/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.acte;

import fr.paris.lutece.plugins.ods.business.direction.IDirectionHome;
import fr.paris.lutece.plugins.ods.dto.acte.Acte;
import fr.paris.lutece.plugins.ods.dto.acte.IActeFilter;
import fr.paris.lutece.plugins.ods.dto.direction.Direction;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;


/**
 * 
 * ActeHome
 *
 * @param <GSeance> ISeance
 * @param <GFichier> IFichier
 * @param <GActeFilter> IActeFilter
 */
public class ActeHome<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>, GActeFilter extends IActeFilter<GFichier, GSeance>>
{
    @Autowired
    private IActeDAO<GSeance, GFichier, GActeFilter> _acteDAO;
    @Autowired
    private StatutActeHome _statutActeHome;
    @Autowired
    private IDirectionHome _directionHome;

    /**
     * Constructeur vide
     */
    public ActeHome(  )
    {
    }

    /**
    * Cre un nouvel acte  partir de lobjet acte pass en paramtre
    * @param acte le statut  insrer
    * @param plugin Plugin
    */
    public void create( Acte<GSeance, GFichier> acte, Plugin plugin )
    {
        _acteDAO.insert( acte, plugin );
    }

    /**
     * Modifie l'acte correspondant  lobjet acte pass en paramtre
     * @param acte le statut  modifier
     * @param plugin Plugin
     */
    public void update( Acte<GSeance, GFichier> acte, Plugin plugin )
    {
        _acteDAO.store( acte, plugin );
    }

    /**
     * Supprime l'acte correspondant  lobjet acte pass en paramtre
     * @param acte le statut  supprimer
     * @param plugin Plugin
     * @throws AppException exception lev si une dependance existe
     */
    public void remove( Acte<GSeance, GFichier> acte, Plugin plugin )
        throws AppException
    {
        _acteDAO.delete( acte, plugin );
    }

    /**
     * renvoie l'acte ayant comme identifiant nKey
     * @param nKey idenfiant de l'acte  supprimer
     * @param plugin Plugin
     * @return objet statut
     */
    public Acte<GSeance, GFichier> findByPrimaryKey( int nKey, Plugin plugin )
    {
        Acte<GSeance, GFichier> acte = _acteDAO.load( nKey, plugin );
        int nIdStatut = acte.getStatutActe(  ).getIdStatut(  );
        acte.getStatutActe(  ).setLibelle( _statutActeHome.findByPrimaryKey( nIdStatut, plugin ).getLibelle(  ) );

        Direction direction = acte.getDirection(  );

        if ( direction != null )
        {
            int nIdDirection = acte.getDirection(  ).getIdDirection(  );
            acte.getDirection(  )
                .setLibelleCourt( _directionHome.findByPrimaryKey( nIdDirection, plugin ).getLibelleCourt(  ) );
        }

        return acte;
    }

    /**
     * renvoie la liste des actes
     * @param acteFilter le filtre de recherche
     * @param plugin Plugin
     * @return liste d'objets acte
     */
    public List<Acte<GSeance, GFichier>> findByFilter( GActeFilter acteFilter, Plugin plugin )
    {
        List<Acte<GSeance, GFichier>> listActes = _acteDAO.selectByFilter( acteFilter, plugin );

        for ( Acte<GSeance, GFichier> acte : listActes )
        {
            int nIdStatut = acte.getStatutActe(  ).getIdStatut(  );
            acte.getStatutActe(  ).setLibelle( _statutActeHome.findByPrimaryKey( nIdStatut, plugin ).getLibelle(  ) );
        }

        return listActes;
    }

    /**
     * renvoie la liste des ids des actes correspondant au filtre passe en parametre
     * @param acteFilter le filtre de recherche
     * @param plugin Plugin
     * @return liste d'objets acte
     */
    public List<Integer> findIdsByFilter( GActeFilter acteFilter, Plugin plugin )
    {
        return _acteDAO.selectIdsByFilter( acteFilter, plugin );
    }

    /**
     * Gnre un nouvau numero de dlibr par rapport  la rfrnce passe en parametre
     * @param strReference la reference
     * @param plugin plugin
     * @return Retourne le nouvau numero gnr
     */
    public int createNewNumeroDelibere( String strReference, Plugin plugin )
    {
        return _acteDAO.newNumeroDelibere( strReference, plugin );
    }

    /**
     * renvoie la liste des actes
     * @param plugin Plugin
     * @return liste d'objets acte
     */
    public List<Acte<GSeance, GFichier>> findActeList( Plugin plugin )
    {
        return _acteDAO.selectActeList( plugin );
    }

    /**
     * Permet de publier ou de dpublier un acte
         * Si isPublie gale  TRUE, alors la mthode <b>publie</b> l'acte
         * Sinon la mthode change le statut enLigne de l'acte a <b>false</b>
     * @param nKey l'identifiant de l'acte
     * @param isPublier le statut de l'arrete
     * @param plugin le plugin
     */
    public void publication( int nKey, boolean isPublier, Plugin plugin )
    {
        _acteDAO.publication( nKey, isPublier, plugin );
    }

    /**
     * insert un commentaire lors de l'annulation d'un acte
     * @param acte Acte
     * @param plugin Plugin
     */
    public void createCommentaireAnnulation( Acte<GSeance, GFichier> acte, Plugin plugin )
    {
        _acteDAO.insertCommentaireAnnulation( acte, plugin );
    }
}
