/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.acte;

import fr.paris.lutece.plugins.ods.business.fichier.IFichierHome;
import fr.paris.lutece.plugins.ods.dto.acte.Acte;
import fr.paris.lutece.plugins.ods.dto.acte.DocumentActe;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter;
import fr.paris.lutece.plugins.ods.dto.fichier.ITypeDocument;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.ArrayList;
import java.util.List;


/**
 * DocumentActeHome
 * @param <GFichier> fichier
 * @param <GFichierFilter> filtre pour fichier
 * @param <GTypeDocument> Type de document
 * @param <GSeance> seance
 */
public final class DocumentActeHome<GFichier extends IFichier<GSeance, GFichier>, GFichierFilter extends IFichierFilter, GTypeDocument extends ITypeDocument, GSeance extends ISeance>
{
    @Autowired
    private IDocumentActeDAO<GSeance, GFichier> _documentActeDAO;
    @Autowired
    private IFichierHome<GFichier, GFichierFilter, GTypeDocument, GSeance> _fichierHome;

    /**
     * Constructeur vide
     */
    private DocumentActeHome(  )
    {
    }

    /**
    * Cre un nouveau document  partir de lobjet document pass en paramtre
    * @param document le document  insrer
    * @param plugin Plugin
    */
    public void create( DocumentActe document, Plugin plugin )
    {
        _documentActeDAO.insert( document, plugin );
    }

    /**
     * Modifie le document correspondant  lobjet document pass en paramtre
     * @param document le document  modifier
     * @param plugin Plugin
     */
    public void update( DocumentActe document, Plugin plugin )
    {
        _documentActeDAO.store( document, plugin );
    }

    /**
     * Supprime le document correspondant  lobjet document pass en paramtre
     * @param document le document  supprimer
     * @param plugin Plugin
     * @throws AppException exception lev si une dependance existe
     */
    public void remove( DocumentActe document, Plugin plugin )
        throws AppException
    {
        _documentActeDAO.delete( document, plugin );
    }

    /**
     * Supprime le document correspondant  lobjet Acte pass en paramtre
     * @param acte l'acte  supprimer
     * @param plugin Plugin
     * @throws AppException exception lev si une dependance existe
     */
    public void removeByActe( Acte<GSeance, GFichier> acte, Plugin plugin )
        throws AppException
    {
        _documentActeDAO.deleteByActe( acte, plugin );
    }

    /**
     * Supprime le document correspondant  lid document pass en paramtre
     * @param nIdDocument l'id du document
     * @param plugin Plugin
     * @throws AppException exception lev si une dependance existe
     */
    public void removeByDocument( int nIdDocument, Plugin plugin )
        throws AppException
    {
        _documentActeDAO.deleteByDocument( nIdDocument, plugin );
    }

    /**
     * renvoie le document ayant comme identifiant nKey
     * @param nKey idenfiant de le document  trouver
     * @param plugin Plugin
     * @return objet document
     */
    public DocumentActe findByPrimaryKey( int nKey, Plugin plugin )
    {
        return _documentActeDAO.load( nKey, plugin );
    }

    /**
     * renvoie la liste des statuts
     * @param plugin Plugin
     * @return liste d'objets statut
     */
    public List<DocumentActe> findDocumentList( Plugin plugin )
    {
        return _documentActeDAO.selectDocumentActeList( plugin );
    }

    /**
     * renvoie si la paire document/acte existe
     * @param document le document
     * @param acte l'acte
     * @param plugin le plugin
     * @return boolean
     */
    public boolean isPairExist( DocumentActe document, Acte<GSeance, GFichier> acte, Plugin plugin )
    {
        return _documentActeDAO.isPairExist( document, acte, plugin );
    }

    /**
     * Renvoie la liste des documents de l'acte
     * @param acte l'acte
     * @param plugin le plugin
     * @return la liste des documents
     */
    public List<GFichier> findByActe( Acte<GSeance, GFichier> acte, Plugin plugin )
    {
        List<GFichier> listFichiers = new ArrayList<GFichier>(  );
        List<Integer> listIdFichiers = _documentActeDAO.selectDocumentByActe( acte, plugin );

        for ( int idFichier : listIdFichiers )
        {
            listFichiers.add( _fichierHome.findByPrimaryKey( idFichier, plugin ) );
        }

        return listFichiers;
    }
}
