/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.acte;

import fr.paris.lutece.plugins.ods.dto.acte.Acte;
import fr.paris.lutece.plugins.ods.dto.acte.IActeFilter;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;

import java.util.List;


/**
 * IActeDAO
 * @param <GActeFilter> filtre pour acte
 * @param <GSeance> seance
 * @param <GFichier> fichier
 */
public interface IActeDAO<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>, GActeFilter extends IActeFilter<GFichier, GSeance>>
{
    /**
    * Insre un nouvel enregistrement dans la table ods_acte  partir de lobjet Acte pass en paramtre
    * @param acte  l'acte  insrer dans la table ods_acte
    * @param plugin plugin
    */
    void insert( Acte<GSeance, GFichier> acte, Plugin plugin );

    /**
     * Modifie l'enregistrement dans la table ods_acte correspondant   lobjet Acte pass en paramtre
     * @param acte l'acte  modifier dans la table ods_acte
     * @param plugin plugin
     */
    void store( Acte<GSeance, GFichier> acte, Plugin plugin );

    /**
     * Supprime l'enregistrement dans la table ods_acte correspondant  lobjet Acte pass en paramtre
     * @param acte l'acte  supprimer dans la table ods_acte
     * @param plugin plugin
     * @throws AppException si le StatutActe est rfrenc par un objet tiers
     */
    void delete( Acte<GSeance, GFichier> acte, Plugin plugin )
        throws AppException;

    /**
     * renvoie l'acte correspondant  l'enregistrement dans la table ods_acte ayant comme identifiant nKey
     * @param nKey l'identifiant de l'acte
     * @param plugin plugin
     * @return le StatutActe correspondant
     */
    Acte<GSeance, GFichier> load( int nKey, Plugin plugin );

    /**
     * renvoie la liste des Actes correspondant au filtre acteFilter
     * @param acteFilter le ActeFilter
     * @param plugin le plugin
     * @return liste d'objets Acte
     */
    List<Acte<GSeance, GFichier>> selectByFilter( GActeFilter acteFilter, Plugin plugin );

    /**
     * renvoie la liste des ids des Actes correspondant au filtre acteFilter
     * @param acteFilter le ActeFilter
     * @param plugin le plugin
     * @return liste d'objets Acte
     */
    List<Integer> selectIdsByFilter( GActeFilter acteFilter, Plugin plugin );

    /**
     * Gnre un nouvau numero de dlibr par rapport  la rfrnce passe en parametre
     * @param strReference la reference
     * @param plugin plugin
     * @return Retourne le nouvau numero gnr
     */
    int newNumeroDelibere( String strReference, Plugin plugin );

    /**
     * renvoie la liste des Actes
     * @param plugin plugin
     * @return Retourne la liste des actes
     */
    List<Acte<GSeance, GFichier>> selectActeList( Plugin plugin );

    /**
     * insert un commentaire lors de l'annulation d'un acte
     * @param acte Acte
     * @param plugin Plugin
     */
    void insertCommentaireAnnulation( Acte<GSeance, GFichier> acte, Plugin plugin );

    /**
     * Permet de publier ou de dpublier un acte
         * Si isPublie gale  TRUE, alors la mthode <b>publie</b> l'acte
         * Sinon la mthode change le statut enLigne de l'acte a <b>false</b>
     * @param nKey l'identifiant de l'acte
     * @param isPublier le statut de l'arrete
     * @param plugin le plugin
     */
    void publication( int nKey, boolean isPublier, Plugin plugin );
}
