/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.acte;

import fr.paris.lutece.plugins.ods.dto.acte.Acte;
import fr.paris.lutece.plugins.ods.dto.acte.DocumentActe;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;

import java.util.List;


/**
 * IStatutActeDAO
 * @param <GSeance> seance
 * @param <GFichier> fichier
 */
public interface IDocumentActeDAO<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>>
{
    /**
    * Insre un nouvel enregistrement dans la table ods_acte_statut  partir de lobjet DocumentActe pass en paramtre
    * @param documentActe  le DocumentActe  insrer dans la table ods_acte_document
    * @param plugin plugin
    */
    void insert( DocumentActe documentActe, Plugin plugin );

    /**
     * Modifie l'enregistrement dans la table ods_acte_statut correspondant   lobjet DocumentActe pass en paramtre
     * @param documentActe le DocumentActe  modifier dans la table ods_acte_document
     * @param plugin plugin
     */
    void store( DocumentActe documentActe, Plugin plugin );

    /**
     * Supprime l'enregistrement dans la table ods_acte_statut correspondant  lobjet DocumentActe pass en paramtre
     * @param documentActe le DocumentActe  supprimer dans la table ods_acte_document
     * @param plugin plugin
     * @throws AppException si le DocumentActe est rfrenc par un objet tiers
     */
    void delete( DocumentActe documentActe, Plugin plugin )
        throws AppException;

    /**
     * Supprime le document correspondant  lobjet Acte pass en paramtre
     * @param acte l'acte  supprimer
     * @param plugin Plugin
     * @throws AppException exception lev si une dependance existe
     */
    void deleteByActe( Acte<GSeance, GFichier> acte, Plugin plugin )
        throws AppException;

    /**
     * Supprime le document correspondant  l'idDocument pass en paramtre
     * @param nIdDocument l'id du document
     * @param plugin Plugin
     * @throws AppException exception lev si une dependance existe
     */
    void deleteByDocument( int nIdDocument, Plugin plugin )
        throws AppException;

    /**
     * renvoie le DocumentActe correspondant  l'enregistrement dans la table ods_acte_document ayant comme identifiant nKey
     * @param nKey l'identifiant du StatutActe
     * @param plugin plugin
     * @return le DocumentActe correspondant
     */
    DocumentActe load( int nKey, Plugin plugin );

    /**
     * renvoie la liste des StatutActes prsents dans la table ods_StatutActe
     * @param plugin le plugin
     * @return liste d'objets StatutActe
     */
    List<DocumentActe> selectDocumentActeList( Plugin plugin );

    /**
     * renvoie la paire document/acte correspondant  l'id nKe
     * @param document le document
     * @param acte l'acte
     * @param plugin Plugin
     * @return boolean
     */
    boolean isPairExist( DocumentActe document, Acte<GSeance, GFichier> acte, Plugin plugin );

    /**
     * renvoie les documents lis  l'acte
     * @param acte l'acte
     * @param plugin Plugin
     * @return boolean
     */
    List<Integer> selectDocumentByActe( Acte<GSeance, GFichier> acte, Plugin plugin );
}
