/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.acte;

import fr.paris.lutece.plugins.ods.dto.acte.NatureActe;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;

import java.util.List;


/**
 * INatureActeDAO
 */
public interface INatureActeDAO
{
    /**
    * Insre un nouvel enregistrement dans la table ods_acte_statut  partir de lobjet NatureActe pass en paramtre
    * @param natureActe  le natureActe  insrer dans la table ods_acte_nature
    * @param plugin plugin
    */
    void insert( NatureActe natureActe, Plugin plugin );

    /**
     * Modifie l'enregistrement dans la table ods_acte_statut correspondant   lobjet NatureActe pass en paramtre
     * @param natureActe le natureActe  modifier dans la table ods_acte_nature
     * @param plugin plugin
     */
    void store( NatureActe natureActe, Plugin plugin );

    /**
     * Supprime l'enregistrement dans la table ods_acte_statut correspondant  lobjet NatureActe pass en paramtre
     * @param natureActe le natureActe  supprimer dans la table ods_acte_nature
     * @param plugin plugin
     * @throws AppException si le StatutActe est rfrenc par un objet tiers
     */
    void delete( NatureActe natureActe, Plugin plugin )
        throws AppException;

    /**
     * renvoie le StatutActe correspondant  l'enregistrement dans la table ods_acte_nature ayant comme identifiant nKey
     * @param nKey l'identifiant du NatureActe
     * @param plugin plugin
     * @return le natureActe correspondant
     */
    NatureActe load( int nKey, Plugin plugin );

    /**
     * renvoie la liste des StatutActes prsents dans la table ods_acte_nature
     * @param plugin le plugin
     * @return liste d'objets StatutActe
     */
    List<NatureActe> selectNatureActeList( Plugin plugin );
}
