/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.acte;

import fr.paris.lutece.plugins.ods.dto.acte.NatureActe;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.util.sql.DAOUtil;

import java.util.ArrayList;
import java.util.List;


/**
 * StatutDAO
 */
public class NatureActeDAO implements INatureActeDAO
{
    /*
     * Requetes SQL
     */
    private static final String SQL_QUERY_NEW_PK = "SELECT max( id_nature_acte ) FROM ods_acte_nature ";
    private static final String SQL_QUERY_INSERT = "INSERT INTO ods_acte_nature " + " ( id_nature_acte, libelle )" +
        "VALUES (?,?) ";
    private static final String SQL_QUERY_FIND_BY_PRIMARY_KEY = "SELECT " +
        " id_nature_acte, libelle FROM ods_acte_nature WHERE id_nature_acte = ? ";
    private static final String SQL_QUERY_NATURE_LIST = "SELECT id_nature_acte, libelle FROM ods_acte_nature ";
    private static final String SQL_QUERY_DELETE = " DELETE FROM ods_acte_nature WHERE id_nature_acte = ?  ";
    private static final String SQL_QUERY_UPDATE = "UPDATE ods_acte_nature SET libelle = ?" +
        " WHERE id_nature_acte = ? ";
    private static final String SQL_QUERY_ORDER_BY_LIBELLE = " ORDER BY libelle ASC ";

    /**
     * Gnre un nouvel identifiant
     * @param plugin plugin
     * @return Retourne lidentifiant gnr
     */
    int newPrimaryKey( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey;

        if ( !daoUtil.next(  ) )
        {
            nKey = 1;
        }

        nKey = daoUtil.getInt( 1 ) + 1;
        daoUtil.free(  );

        return nKey;
    }

    /**
     * Supprime la nature correspondant  lobjet NatureActe pass en paramtre
     * @param natureActe la nature  supprimer
     * @param plugin Plugin
     * @throws AppException exception lev si une dependance existe
     */
    public void delete( NatureActe natureActe, Plugin plugin )
        throws AppException
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE, plugin );
        daoUtil.setInt( 1, natureActe.getIdNatureDocument(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Insre la nature correspondant  lobjet NatureActe pass en paramtre
     * @param natureActe la nature  insrer
     * @param plugin Plugin
     */
    public void insert( NatureActe natureActe, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_INSERT, plugin );
        int nKey = newPrimaryKey( plugin );

        daoUtil.setInt( 1, nKey );
        daoUtil.setString( 2, natureActe.getLibelle(  ) );

        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Rcupre la nature correspondant  l'id nKey
     * @param nKey l'id de la nature  rcuprer
     * @param plugin Plugin
     * @return la nature correspondant  l'id nKey
     */
    public NatureActe load( int nKey, Plugin plugin )
    {
        NatureActe natureActe = null;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_BY_PRIMARY_KEY, plugin );
        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            natureActe = new NatureActe(  );
            natureActe.setIdNatureDocument( daoUtil.getInt( 1 ) );
            natureActe.setLibelle( daoUtil.getString( 2 ) );
        }

        daoUtil.free(  );

        return natureActe;
    }

    /**
     * Rcupre la liste des natures
     * @param plugin Plugin
     * @return la liste des natures
     */
    public List<NatureActe> selectNatureActeList( Plugin plugin )
    {
        List<NatureActe> listNatures = new ArrayList<NatureActe>(  );
        NatureActe natureActe;

        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NATURE_LIST + SQL_QUERY_ORDER_BY_LIBELLE, plugin );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            natureActe = new NatureActe(  );
            natureActe.setIdNatureDocument( daoUtil.getInt( 1 ) );
            natureActe.setLibelle( daoUtil.getString( 2 ) );

            listNatures.add( natureActe );
        }

        daoUtil.free(  );

        return listNatures;
    }

    /**
     * Met  jour la nature passe en paramtre dans la base de donnes
     * @param natureActe la nature  mettre  jour
     * @param plugin Plugin
     */
    public void store( NatureActe natureActe, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_UPDATE, plugin );

        daoUtil.setString( 1, natureActe.getLibelle(  ) );
        daoUtil.setInt( 2, natureActe.getIdNatureDocument(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }
}
