/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.acte;

import fr.paris.lutece.plugins.ods.dto.acte.NatureActe;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;


/**
 * NatureActeHome
 */
public final class NatureActeHome
{
    @Autowired
    private INatureActeDAO _natureActeDAO;

    /**
     * Constructeur vide
     */
    private NatureActeHome(  )
    {
    }

    /**
    * Cre un nouvel acte  partir de lobjet acte pass en paramtre
    * @param natureActe la nature  insrer
    * @param plugin Plugin
    */
    public void create( NatureActe natureActe, Plugin plugin )
    {
        _natureActeDAO.insert( natureActe, plugin );
    }

    /**
     * Modifie l'acte correspondant  lobjet acte pass en paramtre
     * @param natureActe la nature de l'acte  modifier
     * @param plugin Plugin
     */
    public void update( NatureActe natureActe, Plugin plugin )
    {
        _natureActeDAO.store( natureActe, plugin );
    }

    /**
     * Supprime l'acte correspondant  lobjet acte pass en paramtre
     * @param natureActe la nature de l'acte  supprimer
     * @param plugin Plugin
     * @throws AppException exception lev si une dependance existe
     */
    public void remove( NatureActe natureActe, Plugin plugin )
        throws AppException
    {
        _natureActeDAO.delete( natureActe, plugin );
    }

    /**
     * renvoie l'acte ayant comme identifiant nKey
     * @param nKey idenfiant de la nature
     * @param plugin Plugin
     * @return objet natureActe
     */
    public NatureActe findByPrimaryKey( int nKey, Plugin plugin )
    {
        return _natureActeDAO.load( nKey, plugin );
    }

    /**
     * renvoie la liste des actes
     * @param plugin Plugin
     * @return liste d'objets Acte
     */
    public List<NatureActe> findActesList( Plugin plugin )
    {
        return _natureActeDAO.selectNatureActeList( plugin );
    }
}
