/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.acte;

import fr.paris.lutece.plugins.ods.dto.acte.StatutActe;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.util.sql.DAOUtil;

import java.util.ArrayList;
import java.util.List;


/**
 * StatutDAO
 */
public class StatutActeDAO implements IStatutActeDAO
{
    /*
     * Requetes SQL
     */
    private static final String SQL_QUERY_NEW_PK = "SELECT max( id_statut_acte ) FROM ods_acte_statut ";
    private static final String SQL_QUERY_INSERT = "INSERT INTO ods_acte_statut " + " ( id_statut_acte, libelle )" +
        "VALUES (?,?) ";
    private static final String SQL_QUERY_FIND_BY_PRIMARY_KEY = "SELECT " +
        " id_statut_acte, libelle FROM ods_acte_statut WHERE id_statut_acte = ? ";
    private static final String SQL_QUERY_STATUT_LIST = "SELECT id_statut_acte, libelle FROM ods_acte_statut ";
    private static final String SQL_QUERY_DELETE = " DELETE FROM ods_acte_statut WHERE id_statut_acte = ?  ";
    private static final String SQL_QUERY_UPDATE = "UPDATE ods_acte_statut SET libelle = ?" +
        " WHERE id_statut_acte = ? ";
    private static final String SQL_QUERY_ORDER_BY_LIBELLE = " ORDER BY libelle ASC ";

    /**
     * Gnre un nouvel identifiant
     * @param plugin plugin
     * @return Retourne lidentifiant gnr
     */
    int newPrimaryKey( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey;

        if ( !daoUtil.next(  ) )
        {
            nKey = 1;
        }

        nKey = daoUtil.getInt( 1 ) + 1;
        daoUtil.free(  );

        return nKey;
    }

    /**
     * Supprime le statut correspondant  lobjet statut pass en paramtre
     * @param statut le statut  supprimer
     * @param plugin Plugin
     * @throws AppException exception lev si une dependance existe
     */
    public void delete( StatutActe statut, Plugin plugin )
        throws AppException
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE, plugin );
        daoUtil.setInt( 1, statut.getIdStatut(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Insre le statut correspondant  lobjet statut pass en paramtre
     * @param statut le statut  insrer
     * @param plugin Plugin
     */
    public void insert( StatutActe statut, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_INSERT, plugin );
        int nKey = newPrimaryKey( plugin );

        daoUtil.setInt( 1, nKey );
        daoUtil.setString( 2, statut.getLibelle(  ) );

        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Rcupre le statut correspondant  l'id nKey
     * @param nKey l'id du statut  rcuprer
     * @param plugin Plugin
     * @return le statut correspondant  l'id nKey
     */
    public StatutActe load( int nKey, Plugin plugin )
    {
        StatutActe statut = null;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_BY_PRIMARY_KEY, plugin );
        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            statut = new StatutActe(  );
            statut.setIdStatut( daoUtil.getInt( 1 ) );
            statut.setLibelle( daoUtil.getString( 2 ) );
        }

        daoUtil.free(  );

        return statut;
    }

    /**
     * Rcupre la liste des statuts
     * @param plugin Plugin
     * @return la liste des statuts
     */
    public List<StatutActe> selectStatutActeList( Plugin plugin )
    {
        List<StatutActe> listStatuts = new ArrayList<StatutActe>(  );
        StatutActe statut;

        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_STATUT_LIST + SQL_QUERY_ORDER_BY_LIBELLE, plugin );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            statut = new StatutActe(  );
            statut.setIdStatut( daoUtil.getInt( 1 ) );
            statut.setLibelle( daoUtil.getString( 2 ) );

            listStatuts.add( statut );
        }

        daoUtil.free(  );

        return listStatuts;
    }

    /**
     * Met  jour le statut pass en paramtre dans la base de donnes
     * @param statut le statut  mettre  jour
     * @param plugin Plugin
     */
    public void store( StatutActe statut, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_UPDATE, plugin );

        daoUtil.setString( 1, statut.getLibelle(  ) );
        daoUtil.setInt( 2, statut.getIdStatut(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }
}
