/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.acte;

import fr.paris.lutece.plugins.ods.dto.acte.StatutActe;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;


/**
 * StatutActeHome
 */
public final class StatutActeHome
{
    @Autowired
    private IStatutActeDAO _statutActeDAO;

    /**
     * Constructeur vide
     */
    private StatutActeHome(  )
    {
    }

    /**
    * Cre un nouveau statut  partir de lobjet statutActe pass en paramtre
    * @param statutActe le statut  insrer
    * @param plugin Plugin
    */
    public void create( StatutActe statutActe, Plugin plugin )
    {
        _statutActeDAO.insert( statutActe, plugin );
    }

    /**
     * Modifie le statut correspondant  lobjet statut pass en paramtre
     * @param statutActe le statut  modifier
     * @param plugin Plugin
     */
    public void update( StatutActe statutActe, Plugin plugin )
    {
        _statutActeDAO.store( statutActe, plugin );
    }

    /**
     * Supprime le statut correspondant  lobjet statut pass en paramtre
     * @param statutActe le statut  supprimer
     * @param plugin Plugin
     * @throws AppException exception lev si une dependance existe
     */
    public void remove( StatutActe statutActe, Plugin plugin )
        throws AppException
    {
        _statutActeDAO.delete( statutActe, plugin );
    }

    /**
     * renvoie le statut ayant comme identifiant nKey
     * @param nKey idenfiant de le statut  supprimer
     * @param plugin Plugin
     * @return objet statut
     */
    public StatutActe findByPrimaryKey( int nKey, Plugin plugin )
    {
        return _statutActeDAO.load( nKey, plugin );
    }

    /**
     * renvoie la liste des statuts
     * @param plugin Plugin
     * @return liste d'objets statut
     */
    public List<StatutActe> findStatutList( Plugin plugin )
    {
        return _statutActeDAO.selectStatutActeList( plugin );
    }
}
