/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.arrete;

import fr.paris.lutece.plugins.ods.dto.arrete.Arrete;
import fr.paris.lutece.plugins.ods.dto.arrete.ArreteFilter;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.portal.service.plugin.Plugin;

import java.sql.Timestamp;

import java.util.List;


/**
 * IArreteDAO
 *
 * @param <GSeance> seance
 * @param <GFichier> fichier
 */
public interface IArreteDAO<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>>
{
    /**
     * Cre un nouvel arrete  partir de lobjet arrete pass en paramtre
     * @param arrete l'arrete  insrer
     * @param plugin le Plugin actif
     * @return int primaryKey
     */
    int insert( Arrete<GSeance, GFichier> arrete, Plugin plugin );

    /**
     * Modifie l'arrete correspondant  lobjet arrete pass en paramtre
     * @param arrete l'arrete  modifier
     * @param plugin le Plugin actif
     */
    void store( Arrete<GSeance, GFichier> arrete, Plugin plugin );

    /**
     * Supprime l'arrete correspondant  lobjet arrete pass en paramtre
     * @param arrete l'arrete  supprimer
     * @param plugin le Plugin actif
     */
    void delete( Arrete<GSeance, GFichier> arrete, Plugin plugin );

    /**
     * renvoie l'arrete ayant comme identifiant nKey
     * @param nKey idenfiant de ll'arreten  supprimer
     * @param plugin le Plugin actif
     * @return un arrete
     */
    Arrete<GSeance, GFichier> load( int nKey, Plugin plugin );

    /**
     * rcupre la liste des arretes correspondant au filtre donn
     * @param filter le filtre
     * @param plugin le Plugin actif
     * @return uen liste d'arrete
     */
    List<Arrete<GSeance, GFichier>> selectByFilter( ArreteFilter filter, Plugin plugin );

    /**
     * renvoie l'arrete ayant comme identifiant de fichier nKey
     * @param nKey idenfiant du fichier
     * @param plugin le Plugin actif
     * @return l'arrete
     */
    Arrete<GSeance, GFichier> selectByFichier( int nKey, Plugin plugin );

    /**
     * Permet de publier ou de dpublier un arrete
     * Publication d'un arrete
     * Si isPublier gale  TRUE, alors la mthode <b>publie</b> l'arrete et incrmente la version<BR>
     * Sinon la mthode change le statut enLigne d'un arrete a FALSE
     * @param nKey id de l'arrete
     * @param tsDatePublication TimeStamp date de publication
     * @param version version
     * @param isPublier enLigne
     * @param plugin le Plugin actif
     */
    void publication( int nKey, Timestamp tsDatePublication, int version, boolean isPublier, Plugin plugin );
}
