/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.autremodele;

import fr.paris.lutece.plugins.ods.business.fichier.FichierPhysiqueHome;
import fr.paris.lutece.plugins.ods.dto.autremodele.AutreModeleFilter;
import fr.paris.lutece.plugins.ods.dto.autremodele.IAutreModele;
import fr.paris.lutece.plugins.ods.dto.fichier.FichierPhysique;
import fr.paris.lutece.portal.service.plugin.Plugin;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;


/**
 * AutreModeleHome
 * @param <GAutreModele> IAutreModele
 */
public final class AutreModeleHome<GAutreModele extends IAutreModele>
{
    @Autowired
    private IAutreModeleDAO<GAutreModele> _autreModeleDAO;
    @Autowired
    private FichierPhysiqueHome _fichierPhysiqueHome;

    /**
     * Constructeur vide
     */
    private AutreModeleHome(  )
    {
    }

    /**
     * Cre un modele d'ordre du jour dans la base de donnes
     * @param autreModele le modele d'ordre du jour
     * @param plugin le plugin ods
     * @return l'identifiant du modele cr
     */
    public int create( GAutreModele autreModele, Plugin plugin )
    {
        return _autreModeleDAO.insert( autreModele, plugin );
    }

    /**
     * modifie le modele d'ordre du jour dans la base de donnes
     * @param autreModele le modele d'ordre du jour
     * @param plugin le plugin ods
     */
    public void update( GAutreModele autreModele, Plugin plugin )
    {
        _autreModeleDAO.store( autreModele, plugin );
    }

    /**
     * supprime le modele d'ordre du jour dans la base de donnes
     * @param autreModele le modele d'ordre du jour
     * @param plugin le plugin ods
     */
    public void remove( GAutreModele autreModele, Plugin plugin )
    {
        _autreModeleDAO.delete( autreModele, plugin );
    }

    /**
     * renvoie le modele d'ordre du jour ayant l'identifiant donn en parametre
     * @param nKey l'id du modele d'ordre du jour
     * @param plugin le plugin ods
     * @return le modele d'ordre du jour
     */
    public IAutreModele findByPrimaryKey( int nKey, Plugin plugin )
    {
        return _autreModeleDAO.load( nKey, plugin );
    }

    /**
     * verifie si le modele d'ordre du jour existe deja dans la base de donnes
     * @param nIdTypeAutreModele identifiant du type de modele
     * @param nIdAutreModele l'id de l'autre modle
     * @param plugin plugin ods
     * @return TRUE si le modele d'ordre du jour existe deja dans la base de donnes, FALSE sinon
     */
    public boolean isAlreadyExist( int nIdTypeAutreModele, int nIdAutreModele, Plugin plugin )
    {
        return _autreModeleDAO.isAlreadyExist( nIdTypeAutreModele, nIdAutreModele, plugin );
    }

    /**
     * renvoie le modele d'ordre du jour correspondant au criteres du filtre donn en parametre
     * @param filter le filtre
     * @param plugin plugin ods
     * @return le modele d'ordre du jour
     */
    public List<GAutreModele> findByFilter( AutreModeleFilter filter, Plugin plugin )
    {
        List<GAutreModele> listAutreModele = _autreModeleDAO.selectListByFilter( filter, plugin );

        for ( GAutreModele autreModele : listAutreModele )
        {
            setOtherProperty( autreModele, plugin );
        }

        return listAutreModele;
    }

    /**
     * Permet de complter le modle avec les proprits qui ont des contraintes sur d'autres tables
     * @param autreModele le modele  complter
     * @param plugin Plugin
     */
    private void setOtherProperty( GAutreModele autreModele, Plugin plugin )
    {
        if ( autreModele.getEnteteDocument(  ) != null )
        {
            FichierPhysique fichierPhysique = _fichierPhysiqueHome.findByPrimaryKey( autreModele.getEnteteDocument(  )
                                                                                                .getIdFichier(  ),
                    plugin );
            autreModele.setEnteteDocument( fichierPhysique );
        }

        if ( autreModele.getPiedDocument(  ) != null )
        {
            FichierPhysique fichierPhysique = _fichierPhysiqueHome.findByPrimaryKey( autreModele.getPiedDocument(  )
                                                                                                .getIdFichier(  ),
                    plugin );
            autreModele.setPiedDocument( fichierPhysique );
        }
    }
}
