/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.autremodele;

import fr.paris.lutece.plugins.ods.dto.autremodele.AutreModeleFilter;
import fr.paris.lutece.plugins.ods.dto.autremodele.IAutreModele;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;

import java.util.List;


/**
 * IAutreModeleDAO
 * @param <GAutreModele> IAutreModele
 */
public interface IAutreModeleDAO<GAutreModele extends IAutreModele>
{
    /**
     * Cre un nouveau MODJ  partir de lobjet MODJ pass en paramtre
     * @param autreModele le MODJ  insrer
     * @param plugin le plugin
     * @return int primaryKey
     */
    int insert( GAutreModele autreModele, Plugin plugin );

    /**
     * Modifie le MODJ correspondant  lobjet MODJ pass en paramtre
     * @param autreModele le MODJ  modifier
     * @param plugin le plugin
     */
    void store( GAutreModele autreModele, Plugin plugin );

    /**
     * Supprime le MODJ correspondant  lobjet pass en paramtre
     * @param autreModele  le  MODJ  supprimer
     * @param plugin le plugin
     * @throws AppException si contrainte de cl trangre
     */
    void delete( GAutreModele autreModele, Plugin plugin )
        throws AppException;

    /**
     * Retourne le AutreModele ayant comme identifiant nKey
     * @param nKey identifiant du AutreModele  charger
     * @param plugin le plugin
     * @return le fichier identifi par nKey
     */
    IAutreModele load( int nKey, Plugin plugin );

    /**
     * Retourne le AutreModelecorrespondant  nIdTypeAutreModele
     * @param filter le filtre d'autre modle
     * @param plugin le plugin
     * @return la liste des AutreModelea correspondant au filtre
     */
    List<GAutreModele> selectListByFilter( AutreModeleFilter filter, Plugin plugin );

    /**
     *  Retourne true si le modle existe dj, false sinon
     * @param nIdtypeAutreModele l'id du type de l'autre modle
     * @param nIdAutreModele l'id de l'autre modle
     * @param plugin le plugin
     * @return true si le modle existe dj, false sinon
     */
    boolean isAlreadyExist( int nIdtypeAutreModele, int nIdAutreModele, Plugin plugin );
}
