/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.autremodele;

import fr.paris.lutece.plugins.ods.dto.autremodele.TypeAutreModele;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.util.sql.DAOUtil;

import java.util.ArrayList;
import java.util.List;


/**
 * Cette classe permet d'accder/grer la table ods_type_autre_modele
 */
public class TypeAutreModeleDAO implements ITypeAutreModeleDAO
{
    private static final String SQL_QUERY_SELECT = "SELECT id_type_autre_modele,libelle_type FROM ods_type_autre_modele ";
    private static final String SQL_QUERY_FIND_BY_PRIMARY_KEY = SQL_QUERY_SELECT + " WHERE id_type_autre_modele = ? ";
    private static final String SQL_QUERY_TYPE_ODJ_LIST = SQL_QUERY_SELECT;

    /**
     * Retourne le type d'autre modle identifi par <b>nKey</b>
     * @param nKey l'identifiant du type d'autre modle  charger
     * @param plugin le plugin
     * @return le type d'autre modle identifi par <b>nKey</b>
     */
    public TypeAutreModele load( int nKey, Plugin plugin )
    {
        TypeAutreModele typeAutreModele = null;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_BY_PRIMARY_KEY, plugin );
        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            typeAutreModele = setTypeAutreModeleValues( daoUtil );
        }

        daoUtil.free(  );

        return typeAutreModele;
    }

    /**
     * Retourne la liste des formations de conseil
     * @param plugin le plugin
     * @return le libell de la formation de conseil
     */
    public List<TypeAutreModele> selectTypeAutreModeleList( Plugin plugin )
    {
        List<TypeAutreModele> typeAutreModeles = new ArrayList<TypeAutreModele>(  );
        TypeAutreModele typeAutreModele = null;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_TYPE_ODJ_LIST, plugin );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            typeAutreModele = setTypeAutreModeleValues( daoUtil );
            typeAutreModeles.add( typeAutreModele );
        }

        daoUtil.free(  );

        return typeAutreModeles;
    }

    /**
     * Cre un nouvel objet avec les valeurs contenues dans daoUtil
     * @param daoUtil le DAOUtil
     * @return TypeAutreModele
     */
    private TypeAutreModele setTypeAutreModeleValues( DAOUtil daoUtil )
    {
        TypeAutreModele typeAutreModele = new TypeAutreModele(  );
        typeAutreModele.setIdTypeAutreModele( daoUtil.getInt( "id_type_autre_modele" ) );
        typeAutreModele.setLibelle( daoUtil.getString( "libelle_type" ) );

        return typeAutreModele;
    }
}
