/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.autremodele;

import fr.paris.lutece.plugins.ods.dto.autremodele.TypeAutreModele;
import fr.paris.lutece.portal.service.plugin.Plugin;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;


/**
 * Cette classe permet de manipuler des objets TypeAutreModele
 */
public final class TypeAutreModeleHome
{
    @Autowired
    private ITypeAutreModeleDAO _typeAutreModeleDAO;

    /**
     * Constructeur vide
     */
    private TypeAutreModeleHome(  )
    {
    }

    /**
     * Retourne la formation identifie par <b>nKey</b>
     * @param nKey l'identifiant de la formation
     * @param plugin le plugin
     * @return la formation de conseil identifie par <b>nKey</b>
     */
    public TypeAutreModele findByPrimaryKey( int nKey, Plugin plugin )
    {
        return _typeAutreModeleDAO.load( nKey, plugin );
    }

    /**
     * Retourne la liste des formations de conseil
     * @param plugin le plugin
     * @return la liste des formations de conseil
     */
    public List<TypeAutreModele> findTypeAutreModeleList( Plugin plugin )
    {
        return _typeAutreModeleDAO.selectTypeAutreModeleList( plugin );
    }
}
