/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.categoriectrllegalite;

import fr.paris.lutece.plugins.ods.dto.categoriectrllegalite.CategorieCtrlLegalite;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.util.sql.DAOUtil;

import java.util.ArrayList;
import java.util.List;


/**
 * Cette classe permet d'accder aux donnes de grer la table ods_categorie_ctrl_legalite
 */
public class CategorieCtrlLegaliteDAO implements ICategorieCtrlLegaliteDAO
{
    private static final String SQL_QUERY_FIND_BY_PRIMARY_KEY = "SELECT id_categorie, libelle_categorie, codeMatiere1, " +
        "codeMatiere2 FROM ods_categorie_ctrl_legalite " + "WHERE id_categorie = ? ";
    private static final String SQL_QUERY_FIND_ALL = "SELECT id_categorie,libelle_categorie,codeMatiere1,codeMatiere2 " +
        "FROM ods_categorie_ctrl_legalite ";
    private static final String SQL_QUERY_ORDER_BY_LIBELLE = " ORDER BY libelle_categorie ";
    private static final String SQL_QUERY_FIND_BY_LIBELLE = " WHERE libelle_categorie = ? ";

    /**
     * Retourne la catgorie associe  la cl passe en paramtre
     * @param nKey l'id unique de la catgorie dans la table ods_categorie_ctrl_legalite
     * @param plugin spcifie le plugin actif
     * @return la catgorie associe  l'identifiant nKey
     */
    public CategorieCtrlLegalite load( int nKey, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_BY_PRIMARY_KEY, plugin );
        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        CategorieCtrlLegalite categorie = null;

        if ( daoUtil.next(  ) )
        {
            categorie = getBaseProperty( daoUtil );
        }

        daoUtil.free(  );

        return categorie;
    }

    /**
     * Retourne la catgorie du controle de legalite
     * @param strLibelle Le libelle de la catgorie
     * @param plugin spcifie le plugin actif
     * @return une liste de catgorie du controle de legalite
     */
    public CategorieCtrlLegalite findByLibelle( String strLibelle, Plugin plugin )
    {
        CategorieCtrlLegalite categorie;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_ALL + SQL_QUERY_FIND_BY_LIBELLE, plugin );
        daoUtil.setString( 1, strLibelle );
        daoUtil.executeQuery(  );

        categorie = getBaseProperty( daoUtil );

        daoUtil.free(  );

        return categorie;
    }

    /**
     * Retourne la liste des catgories de dlibrations
     * @param plugin spcifie le plugin actif
     * @return une liste de catgorie de dlibrations
     */
    public List<CategorieCtrlLegalite> loadListeCategorieCtrlLegalite( Plugin plugin )
    {
        List<CategorieCtrlLegalite> listCategorie = new ArrayList<CategorieCtrlLegalite>(  );

        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_ALL + SQL_QUERY_ORDER_BY_LIBELLE, plugin );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            CategorieCtrlLegalite categorie = getBaseProperty( daoUtil );
            listCategorie.add( categorie );
        }

        daoUtil.free(  );

        return listCategorie;
    }

    /**
     * retourne la catgorie avec tous les parametres renseigns  partir de daoUtil
     * @param daoUtil DAOUtil
     * @return CategorieCtrlLegalite la catgorie
     */
    private CategorieCtrlLegalite getBaseProperty( DAOUtil daoUtil )
    {
        CategorieCtrlLegalite categorie = new CategorieCtrlLegalite(  );
        categorie.setIdCategorie( daoUtil.getInt( "id_categorie" ) );
        categorie.setLibelle( daoUtil.getString( "libelle_categorie" ) );
        categorie.setCodeMatiere1( daoUtil.getInt( "codeMatiere1" ) );
        categorie.setCodeMatiere2( daoUtil.getInt( "codeMatiere2" ) );

        return categorie;
    }
}
