/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.categoriectrllegalite;

import fr.paris.lutece.plugins.ods.dto.categoriectrllegalite.CategorieCtrlLegalite;
import fr.paris.lutece.portal.service.plugin.Plugin;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;


/**
 * Cette classe permet l'accs aux objets CatgorieDlibration.
 */
public final class CategorieCtrlLegaliteHome
{
    @Autowired
    private ICategorieCtrlLegaliteDAO _categorieCtrlLegaliteDAO;

    /**
     * Constructeur vide
     */
    private CategorieCtrlLegaliteHome(  )
    {
    }

    /**
     * Retourne la catgorie de ayant pour identifiant nKey.
     * @param nKey l'identifiant de la catgorie
     * @param plugin le plugin actif
     * @return la catgorie recherche
     */
    public CategorieCtrlLegalite findByPrimaryKey( int nKey, Plugin plugin )
    {
        return _categorieCtrlLegaliteDAO.load( nKey, plugin );
    }

    /**
     * Retourne la liste de toutes les catgories.
     * @param plugin le plugin actif
     * @return la liste des catgories
     */
    public List<CategorieCtrlLegalite> findCategorieCtrlLegaliteList( Plugin plugin )
    {
        return _categorieCtrlLegaliteDAO.loadListeCategorieCtrlLegalite( plugin );
    }

    /**
     * Retourne la catgorie de ayant pour Libelle strLibelle
     * @param strLibelle Le libelle de la categorie de CL
     * @param plugin le plugin actif
     * @return la catgorie recherche
     */
    public CategorieCtrlLegalite findByLibelle( String strLibelle, Plugin plugin )
    {
        return _categorieCtrlLegaliteDAO.findByLibelle( strLibelle, plugin );
    }
}
