/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.categoriedeliberation;

import fr.paris.lutece.plugins.ods.dto.categoriedeliberation.CategorieDeliberation;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;

import java.util.List;


/**
 * Interface pour accder aux / grer les donnes de la table ods_categorie_deliberation
 */
public interface ICategorieDeliberationDAO
{
    /**
     * Cre une nouvelle catgorie de dlibration dans la table ods_categorie_deliberation
     * @param categorieDeliberation la catgorie de dlibration  insrer
     * @param plugin spcifie le plugin actif
     */
    void insert( CategorieDeliberation categorieDeliberation, Plugin plugin );

    /**
     * Supprime la catgorie passe en paramtre de la table ods_categorie_deliberation
     * @param categorieDeliberation la catgorie de dlibration  supprimer
     * @param plugin spcifie le plugin actif
     * @throws AppException si la catgorie de dlibration est rfrence par un PDD
     */
    void delete( CategorieDeliberation categorieDeliberation, Plugin plugin )
        throws AppException;

    /**
     * Retourne la catgorie de dlibration associe  la cl passe en paramtre
     * @param nKey l'id unique de la catgorie de dlibration dans la table ods_categorie_deliberation
     * @param plugin spcifie le plugin actif
     * @return la catgorie de dlibration associe  l'identifiant nKey
     */
    CategorieDeliberation load( int nKey, Plugin plugin );

    /**
     * Met  jour les informations d'une catgorie de dlibration dans la table ods_categorie_deliberation
     * @param categorieDeliberation contient les nouvelles valeurs de la catgorie de dlibration
     * @param plugin spcifie le plugin actif
     */
    void store( CategorieDeliberation categorieDeliberation, Plugin plugin );

    /**
     * Retourne la liste des catgories de dlibrations
     * @param plugin spcifie le plugin actif
     * @return une liste de catgorie de dlibrations
     */
    List<CategorieDeliberation> loadListeCategorieDeliberation( Plugin plugin );

    /**
     * Retourne la liste des codes de la table ods_categorie_deliberation;
     * Exclut de la requte la catgorie de dlibration associe  l'identifiant exceptIdcatgorie de dlibration.
     * @param exceptCategorie la catgorie de dlibration  exclure de la liste ( =null si aucune catgorie de dlibration  exclure)
     * @param plugin spcifie le plugin actif
     * @return la liste des codes de catgorie de dlibration
     */
    List<String> listCodes( CategorieDeliberation exceptCategorie, Plugin plugin );

    /**
     * Retourne l'id de la catgorie correspondant au code pass en paramtre
     * @param nCodeCategorie le code de la ctagorie
     * @param plugin le plugin
     * @return l'id de la catgorie
     */
    int listIdCategorieByCode( int nCodeCategorie, Plugin plugin );

    /**
     * Retourne la catgorie correspondant au code pass en paramtre
     * @param nCodeCategorie le code de la ctagorie
     * @param plugin le plugin
     * @return la catgorie
     */
    CategorieDeliberation listCategorieByCode( int nCodeCategorie, Plugin plugin );

    /**
     * Retourne la liste des catgories de dlibration actives.
     * @param plugin spcifie le plugin actif
     * @return liste des catgories de dlibration actives
     */
    List<CategorieDeliberation> listCategoriesActives( Plugin plugin );

    /**
     * Retourne la liste des catgories de dlibration ordonnes par ordre alphabtiqque des libelles
     * @param plugin spcifie le plugin actif
     * @return la liste des catgories de dlibration ordonnes
     */

    /**
     * Retourne la liste des code des catgories de type march.
     * @param plugin spcifie le plugin actif
     * @return liste des codes des catgories de type march
     */
    List<String> listCodesCategoriesExportParisFr( Plugin plugin );

    /**
     * Retourne la liste des catgories de dlibration ordonnes par ordre alphabtiqque des libelles
     * @param plugin le plugin
     * @return la liste des catgories de dlibration ordonnes
     */
    List<CategorieDeliberation> listCategoriesOrderByLibelle( Plugin plugin );
}
