/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.categoriedeliberation;

import fr.paris.lutece.plugins.ods.dto.categoriedeliberation.CategorieDeliberation;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;

import java.util.List;


/**
 *
 * ICategorieDeliberationHome
 *
 */
public interface ICategorieDeliberationHome
{
    /**
     * Cre une nouvelle catgorie de dlibration  partir de l'objet pass en argument.
     * @param categorieDeliberation la ctagorie  crer
     * @param plugin le plugin actif
     */
    void create( CategorieDeliberation categorieDeliberation, Plugin plugin );

    /**
     * Retourne la catgorie de dlibration ayant pour identifiant nKey.
     * @param nKey l'identifiant de la catgorie
     * @param plugin le plugin actif
     * @return la catgorie de dlibration recherche
     */
    CategorieDeliberation findByPrimaryKey( int nKey, Plugin plugin );

    /**
     * Supprime une catgories de dlibration de la liste des catgories.
     * @param categorieDeliberation la catgorie  supprimer
     * @param plugin le plugin actif
     * @throws AppException si la catgorie de dlibration est rfrence par un PDD
     */
    void remove( CategorieDeliberation categorieDeliberation, Plugin plugin )
        throws AppException;

    /**
     * Met  jour une catgorie de dlibration
     * @param categorieDeliberation la catgorie  modifier
     * @param plugin le plugin actif
     */
    void update( CategorieDeliberation categorieDeliberation, Plugin plugin );

    /**
     * Retourne la liste de toutes les catgories de dlibration.
     * @param plugin le plugin actif
     * @return la liste des catgories de dlibration
     */
    List<CategorieDeliberation> findCategorieDeliberationList( Plugin plugin );

    /**
     * Indique si le code pass en argument existe dj dans la table ods_categorie_deliberation;
     * Exclut de la recherche la catgorie associe  exceptIdCategorie
     * @param exceptCategorie la categorie  exclure (pas d'exclusion si =null)
     * @param strCode le code recherch
     * @param plugin le plugin
     * @return true si le code est dj utilis, sinon false
     */
    boolean containsCode( CategorieDeliberation exceptCategorie, String strCode, Plugin plugin );

    /**
     * Retourne l'id de la catgorie correspondant au code pass en paramtre
     * @param nCodeCategorie le code de la ctagorie
     * @param plugin le plugin
     * @return l'id de la catgorie
     */
    int findIdCategorieByCode( int nCodeCategorie, Plugin plugin );

    /**
     * Retourne la catgorie correspondant au code pass en paramtre
     * @param nCodeCategorie le code de la ctagorie
     * @param plugin le plugin
     * @return la catgorie
     */
    CategorieDeliberation findCategorieByCode( int nCodeCategorie, Plugin plugin );

    /**
     * Retourne la liste des catgories de dlibration actives.
     * @param plugin le plugin
     * @return la liste des catgories de dlibration actives
     */
    List<CategorieDeliberation> findAllCategoriesActives( Plugin plugin );

    /**
     * Retourne la liste des catgories actives, tries par ordre alphabtique des libells
     * @param plugin le plugin
     * @return la liste ordonne par libell
     */
    List<CategorieDeliberation> findAllActivesOrderByLibelle( Plugin plugin );

    /**
     * Retourne la liste des codes des catgories de type march.
     * @param plugin le plugin
     * @return la liste des codes des catgories de type march
     */
    List<String> findAllCodesCategoriesExportParisFr( Plugin plugin );
}
