/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.deliberationdesignation;

import fr.paris.lutece.plugins.ods.business.IBusinessHome;
import fr.paris.lutece.plugins.ods.business.acte.ActeHome;
import fr.paris.lutece.plugins.ods.business.fichier.IFichierHome;
import fr.paris.lutece.plugins.ods.business.formationconseil.FormationConseilHome;
import fr.paris.lutece.plugins.ods.dto.acte.Acte;
import fr.paris.lutece.plugins.ods.dto.acte.IActeFilter;
import fr.paris.lutece.plugins.ods.dto.deliberationdesignation.DeliberationDesignationFilter;
import fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter;
import fr.paris.lutece.plugins.ods.dto.fichier.ITypeDocument;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.typeentite.ITypeEntite;
import fr.paris.lutece.plugins.ods.service.acte.IActeService;
import fr.paris.lutece.portal.service.plugin.Plugin;

import org.springframework.beans.factory.annotation.Autowired;

import java.sql.Timestamp;

import java.util.List;


/**
 * 
 * AbstractDeliberationDesignationHome
 *
 * @param <GSeance> ISeance
 * @param <GFichier> IFichier
 * @param <GDeliberatioDesignation> IDeliberationDesignation
 * @param <GActeFilter> IActeFilter
 */
public abstract class AbstractDeliberationDesignationHome<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>, GDeliberatioDesignation extends IDeliberationDesignation<GSeance, GFichier>, GActeFilter extends IActeFilter<GFichier, GSeance>>
    implements IBusinessHome, IDeliberationDesignationHome<GSeance, GFichier, GDeliberatioDesignation, GActeFilter>
{
    @Autowired
    private IDeliberationDesignationDAO<GSeance, GFichier, GDeliberatioDesignation> _deliberationDesignationDAO;
    @Autowired
    private ActeHome<GSeance, GFichier, GActeFilter> _acteHome;
    @Autowired
    private IFichierHome<GFichier, IFichierFilter, ITypeDocument, GSeance> _fichierHome;
    @Autowired
    private FormationConseilHome _formationConseilHome;
    @Autowired
    private IActeService<GSeance, GFichier, GActeFilter> _acteService;

    /**
     * Renvoie l'acte courant
     * @param listActes la liste des actes
     * @return l'acte courant
     */
    protected abstract Acte<GSeance, GFichier> getActeCourant( List<Acte<GSeance, GFichier>> listActes );

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.business.deliberationdesignation.IDeliberationDesignationHome#create(GDeliberatioDesignation, fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public int create( GDeliberatioDesignation deliberation, Plugin plugin )
    {
        deliberation.setId( _deliberationDesignationDAO.insert( deliberation, plugin ) );

        return deliberation.getId(  );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.business.deliberationdesignation.IDeliberationDesignationHome#update(GDeliberatioDesignation, fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public void update( GDeliberatioDesignation deliberation, Plugin plugin )
    {
        _deliberationDesignationDAO.store( deliberation, plugin );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.business.deliberationdesignation.IDeliberationDesignationHome#remove(GDeliberatioDesignation, fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public void remove( GDeliberatioDesignation deliberation, Plugin plugin )
    {
        _deliberationDesignationDAO.delete( deliberation, plugin );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.business.deliberationdesignation.IDeliberationDesignationHome#findByPrimaryKey(int, fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public GDeliberatioDesignation findByPrimaryKey( int nKey, Plugin plugin )
    {
        GDeliberatioDesignation deliberation = _deliberationDesignationDAO.load( nKey, plugin );

        if ( deliberation != null )
        {
            List<Acte<GSeance, GFichier>> listActes = getListeActes( nKey, plugin );
            deliberation.setActes( listActes );

            if ( listActes != null )
            {
                deliberation.setActeCourant( getActeCourant( listActes ) );
            }

            setOtherPropertyDeliberation( deliberation, plugin );
        }

        return deliberation;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.business.deliberationdesignation.IDeliberationDesignationHome#findByFilter(fr.paris.lutece.plugins.ods.dto.deliberationdesignation.DeliberationDesignationFilter, fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public List<GDeliberatioDesignation> findByFilter( DeliberationDesignationFilter filter, Plugin plugin )
    {
        List<GDeliberatioDesignation> listDeliberationsDesignation = _deliberationDesignationDAO.selectByFilter( filter,
                plugin );

        for ( IDeliberationDesignation<GSeance, GFichier> deliberation : listDeliberationsDesignation )
        {
            List<Acte<GSeance, GFichier>> listActes = getListeActes( deliberation.getId(  ), plugin );
            deliberation.setActes( listActes );

            if ( deliberation.getFormationConseil(  ) != null )
            {
                FormationConseil formationConseil = _formationConseilHome.findByPrimaryKey( deliberation.getFormationConseil(  )
                                                                                                        .getIdFormationConseil(  ),
                        plugin );
                deliberation.setFormationConseil( formationConseil );
            }
        }

        return listDeliberationsDesignation;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.business.deliberationdesignation.IDeliberationDesignationHome#findByFichier(int, fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public GDeliberatioDesignation findByFichier( int nKey, Plugin plugin )
    {
        return _deliberationDesignationDAO.selectByFichier( nKey, plugin );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.business.deliberationdesignation.IDeliberationDesignationHome#publication(int, java.sql.Timestamp, int, boolean, fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public void publication( int nKey, Timestamp datePublication, int version, boolean isPublier, Plugin plugin )
    {
        _deliberationDesignationDAO.publication( nKey, datePublication, version, isPublier, plugin );
    }

    /**
     * Permet de complter la dlibration avec les proprits qui ont des contraintes sur d'autres tables
     * @param deliberation deliberation  complter  partir des informations du daoUtil
     * @param plugin Plugin
     */
    private void setOtherPropertyDeliberation( IDeliberationDesignation<GSeance, GFichier> deliberation, Plugin plugin )
    {
        if ( deliberation.getFichierOr(  ) != null )
        {
            GFichier fichier = _fichierHome.findByPrimaryKey( deliberation.getFichierOr(  ).getId(  ), plugin );
            deliberation.setFichierOr( fichier );
        }

        if ( deliberation.getFichierSc(  ) != null )
        {
            GFichier fichier = _fichierHome.findByPrimaryKey( deliberation.getFichierSc(  ).getId(  ), plugin );
            deliberation.setFichierSc( fichier );
        }
    }

    /**
     * retourne la liste des actes rattachs  la liste
     * @param nIdEntite l'identifiant de l'entite
     * @param plugin le plugin
     * @return la liste des identifiants des actes
     */
    private List<Acte<GSeance, GFichier>> getListeActes( int nIdEntite, Plugin plugin )
    {
        GActeFilter acteFilter = _acteService.getActeFilter(  );
        acteFilter.setIdEntite( nIdEntite );
        acteFilter.setIdTypeEntite( ITypeEntite.DESIGNATIONS_DE_REPRESENTANTS );

        List<Acte<GSeance, GFichier>> listActes = _acteHome.findByFilter( acteFilter, plugin );

        return listActes;
    }
}
