/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.deliberationdesignation;

import fr.paris.lutece.plugins.ods.dto.deliberationdesignation.DeliberationDesignationFilter;
import fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.portal.service.plugin.Plugin;

import java.sql.Timestamp;

import java.util.List;


/**
 * IDeliberationDesignationDAO
 *
 * @param <GSeance> seance
 * @param <GFichier> fichier
 * @param <GDeliberatioDesignation> DeliberationDesignation
 */
public interface IDeliberationDesignationDAO<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>, GDeliberatioDesignation extends IDeliberationDesignation<GSeance, GFichier>>
{
    /**
     * Cre une nouvelle dlibration  partir de lobjet deliberation pass en paramtre
     * @param deliberation la deliberation  insrer
     * @param plugin le Plugin actif
     * @return int primaryKey
     */
    int insert( GDeliberatioDesignation deliberation, Plugin plugin );

    /**
     * Modifie la deliberation correspondant  lobjet deliberation pass en paramtre
     * @param deliberation la deliberation  modifier
     * @param plugin le Plugin actif
     */
    void store( GDeliberatioDesignation deliberation, Plugin plugin );

    /**
     * Supprime la deliberation correspondant  lobjet deliberation pass en paramtre
     * @param deliberation la deliberation  supprimer
     * @param plugin le Plugin actif
     */
    void delete( GDeliberatioDesignation deliberation, Plugin plugin );

    /**
     * renvoie la deliberation ayant comme identifiant nKey
     * @param nKey idenfiant de la deliberation  supprimer
     * @param plugin le Plugin actif
     * @return une deliberation
     */
    GDeliberatioDesignation load( int nKey, Plugin plugin );

    /**
     * rcupre la liste des deliberations correspondant au filtre donn
     * @param filter le filtre
     * @param plugin le Plugin actif
     * @return uen liste de deliberations
     */
    List<GDeliberatioDesignation> selectByFilter( DeliberationDesignationFilter filter, Plugin plugin );

    /**
     * renvoie la dlibration de dsignation ayant comme identifiant de fichier nKey
     * @param nKey idenfiant du fichier
     * @param plugin le Plugin actif
     * @return la dlibration de dsignation
     */
    GDeliberatioDesignation selectByFichier( int nKey, Plugin plugin );

    /**
     * Permet de publier ou de dpublier unedeliberation
     * Publication d'unz deliberation
     * Si isPublier gale  TRUE, alors la mthode <b>publie</b> la deliberation et incrmente la version<BR>
     * Sinon la mthode change le statut enLigne d'une deliberation a FALSE
     * @param nKey id de la deliberation
     * @param tsDatePublication TimeStamp date de publication
     * @param version version
     * @param isPublier enLigne
     * @param plugin le Plugin actif
     */
    void publication( int nKey, Timestamp tsDatePublication, int version, boolean isPublier, Plugin plugin );
}
