/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.elu;

import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;

import java.util.List;


/**
 *
 * IEluHome
 *
 * @param <GElu> IElu
 */
public interface IEluHome<GElu extends IElu>
{
    /**
     * Cre un nouvel lu   partir de lobjet elu pass en paramtre
     * @param elu  l'lu   insrer
     * @param plugin plugin
     * @return l'id de l'lu nouvellement cr
     */
    int create( GElu elu, Plugin plugin );

    /**
     * Modifie  l'lu correspondant  lobjet elu pass en paramtre
     * @param elu l'lu  modifier
     * @param plugin plugin
     */
    void update( GElu elu, Plugin plugin );

    /**
     * Supprime l'lu correspondant   lobjet elu pass en paramtre
     * @param elu l'lu  supprimer
     * @param plugin plugin
     * @throws AppException si l'lu est rfrenc par un objet tiers
     */
    void remove( GElu elu, Plugin plugin ) throws AppException;

    /**
     * Retourne l'lu  ayant comme identifiant nKey
     * @param nKey l'identifiant de l'lu
     * @param plugin plugin
     * @return l'lu  ayant comme identifiant nKey
     */
    GElu findByPrimaryKey( int nKey, Plugin plugin );

    /**
     * Retourne la liste des lus
     * @param plugin plugin
     * @return liste d'objets Elu
     */
    List<GElu> findEluList( Plugin plugin );

    /**
     * Retourne la liste des lus ayant un groupe politique dont l'identifiant est gal  nKey
     * @param nKey  identifiant du groupe politique
     * @param plugin plugin
     * @return la liste des lus ayant un groupe politique dont l'identifiant est gal  nKey
     */
    List<GElu> findEluListbyIdGroupe( int nKey, Plugin plugin );

    /**
     * Retourne la liste des lus dont le statut est actif.
     * @param plugin plugin
     * @return la liste de lus actifs
     */
    List<GElu> findElusActifs( Plugin plugin );

    /**
     * Retourne la liste des lus remplaants d'un autre lu
     * @param plugin le plugin
     * @return la liste des lus remplaants d'un autre lu
     */
    List<GElu> findEluRemplacantsActifs( Plugin plugin );

    /**
     * Retourne la liste des lus rapporteurs dans une commission
     * @param plugin le plugin
     * @return la liste des rapporteurs
     */
    List<GElu> findElusRapporteursActifs( Plugin plugin );

    /**
     * Retourne la liste de tous les rapporteurs
     * @param plugin le plugin
     * @return la liste de tous les rapporteurs
     */
    List<GElu> findAllRapporteurs( Plugin plugin );
}
