/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.fichier;

import fr.paris.lutece.plugins.ods.business.IBusinessHome;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter;
import fr.paris.lutece.plugins.ods.dto.fichier.ITypeDocument;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.portal.service.plugin.Plugin;

import org.springframework.beans.factory.annotation.Autowired;

import java.sql.Timestamp;

import java.util.List;


/**
 *
 * AbstractFichierHome
 *
 * @param <GFichier> IFichier
 * @param <GFichierFilter> IFichierFilter
 * @param <GTypeDocument> ITypeDocument
 * @param <GFichierDAO> IFichierDAO
 * @param <GSeance> ISeance
 */
public abstract class AbstractFichierHome<GFichier extends IFichier<GSeance, GFichier>, GFichierFilter extends IFichierFilter, GTypeDocument extends ITypeDocument, GFichierDAO extends IFichierDAO<GFichier, GFichierFilter, GTypeDocument, GSeance>, GSeance extends ISeance>
    implements IBusinessHome, IFichierHome<GFichier, GFichierFilter, GTypeDocument, GSeance>
{
    @Autowired
    protected GFichierDAO _fichierDAO;

    /**
     * Cre une nouvelle instance du filtre
     * @return une nouvelle instance du filtre
     */
    public abstract GFichierFilter newFichierFilterInstance(  );

    /**
     * Cre un nouveau fichier  partir de lobjet fichier pass en paramtre
     * @param fichier le fichier  insrer
     * @param plugin le Plugin actif
     * @return int primaryKey
     */
    public int create( GFichier fichier, Plugin plugin )
    {
        fichier.setId( _fichierDAO.insert( fichier, plugin ) );

        return fichier.getId(  );
    }

    /**
     * Modifie le fichier correspondant  lobjet fichier pass en paramtre
     * @param fichier le fichier  modifier
     * @param plugin le Plugin actif
     */
    public void update( GFichier fichier, Plugin plugin )
    {
        _fichierDAO.store( fichier, plugin );
    }

    /**
     * Supprime le fichier correspondant  lobjet fichier pass en paramtre
     * @param fichier le fichier  supprimer
     * @param plugin le Plugin actif
     */
    public void remove( GFichier fichier, Plugin plugin )
    {
        _fichierDAO.delete( fichier, plugin );
    }

    /**
     * renvoie le Fichier ayant comme identifiant nKey
     * @param nKey idenfiant du Fichier  supprimer
     * @param plugin le Plugin actif
     * @return un fichier
     */
    public GFichier findByPrimaryKey( int nKey, Plugin plugin )
    {
        return _fichierDAO.load( nKey, plugin );
    }

    /**
     * rcupre la liste des fichiers correspondant au filtre donn
     * @param filter le filtre
     * @param plugin le Plugin actif
     * @return uen liste de fichiers
     */
    public List<GFichier> findByFilter( GFichierFilter filter, Plugin plugin )
    {
        return _fichierDAO.selectByFilter( filter, plugin );
    }

    /**
     * renvoie la liste des Types de documents
     * @param plugin le Plugin actif
     * @return liste d'objets TypeDocument
     */
    public List<GTypeDocument> findTypeDocumentsList( Plugin plugin )
    {
        return _fichierDAO.loadTypeDocumentList( plugin );
    }

    /**
     * renvoie la liste des Types de documents pour la prochaine sance
     * @param plugin le Plugin actif
     * @return liste d'objets TypeDocument
     */
    public List<GTypeDocument> findTypeDocumentsProchaineSeanceList( Plugin plugin )
    {
        return _fichierDAO.loadTypeDocumentProchaineSeanceList( plugin );
    }

    /**
     * renvoie la liste des Types de documents aval
     * @param plugin le Plugin actif
     * @return liste d'objets TypeDocument
     */
    public List<GTypeDocument> findTypeDocumentsAvalList( Plugin plugin )
    {
        return _fichierDAO.loadTypeDocumentAvalList( plugin );
    }

    /**
     * Retourne le type de document en fonction de l'id pass en parametre
     * @param nKey id dy type de document que l'on souhaite
     * @param plugin le Plugin actif
     * @return TypeDocument
     */
    public GTypeDocument findTypeDocumentsById( int nKey, Plugin plugin )
    {
        return _fichierDAO.loadTypeDocumentsById( nKey, plugin );
    }

    /**
     * Permet de publier ou de dpublier un fichier
     * Publication d'un fichier
     * Si isPublier gale  TRUE, alors la mthode <b>publie</b> le fichier et incrmente la version<BR>
     * Sinon la mthode change le statut enLigne d'un fichier a FALSE
     * @param nKey id du fichier
     * @param datePublication TimeStamp date de publication
     * @param version verssion
     * @param isPublier enLigne
     * @param plugin le Plugin actif
     */
    public void publication( int nKey, Timestamp datePublication, int version, boolean isPublier, Plugin plugin )
    {
        _fichierDAO.publication( nKey, datePublication, version, isPublier, plugin );
    }
}
