/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.historique;

import fr.paris.lutece.plugins.ods.dto.historique.Historique;
import fr.paris.lutece.plugins.ods.dto.historique.IEntiteHistorique;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.util.sql.DAOUtil;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;


/**
 *
 * HistoriqueDAO est responsable de la communication avec la BDD
 *
 */
public class HistoriqueDAO implements IHistoriqueDAO
{
    private static final String SQL_QUERY_NEW_PK = " SELECT max( id_historique ) FROM ods_historique ";
    private static final String SQL_QUERY_HISTORIQUES_LIST = "SELECT id_historique, version, date_publication FROM ods_historique where id_entite = ? AND id_type_entite = ? ";
    private static final String SQL_QUERY_DELETE = " DELETE FROM ods_historique WHERE id_entite = ? AND id_type_entite = ?  ";
    private static final String SQL_QUERY_INSERT = " INSERT INTO ods_historique (id_historique, version, date_publication, id_entite, id_type_entite) VALUES ( ?, ?, ?, ?, ?) ";

    /**
     * Gnre un nouvel identifiant
     * @param plugin Plugin
     * @return int Retourne lidentifiant gnr
     */
    int newPrimaryKey( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey;

        if ( !daoUtil.next(  ) )
        {
            nKey = 1;
        }

        Random random = new Random(  );
        nKey = daoUtil.getInt( 1 ) + 1 + random.nextInt( 99 );
        daoUtil.free(  );

        return nKey;
    }

    /**
     * Insere l'objet historique en base
     * @param historique Historique
     * @param plugin Plugin
     */
    public void insert( Historique historique, Plugin plugin )
    {
        int newPrimaryKey = newPrimaryKey( plugin );
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_INSERT, plugin );
        historique.setId( newPrimaryKey );
        daoUtil.setInt( 1, historique.getId(  ) );
        daoUtil.setInt( 2, historique.getVersion(  ) );
        daoUtil.setTimestamp( 3, historique.getDatePublication(  ) );

        if ( historique.getIdEntite(  ) != -1 )
        {
            daoUtil.setInt( 4, historique.getIdEntite(  ) );
        }
        else
        {
            daoUtil.setIntNull( 4 );
        }

        if ( historique.getIdTypeEntite(  ) != -1 )
        {
            daoUtil.setInt( 5, historique.getIdTypeEntite(  ) );
        }
        else
        {
            daoUtil.setIntNull( 5 );
        }

        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Retourne une liste d'historiques concernant l'entite pass en paramtre
     *
     * @param entite l'entite
     * @param plugin Plugin
     * @return List<Historique> liste d'historiques
     */
    public List<Historique> loadList( IEntiteHistorique entite, Plugin plugin )
    {
        List<Historique> listHistoriques = new ArrayList<Historique>(  );

        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_HISTORIQUES_LIST, plugin );
        daoUtil.setInt( 1, entite.getId(  ) );
        daoUtil.setInt( 2, entite.getTypeEntite(  ) );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            Historique historique = new Historique(  );
            historique.setId( daoUtil.getInt( 1 ) );
            historique.setVersion( daoUtil.getInt( 2 ) );
            historique.setDatePublication( daoUtil.getTimestamp( 3 ) );

            listHistoriques.add( historique );
        }

        daoUtil.free(  );

        return listHistoriques;
    }

    /**
     * Supprime tous les historiques concernant l'entite pass en paramtre
     *
     * @param entite l'entit en question
     * @param plugin Plugin
     */
    public void delete( IEntiteHistorique entite, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE, plugin );
        daoUtil.setInt( 1, entite.getId(  ) );
        daoUtil.setInt( 2, entite.getTypeEntite(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }
}
