/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.historique;

import fr.paris.lutece.plugins.ods.dto.historique.Historique;
import fr.paris.lutece.plugins.ods.dto.historique.IEntiteHistorique;
import fr.paris.lutece.portal.service.plugin.Plugin;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;


/**
 *
 * HistoriqueHome
 */
public final class HistoriqueHome
{
    @Autowired
    private IHistoriqueDAO _historiqueDAO;

    /**
     * Constructeur vide
     */
    private HistoriqueHome(  )
    {
    }

    /**
         * Insere l'objet historique en base
         * @param historique Historique
         * @param plugin Plugin
         */
    public void create( Historique historique, Plugin plugin )
    {
        _historiqueDAO.insert( historique, plugin );
    }

    /**
     * Retourne une liste d'historiques concernant l'entite pass en paramtre
     *
     * @param entite l'entite
     * @param plugin Plugin
     * @return List<Historique> liste d'historiques
     */
    public List<Historique> getHistoriqueList( IEntiteHistorique entite, Plugin plugin )
    {
        return _historiqueDAO.loadList( entite, plugin );
    }

    /**
     * Retourne une liste d'historiques concernant le fichier pass en paramtre
     *
     * @param fichier Fichier
     * @param plugin Plugin
     * @return List<Historique> liste d'historiques
     */

    /*  public List<Historique> getHistoriqueList( Fichier fichier, Plugin plugin )
      {
          return historiqueDAO.loadByFichier( fichier, plugin );
      }*/

    /**
     * Retourne une liste d'historiques concernant le pdd pass en paramtre
     *
     * @param pdd PDD
     * @param plugin Plugin
     * @return List<Historique> liste d'historiques
     */

    /* public List<Historique> getHistoriqueList( PDD pdd, Plugin plugin )
     {
         return historiqueDAO.loadByPDD( pdd, plugin );
     }
    
     /**
      * Retourne une liste d'historiques concernant le voeuAmendement pass en paramtre
      *
      * @param voeuAmendement VoeuAmendement
      * @param plugin Plugin
      * @return List<Historique> liste d'historiques
      */

    /* public List<Historique> getHistoriqueList( VoeuAmendement voeuAmendement, Plugin plugin )
    {
        return historiqueDAO.loadByVoeuAmendement( voeuAmendement, plugin );
    }
    
    /**
     * Retourne une liste d'historiques concernant la dlibrtion de dsignation passe en paramtre
     *
     * @param deliberation DeliberationDesignation
     * @param plugin Plugin
     * @return List<Historique> liste d'historiques
     */

    /* public List<Historique> getHistoriqueList( DeliberationDesignation deliberation, Plugin plugin )
    {
        return historiqueDAO.loadByDeliberation( deliberation, plugin );
    }
    
    /**
     * Retourne une liste d'historiques concernant l'arret passe en paramtre
     *
     * @param arrete Arrete
     * @param plugin Plugin
     * @return List<Historique> liste d'historiques
     */

    /* public List<Historique> getHistoriqueList( Arrete arrete, Plugin plugin )
    {
        return historiqueDAO.loadByArrete( arrete, plugin );
    }
    
    /**
     * Retourne une liste d'historiques concernant la liste de membre prsent passe en paramtre
     *
     * @param listeMembrePresent ListeMembrePresent
     * @param plugin Plugin
     * @return List<Historique> liste d'historiques
     */

    /* public List<Historique> getHistoriqueList( ListeMembrePresent listeMembrePresent, Plugin plugin )
    {
        return historiqueDAO.loadByListeMembrePresent( listeMembrePresent, plugin );
    }*/

    /**
    * Supprime tous les historiques concernant l'entite pass en paramtre
    *
    * @param entite l'entite
    * @param plugin Plugin
    */
    public void remove( IEntiteHistorique entite, Plugin plugin )
    {
        _historiqueDAO.delete( entite, plugin );
    }

    /**
     * Supprime tous les historiques concernant le fichier pass en paramtre
     *
     * @param fichier Fichier
     * @param plugin Plugin
     */

    /* public void remove( Fichier fichier, Plugin plugin )
     {
         historiqueDAO.delete( fichier, plugin );
     }
    
     /**
      * Supprime tous les historiques concernant la dlibration passe en paramtre
      *
      * @param deliberation Dliberation
      * @param plugin Plugin
      */

    /*  public void remove( DeliberationDesignation deliberation, Plugin plugin )
    {
        historiqueDAO.delete( deliberation, plugin );
    }
    
    /**
     * Supprime tous les historiques concernant l'arret pass en paramtre
     *
     * @param arrete Arrete
     * @param plugin Plugin
     */

    /* public void remove( Arrete arrete, Plugin plugin )
    {
        historiqueDAO.delete( arrete, plugin );
    }
    
    /**
     * Supprime tous les historiques concernant le pdd pass en paramtre
     *
     * @param pdd PDD
     * @param plugin Plugin
     */

    /* public void remove( PDD pdd, Plugin plugin )
    {
        historiqueDAO.delete( pdd, plugin );
    }
    
    /**
     * Supprime tous les historiques concernant le voeuAmendement pass en paramtre
     *
     * @param voeuAmendement VoeuAmendement
     * @param plugin Plugin
     */

    /* public void remove( VoeuAmendement voeuAmendement, Plugin plugin )
    {
        historiqueDAO.delete( voeuAmendement, plugin );
    }
    
    /**
     * Supprime tous les historiques concernant la liste des membres prsents pass en paramtre
     *
     * @param listeMembrePresent liste de membre prsent
     * @param plugin Plugin
     */

    /*  public void remove( ListeMembrePresent listeMembrePresent, Plugin plugin )
    {
        historiqueDAO.delete( listeMembrePresent, plugin );
    }*/
}
