/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.horodatage;

import fr.paris.lutece.plugins.ods.dto.horodatage.HorodatageActionEnum;
import fr.paris.lutece.plugins.ods.dto.horodatage.IEntiteHorodatage;
import fr.paris.lutece.plugins.ods.dto.horodatage.ITrace;
import fr.paris.lutece.portal.service.plugin.Plugin;

import java.sql.Timestamp;

import java.util.List;


/**
 *
 * IHorodatageDAO
 *
 */
public interface IHorodatageDAO
{
    /**
     * Permet de trac la publication ou la dpublication d'un pdd
     *
     * @param entite l'entite
     * @param dateSeance Date du premier jour de la sance
     * @param action Type d'action effectue (publication ou dpublication)
     * @param signature Liste des signatures numriques SHA224 des fichiers (les signatures sont spares par le caractre '-')
     * @param plugin le plugin actif
     * @return id qui a ete gnr
     */
    int insertTracePublication( IEntiteHorodatage entite, Timestamp dateSeance, HorodatageActionEnum action,
        String signature, Plugin plugin );

    /**
     * Permet de trac l'envoie de la notification
     *
     * @param objet Objet du mail
     * @param contenu Contenu du message
     * @param signature Signature SHA224 du contenu du mail
     * @param typeNotif le type de notification (F = Fichier, P = PDD, R = Requetes personnelles)
     * @param plugin le plugin actif
     * @return id qui a ete gnr
     */
    int insertTraceNotification( String objet, String contenu, String signature, String typeNotif, Plugin plugin );

    /**
     * Permet de trac l'envoie de la notification des destinataires
     *
     * @param idTraceNotification Cl trangre vers le champ id_trace de la table ods_horodatage_notifications
     * @param idDestinataire Cl trangre vers le champ id_utilisateur de la table ods_utilisateur
     * @param dateEnvoi Date d'envoi du mail de notification
     * @param mails Liste des destinataires (spars par le caractre ';') lis  un mme utilisateur
     * @param plugin le plugin actif
     */
    void insertTraceNotificationDestinaire( int idTraceNotification, String idDestinataire, Timestamp dateEnvoi,
        String mails, Plugin plugin );

    /**
     * Retourne la date de na dernire notification
     * @param plugin plugin
     * @return la date de na dernire notification
     */
    Timestamp selectLastNotification( Plugin plugin );

    /**
     * Retourne une liste de traces dont la date d'ajout est comprise entre dateStart et dateEnd et dont le type est typeHorodatage
     *
     * @param plugin le plugin actif
     * @param typeHorodatage Type PUBLICATION ou NOTIFICATION
     * @param dateStart Filtre sur la date de dbut des traces
    * @param dateEnd Filtre sur la date de fin des traces
     * @return Liste de traces
     */
    List<ITrace> selectAllTraces( Plugin plugin, String typeHorodatage, Timestamp dateStart, Timestamp dateEnd );
}
