/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.indexer;

import fr.paris.lutece.plugins.ods.dto.indexer.Indexer;
import fr.paris.lutece.portal.service.plugin.Plugin;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;


/**
 * IndexerHome : fait appel au DAO pour grer les serveurs d'indexation
 */
public final class IndexerHome
{
    @Autowired
    private IIndexerDAO _indexerDAO;

    /**
     * Constructeur vide
     *
     */
    private IndexerHome(  )
    {
    }

    /**
     * Enregistre un nouvel indexer dans la base
     * @param indexer l'indexer  insrer
     * @param plugin le plugin
     */
    public void create( Indexer indexer, Plugin plugin )
    {
        _indexerDAO.insert( indexer, plugin );
    }

    /**
     * Supprime un indexer de la base
     * @param indexer l'indexer  supprimer
     * @param plugin le plugin
     */
    public void remove( Indexer indexer, Plugin plugin )
    {
        _indexerDAO.delete( indexer, plugin );
    }

    /**
     * Modifie un indexer
     * @param indexer l'indexer modifi
     * @param plugin le plugin
     */
    public void update( Indexer indexer, Plugin plugin )
    {
        _indexerDAO.store( indexer, plugin );
    }

    /**
     * Charge l'indexer identifi par nKey
     * @param nKey l'identifiant de l'indexer
     * @param plugin le plugin
     * @return l'indexer identifi par nKey
     */
    public Indexer findByPrimaryKey( int nKey, Plugin plugin )
    {
        return _indexerDAO.load( nKey, plugin );
    }

    /**
     * Retourne l'indexer dont le nom est strNomIndexer
     * @param strNomIndexer le nom de l'indexer recherch
     * @param plugin le plugin
     * @return l'indexer dont le nom est strNomIndexer
     */
    public Indexer findByName( String strNomIndexer, Plugin plugin )
    {
        return _indexerDAO.selectByNomIndexer( strNomIndexer, plugin );
    }

    /**
     * Retourne l'indexer dont le nom est strNomIndexer
     * @param nIdSeance l'identifiant de la sance
     * @param plugin le plugin
     * @return les indexer dont la sance a pour identifiant nIdSeance
     */
    public List<Indexer> findBySeance( int nIdSeance, Plugin plugin )
    {
        return _indexerDAO.selectByIdSeance( nIdSeance, plugin );
    }

    /**
     * Retourne la liste des indexers
     * @param plugin le plugin
     * @return la liste des indexers
     */
    public List<Indexer> findAllIndexers( Plugin plugin )
    {
        return _indexerDAO.selectAllIndexer( plugin );
    }
}
