/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.listemembrepresent;

import fr.paris.lutece.plugins.ods.dto.acte.StatutActeEnum;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.listemembrepresent.ListeMembrePresent;
import fr.paris.lutece.plugins.ods.dto.listemembrepresent.ListeMembrePresentFilter;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.typeentite.ITypeEntite;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.util.sql.DAOUtil;

import java.sql.Timestamp;

import java.util.ArrayList;
import java.util.List;


/**
 * Cette classe permet d'accder/grer la table ods_fichier
 *
 * @param <GFichier> IFichier
 * @param <GSeance> ISeance
 */
public abstract class AbstractListeMembrePresentDAO<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>>
    implements IListeMembrePresentDAO<GSeance, GFichier>
{
    private static final String SQL_QUERY_FIND_BY_PRIMARY_KEY = " SELECT lmp.id_liste,lmp.id_seance, lmp.id_formation_conseil,  " +
        " lmp.intitule, lmp.id_fichier_sc,lmp.date_publication, lmp.date_transmission_ctrl_legalite, " +
        "lmp.en_ligne, lmp.version, lmp.id_fichier_genere, fc.taille FROM ods_liste_membres_presents lmp, ods_seance seance, ods_fichier fc " +
        " WHERE lmp.id_liste = ? AND lmp.id_seance = seance.id_seance AND lmp.id_fichier_genere = fc.id_document";
    private static final String SQL_QUERY_LISTE = " SELECT lmp.id_liste,lmp.id_seance, lmp.id_formation_conseil,  " +
        " lmp.intitule, lmp.id_fichier_sc,lmp.date_publication, lmp.date_transmission_ctrl_legalite, " +
        "lmp.en_ligne, lmp.version, lmp.id_fichier_genere, fc.taille FROM ods_liste_membres_presents lmp LEFT JOIN ods_fichier fc ON fc.id_document=lmp.id_fichier_genere";
    private static final String SQL_QUERY_NEW_PK = " SELECT max( id_liste ) FROM ods_liste_membres_presents ";
    private static final String SQL_QUERY_INSERT = " INSERT INTO ods_liste_membres_presents " +
        "(id_liste,id_seance, id_formation_conseil,intitule, id_fichier_sc,date_publication, date_transmission_ctrl_legalite, " +
        "en_ligne, version, id_fichier_genere )VALUES(?,?,?,?,?,?,?,?,?,?) ";
    private static final String SQL_QUERY_INSERT_ELU = " INSERT INTO ods_elu_present (id_liste, id_elu) VALUES (?, ?)";
    private static final String SQL_QUERY_UPDATE = " UPDATE ods_liste_membres_presents SET " +
        " id_seance=?, id_formation_conseil=?,intitule=?, id_fichier_sc=?,date_publication=?, date_transmission_ctrl_legalite=?, " +
        "en_ligne=?,version=?,id_fichier_genere=? WHERE id_liste=? ";
    private static final String SQL_QUERY_DELETE = " DELETE FROM ods_liste_membres_presents WHERE id_liste = ?  ";
    private static final String SQL_QUERY_LISTE_ELUS_PRESENTS = "SELECT elp.id_elu, el.nom_elu FROM ods_elu_present elp, ods_elu el " +
        " WHERE elp.id_liste=? AND elp.id_elu = el.id_elu ORDER BY el.nom_elu";
    private static final String SQL_QUERY_DELETE_ALL_ELUS = " DELETE FROM ods_elu_present WHERE id_liste = ?  ";
    private static final String SQL_FILTER_SEANCE = " AND lmp.id_seance = ? ";
    private static final String SQL_FILTER_FORMATION_CONSEIL = " AND lmp.id_formation_conseil = ? ";
    private static final String SQL_FILTER_ID_LISTE = " AND lmp.id_liste <> ? ";
    private static final String SQL_FILTER_PUBLICATION = " AND lmp.en_ligne = ? ";
    private static final String SQL_QUERY_ORDER_BY_DATE_MODIFICATION = " ORDER BY lmp.date_publication DESC ";
    private static final String SQL_QUERY_PUBLIER = " UPDATE ods_liste_membres_presents SET date_publication = ?, en_ligne = ?, version = ? WHERE id_liste = ?  ";
    private static final String SQL_WHERE_1_1 = " WHERE 1=1";
    private static final String SQL_FILTER_STATUT_ACTE = "AND ( SELECT count(*) FROM ods_acte WHERE id_type_entite = ? AND id_entite=lmp.id_liste AND id_statut<>? ) ";
    private static final String SQL_FILTER_STATUT_A_PREPARER = " = 0";
    private static final String SQL_FILTER_STATUT_UN_ACTE_AU_MOINS = " >= 1";

    /**
     * Cre une nouvelle instance de Fichier
     * @return une nouvelle instance de Fichier
     */
    protected abstract GFichier newFichierInstance(  );

    /**
     * Cre une nouvelle instance de Seance
     * @return une nouvelle instance de Sance
     */
    protected abstract GSeance newSeanceInstance(  );

    /**
     * Cre une nouvelle liste de membres prsents   partir de l'objet listeMembrePresent pass en paramtre
     * @param listeMembrePresent la liste des membres prsents  insrer
     * @param plugin le plugin
     * @return int primaryKey
     */
    public int insert( ListeMembrePresent<GSeance, GFichier> listeMembrePresent, Plugin plugin )
    {
        int newPrimaryKey = newPrimaryKey( plugin );
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_INSERT, plugin );
        listeMembrePresent.setId( newPrimaryKey );
        daoUtil.setInt( 1, listeMembrePresent.getId(  ) );

        if ( listeMembrePresent.getSeance(  ) != null )
        {
            daoUtil.setInt( 2, listeMembrePresent.getSeance(  ).getIdSeance(  ) );
        }
        else
        {
            daoUtil.setIntNull( 2 );
        }

        if ( listeMembrePresent.getFormationConseil(  ) == null )
        {
            daoUtil.setIntNull( 3 );
        }
        else
        {
            daoUtil.setInt( 3, listeMembrePresent.getFormationConseil(  ).getIdFormationConseil(  ) );
        }

        daoUtil.setString( 4, listeMembrePresent.getIntitule(  ) );

        if ( listeMembrePresent.getFichierSc(  ) == null )
        {
            daoUtil.setIntNull( 5 );
        }
        else
        {
            daoUtil.setInt( 5, listeMembrePresent.getFichierSc(  ).getId(  ) );
        }

        daoUtil.setTimestamp( 6, new Timestamp( System.currentTimeMillis(  ) ) );
        daoUtil.setTimestamp( 7, listeMembrePresent.getDateTransmissionControleLegalite(  ) );
        daoUtil.setBoolean( 8, listeMembrePresent.getEnLigne(  ) );
        daoUtil.setInt( 9, listeMembrePresent.getVersion(  ) );

        if ( listeMembrePresent.getFichierGenere(  ) == null )
        {
            daoUtil.setIntNull( 10 );
        }
        else
        {
            daoUtil.setInt( 10, listeMembrePresent.getFichierGenere(  ).getId(  ) );
        }

        daoUtil.executeUpdate(  );
        daoUtil.free(  );

        return newPrimaryKey;
    }

    /**
         * Modifie la liste des membres prsents correspondante  l objet listeMembrePresent pass en paramtre
         * @param listeMembrePresent la liste des membres prsents
         * @param plugin le plugin
         */
    public void store( ListeMembrePresent<GSeance, GFichier> listeMembrePresent, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_UPDATE, plugin );

        if ( listeMembrePresent.getSeance(  ) != null )
        {
            daoUtil.setInt( 1, listeMembrePresent.getSeance(  ).getIdSeance(  ) );
        }
        else
        {
            daoUtil.setIntNull( 1 );
        }

        if ( listeMembrePresent.getFormationConseil(  ) == null )
        {
            daoUtil.setIntNull( 2 );
        }
        else
        {
            daoUtil.setInt( 2, listeMembrePresent.getFormationConseil(  ).getIdFormationConseil(  ) );
        }

        daoUtil.setString( 3, listeMembrePresent.getIntitule(  ) );

        if ( listeMembrePresent.getFichierSc(  ) == null )
        {
            daoUtil.setIntNull( 4 );
        }
        else
        {
            daoUtil.setInt( 4, listeMembrePresent.getFichierSc(  ).getId(  ) );
        }

        daoUtil.setTimestamp( 5, new Timestamp( System.currentTimeMillis(  ) ) );
        daoUtil.setTimestamp( 6, listeMembrePresent.getDateTransmissionControleLegalite(  ) );
        daoUtil.setBoolean( 7, listeMembrePresent.getEnLigne(  ) );
        daoUtil.setInt( 8, listeMembrePresent.getVersion(  ) );

        if ( listeMembrePresent.getFichierGenere(  ) == null )
        {
            daoUtil.setIntNull( 9 );
        }
        else
        {
            daoUtil.setInt( 9, listeMembrePresent.getFichierGenere(  ).getId(  ) );
        }

        daoUtil.setInt( 10, listeMembrePresent.getId(  ) );

        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Supprime la liste des membres prsents ayant comme identifiant l'identifiant pass en parametre
     * @param nIdListe l'identifiant de la liste
     * @param plugin le plugin
     * @throws AppException si contrainte de cl trangre
     */
    public void delete( int nIdListe, Plugin plugin ) throws AppException
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE, plugin );
        daoUtil.setInt( 1, nIdListe );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Retourne une  liste contenant les listes de membres prsents  correspondants aux critres du filtre.
     * @param filter les critres de recherche de la liste
     * @param plugin le plugin
     * @return liste contenant les listes de membres prsents  correspondants aux critres   de recherche
     */
    public List<ListeMembrePresent<GSeance, GFichier>> selectByFilter( ListeMembrePresentFilter filter, Plugin plugin )
    {
        List<ListeMembrePresent<GSeance, GFichier>> listes = new ArrayList<ListeMembrePresent<GSeance, GFichier>>(  );

        StringBuffer sbSQL = new StringBuffer(  );
        sbSQL.append( SQL_QUERY_LISTE + SQL_WHERE_1_1 );
        sbSQL.append( ( ( filter.containsSeanceCriteria(  ) ) ? SQL_FILTER_SEANCE : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( filter.containsFormationConseilCriteria(  ) ) ? SQL_FILTER_FORMATION_CONSEIL
                                                                        : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( filter.containsPublicationCriteria(  ) ) ? SQL_FILTER_PUBLICATION
                                                                   : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( filter.containsStatutActeCriteria(  ) ) ? SQL_FILTER_STATUT_ACTE
                                                                  : OdsConstants.CONSTANTE_CHAINE_VIDE ) );

        if ( filter.getStatutActe(  ) == ListeMembrePresentFilter.A_PREPARER )
        {
            sbSQL.append( SQL_FILTER_STATUT_A_PREPARER );
        }
        else if ( filter.getStatutActe(  ) == ListeMembrePresentFilter.UN_ACTE_AU_MOINS )
        {
            sbSQL.append( SQL_FILTER_STATUT_UN_ACTE_AU_MOINS );
        }

        sbSQL.append( SQL_QUERY_ORDER_BY_DATE_MODIFICATION );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );
        int nIndex = 1;

        if ( filter.containsSeanceCriteria(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdSeance(  ) );
            nIndex++;
        }

        if ( filter.containsFormationConseilCriteria(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdFormationConseil(  ) );
            nIndex++;
        }

        if ( filter.containsPublicationCriteria(  ) )
        {
            daoUtil.setInt( nIndex, filter.getPublication(  ) );
            nIndex++;
        }

        if ( filter.containsStatutActeCriteria(  ) )
        {
            daoUtil.setInt( nIndex, ITypeEntite.LISTE_DES_MEMBRES_PRESENTS );
            nIndex++;
            daoUtil.setInt( nIndex, StatutActeEnum.ANNULE.getId(  ) );
            nIndex++;
        }

        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            ListeMembrePresent<GSeance, GFichier> liste = getPropertyListeMembresPresents( daoUtil );
            listes.add( liste );
        }

        daoUtil.free(  );

        return listes;
    }

    /**
     * Retourne la liste de membre prsent ayant comme identifiant nKey
     * @param nKey identifiant de la liste de membres prsents
     * @param plugin le plugin
     * @return la liste de membre prsent ayant comme identifiant nKey
     */
    public ListeMembrePresent<GSeance, GFichier> load( int nKey, Plugin plugin )
    {
        ListeMembrePresent<GSeance, GFichier> listeMembrePresent = null;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_BY_PRIMARY_KEY, plugin );
        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        if ( daoUtil.next(  ) )
        {
            listeMembrePresent = getPropertyListeMembresPresents( daoUtil );
        }

        daoUtil.free(  );

        return listeMembrePresent;
    }

    /**
     * Permet de publier ou de dpublier une liste des membres prsents
     * Publication d'ne liste des membres prsents
     * Si isPublier gale  TRUE, alors la mthode <b>publie</b> la liste des membres prsents et incrmente la version<BR>
     * Sinon la mthode change le statut enLigne d'une liste des membres prsents a FALSE
     * @param nKey id de la liste des membres prsents
     * @param tsDatePublication TimeStamp date de publication
     * @param version version
     * @param isPublie enLigne
     * @param plugin le Plugin actif
     */
    public void publication( int nKey, Timestamp tsDatePublication, int version, boolean isPublie, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_PUBLIER, plugin );
        daoUtil.setTimestamp( 1, tsDatePublication );
        daoUtil.setBoolean( 2, isPublie );

        if ( isPublie )
        {
            daoUtil.setInt( 3, version + 1 );
        }
        else
        {
            daoUtil.setInt( 3, version );
        }

        daoUtil.setInt( 4, nKey );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Vrfie qu'aucune liste de mme sance et mme formation conseil existe dj
     * @param listeMembrePresent la liste que l'on souhaite insrer
     * @param plugin le Plugin actif
     * @return false si aucune liste de mme sance et de mme formation conseil existe, true sinon
     */
    public boolean isAlreadyExist( ListeMembrePresent<GSeance, GFichier> listeMembrePresent, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_LISTE + SQL_WHERE_1_1 + SQL_FILTER_SEANCE +
                SQL_FILTER_FORMATION_CONSEIL + SQL_FILTER_ID_LISTE, plugin );
        daoUtil.setInt( 1, listeMembrePresent.getSeance(  ).getIdSeance(  ) );
        daoUtil.setInt( 2, listeMembrePresent.getFormationConseil(  ).getIdFormationConseil(  ) );
        daoUtil.setInt( 3, listeMembrePresent.getId(  ) );
        daoUtil.executeQuery(  );

        if ( daoUtil.next(  ) )
        {
            daoUtil.free(  );

            return true;
        }

        daoUtil.free(  );

        return false;
    }

    /**
    * retourne la liste des identifiants des lus tant prsents
    * @param nIdListe l'identifiant de la liste
    * @param plugin le plugin
    * @return la liste des identifiants des lus tant prsents
    */
    public List<Integer> getListeIdsElusPresents( int nIdListe, Plugin plugin )
    {
        List<Integer> listIdElusPresents = new ArrayList<Integer>(  );
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_LISTE_ELUS_PRESENTS, plugin );
        daoUtil.setInt( 1, nIdListe );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            listIdElusPresents.add( daoUtil.getInt( 1 ) );
        }

        daoUtil.free(  );

        return listIdElusPresents;
    }

    /**
     * Permet d'insrer un lu prsent dans une liste de membres prsents
     * @param nIdElu l'id de l'lu  insrer
     * @param nIdListe l'id de la liste
     * @param plugin le Plugin
     */
    public void insertElus( int nIdElu, int nIdListe, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_INSERT_ELU, plugin );
        daoUtil.setInt( 1, nIdListe );
        daoUtil.setInt( 2, nIdElu );

        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Supprime les lus prsents de la liste de membre prsent dont l'identifiant est pass en parametre
     * @param nIdListeMembrePresent identifiant de la liste des membres prsents
     * @param plugin le Plugin actif
     */
    public void deleteAllElus( int nIdListeMembrePresent, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE_ALL_ELUS, plugin );
        daoUtil.setInt( 1, nIdListeMembrePresent );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Permet de creer un objet liste membre prsent  avec ses proprits de base
     * @param daoUtil contient les informations permettant de construire l'objet
     * @return  liste membre prsent avec les informations de la requte
     */
    private ListeMembrePresent<GSeance, GFichier> getPropertyListeMembresPresents( DAOUtil daoUtil )
    {
        ListeMembrePresent<GSeance, GFichier> listeMembre = new ListeMembrePresent<GSeance, GFichier>(  );
        listeMembre.setId( daoUtil.getInt( "lmp.id_liste" ) );
        listeMembre.setVersion( daoUtil.getInt( "lmp.version" ) );
        listeMembre.setEnLigne( daoUtil.getBoolean( "lmp.en_ligne" ) );
        listeMembre.setDatePublication( daoUtil.getTimestamp( "lmp.date_publication" ) );
        listeMembre.setDateTransmissionControleLegalite( daoUtil.getTimestamp( "lmp.date_transmission_ctrl_legalite" ) );
        listeMembre.setIntitule( daoUtil.getString( "lmp.intitule" ) );

        if ( daoUtil.getObject( "lmp.id_seance" ) != null )
        {
            GSeance seance = newSeanceInstance(  );
            seance.setIdSeance( daoUtil.getInt( "lmp.id_seance" ) );
            listeMembre.setSeance( seance );
        }

        if ( daoUtil.getObject( "lmp.id_formation_conseil" ) != null )
        {
            FormationConseil formationConseil = new FormationConseil(  );
            formationConseil.setIdFormationConseil( daoUtil.getInt( "lmp.id_formation_conseil" ) );
            listeMembre.setFormationConseil( formationConseil );
        }

        if ( daoUtil.getObject( "lmp.id_fichier_sc" ) != null )
        {
            GFichier fichier = newFichierInstance(  );
            fichier.setId( daoUtil.getInt( "lmp.id_fichier_sc" ) );
            listeMembre.setFichierSc( fichier );
        }

        if ( daoUtil.getObject( "lmp.id_fichier_genere" ) != null )
        {
            GFichier fichier = newFichierInstance(  );

            if ( daoUtil.getObject( "fc.taille" ) != null )
            {
                fichier.setTaille( daoUtil.getInt( "fc.taille" ) );
            }

            fichier.setId( daoUtil.getInt( "lmp.id_fichier_genere" ) );
            listeMembre.setFichierGenere( fichier );
        }

        return listeMembre;
    }

    /**
     * Gnre un nouvel identifiant
     * @param plugin le plugin
     * @return Retourne lidentifiant gnr
     */
    int newPrimaryKey( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey = 1;

        if ( daoUtil.next(  ) )
        {
            nKey = daoUtil.getInt( 1 ) + 1;
        }

        daoUtil.free(  );

        return nKey;
    }
}
