/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.listemembrepresent;

import fr.paris.lutece.plugins.ods.business.IBusinessHome;
import fr.paris.lutece.plugins.ods.business.acte.ActeHome;
import fr.paris.lutece.plugins.ods.business.fichier.IFichierHome;
import fr.paris.lutece.plugins.ods.business.formationconseil.FormationConseilHome;
import fr.paris.lutece.plugins.ods.business.seance.ISeanceHome;
import fr.paris.lutece.plugins.ods.dto.acte.Acte;
import fr.paris.lutece.plugins.ods.dto.acte.IActeFilter;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter;
import fr.paris.lutece.plugins.ods.dto.fichier.ITypeDocument;
import fr.paris.lutece.plugins.ods.dto.listemembrepresent.ListeMembrePresent;
import fr.paris.lutece.plugins.ods.dto.listemembrepresent.ListeMembrePresentFilter;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.typeentite.ITypeEntite;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.service.acte.IActeService;
import fr.paris.lutece.portal.service.plugin.Plugin;

import org.springframework.beans.factory.annotation.Autowired;

import java.sql.Timestamp;

import java.util.List;


/**
 * 
 * ListeMembrePresentHome
 *
 * @param <GFichier> IFichier
 * @param <GSeance> ISeance
 * @param <GActeFilter> IActeFilter
 * @param <GElu> IElu
 * @param <GVoeuAmendement> IVoeuAmendement
 * @param <GPDD> IPDD
 */
public abstract class AbstractListeMembrePresentHome
<
	GSeance extends ISeance, 
	GFichier extends IFichier<GSeance, GFichier>, 
	GActeFilter extends IActeFilter<GFichier, GSeance>, 
	GElu extends IElu, 
	GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, 
	GSeance, GElu, GFichier, GPDD>, 
	GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>
>
implements IBusinessHome, IListeMembrePresentHome<GSeance, GFichier, GActeFilter>
{
    @Autowired
    private IListeMembrePresentDAO<GSeance, GFichier> _listeMembrePresentDAO;
    @Autowired
    private ActeHome<GSeance, GFichier, GActeFilter> _acteHome;
    @Autowired
    private IFichierHome<GFichier, IFichierFilter, ITypeDocument, GSeance> _fichierHome;
    @Autowired
    private FormationConseilHome _formationConseilHome;
    @Autowired
    private ISeanceHome<GSeance, ISeanceFilter, GFichier, GElu, GVoeuAmendement, GPDD> _seanceHome;
    @Autowired
    private IActeService<GSeance, GFichier, GActeFilter> _acteService;

    protected abstract Acte<GSeance, GFichier> getActeCourant( List<Acte<GSeance, GFichier>> listActes );

    /**
    * Cre une nouvelle liste de membres prsents  partir de l'objet deliberation pass en paramtre
    * @param listeMembrePresent la liste de memnre prsent  insrer
    * @param plugin le Plugin actif
    * @return int primaryKey
    */
    public int create( ListeMembrePresent<GSeance, GFichier> listeMembrePresent, Plugin plugin )
    {
        return _listeMembrePresentDAO.insert( listeMembrePresent, plugin );
    }

    /**
     * Modifie la liste de membres prsents  partir de l'objet pass en paramtre
     * @param listeMembrePresent la liste des membres prsents  modifier
     * @param plugin le Plugin actif
     */
    public void update( ListeMembrePresent<GSeance, GFichier> listeMembrePresent, Plugin plugin )
    {
        _listeMembrePresentDAO.store( listeMembrePresent, plugin );
    }

    /**
     * Supprime la liste de membre prsent dont l'identifiant est pass en parametre
     * @param nIdListeMembrePresent identifiant de la liste des membres prsents
     * @param plugin le Plugin actif
     */
    public void remove( int nIdListeMembrePresent, Plugin plugin )
    {
        _listeMembrePresentDAO.delete( nIdListeMembrePresent, plugin );
    }

    /**
     * renvoie la liste des membres prsents  ayant comme identifiant nKey
     * @param nKey idenfiant de la liste des membres prsents
     * @param plugin le Plugin actif
     * @return une liste de membres prsents
     */
    public ListeMembrePresent<GSeance, GFichier> findByPrimaryKey( int nKey, Plugin plugin )
    {
        ListeMembrePresent<GSeance, GFichier> listeMembrePresent = _listeMembrePresentDAO.load( nKey, plugin );

        List<Acte<GSeance, GFichier>> listActes = getListeActes( nKey, plugin );
        listeMembrePresent.setActes( listActes );

        if ( listActes != null )
        {
            listeMembrePresent.setActeCourant( getActeCourant( listActes ) );
        }

        //rcupration des objets associs 
        setOthersProperties( listeMembrePresent, plugin );

        return listeMembrePresent;
    }

    /**
     * rcupre la liste des membres prsents correspondant au filtre donn
     * @param filter le filtre
     * @param plugin le Plugin actif
     * @return la liste des membres prsents
     */
    public List<ListeMembrePresent<GSeance, GFichier>> findByFilter( ListeMembrePresentFilter filter, Plugin plugin )
    {
        List<ListeMembrePresent<GSeance, GFichier>> listMembrePresent = _listeMembrePresentDAO.selectByFilter( filter,
                plugin );

        for ( ListeMembrePresent<GSeance, GFichier> membrePresent : listMembrePresent )
        {
            List<Acte<GSeance, GFichier>> listActes = getListeActes( membrePresent.getId(  ), plugin );
            membrePresent.setActes( listActes );

            setOthersProperties( membrePresent, plugin );
        }

        return listMembrePresent;
    }

    /**
     * Permet de publier ou de dpublier une liste des membres prsents
     * Publication d'ne liste des membres prsents
     * Si isPublier gale  TRUE, alors la mthode <b>publie</b> la liste des membres prsents et incrmente la version<BR>
     * Sinon la mthode change le statut enLigne d'une liste des membres prsents a FALSE
     * @param nKey id de la liste des membres prsents
     * @param datePublication TimeStamp date de publication
     * @param version version
     * @param isPublier enLigne
     * @param plugin le Plugin actif
     */
    public void publication( int nKey, Timestamp datePublication, int version, boolean isPublier, Plugin plugin )
    {
        _listeMembrePresentDAO.publication( nKey, datePublication, version, isPublier, plugin );
    }

    /**
    * Vrfie qu'aucune liste de mme sance et mme formation conseil existe dj
    * @param listeMembrePresent la liste que l'on souhaite insrer
    * @param plugin le Plugin actif
    * @return false si aucune liste de mme sance et de mme formation conseil existe, true sinon
    */
    public boolean isAlreadyExist( ListeMembrePresent<GSeance, GFichier> listeMembrePresent, Plugin plugin )
    {
        return _listeMembrePresentDAO.isAlreadyExist( listeMembrePresent, plugin );
    }

    /**
     * retourne la liste des identifiants des lus tant prsents
     * @param nIdListMembresPresents l'identifiant de la liste
     * @param plugin le plugin
     * @return la liste des identifiants des lus tant prsents
     */
    public List<Integer> getListeIdsElusPresents( int nIdListMembresPresents, Plugin plugin )
    {
        return _listeMembrePresentDAO.getListeIdsElusPresents( nIdListMembresPresents, plugin );
    }

    /**
         * Permet d'insrer un lu prsent dans une liste de membres prsents
         * @param nIdElu l'id de l'lu  insrer
         * @param nIdListe l'id de la liste
         * @param plugin le Plugin
         */
    public void insertElus( int nIdElu, int nIdListe, Plugin plugin )
    {
        _listeMembrePresentDAO.insertElus( nIdElu, nIdListe, plugin );
    }

    /**
     * Supprime les lus prsents de la liste de membre prsent dont l'identifiant est pass en parametre
     * @param nIdListeMembrePresent identifiant de la liste des membres prsents
     * @param plugin le Plugin actif
     */
    public void removeAllElus( int nIdListeMembrePresent, Plugin plugin )
    {
        _listeMembrePresentDAO.deleteAllElus( nIdListeMembrePresent, plugin );
    }

    /**
     * insere dans l'objet liste membre prsent les objets associs
     * @param listeMembrePresent l'objet
     * @param plugin Plugin
     */
    private void setOthersProperties( ListeMembrePresent<GSeance, GFichier> listeMembrePresent, Plugin plugin )
    {
        if ( listeMembrePresent.getSeance(  ) != null )
        {
            listeMembrePresent.setSeance( _seanceHome.findByPrimaryKey( 
                    listeMembrePresent.getSeance(  ).getIdSeance(  ), plugin ) );
        }

        if ( listeMembrePresent.getFormationConseil(  ) != null )
        {
            listeMembrePresent.setFormationConseil( _formationConseilHome.findByPrimaryKey( 
                    listeMembrePresent.getFormationConseil(  ).getIdFormationConseil(  ), plugin ) );
        }

        if ( listeMembrePresent.getFichierGenere(  ) != null )
        {
            listeMembrePresent.setFichierGenere( _fichierHome.findByPrimaryKey( 
                    listeMembrePresent.getFichierGenere(  ).getId(  ), plugin ) );
        }
    }

    /**
     * retourne la liste des actes rattachs  la liste
     * @param nIdEntite l'identifiant de la liste
     * @param plugin le plugin
     * @return la liste des identifiants des actes
     */
    private List<Acte<GSeance, GFichier>> getListeActes( int nIdEntite, Plugin plugin )
    {
        GActeFilter acteFilter = _acteService.getActeFilter(  );
        acteFilter.setIdEntite( nIdEntite );
        acteFilter.setIdTypeEntite( ITypeEntite.LISTE_DES_MEMBRES_PRESENTS );

        List<Acte<GSeance, GFichier>> listActes = _acteHome.findByFilter( acteFilter, plugin );

        return listActes;
    }
}
