/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.listemembrepresent;

import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.listemembrepresent.ListeMembrePresent;
import fr.paris.lutece.plugins.ods.dto.listemembrepresent.ListeMembrePresentFilter;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;

import java.sql.Timestamp;

import java.util.List;


/**
 *
 * IListeMembrePresentDAO
 *
 * @param <GFichier> IFichier
 * @param <GSeance> ISeance
 */
public interface IListeMembrePresentDAO<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>>
{
    /**
     * Cre une nouvelle liste de membres prsents   partir de l'objet listeMembrePresent pass en paramtre
     * @param listeMembrePresent la liste des membres prsents  insrer
     * @param plugin le plugin
     * @return int primaryKey
     */
    int insert( ListeMembrePresent<GSeance, GFichier> listeMembrePresent, Plugin plugin );

    /**
     * Modifie la liste des membres prsents correspondante  l objet listeMembrePresent pass en paramtre
     * @param listeMembrePresent la liste des membres prsents
     * @param plugin le plugin
     */
    void store( ListeMembrePresent<GSeance, GFichier> listeMembrePresent, Plugin plugin );

    /**
     * Supprime la liste des membres prsents ayant comme identifiant l'identifiant pass en parametre
     * @param nIdListe l'identifiant de la liste
     * @param plugin le plugin
     * @throws AppException si contrainte de cl trangre
     */
    void delete( int nIdListe, Plugin plugin ) throws AppException;

    /**
     * Retourne une  liste contenant les listes de membres prsents  correspondants aux critres du filtre.
     * @param filter les critres de recherche de la liste
     * @param plugin le plugin
     * @return liste contenant les listes de membres prsents  correspondants aux critres   de recherche
     */
    List<ListeMembrePresent<GSeance, GFichier>> selectByFilter( ListeMembrePresentFilter filter, Plugin plugin );

    /**
     * Retourne la liste de membre prsent ayant comme identifiant nKey
     * @param nKey identifiant de la liste de membres prsents
     * @param plugin le plugin
     * @return la liste de membre prsent ayant comme identifiant nKey
     */
    ListeMembrePresent<GSeance, GFichier> load( int nKey, Plugin plugin );

    /**
     * Permet de publier ou de dpublier une liste des membres prsents
     * Publication d'ne liste des membres prsents
     * Si isPublier gale  TRUE, alors la mthode <b>publie</b> la liste des membres prsents et incrmente la version<BR>
     * Sinon la mthode change le statut enLigne d'une liste des membres prsents a FALSE
     * @param nKey id de la liste des membres prsents
     * @param datePublication TimeStamp date de publication
     * @param version version
     * @param isPublier enLigne
     * @param plugin le Plugin actif
     */
    void publication( int nKey, Timestamp datePublication, int version, boolean isPublier, Plugin plugin );

    /**
     * Vrfie qu'aucune liste de mme sance et mme formation conseil existe dj
     * @param listeMembrePresent la liste que l'on souhaite insrer
     * @param plugin le Plugin actif
     * @return false si aucune liste de mme sance et de mme formation conseil existe, true sinon
     */
    boolean isAlreadyExist( ListeMembrePresent<GSeance, GFichier> listeMembrePresent, Plugin plugin );

    /**
    * retourne la liste des identifiants des lus tant prsents
    * @param nIdListe l'identifiant de la liste
    * @param plugin le plugin
    * @return la liste des identifiants des lus tant prsents
    */
    List<Integer> getListeIdsElusPresents( int nIdListe, Plugin plugin );

    /**
     * retourne la liste des actes rattachs  la liste
     * @param nIdListe l'identifiant de la liste
     * @param plugin le plugin
     * @return la liste des identifiants des actes
     */

    //List<Acte> getListeActes( int nIdListe, Plugin plugin );

    /**
     * Permet d'insrer un lu prsent dans une liste de membres prsents
     * @param nIdElu l'id de l'lu  insrer
     * @param nIdListe l'id de la liste
     * @param plugin le Plugin
     */
    void insertElus( int nIdElu, int nIdListe, Plugin plugin );

    /**
     * Supprime les lus prsents de la liste de membre prsent dont l'identifiant est pass en parametre
     * @param nIdListeMembrePresent identifiant de la liste des membres prsents
     * @param plugin le Plugin actif
     */
    void deleteAllElus( int nIdListeMembrePresent, Plugin plugin );
}
