/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.modeleordredujour;

import fr.paris.lutece.plugins.ods.dto.modeleordredujour.IModeleOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;

import java.util.List;


/**
 * IModeleOrdreDuJourDAO
 * @param <GOrdreDuJourFilter> {@link IOrdreDuJourFilter}
 * @param <GModeleOrdreDuJour> {@link IModeleOrdreDuJour}
 *
 */
public interface IModeleOrdreDuJourDAO<GOrdreDuJourFilter extends IOrdreDuJourFilter, GModeleOrdreDuJour extends IModeleOrdreDuJour>
{
    /**
     * Cre un nouveau MODJ  partir de lobjet MODJ pass en paramtre
     * @param modeleOrdreDuJour le MODJ  insrer
     * @param plugin le plugin
     * @return int primaryKey
     */
    int insert( GModeleOrdreDuJour modeleOrdreDuJour, Plugin plugin );

    /**
     * Modifie le MODJ correspondant  lobjet MODJ pass en paramtre
     * @param modeleOrdreDuJour le MODJ  modifier
     * @param plugin le plugin
     */
    void store( GModeleOrdreDuJour modeleOrdreDuJour, Plugin plugin );

    /**
     * Supprime le MODJ correspondant  lobjet pass en paramtre
     * @param modeleOrdreDuJour  le  MODJ  supprimer
     * @param plugin le plugin
     * @throws AppException si contrainte de cl trangre
     */
    void delete( GModeleOrdreDuJour modeleOrdreDuJour, Plugin plugin )
        throws AppException;

    /**
     * Retourne la liste des ModeleOrdreDuJour disponibles
     * @param plugin le plugin
     * @return la liste des ModeleOrdreDuJour
     */
    List<GModeleOrdreDuJour> selectListModeleOrdreDuJourList( Plugin plugin );

    /**
     * Retourne le ModeleOrdreDuJour ayant comme identifiant nKey
     * @param nKey identifiant du ModeleOrdreDuJour  charger
     * @param plugin le plugin
     * @return le fichier identifi par nKey
     */
    GModeleOrdreDuJour load( int nKey, Plugin plugin );

    /**
    * Retourne le ModeleOrdreDuJour rpondant au filtre ordreDuJourFilter
    * @param filter le filtre d'Ordre du jour
    * @param plugin le plugin
    * @return le ModeleOrdreDuJour rpondant au filtre ordreDuJourFilter
    */
    GModeleOrdreDuJour load( GOrdreDuJourFilter filter, Plugin plugin );
}
