/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.modeleordredujour;

import fr.paris.lutece.plugins.ods.dto.modeleordredujour.IModeleOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter;
import fr.paris.lutece.portal.service.plugin.Plugin;

import java.util.List;


/**
 *
 * IModeleOrdreDuJourHome
 *
 * @param <GOrdreDuJourFilter> {@link IOrdreDuJourFilter}
 * @param <GModeleOrdreDuJour> {@link IModeleOrdreDuJour}
 */
public interface IModeleOrdreDuJourHome<GOrdreDuJourFilter extends IOrdreDuJourFilter, GModeleOrdreDuJour extends IModeleOrdreDuJour>
{
    /**
     * Cre un modele d'ordre du jour dans la base de donnes
     * @param modeleOrdreDuJour le modele d'ordre du jour
     * @param plugin le plugin ods
     * @return l'identifiant du modele cr
     */
    int create( GModeleOrdreDuJour modeleOrdreDuJour, Plugin plugin );

    /**
     * modifie le modele d'ordre du jour dans la base de donnes
     * @param modeleOrdreDuJour le modele d'ordre du jour
     * @param plugin le plugin ods
     */
    void update( GModeleOrdreDuJour modeleOrdreDuJour, Plugin plugin );

    /**
     * supprime le modele d'ordre du jour dans la base de donnes
     * @param modeleOrdreDuJour le modele d'ordre du jour
     * @param plugin le plugin ods
     */
    void remove( GModeleOrdreDuJour modeleOrdreDuJour, Plugin plugin );

    /**
     * renvoie le modele d'ordre du jour ayant l'identifiant donn en parametre
     * @param nKey l'id du modele d'ordre du jour
     * @param plugin le plugin ods
     * @return le modele d'ordre du jour
     */
    GModeleOrdreDuJour findByPrimaryKey( int nKey, Plugin plugin );

    /**
     * renvoie la liste des modeles d'ordre du jour
     * @param plugin le plugin ods
     * @return la liste des modeles d'ordre du jour
     */
    List<GModeleOrdreDuJour> findModeleOrdreDuJourList( Plugin plugin );

    /**
     * renvoie le modele d'ordre du jour correspondant au criteres du filtre donn en parametre
     * @param filter le filtre
     * @param plugin plugin ods
     * @return le modele d'ordre du jour
     */
    GModeleOrdreDuJour findByFilter( GOrdreDuJourFilter filter, Plugin plugin );
}
