/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.ordredujour;

import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.ordredujour.EntreeOrdreDuJourFilter;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.TypeOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.typeentite.ITypeEntite;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.util.sql.DAOUtil;

import java.util.ArrayList;
import java.util.List;


/**
 * AbstractEntreeOrdreDuJourDAO
 * @param <GEntreeOrdreDuJour> {@link IEntreeOrdreDuJour}
 * @param <GElu> {@link IElu}
 * @param <GSeance> {@link ISeance}
 * @param <GOrdreDuJour> {@link IOrdreDuJour}
 * @param <GFichier> {@link IFichier}
 *
 */
public abstract class AbstractEntreeOrdreDuJourDAO
<
	GEntreeOrdreDuJour extends IEntreeOrdreDuJour<GElu, GSeance, GFichier, GEntreeOrdreDuJour, GOrdreDuJour>, 
	GElu extends IElu, 
	GSeance extends ISeance, 
	GOrdreDuJour extends IOrdreDuJour<GEntreeOrdreDuJour, GSeance, GFichier, GElu, GOrdreDuJour>, 
	GFichier extends IFichier<GSeance, GFichier>
>
implements IEntreeOrdreDuJourDAO<GOrdreDuJour, GEntreeOrdreDuJour, GElu, GSeance, GFichier>
{
    private static final String SQL_QUERY_NEW_PK = "SELECT max( id_entree ) FROM ods_entree_ordre_jour ";
    private static final String SQL_QUERY_NEW_NUMERO_ORDRE = "SELECT max( numero_ordre ) FROM ods_entree_ordre_jour where id_odj = ? ";
    private static final String SQL_QUERY_NEW_NUMERO_ORDRE_ELU = "SELECT max( numero_ordre ) FROM ods_entree_elus where id_entree = ? ";
    private static final String SQL_QUERY_DELETE = "DELETE FROM ods_entree_ordre_jour WHERE id_entree = ? ";
    private static final String SQL_QUERY_FIND_BY_PRIMARY_KEY = "SELECT en.id_odj, en.numero_ordre, " +
        "en.reference_affichee, en.objet, en.style, en.id_entite, en.id_type_entite, " +
        "na.id_nature, na.libelle_nature, na.num_nature " + "FROM ods_entree_ordre_jour en " +
        "LEFT JOIN ods_nature_dossier na on( en.id_nature = na.id_nature ) WHERE id_entree = ? ";
    private static final String SQL_QUERY_SELECT_COUNT_ENTREES_BY_FILTER = "SELECT COUNT( en.id_entree ) " +
        "FROM ods_entree_ordre_jour en " + "LEFT JOIN ods_nature_dossier na ON( en.id_nature = na.id_nature ) " +
        "LEFT JOIN ods_odj odj ON( en.id_odj = odj.id_odj ) ";
    private static final String SQL_QUERY_DELETE_RAPPORTEUR = "DELETE FROM ods_entree_elus WHERE id_entree = ? ";
    private static final String SQL_FILTER_ODJ = " AND odj.id_odj = ? ";
    private static final String SQL_FILTER_PUBLICATION = " AND odj.publie = ? ";
    private static final String SQL_FILTER_SEANCE = " AND odj.id_seance = ? ";
    private static final String SQL_ORDER_BY_NUMERO_ORDRE = " ORDER BY en.numero_ordre ";
    private static final String SQL_LEFT_JOIN_PDD = " LEFT JOIN ods_pdd pdd ON( en.id_type_entite = 6 AND en.id_entite = pdd.id_pdd ) " +
        "LEFT JOIN ods_direction dir ON( pdd.id_direction = dir.id_direction ) " +
        "LEFT JOIN ods_formation_conseil fc ON( pdd.id_formation_conseil = fc.id_formation_conseil ) ";
    private static final String SQL_LEFT_JOIN_VA = " LEFT JOIN ods_voeu_amendement va ON( " + " en.id_type_entite = " +
        ITypeEntite.VOEUX + " AND en.id_entite = va.id_va ) " +
        "LEFT JOIN ods_fascicule fa ON( va.id_fascicule = fa.id_fascicule ) ";
    private static final String SQL_WHERE_1 = " WHERE 1 = 1 ";
    private static final String SQL_FILTER_ENTITE = " AND en.id_entite = ? ";
    private static final String SQL_FILTER_TYPE_ENTITE = " AND en.id_type_entite = ? ";

    /**
     * Cre une nouvelle instance d'EntreeOrdreDuJour
     * @return une nouvelle instance d'EntreeOrdreDuJour
     */
    protected abstract GEntreeOrdreDuJour getEntreeOrdreDuJour(  );

    /**
     * Cre une nouvelle instance d'OrdreDuJour
     * @return une nouvelle instance d'OrdreDuJour
     */
    protected abstract GOrdreDuJour newOrdreDuJourInstance(  );

    /**
     * Charge les valeurs spcifiques
     * @param daoUtil le daOutil
     * @param entreeOrdreDuJour l'entreOrdreDuJour
     * @param plugin le plugin
     */
    protected abstract void loadSpec( DAOUtil daoUtil, GEntreeOrdreDuJour entreeOrdreDuJour, Plugin plugin );

    /**
     * Supprime les informations spcifiques
     * @param entree l'entree
     * @param plugin le plugin
     */
    protected abstract void deleteSpec( GEntreeOrdreDuJour entree, Plugin plugin );

    /**
     * Replit les champs spcifiques
     * @param daoUtil daoUtil
     * @param entreeOrdreDuJour l'entree
     * @param plugin le plguin
     */
    protected abstract void remplirListeEntreesSpec( DAOUtil daoUtil, GEntreeOrdreDuJour entreeOrdreDuJour,
        Plugin plugin );

    /**
     * Retourne la requete de slection de la liste des entre
     * @return la requete de slection de la liste des entre
     */
    protected abstract String getSQLfindEntreesList(  );

    /**
     * Retourne la liste des lus rapporteurs pour l'entre dont l'identifiant est nKey
     * @param nKey l'identifiant de l'entre
     * @param plugin le plugin
     * @return la liste des lus
     */
    protected abstract List<GElu> selectRapporteursbyIdEntree( int nKey, Plugin plugin );

    /**
     * Gnre un nouvel identifiant
     * @param plugin le Plugin
     * @return Retourne lidentifiant gnr
     */
    protected int newPrimaryKey( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey;

        if ( !daoUtil.next(  ) )
        {
            nKey = 1;
        }

        nKey = daoUtil.getInt( 1 ) + 1;
        daoUtil.free(  );

        return nKey;
    }

    /**
     * Gnre un nouveau numero d'ordre
     * @param plugin le Plugin
     * @param nIdOdj identifiant de l'OdJ
     * @return Retourne le numero ordre gnr
     */
    protected int newEntreeNumeroOrdre( Plugin plugin, int nIdOdj )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_NUMERO_ORDRE, plugin );
        daoUtil.setInt( 1, nIdOdj );
        daoUtil.executeQuery(  );

        int nKey;

        if ( !daoUtil.next(  ) )
        {
            nKey = 1;
        }

        nKey = daoUtil.getInt( 1 ) + 1;
        daoUtil.free(  );

        return nKey;
    }

    /**
     * Gnre un nouveau numero d'ordre
     * @param plugin le Plugin
     * @param nIdEntree identifiant de l'entre
     * @return Retourne le numero ordre gnr
     */
    private int newEluNumeroOrdre( Plugin plugin, int nIdEntree )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_NUMERO_ORDRE_ELU, plugin );
        daoUtil.setInt( 1, nIdEntree );
        daoUtil.executeQuery(  );

        int nKey;

        if ( !daoUtil.next(  ) )
        {
            nKey = 1;
        }

        nKey = daoUtil.getInt( 1 ) + 1;
        daoUtil.free(  );

        return nKey;
    }

    /**
     * supprime l'entre d'ordre du jour passe en parametre
     * @param entree EntreeOrdreDuJour
     * @param plugin Plugin
     */
    public void delete( GEntreeOrdreDuJour entree, Plugin plugin )
    {
        deleteSpec( entree, plugin );

        // Suppression des rapporteurs  lis  l'entre 
        if ( entree.getElus(  ) != null )
        {
            deleteRapporteur( entree.getIdEntreeOrdreDuJour(  ), plugin );
        }

        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE, plugin );
        daoUtil.setInt( 1, entree.getIdEntreeOrdreDuJour(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * renvoie l'entre d'ordre du jour d'id nKey
     * @param nKey int
     * @param plugin Plugin
     * @return EntreeOrdreDuJour entree
     */
    public GEntreeOrdreDuJour load( int nKey, Plugin plugin )
    {
        GEntreeOrdreDuJour entreeOrdreDuJour = null;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_BY_PRIMARY_KEY, plugin );
        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            entreeOrdreDuJour = getEntreeOrdreDuJour(  );
            entreeOrdreDuJour.setIdEntreeOrdreDuJour( nKey );

            GOrdreDuJour ordreDuJour = newOrdreDuJourInstance(  );
            ordreDuJour.setIdOrdreDuJour( daoUtil.getInt( 1 ) );
            entreeOrdreDuJour.setOrdreDuJour( ordreDuJour );

            entreeOrdreDuJour.setNumeroOrdre( daoUtil.getInt( 2 ) );
            entreeOrdreDuJour.setReference( daoUtil.getString( 3 ) );
            entreeOrdreDuJour.setObjet( daoUtil.getString( 4 ) );
            entreeOrdreDuJour.setStyle( daoUtil.getString( 5 ) );

            if ( daoUtil.getObject( 6 ) != null )
            {
                entreeOrdreDuJour.setIdEntite( daoUtil.getInt( 6 ) );
            }

            entreeOrdreDuJour.setIdTypeEntite( daoUtil.getInt( 7 ) );

            // Rcupration de la liste des rapporteurs associs
            entreeOrdreDuJour.setElus( selectRapporteursbyIdEntree( entreeOrdreDuJour.getIdEntreeOrdreDuJour(  ), plugin ) );

            loadSpec( daoUtil, entreeOrdreDuJour, plugin );
        }

        daoUtil.free(  );

        return entreeOrdreDuJour;
    }

    /**
     * Retourne la liste d'entres de l'ordre du jour pour l'ordre du jour identifi par nKey
     * @param nKey int l'id de l'ordre du jour
     * @param plugin le plugin
     * @return la liste d'entres de l'ordre du jour d'id nKey
     */
    public List<GEntreeOrdreDuJour> selectEntreesByIdOrdreDuJour( int nKey, Plugin plugin )
    {
        List<GEntreeOrdreDuJour> listentreesOrdreDuJour = new ArrayList<GEntreeOrdreDuJour>(  );

        StringBuffer sbSQL = new StringBuffer(  );
        sbSQL.append( getSQLfindEntreesList(  ) );
        sbSQL.append( SQL_WHERE_1 );
        sbSQL.append( SQL_FILTER_ODJ + SQL_ORDER_BY_NUMERO_ORDRE );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );

        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        remplirListeEntrees( listentreesOrdreDuJour, daoUtil, plugin );

        daoUtil.free(  );

        return listentreesOrdreDuJour;
    }

    /**
     * Retourne la liste d'entres de l'ordre du jour pour l'ordre du jour identifi par nKey
     * @param nKey int l'id de l'ordre du jour
     * @param plugin le plugin
     * @return le nombre d'entres de l'ordre du jour d'id nKey
     */
    public int selectCountEntreesByIdOrdreDuJour( int nKey, Plugin plugin )
    {
        int nResult = 0;

        StringBuffer sbSQL = new StringBuffer(  );
        sbSQL.append( SQL_QUERY_SELECT_COUNT_ENTREES_BY_FILTER );
        sbSQL.append( SQL_WHERE_1 );
        sbSQL.append( SQL_FILTER_ODJ + SQL_ORDER_BY_NUMERO_ORDRE );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );

        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        if ( daoUtil.next(  ) )
        {
            nResult = daoUtil.getInt( 1 );
        }

        daoUtil.free(  );

        return nResult;
    }

    /**
     * Dfinit les filtres et les paramtres pour les requetes de selectCountEntreesByFilter et findEntreesListByFilter
     * @param sbSQL la requete
     * @param filter le filtre
     * @param plugin le plugin
     * @return DAOUtil
     */
    private DAOUtil setFiltersForSelectByFilter( StringBuffer sbSQL, EntreeOrdreDuJourFilter filter, Plugin plugin )
    {
        if ( filter.containsIdTypeEntiteCriteria(  ) && filter.containsIdEntiteCriteria(  ) )
        {
            if ( filter.getIdTypeEntite(  ) == ITypeEntite.PDD )
            {
                sbSQL.append( SQL_LEFT_JOIN_PDD );
            }
            else if ( filter.getIdTypeEntite(  ) == ITypeEntite.VOEUX )
            {
                sbSQL.append( SQL_LEFT_JOIN_VA );
            }
        }

        sbSQL.append( SQL_WHERE_1 );

        sbSQL.append( ( ( filter.containsIdOrdreDuJourCriteria(  ) ) ? SQL_FILTER_ODJ : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( filter.containsIdPublieCriteria(  ) ) ? SQL_FILTER_PUBLICATION
                                                                : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( filter.containsIdSeanceCriteria(  ) ) ? SQL_FILTER_SEANCE : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( filter.containsIdEntiteCriteria(  ) ) ? SQL_FILTER_ENTITE : OdsConstants.CONSTANTE_CHAINE_VIDE ) );
        sbSQL.append( ( ( filter.containsIdTypeEntiteCriteria(  ) ) ? SQL_FILTER_TYPE_ENTITE
                                                                    : OdsConstants.CONSTANTE_CHAINE_VIDE ) );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );

        int nIndex = 1;

        // Filtre sur ordre du jour
        if ( filter.containsIdOrdreDuJourCriteria(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdOrdreDuJour(  ) );
            nIndex++;
        }

        // Filtre sur publication
        if ( filter.containsIdPublieCriteria(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdPublie(  ) );
            nIndex++;
        }

        // Filtre sur seance
        if ( filter.containsIdSeanceCriteria(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdSeance(  ) );
            nIndex++;
        }

        // Filtre sur l'entit
        if ( filter.containsIdEntiteCriteria(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdEntite(  ) );
            nIndex++;
        }

        // Filtre sur le type de l'entit
        if ( filter.containsIdTypeEntiteCriteria(  ) )
        {
            daoUtil.setInt( nIndex, filter.getIdTypeEntite(  ) );
            nIndex++;
        }

        return daoUtil;
    }

    /**
     * Retourne le nombre d'entres de l'ordre du jour rpondant au filtre
     * @param filter le filtre
     * @param plugin le plugin
     * @return le nombre d'entres de l'ordre du jour rpondant au filtre
     */
    public int selectCountEntreesByFilter( EntreeOrdreDuJourFilter filter, Plugin plugin )
    {
        int nResult = 0;

        StringBuffer sbSQL = new StringBuffer(  );
        sbSQL.append( SQL_QUERY_SELECT_COUNT_ENTREES_BY_FILTER );

        DAOUtil daoUtil = setFiltersForSelectByFilter( sbSQL, filter, plugin );

        daoUtil.executeQuery(  );

        if ( daoUtil.next(  ) )
        {
            nResult = daoUtil.getInt( 1 );
        }

        daoUtil.free(  );

        return nResult;
    }

    /**
     * renvoie la liste d'entres de l'ordre du jour correspondant aux filtres appliqus
     * @param filter Filtre sur les entres d'ordre du jour
     * @param plugin le plugin ODS
     * @return la liste d'entres de l'ordre du jour correspondant aux filtres appliqus
     */
    public List<GEntreeOrdreDuJour> findEntreesListByFilter( EntreeOrdreDuJourFilter filter, Plugin plugin )
    {
        List<GEntreeOrdreDuJour> listentreesOrdreDuJour = new ArrayList<GEntreeOrdreDuJour>(  );

        StringBuffer sbSQL = new StringBuffer(  );
        sbSQL.append( getSQLfindEntreesList(  ) );

        DAOUtil daoUtil = setFiltersForSelectByFilter( sbSQL, filter, plugin );

        daoUtil.executeQuery(  );

        remplirListeEntrees( listentreesOrdreDuJour, daoUtil, plugin );

        daoUtil.free(  );

        return listentreesOrdreDuJour;
    }

    /**
     * Insere un rapporteur  l 'entre d'id nIdEntree,
     * @param nIdEntree l'id de l'entre
     * @param elu  l lu  insrer
     * @param plugin le Plugin
     */
    public void insertRapporteur( int nIdEntree, GElu elu, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( getSQLInsertRapporteur(  ), plugin );
        int nIndex = 1;
        daoUtil.setInt( nIndex, nIdEntree );
        nIndex++;
        daoUtil.setInt( nIndex, elu.getIdElu(  ) );
        nIndex++;
        daoUtil.setInt( nIndex, newEluNumeroOrdre( plugin, nIdEntree ) );
        nIndex++;
        nIndex = insertRapporteurSpec( elu, nIndex, daoUtil );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Insertion des valeurs spcifiques
     * @param elu l'elu
     * @param nIndex l'index
     * @param daoUtil daoUtil
     * @return l'index de sortie
     */
    protected abstract int insertRapporteurSpec( GElu elu, int nIndex, DAOUtil daoUtil );

    /**
     * Retourne la requete spcifique pour l'insertion
     * @return la requete spcifique pour l'insertion
     */
    protected abstract String getSQLInsertRapporteur(  );

    /**
     *Supprime tous les rapporteurs lis  l'entre d'id nIdEntree,
     * @param nIdEntree l'id de l'entre
     * @param plugin le Plugin
     */
    public void deleteRapporteur( int nIdEntree, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE_RAPPORTEUR, plugin );
        daoUtil.setInt( 1, nIdEntree );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Remplit la liste des entres d'ordre du jour  partir des rsultats de la requte sql
     * @param listeEntrees liste d'entres  remplir
     * @param daoUtil le rsultat de la requte SQL
     * @param plugin le plugin
     */
    private void remplirListeEntrees( List<GEntreeOrdreDuJour> listeEntrees, DAOUtil daoUtil, Plugin plugin )
    {
        GEntreeOrdreDuJour entreeOrdreDuJour = null;

        while ( daoUtil.next(  ) )
        {
            entreeOrdreDuJour = getEntreeOrdreDuJour(  );
            entreeOrdreDuJour.setIdEntreeOrdreDuJour( daoUtil.getInt( "en.id_entree" ) );

            GOrdreDuJour ordreDuJour = newOrdreDuJourInstance(  );
            ordreDuJour.setIdOrdreDuJour( daoUtil.getInt( "en.id_odj" ) );

            TypeOrdreDuJour typeOrdreDuJour = new TypeOrdreDuJour(  );
            typeOrdreDuJour.setIdTypeOrdreDuJour( daoUtil.getInt( "odj.id_type_odj" ) );
            ordreDuJour.setTypeOrdreDuJour( typeOrdreDuJour );

            entreeOrdreDuJour.setOrdreDuJour( ordreDuJour );

            entreeOrdreDuJour.setNumeroOrdre( daoUtil.getInt( "en.numero_ordre" ) );
            entreeOrdreDuJour.setReference( daoUtil.getString( "en.reference_affichee" ) );
            entreeOrdreDuJour.setObjet( daoUtil.getString( "en.objet" ) );
            entreeOrdreDuJour.setStyle( daoUtil.getString( "en.style" ) );

            if ( daoUtil.getObject( "en.id_entite" ) != null )
            {
                entreeOrdreDuJour.setIdEntite( daoUtil.getInt( "en.id_entite" ) );
            }

            entreeOrdreDuJour.setIdTypeEntite( daoUtil.getInt( "en.id_type_entite" ) );

            // Rcupration de la liste des rapporteurs associs
            entreeOrdreDuJour.setElus( selectRapporteursbyIdEntree( entreeOrdreDuJour.getIdEntreeOrdreDuJour(  ), plugin ) );

            loadSpec( daoUtil, entreeOrdreDuJour, plugin );
            remplirListeEntreesSpec( daoUtil, entreeOrdreDuJour, plugin );

            listeEntrees.add( entreeOrdreDuJour );
        }
    }
}
