/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.ordredujour;

import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.ordredujour.EntreeOrdreDuJourFilter;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.portal.service.plugin.Plugin;

import java.util.List;


/**
 * 
 * IEntreeOrdreDuJourHome
 * 
 * @param <GOrdreDuJour> {@link IOrdreDuJourr}
 * @param <GEntreeOrdreDuJour> {@link IEntreeOrdreDuJour}
 * @param <GElu> {@link IElu}
 * @param <GSeance> {@link ISeance}
 * @param <GFichier> {@link IFichier}
 */
public interface IEntreeOrdreDuJourHome
<
	GOrdreDuJour extends IOrdreDuJour<GEntreeOrdreDuJour, GSeance, GFichier, GElu, GOrdreDuJour>, 
	GEntreeOrdreDuJour extends IEntreeOrdreDuJour<GElu, GSeance, GFichier, GEntreeOrdreDuJour, GOrdreDuJour>, 
	GElu extends IElu, 
	GSeance extends ISeance, 
	GFichier extends IFichier<GSeance, GFichier>
>
{
    /**
     * insere l'entre d'ordre du jour pass en parametre dans la table ods_entree_ordre_jour
     * @param entree EntreeOrdreDuJour
     * @param plugin Plugin
     * @return l'id de la nouvelle entre
     */
    int create( GEntreeOrdreDuJour entree, Plugin plugin );

    /**
     * renvoie l'entre d'ordre du jour d'id nKey
     * @param nKey int
     * @param plugin Plugin
     * @return EntreeOrdreDuJour entree
     */
    GEntreeOrdreDuJour findByPrimaryKey( int nKey, Plugin plugin );

    /**
     * supprime l'entre d'ordre du jour passe en parametre
     * @param entree EntreeOrdreDuJour
     * @param plugin Plugin
     */
    void remove( GEntreeOrdreDuJour entree, Plugin plugin );

    /**
     *        modifie l'entre d'ordre du jour transmise en parametre
     * @param entree EntreeOrdreDuJour
     * @param plugin Plugin
     */
    void update( GEntreeOrdreDuJour entree, Plugin plugin );

    /**
     * renvoie la liste d'entres de l'ordre du jour ayant comme id nKey
     * @param nKey int l'id de l'ordre du jour
     * @param plugin Plugin
     * @return la liste d'entres de l'ordre du jour d'id nKey
     */
    List<GEntreeOrdreDuJour> findEntreesListByIdOrdreDuJour( int nKey, Plugin plugin );

    /**
     * renvoie la liste d'entres de l'ordre du jour ayant comme id nKey
     * @param nKey int l'id de l'ordre du jour
     * @param plugin Plugin
     * @return le nombre d'entres de l'ordre du jour d'id nKey
     */
    int findCountEntreesListByIdOrdreDuJour( int nKey, Plugin plugin );

    /**
     * Retourne le nombre d'entres de l'ordre du jour rpondant au filtre
     * @param filter le filtre
     * @param plugin le plugin
     * @return le nombre d'entres de l'ordre du jour rpondant au filtre
     */
    int findCountEntreesByFilter( EntreeOrdreDuJourFilter filter, Plugin plugin );

    /**
     * Renvoie la liste d'entres de l'ordre du jour correspondant aux filtres appliqus
     * @param filter Filtre sur les entres d'ordre du jour
     * @param plugin le plugin ODS
     * @return la liste d'entres de l'ordre du jour correspondant aux filtres appliqus
     */
    List<GEntreeOrdreDuJour> findEntreesListByFilter( EntreeOrdreDuJourFilter filter, Plugin plugin );

    /**
     * Insere un rapporteur  l 'entre d'id nIdEntree,
     * @param nIdEntree l'id de l'entre
     * @param elu l'elu  insrer
     * @param plugin le Plugin
     */
    void insertRapporteur( int nIdEntree, GElu elu, Plugin plugin );

    /**
     *Supprime tous les rapporteurs lis  l'entre d'id nIdEntree,
     * @param nIdEntree l'id de l'entre
     * @param plugin le Plugin
     */
    void deleteRapporteur( int nIdEntree, Plugin plugin );

    /** Retourne la liste des Rapporteur ajout a l'entree de l'ordre du jour
     *
     * @param nKey id de l'entree de l'ordre du jour
     * @param plugin Plugin
     * @return la liste des lus
     */
    List<GElu> findRapporteursbyIdEntree( int nKey, Plugin plugin );
}
