/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.ordredujour;

import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.portal.service.plugin.Plugin;

import org.springframework.beans.factory.annotation.Autowired;

import java.sql.Timestamp;

import java.util.List;


/**
 * OrdreDuJourHome
 * @param <GOrdreDuJour> {@link IOrdreDuJour}
 * @param <GEntreeOrdreDuJour> {@link IEntreeOrdreDuJour}
 * @param <GOrdreDuJourFilter> {@link IOrdreDuJourFilter}
 * @param <GSeance> {@link ISeance}
 * @param <GElu> {@link IElu}
 * @param <GFichier> {@link IFichier}
 */
public final class OrdreDuJourHome
<
	GOrdreDuJour extends IOrdreDuJour<GEntreeOrdreDuJour, GSeance, GFichier, GElu, GOrdreDuJour>, 
	GEntreeOrdreDuJour extends IEntreeOrdreDuJour<GElu, GSeance, GFichier, GEntreeOrdreDuJour, GOrdreDuJour>, 
	GOrdreDuJourFilter extends IOrdreDuJourFilter, 
	GSeance extends ISeance, 
	GElu extends IElu, 
	GFichier extends IFichier<GSeance, GFichier>
>
implements IOrdreDuJourHome<GOrdreDuJour, GEntreeOrdreDuJour, GOrdreDuJourFilter, GSeance, GElu, GFichier>
{
    @Autowired
    private IOrdreDuJourDAO<GOrdreDuJour, GEntreeOrdreDuJour, GOrdreDuJourFilter, GSeance, GElu, GFichier> _ordreDuJourDAO;

    /**
     * Constructeur vide
     */
    private OrdreDuJourHome(  )
    {
    }

    /**
     * cr un nouvel ordre du jour
     * @param odj OrdreDuJour
     * @param plugin Plugin
     * @return l'id de l'odj cr
     */
    public int create( GOrdreDuJour odj, Plugin plugin )
    {
        return _ordreDuJourDAO.insert( odj, plugin );
    }

    /**
     * retourne l'ordre du jour d'id nKey
     * @param nKey int
     * @param plugin Plugin
     * @return l'ordre du jour d'id nKey
     */
    public GOrdreDuJour findByPrimaryKey( int nKey, Plugin plugin )
    {
        return _ordreDuJourDAO.load( nKey, plugin );
    }

    /**
     * supprime l'ordre du jour pass en parametre
     * @param nIdOrdreDuJour OrdreDuJour
     * @param plugin Plugin
     */
    public void remove( int nIdOrdreDuJour, Plugin plugin )
    {
        _ordreDuJourDAO.delete( nIdOrdreDuJour, plugin );
    }

    /**
     *        modifie l'ordre du jour odj
     * @param odj OrdreDuJour
     * @param plugin Plugin
     */
    public void update( GOrdreDuJour odj, Plugin plugin )
    {
        _ordreDuJourDAO.store( odj, plugin );
    }

    /**
    * renvoie la liste des ordres du jour rpondant au filtre filter pass en parametre
     * @param filter le filtre appliqu pour la recherche
     * @param bFrontOffice true si la liste est appelle par le front office false sinon
    * @param plugin Plugin
    * @return la liste des ordres du jour rpondant au filtre filter
    */
    public List<GOrdreDuJour> findOrdreDuJourList( Plugin plugin, GOrdreDuJourFilter filter, boolean bFrontOffice )
    {
        return _ordreDuJourDAO.selectOrdreDuJourList( plugin, filter, bFrontOffice );
    }

    /**
     * Test si un odj rpondant au filtre 'filter' existe en base.
     * @param filter le filtre appliqu pour la recherche
     * @param plugin  plugin
     * @return true si un odj rpondant au filtre 'filter' existe en base, false sinon
     */
    public boolean isAlreadyExist( GOrdreDuJourFilter filter, Plugin plugin )
    {
        return _ordreDuJourDAO.isAlreadyExist( filter, plugin );
    }

    /**
     * renvoie l'odj rpondant au filtre 'filter' .
     * @param filter le filtre de l'odj
     * @param plugin Plugin
     * @return OrdreDuJour ordreDuJour
     */
    public GOrdreDuJour findByOdjFilter( GOrdreDuJourFilter filter, Plugin plugin )
    {
        return _ordreDuJourDAO.findByOdjFilter( filter, plugin );
    }

    /**
     * renvoie l'odj de reference rpondant au filtre 'filter'
     * lordre du jour de rfrence est le dernier ordre du jour qui a t publi..
     * @param filter le filtre de l'odj
     * @param plugin Plugin
     * @return OrdreDuJour ordreDuJour
     */
    public GOrdreDuJour findOdjReferenceByFilter( GOrdreDuJourFilter filter, Plugin plugin )
    {
        return _ordreDuJourDAO.findOdjReferenceByFilter( filter, plugin );
    }

    /**
     * Retourne les ODJ dont l'une des entres a cet lu dans ses rapporteurs
     * @param nIdRapporteur l'id de l'u rapporteur
     * @param bPourPdd l'odj est-il pr un pdd ?
     * @param plugin le plugin
     * @return les ODJ filtrs par rapporteur
     */
    public List<GOrdreDuJour> findOdjByRapporteur( int nIdRapporteur, boolean bPourPdd, Plugin plugin )
    {
        return _ordreDuJourDAO.selectByRapporteur( nIdRapporteur, bPourPdd, plugin );
    }

    /**
     * Enregistre la date de dbut du processus de publication
     * @param nIdOdj l'id de l'ordre du jour en cours de publication
     * @param dateDebutPublication la date de dbut de publication
     * @param plugin Plugin
     */
    public void updateDateDebutPublication( int nIdOdj, Timestamp dateDebutPublication, Plugin plugin )
    {
        _ordreDuJourDAO.storeDateDebutPublication( nIdOdj, dateDebutPublication, plugin );
    }
}
