/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.panier;

import fr.paris.lutece.plugins.ods.dto.panier.Panier;
import fr.paris.lutece.plugins.ods.utils.commons.DateUtils;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.util.sql.DAOUtil;

import java.sql.Timestamp;

import java.util.ArrayList;
import java.util.List;


/**
 * Permet l'accs aux donnes de la table ods_panier.
 */
public class PanierDAO implements IPanierDAO
{
    private static final String SQL_QUERY_NEW_PK = "SELECT max( id_element_panier ) FROM ods_panier ";
    private static final String SQL_QUERY_INSERT = "INSERT INTO ods_panier (id_element_panier, id_document, id_utilisateur, date_ajout, est_zip) " +
        " VALUES (?,?,?,?,?) ";
    private static final String SQL_QUERY_DELETE_ELEMENTS_PANIER = "DELETE FROM ods_panier WHERE id_utilisateur = ? AND id_document = ? AND est_zip = false ";
    private static final String SQL_QUERY_FIND_ELEMENTS_PANIER_BY_USER = "SELECT id_element_panier, id_document, id_utilisateur, date_ajout, est_zip FROM ods_panier WHERE id_utilisateur = ? AND est_zip = false ORDER BY date_ajout ASC ";
    private static final String SQL_QUERY_FIND_ZIP_BY_USER = "SELECT id_element_panier, id_document, id_utilisateur, date_ajout, est_zip FROM ods_panier WHERE id_utilisateur = ? AND est_zip = true ORDER BY date_ajout ASC ";
    private static final String SQL_QUERY_FIND_ZIP_BY_DATE_AJOUT = "SELECT id_element_panier, id_document, id_utilisateur, date_ajout, est_zip FROM ods_panier WHERE est_zip = true AND date_ajout = ? ";
    private static final String SQL_QUERY_FIND_ALL_ZIP = "SELECT id_element_panier, id_document, id_utilisateur, date_ajout, est_zip FROM ods_panier ";
    private static final String SQL_QUERY_DELETE_ALL_PANIER = "DELETE FROM ods_panier WHERE est_zip = 1 ";
    private static final String SQL_QUERY_DELETE_ALL_FICHIER_PANIER = "DELETE FROM ods_fichier_panier ";
    private static final String SQL_QUERY_DELETE_ALL_FICHIER_PHYSIQUE_PANIER = "DELETE FROM ods_fichier_physique_panier ";
    private static final String SQL_QUERY_FIND_ELEMENT_PANIER_BY_ID_DOCUMENT = "SELECT id_element_panier FROM ods_panier WHERE id_document = ? ";
    private static final String SQL_QUERY_DELETE_DOCUMENT_FROM_PANIER = "DELETE FROM ods_panier WHERE id_document = ?";

    /**
     * Cration de l'element du panier qui possede l'id de l'utilisateur et l'id du fichier pass en paramtre
     * @param idUser id du user
     * @param idFichier id du fichier
     * @param plugin le plugin actif
     */
    public void createElementPanier( String idUser, int idFichier, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_INSERT, plugin );
        daoUtil.setInt( 1, newPrimaryKey( plugin ) );
        daoUtil.setInt( 2, idFichier );
        daoUtil.setString( 3, idUser );
        daoUtil.setTimestamp( 4, DateUtils.getCurrentDate(  ) );
        daoUtil.setBoolean( 5, false );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Supprime l'element du panier qui possede l'id de l'utilisateur et l'id du fichier pass en paramtre
     * @param idUser id du user
     * @param idFichier id du fichier
     * @param plugin le plugin actif
     */
    public void deleteElementPanier( String idUser, int idFichier, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE_ELEMENTS_PANIER, plugin );
        daoUtil.setString( 1, idUser );
        daoUtil.setInt( 2, idFichier );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Retourne Tableau contenant les ids des lments du panier appartenant au user pass en paramtre
     *
     * @param idUser id du user
     * @param plugin le plugin actif
     * @return Tableau contenant les ids des lments du panier appartenant au user pass en paramtre
     */
    public String[] selectElementsPanierByIdUser( String idUser, Plugin plugin )
    {
        List<String> listIds = new ArrayList<String>(  );
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_ELEMENTS_PANIER_BY_USER, plugin );
        daoUtil.setString( 1, idUser );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            int idFichier = daoUtil.getInt( "id_document" );
            listIds.add( "" + idFichier );
        }

        daoUtil.free(  );

        String[] ids = new String[listIds.size(  )];
        int i = 0;

        for ( String s : listIds )
        {
            ids[i] = s;
            i++;
        }

        return ids;
    }

    /**
    * Retourne une nouvelle cl primaire unique sur la table ods_panier
    * @param plugin le plugin actif
    * @return nouvelle cl primaire
    */
    private int newPrimaryKey( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey;

        if ( !daoUtil.next(  ) )
        {
            nKey = 1;
        }

        nKey = daoUtil.getInt( 1 ) + 1;
        daoUtil.free(  );

        return nKey;
    }

    /**
     * Retourne toutes les archives de l'utilisateur pass en paramtre
     *
     * @param idUser id du user
     * @param plugin le plugin actif
     * @return Liste des zip - List<Panier> -qui ont t tlcharg par le user pass en paramtre
     */
    public List<Panier> selectZipByIdUser( String idUser, Plugin plugin )
    {
        List<Panier> paniers = new ArrayList<Panier>(  );
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_ZIP_BY_USER, plugin );
        daoUtil.setString( 1, idUser );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            Panier panier = new Panier(  );
            panier.setIdElementPanier( daoUtil.getInt( "id_element_panier" ) );
            panier.setIdDocument( daoUtil.getInt( "id_document" ) );
            panier.setIdUtilisateur( daoUtil.getString( "id_utilisateur" ) );
            panier.setDateAjout( daoUtil.getTimestamp( "date_ajout" ) );
            paniers.add( panier );
        }

        daoUtil.free(  );

        return paniers;
    }

    /**
     * Supprime tous les documents.
     *
     * @param plugin le plugin actif
     */
    public void deleteAll( Plugin plugin )
    {
        DAOUtil daoUtil;

        daoUtil = new DAOUtil( SQL_QUERY_DELETE_ALL_FICHIER_PHYSIQUE_PANIER, plugin );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );

        daoUtil = new DAOUtil( SQL_QUERY_DELETE_ALL_FICHIER_PANIER, plugin );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );

        daoUtil = new DAOUtil( SQL_QUERY_DELETE_ALL_PANIER, plugin );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Renvoie tous les documents.
     *
     * @param plugin le plugin actif
     * @return la liste des lements du panier
     */
    public List<Panier> selectAll( Plugin plugin )
    {
        List<Panier> paniers = new ArrayList<Panier>(  );
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_ALL_ZIP, plugin );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            Panier panier = new Panier(  );
            panier.setIdElementPanier( daoUtil.getInt( "id_element_panier" ) );
            panier.setIdDocument( daoUtil.getInt( "id_document" ) );
            panier.setIdUtilisateur( daoUtil.getString( "id_utilisateur" ) );
            panier.setDateAjout( daoUtil.getTimestamp( "date_ajout" ) );
            paniers.add( panier );
        }

        daoUtil.free(  );

        return paniers;
    }

    /**
     * Retourne l'archive ZIP qui  t tlcharg a la date pass en paramtre
     *
     * @param strDateAjout Date qui correspond  la demande du tlchargement de l'archive
     * @param plugin le plugin actif
     * @return L'element du Panier
     */
    public Panier loadZipByDateAjout( String strDateAjout, Plugin plugin )
    {
        Panier panier = null;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_ZIP_BY_DATE_AJOUT, plugin );
        daoUtil.setTimestamp( 1, new Timestamp( Long.parseLong( strDateAjout ) ) );
        daoUtil.executeQuery(  );

        if ( daoUtil.next(  ) )
        {
            panier = new Panier(  );
            panier.setIdElementPanier( daoUtil.getInt( "id_element_panier" ) );
            panier.setIdDocument( daoUtil.getInt( "id_document" ) );
            panier.setIdUtilisateur( daoUtil.getString( "id_utilisateur" ) );
            panier.setDateAjout( daoUtil.getTimestamp( "date_ajout" ) );
        }

        daoUtil.free(  );

        return panier;
    }

    /**
     * Retourne tous les paniers lis au document
     *
     * @param plugin le plugin actif
     * @param nIdDocument le document recherch
     * @return Liste des paniers lis au document
     */
    public List<String> selectPanierByDocument( int nIdDocument, Plugin plugin )
    {
        List<String> listPanier = new ArrayList<String>(  );
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_ELEMENT_PANIER_BY_ID_DOCUMENT, plugin );
        daoUtil.setInt( 1, nIdDocument );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            int idFichier = daoUtil.getInt( "id_element_panier" );
            listPanier.add( "" + idFichier );
        }

        daoUtil.free(  );

        return listPanier;
    }

    /**
     * Supprime un document du panier
     *
     * @param plugin le plugin actif
     * @param nIdDocument le document recherch
     */
    public void deleteDocumentFromPanier( int nIdDocument, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE_DOCUMENT_FROM_PANIER, plugin );
        daoUtil.setInt( 1, nIdDocument );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }
}
