/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.panier;

import fr.paris.lutece.plugins.ods.dto.panier.Panier;
import fr.paris.lutece.portal.service.plugin.Plugin;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;


/**
 * Cette classe permet de grer des objets Paniers.
 */
public final class PanierHome
{
    @Autowired
    private IPanierDAO _panierDAO;

    /**
     * Constructeur vide
     */
    private PanierHome(  )
    {
    }

    /**
     * Retourne Tableau contenant les ids des lments du panier appartenant au user pass en paramtre
     *
     * @param idUser id du user
     * @param plugin le plugin actif
     * @return Tableau contenant les ids des lments du panier appartenant au user pass en paramtre
     */
    public String[] findElementPanierByIdUser( String idUser, Plugin plugin )
    {
        return _panierDAO.selectElementsPanierByIdUser( idUser, plugin );
    }

    /**
     * Retourne toutes les archives de l'utilisateur pass en paramtre
     *
     * @param idUser id du user
     * @param plugin le plugin actif
     * @return Liste des zip - List<Panier> -qui ont t tlcharg par le user pass en paramtre
     */
    public List<Panier> findZipByIdUser( String idUser, Plugin plugin )
    {
        return _panierDAO.selectZipByIdUser( idUser, plugin );
    }

    /**
     * Retourne toutes les archives
     *
     * @param plugin le plugin actif
     * @return Liste des zip - List<Panier>
     */
    public List<Panier> findAll( Plugin plugin )
    {
        return _panierDAO.selectAll( plugin );
    }

    /**
     * Supprime l'element du panier qui possede l'id de l'utilisateur et l'id du fichier pass en paramtre
     * @param idUser id du user
     * @param idFichier id du fichier
     * @param plugin le plugin actif
     */
    public void removeElementPanier( String idUser, int idFichier, Plugin plugin )
    {
        _panierDAO.deleteElementPanier( idUser, idFichier, plugin );
    }

    /**
    * Cration de l'element du panier qui possede l'id de l'utilisateur et l'id du fichier pass en paramtre
    * @param idUser id du user
    * @param idFichier id du fichier
    * @param plugin le plugin actif
    */
    public void addElementPanier( String idUser, int idFichier, Plugin plugin )
    {
        _panierDAO.createElementPanier( idUser, idFichier, plugin );
    }

    /**
     * Retourne l'archive ZIP qui  t tlcharg a la date pass en paramtre
     *
     * @param dateAjout Date qui correspond  la demande du tlchargement de l'archive
     * @param plugin le plugin actif
     * @return L'element du Panier
     */
    public Panier findZipByDateAjout( String dateAjout, Plugin plugin )
    {
        return _panierDAO.loadZipByDateAjout( dateAjout, plugin );
    }

    /**
     * Supprime tous les documents.
     *
     * @param plugin le plugin actif
     */
    public void removeAll( Plugin plugin )
    {
        _panierDAO.deleteAll( plugin );
    }

    /**
     * Retourne le panier contennat le document spcifi en paramtre
     * @param idDocument le document
     * @param plugin Le plugin
     * @return La liste des paniers contenant le document
     */
    public List<String> findPanierByDocuments( int idDocument, Plugin plugin )
    {
        return _panierDAO.selectPanierByDocument( idDocument, plugin );
    }

    /**
     * Supprime le document spcifi en paramtre du panier
     * @param nIdDocument Le document
     * @param plugin Le plugin
     */
    public void removeDocument( int nIdDocument, Plugin plugin )
    {
        _panierDAO.deleteDocumentFromPanier( nIdDocument, plugin );
    }
}
