/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.pdd;

import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.pdd.DirectionCoEmetrice;
import fr.paris.lutece.plugins.ods.dto.pdd.IArrondissement;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDDFilter;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendementFilter;
import fr.paris.lutece.portal.service.plugin.Plugin;

import java.sql.Timestamp;

import java.util.List;


/**
 * PDDDAO est responsable de la communication avec la BDD
 * @param <GPDD> {@link IPDD}
 * @param <GVoeuAmendementFilter> {@link IVoeuAmendementFilter}
 * @param <GPDDFilter> {@link IPDDFilter}
 * @param <GSeance> {@link ISeance}
 * @param <GElu> {@link IElu}
 * @param <GFichier> {@link IFichier}
 * @param <GVoeuAmendement> {@link IVoeuAmendement} 
 * @param <GArrondissement> {@link IArrondissement}
 *
 */
public interface IPDDDAO
<
	GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, 
	GPDDFilter extends IPDDFilter, 
	GVoeuAmendementFilter extends IVoeuAmendementFilter, 
	GSeance extends ISeance, 
	GElu extends IElu, 
	GFichier extends IFichier<GSeance, GFichier>, 
	GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, 
	GArrondissement extends IArrondissement
>
{
    /**
    * renvoie la liste des pdd de la prochaine seance
    * @param bIsSimple TRUE si on ne rcupre que les attributs de la table ods_pdd, FALSE sinon
    * @param filter PDDFilter
    * @param plugin Plugin
    * @return List<PDD> liste d'objets Fichier
    */
    List<GPDD> selectByFilter( boolean bIsSimple, GPDDFilter filter, Plugin plugin );

    /**
    * renvoie la liste des identifiants de pdd de la prochaine seance
    * @param plugin Plugin
    * @param filter PDDFilter
    * @return List<int> liste d'identifiants de Fichier
    */
    List<Integer> selectIdByFilter( GPDDFilter filter, Plugin plugin );

    /**
    * Renvoie le nombre de PDD rpondant aux filtres
    * @param plugin Plugin
    * @param filter PDDFilter
    * @return le nombre de PDD
    */
    int selectCountByFilter( GPDDFilter filter, Plugin plugin );

    /**
     * renvoie la liste des pdd de la prochaine seance
     * @param plugin Plugin
     * @param filter PDDFilter
     * @param  nIdDirection nIdDirection
     * @return List<PDD> liste d'objets Fichier
     */
    List<GPDD> selectByFilterByDirectionOrDirectionCoemetrice( GPDDFilter filter, Plugin plugin, int nIdDirection );

    /**
     * renvoie la liste des identifiants de pdd de la prochaine seance
     * @param plugin Plugin
     * @param filter PDDFilter
     * @param  nIdDirection nIdDirection
     * @return List<PDD> liste d'identifiants de Fichier
     */
    List<Integer> selectIdByFilterByDirectionOrDirectionCoemetrice( GPDDFilter filter, Plugin plugin, int nIdDirection );

    /**
     * renvoie la liste des identifiants de pdd de la prochaine seance
     * @param plugin Plugin
     * @param filter PDDFilter
     * @param  nIdDirection nIdDirection
     * @return le nombre de pdd correspondant
     */
    int selectCountByFilterByDirectionOrDirectionCoemetrice( GPDDFilter filter, Plugin plugin, int nIdDirection );

    /**
     * renvoie le pdd en fonction de l'id passe en parametre
     * @param nKey int id du PDD
     * @param plugin Plugin
     * @return PDD le pdd qui a ete trouv
     */
    GPDD selectByPrimaryKey( int nKey, Plugin plugin );

    /**
     * renvoie le pdd en fonction de l'id passe en parametre
     * @param nKey int id du PDD
     * @param plugin Plugin
     * @return PDD le pdd qui a ete trouv
     */
    GPDD selectByPrimaryKeyForODJ( int nKey, Plugin plugin );

    /**
     * Cre un nouveau PDD  partir de lobjet pdd pass en paramtre
     * @param pdd la Projet de deliberation  insrer
     * @param plugin Plugin
     * @return ProjetDeDeliberation pdd creer
     */
    GPDD insert( GPDD pdd, Plugin plugin );

    /**
     * Modifie un PDD  partir de lobjet pdd pass en paramtre
     * @param pdd la Projet de deliberation  modifier
     * @param plugin Plugin
     */
    void store( GPDD pdd, Plugin plugin );

    /**
     * Supprime le PDD dont l'id est celui pass en paramtre
     * @param pdd le pdd
     * @param plugin Plugin
     */
    void delete( GPDD pdd, Plugin plugin );

    /**
     * Permet de publier ou de dpublier un pdd
     * Publication d'un pdd
     * Si isPublier gale  TRUE, alors la mthode <b>publie</b> le pdd et incrmente la version<BR>
     * Sinon la mthode change le statut enLigne d'un pdd a FALSE
     *
     * @param nKey id du pdd
     * @param datePublication TimeStamp date de publication
     * @param version verssion
     * @param isPublier enLigne
     * @param plugin Plugin
     */
    void publication( int nKey, Timestamp datePublication, int version, boolean isPublier, Plugin plugin );

    /**
     * Cre une nouvelle Direction CoEmetrice  partir de lobjet directionCoEmetrice pass en paramtre
     * @param directionCoEmetrice La direction CoEmetrice  inserer
     * @param plugin Plugin
     */
    void insert( DirectionCoEmetrice directionCoEmetrice, Plugin plugin );

    /**
     * Cre un nouvel Arrondissement  partir de lobjet arrondissement pass en paramtre
     * @param arrondissement L'arrondissement  inserer
     * @param plugin Plugin
     */
    void insert( GArrondissement arrondissement, Plugin plugin );

    /**
     * renvoie la liste des Directions CoEmetrices de l'id du pdd pass en paramtre
     * @param pddPrimaryKey Id du pdd
     * @param plugin Plugin
     * @return List<Arrondissement> Liste d'objets DirectionCoEmetrice
     */
    List<DirectionCoEmetrice> selectDirectionsCoEmtricesByPDD( int pddPrimaryKey, Plugin plugin );

    /**
     * renvoie la liste des Arrondissements de l'id du pdd pass en paramtre
     * @param pddPrimaryKey Id du pdd
     * @param plugin Plugin
     * @return List<Arrondissement> Liste d'objets Arrondissement
     */
    List<GArrondissement> selectArrondissementsByPDD( int pddPrimaryKey, Plugin plugin );

    /**
     * Supprime toutes les arrondissements possedant l'id du pdd pass en paramtre
     * @param nIdPDD id du pdd
     * @param plugin Plugin
     */
    void deleteArrondissements( int nIdPDD, Plugin plugin );

    /**
     * Supprime toutes les directionCoEmetrice possedant l'id du pdd pass en paramtre
     * @param nIdPDD id du pdd
     * @param plugin Plugin
     */
    void deleteCoEmetrices( int nIdPDD, Plugin plugin );

    /**
     * Ajoute un VoeuxAmendement au pdd
     *
     * @param idVA id du Voeux amendement
     * @param idPdd id du pdd
     * @param nNumeroOrdre le numro d'ordre du va dans le pdd
     * @param plugin Plugin
     */
    void addVoeuAmendement( int idVA, int idPdd, int nNumeroOrdre, Plugin plugin );

    /**
     * Supprime tous les lements contenus dans oldVoeuAmendements
     * et insre les nouveaux veuxAmendements rattachs aux pdd
     *
     * @param oldVoeuAmendements List<VoeuAmendement>
     * @param newVoeuAmendements List<VoeuAmendement>
     * @param idPDD idPDD
     * @param plugin Plugin
     */
    void updateVoeuxAmendement( List<GVoeuAmendement> oldVoeuAmendements, List<GVoeuAmendement> newVoeuAmendements,
        int idPDD, Plugin plugin );

    /**
     * Retourne l'identifiant unique pour le couple ( arrondissement, pdd )
     * @param strArrondissement l'arrondissement
     * @param pdd le pdd
     * @param plugin le plugin
     * @return l'identifiant unique pour le couple ( arrondissement, pdd )
     */
    int selectIdArrondissement( String strArrondissement, GPDD pdd, Plugin plugin );

    /**
     * Modifie le numro d'ordre du va dans le pdd
     * @param nIdPdd l'id du pdd a rattache au VA
     * @param nIdVoeuAmendement l'id du VoeuAmendement
     * @param nNumeroOrdre numro ordre
     * @param plugin plugin
     */
    void updateVAInRattachePdd( int nIdPdd, int nIdVoeuAmendement, int nNumeroOrdre, Plugin plugin );
}
