/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.requete;

import fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.portal.service.plugin.Plugin;

import java.util.List;


/**
 * IRequeteUtilisateurDAO : interface pour grer les requtes sauvegardes par les utilisateurs
 * @param <GSeance> {@link ISeance}
 * @param <GSeanceFilter> {@link ISeanceFilter}
 * @param <GRequeteUtilisateur> {@link IRequeteUtilisateur}
 */
public interface IRequeteUtilisateurDAO<GSeance extends ISeance, GSeanceFilter extends ISeanceFilter, GRequeteUtilisateur extends IRequeteUtilisateur<GSeance, GSeanceFilter>>
{
    /**
     * Sauvegarde une nouvelle requte utilisateur dans la base
     * @param requete la requte utilisateur
     * @param plugin le plugin
     */
    void insert( GRequeteUtilisateur requete, Plugin plugin );

    /**
     * Supprime une requte utilisateur sauvegarde
     * @param requete la requte utilisateur  supprimer
     * @param plugin le plugin
     */
    void delete( GRequeteUtilisateur requete, Plugin plugin );

    /**
     * Met  jour les informations de la requte utilisateur<br/>
     * Attention cette mthode <u><b>ne modifie pas le nombre, ou la valeur des critres</b></u>
     * @param requete la requete  modifier
     * @param plugin le plugin
     */
    void update( GRequeteUtilisateur requete, Plugin plugin );

    /**
     * Retourne la requte utilisateur identifie par nKey
     * @param nKey l'identifiant de la requte
     * @param plugin le plugin
     * @return la requte utilisateur identifie par nKey
     */
    GRequeteUtilisateur load( int nKey, Plugin plugin );

    /**
     * Retourne la liste des requtes sauvegardes par un utilisateur
     * @param strUserName l'utilisateur
     * @param plugin le plugin
     * @return la liste des requtes de l'utilisateur
     */
    List<GRequeteUtilisateur> selectRequetesByUser( String strUserName, Plugin plugin );
}
