/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.search;

import fr.paris.lutece.plugins.ods.business.categoriedeliberation.CategorieDeliberationHome;
import fr.paris.lutece.plugins.ods.business.fichier.IFichierHome;
import fr.paris.lutece.plugins.ods.business.formationconseil.FormationConseilHome;
import fr.paris.lutece.plugins.ods.business.seance.ISeanceHome;
import fr.paris.lutece.plugins.ods.dto.arrete.Arrete;
import fr.paris.lutece.plugins.ods.dto.deliberationdesignation.DeliberationDesignation;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter;
import fr.paris.lutece.plugins.ods.dto.fichier.ITypeDocument;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.portal.service.plugin.Plugin;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;


/**
 *
 * AbstractSearchHome
 *
 * @param <GFichier> {@link IFichier}
 * @param <GSeance> {@link ISeance}
 * @param <GSeanceFilter> {@link ISeanceFilter}
 * @param <GRequeteUtilisateur> {@link IRequeteUtilisateur}
 * @param <GElu> {@link IElu}
 * @param <GVoeuAmendement> {@link IVoeuAmendement}
 * @param <GPDD> {@link IPDD}
 * @param <GSearchDAO> {@link ISearchDAO}
 */
public abstract class AbstractSearchHome
<
	GFichier extends IFichier<GSeance, GFichier>, 
	GSeance extends ISeance, 
	GSeanceFilter extends ISeanceFilter, 
	GRequeteUtilisateur extends IRequeteUtilisateur<GSeance, GSeanceFilter>, 
	GSearchDAO extends ISearchDAO<GFichier, GSeance, GSeanceFilter, GRequeteUtilisateur, GElu, GVoeuAmendement, GPDD>, 
	GElu extends IElu, 
	GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, 
	GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>
>
implements ISearchHome<GFichier, GSeance, GSeanceFilter, GRequeteUtilisateur, GSearchDAO, GElu, GVoeuAmendement, GPDD>
{
    @Autowired
    private GSearchDAO _searchDAO;
    @Autowired
    private CategorieDeliberationHome _categorieDeliberationHome;
    @Autowired
    private IFichierHome<GFichier, IFichierFilter, ITypeDocument, GSeance> _fichierHome;
    @Autowired
    private FormationConseilHome _formationConseilHome;
    @Autowired
    private ISeanceHome<GSeance, GSeanceFilter, GFichier, GElu, GVoeuAmendement, GPDD> _seanceHome;

    /**
     * Constructeur vide
     */
    protected AbstractSearchHome(  )
    {
    }

    /**
     * Retourne la liste des projets de dlibration recherchs par l'utilisateur
     * @param requete la requte de l'utilisateur
     * @param prochaineSeance la prochaine sance
     * @param listIdsProjets la liste des ids de projets trouvs dans l'index et dans lesquels on retrouve le mot recherch
     *                                 (<code>null</code> si pas de recherche par mot-cl, vide si mot-cl non-trouv)
     * @param plugin le plugin
     * @return la liste des projets de dlibration recherchs par l'utilisateur
     */
    public List<GPDD> findProjetsDeliberation( GRequeteUtilisateur requete, GSeance prochaineSeance,
        List<String> listIdsProjets, Plugin plugin )
    {
        return _searchDAO.selectProjetsDeliberation( requete, prochaineSeance, listIdsProjets, plugin );
    }

    /**
     * Retourne la liste des propositions de dlibration recherchs par l'utilisateur
     * @param requete la requte de l'utilisateur
     * @param prochaineSeance la prochaine sance
     * @param listIdsPropositions la liste des ids de projets trouvs dans l'index et dans lesquels on retrouve le mot recherch
     *                                 (<code>null</code> si pas de recherche par mot-cl, vide si mot-cl non-trouv)
     * @param plugin le plugin
     * @return la liste des propositions de dlibration recherchs par l'utilisateur
     */
    public List<GPDD> findPropositionsDeliberation( GRequeteUtilisateur requete, GSeance prochaineSeance,
        List<String> listIdsPropositions, Plugin plugin )
    {
        return _searchDAO.selectPropositionsDeliberation( requete, prochaineSeance, listIdsPropositions, plugin );
    }

    /**
     * Retourne la liste des amendements filtrs par les critres de recherche
     * @param requete la requte utilisateur
     * @param prochaineSeance la prochaine sance
     * @param listIdsAmendements la liste des amendements retourns par la recherche dans l'index
     *                         (<code>null</code> si pas de mot-cl renseign)
     * @param plugin le plugin
     * @return la liste des {@link VoeuAmendement} de type Amendement ou Lettre Rectificative recherchs
     */
    public List<GVoeuAmendement> findAmendements( GRequeteUtilisateur requete, GSeance prochaineSeance,
        List<String> listIdsAmendements, Plugin plugin )
    {
        return _searchDAO.selectAmendements( requete, prochaineSeance, listIdsAmendements, plugin );
    }

    /**
     * Retourne la liste des voeux filtrs par les critres de recherche
     * @param requete la requte utilisateur
     * @param prochaineSeance la prochaine sance
     * @param listIdsVoeux la liste des voeux retourns par la recherche dans l'index
     *                         (<code>null</code> si pas de mot-cl renseign)
     * @param plugin le plugin
     * @return la liste des {@link VoeuAmendement} de type Voeu ou Voeu Non Rattach recherchs
     */
    public List<GVoeuAmendement> findVoeux( GRequeteUtilisateur requete, GSeance prochaineSeance,
        List<String> listIdsVoeux, Plugin plugin )
    {
        return _searchDAO.selectVoeux( requete, prochaineSeance, listIdsVoeux, plugin );
    }

    /**
     * Retourne la liste des documents de sance filtrs par les critres de recherche
     * @param requete la requte utilisateur
     * @param listIdsFichiers la liste des identifiants des fichiers
     * @param plugin le plugin ODS
     * @return la liste des {@link Fichier} recherchs
     */
    public List<GFichier> findAutresDocuments( GRequeteUtilisateur requete, List<String> listIdsFichiers, Plugin plugin )
    {
        return _searchDAO.selectAutresDocuments( requete, listIdsFichiers, plugin );
    }

    /**
     * Retourne la liste des arrets filtrs par les critres de recherche
     * @param requete la requte utilisateur
     * @param listIdsArretes la liste des identifiants des arretes
     * @param plugin le plugin ODS
     * @return la liste des arretes recherchs
     */
    public List<Arrete<GSeance, GFichier>> findArretes( GRequeteUtilisateur requete, List<String> listIdsArretes,
        Plugin plugin )
    {
        List<Arrete<GSeance, GFichier>> listArretes = _searchDAO.selectArretes( requete, listIdsArretes, plugin );

        for ( Arrete<GSeance, GFichier> arrete : listArretes )
        {
            arrete.setFichierOr( _fichierHome.findByPrimaryKey( arrete.getFichierOr(  ).getId(  ), plugin ) );
            arrete.setFichierSc( _fichierHome.findByPrimaryKey( arrete.getFichierSc(  ).getId(  ), plugin ) );

            arrete.setFormationConseil( _formationConseilHome.findByPrimaryKey( 
                    arrete.getFormationConseil(  ).getIdFormationConseil(  ), plugin ) );

            arrete.setCategorie( _categorieDeliberationHome.findByPrimaryKey( 
                    arrete.getCategorie(  ).getIdCategorie(  ), plugin ) );

            arrete.setSeance( _seanceHome.findByPrimaryKey( arrete.getSeance(  ).getIdSeance(  ), plugin ) );
        }

        return listArretes;
    }

    /**
     * Retourne la liste des dlibration de dsignation filtres par les critres de recherche
     * @param requete la requte utilisateur
     * @param listIdsDeliberationsDesignation la liste des identifiants des dlibration de dsignation
     * @param plugin le plugin ODS
     * @return la liste des dlibration de dsignation recherches
     */
    public List<DeliberationDesignation<GSeance, GFichier>> findDeliberationsDesignation( GRequeteUtilisateur requete,
        List<String> listIdsDeliberationsDesignation, Plugin plugin )
    {
        List<DeliberationDesignation<GSeance, GFichier>> listDeliberation = _searchDAO.selectDeliberationsDesignation( requete,
                listIdsDeliberationsDesignation, plugin );

        for ( DeliberationDesignation<GSeance, GFichier> deliberation : listDeliberation )
        {
            if ( deliberation.getFichierOr(  ) != null )
            {
                deliberation.setFichierOr( _fichierHome.findByPrimaryKey( deliberation.getFichierOr(  ).getId(  ),
                        plugin ) );
            }

            deliberation.setFichierSc( _fichierHome.findByPrimaryKey( deliberation.getFichierSc(  ).getId(  ), plugin ) );

            deliberation.setFormationConseil( _formationConseilHome.findByPrimaryKey( 
                    deliberation.getFormationConseil(  ).getIdFormationConseil(  ), plugin ) );

            deliberation.setCategorie( _categorieDeliberationHome.findByPrimaryKey( 
                    deliberation.getCategorie(  ).getIdCategorie(  ), plugin ) );

            deliberation.setSeance( _seanceHome.findByPrimaryKey( deliberation.getSeance(  ).getIdSeance(  ), plugin ) );
        }

        return listDeliberation;
    }

    /**
     * Retourne la liste des dlibrations de dsignation  indexer
     * @param plugin le plugin
     * @return une liste de dlibrations de dsignations
     */
    public List<DeliberationDesignation<GSeance, GFichier>> findDeliberationDesignationAIndexer( Plugin plugin )
    {
        List<DeliberationDesignation<GSeance, GFichier>> listDeliberation = _searchDAO.selectDeliberationDesignationAIndexer( plugin );

        for ( DeliberationDesignation<GSeance, GFichier> deliberation : listDeliberation )
        {
            if ( deliberation.getFichierOr(  ) != null )
            {
                deliberation.setFichierOr( _fichierHome.findByPrimaryKey( deliberation.getFichierOr(  ).getId(  ),
                        plugin ) );
            }

            deliberation.setFichierSc( _fichierHome.findByPrimaryKey( deliberation.getFichierSc(  ).getId(  ), plugin ) );

            deliberation.setFormationConseil( _formationConseilHome.findByPrimaryKey( 
                    deliberation.getFormationConseil(  ).getIdFormationConseil(  ), plugin ) );

            deliberation.setCategorie( _categorieDeliberationHome.findByPrimaryKey( 
                    deliberation.getCategorie(  ).getIdCategorie(  ), plugin ) );

            deliberation.setSeance( _seanceHome.findByPrimaryKey( deliberation.getSeance(  ).getIdSeance(  ), plugin ) );
        }

        return listDeliberation;
    }

    /**
     * Retourne la liste des fichiers du PDD correspondant aux critres de recherche
     * @param listIdsFichiers la liste des ids de fichier correspondants aux critres de recherche (<code>null</code> si pas de mot-cl)
     * @param nIdPdd l'id du pdd
     * @param plugin le plugin
     * @return une liste de fichiers
     */
    public List<GFichier> findFichiersDuPdd( List<String> listIdsFichiers, int nIdPdd, Plugin plugin )
    {
        return _searchDAO.selectFichiersDuPdd( listIdsFichiers, nIdPdd, plugin );
    }

    /**
     * Retourne la liste des Pdds  indexer
     * @param bArchive indique si l'on veut les Pdds archivs ou de la prochaine sance
     * @param prochaineSeance la prochaine sance (<code>null</code> si pas de prochaine sance)
     * @param plugin le plugin
     * @return une liste de Pdds
     */
    public List<GPDD> findPDDsAIndexer( boolean bArchive, GSeance prochaineSeance, Plugin plugin )
    {
        return _searchDAO.selectPDDsAIndexer( bArchive, prochaineSeance, plugin );
    }

    /**
     * Retourne la liste des Pdds  ajouter dans l'index des archives
     * @param nIdAncienneSeance l'id de la dernire sance coule
     * @param plugin le plugin
     * @return une liste de Pdds
     */
    public List<GPDD> findPDDsChangementSeance( int nIdAncienneSeance, Plugin plugin )
    {
        return _searchDAO.selectPDDsChangementSeance( nIdAncienneSeance, plugin );
    }

    /**
     * Retourne la liste des voeux et amendements  indexer
     * @param bArchive si archive
     * @param prochaineSeance la prochaine seance
     * @param plugin le plugin
     * @return une liste de VoeuAmendement
     */
    public List<GVoeuAmendement> findVAAIndexer( boolean bArchive, GSeance prochaineSeance, Plugin plugin )
    {
        return _searchDAO.selectVAAIndexer( bArchive, prochaineSeance, plugin );
    }

    /**
     * Retourne la liste des fichiers  indexer
     * @param plugin le plugin
     * @return la liste des fichiers
     */
    public List<GFichier> findFichiersAIndexer( Plugin plugin )
    {
        return _searchDAO.selectFichiersAIndexer( plugin );
    }

    /**
     * Retourne la liste des Voeux et Amendements  indexer dans les archives aprs un changement de sance
     * @param nIdAncienneSeance la dernire sance passe
     * @param plugin le plugin
     * @return une liste de VoeuAmendment
     */
    public List<GVoeuAmendement> findVAChangementSeance( int nIdAncienneSeance, Plugin plugin )
    {
        return _searchDAO.selectVAChangementSeance( nIdAncienneSeance, plugin );
    }

    /**
     * Retourne la liste des fichiers  indexer lors du changement de sance
     * @param nIdDerniereSeance l'id de la sance qui se termine
     * @param plugin le plugin
     * @return une liste de fichiers
     */
    public List<GFichier> findFichiersChangementSeance( int nIdDerniereSeance, Plugin plugin )
    {
        return _searchDAO.selectFichiersChangementSeance( nIdDerniereSeance, plugin );
    }
}
