/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.search;

import fr.paris.lutece.plugins.ods.dto.arrete.Arrete;
import fr.paris.lutece.plugins.ods.dto.deliberationdesignation.DeliberationDesignation;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.portal.service.plugin.Plugin;

import java.util.List;


/**
 *
 * ISearchDAO
 *
 * @param <GFichier> {@link IFichier}
 * @param <GSeance> {@link ISeance}
 * @param <GSeanceFilter> {@link ISeanceFilter}
 * @param <GRequeteUtilisateur> {@link IRequeteUtilisateur}
 * @param <GElu> {@link IElu}
 * @param <GVoeuAmendement> {@link IVoeuAmendement}
 * @param <GPDD> {@link IPDD}
 */
public interface ISearchDAO
<
	GFichier extends IFichier<GSeance, GFichier>, 
	GSeance extends ISeance, 
	GSeanceFilter extends ISeanceFilter, 
	GRequeteUtilisateur extends IRequeteUtilisateur<GSeance, GSeanceFilter>, 
	GElu extends IElu, 
	GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, 
	GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>
>
{
    /**
     * Retourne la liste des projets filtrs par les critres de recherche
     * @param requete la requte utilisateur
     * @param prochaineSeance la prochaine sance
     * @param listIdsProjets la liste des PDDs retourns par la recherche dans l'index
     *                         (<code>null</code> si pas de mot-cl renseign)
     * @param plugin le plugin
     * @return la liste des {@link PDD} de type projet recherchs
     */
    List<GPDD> selectProjetsDeliberation( GRequeteUtilisateur requete, GSeance prochaineSeance,
        List<String> listIdsProjets, Plugin plugin );

    /**
     * Retourne la liste des propositions filtrs par les critres de recherche
     * @param requete la requte utilisateur
     * @param prochaineSeance la prochaine sance
     * @param listIdsPropositions la liste des PDDs retourns par la recherche dans l'index
     *                         (<code>null</code> si pas de mot-cl renseign)
     * @param plugin le plugin
     * @return la liste des {@link PDD} de type proposition recherchs
     */
    List<GPDD> selectPropositionsDeliberation( GRequeteUtilisateur requete, GSeance prochaineSeance,
        List<String> listIdsPropositions, Plugin plugin );

    /**
     * Retourne la liste des amendements filtrs par les critres de recherche
     * @param requete la requte utilisateur
     * @param prochaineSeance la prochaine sance
     * @param listIdsAmendements la liste des amendements retourns par la recherche dans l'index
     *                         (<code>null</code> si pas de mot-cl renseign)
     * @param plugin le plugin
     * @return la liste des {@link VoeuAmendement} de type Amendement ou Lettre Rectificative recherchs
     */
    List<GVoeuAmendement> selectAmendements( GRequeteUtilisateur requete, GSeance prochaineSeance,
        List<String> listIdsAmendements, Plugin plugin );

    /**
     * Retourne la liste des voeux filtrs par les critres de recherche
     * @param requete la requte utilisateur
     * @param prochaineSeance la prochaine sance
     * @param listIdsVoeux la liste des voeux retourns par la recherche dans l'index
     *                         (<code>null</code> si pas de mot-cl renseign)
     * @param plugin le plugin
     * @return la liste des {@link VoeuAmendement} de type Voeu ou Voeu Non Rattach recherchs
     */
    List<GVoeuAmendement> selectVoeux( GRequeteUtilisateur requete, GSeance prochaineSeance, List<String> listIdsVoeux,
        Plugin plugin );

    /**
     * Retourne la liste des documents de sance filtrs par les critres de recherche
     * @param requete la requte utilisateur
     * @param listIdsFichiers la liste des id de fichier retourns par la recherche dans l'index
     * @param plugin le plugin
     * @return la liste des {@link Fichier} recherchs
     */
    List<GFichier> selectAutresDocuments( GRequeteUtilisateur requete, List<String> listIdsFichiers, Plugin plugin );

    /**
     * Retourne la liste arretes filtrs par les critres de recherche
     * @param requete la requte utilisateur
     * @param listIdsArretes la liste des id d'arrtes retourns par la recherche dans l'index
     * @param plugin le plugin
     * @return la liste des arretes recherchs
     */
    List<Arrete<GSeance, GFichier>> selectArretes( GRequeteUtilisateur requete, List<String> listIdsArretes,
        Plugin plugin );

    /**
     * Retourne la liste des dlibration de dsignation filtres par les critres de recherche
     * @param requete la requte utilisateur
     * @param listIdsDeliberationsDesignation la liste des identifiants des dlibration de dsignation
     * @param plugin le plugin ODS
     * @return la liste des dlibration de dsignation recherches
     */
    List<DeliberationDesignation<GSeance, GFichier>> selectDeliberationsDesignation( GRequeteUtilisateur requete,
        List<String> listIdsDeliberationsDesignation, Plugin plugin );

    /**
     * Retourne la liste des dlibrations de dsignation  indexer
     * @param plugin le plugin
     * @return une liste de DeliberationDesignation
     */
    List<DeliberationDesignation<GSeance, GFichier>> selectDeliberationDesignationAIndexer( Plugin plugin );

    /**
     * Retourne la liste des fichiers du PDD correspondant aux critres de recherche
     * @param listIdsFichiers la liste des ids de fichier correspondants aux critres de recherche (<code>null</code> si pas de mot-cl)
     * @param nIdPdd l'id du pdd
     * @param plugin le plugin
     * @return une liste de fichiers
     */
    List<GFichier> selectFichiersDuPdd( List<String> listIdsFichiers, int nIdPdd, Plugin plugin );

    /**
     * Retourne la liste des Pdds  indexer
     * @param bArchive indique si l'on veut les Pdds archivs ou de la prochaine sance
     * @param prochaineSeance la prochaine sance (<code>null</code> si pas de prochaine sance)
     * @param plugin le plugin
     * @return une liste de Pdds
     */
    List<GPDD> selectPDDsAIndexer( boolean bArchive, GSeance prochaineSeance, Plugin plugin );

    /**
     * Retourne la liste des Pdds  ajouter dans l'index des archives
     * @param nIdAncienneSeance l'id de la dernire sance coule
     * @param plugin le plugin
     * @return une liste de Pdds
     */
    List<GPDD> selectPDDsChangementSeance( int nIdAncienneSeance, Plugin plugin );

    /**
     * Retourne la liste des voeux et amendements  indexer
     * @param bArchive indique si l'on veut les VAs archivs ou de la prochaine sance
     * @param prochaineSeance la prochaine sance (<code>null</code> si pas de prochaine sance)
     * @param plugin le plugin
     * @return une liste de VoeuAmendement
     */
    List<GVoeuAmendement> selectVAAIndexer( boolean bArchive, GSeance prochaineSeance, Plugin plugin );

    /**
     * Retourne la liste des fichiers  indexer dans les archives
     * @param plugin le plugin
     * @return une liste de fichiers
     */
    List<GFichier> selectFichiersAIndexer( Plugin plugin );

    /**
     * Retourne la liste des Voeux et Amendements  indexer dans les archives aprs un changement de sance
     * @param nIdAncienneSeance la dernire sance passe
     * @param plugin le plugin
     * @return une liste de VoeuAmendment
     */
    List<GVoeuAmendement> selectVAChangementSeance( int nIdAncienneSeance, Plugin plugin );

    /**
     * Retourne la liste des fichiers  indexer lors du changement de sance
     * @param nIdDerniereSeance l'id de la sance qui se termine
     * @param plugin le plugin
     * @return une liste de fichiers
     */
    List<GFichier> selectFichiersChangementSeance( int nIdDerniereSeance, Plugin plugin );
}
