/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.textelibre;

import fr.paris.lutece.plugins.ods.dto.textelibre.TexteLibre;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.util.sql.DAOUtil;


/**
 * Cette classe permet d'accder/grer la table ods_texte_libre
 */
public class TexteLibreDAO implements ITexteLibreDAO
{
    private static final String SQL_QUERY_FIND_BY_PRIMARY_KEY = " SELECT libelle FROM ods_texte_libre WHERE id_entite = ? ";
    private static final String SQL_QUERY_NEW_PK = " SELECT max( id_entite ) FROM ods_texte_libre ";
    private static final String SQL_QUERY_INSERT = " INSERT INTO ods_texte_libre (id_entite, libelle) VALUES ( ?, ?) ";
    private static final String SQL_QUERY_UPDATE = " UPDATE ods_texte_libre SET libelle = ? WHERE id_entite = ?  ";
    private static final String SQL_QUERY_DELETE = " DELETE FROM ods_texte_libre WHERE id_entite = ?  ";

    /**
    * Cre un nouveau fichier  partir de lobjet fichier pass en paramtre
    * @param texteLibre le fichier  insrer
    * @param plugin le plugin
    * @return int primaryKey
    */
    public int insert( TexteLibre texteLibre, Plugin plugin )
    {
        int newPrimaryKey = newPrimaryKey( plugin );
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_INSERT, plugin );
        texteLibre.setId( newPrimaryKey );

        daoUtil.setInt( 1, texteLibre.getId(  ) );
        daoUtil.setString( 2, texteLibre.getLibelle(  ) );

        daoUtil.executeUpdate(  );
        daoUtil.free(  );

        return newPrimaryKey;
    }

    /**
     * Modifie le fichier correspondant  lobjet fichier pass en paramtre
     * @param texteLibre le texteLibre  modifier
     * @param plugin le plugin
     */
    public void store( TexteLibre texteLibre, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_UPDATE, plugin );

        daoUtil.setString( 1, texteLibre.getLibelle(  ) );
        daoUtil.setInt( 2, texteLibre.getId(  ) );

        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Supprime le fichier correspondant  lobjet fichier pass en paramtre
     * @param texteLibre le fichier  supprimer
     * @param plugin le plugin
     * @throws AppException si contrainte de cl trangre
     */
    public void delete( TexteLibre texteLibre, Plugin plugin )
        throws AppException
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE, plugin );
        daoUtil.setInt( 1, texteLibre.getId(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Retourne le Fichier ayant comme identifiant nKey
     * @param nKey identifiant du fichier  charger
     * @param plugin le plugin
     * @return le fichier identifi par nKey
     */
    public TexteLibre load( int nKey, Plugin plugin )
    {
        TexteLibre texteLibre = null;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_BY_PRIMARY_KEY, plugin );
        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            texteLibre = new TexteLibre(  );
            texteLibre.setId( nKey );
            texteLibre.setLibelle( daoUtil.getString( 1 ) );
        }

        daoUtil.free(  );

        return texteLibre;
    }

    /**
     * Gnre un nouvel identifiant
     * @param plugin le plugin
     * @return Retourne lidentifiant gnr
     */
    private int newPrimaryKey( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey = 1;

        if ( daoUtil.next(  ) )
        {
            nKey = daoUtil.getInt( 1 ) + 1;
        }

        daoUtil.free(  );

        return nKey;
    }
}
