/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.typeentite;

import fr.paris.lutece.plugins.ods.dto.typeentite.TypeEntite;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;

import java.util.List;


/**
 * IEntiteDAO
 */
public interface ITypeEntiteDAO
{
    /**
    * Insre un nouvel enregistrement dans la table ods_entite  partir de lobjet Entite pass en paramtre
    * @param entiteActe  le natureActe  insrer dans la table ods_entite
    * @param plugin plugin
    */
    void insert( TypeEntite entiteActe, Plugin plugin );

    /**
     * Modifie l'enregistrement dans la table ods_acte_statut correspondant   lobjet Entite pass en paramtre
     * @param entiteActe le natureActe  modifier dans la table ods_entite
     * @param plugin plugin
     */
    void store( TypeEntite entiteActe, Plugin plugin );

    /**
     * Supprime l'enregistrement dans la table ods_acte_statut correspondant  lobjet Entite pass en paramtre
     * @param entiteActe le natureActe  supprimer dans la table ods_entite
     * @param plugin plugin
     * @throws AppException si le StatutActe est rfrenc par un objet tiers
     */
    void delete( TypeEntite entiteActe, Plugin plugin )
        throws AppException;

    /**
     * renvoie le EntiteActe correspondant  l'enregistrement dans la table ods_entite ayant comme identifiant nKey
     * @param nKey l'identifiant du EntiteActe
     * @param plugin plugin
     * @return le entiteActe correspondant
     */
    TypeEntite load( int nKey, Plugin plugin );

    /**
     * renvoie la liste des EntiteActe prsents dans la table ods_entite
     * @param plugin le plugin
     * @return liste d'objets EntiteActe
     */
    List<TypeEntite> selectEntiteActeList( Plugin plugin );
}
