/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.typeentite;

import fr.paris.lutece.plugins.ods.dto.typeentite.TypeEntite;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.util.sql.DAOUtil;

import java.util.ArrayList;
import java.util.List;


/**
 * StatutDAO
 */
public class TypeEntiteDAO implements ITypeEntiteDAO
{
    /*
     * Requetes SQL
     */
    private static final String SQL_QUERY_NEW_PK = "SELECT max( id_entite ) FROM ods_type_entite ";
    private static final String SQL_QUERY_INSERT = "INSERT INTO ods_type_entite ( id_entite, libelle, est_pour_acte )" +
        "VALUES (?,?,?) ";
    private static final String SQL_QUERY_FIND_BY_PRIMARY_KEY = "SELECT " +
        " id_entite, libelle, est_pour_acte FROM ods_acte_entite WHERE id_entite = ? ";
    private static final String SQL_QUERY_NATURE_LIST = "SELECT id_entite, libelle, est_pour_acte FROM ods_type_entite ";
    private static final String SQL_QUERY_DELETE = " DELETE FROM ods_type_entite WHERE id_entite = ?  ";
    private static final String SQL_QUERY_UPDATE = "UPDATE ods_type_entite SET libelle = ?, est_pour_acte = ? WHERE id_entite = ? ";
    private static final String SQL_QUERY_ORDER_BY_LIBELLE = " ORDER BY libelle ASC ";
    private static final String SQL_QUERY_WHERE_EST_POUR_ACTE = " WHERE est_pour_acte = 1 ";

    /**
     * Gnre un nouvel identifiant
     * @param plugin plugin
     * @return Retourne lidentifiant gnr
     */
    int newPrimaryKey( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey;

        if ( !daoUtil.next(  ) )
        {
            nKey = 1;
        }

        nKey = daoUtil.getInt( 1 ) + 1;
        daoUtil.free(  );

        return nKey;
    }

    /**
     * Supprime l'entite correspondant  lobjet EntiteActe pass en paramtre
     * @param entiteActe l'entite  supprimer
     * @param plugin Plugin
     * @throws AppException exception lev si une dependance existe
     */
    public void delete( TypeEntite entiteActe, Plugin plugin )
        throws AppException
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE, plugin );
        daoUtil.setInt( 1, entiteActe.getIdEntite(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Insre l'entite correspondant  lobjet EntiteActe pass en paramtre
     * @param entiteActe le statut  insrer
     * @param plugin Plugin
     */
    public void insert( TypeEntite entiteActe, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_INSERT, plugin );
        int nKey = newPrimaryKey( plugin );

        daoUtil.setInt( 1, nKey );
        daoUtil.setString( 2, entiteActe.getLibelle(  ) );
        daoUtil.setBoolean( 3, entiteActe.isEstPourActe(  ) );

        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Rcupre le statut correspondant  l'id nKey
     * @param nKey l'id du statut  rcuprer
     * @param plugin Plugin
     * @return l'EntiteActe correspondant  l'id nKey
     */
    public TypeEntite load( int nKey, Plugin plugin )
    {
        TypeEntite entiteActe = null;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_BY_PRIMARY_KEY, plugin );
        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            entiteActe = getTypeEntiteValues( daoUtil );
        }

        daoUtil.free(  );

        return entiteActe;
    }

    /**
     * Rcupre la liste des entite
     * @param plugin Plugin
     * @return la liste des entite
     */
    public List<TypeEntite> selectEntiteActeList( Plugin plugin )
    {
        List<TypeEntite> listEntite = new ArrayList<TypeEntite>(  );
        TypeEntite entiteActe;

        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NATURE_LIST + SQL_QUERY_WHERE_EST_POUR_ACTE +
                SQL_QUERY_ORDER_BY_LIBELLE, plugin );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            entiteActe = getTypeEntiteValues( daoUtil );

            listEntite.add( entiteActe );
        }

        daoUtil.free(  );

        return listEntite;
    }

    /**
     * Met  jour l'entite pass en paramtre dans la base de donnes
     * @param entiteActe l'entite  mettre  jour
     * @param plugin Plugin
     */
    public void store( TypeEntite entiteActe, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_UPDATE, plugin );

        daoUtil.setString( 1, entiteActe.getLibelle(  ) );
        daoUtil.setBoolean( 2, entiteActe.isEstPourActe(  ) );
        daoUtil.setInt( 3, entiteActe.getIdEntite(  ) );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Cre et remplit un objet TypeEntite avec les valeurs contenus dans le DAOUtil
     * @param daoUtil daoUtil
     * @return l'objet TypeEntite
     */
    private TypeEntite getTypeEntiteValues( DAOUtil daoUtil )
    {
        TypeEntite typeEntite = new TypeEntite(  );
        typeEntite.setIdEntite( daoUtil.getInt( "id_entite" ) );
        typeEntite.setLibelle( daoUtil.getString( "libelle" ) );
        typeEntite.setEstPourActe( daoUtil.getBoolean( "est_pour_acte" ) );

        return typeEntite;
    }
}
