/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.typeentite;

import fr.paris.lutece.plugins.ods.dto.typeentite.TypeEntite;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;


/**
 * EntiteActeHome
 */
public final class TypeEntiteHome
{
    @Autowired
    private ITypeEntiteDAO _entiteAO;

    /**
     * Constructeur vide
     */
    private TypeEntiteHome(  )
    {
    }

    /**
    * Cre une nouvelle entite  partir de lobjet entite pass en paramtre
    * @param entite la nature  insrer
    * @param plugin Plugin
    */
    public void create( TypeEntite entite, Plugin plugin )
    {
        _entiteAO.insert( entite, plugin );
    }

    /**
     * Modifie l'acte correspondant  lobjet entiteActe pass en paramtre
     * @param entite l'entite  modifier
     * @param plugin Plugin
     */
    public void update( TypeEntite entite, Plugin plugin )
    {
        _entiteAO.store( entite, plugin );
    }

    /**
     * Supprime l'acte correspondant  lobjet acte pass en paramtre
     * @param entite l'entite  supprimer
     * @param plugin Plugin
     * @throws AppException exception lev si une dependance existe
     */
    public void remove( TypeEntite entite, Plugin plugin )
        throws AppException
    {
        _entiteAO.delete( entite, plugin );
    }

    /**
     * renvoie l'entite ayant comme identifiant nKey
     * @param nKey idenfiant de l'entite
     * @param plugin Plugin
     * @return objet entiteActe
     */
    public TypeEntite findByPrimaryKey( int nKey, Plugin plugin )
    {
        return _entiteAO.load( nKey, plugin );
    }

    /**
     * renvoie la liste des entite
     * @param plugin Plugin
     * @return liste d'objets entite
     */
    public List<TypeEntite> findEntiteActesList( Plugin plugin )
    {
        return _entiteAO.selectEntiteActeList( plugin );
    }
}
