/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.voeuamendement;

import fr.paris.lutece.plugins.ods.dto.IDepositaire;
import fr.paris.lutece.plugins.ods.dto.categoriedeliberation.CategorieDeliberation;
import fr.paris.lutece.plugins.ods.dto.direction.Direction;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.groupepolitique.IGroupePolitique;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.statut.StatutEnum;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendementFilter;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.util.sql.DAOUtil;

import java.util.ArrayList;
import java.util.List;


/**
 *
 * AbstractVoeuAmendementDAO
 *
 * @param <GVoeuAmendement> {@link IVoeuAmendement}
 * @param <GVoeuAmendementFilter> {@link IVoeuAmendementFilter}
 * @param <GSeance> {@link ISeance}
 * @param <GElu> {@link IElu}
 * @param <GFichier> {@link IFichier}
 * @param <GPDD> {@link IPDD}
 */
public abstract class AbstractVoeuAmendementDAO
<
	GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, 
	GVoeuAmendementFilter extends IVoeuAmendementFilter, 
	GSeance extends ISeance, 
	GElu extends IElu, 
	GFichier extends IFichier<GSeance, GFichier>, 
	GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>
>
implements IVoeuAmendementDAO<GVoeuAmendement, GVoeuAmendementFilter, GSeance, GElu, GFichier, GPDD>
{
    protected static final String SQL_QUERY_DELETE_DEPOSITAIRE_IN_DEPOSE_PAR = "DELETE FROM ods_va_depose_par WHERE id_va = ?";
    protected static final String SQL_QUERY_DELETE = "DELETE FROM ods_voeu_amendement WHERE id_va = ?";
    protected static final String SQL_QUERY_DELETE_VA_IN_RATTACHE_PDD = "DELETE FROM ods_va_rattache_pdd WHERE id_va = ?";
    protected static final String SQL_FILTER_FORMATION_CONSEIL = " AND va.id_formation_conseil = ? ";
    protected static final String SQL_FILTER_ID_ODJ = " AND odj.id_odj = ? ";
    protected static final String SQL_FILTER_PUBLIE = " AND va.en_ligne = ? ";
    protected static final String SQL_FILTER_SEANCE = " AND se.id_seance = ? ";
    protected static final String SQL_FILTER_TYPE = " AND va.type = ? ";
    protected static final String SQL_FILTER_PARENT = " AND va.id_va IN (SELECT id_va FROM ods_va_parent WHERE id_va_parent = ? ) ";
    protected static final String SQL_FILTER_PAS_ENFANT_DE = " AND va.id_va NOT IN (SELECT id_va FROM ods_va_parent WHERE id_va_parent = ? ) ";
    protected static final String SQL_FILTER_STATUT = " AND va.id_statut = ? ";
    protected static final String SQL_FILTER_EXECUTIF = " AND va.depose_executif = ? ";
    protected static final String SQL_FILTER_TYPE_ORDRE_DU_JOUR = " AND odj.id_type_odj = ? ";
    protected static final String SQL_FILTER_TYPE_AMENDEMENT = " AND ( va.type='A' OR va.type='LR' ) ";
    protected static final String SQL_FILTER_TYPE_TOUS_VOEUX = " AND ( va.type='V' OR va.type='VNR' ) ";
    protected static final String SQL_FILTER_DATE_PUBLICATION = " AND va.date_publication > ? ";
    protected static final String SQL_FILTER_STATUT_ACTE = " AND ( SELECT count(*) FROM ods_acte WHERE id_type_entite = ? AND id_entite = va.id_va AND id_statut <> ? ) ";
    protected static final String SQL_FILTER_STATUT_A_PREPARER = " = 0 ";
    protected static final String SQL_FILTER_STATUT_UN_ACTE_AU_MOINS = " >= 1 ";
    protected static final String SQL_FILTER_VA_STATUT_ADOPTE_OU_AMENDE = " AND va.id_statut IN ( " +
        StatutEnum.ADOPTE.getId(  ) + " , " + StatutEnum.AMENDE.getId(  ) + " ) ";
    protected static final String SQL_FILTER_DATE_VOTE_RENSEIGNEE = " AND date_vote IS NOT NULL ";
    protected static final String GROUP_BY_ID_VA = " GROUP BY va.id_va ";
    private static final String SQL_QUERY_NEW_PK = " SELECT max( id_va ) FROM ods_voeu_amendement ";
    private static final String SQL_QUERY_DELETE_LIAISONS_VA_ENFANTS = "DELETE FROM ods_va_parent WHERE id_va_parent = ?";
    private static final String SQL_QUERY_DELETE_LIAISONS_VA_PARENTS = "DELETE FROM ods_va_parent WHERE id_va = ?";
    private static final String SQL_QUERY_DEPOSITAIRE_LIST_BY_ID_VOEUAMENDEMENT = " SELECT id_elu, id_groupe FROM ods_va_depose_par WHERE id_va = ? " +
        " ORDER BY numero_ordre ";
    private static final String SQL_QUERY_FIND_ELU_BY_PRIMARY_KEY = " SELECT elu.id_elu, elu.civilite, elu.nom_elu, elu.prenom_elu, groupe.id_groupe, " +
        " groupe.nom_groupe, groupe.nom_complet FROM ods_elu elu LEFT JOIN ods_groupe groupe ON (elu.id_groupe = groupe.id_groupe) WHERE elu.id_elu = ? ";
    private static final String SQL_QUERY_FIND_GROUPE_BY_PRIMARY_KEY = " SELECT id_groupe, nom_groupe, nom_complet FROM ods_groupe WHERE id_groupe = ? ";
    private static final String SQL_QUERY_INSERT_DEPOSITAIRE_IN_DEPOSE_PAR = "INSERT INTO ods_va_depose_par(id_elu,id_groupe,id_va,numero_ordre)" +
        " VALUES(?,?,?,?)";
    private static final String SQL_QUERY_IS_PDD_ALREADY_IN_VA = "SELECT COUNT(id_pdd) FROM ods_va_rattache_pdd where id_pdd=? and id_va= ?";
    private static final String SQL_QUERY_NEW_NUMERO_ORDRE_FOR_PDD = "SELECT MAX( numero_ordre ) FROM ods_va_rattache_pdd where id_pdd=?";
    private static final String SQL_QUERY_INSERT_PDD_IN_RATTACHE_PDD = "INSERT INTO ods_va_rattache_pdd(id_va,id_pdd,numero_ordre)" +
        " VALUES(?,?,?)";
    private static final String SQL_QUERY_INSERT_LIAISONS_VA_PARENT = "INSERT INTO  ods_va_parent(id_va,id_va_parent,numero_ordre)VALUES( ? , ? , ? ) ";
    private static final String SQL_QUERY_SELECT_ID_PDD_IN_RATTACHE_PDD = "SELECT id_pdd FROM ods_va_rattache_pdd where id_va= ? ";
    private static final String SQL_QUERY_DELETE_PDD_IN_VA = "DELETE FROM  ods_va_rattache_pdd where id_pdd=? and id_va=?";
    private static final String SQL_QUERY_FIND_ID_BY_DELIBERATION = "SELECT id_va FROM ods_voeu_amendement WHERE id_delib = ? ";
    private static final String SQL_QUERY_FIND_ID_BY_TEXTE_INITIAL = "SELECT id_va FROM ods_voeu_amendement WHERE id_texte_initial = ? ";
    private static final String SQL_QUERY_VOEUAMENDEMENT_LIST_ENFANT = "SELECT id_va   FROM ods_va_parent WHERE id_va_parent= ? ORDER BY numero_ordre ";
    private static final String SQL_QUERY_VOEUAMENDEMENT_LIST_PARENT = "SELECT id_va_parent  FROM ods_va_parent WHERE id_va= ? ORDER BY numero_ordre ";
    private static final String SQL_QUERY_UPDATE_REFERENCE_FRONT = " UPDATE ods_voeu_amendement SET reference_va_front=? where id_va=? ";
    private static final String SQL_QUERY_PDD_LISTBY_ID_VOEUAMENDEMENT = "SELECT pdd.id_pdd, pdd.id_categorie, pdd.id_direction, " +
        " fc.id_formation_conseil, fc.libelle_formation_conseil, pdd.id_statut, pdd.reference, type_pdd, pdd.delegations_services, " +
        " pdd.mode_introduction, pdd.objet, pdd.pieces_manuelles, pdd.date_vote, pdd.en_ligne, pdd.date_publication, pdd.version, " +
        " ca.id_categorie, ca.libelle_categorie FROM ods_pdd pdd, ods_va_rattache_pdd vrp, ods_formation_conseil fc, " +
        " ods_categorie_deliberation ca WHERE vrp.id_va = ? AND vrp.id_pdd = pdd.id_pdd AND pdd.id_formation_conseil = fc.id_formation_conseil " +
        " AND pdd.id_categorie = ca.id_categorie ORDER BY vrp.numero_ordre ";
    private static final String ORDER_BY_LIST = " ORDER BY ";

    /**
     * cre une nouvelle instance de IElu
     * @return une nouvelle instance de IElu
     */
    protected abstract IElu newElu(  );

    /**
     * Cre une nouvelle instance de IGroupePolitique
     * @return une nouvelle instance de IGroupePolitique
     */
    protected abstract IGroupePolitique newGroupePolitique(  );

    /**
     * Cre une nouvelle instance de IPDD
     * @return une nouvelle instance de IPDD
     */
    protected abstract GPDD newPDD(  );

    /**
     * Retourne la requete pour compter le nombre de VA dans la liasse
     * @return la requete pour compter le nombre de VA dans la liasse
     */
    protected abstract String getQueryCountLiasse(  );

    /**
    * Gnre un nouvel identifiant
    * @param plugin plugin
    * @return Retourne lidentifiant gnr
    */
    protected int newPrimaryKey( Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_PK, plugin );
        daoUtil.executeQuery(  );

        int nKey;

        if ( !daoUtil.next(  ) )
        {
            nKey = 1;
        }

        nKey = daoUtil.getInt( 1 ) + 1;
        daoUtil.free(  );

        return nKey;
    }

    /**
     * Supression des liens entre un VA et ses enfants
     * @param nIdVA l'id du va
     * @param plugin le plugin
     */
    protected void deleteLiaisonVoeuAmendementEnfants( int nIdVA, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE_LIAISONS_VA_ENFANTS, plugin );
        daoUtil.setInt( 1, nIdVA );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Supression des liens entre un VA et ses parents
     * @param nIdVA l'id du va
     * @param plugin le plugin
     */
    protected void deleteLiaisonVoeuAmendementParents( int nIdVA, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE_LIAISONS_VA_PARENTS, plugin );
        daoUtil.setInt( 1, nIdVA );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Retourne la liste des dpositaires ayant dpos le VoeuAmendement d'id nKey
     * @param nKey id du VA
     * @param plugin plugin
     * @return liste des dpositaires du va
     */
    protected List<IDepositaire> selectDepositaireListbyIdVoeuAmendement( int nKey, Plugin plugin )
    {
        List<IDepositaire> depositaires = new ArrayList<IDepositaire>(  );

        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DEPOSITAIRE_LIST_BY_ID_VOEUAMENDEMENT, plugin );
        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            if ( daoUtil.getObject( 1 ) != null )
            {
                depositaires.add( loadEluByPrimaryKey( daoUtil.getInt( 1 ), plugin ) );
            }
            else
            {
                depositaires.add( loadGroupeByPrimaryKey( daoUtil.getInt( 2 ), plugin ) );
            }
        }

        daoUtil.free(  );

        return depositaires;
    }

    /**
     * Retourne l'lu correspondant  l'enregistrement dans la table ods_elu ayant comme identifiant nKey
     * @param nKey l'identifiant de l'lu
     * @param plugin plugin
     * @return  l'lu correspondant  l'enregistrement dans la table ods_elu ayant comme identifiant nKey
     */
    private IElu loadEluByPrimaryKey( int nKey, Plugin plugin )
    {
        IElu elu = null;
        IGroupePolitique groupePolitique = null;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_ELU_BY_PRIMARY_KEY, plugin );
        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        if ( daoUtil.next(  ) )
        {
            elu = newElu(  );
            elu.setIdElu( daoUtil.getInt( 1 ) );
            elu.setCivilite( daoUtil.getString( 2 ) );
            elu.setNomElu( daoUtil.getString( 3 ) );
            elu.setPrenomElu( daoUtil.getString( 4 ) );
            //on recupere le groupe politique de l'lu en crant un objet GroupePolitique et on l'affecte  l'objet elu
            groupePolitique = newGroupePolitique(  );
            groupePolitique.setIdGroupe( daoUtil.getInt( 5 ) );
            groupePolitique.setNomGroupe( daoUtil.getString( 6 ) );
            groupePolitique.setNomComplet( daoUtil.getString( 7 ) );
            elu.setGroupe( groupePolitique );
        }

        daoUtil.free(  );

        return elu;
    }

    /**
     * renvoie le groupe politique  correspondant  l'enregistrement dans la table ods_groupe ayant comme identifiant nKey
     * @param nKey idenfiant du groupe politique  supprimer
     * @param plugin plugin
     * @return objet GroupePolitique
     */
    private IGroupePolitique loadGroupeByPrimaryKey( int nKey, Plugin plugin )
    {
        IGroupePolitique groupePolitique = null;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_GROUPE_BY_PRIMARY_KEY, plugin );
        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        if ( daoUtil.next(  ) )
        {
            groupePolitique = newGroupePolitique(  );
            groupePolitique.setIdGroupe( daoUtil.getInt( 1 ) );
            groupePolitique.setNomGroupe( daoUtil.getString( 2 ) );
            groupePolitique.setNomComplet( daoUtil.getString( 3 ) );
        }

        daoUtil.free(  );

        return groupePolitique;
    }

    /**
     * retourne la clause order by de la requette en fonction du filter
     * @param filter le filtre de selection
     * @return  retourne l'order by de la requette
     */
    protected String buildOrderBy( GVoeuAmendementFilter filter )
    {
        StringBuffer strOrderBy = new StringBuffer(  );
        strOrderBy.append( ORDER_BY_LIST );

        for ( String order : filter.getOrderByList(  ) )
        {
            strOrderBy.append( order );
            strOrderBy.append( OdsConstants.CONSTANTE_VIRGULE );
        }

        return strOrderBy.substring( 0, strOrderBy.length(  ) - 1 );
    }

    /**
     * insere dans la table ods_va_depose_par la liste des dpositaires rattachs au VouAmendement
     * @param depositaire le dpositaire du va
     * @param nIdVoeuAmendement l'id du VoeuAmendement
     * @param nNumeroOrdre numero ordre
     * @param plugin plugin
     */
    protected void insertDepositaireInDeposePar( IDepositaire depositaire, int nIdVoeuAmendement, int nNumeroOrdre,
        Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_INSERT_DEPOSITAIRE_IN_DEPOSE_PAR, plugin );

        if ( depositaire.getType(  ) == IDepositaire.DEPOSITAIRE_TYPE_ELU )
        {
            daoUtil.setInt( 1, ( (IElu) depositaire ).getIdElu(  ) );
            daoUtil.setIntNull( 2 );
        }
        else
        {
            daoUtil.setIntNull( 1 );
            daoUtil.setInt( 2, ( (IGroupePolitique) depositaire ).getIdGroupe(  ) );
        }

        daoUtil.setInt( 3, nIdVoeuAmendement );
        daoUtil.setInt( 4, nNumeroOrdre );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * test si le pdd existe deja dans la liste des pdds du VoeuAmendement
     * @param nIdPdd  id du pdd
     * @param nIdVa id du VA
     * @param plugin plugin
     * @return retourne true si le pdd existe deja dans la liste des pdds du VA false sinon
     */
    protected boolean isPddAlreadyInVa( int nIdPdd, int nIdVa, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_IS_PDD_ALREADY_IN_VA, plugin );
        daoUtil.setInt( 1, nIdPdd );
        daoUtil.setInt( 2, nIdVa );
        daoUtil.executeQuery(  );

        if ( daoUtil.next(  ) && ( daoUtil.getInt( 1 ) != 0 ) )
        {
            daoUtil.free(  );

            return true;
        }

        daoUtil.free(  );

        return false;
    }

    /**
     * retourne le numero d' ordre a inserer avec le pdd d'id nIdPdd
     * @param nIdPdd id du pdd  inserer
     * @param plugin plugin
     * @return  le numero d' ordre a inserer avec le pdd d'id nIdPdd
     */
    protected int newNumeroOrdre( int nIdPdd, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_NEW_NUMERO_ORDRE_FOR_PDD, plugin );
        daoUtil.setInt( 1, nIdPdd );
        daoUtil.executeQuery(  );

        int nNumeroOrdre = 1;

        if ( daoUtil.next(  ) && ( daoUtil.getObject( 1 ) != null ) )
        {
            nNumeroOrdre = daoUtil.getInt( 1 ) + 1;
        }

        daoUtil.free(  );

        return nNumeroOrdre;
    }

    /**
     * insere dans la table ods_va_rattache_pdd la liste des pdds rattachs au VouAmendement
     * @param nIdPdd l'id du pdd a rattache au VA
     * @param nIdVoeuAmendement l'id du VoeuAmendement
     * @param nNumeroOrdre numro ordre
     * @param plugin plugin
     */
    protected void insertPddInRattachePdd( int nIdPdd, int nIdVoeuAmendement, int nNumeroOrdre, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_INSERT_PDD_IN_RATTACHE_PDD, plugin );
        daoUtil.setInt( 1, nIdVoeuAmendement );
        daoUtil.setInt( 2, nIdPdd );
        daoUtil.setInt( 3, nNumeroOrdre );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * insere un parent au va pass en paramre
     * @param nIdVA l'id du va
     * @param nIdVAParent l'id du va parent
     * @param nNumeroOrdre le numero d'ordre du va parent
     * @param plugin plugin
     */
    protected void insertLiasonVoeuAmendementParent( int nIdVA, int nIdVAParent, int nNumeroOrdre, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_INSERT_LIAISONS_VA_PARENT, plugin );
        daoUtil.setInt( 1, nIdVA );
        daoUtil.setInt( 2, nIdVAParent );
        daoUtil.setInt( 3, nNumeroOrdre );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * supprime dans la table va_rattache_pdd tous les pdds qui ne sont pas dans la liste des ppds du va  modifier
     * @param va le voeuAmendement  modifier
     * @param plugin plugin
     */
    protected void deletePddsNotInVa( GVoeuAmendement va, Plugin plugin )
    {
        int nIdPdd;
        boolean bIsInPddList;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_SELECT_ID_PDD_IN_RATTACHE_PDD, plugin );
        daoUtil.setInt( 1, va.getId(  ) );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            bIsInPddList = false;
            nIdPdd = daoUtil.getInt( 1 );

            //boucle sur la liste des pdds du va
            if ( va.getPdds(  ) != null )
            {
                for ( GPDD pdd : va.getPdds(  ) )
                {
                    if ( pdd.getId(  ) == nIdPdd )
                    {
                        bIsInPddList = true;
                    }
                }
            }

            if ( !bIsInPddList )
            {
                deletePdd( nIdPdd, va.getId(  ), plugin );
            }
        }

        daoUtil.free(  );
    }

    /**
     * suppression dans la table ods_va_rattache_pdd du  couple pdd d'id nIdPdd et du va d'id nIdVa
     *@param nIdPdd l'id du pdd
     *@param nIdVa l'id du va
     *@param plugin plugin
     *
     */
    protected void deletePdd( int nIdPdd, int nIdVa, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_DELETE_PDD_IN_VA, plugin );
        daoUtil.setInt( 1, nIdPdd );
        daoUtil.setInt( 2, nIdVa );
        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Retourne le voeu amendement dont la deliberation est le fichier
     * @param fichier le fichier du voeu amendement
     * @param plugin le plugin
     * @return la voeu amendement dont le texte initial est le fichier
     */
    public GVoeuAmendement selectByDeliberation( GFichier fichier, Plugin plugin )
    {
        int nIdVoeuAmendement = -1;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_ID_BY_DELIBERATION, plugin );
        daoUtil.setInt( 1, fichier.getId(  ) );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            nIdVoeuAmendement = daoUtil.getInt( 1 );
        }

        daoUtil.free(  );

        return load( nIdVoeuAmendement, plugin );
    }

    /**
     * Retourne le voeu amendement dont le texte initial est le fichier
     * @param fichier le fichier du voeu amendement
     * @param plugin le plugin
     * @return la voeu amendement dont le texte initial est le fichier
     */
    public GVoeuAmendement selectByTexteInitial( GFichier fichier, Plugin plugin )
    {
        int nIdVoeuAmendement = -1;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_FIND_ID_BY_TEXTE_INITIAL, plugin );
        daoUtil.setInt( 1, fichier.getId(  ) );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            nIdVoeuAmendement = daoUtil.getInt( 1 );
        }

        daoUtil.free(  );

        return load( nIdVoeuAmendement, plugin );
    }

    /**
     * renvoie la liste des voeux et Amendements  prsents dans la table ods_voeu_amendement rpondant
     *  aux criteres de selections du filtre
     * @param nIdFormationConseil l'id de la formation du conseil
     * @param nIdSeance l'id de la sance
     * @param plugin plugin
     * @return le nombre de VAs correspondants
     */
    public int selectCountLiasseByFilter( int nIdFormationConseil, int nIdSeance, Plugin plugin )
    {
        int nResult = 0;

        StringBuffer sbSQL = new StringBuffer(  );

        sbSQL.append( getQueryCountLiasse(  ) );

        DAOUtil daoUtil = new DAOUtil( sbSQL.toString(  ), plugin );
        int nIndex = 1;

        daoUtil.setInt( nIndex, nIdFormationConseil );
        nIndex++;

        daoUtil.setInt( nIndex, nIdSeance );
        nIndex++;

        daoUtil.setInt( nIndex, nIdFormationConseil );
        nIndex++;

        daoUtil.setInt( nIndex, nIdSeance );
        nIndex++;

        daoUtil.executeQuery(  );

        if ( daoUtil.next(  ) )
        {
            nResult = daoUtil.getInt( 1 );
        }

        daoUtil.free(  );

        return nResult;
    }

    /**
    *
    * retourne la liste des voeux amendements enfants associs au voeu amendement d'id nIdVoeuAmendement
    * @param nIdVoeuAmendement l'id du voeu amendement
    * @param plugin yhe plugin
    * @return la liste des voeux amendements enfants associs au voeu amendement d'id nIdVoeuAmendement
    */
    public List<GVoeuAmendement> selectVoeuxAmendementsEnfant( int nIdVoeuAmendement, Plugin plugin )
    {
        List<GVoeuAmendement> listVoeuxAmendementsEnfant = new ArrayList<GVoeuAmendement>(  );
        GVoeuAmendement voeuAmendement;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_VOEUAMENDEMENT_LIST_ENFANT, plugin );
        daoUtil.setInt( 1, nIdVoeuAmendement );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            voeuAmendement = load( daoUtil.getInt( 1 ), plugin );
            listVoeuxAmendementsEnfant.add( voeuAmendement );
        }

        daoUtil.free(  );

        return listVoeuxAmendementsEnfant;
    }

    /**
    *
    * retourne la liste des voeux amendements parents associs au voeu amendement d'id nIdVoeuAmendement
    * @param nIdVoeuAmendement l'id du voeu amendement
    * @param plugin yhe plugin
    * @return la liste des voeux amendements parents associs au voeu amendement d'id nIdVoeuAmendement
    */
    public List<GVoeuAmendement> selectVoeuxAmendementsParent( int nIdVoeuAmendement, Plugin plugin )
    {
        List<GVoeuAmendement> listVoeuxAmendementsParent = new ArrayList<GVoeuAmendement>(  );
        GVoeuAmendement voeuAmendement;
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_VOEUAMENDEMENT_LIST_PARENT, plugin );
        daoUtil.setInt( 1, nIdVoeuAmendement );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            voeuAmendement = load( daoUtil.getInt( 1 ), plugin );
            listVoeuxAmendementsParent.add( voeuAmendement );
        }

        daoUtil.free(  );

        return listVoeuxAmendementsParent;
    }

    /**
     * Modifie la reference du va utilise cot front office dans la table ods_voeu_Amendement
     * @param voeuAmendement le voeuAmendement  modifier dans la table ods_voeu_amendement
     * @param plugin plugin
     */
    public void storeReferenceFront( GVoeuAmendement voeuAmendement, Plugin plugin )
    {
        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_UPDATE_REFERENCE_FRONT, plugin );
        daoUtil.setString( 1, voeuAmendement.getReferenceFront(  ) );
        daoUtil.setInt( 2, voeuAmendement.getId(  ) );

        daoUtil.executeUpdate(  );
        daoUtil.free(  );
    }

    /**
     * Retourne la liste des PDDs associs au VA d'id nKey
     * @param nKey id du VA
     * @param plugin plugin
     * @return la liste des PDDs associs au VA
     */
    public List<GPDD> selectPddListbyIdVoeuAmendement( int nKey, Plugin plugin )
    {
        List<GPDD> listPDDs = new ArrayList<GPDD>(  );
        GPDD pdd = null;
        FormationConseil formationConseil = null;
        CategorieDeliberation categorieDeliberation = null;
        Direction direction = null;

        DAOUtil daoUtil = new DAOUtil( SQL_QUERY_PDD_LISTBY_ID_VOEUAMENDEMENT, plugin );
        daoUtil.setInt( 1, nKey );
        daoUtil.executeQuery(  );

        while ( daoUtil.next(  ) )
        {
            pdd = newPDD(  );
            pdd.setId( daoUtil.getInt( "pdd.id_pdd" ) );
            pdd.setReference( daoUtil.getString( "pdd.reference" ) );
            pdd.setTypePdd( daoUtil.getString( "pdd.type_pdd" ) );
            pdd.setDelegationsServices( daoUtil.getBoolean( "pdd.delegations_services" ) );
            pdd.setModeIntroduction( daoUtil.getString( "pdd.mode_introduction" ) );
            pdd.setObjet( daoUtil.getString( "pdd.objet" ) );
            pdd.setPiecesManuelles( daoUtil.getBoolean( "pdd.pieces_manuelles" ) );
            pdd.setDateVote( daoUtil.getTimestamp( "pdd.date_vote" ) );
            pdd.setEnLigne( daoUtil.getBoolean( "pdd.en_ligne" ) );
            pdd.setDatePublication( daoUtil.getTimestamp( "pdd.date_publication" ) );
            pdd.setVersion( daoUtil.getInt( "pdd.version" ) );

            // Cration de l'objet formation de conseil associ au VA
            formationConseil = new FormationConseil(  );
            formationConseil.setIdFormationConseil( daoUtil.getInt( "fc.id_formation_conseil" ) );
            formationConseil.setLibelle( daoUtil.getString( "fc.libelle_formation_conseil" ) );
            pdd.setFormationConseil( formationConseil );

            // Cration de l'objet catgorie de dlibration
            categorieDeliberation = new CategorieDeliberation(  );
            categorieDeliberation.setIdCategorie( daoUtil.getInt( "ca.id_categorie" ) );
            categorieDeliberation.setLibelle( daoUtil.getString( "ca.libelle_categorie" ) );
            pdd.setCategorieDeliberation( categorieDeliberation );

            // Cration de l'objet direction
            direction = new Direction(  );
            direction.setIdDirection( daoUtil.getInt( "pdd.id_direction" ) );
            pdd.setDirection( direction );

            listPDDs.add( pdd );
        }

        daoUtil.free(  );

        return listPDDs;
    }
}
