/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.voeuamendement;

import fr.paris.lutece.plugins.ods.business.acte.ActeHome;
import fr.paris.lutece.plugins.ods.business.statut.StatutHome;
import fr.paris.lutece.plugins.ods.dto.acte.Acte;
import fr.paris.lutece.plugins.ods.dto.acte.IActeFilter;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.typeentite.ITypeEntite;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendementFilter;
import fr.paris.lutece.plugins.ods.service.acte.IActeService;
import fr.paris.lutece.portal.service.plugin.Plugin;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;


/**
 *
 * AbstractVoeuAmendementHome
 *
 * @param <GVoeuAmendement> {@link IVoeuAmendement}
 * @param <GVoeuAmendementFilter> {@link IVoeuAmendementFilter}
 * @param <GVoeuAmendementDAO> {@link IVoeuAmendementDAO}
 * @param <GSeance> {@link ISeance}
 * @param <GElu> {@link IElu}
 * @param <GFichier> {@link IFichier}
 * @param <GActeFilter> {@link IActeFilter}
 * @param <GPDD> {@link IPDD}
 */
public abstract class AbstractVoeuAmendementHome
<
	GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, 
	GVoeuAmendementFilter extends IVoeuAmendementFilter, 
	GVoeuAmendementDAO extends IVoeuAmendementDAO<GVoeuAmendement, GVoeuAmendementFilter, GSeance, GElu, GFichier, GPDD>, 
	GSeance extends ISeance, 
	GElu extends IElu, 
	GFichier extends IFichier<GSeance, GFichier>, 
	GActeFilter extends IActeFilter<GFichier, GSeance>, 
	GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>
>
implements IVoeuAmendementHome<GVoeuAmendement, GVoeuAmendementFilter, GSeance, GElu, GFichier, GPDD>
{
    @Autowired
    protected GVoeuAmendementDAO _voeuAmendementDAO;
    @Autowired
    private ActeHome<GSeance, GFichier, GActeFilter> _acteHome;
    @Autowired
    private IActeService<GSeance, GFichier, GActeFilter> _acteService;
    @Autowired
    private StatutHome _statutHome;

    protected abstract Acte<GSeance, GFichier> getActeCourant( List<Acte<GSeance, GFichier>> listActes );

    /**
     * Insre un VoeuAmendement  partir de lobjet VoeuAmendement pass en parametre
     *  pass  en paramtre
     * @param   voeuAmendement le voeuAmendement  insrer dans la table ods_voeu_amendement
     * @param plugin plugin
     * @return l'id du VoeuAmendement cr
     */
    public int create( GVoeuAmendement voeuAmendement, Plugin plugin )
    {
        voeuAmendement.setId( _voeuAmendementDAO.insert( voeuAmendement, plugin ) );

        return voeuAmendement.getId(  );
    }

    /**
     * renvoie le VoeuAmendement  ayant comme identifiant nKey
     * @param nKey l'identifiant de l'lu
     * @param plugin plugin
     * @return le voeuAmendement d'id=nkey
     */
    public GVoeuAmendement findByPrimaryKey( int nKey, Plugin plugin )
    {
        GVoeuAmendement voeuAmendement = _voeuAmendementDAO.load( nKey, plugin );

        // On instancie la liste des VAs parents
        if ( voeuAmendement != null )
        {
            // Gestion des actes rattachs
            List<Acte<GSeance, GFichier>> listActes = getListeActes( nKey, plugin );
            voeuAmendement.setActes( listActes );

            if ( listActes != null )
            {
                voeuAmendement.setActeCourant( getActeCourant( listActes ) );
            }

            voeuAmendement.setParents( findVoeuxAmendementsParent( nKey, plugin ) );
        }

        return voeuAmendement;
    }

    /**
     * Supprime le VoeuAmendement pass en paramtre
     * @param voeuAmendement le voeuAmendement  supprimer
     * @param plugin plugin
     */
    public void remove( GVoeuAmendement voeuAmendement, Plugin plugin )
    {
        _voeuAmendementDAO.delete( voeuAmendement, plugin );
    }

    /**
     * Modifie le VoeuAmendement pass en paramtre
     * @param voeuAmendement le voeuAmendement  modifier dans la table ods_voeu_amendement
     * @param plugin plugin
     */
    public void update( GVoeuAmendement voeuAmendement, Plugin plugin )
    {
        _voeuAmendementDAO.store( voeuAmendement, plugin );
    }

    /**
     * Modifie la reference du va utilise cot front office
     * @param voeuAmendement le voeuAmendement  modifier
     * @param plugin plugin
     */
    public void updateReferenceFront( GVoeuAmendement voeuAmendement, Plugin plugin )
    {
        _voeuAmendementDAO.storeReferenceFront( voeuAmendement, plugin );
    }

    /**
     * renvoie la liste des voeux et Amendements   rpondant
     *  aux criteres de selections du filtre
     * @param filter le filtre de selection
     * @param plugin plugin
     * @return liste d'objets voeuAmendement
     */
    public List<GVoeuAmendement> findVoeuAmendementListByFilter( GVoeuAmendementFilter filter, Plugin plugin )
    {
        List<GVoeuAmendement> listVA = _voeuAmendementDAO.selectVoeuAmendementListByFilter( filter, plugin );

        for ( GVoeuAmendement va : listVA )
        {
            List<Acte<GSeance, GFichier>> listActes = getListeActes( va.getId(  ), plugin );
            va.setActes( listActes );

            if ( va.getStatut(  ) != null )
            {
                va.setStatut( _statutHome.findByPrimaryKey( va.getStatut(  ).getIdStatut(  ), plugin ) );
            }
        }

        return listVA;
    }

    /**
     * renvoie la liasse des Voeux  Amendements   rpondant
     *  aux criteres de selections du filtre
     * @param nIdFormationConseil l'id de la formation conseil
     * @param nIdSeance l'id de la sance
     * @param plugin plugin
     * @return liste d'objets voeuAmendement
     */
    public List<GVoeuAmendement> findLiasseByFilter( int nIdFormationConseil, int nIdSeance, Plugin plugin )
    {
        return _voeuAmendementDAO.selectLiasseByFilter( nIdFormationConseil, nIdSeance, plugin );
    }

    /**
     * renvoie la liste des voeux et Amendements  prsents dans la table ods_voeu_amendement rpondant
     *  aux criteres de selections du filtre
     * @param nIdFormationConseil l'id de la formation du conseil
     * @param nIdSeance l'id de la seance
     * @param plugin plugin
     * @return liste d'objets voeuAmendement
     */
    public int findCountLiasseByFilter( int nIdFormationConseil, int nIdSeance, Plugin plugin )
    {
        return _voeuAmendementDAO.selectCountLiasseByFilter( nIdFormationConseil, nIdSeance, plugin );
    }

    /**
     * Retourne la liste des pdds associ au VoeuAmendement d'id nKey
     * @param nKey id du VA
     * @param plugin plugin
     * @return la liste des pdds du va
     */
    public List<GPDD> findPddListbyIdVoeuAmendement( int nKey, Plugin plugin )
    {
        return _voeuAmendementDAO.selectPddListbyIdVoeuAmendement( nKey, plugin );
    }

    /**
     * Retourne le voeu amendement dont le texte initial est le fichier
     * @param fichier le fichier du voeu amendement
     * @param plugin le plugin
     * @return la voeu amendement dont le texte initial est le fichier
     */
    public GVoeuAmendement findByTexteInitial( GFichier fichier, Plugin plugin )
    {
        return _voeuAmendementDAO.selectByTexteInitial( fichier, plugin );
    }

    /**
     * Retourne le voeu amendement dont la dlibration est le fichier
     * @param fichier le fichier du voeu amendement
     * @param plugin le plugin
     * @return la voeu amendement dont le texte initial est le fichier
     */
    public GVoeuAmendement findByDeliberation( GFichier fichier, Plugin plugin )
    {
        return _voeuAmendementDAO.selectByDeliberation( fichier, plugin );
    }

    /**
     *
     * retourne la liste des voeux amendements parents associs au voeu amendement d'id nIdVoeuAmendement
     * @param nIdVoeuAmendement l'id du voeu amendement
     * @param plugin yhe plugin
     * @return la liste des voeux amendements parents associs au voeu amendement d'id nIdVoeuAmendement
     */
    public List<GVoeuAmendement> findVoeuxAmendementsParent( int nIdVoeuAmendement, Plugin plugin )
    {
        return _voeuAmendementDAO.selectVoeuxAmendementsParent( nIdVoeuAmendement, plugin );
    }

    /**
    *
    * retourne la liste des voeux amendements enfants associs au voeu amendement d'id nIdVoeuAmendement
    * @param nIdVoeuAmendement l'id du voeu amendement
    * @param plugin yhe plugin
    * @return la liste des voeux amendements enfants associs au voeu amendement d'id nIdVoeuAmendement
    */
    public List<GVoeuAmendement> findVoeuxAmendementsEnfant( int nIdVoeuAmendement, Plugin plugin )
    {
        return _voeuAmendementDAO.selectVoeuxAmendementsEnfant( nIdVoeuAmendement, plugin );
    }

    /**
     * retourne la liste des actes rattachs au voeu
     * @param nIdEntite l'identifiant du va
     * @param plugin le plugin
     * @return la liste des identifiants des actes
     */
    private List<Acte<GSeance, GFichier>> getListeActes( int nIdEntite, Plugin plugin )
    {
        GActeFilter acteFilter = _acteService.getActeFilter(  );
        acteFilter.setIdEntite( nIdEntite );
        acteFilter.setIdTypeEntite( ITypeEntite.VOEUX );

        List<Acte<GSeance, GFichier>> listActes = _acteHome.findByFilter( acteFilter, plugin );

        return listActes;
    }
}
