/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.business.voeuamendement;

import fr.paris.lutece.plugins.ods.dto.acte.IActeFilter;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendementFilter;
import fr.paris.lutece.portal.service.plugin.Plugin;

import java.util.List;


/**
 *
 * IVoeuAmendementDAO
 *
 * @param <GVoeuAmendement> {@link IVoeuAmendement}
 * @param <GVoeuAmendementFilter> {@link IVoeuAmendementFilter}
 * @param <GSeance> {@link ISeance}
 * @param <GElu> {@link IElu}
 * @param <GFichier> {@link IFichier}
 * @param <GPDD> {@link IPDD}
 */
public interface IVoeuAmendementDAO
<
	GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, 
	GVoeuAmendementFilter extends IVoeuAmendementFilter, 
	GSeance extends ISeance, 
	GElu extends IElu, 
	GFichier extends IFichier<GSeance, GFichier>, 
	GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>
>
{
    /**
     * Insre un nouvel enregistrement dans la table ods_voeu_amendement  partir de lobjet voeu_amendement
     *  pass  en paramtre
     * @param   voeuAmendement le voeuAmendement  insrer dans la table ods_voeu_amendement
     * @param plugin plugin
     * @return l'id du VoeuAmendement cr
     */
    int insert( GVoeuAmendement voeuAmendement, Plugin plugin );

    /**
     * Modifie l'enregistrement dans la table ods_voeu_Amendement et les tables associes correspondant   lobjet voeuAmendementpass en paramtre
     * @param voeuAmendement le voeuAmendement  modifier dans la table ods_voeu_amendement
     * @param plugin plugin
     */
    void store( GVoeuAmendement voeuAmendement, Plugin plugin );

    /**
     * Supprime l' enregistrement dans la table ods_voeu_amendement correspondant   lobjet VoeuAmendement pass en paramtre
     * @param voeuAmendement le voeuAmendement  supprimer dans la table ods_voeu_amendement
     * @param plugin plugin
     */
    void delete( GVoeuAmendement voeuAmendement, Plugin plugin );

    /**
     * Renvoie le VA ayant comme identifiant nKey
     * @param nKey Identifiant du VA
     * @param plugin Plugin
     * @return Objet VA
     */
    GVoeuAmendement load( int nKey, Plugin plugin );

    /**
     * renvoie la liste des voeux et Amendements  prsents dans la table ods_voeu_amendement rpondant
     *  aux criteres de selections du filtre
     * @param filter le filtre de selection
     * @param plugin  plugin
     * @return liste d'objets voeuAmendement
     */
    List<GVoeuAmendement> selectVoeuAmendementListByFilter( GVoeuAmendementFilter filter, Plugin plugin );

    /**
    * renvoie la liste des voeux et Amendements  prsents dans la table ods_voeu_amendement rpondant
    *  aux criteres de selections du filtre
    * @param nIdFormationConseil l'id de la formation du conseil
    * @param nIdSeance l'id de la seance
    * @param plugin plugin
    * @return liste d'objets voeuAmendement
    */
    List<GVoeuAmendement> selectLiasseByFilter( int nIdFormationConseil, int nIdSeance, Plugin plugin );

    /**
     * renvoie la liste des voeux et Amendements  prsents dans la table ods_voeu_amendement rpondant
     *  aux criteres de selections du filtre
     * @param nIdFormationConseil l'id de la formation du conseil
     * @param nIdSeance l'id de la sance
     * @param plugin plugin
     * @return le nombre de VAs correspondants
     */
    int selectCountLiasseByFilter( int nIdFormationConseil, int nIdSeance, Plugin plugin );

    /**
     * Retourne la liste des pdds associ au VoeuAmendement d'id nKey
     * @param nKey id du VA
     * @param plugin plugin
     * @return la liste des pdds du va
     */
    List<GPDD> selectPddListbyIdVoeuAmendement( int nKey, Plugin plugin );

    /**
     * Retourne le voeu amendement dont le texte initial est le fichier
     * @param fichier le fichier du voeu amendement
     * @param plugin le plugin
     * @return la voeu amendement dont le texte initial est le fichier
     */
    GVoeuAmendement selectByTexteInitial( GFichier fichier, Plugin plugin );

    /**
     * Retourne le voeu amendement dont la deliberation est le fichier
     * @param fichier le fichier du voeu amendement
     * @param plugin le plugin
     * @return la voeu amendement dont le texte initial est le fichier
     */
    GVoeuAmendement selectByDeliberation( GFichier fichier, Plugin plugin );

    /**
     * Modifie la reference du va utilise cot front office dans la table ods_voeu_Amendement
     * @param voeuAmendement le voeuAmendement  modifier dans la table ods_voeu_amendement
     * @param plugin plugin
     */
    void storeReferenceFront( GVoeuAmendement voeuAmendement, Plugin plugin );

    /**
     *
     * retourne la liste des voeux amendements parents associs au voeu amendement d'id nIdVoeuAmendement
     * @param nIdVoeuAmendement l'id du voeu amendement
     * @param plugin yhe plugin
     * @return la liste des voeux amendements parents associs au voeu amendement d'id nIdVoeuAmendement
     */
    List<GVoeuAmendement> selectVoeuxAmendementsParent( int nIdVoeuAmendement, Plugin plugin );

    /**
    *
    * retourne la liste des voeux amendements enfants associs au voeu amendement d'id nIdVoeuAmendement
    * @param nIdVoeuAmendement l'id du voeu amendement
    * @param plugin yhe plugin
    * @return la liste des voeux amendements enfants associs au voeu amendement d'id nIdVoeuAmendement
    */
    List<GVoeuAmendement> selectVoeuxAmendementsEnfant( int nIdVoeuAmendement, Plugin plugin );
}
