/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto;

import fr.paris.lutece.plugins.ods.dto.categoriedeliberation.CategorieDeliberation;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;

import java.sql.Timestamp;


/**
 *
 * IDesignation
 *
 * @param <GSeance> seance
 * @param <GFichier> fichier
 */
public interface IDesignation<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>>
    extends IBusinessItem
{
    /**
     * Retourne la date de publication
     * @return Timestamp _tsDatePublication
     */
    Timestamp getDatePublication(  );

    /**
     * Affecte la date de publication
     * @param tsDatePublication Timestamp
     */
    void setDatePublication( Timestamp tsDatePublication );

    /**
     * Retourne le boolean qui spcifie la mise en ligne
     * @return Boolean
     */
    Boolean getEnLigne(  );

    /**
     * Affecte le boolean qui spcifie la mise en ligne
     * @param bEnLigne le statut du fichier
     */
    void setEnLigne( Boolean bEnLigne );

    /**
     * Retourne l'objet Seance
     * @return Seance _seance
     */
    GSeance getSeance(  );

    /**
     * Affecte l'objet Seance
     * @param seance la sance
     */
    void setSeance( GSeance seance );

    /**
     * Affecte l'intitule
     * @param strIntitule le texte
     */
    void setIntitule( String strIntitule );

    /**
     * Retourne la Version
     * @return int _nVersion
     */
    int getVersion(  );

    /**
     * Affecte la Version
     * @param nVersion le numro de la version
     */
    void setVersion( int nVersion );

    /**
     * Affecte Id
     * @param nId l'identifiant d'un fichier
     */
    void setId( int nId );

    /**
     * renvoie le hashcode de l'objet
     * @return le hashcode
     */
    int hashCode(  );

    /**
     * @return la formation conseil
     */
    FormationConseil getFormationConseil(  );

    /**
     * @param conseil la formation conseil
     */
    void setFormationConseil( FormationConseil conseil );

    /**
     * @return le numero
     */
    String getNumero(  );

    /**
     * @param numero le numero
     */
    void setNumero( String numero );

    /**
     * @return le fichier original
     */
    GFichier getFichierOr(  );

    /**
     * @param fichierOr le fichier original
     */
    void setFichierOr( GFichier fichierOr );

    /**
     * @return le fichier scanne
     */
    GFichier getFichierSc(  );

    /**
     * @param fichierSc le fichier scanne
     */
    void setFichierSc( GFichier fichierSc );

    /**
     * @return la categorie
     */
    CategorieDeliberation getCategorie(  );

    /**
     * @param categorie la categorie
     */
    void setCategorie( CategorieDeliberation categorie );

    /**
     * gnration XML
     * @return gnre Dlibration en XML
     */
    String getXml(  );
}
