/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.acte;

import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;

import java.sql.Timestamp;


/**
 *
 * AbstractActeFilter
 *
 * @param <GFichier> IFichier
 * @param <GSeance> ISeance
 */
public class AbstractActeFilter<GFichier extends IFichier<GSeance, GFichier>, GSeance extends ISeance>
    implements IActeFilter<GFichier, GSeance>
{
    // Variables declarations
    private int _nIdFormationConseil = ALL_INT;
    private int _nSeance = ALL_INT;
    private int _nCategorieDeliberation = ALL_INT;
    private int _nStatut = ALL_INT;
    private int _nIdTypeEntite = ALL_INT;
    private int _nIdEntite = ALL_INT;
    private Timestamp _tsDateRetourControleDeLegalite;
    private String _strReference = ALL_STRING;
    private GFichier _fichierDeliberationFinale;
    private boolean _bOrderByFormationConseil;
    private boolean _bOrderByStatut;
    private boolean _bOrderByTransmission;

    /**
     * @return _nSeance
     */
    public int getIdSeance(  )
    {
        return _nSeance;
    }

    /**
     * @param seance _nSeance to set
     */
    public void setIdSeance( int seance )
    {
        _nSeance = seance;
    }

    /**
     * @return boolean TRUE si _nSeance est actif, FALSE sinon
     */
    public boolean containsSeanceCriteria(  )
    {
        return ( _nSeance != ALL_INT );
    }

    /**
     * @return _nIdFormationConseil int
     */
    public int getIdFormationConseil(  )
    {
        return _nIdFormationConseil;
    }

    /**
     * @param idFormationConseil _nIdFormationConseil to set
     */
    public void setIdFormationConseil( int idFormationConseil )
    {
        _nIdFormationConseil = idFormationConseil;
    }

    /**
     * @return boolean TRUE si _nIdFormationConseil est actif, FALSE sinon
     */
    public boolean containsFormationConseilCriteria(  )
    {
        return ( _nIdFormationConseil != ALL_INT );
    }

    /**
     * @return _nCategorieDeliberation int
     */
    public int getCategorieDeliberation(  )
    {
        return _nCategorieDeliberation;
    }

    /**
     * @param categorieDeliberation _nCategorieDeliberation to set
     */
    public void setCategorieDeliberation( int categorieDeliberation )
    {
        _nCategorieDeliberation = categorieDeliberation;
    }

    /**
     *
     * @return boolean TRUE si _nCategorieDeliberation est actif, FALSE sinon
     */
    public boolean containsCategorieDeliberationCriteria(  )
    {
        return ( _nCategorieDeliberation != ALL_INT );
    }

    /**
     * @return _nStatut int
     */
    public int getStatut(  )
    {
        return _nStatut;
    }

    /**
     * @param statut _nStatut to set
     */
    public void setStatut( int statut )
    {
        _nStatut = statut;
    }

    /**
     * @return boolean TRUE si _nStatut est actif, FALSE sinon
     */
    public boolean containsStatutCriteria(  )
    {
        return ( _nStatut != ALL_INT );
    }

    /**
     * @return _tsDateRetourControleDeLegalite Timestamp
     */
    public Timestamp getDateRetourControleDeLegalite(  )
    {
        return _tsDateRetourControleDeLegalite;
    }

    /**
     * @param tsDateRetourControleDeLegalite _tsDateRetourControleDeLegalite to set
     */
    public void setDateRetourControleDeLegalite( Timestamp tsDateRetourControleDeLegalite )
    {
        _tsDateRetourControleDeLegalite = tsDateRetourControleDeLegalite;
    }

    /**
     * @return boolean TRUE si _tsDateRetourControleDeLegalite est actif, FALSE sinon
     */
    public boolean containsDateRetourControleDeLegaliteCriteria(  )
    {
        return ( _tsDateRetourControleDeLegalite != null );
    }

    /**
     * @return _fichierDeliberationFinale Fichier
     */
    public GFichier getFichierDeliberationFinale(  )
    {
        return _fichierDeliberationFinale;
    }

    /**
     * @param fichierDeliberationFinale _fichierDeliberationFinale to set
     */
    public void setFichierDeliberationFinale( GFichier fichierDeliberationFinale )
    {
        _fichierDeliberationFinale = fichierDeliberationFinale;
    }

    /**
     * @return boolean TRUE si _fichierDeliberationFinale est actif, FALSE sinon
     */
    public boolean containsDeliberationFinaleCriteria(  )
    {
        return ( _fichierDeliberationFinale != null );
    }

    /**
     * @return _nIdEntite int
     */
    public int getIdEntite(  )
    {
        return _nIdEntite;
    }

    /**
     * @param nIdEntite _nIdEntite to set
     */
    public void setIdEntite( int nIdEntite )
    {
        _nIdEntite = nIdEntite;
    }

    /**
     * @return boolean TRUE si _nIdEntite est actif, FALSE sinon
     */
    public boolean containsIdEntiteCriteria(  )
    {
        return ( _nIdEntite != ALL_INT );
    }

    /**
     * @return the _strTypeEntite
     */
    public int getIdTypeEntite(  )
    {
        return _nIdTypeEntite;
    }

    /**
     * @param nIdTypeEntite the _nIdTypeEntite to set
     */
    public void setIdTypeEntite( int nIdTypeEntite )
    {
        _nIdTypeEntite = nIdTypeEntite;
    }

    /**
     * @return boolean TRUE si _strTypeEntite est actif, FALSE sinon
     */
    public boolean containsTypeEntiteCriteria(  )
    {
        return ( _nIdTypeEntite != ALL_INT );
    }

    /**
     * @return the _strReference
     */
    public String getReference(  )
    {
        return _strReference;
    }

    /**
     * @param strReference the _strTypeEntite to set
     */
    public void setrefrence( String strReference )
    {
        _strReference = strReference;
    }

    /**
     * @return boolean TRUE si _strReference est actif, FALSE sinon
     */
    public boolean containsReferenceCriteria(  )
    {
        return ( !_strReference.equals( ALL_STRING ) );
    }

    /**
     * @return boolean TRUE si _bOrderByFormationConseil est actif, FALSE sinon
     */
    public boolean getOrderByFormationConseil(  )
    {
        return _bOrderByFormationConseil;
    }

    /**
     * @param bOrderByFormationConseil the _bOrderByFormationConseil to set
     */
    public void setOrderByFormationConseil( boolean bOrderByFormationConseil )
    {
        _bOrderByFormationConseil = bOrderByFormationConseil;
    }

    /**
     * @return boolean TRUE si _bOrderByStatut est actif, FALSE sinon
     */
    public boolean getOrderByStatut(  )
    {
        return _bOrderByStatut;
    }

    /**
     * @param bOrderByStatut the _bOrderByStatut to set
     */
    public void setOrderByStatut( boolean bOrderByStatut )
    {
        _bOrderByStatut = bOrderByStatut;
    }

    /**
     * @return boolean TRUE si _bOrderByTransmission est actif, FALSE sinon
     */
    public boolean getOrderByTransmission(  )
    {
        return _bOrderByTransmission;
    }

    /**
     * @param bOrderByTransmission the _bOrderByTransmission to set
     */
    public void setOrderByTransmission( boolean bOrderByTransmission )
    {
        _bOrderByTransmission = bOrderByTransmission;
    }
}
