/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.acte;

import fr.paris.lutece.plugins.ods.dto.categoriedeliberation.CategorieDeliberation;
import fr.paris.lutece.plugins.ods.dto.direction.Direction;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.util.xml.XmlUtil;

import java.sql.Timestamp;

import java.util.HashMap;


/**
 * Acte
 *
 * @param <GSeance> seance
 * @param <GFichier> fichier
 */
public class Acte<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>>
{
    public static final String CONSTANTE_TYPE_PDD = "PDD";
    public static final String CONSTANTE_TYPE_VA = "VA";
    public static final String CONSTANTE_TYPE_ARRETE = "ARRETE";
    public static final String CONSTANTE_TYPE_DELIBERATION_DESIGNATION = "DESIGNATION";
    public static final String CONSTANTE_TYPE_MEMBRE_PRESENT = "MEMBRE";
    public static final int CONSTANTE_TRANSMISSION_AUTOMATIQUE = 0;
    public static final int CONSTANTE_TRANSMISSION_MANUELLE = 1;
    public static final String MARK_ACTE = "acte";
    private int _nId;
    private String _strReference;
    private int _nNumeroDelibere;
    private int _nIdEntite;
    private int _nIdTypeEntite;
    private ISeance _seance;
    private FormationConseil _formationConseil;
    private NatureActe _natureDocument;
    private StatutActe _statutActe;
    private CategorieDeliberation _categorie;
    private Direction _direction;
    private Timestamp _tsDateRetourControleLegalite;
    private GFichier _fDeliberationFinale;
    private int _dPoids;
    private int _nIdTransmission = 1;
    private String _strAnnulation;
    private int _nIdGroupe;
    private Boolean _bEnLigne;

    /**
     * Retourne l'id de l'acte
     * @return int _nId
     */
    public int getId(  )
    {
        return _nId;
    }

    /**
     * Affecte l'id de l'acte
     * @param nId int
     */
    public void setId( int nId )
    {
        this._nId = nId;
    }

    /**
     * Retourne l'id
     * @return int _nId
     */
    public int hashCode(  )
    {
        return getId(  );
    }

    /**
     * Retourne la catgorie de deliberation
     * @return CategorieDeliberation _categorie
     */
    public CategorieDeliberation getCategorie(  )
    {
        return _categorie;
    }

    /**
     * Affecte la categorie de deliberation
     * @param categorie CategorieDeliberation
     */
    public void setCategorie( CategorieDeliberation categorie )
    {
        this._categorie = categorie;
    }

    /**
     * Retourne la date de retour du contrle de lgalit
     * @return Timestamp _tsDateRetourControleLegalite
     */
    public Timestamp getDateRetourControleLegalite(  )
    {
        return _tsDateRetourControleLegalite;
    }

    /**
     * Affecte la date de retour du contrle de lgalit
     * @param dateRetourControleLegalite Timestamp
     */
    public void setDateRetourControleLegalite( Timestamp dateRetourControleLegalite )
    {
        _tsDateRetourControleLegalite = dateRetourControleLegalite;
    }

    /**
     * Retourne la reference de l'acte
     * @return String _strReference
     */
    public String getReference(  )
    {
        return _strReference;
    }

    /**
    * Affecte la reference de l'acte
    * @param strReference la reference de l'acte
    */
    public void setReference( String strReference )
    {
        _strReference = strReference;
    }

    /**
    * Retourne le numro de dlibr
    * @return _nNumeroDelibere le numro de dlibr
    */
    public int getNumeroDelibere(  )
    {
        return _nNumeroDelibere;
    }

    /**
    * Affecte le numro de dlibr
    * @param  nNumeroDelibere le numro de dlibr
    */
    public void setNumeroDelibere( int nNumeroDelibere )
    {
        _nNumeroDelibere = nNumeroDelibere;
    }

    /**
    * Retourne l'entite de l'acte
    * @return int _nIdEntite
    */
    public int getIdEntite(  )
    {
        return _nIdEntite;
    }

    /**
    * Affecte l'entite de l'acte
    * @param  idEntite l'entite de l'acte
    */
    public void setIdEntite( int idEntite )
    {
        _nIdEntite = idEntite;
    }

    /**
    * Retourne l'id du type de l'entite de l'acte
    * @return int _nIdTypeEntite
    */
    public int getIdTypeEntite(  )
    {
        return _nIdTypeEntite;
    }

    /**
    * Affecte l'id du type de l'entite de l'acte
    * @param nIdTypeEntite l'id du type de l'entite de l'acte
    */
    public void setIdTypeEntite( int nIdTypeEntite )
    {
        _nIdTypeEntite = nIdTypeEntite;
    }

    /**
    * Retourne la formation du conseil de l'acte
    * @return FormationConseil _formationConseil
    */
    public FormationConseil getFormationConseil(  )
    {
        return _formationConseil;
    }

    /**
    * Affecte la formation du conseil de l'acte
    * @param formationConseil la formation du conseil de l'acte
    */
    public void setFormationConseil( FormationConseil formationConseil )
    {
        _formationConseil = formationConseil;
    }

    /**
    * Retourne la nature de l'acte
    * @return int _nIdNature
    */
    public NatureActe getNatureDocument(  )
    {
        return _natureDocument;
    }

    /**
    * Affecte la nature de l'acte
    * @param natureDocument la nature de l'acte
    */
    public void setNatureDocument( NatureActe natureDocument )
    {
        _natureDocument = natureDocument;
    }

    /**
    * Retourne le statut de l'acte
    * @return StatutActe _statutActe
    */
    public StatutActe getStatutActe(  )
    {
        return _statutActe;
    }

    /**
    * Affecte le statut de l'acte
    * @param statutActe le statut de l'acte
    */
    public void setStatutActe( StatutActe statutActe )
    {
        _statutActe = statutActe;
    }

    /**
    * Retourne la direction
    * @return Direction _direction
    */
    public Direction getDirection(  )
    {
        return _direction;
    }

    /**
    * Affecte la direction
    * @param direction la direction
    */
    public void setDirection( Direction direction )
    {
        this._direction = direction;
    }

    /**
    * Retourne la dlibration finale
    * @return Fichier _fDeliberationFinale
    */
    public GFichier getDeliberationFinale(  )
    {
        return _fDeliberationFinale;
    }

    /**
    * Affecte la dlibration finale
    * @param fDeliberationFinale la dlibration finale
    */
    public void setDeliberationFinale( GFichier fDeliberationFinale )
    {
        _fDeliberationFinale = fDeliberationFinale;
    }

    /**
    * Retourne le poids des fichiers selectionnes
    * @return double _dPoids
    */
    public int getPoids(  )
    {
        return _dPoids;
    }

    /**
    * Affecte le poids des fichiers selectionnes
    * @param  dPoids le poids des fichiers selectionnes
    */
    public void setPoids( int dPoids )
    {
        _dPoids = dPoids;
    }

    /**
    * Retourne l'id de transmission
    * @return int _nIdTransmission
    */
    public int getIdTransmission(  )
    {
        return _nIdTransmission;
    }

    /**
    * Affecte l'id de transmission
    * @param nIdTransmission l'id de transmission
    */
    public void setIdTransmission( int nIdTransmission )
    {
        _nIdTransmission = nIdTransmission;
    }

    /**
    * Retourne le commentaire d'annulation de l'acte
    * @return String _strAnnulation
    */
    public String getComAnnulation(  )
    {
        return _strAnnulation;
    }

    /**
    * Affecte le commentaire d'annulation de l'acte
    * @param strAnnulation le commentaire d'annulation de l'acte
    */
    public void setComAnnulation( String strAnnulation )
    {
        _strAnnulation = strAnnulation;
    }

    /**
    * Retourne l'id du groupe politique
    * @return int _nIdGroupe
    */
    public int getIdGroupe(  )
    {
        return _nIdGroupe;
    }

    /**
     * Retourne la sance
     * @return Seance _seance
     */
    public ISeance getSeance(  )
    {
        return _seance;
    }

    /**
     * Affecte la seance
     * @param  seance la seance
     */
    public void setSeance( ISeance seance )
    {
        this._seance = seance;
    }

    /**
    * Affecte l'id du groupe politique
    * @param nIdGroupe l'id du groupe politique
    */
    public void setIdGroupe( int nIdGroupe )
    {
        _nIdGroupe = nIdGroupe;
    }

    /**
    * Retourne TRUE s'il est en ligne, FALSE sinon
    * @return Boolean _bEnLigne
    */
    public Boolean getEnLigne(  )
    {
        return _bEnLigne;
    }

    /**
     * Affecte le boolen enLigne
     * @param bEnLigne Boolean
     */
    public void setEnLigne( Boolean bEnLigne )
    {
        this._bEnLigne = bEnLigne;
    }

    /**
     * Retourne le poids en Mo
     * @return double le poids en Mo
     */
    public double getPoidsMo(  )
    {
        double poidsMo = (double) _dPoids / 1000000.0;

        return Math.round( poidsMo * 100 ) * 0.01;
    }

    /**
    * Retourne le flux XML
    * @return String le flux XML
    */
    public String getXml(  )
    {
        StringBuffer buffer = new StringBuffer(  );
        HashMap<String, Object> attributes = new HashMap<String, Object>(  );
        XmlUtil.beginElement( buffer, MARK_ACTE, attributes );
        XmlUtil.endElement( buffer, MARK_ACTE );

        return buffer.toString(  );
    }
}
