/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.acte;

import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.util.xml.XmlUtil;

import java.util.HashMap;


/**
 * Statut d'un acte
 */
public class DocumentActe
{
    private static final String TAG_ID_ACTE = "id_acte";
    private static final String TAG_ID_DOCUMENT = "id_document";
    private static final String TAG_DOCUMENT = "documentActe";
    private int _nIdDocument = -1;
    private int _nIdActe;

    /**
     * Retourne l'id du document de l'acte
     * @return l'id du document de l'acte
     */
    public int getIdDocument(  )
    {
        return _nIdDocument;
    }

    /**
     * Fixe l'id du document de l'acte
     * @param nIdDocument l'id du document de l'acte
     */
    public void setIdDocument( int nIdDocument )
    {
        _nIdDocument = nIdDocument;
    }

    /**
     * Retourne l'id de l'acte
     * @return l'id de l'acte
     */
    public int getIdActe(  )
    {
        return _nIdActe;
    }

    /**
     * Fixe l'id de l'acte
     * @param nIdActe l'id de l'acte
     */
    public void setIdActe( int nIdActe )
    {
        _nIdActe = nIdActe;
    }

    /**
    * Gnration du XML d'un objet
    * @return Le code XML reprsentant le document d'un acte
    */
    public String getXml(  )
    {
        StringBuffer buffer = new StringBuffer(  );
        HashMap<String, Object> attributes = new HashMap<String, Object>(  );

        XmlUtil.beginElement( buffer, TAG_DOCUMENT, attributes );

        OdsUtils.addElement( buffer, TAG_ID_DOCUMENT, _nIdDocument );

        if ( _nIdActe != -1 )
        {
            OdsUtils.addElement( buffer, TAG_ID_ACTE, _nIdActe );
        }

        XmlUtil.endElement( buffer, TAG_DOCUMENT );

        return buffer.toString(  );
    }
}
