/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.arrete;

import fr.paris.lutece.plugins.ods.dto.IDesignation;
import fr.paris.lutece.plugins.ods.dto.categoriedeliberation.CategorieDeliberation;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.historique.IEntiteHistorique;
import fr.paris.lutece.plugins.ods.dto.horodatage.IEntiteHorodatage;
import fr.paris.lutece.plugins.ods.dto.horodatage.IHorodatageFamilleDocument;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.typeentite.ITypeEntite;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.util.xml.XmlUtil;

import java.sql.Timestamp;

import java.util.HashMap;


/**
 * Arrete
 *
 * @param <GSeance> seance
 * @param <GFichier> fichier
 */
public class Arrete<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>> implements IDesignation<GSeance, GFichier>,
    IEntiteHorodatage, IEntiteHistorique
{
    private static final String SEANCE = "seance";
    private static final String FORMATION_CONSEIL = "formationConseil";
    private static final String VERSION = "version";
    private static final String DATE_PUBLICATION = "datePublication";
    private static final String CATEGORIE = "categorieDeliberation";
    private static final String DATE_RETOUR_CONTROLE_LEGALITE = "dateRetourControleLegalite";
    private static final String EN_LIGNE = "enLigne";
    private static final String ID = "id";
    private int _nId;
    private GSeance _seance;
    private FormationConseil _formationConseil;
    private String _strNumero;
    private String _strIntitule;
    private GFichier _fichierOr;
    private GFichier _fichierSc;
    private CategorieDeliberation _categorie;
    private Timestamp _tsDateRetourControleLegalite;
    private Boolean _bEnLigne;
    private Timestamp _tsDatePublication;
    private int _nVersion;

    /**
     * Retourne la date de publication
     * @return Timestamp _tsDatePublication
     */
    public Timestamp getDatePublication(  )
    {
        return _tsDatePublication;
    }

    /**
     * Affecte la date de publication
     * @param tsDatePublication Timestamp
     */
    public void setDatePublication( Timestamp tsDatePublication )
    {
        this._tsDatePublication = tsDatePublication;
    }

    /**
     * Retourne TRUE s'il est en ligne, FALSE sinon
     * @return Boolean _bEnLigne
     */
    public Boolean getEnLigne(  )
    {
        return _bEnLigne;
    }

    /**
     * Affecte le boolen enLigne
     * @param bEnLigne Boolean
     */
    public void setEnLigne( Boolean bEnLigne )
    {
        this._bEnLigne = bEnLigne;
    }

    /**
     * Retourne la sance
     * @return Seance _seance
     */
    public GSeance getSeance(  )
    {
        return _seance;
    }

    /**
     * Affecte la sance
     * @param seance Seance
     */
    public void setSeance( GSeance seance )
    {
        this._seance = seance;
    }

    /**
     * Retourne l'intitule
     * @return String _strIntitule
     */
    public String getIntitule(  )
    {
        return _strIntitule;
    }

    /**
     * Affecte l'intitule
     * @param strIntitule String
     */
    public void setIntitule( String strIntitule )
    {
        this._strIntitule = strIntitule;
    }

    /**
     * Retourne la version
     * @return int _nVersion
     */
    public int getVersion(  )
    {
        return _nVersion;
    }

    /**
     * Affecte la verison
     * @param nVersion int
     */
    public void setVersion( int nVersion )
    {
        this._nVersion = nVersion;
    }

    /**
     * Retourne la version
     * @return int _nVersion
     */
    public int getId(  )
    {
        return _nId;
    }

    /**
     * Affecte l'id
     * @param nId int
     */
    public void setId( int nId )
    {
        this._nId = nId;
    }

    /**
     * Retourne l'id
     * @return int _nId
     */
    public int hashCode(  )
    {
        return getId(  );
    }

    /**
     * Retourne la formation conseil
     * @return FormationConseil _formationConseil
     */
    public FormationConseil getFormationConseil(  )
    {
        return _formationConseil;
    }

    /**
     * Affecte la formation conseil
     * @param conseil FormationConseil
     */
    public void setFormationConseil( FormationConseil conseil )
    {
        _formationConseil = conseil;
    }

    /**
     * Retourne le numro
     * @return String _strNumero
     */
    public String getNumero(  )
    {
        return _strNumero;
    }

    /**
     * Affecte le numero
     * @param strNumero String
     */
    public void setNumero( String strNumero )
    {
        _strNumero = strNumero;
    }

    /**
     * Retourne le fichier original
     * @return Fichier _fichierOr
     */
    public GFichier getFichierOr(  )
    {
        return _fichierOr;
    }

    /**
     * Affecte le fichier original
     * @param fichierOr Fichier
     */
    public void setFichierOr( GFichier fichierOr )
    {
        this._fichierOr = fichierOr;
    }

    /**
     * Retourne le fichier scanne
     * @return Fichier _fichierSc
     */
    public GFichier getFichierSc(  )
    {
        return _fichierSc;
    }

    /**
     * Affecte le fichier scanne
     * @param fichierSc Fichier
     */
    public void setFichierSc( GFichier fichierSc )
    {
        this._fichierSc = fichierSc;
    }

    /**
     * Retourne la catgorie de deliberation
     * @return CategorieDeliberation _categorie
     */
    public CategorieDeliberation getCategorie(  )
    {
        return _categorie;
    }

    /**
     * Affecte la categorie de deliberation
     * @param categorie CategorieDeliberation
     */
    public void setCategorie( CategorieDeliberation categorie )
    {
        this._categorie = categorie;
    }

    /**
     * Retourne la date de retour du contrle de lgalit
     * @return Timestamp _tsDateRetourControleLegalite
     */
    public Timestamp getDateRetourControleLegalite(  )
    {
        return _tsDateRetourControleLegalite;
    }

    /**
     * Affecte la date de retour du contrle de lgalit
     * @param dateRetourControleLegalite Timestamp
     */
    public void setDateRetourControleLegalite( Timestamp dateRetourControleLegalite )
    {
        _tsDateRetourControleLegalite = dateRetourControleLegalite;
    }

    /**
     * Retourne le flux XML
     * @return String le flux XML
     */
    public String getXml(  )
    {
        StringBuffer buffer = new StringBuffer(  );
        HashMap<String, Object> attributes = new HashMap<String, Object>(  );
        attributes.put( ID, OdsConstants.CONSTANTE_CHAINE_VIDE + _nId );
        XmlUtil.beginElement( buffer, OdsMarks.MARK_ARRETE, attributes );

        if ( _seance != null )
        {
            OdsUtils.addElement( buffer, SEANCE, _seance.getIdSeance(  ) );
        }

        if ( _formationConseil != null )
        {
            OdsUtils.addElement( buffer, FORMATION_CONSEIL, _formationConseil.getLibelle(  ) );
        }

        if ( _categorie != null )
        {
            OdsUtils.addElement( buffer, CATEGORIE, _categorie.getLibelle(  ) );
        }

        OdsUtils.addElement( buffer, EN_LIGNE, OdsConstants.CONSTANTE_CHAINE_VIDE + _bEnLigne );

        if ( _tsDatePublication != null )
        {
            OdsUtils.addElement( buffer, DATE_PUBLICATION, OdsConstants.CONSTANTE_CHAINE_VIDE + _tsDatePublication );
        }

        if ( _tsDateRetourControleLegalite != null )
        {
            OdsUtils.addElement( buffer, DATE_RETOUR_CONTROLE_LEGALITE,
                OdsConstants.CONSTANTE_CHAINE_VIDE + _tsDateRetourControleLegalite );
        }

        if ( _nVersion != -1 )
        {
            OdsUtils.addElement( buffer, VERSION, OdsConstants.CONSTANTE_CHAINE_VIDE + _nVersion );
        }

        XmlUtil.endElement( buffer, OdsMarks.MARK_ARRETE );

        return buffer.toString(  );
    }

    /**
     * @return le type de document
     */
    public String getFamilleDocumentEnum(  )
    {
        return IHorodatageFamilleDocument.ARRETE;
    }

    /**
     * @return la reference
     */
    public String getReference(  )
    {
        return null;
    }

    /**
     * @return le type d'entite
     */
    public int getTypeEntite(  )
    {
        return ITypeEntite.ARRETES;
    }
}
