/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.arrete;

import java.sql.Timestamp;


/**
 * Un objet ArreteFilter est utilis pour retrouver les arrets selon certains critres:
 * la sance, la formation conseil, s'il est publi ou non.
 */
public class ArreteFilter
{
    public static final String ALL_STRING = "all";
    public static final int ALL_INT = -1;
    public static final int TYPE_DOCUMENT_GESTION_AVAL = 2;
    private int _nFormationConseil = ALL_INT;
    private int _nIdSeance = ALL_INT;
    private Timestamp _tsDatePublication;

    /**
     * @return _tsDatePublication Timestamp
     */
    public Timestamp getDatePublication(  )
    {
        return _tsDatePublication;
    }

    /**
     * @param tsDatePubliocation _tsDatePublication to set
     */
    public void setDatePublication( Timestamp tsDatePubliocation )
    {
        _tsDatePublication = tsDatePubliocation;
    }

    /**
     * @return boolean TRUE si _tsDatePublication est actif, FALSE sinon
     */
    public boolean containsDatePublicationCriteria(  )
    {
        return ( _tsDatePublication != null );
    }

    /**
     * Retourne la sance  filtrer
     * @return la sance  filtrer
     */
    public int getIdSeance(  )
    {
        return _nIdSeance;
    }

    /**
     * Fixe la valeur du filtre sance
     * @param nIdSeance la sance  filtrer
     */
    public void setIdSeance( int nIdSeance )
    {
        _nIdSeance = nIdSeance;
    }

    /**
     * Indique si le filtre sur la sance est activ.
     * @return <b>true</b> si le filtre sur lsance est positionn, <b>false</b> sinon
     */
    public boolean containsSeanceCriteria(  )
    {
        return ( _nIdSeance != ALL_INT );
    }

    /**
     * Retourne la formation conseil  filtrer
     * @return la formation conseil  filtrer
     */
    public int getIdFormationConseil(  )
    {
        return _nFormationConseil;
    }

    /**
     * Fixe le filtre sur la formation Conseil
     * @param nFormationConseil la formation conseil  filtrer
     */
    public void setIdFormationConseil( int nFormationConseil )
    {
        _nFormationConseil = nFormationConseil;
    }

    /**
     * Indique si le filtre sur la formation conseil est positionn
     * @return <b>true</b> si le filtre sur lla formation conseil est positionn, <b>false</b> sinon
     */
    public boolean containsFormationConseilCriteria(  )
    {
        return ( _nFormationConseil != ALL_INT );
    }
}
