/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.autremodele;

import fr.paris.lutece.plugins.ods.dto.fichier.FichierPhysique;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.util.xml.XmlUtil;

import java.util.HashMap;


/**
 *
 * AbstractAutreModele
 *
 */
public abstract class AbstractAutreModele implements IAutreModele
{
    private static final String TAG_TITRE = "titre";
    private static final String TAG_ID = "id";
    private static final String TAG_AUTRE_MODELE = "autreModele";
    private int _nIdAutreModele;
    private String _strTitre;
    private TypeAutreModele _typeAutreModele;
    private FichierPhysique _fEnteteDocument;
    private FichierPhysique _fPiedDocument;

    /**
     * Generation du flux XML specifique
     * @param buffer StringBuffer
     */
    protected abstract void getXmlSpec( StringBuffer buffer );

    /**
     * @return _fEnteteDocument
     */
    public FichierPhysique getEnteteDocument(  )
    {
        return _fEnteteDocument;
    }

    /**
     * @param enTete l'en tete
     */
    public void setEnteteDocument( FichierPhysique enTete )
    {
        _fEnteteDocument = enTete;
    }

    /**
     * @return _fPiedDocument
     */
    public FichierPhysique getPiedDocument(  )
    {
        return _fPiedDocument;
    }

    /**
     * @param piedDocument le pied du document
     */
    public void setPiedDocument( FichierPhysique piedDocument )
    {
        _fPiedDocument = piedDocument;
    }

    /**
     * @return _nIdAutreModele
     */
    public int getIdAutreModele(  )
    {
        return _nIdAutreModele;
    }

    /**
     * @param nIdAutreModele l'identifiant du modele
     */
    public void setIdAutreModele( int nIdAutreModele )
    {
        _nIdAutreModele = nIdAutreModele;
    }

    /**
     * @return _strTitre
     */
    public String getTitre(  )
    {
        return _strTitre;
    }

    /**
     * @param titre le titre
     */
    public void setTitre( String titre )
    {
        _strTitre = titre;
    }

    /**
     * @return _typeOrdreDuJour
     */
    public TypeAutreModele getTypeAutreModele(  )
    {
        return _typeAutreModele;
    }

    /**
     * @param typeAutreModele le type d'autre modle
     */
    public void setTypeAutreModele( TypeAutreModele typeAutreModele )
    {
        _typeAutreModele = typeAutreModele;
    }

    /**
     * Gnration du XML d'un objet
     * @return Le code XML reprsentant un modle ordre du jour
     */
    public String getXml(  )
    {
        StringBuffer buffer = new StringBuffer(  );
        HashMap<String, Object> attributes = new HashMap<String, Object>(  );

        attributes.put( TAG_ID, _nIdAutreModele );
        XmlUtil.beginElement( buffer, TAG_AUTRE_MODELE, attributes );

        if ( _strTitre != null )
        {
            OdsUtils.addElement( buffer, TAG_TITRE, _strTitre );
        }

        if ( _fEnteteDocument != null )
        {
            buffer.append( _fEnteteDocument.getXml(  ) );
        }

        if ( _fPiedDocument != null )
        {
            buffer.append( _fPiedDocument.getXml(  ) );
        }

        if ( _typeAutreModele != null )
        {
            buffer.append( _typeAutreModele.getXml(  ) );
        }

        getXmlSpec( buffer );

        XmlUtil.endElement( buffer, TAG_AUTRE_MODELE );

        return buffer.toString(  );
    }
}
