/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.deliberationdesignation;

import fr.paris.lutece.plugins.ods.dto.IDesignation;
import fr.paris.lutece.plugins.ods.dto.acte.Acte;
import fr.paris.lutece.plugins.ods.dto.acte.IEntiteActe;
import fr.paris.lutece.plugins.ods.dto.acte.StatutActeEnum;
import fr.paris.lutece.plugins.ods.dto.categoriedeliberation.CategorieDeliberation;
import fr.paris.lutece.plugins.ods.dto.certificataffichage.IEntiteCertificatAffichage;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.historique.IEntiteHistorique;
import fr.paris.lutece.plugins.ods.dto.horodatage.IEntiteHorodatage;
import fr.paris.lutece.plugins.ods.dto.horodatage.IHorodatageFamilleDocument;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.typeentite.ITypeEntite;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.util.xml.XmlUtil;

import java.sql.Timestamp;

import java.util.HashMap;
import java.util.List;


/**
 * DeliberationDesignation
 *
 * @param <GSeance> seance
 * @param <GFichier> fichier
 */
public class DeliberationDesignation<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>>
    implements IDesignation<GSeance, GFichier>, IEntiteActe, IEntiteHorodatage, IEntiteCertificatAffichage,
        IEntiteHistorique, IDeliberationDesignation<GSeance, GFichier>
{
    private static final String SEANCE = "seance";
    private static final String NUMERO = "numero";
    private static final String INTITULE = "intitule";
    private static final String VERSION = "version";
    private static final String DATE_PUBLICATION = "datePublication";
    private static final String DATE_VOTE = "dateVote";
    private static final String CATEGORIE = "categorieDeliberation";
    private static final String EN_LIGNE = "enLigne";
    private static final String ID = "id";
    private int _nId;
    private GSeance _seance;
    private FormationConseil _formationConseil;
    private String _strNumero;
    private String _strIntitule;
    private GFichier _fichierOr;
    private GFichier _fichierSc;
    private CategorieDeliberation _categorie;
    private Timestamp _tsDateVote;

    /*private Timestamp _tsDateRetourControleLegalite;*/
    private Boolean _bEnLigne;
    private Timestamp _tsDatePublication;
    private int _nVersion;
    private Acte<GSeance, GFichier> _acteCourant;
    private List<Acte<GSeance, GFichier>> _listActes;

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#getDatePublication()
         */
    public Timestamp getDatePublication(  )
    {
        return _tsDatePublication;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#setDatePublication(java.sql.Timestamp)
         */
    public void setDatePublication( Timestamp tsDatePublication )
    {
        this._tsDatePublication = tsDatePublication;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#getActes()
         */
    public List<Acte<GSeance, GFichier>> getActes(  )
    {
        return _listActes;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#setActes(java.util.List)
         */
    public void setActes( List<Acte<GSeance, GFichier>> listActes )
    {
        this._listActes = listActes;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#getActeCourant()
         */
    public Acte<GSeance, GFichier> getActeCourant(  )
    {
        return _acteCourant;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#setActeCourant(fr.paris.lutece.plugins.ods.dto.acte.Acte)
         */
    public void setActeCourant( Acte<GSeance, GFichier> acteCourant )
    {
        this._acteCourant = acteCourant;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#getEnLigne()
         */
    public Boolean getEnLigne(  )
    {
        return _bEnLigne;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#setEnLigne(java.lang.Boolean)
         */
    public void setEnLigne( Boolean bEnLigne )
    {
        this._bEnLigne = bEnLigne;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#getSeance()
         */
    public GSeance getSeance(  )
    {
        return _seance;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#setSeance(fr.paris.lutece.plugins.ods.dto.seance.ISeance)
         */
    public void setSeance( GSeance seance )
    {
        this._seance = seance;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#getIntitule()
         */
    public String getIntitule(  )
    {
        return _strIntitule;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#setIntitule(java.lang.String)
         */
    public void setIntitule( String strIntitule )
    {
        this._strIntitule = strIntitule;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#getVersion()
         */
    public int getVersion(  )
    {
        return _nVersion;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#setVersion(int)
         */
    public void setVersion( int nVersion )
    {
        this._nVersion = nVersion;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#getId()
         */
    public int getId(  )
    {
        return _nId;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#setId(int)
         */
    public void setId( int nId )
    {
        this._nId = nId;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#hashCode()
         */
    public int hashCode(  )
    {
        return getId(  );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#getFormationConseil()
         */
    public FormationConseil getFormationConseil(  )
    {
        return _formationConseil;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#setFormationConseil(fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil)
         */
    public void setFormationConseil( FormationConseil conseil )
    {
        _formationConseil = conseil;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#getNumero()
         */
    public String getNumero(  )
    {
        return _strNumero;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#getNombreActes()
         */
    public int getNombreActes(  )
    {
        int resultat = 0;

        if ( _listActes != null )
        {
            for ( Acte<GSeance, GFichier> acte : _listActes )
            {
                if ( ( acte.getStatutActe(  ).getIdStatut(  ) != StatutActeEnum.ANNULE.getId(  ) ) &&
                        ( acte.getStatutActe(  ).getIdStatut(  ) != StatutActeEnum.ERREUR_TRANSMISSION.getId(  ) ) )
                {
                    resultat += 1;
                }
            }
        }

        return resultat;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#setNumero(java.lang.String)
         */
    public void setNumero( String strNumero )
    {
        _strNumero = strNumero;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#getFichierOr()
         */
    public GFichier getFichierOr(  )
    {
        return _fichierOr;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#setFichierOr(fr.paris.lutece.plugins.ods.dto.fichier.IFichier)
         */
    public void setFichierOr( GFichier fichierOr )
    {
        this._fichierOr = fichierOr;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#getFichierSc()
         */
    public GFichier getFichierSc(  )
    {
        return _fichierSc;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#setFichierSc(fr.paris.lutece.plugins.ods.dto.fichier.IFichier)
         */
    public void setFichierSc( GFichier fichierSc )
    {
        this._fichierSc = fichierSc;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#getCategorie()
         */
    public CategorieDeliberation getCategorie(  )
    {
        return _categorie;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#setCategorie(fr.paris.lutece.plugins.ods.dto.categoriedeliberation.CategorieDeliberation)
         */
    public void setCategorie( CategorieDeliberation categorie )
    {
        this._categorie = categorie;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#getDateVote()
         */
    public Timestamp getDateVote(  )
    {
        return _tsDateVote;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#setDateVote(java.sql.Timestamp)
         */
    public void setDateVote( Timestamp dateVote )
    {
        _tsDateVote = dateVote;
    }

    /**
     * @return _tsDateRetourControleLegalite la date de retour du controle de legalite
     */

    /* *public Timestamp getDateRetourControleLegalite(  )
    {
        return _tsDateRetourControleLegalite;
    }*/

    /**
     * @param dateRetourControleLegalite la date de retour du controle de legalite
     */

    /* *public void setDateRetourControleLegalite( Timestamp dateRetourControleLegalite )
    {
        _tsDateRetourControleLegalite = dateRetourControleLegalite;
    }*/

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#getXml()
         */
    public String getXml(  )
    {
        StringBuffer buffer = new StringBuffer(  );
        HashMap<String, Object> attributes = new HashMap<String, Object>(  );
        attributes.put( ID, OdsConstants.CONSTANTE_CHAINE_VIDE + _nId );
        XmlUtil.beginElement( buffer, OdsMarks.MARK_DELIBERATION_DESIGNATION, attributes );

        if ( _seance != null )
        {
            OdsUtils.addElement( buffer, SEANCE, _seance.getIdSeance(  ) );
        }

        if ( _formationConseil != null )
        {
            buffer.append( _formationConseil.getXml(  ) );
        }

        if ( _strNumero != null )
        {
            OdsUtils.addElement( buffer, NUMERO, _strNumero );
        }

        if ( _strIntitule != null )
        {
            OdsUtils.addElement( buffer, INTITULE, _strIntitule );
        }

        if ( _categorie != null )
        {
            OdsUtils.addElement( buffer, CATEGORIE, _categorie.getLibelle(  ) );
        }

        OdsUtils.addElement( buffer, EN_LIGNE, OdsConstants.CONSTANTE_CHAINE_VIDE + _bEnLigne );

        if ( _tsDatePublication != null )
        {
            OdsUtils.addElement( buffer, DATE_PUBLICATION, OdsConstants.CONSTANTE_CHAINE_VIDE + _tsDatePublication );
        }

        if ( _tsDateVote != null )
        {
            OdsUtils.addElement( buffer, DATE_VOTE, OdsConstants.CONSTANTE_CHAINE_VIDE + _tsDateVote );
        }

        if ( _nVersion != -1 )
        {
            OdsUtils.addElement( buffer, VERSION, OdsConstants.CONSTANTE_CHAINE_VIDE + _nVersion );
        }

        XmlUtil.endElement( buffer, OdsMarks.MARK_DELIBERATION_DESIGNATION );

        return buffer.toString(  );
    }

    /* (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#getXmlForCertificat(int)
     */
    public String getXmlForCertificat( int nIdTypeEntite )
    {
        return getXml(  );
    }

    /* (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#getTypeEntite()
     */
    public int getTypeEntite(  )
    {
        return ITypeEntite.DESIGNATIONS_DE_REPRESENTANTS;
    }

    /* (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#getFamilleDocumentEnum()
     */
    public String getFamilleDocumentEnum(  )
    {
        return IHorodatageFamilleDocument.DELIBERATION_DESIGNATION;
    }

    /* (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation#getReference()
     */
    public String getReference(  )
    {
        return null;
    }
}
