/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.elu;

import fr.paris.lutece.plugins.ods.dto.IDepositaire;
import fr.paris.lutece.plugins.ods.dto.groupepolitique.IGroupePolitique;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.util.xml.XmlUtil;

import java.util.HashMap;


/**
 * Elu
 */
public abstract class AbstractElu implements IDepositaire, IElu
{
    private int _nIdElu;
    private IGroupePolitique _groupe;
    private String _strCivilite;
    private String _strNomElu;
    private String _strPrenomElu;
    private boolean _bActif;
    private IElu _eluRemplace;

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.elu.IElu#isActif()
         */
    public boolean isActif(  )
    {
        return _bActif;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.elu.IElu#setActif(boolean)
         */
    public void setActif( boolean bActif )
    {
        _bActif = bActif;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.elu.IElu#getGroupe()
         */
    public IGroupePolitique getGroupe(  )
    {
        return _groupe;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.elu.IElu#setGroupe(fr.paris.lutece.plugins.ods.dto.groupepolitique.GroupePolitique)
         */
    public void setGroupe( IGroupePolitique groupe )
    {
        _groupe = groupe;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.elu.IElu#getIdElu()
         */
    public int getIdElu(  )
    {
        return _nIdElu;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.elu.IElu#setIdElu(int)
         */
    public void setIdElu( int nIdElu )
    {
        _nIdElu = nIdElu;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.elu.IElu#getCivilite()
         */
    public String getCivilite(  )
    {
        return _strCivilite;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.elu.IElu#setCivilite(java.lang.String)
         */
    public void setCivilite( String strCivilite )
    {
        _strCivilite = strCivilite;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.elu.IElu#getNomElu()
         */
    public String getNomElu(  )
    {
        return _strNomElu;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.elu.IElu#setNomElu(java.lang.String)
         */
    public void setNomElu( String strNomElu )
    {
        _strNomElu = strNomElu;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.elu.IElu#getPrenomElu()
         */
    public String getPrenomElu(  )
    {
        return _strPrenomElu;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.elu.IElu#setPrenomElu(java.lang.String)
         */
    public void setPrenomElu( String strPrenomElu )
    {
        _strPrenomElu = strPrenomElu;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.elu.IElu#getXml()
         */
    public String getXml(  )
    {
        StringBuffer buffer = new StringBuffer(  );
        HashMap<String, Object> attributes = new HashMap<String, Object>(  );
        attributes.put( TAG_ID_ELU, OdsConstants.CONSTANTE_CHAINE_VIDE + _nIdElu );
        XmlUtil.beginElement( buffer, TAG_ELU, attributes );

        if ( _strCivilite != null )
        {
            OdsUtils.addElement( buffer, TAG_CIVILITE, _strCivilite );
        }

        if ( _strNomElu != null )
        {
            OdsUtils.addElement( buffer, TAG_NOM_ELU, _strNomElu );
        }

        if ( _strPrenomElu != null )
        {
            OdsUtils.addElement( buffer, TAG_PRENOM_ELU, _strPrenomElu );
        }

        OdsUtils.addElement( buffer, TAG_ACTIF, OdsConstants.CONSTANTE_CHAINE_VIDE + _bActif );

        if ( _groupe != null )
        {
            buffer.append( _groupe.getXml(  ) );
        }

        // eluspec
        getXmlSpec( buffer );

        XmlUtil.endElement( buffer, TAG_ELU );

        return buffer.toString(  );
    }

    /**
     * Retourne un int unique par lu
     * @return l'id de l'lu
     */
    public int hashCode(  )
    {
        return getIdElu(  );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.elu.IElu#getType()
         */
    public int getType(  )
    {
        return IDepositaire.DEPOSITAIRE_TYPE_ELU;
    }

    public IElu getEluRemplace(  )
    {
        return _eluRemplace;
    }

    public void setEluRemplace( IElu eluRemplace )
    {
        _eluRemplace = eluRemplace;
    }

    protected abstract void getXmlSpec( StringBuffer buffer );
}
