/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.fichier;

import fr.paris.lutece.plugins.ods.dto.acte.Acte;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.horodatage.IHorodatageFamilleDocument;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.typeentite.ITypeEntite;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.util.xml.XmlUtil;

import java.sql.Timestamp;

import java.util.HashMap;


/**
 * 
 * AbstractFichier
 *
 * @param <GSeance> {@link ISeance}
 * @param <GFichier> {@link IFichier}
 */
public class AbstractFichier
<
	GSeance extends ISeance, 
	GFichier extends IFichier<GSeance, GFichier>
>
implements IFichier<GSeance, GFichier>
{
    private static final String VERSION = "version";
    private static final String DATE_PUBLICATION = "datePublication";
    private static final String EN_LIGNE = "enLigne";
    private static final String TEXTE = "texte";
    private static final String TAILLE = "taille";
    private static final String NOM = "nom";
    private static final String EXTENSION = "extension";
    private static final String TITRE = "titre";
    private static final String ID = "id";
    private int _nId;
    private String _strIntitule;
    private String _strExtension;
    private String _strNom;
    private int _nTaille;
    private String _strTexte;
    private Boolean _bEnLigne;
    private Timestamp _tsDatePublication;
    private int _nVersion;
    private ITypeDocument _typdeDocument;
    private GSeance _seance;
    private IPDD _pdd;
    private FichierPhysique _fichier;
    private FormationConseil _formationConseil;
    private Acte<GSeance, GFichier> _acte;

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#getDatePublication()
         */
    public Timestamp getDatePublication(  )
    {
        return _tsDatePublication;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#setDatePublication(java.sql.Timestamp)
         */
    public void setDatePublication( Timestamp tsDatePublication )
    {
        this._tsDatePublication = tsDatePublication;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#getPDD()
         */
    public IPDD getPDD(  )
    {
        return _pdd;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#setPDD(fr.paris.lutece.plugins.ods.dto.pdd.IPDD)
         */
    public void setPDD( IPDD pdd )
    {
        this._pdd = pdd;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#getActe()
         */
    public Acte<GSeance, GFichier> getActe(  )
    {
        return _acte;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#setActe(fr.paris.lutece.plugins.ods.dto.acte.Acte)
         */
    public void setActe( Acte<GSeance, GFichier> acte )
    {
        this._acte = acte;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#getEnLigne()
         */
    public Boolean getEnLigne(  )
    {
        return _bEnLigne;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#setEnLigne(java.lang.Boolean)
         */
    public void setEnLigne( Boolean bEnLigne )
    {
        this._bEnLigne = bEnLigne;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#getFichier()
         */
    public FichierPhysique getFichier(  )
    {
        return _fichier;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#setFichier(fr.paris.lutece.plugins.ods.dto.fichier.FichierPhysique)
         */
    public void setFichier( FichierPhysique fichier )
    {
        this._fichier = fichier;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#getSeance()
         */
    public GSeance getSeance(  )
    {
        return _seance;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#setSeance(fr.paris.lutece.plugins.ods.dto.seance.ISeance)
         */
    public void setSeance( GSeance seance )
    {
        this._seance = seance;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#getTexte()
         */
    public String getTexte(  )
    {
        return _strTexte;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#setTexte(java.lang.String)
         */
    public void setTexte( String strTexte )
    {
        this._strTexte = strTexte;
    }

    /**
         * Retourne le _strIntitule d'un _fichier
         * @return String _strIntitule
         */
    public String getIntitule(  )
    {
        return _strIntitule;
    }

    /**
         * Affecte le _strIntitule a un _fichier
         * @param _strIntitule le titre du fichier
         */
    public void setIntitule( String strIntitule )
    {
        this._strIntitule = strIntitule;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#getTypdeDocument()
         */
    public ITypeDocument getTypdeDocument(  )
    {
        return _typdeDocument;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#setTypdeDocument(fr.paris.lutece.plugins.ods.dto.fichier.TypeDocument)
         */
    public void setTypdeDocument( ITypeDocument typdeDocument )
    {
        this._typdeDocument = typdeDocument;
    }

    /**
         * Retourne la _nVersion d'un _fichier
         * @return int _nVersion
         */
    public int getVersion(  )
    {
        return _nVersion;
    }

    /**
         * Affecte la _nVersion  un _fichier
         * @param nVersion le numro de la version
         */
    public void setVersion( int nVersion )
    {
        this._nVersion = nVersion;
    }

    /**
         * Retourne l'_nId d'un _fichier
         * @return int _nId
         */
    public int getId(  )
    {
        return _nId;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#setId(int)
         */
    public void setId( int nId )
    {
        this._nId = nId;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#getExtension()
         */
    public String getExtension(  )
    {
        return _strExtension;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#setExtension(java.lang.String)
         */
    public void setExtension( String strExtension )
    {
        this._strExtension = strExtension;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#getTaille()
         */
    public int getTaille(  )
    {
        return _nTaille;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#getTailleKo()
         */
    public int getTailleKo(  )
    {
        return _nTaille / 1000;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#getTailleMo()
         */
    public double getTailleMo(  )
    {
        double result = (double) getTailleKo(  ) / 1000.0;

        return Math.round( result * 100 ) * 0.01;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#setTaille(int)
         */
    public void setTaille( int taille )
    {
        this._nTaille = taille;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#getNom()
         */
    public String getNom(  )
    {
        return _strNom;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#setNom(java.lang.String)
         */
    public void setNom( String strNom )
    {
        this._strNom = strNom;
    }

    /**
     * renvoie le hashcode de l'objet
     * @return le hashcode
     */
    public int hashCode(  )
    {
        return getId(  );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#equals(java.lang.Object)
         */
    public boolean equals( Object fichier )
    {
        if ( fichier instanceof IFichier<?, ?> )
        {
            return ( (IFichier<?, ?>) fichier ).getId(  ) == getId(  );
        }

        return false;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#getFormationConseil()
         */
    public FormationConseil getFormationConseil(  )
    {
        return _formationConseil;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#setFormationConseil(fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil)
         */
    public void setFormationConseil( FormationConseil conseil )
    {
        _formationConseil = conseil;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#getXml()
         */
    public String getXml(  )
    {
        StringBuffer buffer = new StringBuffer(  );
        HashMap<String, Object> attributes = new HashMap<String, Object>(  );
        attributes.put( ID, OdsConstants.CONSTANTE_CHAINE_VIDE + _nId );
        XmlUtil.beginElement( buffer, OdsMarks.MARK_FICHIER, attributes );

        if ( _strIntitule != null )
        {
            OdsUtils.addElement( buffer, TITRE, _strIntitule );
        }

        if ( _strExtension != null )
        {
            OdsUtils.addElement( buffer, EXTENSION, _strExtension );
        }

        if ( _strNom != null )
        {
            OdsUtils.addElement( buffer, NOM, _strNom );
        }

        if ( _nTaille != -1 )
        {
            OdsUtils.addElement( buffer, TAILLE, OdsConstants.CONSTANTE_CHAINE_VIDE + _nTaille );
        }

        if ( _strTexte != null )
        {
            OdsUtils.addElement( buffer, TEXTE, _strTexte );
        }

        OdsUtils.addElement( buffer, EN_LIGNE, OdsConstants.CONSTANTE_CHAINE_VIDE + _bEnLigne );

        if ( _tsDatePublication != null )
        {
            OdsUtils.addElement( buffer, DATE_PUBLICATION, OdsConstants.CONSTANTE_CHAINE_VIDE + _tsDatePublication );
        }

        if ( _nVersion != -1 )
        {
            OdsUtils.addElement( buffer, VERSION, OdsConstants.CONSTANTE_CHAINE_VIDE + _nVersion );
        }

        if ( _typdeDocument != null )
        {
            buffer.append( _typdeDocument.getXml(  ) );
        }

        XmlUtil.endElement( buffer, OdsMarks.MARK_FICHIER );

        return buffer.toString(  );
    }

    /* (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#getFamilleDocumentEnum()
     */
    public String getFamilleDocumentEnum(  )
    {
        return IHorodatageFamilleDocument.FICHIER;
    }

    /* (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#getReference()
     */
    public String getReference(  )
    {
        return null;
    }

    /* (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichier#getTypeEntite()
     */
    public int getTypeEntite(  )
    {
        return ITypeEntite.FICHIERS;
    }
}
