/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.fichier;

import java.sql.Timestamp;

import java.util.ArrayList;
import java.util.List;


public abstract class AbstractFichierFilter implements IFichierFilter
{
    private int _nTypeDocument = ALL_INT;
    private int _nIdSeance = ALL_INT;
    private int _nIdTypeDocument = ALL_INT;
    private int _nPublication = ALL_INT;
    private int _nPDD = ALL_INT;
    private String _strIntitule = ALL_STRING;
    private int _nAutreDocumentsSeance = ALL_INT;
    private int _nIdFormationConseil = ALL_INT;
    private Timestamp _tsDatePublication;
    private List<String> _orderByList = new ArrayList<String>(  );
    private List<ITypeDocument> _typesDocList = new ArrayList<ITypeDocument>(  );

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#getDatePublication()
         */
    public Timestamp getDatePublication(  )
    {
        return _tsDatePublication;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#setDatePublication(java.sql.Timestamp)
         */
    public void setDatePublication( Timestamp tsDatePubliocation )
    {
        _tsDatePublication = tsDatePubliocation;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#containsDatePublicationCriteria()
         */
    public boolean containsDatePublicationCriteria(  )
    {
        return ( _tsDatePublication != null );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#getIdSeance()
         */
    public int getIdSeance(  )
    {
        return _nIdSeance;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#setIdSeance(int)
         */
    public void setIdSeance( int nIdSeance )
    {
        _nIdSeance = nIdSeance;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#containsSeanceCriteria()
         */
    public boolean containsSeanceCriteria(  )
    {
        return ( _nIdSeance != ALL_INT );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#getIdTypeDocument()
         */
    public int getIdTypeDocument(  )
    {
        return _nIdTypeDocument;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#setIdTypeDocument(int)
         */
    public void setIdTypeDocument( int nIdTypeDocument )
    {
        _nIdTypeDocument = nIdTypeDocument;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#containsTypeDocumentCriteria()
         */
    public boolean containsTypeDocumentCriteria(  )
    {
        return ( _nIdTypeDocument != ALL_INT );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#isPublication()
         */
    public boolean isPublication(  )
    {
        if ( _nPublication == 1 )
        {
            return true;
        }

        return false;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#setPublication(int)
         */
    public void setPublication( int nPublication )
    {
        _nPublication = nPublication;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#containsPublicationCriteria()
         */
    public boolean containsPublicationCriteria(  )
    {
        return ( _nPublication != ALL_INT );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#getIdPDD()
         */
    public int getIdPDD(  )
    {
        return _nPDD;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#setPDD(int)
         */
    public void setPDD( int nPDD )
    {
        _nPDD = nPDD;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#containsPDDCriteria()
         */
    public boolean containsPDDCriteria(  )
    {
        return ( _nPDD != ALL_INT );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#getIntitule()
         */
    public String getIntitule(  )
    {
        return _strIntitule;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#setIntitule(java.lang.String)
         */
    public void setIntitule( String strTitre )
    {
        _strIntitule = strTitre;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#containsIntituleCriteria()
         */
    public boolean containsIntituleCriteria(  )
    {
        return ( !_strIntitule.equals( ALL_STRING ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#getAutreDocumentsSeance()
         */
    public int getAutreDocumentsSeance(  )
    {
        return _nAutreDocumentsSeance;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#setAutreDocumentsSeance(int)
         */
    public void setAutreDocumentsSeance( int autreDocumentsSeance )
    {
        _nAutreDocumentsSeance = autreDocumentsSeance;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#containsAutreDocumentsSeanceCriteria()
         */
    public boolean containsAutreDocumentsSeanceCriteria(  )
    {
        return ( _nAutreDocumentsSeance != ALL_INT );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#getIdFormationConseil()
         */
    public int getIdFormationConseil(  )
    {
        return _nIdFormationConseil;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#setIdFormationConseil(int)
         */
    public void setIdFormationConseil( int idFormationConseil )
    {
        _nIdFormationConseil = idFormationConseil;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#containsIdFormationConseilCriteria()
         */
    public boolean containsIdFormationConseilCriteria(  )
    {
        return ( _nIdFormationConseil != ALL_INT );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#getOrderByList()
         */
    public List<String> getOrderByList(  )
    {
        return _orderByList;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#setOrderByList(java.util.List)
         */
    public void setOrderByList( List<String> orderByList )
    {
        _orderByList = orderByList;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#addInOrderBy(java.lang.String)
         */
    public void addInOrderBy( String strOrder )
    {
        _orderByList.add( strOrder );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#containsOrderByCriteria()
         */
    public boolean containsOrderByCriteria(  )
    {
        return ( !_orderByList.isEmpty(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#getFilterTypeDocument()
         */
    public int getFilterTypeDocument(  )
    {
        return _nTypeDocument;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#setFilterTypeDocument(int)
         */
    public void setFilterTypeDocument( int type )
    {
        _nTypeDocument = type;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#containsFilterTypeDocumentCriteria()
         */
    public boolean containsFilterTypeDocumentCriteria(  )
    {
        return ( ( _nTypeDocument == 1 ) || ( _nTypeDocument == 2 ) );
    }

    /* (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#getTypesDocList()
     */
    public List<ITypeDocument> getTypesDocList(  )
    {
        return _typesDocList;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#setTypesDocList(java.util.List)
         */
    public void setTypesDocList( List<ITypeDocument> typesDocList )
    {
        _typesDocList = typesDocList;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#addInTypeDoc(fr.paris.lutece.plugins.ods.dto.fichier.TypeDocumentEnum)
         */
    public void addInTypeDoc( ITypeDocument typeDoc )
    {
        _typesDocList.add( typeDoc );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter#containsListeTypesDocumentsCriteria()
         */
    public boolean containsListeTypesDocumentsCriteria(  )
    {
        return ( !_typesDocList.isEmpty(  ) );
    }
}
