/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.fichier;

import fr.paris.lutece.plugins.ods.dto.IBusinessItem;
import fr.paris.lutece.plugins.ods.dto.acte.Acte;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.historique.IEntiteHistorique;
import fr.paris.lutece.plugins.ods.dto.horodatage.IEntiteHorodatage;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;

import java.sql.Timestamp;


public interface IFichier<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>>
    extends IEntiteHorodatage, IEntiteHistorique, IBusinessItem
{
    /**
     * Retourne la date de publication d'un _fichier
     * @return Timestamp _tsDatePublication
     */
    Timestamp getDatePublication(  );

    /**
     * Affecte la date de publication
     * @param tsDatePublication Timestamp
     */
    void setDatePublication( Timestamp tsDatePublication );

    /**
     * Retourne l'objet Dliberation associ  un _fichier
     * @return Objet Deliberation
     */
    IPDD getPDD(  );

    /**
     * Affecte l'objet Dliberation  un _fichier
     * @param pdd le PDD
     */
    void setPDD( IPDD pdd );

    /**
     * Retourne l'objet Dliberation associ  un _fichier
     * @return Objet Deliberation
     */
    Acte<GSeance, GFichier> getActe(  );

    /**
     * Affecte l'objet Dliberation  un _fichier
     * @param acte l'Acte
     */
    void setActe( Acte<GSeance, GFichier> acte );

    /**
     * Retourne le boolean qui spcifie la mise en ligne d'un _fichier
     * @return Boolean
     */
    Boolean getEnLigne(  );

    /**
     * Affecte le boolean qui spcifie la mise en ligne d'un _fichier
     * @param bEnLigne le statut du fichier
     */
    void setEnLigne( Boolean bEnLigne );

    /**
     * Retourne l'objet FichierPhysique associ  un _fichier
     * @return FichierPhysique _fichier
     */
    FichierPhysique getFichier(  );

    /**
     *  Affecte l'objet Fichier  un _fichier
     * @param fichier le fichier
     */
    void setFichier( FichierPhysique fichier );

    /**
     * Retourne l'objet Seance associ  un _fichier
     * @return Seance _seance
     */
    GSeance getSeance(  );

    /**
     * Affecte l'objet Seance  un _fichier
     * @param seance la sance
     */
    void setSeance( GSeance seance );

    /**
     * Retourne le _strTexte qui sert a la recherche
     * @return String _strTexte
     */
    String getTexte(  );

    /**
     * Affecte le _strTexte qui sert a la recherche
     * @param strTexte le texte
     */
    void setTexte( String strTexte );

    /**
     * Retourne l'objet TypeDocument associ  un _fichier
     * @return TypeDocument _typdeDocument
     */
    ITypeDocument getTypdeDocument(  );

    /**
     * Affecte l'Objet TypeDocument  un _fichier
     * @param typdeDocument le type de document
     */
    void setTypdeDocument( ITypeDocument typdeDocument );

    /**
     * Affecte la _nVersion  un _fichier
     * @param nVersion le numro de la version
     */
    void setVersion( int nVersion );

    /**
     * Affecte l'_nId  un _fichier
     * @param nId l'identifiant d'un fichier
     */
    void setId( int nId );

    /**
     * Retourne l'_strExtension d'un _fichier
     * @return String _strExtension
     */
    String getExtension(  );

    /**
     * Affecte l'_strExtension  un _fichier
     * @param strExtension l'extension du fichier
     */
    void setExtension( String strExtension );

    /**
     * Retourne la taille du fichier physique
     * @return int la taille du fichier physique
     */
    int getTaille(  );

    /**
     * Retourne la taille du fichier physique en ko
     * @return int la taille du fichier physique
     */
    int getTailleKo(  );

    /**
     * Retourne la taille du fichier physique en mo
     * @return double la taille du fichier physique
     */
    double getTailleMo(  );

    /**
     * Affecte la _lTaille  un _fichier
     * @param taille la taille du fichier physique
     */
    void setTaille( int taille );

    /**
     * Retourne le _strNom d'un _fichier
     * @return String _strNom
     */
    String getNom(  );

    /**
     * Affecte le _strNom  un _fichier
     * @param strNom le nom du fichier
     */
    void setNom( String strNom );

    /**
     * verifie l'equivalence entre 2 objets
     * @param fichier objet fichier  comparer
     * @return TRUE si l'id est identique, FALSE sinon
     */
    boolean equals( Object fichier );

    /**
     *
     * @return la formation conseil du fichier
     */
    FormationConseil getFormationConseil(  );

    /**
     *
     * @param conseil la formation conseil du fichier
     */
    void setFormationConseil( FormationConseil conseil );

    /**
     * gnration XML
     * @return gnre Fichier en XML
     */
    String getXml(  );

    String getFamilleDocumentEnum(  );

    String getReference(  );

    int getTypeEntite(  );

    /**
     * Affecte le strIntitule a un _fichier
     * @param strIntitule le titre du fichier
     */
    void setIntitule( String strIntitule );
}
