/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.fichier;

import java.sql.Timestamp;

import java.util.List;


public interface IFichierFilter
{
    String ORDER_DATE_PUBLICATION = "fc.date_publication";
    String ALL_STRING = "all";
    int ALL_INT = -1;
    int TYPE_DOCUMENT_PROCHAINE_SEANCE = 1;
    int TYPE_DOCUMENT_GESTION_AVAL = 2;

    /**
     * @return _tsDatePublication Timestamp
     */
    Timestamp getDatePublication(  );

    /**
     * @param tsDatePubliocation _tsDatePublication to set
     */
    void setDatePublication( Timestamp tsDatePubliocation );

    /**
     * @return boolean TRUE si _tsDatePublication est actif, FALSE sinon
     */
    boolean containsDatePublicationCriteria(  );

    /**
     * Retourne la sance  filtrer
     * @return la sance  filtrer
     */
    int getIdSeance(  );

    /**
     * Fixe la valeur du filtre sance
     * @param nIdSeance la sance  filtrer
     */
    void setIdSeance( int nIdSeance );

    /**
     * Indique si le filtre sur la sance est activ.
     * @return <b>true</b> si le filtre sur lsance est positionn, <b>false</b> sinon
     */
    boolean containsSeanceCriteria(  );

    /**
     * Retourne le type de documents  filtrer
     * @return le type de cdocument  filtrer
     */
    int getIdTypeDocument(  );

    /**
     * Fixe le filtre sur le type de document
     * @param nIdTypeDocument la valeur  filtrer
     */
    void setIdTypeDocument( int nIdTypeDocument );

    /**
     * Indique si le filtre sur le type de documents est positionn
     * @return <b>true</b> si le filtre sur le type de documents est positionn, <b>false</b> sinon
     */
    boolean containsTypeDocumentCriteria(  );

    /**
     * Retourne la valeur de publication  filtrer
     * @return <b>true</b> si _nPublication vaut 1, <b>false</b> sinon
     */
    boolean isPublication(  );

    /**
     * Fixe le filtre sur la publication
     * 1 = publi
     * 0 = non publi
     * @param nPublication la valeur  filtrer
     */
    void setPublication( int nPublication );

    /**
     * Indique si le filtre sur la publication est activ
     * @return <b>true</b> si le filtre sur la publication est positionn, <b>false</b> sinon
     */
    boolean containsPublicationCriteria(  );

    /**
     * Retourne la valeur de l'identifiant du PDD  filtrer
     * @return l'identifiant du PDD  filtrer
     */
    int getIdPDD(  );

    /**
     * Fixe l'identifiant du PDD  filtrer
     * @param nPDD l'identifiant du PDD
     */
    void setPDD( int nPDD );

    /**
     * Indique si le filtre doit prendre en compte le PDD
     * @return <b>true</b> si le filtre sur le PDD est positionn, <b>false</b> sinon
     */
    boolean containsPDDCriteria(  );

    /**
     * retourne l'intitule du fichier
     * @return le titre du fichier
     */
    String getIntitule(  );

    /**
     *initialise l'intitule du fichier
     * @param strTitre le titre du fichier
     */
    void setIntitule( String strTitre );

    /**
     * retourne true si le filtre contient le critere Titre false sinon
     * @return retourne si le filtre contient le critere Titre false sinon
     */
    boolean containsIntituleCriteria(  );

    /**
     * @return the _nAutreDocumentsSeance
     */
    int getAutreDocumentsSeance(  );

    /**
     * @param autreDocumentsSeance the _nAutreDocumentsSeance to set
     */
    void setAutreDocumentsSeance( int autreDocumentsSeance );

    /**
     * retourne true si le filtre pour rechercher les autres documents de seances est activ
     * @return retourne true si le filtre pour rechercher les autres documents de seances est activ  false sinon
     */
    boolean containsAutreDocumentsSeanceCriteria(  );

    /**
     * retourne l'id de la formation conseil prsent dans le filtre
     * @return id de la formation conseil prsent dans le filtre
     */
    int getIdFormationConseil(  );

    /**
     *  initialise l'id de la formation conseil  filtrer
     * @param idFormationConseil id de la formation conseil prsent dans le filtre
     */
    void setIdFormationConseil( int idFormationConseil );

    /**
     * retourne true si le filtre contient l'id de la formation conseil  false sinon
     * @return retourne true si le filtre contient l'id de la formation conseil  false sinon
     */
    boolean containsIdFormationConseilCriteria(  );

    /**
     * retourne la liste des criteres de tries
     * @return retourne la liste des criteres de tries
     */
    List<String> getOrderByList(  );

    /**
     * initialise la liste des criteres de tries
     * @param orderByList  liste des criteres de tries
     */
    void setOrderByList( List<String> orderByList );

    /**
     * ajoute un critere de tri
     * @param strOrder critere de tri
     */
    void addInOrderBy( String strOrder );

    /**
     * retourne true si le filtre contient des criteres de tris sinon false
     * @return retourne true si le filtre contient des criteres de tris sinon false
     */
    boolean containsOrderByCriteria(  );

    /**
     * Retourne :
     * 1 si on veut filter que les types de documents propres  la prochaine sance :
     *         - Convocation du maire
     *         - Document Libre
     *         - Liste intgrale des questions
     *         - Relev de la confrence d'organisation
     *         - Rsum des questions
     *         - Dsignation
     * 2 si on veut filtrer ceux de la gestion aval :
     *         - pareil que pour la prochaine sance
     *         + Compte rendu
     *         + Dlibration de dsignation
     * @return 1 pour la prochaine sance, 2 pour la gestion aval
     */
    int getFilterTypeDocument(  );

    /**
     * Affecte le filtrage par type de document suivant si c'est la liste
     * pour la prochaine sance ou pour la gestion aval
     * @param type 1 pour la prochaine sance, 2 pour la gestion aval
     */
    void setFilterTypeDocument( int type );

    /**
     * retourne true si le filtre contient 1 ou 2 suivant le type de document
     * @return retourne true si le filtre contient 1 ou 2 suivant le type de document
     */
    boolean containsFilterTypeDocumentCriteria(  );

    /**
     * Retourne la liste des types de document voulus
     * @return la liste des types de documents
     */
    List<ITypeDocument> getTypesDocList(  );

    /**
     * Fixe la liste des types de documents voulus
     * @param typesDocList la liste des types de documents voulus
     */
    void setTypesDocList( List<ITypeDocument> typesDocList );

    /**
     * Ajoute un type de document  filtrer
     * @param typeDoc le type  filtrer
     */
    void addInTypeDoc( ITypeDocument typeDoc );

    /**
     * Indique si l'on veut filtrer par type de documents
     * @return <b>true</b> si la liste des types de documents n'est pas vide, <b>false</b> sinon
     */
    boolean containsListeTypesDocumentsCriteria(  );
}
