/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.listemembrepresent;

import fr.paris.lutece.plugins.ods.dto.acte.Acte;
import fr.paris.lutece.plugins.ods.dto.acte.IEntiteActe;
import fr.paris.lutece.plugins.ods.dto.acte.StatutActeEnum;
import fr.paris.lutece.plugins.ods.dto.certificataffichage.IEntiteCertificatAffichage;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.historique.IEntiteHistorique;
import fr.paris.lutece.plugins.ods.dto.horodatage.IEntiteHorodatage;
import fr.paris.lutece.plugins.ods.dto.horodatage.IHorodatageFamilleDocument;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.typeentite.ITypeEntite;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.util.xml.XmlUtil;

import java.sql.Timestamp;

import java.util.HashMap;
import java.util.List;


/**
 * ListeMembrePresent
 *
 * @param <GFichier> IFichier
 * @param <GSeance> ISeance
 */
public class ListeMembrePresent<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>>
    implements IEntiteActe, IEntiteHorodatage, IEntiteCertificatAffichage, IEntiteHistorique
{
    private static final String TAG_LISTE_MEMBRES_PRESENTS = "listeMembresPresents";
    private static final String TAG_VERSION = "version";
    private static final String TAG_DATE_PUBLICATION = "datePublication";
    private static final String TAG_DATE_TRANSMISSION_CONTROLE_LEGALITE = "dateTransmissionControleLegalite";
    private static final String TAG_EN_LIGNE = "enLigne";
    private static final String TAG_ID = "id";
    private static final String TAG_LISTE_ELUS_PRESENTS = "listeElusPresents";
    private int _nId;
    private GSeance _seance;
    private FormationConseil _formationConseil;
    private String _strIntitule;
    private GFichier _fichierSc;
    private GFichier _fichierGenere;
    private Timestamp _tsDateTransmissionControleLegalite;
    private Boolean _bEnLigne;
    private Timestamp _tsDatePublication;
    private int _nVersion;
    private List<IElu> _listElusPresents;
    private List<Acte<GSeance, GFichier>> _listActes;
    private Acte<GSeance, GFichier> _acteCourant;

    /**
     * Retourne la date de publication
     * @return Timestamp _tsDatePublication
     */
    public Timestamp getDatePublication(  )
    {
        return _tsDatePublication;
    }

    /**
     * Affecte la date de publication
     * @param tsDatePublication Timestamp
     */
    public void setDatePublication( Timestamp tsDatePublication )
    {
        this._tsDatePublication = tsDatePublication;
    }

    /**
     * Retourne le boolean qui spcifie la mise en ligne
     * @return Boolean
     */
    public Boolean getEnLigne(  )
    {
        return _bEnLigne;
    }

    /**
     * Affecte le boolean qui spcifie la mise en ligne
     * @param bEnLigne le statut du fichier
     */
    public void setEnLigne( Boolean bEnLigne )
    {
        this._bEnLigne = bEnLigne;
    }

    /**
     * Retourne l'objet Seance
     * @return Seance _seance
     */
    public GSeance getSeance(  )
    {
        return _seance;
    }

    /**
     * Affecte l'objet Seance
     * @param seance la sance
     */
    public void setSeance( GSeance seance )
    {
        this._seance = seance;
    }

    /**
     * Retourne l'intitule
     * @return String _strIntitule
     */
    public String getIntitule(  )
    {
        return _strIntitule;
    }

    /**
     * Affecte l'intitule
     * @param strIntitule le texte
     */
    public void setIntitule( String strIntitule )
    {
        this._strIntitule = strIntitule;
    }

    /**
     * Retourne la Version
     * @return int _nVersion
     */
    public int getVersion(  )
    {
        return _nVersion;
    }

    /**
     * Affecte la Version
     * @param nVersion le numro de la version
     */
    public void setVersion( int nVersion )
    {
        this._nVersion = nVersion;
    }

    /**
     * Retourne l'id de la liste des membres prsents
     * @return int _nId
     */
    public int getId(  )
    {
        return _nId;
    }

    /**
     * Affecte l'id de la liste des membres prsents
     * @param nId l'identifiant
     */
    public void setId( int nId )
    {
        this._nId = nId;
    }

    /**
     * renvoie le hashcode de l'objet
     * @return le hashcode
     */
    public int hashCode(  )
    {
        return getId(  );
    }

    /**
     *
     * @return la formation conseil
     */
    public FormationConseil getFormationConseil(  )
    {
        return _formationConseil;
    }

    /**
     *
     * @param conseil la formation conseil
     */
    public void setFormationConseil( FormationConseil conseil )
    {
        _formationConseil = conseil;
    }

    /**
     *
     * @return le fichier scann de la liste des membres  prsents
     */
    public GFichier getFichierSc(  )
    {
        return _fichierSc;
    }

    /**
     *  le fichier scann de la liste des membres  prsents
     *  @param fichierSc le fichier scanne de la liste des membres  prsents
     */
    public void setFichierSc( GFichier fichierSc )
    {
        this._fichierSc = fichierSc;
    }

    /**
    *
    * @return le fichier gnr de la liste des membres  prsents
    */
    public GFichier getFichierGenere(  )
    {
        return _fichierGenere;
    }

    /**
     *  le fichier gnr de la liste des membres  prsents
     *  @param fichierGenere le fichier gnr de la liste des membres  prsents
     */
    public void setFichierGenere( GFichier fichierGenere )
    {
        this._fichierGenere = fichierGenere;
    }

    /**
     * La date de transmission au controle de legalite
     * @return La date de transmission au controle de legalite
     */
    public Timestamp getDateTransmissionControleLegalite(  )
    {
        return _tsDateTransmissionControleLegalite;
    }

    /**
     * retourne la date de transmission au controle de legalite
     * @param dateTransmissionControleLegalite Timestamp
     */
    public void setDateTransmissionControleLegalite( Timestamp dateTransmissionControleLegalite )
    {
        _tsDateTransmissionControleLegalite = dateTransmissionControleLegalite;
    }

    /**
     *
     * @return la liste des lus prsents
     */
    public List<IElu> getElusPresents(  )
    {
        return _listElusPresents;
    }

    /**
     * set la liste des lus prsents
     * @param listEluPresent la liste des lus prsents
     */
    public void setElusPresents( List<IElu> listEluPresent )
    {
        this._listElusPresents = listEluPresent;
    }

    /**
    *
    * @return la liste des actes associs  cette liste de membres
    */
    public List<Acte<GSeance, GFichier>> getActes(  )
    {
        return _listActes;
    }

    /**
     * set la liste des actes associs  cette liste de membres
     * @param listActes la liste des actes associs  cette liste de membres
     */
    public void setActes( List<Acte<GSeance, GFichier>> listActes )
    {
        this._listActes = listActes;
    }

    /**
        *
        * @return le nombre d'acte non annuls associs  cette liste de membres
        */
    public int getNombreActes(  )
    {
        int resultat = 0;

        if ( _listActes != null )
        {
            for ( Acte<GSeance, GFichier> acte : _listActes )
            {
                if ( ( acte.getStatutActe(  ).getIdStatut(  ) != StatutActeEnum.ANNULE.getId(  ) ) &&
                        ( acte.getStatutActe(  ).getIdStatut(  ) != StatutActeEnum.ERREUR_TRANSMISSION.getId(  ) ) )
                {
                    resultat += 1;
                }
            }
        }

        return resultat;
    }

    /**
     * Retourne l'acte courant
     * @return Acte _acteCourant
     */
    public Acte<GSeance, GFichier> getActeCourant(  )
    {
        return _acteCourant;
    }

    /**
     * Affecte l'objet Acte acteCourant
     * @param acteCourant l'acte
     */
    public void setActeCourant( Acte<GSeance, GFichier> acteCourant )
    {
        this._acteCourant = acteCourant;
    }

    /**
     * gnration XML
     * @return gnre ListeMembrePresent en XML
     */
    public String getXml(  )
    {
        StringBuffer buffer = new StringBuffer(  );
        HashMap<String, Object> attributes = new HashMap<String, Object>(  );
        attributes.put( TAG_ID, OdsConstants.CONSTANTE_CHAINE_VIDE + _nId );
        XmlUtil.beginElement( buffer, ListeMembrePresent.TAG_LISTE_MEMBRES_PRESENTS, attributes );

        if ( _seance != null )
        {
            buffer.append( _seance.getXml(  ) );
        }

        if ( _formationConseil != null )
        {
            buffer.append( _formationConseil.getXml(  ) );
        }

        OdsUtils.addElement( buffer, TAG_EN_LIGNE, OdsConstants.CONSTANTE_CHAINE_VIDE + _bEnLigne );

        if ( _tsDatePublication != null )
        {
            OdsUtils.addElement( buffer, TAG_DATE_PUBLICATION, OdsConstants.CONSTANTE_CHAINE_VIDE + _tsDatePublication );
        }

        if ( _tsDateTransmissionControleLegalite != null )
        {
            OdsUtils.addElement( buffer, TAG_DATE_TRANSMISSION_CONTROLE_LEGALITE,
                OdsConstants.CONSTANTE_CHAINE_VIDE + _tsDateTransmissionControleLegalite );
        }

        if ( _nVersion != -1 )
        {
            OdsUtils.addElement( buffer, TAG_VERSION, OdsConstants.CONSTANTE_CHAINE_VIDE + _nVersion );
        }

        if ( ( _listElusPresents != null ) && !_listElusPresents.isEmpty(  ) )
        {
            XmlUtil.beginElement( buffer, TAG_LISTE_ELUS_PRESENTS );

            for ( IElu elu : _listElusPresents )
            {
                if ( elu != null )
                {
                    buffer.append( elu.getXml(  ) );
                }
            }

            XmlUtil.endElement( buffer, TAG_LISTE_ELUS_PRESENTS );
        }

        XmlUtil.endElement( buffer, ListeMembrePresent.TAG_LISTE_MEMBRES_PRESENTS );

        return buffer.toString(  );
    }

    /**
     * @return le flux XML pour le certificat
     * @param nIdTypeEntite l'id du type d'entite
     */
    public String getXmlForCertificat( int nIdTypeEntite )
    {
        return getXml(  );
    }

    /**
     * @return le type d'entite
     */
    public int getTypeEntite(  )
    {
        return ITypeEntite.LISTE_DES_MEMBRES_PRESENTS;
    }

    /**
     * @return le type de document
     */
    public String getFamilleDocumentEnum(  )
    {
        return IHorodatageFamilleDocument.LISTE_MEMBRE_PRESENT;
    }

    /**
     * @return la reference
     */
    public String getReference(  )
    {
        return null;
    }
}
