/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.modeleordredujour;

import fr.paris.lutece.plugins.ods.dto.fichier.FichierPhysique;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.ordredujour.TypeOrdreDuJour;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.util.xml.XmlUtil;

import java.util.HashMap;


public abstract class AbstractModeleOrdreDuJour implements IModeleOrdreDuJour
{
    protected static final String TAG_TITRE = "titre";
    protected static final String TAG_ID = "id";
    protected static final String TAG_MODELE_ORDRE_DU_JOUR = "modeleOrdreDuJour";
    private int _nIdModele;
    private FichierPhysique _fEnteteDocument;
    private FichierPhysique _fPiedDocument;
    private TypeOrdreDuJour _typeOrdreDuJour;
    private FormationConseil _formationConseil;
    private String _strTitre;

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.modeleordredujour.IModeleOrdreDuJour#getEnteteDocument()
         */
    public FichierPhysique getEnteteDocument(  )
    {
        return _fEnteteDocument;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.modeleordredujour.IModeleOrdreDuJour#setEnteteDocument(fr.paris.lutece.plugins.ods.dto.fichier.FichierPhysique)
         */
    public void setEnteteDocument( FichierPhysique enTete )
    {
        _fEnteteDocument = enTete;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.modeleordredujour.IModeleOrdreDuJour#getFormationConseil()
         */
    public FormationConseil getFormationConseil(  )
    {
        return _formationConseil;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.modeleordredujour.IModeleOrdreDuJour#setFormationConseil(fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil)
         */
    public void setFormationConseil( FormationConseil formationConseil )
    {
        _formationConseil = formationConseil;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.modeleordredujour.IModeleOrdreDuJour#getPiedDocument()
         */
    public FichierPhysique getPiedDocument(  )
    {
        return _fPiedDocument;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.modeleordredujour.IModeleOrdreDuJour#setPiedDocument(fr.paris.lutece.plugins.ods.dto.fichier.FichierPhysique)
         */
    public void setPiedDocument( FichierPhysique piedDocument )
    {
        _fPiedDocument = piedDocument;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.modeleordredujour.IModeleOrdreDuJour#getIdModele()
         */
    public int getIdModele(  )
    {
        return _nIdModele;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.modeleordredujour.IModeleOrdreDuJour#setIdModele(int)
         */
    public void setIdModele( int idModele )
    {
        _nIdModele = idModele;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.modeleordredujour.IModeleOrdreDuJour#getTitre()
         */
    public String getTitre(  )
    {
        return _strTitre;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.modeleordredujour.IModeleOrdreDuJour#setTitre(java.lang.String)
         */
    public void setTitre( String titre )
    {
        _strTitre = titre;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.modeleordredujour.IModeleOrdreDuJour#getTypeOrdreDuJour()
         */
    public TypeOrdreDuJour getTypeOrdreDuJour(  )
    {
        return _typeOrdreDuJour;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.modeleordredujour.IModeleOrdreDuJour#setTypeOrdreDuJour(fr.paris.lutece.plugins.ods.dto.ordredujour.TypeOrdreDuJour)
         */
    public void setTypeOrdreDuJour( TypeOrdreDuJour ordreDuJour )
    {
        _typeOrdreDuJour = ordreDuJour;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.modeleordredujour.IModeleOrdreDuJour#getXml()
         */
    public String getXml(  )
    {
        StringBuffer buffer = new StringBuffer(  );
        HashMap<String, Object> attributes = new HashMap<String, Object>(  );

        attributes.put( TAG_ID, _nIdModele );
        XmlUtil.beginElement( buffer, TAG_MODELE_ORDRE_DU_JOUR, attributes );

        if ( _strTitre != null )
        {
            OdsUtils.addElement( buffer, TAG_TITRE, _strTitre );
        }

        if ( _fEnteteDocument != null )
        {
            buffer.append( _fEnteteDocument.getXml(  ) );
        }

        if ( _fPiedDocument != null )
        {
            buffer.append( _fPiedDocument.getXml(  ) );
        }

        if ( _typeOrdreDuJour != null )
        {
            buffer.append( _typeOrdreDuJour.getXml(  ) );
        }

        if ( _formationConseil != null )
        {
            buffer.append( _formationConseil.getXml(  ) );
        }

        getXmlSpec( buffer );

        XmlUtil.endElement( buffer, TAG_MODELE_ORDRE_DU_JOUR );

        return buffer.toString(  );
    }

    protected abstract void getXmlSpec( StringBuffer buffer );
}
