/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.ordredujour;

import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.util.xml.XmlUtil;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;


/**
 * 
 * Objet EntreOrdreDuJour, correspond  une entre de la table
 * ods_entree_ordre_jour
 *
 * @param <GEntreeOrdreDuJour> {@link IEntreeOrdreDuJour}
 * @param <GOrdreDuJour> {@link IOrdreDuJour}
 * @param <GOrdreDuJourFilter> {@link IOrdreDuJourFilter}
 * @param <GSeance> {@link ISeance}
 * @param <GElu> {@link IElu}
 * @param <GFichier> {@link IFichier}
 */
public abstract class AbstractEntreeOrdreDuJour
<
	GEntreeOrdreDuJour extends IEntreeOrdreDuJour<GElu, GSeance, GFichier, GEntreeOrdreDuJour, GOrdreDuJour>, 
	GOrdreDuJour extends IOrdreDuJour<GEntreeOrdreDuJour, GSeance, GFichier, GElu, GOrdreDuJour>, 
	GOrdreDuJourFilter extends IOrdreDuJourFilter, 
	GSeance extends ISeance, 
	GElu extends IElu, 
	GFichier extends IFichier<GSeance, GFichier>
>
implements IEntreeOrdreDuJour<GElu, GSeance, GFichier, GEntreeOrdreDuJour, GOrdreDuJour>
{
    private static final String TAG_LISTE_ELUS = "listeElus";
    private static final String TAG_STYLE = "style";
    private static final String TAG_OBJET = "objet";
    private static final String TAG_NUMERO_ORDRE = "numeroOrdre";
    private static final String TAG_TYPE = "typeEntite";
    private static final String TAG_ID = "id";
    private static final String TAG_ENTREE_ORDRE_DU_JOUR = "entreeOrdreDuJour";
    private int _nIdEntreeOrdreDuJour;
    private IEntiteEntreeOrdreDuJour _entite;
    private int _nIdEntite;
    private int _nIdTypeEntite;
    protected GOrdreDuJour _ordreDuJour;
    private int _nNumeroOrdre;
    private String _strReference;
    private String _strObjet;
    private String _strStyle;
    protected List<GElu> _elus;

    protected abstract void getXmlSpec( StringBuffer buffer );

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour#getNumeoOrdre()
         */
    public int getNumeroOrdre(  )
    {
        return _nNumeroOrdre;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour#setNumeroOrdre(int)
         */
    public void setNumeroOrdre( int nOrdre )
    {
        _nNumeroOrdre = nOrdre;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour#getOrdreDuJour()
         */
    public GOrdreDuJour getOrdreDuJour(  )
    {
        return _ordreDuJour;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour#setOrdreDuJour(fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJour)
         */
    public void setOrdreDuJour( GOrdreDuJour ordreDuJour )
    {
        _ordreDuJour = ordreDuJour;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour#getObjet()
         */
    public String getObjet(  )
    {
        return _strObjet;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour#setObjet(java.lang.String)
         */
    public void setObjet( String strObjet )
    {
        _strObjet = strObjet;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour#getReference()
         */
    public String getReference(  )
    {
        return _strReference;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour#setReference(java.lang.String)
         */
    public void setReference( String strReference )
    {
        _strReference = strReference;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour#getStyle()
         */
    public String getStyle(  )
    {
        return _strStyle;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour#setStyle(java.lang.String)
         */
    public void setStyle( String strStyle )
    {
        _strStyle = strStyle;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour#getIdEntreeOrdreDuJour()
         */
    public int getIdEntreeOrdreDuJour(  )
    {
        return _nIdEntreeOrdreDuJour;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour#setIdEntreeOrdreDuJour(int)
         */
    public void setIdEntreeOrdreDuJour( int idEntreeOrdreDuJour )
    {
        _nIdEntreeOrdreDuJour = idEntreeOrdreDuJour;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour#getEntite()
         */
    public IEntiteEntreeOrdreDuJour getEntite(  )
    {
        return _entite;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour#setEntite(fr.paris.lutece.plugins.ods.dto.ordredujour.IEntiteEntreeOrdreDuJour)
         */
    public void setEntite( IEntiteEntreeOrdreDuJour entite )
    {
        _entite = entite;
    }

    /* (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour#setIdEntite(int)
     */
    public void setIdEntite( int nIdEntite )
    {
        _nIdEntite = nIdEntite;
    }

    /* (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour#getIdEntite()
     */
    public int getIdEntite(  )
    {
        return _nIdEntite;
    }

    /* (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour#getIdTypeEntite()
     */
    public int getIdTypeEntite(  )
    {
        return _nIdTypeEntite;
    }

    /* (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour#setIdTypeEntite(int)
     */
    public void setIdTypeEntite( int nIdTypeEntite )
    {
        _nIdTypeEntite = nIdTypeEntite;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour#getElus()
         */
    public List<GElu> getElus(  )
    {
        return _elus;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour#setElus(java.util.List)
         */
    public void setElus( List<GElu> elus )
    {
        this._elus = elus;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour#getEqualListRapporteursCommission(java.util.List, int)
         */
    public boolean getEqualListRapporteursCommission( List<GElu> listElus, int nIdCommission )
    {
        List<GElu> listRapporteursEntree = _elus;

        if ( listRapporteursEntree.size(  ) != listElus.size(  ) )
        {
            return false;
        }

        for ( int cpt = 0; cpt < listRapporteursEntree.size(  ); cpt++ )
        {
            if ( listRapporteursEntree.get( cpt ).getIdElu(  ) != listElus.get( cpt ).getIdElu(  ) )
            {
                return false;
            }
        }

        return true;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour#compareListRapporteurs(java.util.List, int)
         */
    public int compareListRapporteurs( List<GElu> listElus, int nIdCommission )
    {
        int nRetour = 666;

        // Si les 2 listes sont nulles, elles sont gales
        if ( ( _elus == null ) && ( listElus == null ) )
        {
            nRetour = 0;
        }

        // Si _elus est nulle et listElus non nulle, cela dpend si listElus est vide ou pas.
        // listElus est vide alors les 2 listes sont gales, sinon non
        else if ( ( _elus == null ) && ( listElus != null ) )
        {
            if ( listElus.isEmpty(  ) )
            {
                nRetour = 0;
            }
            else
            {
                nRetour = 1;
            }
        }

        // Si _elus est non nulle et listElus nulle, cela dpend si _elus est vide ou pas.
        // _elus est vide alors les 2 listes sont gales, sinon non
        else if ( ( _elus != null ) && ( listElus == null ) )
        {
            if ( _elus.isEmpty(  ) )
            {
                nRetour = 0;
            }
            else
            {
                nRetour = -1;
            }
        }

        // Sinon on est dans le cas o ni _elus ni listElus ne sont vides ou nulles
        else
        {
            // Si la _elus est plus grande que listElus, on renvoie 1
            if ( _elus.size(  ) > listElus.size(  ) )
            {
                nRetour = -1;
            }

            // Si la _elus est plus grande que listElus, on renvoie 1
            else if ( _elus.size(  ) < listElus.size(  ) )
            {
                nRetour = 1;
            }

            // Si les 2 listes sont identiques
            else if ( getEqualListRapporteursCommission( listElus, nIdCommission ) )
            {
                nRetour = 0;
            }
        }

        if ( nRetour != 666 )
        {
            return nRetour;
        }

        List<String> nomsElus = new ArrayList<String>( _elus.size(  ) );

        for ( GElu elu : getElus(  ) )
        {
            nomsElus.add( elu.getNomElu(  ) );
        }

        Collections.sort( nomsElus );

        List<String> nomsNewElus = new ArrayList<String>( listElus.size(  ) );

        for ( GElu elu : listElus )
        {
            nomsNewElus.add( elu.getNomElu(  ) );
        }

        Collections.sort( nomsNewElus );

        for ( int i = 0; i < nomsElus.size(  ); i++ )
        {
            int nCompare = nomsNewElus.get( i ).compareToIgnoreCase( nomsElus.get( i ) );

            if ( nCompare != 0 )
            {
                return nCompare;
            }
        }

        return 0;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour#getXml()
         */
    public String getXml(  )
    {
        StringBuffer buffer = new StringBuffer(  );
        HashMap<String, Object> attributes = new HashMap<String, Object>(  );

        attributes.put( TAG_ID, _nIdEntreeOrdreDuJour );
        XmlUtil.beginElement( buffer, TAG_ENTREE_ORDRE_DU_JOUR, attributes );

        if ( _entite != null )
        {
            OdsUtils.addElement( buffer, TAG_TYPE, OdsConstants.CONSTANTE_CHAINE_VIDE + _entite.getTypeEntite(  ) );
        }
        else
        {
            OdsUtils.addElement( buffer, TAG_TYPE, OdsConstants.CONSTANTE_CHAINE_VIDE + getIdTypeEntite(  ) );
        }

        if ( _nNumeroOrdre >= 0 )
        {
            OdsUtils.addElement( buffer, TAG_NUMERO_ORDRE, OdsConstants.CONSTANTE_CHAINE_VIDE + _nNumeroOrdre );
        }

        if ( _strReference != null )
        {
            OdsUtils.addElement( buffer, OdsParameters.REFERENCE, _strReference );
        }

        if ( _strObjet != null )
        {
            OdsUtils.addElement( buffer, TAG_OBJET, _strObjet );
        }

        if ( _strStyle != null )
        {
            OdsUtils.addElement( buffer, TAG_STYLE, _strStyle );
        }

        if ( _entite != null )
        {
            String strXml = _entite.getXml(  );

            if ( strXml != null )
            {
                buffer.append( strXml );
            }
        }

        getXmlSpec( buffer );

        if ( ( _elus != null ) && !_elus.isEmpty(  ) )
        {
            XmlUtil.beginElement( buffer, TAG_LISTE_ELUS );

            for ( GElu elu : _elus )
            {
                if ( elu != null )
                {
                    buffer.append( elu.getXml(  ) );
                }
            }

            XmlUtil.endElement( buffer, TAG_LISTE_ELUS );
        }

        XmlUtil.endElement( buffer, TAG_ENTREE_ORDRE_DU_JOUR );

        return buffer.toString(  );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour#getEqualListRapporteurs(java.util.List)
         */
    public boolean getEqualListRapporteurs( List<GElu> listElus )
    {
        if ( _elus.size(  ) != listElus.size(  ) )
        {
            return false;
        }

        for ( int cpt = 0; cpt < _elus.size(  ); cpt++ )
        {
            if ( ( _elus.get( cpt ).getIdElu(  ) != listElus.get( cpt ).getIdElu(  ) ) ||
                    ( _elus.get( cpt ).getIdElu(  ) == listElus.get( cpt ).getIdElu(  ) ) )
            {
                return false;
            }
        }

        return true;
    }
}
