/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.ordredujour;

import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.business.OrdreDuJourUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.util.xml.XmlUtil;

import java.io.FileInputStream;

import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;

import java.sql.Timestamp;

import java.util.HashMap;
import java.util.List;


/**
 *
 * AbstractOrdreDuJour
 *
 * @param <GEntreeOrdreDuJour> IEntreeOrdreDuJour
 * @param <GSeance> ISeance
 * @param <GElu> IElu
 * @param <GFichier> IFichier
 * @param <GOrdreDuJour> IOrdreDuJour
 */
public abstract class AbstractOrdreDuJour
<
	GEntreeOrdreDuJour extends IEntreeOrdreDuJour<GElu, GSeance, GFichier, GEntreeOrdreDuJour, GOrdreDuJour>, 
	GOrdreDuJour extends IOrdreDuJour<GEntreeOrdreDuJour, GSeance, GFichier, GElu, GOrdreDuJour>, 
	GSeance extends ISeance, 
	GElu extends IElu, 
	GFichier extends IFichier<GSeance, GFichier>
>
implements IOrdreDuJour<GEntreeOrdreDuJour, GSeance, GFichier, GElu, GOrdreDuJour>
{
    protected TypeOrdreDuJour _typeOrdreDuJour;
    private int _nIdOrdreDuJour;
    private GSeance _seance;
    private FormationConseil _formationConseil;
    private GOrdreDuJour _ordreDuJourSauveGarde;
    private String _strIntitule;
    private String _strModeClassement;
    private Boolean _bPublie;
    private Timestamp _datePublication;
    private Timestamp _dateDebutPublication;
    private Boolean _bEstSauvegarde;
    private List<GEntreeOrdreDuJour> _listEntrees;
    private String _strXmlPublication;
    private String _strXmlEntete;
    private String _strXmlPiedDePage;
    private GFichier _fichier;
    private Boolean _bEstDepubliable = false;

    /**
     * Renvoie le flux XML specifique
     * @param buffer le StringBuffer
     */
    protected abstract void getXmlSpec( StringBuffer buffer );

    /**
     * @return le type d'ODS
     */
    protected abstract String getTypeODS(  );

    /**
     * @return retourne le fichier rattach  l'ordre du jour
     */
    public GFichier getFichier(  )
    {
        return _fichier;
    }

    /**
     *
     * @param fichier rattache le fichier  l'ordre du jour
     */
    public void setFichier( GFichier fichier )
    {
        _fichier = fichier;
    }

    /**
     *
     *@return retourne true si l'ordre du jour est publi false sinon
     */
    public Boolean getPublie(  )
    {
        return _bPublie;
    }

    /**
     * @return retourne true si l'ordre du jour est une sauvegarde false sinon
     */
    public Boolean getEstSauvegarde(  )
    {
        return _bEstSauvegarde;
    }

    /**
     *
     * @param estSauvegarde affecte true  si l'ordre du jour est une false sinon
     */
    public void setEstSauvegarde( Boolean estSauvegarde )
    {
        _bEstSauvegarde = estSauvegarde;
    }

    /**
     *
     * @param publie  affecte true si l'ordre du jour est publi  false sinon
     */
    public void setPublie( Boolean publie )
    {
        _bPublie = publie;
    }

    /**
     *
     * @return retourne la date de publication de l'ordre du jour
     */
    public Timestamp getDatePublication(  )
    {
        return _datePublication;
    }

    /**
     *
     * @param publication affecte une date de publication  l'ordre du jour
     */
    public void setDatePublication( Timestamp publication )
    {
        _datePublication = publication;
    }

    /**
    *
    * @return retourne la date de dbut du processus de  publication de l'ordre du jour
    */
    public Timestamp getDateDebutPublication(  )
    {
        return _dateDebutPublication;
    }

    /**
     *
     * @param publication affecte une date de dbut du processus de  publication  l'ordre du jour
     */
    public void setDateDebutPublication( Timestamp publication )
    {
        _dateDebutPublication = publication;
    }

    /**
     *
     * @return retourne la formation conseil de l'ordre du jour
     */
    public FormationConseil getFormationConseil(  )
    {
        return _formationConseil;
    }

    /**
         *
         * @param conseil affecte une formation conseil  l'ordre du jour
         */
    public void setFormationConseil( FormationConseil conseil )
    {
        _formationConseil = conseil;
    }

    /**
     *
     * @return retourne la liste des entres d'ordre du jour
     */
    public List<GEntreeOrdreDuJour> getListEntrees(  )
    {
        return _listEntrees;
    }

    /**
         *
         * @param entrees affecte une liste d'entres  l'ordre du jour
         */
    public void setListEntrees( List<GEntreeOrdreDuJour> entrees )
    {
        _listEntrees = entrees;
    }

    /**
     *
     * @return retourne l'id de l'ordre du jour
     */
    public int getIdOrdreDuJour(  )
    {
        return _nIdOrdreDuJour;
    }

    /**
     *
     * @param idOrdreDuJour affecte un id a l'ordre du jour
     */
    public void setIdOrdreDuJour( int idOrdreDuJour )
    {
        _nIdOrdreDuJour = idOrdreDuJour;
    }

    /**
     *
     * @return retourne la sauvegarde de l'ordre du jour
     */
    public GOrdreDuJour getOrdreDuJourSauveGarde(  )
    {
        return _ordreDuJourSauveGarde;
    }

    /**
     *
     * @param ordreDuJourSauveGarde affecte une sauvegarde  l'ordre du jour
     */
    public void setOrdreDuJourSauveGarde( GOrdreDuJour ordreDuJourSauveGarde )
    {
        _ordreDuJourSauveGarde = ordreDuJourSauveGarde;
    }

    /**
     *
     * @return retourne la seance de l'ordre du jour
     */
    public GSeance getSeance(  )
    {
        return _seance;
    }

    /**
     *
     * @param seance affecte une seance  l'ordre du jour
     */
    public void setSeance( GSeance seance )
    {
        _seance = seance;
    }

    /**
     *
     * @return retourne l'intitul de l'ordre du jour
     */
    public String getIntitule(  )
    {
        return _strIntitule;
    }

    /**
     *
     * @param intitule affecte un intitul  l'ordre du jour
     */
    public void setIntitule( String intitule )
    {
        _strIntitule = intitule;
    }

    /**
     *
     * @return retourne le mode de classement de l'ordre du jour (Automatique,Manuel)
     */
    public String getModeClassement(  )
    {
        return _strModeClassement;
    }

    /**
     *
     * @param modeClassement affecte un mode de classement  l'ordre du jour (Automatique,Manuel)
     */
    public void setModeClassement( String modeClassement )
    {
        _strModeClassement = modeClassement;
    }

    /**
     *
     * @return le type d'ordre du jour
     */
    public TypeOrdreDuJour getTypeOrdreDuJour(  )
    {
        return _typeOrdreDuJour;
    }

    /**
     *
     * @param typeOrdreDuJour affecte un  type d'ordre du jour
     */
    public void setTypeOrdreDuJour( TypeOrdreDuJour typeOrdreDuJour )
    {
        _typeOrdreDuJour = typeOrdreDuJour;
    }

    /**
     * @return retourne true si l'ordre du jour est dpubliable false sinon
     */
    public Boolean getEstDepubliable(  )
    {
        return _bEstDepubliable;
    }

    /**
     *
     * @param estDepubliable affecte true  si l'ordre du jour est depubliable false sinon
     */
    public void setEstDepubliable( Boolean estDepubliable )
    {
        _bEstDepubliable = estDepubliable;
    }

    /**
     *
     * @return l'entete de l'ordre du jour ,stocke au moment de la publication de ce dernier
     */
    public String getXmlEntete(  )
    {
        return _strXmlEntete;
    }

    /**
     *
     * @param entete initialise l'entete de l'ordre du jour au moment de la publication de ce dernier
     */
    public void setXmlEntete( String entete )
    {
        _strXmlEntete = entete;
    }

    /**
     *
     * @return le pied de page  de l'ordre du jour ,stock au moment de la publication de ce dernier
     */
    public String getXmlPiedDePage(  )
    {
        return _strXmlPiedDePage;
    }

    /**
     *
     * @param piedDePage  initialise le pied de page  de l'ordre du jour au moment de la publication de ce dernier
     */
    public void setXmlPiedDePage( String piedDePage )
    {
        _strXmlPiedDePage = piedDePage;
    }

    /**
     *
     * @return le fichier xml de l'ordre du jour ,stock au moment de la publication de ce dernier
     */
    public String getXmlPublication(  )
    {
        return _strXmlPublication;
    }

    /**
     *
     * @param xmlPublication  initialise le fichier xml  de l'ordre du jour au moment de la publication de ce dernier
     */
    public void setXmlPublication( String xmlPublication )
    {
        _strXmlPublication = xmlPublication;
    }

    /**
    * Gnration du XML d'un objet
    * @return Le code XML reprsentant un ordre du jour
    */
    public String getXml(  )
    {
        StringBuffer buffer = new StringBuffer(  );
        HashMap<String, Object> attributes = new HashMap<String, Object>(  );

        attributes.put( TAG_ID, _nIdOrdreDuJour );
        attributes.put( TAG_TYPE_ODS, getTypeODS(  ) );
        XmlUtil.beginElement( buffer, TAG_ORDRE_DU_JOUR, attributes );

        if ( _strXmlEntete == null )
        {
            //on ajoute un entete vide
            OdsUtils.addElement( buffer, OrdreDuJourUtils.TAG_ENTETE, _strXmlEntete );
        }
        else
        {
            //on ajoute le contenu du fichier xml	
            buffer.append( _strXmlEntete );
        }

        if ( _strXmlPiedDePage == null )
        {
            //on ajoute un pied de page vide
            OdsUtils.addElement( buffer, OrdreDuJourUtils.TAG_PIED_DE_PAGE, _strXmlPiedDePage );
        }
        else
        {
            //on ajoute le contenu du fichier xml	
            buffer.append( _strXmlPiedDePage );
        }

        if ( _seance != null )
        {
            buffer.append( _seance.getXml(  ) );
        }

        if ( _formationConseil != null )
        {
            buffer.append( _formationConseil.getXml(  ) );
        }

        if ( _typeOrdreDuJour != null )
        {
            buffer.append( _typeOrdreDuJour.getXml(  ) );
        }

        OdsUtils.addElement( buffer, OrdreDuJourUtils.TAG_INTITULE, _strIntitule );
        OdsUtils.addElement( buffer, OrdreDuJourUtils.TAG_MODE_CLASSEMENT, _strModeClassement );
        OdsUtils.addElement( buffer, OrdreDuJourUtils.TAG_DATE_PUBLICATION,
            OdsConstants.CONSTANTE_CHAINE_VIDE + _datePublication );
        OdsUtils.addElement( buffer, OrdreDuJourUtils.TAG_EST_PUBLIE, OdsConstants.CONSTANTE_CHAINE_VIDE + _bPublie );
        OdsUtils.addElement( buffer, OrdreDuJourUtils.TAG_EST_SAUVEGARDE,
            OdsConstants.CONSTANTE_CHAINE_VIDE + _bEstSauvegarde );

        getXmlSpec( buffer );

        if ( ( _listEntrees != null ) )
        {
            XmlUtil.beginElement( buffer, OrdreDuJourUtils.TAG_LISTE_ENTREES );

            for ( GEntreeOrdreDuJour entree : _listEntrees )
            {
                if ( entree != null )
                {
                    buffer.append( entree.getXml(  ) );
                }
            }

            XmlUtil.endElement( buffer, OrdreDuJourUtils.TAG_LISTE_ENTREES );
        }

        XmlUtil.endElement( buffer, TAG_ORDRE_DU_JOUR );

        return buffer.toString(  );
    }

    /**
     * Lit le contenue d'un fichier et le renvoie sous la forme d'une liste de byte
     * @param file le fichier
     * @return une liste d'octets
     * @throws Exception en cas d'erreur
     */
    public byte[] read( String file ) throws Exception // lots of exceptions
    {
        FileInputStream fis = new FileInputStream( file );
        FileChannel fc = fis.getChannel(  );
        byte[] data = new byte[(int) fc.size(  )]; // fc.size returns the size
                                                   // of the
                                                   // file which backs the channel

        ByteBuffer bb = ByteBuffer.wrap( data );
        fc.read( bb );

        return data;
    }
}
