/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.ordredujour;

import java.sql.Timestamp;


/**
* filtre commun sur les ordres du jour
*
*/
public abstract class AbstractOrdreDuJourFilter implements IOrdreDuJourFilter
{
    private int _nIdFormationConseil = ALL_INT;
    private int _nIdSeance = ALL_INT;
    private int _nIdPublie = ALL_INT;
    private int _nIdType = ALL_INT;
    private int _nIdSauvegarde = ALL_INT;
    private Timestamp _tsDatePublication;

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter#getDatePublication()
         */
    public Timestamp getDatePublication(  )
    {
        return _tsDatePublication;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter#setDatePublication(java.sql.Timestamp)
         */
    public void setDatePublication( Timestamp tsDatePubliocation )
    {
        _tsDatePublication = tsDatePubliocation;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter#containsDatePublicationCriteria()
         */
    public boolean containsDatePublicationCriteria(  )
    {
        return ( _tsDatePublication != null );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter#getIdFormationConseil()
         */
    public int getIdFormationConseil(  )
    {
        return _nIdFormationConseil;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter#setIdFormationConseil(int)
         */
    public void setIdFormationConseil( int idFormationConseil )
    {
        _nIdFormationConseil = idFormationConseil;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter#containsIdFormationConseilCriteria()
         */
    public boolean containsIdFormationConseilCriteria(  )
    {
        return ( _nIdFormationConseil != ALL_INT );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter#getIdPublie()
         */
    public int getIdPublie(  )
    {
        return _nIdPublie;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter#setIdPublie(int)
         */
    public void setIdPublie( int idPublie )
    {
        _nIdPublie = idPublie;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter#containsIdPublieCriteria()
         */
    public boolean containsIdPublieCriteria(  )
    {
        return ( _nIdPublie != ALL_INT );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter#getIdSeance()
         */
    public int getIdSeance(  )
    {
        return _nIdSeance;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter#setIdSeance(int)
         */
    public void setIdSeance( int idSeance )
    {
        _nIdSeance = idSeance;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter#containsIdSeanceCriteria()
         */
    public boolean containsIdSeanceCriteria(  )
    {
        return ( _nIdSeance != ALL_INT );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter#getIdType()
         */
    public int getIdType(  )
    {
        return _nIdType;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter#setIdType(int)
         */
    public void setIdType( int idType )
    {
        _nIdType = idType;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter#containsIdTypeCriteria()
         */
    public boolean containsIdTypeCriteria(  )
    {
        return ( _nIdType != ALL_INT );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter#getIdSauvegarde()
         */
    public int getIdSauvegarde(  )
    {
        return _nIdSauvegarde;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter#setIdSauvegarde(int)
         */
    public void setIdSauvegarde( int idSauvegarde )
    {
        _nIdSauvegarde = idSauvegarde;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter#containsIdSauvegardeCriteria()
         */
    public boolean containsIdSauvegardeCriteria(  )
    {
        return ( _nIdSauvegarde != ALL_INT );
    }
}
