/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.ordredujour;

import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;

import java.util.List;


public interface IEntreeOrdreDuJour<GElu extends IElu, GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>, GEntreeOrdreDuJour extends IEntreeOrdreDuJour<GElu, GSeance, GFichier, GEntreeOrdreDuJour, GOrdreDuJour>, GOrdreDuJour extends IOrdreDuJour<GEntreeOrdreDuJour, GSeance, GFichier, GElu, GOrdreDuJour>>
{
    /**
     * Retourne le numro d'ordre de l'entre
     *
     * @return le numro d'ordre de l'entre
     */
    int getNumeroOrdre(  );

    /**
     * Fixe le numro d'ordre de l'entre
     *
     * @param nOrdre
     *            le numro d'ordre de l'entre
     */
    void setNumeroOrdre( int nOrdre );

    /**
     * Retourne l'ordre du jour auquel est rattache l'entre
     *
     * @return l'ordre du jour auquel est rattache l'entre
     */
    GOrdreDuJour getOrdreDuJour(  );

    /**
     * Fixe l'ordre du jour auquel est rattache l'entre
     *
     * @param ordreDuJour
     *            l'ordre du jour de l'entre
     */
    void setOrdreDuJour( GOrdreDuJour ordreDuJour );

    /**
     * Retourne l'objet de l'entre
     *
     * @return l'objet de l'entre
     */
    String getObjet(  );

    /**
     * Fixe l'objet de l'entre
     *
     * @param strObjet
     *            l'objet de l'entre
     */
    void setObjet( String strObjet );

    /**
     * Retourne la rfrence de l'entre
     *
     * @return la rfrence de l'entre
     */
    String getReference(  );

    /**
     * Fixe la rfrence de l'entre
     *
     * @param strReference
     *            la rfrence de l'entre
     */
    void setReference( String strReference );

    /**
     * Retourne le style de l'entre
     *
     * @return le style de l'entre
     */
    String getStyle(  );

    /**
     * Fixe le style de l'entre
     *
     * @param strStyle
     *            le style de l'entre
     */
    void setStyle( String strStyle );

    /**
     *
     * @return l'id de l'entree d'ordre du jour
     */
    int getIdEntreeOrdreDuJour(  );

    /**
     *
     * @param idEntreeOrdreDuJour
     *            affecte un id a l'entree d'ordre du jour
     */
    void setIdEntreeOrdreDuJour( int idEntreeOrdreDuJour );

    /**
     * Retourne l'entit de l'entre d'ordre du jour
     * @return IEntiteEntreeOrdreDuJour _entite
     */
    IEntiteEntreeOrdreDuJour getEntite(  );

    /**
     * Affecte l'entit  de l'entre d'ordre du jour
     * @param entite l'id du type de l'entite de l'acte
     */
    void setEntite( IEntiteEntreeOrdreDuJour entite );

    /**
     * Affecte l'id de l'entit de l'entre d'ordre du jour
     * @param nIdEntite l'id de l'entit de l'entre d'ordre du jour
     */
    void setIdEntite( int nIdEntite );

    /**
     * Retourne l'id de l'entit de l'entre d'ordre du jour
     * @return int _nIdEntite
     */
    int getIdEntite(  );

    /**
     * Retourne l'id du type de l'entit de l'entre d'ordre du jour
     * @return int _nIdTypeEntite
     */
    int getIdTypeEntite(  );

    /**
     * Affecte l'id du type de l'entit de l'entre d'ordre du jour
     * @param nIdTypeEntite l'id du type de l'entit de l'entre d'ordre du jour
     */
    void setIdTypeEntite( int nIdTypeEntite );

    /**
     * @return retourne liste des rapporteurs de l'entre d'ordre du jour
     */
    List<GElu> getElus(  );

    /**
     * @return retourne liste des rapporteurs d'une commission donne de l'entre d'ordre du jour
     */
    List<GElu> getElusCommission(  );

    /**
     *
     * @param elus
     *            affecte une liste de rapporteur a l'entre d'ordre du jour
     */
    void setElus( List<GElu> elus );

    /**
     * Test si la liste des lus passe en parametre est egale a la liste des
     * rapporteurs de l'entre
     *
     * @param listElus
     *            liste de rapporteurs
     * @return true si la liste des rapporteurs de l'entre est egale a la liste
     *         pass en parametre
     */
    boolean getEqualListRapporteurs( List<GElu> listElus );

    /**
     * Teste si la liste des lus passe en paramtre est gale a la liste des
     * rapporteurs de l'entre appartenant  la commission donne en paramtres
     *
     * @param listElus liste de rapporteurs
     * @param nIdCommission l'identifiant de la commission
     * @return true si la liste des rapporteurs de l'entre est egale a la liste
     *         pass en parametre
     */
    boolean getEqualListRapporteursCommission( List<GElu> listElus, int nIdCommission );

    /**
     * Compare deux listes de rapporteurs pour les classer
     *
     * @param listElus liste de rapporteurs
     * @param nIdCommission l'identifiant de la commission
     * @return 0 si la liste des rapporteurs de l'entre est egale a la liste pass en parametre,
     *                    1 si la liste pass en paramtre doit se situer apres,
     *         -1 si elle doit se situer avant.
     */
    int compareListRapporteurs( List<GElu> listElus, int nIdCommission );

    /**
     * Gnration du XML d'un objet
     * @return Le code XML reprsentant une entre d'ordre du jour
     */
    String getXml(  );
}
