/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.pdd;

import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.util.xml.XmlUtil;

import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;


/**
 *
 * Reprsente un arrondissement
 *
 */
public abstract class AbstractArrondissement implements IArrondissement
{
    private static final String TAG_ID_PDD = "idPDD";
    private static final String TAG_NUMERO_ARRONDISSEMENT = "numeroArrondissement";
    private static final String TAG_ARRONDISSEMENT = "arrondissement";
    private static final String TAG_ID_ARRONDISSEMENT = "id";
    private int _nIdArrondissement;
    private int _nArrondissement;
    private int _nIdPDD;

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.pdd.IArrondissement#getArrondissement()
         */
    public int getArrondissement(  )
    {
        return _nArrondissement;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.pdd.IArrondissement#setArrondissement(int)
         */
    public void setArrondissement( int nArrondissement )
    {
        this._nArrondissement = nArrondissement;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.pdd.IArrondissement#getIdArrondissement()
         */
    public int getIdArrondissement(  )
    {
        return _nIdArrondissement;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.pdd.IArrondissement#setIdArrondissement(int)
         */
    public void setIdArrondissement( int nIdArrondissement )
    {
        this._nIdArrondissement = nIdArrondissement;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.pdd.IArrondissement#getIdPDD()
         */
    public int getIdPDD(  )
    {
        return _nIdPDD;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.pdd.IArrondissement#setIdPDD(int)
         */
    public void setIdPDD( int nIdPDD )
    {
        this._nIdPDD = nIdPDD;
    }

    /**
     * calcule le hashcode
     * @return le hashcode
     */
    public int hashCode(  )
    {
        return getIdArrondissement(  );
    }

    /**
     * vrifie l'galit entre 2 objets
     * @param arrondissement l'objet arrondissement
     * @return TRUE si les 2 objets ont le meme id, FALSE sinon
     */
    public boolean equals( Object arrondissement )
    {
        if ( arrondissement instanceof AbstractArrondissement )
        {
            return ( getArrondissement(  ) == ( (IArrondissement) arrondissement ).getArrondissement(  ) ) &&
            ( getIdPDD(  ) == ( (IArrondissement) arrondissement ).getIdPDD(  ) );
        }

        return false;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.pdd.IArrondissement#getXml(javax.servlet.http.HttpServletRequest)
         */
    public String getXml( HttpServletRequest request )
    {
        StringBuffer buffer = new StringBuffer(  );
        HashMap<String, Object> attributes = new HashMap<String, Object>(  );
        attributes.put( TAG_ID_ARRONDISSEMENT, _nIdArrondissement );
        XmlUtil.beginElement( buffer, TAG_ARRONDISSEMENT, attributes );

        if ( _nArrondissement != -1 )
        {
            OdsUtils.addElement( buffer, TAG_NUMERO_ARRONDISSEMENT,
                OdsConstants.CONSTANTE_CHAINE_VIDE + _nArrondissement );
        }

        if ( _nIdPDD != -1 )
        {
            OdsUtils.addElement( buffer, TAG_ID_PDD, OdsConstants.CONSTANTE_CHAINE_VIDE + _nIdPDD );
        }

        XmlUtil.endElement( buffer, TAG_ARRONDISSEMENT );

        return buffer.toString(  );
    }
}
