/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.pdd;

import fr.paris.lutece.plugins.ods.dto.acte.Acte;
import fr.paris.lutece.plugins.ods.dto.acte.IEntiteActe;
import fr.paris.lutece.plugins.ods.dto.annotations.Settings;
import fr.paris.lutece.plugins.ods.dto.categoriedeliberation.CategorieDeliberation;
import fr.paris.lutece.plugins.ods.dto.certificataffichage.IEntiteCertificatAffichage;
import fr.paris.lutece.plugins.ods.dto.direction.Direction;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.groupepolitique.IGroupePolitique;
import fr.paris.lutece.plugins.ods.dto.historique.IEntiteHistorique;
import fr.paris.lutece.plugins.ods.dto.horodatage.IEntiteHorodatage;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IEntiteEntreeOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.statut.Statut;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;

import java.sql.Timestamp;

import java.util.List;


/**
 *
 * IPDD
 *
 * @param <GVoeuAmendement> IVoeuAmendement
 * @param <GSeance> ISeance
 * @param <GElu> IElu
 * @param <GFichier> IFichier
 * @param <GPDD> IPDD
 */
public interface IPDD
<
	GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, 
	GSeance extends ISeance, 
	GElu extends IElu, 
	GFichier extends IFichier<GSeance, GFichier>, 
	GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>
>
extends IEntiteActe, IEntiteHorodatage, IEntiteCertificatAffichage, IEntiteHistorique, IEntiteEntreeOrdreDuJour
{
    String CONSTANTE_TYPE_PROJET = "PJ";
    String CONSTANTE_TYPE_PROP = "PP";
    String CONSTANTE_TYPE_DSP = "DSP";

    /**
     * @return the categorieDeliberation
     */
    CategorieDeliberation getCategorieDeliberation(  );

    /**
     * @param categorieDeliberation the categorieDeliberation to set
     */
    void setCategorieDeliberation( CategorieDeliberation categorieDeliberation );

    /**
     * @return the datePublication
     */
    Timestamp getDatePublication(  );

    /**
     * @param datePublication the datePublication to set
     */
    void setDatePublication( Timestamp datePublication );

    /**
     * @return the dateVote
     */
    Timestamp getDateVote(  );

    /**
     * @param dateVote the dateVote to set
     */
    void setDateVote( Timestamp dateVote );

    /**
     * @return the delegationsServices
     */
    boolean isDelegationsServices(  );

    /**
     * @param delegationsServices the delegationsServices to set
     */
    void setDelegationsServices( boolean delegationsServices );

    /**
     * @return the direction
     */
    Direction getDirection(  );

    /**
     * @param direction the direction to set
     */
    void setDirection( Direction direction );

    /**
     * @return the enLigne
     */
    boolean isEnLigne(  );

    /**
     * @param enLigne the enLigne to set
     */
    void setEnLigne( boolean enLigne );

    /**
     * @return the formationConseil
     */
    FormationConseil getFormationConseil(  );

    /**
     * @param formationConseil the formationConseil to set
     */
    void setFormationConseil( FormationConseil formationConseil );

    /**
     * @return the groupePolitique
     */
    IGroupePolitique getGroupePolitique(  );

    /**
     * @param groupePolitique the groupePolitique to set
     */
    void setGroupePolitique( IGroupePolitique groupePolitique );

    /**
     * @return the idPdd
     */
    int getId(  );

    /**
     * @param id the id to set
     */
    void setId( int id );

    /**
     * @return the mode_introduction
     */
    String getModeIntroduction(  );

    /**
     * @param modeIntroduction the modeIntroduction to set
     */
    void setModeIntroduction( String modeIntroduction );

    /**
     * @return the objet
     */
    @Settings( length = 30, maxSize = 50, required = true )
    String getObjet(  );

    /**
     * @param objet the objet to set
     */
    void setObjet( String objet );

    /**
     * @return the piecesManuelles
     */
    boolean isPiecesManuelles(  );

    /**
     * @param piecesManuelles the piecesManuelles to set
     */
    void setPiecesManuelles( boolean piecesManuelles );

    /**
     * @return the reference
     */
    String getReference(  );

    /**
     * @param reference the reference to set
     */
    void setReference( String reference );

    /**
     * @return the statut
     */
    Statut getStatut(  );

    /**
     * @param statut the statut to set
     */
    void setStatut( Statut statut );

    /**
     * @return the typePdd
     */
    String getTypePdd(  );

    /**
     * @param typePdd the typePdd to set
     */
    void setTypePdd( String typePdd );

    /**
     * @return the version
     */
    int getVersion(  );

    /**
     * @param version the version to set
     */
    void setVersion( int version );

    /**
     * Retourne l'annee qui est stock� dans la reference
     *  (reference = ANNEE +' '+ CODE_DIRECTION  +' '+ NUMERO )
     * @return String Annee
     */
    String getAnnee(  );

    /**
     * Retourne le numero qui est stock� dans la reference
     * (reference = ANNEE +' '+ CODE_DIRECTION  +' '+ NUMERO )
     * @return String Numero
     */
    String getNumero(  );

    /**
     *
     * @return le nombre d'acte non annul�s associ�s � cette d�liberation
     */
    int getNombreActes(  );

    /**
     *
     * @param nombreActes le nombre d'acte non annul�s associ�s � cette d�liberation
     */
    void setNombreActes( int nombreActes );

    /**
     * @return the directionsCoEmetrices
     */
    List<DirectionCoEmetrice> getDirectionsCoEmetrices(  );

    /**
     * @param directionsCoEmetrices the directionsCoEmetrices to set
     */
    void setDirectionsCoEmetrices( List<DirectionCoEmetrice> directionsCoEmetrices );

    /**
     * @return the amendements
     */
    List<GVoeuAmendement> getAmendements(  );

    /**
     * @param amendements the amendements to set
     */
    void setAmendements( List<GVoeuAmendement> amendements );

    /**
     * @return the voeuxNonRattaches
     */
    List<GVoeuAmendement> getVoeuxNonRattaches(  );

    /**
     * @param voeuxNonRattaches the voeuxNonRattaches to set
     */
    void setVoeuxNonRattaches( List<GVoeuAmendement> voeuxNonRattaches );

    /**
     * @return the voeuxRattaches
     */
    List<GVoeuAmendement> getVoeuxRattaches(  );

    /**
     * @param voeuxRattaches the voeuxRattaches to set
     */
    void setVoeuxRattaches( List<GVoeuAmendement> voeuxRattaches );

    /**
     * @return the voeuxvoeuxAmendements
     */
    List<GVoeuAmendement> getVoeuxAmendements(  );

    /**
     * @param voeuxAmendements the voeuxAmendements to set
     */
    void setVoeuxAmendements( List<GVoeuAmendement> voeuxAmendements );

    /**
     *
     * @param va VoeuAmendement
     * @return boolean TRUE si le VoeuAmendement est contenu dans une des listes
     */
    boolean containsVoeuxAmandements( GVoeuAmendement va );

    /**
     * Indique si les 2 PDD sont �gaux, cad s'ils ont le m�me id
     * @param pdd le PDD � comparer
     * @return <b>true</b> si les 2 PDD sont �gaux, <b>false</b> sinon
     */
    boolean equals( Object pdd );

    /**
     * Retourne la liste des fichiers reli�s au pdd
     * @return la liste des fichiers du pdd
     */
    List<GFichier> getFichiers(  );

    /**
     * Fixe la liste des fichiers du pdd
     * @param fichiers la liste des fichiers du pdd
     */
    void setFichiers( List<GFichier> fichiers );

    /**
     * @return the estModifie
     */
    boolean isModifie(  );

    /**
     * @param bEstModifie the estModifie to set
     */
    void setModifie( boolean bEstModifie );

    /**
     *
     * @return retourne le mode de classement du pdd (Automatique,Manuel)
     */
    String getModeClassement(  );

    /**
     *
     * @param modeClassement affecte un mode de classement au pdd (Automatique,Manuel)
     */
    void setModeClassement( String modeClassement );

    /**
     * @return le hashcode de l'objet
     */
    int hashCode(  );

    /**
     *
     * @return la liste des actes associ�s � cette liste de membres
     */
    List<Acte<GSeance, GFichier>> getListeActes(  );

    /**
     * set la liste des actes associ�s
     * @param listActes la liste des actes associ�s
     */
    void setListeActes( List<Acte<GSeance, GFichier>> listActes );

    /**
     *
     * @return le code XML correspondant a l'objet
     */
    String getXml(  );

    /**
     * @param nIdTypeEntite le type d'entite
     * @return le code XML correspondant a l'objet pour le certificat
     */
    String getXmlForCertificat( int nIdTypeEntite );

    /**
     * @return l'intitule
     */
    String getIntitule(  );

    /**
     * @return le type d'entite
     */
    int getTypeEntite(  );

    /**
     * @return la famille du document
     */
    String getFamilleDocumentEnum(  );

    /**
     * @return le type d'entite
     */
    String getTypeEntree(  );

    /** MA **/
    /**
     * @return les questions diverses
     */
    String getQuestionsDiverses(  );

    /**
     *
     * @param strQuestionsDiverses les questions diverses
     */
    void setQuestionsDiverses( String strQuestionsDiverses );

    /**
     *
     * @return la seance previsionnelle de passage en CA
     */
    ISeance getSeancePrevisPassageCA(  );
}
