/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.pdd;

import java.sql.Timestamp;


/**
 *
 * IPDDFilter
 *
 */
public interface IPDDFilter
{
    String ALL_STRING = "all";
    int ALL_INT = -1;
    int A_PREPARER = 1;
    int UN_ACTE_AU_MOINS = 2;
    int ACTE_ANNULE = 3;

    /**
     * @return _tsDateRetourControleDeLegalite Timestamp
     */
    Timestamp getDateRetourControleDeLegalite(  );

    /**
     * @param tsDateRetourControleDeLegalite _tsDateRetourControleDeLegalite to set
     */
    void setDateRetourControleDeLegalite( Timestamp tsDateRetourControleDeLegalite );

    /**
     * @return boolean TRUE si _tsDateRetourControleDeLegalite est actif, FALSE sinon
     */
    boolean containsDateRetourControleDeLegaliteCriteria(  );

    /**
     * @return _tsDatePublication Timestamp
     */
    Timestamp getDatePublication(  );

    /**
     * @param tsDatePubliocation _tsDatePublication to set
     */
    void setDatePublication( Timestamp tsDatePubliocation );

    /**
     * @return boolean TRUE si _tsDatePublication est actif, FALSE sinon
     */
    boolean containsDatePublicationCriteria(  );

    /**
     * @return _nCategorieDeliberation int
     */
    int getCategorieDeliberation(  );

    /**
     * @param categorieDeliberation _nCategorieDeliberation to set
     */
    void setCategorieDeliberation( int categorieDeliberation );

    /**
     *
     * @return boolean TRUE si _nCategorieDeliberation est actif, FALSE sinon
     */
    boolean containsCategorieDeliberationCriteria(  );

    /**
     * @return _nStatut int
     */
    int getStatut(  );

    /**
     * @param statut _nStatut to set
     */
    void setStatut( int statut );

    /**
     * @return boolean TRUE si _nStatut est actif, FALSE sinon
     */
    boolean containsStatutCriteria(  );

    /**
     * @return _nIdFormationConseil int
     */
    int getIdFormationConseil(  );

    /**
     * @param idFormationConseil _nIdFormationConseil to set
     */
    void setIdFormationConseil( int idFormationConseil );

    /**
     * @return boolean TRUE si _nIdFormationConseil est actif, FALSE sinon
     */
    boolean containsFormationConseilCriteria(  );

    /**
     * @return retourne 0 si le pdd ne doit pas etre inscrit a un ordre du jour de reference
     *                                  1 si le pdd  doit etre inscrit a un ordre du jour de reference
     */
    int getInscritODJ(  );

    /**
     * @param inscritODJ 0 si le pdd ne doit pas etre inscrit a un ordre du jour de reference
     *                                   1 si le pdd  doit etre inscrit a un ordre du jour de reference
     */
    void setInscritODJ( int inscritODJ );

    /**
     * @return boolean TRUE si le pdd doit  etre inscrit a l'odj de reference
     */
    boolean containsInscritODJCriteria(  );

    /**
     * @return the _nModeIntroduction
     */
    String getModeIntroduction(  );

    /**
     * @param modeIntroduction the _nModeIntroduction to set
     */
    void setModeIntroduction( String modeIntroduction );

    /**
     * @return boolean TRUE si _strModeIntroduction est actif, FALSE sinon
     */
    boolean containsModeIntroductionCriteria(  );

    /**
     * @return the _nSeance
     */
    int getSeance(  );

    /**
     * @param seance _nSeance to set
     */
    void setSeance( int seance );

    /**
     * @return boolean TRUE si _nProchaineSeance est actif, FALSE sinon
     */
    boolean containsProchaineSeanceCriteria(  );

    /**
     * @return the _nProchaineSeance
     */
    int getProchaineSeance(  );

    /**
     * @param prochaineSeance _nProchaineSeance to set
     */
    void setProchaineSeance( int prochaineSeance );

    /**
     * @return boolean TRUE si _nArchive est actif, FALSE sinon
     */
    boolean containsArchiveAvecPSCriteria(  );

    /**
     * Retourne le filtre Archive
     * @return le filtre Archive
     */
    int getArchiveAvecPS(  );

    /**
     * Fixe le filtre archive avec l'id de la prochaine sance
     * @param prochaineSeance l'id de la prochaine sance
     */
    void setArchiveAvecPS( int prochaineSeance );

    /**
     * Indique si la recherche porte sur les archives sans prochaine sance dans la base
     * @return <b>true</b> si la recherche porte sur les archives sans prochaine sance dans la base, <b>false</b> sinon
     */
    boolean getArchiveSansPS(  );

    /**
     * Fixe le filtre archive sans prochaine sance
     * @param bArchiveSansPS <b>true</b> si la recherche porte sur les archives sans prochaine sance dans la base, <b>false</b> sinon
     */
    void setArchiveSansPS( boolean bArchiveSansPS );

    /**
     * @return boolean TRUE si _nSeance est actif, FALSE sinon
     */
    boolean containsSeanceCriteria(  );

    /**
     * @return _strDirection String
     */
    String getDirection(  );

    /**
     * @param direction the _strDirection to set
     */
    void setDirection( String direction );

    /**
     * @return boolean TRUE si _strDirection est actif, FALSE sinon
     */
    boolean containsDirectionCriteria(  );

    /**
     * @return _nPublication int
     */
    int getPublication(  );

    /**
     * @param nPublication _nPublication to set
     */
    void setPublication( int nPublication );

    /**
     * @return boolean TRUE si _nPublication est actif, FALSE sinon
     */
    boolean containsPublicationCriteria(  );

    /**
     * @return _strGroupeDepositaire
     */
    String getGroupeDepositaire(  );

    /**
     * @param groupeDepositaire _strGroupeDepositaire to set
     */
    void setGroupeDepositaire( String groupeDepositaire );

    /**
     * @return boolean TRUE si _strGroupeDepositaire est actif, FALSE sinon
     */
    boolean containsGroupeDepositaireCriteria(  );

    /**
     * @return _strType String
     */
    String getType(  );

    /**
     * @param type _strType to set
     */
    void setType( String type );

    /**
     * @return boolean TRUE si _strType est actif, FALSE sinon
     */
    boolean containsTypeCriteria(  );

    /**
     *
     * @return _strReference String
     */
    String getReference(  );

    /**
     *
     * @param reference String
     */
    void setReference( String reference );

    /**
     *
     * @return boolean TRUE si _strReference est actif, FALSE sinon
     */
    boolean containsReferenceCriteria(  );

    /**
     * @return _nDelegationService
     */
    int getDelegationService(  );

    /**
     * @param delegationService _nDelegationService to set
     */
    void setDelegationService( int delegationService );

    /**
     * @return boolean TRUE si _nDelegationService est actif, FALSE sinon
     */
    boolean containsDelegationServiceCriteria(  );

    /**
     * @return _nTypeOrdreDuJour
     */
    int getTypeOrdreDuJour(  );

    /**
     * @param typeOrdreDuJour _nTypeOrdreDuJour to set
     */
    void setTypeOrdreDuJour( int typeOrdreDuJour );

    /**
     * @return boolean TRUE si _nTypeOrdreDuJour est actif, FALSE sinon
     */
    boolean containsTypeOrdreDuJourCriteria(  );

    /**
     * Fixe le filtre sur le statut de l'acte
     * @param nStatutActe le statut de l'acte
     */
    void setStatutActe( int nStatutActe );

    /**
     * Retourne le statut de l'acte  filtrer
     * @return le statut de l'acte  filtrer
     */
    int getStatutActe(  );

    /**
     * Indique si le filtre sur l'acte est positionn
     * @return <b>true</b> si le filtre sur l'acte est positionn, <b>false</b> sinon
     */
    boolean containsStatutActeCriteria(  );

    /**
     *
     * @return le type d'entite
     */
    int getIdTypeEntite(  );

    /**
     *
     * @param nIdTypeEntite le type d'entite
     */
    void setIdTypeEntite( int nIdTypeEntite );
}
