/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.requete;

import fr.paris.lutece.plugins.ods.business.categoriedeliberation.ICategorieDeliberationHome;
import fr.paris.lutece.plugins.ods.business.direction.IDirectionHome;
import fr.paris.lutece.plugins.ods.business.elu.IEluHome;
import fr.paris.lutece.plugins.ods.business.formationconseil.FormationConseilHome;
import fr.paris.lutece.plugins.ods.business.groupepolitique.GroupePolitiqueHome;
import fr.paris.lutece.plugins.ods.business.statut.StatutHome;
import fr.paris.lutece.plugins.ods.dto.categoriedeliberation.CategorieDeliberation;
import fr.paris.lutece.plugins.ods.dto.direction.Direction;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.groupepolitique.IGroupePolitique;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.statut.Statut;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.spring.SpringContextService;

import java.sql.Timestamp;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.StringTokenizer;


/**
 * Objet requte utilisateur: pour sauvegarder une requte de recherche
 *
 * @param <GSeanceFilter> ISeanceFilter
 * @param <GSeance> ISeance
 */
public abstract class AbstractRequeteUtilisateur<GSeance extends ISeance, GSeanceFilter extends ISeanceFilter>
    implements IRequeteUtilisateur<GSeance, GSeanceFilter>
{
    private static final String MARK_LISTE_TYPES_DOC = "liste_types_doc";
    private static final String MARK_DATE_1 = "date_1";
    private static final String MARK_DATE_2 = "date_2";
    private static final String MARK_MOTS_RECHERCHES = "mots_recherches";
    private static final String MARK_LISTE_RAPPORTEURS = "liste_rapporteurs";
    private static final String MARK_LISTE_DIRECTIONS = "liste_directions";
    private static final String MARK_LISTE_CATEGORIES = "liste_categories";
    private static final String MARK_LISTE_GROUPES = "liste_groupes";
    private static final String MARK_DEPOSE_EXECUTIF = "depose_executif";
    private static final String MARK_LISTE_ARRONDISSEMENTS = "liste_arrondissements";
    private static final String MARK_LISTE_STATUTS_PROJETS = "liste_statuts_projets";
    private static final String MARK_LISTE_STATUTS_PROPOSITIONS = "liste_statuts_propositions";
    private static final String MARK_LISTE_STATUTS_VOEUX_AMENDEMENTS = "liste_statuts_voeux_amendements";
    private static final String MARK_LISTE_ELUS = "liste_elus";
    protected String _strHtml;
    private int _nIdRequete;
    private String _strUserName = OdsConstants.CONSTANTE_CHAINE_VIDE;
    private String _strNomRequete = OdsConstants.CONSTANTE_CHAINE_VIDE;
    private boolean _bRechercheArchive;
    protected Timestamp _tsPremiereDate;
    protected Timestamp _tsDeuxiemeDate;
    private String _strTypeRequete = OdsConstants.CONSTANTE_CHAINE_VIDE;
    private String _strChampRecherche = OdsConstants.CONSTANTE_CHAINE_VIDE;
    private String _strListeTypesDocument = OdsConstants.CONSTANTE_CHAINE_VIDE;
    private String _strListeFormationsConseil = OdsConstants.CONSTANTE_CHAINE_VIDE;
    private String _strListeRapporteurs = OdsConstants.CONSTANTE_CHAINE_VIDE;
    private String _strListeArrondissements = OdsConstants.CONSTANTE_CHAINE_VIDE;
    private String _strListeDirections = OdsConstants.CONSTANTE_CHAINE_VIDE;
    private String _strListeCategoriesDeliberation = OdsConstants.CONSTANTE_CHAINE_VIDE;
    private String _strListeGroupesDepositaires = OdsConstants.CONSTANTE_CHAINE_VIDE;
    private String _strListeElusDepositaires = OdsConstants.CONSTANTE_CHAINE_VIDE;
    private String _strListeStatutsVoeuxAmendements = OdsConstants.CONSTANTE_CHAINE_VIDE;
    private String _strListeStatutsProjets = OdsConstants.CONSTANTE_CHAINE_VIDE;
    private String _strListeStatutsPropositions = OdsConstants.CONSTANTE_CHAINE_VIDE;
    private boolean _bIsNotifie;
    private boolean _bIsDeposeExecutif;
    private Timestamp _tsDateCreation;
    private ICategorieDeliberationHome _categorieDeliberationHome = ( (ICategorieDeliberationHome) SpringContextService.getPluginBean( OdsConstants.CONSTANTE_PLUGIN_ODS,
            "categorieDeliberationHome" ) );
    private IDirectionHome _directionHome = ( (IDirectionHome) SpringContextService.getPluginBean( OdsConstants.CONSTANTE_PLUGIN_ODS,
            "directionHome" ) );
    private IEluHome<?> _eluHome = ( (IEluHome<?>) SpringContextService.getPluginBean( OdsConstants.CONSTANTE_PLUGIN_ODS,
            "eluHome" ) );
    private FormationConseilHome _formationConseilHome = ( (FormationConseilHome) SpringContextService.getPluginBean( OdsConstants.CONSTANTE_PLUGIN_ODS,
            "formationConseilHome" ) );
    private GroupePolitiqueHome _groupePolitiqueHome = ( (GroupePolitiqueHome) SpringContextService.getPluginBean( OdsConstants.CONSTANTE_PLUGIN_ODS,
            "groupePolitiqueHome" ) );
    private StatutHome _statutHome = ( (StatutHome) SpringContextService.getPluginBean( OdsConstants.CONSTANTE_PLUGIN_ODS,
            "statutHome" ) );

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getIdRequete()
         */
    public int getIdRequete(  )
    {
        return _nIdRequete;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setIdRequete(int)
         */
    public void setIdRequete( int nIdRequete )
    {
        _nIdRequete = nIdRequete;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getUserName()
         */
    public String getUserName(  )
    {
        return _strUserName;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setUserName(java.lang.String)
         */
    public void setUserName( String strUserName )
    {
        _strUserName = strUserName;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getNomRequete()
         */
    public String getNomRequete(  )
    {
        return _strNomRequete;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setNomRequete(java.lang.String)
         */
    public void setNomRequete( String strNomRequete )
    {
        _strNomRequete = strNomRequete;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#isRechercheArchive()
         */
    public boolean isRechercheArchive(  )
    {
        return _bRechercheArchive;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setRechercheArchive(boolean)
         */
    public void setRechercheArchive( boolean bRechercheArchive )
    {
        _bRechercheArchive = bRechercheArchive;
    }

    /* (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getPremiereDate()
     */
    public Timestamp getPremiereDate(  )
    {
        return _tsPremiereDate;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setPremiereDate(java.sql.Timestamp)
         */
    public void setPremiereDate( Timestamp tsPremiereDate )
    {
        _tsPremiereDate = tsPremiereDate;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getDeuxiemeDate()
         */
    public Timestamp getDeuxiemeDate(  )
    {
        return _tsDeuxiemeDate;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setDeuxiemeDate(java.sql.Timestamp)
         */
    public void setDeuxiemeDate( Timestamp tsDeuxiemeDate )
    {
        _tsDeuxiemeDate = tsDeuxiemeDate;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getTypeRequete()
         */
    public String getTypeRequete(  )
    {
        return _strTypeRequete;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setTypeRequete(java.lang.String)
         */
    public void setTypeRequete( String strTypeRequete )
    {
        _strTypeRequete = strTypeRequete;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getChampRecherche()
         */
    public String getChampRecherche(  )
    {
        return _strChampRecherche;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setChampRecherche(java.lang.String)
         */
    public void setChampRecherche( String strChampRecherche )
    {
        _strChampRecherche = strChampRecherche;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getListeTypesDocument()
         */
    public String getListeTypesDocument(  )
    {
        return _strListeTypesDocument;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setListeTypesDocument(java.lang.String)
         */
    public void setListeTypesDocument( String strListeTypesDocument )
    {
        _strListeTypesDocument = strListeTypesDocument;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setTypesDocumentFromListe(java.util.List)
         */
    public void setTypesDocumentFromListe( List<String> listeTypesDocument )
    {
        StringBuilder sbRepresentation = new StringBuilder(  );

        if ( listeTypesDocument != null )
        {
            for ( String typeDocument : listeTypesDocument )
            {
                if ( sbRepresentation.length(  ) != 0 )
                {
                    sbRepresentation.append( ", " );
                }

                sbRepresentation.append( typeDocument );
            }
        }

        setListeTypesDocument( sbRepresentation.toString(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getListeTypesDocumentFromRepresentation()
         */
    public List<String> getListeTypesDocumentFromRepresentation(  )
    {
        List<String> listeTypesDocument = new ArrayList<String>(  );
        StringTokenizer stringTokenizer = new StringTokenizer( getListeTypesDocument(  ), ":, " );

        while ( stringTokenizer.hasMoreTokens(  ) )
        {
            listeTypesDocument.add( stringTokenizer.nextToken(  ) );
        }

        return listeTypesDocument;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getListeFormationsConseil()
         */
    public String getListeFormationsConseil(  )
    {
        return _strListeFormationsConseil;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setListeFormationsConseil(java.lang.String)
         */
    public void setListeFormationsConseil( String strListeFormationsConseil )
    {
        _strListeFormationsConseil = strListeFormationsConseil;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#isFiltreFormationsConseilActif(fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public boolean isFiltreFormationsConseilActif( Plugin plugin )
    {
        List<FormationConseil> listFormations = getListeFormationsConseilFromRepresentation( plugin );

        return listFormations.size(  ) == 1;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setFormationsConseilFromListe(java.util.List)
         */
    public void setFormationsConseilFromListe( List<FormationConseil> listeFormationsConseil )
    {
        StringBuilder sbRepresentation = new StringBuilder(  );

        if ( getListeFormationsConseil(  ) != null )
        {
            for ( FormationConseil formationConseil : listeFormationsConseil )
            {
                if ( sbRepresentation.length(  ) != 0 )
                {
                    sbRepresentation.append( ", " );
                }

                sbRepresentation.append( formationConseil.getIdFormationConseil(  ) );
            }
        }

        setListeFormationsConseil( sbRepresentation.toString(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getListeFormationsConseilFromRepresentation(fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public List<FormationConseil> getListeFormationsConseilFromRepresentation( Plugin plugin )
    {
        List<FormationConseil> listeFormationsConseil = new ArrayList<FormationConseil>(  );
        StringTokenizer stringTokenizer = new StringTokenizer( getListeFormationsConseil(  ), ":, " );

        while ( stringTokenizer.hasMoreTokens(  ) )
        {
            listeFormationsConseil.add( _formationConseilHome.findByPrimaryKey( Integer.parseInt( 
                        stringTokenizer.nextToken(  ) ), plugin ) );
        }

        return listeFormationsConseil;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getListeRapporteurs()
         */
    public String getListeRapporteurs(  )
    {
        return _strListeRapporteurs;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setListeRapporteurs(java.lang.String)
         */
    public void setListeRapporteurs( String strListeRapporteurs )
    {
        _strListeRapporteurs = strListeRapporteurs;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#isFiltreRapporteursActif()
         */
    public boolean isFiltreRapporteursActif(  )
    {
        return _strListeRapporteurs.length(  ) != 0;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setRapporteursFromListe(java.util.List)
         */
    public void setRapporteursFromListe( List<IElu> listeRapporteurs )
    {
        StringBuilder sbRepresentation = new StringBuilder(  );

        if ( listeRapporteurs != null )
        {
            for ( IElu rapporteur : listeRapporteurs )
            {
                if ( sbRepresentation.length(  ) != 0 )
                {
                    sbRepresentation.append( ", " );
                }

                sbRepresentation.append( rapporteur.getIdElu(  ) );
            }
        }

        setListeRapporteurs( sbRepresentation.toString(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getListeRapporteursFromRepresentation(fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public List<IElu> getListeRapporteursFromRepresentation( Plugin plugin )
    {
        List<IElu> listeRapporteurs = new ArrayList<IElu>(  );
        StringTokenizer stringTokenizer = new StringTokenizer( getListeRapporteurs(  ), ":, " );

        while ( stringTokenizer.hasMoreTokens(  ) )
        {
            listeRapporteurs.add( _eluHome.findByPrimaryKey( Integer.parseInt( stringTokenizer.nextToken(  ) ), plugin ) );
        }

        return listeRapporteurs;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getListeArrondissements()
         */
    public String getListeArrondissements(  )
    {
        return _strListeArrondissements;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setListeArrondissements(java.lang.String)
         */
    public void setListeArrondissements( String strListeArrondissements )
    {
        _strListeArrondissements = strListeArrondissements;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#isFiltreArrondissementsActif()
         */
    public boolean isFiltreArrondissementsActif(  )
    {
        return _strListeArrondissements.length(  ) != 0;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setArrondissementsFromListe(int[])
         */
    public void setArrondissementsFromListe( int[] listeArrondissements )
    {
        StringBuilder sbRepresentation = new StringBuilder(  );

        if ( getListeArrondissements(  ) != null )
        {
            for ( Integer arrondissement : listeArrondissements )
            {
                if ( sbRepresentation.length(  ) != 0 )
                {
                    sbRepresentation.append( ", " );
                }

                sbRepresentation.append( arrondissement.toString(  ) );
            }
        }

        setListeArrondissements( sbRepresentation.toString(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getListeArrondissementsFromRepresentation(fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public int[] getListeArrondissementsFromRepresentation( Plugin plugin )
    {
        StringTokenizer stringTokenizer = new StringTokenizer( getListeArrondissements(  ), ":, " );
        int[] listeArrondissements = new int[stringTokenizer.countTokens(  )];
        int nIndex = 0;

        while ( stringTokenizer.hasMoreTokens(  ) )
        {
            listeArrondissements[nIndex++] = Integer.parseInt( stringTokenizer.nextToken(  ) );
        }

        return listeArrondissements;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getListeDirections()
         */
    public String getListeDirections(  )
    {
        return _strListeDirections;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setListeDirections(java.lang.String)
         */
    public void setListeDirections( String strListeDirections )
    {
        _strListeDirections = strListeDirections;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#isFiltreDirectionsActif()
         */
    public boolean isFiltreDirectionsActif(  )
    {
        return _strListeDirections.length(  ) != 0;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setDirectionsFromListe(java.util.List)
         */
    public void setDirectionsFromListe( List<Direction> listeDirections )
    {
        StringBuilder sbRepresentation = new StringBuilder(  );

        if ( listeDirections != null )
        {
            for ( Direction direction : listeDirections )
            {
                if ( sbRepresentation.length(  ) != 0 )
                {
                    sbRepresentation.append( ", " );
                }

                sbRepresentation.append( direction.getIdDirection(  ) );
            }
        }

        setListeDirections( sbRepresentation.toString(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getListeDirectionsFromRepresentation(fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public List<Direction> getListeDirectionsFromRepresentation( Plugin plugin )
    {
        List<Direction> listeDirections = new ArrayList<Direction>(  );
        StringTokenizer stringTokenizer = new StringTokenizer( getListeDirections(  ), ":, " );

        while ( stringTokenizer.hasMoreTokens(  ) )
        {
            listeDirections.add( _directionHome.findByPrimaryKey( Integer.parseInt( stringTokenizer.nextToken(  ) ),
                    plugin ) );
        }

        return listeDirections;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getListeCategoriesDeliberation()
         */
    public String getListeCategoriesDeliberation(  )
    {
        return _strListeCategoriesDeliberation;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setListeCategoriesDeliberation(java.lang.String)
         */
    public void setListeCategoriesDeliberation( String strListeCategoriesDeliberation )
    {
        _strListeCategoriesDeliberation = strListeCategoriesDeliberation;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#isFiltreCategoriesDeliberationActif()
         */
    public boolean isFiltreCategoriesDeliberationActif(  )
    {
        return _strListeCategoriesDeliberation.length(  ) != 0;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setCategoriesDeliberationFromListe(java.util.List)
         */
    public void setCategoriesDeliberationFromListe( List<CategorieDeliberation> listeCategoriesDeliberation )
    {
        StringBuilder sbRepresentation = new StringBuilder(  );

        if ( getListeCategoriesDeliberation(  ) != null )
        {
            for ( CategorieDeliberation categorie : listeCategoriesDeliberation )
            {
                if ( sbRepresentation.length(  ) != 0 )
                {
                    sbRepresentation.append( ", " );
                }

                sbRepresentation.append( categorie.getIdCategorie(  ) );
            }
        }

        setListeCategoriesDeliberation( sbRepresentation.toString(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getListeCategorieDeliberationFromRepresentation(fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public List<CategorieDeliberation> getListeCategorieDeliberationFromRepresentation( Plugin plugin )
    {
        List<CategorieDeliberation> listeCategoriesDeliberation = new ArrayList<CategorieDeliberation>(  );
        StringTokenizer stringTokenizer = new StringTokenizer( getListeCategoriesDeliberation(  ), ":, " );

        while ( stringTokenizer.hasMoreTokens(  ) )
        {
            listeCategoriesDeliberation.add( _categorieDeliberationHome.findByPrimaryKey( Integer.parseInt( 
                        stringTokenizer.nextToken(  ) ), plugin ) );
        }

        return listeCategoriesDeliberation;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getListeGroupesDepositaires()
         */
    public String getListeGroupesDepositaires(  )
    {
        return _strListeGroupesDepositaires;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setListeGroupesDepositaires(java.lang.String)
         */
    public void setListeGroupesDepositaires( String strListeGroupesDepositaires )
    {
        _strListeGroupesDepositaires = strListeGroupesDepositaires;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#isFiltreGroupesDepositairesActif()
         */
    public boolean isFiltreGroupesDepositairesActif(  )
    {
        return _strListeGroupesDepositaires.length(  ) != 0;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setGroupesDepositairesFromListe(java.util.List)
         */
    public void setGroupesDepositairesFromListe( List<IGroupePolitique> listeGroupesDepositaires )
    {
        StringBuilder sbRepresentation = new StringBuilder(  );

        if ( listeGroupesDepositaires != null )
        {
            for ( IGroupePolitique groupe : listeGroupesDepositaires )
            {
                if ( sbRepresentation.length(  ) != 0 )
                {
                    sbRepresentation.append( ", " );
                }

                sbRepresentation.append( groupe.getIdGroupe(  ) );
            }
        }

        setListeGroupesDepositaires( sbRepresentation.toString(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getListeGroupesDepositairesFromRepresentation(fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public List<IGroupePolitique> getListeGroupesDepositairesFromRepresentation( Plugin plugin )
    {
        List<IGroupePolitique> listeGroupesDepositaires = new ArrayList<IGroupePolitique>(  );
        StringTokenizer stringTokenizer = new StringTokenizer( getListeGroupesDepositaires(  ), ":, " );

        while ( stringTokenizer.hasMoreTokens(  ) )
        {
            listeGroupesDepositaires.add( _groupePolitiqueHome.findByPrimaryKey( Integer.parseInt( 
                        stringTokenizer.nextToken(  ) ), plugin ) );
        }

        return listeGroupesDepositaires;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getListeElusDepositaires()
         */
    public String getListeElusDepositaires(  )
    {
        return _strListeElusDepositaires;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setListeElusDepositaires(java.lang.String)
         */
    public void setListeElusDepositaires( String strListeElusDepositaires )
    {
        _strListeElusDepositaires = strListeElusDepositaires;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#isFiltreElusDepositairesActif()
         */
    public boolean isFiltreElusDepositairesActif(  )
    {
        return _strListeElusDepositaires.length(  ) != 0;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setElusDepositairesFromListe(java.util.List)
         */
    public void setElusDepositairesFromListe( List<IElu> listeElusDepositaires )
    {
        StringBuilder sbRepresentation = new StringBuilder(  );

        if ( listeElusDepositaires != null )
        {
            for ( IElu eluDepositaire : listeElusDepositaires )
            {
                if ( sbRepresentation.length(  ) != 0 )
                {
                    sbRepresentation.append( " ," );
                }

                sbRepresentation.append( eluDepositaire.getIdElu(  ) );
            }
        }

        setListeElusDepositaires( sbRepresentation.toString(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getListeElusDepositairesFromRepresentation(fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public List<IElu> getListeElusDepositairesFromRepresentation( Plugin plugin )
    {
        List<IElu> listeElusDepositaires = new ArrayList<IElu>(  );
        StringTokenizer stringTokenizer = new StringTokenizer( getListeElusDepositaires(  ), ":, " );

        while ( stringTokenizer.hasMoreTokens(  ) )
        {
            listeElusDepositaires.add( _eluHome.findByPrimaryKey( Integer.parseInt( stringTokenizer.nextToken(  ) ),
                    plugin ) );
        }

        return listeElusDepositaires;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#isDeposeExecutif()
         */
    public boolean isDeposeExecutif(  )
    {
        return _bIsDeposeExecutif;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setDeposeExecutif(boolean)
         */
    public void setDeposeExecutif( boolean bIsDeposeExecutif )
    {
        _bIsDeposeExecutif = bIsDeposeExecutif;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#isNotifie()
         */
    public boolean isNotifie(  )
    {
        return _bIsNotifie;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setNotifie(boolean)
         */
    public void setNotifie( boolean bIsNotifie )
    {
        _bIsNotifie = bIsNotifie;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getDateCreation()
         */
    public Timestamp getDateCreation(  )
    {
        return _tsDateCreation;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setDateCreation(java.sql.Timestamp)
         */
    public void setDateCreation( Timestamp tsDateCreation )
    {
        _tsDateCreation = tsDateCreation;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getHtml()
         */
    public String getHtml(  )
    {
        return _strHtml;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setHtml(java.lang.String)
         */
    public void setHtml( String strHtml )
    {
        _strHtml = strHtml;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#toHtml(fr.paris.lutece.portal.service.plugin.Plugin, java.util.Locale)
         */
    protected HashMap<String, Object> toHtmlCommun( Plugin plugin, Locale locale )
    {
        HashMap<String, Object> model = new HashMap<String, Object>(  );
        model.put( MARK_LISTE_TYPES_DOC, getListeTypesDocumentFromRepresentation(  ) );
        model.put( MARK_DATE_1, getPremiereDate(  ) );
        model.put( MARK_DATE_2, getDeuxiemeDate(  ) );
        model.put( MARK_MOTS_RECHERCHES, getChampRecherche(  ) );
        model.put( MARK_LISTE_ARRONDISSEMENTS, getListeArrondissementsFromRepresentation( plugin ) );
        model.put( MARK_LISTE_RAPPORTEURS, getListeRapporteursFromRepresentation( plugin ) );
        model.put( MARK_LISTE_DIRECTIONS, getListeDirectionsFromRepresentation( plugin ) );
        model.put( MARK_LISTE_CATEGORIES, getListeCategorieDeliberationFromRepresentation( plugin ) );
        model.put( MARK_LISTE_GROUPES, getListeGroupesDepositairesFromRepresentation( plugin ) );
        model.put( MARK_DEPOSE_EXECUTIF, isDeposeExecutif(  ) );
        model.put( MARK_LISTE_STATUTS_PROJETS, getListeStatutsProjetsFromRepresentation( plugin ) );
        model.put( MARK_LISTE_STATUTS_PROPOSITIONS, getListeStatutsPropositionsFromRepresentation( plugin ) );
        model.put( MARK_LISTE_STATUTS_VOEUX_AMENDEMENTS, getListeStatutsVoeuxAmendementsFromRepresentation( plugin ) );
        model.put( MARK_LISTE_ELUS, getListeElusDepositairesFromRepresentation( plugin ) );

        return model;
    }

    /* (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#hashCode()
     */
    public int hashCode(  )
    {
        return getIdRequete(  );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#toString()
         */
    public String toString(  )
    {
        return getNomRequete(  );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getListeStatutsProjets()
         */
    public String getListeStatutsProjets(  )
    {
        return _strListeStatutsProjets;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setListeStatutsProjets(java.lang.String)
         */
    public void setListeStatutsProjets( String listeStatutsProjets )
    {
        _strListeStatutsProjets = listeStatutsProjets;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setStatutsProjetsFromListe(java.util.List)
         */
    public void setStatutsProjetsFromListe( List<Statut> listeStatuts )
    {
        StringBuilder sbRepresentation = new StringBuilder(  );

        if ( listeStatuts != null )
        {
            for ( Statut statut : listeStatuts )
            {
                if ( sbRepresentation.length(  ) != 0 )
                {
                    sbRepresentation.append( " ," );
                }

                sbRepresentation.append( statut.getIdStatut(  ) );
            }
        }

        setListeStatutsProjets( sbRepresentation.toString(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getListeStatutsProjetsFromRepresentation(fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public List<Statut> getListeStatutsProjetsFromRepresentation( Plugin plugin )
    {
        List<Statut> listeStatut = new ArrayList<Statut>(  );
        StringTokenizer stringTokenizer = new StringTokenizer( getListeStatutsProjets(  ), ":, " );

        while ( stringTokenizer.hasMoreTokens(  ) )
        {
            listeStatut.add( _statutHome.findByPrimaryKey( Integer.parseInt( stringTokenizer.nextToken(  ) ), plugin ) );
        }

        return listeStatut;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#isFiltreStatutsProjets()
         */
    public boolean isFiltreStatutsProjets(  )
    {
        return _strListeStatutsProjets.length(  ) != 0;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getListeStatutsPropositions()
         */
    public String getListeStatutsPropositions(  )
    {
        return _strListeStatutsPropositions;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setListeStatutsPropositions(java.lang.String)
         */
    public void setListeStatutsPropositions( String listeStatutsPropositions )
    {
        _strListeStatutsPropositions = listeStatutsPropositions;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setStatutsPropositionsFromListe(java.util.List)
         */
    public void setStatutsPropositionsFromListe( List<Statut> listeStatuts )
    {
        StringBuilder sbRepresentation = new StringBuilder(  );

        if ( listeStatuts != null )
        {
            for ( Statut statut : listeStatuts )
            {
                if ( sbRepresentation.length(  ) != 0 )
                {
                    sbRepresentation.append( " ," );
                }

                sbRepresentation.append( statut.getIdStatut(  ) );
            }
        }

        setListeStatutsPropositions( sbRepresentation.toString(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#isFiltreStatutsPropositions()
         */
    public boolean isFiltreStatutsPropositions(  )
    {
        return _strListeStatutsPropositions.length(  ) != 0;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getListeStatutsPropositionsFromRepresentation(fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public List<Statut> getListeStatutsPropositionsFromRepresentation( Plugin plugin )
    {
        List<Statut> listeStatut = new ArrayList<Statut>(  );
        StringTokenizer stringTokenizer = new StringTokenizer( getListeStatutsPropositions(  ), ":, " );

        while ( stringTokenizer.hasMoreTokens(  ) )
        {
            listeStatut.add( _statutHome.findByPrimaryKey( Integer.parseInt( stringTokenizer.nextToken(  ) ), plugin ) );
        }

        return listeStatut;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getListeStatutsVoeuxAmendements()
         */
    public String getListeStatutsVoeuxAmendements(  )
    {
        return _strListeStatutsVoeuxAmendements;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setListeStatutsVoeuxAmendements(java.lang.String)
         */
    public void setListeStatutsVoeuxAmendements( String listeStatutsVoeuxAmendements )
    {
        _strListeStatutsVoeuxAmendements = listeStatutsVoeuxAmendements;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#setStatutsVoeuxAmendementsFromListe(java.util.List)
         */
    public void setStatutsVoeuxAmendementsFromListe( List<Statut> listeStatuts )
    {
        StringBuilder sbRepresentation = new StringBuilder(  );

        if ( listeStatuts != null )
        {
            for ( Statut statut : listeStatuts )
            {
                if ( sbRepresentation.length(  ) != 0 )
                {
                    sbRepresentation.append( " ," );
                }

                sbRepresentation.append( statut.getIdStatut(  ) );
            }
        }

        setListeStatutsVoeuxAmendements( sbRepresentation.toString(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#getListeStatutsVoeuxAmendementsFromRepresentation(fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public List<Statut> getListeStatutsVoeuxAmendementsFromRepresentation( Plugin plugin )
    {
        List<Statut> listeStatut = new ArrayList<Statut>(  );
        StringTokenizer stringTokenizer = new StringTokenizer( getListeStatutsVoeuxAmendements(  ), ":, " );

        while ( stringTokenizer.hasMoreTokens(  ) )
        {
            listeStatut.add( _statutHome.findByPrimaryKey( Integer.parseInt( stringTokenizer.nextToken(  ) ), plugin ) );
        }

        return listeStatut;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur#isFiltreStatutsVoeuxAmendements()
         */
    public boolean isFiltreStatutsVoeuxAmendements(  )
    {
        return _strListeStatutsVoeuxAmendements.length(  ) != 0;
    }
}
