/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.requete;

import fr.paris.lutece.plugins.ods.dto.IDesignation;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;

import java.util.List;


/**
 * 
  * ResultatRequete
 *
 * @param <GSeanceFilter> ISeanceFilter
 * @param <GSeance> ISeance
 * @param <GRequeteUtilisateur> IRequeteUtilisateur
 * @param <GElu> IElu
 * @param <GFichier> IFichier
 * @param <GVoeuAmendement> {@link IVoeuAmendement}
 * @param <GPDD> {@link IPDD}
 */
public abstract class AbstractResultatRequete
<
	GSeance extends ISeance, 
	GSeanceFilter extends ISeanceFilter, GRequeteUtilisateur extends IRequeteUtilisateur<GSeance, GSeanceFilter>, 
	GElu extends IElu, 
	GFichier extends IFichier<GSeance, GFichier>, 
	GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, 
	GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>
>
implements IResultatRequete<GSeance, GSeanceFilter, GRequeteUtilisateur, GElu, GFichier, GVoeuAmendement, GPDD>
{
    private GRequeteUtilisateur _requeteUtilisateur;
    private int _nNombreResultats;
    private int _nNombreObjetsDetailles;
    private List<GPDD> _listProjets;
    private List<GPDD> _listPropositions;
    private List<GVoeuAmendement> _listAmendements;
    private List<GVoeuAmendement> _listVoeux;
    private List<GSeance> _listSeances;
    private List<GFichier> _listAutresDocuments;
    private List<IDesignation<GSeance, GFichier>> _listDesignations;

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete#getRequeteUtilisateur()
         */
    public GRequeteUtilisateur getRequeteUtilisateur(  )
    {
        return _requeteUtilisateur;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete#setRequeteUtilisateur(fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur)
         */
    public void setRequeteUtilisateur( GRequeteUtilisateur requeteUtilisateur )
    {
        _requeteUtilisateur = requeteUtilisateur;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete#getNombreResultats()
         */
    public int getNombreResultats(  )
    {
        return _nNombreResultats;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete#setNombreResultats(int)
         */
    public void setNombreResultats( int nNombreResultats )
    {
        _nNombreResultats = nNombreResultats;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete#getNombreObjetsDetailles()
         */
    public int getNombreObjetsDetailles(  )
    {
        return _nNombreObjetsDetailles;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete#setNombreObjetsDetailles(int)
         */
    public void setNombreObjetsDetailles( int nNombreObjetsDetailles )
    {
        _nNombreObjetsDetailles = nNombreObjetsDetailles;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete#getListeProjets()
         */
    public List<GPDD> getListeProjets(  )
    {
        return _listProjets;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete#setListeProjets(java.util.List)
         */
    public void setListeProjets( List<GPDD> listProjets )
    {
        _listProjets = listProjets;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete#getListePropositions()
         */
    public List<GPDD> getListePropositions(  )
    {
        return _listPropositions;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete#setListePropositions(java.util.List)
         */
    public void setListePropositions( List<GPDD> listPropositions )
    {
        _listPropositions = listPropositions;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete#getListeAmendements()
         */
    public List<GVoeuAmendement> getListeAmendements(  )
    {
        return _listAmendements;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete#setListeAmendements(java.util.List)
         */
    public void setListeAmendements( List<GVoeuAmendement> listAmendements )
    {
        _listAmendements = listAmendements;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete#getListeVoeux()
         */
    public List<GVoeuAmendement> getListeVoeux(  )
    {
        return _listVoeux;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete#setListeVoeux(java.util.List)
         */
    public void setListeVoeux( List<GVoeuAmendement> listVoeux )
    {
        _listVoeux = listVoeux;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete#getListeSeances()
         */
    public List<GSeance> getListeSeances(  )
    {
        return _listSeances;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete#setListeSeances(java.util.List)
         */
    public void setListeSeances( List<GSeance> listSeances )
    {
        _listSeances = listSeances;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete#getListeDesignations()
         */
    public List<IDesignation<GSeance, GFichier>> getListeDesignations(  )
    {
        return _listDesignations;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete#setListeDesignations(java.util.List)
         */
    public void setListeDesignations( List<IDesignation<GSeance, GFichier>> listDesignations )
    {
        _listDesignations = listDesignations;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete#getListeAutresDocuments()
         */
    public List<GFichier> getListeAutresDocuments(  )
    {
        return _listAutresDocuments;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete#setListeAutresDocuments(java.util.List)
         */
    public void setListeAutresDocuments( List<GFichier> listAutresDocuments )
    {
        _listAutresDocuments = listAutresDocuments;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete#addAll(fr.paris.lutece.plugins.ods.dto.requete.ResultatRequete)
         */
    public void addAll( 
        IResultatRequete<GSeance, GSeanceFilter, GRequeteUtilisateur, GElu, GFichier, GVoeuAmendement, GPDD> resultat )
    {
        /*
         * Ajout des nouveaux projets
         */
        if ( _listProjets == null )
        {
            _listProjets = resultat.getListeProjets(  );
        }
        else if ( resultat.getListeProjets(  ) != null )
        {
            List<GPDD> listPdds = resultat.getListeProjets(  );
            boolean bAjouter;

            for ( GPDD projet1 : listPdds )
            {
                bAjouter = true;

                for ( GPDD projet2 : _listProjets )
                {
                    if ( projet1.equals( projet2 ) )
                    {
                        bAjouter = false;

                        break;
                    }
                }

                if ( bAjouter )
                {
                    _listProjets.add( projet1 );
                }
            }
        }

        /*
         * Ajout des nouvelles propositions
         */
        if ( _listPropositions == null )
        {
            _listPropositions = resultat.getListeProjets(  );
        }
        else if ( resultat.getListeProjets(  ) != null )
        {
            List<GPDD> listPdds = resultat.getListeProjets(  );
            boolean bAjouter;

            for ( GPDD proposition1 : listPdds )
            {
                bAjouter = true;

                for ( GPDD proposition2 : _listPropositions )
                {
                    if ( proposition1.equals( proposition2 ) )
                    {
                        bAjouter = false;

                        break;
                    }
                }

                if ( bAjouter )
                {
                    _listPropositions.add( proposition1 );
                }
            }
        }

        /*
         * Ajout des nouveaux Amendements
         */
        if ( _listAmendements == null )
        {
            _listAmendements = resultat.getListeAmendements(  );
        }
        else if ( resultat.getListeAmendements(  ) != null )
        {
            List<GVoeuAmendement> listAmendements = resultat.getListeAmendements(  );
            boolean bAjouter;

            for ( GVoeuAmendement amendement1 : listAmendements )
            {
                bAjouter = true;

                for ( GVoeuAmendement amendement2 : _listAmendements )
                {
                    if ( amendement1.equals( amendement2 ) )
                    {
                        bAjouter = false;

                        break;
                    }
                }

                if ( bAjouter )
                {
                    _listAmendements.add( amendement1 );
                }
            }
        }

        /*
         * Ajout des nouveaux Voeux
         */
        if ( _listVoeux == null )
        {
            _listVoeux = resultat.getListeVoeux(  );
        }
        else if ( resultat.getListeVoeux(  ) != null )
        {
            List<GVoeuAmendement> listVoeux = resultat.getListeVoeux(  );
            boolean bAjouter;

            for ( GVoeuAmendement voeu1 : listVoeux )
            {
                bAjouter = true;

                for ( GVoeuAmendement voeu2 : _listVoeux )
                {
                    if ( voeu1.equals( voeu2 ) )
                    {
                        bAjouter = false;

                        break;
                    }
                }

                if ( bAjouter )
                {
                    _listVoeux.add( voeu1 );
                }
            }
        }

        /*
         * Ajout des nouvelles sances
         */
        if ( _listSeances == null )
        {
            _listSeances = resultat.getListeSeances(  );
        }
        else if ( resultat.getListeSeances(  ) != null )
        {
            _listSeances.addAll( resultat.getListeSeances(  ) );
        }

        /*
         * Ajout des nouveaux autres documents
         */
        if ( _listAutresDocuments == null )
        {
            _listAutresDocuments = resultat.getListeAutresDocuments(  );
        }
        else if ( resultat.getListeAutresDocuments(  ) != null )
        {
            _listAutresDocuments.addAll( resultat.getListeAutresDocuments(  ) );
        }

        /*
         * Ajout des nouvelles dsignations
         */
        if ( _listDesignations == null )
        {
            _listDesignations = resultat.getListeDesignations(  );
        }
        else if ( resultat.getListeDesignations(  ) != null )
        {
            _listDesignations.addAll( resultat.getListeDesignations(  ) );
        }
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete#isEmpty()
         */
    public boolean isEmpty(  )
    {
        boolean bRetour = true;

        if ( ( _listProjets != null ) && !_listProjets.isEmpty(  ) )
        {
            bRetour = false;
        }

        else if ( ( _listPropositions != null ) && !_listPropositions.isEmpty(  ) )
        {
            bRetour = false;
        }

        else if ( ( _listAmendements != null ) && !_listAmendements.isEmpty(  ) )
        {
            bRetour = false;
        }

        else if ( ( _listVoeux != null ) && !_listVoeux.isEmpty(  ) )
        {
            bRetour = false;
        }

        else if ( ( _listSeances != null ) && !_listSeances.isEmpty(  ) )
        {
            bRetour = false;
        }

        else if ( ( _listAutresDocuments != null ) && !_listAutresDocuments.isEmpty(  ) )
        {
            bRetour = false;
        }

        else if ( ( _listDesignations != null ) && !_listDesignations.isEmpty(  ) )
        {
            bRetour = false;
        }

        return bRetour;
    }
}
