/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.requete;

import fr.paris.lutece.plugins.ods.dto.categoriedeliberation.CategorieDeliberation;
import fr.paris.lutece.plugins.ods.dto.direction.Direction;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.groupepolitique.IGroupePolitique;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.statut.Statut;
import fr.paris.lutece.portal.service.plugin.Plugin;

import java.sql.Timestamp;

import java.util.List;
import java.util.Locale;


/**
 *
 * IRequeteUtilisateur
 *
 * @param <GSeance> ISeance
 * @param <GSeanceFilter> ISeanceFilter
 */
public interface IRequeteUtilisateur<GSeance extends ISeance, GSeanceFilter extends ISeanceFilter>
{
    /**
     * Retourne l'id de la requte
     * @return l'id de la requte
     */
    int getIdRequete(  );

    /**
     * Fixe l'id de la requte
     * @param nIdRequete l'id de la requte
     */
    void setIdRequete( int nIdRequete );

    /**
     * Retourne l'id de l'utilisateur qui a enregistr sa requte
     * @return l'id de l'utilisateur qui a enregistr sa requte
     */
    String getUserName(  );

    /**
     * Fixe l'id de l'utilisateur qui a enregistr sa requte
     * @param strUserName l'id de l'utilisateur qui a enregistr sa requte
     */
    void setUserName( String strUserName );

    /**
     * Retourne le nom de sauvegarde de la requte
     * @return le nom de sauvegarde de la requte
     */
    String getNomRequete(  );

    /**
     * Fixe le nom de sauvegarde de la requte
     * @param strNomRequete le nom de sauvegarde de la requte
     */
    void setNomRequete( String strNomRequete );

    /**
     * Indique si la recherche concerne les archives ou la prochaine sance
     * @return <b>true</b> si la recherche concerne les archives, <b>false</b> sinon
     */
    boolean isRechercheArchive(  );

    /**
     * Pour rechercher sur les archives ou sur la prochaine sance
     * @param bRechercheArchive recherche sur archives ou non
     */
    void setRechercheArchive( boolean bRechercheArchive );

    /**
     * Retourne le type de la requte: simple (par le bandeau haut), par rfrence ou multi-critre
     * @return le type de la requte
     */
    String getTypeRequete(  );

    /**
     * Fixe le type de la requte
     * @param strTypeRequete le type de la requte
     */
    void setTypeRequete( String strTypeRequete );

    /**
     * Retourne la premiere date de sance renseigne
     * @return la premiere date de sance renseigne
     */
    Timestamp getPremiereDate(  );

    /**
     * Fixe la premiere date de sance renseigne
     * @param tsPremiereDate la premiere date de sance renseigne
     */
    void setPremiereDate( Timestamp tsPremiereDate );

    /**
     * Retourne la deuxime date de sance renseigne
     * @return la deuxime date de sance renseigne
     */
    Timestamp getDeuxiemeDate(  );

    /**
     * Fixe la deuxime date de sance renseigne
     * @param tsDeuxiemeDate la deuxime date de sance renseigne
     */
    void setDeuxiemeDate( Timestamp tsDeuxiemeDate );

    /**
     * Renvoie la liste des sances comprises entre les deux dates de la requete,
     * ou toutes les sances passes si aucune date n'est renseigne
     * @param plugin le plugin ODS
     * @return une liste de sances
     */
    List<GSeance> getListeSeances( Plugin plugin );

    /**
     * Retourne les mots (ou la rfrence) recherchs
     * @return les mots (ou la rfrence) recherche
     */
    String getChampRecherche(  );

    /**
     * Fixe le champ de saisie sur les mots (ou la rfrence) recherche
     * @param strChampRecherche les mots (ou la rfrence) recherche
     */
    void setChampRecherche( String strChampRecherche );

    /**
     * Retourne la liste des types de document recherchs
     * @return la liste des types de document recherchs
     */
    String getListeTypesDocument(  );

    /**
     * Fixe la liste des types de document recherchs
     * @param strListeTypesDocument la liste des types de document recherchs
     */
    void setListeTypesDocument( String strListeTypesDocument );

    /**
     * Fixe la reprsentation en base de la liste des types de documents
     * @param listeTypesDocument la liste des types de documents
     */
    void setTypesDocumentFromListe( List<String> listeTypesDocument );

    /**
     * Retourne la liste des types de document  partir de la reprsentation en base
     * @return la liste des types de document  partir de la reprsentation en base
     */
    List<String> getListeTypesDocumentFromRepresentation(  );

    /**
     * Retourne la liste des formations de conseil slectionnes
     * @return la liste des formations de conseil slectionnes
     */
    String getListeFormationsConseil(  );

    /**
     * Fixe la liste des formations de conseil slectionnes
     * @param strListeFormationsConseil la liste des formations de conseil slectionnes
     */
    void setListeFormationsConseil( String strListeFormationsConseil );

    /**
     * Indique si la requte contient un filtre sur les formations de conseil
     * @param plugin le plugin ODS
     * @return <code>true</code> si la requte porte sur au moins une formation de conseil, <code>false</code> sinon
     */
    boolean isFiltreFormationsConseilActif( Plugin plugin );

    /**
     * Fixe la reprsentation en base de la liste des formations slectionnes
     * @param listeFormationsConseil la liste des formations slectionnes
     */
    void setFormationsConseilFromListe( List<FormationConseil> listeFormationsConseil );

    /**
     * Retourne la liste des formations de conseil slectionnes  partir de la reprsentation en base
     * @param plugin le plugin
     * @return la liste des formations de conseil slectionnes
     */
    List<FormationConseil> getListeFormationsConseilFromRepresentation( Plugin plugin );

    /**
     * Retourne la liste des lus rapporteurs slectionns
     * @return la liste des lus rapporteurs slectionns
     */
    String getListeRapporteurs(  );

    /**
     * Fixe la liste des lus rapporteurs slectionns
     * @param strListeRapporteurs la liste des lus rapporteurs slectionns
     */
    void setListeRapporteurs( String strListeRapporteurs );

    /**
     * Indique si la requte contient un filtre sur les rapporteurs
     * @return <code>true</code> si la requte porte sur au moins un rapporteur, <code>false</code> sinon
     */
    boolean isFiltreRapporteursActif(  );

    /**
     * Fixe la reprsentation en base de la liste des lus rapporteurs slectionns
     * @param listeRapporteurs la liste des lus rapporteurs slectionns
     */
    void setRapporteursFromListe( List<IElu> listeRapporteurs );

    /**
     * Retourne la liste des lus rapporteurs  partir de la reprsentation en base
     * @param plugin le plugin
     * @return la liste des lus rapporteurs  partir de la reprsentation en base
     */
    List<IElu> getListeRapporteursFromRepresentation( Plugin plugin );

    /**
     * Retourne la liste des arrondissements slectionns
     * @return la liste des arrondissements slectionns
     */
    String getListeArrondissements(  );

    /**
     * Fixe la liste des arrondissements slectionns
     * @param strListeArrondissements la liste des arrondissements slectionns
     */
    void setListeArrondissements( String strListeArrondissements );

    /**
     * Indique si la requte contient un filtre sur les arrondissements
     * @return <code>true</code> si la requte porte sur au moins un arrondissement, <code>false</code> sinon
     */
    boolean isFiltreArrondissementsActif(  );

    /**
     * Fixe la reprsentation en base de la liste des arrondissements
     * @param listeArrondissements la liste des arrondissements
     */
    void setArrondissementsFromListe( int[] listeArrondissements );

    /**
     * Retourne la liste de arrondissements slectionns  partir de la reprsentation en base
     * @param plugin le plugin
     * @return la liste des arrondissements slectionns
     */
    int[] getListeArrondissementsFromRepresentation( Plugin plugin );

    /**
     * Retourne la liste des directions slectionnes
     * @return la liste des directions slectionnes
     */
    String getListeDirections(  );

    /**
     * Fixe la liste des directions slectionnes
     * @param strListeDirections la liste des directions slectionnes
     */
    void setListeDirections( String strListeDirections );

    /**
     * Indique si la requte contient un filtre sur les directions
     * @return <code>true</code> si la requte porte sur au moins une direction, <code>false</code> sinon
     */
    boolean isFiltreDirectionsActif(  );

    /**
     * Fixe la reprsentation en base de la liste des directions slectionnes
     * @param listeDirections la liste des directions slectionnes
     */
    void setDirectionsFromListe( List<Direction> listeDirections );

    /**
     * Retourne la liste des directions slectionnes  partir de la reprsentation en base
     * @param plugin le plugin
     * @return la liste des directions slectionnes
     */
    List<Direction> getListeDirectionsFromRepresentation( Plugin plugin );

    /**
     * Retourne la liste des catgories de dlibration slectionnes
     * @return la liste des catgories de dlibration slectionnes
     */
    String getListeCategoriesDeliberation(  );

    /**
     * Fixe la liste des catgories de dlibration slectionnes
     * @param strListeCategoriesDeliberation la liste des catgories de dlibration slectionnes
     */
    void setListeCategoriesDeliberation( String strListeCategoriesDeliberation );

    /**
     * Indique si la requte contient un filtre sur les catgories de dlibration
     * @return <code>true</code> si la requte porte sur au moins une catgorie de dlibration, <code>false</code> sinon
     */
    boolean isFiltreCategoriesDeliberationActif(  );

    /**
     * Fixe la reprsentation en base de la liste des catgories de dlibration slectionnes
     * @param listeCategoriesDeliberation la liste des catgories de dlibration slectionnes
     */
    void setCategoriesDeliberationFromListe( List<CategorieDeliberation> listeCategoriesDeliberation );

    /**
     * Retourne la liste des catgories de dlibration slectionnes  partir de la reprsentation en base
     * @param plugin le plugin
     * @return la liste des catgories de dlibration
     */
    List<CategorieDeliberation> getListeCategorieDeliberationFromRepresentation( Plugin plugin );

    /**
     * Retourne la liste des groupes dpositaires slectionns
     * @return la liste des groupes dpositaires slectionns
     */
    String getListeGroupesDepositaires(  );

    /**
     * Fixe la liste des groupes dpositaires slectionns
     * @param strListeGroupesDepositaires la liste des groupes dpositaires slectionns
     */
    void setListeGroupesDepositaires( String strListeGroupesDepositaires );

    /**
     * Indique si la requte contient un filtre sur les groupes dpositaires
     * @return <code>true</code> si la requte porte sur au moins un groupe dpositaire, <code>false</code> sinon
     */
    boolean isFiltreGroupesDepositairesActif(  );

    /**
     * Fixe la reprsentation en base de la liste des groupes dpositaires slectionns
     * @param listeGroupesDepositaires la liste des groupes dpositaires slectionns
     */
    void setGroupesDepositairesFromListe( List<IGroupePolitique> listeGroupesDepositaires );

    /**
     * Retourne la liste des groupes dpositaires slectionns  partir de la reprsentation en base
     * @param plugin le plugin
     * @return la liste des groupes dpositaires slectionns
     */
    List<IGroupePolitique> getListeGroupesDepositairesFromRepresentation( Plugin plugin );

    /**
     * Retourne la liste des lus dpositaires slectionns
     * @return la liste des lus dpositaires slectionns
     */
    String getListeElusDepositaires(  );

    /**
     * Fixe la liste des lus dpositaires slectionns
     * @param strListeElusDepositaires la liste des lus dpositaires slectionns
     */
    void setListeElusDepositaires( String strListeElusDepositaires );

    /**
     * Indique si la requte contient un filtre sur les lus dpositaires
     * @return <code>true</code> si la requte porte sur au moins un lu dpositaire, <code>false</code> sinon
     */
    boolean isFiltreElusDepositairesActif(  );

    /**
     * Fixe la reprsentation en base de la liste des lus dpositaires slectionns
     * @param listeElusDepositaires la liste des lus slectionns
     */
    void setElusDepositairesFromListe( List<IElu> listeElusDepositaires );

    /**
     * Retourne la liste des lus dpositaires slectionns  partir de la reprsentation en base
     * @param plugin le plugin
     * @return la liste des lus dpositaires slectionns
     */
    List<IElu> getListeElusDepositairesFromRepresentation( Plugin plugin );

    /**
     * Indique si les voeux amendements retourns par la recherche doivent etre dposs par l'executif
     * @return <b>true</b> si les voeux amendements retourns par la recherche doivent etre dposs par l'executif <b>false</b> sinon
     */
    boolean isDeposeExecutif(  );

    /**
     * Fixe le paramtre dpos par l'excutif
     * @param bIsDeposeExecutif la valeur  fixer
     */
    void setDeposeExecutif( boolean bIsDeposeExecutif );

    /**
     * Indique si cette requte doit tre notifie  l'utilisateur si le rsultat a chang
     * @return <b>true</b> si l'utilisateur veut tre alert, <b>false</b> sinon
     */
    boolean isNotifie(  );

    /**
     * Fixe le paramtre de notification
     * @param bIsNotifie la valeur  fixer
     */
    void setNotifie( boolean bIsNotifie );

    /**
     * Retuorne la date d'enregistrement de la requte
     * @return la date d'enregistrement de la requte
     */
    Timestamp getDateCreation(  );

    /**
     * Fixe la date d'enregistrement de la requte
     * @param tsDateCreation la date d'enregistrement de la requte
     */
    void setDateCreation( Timestamp tsDateCreation );

    /**
     * Retourne le HTML
     * @return String HTML
     */
    String getHtml(  );

    /**
     *
     * @param strHtml le code Html
     */
    void setHtml( String strHtml );

    /**
     * Retourne le code HTML correspondant  la requte
     *
     * Retourne la requte prpare pour l'affichage dans la liste des requtes
     * @param plugin Plugin
     * @param locale Local
     */
    void toHtml( Plugin plugin, Locale locale );

    /**
     * Retourne un identifiant unique par RequeteUtilisateur
     * @return l'identifiant unique de la requte
     */
    int hashCode(  );

    /**
     * 2 requtes utilisateurs sont considres gales si elles ont le mme nom pour le mme utilisateur
     * @param requete une requete
     * @return <b>true</b> si l'objet pass en argument est une requte utilisateur et si le couple
     *           (nom de requte / utilisateur) est gal, <b>false</b> sinon
     */
    boolean equals( Object requete );

    /**
     * Texte affich lorsque l'on affiche l'objet (concrtement le nom de la requte)
     * @return le nom de la requte
     */
    String toString(  );

    /**
     * retourne la liste des statuts des projets de dlibration
     * @return  la liste des statuts des projets de dlibration
     */
    String getListeStatutsProjets(  );

    /**
     *  Fixe la liste des statuts des projets de dlibration
     * @param listeStatutsProjets la liste des statuts des projets de dlibration
     */
    void setListeStatutsProjets( String listeStatutsProjets );

    /**
     * Fixe la reprsentation en base de la liste des statuts des projets de dlibration slectionns
     * @param listeStatuts la liste des statuts slectionns
     */
    void setStatutsProjetsFromListe( List<Statut> listeStatuts );

    /**
     * Retourne la liste des statuts des pojets  partir de la reprsentation en base
     * @param plugin le plugin
     * @return la liste des statuts des pojets  partir de la reprsentation en base
     */
    List<Statut> getListeStatutsProjetsFromRepresentation( Plugin plugin );

    /**
     * Indique si la requte contient un filtre sur les statuts des projets
     * @return <code>true</code> si la requte porte sur au moins un statut, <code>false</code> sinon
     */
    boolean isFiltreStatutsProjets(  );

    /**
     * retourne la liste des statuts des propositions de dlibration
     * @return la liste des statuts des propositions de dlibration
     */
    String getListeStatutsPropositions(  );

    /**
     *  Fixe la liste des statuts des propositions de dlibration
     * @param listeStatutsPropositions liste des statuts des propositions de dlibration
     */
    void setListeStatutsPropositions( String listeStatutsPropositions );

    /**
     * Fixe la reprsentation en base de la liste des statuts des propositions de dlibration slectionns
     * @param listeStatuts la liste des statuts slectionns
     */
    void setStatutsPropositionsFromListe( List<Statut> listeStatuts );

    /**
     * Indique si la requte contient un filtre sur les statuts des propositions
     * @return <code>true</code> si la requte porte sur au moins un statut, <code>false</code> sinon
     */
    boolean isFiltreStatutsPropositions(  );

    /**
     * Retourne la liste des statuts des popositions  partir de la reprsentation en base
     * @param plugin le plugin
     * @return la liste des statuts des popositions  partir de la reprsentation en base
     */
    List<Statut> getListeStatutsPropositionsFromRepresentation( Plugin plugin );

    /**
     * retourne la liste des statuts des voeux amendements
     * @return la liste des statuts des voeux amendements
     */
    String getListeStatutsVoeuxAmendements(  );

    /**
     *  fixe la liste des statuts des voeux amendements
     * @param listeStatutsVoeuxAmendements la liste des statuts des voeux amendements
     */
    void setListeStatutsVoeuxAmendements( String listeStatutsVoeuxAmendements );

    /**
     * Fixe la reprsentation en base de la liste des statuts des voeux et amendements  slectionns
     * @param listeStatuts la liste des statuts slectionns
     */
    void setStatutsVoeuxAmendementsFromListe( List<Statut> listeStatuts );

    /**
     * Retourne la liste des statuts des voeux amendements  partir de la reprsentation en base
     * @param plugin le plugin
     * @return la liste des statuts des voeux amendemens  partir de la reprsentation en base
     */
    List<Statut> getListeStatutsVoeuxAmendementsFromRepresentation( Plugin plugin );

    /**
     * Indique si la requte contient un filtre sur les statuts des voeux amendements
     * @return <code>true</code> si la requte porte sur au moins un statut, <code>false</code> sinon
     */
    boolean isFiltreStatutsVoeuxAmendements(  );
}
