/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.requete;

import fr.paris.lutece.plugins.ods.dto.IDesignation;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;

import java.util.List;


/**
 * 
 * IResultatRequete
 * 
 * @param <GSeance> ISeance
 * @param <GSeanceFilter> ISeanceFilter
 * @param <GRequeteUtilisateur> IRequeteUtilisateur
 * @param <GElu> {@link IElu}
 * @param <GFichier> {@link IFichier}
 * @param <GVoeuAmendement> {@link IVoeuAmendement}
 * @param <GPDD> {@link IPDD}
 */
public interface IResultatRequete
<
	GSeance extends ISeance, 
	GSeanceFilter extends ISeanceFilter, 
	GRequeteUtilisateur extends IRequeteUtilisateur<GSeance, GSeanceFilter>, 
	GElu extends IElu, 
	GFichier extends IFichier<GSeance, GFichier>, 
	GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, 
	GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>
>
{
    /**
     * Retourne la requete utilisateur dont elle est le rsultat
     * @return la requete utilisateur dont elle est le rsultat
     */
    GRequeteUtilisateur getRequeteUtilisateur(  );

    /**
     * Fixe la requete utilisateur dont elle est le rsultat
     * @param requeteUtilisateur la requete utilisateur dont elle est le rsultat
     */
    void setRequeteUtilisateur( GRequeteUtilisateur requeteUtilisateur );

    /**
     * Retuorne le nombre de rsultats de la recherche
     * @return le nombre de rsultats de la recherche
     */
    int getNombreResultats(  );

    /**
     * Fixe le nombre de rsultats de la recherche
     * @param nNombreResultats le nombre de rsultats
     */
    void setNombreResultats( int nNombreResultats );

    /**
     * Retuorne le nombre d'objets retourns par la recherche
     * @return le nombre d'objets retourns par la recherche
     */
    int getNombreObjetsDetailles(  );

    /**
     * Fixe le nombre d'objets retourns par la recherche
     * @param nNombreObjetsDetailles le nombre d'objets retourns par la recherche
     */
    void setNombreObjetsDetailles( int nNombreObjetsDetailles );

    /**
     * Retourne la liste des projets de la requete tris par rfrence
     * @return la liste des projets de la requete
     */
    List<GPDD> getListeProjets(  );

    /**
     * Fixe la liste des projets de la requete
     * @param listProjets la liste des projets de la requete
     */
    void setListeProjets( List<GPDD> listProjets );

    /**
     * Retourne la liste des propositions de la requete tris par rfrence
     * @return la liste des propositions de la requete
     */
    List<GPDD> getListePropositions(  );

    /**
     * Fixe la liste des propositions de la requete
     * @param listPropositions la liste des propositions de la requete
     */
    void setListePropositions( List<GPDD> listPropositions );

    /**
     * Retourne la liste des amendements de la requete tris par numro
     * @return la liste des amendements de la requete
     */
    List<GVoeuAmendement> getListeAmendements(  );

    /**
     * Fixe la liste des amendements de la requete
     * @param listAmendements la liste des amendements de la requete
     */
    void setListeAmendements( List<GVoeuAmendement> listAmendements );

    /**
     * Retourne la liste des voeux de la requete tris par numro
     * @return la liste des voeux de la requete
     */
    List<GVoeuAmendement> getListeVoeux(  );

    /**
     * Fixe la liste des voeux de la requete
     * @param listVoeux la liste des voeux de la requete
     */
    void setListeVoeux( List<GVoeuAmendement> listVoeux );

    /**
     * Retourne la liste des sances de la requete tries par date de sance (de la plus rcente  la plus ancienne)
     * @return la liste des sances de la requete
     */
    List<GSeance> getListeSeances(  );

    /**
     * Fixe la liste des sances de la requete
     * @param listSeances la liste des sances de la requete
     */
    void setListeSeances( List<GSeance> listSeances );

    /**
     * Retourne la liste des arretes de la requete
     * @return la liste des arretes de la requete
     */
    List<IDesignation<GSeance, GFichier>> getListeDesignations(  );

    /**
     * Fixe la liste des arretes de la requete
     * @param listDesignations la liste des dsignations
     */
    void setListeDesignations( List<IDesignation<GSeance, GFichier>> listDesignations );

    /**
     * Retourne la liste des autres documents de sance de la requete
     * @return la liste des autres documents de sance de la requete
     */
    List<GFichier> getListeAutresDocuments(  );

    /**
     * Fixe la liste des autres documents de sance de la requete
     * @param listAutresDocuments la liste des autres documents de sance de la requete
     */
    void setListeAutresDocuments( List<GFichier> listAutresDocuments );

    /**
     * Ajoute les rsultats d'une autre requte  ce rsultat
     * @param resultat l'autre rsultat de requte
     */
    void addAll( 
        IResultatRequete<GSeance, GSeanceFilter, GRequeteUtilisateur, GElu, GFichier, GVoeuAmendement, GPDD> resultat );

    /**
     *
     * @return true si aucun resultat , false sinon
     */
    boolean isEmpty(  );
}
