/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.seance;

import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.util.xml.XmlUtil;

import java.sql.Timestamp;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;


/**
 *
 * Reponsable de decrire une Seance
 *
 */
public abstract class AbstractSeance implements ISeance
{
    protected static final String TAG_NUMERO_SEANCE = "numeroSeance";
    private int _nIdSeance;
    private Timestamp _dateSeance;
    private String _strNumero;
    private long _tailleArchive = -1;
    private Timestamp _datePublicationArchive;

    /**
     * Retourne la date de dbut de la sance
     * @return dateSeance date de dbut de la sance
     */
    public Timestamp getDateSeance(  )
    {
        return _dateSeance;
    }

    /**
     * Retourne le moi de la seance
     * @return dateSeance date de dbut de la sance
     */
    public String getMoisSeance(  )
    {
        Calendar gc = new GregorianCalendar(  );
        gc.setTimeInMillis( _dateSeance.getTime(  ) );

        return Integer.toString( gc.get( Calendar.MONTH ) + 1 );
    }

    /**
     * Affecte la date de dbut de sance
     * @param dateSeance date de dbut de sance
     */
    public void setDateSeance( java.sql.Timestamp dateSeance )
    {
        this._dateSeance = dateSeance;
    }

    /**
     * Retourne l'identifiant de la sance
     * @return _nIdSeance identifiant de la sance
     */
    public int getIdSeance(  )
    {
        return _nIdSeance;
    }

    /**
     * Affecte l'identifiant de la sance
     * @param nIdSeance int
     */
    public void setIdSeance( int nIdSeance )
    {
        this._nIdSeance = nIdSeance;
    }

    /**
     * Gnration du XML d'un objet
     * @return Le code XML reprsentant un ordre du jour
     */
    public String getXml(  )
    {
        StringBuffer buffer = new StringBuffer(  );
        HashMap<String, Object> attributes = new HashMap<String, Object>(  );

        attributes.put( TAG_ID, _nIdSeance );
        XmlUtil.beginElement( buffer, OdsMarks.MARK_SEANCE, attributes );

        getXmlContent( buffer );

        XmlUtil.endElement( buffer, OdsMarks.MARK_SEANCE );

        return buffer.toString(  );
    }

    /**
     * Gnration du contenu du flux XML
     * @param buffer le StringBuffer
     */
    public abstract void getXmlContent( StringBuffer buffer );

    /**
     * Indique si la sance passe en argument est la mme que la sance actuelle
     * @param seance l'objet  comparer
     * @return <b>true</b> si les sances sont gales, cad si les id des 2 sances sont gaux, <b>false</b> sinon
     */
    public boolean equals( Object seance )
    {
        if ( seance instanceof AbstractSeance )
        {
            return getIdSeance(  ) == ( (AbstractSeance) seance ).getIdSeance(  );
        }

        return false;
    }

    /**
     * Redfinition de la mthode hashcode( )
     * @return l'identifiant de la seance
     */
    public int hashCode(  )
    {
        return _nIdSeance;
    }

    /**
     * retourne le numro de la sance
     * @return le numro de la sance
     */
    public String getNumero(  )
    {
        return _strNumero;
    }

    /**
     * initialise le numro de la sance
     * @param numeroSeance le numro de la sance
     */
    public void setNumero( String numeroSeance )
    {
        _strNumero = numeroSeance;
    }

    /**
     * Retourne la taille de l'archive contenant tous les documents de sance
     * @return _tailleArchive la taille de l'archive contenant tous les documents de sance
     */
    public long getTailleArchive(  )
    {
        return _tailleArchive;
    }

    /**
     * Affecte la taille de l'archive contenant tous les documents de sance
     * @param tailleArchive taille de l'archive contenant tous les documents de sance
     */
    public void setTailleArchive( long tailleArchive )
    {
        this._tailleArchive = tailleArchive;
    }

    /**
     * Retourne la date de publication de la l'archive contenant tous les documents de sance
     * @return getDatePublicationArchive date de publication de la l'archive contenant tous les documents de sance
     */
    public java.sql.Timestamp getDatePublicationArchive(  )
    {
        return _datePublicationArchive;
    }

    /**
     * Affecte la date de publication de la l'archive contenant tous les documents de sance
     * @param datePublicationArchive date de publication de la l'archive contenant tous les documents de sance
     */
    public void setDatePublicationArchive( java.sql.Timestamp datePublicationArchive )
    {
        this._datePublicationArchive = datePublicationArchive;
    }
}
