/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.typeentite;

import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.util.xml.XmlUtil;

import java.util.HashMap;


/**
 * TypeEntite
 */
public class TypeEntite
{
    private static final String TAG_LIBELLE = "libelle";
    private static final String TAG_ID = "id";
    private static final String TAG_ENTITE = "TypeEntite";
    private int _nIdEntite = -1;
    private String _strLibelle;
    private boolean _bEstPourActe;

    /**
    * Retourne l'id de l'entite
    * @return l'id de l'entite
    */
    public int getIdEntite(  )
    {
        return _nIdEntite;
    }

    /**
     * Fixe l'id de l'entite
     * @param nIdEntite l'id de l'entite
     */
    public void setIdEntite( int nIdEntite )
    {
        _nIdEntite = nIdEntite;
    }

    /**
     * Retourne le libell de l'entite
     * @return le libell de l'entite
     */
    public String getLibelle(  )
    {
        return _strLibelle;
    }

    /**
     * Fixe le libell de l'entite
     * @param strLibelle le libell de l'entite
     */
    public void setLibelle( String strLibelle )
    {
        _strLibelle = strLibelle;
    }

    /**
     *
     * @return TRUE si c'est pour un acte, FALSE sinon
     */
    public boolean isEstPourActe(  )
    {
        return _bEstPourActe;
    }

    /**
     *
     * @param bEstPourActe TRUE si c'est pour un acte, FALSE sinon
     */
    public void setEstPourActe( boolean bEstPourActe )
    {
        _bEstPourActe = bEstPourActe;
    }

    /**
    * Gnration du XML d'un objet
    * @return Le code XML reprsentant une entite
    */
    public String getXml(  )
    {
        StringBuffer buffer = new StringBuffer(  );
        HashMap<String, Object> attributes = new HashMap<String, Object>(  );

        XmlUtil.beginElement( buffer, TAG_ENTITE, attributes );

        OdsUtils.addElement( buffer, TAG_ID, _nIdEntite );

        if ( _strLibelle != null )
        {
            OdsUtils.addElement( buffer, TAG_LIBELLE, _strLibelle );
        }

        XmlUtil.endElement( buffer, TAG_ENTITE );

        return buffer.toString(  );
    }
}
