/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.voeuamendement;

import fr.paris.lutece.plugins.ods.dto.IDepositaire;
import fr.paris.lutece.plugins.ods.dto.acte.Acte;
import fr.paris.lutece.plugins.ods.dto.acte.StatutActeEnum;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.historique.Historique;
import fr.paris.lutece.plugins.ods.dto.horodatage.IHorodatageFamilleDocument;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.statut.Statut;
import fr.paris.lutece.plugins.ods.dto.typeentite.ITypeEntite;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.portal.service.util.AppPropertiesService;
import fr.paris.lutece.util.xml.XmlUtil;

import java.sql.Timestamp;

import java.util.HashMap;
import java.util.List;


/**
 *
 * AbstractVoeuAmendement
 *
 * @param <GVoeuAmendement> IVoeuAmendement
 * @param <GSeance> ISeance
 * @param <GElu> IElu
 * @param <GFichier> IFichier
 * @param <GPDD> {@link IPDD}
 */
public abstract class AbstractVoeuAmendement
<
	GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, 
	GSeance extends ISeance, 
	GElu extends IElu, 
	GFichier extends IFichier<GSeance, GFichier>, 
	GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>
>
implements IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>
{
    private static final String PROPERTY_LIMIT_SIZE_AFFICHAGE = "ods.voeuxamendement.limiteSizeAffichage";
    private static final int DEFAULT_LIMIT_SIZE_AFFICHAGE = 100;
    private int _nId = -1;
    private FormationConseil _formationConseil;
    private String _strType;
    private String _strObjet;
    private String _strReference;
    private String _strReferenceFront;
    private Statut _statut;
    private boolean _bEnLigne;
    private Timestamp _timestampDatePublication;
    private int _nVersion;
    private boolean _bDeposeExecutif;
    private List<Historique> _historiques;
    private GFichier _fichier;
    private List<IDepositaire> _depositaires;
    private List<GPDD> _pdds;
    private Timestamp _timestampDateVote;
    private String _strNumeroDeliberation;
    private GFichier _deliberation;
    private List<GVoeuAmendement> _parents;
    private List<GVoeuAmendement> _enfants;
    private List<GElu> _groupeRapporteurs;
    private List<Acte<GSeance, GFichier>> _listActes;
    private Acte<GSeance, GFichier> _acteCourant;

    public abstract GSeance getSeance(  );

    public abstract void setSeance( GSeance seance );

    public abstract boolean equals( Object va );

    protected abstract void getReferenceCompleteSpec( StringBuffer result );

    protected abstract String getReferenceCompleteCertificatAffichageSpec(  );

    protected abstract void getXmlSpec( StringBuffer buffer );

    /**
     * retourne la rfrence complete du va
     * @return  retourne la rfrence complete du va
     */
    public String getReferenceComplete(  )
    {
        StringBuffer result = new StringBuffer(  );

        if ( OdsConstants.CONSTANTE_TYPE_VNR.equals( _strType ) )
        {
            result.append( OdsConstants.CONSTANTE_ENTREE_TYPE_VOEU );
        }
        else
        {
            result.append( _strType );
        }

        if ( _strReference != null )
        {
            result.append( _strReference );
        }

        // Appel au service spcifique
        getReferenceCompleteSpec( result );

        return result.toString(  );
    }

    /**
     * retourne la rfrence complete du va pour le certificat d'affichage :
     * affichage du numero du voeu adopt  la place de la reference
     * @return  retourne la rfrence complete du va pour le certificat d'affichage
     */

    /*public String getReferenceCompleteCertificatAffichage(  )
    {
        StringBuffer result = new StringBuffer(  );
    
        if ( OdsConstants.CONSTANTE_TYPE_VNR.equals( _strType ) )
        {
            result.append( OdsConstants.CONSTANTE_ENTREE_TYPE_VOEU );
        }
        else
        {
            result.append( _strType );
        }
    
        // Appel au service spcifique
        getReferenceCompleteSpec( result );
    
        if ( _strNumeroDeliberation != null )
        {
            result.append( _strNumeroDeliberation );
        }        return result.toString(  );
    }*/
    public String getReferenceCompleteCertificatAffichage(  )
    {
        return getReferenceCompleteCertificatAffichageSpec(  );
    }

    /**
     * retourne la rfrence complete du va  afficher cot front office
     * @return  retourne la rfrence complete du va  afficher cot front office
     */
    public String getReferenceCompleteFront(  )
    {
        StringBuffer result = new StringBuffer(  );

        if ( OdsConstants.CONSTANTE_TYPE_VNR.equals( _strType ) )
        {
            result.append( OdsConstants.CONSTANTE_ENTREE_TYPE_VOEU );
        }
        else
        {
            result.append( _strType );
        }

        if ( _strReferenceFront != null )
        {
            result.append( _strReferenceFront );
        }

        // Appel au service spcifique
        getReferenceCompleteSpec( result );

        return result.toString(  );
    }

    /**
     * retourne la formation conseil du va
     * @return la formation conseil du va
     */
    public FormationConseil getFormationConseil(  )
    {
        return _formationConseil;
    }

    /**
     * initialise la formationConseil du va
     * @param formationConseil la formationConseil du va
     */
    public void setFormationConseil( FormationConseil formationConseil )
    {
        _formationConseil = formationConseil;
    }

    /**
     * renvoie la liste des dpositaires du va
     * @return la liste des dpositaires du va
     */
    public List<IDepositaire> getDepositaires(  )
    {
        return _depositaires;
    }

    /**
     * initialise la liste des dpositaires du va
     * @param depositaires la liste des dpositaires du va
     */
    public void setDepositaires( List<IDepositaire> depositaires )
    {
        _depositaires = depositaires;
    }

    /**
     * renvoie la liste des  rapporteurs de l'entre d'ordre du jour rattache au va
     * @return la liste des rapporteurs du va
     */
    public List<GElu> getGroupeRapporteurs(  )
    {
        return _groupeRapporteurs;
    }

    /**
     * initialise la liste des  rapporteurs de l'entre d'ordre du jour rattache au va
     * @param elus la liste des rapporteurs du va
     */
    public void setGroupeRapporteurs( List<GElu> elus )
    {
        _groupeRapporteurs = elus;
    }

    /**
     * retourne true si le va est en ligne false sinon
     * @return  true si le va est en ligne false sinon
     */
    public Boolean getEnLigne(  )
    {
        return _bEnLigne;
    }

    /**
     * initialise le statut en ligne a true ou false
     * @param enLigne le statut du va
     */
    public void setEnLigne( Boolean enLigne )
    {
        _bEnLigne = enLigne;
    }

    /**
     * retourne la date de publication du va
     * @return date de publication du va
     */
    public Timestamp getDatePublication(  )
    {
        return _timestampDatePublication;
    }

    /**
     * initialise la date de publication du va
     * @param datePublication date de publication du va
     */
    public void setDatePublication( Timestamp datePublication )
    {
        _timestampDatePublication = datePublication;
    }

    /**
     * retourne la liste d'historiques du va
     * @return liste d'historiques du va
     */
    public List<Historique> getHistoriques(  )
    {
        return _historiques;
    }

    /**
     * initialise la liste d'historiques du va
     * @param historiques la liste d'historiques du va
     */
    public void setHistoriques( List<Historique> historiques )
    {
        _historiques = historiques;
    }

    /**
     * retourne true si le va a t dpos par l'excutif,false sinon
     * @return  true si le va a t dpos par l'excutif,false sinon
     */
    public boolean getDeposeExecutif(  )
    {
        return _bDeposeExecutif;
    }

    /**
     *
     * initialise le statut deposeExecutif du va  true si le va est dpos par l'excutif, false sinon
     * @param deposeExecutif true si le va est dpos par l'excutif, false sinon
     */
    public void setDeposeExecutif( boolean deposeExecutif )
    {
        _bDeposeExecutif = deposeExecutif;
    }

    /**
     * retourne l'id du va
     * @return  l'id du va
     */
    public int getId(  )
    {
        return _nId;
    }

    /**
     * initialise l'id  du va
     * @param nId id  du va
     */
    public void setId( int nId )
    {
        _nId = nId;
    }

    /**
     * retourne la version du va
     * @return la version du va
     */
    public int getVersion(  )
    {
        return _nVersion;
    }

    /**
     * initialise la version du va
     * @param version la version du va
     */
    public void setVersion( int version )
    {
        _nVersion = version;
    }

    /**
     *
     * @return getObjetLimit
     */
    public String getObjetLimit(  )
    {
        int nLimitSize = AppPropertiesService.getPropertyInt( PROPERTY_LIMIT_SIZE_AFFICHAGE,
                DEFAULT_LIMIT_SIZE_AFFICHAGE );

        // objet du vu ou de lamendement tronqu au dernier espace avant le n ime caractre 
        if ( ( _strObjet != null ) && ( _strObjet.length(  ) > ( nLimitSize - 1 ) ) )
        {
            StringBuffer strObjet = new StringBuffer(  );
            int nLastEspace = _strObjet.lastIndexOf( " ", ( nLimitSize - 1 ) ) + 1;
            strObjet.append( _strObjet.substring( 0, nLastEspace ) ).append( "..." );

            return strObjet.toString(  );
        }

        return _strObjet;
    }

    /**
     *retourne l'objet du va
     * @return  l'objet du va
     */
    public String getObjet(  )
    {
        return _strObjet;
    }

    /**
     * initialise l'objet du va
     * @param objet l'objet du va
     */
    public void setObjet( String objet )
    {
        _strObjet = objet;
    }

    /**
     * retourne la reference du va
     * @return  la reference du va
     */
    public String getReference(  )
    {
        return _strReference;
    }

    /**
     * initialise la reference du va
     * @param reference la reference du va
     */
    public void setReference( String reference )
    {
        _strReference = reference;
    }

    /**
     * retourne la reference du va visible cot front office
     * @return  la reference du va visible cot front office
     */
    public String getReferenceFront(  )
    {
        return _strReferenceFront;
    }

    /**
     * initialise la reference du va visible cot front office
     * @param reference la reference du va visible cot front office
     */
    public void setReferenceFront( String reference )
    {
        _strReferenceFront = reference;
    }

    /**
     * retourne le statut du va
     * @return  le statut du va
     */
    public Statut getStatut(  )
    {
        return _statut;
    }

    /**
     * initialise le statut du va
     * @param statut le statut du va
     */
    public void setStatut( Statut statut )
    {
        _statut = statut;
    }

    /**
     * retourne le type du va(Amendement=A,Lettre rectificative=LR,Voeu rattache=V,Voeu non rattache=VNR)
     * @return  le type du va(Amendement=A,Lettre rectificative=LR,Voeu rattache=V,Voeu non rattache=VNR)
     */
    public String getType(  )
    {
        return _strType;
    }

    /**
     * initialise le type du va (Amendement=A,Lettre rectificative=LR,Voeu rattache=V,Voeu non rattache=VNR)
     * @param type le type du va(Amendement=A,Lettre rectificative=LR,Voeu rattache=V,Voeu non rattache=VNR)
     */
    public void setType( String type )
    {
        _strType = type;
    }

    /**
     * le fichier associ au va
     * @return le fichier associ au va
     */
    public GFichier getFichier(  )
    {
        return _fichier;
    }

    /**
     * initialise le fichier du va
     * @param fichier le fichier du va
     */
    public void setFichier( GFichier fichier )
    {
        _fichier = fichier;
    }

    /**
     * retourne  la liste des pdds du va
     * @return  la liste des pdds du va
     */
    public List<GPDD> getPdds(  )
    {
        return _pdds;
    }

    /**
     * initialise la liste des pdds du va
     * @param ds la liste des pdds du va
     */
    public void setPdds( List<GPDD> ds )
    {
        _pdds = ds;
    }

    /**
     * retourne la dlibration du VA.
     * @return la dlibration
     */
    public GFichier getDeliberation(  )
    {
        return _deliberation;
    }

    /**
     * initialise la dlibration du VA.
     * @param deliberation la dlibratiion
     */
    public void setDeliberation( GFichier deliberation )
    {
        this._deliberation = deliberation;
    }

    /**
     * retourne le numro de la dlibration.
     * @return le numro de la dlibration
     */
    public String getNumeroDeliberation(  )
    {
        return _strNumeroDeliberation;
    }

    /**
          *
          * @return le nombre d'acte non annuls associs  cette liste de membres
          */
    public int getNombreActes(  )
    {
        int resultat = 0;

        if ( _listActes != null )
        {
            for ( Acte<GSeance, GFichier> acte : _listActes )
            {
                if ( ( acte.getStatutActe(  ).getIdStatut(  ) != StatutActeEnum.ANNULE.getId(  ) ) &&
                        ( acte.getStatutActe(  ).getIdStatut(  ) != StatutActeEnum.ERREUR_TRANSMISSION.getId(  ) ) )
                {
                    resultat += 1;
                }
            }
        }

        return resultat;
    }

    /**
     * initialise le numro de la dlibration.
     * @param numeroDeliberation le numro de la dlibration
     */
    public void setNumeroDeliberation( String numeroDeliberation )
    {
        _strNumeroDeliberation = numeroDeliberation;
    }

    /**
     * retourne la date du vote.
     * @return la date du vote
     */
    public Timestamp getDateVote(  )
    {
        return _timestampDateVote;
    }

    /**
     * initialise la date du vote.
     * @param dateVote la date du vote
     */
    public void setDateVote( Timestamp dateVote )
    {
        _timestampDateVote = dateVote;
    }

    /**
     * calcule le hashcode de l'objet
     * @return le hashcode
     */
    public int hashCode(  )
    {
        return getId(  );
    }

    /**
     * Retourne la liste des parents du voeu/amendement
     * @return la liste des parents
     */
    public List<GVoeuAmendement> getParents(  )
    {
        return _parents;
    }

    /**
     * intialise la liste des parents du voeu/amendement
     * @param listParent la liste des parents du voeu/amendement
     */
    public void setParents( List<GVoeuAmendement> listParent )
    {
        _parents = listParent;
    }

    /**
     * Retourne la liste des enfants du voeu/amendement
     * @return la liste des enfants
     */
    public List<GVoeuAmendement> getEnfants(  )
    {
        return _enfants;
    }

    /**
     * intialise la liste des enfants du voeu/amendement
     * @param listEnfants la liste des enfants du voeu/amendement
     */
    public void setEnfants( List<GVoeuAmendement> listEnfants )
    {
        _enfants = listEnfants;
    }

    /**
    *
    * @return la liste des actes associs  cette liste de membres
    */
    public List<Acte<GSeance, GFichier>> getActes(  )
    {
        return _listActes;
    }

    /**
    * set la liste des actes associs  ce VoeuAmendement
    * @param listActes la liste des actes associs  ce VoeuAmendement
    */
    public void setActes( List<Acte<GSeance, GFichier>> listActes )
    {
        this._listActes = listActes;
    }

    /**
    * Retourne l'acte courant
    * @return Acte _acteCourant
    */
    public Acte<GSeance, GFichier> getActeCourant(  )
    {
        return _acteCourant;
    }

    /**
    * Affecte l'objet Acte acteCourant
    * @param acteCourant l'acte
    */
    public void setActeCourant( Acte<GSeance, GFichier> acteCourant )
    {
        this._acteCourant = acteCourant;
    }

    /**
     * Test si la liste des lus passe en parametre est egale au groupe de rapporteurs du va
     *
     *@param listElus
     *            liste de rapporteurs
     * @return true Test si la liste des lus passe en parametre est egale
     * au groupe de rapporteurs du va
     */
    public boolean getEqualListRapporteurs( List<GElu> listElus )
    {
        if ( _groupeRapporteurs.size(  ) != listElus.size(  ) )
        {
            return false;
        }

        for ( int cpt = 0; cpt < _groupeRapporteurs.size(  ); cpt++ )
        {
            if ( _groupeRapporteurs.get( cpt ).getIdElu(  ) != listElus.get( cpt ).getIdElu(  ) )
            {
                return false;
            }
        }

        return true;
    }

    /**
     * renvoie le code XML correspondant  l'objet
     * @return le code XMl
     */
    @SuppressWarnings( "unchecked" )
    public String getXml(  )
    {
        StringBuffer buffer = new StringBuffer(  );
        HashMap<String, Object> attributes = new HashMap<String, Object>(  );
        attributes.put( TAG_ID_VOEU_AMENDEMENT, OdsConstants.CONSTANTE_CHAINE_VIDE + _nId );
        XmlUtil.beginElement( buffer, TAG_VOEU_AMENDEMENT, attributes );

        OdsUtils.addElement( buffer, TAG_TYPE, _strType );
        OdsUtils.addElement( buffer, TAG_OBJET, _strObjet );
        OdsUtils.addElement( buffer, OdsParameters.REFERENCE, _strReference );
        OdsUtils.addElement( buffer, TAG_REFERENCE_COMPLETE, getReferenceComplete(  ) );
        OdsUtils.addElement( buffer, TAG_REFERENCE_COMPLETE_CERTIFICAT, getReferenceCompleteCertificatAffichage(  ) );
        OdsUtils.addElement( buffer, TAG_DATE_PUBLICATION,
            OdsConstants.CONSTANTE_CHAINE_VIDE + _timestampDatePublication );
        OdsUtils.addElement( buffer, TAG_VERSION, _nVersion );
        OdsUtils.addElement( buffer, TAG_DEPOSE_EXECUTIF, OdsConstants.CONSTANTE_CHAINE_VIDE + _bDeposeExecutif );

        OdsUtils.addElement( buffer, TAG_DATE_VOTE, OdsConstants.CONSTANTE_CHAINE_VIDE + _timestampDateVote );

        if ( _formationConseil != null )
        {
            buffer.append( _formationConseil.getXml(  ) );
        }

        // Appel au service spcifique
        getXmlSpec( buffer );

        if ( _fichier != null )
        {
            buffer.append( _fichier.getXml(  ) );
        }

        XmlUtil.beginElement( buffer, TAG_LISTE_ELUS );

        if ( _depositaires != null )
        {
            for ( IDepositaire depo : _depositaires )
            {
                if ( depo.getType(  ) == IDepositaire.DEPOSITAIRE_TYPE_ELU )
                {
                    buffer.append( ( (GElu) depo ).getXml(  ) );
                }
            }
        }

        XmlUtil.endElement( buffer, TAG_LISTE_ELUS );

        if ( _deliberation != null )
        {
            buffer.append( _deliberation.getXml(  ) );
        }

        if ( _statut != null )
        {
            buffer.append( _statut.getXml(  ) );
        }

        XmlUtil.endElement( buffer, TAG_VOEU_AMENDEMENT );

        return buffer.toString(  );
    }

    public String getXmlForCertificat( int nIdTypeEntite )
    {
        return getXml(  );
    }

    public String getIntitule(  )
    {
        return null;
    }

    public int getTypeEntite(  )
    {
        return ITypeEntite.VA;
    }

    public String getFamilleDocumentEnum(  )
    {
        return IHorodatageFamilleDocument.VOEU_AMENDEMENT;
    }

    public String getTypeEntree(  )
    {
        return OdsConstants.CONSTANTE_ENTREE_TYPE_VOEU;
    }
}
