/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.voeuamendement;

import java.sql.Timestamp;

import java.util.ArrayList;
import java.util.List;


/**
 * Filtre sur les voeux et les amendements
 */
public abstract class AbstractVoeuAmendementFilter implements IVoeuAmendementFilter
{
    public static final String ALL_STRING = "all";
    public static final int ALL_INT = -1;
    public static final String ORDER_REFERENCE_VA = "va.reference_va";
    public static final String ORDER_DATE_PUBLICATION = "va.date_publication";
    private static final int A_PREPARER = 1;
    private static final int UN_ACTE_AU_MOINS = 2;
    private int _nIdSeance = ALL_INT;
    private int _nIdPublie = ALL_INT;
    private String _strType = ALL_STRING;
    private int _nIdParent = ALL_INT;
    private int _nIdParentNotIn = ALL_INT;
    private int _nIdStatut = ALL_INT;
    private int _nStatutActe = ALL_INT;
    private List<String> _orderByList = new ArrayList<String>(  );
    private int _nTypeOrdreDuJour = ALL_INT;
    private int _nIdOrdreDuJour = ALL_INT;
    private Timestamp _tsDatePublication;
    private Timestamp _tsDateSeance;
    private int _nIdFormationConseil = ALL_INT;

    /**
     * @return _tsDatePublication Timestamp
     */
    public Timestamp getDatePublication(  )
    {
        return _tsDatePublication;
    }

    /**
     * @param tsDatePubliocation _tsDatePublication to set
     */
    public void setDatePublication( Timestamp tsDatePubliocation )
    {
        _tsDatePublication = tsDatePubliocation;
    }

    /**
     * @return boolean TRUE si _tsDatePublication est actif, FALSE sinon
     */
    public boolean containsDatePublicationCriteria(  )
    {
        return ( _tsDatePublication != null );
    }

    /**
     * retourne le type du va(Amendement=A,Lettre rectificative=LR,Voeu rattache=V,Voeu non rattache=VNR)
     * @return type du va(Amendement=A,Lettre rectificative=LR,Voeu rattache=V,Voeu non rattache=VNR)
     */
    public String getType(  )
    {
        return _strType;
    }

    /**
     *initialise le type du va (Amendement=A,Lettre rectificative=LR,Voeu rattache=V,Voeu non rattache=VNR)  filtrer
     * @param strType le type du va (Amendement=A,Lettre rectificative=LR,Voeu rattache=V,Voeu non rattache=VNR)
     */
    public void setType( String strType )
    {
        _strType = strType;
    }

    /**
         * retourne true si le filtre contient le critere Type false sinon
         * @return retourne si le filtre contient le critere Type false sinon
         */
    public boolean containsTypeCriteria(  )
    {
        return ( !_strType.equals( ALL_STRING ) );
    }

    /**
     * retourne l'id publie prsent dans le filtre (0 ou 1)
     * @return l'id publie prsent dans le filtre (0 ou 1)
     */
    public int getIdPublie(  )
    {
        return _nIdPublie;
    }

    /**
     * initialise l'id publie  filtrer (0 ou 1)
     * @param nIdPublie l'id publie   filtrer (0 ou 1)
     */
    public void setIdPublie( int nIdPublie )
    {
        _nIdPublie = nIdPublie;
    }

    /**
         * retourne true si le filtre contient l'id publie false sinon
         * @return retourne true si le filtre contient l'id publie false sinon
         */
    public boolean containsIdPublieCriteria(  )
    {
        return ( _nIdPublie != ALL_INT );
    }

    /**
     * retourne  l'id de la seance prsent dans le filtre
     * @return l'id de la seance prsent dans le filtre
     */
    public int getIdSeance(  )
    {
        return _nIdSeance;
    }

    /**
     * initialise l'id de la seance  filtrer
     * @param nIdSeance l'id de la seance   filtrer
     */
    public void setIdSeance( int nIdSeance )
    {
        _nIdSeance = nIdSeance;
    }

    /**
         * retourne true si le filtre contient l'id de la seance false sinon
         * @return retourne true si le filtre contient l'id de la seance false sinon
         */
    public boolean containsIdSeanceCriteria(  )
    {
        return ( _nIdSeance != ALL_INT );
    }

    /**
    * retourne la liste des criteres de tries
    * @return retourne la liste des criteres de tries
    */
    public List<String> getOrderByList(  )
    {
        return _orderByList;
    }

    /**
     * initialise la liste des criteres de tries
     * @param orderByList  liste des criteres de tries
     */
    public void setOrderByList( List<String> orderByList )
    {
        _orderByList = orderByList;
    }

    /**
     * ajoute un critere de trie
     * @param strOrder critere de trie
     */
    public void addInOrderBy( String strOrder )
    {
        _orderByList.add( strOrder );
    }

    /**
         * retourne true si le filtre contient des criters de tries sinon false
         * @return retourne true si le filtre contient des criters de tries sinon false
         */
    public boolean containsOrderByCriteria(  )
    {
        return ( !_orderByList.isEmpty(  ) );
    }

    /**
     * retourne l'id du parent  filtrer
     * @return l'id du parent  filtrer
     */
    public int getIdParent(  )
    {
        return _nIdParent;
    }

    /**
     * fixe l'id du parent  filtrer
     * @param nIdParent l'id du parent  filtrer
     */
    public void setIdParent( int nIdParent )
    {
        _nIdParent = nIdParent;
    }

    /**
     * indique si le filtre sur le parent est actif
     * @return <b>true</b> si le filtre sur parent est activ, <b>false</b> sinon
     */
    public boolean containsIdParentCriteria(  )
    {
        return ( _nIdParent != ALL_INT );
    }

    /**
     * retourne l'id du parent  exclure
     * @return l'id du parent   exclure
     */
    public int getIdParentNotIn(  )
    {
        return _nIdParentNotIn;
    }

    /**
     * fixe l'id du parent   exclure
     * @param nIdParent l'id du parent   exclure
     */
    public void setIdParentNotIn( int nIdParent )
    {
        _nIdParentNotIn = nIdParent;
    }

    /**
     * indique si le filtre sur le parent not in  est actif
     * @return <b>true</b> si le filtre sur parent not in est activ, <b>false</b> sinon
     */
    public boolean containsIdParentNotInCriteria(  )
    {
        return ( _nIdParentNotIn != ALL_INT );
    }

    /**
     * retourne l'id du statut a filtrer
     * @return l'id du statut a filtrer
     */
    public int getIdStatut(  )
    {
        return _nIdStatut;
    }

    /**
     * fixe l'id du statut a filtrer
     * @param nIdStatut l'id du statut a filtrer
     */
    public void setIdStatut( int nIdStatut )
    {
        _nIdStatut = nIdStatut;
    }

    /**
     * indique si le filtre sur le statut est actif
     * @return <b>true</b> si le filtre sur statut est activ, <b>false</b> sinon
     */
    public boolean containsIdStatutCriteria(  )
    {
        return ( _nIdStatut != ALL_INT );
    }

    /**
     * @return _nTypeOrdreDuJour
     */
    public int getTypeOrdreDuJour(  )
    {
        return _nTypeOrdreDuJour;
    }

    /**
     * @param typeOrdreDuJour _nTypeOrdreDuJour to set
     */
    public void setTypeOrdreDuJour( int typeOrdreDuJour )
    {
        _nTypeOrdreDuJour = typeOrdreDuJour;
    }

    /**
     * @return boolean TRUE si _nTypeOrdreDuJour est actif, FALSE sinon
     */
    public boolean containsTypeOrdreDuJourCriteria(  )
    {
        return ( _nTypeOrdreDuJour != ALL_INT );
    }

    /**
     * Retourne l'id d'ODJ  filtrer
     * @return l'id d'ODJ  filtrer
     */
    public int getIdOrdreDuJour(  )
    {
        return _nIdOrdreDuJour;
    }

    /**
     * Fixe l'id d'ODJ  filtrer
     * @param nIdOrdreDuJour l'id d'ODJ  filtrer
     */
    public void setIdOrdreDuJour( int nIdOrdreDuJour )
    {
        _nIdOrdreDuJour = nIdOrdreDuJour;
    }

    /**
     * Indique si l'id d'ODJ est fix
     * @return <b>true</b> si l'id d'ODJ est fix, <b>false</b> sinon
     */
    public boolean containsIdOrdreDuJourCriteria(  )
    {
        return ( _nIdOrdreDuJour != ALL_INT );
    }

    /**
     * Fixe le filtre sur le statut de l'acte
     * @param nStatutActe le statut de l'acte
     */
    public void setStatutActe( int nStatutActe )
    {
        _nStatutActe = nStatutActe;
    }

    /**
    * Retourne le statut de l'acte  filtrer
    * @return le statut de l'acte  filtrer
    */
    public int getStatutActe(  )
    {
        return _nStatutActe;
    }

    /**
     * Indique si le filtre sur l'acte est positionn
     * @return <b>true</b> si le filtre sur l'acte est positionn, <b>false</b> sinon
     */
    public boolean containsStatutActeCriteria(  )
    {
        return ( _nStatutActe != ALL_INT );
    }

    /**
     * retourne l'id de la formation conseil prsent dans le filtre
     * @return id de la formation conseil prsent dans le filtre
     */
    public int getIdFormationConseil(  )
    {
        return _nIdFormationConseil;
    }

    /**
     *  initialise l'id de la formation conseil  filtrer
     * @param nIdFormationConseil id de la formation conseil prsent dans le filtre
     */
    public void setIdFormationConseil( int nIdFormationConseil )
    {
        _nIdFormationConseil = nIdFormationConseil;
    }

    /**
     * retourne true si le filtre contient l'id de la formation conseil  false sinon
     * @return retourne true si le filtre contient l'id de la formation conseil  false sinon
     */
    public boolean containsIdFormationConseilCriteria(  )
    {
        return ( _nIdFormationConseil != ALL_INT );
    }

    /**
     *
     * @return
     */
    public int getAllInt(  )
    {
        return ALL_INT;
    }

    /**
     *
     * @return
     */
    public String getAllString(  )
    {
        return ALL_STRING;
    }

    /**
     *
     * @return
     */
    public String getOrderDatePublication(  )
    {
        return ORDER_DATE_PUBLICATION;
    }

    /**
     *
     * @return
     */
    public String getOrderReferenceVa(  )
    {
        return ORDER_REFERENCE_VA;
    }

    /**
     *
     * @return
     */
    public int getAPreparer(  )
    {
        return A_PREPARER;
    }

    /**
     *
     * @return
     */
    public int getUnActeAuMoins(  )
    {
        return UN_ACTE_AU_MOINS;
    }

    /**
     * Retourne la date seance prsent dans le filtre
     * @return la date seance prsent dans le filtre
     */
    public Timestamp getDateSeance(  )
    {
        return _tsDateSeance;
    }

    /**
     * Fixe l'id la date de seance  filtrer
     * @param dateSeance la date de seance  filtrer
     */
    public void setDateSeance( Timestamp dateSeance )
    {
        _tsDateSeance = dateSeance;
    }

    /**
    *
    * @return
    */
    public boolean containsDateSeanceCriteria(  )
    {
        return ( _tsDateSeance != null );
    }
}
