/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.voeuamendement;

import fr.paris.lutece.plugins.ods.dto.IBusinessItem;
import fr.paris.lutece.plugins.ods.dto.IDepositaire;
import fr.paris.lutece.plugins.ods.dto.acte.Acte;
import fr.paris.lutece.plugins.ods.dto.acte.IEntiteActe;
import fr.paris.lutece.plugins.ods.dto.certificataffichage.IEntiteCertificatAffichage;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.historique.Historique;
import fr.paris.lutece.plugins.ods.dto.historique.IEntiteHistorique;
import fr.paris.lutece.plugins.ods.dto.horodatage.IEntiteHorodatage;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IEntiteEntreeOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.statut.Statut;

import java.sql.Timestamp;

import java.util.List;


/**
 * 
 * IVoeuAmendement
 *
 * @param <GVoeuAmendement> {@link IVoeuAmendement}
 * @param <GSeance> {@link ISeance}
 * @param <GElu> {@link IElu}
 * @param <GFichier> {@link IFichier}
 * @param <GPDD> {@link IPDD}
 */
public interface IVoeuAmendement
<
	GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, 
	GSeance extends ISeance, 
	GElu extends IElu, 
	GFichier extends IFichier<GSeance, GFichier>, 
	GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>
>
extends IBusinessItem, IEntiteActe, IEntiteHorodatage, IEntiteCertificatAffichage, IEntiteHistorique,
        IEntiteEntreeOrdreDuJour
{
    String TAG_LISTE_ELUS = "listeElus";
    String TAG_DATE_VOTE = "dateVote";
    String TAG_DEPOSE_EXECUTIF = "deposeExecutif";
    String TAG_VERSION = "version";
    String TAG_DATE_PUBLICATION = "datePublication";
    String TAG_REFERENCE_COMPLETE = "referenceComplete";
    String TAG_REFERENCE_COMPLETE_CERTIFICAT = "referenceCompleteCertificat";
    String TAG_OBJET = "objet";
    String TAG_TYPE = "type";
    String TAG_VOEU_AMENDEMENT = "voeuAmendement";
    String TAG_ID_VOEU_AMENDEMENT = "id";

    int getId(  );

    void setId( int nId );

    FormationConseil getFormationConseil(  );

    void setFormationConseil( FormationConseil formationConseil );

    String getType(  );

    void setType( String type );

    String getObjet(  );

    void setObjet( String objet );

    String getReference(  );

    void setReference( String reference );

    String getReferenceFront(  );

    void setReferenceFront( String reference );

    Statut getStatut(  );

    void setStatut( Statut statut );

    Boolean getEnLigne(  );

    void setEnLigne( Boolean enLigne );

    Timestamp getDatePublication(  );

    void setDatePublication( Timestamp datePublication );

    int getVersion(  );

    void setVersion( int version );

    boolean getDeposeExecutif(  );

    void setDeposeExecutif( boolean deposeExecutif );

    List<Historique> getHistoriques(  );

    void setHistoriques( List<Historique> historiques );

    GFichier getFichier(  );

    void setFichier( GFichier fichier );

    List<IDepositaire> getDepositaires(  );

    void setDepositaires( List<IDepositaire> depositaires );

    List<GPDD> getPdds(  );

    void setPdds( List<GPDD> ds );

    Timestamp getDateVote(  );

    void setDateVote( Timestamp dateVote );

    String getNumeroDeliberation(  );

    void setNumeroDeliberation( String numeroDeliberation );

    GFichier getDeliberation(  );

    void setDeliberation( GFichier deliberation );

    List<GVoeuAmendement> getParents(  );

    void setParents( List<GVoeuAmendement> listParent );

    List<GVoeuAmendement> getEnfants(  );

    void setEnfants( List<GVoeuAmendement> listEnfants );

    List<GElu> getGroupeRapporteurs(  );

    void setGroupeRapporteurs( List<GElu> elus );

    List<Acte<GSeance, GFichier>> getActes(  );

    void setActes( List<Acte<GSeance, GFichier>> listActes );

    Acte<GSeance, GFichier> getActeCourant(  );

    void setActeCourant( Acte<GSeance, GFichier> acteCourant );

    String getReferenceComplete(  );

    String getReferenceCompleteCertificatAffichage(  );

    String getReferenceCompleteFront(  );

    boolean getEqualListRapporteurs( List<GElu> listElus );

    GSeance getSeance(  );

    void setSeance( GSeance seance );
}
