/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.dto.voeuamendement;

import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.utils.commons.StringUtils;

import java.io.Serializable;

import java.util.Comparator;


/**
 * 
 * VoeuAmendementComparator
 * Reprsente un comparateur de voeu ou amendement
 * Cette classe permet de classer une liste de VA par leur numro
 *
 * @param <GVoeuAmendement> {@link IVoeuAmendement}
 * @param <GSeance> {@link ISeance}
 * @param <GElu> {@link IElu}
 * @param <GFichier> {@link IFichier}
 * @param <GPDD> {@link IPDD}
 */
public class VoeuAmendementCertificatComparator
<
	GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, 
	GSeance extends ISeance, 
	GElu extends IElu, 
	GFichier extends IFichier<GSeance, GFichier>, 
	GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>
>
implements Comparator<GVoeuAmendement>, Serializable
{
    private static final long serialVersionUID = 4567334590516828409L;

    /**
     * methode qui compare la reference de deux vas pour le certificat d'affichage.
     * @param va1 voeuAmendement
     * @param va2 voeuAmendement
     * @return -1 si la reference de va1 est avant la reference de va2 dans l'ordre alphabetique
     *                         0 si egale
     *                         1 si apres
     */
    public int compare( GVoeuAmendement va1, GVoeuAmendement va2 )
    {
        return StringUtils.compare( va1.getReferenceCompleteCertificatAffichage(  ),
            va2.getReferenceCompleteCertificatAffichage(  ) );
    }
}
