/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.acte;

import fr.paris.lutece.plugins.actes.modules.fast.service.FastService;
import fr.paris.lutece.plugins.ods.business.IBusinessHome;
import fr.paris.lutece.plugins.ods.business.acte.ActeHome;
import fr.paris.lutece.plugins.ods.business.acte.DocumentActeHome;
import fr.paris.lutece.plugins.ods.business.acte.NatureActeHome;
import fr.paris.lutece.plugins.ods.business.acte.StatutActeHome;
import fr.paris.lutece.plugins.ods.business.categoriectrllegalite.CategorieCtrlLegaliteHome;
import fr.paris.lutece.plugins.ods.business.categoriedeliberation.CategorieDeliberationHome;
import fr.paris.lutece.plugins.ods.business.fichier.FichierPhysiqueHome;
import fr.paris.lutece.plugins.ods.business.fichier.IFichierHome;
import fr.paris.lutece.plugins.ods.business.formationconseil.FormationConseilHome;
import fr.paris.lutece.plugins.ods.business.historique.HistoriqueHome;
import fr.paris.lutece.plugins.ods.business.pdd.IPDDHome;
import fr.paris.lutece.plugins.ods.business.seance.ISeanceHome;
import fr.paris.lutece.plugins.ods.business.typeentite.TypeEntiteHome;
import fr.paris.lutece.plugins.ods.dto.IBusinessItem;
import fr.paris.lutece.plugins.ods.dto.acte.Acte;
import fr.paris.lutece.plugins.ods.dto.acte.ActeComparator;
import fr.paris.lutece.plugins.ods.dto.acte.DocumentActe;
import fr.paris.lutece.plugins.ods.dto.acte.IActeFilter;
import fr.paris.lutece.plugins.ods.dto.acte.IEntiteActe;
import fr.paris.lutece.plugins.ods.dto.acte.NatureActe;
import fr.paris.lutece.plugins.ods.dto.acte.NatureActeEnum;
import fr.paris.lutece.plugins.ods.dto.acte.StatutActe;
import fr.paris.lutece.plugins.ods.dto.acte.StatutActeEnum;
import fr.paris.lutece.plugins.ods.dto.categoriectrllegalite.CategorieCtrlLegalite;
import fr.paris.lutece.plugins.ods.dto.categoriedeliberation.CategorieDeliberation;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.FichierPhysique;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter;
import fr.paris.lutece.plugins.ods.dto.fichier.ITypeDocument;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.historique.Historique;
import fr.paris.lutece.plugins.ods.dto.pdd.AbstractPDDFilter;
import fr.paris.lutece.plugins.ods.dto.pdd.IArrondissement;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDDFilter;
import fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur;
import fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.typeentite.ITypeEntite;
import fr.paris.lutece.plugins.ods.dto.typeentite.TypeEntite;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendementFilter;
import fr.paris.lutece.plugins.ods.service.categoriedeliberation.ICategorieDeliberationService;
import fr.paris.lutece.plugins.ods.service.fichier.IFichierService;
import fr.paris.lutece.plugins.ods.service.formationconseil.IFormationConseilService;
import fr.paris.lutece.plugins.ods.service.pdd.IProjetDeDeliberationService;
import fr.paris.lutece.plugins.ods.service.search.ISearchService;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.commons.DateUtils;
import fr.paris.lutece.plugins.ods.utils.commons.StringUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.service.util.AppPathService;
import fr.paris.lutece.portal.service.util.AppPropertiesService;
import fr.paris.lutece.portal.web.upload.MultipartHttpServletRequest;
import fr.paris.lutece.util.ReferenceList;
import fr.paris.lutece.util.html.Paginator;
import fr.paris.lutece.util.url.UrlItem;

import org.apache.commons.fileupload.FileItem;

import org.springframework.beans.factory.annotation.Autowired;

import java.io.IOException;

import java.sql.Timestamp;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;


/**
 *
 * AbstractActeService
 *
 * @param <GSeanceHome> ISeanceHome
 * @param <GSeance> ISeance
 * @param <GSeanceFilter> ISeanceFilter
 * @param <GActeFilter> IActeFilter
 */
public abstract class AbstractActeService<GSeanceHome extends ISeanceHome<GSeance, GSeanceFilter, GFichier, GElu, GVoeuAmendement, GPDD>, GSeance extends ISeance, GSeanceFilter extends ISeanceFilter, GActeFilter extends IActeFilter<GFichier, GSeance>, GFichier extends IFichier<GSeance, GFichier>, GElu extends IElu, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GPDDFilter extends IPDDFilter, GVoeuAmendementFilter extends IVoeuAmendementFilter, GArrondissement extends IArrondissement, GRequeteUtilisateur extends IRequeteUtilisateur<GSeance, GSeanceFilter>, GResultatRequete extends IResultatRequete<GSeance, GSeanceFilter, GRequeteUtilisateur, GElu, GFichier, GVoeuAmendement, GPDD>>
    implements IActeService<GSeance, GFichier, GActeFilter>
{
    protected static final String JSP_URL_GESTION_ACTE = "jsp/admin/plugins/ods/acte/ActeGestionAval.jsp";
    protected static final String PARAMETER_STATUT = "statut";
    protected static final String PARAMETER_ENTITE = "entite";
    protected static final String PARAMETER_FORMATION_CONSEIL = "formation_conseil";
    protected static final String PARAMETER_ORDER_BY = "order_hidden";
    protected static final String PARAM_CHECKBOX_DELETE = "checkbox_delete";
    protected static final String ATTRIBUT_SEANCE = "attribut_seance";
    protected static final String ATTRIBUT_FORMATION_CONSEIL = "attribut_formation_conseil";
    protected static final String ATTRIBUT_STATUT = "attribut_statut";
    protected static final String ATTRIBUT_ENTITE = "attribut_entite";
    protected static final String ATTRIBUT_ORDER_BY = "attribut_orderby";
    protected static final String MARK_ACTE = "acte";
    protected static final String MARK_LISTE_ACTES = "liste_acte";
    protected static final String MARK_LISTE_SEANCES = "liste_seances";
    protected static final String MARK_LISTE_FORMATION_CONSEIL = "liste_formation_conseil";
    protected static final String MARK_ID_FORMATION_CONSEIL_SELECTED = "id_formation_conseil_selected";
    protected static final String MARK_URL = "url";
    protected static final String MARK_LISTE_PIECES_ANNEXES = "liste_pieces_annexes";
    protected static final String MARK_EXPO_MOTIF_CHECK = "expo_motif_check";
    protected static final String MARK_PROJET_DELIBERE_CHECK = "projet_delibere_check";
    protected static final String MARK_ANNEXES_CHECKED = "list_annexes_check";
    protected static final String STRING_ODS = "ODS";
    protected static final String STRING_ZERO = "0";
    private static final String JSP_URL_DO_SUPPRESSION_ACTE = "jsp/admin/plugins/ods/acte/DoSuppressionActe.jsp";
    private static final String JSP_URL_DO_REJETER_ACTE = "jsp/admin/plugins/ods/acte/DoRejeterActe.jsp";
    private static final String JSP_URL_FINALISER = "jsp/admin/plugins/ods/acte/FinalisationActe.jsp";
    private static final String JSP_URL_MODIFICATION_ACTE = "jsp/admin/plugins/ods/acte/ModificationActe.jsp";
    private static final String JSP_URL_DO_ANNULER_ACTE = "jsp/admin/plugins/ods/acte/DoAnnulerActe.jsp";
    private static final String JSP_URL_DO_RECEPTIONNER_ACTE = "jsp/admin/plugins/ods/acte/DoReceptionnerActe.jsp";
    private static final String JSP_URL_DO_ERREUR_TRANSMISSION_ACTE = "jsp/admin/plugins/ods/acte/DoErreurTransmissionActe.jsp";
    private static final String JSP_URL_MODIFICATION_ACTE_PDD = "jsp/admin/plugins/ods/acte/ModificationActePDD.jsp";
    private static final String JSP_RETOUR_VISUALISATION_ACTE = "jsp/admin/plugins/ods/acte/DoVisualisationActeForSuiviActe.jsp";
    private static final String PROPERTY_FICHIERS_PER_PAGE = "ods.itemPerPage.label";
    private static final String CONSTANTE_PAGE_FIRST = "1";
    private static final String MESSAGE_CONFIRME_DELETE_ACTE = "ods.acte.message.confirmeSuppression";
    private static final String MESSAGE_DELETE_ACTE = "ods.acte.message.suppression";
    private static final String MESSAGE_CONFIRMATION_TO_DELETE = "ods.acte.message.confirmationSuppression";
    private static final String CONSTANTE_ACTION_CANCEL = "ods.all.button.cancel";
    private static final String CONSTANTE_ACTION_APPLY = "ods.pdd.button.appliquer";
    private static final String CONSTANTE_ACTION_FINALISER = "ods.pdd.button.finaliseracte";
    private static final String CONSTANTE_ACTION_ENREGISTRER = "ods.pdd.button.enregistrer";
    private static final String PROPERTY_LABEL_FICHIER = "ods.acte.label.fichier.deliberationfinale";
    private static final String PROPERTY_LABEL_ANNULATION = "ods.acte.message.annulation.motif";
    private static final String PROPERTY_LABEL_DATE_VOTE_RETOUR_CL = "ods.pdd.label.dateControleLegalite";
    private static final String PROPERTY_LABEL_FICHIER_NO_PDF = "ods.acte.message.deliberationFinale.noPDF";
    private static final String MESSAGE_CONFIRMATION_TO_REJET = "ods.acte.message.rejeter.confirmation";
    private static final String MESSAGE_ERROR_ACTE_CATEGORIE = "ods.acte.message.error.categorie";
    private static final String MESSAGE_CONFIRMATION_TO_ERREUR_TRANSMISSION = "ods.acte.message.erreurTransmission.confirmation";
    private static final String PARAM_CHECKBOX_VALIDATE = "checkbox_validate";
    private static final String VALUE_ORDER_BY_STATUS = "orderbyStatus";
    private static final String VALUE_ORDER_BY_REFERENCE = "orderbyReference";
    private static final String VALUE_ORDER_BY_TRANSMISSION = "orderbyTransmission";
    private static final String VALUE_NO_ORDER_BY = "no";
    private static final String MARK_LISTE_STATUTS = "liste_statuts";
    private static final String MARK_LISTE_ENTITE = "liste_entite";
    private static final String MARK_ID_SEANCE_SELECTED = "id_seance_selected";
    private static final String MARK_ID_STATUT_SELECTED = "id_statut_selected";
    private static final String MARK_ID_ENTITE_SELECTED = "id_entite_selected";
    private static final String MARK_NBR_RESULTAT = "nbr_resultat";
    private static final String MARK_NBR_TRANSMIS = "nbr_transmis";
    private static final String MARK_NBR_PREPARATION = "nbr_preparation";
    private static final String MARK_NBR_PUBLIE = "nbr_publie";
    private static final String MARK_NBR_ANNULE = "nbr_annule";
    private static final String MARK_NBR_A_TRANSMETTRE = "nbr_a_transmettre";
    private static final String MARK_TEXT_SUPPRESSION = "text_suppression";
    private static final String MARK_SUPPRESSION = "suppression";
    private static final String MARK_NUMERO = "numero";
    private static final String MARK_FICHIER_DELIBERATION_FINALE = "fichier_deliberation_finale";
    private static final String MARK_DATE_VOTE = "date_vote";
    private static final String MARK_DATE_LEGALITE = "date_legalite";
    private static final String MARK_NATURE_DOCUMENTS = "nature_documents";
    private static final String MARK_LISTE_CATEGORIE_DELIBERATION = "liste_categorie_deliberation";
    private static final String MARK_ID_CATEGORIE_DELIBERATION_SELECTED = "id_categorie_deliberation_selected";
    private static final String MARK_ID_NATURE_DOCUMENTS_SELECTED = "id_nature_document_selected";
    private static final String MARK_STATUT_ACTE = "statut";
    private static final String MARK_EXPOSE_DE_MOTIFS = "fichier_expose_des_motifs";
    private static final String MARK_PROJET_DELIBERE = "fichier_projet_de_delibere";
    private static final String MARK_ENTITE = "entite";
    private static final String MARK_PDD = "pdd";
    private static final String MARK_ID_ACTE = "id_acte";
    private static final String MARK_RETOUR_SUIVI_ACTE = "retour_suivi_acte";
    private static final String MARK_VISUALISATION_ACTION_URL = "visualisation_action_url";
    private static final int RETOUR_LISTE_ACTE = 1;

    //Gere le nombre d'item par page
    protected int _nDefaultItemsPerPage = 5;
    protected String _strCurrentPageIndex;
    protected int _nTypeRetour = -1;
    protected int _nItemsPerPage;
    protected ReferenceList _referenceCategorieDeliberationList;
    protected int _nIdCategorieDeliberationSelected;

    // url
    private String _strUrlActionVisualisation;
    private String _strUrlGestionActe;
    @Autowired
    private ICategorieDeliberationService _categorieDeliberationService;
    @Autowired
    private TypeEntiteHome _entiteActeHome;
    @Autowired
    private StatutActeHome _statutActeHome;
    @Autowired
    private ActeHome<GSeance, GFichier, GActeFilter> _acteHome;
    @Autowired
    private NatureActeHome _natureActeHome;
    @Autowired
    protected DocumentActeHome<GFichier, IFichierFilter, ITypeDocument, GSeance> _documentActeHome;
    @Autowired
    private FormationConseilHome _formationConseilHome;
    @Autowired
    private FichierPhysiqueHome _fichierPhysiqueHome;
    @Autowired
    private ISearchService<GSeance, GSeanceFilter, GRequeteUtilisateur, GResultatRequete, GElu, GFichier, GVoeuAmendement, GPDD> _searchService;
    @Autowired
    protected IFichierHome<GFichier, IFichierFilter, ITypeDocument, GSeance> _fichierHome;
    @Autowired
    private IFichierService<GSeance, GFichier> _fichierService;
    @Autowired
    private HistoriqueHome _historiqueHome;
    @Autowired
    private IPDDHome<GPDD, GPDDFilter, GVoeuAmendementFilter, GSeance, GElu, GVoeuAmendement, GFichier, GArrondissement> _projetDeliberationHome;
    @Autowired
    private IProjetDeDeliberationService _projetDeDeliberationService;
    @Autowired
    protected IFormationConseilService _formationConseilService;
    @Autowired
    private CategorieCtrlLegaliteHome _categorieCtrlLegaliteHome;
    @Autowired
    private CategorieDeliberationHome _categorieDeliberationHome;
    private Map<Integer, IActeServiceManaged<GSeance, GFichier>> _mappingService;
    private Map<Integer, IBusinessHome> _mappingHome;

    public Map<Integer, IBusinessHome> getMappingHome(  )
    {
        return _mappingHome;
    }

    public void setMappingHome( Map<Integer, IBusinessHome> mappingHome )
    {
        this._mappingHome = mappingHome;
    }

    public Map<Integer, IActeServiceManaged<GSeance, GFichier>> getMappingService(  )
    {
        return _mappingService;
    }

    public void setMappingService( Map<Integer, IActeServiceManaged<GSeance, GFichier>> mappingService )
    {
        this._mappingService = mappingService;
    }

    protected abstract Plugin getPlugin(  );

    public abstract GActeFilter getActeFilter(  );

    protected abstract GSeanceHome getSeanceHome(  );

    protected abstract void getActeListSpec( HttpServletRequest request, GActeFilter acteFilter,
        HashMap<String, Object> model );

    protected abstract IFichierFilter newFichierFilterInstance(  );

    protected abstract String getModificationPDDUrl(  );

    protected abstract GActeFilter newActeFilterInstance(  );

    protected abstract GPDDFilter newPDDFilterInstance(  );

    protected abstract String getNumeroInterneActe( Acte<GSeance, GFichier> acte );

    protected abstract boolean getFormationConseil( Acte<GSeance, GFichier> acte );

    protected abstract void sortFichiersByType( IFichierFilter fichierFilter,
        FichierContainer fichierExposeDesMotifsContainer, FichierContainer fichierProjetDeDelibereContainer,
        HashMap<String, Object> model );

    protected abstract void sortFichiersSelectedByType( Acte<GSeance, GFichier> acte, GFichier fichierProjetDeDelibere,
        GFichier fichierExposeDesMotifs, HashMap<String, Object> model );

    /**
     * Methode qui initialise les variables se sessions si elles n'ont pas ts auparavant initialises
     */
    public void init(  )
    {
        if ( -1 == _nDefaultItemsPerPage )
        {
            _nDefaultItemsPerPage = AppPropertiesService.getPropertyInt( PROPERTY_FICHIERS_PER_PAGE, 10 );
        }

        _nTypeRetour = -1;
    }

    /**
    * Methode qui initialise la session
    * @param request la requete
    */
    public void initSession( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        _strUrlGestionActe = JSP_URL_GESTION_ACTE;
        _strUrlActionVisualisation = JSP_RETOUR_VISUALISATION_ACTE;

        if ( -1 == _nDefaultItemsPerPage )
        {
            _nDefaultItemsPerPage = AppPropertiesService.getPropertyInt( PROPERTY_FICHIERS_PER_PAGE, 10 );
        }

        if ( request.getParameter( OdsParameters.ID_SEANCE ) != null )
        {
            int idSeance = Integer.parseInt( (String) request.getParameter( OdsParameters.ID_SEANCE ) );
            request.getSession(  ).setAttribute( ATTRIBUT_SEANCE, idSeance );
        }
        else
        {
            ISeance derniereSeance = getSeanceHome(  ).getDerniereSeance( plugin );

            if ( derniereSeance != null )
            {
                request.getSession(  ).setAttribute( ATTRIBUT_SEANCE, derniereSeance.getIdSeance(  ) );
            }
            else
            {
                request.getSession(  ).setAttribute( ATTRIBUT_SEANCE, -1 );
            }
        }

        request.getSession(  ).setAttribute( ATTRIBUT_FORMATION_CONSEIL, -1 );
        request.getSession(  ).setAttribute( ATTRIBUT_STATUT, -1 );
        request.getSession(  ).setAttribute( ATTRIBUT_ENTITE, -1 );
        request.getSession(  ).removeAttribute( ATTRIBUT_ORDER_BY );

        _nTypeRetour = -1;
    }

    /**
    *
    * initialisation du numero de page pour la pagination
    */
    private void resetPageIndex(  )
    {
        _strCurrentPageIndex = CONSTANTE_PAGE_FIRST;
    }

    /**
     * Initialisation du pdd
     * @param request la requte HTTP
     * @return le droit
     */
    public String initPDD( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        int nIdPdd = OdsUtils.getIntegerParameter( OdsParameters.ID_PDD, request );

        if ( nIdPdd > 0 )
        {
            GPDD pdd = _projetDeliberationHome.findByPrimaryKey( nIdPdd, plugin );

            if ( pdd != null )
            {
                return _projetDeDeliberationService.getRight(  );
            }
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * Permet de rcuprer la liste des statuts
     * @return ReferenceList La liste des statuts disponibles
     */
    protected ReferenceList initRefListStatut(  )
    {
        Plugin plugin = getPlugin(  );

        ReferenceList refListStatut = new ReferenceList(  );
        List<StatutActe> listStatut = _statutActeHome.findStatutList( plugin );

        refListStatut.addItem( -1, OdsConstants.CONSTANTE_CHAINE_VIDE );

        if ( ( listStatut != null ) && !listStatut.isEmpty(  ) )
        {
            for ( StatutActe statut : listStatut )
            {
                refListStatut.addItem( statut.getIdStatut(  ), statut.getLibelle(  ) );
            }
        }

        return refListStatut;
    }

    /**
     * Permet de rcuprer la liste des entits
     * @return ReferenceList La liste des entits disponibles
     */
    protected ReferenceList initRefListEntite(  )
    {
        Plugin plugin = getPlugin(  );

        ReferenceList refListEntite = new ReferenceList(  );
        refListEntite.addItem( -1, OdsConstants.CONSTANTE_CHAINE_VIDE );

        List<TypeEntite> listEntiteActe = _entiteActeHome.findEntiteActesList( plugin );

        for ( TypeEntite entiteActe : listEntiteActe )
        {
            refListEntite.addItem( entiteActe.getIdEntite(  ), entiteActe.getLibelle(  ) );
        }

        return refListEntite;
    }

    /**
     * Permet d'initialiser la liste des categorie de deliberations
     *
     * @param request
     *            la requte HTTP
     */
    protected void initCategorieDeliberation( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        int nIdCategorie = OdsUtils.getIntegerParameter( OdsParameters.ID_CATEGORIE, request );

        if ( nIdCategorie == -1 )
        {
            _referenceCategorieDeliberationList = _categorieDeliberationService.initRefListCategorieDeliberation( plugin,
                    true );
        }

        _nIdCategorieDeliberationSelected = nIdCategorie;
    }

    /**
     * @param request la requte HTTP
     * @param filter PDDFilter
     * @return int l'id de la Categorie de Deliberation qui a ete choisi dans le filtre
     */
    protected int getCategorieDeliberation( HttpServletRequest request, IPDDFilter filter )
    {
        int nCodeCategorieDeliberationFilter = OdsUtils.getIntegerParameter( OdsParameters.ID_CATEGORIE, request );

        if ( nCodeCategorieDeliberationFilter == -1 )
        {
            if ( _nIdCategorieDeliberationSelected != -1 )
            {
                nCodeCategorieDeliberationFilter = _nIdCategorieDeliberationSelected;
            }
            else
            {
                nCodeCategorieDeliberationFilter = AbstractPDDFilter.ALL_INT;
            }
        }

        if ( ( filter != null ) && ( nCodeCategorieDeliberationFilter != AbstractPDDFilter.ALL_INT ) )
        {
            filter.setCategorieDeliberation( nCodeCategorieDeliberationFilter );
        }

        if ( nCodeCategorieDeliberationFilter != _nIdCategorieDeliberationSelected )
        {
            resetPageIndex(  );
        }

        return nCodeCategorieDeliberationFilter;
    }

    /**
     * Permet de rcuprer un objet rfrenceList contenant la liste des natures d'actes
     * @return referenceNatureActeList la liste des natures d'actes
     */
    public ReferenceList getRefListNatureActe(  )
    {
        Plugin plugin = getPlugin(  );

        ReferenceList referenceNatureActeList = new ReferenceList(  );
        List<NatureActe> listNatureActe = _natureActeHome.findActesList( plugin );

        //ajout dans la referenceList de l'ensemble des items reprsentant les natures d'acte
        for ( NatureActe nature : listNatureActe )
        {
            referenceNatureActeList.addItem( nature.getIdNatureDocument(  ), nature.getLibelle(  ) );
        }

        return referenceNatureActeList;
    }

    /**
     * retourne l'interface de gestion des actes
     * @param request la requete HTTP
     * @return model la map contenant les lments utiles au template HTML pour affiche la liste des actes
     */
    public HashMap<String, Object> getActeList( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        HashMap<String, Object> model = new HashMap<String, Object>(  );
        GActeFilter acteFilter = getActeFilter(  );

        int nIdSeance = ( (Integer) request.getSession(  ).getAttribute( ATTRIBUT_SEANCE ) ).intValue(  );

        if ( request.getParameter( OdsParameters.ID_SEANCE ) != null )
        {
            nIdSeance = Integer.parseInt( request.getParameter( OdsParameters.ID_SEANCE ) );
            request.getSession(  ).setAttribute( ATTRIBUT_SEANCE, nIdSeance );
        }

        if ( nIdSeance != -1 )
        {
            acteFilter.setIdSeance( nIdSeance );
        }

        int nIdFirstSeanceForActe = AppPropertiesService.getPropertyInt( OdsProperties.PROPERTY_CREATION_ID_SEANCE, 1 );
        model.put( MARK_LISTE_SEANCES, getSeanceHome(  ).findSeanceAfter( nIdFirstSeanceForActe, plugin ) );
        model.put( MARK_ID_SEANCE_SELECTED, nIdSeance );

        getActeListCommon( request, acteFilter, model );
        getActeListSpec( request, acteFilter, model );
        getAndSortListActes( request, model, acteFilter, true );

        return model;
    }

    protected void getActeListCommon( HttpServletRequest request, GActeFilter acteFilter, HashMap<String, Object> model )
    {
        int nIdStatut = -1;

        if ( request.getSession(  ).getAttribute( ATTRIBUT_STATUT ) != null )
        {
            nIdStatut = ( (Integer) request.getSession(  ).getAttribute( ATTRIBUT_STATUT ) ).intValue(  );
        }

        if ( request.getParameter( PARAMETER_STATUT ) != null )
        {
            nIdStatut = Integer.parseInt( request.getParameter( PARAMETER_STATUT ) );
            request.getSession(  ).setAttribute( ATTRIBUT_STATUT, nIdStatut );
        }

        if ( nIdStatut != -1 )
        {
            acteFilter.setStatut( nIdStatut );
        }

        model.put( MARK_LISTE_STATUTS, initRefListStatut(  ) );
        model.put( MARK_ID_STATUT_SELECTED, nIdStatut );

        int nIdTypeEntite = -1;

        if ( request.getSession(  ).getAttribute( ATTRIBUT_ENTITE ) != null )
        {
            nIdTypeEntite = ( (Integer) request.getSession(  ).getAttribute( ATTRIBUT_ENTITE ) ).intValue(  );
        }

        if ( request.getParameter( PARAMETER_ENTITE ) != null )
        {
            nIdTypeEntite = Integer.parseInt( request.getParameter( PARAMETER_ENTITE ) );
            request.getSession(  ).setAttribute( ATTRIBUT_ENTITE, nIdTypeEntite );
        }

        if ( nIdTypeEntite != -1 )
        {
            acteFilter.setIdTypeEntite( nIdTypeEntite );
        }

        model.put( MARK_LISTE_ENTITE, initRefListEntite(  ) );
        model.put( MARK_ID_ENTITE_SELECTED, Integer.toString( nIdTypeEntite ) );
    }

    /**
     * Retourne l'ecran de confirmation de rejet
     *
     * @param request la requete HTTP
     * @return model la map contenant l'ensemble des lments utiles au template
     */
    public HashMap<String, String> getConfirmRejeter( HttpServletRequest request )
    {
        HashMap<String, String> model = new HashMap<String, String>(  );

        String strIdPDD = request.getParameter( OdsParameters.ID_PDD );
        String strIdActe = request.getParameter( OdsParameters.ID_ACTE );

        UrlItem url = new UrlItem( JSP_URL_DO_REJETER_ACTE );
        url.addParameter( OdsParameters.ID_PDD, strIdPDD );
        url.addParameter( OdsParameters.ID_ACTE, strIdActe );

        model.put( MARK_URL, url.getUrl(  ) );

        return model;
    }

    protected boolean initOrderBy( HttpServletRequest request, GActeFilter acteFilter )
    {
        boolean bOderByReference = false;

        if ( request.getParameter( PARAMETER_ORDER_BY ) != null )
        {
            if ( VALUE_NO_ORDER_BY.equals( request.getParameter( PARAMETER_ORDER_BY ) ) )
            {
                request.getSession(  ).removeAttribute( ATTRIBUT_ORDER_BY );
            }
            else if ( VALUE_ORDER_BY_REFERENCE.equals( request.getParameter( PARAMETER_ORDER_BY ) ) )
            {
                bOderByReference = true;
                request.getSession(  ).setAttribute( ATTRIBUT_ORDER_BY, VALUE_ORDER_BY_REFERENCE );
            }
            else if ( VALUE_ORDER_BY_STATUS.equals( request.getParameter( PARAMETER_ORDER_BY ) ) )
            {
                acteFilter.setOrderByStatut( true );
                request.getSession(  ).setAttribute( ATTRIBUT_ORDER_BY, VALUE_ORDER_BY_STATUS );
            }
            else if ( VALUE_ORDER_BY_TRANSMISSION.equals( request.getParameter( PARAMETER_ORDER_BY ) ) )
            {
                acteFilter.setOrderByTransmission( true );
                request.getSession(  ).setAttribute( ATTRIBUT_ORDER_BY, VALUE_ORDER_BY_TRANSMISSION );
            }
        }
        else if ( request.getSession(  ).getAttribute( ATTRIBUT_ORDER_BY ) != null )
        {
            if ( VALUE_ORDER_BY_REFERENCE.equals( request.getSession(  ).getAttribute( ATTRIBUT_ORDER_BY ) ) )
            {
                bOderByReference = true;
                request.getSession(  ).setAttribute( ATTRIBUT_ORDER_BY, VALUE_ORDER_BY_REFERENCE );
            }
            else if ( VALUE_ORDER_BY_STATUS.equals( request.getSession(  ).getAttribute( ATTRIBUT_ORDER_BY ) ) )
            {
                acteFilter.setOrderByStatut( true );
                request.getSession(  ).setAttribute( ATTRIBUT_ORDER_BY, VALUE_ORDER_BY_STATUS );
            }
            else if ( VALUE_ORDER_BY_TRANSMISSION.equals( request.getSession(  ).getAttribute( ATTRIBUT_ORDER_BY ) ) )
            {
                acteFilter.setOrderByTransmission( true );
                request.getSession(  ).setAttribute( ATTRIBUT_ORDER_BY, VALUE_ORDER_BY_TRANSMISSION );
            }
        }

        return bOderByReference;
    }

    protected void getAndSortListActes( HttpServletRequest request, HashMap<String, Object> model,
        GActeFilter acteFilter, boolean isGetList )
    {
        Plugin plugin = getPlugin(  );

        boolean bOderByReference = initOrderBy( request, acteFilter );

        List<Acte<GSeance, GFichier>> listActes = new ArrayList<Acte<GSeance, GFichier>>(  );

        if ( isGetList )
        {
            listActes = _acteHome.findByFilter( acteFilter, plugin );
        }

        int nActeTransmis = 0;
        int nActePreparation = 0;
        int nActeATransmettre = 0;
        int nActePublie = 0;
        int nActeAnnule = 0;

        for ( Acte<GSeance, GFichier> acte : listActes )
        {
            int nIdStatutActe = acte.getStatutActe(  ).getIdStatut(  );
            StatutActe statut = _statutActeHome.findByPrimaryKey( nIdStatutActe, plugin );
            acte.setStatutActe( statut );

            FormationConseil formationConseil = _formationConseilHome.findByPrimaryKey( acte.getFormationConseil(  )
                                                                                            .getIdFormationConseil(  ),
                    plugin );
            acte.setFormationConseil( formationConseil );

            switch ( nIdStatutActe )
            {
                case 1:
                    nActePreparation++;

                    break;

                case 2:
                    nActeATransmettre++;

                    break;

                case 3:
                    nActeTransmis++;

                    break;

                case 5:
                    nActePublie++;

                    break;

                case 6:
                    nActeAnnule++;

                    break;

                default:
                    break;
            }
        }

        if ( bOderByReference )
        {
            Collections.sort( listActes, new ActeComparator<GSeance, GFichier>(  ) );
        }

        _strCurrentPageIndex = Paginator.getPageIndex( request, Paginator.PARAMETER_PAGE_INDEX, _strCurrentPageIndex );
        _nItemsPerPage = Paginator.getItemsPerPage( request, Paginator.PARAMETER_ITEMS_PER_PAGE, _nItemsPerPage,
                _nDefaultItemsPerPage );

        Paginator paginator = new Paginator( listActes, _nItemsPerPage, getUrlGestionActe(  ),
                Paginator.PARAMETER_PAGE_INDEX, _strCurrentPageIndex );

        model.put( MARK_LISTE_ACTES, paginator.getPageItems(  ) );
        model.put( OdsMarks.MARK_NOMBRE_RESULTATS, listActes.size(  ) );
        model.put( OdsMarks.MARK_PAGINATOR, paginator );
        model.put( OdsMarks.MARK_NB_ITEMS_PER_PAGE, OdsConstants.CONSTANTE_CHAINE_VIDE + _nItemsPerPage );

        model.put( MARK_LISTE_ACTES, paginator.getPageItems(  ) );
        model.put( MARK_NBR_RESULTAT, listActes.size(  ) );
        model.put( MARK_NBR_TRANSMIS, nActeTransmis );
        model.put( MARK_NBR_PREPARATION, nActePreparation );
        model.put( MARK_NBR_PUBLIE, nActePublie );
        model.put( MARK_NBR_ANNULE, nActeAnnule );
        model.put( MARK_NBR_A_TRANSMETTRE, nActeATransmettre );
    }

    /**
     * Retourne l'ecran de visualisation de l'acte suivant le type d'entit
     *
     * @param request la requete HTTP
     * @return String template
     */
    public String getModificationActe( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        int nIdActe = OdsUtils.getIntegerParameter( OdsParameters.ID_ACTE, request );
        Acte<GSeance, GFichier> acte = _acteHome.findByPrimaryKey( nIdActe, plugin );

        int nIdTypeEntite = acte.getIdTypeEntite(  );

        IBusinessItem entite = _mappingHome.get( nIdTypeEntite ).findByPrimaryKey( acte.getIdEntite(  ), plugin );

        IActeServiceManaged<GSeance, GFichier> acteServiceManaged = _mappingService.get( nIdTypeEntite );

        String strReturn = acteServiceManaged.getJspUrlGetModificationForActe( request, entite.getId(  ), nIdActe );

        return strReturn;
    }

    /**
     * Retourne la map contenant l'ensemble des lments utiles au template pour la suppression d'un acte
     *
     * @param request
     *            la requte Http
     * @return model la map contenant l'ensemble des lments utiles au template pour la suppression d'un acte
     */
    public HashMap<String, String> getSuppressionActe( HttpServletRequest request )
    {
        HashMap<String, String> model = new HashMap<String, String>(  );

        String strIdPDD = request.getParameter( OdsParameters.ID_PDD );
        String strIdActe = request.getParameter( OdsParameters.ID_ACTE );

        UrlItem url = new UrlItem( JSP_URL_DO_SUPPRESSION_ACTE );

        if ( strIdActe != null )
        {
            url.addParameter( OdsParameters.ID_ACTE, strIdActe );
        }

        if ( strIdPDD != null )
        {
            url.addParameter( OdsParameters.ID_PDD, strIdPDD );
        }

        String strMessageSuppression = I18nService.getLocalizedString( MESSAGE_CONFIRME_DELETE_ACTE,
                request.getLocale(  ) );
        String strSuppression = I18nService.getLocalizedString( MESSAGE_DELETE_ACTE, request.getLocale(  ) );

        model.put( MARK_TEXT_SUPPRESSION, strMessageSuppression );
        model.put( MARK_SUPPRESSION, strSuppression );
        model.put( MARK_URL, url.getUrl(  ) );

        return model;
    }

    /**
     * Retourne la map contenant l'ensemble des lments utiles au template
     *
     * @param request la requete HTTP
     * @return model la map contenant l'ensemble des lments utiles au template
     */
    public HashMap<String, Object> getConfirmFinaliser( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        HashMap<String, Object> model = new HashMap<String, Object>(  );

        int nIdActe = OdsUtils.getIntegerParameter( OdsParameters.ID_ACTE, request );
        Acte<GSeance, GFichier> acte = _acteHome.findByPrimaryKey( nIdActe, plugin );

        if ( acte != null )
        {
            model.put( MARK_ACTE, acte );
        }

        return model;
    }

    /**
     * Effectue la suppression d'acte  partir des informations contenues dans la requte
     *
     * @param request
     *            la requte Http
     * @return l'url de gestion des actes
     */
    public String doSuppressionActe( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        int nIdPdd = OdsUtils.getIntegerParameter( OdsParameters.ID_PDD, request );
        int nIdActe = OdsUtils.getIntegerParameter( OdsParameters.ID_ACTE, request );

        String strConfirmationSuppression = request.getParameter( PARAM_CHECKBOX_DELETE );

        if ( strConfirmationSuppression == null )
        {
            return AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRMATION_TO_DELETE, AdminMessage.TYPE_WARNING );
        }
        else
        {
            Acte<GSeance, GFichier> acte = new Acte<GSeance, GFichier>(  );
            acte.setId( nIdActe );
            _documentActeHome.removeByActe( acte, plugin );
            _acteHome.remove( acte, plugin );
        }

        String strUrl = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( nIdPdd != -1 )
        {
            strUrl = AppPathService.getBaseUrl( request ) + getModificationPDDUrl(  ) +
                OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_PDD + OdsConstants.CONSTANTE_EGAL +
                nIdPdd;
        }
        else
        {
            strUrl = AppPathService.getBaseUrl( request ) + JSP_URL_GESTION_ACTE;
        }

        return strUrl;
    }

    /**
     * Publie ou dpublie l'acte l'acte
     *
     * @param nIdActe l'id de l'acte
     * @param bIsPublie booleen pour savoir si l'acte est  publier ou pas
     */
    public void doPublicationActe( int nIdActe, boolean bIsPublie )
    {
        Plugin plugin = getPlugin(  );

        if ( nIdActe != 1 )
        {
            Acte<GSeance, GFichier> acte = _acteHome.findByPrimaryKey( nIdActe, plugin );

            if ( acte != null )
            {
                Timestamp dateForVersion = DateUtils.getCurrentDate(  );

                acte.setEnLigne( bIsPublie );
                _acteHome.publication( acte.getId(  ), bIsPublie, plugin );

                //On publie le fichier associ
                _fichierHome.publication( acte.getDeliberationFinale(  ).getId(  ), dateForVersion, 0, bIsPublie, plugin );

                if ( ( acte.getIdTypeEntite(  ) == ITypeEntite.LISTE_DES_MEMBRES_PRESENTS ) ||
                        ( acte.getIdTypeEntite(  ) == ITypeEntite.DESIGNATIONS_DE_REPRESENTANTS ) )
                {
                    IActeServiceManaged<GSeance, GFichier> service = _mappingService.get( acte.getIdTypeEntite(  ) );
                    service.doPublicationActe( acte, bIsPublie, dateForVersion );
                }
            }
        }
    }

    /**
     * Permet de creer un fichier du type qui est pass en paramtre et de
     * l'associer  l'acte pass en parametre
     *
     * @param acteCreated Acte
     * @param fichier FileItem
     * @param typeDocumentEnum TypeDocumentEnum
     * @return Fichier le fichier cr
     */
    protected GFichier createFichier( final Acte<GSeance, GFichier> acteCreated, final FileItem fichier,
        int typeDocumentEnum )
    {
        Plugin plugin = getPlugin(  );
        FichierPhysique newFichierPhysique = new FichierPhysique(  );
        newFichierPhysique.setDonnees( fichier.get(  ) );

        int idFichierPhysique = _fichierPhysiqueHome.create( newFichierPhysique, plugin );
        newFichierPhysique.setIdFichier( idFichierPhysique );

        GFichier newFichier = _fichierService.newFichier(  );
        newFichier.setFichier( newFichierPhysique );
        newFichier.setActe( acteCreated );
        newFichier.setPDD( _projetDeliberationHome.findByPrimaryKey( acteCreated.getIdEntite(  ), plugin ) );
        newFichier.setNom( StringUtils.getNameCourt( fichier ) );
        newFichier.setExtension( fichier.getName(  ).substring( fichier.getName(  ).length(  ) - 3 ).toUpperCase(  ) );
        newFichier.setTaille( (int) fichier.getSize(  ) );

        ITypeDocument typDocument = _fichierHome.findTypeDocumentsById( typeDocumentEnum, plugin );
        newFichier.setTypdeDocument( typDocument );
        newFichier.setIntitule( typDocument.getLibelle(  ) );
        newFichier.setEnLigne( false );

        _fichierHome.create( newFichier, plugin );

        if ( newFichier.getSeance(  ) != null )
        {
            ISeance seanceByFichier = newFichier.getSeance(  );
            ISeance nextSeance = getSeanceHome(  ).getProchaineSeance( plugin );

            if ( ( nextSeance != null ) && !seanceByFichier.equals( nextSeance ) &&
                    ( ( newFichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.DOCUMENT_LIBRE ) ||
                    ( newFichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.COMPTE_RENDU ) ||
                    ( newFichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.LISTE_DE_DESIGNATION ) ) )
            {
                _searchService.addObjectToIndex( newFichier, true, plugin );
            }
        }

        return newFichier;
    }

    /**
     * Permet de mettre  jour une dlibration finale d'un acte qui est pass en paramtre     *
     * @param acteCreated Acte
     * @param filteItem FileItem
     * @return Fichier le fichier mis  jour
     */
    protected GFichier updateFichier( final Acte<GSeance, GFichier> acteCreated, final FileItem filteItem )
    {
        Plugin plugin = getPlugin(  );

        GFichier fichier = _fichierHome.findByPrimaryKey( acteCreated.getDeliberationFinale(  ).getId(  ), plugin );

        FichierPhysique fichierPhysique = fichier.getFichier(  );
        fichierPhysique.setDonnees( filteItem.get(  ) );
        _fichierPhysiqueHome.update( fichierPhysique, plugin );

        fichier.setFichier( fichierPhysique );
        fichier.setNom( StringUtils.getNameCourt( filteItem ) );
        fichier.setExtension( filteItem.getName(  ).substring( filteItem.getName(  ).length(  ) - 3 ).toUpperCase(  ) );
        fichier.setTaille( (int) filteItem.getSize(  ) );

        if ( fichier.getSeance(  ) != null )
        {
            ISeance seanceByFichier = fichier.getSeance(  );
            ISeance nextSeance = getSeanceHome(  ).getProchaineSeance( plugin );

            if ( ( nextSeance != null ) && !seanceByFichier.equals( nextSeance ) &&
                    ( ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.DOCUMENT_LIBRE ) ||
                    ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.COMPTE_RENDU ) ||
                    ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.LISTE_DE_DESIGNATION ) ) )
            {
                _searchService.updateObjectInIndex( fichier, true, plugin );
            }
        }

        _fichierHome.update( fichier, plugin );

        return fichier;
    }

    /**
     * Permet de creer un fichier du type qui est pass en parametre et de
     * l'associer  l'acte pass en parametre
     *
     * @param acteCreated Acte
     * @param fichier FileItem
     * @param typeDocumentEnum TypeDocumentEnum
     * @return Fichier le fichier cr
     */
    protected GFichier dupliquerFichier( Acte<GSeance, GFichier> acteCreated, GFichier fichier, int typeDocumentEnum )
    {
        Plugin plugin = getPlugin(  );

        FichierPhysique newFichierPhysique = new FichierPhysique(  );
        newFichierPhysique.setDonnees( fichier.getFichier(  ).getDonnees(  ) );

        int idFichierPhysique = _fichierPhysiqueHome.create( newFichierPhysique, plugin );
        newFichierPhysique.setIdFichier( idFichierPhysique );

        GFichier newFichier = _fichierService.newFichier(  );
        newFichier.setFichier( fichier.getFichier(  ) );
        newFichier.setActe( acteCreated );
        newFichier.setPDD( _projetDeliberationHome.findByPrimaryKey( acteCreated.getIdEntite(  ), plugin ) );
        newFichier.setNom( fichier.getNom(  ) );
        newFichier.setExtension( fichier.getExtension(  ) );
        newFichier.setTaille( fichier.getTaille(  ) );

        ITypeDocument typDocument = _fichierHome.findTypeDocumentsById( typeDocumentEnum, plugin );
        newFichier.setTypdeDocument( typDocument );
        newFichier.setIntitule( typDocument.getLibelle(  ) );

        Timestamp dateForVersion = new Timestamp( System.currentTimeMillis(  ) );

        newFichier.setEnLigne( true );
        newFichier.setVersion( 1 );
        newFichier.setDatePublication( dateForVersion );

        _fichierHome.create( newFichier, plugin );

        if ( newFichier.getSeance(  ) != null )
        {
            ISeance seanceByFichier = newFichier.getSeance(  );
            ISeance nextSeance = getSeanceHome(  ).getProchaineSeance( plugin );

            if ( ( nextSeance != null ) && !seanceByFichier.equals( nextSeance ) &&
                    ( ( newFichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.DOCUMENT_LIBRE ) ||
                    ( newFichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.COMPTE_RENDU ) ||
                    ( newFichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.LISTE_DE_DESIGNATION ) ) )
            {
                _searchService.addObjectToIndex( newFichier, true, plugin );
            }
        }

        // Ajout de l'historique
        Historique historique = new Historique(  );
        historique.setVersion( 1 );
        historique.setDatePublication( dateForVersion );
        historique.setIdEntite( newFichier.getId(  ) );
        historique.setIdTypeEntite( ITypeEntite.FICHIERS );
        _historiqueHome.create( historique, plugin );

        return newFichier;
    }

    /**
     * Retourne la map contenant l'ensemble des lments utiles au template  la prparation de l'acte
     *
     * @param request la requete HTTP
     * @return HashMap<String, Object> la map contenant l'ensemble des lments utiles au template
     */
    public HashMap<String, Object> getPreparationActe( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        boolean bPermissionModification = true;

        int nIdPddPrimaryKey = OdsUtils.getIntegerParameter( OdsParameters.ID_PDD, request );
        IFichierFilter fichierFilter = newFichierFilterInstance(  );

        fichierFilter.setPDD( nIdPddPrimaryKey );

        // Classement des fichiers par type de document et par identifiant
        List<String> listOrderFichiers = new ArrayList<String>(  );
        listOrderFichiers.add( OdsMarks.MARK_SQL_COLONNE_TYPE_DOCUMENT );
        listOrderFichiers.add( OdsMarks.MARK_SQL_COLONNE_ID );
        fichierFilter.setOrderByList( listOrderFichiers );

        GPDD pdd = _projetDeliberationHome.findByPrimaryKey( nIdPddPrimaryKey, plugin );
        OdsUtils.testNullObject( pdd, request );

        model.put( MARK_PDD, pdd );

        if ( pdd.getReference(  ) != null )
        {
            model.put( MARK_NUMERO, pdd.getReference(  ) );
        }

        if ( pdd.getDateVote(  ) != null )
        {
            model.put( MARK_DATE_VOTE, pdd.getDateVote(  ) );
        }

        boolean bIsProjet = false;

        if ( pdd.getTypePdd(  ).equals( IPDD.CONSTANTE_TYPE_PROJET ) )
        {
            bIsProjet = true;
        }

        model.put( OdsMarks.MARK_FONCTIONNALITE_IS_PROJET, bIsProjet );

        FichierContainer fichierExposeDesMotifsContainer = new FichierContainer(  );
        FichierContainer fichierProjetDeDelibereContainer = new FichierContainer(  );

        sortFichiersByType( fichierFilter, fichierExposeDesMotifsContainer, fichierProjetDeDelibereContainer, model );

        GFichier fichierExposeDesMotifs = fichierExposeDesMotifsContainer.getFichier(  );
        GFichier fichierProjetDeDelibere = fichierProjetDeDelibereContainer.getFichier(  );

        if ( fichierExposeDesMotifs != null )
        {
            model.put( MARK_EXPOSE_DE_MOTIFS, fichierExposeDesMotifs );
        }

        if ( fichierProjetDeDelibere != null )
        {
            model.put( MARK_PROJET_DELIBERE, fichierProjetDeDelibere );
        }

        // Nature documents
        ReferenceList listNatureDocuments = getRefListNatureActe(  );

        model.put( MARK_NATURE_DOCUMENTS, listNatureDocuments );
        model.put( MARK_ID_NATURE_DOCUMENTS_SELECTED, NatureActeEnum.DELIBERATION.getId(  ) );

        // Catgorie de dlibration
        if ( _referenceCategorieDeliberationList == null )
        {
            initCategorieDeliberation( request );
        }

        _nIdCategorieDeliberationSelected = pdd.getCategorieDeliberation(  ).getIdCategorie(  );

        model.put( MARK_LISTE_CATEGORIE_DELIBERATION, _referenceCategorieDeliberationList );
        model.put( MARK_ID_CATEGORIE_DELIBERATION_SELECTED, _nIdCategorieDeliberationSelected );

        // Statut
        model.put( MARK_STATUT_ACTE, _statutActeHome.findByPrimaryKey( StatutActeEnum.A_PREPARER.getId(  ), plugin ) );

        // Pour la modification
        model.put( OdsMarks.MARK_PERMISSION_MODIFICATION, bPermissionModification );

        return model;
    }

    /**
     * Retourne la map contenant l'ensemble des lments utiles au templates
     *
     * @param request la requete HTTP
     * @return HashMap<String, Object> la map contenant l'ensemble des lments utiles au template
     */
    public HashMap<String, Object> getModificationActePDD( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        boolean bPermissionModification = true;

        int nPddPrimaryKey = OdsUtils.getIntegerParameter( OdsParameters.ID_PDD, request );

        int nIdActe = OdsUtils.getIntegerParameter( OdsParameters.ID_ACTE, request );
        Acte<GSeance, GFichier> acte = _acteHome.findByPrimaryKey( nIdActe, plugin );
        GPDD pdd = null;

        IFichierFilter fichierFilter = newFichierFilterInstance(  );

        if ( nPddPrimaryKey <= 0 )
        {
            nPddPrimaryKey = acte.getIdEntite(  );
        }

        GActeFilter acteFilter = newActeFilterInstance(  );
        acteFilter.setIdEntite( nPddPrimaryKey );

        List<Acte<GSeance, GFichier>> listActe = _acteHome.findByFilter( acteFilter, plugin );

        for ( Acte<GSeance, GFichier> act : listActe )
        {
            if ( act.getId(  ) == nIdActe )
            {
                acte = act;
            }
        }

        // Statut
        acte.setStatutActe( _statutActeHome.findByPrimaryKey( acte.getStatutActe(  ).getIdStatut(  ), plugin ) );

        if ( acte != null )
        {
            model.put( MARK_ACTE, acte );
        }

        fichierFilter.setPDD( nPddPrimaryKey );

        // Classement des fichiers par type de document et par identifiant
        List<String> listOrderFichiers = new ArrayList<String>(  );
        listOrderFichiers.add( OdsMarks.MARK_SQL_COLONNE_TYPE_DOCUMENT );
        listOrderFichiers.add( OdsMarks.MARK_SQL_COLONNE_ID );
        fichierFilter.setOrderByList( listOrderFichiers );

        pdd = _projetDeliberationHome.findByPrimaryKey( nPddPrimaryKey, plugin );
        OdsUtils.testNullObject( pdd, request );

        if ( ( acte.getReference(  ) != null ) && ( acte.getNumeroDelibere(  ) > 0 ) )
        {
            model.put( MARK_NUMERO, acte.getReference(  ) + OdsConstants.CONSTANTE_TIRET + acte.getNumeroDelibere(  ) );
        }

        if ( pdd.getDateVote(  ) != null )
        {
            model.put( MARK_DATE_VOTE, pdd.getDateVote(  ) );
        }

        boolean bIsProjet = false;

        if ( pdd.getTypePdd(  ).equals( IPDD.CONSTANTE_TYPE_PROJET ) )
        {
            bIsProjet = true;
        }

        model.put( OdsMarks.MARK_FONCTIONNALITE_IS_PROJET, bIsProjet );

        FichierContainer fichierExposeDesMotifsContainer = new FichierContainer(  );
        FichierContainer fichierProjetDeDelibereContainer = new FichierContainer(  );

        sortFichiersByType( fichierFilter, fichierExposeDesMotifsContainer, fichierProjetDeDelibereContainer, model );

        GFichier fichierExposeDesMotifs = fichierExposeDesMotifsContainer.getFichier(  );
        GFichier fichierProjetDeDelibere = fichierProjetDeDelibereContainer.getFichier(  );

        if ( fichierExposeDesMotifs != null )
        {
            model.put( MARK_EXPOSE_DE_MOTIFS, fichierExposeDesMotifs );
        }

        if ( fichierProjetDeDelibere != null )
        {
            model.put( MARK_PROJET_DELIBERE, fichierProjetDeDelibere );
        }

        sortFichiersSelectedByType( acte, fichierProjetDeDelibere, fichierExposeDesMotifs, model );

        GFichier delibFinal = _fichierHome.findByPrimaryKey( acte.getDeliberationFinale(  ).getId(  ), plugin );

        if ( delibFinal != null )
        {
            model.put( MARK_FICHIER_DELIBERATION_FINALE, delibFinal );
        }

        // Nature documents
        ReferenceList listNatureDocuments = getRefListNatureActe(  );
        model.put( MARK_NATURE_DOCUMENTS, listNatureDocuments );

        // Catgorie de dlibration
        if ( _referenceCategorieDeliberationList == null )
        {
            initCategorieDeliberation( request );
        }

        model.put( MARK_LISTE_CATEGORIE_DELIBERATION, _referenceCategorieDeliberationList );
        _nIdCategorieDeliberationSelected = acte.getCategorie(  ).getIdCategorie(  );

        //dans le cas ou l'on provient de la liste de suivi des actes
        int nIdPDD = OdsUtils.getIntegerParameter( OdsParameters.ID_PDD, request );

        if ( nIdPDD == -1 )
        {
            model.put( MARK_RETOUR_SUIVI_ACTE, true );
        }

        // Pour la modification
        model.put( OdsMarks.MARK_PERMISSION_MODIFICATION, bPermissionModification );

        return model;
    }

    /**
     * Retourne la map contenant l'ensemble des lments utiles au templates utile  la visualisation de l'acte
     *
     * @param request la requete HTTP
     @return HashMap<String, Object> la map contenant l'ensemble des lments utiles au template
     */
    public HashMap<String, Object> getVisualisationActe( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        int nIdAacte = OdsUtils.getIntegerParameter( OdsParameters.ID_ACTE, request );
        Acte<GSeance, GFichier> acte = _acteHome.findByPrimaryKey( nIdAacte, plugin );

        //int nIdTypeEntite = acte.getIdTypeEntite(  );
        int nIdEntite = 0;

        GPDD entite = _projetDeliberationHome.findByPrimaryKey( acte.getIdEntite(  ), plugin );
        nIdEntite = entite.getId(  );
        model.put( MARK_ENTITE, nIdEntite );
        model.put( MARK_DATE_VOTE, entite.getDateVote(  ) );

        boolean bIsProjet = false;

        if ( entite.getTypePdd(  ).equals( IPDD.CONSTANTE_TYPE_PROJET ) )
        {
            bIsProjet = true;
        }

        model.put( OdsMarks.MARK_FONCTIONNALITE_IS_PROJET, bIsProjet );

        GFichier fDeliberationFinale = _fichierHome.findByPrimaryKey( acte.getDeliberationFinale(  ).getId(  ), plugin );
        acte.setDeliberationFinale( fDeliberationFinale );

        NatureActe natureDocument = _natureActeHome.findByPrimaryKey( acte.getNatureDocument(  ).getIdNatureDocument(  ),
                plugin );
        acte.setNatureDocument( natureDocument );

        StatutActe statutActe = _statutActeHome.findByPrimaryKey( acte.getStatutActe(  ).getIdStatut(  ), plugin );
        acte.setStatutActe( statutActe );

        CategorieDeliberation categorieDeliberation = _categorieDeliberationHome.findByPrimaryKey( acte.getCategorie(  )
                                                                                                       .getIdCategorie(  ),
                plugin );
        acte.setCategorie( categorieDeliberation );

        if ( acte != null )
        {
            model.put( MARK_ACTE, acte );
            model.put( MARK_ID_ACTE, nIdAacte );
        }

        // Tri des fichiers selon leur type
        IFichierFilter fichierFilter = newFichierFilterInstance(  );
        fichierFilter.setPDD( nIdEntite );

        // Classement des fichiers par type de document et par identifiant
        List<String> listOrderFichiers = new ArrayList<String>(  );
        listOrderFichiers.add( OdsMarks.MARK_SQL_COLONNE_TYPE_DOCUMENT );
        listOrderFichiers.add( OdsMarks.MARK_SQL_COLONNE_ID );
        fichierFilter.setOrderByList( listOrderFichiers );

        FichierContainer fichierExposeDesMotifsContainer = new FichierContainer(  );
        FichierContainer fichierProjetDeDelibereContainer = new FichierContainer(  );

        sortFichiersByType( fichierFilter, fichierExposeDesMotifsContainer, fichierProjetDeDelibereContainer, model );

        GFichier fichierExposeDesMotifs = fichierExposeDesMotifsContainer.getFichier(  );
        GFichier fichierProjetDeDelibere = fichierProjetDeDelibereContainer.getFichier(  );

        if ( fichierExposeDesMotifs != null )
        {
            model.put( MARK_EXPOSE_DE_MOTIFS, fichierExposeDesMotifs );
        }

        if ( fichierProjetDeDelibere != null )
        {
            model.put( MARK_PROJET_DELIBERE, fichierProjetDeDelibere );
        }

        sortFichiersSelectedByType( acte, fichierProjetDeDelibere, fichierExposeDesMotifs, model );

        //dans le cas ou l'on provient de la liste de suivi des actes
        int nIdPDD = OdsUtils.getIntegerParameter( OdsParameters.ID_PDD, request );

        if ( nIdPDD == -1 )
        {
            model.put( MARK_VISUALISATION_ACTION_URL, getVisualisationActionUrl(  ) );
            model.put( MARK_RETOUR_SUIVI_ACTE, true );
        }

        return model;
    }

    /**
     * Retourne la map contenant l'ensemble des lments utiles au template pour la reception de l'acte
     *
     * @param request la requete HTTP
     * @return model la map contenant l'ensemble des lments utiles au template
     */
    public HashMap<String, Object> getReceptionActe( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        int nIdActe = OdsUtils.getIntegerParameter( OdsParameters.ID_ACTE, request );
        UrlItem url = new UrlItem( JSP_URL_DO_RECEPTIONNER_ACTE );

        if ( nIdActe != -1 )
        {
            url.addParameter( OdsParameters.ID_ACTE, nIdActe );
        }

        model.put( MARK_URL, url.getUrl(  ) );

        Acte<GSeance, GFichier> acte = _acteHome.findByPrimaryKey( nIdActe, plugin );

        if ( acte != null )
        {
            model.put( MARK_ACTE, acte );
        }

        model.put( MARK_DATE_LEGALITE, new Timestamp( System.currentTimeMillis(  ) ) );

        return model;
    }

    /**
     * Retourne la map contenant l'ensemble des lments utiles au template pour l'annulation d'un acte
     *
     * @param request la requete HTTP
     * @return model la map contenant l'ensemble des lments utiles au template pour l'annulation d'un acte
     */
    public HashMap<String, Object> getAnnulationActe( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        int nIdActe = OdsUtils.getIntegerParameter( OdsParameters.ID_ACTE, request );
        Acte<GSeance, GFichier> acte = _acteHome.findByPrimaryKey( nIdActe, plugin );

        if ( acte != null )
        {
            model.put( MARK_ACTE, acte );
        }

        UrlItem url = new UrlItem( JSP_URL_DO_ANNULER_ACTE );
        url.addParameter( OdsParameters.ID_ACTE, nIdActe );
        model.put( MARK_URL, url.getUrl(  ) );

        return model;
    }

    /*
     *
     * ********** TRAITEMENT METIER **********
     *
     */

    /**
     * Cr la modification de l'acte
     *
     * @param request la requte HTTP
     * @param locale la locale
     * @return String template
     */
    public String doPreparationActe( HttpServletRequest request, Locale locale )
    {
        Plugin plugin = getPlugin(  );
        String strUrl = OdsConstants.CONSTANTE_CHAINE_VIDE;

        boolean bRetourList = false;
        int nPoidsPiece = 0;

        int nIdPdd = OdsUtils.getIntegerParameter( OdsParameters.ID_PDD, request );
        GPDD pdd = _projetDeliberationHome.findByPrimaryKey( nIdPdd, plugin );
        Acte<GSeance, GFichier> acte = new Acte<GSeance, GFichier>(  );

        NatureActe natureActe = new NatureActe(  );
        StatutActe statutActe = new StatutActe(  );
        statutActe.setIdStatut( StatutActeEnum.EN_PREPARATION.getId(  ) );
        statutActe.setLibelle( _statutActeHome.findByPrimaryKey( StatutActeEnum.EN_PREPARATION.getId(  ), plugin )
                                              .getLibelle(  ) );

        // Agit selon le bouton cliqu
        String strAction = request.getParameter( OdsParameters.A_FAIRE );

        if ( ( strAction == null ) ||
                strAction.trim(  )
                             .equals( I18nService.getLocalizedString( CONSTANTE_ACTION_CANCEL, request.getLocale(  ) ) ) )
        {
            _nIdCategorieDeliberationSelected = -1;
            strUrl = AppPathService.getBaseUrl( request ) + getModificationPDDUrl(  ) +
                OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_PDD + OdsConstants.CONSTANTE_EGAL +
                nIdPdd;

            return strUrl;
        }
        else if ( strAction.trim(  )
                               .equals( I18nService.getLocalizedString( CONSTANTE_ACTION_APPLY, request.getLocale(  ) ) ) )
        {
            bRetourList = false;
        }
        else if ( strAction.trim(  )
                               .equals( I18nService.getLocalizedString( CONSTANTE_ACTION_FINALISER,
                        request.getLocale(  ) ) ) )
        {
            statutActe.setIdStatut( StatutActeEnum.A_TRANSMETTRE.getId(  ) );
            statutActe.setLibelle( _statutActeHome.findByPrimaryKey( StatutActeEnum.A_TRANSMETTRE.getId(  ), plugin )
                                                  .getLibelle(  ) );
            bRetourList = true;
        }
        else if ( strAction.trim(  )
                               .equals( I18nService.getLocalizedString( CONSTANTE_ACTION_ENREGISTRER,
                        request.getLocale(  ) ) ) )
        {
            bRetourList = true;
        }
        else
        {
            bRetourList = false;
        }

        // Entite Acte
        acte.setIdEntite( pdd.getId(  ) );

        if ( pdd.getTypePdd(  ).equals( IPDD.CONSTANTE_TYPE_PROJET ) )
        {
            acte.setIdTypeEntite( ITypeEntite.PROJETS_DE_DELIBERATIONS );
        }
        else
        {
            acte.setIdTypeEntite( ITypeEntite.PROPOSITIONS_DE_DELIBERATIONS );
        }

        // Numero Delibere
        acte.setNumeroDelibere( _acteHome.createNewNumeroDelibere( pdd.getReference(  ), plugin ) );

        // Direction ou Groupe
        if ( pdd.getTypePdd(  ).equals( IPDD.CONSTANTE_TYPE_PROJET ) )
        {
            acte.setDirection( pdd.getDirection(  ) );
        }
        else
        {
            acte.setIdGroupe( pdd.getGroupePolitique(  ).getIdGroupe(  ) );
        }

        //Seance
        ISeance seance = getSeanceHome(  ).findSeanceByPdd( pdd.getId(  ), plugin );

        if ( seance != null )
        {
            acte.setSeance( seance );
        }

        // Formation Conseil
        acte.setFormationConseil( pdd.getFormationConseil(  ) );

        //Nature Document
        int nNatureDocumentSelected = OdsUtils.getIntegerParameter( OdsParameters.NATURE_DOCUMENT, request );
        natureActe.setIdNatureDocument( nNatureDocumentSelected );
        acte.setNatureDocument( natureActe );

        acte.setReference( pdd.getReference(  ) );

        acte.setStatutActe( statutActe );

        // Catgorie selectionne
        IPDDFilter filter = newPDDFilterInstance(  );

        _nIdCategorieDeliberationSelected = getCategorieDeliberation( request, filter );

        if ( ( filter != null ) && ( _nIdCategorieDeliberationSelected != AbstractPDDFilter.ALL_INT ) )
        {
            filter.setCategorieDeliberation( _nIdCategorieDeliberationSelected );
        }

        acte.setCategorie( _categorieDeliberationHome.findByPrimaryKey( _nIdCategorieDeliberationSelected, plugin ) );

        // Deliberation finale
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        FileItem fileItem = multipartRequest.getFile( OdsParameters.DELIB_FINAL );

        if ( ( fileItem != null ) && ( fileItem.getName(  ) != null ) &&
                !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItem.getName(  ) ) )
        {
            String strExtension = fileItem.getName(  ).substring( fileItem.getName(  ).length(  ) - 3 ).toUpperCase(  );

            if ( ( strExtension != null ) && ( strExtension.length(  ) == 3 ) &&
                    OdsConstants.CONSTANTE_PDF.equals( strExtension ) )
            {
                GFichier fichierDelibFinal = createFichier( acte, fileItem, ITypeDocument.DELIBERATION );
                nPoidsPiece += fichierDelibFinal.getTaille(  );
                acte.setDeliberationFinale( fichierDelibFinal );
            }
            else
            {
                return AdminMessageService.getMessageUrl( request, PROPERTY_LABEL_FICHIER_NO_PDF, AdminMessage.TYPE_STOP );
            }
        }
        else
        {
            Object[] messagesArgs = { I18nService.getLocalizedString( PROPERTY_LABEL_FICHIER, locale ) };

            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED,
                messagesArgs, AdminMessage.TYPE_STOP );
        }

        acte.setEnLigne( false );

        _acteHome.create( acte, plugin );

        //Poids fichiers
        String[] strPddTransmit = request.getParameterValues( OdsParameters.PDD_TRANSMISSION );
        String strExpoMotif = request.getParameter( OdsParameters.EXPO_MOTIF );
        String strProjetDelibere = request.getParameter( OdsParameters.PROJET_DELIBERE );

        DocumentActe docExpoMotif = new DocumentActe(  );
        DocumentActe docProjDelibere = new DocumentActe(  );
        docExpoMotif.setIdActe( acte.getId(  ) );
        docProjDelibere.setIdActe( acte.getId(  ) );

        if ( strProjetDelibere != null )
        {
            docProjDelibere.setIdDocument( Integer.parseInt( strProjetDelibere ) );

            if ( !_documentActeHome.isPairExist( docProjDelibere, acte, plugin ) )
            {
                _documentActeHome.create( docProjDelibere, plugin );
            }

            nPoidsPiece += _fichierHome.findByPrimaryKey( Integer.parseInt( strProjetDelibere ), plugin ).getTaille(  );
        }
        else
        {
            if ( _documentActeHome.isPairExist( docProjDelibere, acte, plugin ) )
            {
                _documentActeHome.remove( docProjDelibere, plugin );
            }
        }

        if ( strExpoMotif != null )
        {
            docExpoMotif.setIdDocument( Integer.parseInt( strExpoMotif ) );

            if ( !_documentActeHome.isPairExist( docExpoMotif, acte, plugin ) )
            {
                _documentActeHome.create( docExpoMotif, plugin );
            }

            nPoidsPiece += _fichierHome.findByPrimaryKey( Integer.parseInt( strExpoMotif ), plugin ).getTaille(  );
        }
        else
        {
            if ( _documentActeHome.isPairExist( docExpoMotif, acte, plugin ) )
            {
                _documentActeHome.remove( docExpoMotif, plugin );
            }
        }

        DocumentActe docPieceAnnexe = new DocumentActe(  );

        if ( strPddTransmit != null )
        {
            for ( int i = 0; i < strPddTransmit.length; i++ )
            {
                if ( strPddTransmit[i] != null )
                {
                    docPieceAnnexe.setIdDocument( Integer.parseInt( strPddTransmit[i] ) );
                    docPieceAnnexe.setIdActe( acte.getId(  ) );

                    if ( !_documentActeHome.isPairExist( docPieceAnnexe, acte, plugin ) )
                    {
                        _documentActeHome.create( docPieceAnnexe, plugin );
                    }

                    nPoidsPiece += _fichierHome.findByPrimaryKey( Integer.parseInt( strPddTransmit[i] ), plugin )
                                               .getTaille(  );
                }
                else
                {
                    if ( _documentActeHome.isPairExist( docPieceAnnexe, acte, plugin ) )
                    {
                        _documentActeHome.remove( docPieceAnnexe, plugin );
                    }
                }
            }
        }

        acte.setPoids( nPoidsPiece );

        int nMaxPoids = AppPropertiesService.getPropertyInt( OdsProperties.PROPERTY_POIDS_MAX_ACTE,
                OdsConstants.DEFAULT_POIDS_MAX_ACTE );

        int nIdModeTransmission = Integer.parseInt( request.getParameter( OdsParameters.MODE_TRANSMISSION ) );

        if ( ( nIdModeTransmission == 1 ) && ( acte.getPoids(  ) > nMaxPoids ) )
        {
            //Mode de transmission automatique, mais le poids est suprieur
            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_MODE_TRANSMISSION_ERROR,
                AdminMessage.TYPE_STOP );
        }

        acte.setIdTransmission( nIdModeTransmission );

        _acteHome.update( acte, plugin );

        // Choix de la page de retour
        if ( bRetourList )
        {
            if ( ( acte.getIdTransmission(  ) == 1 ) && ( acte.getPoids(  ) > nMaxPoids ) )
            {
                strUrl = AppPathService.getBaseUrl( request ) + JSP_URL_FINALISER +
                    OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_PDD + OdsConstants.CONSTANTE_EGAL +
                    nIdPdd + OdsConstants.CONSTANTE_ET + OdsParameters.ID_ACTE + OdsConstants.CONSTANTE_EGAL +
                    acte.getId(  );
            }
            else
            {
                strUrl = AppPathService.getBaseUrl( request ) + getModificationPDDUrl(  ) +
                    OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_PDD + OdsConstants.CONSTANTE_EGAL +
                    nIdPdd;
            }
        }
        else
        {
            strUrl = AppPathService.getBaseUrl( request ) + JSP_URL_MODIFICATION_ACTE +
                OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_PDD + OdsConstants.CONSTANTE_EGAL +
                nIdPdd + OdsConstants.CONSTANTE_ET + OdsParameters.ID_ACTE + OdsConstants.CONSTANTE_EGAL +
                acte.getId(  );
        }

        return strUrl;
    }

    /**
     * Modifie l'acte
     *
     * @param request la requete HTTP
     * @param nTypeRetour l'id du type de retour
     * @return String l'URL de destination
     */
    public String doModificationActe( HttpServletRequest request, int nTypeRetour )
    {
        Plugin plugin = getPlugin(  );
        String strUrl = OdsConstants.CONSTANTE_CHAINE_VIDE;

        boolean bRetourList = false;
        int nIdActe = OdsUtils.getIntegerParameter( OdsParameters.ID_ACTE, request );
        Acte<GSeance, GFichier> acte = _acteHome.findByPrimaryKey( nIdActe, plugin );
        int nIdPdd = acte.getIdEntite(  );
        _nTypeRetour = nTypeRetour;

        //Fichiers lis au PDD	    	
        IFichierFilter fichierFilter = newFichierFilterInstance(  );
        fichierFilter.setPDD( nIdPdd );

        // Classement des fichiers par type de document et par identifiant	        
        List<String> listOrderFichiers = new ArrayList<String>(  );
        listOrderFichiers.add( OdsMarks.MARK_SQL_COLONNE_TYPE_DOCUMENT );
        listOrderFichiers.add( OdsMarks.MARK_SQL_COLONNE_ID );
        fichierFilter.setOrderByList( listOrderFichiers );

        List<GFichier> fichiers = _fichierHome.findByFilter( fichierFilter, plugin );
        List<GFichier> piecesAnnexes = new ArrayList<GFichier>(  );
        List<GFichier> piecesAnnexesMA = new ArrayList<GFichier>(  );
        GFichier fichierExposeDesMotifs = null;
        GFichier fichierProjetDeDelibere = null;

        for ( GFichier f : fichiers )
        {
            if ( ( f.getTypdeDocument(  ) != null ) &&
                    ( ITypeDocument.PIECE_ANNEXE == f.getTypdeDocument(  ).getId(  ) ) )
            {
                piecesAnnexes.add( f );
            }
            else if ( ( f.getTypdeDocument(  ) != null ) &&
                    ( ITypeDocument.PIECE_ANNEXE_MA == f.getTypdeDocument(  ).getId(  ) ) )
            {
                piecesAnnexesMA.add( f );
            }
            else if ( ( f.getTypdeDocument(  ) != null ) &&
                    ( ITypeDocument.EXPOSE_DES_MOTIFS == f.getTypdeDocument(  ).getId(  ) ) )
            {
                fichierExposeDesMotifs = f;
            }
            else if ( ( f.getTypdeDocument(  ) != null ) &&
                    ( ITypeDocument.PROJET_DE_DELIBERE == f.getTypdeDocument(  ).getId(  ) ) )
            {
                fichierProjetDeDelibere = f;
            }
        }

        int nPoidsPiece = 0;
        StatutActe statutActe = new StatutActe(  );
        NatureActe natureActe = new NatureActe(  );
        String strAction = request.getParameter( OdsParameters.A_FAIRE );

        if ( ( strAction == null ) ||
                strAction.trim(  )
                             .equals( I18nService.getLocalizedString( CONSTANTE_ACTION_CANCEL, request.getLocale(  ) ) ) )
        {
            _nIdCategorieDeliberationSelected = -1;

            if ( nTypeRetour == RETOUR_LISTE_ACTE )
            {
                strUrl = AppPathService.getBaseUrl( request ) + JSP_URL_GESTION_ACTE;
            }
            else
            {
                strUrl = AppPathService.getBaseUrl( request ) + getModificationPDDUrl(  ) +
                    OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_PDD + OdsConstants.CONSTANTE_EGAL +
                    nIdPdd;
            }

            return strUrl;
        }
        else if ( strAction.trim(  )
                               .equals( I18nService.getLocalizedString( CONSTANTE_ACTION_APPLY, request.getLocale(  ) ) ) )
        {
            bRetourList = false;
        }
        else if ( strAction.trim(  )
                               .equals( I18nService.getLocalizedString( CONSTANTE_ACTION_FINALISER,
                        request.getLocale(  ) ) ) )
        {
            statutActe.setIdStatut( StatutActeEnum.A_TRANSMETTRE.getId(  ) );
            statutActe.setLibelle( _statutActeHome.findByPrimaryKey( StatutActeEnum.A_TRANSMETTRE.getId(  ), plugin )
                                                  .getLibelle(  ) );
            acte.setStatutActe( statutActe );
            bRetourList = true;
        }
        else if ( strAction.trim(  )
                               .equals( I18nService.getLocalizedString( CONSTANTE_ACTION_ENREGISTRER,
                        request.getLocale(  ) ) ) )
        {
            bRetourList = true;
        }
        else
        {
            bRetourList = false;
        }

        //Nature Document
        int nNatureDocumentSelected = OdsUtils.getIntegerParameter( OdsParameters.NATURE_DOCUMENT, request );
        natureActe.setIdNatureDocument( nNatureDocumentSelected );
        acte.setNatureDocument( natureActe );

        //Poids fichiers
        String[] strPddTransmit = request.getParameterValues( OdsParameters.PDD_TRANSMISSION );
        String strExpoMotif = request.getParameter( OdsParameters.EXPO_MOTIF );
        String strProjetDelibere = request.getParameter( OdsParameters.PROJET_DELIBERE );
        DocumentActe docExpoMotif = new DocumentActe(  );
        DocumentActe docProjDelibere = new DocumentActe(  );
        docExpoMotif.setIdActe( acte.getId(  ) );
        docProjDelibere.setIdActe( acte.getId(  ) );

        if ( strProjetDelibere != null )
        {
            docProjDelibere.setIdDocument( Integer.parseInt( strProjetDelibere ) );

            if ( !_documentActeHome.isPairExist( docProjDelibere, acte, plugin ) )
            {
                _documentActeHome.create( docProjDelibere, plugin );
            }

            nPoidsPiece += _fichierHome.findByPrimaryKey( Integer.parseInt( strProjetDelibere ), plugin ).getTaille(  );
        }
        else
        {
            docProjDelibere.setIdDocument( fichierProjetDeDelibere.getId(  ) );

            if ( _documentActeHome.isPairExist( docProjDelibere, acte, plugin ) )
            {
                _documentActeHome.remove( docProjDelibere, plugin );
            }
        }

        if ( strExpoMotif != null )
        {
            docExpoMotif.setIdDocument( Integer.parseInt( strExpoMotif ) );

            if ( !_documentActeHome.isPairExist( docExpoMotif, acte, plugin ) )
            {
                _documentActeHome.create( docExpoMotif, plugin );
            }

            nPoidsPiece += _fichierHome.findByPrimaryKey( Integer.parseInt( strExpoMotif ), plugin ).getTaille(  );
        }
        else
        {
            docExpoMotif.setIdDocument( fichierExposeDesMotifs.getId(  ) );

            if ( _documentActeHome.isPairExist( docExpoMotif, acte, plugin ) )
            {
                _documentActeHome.remove( docExpoMotif, plugin );
            }
        }

        DocumentActe docPieceAnnexe = new DocumentActe(  );
        docPieceAnnexe.setIdActe( acte.getId(  ) );

        if ( strPddTransmit != null )
        {
            for ( int i = 0; i < strPddTransmit.length; i++ )
            {
                if ( strPddTransmit[i] != null )
                {
                    docPieceAnnexe.setIdDocument( Integer.parseInt( strPddTransmit[i] ) );

                    if ( !_documentActeHome.isPairExist( docPieceAnnexe, acte, plugin ) )
                    {
                        _documentActeHome.create( docPieceAnnexe, plugin );
                    }

                    nPoidsPiece += _fichierHome.findByPrimaryKey( Integer.parseInt( strPddTransmit[i] ), plugin )
                                               .getTaille(  );
                }
                else
                {
                    if ( _documentActeHome.isPairExist( docPieceAnnexe, acte, plugin ) )
                    {
                        _documentActeHome.remove( docPieceAnnexe, plugin );
                    }
                }
            }

            List<GFichier> listFichierTmp = new ArrayList<GFichier>( piecesAnnexes );

            for ( GFichier pieceAnnexe : piecesAnnexes )
            {
                for ( int i = 0; i < strPddTransmit.length; i++ )
                {
                    if ( pieceAnnexe.getId(  ) == Integer.parseInt( strPddTransmit[i] ) )
                    {
                        listFichierTmp.remove( pieceAnnexe );
                    }
                }
            }

            for ( GFichier pieceAnnexe : listFichierTmp )
            {
                docPieceAnnexe.setIdDocument( pieceAnnexe.getId(  ) );

                if ( _documentActeHome.isPairExist( docPieceAnnexe, acte, plugin ) )
                {
                    _documentActeHome.remove( docPieceAnnexe, plugin );
                }
            }

            List<GFichier> listFichierMATmp = new ArrayList<GFichier>( piecesAnnexesMA );

            for ( GFichier pieceAnnexeMA : piecesAnnexesMA )
            {
                for ( int i = 0; i < strPddTransmit.length; i++ )
                {
                    if ( pieceAnnexeMA.getId(  ) == Integer.parseInt( strPddTransmit[i] ) )
                    {
                        listFichierMATmp.remove( pieceAnnexeMA );
                    }
                }
            }

            for ( GFichier pieceAnnexeMA : listFichierMATmp )
            {
                docPieceAnnexe.setIdDocument( pieceAnnexeMA.getId(  ) );

                if ( _documentActeHome.isPairExist( docPieceAnnexe, acte, plugin ) )
                {
                    _documentActeHome.remove( docPieceAnnexe, plugin );
                }
            }
        }
        else
        {
            //remove all piece annexes
            for ( GFichier pieceAnnexe : piecesAnnexes )
            {
                docPieceAnnexe.setIdDocument( pieceAnnexe.getId(  ) );
                docPieceAnnexe.setIdActe( acte.getId(  ) );

                if ( _documentActeHome.isPairExist( docPieceAnnexe, acte, plugin ) )
                {
                    _documentActeHome.remove( docPieceAnnexe, plugin );
                }
            }

            //remove all piece annexes MA
            for ( GFichier pieceAnnexeMA : piecesAnnexesMA )
            {
                docPieceAnnexe.setIdDocument( pieceAnnexeMA.getId(  ) );
                docPieceAnnexe.setIdActe( acte.getId(  ) );

                if ( _documentActeHome.isPairExist( docPieceAnnexe, acte, plugin ) )
                {
                    _documentActeHome.remove( docPieceAnnexe, plugin );
                }
            }
        }

        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        FileItem fileItem = multipartRequest.getFile( OdsParameters.DELIB_FINAL );
        GFichier fichierDelibFinale = null;

        if ( ( fileItem != null ) && ( fileItem.getName(  ) != null ) &&
                !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItem.getName(  ) ) )
        {
            String strExtension = fileItem.getName(  ).substring( fileItem.getName(  ).length(  ) - 3 ).toUpperCase(  );

            if ( ( strExtension != null ) && ( strExtension.length(  ) == 3 ) &&
                    OdsConstants.CONSTANTE_PDF.equals( strExtension ) )
            {
                fichierDelibFinale = updateFichier( acte, fileItem );
                nPoidsPiece += fichierDelibFinale.getTaille(  );
                acte.setDeliberationFinale( fichierDelibFinale );
            }
            else
            {
                return AdminMessageService.getMessageUrl( request, PROPERTY_LABEL_FICHIER_NO_PDF, AdminMessage.TYPE_STOP );
            }
        }
        else
        {
            fichierDelibFinale = _fichierHome.findByPrimaryKey( acte.getDeliberationFinale(  ).getId(  ), plugin );
            nPoidsPiece += fichierDelibFinale.getTaille(  );
        }

        acte.setPoids( nPoidsPiece );

        //Transmission
        int nMaxPoids = AppPropertiesService.getPropertyInt( OdsProperties.PROPERTY_POIDS_MAX_ACTE,
                OdsConstants.DEFAULT_POIDS_MAX_ACTE );

        int nIdModeTransmission = Integer.parseInt( request.getParameter( OdsParameters.MODE_TRANSMISSION ) );

        if ( ( nIdModeTransmission == 1 ) && ( acte.getPoids(  ) > nMaxPoids ) )
        {
            //Mode de transmission automatique, mais le poids est suprieur
            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_MODE_TRANSMISSION_ERROR,
                AdminMessage.TYPE_STOP );
        }

        acte.setIdTransmission( nIdModeTransmission );

        // Catgorie selectionne
        IPDDFilter filter = newPDDFilterInstance(  );

        _nIdCategorieDeliberationSelected = getCategorieDeliberation( request, filter );

        if ( ( filter != null ) && ( _nIdCategorieDeliberationSelected != AbstractPDDFilter.ALL_INT ) )
        {
            filter.setCategorieDeliberation( _nIdCategorieDeliberationSelected );
        }

        acte.setCategorie( _categorieDeliberationHome.findByPrimaryKey( _nIdCategorieDeliberationSelected, plugin ) );

        _acteHome.update( acte, plugin );

        if ( bRetourList )
        {
            if ( ( acte.getIdTransmission(  ) == 1 ) && ( acte.getPoids(  ) > nMaxPoids ) )
            {
                strUrl = AppPathService.getBaseUrl( request ) + JSP_URL_FINALISER +
                    OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_PDD + OdsConstants.CONSTANTE_EGAL +
                    nIdPdd + OdsConstants.CONSTANTE_ET + OdsParameters.ID_ACTE + OdsConstants.CONSTANTE_EGAL +
                    acte.getId(  );
            }
            else
            {
                if ( nTypeRetour == RETOUR_LISTE_ACTE )
                {
                    strUrl = AppPathService.getBaseUrl( request ) + JSP_URL_GESTION_ACTE;
                }
                else
                {
                    strUrl = AppPathService.getBaseUrl( request ) + getModificationPDDUrl(  ) +
                        OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_PDD +
                        OdsConstants.CONSTANTE_EGAL + acte.getIdEntite(  );
                }
            }
        }
        else
        {
            if ( nTypeRetour == RETOUR_LISTE_ACTE )
            {
                strUrl = AppPathService.getBaseUrl( request ) + JSP_URL_MODIFICATION_ACTE_PDD +
                    OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_ACTE + OdsConstants.CONSTANTE_EGAL +
                    acte.getId(  );
            }
            else
            {
                strUrl = AppPathService.getBaseUrl( request ) + JSP_URL_MODIFICATION_ACTE +
                    OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_PDD + OdsConstants.CONSTANTE_EGAL +
                    acte.getIdEntite(  ) + OdsConstants.CONSTANTE_ET + OdsParameters.ID_ACTE +
                    OdsConstants.CONSTANTE_EGAL + acte.getId(  );
            }
        }

        return strUrl;
    }

    /**
     * Permet de retourner  la liste des actes
     *
     * @param request
     *            la requte HTTP
     * @return strUrl url de retour
     */
    public String doVisualiserActe( HttpServletRequest request )
    {
        String strUrl = OdsConstants.CONSTANTE_CHAINE_VIDE;
        int nIdPdd = OdsUtils.getIntegerParameter( OdsParameters.ID_PDD, request );

        strUrl = AppPathService.getBaseUrl( request ) + getModificationPDDUrl(  ) +
            OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_PDD + OdsConstants.CONSTANTE_EGAL + nIdPdd;

        return strUrl;
    }

    /**
     * Permet de retourner  la liste de suivi des actes
     *
     * @param request
     *            la requte HTTP
     * @return strUrl url de retour
     */
    public String doVisualiserActeForSuiviActe( HttpServletRequest request )
    {
        String strUrl = AppPathService.getBaseUrl( request ) + getUrlGestionActe(  );

        return strUrl;
    }

    /**
     * Permet de finaliser un acte
     *
     * @param request
     *            la requte HTTP
     * @return strUrl url de retour
     */
    public String doFinaliserActe( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        String strUrl = OdsConstants.CONSTANTE_CHAINE_VIDE;
        int nIdPdd = OdsUtils.getIntegerParameter( OdsParameters.ID_PDD, request );
        int nIdActe = OdsUtils.getIntegerParameter( OdsParameters.ID_ACTE, request );
        Acte<GSeance, GFichier> acte = _acteHome.findByPrimaryKey( nIdActe, plugin );

        String strAction = request.getParameter( OdsParameters.A_FAIRE );

        if ( ( strAction == null ) ||
                strAction.trim(  )
                             .equals( I18nService.getLocalizedString( CONSTANTE_ACTION_CANCEL, request.getLocale(  ) ) ) )
        {
            StatutActe statutActe = new StatutActe(  );
            statutActe.setIdStatut( StatutActeEnum.EN_PREPARATION.getId(  ) );
            statutActe.setLibelle( _statutActeHome.findByPrimaryKey( StatutActeEnum.EN_PREPARATION.getId(  ), plugin )
                                                  .getLibelle(  ) );

            acte.setStatutActe( statutActe );

            _acteHome.update( acte, plugin );

            if ( _nTypeRetour == RETOUR_LISTE_ACTE )
            {
                strUrl = AppPathService.getBaseUrl( request ) + JSP_URL_MODIFICATION_ACTE +
                    OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_ACTE + OdsConstants.CONSTANTE_EGAL +
                    nIdActe;
            }
            else
            {
                strUrl = AppPathService.getBaseUrl( request ) + JSP_URL_MODIFICATION_ACTE_PDD +
                    OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_PDD + OdsConstants.CONSTANTE_EGAL +
                    nIdPdd + OdsConstants.CONSTANTE_ET + OdsParameters.ID_ACTE + OdsConstants.CONSTANTE_EGAL + nIdActe;
            }

            return strUrl;
        }
        else if ( strAction.trim(  )
                               .equals( I18nService.getLocalizedString( CONSTANTE_ACTION_FINALISER,
                        request.getLocale(  ) ) ) )
        {
            String strActe = request.getParameter( OdsParameters.CHECKBOX_DELETE );

            if ( strActe != null )
            {
                StatutActe statutActe = new StatutActe(  );
                statutActe.setIdStatut( StatutActeEnum.A_TRANSMETTRE.getId(  ) );
                statutActe.setLibelle( _statutActeHome.findByPrimaryKey( StatutActeEnum.A_TRANSMETTRE.getId(  ), plugin )
                                                      .getLibelle(  ) );

                acte.setStatutActe( statutActe );
                acte.setIdTransmission( 0 );
                _acteHome.update( acte, plugin );
            }
            else
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_CHECKBOX,
                    AdminMessage.TYPE_STOP );
            }
        }

        if ( _nTypeRetour == RETOUR_LISTE_ACTE )
        {
            strUrl = AppPathService.getBaseUrl( request ) + JSP_URL_GESTION_ACTE;
        }
        else
        {
            strUrl = AppPathService.getBaseUrl( request ) + getModificationPDDUrl(  ) +
                OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_PDD + OdsConstants.CONSTANTE_EGAL +
                nIdPdd;
        }

        return strUrl;
    }

    /**
     * Permet de transmettre manuellement un acte
     *
     * @param request
     *            la requte HTTP
     * @return strUrl url de retour
     */
    public String doTransmissionManuelleActe( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        String strUrl = OdsConstants.CONSTANTE_CHAINE_VIDE;
        int nIdPdd = OdsUtils.getIntegerParameter( OdsParameters.ID_PDD, request );
        int nIdAacte = OdsUtils.getIntegerParameter( OdsParameters.ID_ACTE, request );

        Acte<GSeance, GFichier> acte = _acteHome.findByPrimaryKey( nIdAacte, plugin );

        StatutActe statutActe = new StatutActe(  );
        statutActe.setIdStatut( StatutActeEnum.TRANSMIS.getId(  ) );
        statutActe.setLibelle( _statutActeHome.findByPrimaryKey( StatutActeEnum.TRANSMIS.getId(  ), plugin ).getLibelle(  ) );

        acte.setStatutActe( statutActe );

        _acteHome.update( acte, plugin );

        if ( nIdPdd != -1 )
        {
            strUrl = AppPathService.getBaseUrl( request ) + getModificationPDDUrl(  ) +
                OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_PDD + OdsConstants.CONSTANTE_EGAL +
                nIdPdd;
        }
        else
        {
            strUrl = AppPathService.getBaseUrl( request ) + getUrlGestionActe(  );
        }

        return strUrl;
    }

    /**
     * Transmet l'acte
     * @param request la requete
     * @return String la page de retour
     */
    public String doTransmissionActe( HttpServletRequest request )
    {
        boolean transmis = false;
        List<String> listErrors = new ArrayList<String>(  );

        String[] strValue = (String[]) request.getParameterMap(  ).get( "transmission" );

        if ( strValue != null )
        {
            try
            {
                transmis = sendActe( strValue, listErrors );
            }
            catch ( Exception e )
            {
                AppLogService.error( "Error during the transmission : " + e.getMessage(  ) );

                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_TRANSMISSION,
                    AdminMessage.TYPE_ERROR );
            }

            if ( !transmis )
            {
                return AdminMessageService.getMessageUrl( request, listErrors.get( 0 ), AdminMessage.TYPE_ERROR );
            }
            else
            {
                return AppPathService.getBaseUrl( request ) + getUrlGestionActe(  );
            }
        }
        else
        {
            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_STOP_TRANSMISSION,
                AdminMessage.TYPE_STOP );
        }
    }

    /**
     * Permet de receptionner un acte (si transmis manuellement)
     *
     * @param request
     *            la requte HTTP
     * @return strUrl url de retour
     */
    public String doReceptionActe( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        int nIdActe = OdsUtils.getIntegerParameter( OdsParameters.ID_ACTE, request );
        Acte<GSeance, GFichier> acte = _acteHome.findByPrimaryKey( nIdActe, plugin );

        String strDateRetourCL = request.getParameter( OdsParameters.DATE_RETOUR_CONTROLE_LEGALITE );
        Timestamp tsDateRetourControleLegalite = DateUtils.getDate( strDateRetourCL, true );

        if ( tsDateRetourControleLegalite != null )
        {
            acte.setDateRetourControleLegalite( tsDateRetourControleLegalite );
        }
        else
        {
            Object[] messagesArgs = 
                {
                    I18nService.getLocalizedString( PROPERTY_LABEL_DATE_VOTE_RETOUR_CL, request.getLocale(  ) ),
                };

            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_DATE_FORMAT_ERROR,
                messagesArgs, AdminMessage.TYPE_WARNING );
        }

        StatutActe statutActe = new StatutActe(  );
        statutActe.setIdStatut( StatutActeEnum.PUBLIE.getId(  ) );
        acte.setStatutActe( statutActe );

        if ( ( acte.getIdTypeEntite(  ) == ITypeEntite.PROJETS_DE_DELIBERATIONS ) ||
                ( acte.getIdTypeEntite(  ) == ITypeEntite.PROPOSITIONS_DE_DELIBERATIONS ) )
        {
            GPDD pdd = _projetDeliberationHome.findByPrimaryKey( acte.getIdEntite(  ), plugin );

            if ( pdd.isEnLigne(  ) )
            {
                //Publication des pieces annexes du pdd qui sont rattachs  l'acte
                Timestamp dateForVersion = new Timestamp( System.currentTimeMillis(  ) );

                //int nNewVersion = pdd.getVersion(  ) + 1;
                List<GFichier> listFichiers = _documentActeHome.findByActe( acte, plugin );

                for ( GFichier fichier : listFichiers )
                {
                    if ( !fichier.getEnLigne(  ) &&
                            ( fichier.getTypdeDocument(  ).getId(  ) != ITypeDocument.PIECE_ANNEXE_MA ) )
                    {
                        //on publie le fichier si ce dernier n'est pas dj publi
                        fichier.setEnLigne( true );
                        fichier.setVersion( 1 );
                        fichier.setDatePublication( dateForVersion );
                        _fichierHome.update( fichier, plugin );

                        // Ajout de l'historique du fichier
                        Historique historique = new Historique(  );
                        historique.setVersion( 1 );
                        historique.setDatePublication( dateForVersion );
                        historique.setIdEntite( fichier.getId(  ) );
                        historique.setIdTypeEntite( ITypeEntite.FICHIERS );
                        _historiqueHome.create( historique, plugin );
                    }
                }

                //ODS-430 & ODS-464 : La publication d'un acte ne modifie pas la version et la date de publication du PDD

                /*
                        //on incrmente la version du PDD et on modifie sa date de publication
                        pdd.setVersion( nNewVersion );
                        pdd.setDatePublication( dateForVersion );
                
                        boolean bArchive = true;
                        GPDDFilter filter = newPDDFilterInstance(  );
                
                        GSeance nextSeance = getSeanceHome(  ).getProchaineSeance( plugin );
                
                        if ( nextSeance != null )
                        {
                            filter.setProchaineSeance( nextSeance.getIdSeance(  ) );
                
                            List<Integer> listPDDProchaineSeance = _projetDeliberationHome.findIdByFilter( filter, plugin );
                
                            if ( ( listPDDProchaineSeance != null ) && listPDDProchaineSeance.contains( pdd.getId(  ) ) )
                            {
                                bArchive = false;
                            }
                        }
                
                        _searchService.updateObjectInIndex( pdd, bArchive, plugin );
                        _projetDeliberationHome.update( pdd, plugin );
                
                        // Ajout de l'historique du PDD
                        Historique historiquePDD = new Historique(  );
                        historiquePDD.setVersion( nNewVersion );
                        historiquePDD.setDatePublication( dateForVersion );
                        historiquePDD.setIdEntite( pdd.getId(  ) );
                        historiquePDD.setIdTypeEntite( ITypeEntite.PDD );
                        _historiqueHome.create( historiquePDD, plugin );
                */
            }
        }

        _acteHome.update( acte, plugin );
        doPublicationActe( nIdActe, true );

        return AppPathService.getBaseUrl( request ) + getUrlGestionActe(  );
    }

    /**
     * Permet de rejetter un acte
     *
     * @param request
     *            la requte HTTP
     * @return strUrl url de retour
     */
    public String doRejeterActe( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        String strUrl = OdsConstants.CONSTANTE_CHAINE_VIDE;

        int nIdPdd = OdsUtils.getIntegerParameter( OdsParameters.ID_PDD, request );
        int nIdActe = OdsUtils.getIntegerParameter( OdsParameters.ID_ACTE, request );

        String strConfirmationSuppression = request.getParameter( PARAM_CHECKBOX_DELETE );
        Acte<GSeance, GFichier> acte = _acteHome.findByPrimaryKey( nIdActe, plugin );

        if ( strConfirmationSuppression == null )
        {
            return AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRMATION_TO_REJET, AdminMessage.TYPE_WARNING );
        }
        else
        {
            StatutActe statutActe = new StatutActe(  );
            statutActe.setIdStatut( StatutActeEnum.EN_PREPARATION.getId(  ) );
            statutActe.setLibelle( _statutActeHome.findByPrimaryKey( StatutActeEnum.EN_PREPARATION.getId(  ), plugin )
                                                  .getLibelle(  ) );

            acte.setStatutActe( statutActe );

            _acteHome.update( acte, plugin );
        }

        if ( nIdPdd != -1 )
        {
            strUrl = AppPathService.getBaseUrl( request ) + getModificationPDDUrl(  ) +
                OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_PDD + OdsConstants.CONSTANTE_EGAL +
                nIdPdd;
        }
        else
        {
            strUrl = AppPathService.getBaseUrl( request ) + getUrlGestionActe(  );
        }

        return strUrl;
    }

    /**
     * Permet d'annuler un acte
     *
     * @param request
     *            la requte HTTP
     * @return strUrl url de retour
     */
    public String doAnnulerActe( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        String strUrl = OdsConstants.CONSTANTE_CHAINE_VIDE;

        int nIdPdd = OdsUtils.getIntegerParameter( OdsParameters.ID_PDD, request );
        int nIdActe = OdsUtils.getIntegerParameter( OdsParameters.ID_ACTE, request );
        boolean transmis = false;

        if ( nIdActe != -1 )
        {
            Acte<GSeance, GFichier> acte = _acteHome.findByPrimaryKey( nIdActe, plugin );
            StatutActe statutActe = new StatutActe(  );

            if ( acte.getIdTransmission(  ) == 1 )
            {
                statutActe.setIdStatut( StatutActeEnum.EN_ATTENTE_ANNULATION.getId(  ) );
                statutActe.setLibelle( _statutActeHome.findByPrimaryKey( 
                        StatutActeEnum.EN_ATTENTE_ANNULATION.getId(  ), plugin ).getLibelle(  ) );
            }
            else
            {
                statutActe.setIdStatut( StatutActeEnum.ANNULE.getId(  ) );
                statutActe.setLibelle( _statutActeHome.findByPrimaryKey( StatutActeEnum.ANNULE.getId(  ), plugin )
                                                      .getLibelle(  ) );
            }

            acte.setStatutActe( statutActe );

            String strAnnulation = request.getParameter( OdsParameters.COMMENTAIRE_ANNULATION );

            if ( strAnnulation != null )
            {
                if ( !strAnnulation.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                {
                    if ( strAnnulation.length(  ) < 250 )
                    {
                        acte.setComAnnulation( strAnnulation );
                        _acteHome.createCommentaireAnnulation( acte, plugin );

                        try
                        {
                            transmis = sendAnnulationActe( nIdActe );
                        }
                        catch ( IOException e )
                        {
                            return AdminMessageService.getMessageUrl( request,
                                OdsProperties.PROPERTY_MESSAGE_TRANSMISSION, AdminMessage.TYPE_ERROR );
                        }

                        if ( !transmis )
                        {
                            return AdminMessageService.getMessageUrl( request,
                                OdsProperties.PROPERTY_MESSAGE_TRANSMISSION, AdminMessage.TYPE_ERROR );
                        }
                    }
                    else
                    {
                        return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_LENGTH_ERROR,
                            AdminMessage.TYPE_WARNING );
                    }
                }
                else
                {
                    Object[] messagesArgs = 
                        {
                            I18nService.getLocalizedString( PROPERTY_LABEL_ANNULATION, request.getLocale(  ) ),
                        };

                    return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED,
                        messagesArgs, AdminMessage.TYPE_WARNING );
                }
            }

            if ( acte.getEnLigne(  ) )
            {
                //On dpublie l'acte du Front-Office
                acte.setEnLigne( false );
                _fichierHome.publication( acte.getDeliberationFinale(  ).getId(  ), DateUtils.getCurrentDate(  ), 0,
                    false, plugin );
            }

            _acteHome.update( acte, plugin );
        }

        if ( nIdPdd != -1 )
        {
            strUrl = AppPathService.getBaseUrl( request ) + getModificationPDDUrl(  ) +
                OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_PDD + OdsConstants.CONSTANTE_EGAL +
                nIdPdd;
        }
        else
        {
            strUrl = AppPathService.getBaseUrl( request ) + getUrlGestionActe(  );
        }

        return strUrl;
    }

    /**
    * Permet de cloner un acte
    *
    * @param request
    *            la requte HTTP
    * @return strUrl url de retour
    */
    public String doClonerActe( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        String strUrl = OdsConstants.CONSTANTE_CHAINE_VIDE;

        int nIdActe = OdsUtils.getIntegerParameter( OdsParameters.ID_ACTE, request );
        int nIdPdd = OdsUtils.getIntegerParameter( OdsParameters.ID_PDD, request );

        Acte<GSeance, GFichier> acte = _acteHome.findByPrimaryKey( nIdActe, plugin );

        if ( ( acte.getIdTypeEntite(  ) != ITypeEntite.PROJETS_DE_DELIBERATIONS ) &&
                ( acte.getIdTypeEntite(  ) != ITypeEntite.PROPOSITIONS_DE_DELIBERATIONS ) )
        {
            int nIdEntite = acte.getIdEntite(  );
            int nNombreActe = 0;

            IEntiteActe item = (IEntiteActe) _mappingHome.get( acte.getIdTypeEntite(  ) )
                                                         .findByPrimaryKey( nIdEntite, plugin );
            nNombreActe = item.getNombreActes(  );

            if ( nNombreActe > 0 )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_ACTE_EN_COURS,
                    AdminMessage.TYPE_STOP );
            }
        }

        List<GFichier> listDocuments = _documentActeHome.findByActe( acte, plugin );

        int nNumeroDelibere = acte.getNumeroDelibere(  );
        Acte<GSeance, GFichier> newActe = acte;

        newActe.setNumeroDelibere( nNumeroDelibere );
        newActe.setDateRetourControleLegalite( null );

        GFichier fileToClone = _fichierHome.findByPrimaryKey( acte.getDeliberationFinale(  ).getId(  ), plugin );
        FichierPhysique filePhysique = _fichierPhysiqueHome.findByPrimaryKey( fileToClone.getFichier(  ).getIdFichier(  ),
                plugin );
        fileToClone.setFichier( filePhysique );

        GFichier fichierActe = dupliquerFichier( acte, fileToClone, ITypeDocument.DELIBERATION );

        newActe.setDeliberationFinale( fichierActe );

        StatutActe statutActe = new StatutActe(  );
        statutActe.setIdStatut( StatutActeEnum.EN_PREPARATION.getId(  ) );
        statutActe.setLibelle( _statutActeHome.findByPrimaryKey( StatutActeEnum.EN_PREPARATION.getId(  ), plugin )
                                              .getLibelle(  ) );

        newActe.setStatutActe( statutActe );

        _acteHome.create( newActe, plugin );

        for ( GFichier fichier : listDocuments )
        {
            DocumentActe documentActe = new DocumentActe(  );
            documentActe.setIdActe( newActe.getId(  ) );
            documentActe.setIdDocument( fichier.getId(  ) );

            if ( !_documentActeHome.isPairExist( documentActe, newActe, plugin ) )
            {
                _documentActeHome.create( documentActe, plugin );
            }
        }

        if ( nIdPdd != -1 )
        {
            strUrl = AppPathService.getBaseUrl( request ) + getModificationPDDUrl(  ) +
                OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_PDD + OdsConstants.CONSTANTE_EGAL +
                nIdPdd;
        }
        else
        {
            strUrl = AppPathService.getBaseUrl( request ) + JSP_URL_GESTION_ACTE;
        }

        return strUrl;
    }

    /**
     * Telecharge un fichier. Renvoie un flux de bytes du fichier dans l'objet
     * HttpServletResponse response
     *
     * @param request la requete HTTP
     * @return fichierTelecharger le fichier  tlcharger
     */
    public GFichier doDownloadFichier( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        GFichier fichierTelecharger = null;

        String strIdFichier = request.getParameter( OdsParameters.ID_FICHIER );

        if ( strIdFichier != null )
        {
            int nIdFichier = OdsUtils.getIntegerParameter( OdsParameters.ID_FICHIER, request );

            fichierTelecharger = _fichierHome.findByPrimaryKey( nIdFichier, plugin );

            FichierPhysique fichierPhysique = _fichierPhysiqueHome.findByPrimaryKey( fichierTelecharger.getFichier(  )
                                                                                                       .getIdFichier(  ),
                    plugin );

            fichierTelecharger.setFichier( fichierPhysique );
        }

        return fichierTelecharger;
    }

    /*
     *
     * ********** TRANSMISSION ACTE **********
     *
     */

    /**
     * Permet la transmission d'un acte
     * @param strIdActe tableau d'id acte  transmettre
     * @param listErrors la liste contenant les messages d'erreurs
     * @throws IOException IOException
     * @return true si la transmission s'est bien droule, false sinon
     */
    public synchronized boolean sendActe( String[] strIdActe, List<String> listErrors )
        throws IOException
    {
        Plugin plugin = getPlugin(  );
        boolean bTransmis = false;

        for ( int i = 0; i < strIdActe.length; i++ )
        {
            List<byte[]> listFile = new ArrayList<byte[]>(  );

            Acte<GSeance, GFichier> acte = _acteHome.findByPrimaryKey( Integer.parseInt( strIdActe[i] ), plugin );

            // rcupration des pice jointe suivant le type d'entit
            int nIdTypeEntite = acte.getIdTypeEntite(  );

            if ( ( nIdTypeEntite == ITypeEntite.PROJETS_DE_DELIBERATIONS ) ||
                    ( nIdTypeEntite == ITypeEntite.PROPOSITIONS_DE_DELIBERATIONS ) )
            {
                List<GFichier> listFichier = _documentActeHome.findByActe( acte, plugin );

                for ( GFichier file : listFichier )
                {
                    FichierPhysique fichierPhysique = file.getFichier(  );
                    fichierPhysique = _fichierPhysiqueHome.findByPrimaryKey( fichierPhysique.getIdFichier(  ), plugin );

                    byte[] b = fichierPhysique.getDonnees(  );
                    listFile.add( b );
                }
            }

            //rcupration des codes matires
            CategorieDeliberation catDelib = _categorieDeliberationHome.findByPrimaryKey( acte.getCategorie(  )
                                                                                              .getIdCategorie(  ),
                    plugin );
            CategorieCtrlLegalite catCtrlLegal = _categorieCtrlLegaliteHome.findByPrimaryKey( catDelib.getCategorieCtrlLegalite(  ),
                    plugin );

            if ( catCtrlLegal != null )
            {
                //Construction de l'objet de l'acte
                String strObjet = getMappingService(  ).get( acte.getIdTypeEntite(  ) ).getObjetActe( acte );

                if ( strObjet.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                {
                    strObjet = acte.getReference(  );
                }

                // rcupration de la dlibration finale
                GFichier deliberation = acte.getDeliberationFinale(  );
                deliberation = _fichierHome.findByPrimaryKey( deliberation.getId(  ), plugin );

                FichierPhysique deliberationPhysique = deliberation.getFichier(  );
                deliberationPhysique = _fichierPhysiqueHome.findByPrimaryKey( deliberationPhysique.getIdFichier(  ),
                        plugin );

                byte[] deliberationFinal = deliberationPhysique.getDonnees(  );

                //Transmission de l'acte
                FastService fastService = new FastService(  );
                bTransmis = fastService.sendActe( getNumeroInterneActe( acte ), listFile, deliberationFinal, strObjet,
                        catCtrlLegal.getCodeMatiere1(  ), catCtrlLegal.getCodeMatiere2(  ), getFormationConseil( acte ) );

                //Mise a jour du statut
                if ( bTransmis )
                {
                    StatutActe statutActe = new StatutActe(  );
                    statutActe.setIdStatut( StatutActeEnum.TRANSMIS.getId(  ) );
                    statutActe.setLibelle( _statutActeHome.findByPrimaryKey( StatutActeEnum.TRANSMIS.getId(  ), plugin )
                                                          .getLibelle(  ) );
                    acte.setStatutActe( statutActe );
                    _acteHome.update( acte, plugin );
                }
                else
                {
                    listErrors.add( OdsProperties.PROPERTY_MESSAGE_TRANSMISSION );
                }
            }
            else
            {
                AppLogService.error( "ActeService : sendActe -  La catgorie du contrle de lgalit n'est pas " +
                    "renseigne pour la catgorie de dlibration de code : " + catDelib.getCode(  ) );
                listErrors.add( MESSAGE_ERROR_ACTE_CATEGORIE );

                bTransmis = false;
            }
        }

        return bTransmis;
    }

    /**
     * Permet la transmission d'une annulation d'acte
     * @param nIdActe id de l'annulation de l'acte  transmettre
     * @throws IOException IOException
     * @return true si la transmission s'est bien droule, false sinon
     */
    public synchronized boolean sendAnnulationActe( int nIdActe )
        throws IOException
    {
        Plugin plugin = getPlugin(  );

        Acte<GSeance, GFichier> acte = _acteHome.findByPrimaryKey( nIdActe, plugin );

        //Transmission de l'acte
        FastService fastService = new FastService(  );
        boolean bTransmis = fastService.sendAnnulationActe( getNumeroInterneActe( acte ), getFormationConseil( acte ) );

        return bTransmis;
    }

    public String getVisualisationActionUrl(  )
    {
        return _strUrlActionVisualisation;
    }

    public void setVisualisationActionUrl( String strUrlActionVisualisation )
    {
        _strUrlActionVisualisation = strUrlActionVisualisation;
    }

    public void setUrlGestionActe( String strUrlGestionActe )
    {
        _strUrlGestionActe = strUrlGestionActe;
    }

    public String getUrlGestionActe(  )
    {
        return _strUrlGestionActe;
    }

    /**
     * Retourne l'ecran de confirmation de l'erreur de transmission si la personne est autorisee
     * sinon retourne un msg d'erreur
     * @param request la requete HTTP
     * @return String template
     */
    public HashMap<String, Object> getErreurTransmissionActe( HttpServletRequest request )
    {
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        String strIdActe = request.getParameter( OdsParameters.ID_ACTE );

        UrlItem url = new UrlItem( JSP_URL_DO_ERREUR_TRANSMISSION_ACTE );

        if ( strIdActe != null )
        {
            url.addParameter( OdsParameters.ID_ACTE, strIdActe );
        }

        model.put( MARK_URL, url.getUrl(  ) );

        int nIdActe = OdsUtils.getIntegerParameter( OdsParameters.ID_ACTE, request );
        Acte<GSeance, GFichier> acte = _acteHome.findByPrimaryKey( nIdActe, getPlugin(  ) );

        if ( acte != null )
        {
            model.put( MARK_ACTE, acte );
        }

        return model;
    }

    /**
     * Permet de passer l'acte a l'tat "Erreur de transmission"
     *
     * @param request
     *            la requte HTTP
     * @return strUrl url de retour
     */
    public String doErreurTransmissionActe( HttpServletRequest request )
    {
        int nIdActe = OdsUtils.getIntegerParameter( OdsParameters.ID_ACTE, request );

        String strConfirmationErreurTransmission = request.getParameter( PARAM_CHECKBOX_VALIDATE );

        if ( strConfirmationErreurTransmission == null )
        {
            return AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRMATION_TO_ERREUR_TRANSMISSION,
                AdminMessage.TYPE_WARNING );
        }
        else
        {
            Acte<GSeance, GFichier> acte = _acteHome.findByPrimaryKey( nIdActe, getPlugin(  ) );

            StatutActe statutActe = new StatutActe(  );
            statutActe.setIdStatut( StatutActeEnum.ERREUR_TRANSMISSION.getId(  ) );

            acte.setStatutActe( statutActe );

            _acteHome.update( acte, getPlugin(  ) );

            return AppPathService.getBaseUrl( request ) + getUrlGestionActe(  );
        }
    }

    /**
     * Class FichierContainer
     * @author rFlachaire
     *
     */
    protected final class FichierContainer
    {
        private GFichier _fichier;

        public FichierContainer(  )
        {
            _fichier = null;
        }

        public void setFichier( GFichier fichier )
        {
            _fichier = fichier;
        }

        public GFichier getFichier(  )
        {
            return _fichier;
        }
    }
}
