/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.acte;

import fr.paris.lutece.plugins.ods.dto.acte.IActeFilter;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.service.IODSService;
import fr.paris.lutece.util.ReferenceList;

import java.io.IOException;

import java.util.HashMap;
import java.util.List;
import java.util.Locale;

import javax.servlet.http.HttpServletRequest;


/**
 *
 * IActeService
 *
 * @param <GActeFilter> IActeFilter
 */
public interface IActeService<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>, GActeFilter extends IActeFilter<GFichier, GSeance>>
    extends IODSService
{
    /**
     * Methode qui initialise les variables se sessions si elles n'ont pas ts auparavant initialises
     */
    void init(  );

    /**
     * Methode qui initialise la session
     * @param request la requete
     */
    void initSession( HttpServletRequest request );

    /**
     * Initialisation du pdd
     * @param request la requte HTTP
     */
    String initPDD( HttpServletRequest request );

    /**
     *
     * @return le droit
     */
    String getRight(  );

    /**
     *
     * @return GActeFilter
     */
    GActeFilter getActeFilter(  );

    /**
     * Renvoie l'URL de gestion des actes
     * @return l'URL de gestion des actes
     */
    String getUrlGestionActe(  );

    /**
     * retourne l'interface de gestion des actes
     * @param request la requete HTTP
     * @return model la map contenant les lments utiles au template HTML pour affiche la liste des actes
     */
    HashMap<String, Object> getActeList( HttpServletRequest request );

    /**
     * Retourne la map contenant l'ensemble des lments utiles au template  la prparation de l'acte
     *
     * @param request la requete HTTP
     * @return HashMap<String, Object> la map contenant l'ensemble des lments utiles au template
     */
    HashMap<String, Object> getPreparationActe( HttpServletRequest request );

    /**
     * Retourne l'ecran de visualisation de l'acte suivant le type d'entit
     *
     * @param request la requete HTTP
     * @return String template
     */
    String getModificationActe( HttpServletRequest request );

    /**
     * Retourne la map contenant l'ensemble des lments utiles au templates
     *
     * @param request la requete HTTP
     * @return HashMap<String, Object> la map contenant l'ensemble des lments utiles au template
     */
    HashMap<String, Object> getModificationActePDD( HttpServletRequest request );

    /**
     * Retourne la map contenant l'ensemble des lments utiles au templates utile  la visualisation de l'acte
     *
     * @param request la requete HTTP
     @return HashMap<String, Object> la map contenant l'ensemble des lments utiles au template
     */
    HashMap<String, Object> getVisualisationActe( HttpServletRequest request );

    /**
     * Retourne la map contenant l'ensemble des lments utiles au template
     *
     * @param request la requete HTTP
     * @return model la map contenant l'ensemble des lments utiles au template
     */
    HashMap<String, Object> getConfirmFinaliser( HttpServletRequest request );

    /**
     * Retourne l'ecran de confirmation de rejet
     *
     * @param request la requete HTTP
     * @return model la map contenant l'ensemble des lments utiles au template
     */
    HashMap<String, String> getConfirmRejeter( HttpServletRequest request );

    /**
     * Retourne la map contenant l'ensemble des lments utiles au template pour la reception de l'acte
     *
     * @param request la requete HTTP
     * @return model la map contenant l'ensemble des lments utiles au template
     */
    HashMap<String, Object> getReceptionActe( HttpServletRequest request );

    /**
     * Retourne la map contenant l'ensemble des lments utiles au template pour la suppression d'un acte
     *
     * @param request
     *            la requte Http
     * @return model la map contenant l'ensemble des lments utiles au template pour la suppression d'un acte
     */
    HashMap<String, String> getSuppressionActe( HttpServletRequest request );

    /**
     * Retourne la map contenant l'ensemble des lments utiles au template pour l'annulation d'un acte
     *
     * @param request la requete HTTP
     * @return model la map contenant l'ensemble des lments utiles au template pour l'annulation d'un acte
     */
    HashMap<String, Object> getAnnulationActe( HttpServletRequest request );

    /**
     * Permet de rcuprer un objet rfrenceList contenant la liste des natures d'actes
     * @return referenceNatureActeList la liste des natures d'actes
     */
    ReferenceList getRefListNatureActe(  );

    /**
     * Cr la modification de l'acte
     *
     * @param request la requte HTTP
     * @param locale la locale
     * @return String template
     */
    String doPreparationActe( HttpServletRequest request, Locale locale );

    /**
     * Modifie l'acte
     *
     * @param request la requete HTTP
     * @param nTypeRetour l'id du type de retour
     * @return String l'URL de destination
     */
    String doModificationActe( HttpServletRequest request, int nTypeRetour );

    /**
     * Permet de retourner  la liste des actes
     *
     * @param request
     *            la requte HTTP
     * @return strUrl url de retour
     */
    String doVisualiserActe( HttpServletRequest request );

    /**
     * Permet de retourner  la liste de suivi des actes
     *
     * @param request
     *            la requte HTTP
     * @return strUrl url de retour
     */
    String doVisualiserActeForSuiviActe( HttpServletRequest request );

    /**
     * Permet de finaliser un acte
     *
     * @param request
     *            la requte HTTP
     * @return strUrl url de retour
     */
    String doFinaliserActe( HttpServletRequest request );

    /**
     * Transmet l'acte
     * @param request la requete
     * @return String la page de retour
     */
    String doTransmissionActe( HttpServletRequest request );

    /**
     * Permet de transmettre manuellement un acte
     *
     * @param request
     *            la requte HTTP
     * @return strUrl url de retour
     */
    String doTransmissionManuelleActe( HttpServletRequest request );

    /**
     * Permet de receptionner un acte (si transmis manuellement)
     *
     * @param request
     *            la requte HTTP
     * @return strUrl url de retour
     */
    String doReceptionActe( HttpServletRequest request );

    /**
     * Publie ou dpublie l'acte l'acte
     *
     * @param nIdActe l'id de l'acte
     * @param bIsPublie booleen pour savoir si l'acte est  publier ou pas
     */
    void doPublicationActe( int nIdActe, boolean bIsPublie );

    /**
     * Permet d'annuler un acte
     *
     * @param request
     *            la requte HTTP
     * @return strUrl url de retour
     */
    String doAnnulerActe( HttpServletRequest request );

    /**
     * Permet de cloner un acte
     *
     * @param request
     *            la requte HTTP
     * @return strUrl url de retour
     */
    String doClonerActe( HttpServletRequest request );

    /**
     * Effectue la suppression d'acte  partir des informations contenues dans la requte
     *
     * @param request
     *            la requte Http
     * @return l'url de gestion des actes
     */
    String doSuppressionActe( HttpServletRequest request );

    /**
     * Permet de rejetter un acte
     *
     * @param request
     *            la requte HTTP
     * @return strUrl url de retour
     */
    String doRejeterActe( HttpServletRequest request );

    /**
     * Telecharge un fichier. Renvoie un flux de bytes du fichier dans l'objet
     * HttpServletResponse response
     *
     * @param request la requete HTTP
     * @return fichierTelecharger le fichier  tlcharger
     */
    GFichier doDownloadFichier( HttpServletRequest request );

    /**
     * Permet la transmission d'un acte
     * @param strIdActe tableau d'id acte  transmettre
     * @param listErrors la liste contenant les messages d'erreurs
     * @throws IOException IOException
     * @return true si la transmission s'est bien droule, false sinon
     */
    boolean sendActe( String[] strIdActe, List<String> listErrors )
        throws IOException;

    /**
     * Permet la transmission d'une annulation d'acte
     * @param nIdActe id de l'annulation de l'acte  transmettre
     * @throws IOException IOException
     * @return true si la transmission s'est bien droule, false sinon
     */
    boolean sendAnnulationActe( int nIdActe ) throws IOException;

    /**
     * Retourne l'ecran de confirmation de l'erreur de transmission si la personne est autorisee
     * sinon retourne un msg d'erreur
     * @param request la requete HTTP
     * @return String template
     */
    HashMap<String, Object> getErreurTransmissionActe( HttpServletRequest request );

    /**
     * Permet de passer l'acte a l'tat "Erreur de transmission"
     *
     * @param request
     *            la requte HTTP
     * @return strUrl url de retour
     */
    String doErreurTransmissionActe( HttpServletRequest request );
}
