/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.arrete;

import fr.paris.lutece.plugins.ods.business.arrete.ArreteHome;
import fr.paris.lutece.plugins.ods.business.categoriedeliberation.CategorieDeliberationHome;
import fr.paris.lutece.plugins.ods.business.fichier.FichierPhysiqueHome;
import fr.paris.lutece.plugins.ods.business.fichier.IFichierHome;
import fr.paris.lutece.plugins.ods.business.formationconseil.FormationConseilHome;
import fr.paris.lutece.plugins.ods.business.historique.HistoriqueHome;
import fr.paris.lutece.plugins.ods.business.seance.ISeanceHome;
import fr.paris.lutece.plugins.ods.dto.IBusinessItem;
import fr.paris.lutece.plugins.ods.dto.arrete.Arrete;
import fr.paris.lutece.plugins.ods.dto.arrete.ArreteFilter;
import fr.paris.lutece.plugins.ods.dto.categoriedeliberation.CategorieDeliberation;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.FichierPhysique;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter;
import fr.paris.lutece.plugins.ods.dto.fichier.ITypeDocument;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.historique.Historique;
import fr.paris.lutece.plugins.ods.dto.horodatage.HorodatageActionEnum;
import fr.paris.lutece.plugins.ods.dto.horodatage.IEntiteHorodatage;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur;
import fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.typeentite.ITypeEntite;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.service.categoriedeliberation.ICategorieDeliberationService;
import fr.paris.lutece.plugins.ods.service.formationconseil.IFormationConseilService;
import fr.paris.lutece.plugins.ods.service.horodatage.IHorodatageService;
import fr.paris.lutece.plugins.ods.service.horodatage.IHorodatageServiceManaged;
import fr.paris.lutece.plugins.ods.service.search.ISearchService;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.commons.DateUtils;
import fr.paris.lutece.plugins.ods.utils.commons.ShaUtils;
import fr.paris.lutece.plugins.ods.utils.commons.StringUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.service.admin.AccessDeniedException;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.service.util.AppPathService;
import fr.paris.lutece.portal.service.util.AppPropertiesService;
import fr.paris.lutece.portal.web.constants.Messages;
import fr.paris.lutece.portal.web.upload.MultipartHttpServletRequest;
import fr.paris.lutece.util.html.Paginator;
import fr.paris.lutece.util.url.UrlItem;

import org.apache.commons.fileupload.FileItem;

import org.springframework.beans.factory.annotation.Autowired;

import java.sql.Timestamp;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.servlet.http.HttpServletRequest;


public abstract class AbstractArreteService<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>, GElu extends IElu, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GSeanceFilter extends ISeanceFilter, GRequeteUtilisateur extends IRequeteUtilisateur<GSeance, GSeanceFilter>, GResultatRequete extends IResultatRequete<GSeance, GSeanceFilter, GRequeteUtilisateur, GElu, GFichier, GVoeuAmendement, GPDD>>
    implements IArreteService<GSeance, GFichier>, IHorodatageServiceManaged
{
    protected static final String MARK_ID_FORMATION_CONSEIL_SELECTED = "id_formation_conseil_selected";
    private static final String JSP_URL_LISTE_ARRETES = "jsp/admin/plugins/ods/arretes/ArretesGestionAvalNoReinit.jsp";
    private static final String PROPERTY_ARRETES_PER_PAGE = "ods.itemPerPage.label";
    private static final String PROPERTY_URL_DO_SUPPRESSION_ARRETE_GESTION_AVAL_JSP = "jsp/admin/plugins/ods/arretes/DoSuppressionArreteGestionAval.jsp";
    private static final String PROPERTY_LABEL_SEANCE = "ods.creationarrete.label.seance";
    private static final String PROPERTY_LABEL_INTITULE = "ods.creationarrete.label.intitule";
    private static final String PROPERTY_LABEL_FICHIER_ORIGINAL = "ods.creationarrete.label.arreteoriginal";
    private static final String PROPERTY_LABEL_FICHIER_SCANNE = "ods.creationarrete.label.arretescanne";
    private static final String PROPERTY_LABEL_NUMERO = "ods.creationarrete.label.numero";
    private static final String PROPERTY_LABEL_FORMATION_CONSEIL = "ods.creationarrete.label.formationConseil";
    private static final String PROPERTY_LABEL_CATEGORIE = "ods.creationarrete.label.categorieDeDeliberation";
    private static final String PROPERTY_LABEL_DATE_RETOUR_CTRL_LEGALITE = "ods.creationarrete.label.dateControleLegalite";
    private static final String PROPERTY_LABEL_FICHIER_NO_PDF = "ods.creationarrete.label.fichier.noPDF";
    private static final String PROPERTY_MESSAGE_FILES_NOT_FOUND = "ods.arrete.error.filesnotfound";
    private static final String PROPERTY_CODE_DELIBERATION_DEFAUT = "ods.arrete.categoriedeliberation.defaut.code";
    private static final String MARK_LISTE_ARRETES = "liste_arretes";
    private static final String MARK_LISTE_HISTORIQUES = "liste_historiques";
    private static final String MARK_LISTE_ARRETES_SIZE_TOTAL = "liste_arretes_size_total";
    private static final String MARK_LISTE_CATEGORIE_DELIBERATION = "liste_categorie_deliberation";
    private static final String MARK_ID_CATEGORIE_DELIBERATION_SELECTED = "id_categorie_deliberation_selected";
    private static final String MARK_LISTE_SEANCES = "liste_seances";
    private static final String MARK_ID_SEANCE_SELECTED = "id_seance_selected";
    private static final String MESSAGE_CONFIRM_DELETE_ARRETE = "ods.message.confirmDeleteArrete";
    private static final int DEFAULT_CODE_DELIBERATION = 34;

    //Gre la sance slectionne
    private int _nIdSeanceSelected = -1;

    //Gre la formation conseil slectionne
    protected int _nIdFormationConseilSelected = -1;

    // Gere la page du Paginator
    private String _strCurrentPageIndex;

    //Gere le nombre d'item par page
    private int _nItemsPerPage;
    private int _nDefaultItemsPerPage;

    //gre si il ya eu des changements dans un arrete avant enregistrement
    private boolean _bHasChanged;
    @Autowired
    protected ArreteHome<GSeance, GFichier> _arreteHome;
    @Autowired
    protected IHorodatageService _horodatageService;
    @Autowired
    protected ISearchService<GSeance, GSeanceFilter, GRequeteUtilisateur, GResultatRequete, GElu, GFichier, GVoeuAmendement, GPDD> _searchService;
    @Autowired
    protected CategorieDeliberationHome _categorieDeliberationHome;
    @Autowired
    protected ICategorieDeliberationService _categorieDeliberationService;
    @Autowired
    protected IFichierHome<GFichier, IFichierFilter, ITypeDocument, GSeance> _fichierHome;
    @Autowired
    protected FichierPhysiqueHome _fichierPhysiqueHome;
    @Autowired
    protected IFormationConseilService _formationConseilService;
    @Autowired
    protected FormationConseilHome _formationConseilHome;
    @Autowired
    protected HistoriqueHome _historiqueHome;
    @Autowired
    protected ISeanceHome<GSeance, ISeanceFilter, GFichier, GElu, GVoeuAmendement, GPDD> _seanceHome;

    /**
     *  Creates a new instance of ArreteService
     */
    public AbstractArreteService(  )
    {
        _nDefaultItemsPerPage = AppPropertiesService.getPropertyInt( PROPERTY_ARRETES_PER_PAGE, 10 );
        _nIdFormationConseilSelected = -1;
        _nIdSeanceSelected = -1;
        _bHasChanged = false;
    }

    /**
     * Rajoute les parametres necessaires pour l'affichage
     * @param request HttpServletRequest
     * @param model le model
     */
    protected abstract void getArreteListSpec( HttpServletRequest request, HashMap<String, Object> model );

    /**
     * Rajoute les parametres necessaires pour l'affichage
     * @param request HttpServletRequest
     * @param model le model
     */
    protected abstract void getCreationArreteSpec( HttpServletRequest request, HashMap<String, Object> model );

    /**
     * Rajoute les parametres necessaires pour l'affichage
     * @param request HttpServletRequest
     * @param model le model
     */
    protected abstract void getModificationArreteSpec( HttpServletRequest request, HashMap<String, Object> model );

    /**
     * Rajoute les informations necessaires dans l'objet arrete
     * @param request HttpServletRequest
     * @param arrete l'arrete
     */
    protected abstract void doCreationArreteSpec( HttpServletRequest request, Arrete<GSeance, GFichier> arrete );

    /**
     * Rajoute les informations necessaires dans l'objet arrete
     * @param request HttpServletRequest
     * @param arrete l'arrete
     */
    protected abstract void doModificationArreteSpec( HttpServletRequest request, Arrete<GSeance, GFichier> arrete );

    /**
     * Rajoute les options supplmentaires dans le filtre
     * @param request la requete
     * @param arreteFilter le filtre
     */
    protected abstract void getArreteFilterSpec( HttpServletRequest request, ArreteFilter arreteFilter );

    /**
     * Cre une nouvelle instance d'IFichier
     * @return l'instance d'IFichier
     */
    protected abstract GFichier newFichierInstance(  );

    /**
     * Renvoie l'instance du PLugin
     * @return le plugin
     */
    protected abstract Plugin getPlugin(  );

    /**
     * Reinitialise tous les attributs de session
     * @param request requete HTTP
     * @throws AccessDeniedException   AccessDeniedException
     */
    public void reinitSession( HttpServletRequest request )
        throws AccessDeniedException
    {
        _bHasChanged = false;
        _nIdFormationConseilSelected = -1;
        _nIdSeanceSelected = -1;

        if ( -1 == _nDefaultItemsPerPage )
        {
            _nDefaultItemsPerPage = AppPropertiesService.getPropertyInt( PROPERTY_ARRETES_PER_PAGE, 10 );
        }
    }

    /**
     * retourne l'URL de la page d'accueil des arretes
     * @param request la requete
     * @return l'URL de la page d'accueil des arretes
     */
    private String getHomeUrl( HttpServletRequest request )
    {
        String strJspReturn = AppPathService.getBaseUrl( request );
        strJspReturn += JSP_URL_LISTE_ARRETES;

        return strJspReturn;
    }

    /**
      * Retourne l'interface de gestion des arretes.
      * @param request la requte http
      * @return model la map contenant les lments utiles au template HTML de la liste
      */
    public HashMap<String, Object> getArreteList( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        _strCurrentPageIndex = Paginator.getPageIndex( request, Paginator.PARAMETER_PAGE_INDEX, _strCurrentPageIndex );
        _nItemsPerPage = Paginator.getItemsPerPage( request, Paginator.PARAMETER_ITEMS_PER_PAGE, _nItemsPerPage,
                _nDefaultItemsPerPage );

        ArreteFilter filter = new ArreteFilter(  );

        // on recupre la liste des arretes correspondants au filtre
        if ( request.getParameter( OdsParameters.ID_SEANCE ) != null )
        {
            _nIdSeanceSelected = OdsUtils.getIntegerParameter( OdsParameters.ID_SEANCE, request );
        }

        filter.setIdSeance( _nIdSeanceSelected );

        getArreteFilterSpec( request, filter );

        List<Arrete<GSeance, GFichier>> arretes = _arreteHome.findByFilter( filter, plugin );

        Paginator paginator = new Paginator( arretes, _nItemsPerPage, getHomeUrl( request ),
                Paginator.PARAMETER_PAGE_INDEX, _strCurrentPageIndex );

        // Ajouts des lments ncessaire dans la Hashmap
        model.put( MARK_LISTE_SEANCES, _seanceHome.findOldSeance( plugin ) );
        model.put( MARK_ID_SEANCE_SELECTED, _nIdSeanceSelected );

        model.put( MARK_LISTE_ARRETES, paginator.getPageItems(  ) );
        model.put( MARK_LISTE_ARRETES_SIZE_TOTAL, arretes.size(  ) );
        model.put( OdsMarks.MARK_PAGINATOR, paginator );
        model.put( OdsMarks.MARK_NB_ITEMS_PER_PAGE, OdsConstants.CONSTANTE_CHAINE_VIDE + _nItemsPerPage );

        getArreteListSpec( request, model );

        return model;
    }

    /**
     * retourne la map contenant les lments utiles au template HTML de la creation
     * @param request la requte http
     * @return model la map contenant les lments utiles au template HTML de la creation
     */
    public HashMap<String, Object> getCreationArrete( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        HashMap<String, Object> model = new HashMap<String, Object>(  );

        model.put( MARK_LISTE_SEANCES, _seanceHome.findOldSeance( plugin ) );
        model.put( MARK_ID_SEANCE_SELECTED, _nIdSeanceSelected );
        model.put( MARK_LISTE_CATEGORIE_DELIBERATION,
            _categorieDeliberationService.initRefListCategorieDeliberation( plugin, false ) );

        int nCodeCategorieDefaut = AppPropertiesService.getPropertyInt( PROPERTY_CODE_DELIBERATION_DEFAUT,
                DEFAULT_CODE_DELIBERATION );
        int nIdCategorieDefaut = _categorieDeliberationHome.findIdCategorieByCode( nCodeCategorieDefaut, plugin );
        model.put( MARK_ID_CATEGORIE_DELIBERATION_SELECTED, nIdCategorieDefaut );

        getCreationArreteSpec( request, model );

        return model;
    }

    /**
     * retourne la map contenant les lments utiles au template HTML de la modification
     * @param request la requte http
     * @return model la map contenant les lments utiles au template HTML de la modification
     */
    public HashMap<String, Object> getModificationArrete( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        int nIdArrete = OdsUtils.getIntegerParameter( OdsParameters.ID_ARRETE, request );
        Arrete<GSeance, GFichier> arrete = _arreteHome.findByPrimaryKey( nIdArrete, plugin );

        if ( arrete == null )
        {
            return null;
        }

        model.put( OdsMarks.MARK_ARRETE, arrete );
        model.put( MARK_LISTE_SEANCES, _seanceHome.findOldSeance( plugin ) );
        model.put( MARK_LISTE_CATEGORIE_DELIBERATION,
            _categorieDeliberationService.initRefListCategorieDeliberation( plugin, false ) );

        getModificationArreteSpec( request, model );

        return model;
    }

    /**
     * retourne la map contenant les lments utiles au template HTML de l'historique
     * @param request la requte http
     * @return model la map contenant les lments utiles au template HTML de l'historique
     */
    public HashMap<String, Object> getHistoriqueArrete( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        int nIdArrete = OdsUtils.getIntegerParameter( OdsParameters.ID_ARRETE, request );
        Arrete<GSeance, GFichier> arrete = _arreteHome.findByPrimaryKey( nIdArrete, plugin );

        if ( arrete == null )
        {
            return null;
        }

        List<Historique> historiques = _historiqueHome.getHistoriqueList( arrete, plugin );
        model.put( MARK_LISTE_HISTORIQUES, historiques );

        return model;
    }

    /**
     * retourne une demande de confirmation pour la suppression de l'arrete choisi
     * @param request la requte http
     * @return la page de confirmation de suppression d'un arrete
     */
    public String getSuppressionArrete( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        if ( request.getParameter( OdsParameters.ID_ARRETE ) != null )
        {
            int nIdArrete = OdsUtils.getIntegerParameter( OdsParameters.ID_ARRETE, request );
            Arrete<GSeance, GFichier> arrete = _arreteHome.findByPrimaryKey( nIdArrete, plugin );

            if ( arrete == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            UrlItem url = new UrlItem( PROPERTY_URL_DO_SUPPRESSION_ARRETE_GESTION_AVAL_JSP );
            url.addParameter( OdsParameters.ID_ARRETE, nIdArrete );

            return AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRM_DELETE_ARRETE, url.getUrl(  ),
                AdminMessage.TYPE_CONFIRMATION );
        }
        else
        {
            return getHomeUrl( request );
        }
    }

    /**
     * Creation d'un arrete
     * si tous les champs obligatoires du formulaire de cration d'un arrete ont t renseign,<BR>
     * la mthode retourne l'url de gestion des arretes,<BR>
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request la requte http
     * @return l'url de creation d'un arrete
     */
    public String doCreationArrete( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        List<String> champsNonSaisie = new ArrayList<String>(  );
        String strChampsError = OdsConstants.CONSTANTE_CHAINE_VIDE;

        //Si on a cliqu sur le bouton "Annuler"
        if ( request.getParameter( OdsParameters.CONSTANTE_ACTION_CANCEL ) != null )
        {
            return AppPathService.getBaseUrl( request ) + JSP_URL_LISTE_ARRETES;
        }

        if ( isValideForme( champsNonSaisie, request, true ) )
        {
            MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;

            Arrete<GSeance, GFichier> arrete = new Arrete<GSeance, GFichier>(  );

            arrete.setIntitule( request.getParameter( OdsParameters.ARRETE_INTITULE ).trim(  ) );
            arrete.setNumero( request.getParameter( OdsParameters.ARRETE_NUMERO ).trim(  ) );
            arrete.setEnLigne( false );

            int nModificationIdSeance = OdsUtils.getIntegerParameter( OdsParameters.ID_SEANCE, request );
            int nIdFormationConseil = OdsUtils.getIntegerParameter( OdsParameters.ID_FORMATION_CONSEIL, request );
            int nIdCategorie = OdsUtils.getIntegerParameter( OdsParameters.ID_CATEGORIE, request );

            GSeance seance = _seanceHome.findByPrimaryKey( nModificationIdSeance, plugin );
            arrete.setSeance( seance );

            FormationConseil conseil = _formationConseilHome.findByPrimaryKey( nIdFormationConseil, plugin );
            arrete.setFormationConseil( conseil );

            CategorieDeliberation categorie = _categorieDeliberationHome.findByPrimaryKey( nIdCategorie, plugin );
            arrete.setCategorie( categorie );

            /* Vrification de l'extension du fichier de l'arrt original */
            FileItem fileItemOriginale = multipartRequest.getFile( OdsParameters.ARRETE_OR );

            if ( ( fileItemOriginale != null ) && ( fileItemOriginale.getName(  ) != null ) &&
                    !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItemOriginale.getName(  ) ) )
            {
                String strExtension = fileItemOriginale.getName(  )
                                                       .substring( fileItemOriginale.getName(  ).length(  ) - 3 )
                                                       .toUpperCase(  );

                if ( ( strExtension == null ) || ( strExtension.length(  ) != 3 ) ||
                        !OdsConstants.CONSTANTE_PDF.equals( strExtension ) )
                {
                    strChampsError = PROPERTY_LABEL_FICHIER_ORIGINAL;
                }
            }

            /* Vrification de l'extension du fichier de l'arrt original */
            FileItem fileItemScannee = multipartRequest.getFile( OdsParameters.ARRETE_SC );

            if ( ( fileItemScannee != null ) && ( fileItemScannee.getName(  ) != null ) &&
                    !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItemScannee.getName(  ) ) )
            {
                String strExtension = fileItemScannee.getName(  ).substring( fileItemScannee.getName(  ).length(  ) -
                        3 ).toUpperCase(  );

                if ( ( strExtension == null ) || ( strExtension.length(  ) != 3 ) ||
                        !OdsConstants.CONSTANTE_PDF.equals( strExtension ) )
                {
                    if ( strChampsError == OdsConstants.CONSTANTE_CHAINE_VIDE )
                    {
                        strChampsError = PROPERTY_LABEL_FICHIER_SCANNE;
                    }
                }
            }

            if ( !strChampsError.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                Object[] messagesArgs = { I18nService.getLocalizedString( strChampsError, request.getLocale(  ) ), };

                return AdminMessageService.getMessageUrl( request, PROPERTY_LABEL_FICHIER_NO_PDF, messagesArgs,
                    AdminMessage.TYPE_STOP );
            }
            else
            {
                /* Cration du fichier de l'arrt original */
                createFichier( arrete, fileItemOriginale, true );
                /* Cration du fichier de l'arrt scann */
                createFichier( arrete, fileItemScannee, false );
            }

            //rcupration de la date de retour du controle de lgalit
            if ( ( request.getParameter( OdsParameters.DATE_RETOUR_CONTROLE_LEGALITE ) != null ) &&
                    !request.getParameter( OdsParameters.DATE_RETOUR_CONTROLE_LEGALITE ).trim(  )
                                .equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                String strDateRetourCtrlLegalite = request.getParameter( OdsParameters.DATE_RETOUR_CONTROLE_LEGALITE )
                                                          .trim(  );
                Timestamp tsDateRetourCtrlLegalite = DateUtils.getDate( strDateRetourCtrlLegalite, true );

                if ( tsDateRetourCtrlLegalite == null )
                {
                    Object[] messagesArgs = 
                        {
                            I18nService.getLocalizedString( PROPERTY_LABEL_DATE_RETOUR_CTRL_LEGALITE,
                                request.getLocale(  ) ),
                        };

                    return AdminMessageService.getMessageUrl( request,
                        OdsProperties.PROPERTY_MESSAGE_DATE_FORMAT_ERROR, messagesArgs, AdminMessage.TYPE_ERROR );
                }
                else
                {
                    arrete.setDateRetourControleLegalite( tsDateRetourCtrlLegalite );
                }
            }
            else if ( ( request.getParameter( OdsParameters.DATE_RETOUR_CONTROLE_LEGALITE ) != null ) &&
                    request.getParameter( OdsParameters.DATE_RETOUR_CONTROLE_LEGALITE ).trim(  )
                               .equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                arrete.setDateRetourControleLegalite( null );
            }

            // Cration de l'arrt
            arrete.setId( _arreteHome.create( arrete, plugin ) );

            if ( ( arrete != null ) && ( arrete.getSeance(  ) != null ) )
            {
                _searchService.addObjectToIndex( arrete, true, plugin );
            }
        }
        else
        {
            String strChamp = ( champsNonSaisie.get( 0 ) != null ) ? champsNonSaisie.get( 0 ) : Messages.MANDATORY_FIELDS;
            Object[] messagesArgs = { I18nService.getLocalizedString( strChamp, request.getLocale(  ) ) };

            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED,
                messagesArgs, AdminMessage.TYPE_STOP );
        }

        return getHomeUrl( request );
    }

    /**
     * Modifie l'arrete
     * Si l'arrete est en ligne, alors la version est incrmente de 1 et ajout d'un historique
     * si tous les champs obligatoires du formulaire de modification d'un arrete ont t renseigns,<BR>
     * la mthode retourne l'url de gestion des arretes,<BR>
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request la requte http
     * @return l'url de modification d'un arrete
     */
    public String doModificationArrete( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        List<String> champsNonSaisie = new ArrayList<String>(  );
        String strChampsError = OdsConstants.CONSTANTE_CHAINE_VIDE;
        _bHasChanged = false;

        //Si on a cliqu sur le bouton "Annuler"
        if ( request.getParameter( OdsParameters.CONSTANTE_ACTION_CANCEL ) != null )
        {
            return AppPathService.getBaseUrl( request ) + JSP_URL_LISTE_ARRETES;
        }

        if ( isValideForme( champsNonSaisie, request, false ) )
        {
            int nIdArrete = OdsUtils.getIntegerParameter( OdsParameters.ID_ARRETE, request );
            MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;

            if ( ( nIdArrete != -1 ) )
            {
                Arrete<GSeance, GFichier> arrete = _arreteHome.findByPrimaryKey( nIdArrete, plugin );

                if ( arrete == null )
                {
                    return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                        AdminMessage.TYPE_ERROR );
                }

                // recupration des modifications et mise  jour des champs si besoin
                int nModificationIdSeance = OdsUtils.getIntegerParameter( OdsParameters.ID_SEANCE, request );

                if ( nModificationIdSeance != arrete.getSeance(  ).getIdSeance(  ) )
                {
                    GSeance seance = _seanceHome.findByPrimaryKey( nModificationIdSeance, plugin );
                    arrete.setSeance( seance );
                    _bHasChanged = true;
                }

                int nModificationIdFormationConseil = OdsUtils.getIntegerParameter( OdsParameters.ID_FORMATION_CONSEIL,
                        request );

                if ( nModificationIdFormationConseil != arrete.getFormationConseil(  ).getIdFormationConseil(  ) )
                {
                    FormationConseil conseil = _formationConseilHome.findByPrimaryKey( nModificationIdFormationConseil,
                            plugin );
                    arrete.setFormationConseil( conseil );
                    _bHasChanged = true;
                }

                String strModificationNumero = request.getParameter( OdsParameters.ARRETE_NUMERO ).trim(  );

                if ( !strModificationNumero.equals( arrete.getNumero(  ) ) )
                {
                    arrete.setNumero( strModificationNumero );
                    _bHasChanged = true;
                }

                String strModificationIntitule = request.getParameter( OdsParameters.ARRETE_INTITULE ).trim(  );

                if ( !strModificationIntitule.equals( arrete.getIntitule(  ) ) )
                {
                    arrete.setIntitule( strModificationIntitule );
                    _bHasChanged = true;
                }

                /* Vrification de l'extension du fichier de l'arrt oiriginal */
                FileItem fileItemOr = multipartRequest.getFile( OdsParameters.ARRETE_OR );

                if ( ( fileItemOr != null ) && ( fileItemOr.getName(  ) != null ) &&
                        !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItemOr.getName(  ) ) )
                {
                    String strExtension = fileItemOr.getName(  ).substring( fileItemOr.getName(  ).length(  ) - 3 )
                                                    .toUpperCase(  );

                    if ( ( strExtension == null ) || ( strExtension.length(  ) != 3 ) ||
                            !OdsConstants.CONSTANTE_PDF.equals( strExtension ) )
                    {
                        strChampsError = PROPERTY_LABEL_FICHIER_ORIGINAL;
                    }
                }

                /* Vrification de l'extension du fichier de l'arrt scann */
                FileItem fileItemSc = multipartRequest.getFile( OdsParameters.ARRETE_SC );

                if ( ( fileItemSc != null ) && ( fileItemSc.getName(  ) != null ) &&
                        !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItemSc.getName(  ) ) )
                {
                    String strExtension = fileItemSc.getName(  ).substring( fileItemSc.getName(  ).length(  ) - 3 )
                                                    .toUpperCase(  );

                    if ( ( ( strExtension == null ) || ( strExtension.length(  ) != 3 ) ||
                            !OdsConstants.CONSTANTE_PDF.equals( strExtension ) ) &&
                            strChampsError.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                    {
                        if ( strChampsError == OdsConstants.CONSTANTE_CHAINE_VIDE )
                        {
                            strChampsError = PROPERTY_LABEL_FICHIER_SCANNE;
                        }
                    }
                }

                if ( !strChampsError.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                {
                    Object[] messagesArgs = { I18nService.getLocalizedString( strChampsError, request.getLocale(  ) ), };

                    return AdminMessageService.getMessageUrl( request, PROPERTY_LABEL_FICHIER_NO_PDF, messagesArgs,
                        AdminMessage.TYPE_STOP );
                }
                else
                {
                    /* Mise  jour du fichier de l'arrt original */
                    updateFichier( arrete, fileItemOr, true );
                    /* Mise  jour du fichier de l'arrte scann */
                    updateFichier( arrete, fileItemSc, false );
                }

                int nModificationIdFCategorie = OdsUtils.getIntegerParameter( OdsParameters.ID_CATEGORIE, request );

                if ( nModificationIdFCategorie != arrete.getCategorie(  ).getIdCategorie(  ) )
                {
                    CategorieDeliberation categorie = _categorieDeliberationHome.findByPrimaryKey( nModificationIdFCategorie,
                            plugin );
                    arrete.setCategorie( categorie );
                    _bHasChanged = true;
                }

                String strModificationDateRetourControleLegalite = request.getParameter( OdsParameters.DATE_RETOUR_CONTROLE_LEGALITE )
                                                                          .trim(  );
                Timestamp tsDateRetourControleLegalite = DateUtils.getDate( strModificationDateRetourControleLegalite,
                        true );

                if ( ( tsDateRetourControleLegalite == null ) &&
                        !strModificationDateRetourControleLegalite.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                {
                    Object[] messagesArgs = 
                        {
                            I18nService.getLocalizedString( PROPERTY_LABEL_DATE_RETOUR_CTRL_LEGALITE,
                                request.getLocale(  ) ),
                        };

                    return AdminMessageService.getMessageUrl( request,
                        OdsProperties.PROPERTY_MESSAGE_DATE_FORMAT_ERROR, messagesArgs, AdminMessage.TYPE_ERROR );
                }
                else
                {
                    if ( ( ( arrete.getDateRetourControleLegalite(  ) == null ) &&
                            ( tsDateRetourControleLegalite != null ) ) ||
                            ( ( arrete.getDateRetourControleLegalite(  ) != null ) &&
                            ( tsDateRetourControleLegalite != null ) &&
                            !tsDateRetourControleLegalite.equals( arrete.getDateRetourControleLegalite(  ) ) ) ||
                            ( ( arrete.getDateRetourControleLegalite(  ) != null ) &&
                            ( tsDateRetourControleLegalite == null ) ) )
                    {
                        arrete.setDateRetourControleLegalite( tsDateRetourControleLegalite );
                        _bHasChanged = true;
                    }
                }

                if ( arrete.getEnLigne(  ) && _bHasChanged )
                {
                    arrete.setVersion( arrete.getVersion(  ) + 1 );

                    Timestamp dateForVersion = DateUtils.getCurrentDate(  );

                    //gestion de l'historique de l'arrt
                    Historique historique = new Historique(  );
                    historique.setVersion( arrete.getVersion(  ) );
                    historique.setDatePublication( dateForVersion );
                    historique.setIdEntite( arrete.getId(  ) );
                    historique.setIdTypeEntite( ITypeEntite.ARRETES );
                    _historiqueHome.create( historique, plugin );
                }

                if ( ( arrete != null ) && ( arrete.getSeance(  ) != null ) )
                {
                    _searchService.updateObjectInIndex( arrete, true, plugin );
                }

                // On met  jour l'arrt
                _arreteHome.update( arrete, plugin );
            }
            else
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_STOP );
            }
        }
        else
        {
            String strChamp = ( champsNonSaisie.get( 0 ) != null ) ? champsNonSaisie.get( 0 ) : Messages.MANDATORY_FIELDS;
            Object[] messagesArgs = { I18nService.getLocalizedString( strChamp, request.getLocale(  ) ) };

            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED,
                messagesArgs, AdminMessage.TYPE_STOP );
        }

        return getHomeUrl( request );
    }

    /**
     * Publication d'un arrete
     * Si "OdsParameters.ARRETER_IS_PUBLIER" gale  TRUE, alors la mthode <b>publie</b> l'arrete et incrmente la version<BR>
     *                 Dans ce cas, un historique est cre
     * Sinon la mthode change le statut enLigne d'un arrete a FALSE
     * @param request la requte http
     * @return l'url
     */
    public String doPublicationArrete( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        Timestamp dateForVersion = DateUtils.getCurrentDate(  );

        String strIsPublier = request.getParameter( OdsParameters.ARRETE_IS_PUBLIER );
        boolean bEstPublication = Boolean.parseBoolean( OdsConstants.CONSTANTE_CHAINE_VIDE + strIsPublier );

        int nIdArrete = OdsUtils.getIntegerParameter( OdsParameters.ID_ARRETE, request );

        if ( nIdArrete != -1 )
        {
            Arrete<GSeance, GFichier> arrete = _arreteHome.findByPrimaryKey( nIdArrete, plugin );

            if ( arrete == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            if ( ( arrete.getFichierOr(  ) == null ) || ( arrete.getFichierSc(  ) == null ) )
            {
                return AdminMessageService.getMessageUrl( request, PROPERTY_MESSAGE_FILES_NOT_FOUND,
                    AdminMessage.TYPE_STOP );
            }

            if ( !( ( arrete.getEnLigne(  ) && bEstPublication ) || ( !arrete.getEnLigne(  ) && !bEstPublication ) ) )
            {
                if ( bEstPublication && ( arrete.getVersion(  ) < 1 ) )
                {
                    arrete.setVersion( 0 );
                }

                //On publie l'arrete
                _arreteHome.publication( arrete.getId(  ), dateForVersion, arrete.getVersion(  ), bEstPublication,
                    plugin );

                //On publie les fichiers associs
                _fichierHome.publication( arrete.getFichierOr(  ).getId(  ), dateForVersion, arrete.getVersion(  ),
                    bEstPublication, plugin );
                _fichierHome.publication( arrete.getFichierSc(  ).getId(  ), dateForVersion, arrete.getVersion(  ),
                    bEstPublication, plugin );

                if ( bEstPublication )
                {
                    Historique historique = new Historique(  );
                    historique.setVersion( arrete.getVersion(  ) + 1 );
                    historique.setDatePublication( dateForVersion );
                    historique.setIdEntite( arrete.getId(  ) );
                    historique.setIdTypeEntite( ITypeEntite.ARRETES );
                    _historiqueHome.create( historique, plugin );
                }

                // on horodate la (de)publication
                HorodatageActionEnum action = ( bEstPublication ) ? HorodatageActionEnum.PUBLICATION
                                                                  : HorodatageActionEnum.DE_PUBLICATION;
                _horodatageService.tracePublication( arrete, action );
            }
        }

        return getHomeUrl( request );
    }

    /**
     * supprime l'arrete choisi
     * @param request la requte http
     * @return l'url de de la liste des arretes
     */
    public String doSuppressionArrete( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        int nIdArrete = OdsUtils.getIntegerParameter( OdsParameters.ID_ARRETE, request );

        if ( nIdArrete != -1 )
        {
            Arrete<GSeance, GFichier> arrete = _arreteHome.findByPrimaryKey( nIdArrete, plugin );

            if ( arrete == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            //Suppression des fichiers rattachs
            GFichier fichierOr = arrete.getFichierOr(  );
            GFichier fichierSc = arrete.getFichierSc(  );

            _historiqueHome.remove( arrete, plugin );

            if ( ( arrete != null ) && ( arrete.getSeance(  ) != null ) )
            {
                _searchService.deleteObjectFromIndex( arrete, true, plugin );
            }

            _arreteHome.remove( arrete, plugin );

            if ( fichierOr != null )
            {
                fichierOr = _fichierHome.findByPrimaryKey( fichierOr.getId(  ), plugin );

                FichierPhysique fichierPhysiqueOr = _fichierPhysiqueHome.findByPrimaryKey( fichierOr.getFichier(  )
                                                                                                    .getIdFichier(  ),
                        plugin );

                _historiqueHome.remove( fichierOr, plugin );

                if ( fichierOr.getSeance(  ) != null )
                {
                    ISeance seanceByFichier = fichierOr.getSeance(  );
                    ISeance nextSeance = _seanceHome.getProchaineSeance( plugin );

                    if ( ( nextSeance != null ) && !seanceByFichier.equals( nextSeance ) &&
                            ( ( fichierOr.getTypdeDocument(  ).getId(  ) == ITypeDocument.DOCUMENT_LIBRE ) ||
                            ( fichierOr.getTypdeDocument(  ).getId(  ) == ITypeDocument.COMPTE_RENDU ) ||
                            ( fichierOr.getTypdeDocument(  ).getId(  ) == ITypeDocument.LISTE_DE_DESIGNATION ) ) )
                    {
                        _searchService.deleteObjectFromIndex( fichierOr, true, plugin );
                    }
                }

                _fichierHome.remove( fichierOr, plugin );
                _fichierPhysiqueHome.remove( fichierPhysiqueOr, plugin );
            }

            if ( fichierSc != null )
            {
                fichierSc = _fichierHome.findByPrimaryKey( fichierSc.getId(  ), plugin );

                FichierPhysique fichierPhysiqueSc = _fichierPhysiqueHome.findByPrimaryKey( fichierSc.getFichier(  )
                                                                                                    .getIdFichier(  ),
                        plugin );

                _historiqueHome.remove( fichierSc, plugin );

                if ( fichierSc.getSeance(  ) != null )
                {
                    ISeance seanceByFichier = fichierSc.getSeance(  );
                    ISeance nextSeance = _seanceHome.getProchaineSeance( plugin );

                    if ( ( nextSeance != null ) && !seanceByFichier.equals( nextSeance ) &&
                            ( ( fichierSc.getTypdeDocument(  ).getId(  ) == ITypeDocument.DOCUMENT_LIBRE ) ||
                            ( fichierSc.getTypdeDocument(  ).getId(  ) == ITypeDocument.COMPTE_RENDU ) ||
                            ( fichierSc.getTypdeDocument(  ).getId(  ) == ITypeDocument.LISTE_DE_DESIGNATION ) ) )
                    {
                        _searchService.deleteObjectFromIndex( fichierSc, true, plugin );
                    }
                }

                _fichierHome.remove( fichierSc, plugin );
                _fichierPhysiqueHome.remove( fichierPhysiqueSc, plugin );
            }
        }

        return getHomeUrl( request );
    }

    /**
     * Tlcharge un fichier. Renvoie un flux de bytes du fichier dans l'objet
     * HttpServletResponse response
     *
     * @param request la requte HTTP
     * @return fichierTelecharger le fichier  tlcharger
     */
    public GFichier doDownloadFichier( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        GFichier fichierTelecharger = null;
        String strIdFichier = request.getParameter( OdsParameters.ID_FICHIER );

        if ( strIdFichier != null )
        {
            int nIdFichier = OdsUtils.getIntegerParameter( OdsParameters.ID_FICHIER, request );

            fichierTelecharger = _fichierHome.findByPrimaryKey( nIdFichier, plugin );

            FichierPhysique fichierPhysique = _fichierPhysiqueHome.findByPrimaryKey( fichierTelecharger.getFichier(  )
                                                                                                       .getIdFichier(  ),
                    plugin );
            fichierTelecharger.setFichier( fichierPhysique );
        }

        return fichierTelecharger;
    }

    /*
     *
     * ********** TRAITEMENT METIER **********
     *
     */

    /**
     * methode qui test si dans le formulaire de saisi d'un arrete,l'ensemble des critres obligatoires ont t renseign
     * @param request la requte http
     * @param champsNonSaisie List<String>
     * @param isCreation true si on est en cration, false si on est en modification
     * @return VRAI si le formulaire envoy est valide
     */
    private boolean isValideForme( List<String> champsNonSaisie, HttpServletRequest request, boolean isCreation )
    {
        boolean isValid = true;

        if ( OdsUtils.getIntegerParameter( OdsParameters.ID_SEANCE, request ) == -1 )
        {
            champsNonSaisie.add( PROPERTY_LABEL_SEANCE );
            isValid = false;
        }

        if ( OdsUtils.getIntegerParameter( OdsParameters.ID_FORMATION_CONSEIL, request ) == -1 )
        {
            champsNonSaisie.add( PROPERTY_LABEL_FORMATION_CONSEIL );
            isValid = false;
        }

        String strNumero = request.getParameter( OdsParameters.ARRETE_NUMERO ).trim(  );

        if ( ( strNumero == null ) || OdsConstants.CONSTANTE_CHAINE_VIDE.equals( strNumero ) )
        {
            champsNonSaisie.add( PROPERTY_LABEL_NUMERO );
            isValid = false;
        }

        String strCreationIntitule = request.getParameter( OdsParameters.ARRETE_INTITULE ).trim(  );

        if ( ( strCreationIntitule == null ) || OdsConstants.CONSTANTE_CHAINE_VIDE.equals( strCreationIntitule ) )
        {
            champsNonSaisie.add( PROPERTY_LABEL_INTITULE );
            isValid = false;
        }

        if ( isCreation )
        {
            MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
            FileItem fileItem = multipartRequest.getFile( OdsParameters.ARRETE_OR );

            if ( ( fileItem == null ) || ( fileItem.getName(  ) == null ) ||
                    OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItem.getName(  ) ) )
            {
                champsNonSaisie.add( PROPERTY_LABEL_FICHIER_ORIGINAL );
                isValid = false;
            }
        }

        if ( OdsUtils.getIntegerParameter( OdsParameters.ID_CATEGORIE, request ) == -1 )
        {
            champsNonSaisie.add( PROPERTY_LABEL_CATEGORIE );
            isValid = false;
        }

        return isValid;
    }

    /**
     * Permet de creer un fichier du type qui est pass en paramtre et de
     * l'associ au pdd pass en parametre
     *
     * @param arrete Arrete
     * @param fileItem FileItem
     * @param isOriginale TRUE si le fichier en question est le fichier original, FALSE sinon
     */
    private void createFichier( final Arrete<GSeance, GFichier> arrete, final FileItem fileItem, boolean isOriginale )
    {
        Plugin plugin = getPlugin(  );

        if ( ( fileItem != null ) && ( fileItem.getName(  ) != null ) &&
                !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItem.getName(  ) ) )
        {
            GFichier newFichier = newFichierInstance(  );
            FichierPhysique newFichierPhysique = new FichierPhysique(  );

            newFichierPhysique.setDonnees( fileItem.get(  ) );
            newFichierPhysique.setIdFichier( _fichierPhysiqueHome.create( newFichierPhysique, plugin ) );
            newFichier.setFichier( newFichierPhysique );
            newFichier.setNom( StringUtils.getNameCourt( fileItem ) );

            String strExtension = fileItem.getName(  ).substring( fileItem.getName(  ).length(  ) - 3 ).toUpperCase(  );
            newFichier.setExtension( strExtension );

            newFichier.setTaille( (int) fileItem.getSize(  ) );

            newFichier.setSeance( arrete.getSeance(  ) );
            newFichier.setFormationConseil( arrete.getFormationConseil(  ) );
            newFichier.setTypdeDocument( _fichierHome.findTypeDocumentsById( ITypeDocument.ARRETE, plugin ) );
            newFichier.setIntitule( arrete.getIntitule(  ) );
            newFichier.setEnLigne( false );
            newFichier.setVersion( 1 );

            _fichierHome.create( newFichier, plugin );

            if ( newFichier.getSeance(  ) != null )
            {
                ISeance seanceByFichier = newFichier.getSeance(  );
                ISeance nextSeance = _seanceHome.getProchaineSeance( plugin );

                if ( ( nextSeance != null ) && !seanceByFichier.equals( nextSeance ) &&
                        ( ( newFichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.DOCUMENT_LIBRE ) ||
                        ( newFichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.COMPTE_RENDU ) ||
                        ( newFichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.LISTE_DE_DESIGNATION ) ) )
                {
                    _searchService.addObjectToIndex( newFichier, true, plugin );
                }
            }

            Timestamp dateForVersion = DateUtils.getCurrentDate(  );

            // Ajout de l'historique
            Historique historique = new Historique(  );
            historique.setVersion( 1 );
            historique.setDatePublication( dateForVersion );
            historique.setIdEntite( newFichier.getId(  ) );
            historique.setIdTypeEntite( ITypeEntite.FICHIERS );
            _historiqueHome.create( historique, plugin );

            if ( isOriginale )
            {
                arrete.setFichierOr( newFichier );
            }
            else
            {
                arrete.setFichierSc( newFichier );
            }
        }
    }

    /**
     * Permet de creer un fichier du type qui est pass en paramtre et de
     * l'associ au pdd pass en parametre
     *
     * @param arrete Arrete
     * @param fileItem FileItem
     * @param isOriginale TRUE si le fichier en question est le fichier original, FALSE sinon
     */
    private void updateFichier( Arrete<GSeance, GFichier> arrete, final FileItem fileItem, boolean isOriginale )
    {
        Plugin plugin = getPlugin(  );

        FichierPhysique fichierPhysique = new FichierPhysique(  );
        GFichier fichier;

        if ( ( fileItem != null ) && ( fileItem.getName(  ) != null ) &&
                !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItem.getName(  ) ) )
        {
            _bHasChanged = true;

            if ( isOriginale )
            {
                fichier = arrete.getFichierOr(  );
            }
            else
            {
                fichier = arrete.getFichierSc(  );
            }

            if ( fichier == null )
            {
                //on cre un fichier vide en base pour pouvoir le mettre  jour
                createFichier( arrete, fileItem, isOriginale );

                if ( isOriginale )
                {
                    fichier = arrete.getFichierOr(  );
                }
                else
                {
                    fichier = arrete.getFichierSc(  );
                }
            }

            fichierPhysique.setIdFichier( fichier.getFichier(  ).getIdFichier(  ) );
            // Modification de l'objet reprsentant le fichier physique
            fichierPhysique.setDonnees( fileItem.get(  ) );
            _fichierPhysiqueHome.update( fichierPhysique, plugin );
            fichier.setFichier( fichierPhysique );
            fichier.setNom( StringUtils.getNameCourt( fileItem ) );

            String strExtension = fileItem.getName(  ).substring( fileItem.getName(  ).length(  ) - 3 ).toUpperCase(  );
            fichier.setExtension( strExtension );
            fichier.setTaille( (int) fileItem.getSize(  ) );

            fichier.setSeance( arrete.getSeance(  ) );

            fichier.setIntitule( arrete.getIntitule(  ) );

            if ( arrete.getEnLigne(  ) )
            {
                Timestamp dateForVersion = DateUtils.getCurrentDate(  );
                // on incremente le numero de version
                fichier.setVersion( fichier.getVersion(  ) + 1 );
                fichier.setDatePublication( dateForVersion );

                //Ajout de l'historique
                Historique historique = new Historique(  );
                historique.setVersion( fichier.getVersion(  ) );
                historique.setDatePublication( dateForVersion );
                historique.setIdEntite( fichier.getId(  ) );
                historique.setIdTypeEntite( ITypeEntite.FICHIERS );
                _historiqueHome.create( historique, plugin );
            }

            if ( fichier.getSeance(  ) != null )
            {
                ISeance seanceByFichier = fichier.getSeance(  );
                ISeance nextSeance = _seanceHome.getProchaineSeance( plugin );

                if ( ( nextSeance != null ) && !seanceByFichier.equals( nextSeance ) &&
                        ( ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.DOCUMENT_LIBRE ) ||
                        ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.COMPTE_RENDU ) ||
                        ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.LISTE_DE_DESIGNATION ) ) )
                {
                    _searchService.updateObjectInIndex( fichier, true, plugin );
                }
            }

            _fichierHome.update( fichier, plugin );

            if ( fichier.getEnLigne(  ) )
            {
                _horodatageService.tracePublication( fichier, HorodatageActionEnum.MODIFICATION );
            }

            if ( isOriginale )
            {
                arrete.setFichierOr( fichier );
            }
            else
            {
                arrete.setFichierSc( fichier );
            }
        }
    }

    @SuppressWarnings( "unchecked" )
    public ISeance getSeance( IBusinessItem entite )
    {
        Plugin plugin = getPlugin(  );

        Arrete<GSeance, GFichier> arrete = (Arrete<GSeance, GFichier>) entite;

        ISeance seance = _seanceHome.findByPrimaryKey( arrete.getSeance(  ).getIdSeance(  ), plugin );

        return seance;
    }

    /**
     * Retourne la signature numriques SHA224 d'un arrete
     *
     * @param entite l'etnite
     * @return la signature
     */
    @SuppressWarnings( "unchecked" )
    public String getSignature( IEntiteHorodatage entite )
    {
        Plugin plugin = getPlugin(  );
        String signature = OdsConstants.CONSTANTE_CHAINE_VIDE;

        Arrete<GSeance, GFichier> arrete = (Arrete<GSeance, GFichier>) entite;
        GFichier fichierSc = arrete.getFichierSc(  );
        fichierSc.setFichier( _fichierPhysiqueHome.findByPrimaryKey( 
                arrete.getFichierSc(  ).getFichier(  ).getIdFichier(  ), plugin ) );
        arrete.setFichierSc( fichierSc );

        if ( ( arrete.getFichierSc(  ) != null ) && ( arrete.getFichierSc(  ).getFichier(  ) != null ) )
        {
            byte[] donnees = arrete.getFichierSc(  ).getFichier(  ).getDonnees(  );
            signature = ShaUtils.toSha224String( donnees );

            if ( signature == null )
            {
                signature = "error_" + arrete.getFichierSc(  ).getNom(  );
                AppLogService.error( "Erreur lors de la recuperation de la signature SHA 224 du fichier : " +
                    arrete.getFichierSc(  ).getNom(  ) );
            }
        }

        return signature;
    }

    /**
     * @return le parametre
     */
    public String getIdParameter(  )
    {
        return null;
    }

    /**
     *
     * @param item l'item
     * @return le type d'entite
     */
    public int getIndexerAction( IBusinessItem item )
    {
        return item.getTypeEntite(  );
    }

    /**
     *
     * @param nIdItem l'id de l'item
     * @return un arrete correspondant  l'id de l'item
     */
    public IBusinessItem createItemToDeleteInIndexer( int nIdItem )
    {
        Arrete<GSeance, GFichier> arrete = new Arrete<GSeance, GFichier>(  );
        arrete.setId( nIdItem );

        return arrete;
    }
}
