/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.arrete;

import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.service.IODSService;
import fr.paris.lutece.portal.service.admin.AccessDeniedException;

import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;


/**
 *
 * IArreteService
 *
 */
public interface IArreteService<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>>
    extends IODSService
{
    /**
     * Reinitialise tous les attributs de session
     * @param request requete HTTP
     * @throws AccessDeniedException   AccessDeniedException
     */
    void reinitSession( HttpServletRequest request ) throws AccessDeniedException;

    /**
     * Retourne l'interface de gestion des arretes.
     * @param request la requte http
     * @return model la map contenant les lments utiles au template HTML de la liste
     */
    HashMap<String, Object> getArreteList( HttpServletRequest request );

    /**
     * retourne la map contenant les lments utiles au template HTML de la creation
     * @param request la requte http
     * @return model la map contenant les lments utiles au template HTML de la creation
     */
    HashMap<String, Object> getCreationArrete( HttpServletRequest request );

    /**
     * retourne la map contenant les lments utiles au template HTML de la modification
     * @param request la requte http
     * @return model la map contenant les lments utiles au template HTML de la modification
     */
    HashMap<String, Object> getModificationArrete( HttpServletRequest request );

    /**
     * retourne la map contenant les lments utiles au template HTML de l'historique
     * @param request la requte http
     * @return model la map contenant les lments utiles au template HTML de l'historique
     */
    HashMap<String, Object> getHistoriqueArrete( HttpServletRequest request );

    /**
     * retourne une demande de confirmation pour la suppression de l'arrete choisi
     * @param request la requte http
     * @return la page de confirmation de suppression d'un arrete
     */
    String getSuppressionArrete( HttpServletRequest request );

    /**
     * Creation d'un arrete
     * si tous les champs obligatoires du formulaire de cration d'un arrete ont t renseign,<BR>
     * la mthode retourne l'url de gestion des arretes,<BR>
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request la requte http
     * @return l'url de creation d'un arrete
     */
    String doCreationArrete( HttpServletRequest request );

    /**
     * Modifie l'arrete
     * Si l'arrete est en ligne, alors la version est incrmente de 1 et ajout d'un historique
     * si tous les champs obligatoires du formulaire de modification d'un arrete ont t renseigns,<BR>
     * la mthode retourne l'url de gestion des arretes,<BR>
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request la requte http
     * @return l'url de modification d'un arrete
     */
    String doModificationArrete( HttpServletRequest request );

    /**
     * Publication d'un arrete
     * Si "OdsParameters.ARRETER_IS_PUBLIER" gale  TRUE, alors la mthode <b>publie</b> l'arrete et incrmente la version<BR>
     *                 Dans ce cas, un historique est cre
     * Sinon la mthode change le statut enLigne d'un arrete a FALSE
     * @param request la requte http
     * @return l'url
     */
    String doPublicationArrete( HttpServletRequest request );

    /**
     * supprime l'arrete choisi
     * @param request la requte http
     * @return l'url de de la liste des arretes
     */
    String doSuppressionArrete( HttpServletRequest request );

    /**
     * Tlcharge un fichier. Renvoie un flux de bytes du fichier dans l'objet
     * HttpServletResponse response
     *
     * @param request la requte HTTP
     * @return fichierTelecharger le fichier  tlcharger
     */
    GFichier doDownloadFichier( HttpServletRequest request );
}
