/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.autremodele;

import fr.paris.lutece.plugins.ods.business.autremodele.AutreModeleHome;
import fr.paris.lutece.plugins.ods.business.autremodele.TypeAutreModeleHome;
import fr.paris.lutece.plugins.ods.business.fichier.FichierPhysiqueHome;
import fr.paris.lutece.plugins.ods.dto.autremodele.AutreModeleFilter;
import fr.paris.lutece.plugins.ods.dto.autremodele.IAutreModele;
import fr.paris.lutece.plugins.ods.dto.autremodele.TypeAutreModele;
import fr.paris.lutece.plugins.ods.dto.fichier.FichierPhysique;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.service.util.AppPathService;
import fr.paris.lutece.portal.web.upload.MultipartHttpServletRequest;
import fr.paris.lutece.util.ReferenceList;
import fr.paris.lutece.util.url.UrlItem;

import org.apache.commons.fileupload.FileItem;

import org.springframework.beans.factory.annotation.Autowired;

import java.sql.SQLException;

import java.util.HashMap;
import java.util.List;

import javax.servlet.http.HttpServletRequest;


/**
 *
 * AbstractAutreModeleService
 *
 */
public abstract class AbstractAutreModeleService implements IAutreModeleService
{
    private static final String PROPERTY_LABEL_TITRE = "ods.autremodele.label.titre";
    private static final String PROPERTY_LABEL_TYPE = "ods.autremodele.label.type";
    private static final String MARK_AUTRE_MODELE = "autre_modele";
    private static final String MARK_LIST_AUTRE_MODELE = "liste_autre_modele";
    private static final String MARK_LISTE_TYPE = "liste_type";
    private static final String MESSAGE_MODELE_ALREADY_EXIST = "ods.autremodele.message.existeDeja";
    private static final String MESSAGE_MODELE_ENTETE_NO_XML = "ods.modele.message.enteteNoXml";
    private static final String MESSAGE_MODELE_PIED_NO_XML = "ods.modele.message.piedNoXml";
    private static final String MESSAGE_CONFIRME_DELETE_MODELE = "ods.autremodele.message.confirmeSuppression";
    private static final String MESSAGE_CANNOT_DELETE_MODELE = "ods.modele.message.suppressionImpossible";
    private static final String JSP_URL_DO_SUPPRESSION_AUTRE_MODELE_JSP = "jsp/admin/plugins/ods/autremodele/DoSuppressionAutreModele.jsp";
    private static final String JSP_URL_LISTE_AUTRES_MODELES = "jsp/admin/plugins/ods/autremodele/AutresModeles.jsp";
    @Autowired
    private AutreModeleHome<IAutreModele> _autreModeleHome;
    @Autowired
    private TypeAutreModeleHome _typeAutreModeleHome;
    @Autowired
    private FichierPhysiqueHome _fichierPhysiqueHome;

    /**
     *
     * @return le plugin
     */
    protected abstract Plugin getPlugin(  );

    /**
     *
     * @return une instance de IAutreModele
     */
    protected abstract IAutreModele newAutreModeleInstance(  );

    /**
     * Permet d'initialiser la liste des choix de type d'autres modles
     * @return la liste des types d'ordre du jour
     */
    private ReferenceList initRefListType(  )
    {
        Plugin plugin = getPlugin(  );

        ReferenceList refListType = new ReferenceList(  );
        List<TypeAutreModele> listType = _typeAutreModeleHome.findTypeAutreModeleList( plugin );

        // ajout dans la referenceList de l'ensemble des items reprsentant les types d'autres modles
        for ( TypeAutreModele type : listType )
        {
            refListType.addItem( type.getIdTypeAutreModele(  ), OdsConstants.CONSTANTE_CHAINE_VIDE +
                type.getLibelle(  ) );
        }

        return refListType;
    }

    /**
     * Rcupre la map contenant les lments utiles au template HTML de la liste
     *@param request la requete
     * @return model la map contenant les lments utiles au template HTML de la liste
     */
    public HashMap<String, Object> getAutresModelesList( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        HashMap<String, Object> model = new HashMap<String, Object>(  );
        AutreModeleFilter filter = new AutreModeleFilter(  );
        filter.setIdType( AutreModeleFilter.ALL_INT );

        List<IAutreModele> autresModeles = _autreModeleHome.findByFilter( filter, plugin );

        model.put( MARK_LIST_AUTRE_MODELE, autresModeles );

        return model;
    }

    /**
     * retourne la map contenant les lments utiles au template HTML de creation
     * @param request la requete
     * @return model la map contenant les lments utiles au template HTML de creation
     */
    public HashMap<String, Object> getCreationAutreModele( HttpServletRequest request )
    {
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        ReferenceList referenceListType = initRefListType(  );
        model.put( MARK_LISTE_TYPE, referenceListType );

        return model;
    }

    /**
     * retourne la map contenant les lments utiles au template HTML de modification
     * @param request la requete
     * @return model la map contenant les lments utiles au template HTML de modification
     */
    public HashMap<String, Object> getModificationAutreModele( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        HashMap<String, Object> model = new HashMap<String, Object>(  );

        int nIdModele = OdsUtils.getIntegerParameter( OdsParameters.ID_AUTRE_MODELE, request );

        IAutreModele autreModele = _autreModeleHome.findByPrimaryKey( nIdModele, plugin );
        OdsUtils.testNullObject( autreModele, request );

        ReferenceList referenceListType = initRefListType(  );

        model.put( MARK_AUTRE_MODELE, autreModele );
        model.put( MARK_LISTE_TYPE, referenceListType );

        return model;
    }

    /**
     * Retourne une demande pour la suppression d'un modele choisi
     * @param request la requete HTTP
     * @return interface de confirmation de suppression
     */
    public String getSuppressionAutreModele( HttpServletRequest request )
    {
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( request.getParameter( OdsParameters.ID_AUTRE_MODELE ) != null )
        {
            String strIdModele = request.getParameter( OdsParameters.ID_AUTRE_MODELE );
            UrlItem url = new UrlItem( JSP_URL_DO_SUPPRESSION_AUTRE_MODELE_JSP );
            url.addParameter( OdsParameters.ID_AUTRE_MODELE, strIdModele );

            strReturn = AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRME_DELETE_MODELE, url.getUrl(  ),
                    AdminMessage.TYPE_CONFIRMATION );
        }

        return strReturn;
    }

    /**
     * Cre un autre modele  partir des parametres contenus dans la requete
     * @param request la requete
     * @param strDefaultReturn l'URL de retour par dfaut
     * @return retour sur l'interface de gestion des autres modeles
     */
    public String doCreationAutreModele( HttpServletRequest request, String strDefaultReturn )
    {
        Plugin plugin = getPlugin(  );

        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        //Si on a cliqu sur le bouton "Annuler"
        if ( request.getParameter( OdsParameters.CONSTANTE_ACTION_CANCEL ) != null )
        {
            return AppPathService.getBaseUrl( request ) + JSP_URL_LISTE_AUTRES_MODELES;
        }

        IAutreModele autreModele = newAutreModeleInstance(  );

        String strMessage = testField( request, autreModele );

        if ( !strMessage.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            if ( strMessage.equals( MESSAGE_MODELE_ALREADY_EXIST ) )
            {
                strReturn = AdminMessageService.getMessageUrl( request, strMessage, AdminMessage.TYPE_STOP );
            }
            else if ( strMessage.equals( OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND ) )
            {
                strReturn = AdminMessageService.getMessageUrl( request,
                        OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND, AdminMessage.TYPE_ERROR );
            }
            else
            {
                Object[] tabRequiredFields = { I18nService.getLocalizedString( strMessage, request.getLocale(  ) ) };

                strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED,
                        tabRequiredFields, AdminMessage.TYPE_STOP );
            }
        }
        else
        {
            MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
            FileItem fileItemEnteteDocument = multipartRequest.getFile( OdsParameters.ENTETE_DOCUMENT );
            FileItem fileItemPiedDocument = multipartRequest.getFile( OdsParameters.PIED_DOCUMENT );

            if ( ( null != fileItemEnteteDocument ) && ( null != ( fileItemEnteteDocument.getName(  ) ) ) &&
                    !( OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItemEnteteDocument.getName(  ) ) ) )
            {
                if ( !isValideFile( fileItemEnteteDocument ) )
                {
                    strReturn = AdminMessageService.getMessageUrl( request, MESSAGE_MODELE_ENTETE_NO_XML,
                            AdminMessage.TYPE_STOP );
                }
                else
                {
                    autreModele.setEnteteDocument( doCreationFichierPhysique( fileItemEnteteDocument ) );
                }
            }

            if ( ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) ) && ( null != fileItemPiedDocument ) &&
                    ( null != fileItemPiedDocument.getName(  ) ) &&
                    !( OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItemPiedDocument.getName(  ) ) ) )
            {
                if ( !isValideFile( fileItemPiedDocument ) )
                {
                    strReturn = AdminMessageService.getMessageUrl( request, MESSAGE_MODELE_PIED_NO_XML,
                            AdminMessage.TYPE_STOP );
                }
                else
                {
                    autreModele.setPiedDocument( doCreationFichierPhysique( fileItemPiedDocument ) );
                }
            }

            if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                _autreModeleHome.create( autreModele, plugin );

                strReturn = strDefaultReturn;
            }
        }

        return strReturn;
    }

    /**
     * modifie un  autre modele a partir des parametres contenus dans la requete
     * @param request la requete
     * @return retour sur l'interface de gestion des autres modeles
     */
    public String doModificationAutreModele( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        //Si on a cliqu sur le bouton "Annuler"
        if ( request.getParameter( OdsParameters.CONSTANTE_ACTION_CANCEL ) != null )
        {
            return AppPathService.getBaseUrl( request ) + JSP_URL_LISTE_AUTRES_MODELES;
        }

        if ( request.getParameter( OdsParameters.ID_AUTRE_MODELE ) != null )
        {
            int nIdModele = OdsUtils.getIntegerParameter( OdsParameters.ID_AUTRE_MODELE, request );
            IAutreModele autreModele = _autreModeleHome.findByPrimaryKey( nIdModele, plugin );

            if ( autreModele == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            String strMessage = testField( request, autreModele );

            if ( !strMessage.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                if ( strMessage.equals( MESSAGE_MODELE_ALREADY_EXIST ) )
                {
                    strReturn = AdminMessageService.getMessageUrl( request, strMessage, AdminMessage.TYPE_STOP );
                }
                else if ( strMessage.equals( OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND ) )
                {
                    strReturn = AdminMessageService.getMessageUrl( request,
                            OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND, AdminMessage.TYPE_ERROR );
                }
                else
                {
                    Object[] tabRequiredFields = { I18nService.getLocalizedString( strMessage, request.getLocale(  ) ) };

                    strReturn = AdminMessageService.getMessageUrl( request,
                            OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED, tabRequiredFields, AdminMessage.TYPE_STOP );
                }

                return strReturn;
            }
            else
            {
                MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
                FileItem fileItemEnteteDocument = multipartRequest.getFile( OdsParameters.ENTETE_DOCUMENT );
                FileItem fileItemPiedDocument = multipartRequest.getFile( OdsParameters.PIED_DOCUMENT );

                if ( ( null != fileItemEnteteDocument ) && ( null != ( fileItemEnteteDocument.getName(  ) ) ) &&
                        !( OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItemEnteteDocument.getName(  ) ) ) &&
                        !isValideFile( fileItemEnteteDocument ) )
                {
                    strReturn = AdminMessageService.getMessageUrl( request, MESSAGE_MODELE_ENTETE_NO_XML,
                            AdminMessage.TYPE_STOP );
                }
                else if ( ( null != fileItemEnteteDocument ) && ( null != ( fileItemEnteteDocument.getName(  ) ) ) &&
                        !( OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItemEnteteDocument.getName(  ) ) ) &&
                        ( null == autreModele.getEnteteDocument(  ) ) )
                {
                    autreModele.setEnteteDocument( doCreationFichierPhysique( fileItemEnteteDocument ) );
                }
                else if ( ( null != fileItemEnteteDocument ) && ( null != ( fileItemEnteteDocument.getName(  ) ) ) &&
                        !( OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItemEnteteDocument.getName(  ) ) ) )
                {
                    FichierPhysique fichierEntete = autreModele.getEnteteDocument(  );
                    doModificationFichierPhysique( fileItemEnteteDocument, fichierEntete );
                }

                if ( ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) ) && ( null != fileItemPiedDocument ) &&
                        ( null != fileItemPiedDocument.getName(  ) ) &&
                        !( OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItemPiedDocument.getName(  ) ) ) &&
                        ( !isValideFile( fileItemPiedDocument ) ) )
                {
                    strReturn = AdminMessageService.getMessageUrl( request, MESSAGE_MODELE_PIED_NO_XML,
                            AdminMessage.TYPE_STOP );
                }
                else if ( ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) ) && ( null != fileItemPiedDocument ) &&
                        ( null != fileItemPiedDocument.getName(  ) ) &&
                        !( OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItemPiedDocument.getName(  ) ) ) &&
                        ( null == autreModele.getPiedDocument(  ) ) )
                {
                    autreModele.setPiedDocument( doCreationFichierPhysique( fileItemPiedDocument ) );
                }
                else if ( ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) ) && ( null != fileItemPiedDocument ) &&
                        ( null != fileItemPiedDocument.getName(  ) ) &&
                        !( OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItemPiedDocument.getName(  ) ) ) )
                {
                    FichierPhysique fichierPiedDocument = autreModele.getPiedDocument(  );
                    doModificationFichierPhysique( fileItemPiedDocument, fichierPiedDocument );
                }

                if ( !strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                {
                    return strReturn;
                }
            }

            _autreModeleHome.update( autreModele, plugin );
        }

        return strReturn;
    }

    /**
     * supprime le modle choisi
     * @param request la requete HTTP
     * @return url de gestion des groupes poliques
     */
    public String doSuppressionAutreModele( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( null != request.getParameter( OdsParameters.ID_AUTRE_MODELE ) )
        {
            int nIdModele = OdsUtils.getIntegerParameter( OdsParameters.ID_AUTRE_MODELE, request );
            IAutreModele autreModele = _autreModeleHome.findByPrimaryKey( nIdModele, plugin );

            if ( autreModele == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            try
            {
                //suppression du  modele
                _autreModeleHome.remove( autreModele, plugin );

                if ( null != autreModele.getEnteteDocument(  ) )
                {
                    _fichierPhysiqueHome.remove( autreModele.getEnteteDocument(  ), plugin );
                }

                if ( null != autreModele.getPiedDocument(  ) )
                {
                    _fichierPhysiqueHome.remove( autreModele.getPiedDocument(  ), plugin );
                }
            }
            catch ( AppException ae )
            {
                AppLogService.error( ae );

                if ( ae.getInitialException(  ) instanceof SQLException )
                {
                    strReturn = AdminMessageService.getMessageUrl( request, MESSAGE_CANNOT_DELETE_MODELE,
                            AdminMessage.TYPE_STOP );
                }
            }
        }

        return strReturn;
    }

    /**
     * Tlecharge un fichier.
     * Renvoie un flux de bytes du fichier dans l'objet HttpServletResponse response
     *
     * @param request la requete http
     * @return fichierPhysique le fichier  tlcharger
     */
    public FichierPhysique doDownloadFichier( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        FichierPhysique fichierPhysique = null;
        String strIdFichier = request.getParameter( OdsParameters.ID_FICHIER );

        if ( strIdFichier != null )
        {
            try
            {
                int nIdFichier = Integer.parseInt( strIdFichier );
                fichierPhysique = _fichierPhysiqueHome.findByPrimaryKey( nIdFichier, plugin );
            }

            catch ( NumberFormatException ne )
            {
                AppLogService.error( ne );
            }
        }

        return fichierPhysique;
    }

    /**
     * Retourne le premier champ requis manquant s'il y'en a .
     * Retourne un message d'erreur si les parametres saisis ne conviennent pas
     * @param request  la requete HTTP
     * @param autreModele l'autre modele
     * @return retourne le premier champ requis manquant s'il y'en a .
     */
    private String testField( HttpServletRequest request, IAutreModele autreModele )
    {
        Plugin plugin = getPlugin(  );

        String strRequiredField = OdsConstants.CONSTANTE_CHAINE_VIDE;
        String strTitre = OdsConstants.CONSTANTE_CHAINE_VIDE;
        int nIdType = 1;

        if ( ( null != request.getParameter( OdsParameters.TITRE ) ) &&
                !request.getParameter( OdsParameters.TITRE ).trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strTitre = request.getParameter( OdsParameters.TITRE ).trim(  );
        }
        else
        {
            strRequiredField = PROPERTY_LABEL_TITRE;
        }

        if ( strRequiredField.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            if ( ( null != request.getParameter( OdsParameters.ID_TYPE_AUTRE_MODELE ) ) &&
                    ( OdsUtils.getIntegerParameter( OdsParameters.ID_TYPE_AUTRE_MODELE, request ) != -1 ) )
            {
                nIdType = OdsUtils.getIntegerParameter( OdsParameters.ID_TYPE_AUTRE_MODELE, request );

                if ( _autreModeleHome.isAlreadyExist( nIdType, autreModele.getIdAutreModele(  ), plugin ) )
                {
                    strRequiredField = MESSAGE_MODELE_ALREADY_EXIST;
                }
            }
            else
            {
                strRequiredField = PROPERTY_LABEL_TYPE;
            }
        }

        if ( strRequiredField.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            TypeAutreModele typeAutreModele = new TypeAutreModele(  );

            //on insere les parametres dans l'objet pass en parametre
            autreModele.setTitre( strTitre );

            typeAutreModele.setIdTypeAutreModele( nIdType );
            autreModele.setTypeAutreModele( typeAutreModele );
        }

        return strRequiredField;
    }

    /**
     * Mthode qui cre le  fichier rcupr de la requette
     * @param fileItem le fichier associ au voeu(rcupr de la requette )
     * @return Fichier cr
     */
    private FichierPhysique doCreationFichierPhysique( FileItem fileItem )
    {
        Plugin plugin = getPlugin(  );

        FichierPhysique newFichierPhysique = new FichierPhysique(  );
        // Creation de l'objet reprsentant le fichier physique
        newFichierPhysique.setDonnees( fileItem.get(  ) );
        newFichierPhysique.setIdFichier( _fichierPhysiqueHome.create( newFichierPhysique, plugin ) );

        return newFichierPhysique;
    }

    /**
     * Modifie le fichier et le fichier physique li au voeuAmendement
     * @param fileItem le fichier
     * @param fichierPhysique le fichier physique
     */
    private void doModificationFichierPhysique( FileItem fileItem, FichierPhysique fichierPhysique )
    {
        Plugin plugin = getPlugin(  );

        // Modification de l'objet reprsentant le fichier physique
        fichierPhysique.setDonnees( fileItem.get(  ) );
        _fichierPhysiqueHome.update( fichierPhysique, plugin );
    }

    /**
     * test si le fichier fourni est valide
     * @param fileItem le fichier a tester
     * @return true si le fichier est valide  ont
     */
    private boolean isValideFile( FileItem fileItem )
    {
        boolean nIsFileGood = true;
        String extensionFile = fileItem.getName(  ).substring( fileItem.getName(  ).length(  ) - 3 ).toUpperCase(  );

        if ( !OdsConstants.CONSTANTE_XML.equals( extensionFile ) )
        {
            nIsFileGood = false;
        }

        return nIsFileGood;
    }
}
