/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.categoriedeliberation;

import fr.paris.lutece.plugins.ods.business.categoriectrllegalite.CategorieCtrlLegaliteHome;
import fr.paris.lutece.plugins.ods.business.categoriedeliberation.CategorieDeliberationHome;
import fr.paris.lutece.plugins.ods.dto.categoriectrllegalite.CategorieCtrlLegalite;
import fr.paris.lutece.plugins.ods.dto.categoriedeliberation.CategorieDeliberation;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.portal.service.util.AppPropertiesService;
import fr.paris.lutece.util.ReferenceList;
import fr.paris.lutece.util.url.UrlItem;

import org.springframework.beans.factory.annotation.Autowired;

import java.sql.SQLException;

import java.util.HashMap;
import java.util.List;

import javax.servlet.http.HttpServletRequest;


/**
 *
 * AbstractCategorieDeliberationService
 *
 */
public abstract class AbstractCategorieDeliberationService implements ICategorieDeliberationService
{
    private static final String PROPERTY_AFFICHER_AVERTISSEMENT_ALPACA = "ods.categoriedeliberation.afficherAvertissement";
    private static final String MARK_LISTE_CATEGORIES = "liste_categories";
    private static final String MARK_CATEGORIE = "categorie";
    private static final String MARK_AVERTISSEMENT = "avertissement";
    private static final String MARK_CATEGORIE_CTRL_LEGALITE_LIST = "categorie_ctrl_legalite";
    private static final String MESSAGE_NFE_ON_CREATE_CODE = "ods.message.errorOnCreateCode";
    private static final String MESSAGE_CANNOT_CREATE_CATEGORIE = "ods.message.cannotCreateCategorieDeliberation";
    private static final String MESSAGE_NFE_ON_MODIFY_CODE = "ods.message.errorOnModifyCode";
    private static final String MESSAGE_CANNOT_MODIFY_CATEGORIE = "ods.message.cannotModifyCategorieDeliberation";
    private static final String MESSAGE_CONFIRM_DELETE_CATEGORIE = "ods.message.confirmDeleteCategorieDeliberation";
    private static final String MESSAGE_CANNOT_DELETE_CATEGORIE = "ods.message.cannotDeleteCategorieDeliberation";
    private static final int DEFAULT_AFFICHER_AVERTISSEMENT_ALPACA = 1;
    private static final String JSP_URL_DO_SUPPRESSION_CATEGORIE_JSP = "jsp/admin/plugins/ods/categoriedeliberation/DoSuppressionCategorieDeliberation.jsp";
    private static final String FIELD_CODE = "ods.creationcategoriedeliberation.label.code";
    private static final String FIELD_LIBELLE = "ods.creationcategoriedeliberation.label.libelle";
    @Autowired
    private CategorieDeliberationHome _categorieDeliberationHome;
    @Autowired
    private CategorieCtrlLegaliteHome _categorieCtrlLegaliteHome;

    /**
     *
     * @return le plugin
     */
    protected abstract Plugin getPlugin(  );

    /**
     *
     * @return la liste des arrondissements
     */
    public static ReferenceList initRefListArrondissement(  )
    {
        ReferenceList refListArrondissement = new ReferenceList(  );
        refListArrondissement.addItem( 1, "1er" );

        for ( int i = 2; i <= 20; i++ )
        {
            refListArrondissement.addItem( i, OdsConstants.CONSTANTE_CHAINE_VIDE + i + "eme" );
        }

        return refListArrondissement;
    }

    /**
     * Retourne la liste du controle de lgalit
     * @return {@link ReferenceList} La liste du controle de lgalit
     */
    private ReferenceList initRefListCtrlLegalite(  )
    {
        Plugin plugin = getPlugin(  );

        ReferenceList refListDirection = new ReferenceList(  );
        List<CategorieCtrlLegalite> listCtrlLegalite = _categorieCtrlLegaliteHome.findCategorieCtrlLegaliteList( plugin );

        refListDirection.addItem( -1, OdsConstants.CONSTANTE_CHAINE_VIDE );

        if ( ( listCtrlLegalite != null ) && !listCtrlLegalite.isEmpty(  ) )
        {
            for ( CategorieCtrlLegalite categorie : listCtrlLegalite )
            {
                refListDirection.addItem( categorie.getIdCategorie(  ), categorie.getLibelle(  ) );
            }
        }

        return refListDirection;
    }

    /**
     * Retourne la map contenant les lments utiles au template HTML de la liste
     * @param request le requte HTTP
     * @return model la map contenant les lments utiles au tem:plate HTML de la liste
     */
    public HashMap<String, Object> getCategorieList( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        List<CategorieDeliberation> listCategorie = _categorieDeliberationHome.findCategorieDeliberationList( plugin );
        model.put( MARK_LISTE_CATEGORIES, listCategorie );

        return model;
    }

    /**
     * Retourne la map contenant les lments utiles au template HTML de creation
     * @param request le requte HTTP
     * @return model la map contenant les lments utiles au template HTML de creation
     */
    public HashMap<String, Object> getCreationCategorie( HttpServletRequest request )
    {
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        model.put( MARK_AVERTISSEMENT,
            AppPropertiesService.getPropertyInt( PROPERTY_AFFICHER_AVERTISSEMENT_ALPACA,
                DEFAULT_AFFICHER_AVERTISSEMENT_ALPACA ) );

        ReferenceList listCtrlLegalite = initRefListCtrlLegalite(  );
        model.put( MARK_CATEGORIE_CTRL_LEGALITE_LIST, listCtrlLegalite );

        return model;
    }

    /**
     * Retourne la map contenant les lments utiles au template HTML de modification
     * @param request le requte HTTP
     * @return model la map contenant les lments utiles au template HTML de modification
     */
    public HashMap<String, Object> getModificationCategorie( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        int nIdCategorie = OdsUtils.getIntegerParameter( OdsParameters.ID_CATEGORIE, request );

        CategorieDeliberation categorie = _categorieDeliberationHome.findByPrimaryKey( nIdCategorie, plugin );
        OdsUtils.testNullObject( categorie, request );

        ReferenceList listCtrlLegalite = initRefListCtrlLegalite(  );
        model.put( MARK_CATEGORIE_CTRL_LEGALITE_LIST, listCtrlLegalite );
        model.put( MARK_CATEGORIE, categorie );
        model.put( OdsMarks.MARK_ACTIF, categorie.isActif(  ) );
        model.put( OdsMarks.MARK_PUBLICATION_PARIS_FR, categorie.isExportParisFr(  ) );
        model.put( MARK_AVERTISSEMENT,
            AppPropertiesService.getPropertyInt( PROPERTY_AFFICHER_AVERTISSEMENT_ALPACA,
                DEFAULT_AFFICHER_AVERTISSEMENT_ALPACA ) );

        return model;
    }

    /**
     * Retourne l'interface de suppression d'une catgorie de dlibration.
     * @param request le requte HTTP
     * @return l'interface de suppression d'une catgorie de dlibration
     */
    public String getSuppressionCategorie( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        int nIdCategorie = OdsUtils.getIntegerParameter( OdsParameters.ID_CATEGORIE, request );

        if ( nIdCategorie == -1 )
        {
            return OdsConstants.CONSTANTE_CHAINE_VIDE;
        }

        UrlItem url = new UrlItem( JSP_URL_DO_SUPPRESSION_CATEGORIE_JSP );
        url.addParameter( OdsParameters.ID_CATEGORIE, OdsConstants.CONSTANTE_CHAINE_VIDE + nIdCategorie );

        String[] strLibelle = new String[1];

        CategorieDeliberation categorie = _categorieDeliberationHome.findByPrimaryKey( nIdCategorie, plugin );

        if ( categorie == null )
        {
            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                AdminMessage.TYPE_ERROR );
        }

        strLibelle[0] = ( categorie.getLibelle(  ) );

        return AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRM_DELETE_CATEGORIE, strLibelle,
            url.getUrl(  ), AdminMessage.TYPE_CONFIRMATION );
    }

    /**
     * Cre une catgorie de dlibration.
     * Si tous les champs obligatoires sont renseigns,
     * et si la valeur du code n'est pas dj utilise par une autre catgorie,
     * la mthode retourne l'url de gestion des catgories de dlibration.
     * Sinon la mthode retourne l'url affichant les messages d'erreur
     * @param request la requte HTTP
     * @return url de la page de gestion si tous les champs sont renseigns et le code valide,<br>
     *  sinon un message d'erreur
     */
    public String doCreationCategorie( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( !requiredField( request ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            Object[] tabRequiredFields = 
                {
                    I18nService.getLocalizedString( requiredField( request ), request.getLocale(  ) ),
                };

            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED,
                tabRequiredFields, AdminMessage.TYPE_STOP );
        }

        CategorieDeliberation categorie = new CategorieDeliberation(  );

        int nCode;

        try
        {
            nCode = Integer.parseInt( request.getParameter( OdsParameters.CODE_CATEGORIE ) );
        }
        catch ( NumberFormatException nfe )
        {
            return AdminMessageService.getMessageUrl( request, MESSAGE_NFE_ON_CREATE_CODE, AdminMessage.TYPE_STOP );
        }

        if ( _categorieDeliberationHome.containsCode( null, OdsConstants.CONSTANTE_CHAINE_VIDE + nCode, plugin ) )
        {
            return AdminMessageService.getMessageUrl( request, MESSAGE_CANNOT_CREATE_CATEGORIE, AdminMessage.TYPE_STOP );
        }

        categorie.setCode( nCode );

        String strLibelle = request.getParameter( OdsParameters.INTITULE ).trim(  );
        categorie.setLibelle( strLibelle );

        String strActif = request.getParameter( OdsParameters.ACTIF );
        boolean bActif;

        if ( null == strActif )
        {
            bActif = false;
        }
        else
        {
            bActif = true;
        }

        categorie.setActif( bActif );

        String strTypeMarche = request.getParameter( OdsParameters.PUBLICATION_PARIS_FR );
        boolean bTypeMarche;

        if ( null == strTypeMarche )
        {
            bTypeMarche = false;
        }
        else
        {
            bTypeMarche = true;
        }

        categorie.setExportParisFr( bTypeMarche );

        String categorieCtrlLegalite = request.getParameter( MARK_CATEGORIE_CTRL_LEGALITE_LIST );
        categorie.setCategorieCtrlLegalite( Integer.parseInt( categorieCtrlLegalite ) );

        _categorieDeliberationHome.create( categorie, plugin );

        return strReturn;
    }

    /**
     * Modifie une catgorie de dlibration.
     * Si tous les champs obligatoires sont renseigns,
     * et si la valeur du code n'est pas dj utilise par une autre catgorie,
     * la mthode retourne l'url de gestion des catgories de dlibration.
     * Sinon la mthode retourne l'url affichant les messages d'erreur
     * @param request la requte HTTP
     * @return une url
     */
    public String doModificationCategorie( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( !requiredField( request ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            Object[] tabRequiredFields = 
                {
                    I18nService.getLocalizedString( requiredField( request ), request.getLocale(  ) ),
                };

            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED,
                tabRequiredFields, AdminMessage.TYPE_STOP );
        }

        int nIdCategorie = OdsUtils.getIntegerParameter( OdsParameters.ID_CATEGORIE, request );

        CategorieDeliberation categorie = _categorieDeliberationHome.findByPrimaryKey( nIdCategorie, plugin );

        if ( categorie == null )
        {
            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                AdminMessage.TYPE_ERROR );
        }

        int nCode = OdsUtils.getIntegerParameter( OdsParameters.CODE_CATEGORIE, request );

        if ( nCode == -1 )
        {
            return AdminMessageService.getMessageUrl( request, MESSAGE_NFE_ON_MODIFY_CODE, AdminMessage.TYPE_STOP );
        }

        if ( _categorieDeliberationHome.containsCode( categorie, OdsConstants.CONSTANTE_CHAINE_VIDE + nCode, plugin ) )
        {
            return AdminMessageService.getMessageUrl( request, MESSAGE_CANNOT_MODIFY_CATEGORIE, AdminMessage.TYPE_STOP );
        }

        categorie.setCode( nCode );

        String strLibelle = request.getParameter( OdsParameters.INTITULE ).trim(  );
        categorie.setLibelle( strLibelle );

        String strActif = request.getParameter( OdsParameters.ACTIF );
        boolean bActif;

        if ( null == strActif )
        {
            bActif = false;
        }
        else
        {
            bActif = true;
        }

        categorie.setActif( bActif );

        String strTypeMarche = request.getParameter( OdsParameters.PUBLICATION_PARIS_FR );
        boolean bTypeMarche;

        if ( null == strTypeMarche )
        {
            bTypeMarche = false;
        }
        else
        {
            bTypeMarche = true;
        }

        categorie.setExportParisFr( bTypeMarche );

        String categorieCtrlLegalite = request.getParameter( MARK_CATEGORIE_CTRL_LEGALITE_LIST );
        categorie.setCategorieCtrlLegalite( Integer.parseInt( categorieCtrlLegalite ) );

        _categorieDeliberationHome.update( categorie, plugin );

        return strReturn;
    }

    /**
     * Supprime une catgorie de dlibration.
     * @param request la requte HTTP
     * @return url de gestion des catgories de dlibration
     */
    public String doSuppressionCategorie( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        int nIdCategorie = OdsUtils.getIntegerParameter( OdsParameters.ID_CATEGORIE, request );

        if ( nIdCategorie == -1 )
        {
            return OdsConstants.CONSTANTE_CHAINE_VIDE;
        }

        CategorieDeliberation categorie = new CategorieDeliberation(  );
        categorie.setIdCategorie( nIdCategorie );

        try
        {
            _categorieDeliberationHome.remove( categorie, plugin );
        }
        catch ( AppException ae )
        {
            if ( ae.getInitialException(  ) instanceof SQLException )
            {
                return AdminMessageService.getMessageUrl( request, MESSAGE_CANNOT_DELETE_CATEGORIE,
                    AdminMessage.TYPE_STOP );
            }
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * Indique si la requte est valide, c'est  dire que tous les champs obligatoires ont t remplis.
     * @param request la requte HTTP
     * @return true si la requte est valide, false sinon
     */
    private String requiredField( HttpServletRequest request )
    {
        if ( ( request.getParameter( OdsParameters.CODE_CATEGORIE ) == null ) ||
                ( request.getParameter( OdsParameters.CODE_CATEGORIE ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) ) )
        {
            return FIELD_CODE;
        }
        else if ( ( request.getParameter( OdsParameters.INTITULE ) == null ) ||
                ( request.getParameter( OdsParameters.INTITULE ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) ) )
        {
            return FIELD_LIBELLE;
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * Permet de rcuprer un objet rfrenceList contenant la liste des catgories de dlibration
     * @param plugin le Plugin
     * @param bIsItemVideEnabled TRUE si l'on souhaite avoir le 1 ligne vide, FALSE sinon
     * @return referenceCategorieDeliberationList la liste des catgories de dlibrations
     */
    public ReferenceList initRefListCategorieDeliberation( Plugin plugin, boolean bIsItemVideEnabled )
    {
        ReferenceList referenceListCategorieDeliberation = new ReferenceList(  );

        if ( bIsItemVideEnabled )
        {
            // ajout dans la referenceList de l'item chaine vide
            referenceListCategorieDeliberation.addItem( OdsConstants.FILTER_ALL, OdsConstants.CONSTANTE_CHAINE_VIDE );
        }

        List<CategorieDeliberation> listCategorieDeliberation = _categorieDeliberationHome.findAllCategoriesActives( plugin );

        //ajout dans la referenceList de l'ensemble des items reprsentant les catgories de dlibration
        for ( CategorieDeliberation categorie : listCategorieDeliberation )
        {
            referenceListCategorieDeliberation.addItem( categorie.getIdCategorie(  ), categorie.getLibelle(  ) );
        }

        return referenceListCategorieDeliberation;
    }
}
