/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.categoriedeliberation;

import fr.paris.lutece.plugins.ods.service.IODSService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.util.ReferenceList;

import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;


/**
 *
 * ICategorieDeliberationService
 *
 */
public interface ICategorieDeliberationService extends IODSService
{
    /**
     * Retourne la map contenant les lments utiles au template HTML de la liste
     * @param request le requte HTTP
     * @return model la map contenant les lments utiles au template HTML de la liste
     */
    HashMap<String, Object> getCategorieList( HttpServletRequest request );

    /**
     * Retourne la map contenant les lments utiles au template HTML de creation
     * @param request le requte HTTP
      * @return model la map contenant les lments utiles au template HTML de creation
     */
    HashMap<String, Object> getCreationCategorie( HttpServletRequest request );

    /**
     * Retourne la map contenant les lments utiles au template HTML de modification
     * @param request le requte HTTP
     * @return model la map contenant les lments utiles au template HTML de modification
     */
    HashMap<String, Object> getModificationCategorie( HttpServletRequest request );

    /**
     * Retourne l'interface de suppression d'une catgorie de dlibration.
     * @param request le requte HTTP
     * @return l'interface de suppression d'une catgorie de dlibration
     */
    String getSuppressionCategorie( HttpServletRequest request );

    /**
     * Cre une catgorie de dlibration.
     * Si tous les champs obligatoires sont renseigns,
     * et si la valeur du code n'est pas dj utilise par une autre catgorie,
     * la mthode retourne l'url de gestion des catgories de dlibration.
     * Sinon la mthode retourne l'url affichant les messages d'erreur
     * @param request la requte HTTP
     * @return url de la page de gestion si tous les champs sont renseigns et le code valide,<br>
     *  sinon un message d'erreur
     */
    String doCreationCategorie( HttpServletRequest request );

    /**
     * Modifie une catgorie de dlibration.
     * Si tous les champs obligatoires sont renseigns,
     * et si la valeur du code n'est pas dj utilise par une autre catgorie,
     * la mthode retourne l'url de gestion des catgories de dlibration.
     * Sinon la mthode retourne l'url affichant les messages d'erreur
     * @param request la requte HTTP
     * @return une url
     */
    String doModificationCategorie( HttpServletRequest request );

    /**
     * Supprime une catgorie de dlibration.
     * @param request la requte HTTP
     * @return url de gestion des catgories de dlibration
     */
    String doSuppressionCategorie( HttpServletRequest request );

    /**
     * Permet de rcuprer un objet rfrenceList contenant la liste des catgories de dlibration
     * @param plugin le Plugin
     * @param bIsItemVideEnabled TRUE si l'on souhaite avoir le 1 ligne vide, FALSE sinon
     * @return referenceCategorieDeliberationList la liste des catgories de dlibrations
     */
    ReferenceList initRefListCategorieDeliberation( Plugin plugin, boolean bIsItemVideEnabled );
}
