/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.certificataffichage;

import fr.paris.lutece.plugins.ods.business.IBusinessHome;
import fr.paris.lutece.plugins.ods.business.acte.ActeHome;
import fr.paris.lutece.plugins.ods.business.autremodele.AutreModeleHome;
import fr.paris.lutece.plugins.ods.business.formationconseil.FormationConseilHome;
import fr.paris.lutece.plugins.ods.business.ordredujour.generated.entete.Entete;
import fr.paris.lutece.plugins.ods.business.ordredujour.generated.pieddepage.PiedDePage;
import fr.paris.lutece.plugins.ods.business.seance.ISeanceHome;
import fr.paris.lutece.plugins.ods.dto.IBusinessItem;
import fr.paris.lutece.plugins.ods.dto.acte.Acte;
import fr.paris.lutece.plugins.ods.dto.acte.IActeFilter;
import fr.paris.lutece.plugins.ods.dto.autremodele.AutreModeleFilter;
import fr.paris.lutece.plugins.ods.dto.autremodele.IAutreModele;
import fr.paris.lutece.plugins.ods.dto.autremodele.TypeAutreModeleEnum;
import fr.paris.lutece.plugins.ods.dto.certificataffichage.IEntiteCertificatAffichage;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.service.acte.IActeService;
import fr.paris.lutece.plugins.ods.service.formationconseil.IFormationConseilService;
import fr.paris.lutece.plugins.ods.utils.business.ModeleUtils;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.commons.DateUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppPropertiesService;
import fr.paris.lutece.util.ReferenceList;
import fr.paris.lutece.util.url.UrlItem;

import org.springframework.beans.factory.annotation.Autowired;

import java.sql.Timestamp;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.servlet.http.HttpServletRequest;


public abstract class AbstractCertificatAffichageService<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>, GActeFilter extends IActeFilter<GFichier, GSeance>, GElu extends IElu, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>>
    implements ICertificatAffichageService
{
    private static final String XML_CERTIFICAT_OPEN = "<certificat>";
    private static final String XML_CERTIFICAT_CLOSE = "</certificat>";
    private static final String MARK_LISTE_SEANCES = "liste_seances";
    private static final String MARK_ID_SEANCE_SELECTED = "id_seance_selected";
    private static final String MARK_DATE_RETOUR_CONTROLE_LEGALITE = "date_retour_controle_legalite";
    private static final String MARK_LISTE_FORMATION_CONSEIL = "liste_formation_conseil";
    private static final String MARK_ID_FORMATION_CONSEIL_SELECTED = "id_formation_conseil_selected";
    private static final String MARK_ERROR = "error";
    private static final String PROPERTY_LABEL_DATE_TRANSMISSION_CTRL_LEGALITE = "ods.certificatAffichage.label.dateTransmissionControleLegalite";
    private static final String PROPERTY_MESSAGE_NO_DATE_TRANSMISSION_CTRL_LEGALITE = "ods.certificatAffichage.error.noDateTransmissionControleLegalite";
    private static final String PROPERTY_MESSAGE_NO_MODELE = "ods.certificatAffichage.error.noModele";
    private static final String PROPERTY_MESSAGE_NO_ENTETE_PIED = "ods.certificatAffichage.error.noEntetePied";
    private static final String JSP_URL_DO_VISUALISATION_CERTIFICAT = "DoVisualisationCertificat.jsp";
    private Timestamp _tsDateRetourCtrlLegalite;
    @Autowired
    private IFormationConseilService _formationConseilService;
    @Autowired
    private ActeHome<GSeance, GFichier, GActeFilter> _acteHome;
    @Autowired
    private AutreModeleHome<IAutreModele> _autreModeleHome;
    @Autowired
    private FormationConseilHome _formationConseilHome;
    @Autowired
    private ISeanceHome<GSeance, ISeanceFilter, GFichier, GElu, GVoeuAmendement, GPDD> _seanceHome;
    @Autowired
    private IActeService<GSeance, GFichier, GActeFilter> _acteService;
    private Map<Integer, IBusinessHome> _mappingHome;
    private Map<Integer, ICertificatAffichageServiceManaged<GSeance, GFichier>> _mappingService;

    public Map<Integer, IBusinessHome> getMappingHome(  )
    {
        return _mappingHome;
    }

    public void setMappingHome( Map<Integer, IBusinessHome> mappingHome )
    {
        this._mappingHome = mappingHome;
    }

    public Map<Integer, ICertificatAffichageServiceManaged<GSeance, GFichier>> getMappingService(  )
    {
        return _mappingService;
    }

    public void setMappingService( Map<Integer, ICertificatAffichageServiceManaged<GSeance, GFichier>> mappingService )
    {
        this._mappingService = mappingService;
    }

    protected abstract IActeFilter<GFichier, GSeance> newActeFilter(  );

    protected abstract String getDateConseilSpec( ISeance seance );

    protected abstract Plugin getPlugin(  );

    /**
     * Retourne la map contenant les lments utiles au template HTML de la liste
     * @param request le requte HTTP
     * @return model la map contenant les lments utiles au template HTML de la liste
     */
    public HashMap<String, Object> getCertificatList( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        int nIdSeance = -1;
        int nIdFormationConseil = -1;
        boolean bIsEntite = false;

        for ( Entry<Integer, ICertificatAffichageServiceManaged<GSeance, GFichier>> entry : _mappingService.entrySet(  ) )
        {
            ICertificatAffichageServiceManaged<GSeance, GFichier> serviceManaged = entry.getValue(  );
            int nIdEntite = OdsUtils.getIntegerParameter( serviceManaged.getIdParameter(  ), request );

            if ( nIdEntite != -1 )
            {
                IEntiteCertificatAffichage entite = (IEntiteCertificatAffichage) _mappingHome.get( entry.getKey(  ) )
                                                                                             .findByPrimaryKey( nIdEntite,
                        plugin );

                if ( entite != null )
                {
                    nIdSeance = serviceManaged.getSeance( entite ).getIdSeance(  );
                    nIdFormationConseil = entite.getFormationConseil(  ).getIdFormationConseil(  );
                }

                bIsEntite = true;

                break;
            }
        }

        if ( !bIsEntite )
        {
            nIdSeance = OdsUtils.getIntegerParameter( OdsParameters.ID_SEANCE, request );
            nIdFormationConseil = OdsUtils.getIntegerParameter( OdsParameters.ID_FORMATION_CONSEIL, request );

            String strDateRetourControleDeLegalite = request.getParameter( OdsParameters.DATE_RETOUR_CONTROLE_LEGALITE );
            _tsDateRetourCtrlLegalite = DateUtils.getDate( strDateRetourControleDeLegalite, true );
        }

        if ( ( nIdSeance != -1 ) && ( nIdFormationConseil != -1 ) && ( _tsDateRetourCtrlLegalite == null ) )
        {
            model.put( MARK_ERROR, true );
        }

        if ( ( nIdSeance == -1 ) && ( _seanceHome.getDerniereSeance( plugin ) != null ) )
        {
            nIdSeance = _seanceHome.getDerniereSeance( plugin ).getIdSeance(  );
        }

        //Generation dynamique des listes d"entits
        HashMap<Integer, List<IEntiteCertificatAffichage>> mapList = new HashMap<Integer, List<IEntiteCertificatAffichage>>(  );

        for ( Entry<Integer, ICertificatAffichageServiceManaged<GSeance, GFichier>> entry : _mappingService.entrySet(  ) )
        {
            mapList.put( entry.getKey(  ), new ArrayList<IEntiteCertificatAffichage>(  ) );
        }

        int nNbResultats = 0;

        if ( ( _tsDateRetourCtrlLegalite != null ) )
        {
            GActeFilter acteFilter = _acteService.getActeFilter(  );
            acteFilter.setIdSeance( nIdSeance );
            acteFilter.setIdFormationConseil( nIdFormationConseil );
            acteFilter.setDateRetourControleDeLegalite( _tsDateRetourCtrlLegalite );
            acteFilter.setStatut( IActeFilter.ALL_EXCEPT_ANNULE );

            assignByEntite( acteFilter, mapList );
            sortByEntite( mapList );

            for ( Entry<Integer, List<IEntiteCertificatAffichage>> entry : mapList.entrySet(  ) )
            {
                nNbResultats += entry.getValue(  ).size(  );

                model.put( _mappingService.get( entry.getKey(  ) ).getMarkForCertificat( entry.getKey(  ) ),
                    entry.getValue(  ) );
            }
        }

        List<GSeance> listeSeances = _seanceHome.findOldSeance( plugin );
        ReferenceList refListFormations = _formationConseilService.initRefListFormationConseil( false );

        model.put( MARK_LISTE_SEANCES, listeSeances );
        model.put( MARK_ID_SEANCE_SELECTED, nIdSeance );
        model.put( MARK_DATE_RETOUR_CONTROLE_LEGALITE, _tsDateRetourCtrlLegalite );
        model.put( MARK_LISTE_FORMATION_CONSEIL, refListFormations );
        model.put( MARK_ID_FORMATION_CONSEIL_SELECTED, nIdFormationConseil );
        model.put( OdsMarks.MARK_NOMBRE_RESULTATS, nNbResultats );

        return model;
    }

    /**
    *
    * @param request la requete HTTP
    * @return l'URL de visualisation du certificat
    */
    public String getVisualisationCertificat( HttpServletRequest request )
    {
        String strDateTransmissionControleDeLegalite = request.getParameter( OdsParameters.DATE_TRANSMISSION_CONTROLE_LEGALITE );
        Timestamp tsDateTransmissionCtrlLegalite;

        if ( ( strDateTransmissionControleDeLegalite == null ) ||
                ( ( strDateTransmissionControleDeLegalite != null ) &&
                strDateTransmissionControleDeLegalite.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) ) )
        {
            return AdminMessageService.getMessageUrl( request, PROPERTY_MESSAGE_NO_DATE_TRANSMISSION_CTRL_LEGALITE,
                AdminMessage.TYPE_STOP );
        }
        else
        {
            tsDateTransmissionCtrlLegalite = DateUtils.getDate( strDateTransmissionControleDeLegalite, true );

            if ( tsDateTransmissionCtrlLegalite == null )
            {
                Object[] messagesArgs = 
                    {
                        I18nService.getLocalizedString( PROPERTY_LABEL_DATE_TRANSMISSION_CTRL_LEGALITE,
                            request.getLocale(  ) ),
                    };

                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_DATE_FORMAT_ERROR,
                    messagesArgs, AdminMessage.TYPE_ERROR );
            }
        }

        int nIdSeance = OdsUtils.getIntegerParameter( OdsParameters.ID_SEANCE, request );
        int nIdFormationConseil = OdsUtils.getIntegerParameter( OdsParameters.ID_FORMATION_CONSEIL, request );
        String strDateRetourControleDeLegalite = request.getParameter( OdsParameters.DATE_RETOUR_CONTROLE_LEGALITE );

        UrlItem url = new UrlItem( JSP_URL_DO_VISUALISATION_CERTIFICAT );
        url.addParameter( OdsParameters.ID_SEANCE, nIdSeance );
        url.addParameter( OdsParameters.ID_FORMATION_CONSEIL, nIdFormationConseil );
        url.addParameter( OdsParameters.DATE_RETOUR_CONTROLE_LEGALITE, strDateRetourControleDeLegalite );
        url.addParameter( OdsParameters.DATE_TRANSMISSION_CONTROLE_LEGALITE, strDateTransmissionControleDeLegalite );

        return url.getUrl(  );
    }

    /**
     * Retourne le certificat sous format PDF
     * @param request la requete Http
         * @return sbXml le flux XML permettant de gnrer le certificat au format PDF, les messages d'erreur sinon
     */
    public String doVisualisationCertificat( HttpServletRequest request )
    {
        StringBuffer sbXml = new StringBuffer(  );
        Plugin plugin = getPlugin(  );

        int nIdSeance = OdsUtils.getIntegerParameter( OdsParameters.ID_SEANCE, request );
        int nIdFormationConseil = OdsUtils.getIntegerParameter( OdsParameters.ID_FORMATION_CONSEIL, request );

        String strDateRetourControleDeLegalite = request.getParameter( OdsParameters.DATE_RETOUR_CONTROLE_LEGALITE );
        Timestamp tsDateRetourControleDeLegalite = DateUtils.getDate( strDateRetourControleDeLegalite, true );

        String strDateTransmissionControleDeLegalite = request.getParameter( OdsParameters.DATE_TRANSMISSION_CONTROLE_LEGALITE );
        Timestamp tsDateTransmissionCtrlLegalite = DateUtils.getDate( strDateTransmissionControleDeLegalite, true );

        ISeance seance = _seanceHome.findByPrimaryKey( nIdSeance, plugin );
        FormationConseil formationConseil = _formationConseilHome.findByPrimaryKey( nIdFormationConseil, plugin );

        //Generation dynamique des listes d"entits
        HashMap<Integer, List<IEntiteCertificatAffichage>> mapList = new HashMap<Integer, List<IEntiteCertificatAffichage>>(  );

        for ( Entry<Integer, ICertificatAffichageServiceManaged<GSeance, GFichier>> entry : _mappingService.entrySet(  ) )
        {
            mapList.put( entry.getKey(  ), new ArrayList<IEntiteCertificatAffichage>(  ) );
        }

        GActeFilter acteFilter = _acteService.getActeFilter(  );
        acteFilter.setIdSeance( nIdSeance );
        acteFilter.setIdFormationConseil( nIdFormationConseil );
        acteFilter.setDateRetourControleDeLegalite( tsDateRetourControleDeLegalite );
        acteFilter.setStatut( IActeFilter.ALL_EXCEPT_ANNULE );

        assignByEntite( acteFilter, mapList );
        sortByEntite( mapList );

        /* Rcupration du modle du certificat pour la gnration PDF */
        AutreModeleFilter filterAutreModele = new AutreModeleFilter(  );
        filterAutreModele.setIdType( TypeAutreModeleEnum.CERTIFICAT_AFFICHAGE.getId(  ) );

        List<IAutreModele> listAutreModele = _autreModeleHome.findByFilter( filterAutreModele, plugin );

        if ( ( listAutreModele != null ) && listAutreModele.isEmpty(  ) )
        {
            return PROPERTY_MESSAGE_NO_MODELE;
        }
        else
        {
            IAutreModele autreModele = listAutreModele.get( 0 );

            if ( ( autreModele.getEnteteDocument(  ) == null ) || ( autreModele.getPiedDocument(  ) == null ) )
            {
                return PROPERTY_MESSAGE_NO_ENTETE_PIED;
            }
            else
            {
                //On rucpre l'entte et le pied de page
                Entete entete = ModeleUtils.getEntete( autreModele.getEnteteDocument(  ).getDonnees(  ) );
                PiedDePage piedDePage = ModeleUtils.getPiedDePage( autreModele.getPiedDocument(  ).getDonnees(  ) );

                HashMap<Integer, Object> xmlParamsEntete = new HashMap<Integer, Object>(  );
                HashMap<Integer, Object> xmlParamsPiedDePage = new HashMap<Integer, Object>(  );

                Timestamp tsCurrentDate = DateUtils.getCurrentDate(  );
                String strNomSecretaireGenerale = AppPropertiesService.getProperty( OdsProperties.PROPERTY_NOM_SECRETAIRE_GENERALE );

                //On genere le flux XML de l'entte
                xmlParamsEntete.put( ModeleUtils.XML_PARAMETER_CODE_DATE_JOUR,
                    DateUtils.getDate( tsCurrentDate, OdsConstants.CONSTANTE_PATTERN_DATE_LONG ) );
                xmlParamsEntete.put( ModeleUtils.XML_PARAMETER_CODE_DATE_TRANSMISSION_CTRL,
                    DateUtils.getDate( tsDateTransmissionCtrlLegalite, OdsConstants.CONSTANTE_PATTERN_DATE_LONG ) );
                xmlParamsEntete.put( ModeleUtils.XML_PARAMETER_CODE_DATE_SEANCE, getDateConseilSpec( seance ) );
                xmlParamsEntete.put( ModeleUtils.XML_PARAMETER_CODE_FORMATION_CONSEIL, formationConseil.getLibelle(  ) );
                xmlParamsEntete.put( ModeleUtils.XML_PARAMETER_CODE_NOM_SECRETAIRE_GENERAL, strNomSecretaireGenerale );

                String strXmlEntete = ModeleUtils.getXmlEntete( entete, xmlParamsEntete );

                //On genere le flux XML du pied de page
                xmlParamsPiedDePage.put( ModeleUtils.XML_PARAMETER_CODE_NOM_SECRETAIRE_GENERAL, strNomSecretaireGenerale );

                String strXmlPiedDePage = ModeleUtils.getXmlPiedDePage( piedDePage, xmlParamsPiedDePage );

                //On genere
                sbXml.append( XML_CERTIFICAT_OPEN );

                sbXml.append( strXmlEntete );
                sbXml.append( strXmlPiedDePage );

                for ( Entry<Integer, List<IEntiteCertificatAffichage>> entry : mapList.entrySet(  ) )
                {
                    for ( IEntiteCertificatAffichage entite : entry.getValue(  ) )
                    {
                        sbXml.append( entite.getXmlForCertificat( entry.getKey(  ) ) );
                    }
                }

                sbXml.append( XML_CERTIFICAT_CLOSE );
            }
        }

        return sbXml.toString(  );
    }

    /**
     * Tri les actes suivant leur entit parent
     * @param acteFilter le filtre d'actes
     * @param mapList la map contenant les listes d'entits
     */
    private void assignByEntite( GActeFilter acteFilter, HashMap<Integer, List<IEntiteCertificatAffichage>> mapList )
    {
        Plugin plugin = getPlugin(  );

        List<Acte<GSeance, GFichier>> listActes = _acteHome.findByFilter( acteFilter, plugin );

        for ( Acte<GSeance, GFichier> acte : listActes )
        {
            int nIdTypeEntite = acte.getIdTypeEntite(  );
            IEntiteCertificatAffichage entite = _mappingService.get( nIdTypeEntite ).getEntite( acte );

            if ( entite != null )
            {
                mapList.get( nIdTypeEntite ).add( entite );
            }
        }
    }

    /**
     * Tri les actes suivant leur entit parent
     * @param mapList la map contenant les listes d'entits
     */
    private void sortByEntite( HashMap<Integer, List<IEntiteCertificatAffichage>> mapList )
    {
        // Tri des diffrentes listes
        for ( Entry<Integer, List<IEntiteCertificatAffichage>> entry : mapList.entrySet(  ) )
        {
            Comparator<IBusinessItem> comparator = _mappingService.get( entry.getKey(  ) ).getComparatorClass(  );

            if ( comparator != null )
            {
                Collections.sort( entry.getValue(  ), comparator );
            }
        }
    }
}
