/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.deliberationdesignation;

import fr.paris.lutece.plugins.ods.business.acte.ActeHome;
import fr.paris.lutece.plugins.ods.business.acte.NatureActeHome;
import fr.paris.lutece.plugins.ods.business.acte.StatutActeHome;
import fr.paris.lutece.plugins.ods.business.categoriedeliberation.CategorieDeliberationHome;
import fr.paris.lutece.plugins.ods.business.deliberationdesignation.AbstractDeliberationDesignationHome;
import fr.paris.lutece.plugins.ods.business.fichier.FichierPhysiqueHome;
import fr.paris.lutece.plugins.ods.business.fichier.IFichierHome;
import fr.paris.lutece.plugins.ods.business.formationconseil.FormationConseilHome;
import fr.paris.lutece.plugins.ods.business.historique.HistoriqueHome;
import fr.paris.lutece.plugins.ods.business.seance.ISeanceHome;
import fr.paris.lutece.plugins.ods.dto.DesignationComparator;
import fr.paris.lutece.plugins.ods.dto.IBusinessItem;
import fr.paris.lutece.plugins.ods.dto.IDesignation;
import fr.paris.lutece.plugins.ods.dto.acte.Acte;
import fr.paris.lutece.plugins.ods.dto.acte.IActeFilter;
import fr.paris.lutece.plugins.ods.dto.acte.NatureActe;
import fr.paris.lutece.plugins.ods.dto.acte.NatureActeEnum;
import fr.paris.lutece.plugins.ods.dto.acte.StatutActe;
import fr.paris.lutece.plugins.ods.dto.acte.StatutActeEnum;
import fr.paris.lutece.plugins.ods.dto.categoriedeliberation.CategorieDeliberation;
import fr.paris.lutece.plugins.ods.dto.certificataffichage.IEntiteCertificatAffichage;
import fr.paris.lutece.plugins.ods.dto.deliberationdesignation.DeliberationDesignationFilter;
import fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.FichierPhysique;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter;
import fr.paris.lutece.plugins.ods.dto.fichier.ITypeDocument;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.historique.Historique;
import fr.paris.lutece.plugins.ods.dto.horodatage.HorodatageActionEnum;
import fr.paris.lutece.plugins.ods.dto.horodatage.IEntiteHorodatage;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur;
import fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.typeentite.ITypeEntite;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.service.acte.IActeService;
import fr.paris.lutece.plugins.ods.service.categoriedeliberation.ICategorieDeliberationService;
import fr.paris.lutece.plugins.ods.service.certificataffichage.ICertificatAffichageServiceManaged;
import fr.paris.lutece.plugins.ods.service.direction.IDirectionService;
import fr.paris.lutece.plugins.ods.service.formationconseil.IFormationConseilService;
import fr.paris.lutece.plugins.ods.service.horodatage.IHorodatageService;
import fr.paris.lutece.plugins.ods.service.horodatage.IHorodatageServiceManaged;
import fr.paris.lutece.plugins.ods.service.search.ISearchService;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.commons.DateUtils;
import fr.paris.lutece.plugins.ods.utils.commons.ShaUtils;
import fr.paris.lutece.plugins.ods.utils.commons.StringUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.service.util.AppPathService;
import fr.paris.lutece.portal.service.util.AppPropertiesService;
import fr.paris.lutece.portal.web.constants.Messages;
import fr.paris.lutece.portal.web.upload.MultipartHttpServletRequest;
import fr.paris.lutece.util.html.Paginator;
import fr.paris.lutece.util.url.UrlItem;

import org.apache.commons.fileupload.FileItem;

import org.springframework.beans.factory.annotation.Autowired;

import java.sql.SQLException;
import java.sql.Timestamp;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;

import javax.servlet.http.HttpServletRequest;


public abstract class AbstractDeliberationDesignationService<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>, GActeFilter extends IActeFilter<GFichier, GSeance>, GDeliberationDesignation extends IDeliberationDesignation<GSeance, GFichier>, GElu extends IElu, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GSeanceFilter extends ISeanceFilter, GRequeteUtilisateur extends IRequeteUtilisateur<GSeance, GSeanceFilter>, GResultatRequete extends IResultatRequete<GSeance, GSeanceFilter, GRequeteUtilisateur, GElu, GFichier, GVoeuAmendement, GPDD>>
    implements IDeliberationDesignationService<GSeance, GFichier>, IHorodatageServiceManaged,
        ICertificatAffichageServiceManaged<GSeance, GFichier>
{
    protected static final String JSP_URL_GET_MODIFICATION_DELIBERATION_DESIGNATION = "jsp/admin/plugins/ods/deliberationsdesignation/ModificationDeliberationDesignationGestionAval.jsp";
    protected static final String JSP_URL_GET_VISUALISATION_DELIBERATION_DESIGNATION_FOR_ACTE = "jsp/admin/plugins/ods/deliberationsdesignation/VisualisationDeliberationDesignationForActe.jsp";
    protected static final String JSP_URL_GET_MODIFICATION_DELIBERATION_DESIGNATION_FOR_ACTE = "jsp/admin/plugins/ods/deliberationsdesignation/ModificationDeliberationDesignationForActe.jsp";
    protected static final String JSP_URL_LISTE_DELIBERATIONS = "jsp/admin/plugins/ods/deliberationsdesignation/DeliberationsDesignationGestionAvalNoReinit.jsp";
    protected static final String JSP_URL_FINALISATION_ACTE = "jsp/admin/plugins/ods/deliberationsdesignation/FinalisationActe.jsp";
    protected static final String PROPERTY_DELIBERATIONS_DESIGNATION_PER_PAGE = "ods.itemPerPage.label";
    protected static final String PROPERTY_URL_DO_SUPPRESSION_DELIBERATION_DESIGNATION_GESTION_AVAL_JSP = "jsp/admin/plugins/ods/deliberationsdesignation/DoSuppressionDeliberationDesignationGestionAval.jsp";
    protected static final String PROPERTY_LABEL_SEANCE = "ods.creationdeliberationdesignation.label.seance";
    protected static final String PROPERTY_LABEL_INTITULE = "ods.creationdeliberationdesignation.label.intitule";
    protected static final String PROPERTY_LABEL_NUMERO = "ods.creationdeliberationdesignation.label.numero";
    protected static final String PROPERTY_LABEL_FICHIER_SCANNE = "ods.creationdeliberationdesignation.label.deliberationscannee";
    protected static final String PROPERTY_LABEL_CATEGORIE = "ods.creationdeliberationdesignation.label.categorieDeDeliberation";
    protected static final String PROPERTY_LABEL_DATE_VOTE = "ods.creationdeliberationdesignation.label.dateVote";
    protected static final String PROPERTY_LABEL_FICHIER_NO_PDF = "ods.creationdeliberationdesignation.label.fichier.noPDF";
    protected static final String PROPERTY_MESSAGE_FILES_NOT_FOUND = "ods.deliberationdesignation.error.filesnotfound";
    protected static final String MARK_LISTE_DELIBERATIONS = "liste_deliberations";
    protected static final String MARK_LISTE_HISTORIQUES = "liste_historiques";
    protected static final String MARK_LISTE_DELIBERATIONS_SIZE_TOTAL = "liste_deliberations_size_total";
    protected static final String MARK_LISTE_CATEGORIE_DELIBERATION = "liste_categorie_deliberation";
    protected static final String MARK_ID_CATEGORIE_DELIBERATION_SELECTED = "id_categorie_deliberation_selected";
    protected static final String MARK_LISTE_SEANCES = "liste_seances";
    protected static final String MARK_ID_SEANCE_SELECTED = "id_seance_selected";
    protected static final String MARK_LISTE_NATURE_DOCUMENT = "liste_nature_document";
    protected static final String MARK_LISTE_DIRECTION = "liste_direction";
    protected static final String MARK_ID_ACTE = "id_acte";
    protected static final String MARK_ACTE = "acte";
    protected static final String MARK_STATUT_ACTE_SELECTED = "statut_acte_selected";
    protected static final String MESSAGE_CANNOT_DELETE_DELIBERATION = "ods.message.cannotDeleteDeliberation";
    protected static final String MESSAGE_CONFIRM_DELETE_DELIBERATION = "ods.message.confirmDeleteDeliberation";
    protected static final String MESSAGE_CONFIRM_DELETE_DELIBERATION_WITH_ACTES = "ods.message.confirmDeleteDeliberationWithActes";
    protected static final String MESSAGE_ERROR_DELETE_DELIBERATION = "ods.message.errorDeleteDeliberation";
    protected static final String PROPERTY_CODE_DELIBERATION_DEFAUT = "ods.deliberation.categoriedeliberation.defaut.code";
    protected static final int DEFAULT_CODE_DELIBERATION = 34;

    //Gre la sance slectionne
    protected int _nIdSeanceSelected = -1;

    //Gre le statut de l'acte slectionn
    protected int _nStatutActeSelected = -1;

    // Gere la page du Paginator
    protected String _strCurrentPageIndex;

    //Gere le nombre d'item par page
    protected int _nItemsPerPage;
    protected int _nDefaultItemsPerPage;

    //gre si il ya eu des changements dans un arrete avant enregistrement
    protected boolean _bHasChanged;

    //variable de session permettant de savoir si le retour ne doit pas ce faire sur l'interface de gestion des vas 
    protected boolean _bUrlRetour;

    //variable de session stockant l'url de retour si le retour ne ce fait pas sur l'interface de gestion des vas 
    protected String _strUrlRetour;

    //variable de session permettant de savoir si le retour se fait sur la gestion des deliberations (0) ou sur le suivi des actes(1)
    protected int _nTypeUrlRetour;

    //variable de session permettant de savoir si l'on vient de la gestion des actes
    protected boolean _bComeFromActe;
    @Autowired
    protected AbstractDeliberationDesignationHome<GSeance, GFichier, GDeliberationDesignation, GActeFilter> _deliberationDesignationHome;
    @Autowired
    protected IHorodatageService _horodatageService;
    @Autowired
    protected ISearchService<GSeance, GSeanceFilter, GRequeteUtilisateur, GResultatRequete, GElu, GFichier, GVoeuAmendement, GPDD> _searchService;
    @Autowired
    protected IActeService<GSeance, GFichier, GActeFilter> _acteService;
    @Autowired
    protected ActeHome<GSeance, GFichier, GActeFilter> _acteHome;
    @Autowired
    protected NatureActeHome _natureActeHome;
    @Autowired
    protected StatutActeHome _statutActeHome;
    @Autowired
    protected CategorieDeliberationHome _categorieDeliberationHome;
    @Autowired
    protected ICategorieDeliberationService _categorieDeliberationService;
    @Autowired
    protected IDirectionService _directionService;
    @Autowired
    protected IFichierHome<GFichier, IFichierFilter, ITypeDocument, GSeance> _fichierHome;
    @Autowired
    protected FichierPhysiqueHome _fichierPhysiqueHome;
    @Autowired
    protected IFormationConseilService _formationConseilService;
    @Autowired
    protected FormationConseilHome _formationConseilHome;
    @Autowired
    protected HistoriqueHome _historiqueHome;
    @Autowired
    protected ISeanceHome<GSeance, ISeanceFilter, GFichier, GElu, GVoeuAmendement, GPDD> _seanceHome;

    /**
     * Constructor.
     */
    public AbstractDeliberationDesignationService(  )
    {
        _nDefaultItemsPerPage = AppPropertiesService.getPropertyInt( PROPERTY_DELIBERATIONS_DESIGNATION_PER_PAGE, 10 );
        _nIdSeanceSelected = -1;
        _nStatutActeSelected = -1;
        _nTypeUrlRetour = -1;
        _bHasChanged = false;
        _bUrlRetour = false;
        _strUrlRetour = null;
        _bComeFromActe = false;
    }

    protected abstract GDeliberationDesignation newDeliberationInstance(  );

    protected abstract void getModificationDeliberationDesignationSpec( HttpServletRequest request,
        GDeliberationDesignation deliberation );

    protected abstract void getDeliberationDesignationListSpec( HashMap<String, Object> model,
        HttpServletRequest request );

    protected abstract void getDeliberationDesignationCreationSpec( HashMap<String, Object> model,
        HttpServletRequest request );

    protected abstract void getModificationDeliberationDesignationSpecEnd( HashMap<String, Object> model,
        HttpServletRequest request );

    protected abstract void getDeliberationDesignationListFilterSpec( DeliberationDesignationFilter filter,
        HttpServletRequest request );

    protected abstract void doCreationDeliberationDesignationSpec( GDeliberationDesignation deliberation,
        HttpServletRequest request );

    protected abstract void doModificationDeliberationDesignationSpec( GDeliberationDesignation deliberation,
        HttpServletRequest request );

    protected abstract void doModificationActeSpec( Acte<GSeance, GFichier> acte, HttpServletRequest request );

    protected abstract boolean isValideFormeSpec( List<String> champsNonSaisie, HttpServletRequest request,
        boolean bIsCreation );

    protected abstract GFichier newFichierInstance(  );

    protected abstract Plugin getPlugin(  );

    protected abstract String getParameterPluginName(  );

    /**
     * Reinitialise tous les attributs de session
     * @param request requete HTTP
     */
    public void reinitSession( HttpServletRequest request )
    {
        _bHasChanged = false;
        _nIdSeanceSelected = -1;
        _nStatutActeSelected = -1;
        _bUrlRetour = false;
        _strUrlRetour = null;
        _nTypeUrlRetour = -1;
        _bComeFromActe = false;
    }

    /**
     * Permet d'initialiser l'url de retour
     * @param strUrl url de retour
     * @param request la requte http
     */
    public void initUrlRetour( String strUrl, HttpServletRequest request )
    {
        _strUrlRetour = strUrl;
        _bUrlRetour = true;

        if ( _strUrlRetour.equals( _acteService.getUrlGestionActe(  ) ) )
        {
            _nTypeUrlRetour = 1;
        }
        else
        {
            _nTypeUrlRetour = 0;
        }

        _strUrlRetour += ( OdsConstants.CONSTANTE_POINT_INTERROGATION + getParameterPluginName(  ) );

        if ( request.getParameter( OdsParameters.ID_DELIBERATION ) != null )
        {
            _strUrlRetour += ( OdsConstants.CONSTANTE_ET + OdsParameters.ID_DELIBERATION + OdsConstants.CONSTANTE_EGAL +
            request.getParameter( OdsParameters.ID_DELIBERATION ) );
        }
    }

    public void comeFromActe(  )
    {
        _bComeFromActe = true;
    }

    /**
     *
     * @param request
     * @return
     */
    public boolean isPublie( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        boolean bIsPublie = false;

        int nIdDeliberation = OdsUtils.getIntegerParameter( OdsParameters.ID_DELIBERATION, request );

        if ( nIdDeliberation != -1 )
        {
            GDeliberationDesignation deliberation = _deliberationDesignationHome.findByPrimaryKey( nIdDeliberation,
                    plugin );

            if ( deliberation != null )
            {
                bIsPublie = deliberation.getEnLigne(  );
            }
        }

        return bIsPublie;
    }

    /**
     * Retourne la map contentant les lments utiles au template HTML de la liste
     * @param request la requte http
     * @param strHomeUrl l'URL de le retour
     * @return model la map contentant les lments utiles au template HTML de la liste
     */
    public HashMap<String, Object> getDeliberationDesignationList( HttpServletRequest request, String strHomeUrl )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        _strCurrentPageIndex = Paginator.getPageIndex( request, Paginator.PARAMETER_PAGE_INDEX, _strCurrentPageIndex );
        _nItemsPerPage = Paginator.getItemsPerPage( request, Paginator.PARAMETER_ITEMS_PER_PAGE, _nItemsPerPage,
                _nDefaultItemsPerPage );

        // on recupre la liste des dsignations de deliberation correspondants au filtre
        DeliberationDesignationFilter filter = new DeliberationDesignationFilter(  );

        if ( request.getParameter( OdsParameters.ID_SEANCE ) != null )
        {
            _nIdSeanceSelected = OdsUtils.getIntegerParameter( OdsParameters.ID_SEANCE, request );
        }

        filter.setIdSeance( _nIdSeanceSelected );

        getDeliberationDesignationListFilterSpec( filter, request );

        if ( request.getParameter( OdsParameters.STATUT_ACTE ) != null )
        {
            _nStatutActeSelected = OdsUtils.getIntegerParameter( OdsParameters.STATUT_ACTE, request );
        }

        filter.setStatutActe( _nStatutActeSelected );

        List<GDeliberationDesignation> deliberations = _deliberationDesignationHome.findByFilter( filter, plugin );

        Paginator paginator = new Paginator( deliberations, _nItemsPerPage, strHomeUrl, Paginator.PARAMETER_PAGE_INDEX,
                _strCurrentPageIndex );

        // Ajouts des lments ncessaire dans la Hashmap
        model.put( MARK_LISTE_SEANCES, _seanceHome.findOldSeance( plugin ) );
        model.put( MARK_ID_SEANCE_SELECTED, _nIdSeanceSelected );

        //rcupration de l'identifiant de la sance  partir de laquelle les crations d'acte sont autorises
        int nIdFirstSeanceForActe = AppPropertiesService.getPropertyInt( OdsProperties.PROPERTY_CREATION_ID_SEANCE, 1 );
        model.put( OdsMarks.MARK_ACTE_CREATION_ID_SEANCE, nIdFirstSeanceForActe );

        model.put( MARK_LISTE_DELIBERATIONS, paginator.getPageItems(  ) );
        model.put( MARK_LISTE_DELIBERATIONS_SIZE_TOTAL, deliberations.size(  ) );
        model.put( OdsMarks.MARK_PAGINATOR, paginator );
        model.put( OdsMarks.MARK_NB_ITEMS_PER_PAGE, OdsConstants.CONSTANTE_CHAINE_VIDE + _nItemsPerPage );
        getDeliberationDesignationListSpec( model, request );
        model.put( MARK_STATUT_ACTE_SELECTED, _nStatutActeSelected );

        return model;
    }

    /**
     * retourne la map contentant les lments utiles au template HTML de creation
     * @param request la requte http
     * @return model la map contentant les lments utiles au template HTML de creation
     */
    public HashMap<String, Object> getCreationDeliberationDesignation( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        model.put( MARK_LISTE_CATEGORIE_DELIBERATION,
            _categorieDeliberationService.initRefListCategorieDeliberation( getPlugin(  ), false ) );
        model.put( MARK_LISTE_SEANCES, _seanceHome.findOldSeance( plugin ) );
        model.put( MARK_ID_SEANCE_SELECTED, _nIdSeanceSelected );

        int nCodeCategorieDefaut = AppPropertiesService.getPropertyInt( PROPERTY_CODE_DELIBERATION_DEFAUT,
                DEFAULT_CODE_DELIBERATION );
        int nIdCategorieDefaut = _categorieDeliberationHome.findIdCategorieByCode( nCodeCategorieDefaut, plugin );
        model.put( MARK_ID_CATEGORIE_DELIBERATION_SELECTED, nIdCategorieDefaut );
        getDeliberationDesignationCreationSpec( model, request );

        return model;
    }

    /**
     * retourne la map contentant les lments utiles au template HTML de modification
     * @param request la requte http
     * @return model la map contentant les lments utiles au template HTML de modification
     */
    public HashMap<String, Object> getModificationDeliberationDesignation( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        int nIdDeliberation = OdsUtils.getIntegerParameter( OdsParameters.ID_DELIBERATION, request );
        GDeliberationDesignation deliberation = _deliberationDesignationHome.findByPrimaryKey( nIdDeliberation, plugin );

        if ( deliberation == null )
        {
            return null;
        }

        getModificationDeliberationDesignationSpec( request, deliberation );

        model.put( MARK_LISTE_CATEGORIE_DELIBERATION,
            _categorieDeliberationService.initRefListCategorieDeliberation( getPlugin(  ), false ) );

        if ( request.getParameter( OdsParameters.PREPARER_ACTE ) != null )
        {
            Acte<GSeance, GFichier> acteCourant = new Acte<GSeance, GFichier>(  );

            NatureActe natureActe = new NatureActe(  );
            natureActe.setIdNatureDocument( NatureActeEnum.DELIBERATION.getId(  ) );
            acteCourant.setNatureDocument( natureActe );

            StatutActe statut = _statutActeHome.findByPrimaryKey( StatutActeEnum.A_PREPARER.getId(  ), plugin );
            acteCourant.setStatutActe( statut );

            CategorieDeliberation categorie = _categorieDeliberationHome.findByPrimaryKey( deliberation.getCategorie(  )
                                                                                                       .getIdCategorie(  ),
                    plugin );
            acteCourant.setCategorie( categorie );
            acteCourant.setReference( deliberation.getNumero(  ) );
            acteCourant.setSeance( deliberation.getSeance(  ) );
            acteCourant.setFormationConseil( deliberation.getFormationConseil(  ) );
            acteCourant.setIdEntite( deliberation.getId(  ) );
            acteCourant.setIdTypeEntite( ITypeEntite.DESIGNATIONS_DE_REPRESENTANTS );

            GFichier deliberationFinale = deliberation.getFichierSc(  );

            if ( deliberationFinale != null )
            {
                acteCourant.setDeliberationFinale( deliberationFinale );
                acteCourant.setPoids( deliberationFinale.getTaille(  ) );

                int nMaxPoids = AppPropertiesService.getPropertyInt( OdsProperties.PROPERTY_POIDS_MAX_ACTE,
                        OdsConstants.DEFAULT_POIDS_MAX_ACTE );

                if ( acteCourant.getPoids(  ) > nMaxPoids )
                {
                    acteCourant.setIdTransmission( 0 );
                }
                else
                {
                    acteCourant.setIdTransmission( 1 );
                }
            }

            deliberation.setActeCourant( acteCourant );
        }

        model.put( OdsMarks.MARK_DELIBERATION_DESIGNATION, deliberation );
        model.put( MARK_LISTE_SEANCES, _seanceHome.findOldSeance( plugin ) );
        model.put( MARK_LISTE_DIRECTION, _directionService.initRefListDirection( true, false ) );

        //rcupration de l'identifiant de la sance  partir de laquelle les crations d'acte sont autorises
        int nIdFirstSeanceForActe = AppPropertiesService.getPropertyInt( OdsProperties.PROPERTY_CREATION_ID_SEANCE, 1 );
        model.put( OdsMarks.MARK_ACTE_CREATION_ID_SEANCE, nIdFirstSeanceForActe );

        //rcupration des nature de document       
        model.put( MARK_LISTE_NATURE_DOCUMENT, _acteService.getRefListNatureActe(  ) );

        //si l'on a cliqu sur visualiser l'acte  partir de la liste de suivi des actes
        int nIdActe = OdsUtils.getIntegerParameter( MARK_ID_ACTE, request );

        if ( nIdActe != -1 )
        {
            model.put( MARK_ID_ACTE, nIdActe );

            Acte<GSeance, GFichier> acte = _acteHome.findByPrimaryKey( nIdActe, plugin );
            model.put( MARK_ACTE, acte );
        }

        getModificationDeliberationDesignationSpecEnd( model, request );

        return model;
    }

    /**
     * retourne la map contentant les lments utiles au template HTML de l'historique
     * @param request la requte http
     * @return model la map contentant les lments utiles au template HTML de l'historique
     */
    public HashMap<String, Object> getHistoriqueDeliberationDesignation( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        int nIdDeliberation = OdsUtils.getIntegerParameter( OdsParameters.ID_DELIBERATION, request );
        IDeliberationDesignation<GSeance, GFichier> deliberation = _deliberationDesignationHome.findByPrimaryKey( nIdDeliberation,
                plugin );

        if ( deliberation == null )
        {
            return null;
        }

        List<Historique> historiques = _historiqueHome.getHistoriqueList( deliberation, plugin );
        model.put( MARK_LISTE_HISTORIQUES, historiques );

        return model;
    }

    /**
     * retourne une demande de confirmation pour la suppression de l dlibration choisie
     * @param request la requte http
     * @return la page de confirmation de suppression d'une dlibration
     */
    public String getSuppressionDeliberationDesignation( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        if ( request.getParameter( OdsParameters.ID_DELIBERATION ) != null )
        {
            int nIdDeliberation = OdsUtils.getIntegerParameter( OdsParameters.ID_DELIBERATION, request );
            IDeliberationDesignation<GSeance, GFichier> deliberation = _deliberationDesignationHome.findByPrimaryKey( nIdDeliberation,
                    plugin );

            UrlItem url = new UrlItem( PROPERTY_URL_DO_SUPPRESSION_DELIBERATION_DESIGNATION_GESTION_AVAL_JSP );
            url.addParameter( OdsParameters.ID_DELIBERATION, nIdDeliberation );

            if ( deliberation == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            if ( deliberation.getActeCourant(  ) != null )
            {
                return AdminMessageService.getMessageUrl( request, MESSAGE_ERROR_DELETE_DELIBERATION,
                    AdminMessage.TYPE_STOP );
            }
            else if ( ( deliberation.getActes(  ) != null ) && !deliberation.getActes(  ).isEmpty(  ) )
            {
                return AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRM_DELETE_DELIBERATION_WITH_ACTES,
                    url.getUrl(  ), AdminMessage.TYPE_CONFIRMATION );
            }

            return AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRM_DELETE_DELIBERATION, url.getUrl(  ),
                AdminMessage.TYPE_CONFIRMATION );
        }
        else
        {
            return OdsConstants.CONSTANTE_CHAINE_VIDE;
        }
    }

    /**
     * retourne la map contentant les lments utiles au template HTML
     * @param request la requte http
     * @return model la map contentant les lments utiles au template HTML
     */
    public HashMap<String, Object> getConfirmFinaliser( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        int nIdDeliberation = OdsUtils.getIntegerParameter( OdsParameters.ID_DELIBERATION, request );
        IDeliberationDesignation<GSeance, GFichier> deliberation = _deliberationDesignationHome.findByPrimaryKey( nIdDeliberation,
                plugin );
        OdsUtils.testNullObject( deliberation, request );

        model.put( OdsMarks.MARK_DELIBERATION_DESIGNATION, deliberation );

        return model;
    }

    /**
     * Creation d'une dlibration de dsignation
     * si tous les champs obligatoires du formulaire de cration d'une dlibration ont t renseign,<BR>
     * la mthode retourne l'url de gestion des dlibrations,<BR>
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request la requte http
     * @return l'url de creation d'une dlibration de dsignation
     */
    public String doCreationDeliberationDesignation( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        List<String> champsNonSaisie = new ArrayList<String>(  );
        String strChampsError = OdsConstants.CONSTANTE_CHAINE_VIDE;

        //Si on a cliqu sur le bouton "Annuler"
        if ( request.getParameter( OdsParameters.CONSTANTE_ACTION_CANCEL ) != null )
        {
            return AppPathService.getBaseUrl( request ) + JSP_URL_LISTE_DELIBERATIONS;
        }

        if ( isValideForme( champsNonSaisie, request, true ) )
        {
            MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;

            GDeliberationDesignation deliberation = newDeliberationInstance(  );

            deliberation.setIntitule( request.getParameter( OdsParameters.DELIBERATION_INTITULE ).trim(  ) );
            deliberation.setNumero( request.getParameter( OdsParameters.DELIBERATION_NUMERO ).trim(  ) );
            deliberation.setEnLigne( false );

            int nModificationIdSeance = OdsUtils.getIntegerParameter( OdsParameters.ID_SEANCE, request );
            doCreationDeliberationDesignationSpec( deliberation, request );

            int nIdCategorie = OdsUtils.getIntegerParameter( OdsParameters.ID_CATEGORIE, request );

            GSeance seance = _seanceHome.findByPrimaryKey( nModificationIdSeance, plugin );
            deliberation.setSeance( seance );

            CategorieDeliberation categorie = _categorieDeliberationHome.findByPrimaryKey( nIdCategorie, plugin );
            deliberation.setCategorie( categorie );

            /* Vrification de l'extension du fichier de l'arrt original */
            FileItem fileItemScannee = multipartRequest.getFile( OdsParameters.DELIBERATION_SC );

            if ( ( fileItemScannee != null ) && ( fileItemScannee.getName(  ) != null ) &&
                    !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItemScannee.getName(  ) ) )
            {
                String strExtension = fileItemScannee.getName(  ).substring( fileItemScannee.getName(  ).length(  ) -
                        3 ).toUpperCase(  );

                if ( ( strExtension == null ) || ( strExtension.length(  ) != 3 ) ||
                        !OdsConstants.CONSTANTE_PDF.equals( strExtension ) )
                {
                    if ( strChampsError.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                    {
                        strChampsError = PROPERTY_LABEL_FICHIER_SCANNE;
                    }
                }
            }

            if ( !strChampsError.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                Object[] messagesArgs = { I18nService.getLocalizedString( strChampsError, request.getLocale(  ) ), };

                return AdminMessageService.getMessageUrl( request, PROPERTY_LABEL_FICHIER_NO_PDF, messagesArgs,
                    AdminMessage.TYPE_STOP );
            }
            else
            {
                /* Cration du fichier de la dlibration scanne */
                createFichier( deliberation, fileItemScannee, false );
            }

            //rcupration de la date du vote
            if ( ( request.getParameter( OdsParameters.DATE_VOTE ) != null ) &&
                    !request.getParameter( OdsParameters.DATE_VOTE ).trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                String strDateVote = request.getParameter( OdsParameters.DATE_VOTE ).trim(  );
                Timestamp tsDateVote = DateUtils.getDate( strDateVote, true );

                if ( tsDateVote == null )
                {
                    Object[] messagesArgs = 
                        {
                            I18nService.getLocalizedString( PROPERTY_LABEL_DATE_VOTE, request.getLocale(  ) ),
                        };

                    return AdminMessageService.getMessageUrl( request,
                        OdsProperties.PROPERTY_MESSAGE_DATE_FORMAT_ERROR, messagesArgs, AdminMessage.TYPE_ERROR );
                }
                else
                {
                    deliberation.setDateVote( tsDateVote );
                }
            }

            // Cration de la dlibration
            deliberation.setId( _deliberationDesignationHome.create( deliberation, plugin ) );

            if ( ( deliberation != null ) && ( deliberation.getSeance(  ) != null ) )
            {
                _searchService.addObjectToIndex( deliberation, true, plugin );
            }
        }
        else
        {
            String strChamp = ( champsNonSaisie.get( 0 ) != null ) ? champsNonSaisie.get( 0 ) : Messages.MANDATORY_FIELDS;
            Object[] messagesArgs = { I18nService.getLocalizedString( strChamp, request.getLocale(  ) ) };

            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED,
                messagesArgs, AdminMessage.TYPE_STOP );
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * Modifie la dlibration de dsignation
     * Si la dlibration est en ligne, alors la version est incrmente de 1 et ajout d'un historique
     * si tous les champs obligatoires du formulaire de modification d'une dlibration ont t renseigns,<BR>
     * la mthode retourne l'url de gestion des dlibrations,<BR>
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request la requte http
     * @return l'url de modification d'une dlibration
     */
    public String doModificationDeliberationDesignation( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        List<String> champsNonSaisie = new ArrayList<String>(  );
        String strChampsError = OdsConstants.CONSTANTE_CHAINE_VIDE;
        _bHasChanged = false;

        String strReturn;

        //Si on a cliqu sur le bouton "Annuler"
        if ( request.getParameter( OdsParameters.CONSTANTE_ACTION_CANCEL ) != null )
        {
            if ( _bUrlRetour )
            {
                strReturn = AppPathService.getBaseUrl( request ) + _strUrlRetour;
                _bUrlRetour = false;
                _strUrlRetour = null;
            }
            else
            {
                strReturn = AppPathService.getBaseUrl( request ) + JSP_URL_LISTE_DELIBERATIONS;
            }

            return strReturn;
        }

        if ( isValideForme( champsNonSaisie, request, false ) )
        {
            int nIdDeliberation = OdsUtils.getIntegerParameter( OdsParameters.ID_DELIBERATION, request );
            MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;

            if ( ( nIdDeliberation != -1 ) )
            {
                GDeliberationDesignation deliberation = _deliberationDesignationHome.findByPrimaryKey( nIdDeliberation,
                        plugin );

                if ( deliberation == null )
                {
                    return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                        AdminMessage.TYPE_ERROR );
                }

                /*
                 * Si Preparer acte est cliqu, on recharge la page avec les champs de prparation de l'acte
                 */
                if ( request.getParameter( OdsParameters.PREPARER_ACTE_BUTTON ) != null )
                {
                    if ( ( deliberation.getActeCourant(  ) == null ) ||
                            ( ( deliberation.getActeCourant(  ) != null ) &&
                            ( deliberation.getActeCourant(  ).getStatutActe(  ).getIdStatut(  ) != StatutActeEnum.A_PREPARER.getId(  ) ) ) )
                    {
                        return AppPathService.getBaseUrl( request ) +
                        JSP_URL_GET_MODIFICATION_DELIBERATION_DESIGNATION + OdsConstants.CONSTANTE_POINT_INTERROGATION +
                        OdsParameters.ID_DELIBERATION + OdsConstants.CONSTANTE_EGAL + nIdDeliberation +
                        OdsConstants.CONSTANTE_ET + "preparer_acte=preparer_acte";
                    }

                    strReturn = AppPathService.getBaseUrl( request ) +
                        JSP_URL_GET_MODIFICATION_DELIBERATION_DESIGNATION + OdsConstants.CONSTANTE_POINT_INTERROGATION +
                        OdsParameters.ID_DELIBERATION + OdsConstants.CONSTANTE_EGAL + nIdDeliberation;

                    return strReturn;
                }

                // recupration des modifications et mise  jour des champs si besoin
                int nModificationIdSeance = OdsUtils.getIntegerParameter( OdsParameters.ID_SEANCE, request );

                if ( nModificationIdSeance != deliberation.getSeance(  ).getIdSeance(  ) )
                {
                    GSeance seance = _seanceHome.findByPrimaryKey( nModificationIdSeance, plugin );
                    deliberation.setSeance( seance );
                    _bHasChanged = true;
                }

                String strModificationNumero = request.getParameter( OdsParameters.DELIBERATION_NUMERO ).trim(  );

                if ( !strModificationNumero.equals( deliberation.getNumero(  ) ) )
                {
                    deliberation.setNumero( strModificationNumero );
                    _bHasChanged = true;
                }

                String strModificationIntitule = request.getParameter( OdsParameters.DELIBERATION_INTITULE ).trim(  );

                if ( !strModificationIntitule.equals( deliberation.getIntitule(  ) ) )
                {
                    deliberation.setIntitule( strModificationIntitule );
                    _bHasChanged = true;
                }

                /* Vrification de l'extension du fichier de la dlibration scanne */
                FileItem fileItemSc = multipartRequest.getFile( OdsParameters.DELIBERATION_SC );

                if ( ( fileItemSc != null ) && ( fileItemSc.getName(  ) != null ) &&
                        !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItemSc.getName(  ) ) )
                {
                    String strExtension = fileItemSc.getName(  ).substring( fileItemSc.getName(  ).length(  ) - 3 )
                                                    .toUpperCase(  );

                    if ( ( strExtension == null ) || ( strExtension.length(  ) != 3 ) ||
                            !OdsConstants.CONSTANTE_PDF.equals( strExtension ) )
                    {
                        if ( strChampsError.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                        {
                            strChampsError = PROPERTY_LABEL_FICHIER_SCANNE;
                        }
                    }
                }

                if ( !strChampsError.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                {
                    Object[] messagesArgs = { I18nService.getLocalizedString( strChampsError, request.getLocale(  ) ), };

                    return AdminMessageService.getMessageUrl( request, PROPERTY_LABEL_FICHIER_NO_PDF, messagesArgs,
                        AdminMessage.TYPE_STOP );
                }
                else
                {
                    /* Mise  jour du fichier de la dlibration scanne */
                    updateFichier( deliberation, fileItemSc, false, request );
                }

                int nModificationIdCategorie = OdsUtils.getIntegerParameter( OdsParameters.ID_CATEGORIE, request );

                if ( nModificationIdCategorie != deliberation.getCategorie(  ).getIdCategorie(  ) )
                {
                    CategorieDeliberation categorie = _categorieDeliberationHome.findByPrimaryKey( nModificationIdCategorie,
                            plugin );
                    deliberation.setCategorie( categorie );
                    _bHasChanged = true;
                }

                Acte<GSeance, GFichier> acteCourant = deliberation.getActeCourant(  );

                if ( request.getParameter( OdsParameters.PREPARER_ACTE ) != null )
                {
                    acteCourant = new Acte<GSeance, GFichier>(  );
                    acteCourant.setStatutActe( StatutActeEnum.A_PREPARER.getStatutOnlyWidthId(  ) );
                }

                int nMaxPoids = AppPropertiesService.getPropertyInt( OdsProperties.PROPERTY_POIDS_MAX_ACTE,
                        OdsConstants.DEFAULT_POIDS_MAX_ACTE );

                doModificationDeliberationDesignationSpec( deliberation, request );

                if ( ( request.getParameter( MARK_ID_ACTE ) == null ) && ( acteCourant != null ) &&
                        ( deliberation.getDateVote(  ) != null ) &&
                        ( ( acteCourant.getStatutActe(  ).getIdStatut(  ) == StatutActeEnum.A_PREPARER.getId(  ) ) ||
                        ( acteCourant.getStatutActe(  ).getIdStatut(  ) == StatutActeEnum.EN_PREPARATION.getId(  ) ) ) )
                {
                    //Ajout de la catgorie
                    CategorieDeliberation categorie = _categorieDeliberationHome.findByPrimaryKey( Integer.parseInt( 
                                request.getParameter( OdsParameters.ID_CATEGORIE ) ), plugin );
                    acteCourant.setCategorie( categorie );

                    int nNatureActeID = Integer.parseInt( request.getParameter( OdsParameters.NATURE_DOCUMENT ) );
                    acteCourant.setNatureDocument( _natureActeHome.findByPrimaryKey( nNatureActeID, plugin ) );

                    GFichier deliberationFinale = deliberation.getFichierSc(  );

                    if ( deliberationFinale != null )
                    {
                        acteCourant.setDeliberationFinale( deliberationFinale );
                        acteCourant.setPoids( deliberationFinale.getTaille(  ) );
                    }

                    int nIdModeTransmission = Integer.parseInt( request.getParameter( OdsParameters.MODE_TRANSMISSION ) );

                    if ( ( nIdModeTransmission == 1 ) && ( acteCourant.getPoids(  ) > nMaxPoids ) )
                    {
                        //Mode de transmission automatique, mais le poids est suprieur
                        return AdminMessageService.getMessageUrl( request,
                            OdsProperties.PROPERTY_MESSAGE_MODE_TRANSMISSION_ERROR, AdminMessage.TYPE_STOP );
                    }

                    acteCourant.setIdTransmission( nIdModeTransmission );

                    acteCourant.setIdEntite( deliberation.getId(  ) );
                    acteCourant.setIdTypeEntite( ITypeEntite.DESIGNATIONS_DE_REPRESENTANTS );

                    acteCourant.setReference( request.getParameter( OdsParameters.REFERENCE ) );
                    acteCourant.setStatutActe( StatutActeEnum.EN_PREPARATION.getStatutOnlyWidthId(  ) );
                    acteCourant.setSeance( deliberation.getSeance(  ) );
                    acteCourant.setEnLigne( false );
                    doModificationActeSpec( acteCourant, request );
                    deliberation.setActeCourant( acteCourant );

                    if ( request.getParameter( OdsParameters.PREPARER_ACTE ) != null )
                    {
                        _acteHome.create( acteCourant, plugin );
                    }
                    else
                    {
                        _acteHome.update( acteCourant, plugin );
                    }
                }

                /*
                 * Finalisation de l'acte (si bouton finaliser cliqu)
                 */
                if ( request.getParameter( OdsParameters.FINALISER_ACTE ) != null )
                {
                    if ( acteCourant.getReference(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                    {
                        return AdminMessageService.getMessageUrl( request,
                            OdsProperties.PROPERTY_MESSAGE_REFERENCE_ERROR, AdminMessage.TYPE_ERROR );
                    }

                    if ( ( acteCourant.getIdTransmission(  ) == 1 ) && ( acteCourant.getPoids(  ) > nMaxPoids ) )
                    {
                        strReturn = AppPathService.getBaseUrl( request ) + JSP_URL_FINALISATION_ACTE +
                            OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_DELIBERATION +
                            OdsConstants.CONSTANTE_EGAL + deliberation.getId(  ) + OdsConstants.CONSTANTE_ET +
                            getParameterPluginName(  );

                        return strReturn;
                    }
                    else
                    {
                        if ( _bUrlRetour )
                        {
                            strReturn = AppPathService.getBaseUrl( request ) + _strUrlRetour;
                            _bUrlRetour = false;
                            _strUrlRetour = null;
                        }
                        else
                        {
                            strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;
                        }

                        acteCourant.setStatutActe( StatutActeEnum.A_TRANSMETTRE.getStatutOnlyWidthId(  ) );
                        _acteHome.update( acteCourant, plugin );

                        return strReturn;
                    }
                }

                String strModificationDateVote = request.getParameter( OdsParameters.DATE_VOTE ).trim(  );
                Timestamp tsDateVote = DateUtils.getDate( strModificationDateVote, true );

                if ( ( tsDateVote == null ) && !strModificationDateVote.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                {
                    Object[] messagesArgs = 
                        {
                            I18nService.getLocalizedString( PROPERTY_LABEL_DATE_VOTE, request.getLocale(  ) ),
                        };

                    return AdminMessageService.getMessageUrl( request,
                        OdsProperties.PROPERTY_MESSAGE_DATE_FORMAT_ERROR, messagesArgs, AdminMessage.TYPE_ERROR );
                }
                else
                {
                    if ( ( tsDateVote != null ) ||
                            ( ( deliberation.getDateVote(  ) != null ) && ( tsDateVote != null ) &&
                            !tsDateVote.equals( deliberation.getDateVote(  ) ) ) ||
                            ( ( deliberation.getDateVote(  ) != null ) && ( tsDateVote == null ) ) )
                    {
                        deliberation.setDateVote( tsDateVote );
                        _bHasChanged = true;
                    }
                }

                if ( deliberation.getEnLigne(  ) && _bHasChanged )
                {
                    deliberation.setVersion( deliberation.getVersion(  ) + 1 );

                    Timestamp dateForVersion = DateUtils.getCurrentDate(  );

                    //gestion de l'historique du voeu
                    Historique historique = new Historique(  );
                    historique.setVersion( deliberation.getVersion(  ) );
                    historique.setDatePublication( dateForVersion );
                    historique.setIdEntite( deliberation.getId(  ) );
                    historique.setIdTypeEntite( ITypeEntite.DESIGNATIONS_DE_REPRESENTANTS );
                    _historiqueHome.create( historique, plugin );
                }

                if ( ( deliberation != null ) && ( deliberation.getSeance(  ) != null ) )
                {
                    _searchService.updateObjectInIndex( deliberation, true, plugin );
                }

                // On met  jour la dlibration
                _deliberationDesignationHome.update( deliberation, plugin );
            }
            else
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_STOP );
            }

            /*
             * Si Appliquer est cliqu, on enregistre et recharge la page
             */
            if ( request.getParameter( OdsParameters.CONSTANTE_ACTION_APPLY ) != null )
            {
                if ( request.getParameter( MARK_ID_ACTE ) != null )
                {
                    strReturn = AppPathService.getBaseUrl( request ) +
                        JSP_URL_GET_VISUALISATION_DELIBERATION_DESIGNATION_FOR_ACTE +
                        OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_DELIBERATION +
                        OdsConstants.CONSTANTE_EGAL + nIdDeliberation;

                    int nIdActe = OdsUtils.getIntegerParameter( MARK_ID_ACTE, request );
                    strReturn += ( OdsConstants.CONSTANTE_ET + OdsParameters.ID_ACTE + OdsConstants.CONSTANTE_EGAL +
                    nIdActe );
                }
                else
                {
                    if ( _nTypeUrlRetour == 1 )
                    {
                        strReturn = AppPathService.getBaseUrl( request ) +
                            JSP_URL_GET_MODIFICATION_DELIBERATION_DESIGNATION_FOR_ACTE +
                            OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_DELIBERATION +
                            OdsConstants.CONSTANTE_EGAL + nIdDeliberation;
                    }
                    else
                    {
                        strReturn = AppPathService.getBaseUrl( request ) +
                            JSP_URL_GET_MODIFICATION_DELIBERATION_DESIGNATION +
                            OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_DELIBERATION +
                            OdsConstants.CONSTANTE_EGAL + nIdDeliberation;
                    }
                }

                return strReturn;
            }
        }
        else
        {
            String strChamp = ( champsNonSaisie.get( 0 ) != null ) ? champsNonSaisie.get( 0 ) : Messages.MANDATORY_FIELDS;
            Object[] messagesArgs = { I18nService.getLocalizedString( strChamp, request.getLocale(  ) ) };

            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED,
                messagesArgs, AdminMessage.TYPE_STOP );
        }

        if ( _bUrlRetour )
        {
            strReturn = AppPathService.getBaseUrl( request ) + _strUrlRetour;
            _bUrlRetour = false;
            _strUrlRetour = null;
        }
        else
        {
            strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;
        }

        return strReturn;
    }

    /**
     * Publication d'une dlibration de dsignation
     * Si "OdsParameters.DELIBERATION_IS_PUBLIER" gale  TRUE, alors la mthode <b>publie</b> la dlibration et incrmente la version<BR>
     *                 Dans ce cas, un historique est cre
     * Sinon la mthode change le statut enLigne d'une dlibration a FALSE
     * @param request la requte http
     * @return l'url
     */
    public String doPublicationDeliberationDesignation( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        Timestamp dateForVersion = DateUtils.getCurrentDate(  );

        String strIsPublier = request.getParameter( OdsParameters.DELIBERATION_IS_PUBLIER );
        boolean bEstPublication = Boolean.parseBoolean( OdsConstants.CONSTANTE_CHAINE_VIDE + strIsPublier );

        int nIdDeliberation = OdsUtils.getIntegerParameter( OdsParameters.ID_DELIBERATION, request );

        if ( nIdDeliberation != -1 )
        {
            IDeliberationDesignation<GSeance, GFichier> deliberation = _deliberationDesignationHome.findByPrimaryKey( nIdDeliberation,
                    plugin );

            if ( deliberation == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            if ( deliberation.getFichierSc(  ) == null )
            {
                return AdminMessageService.getMessageUrl( request, PROPERTY_MESSAGE_FILES_NOT_FOUND,
                    AdminMessage.TYPE_STOP );
            }

            if ( !( ( deliberation.getEnLigne(  ) && bEstPublication ) ||
                    ( !deliberation.getEnLigne(  ) && !bEstPublication ) ) )
            {
                if ( bEstPublication && ( deliberation.getVersion(  ) < 1 ) )
                {
                    deliberation.setVersion( 0 );
                }

                //On publie la dlibration
                _deliberationDesignationHome.publication( deliberation.getId(  ), dateForVersion,
                    deliberation.getVersion(  ), bEstPublication, plugin );

                //On publie galement l'acte rattach s'il existe
                if ( deliberation.getActeCourant(  ) != null )
                {
                    _acteHome.publication( deliberation.getActeCourant(  ).getId(  ), bEstPublication, plugin );
                }

                //On publie les fichiers associs
                _fichierHome.publication( deliberation.getFichierSc(  ).getId(  ), dateForVersion,
                    deliberation.getVersion(  ), bEstPublication, plugin );

                if ( bEstPublication )
                {
                    Historique historique = new Historique(  );
                    historique.setVersion( deliberation.getVersion(  ) + 1 );
                    historique.setDatePublication( dateForVersion );
                    historique.setIdEntite( deliberation.getId(  ) );
                    historique.setIdTypeEntite( ITypeEntite.DESIGNATIONS_DE_REPRESENTANTS );
                    _historiqueHome.create( historique, plugin );
                }

                // on horodate la (de)publication
                HorodatageActionEnum action = ( bEstPublication ) ? HorodatageActionEnum.PUBLICATION
                                                                  : HorodatageActionEnum.DE_PUBLICATION;
                _horodatageService.tracePublication( deliberation, action );
            }
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * supprime la dlibration de dsignation choisie
     * @param request la requte http
     * @return l'url de suppression de la dlibration de dsignation
     */
    public String doSuppressionDeliberationDesignation( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        int nIdDeliberation = OdsUtils.getIntegerParameter( OdsParameters.ID_DELIBERATION, request );

        if ( nIdDeliberation != -1 )
        {
            GDeliberationDesignation deliberation = _deliberationDesignationHome.findByPrimaryKey( nIdDeliberation,
                    plugin );

            if ( deliberation == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            //Suppression des fichiers rattachs
            try
            {
                GFichier fichierOr = deliberation.getFichierOr(  );

                _historiqueHome.remove( deliberation, plugin );

                if ( ( deliberation != null ) && ( deliberation.getSeance(  ) != null ) )
                {
                    _searchService.deleteObjectFromIndex( deliberation, true, plugin );
                }

                _deliberationDesignationHome.remove( deliberation, plugin );

                if ( fichierOr != null )
                {
                    fichierOr = _fichierHome.findByPrimaryKey( fichierOr.getId(  ), plugin );

                    FichierPhysique fichierPhysiqueOr = _fichierPhysiqueHome.findByPrimaryKey( fichierOr.getFichier(  )
                                                                                                        .getIdFichier(  ),
                            plugin );

                    _historiqueHome.remove( fichierOr, plugin );

                    if ( fichierOr.getSeance(  ) != null )
                    {
                        ISeance seanceByFichier = fichierOr.getSeance(  );
                        ISeance nextSeance = _seanceHome.getProchaineSeance( plugin );

                        if ( ( nextSeance != null ) && !seanceByFichier.equals( nextSeance ) &&
                                ( ( fichierOr.getTypdeDocument(  ).getId(  ) == ITypeDocument.DOCUMENT_LIBRE ) ||
                                ( fichierOr.getTypdeDocument(  ).getId(  ) == ITypeDocument.COMPTE_RENDU ) ||
                                ( fichierOr.getTypdeDocument(  ).getId(  ) == ITypeDocument.LISTE_DE_DESIGNATION ) ) )
                        {
                            _searchService.deleteObjectFromIndex( fichierOr, true, plugin );
                        }
                    }

                    _fichierHome.remove( fichierOr, plugin );
                    _fichierPhysiqueHome.remove( fichierPhysiqueOr, plugin );
                }
            }
            catch ( AppException ae )
            {
                if ( ae.getInitialException(  ) instanceof SQLException )
                {
                    return AdminMessageService.getMessageUrl( request, MESSAGE_CANNOT_DELETE_DELIBERATION,
                        AdminMessage.TYPE_STOP );
                }
            }
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * Permet de confirmer la finalisation d'un acte
     *
     * @param request la requete HTTP
     * @return strUrl url de retour
     */
    public String doFinaliserActe( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;
        int nIdDeliberation = OdsUtils.getIntegerParameter( OdsParameters.ID_DELIBERATION, request );
        IDeliberationDesignation<GSeance, GFichier> deliberation = _deliberationDesignationHome.findByPrimaryKey( nIdDeliberation,
                plugin );
        OdsUtils.testNullObject( deliberation, request );

        Acte<GSeance, GFichier> acteCourant = deliberation.getActeCourant(  );

        //Si on a cliqu sur le bouton "Annuler"
        if ( request.getParameter( OdsParameters.CONSTANTE_ACTION_CANCEL ) != null )
        {
            if ( _nTypeUrlRetour == 1 )
            {
                strReturn = AppPathService.getBaseUrl( request ) +
                    JSP_URL_GET_MODIFICATION_DELIBERATION_DESIGNATION_FOR_ACTE +
                    OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_DELIBERATION +
                    OdsConstants.CONSTANTE_EGAL + nIdDeliberation;
            }
            else
            {
                strReturn = AppPathService.getBaseUrl( request ) + JSP_URL_GET_MODIFICATION_DELIBERATION_DESIGNATION +
                    OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_DELIBERATION +
                    OdsConstants.CONSTANTE_EGAL + nIdDeliberation;
            }
        }
        else
        {
            String strActe = request.getParameter( OdsParameters.CHECKBOX_DELETE );

            if ( strActe != null )
            {
                acteCourant.setStatutActe( StatutActeEnum.A_TRANSMETTRE.getStatutOnlyWidthId(  ) );

                GFichier deliberationFinale = deliberation.getFichierSc(  );

                if ( deliberationFinale != null )
                {
                    acteCourant.setDeliberationFinale( deliberationFinale );

                    int nMaxPoids = AppPropertiesService.getPropertyInt( OdsProperties.PROPERTY_POIDS_MAX_ACTE,
                            OdsConstants.DEFAULT_POIDS_MAX_ACTE );

                    if ( deliberationFinale.getTaille(  ) > nMaxPoids )
                    {
                        acteCourant.setIdTransmission( 0 );
                    }
                    else
                    {
                        acteCourant.setIdTransmission( 1 );
                    }
                }

                _acteHome.update( acteCourant, plugin );

                if ( _bUrlRetour )
                {
                    strReturn = AppPathService.getBaseUrl( request ) + _strUrlRetour;
                    _bUrlRetour = false;
                    _strUrlRetour = null;
                }
                else
                {
                    strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;
                }
            }
            else
            {
                Object[] messagesArgs = null;

                strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_CHECKBOX,
                        messagesArgs, AdminMessage.TYPE_STOP );
            }
        }

        return strReturn;
    }

    /**
     * Tlcharge un fichier. Renvoie un flux de bytes du fichier dans l'objet
     * HttpServletResponse response
     *
     * @param request la requte HTTP
     * @return fichierTelecharger le fichier  tlcharger
     */
    public GFichier doDownloadFichier( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        GFichier fichierTelecharger = null;

        String strIdFichier = request.getParameter( OdsParameters.ID_FICHIER );

        if ( strIdFichier != null )
        {
            int nIdFichier = OdsUtils.getIntegerParameter( OdsParameters.ID_FICHIER, request );

            fichierTelecharger = _fichierHome.findByPrimaryKey( nIdFichier, plugin );

            FichierPhysique fichierPhysique = _fichierPhysiqueHome.findByPrimaryKey( fichierTelecharger.getFichier(  )
                                                                                                       .getIdFichier(  ),
                    plugin );
            fichierTelecharger.setFichier( fichierPhysique );
        }

        return fichierTelecharger;
    }

    /**
     * methode qui test si dans le formulaire de saisi d'un fichier,l'ensemble des critres obligatoires ont t renseign
     * @param request la requte http
     * @param champsNonSaisie List<String>
     * @param isCreation true si on est en cration, false si on est en modification
     * @return VRAI si le formulaire envoy est valide
     */
    private boolean isValideForme( List<String> champsNonSaisie, HttpServletRequest request, boolean isCreation )
    {
        boolean isValid = true;

        if ( OdsUtils.getIntegerParameter( OdsParameters.ID_SEANCE, request ) == -1 )
        {
            champsNonSaisie.add( PROPERTY_LABEL_SEANCE );
            isValid = false;
        }

        String strNumero = request.getParameter( OdsParameters.DELIBERATION_NUMERO ).trim(  );

        if ( ( strNumero == null ) || OdsConstants.CONSTANTE_CHAINE_VIDE.equals( strNumero ) )
        {
            champsNonSaisie.add( PROPERTY_LABEL_NUMERO );
            isValid = false;
        }

        String strCreationIntitule = request.getParameter( OdsParameters.DELIBERATION_INTITULE ).trim(  );

        if ( ( strCreationIntitule == null ) || OdsConstants.CONSTANTE_CHAINE_VIDE.equals( strCreationIntitule ) )
        {
            champsNonSaisie.add( PROPERTY_LABEL_INTITULE );
            isValid = false;
        }

        if ( OdsUtils.getIntegerParameter( OdsParameters.ID_CATEGORIE, request ) == -1 )
        {
            champsNonSaisie.add( PROPERTY_LABEL_CATEGORIE );
            isValid = false;
        }

        isValid &= isValideFormeSpec( champsNonSaisie, request, isCreation );

        return isValid;
    }

    /**
     * Permet de creer un fichier du type qui est pass en paramtre et de
     * l'associ au pdd pass en parametre
     *
     * @param deliberation DeliberationDesignation
     * @param fileItem FileItem
     * @param isOriginale boolean
     */
    private void createFichier( final IDeliberationDesignation<GSeance, GFichier> deliberation,
        final FileItem fileItem, boolean isOriginale )
    {
        Plugin plugin = getPlugin(  );

        if ( ( fileItem != null ) && ( fileItem.getName(  ) != null ) &&
                !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItem.getName(  ) ) )
        {
            GFichier newFichier = newFichierInstance(  );
            FichierPhysique newFichierPhysique = new FichierPhysique(  );

            newFichierPhysique.setDonnees( fileItem.get(  ) );
            newFichierPhysique.setIdFichier( _fichierPhysiqueHome.create( newFichierPhysique, plugin ) );
            newFichier.setFichier( newFichierPhysique );
            newFichier.setNom( StringUtils.getNameCourt( fileItem ) );

            String strExtension = fileItem.getName(  ).substring( fileItem.getName(  ).length(  ) - 3 ).toUpperCase(  );
            newFichier.setExtension( strExtension );

            newFichier.setTaille( (int) fileItem.getSize(  ) );

            newFichier.setSeance( deliberation.getSeance(  ) );
            newFichier.setFormationConseil( deliberation.getFormationConseil(  ) );
            newFichier.setTypdeDocument( _fichierHome.findTypeDocumentsById( ITypeDocument.DELIBERATION_DESIGNATION,
                    plugin ) );
            newFichier.setIntitule( deliberation.getNumero(  ) );
            newFichier.setEnLigne( false );
            newFichier.setVersion( 1 );

            _fichierHome.create( newFichier, plugin );

            if ( newFichier.getSeance(  ) != null )
            {
                ISeance seanceByFichier = newFichier.getSeance(  );
                ISeance nextSeance = _seanceHome.getProchaineSeance( plugin );

                if ( ( nextSeance != null ) && !seanceByFichier.equals( nextSeance ) &&
                        ( ( newFichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.DOCUMENT_LIBRE ) ||
                        ( newFichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.COMPTE_RENDU ) ||
                        ( newFichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.LISTE_DE_DESIGNATION ) ) )
                {
                    _searchService.addObjectToIndex( newFichier, true, plugin );
                }
            }

            Timestamp dateForVersion = DateUtils.getCurrentDate(  );

            // Ajout de l'historique
            Historique historique = new Historique(  );
            historique.setVersion( 1 );
            historique.setDatePublication( dateForVersion );
            historique.setIdEntite( newFichier.getId(  ) );
            historique.setIdTypeEntite( ITypeEntite.DESIGNATIONS_DE_REPRESENTANTS );
            _historiqueHome.create( historique, plugin );

            if ( !isOriginale )
            {
                deliberation.setFichierSc( newFichier );
            }
        }
    }

    /**
     * Permet de creer un fichier du type qui est passe en parametre et de
     * l'associe au pdd pass en parametre
     *
     * @param deliberation DeliberationDesignation
     * @param fileItem FileItem
     * @param isOriginale boolean
     */
    private void updateFichier( IDeliberationDesignation<GSeance, GFichier> deliberation, final FileItem fileItem,
        boolean isOriginale, HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        FichierPhysique fichierPhysique = new FichierPhysique(  );
        GFichier fichier = null;

        if ( ( fileItem != null ) && ( fileItem.getName(  ) != null ) &&
                !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItem.getName(  ) ) )
        {
            _bHasChanged = true;

            if ( !isOriginale )
            {
                fichier = deliberation.getFichierSc(  );
            }

            if ( fichier == null )
            {
                //on cre un fichier vide en base pour pouvoir le mettre  jour
                createFichier( deliberation, fileItem, isOriginale );

                if ( !isOriginale )
                {
                    fichier = deliberation.getFichierSc(  );
                }
            }

            OdsUtils.testNullObject( fichier, request );

            fichierPhysique.setIdFichier( fichier.getFichier(  ).getIdFichier(  ) );
            // Modification de l'objet reprsentant le fichier physique
            fichierPhysique.setDonnees( fileItem.get(  ) );
            _fichierPhysiqueHome.update( fichierPhysique, plugin );
            fichier.setFichier( fichierPhysique );
            fichier.setNom( StringUtils.getNameCourt( fileItem ) );

            String strExtension = fileItem.getName(  ).substring( fileItem.getName(  ).length(  ) - 3 ).toUpperCase(  );
            fichier.setExtension( strExtension );
            fichier.setTaille( (int) fileItem.getSize(  ) );

            fichier.setSeance( deliberation.getSeance(  ) );

            fichier.setIntitule( deliberation.getNumero(  ) );

            if ( deliberation.getEnLigne(  ) )
            {
                Timestamp dateForVersion = DateUtils.getCurrentDate(  );
                // on incremente le numero de version
                fichier.setVersion( fichier.getVersion(  ) + 1 );
                fichier.setDatePublication( dateForVersion );

                //Ajout de l'historique
                Historique historique = new Historique(  );
                historique.setVersion( fichier.getVersion(  ) );
                historique.setDatePublication( dateForVersion );
                historique.setIdEntite( fichier.getId(  ) );
                historique.setIdTypeEntite( ITypeEntite.DESIGNATIONS_DE_REPRESENTANTS );
                _historiqueHome.create( historique, plugin );
            }

            if ( fichier.getSeance(  ) != null )
            {
                ISeance seanceByFichier = fichier.getSeance(  );
                ISeance nextSeance = _seanceHome.getProchaineSeance( plugin );

                if ( ( nextSeance != null ) && !seanceByFichier.equals( nextSeance ) &&
                        ( ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.DOCUMENT_LIBRE ) ||
                        ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.COMPTE_RENDU ) ||
                        ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.LISTE_DE_DESIGNATION ) ) )
                {
                    _searchService.updateObjectInIndex( fichier, true, plugin );
                }
            }

            _fichierHome.update( fichier, plugin );

            if ( fichier.getEnLigne(  ) )
            {
                _horodatageService.tracePublication( fichier, HorodatageActionEnum.MODIFICATION );
            }

            if ( !isOriginale )
            {
                deliberation.setFichierSc( fichier );
            }
        }
    }

    public String getJspUrlGetModificationForActe( HttpServletRequest request, int nIdEntite, int nIdActe )
    {
        return AppPathService.getBaseUrl( request ) + JSP_URL_GET_MODIFICATION_DELIBERATION_DESIGNATION_FOR_ACTE +
        OdsConstants.CONSTANTE_POINT_INTERROGATION + getParameterPluginName(  ) + OdsConstants.CONSTANTE_ET +
        getIdParameter(  ) + OdsConstants.CONSTANTE_EGAL + nIdEntite;
    }

    public String getIdParameter(  )
    {
        return OdsParameters.ID_DELIBERATION;
    }

    public void doPublicationActe( Acte<GSeance, GFichier> acte, boolean bIsPublie, Timestamp dateForVersion )
    {
        Plugin plugin = getPlugin(  );

        GDeliberationDesignation deliberation = _deliberationDesignationHome.findByPrimaryKey( acte.getIdEntite(  ),
                plugin );

        if ( !( ( deliberation.getEnLigne(  ) && bIsPublie ) || ( !deliberation.getEnLigne(  ) && !bIsPublie ) ) )
        {
            if ( bIsPublie && ( deliberation.getVersion(  ) < 1 ) )
            {
                deliberation.setVersion( 0 );
            }

            //On publie la dlibration
            _deliberationDesignationHome.publication( deliberation.getId(  ), dateForVersion,
                deliberation.getVersion(  ), bIsPublie, plugin );

            //On publie les fichiers associs
            _fichierHome.publication( deliberation.getFichierSc(  ).getId(  ), dateForVersion,
                deliberation.getVersion(  ), bIsPublie, plugin );

            if ( bIsPublie )
            {
                Historique historique = new Historique(  );
                historique.setVersion( deliberation.getVersion(  ) + 1 );
                historique.setDatePublication( dateForVersion );
                historique.setIdEntite( deliberation.getId(  ) );
                historique.setIdTypeEntite( ITypeEntite.DESIGNATIONS_DE_REPRESENTANTS );
                _historiqueHome.create( historique, plugin );
            }

            // on horodate la (de)publication
            HorodatageActionEnum action = ( bIsPublie ) ? HorodatageActionEnum.PUBLICATION
                                                        : HorodatageActionEnum.DE_PUBLICATION;
            _horodatageService.tracePublication( deliberation, action );
        }
    }

    @SuppressWarnings( "unchecked" )
    public GSeance getSeance( IBusinessItem entite )
    {
        Plugin plugin = getPlugin(  );

        GDeliberationDesignation deliberation = (GDeliberationDesignation) entite;

        GSeance seance = _seanceHome.findByPrimaryKey( deliberation.getSeance(  ).getIdSeance(  ), plugin );

        return seance;
    }

    /**
     * Retourne la signature numriques SHA224 de la dlibration de dsignation
     *
     * @param deliberation
     *            Deliberation
     * @param plugin
     *            le plugin actif
     * @return la signature
     */
    @SuppressWarnings( "unchecked" )
    public String getSignature( IEntiteHorodatage entite )
    {
        Plugin plugin = getPlugin(  );
        String signature = OdsConstants.CONSTANTE_CHAINE_VIDE;

        GDeliberationDesignation deliberation = (GDeliberationDesignation) entite;
        GFichier fichierSc = deliberation.getFichierSc(  );
        fichierSc.setFichier( _fichierPhysiqueHome.findByPrimaryKey( 
                deliberation.getFichierSc(  ).getFichier(  ).getIdFichier(  ), plugin ) );
        deliberation.setFichierSc( fichierSc );

        if ( ( deliberation.getFichierSc(  ) != null ) && ( deliberation.getFichierSc(  ).getFichier(  ) != null ) )
        {
            byte[] donnees = deliberation.getFichierSc(  ).getFichier(  ).getDonnees(  );
            signature = ShaUtils.toSha224String( donnees );

            if ( signature == null )
            {
                signature = "error_" + deliberation.getFichierSc(  ).getNom(  );
                AppLogService.error( "Erreur lors de la recuperation de la signature SHA 224 du fichier : " +
                    deliberation.getFichierSc(  ).getNom(  ) );
            }
        }

        return signature;
    }

    public IEntiteCertificatAffichage getEntite( Acte<GSeance, GFichier> acte )
    {
        Plugin plugin = getPlugin(  );

        GDeliberationDesignation deliberation = _deliberationDesignationHome.findByPrimaryKey( acte.getIdEntite(  ),
                plugin );

        FormationConseil formationConseil = _formationConseilHome.findByPrimaryKey( deliberation.getFormationConseil(  )
                                                                                                .getIdFormationConseil(  ),
                plugin );
        deliberation.setFormationConseil( formationConseil );

        return deliberation;
    }

    public Comparator<IBusinessItem> getComparatorClass(  )
    {
        return new DesignationComparator(  );
    }

    public String getMarkForCertificat( int nIdTypeEntite )
    {
        return OdsMarks.MARK_LISTE_DELIBERATIONS_DESIGNATION;
    }

    public int getIndexerAction( IBusinessItem item )
    {
        return item.getTypeEntite(  );
    }

    public IBusinessItem createItemToDeleteInIndexer( int nIdItem )
    {
        GDeliberationDesignation deliberation = newDeliberationInstance(  );
        deliberation.setId( nIdItem );

        return deliberation;
    }

    /**
     * Renvoie l'objet de l'acte utilis lors de l'envoi vers la plate-forme
     *                         Numro Intitul
     * @param acte l'acte que l'on va transmettre
     * @return strObjet l'objet de l'acte
     */
    public String getObjetActe( Acte<GSeance, GFichier> acte )
    {
        Plugin plugin = getPlugin(  );
        String strObjet = OdsConstants.CONSTANTE_CHAINE_VIDE;

        GDeliberationDesignation deliberationDesignation = _deliberationDesignationHome.findByPrimaryKey( acte.getIdEntite(  ),
                plugin );

        if ( deliberationDesignation != null )
        {
            strObjet = deliberationDesignation.getNumero(  ) + OdsConstants.CONSTANTE_ESPACE +
                deliberationDesignation.getIntitule(  );
        }

        return strObjet;
    }
}
