/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.direction;

import fr.paris.lutece.plugins.ods.business.direction.IDirectionHome;
import fr.paris.lutece.plugins.ods.dto.direction.Direction;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.util.ReferenceList;
import fr.paris.lutece.util.url.UrlItem;

import org.springframework.beans.factory.annotation.Autowired;

import java.sql.SQLException;

import java.util.HashMap;
import java.util.List;

import javax.servlet.http.HttpServletRequest;


public abstract class AbstractDirectionService implements IDirectionService
{
    private static final String MARK_LISTE_DIRECTIONS = "liste_directions";
    private static final String MARK_DIRECTION = "direction";
    private static final String MESSAGE_CONFIRM_DELETE_DIRECTION = "ods.message.confirmDeleteDirection";
    private static final String MESSAGE_CANNOT_CREATE_DIRECTION = "ods.message.cannotCreateDirection";
    private static final String MESSAGE_NFE_ON_MODIFY = "ods.modificationdirection.label.nfeOnModify";
    private static final String MESSAGE_CANNOT_MODIFY_DIRECTION = "ods.message.cannotModifyDirection";
    private static final String MESSAGE_CANNOT_DELETE_DIRECTION = "ods.message.cannotDeleteDirection";
    private static final String JSP_URL_DO_SUPPRESSION_DIRECTION_JSP = "jsp/admin/plugins/ods/direction/DoSuppressionDirection.jsp";
    private static final String FIELD_CODE = "ods.creationdirection.label.code";
    private static final String FIELD_LIBELLE_COURT = "ods.creationdirection.label.libelle_court";
    private static final String FIELD_LIBELLE_LONG = "ods.creationdirection.label.libelle_long";
    @Autowired
    private IDirectionHome _directionHome;

    protected abstract Plugin getPlugin(  );

    public static ReferenceList initRefListArrondissement(  )
    {
        ReferenceList refListArrondissement = new ReferenceList(  );
        //refListArrondissement.addItem( -1, OdsConstants.CONSTANTE_CHAINE_VIDE );
        refListArrondissement.addItem( 1, "1er" );

        for ( int i = 2; i <= 20; i++ )
        {
            refListArrondissement.addItem( i, OdsConstants.CONSTANTE_CHAINE_VIDE + i + "eme" );
        }

        return refListArrondissement;
    }

    /**
     * Retourne la map contenant les lments utiles au template HTML de la liste
     * @param request le requte HTTP
     * @return model la map contenant les lments utiles au template HTML de la liste
     */
    public HashMap<String, Object> getDirectionList( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        List<Direction> listDirection = _directionHome.findDirectionList( plugin );
        model.put( MARK_LISTE_DIRECTIONS, listDirection );

        return model;
    }

    /**
     * Retourne la map contenant les lments utiles au template HTML de modification
     * @param request le requte HTTP
     * @return model la map contenant les lments utiles au template HTML de modification
     */
    public HashMap<String, Object> getModificationDirection( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        if ( request.getParameter( OdsParameters.ID_DIRECTION ) == null )
        {
            return null;
        }

        int nIdDirection = OdsUtils.getIntegerParameter( OdsParameters.ID_DIRECTION, request );
        Direction direction = _directionHome.findByPrimaryKey( nIdDirection, plugin );
        OdsUtils.testNullObject( direction, request );

        model.put( MARK_DIRECTION, direction );
        model.put( OdsMarks.MARK_ACTIF, direction.isActif(  ) );

        return model;
    }

    /**
     * Retourne l'interface de suppression d'une direction.
     * @param request la requte HTTP
     * @return l'interface de cration d'une direction
     */
    public String getSuppressionDirection( HttpServletRequest request )
    {
        if ( request.getParameter( OdsParameters.ID_DIRECTION ) == null )
        {
            return OdsConstants.CONSTANTE_CHAINE_VIDE;
        }

        String strIdDirection = request.getParameter( OdsParameters.ID_DIRECTION );
        UrlItem url = new UrlItem( JSP_URL_DO_SUPPRESSION_DIRECTION_JSP );
        url.addParameter( OdsParameters.ID_DIRECTION, strIdDirection );

        return AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRM_DELETE_DIRECTION, url.getUrl(  ),
            AdminMessage.TYPE_CONFIRMATION );
    }

    /**
     * Cre une direction.
     * Si tous les champs obligatoires sont renseigns,
     * et si la valeur du code n'est pas dj utilise par une autre direction,
     * la mthode retourne l'url de gestion des directions.
     * Sinon la mthode retourne l'url affichant les messages d'erreur
     * @param request la requte HTTP
     * @return une url
     */
    public String doCreationDirection( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        if ( !requiredField( request ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            Object[] tabRequiredFields = 
                {
                    I18nService.getLocalizedString( requiredField( request ), request.getLocale(  ) ),
                };

            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED,
                tabRequiredFields, AdminMessage.TYPE_STOP );
        }

        String strCode = request.getParameter( OdsParameters.CODE_DIRECTION ).trim(  );
        String strLibelleCourt = request.getParameter( OdsParameters.INTITULE_COURT ).trim(  );
        String strLibelleLong = request.getParameter( OdsParameters.INTITULE_LONG ).trim(  );
        String strActif = request.getParameter( OdsParameters.ACTIF );
        boolean bActif;

        if ( null == strActif )
        {
            bActif = false;
        }
        else
        {
            bActif = true;
        }

        Direction direction = new Direction(  );

        if ( _directionHome.containsCode( null, strCode, plugin ) )
        {
            return AdminMessageService.getMessageUrl( request, MESSAGE_CANNOT_CREATE_DIRECTION, AdminMessage.TYPE_STOP );
        }

        direction.setCode( strCode );
        direction.setLibelleCourt( strLibelleCourt );
        direction.setLibelleLong( strLibelleLong );
        direction.setActif( bActif );

        _directionHome.create( direction, plugin );

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * Modifie une direction.
     * Si tous les champs obligatoires sont renseigns,
     * et si la valeur du code n'est pas dj utilise par une autre direction,
     * la mthode retourne l'url de gestion des directions.
     * Sinon la mthode retourne l'url affichant les messages d'erreur
     * @param request la requte HTTP
     * @return une url
     */
    public String doModificationDirection( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        if ( !requiredField( request ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            Object[] tabRequiredFields = 
                {
                    I18nService.getLocalizedString( requiredField( request ), request.getLocale(  ) ),
                };

            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED,
                tabRequiredFields, AdminMessage.TYPE_STOP );
        }

        int nIdDirection = OdsUtils.getIntegerParameter( OdsParameters.ID_DIRECTION, request );

        Direction direction = _directionHome.findByPrimaryKey( nIdDirection, plugin );

        if ( direction == null )
        {
            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                AdminMessage.TYPE_ERROR );
        }

        String strCode = request.getParameter( OdsParameters.CODE_DIRECTION ).trim(  );
        String strLibelleCourt = request.getParameter( OdsParameters.INTITULE_COURT ).trim(  );
        String strLibelleLong = request.getParameter( OdsParameters.INTITULE_LONG ).trim(  );
        String strActif = request.getParameter( OdsParameters.ACTIF );
        boolean bActif;

        if ( null == strActif )
        {
            bActif = false;
        }
        else
        {
            bActif = true;
        }

        if ( _directionHome.containsCode( direction, strCode, plugin ) )
        {
            return AdminMessageService.getMessageUrl( request, MESSAGE_CANNOT_MODIFY_DIRECTION, AdminMessage.TYPE_STOP );
        }

        direction.setCode( strCode );
        direction.setLibelleCourt( strLibelleCourt );
        direction.setLibelleLong( strLibelleLong );
        direction.setActif( bActif );

        _directionHome.update( direction, plugin );

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * Supprime une direction.
     * @param request la requte HTTP
     * @return url de gestion des directions
     */
    public String doSuppressionDirection( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        if ( null == request.getParameter( OdsParameters.ID_DIRECTION ) )
        {
            return OdsConstants.CONSTANTE_CHAINE_VIDE;
        }

        int nIdDirection;

        try
        {
            nIdDirection = Integer.parseInt( request.getParameter( OdsParameters.ID_DIRECTION ) );
        }
        catch ( NumberFormatException ne )
        {
            return AdminMessageService.getMessageUrl( request, MESSAGE_NFE_ON_MODIFY, AdminMessage.TYPE_STOP );
        }

        Direction direction = new Direction(  );
        direction.setIdDirection( nIdDirection );

        try
        {
            _directionHome.remove( direction, plugin );
        }
        catch ( AppException ae )
        {
            if ( ae.getInitialException(  ) instanceof SQLException )
            {
                return AdminMessageService.getMessageUrl( request, MESSAGE_CANNOT_DELETE_DIRECTION,
                    AdminMessage.TYPE_STOP );
            }
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * Indique si la requte est valide, c'est  dire que tous les champs obligatoires ont t remplis.
     * @param request la requte HTTP
     * @return true si la requte est valide, false sinon
     */
    private String requiredField( HttpServletRequest request )
    {
        String strRequiredField = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( ( request.getParameter( OdsParameters.CODE_DIRECTION ) == null ) ||
                ( request.getParameter( OdsParameters.CODE_DIRECTION ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) ) )
        {
            strRequiredField = FIELD_CODE;
        }
        else if ( ( request.getParameter( OdsParameters.INTITULE_COURT ) == null ) ||
                ( request.getParameter( OdsParameters.INTITULE_COURT ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) ) )
        {
            strRequiredField = FIELD_LIBELLE_COURT;
        }
        else if ( ( request.getParameter( OdsParameters.INTITULE_LONG ) == null ) ||
                ( request.getParameter( OdsParameters.INTITULE_LONG ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) ) )
        {
            strRequiredField = FIELD_LIBELLE_LONG;
        }

        return strRequiredField;
    }

    /**
     * Permet de rcuprer la liste des directions
     * @return ReferenceList La liste des directions disponibles
     */
    public ReferenceList initRefListDirection( boolean bIsActive, boolean bIsItemVideEnbaled )
    {
        Plugin plugin = getPlugin(  );

        ReferenceList refListDirection = new ReferenceList(  );
        List<Direction> listDirection;

        if ( bIsActive )
        {
            listDirection = _directionHome.findAllDirectionsActives( plugin );
        }
        else
        {
            listDirection = _directionHome.findDirectionList( plugin );
        }

        if ( bIsItemVideEnbaled )
        {
            // ajout dans la referenceList de l'item chaine vide
            refListDirection.addItem( OdsConstants.FILTER_ALL, OdsConstants.CONSTANTE_CHAINE_VIDE );
        }

        if ( ( listDirection != null ) && !listDirection.isEmpty(  ) )
        {
            for ( Direction direction : listDirection )
            {
                refListDirection.addItem( direction.getIdDirection(  ), direction.getLibelleCourt(  ) );
            }
        }

        return refListDirection;
    }
}
