/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.direction;

import fr.paris.lutece.plugins.ods.service.IODSService;
import fr.paris.lutece.util.ReferenceList;

import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;


public interface IDirectionService extends IODSService
{
    /**
     * Retourne la map contenant les lments utiles au template HTML de la liste
     * @param request le requte HTTP
     * @return model la map contenant les lments utiles au template HTML de la liste
     */
    HashMap<String, Object> getDirectionList( HttpServletRequest request );

    /**
     * Retourne la map contenant les lments utiles au template HTML de modification
     * @param request le requte HTTP
     * @return model la map contenant les lments utiles au template HTML de modification
     */
    HashMap<String, Object> getModificationDirection( HttpServletRequest request );

    /**
     * Retourne l'interface de suppression d'une direction.
     * @param request la requte HTTP
     * @return l'interface de cration d'une direction
     */
    String getSuppressionDirection( HttpServletRequest request );

    /**
     * Cre une direction.
     * Si tous les champs obligatoires sont renseigns,
     * et si la valeur du code n'est pas dj utilise par une autre direction,
     * la mthode retourne l'url de gestion des directions.
     * Sinon la mthode retourne l'url affichant les messages d'erreur
     * @param request la requte HTTP
     * @return une url
     */
    String doCreationDirection( HttpServletRequest request );

    /**
     * Modifie une direction.
     * Si tous les champs obligatoires sont renseigns,
     * et si la valeur du code n'est pas dj utilise par une autre direction,
     * la mthode retourne l'url de gestion des directions.
     * Sinon la mthode retourne l'url affichant les messages d'erreur
     * @param request la requte HTTP
     * @return une url
     */
    String doModificationDirection( HttpServletRequest request );

    /**
     * Supprime une direction.
     * @param request la requte HTTP
     * @return url de gestion des directions
     */
    String doSuppressionDirection( HttpServletRequest request );

    /**
     * Permet de rcuprer la liste des directions
     * @return ReferenceList La liste des directions disponibles
     */
    ReferenceList initRefListDirection( boolean bIsActive, boolean bIsItemVideEnbaled );
}
