/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.elu;

import fr.paris.lutece.plugins.ods.business.elu.IEluHome;
import fr.paris.lutece.plugins.ods.business.groupepolitique.GroupePolitiqueHome;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.groupepolitique.IGroupePolitique;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.util.ReferenceList;
import fr.paris.lutece.util.url.UrlItem;

import org.springframework.beans.factory.annotation.Autowired;

import java.sql.SQLException;

import java.util.HashMap;
import java.util.List;

import javax.servlet.http.HttpServletRequest;


public abstract class AbstractEluService<GElu extends IElu, GEluHome extends IEluHome<GElu>> implements IEluService
{
    protected static final String JSP_CREATION_ELU = "jsp/admin/plugins/ods/elu/CreationElu.jsp";
    protected static final String JSP_MODIFICATION_ELU = "jsp/admin/plugins/ods/elu/ModificationElu.jsp";
    protected static final String JSP_URL_DO_SUPPRESSION_ELU_JSP = "jsp/admin/plugins/ods/elu/DoSuppressionElu.jsp";
    protected static final String CONSTANTE_SESSION = "session";
    protected static final String CONSTANTE_ANNULER = "annuler";
    private static final String MARK_LISTE_ELUS = "liste_elus";
    private static final String MARK_LISTE_GROUPES = "liste_groupes";
    private static final String MARK_LISTE_REMP = "liste_remplacants";
    private static final String MARK_ELU = "elu";
    private static final String MARK_ID_GROUPE_SELECTED = "id_groupe_selected";
    private static final String MESSAGE_CONFIRMDELETEELU = "ods.message.confirmDeleteElu";
    private static final String MESSAGE_CANNOT_DELETE_ELU = "ods.message.cannotDeleteElu";
    private static final String MESSAGE_SESSION_EXPIREE = "ods.voeuamendement.message.sessionExpiree";
    private static final String FIELD_CIVILITE = "ods.creationelu.label.civilite";
    private static final String FIELD_PRENOM_ELU = "ods.creationelu.label.prenom_elu";
    private static final String FIELD_NOM_ELU = "ods.creationelu.label.nom_elu";
    private static final String FIELD_ID_GROUPE = "ods.creationelu.label.groupe";
    @Autowired
    protected GEluHome _eluHome;
    @Autowired
    protected GroupePolitiqueHome _groupePolitiqueHome;
    private GElu _elu;

    /**
     * Retourne la map contenant les lments utiles au template HTML de la liste
     * @param request la requete HTTP
     * @return model la map contenant les lments utiles au template HTML de la liste
     */
    public HashMap<String, Object> getEluList( HttpServletRequest request )
    {
        Plugin plugin = getSpecPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        // Cration d'un objet Referencelist en vue de stocker l'ensemble des items
        // reprsentant les groupes politiques.
        ReferenceList referenceList = new ReferenceList(  );

        List<GElu> elus = null;
        int nIdGroupe = -1;

        // Si un choix de groupe politique a t fait,on retourne la liste des lus
        // appartenant au groupe politique choisi.
        // Dans le cas contraire on retourne la liste complete des lus        
        if ( null != request.getParameter( OdsParameters.ID_GROUPE ) )
        {
            try
            {
                nIdGroupe = Integer.parseInt( request.getParameter( OdsParameters.ID_GROUPE ) );
                elus = _eluHome.findEluListbyIdGroupe( nIdGroupe, plugin );
            }
            catch ( NumberFormatException nfe )
            {
                AppLogService.error( nfe );
            }
        }

        if ( nIdGroupe == -1 )
        {
            elus = _eluHome.findEluList( plugin );
        }

        List<IGroupePolitique> listGroupePolitique = _groupePolitiqueHome.findGroupePolitiqueList( plugin );

        // Ajout dans la referenceList de l'item chaine vide
        referenceList.addItem( -1, OdsConstants.CONSTANTE_CHAINE_VIDE );

        // Ajout dans la referenceList de l'ensemble des items reprsentant
        // les groupes politiques.
        for ( IGroupePolitique groupe : listGroupePolitique )
        {
            referenceList.addItem( groupe.getIdGroupe(  ), groupe.getNomGroupe(  ) );
        }

        // On ajoute l'id du groupe politique slectionn.
        // Si aucun groupe n'a t renseign  c'est la valeur -1 qui est transmise.
        model.put( MARK_ID_GROUPE_SELECTED, nIdGroupe );
        model.put( MARK_LISTE_GROUPES, referenceList );
        model.put( MARK_LISTE_ELUS, elus );

        return model;
    }

    /**
     * Retourne la map contenant les lments utiles au template HTML de cration
     * @param request la requete HTTP
     * @return model la map contenant les lments utiles au template HTML de cration
     */
    public HashMap<String, Object> getCreationElu( HttpServletRequest request )
    {
        Plugin plugin = getSpecPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        // Cration d'un objet Referencelist en vue de stocker l'ensemble 
        // des items reprsentant les groupes politiques, les commissions 
        // et les lus (remplaants possibles).
        ReferenceList referenceListGroupes = new ReferenceList(  );
        ReferenceList referenceListElus = new ReferenceList(  );

        List<IGroupePolitique> listGroupePolitique = _groupePolitiqueHome.findGroupesActifs( plugin );

        // Ajout dans la referenceListGroupes de l'item chaine vide
        referenceListGroupes.addItem( -1, OdsConstants.CONSTANTE_CHAINE_VIDE );

        // Ajout dans la referenceListGroupes de l'ensemble des items reprsentant 
        // les groupes politiques
        for ( IGroupePolitique groupe : listGroupePolitique )
        {
            referenceListGroupes.addItem( groupe.getIdGroupe(  ), groupe.getNomGroupe(  ) );
        }

        model.put( MARK_LISTE_GROUPES, referenceListGroupes );

        if ( ( null == request.getParameter( CONSTANTE_SESSION ) ) )
        {
            _elu = newEluInstance(  );
        }

        getCreationEluSpec( model, _elu, plugin );

        List<GElu> listElusRemplacants = _eluHome.findEluList( plugin );
        referenceListElus.addItem( -1, OdsConstants.CONSTANTE_CHAINE_VIDE );

        for ( GElu remplacant : listElusRemplacants )
        {
            String nom = remplacant.getCivilite(  ) + " " + remplacant.getPrenomElu(  ) + " " +
                remplacant.getNomElu(  ) + " (" + remplacant.getGroupe(  ).getNomGroupe(  ) + ")";
            referenceListElus.addItem( remplacant.getIdElu(  ), nom );
        }

        model.put( MARK_LISTE_REMP, referenceListElus );
        model.put( MARK_ELU, _elu );

        return model;
    }

    protected abstract void getCreationEluSpec( HashMap<String, Object> model, GElu elu, Plugin plugin );

    protected abstract void getModificationEluSpec( HashMap<String, Object> model, GElu elu, Plugin plugin );

    /**
     * Retourne la map contenant les lments utiles au template HTML de modification
     * @param request la requete HTTP
      * @return model la map contenant les lments utiles au template HTML de modification
     */
    public HashMap<String, Object> getModificationElu( HttpServletRequest request )
    {
        Plugin plugin = getSpecPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        if ( ( null == request.getParameter( CONSTANTE_SESSION ) ) )
        {
            int nIdElu = OdsUtils.getIntegerParameter( OdsParameters.ID_ELU, request );

            if ( nIdElu == -1 )
            {
                return null;
            }

            _elu = _eluHome.findByPrimaryKey( nIdElu, plugin );
        }

        OdsUtils.testNullObject( _elu, request );

        // Cration d'un objet Referencelist en vue de stocker
        // l'ensemble des items reprsentant les groupes politiques,
        // les lus et les commissions.
        ReferenceList referenceListGroupes = new ReferenceList(  );
        ReferenceList referenceListElus = new ReferenceList(  );

        List<IGroupePolitique> listGroupePolitique = _groupePolitiqueHome.findGroupesActifs( plugin );

        // Ajout dans la referenceListGroupes de l'item chaine vide
        referenceListGroupes.addItem( -1, OdsConstants.CONSTANTE_CHAINE_VIDE );

        // Ajout dans la referenceListGroupes de l'ensemble des items reprsentant 
        // les groupes politiques
        for ( IGroupePolitique groupe : listGroupePolitique )
        {
            referenceListGroupes.addItem( groupe.getIdGroupe(  ), groupe.getNomGroupe(  ) );
        }

        model.put( MARK_LISTE_GROUPES, referenceListGroupes );

        getModificationEluSpec( model, _elu, plugin );

        List<GElu> listElusRemplacants = _eluHome.findEluList( plugin );
        referenceListElus.addItem( -1, OdsConstants.CONSTANTE_CHAINE_VIDE );

        for ( GElu eluRemplacant : listElusRemplacants )
        {
            if ( eluRemplacant.getIdElu(  ) != _elu.getIdElu(  ) )
            {
                String strNom = eluRemplacant.getCivilite(  ) + " " + eluRemplacant.getPrenomElu(  ) + " " +
                    eluRemplacant.getNomElu(  ) + " (" + eluRemplacant.getGroupe(  ).getNomGroupe(  ) + ")";
                referenceListElus.addItem( eluRemplacant.getIdElu(  ), strNom );
            }
        }

        model.put( MARK_LISTE_REMP, referenceListElus );
        model.put( MARK_ELU, _elu );

        return model;
    }

    /**
     * Retourne une demande pour la suppression de l'lu choisi
     * @param request la requete HTTP
     * @param strReturnDefault l'URL de retour par dfaut
     * @return interface de confirmation de suppression
     */
    public String getSuppressionElu( HttpServletRequest request, String strReturnDefault )
    {
        if ( request.getParameter( OdsParameters.ID_ELU ) == null )
        {
            return strReturnDefault;
        }

        String strEluID = request.getParameter( OdsParameters.ID_ELU );
        UrlItem url = new UrlItem( JSP_URL_DO_SUPPRESSION_ELU_JSP );
        url.addParameter( OdsParameters.ID_ELU, strEluID );

        return AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRMDELETEELU, url.getUrl(  ),
            AdminMessage.TYPE_CONFIRMATION );
    }

    /**
     * cre lu.
     * si tous les champs obligatoires du formulaire de cration d'un lu ont t renseign,
     * la mthode retourne l'url de gestion des lus,
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request request+
     * @param strReturnDefault l'URL de retour par dfaut
     * @return url url
     */
    public String doCreationElu( HttpServletRequest request, String strReturnDefault )
    {
        Plugin plugin = getSpecPlugin(  );

        if ( _elu == null )
        {
            UrlItem url = new UrlItem( strReturnDefault );

            return AdminMessageService.getMessageUrl( request, MESSAGE_SESSION_EXPIREE, url.getUrl(  ),
                AdminMessage.TYPE_ERROR );
        }

        if ( null != request.getParameter( CONSTANTE_ANNULER ) )
        {
            return strReturnDefault;
        }

        // TODO CP
        String strCreationActionSpec = doCreationSpecAction( request, _elu, plugin );

        if ( ( strCreationActionSpec != null ) && !strCreationActionSpec.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            return strCreationActionSpec;
        }

        if ( !requiredField( request ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            Object[] tabRequiredFields = 
                {
                    I18nService.getLocalizedString( requiredField( request ), request.getLocale(  ) ),
                };

            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED,
                tabRequiredFields, AdminMessage.TYPE_STOP );
        }

        String strError = getEluData( _elu, request );

        if ( null == strError )
        {
            _elu.setIdElu( _eluHome.create( _elu, plugin ) );
        }
        else
        {
            return strError;
        }

        // TODO CP
        String strReturnCreationSpec = doCreationEluSpec( request, _elu, plugin );

        if ( ( strReturnCreationSpec != null ) && !strReturnCreationSpec.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            return strReturnCreationSpec;
        }

        // TODO FIN CP
        return strReturnDefault;
    }

    /**
     * Modifie un lu.
     * si tous les champs obligatoires du formulaire de modification d'un lu ont t renseign,
     * la mthode retourne l'url de gestion des lus,
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request request
     * @param strURLDefault l'URL de retour par dfaut
     * @return url url
     */
    public String doModificationElu( HttpServletRequest request, String strURLDefault )
    {
        Plugin plugin = getSpecPlugin(  );

        if ( _elu == null )
        {
            UrlItem url = new UrlItem( strURLDefault );

            return AdminMessageService.getMessageUrl( request, MESSAGE_SESSION_EXPIREE, url.getUrl(  ),
                AdminMessage.TYPE_ERROR );
        }

        if ( null != request.getParameter( CONSTANTE_ANNULER ) )
        {
            return strURLDefault;
        }

        // TODO CP
        String strModifActionSpec = doModificationSpecAction( request, _elu, plugin );

        if ( ( strModifActionSpec != null ) && !strModifActionSpec.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            return strModifActionSpec;
        }

        if ( !requiredField( request ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            Object[] tabRequiredFields = 
                {
                    I18nService.getLocalizedString( requiredField( request ), request.getLocale(  ) ),
                };

            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED,
                tabRequiredFields, AdminMessage.TYPE_STOP );
        }

        String strError = getEluData( _elu, request );

        if ( null == strError )
        {
            _eluHome.update( _elu, plugin );
        }
        else
        {
            return strError;
        }

        // TODO CP
        String strReturnModificationSpec = doModificationEluSpec( request, _elu, plugin );

        if ( ( strReturnModificationSpec != null ) &&
                !strReturnModificationSpec.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            return strReturnModificationSpec;
        }

        // TODO FIN CP
        return strURLDefault;
    }

    protected abstract String doCreationSpecAction( HttpServletRequest request, GElu elu, Plugin plugin );

    protected abstract String doModificationSpecAction( HttpServletRequest request, GElu elu, Plugin plugin );

    protected abstract String doCreationEluSpec( HttpServletRequest request, GElu elu, Plugin plugin );

    protected abstract String doModificationEluSpec( HttpServletRequest request, GElu elu, Plugin plugin );

    /**
     * supprime l'lu choisi
     * @param request la requete HTTP
     * @return url de gestion des groupes poliques
     */
    public String doSuppressionElu( HttpServletRequest request )
    {
        Plugin plugin = getSpecPlugin(  );
        int nIdElu = OdsUtils.getIntegerParameter( OdsParameters.ID_ELU, request );

        if ( nIdElu == -1 )
        {
            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                AdminMessage.TYPE_ERROR );
        }

        GElu elu = newEluInstance(  );
        elu.setIdElu( nIdElu );

        try
        {
            _eluHome.remove( elu, plugin );
        }
        catch ( AppException ae )
        {
            AppLogService.error( ae );

            if ( ae.getInitialException(  ) instanceof SQLException )
            {
                return AdminMessageService.getMessageUrl( request, MESSAGE_CANNOT_DELETE_ELU, AdminMessage.TYPE_STOP );
            }
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * Retourne le premier champ requis manquant s'il y'en a un.
     * @param request  la requete HTTP
     * @return boolean retourne true si l'ensemble des critres obligatoires ont t renseign
     */
    private String requiredField( HttpServletRequest request )
    {
        String strRequiredField = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( ( request.getParameter( OdsParameters.CIVILITE ) == null ) ||
                request.getParameter( OdsParameters.CIVILITE ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strRequiredField = FIELD_CIVILITE;
        }
        else if ( request.getParameter( OdsParameters.PRENOM_ELU ).trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strRequiredField = FIELD_PRENOM_ELU;
        }
        else if ( request.getParameter( OdsParameters.NOM_ELU ).trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strRequiredField = FIELD_NOM_ELU;
        }
        else if ( ( request.getParameter( OdsParameters.ID_GROUPE ) == null ) ||
                ( Integer.parseInt( request.getParameter( OdsParameters.ID_GROUPE ) ) == -1 ) )
        {
            strRequiredField = FIELD_ID_GROUPE;
        }

        return strRequiredField;
    }

    /**
     * initialise l'objet lu avec les paramtres de la requete
     * @param elu l'lu  traiter
     * @param request request
     * @return null si aucune erreurn ne c'est produite sinon retourne l'url du message d'erreur  afficher
     */
    protected String getEluData( GElu elu, HttpServletRequest request )
    {
        Plugin plugin = getSpecPlugin(  );

        String strNom = request.getParameter( OdsParameters.NOM_ELU ).trim(  );
        String strPrenom = request.getParameter( OdsParameters.PRENOM_ELU ).trim(  );
        String strCivilite = request.getParameter( OdsParameters.CIVILITE ).trim(  );
        int nIdGroupe = OdsUtils.getIntegerParameter( OdsParameters.ID_GROUPE, request );

        if ( nIdGroupe != -1 )
        {
            IGroupePolitique groupe = _groupePolitiqueHome.findByPrimaryKey( nIdGroupe, plugin );

            if ( groupe == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            elu.setGroupe( groupe );
        }

        int nIdRemplacant = OdsUtils.getIntegerParameter( OdsParameters.ID_REMPLACANT, request );

        if ( nIdRemplacant != -1 )
        {
            GElu remplacant = null;
            remplacant = _eluHome.findByPrimaryKey( nIdRemplacant, plugin );

            if ( remplacant == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            elu.setEluRemplace( remplacant );
        }
        else
        {
            elu.setEluRemplace( null );
        }

        String strActif = request.getParameter( OdsParameters.ACTIF );
        boolean bActif;
        elu.setNomElu( strNom );
        elu.setPrenomElu( strPrenom );
        elu.setCivilite( strCivilite );

        if ( null == strActif )
        {
            bActif = false;
        }
        else
        {
            bActif = true;
        }

        elu.setActif( bActif );

        return null;
    }

    protected abstract GElu newEluInstance(  );

    protected abstract Plugin getSpecPlugin(  );
}
