/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.elu;

import fr.paris.lutece.plugins.ods.service.IODSService;

import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;


public interface IEluService extends IODSService
{
    /**
    * Retourne la map contenant les lments utiles au template HTML de la liste
    * @param request la requete HTTP
    * @return model la map contenant les lments utiles au template HTML de la liste
    */
    HashMap<String, Object> getEluList( HttpServletRequest request );

    /**
     * Retourne la map contenant les lments utiles au template HTML de cration
     * @param request la requete HTTP
     * @return model la map contenant les lments utiles au template HTML de cration
     */
    HashMap<String, Object> getCreationElu( HttpServletRequest request );

    /**
     * Retourne la map contenant les lments utiles au template HTML de modification
     * @param request la requete HTTP
      * @return model la map contenant les lments utiles au template HTML de modification
     */
    HashMap<String, Object> getModificationElu( HttpServletRequest request );

    /**
     * Retourne une demande pour la suppression de l'lu choisi
     * @param request la requete HTTP
     * @param strReturnDefault l'URL de retour par dfaut
     * @return interface de confirmation de suppression
     */
    String getSuppressionElu( HttpServletRequest request, String strReturnDefault );

    /**
     * cre lu.
     * si tous les champs obligatoires du formulaire de cration d'un lu ont t renseign,
     * la mthode retourne l'url de gestion des lus,
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request request+
     * @param strReturnDefault l'URL de retour par dfaut
     * @return url url
     */
    String doCreationElu( HttpServletRequest request, String strReturnDefault );

    /**
     * Modifie un lu.
     * si tous les champs obligatoires du formulaire de modification d'un lu ont t renseign,
     * la mthode retourne l'url de gestion des lus,
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request request
     * @param strURLDefault l'URL de retour par dfaut
     * @return url url
     */
    String doModificationElu( HttpServletRequest request, String strURLDefault );

    /**
     * supprime l'lu choisi
     * @param request la requete HTTP
     * @return url de gestion des groupes poliques
     */
    String doSuppressionElu( HttpServletRequest request );
}
