/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.expression;

import fr.paris.lutece.plugins.ods.business.expression.IExpressionUsuelleHome;
import fr.paris.lutece.plugins.ods.dto.expression.IExpressionUsuelle;
import fr.paris.lutece.plugins.ods.service.ordredujour.IOrdreDuJourService;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.portal.service.util.AppPropertiesService;
import fr.paris.lutece.portal.web.constants.Messages;
import fr.paris.lutece.util.html.Paginator;
import fr.paris.lutece.util.url.UrlItem;

import org.springframework.beans.factory.annotation.Autowired;

import java.sql.SQLException;

import java.util.HashMap;
import java.util.List;

import javax.servlet.http.HttpServletRequest;


public abstract class AbstractExpressionUsuelleService<GExpressionUsuelle extends IExpressionUsuelle>
    implements IExpressionUsuelleService
{
    private static final String PROPERTY_EXPRESSIONS_PER_PAGE = "ods.listeexpressionusuelle.label.line_per_page";
    private static final String MARK_LISTE_EXPRESSIONS = "liste_expressions";
    private static final String MARK_LISTE_EXPRESSION_COUNT = "liste_expression_count";
    private static final String MARK_EXPRESSION = "expression";
    private static final String MARK_POSITION_CURSEUR = "id_position_curseur";
    private static final String MESSAGE_CONFIRM_DELETE_EXPRESSION = "ods.message.confirmDeleteExpressionUsuelle";
    private static final String MESSAGE_CANNOT_DELETE_EXPRESSION = "ods.message.cannotDeleteExpression";
    private static final String JSP_URL_DO_SUPPRESSION_EXPRESSION = "jsp/admin/plugins/ods/expression/DoSuppressionExpressionUsuelle.jsp";
    private static final String FIELD_EXPRESSION = "ods.creationexpressionusuelle.label.libelle";

    // Attributs du Paginator
    private int _nItemsPerPage;
    private int _nDefaultItemsPerPage = AppPropertiesService.getPropertyInt( PROPERTY_EXPRESSIONS_PER_PAGE, 10 );
    private String _strCurrentPageIndex;
    @Autowired
    private IExpressionUsuelleHome<GExpressionUsuelle> _expressionUsuelleHome;
    @Autowired
    private IOrdreDuJourService<?, ?, ?, ?, ?, ?> _ordreDuJourService;

    protected abstract Plugin getPlugin(  );

    /**
     * Retourn la map contenant les lments utiles au template HTML pour la liste
     * @param request le requte HTTP
     * @param strURLReturn l'URL de retour
     * @return model la map contenant les lments utiles au template HTML pour la liste
     */
    public HashMap<String, Object> getExpressionList( HttpServletRequest request, String strURLReturn )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        _strCurrentPageIndex = Paginator.getPageIndex( request, Paginator.PARAMETER_PAGE_INDEX, _strCurrentPageIndex );
        _nItemsPerPage = Paginator.getItemsPerPage( request, Paginator.PARAMETER_ITEMS_PER_PAGE, _nItemsPerPage,
                _nDefaultItemsPerPage );

        List<GExpressionUsuelle> listExpression = _expressionUsuelleHome.findExpressionList( plugin );

        Paginator paginator = new Paginator( listExpression, _nItemsPerPage, strURLReturn,
                Paginator.PARAMETER_PAGE_INDEX, _strCurrentPageIndex );

        model.put( MARK_LISTE_EXPRESSIONS, paginator.getPageItems(  ) );
        model.put( MARK_LISTE_EXPRESSION_COUNT, listExpression.size(  ) );
        model.put( OdsMarks.MARK_PAGINATOR, paginator );
        model.put( OdsMarks.MARK_NB_ITEMS_PER_PAGE, OdsConstants.CONSTANTE_CHAINE_VIDE + _nItemsPerPage );

        return model;
    }

    /**
     * Retourne la map contenant les lments utiles au template HTML pour la modification
     * @param request le requte HTTP
     * @return model la map contenant les lments utiles au template HTML pour la modification
     */
    public HashMap<String, Object> getModificationExpression( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        if ( request.getParameter( OdsParameters.ID_EXPRESSION ) == null )
        {
            return null;
        }

        int nIdExpression = OdsUtils.getIntegerParameter( OdsParameters.ID_EXPRESSION, request );
        GExpressionUsuelle expression = _expressionUsuelleHome.findByPrimaryKey( nIdExpression, plugin );
        OdsUtils.testNullObject( expression, request );
        model.put( MARK_EXPRESSION, expression );

        return model;
    }

    /**
     * Retourne l'interface de suppression d'une expression usuelle.
     * @param request le requte HTTP
     * @return l'interface de cration d'une expression usuelle
     */
    public String getSuppressionExpression( HttpServletRequest request )
    {
        if ( request.getParameter( OdsParameters.ID_EXPRESSION ) == null )
        {
            return OdsConstants.CONSTANTE_CHAINE_VIDE;
        }

        String strIdExpression = request.getParameter( OdsParameters.ID_EXPRESSION );
        UrlItem url = new UrlItem( JSP_URL_DO_SUPPRESSION_EXPRESSION );
        url.addParameter( OdsParameters.ID_EXPRESSION, strIdExpression );

        return AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRM_DELETE_EXPRESSION, url.getUrl(  ),
            AdminMessage.TYPE_CONFIRMATION );
    }

    /**
     * Retourne la map contenant les lments utiles au template HTML pour la slection
     * @param request le requte HTTP
     * @return model la map contenant les lments utiles au template HTML pour la slection
     */
    public HashMap<String, Object> getSelectionExpression( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        _strCurrentPageIndex = Paginator.getPageIndex( request, Paginator.PARAMETER_PAGE_INDEX, _strCurrentPageIndex );
        _nItemsPerPage = Paginator.getItemsPerPage( request, Paginator.PARAMETER_ITEMS_PER_PAGE, _nItemsPerPage,
                _nDefaultItemsPerPage );

        List<GExpressionUsuelle> listExpression = _expressionUsuelleHome.findExpressionList( plugin );

        Paginator paginator = new Paginator( listExpression, _nItemsPerPage,
                _ordreDuJourService.getUrlSelectionExpression(  ), Paginator.PARAMETER_PAGE_INDEX, _strCurrentPageIndex );

        model.put( MARK_LISTE_EXPRESSIONS, paginator.getPageItems(  ) );
        model.put( OdsMarks.MARK_NOMBRE_RESULTATS, listExpression.size(  ) );
        model.put( OdsMarks.MARK_PAGINATOR, paginator );
        model.put( OdsMarks.MARK_NB_ITEMS_PER_PAGE, OdsConstants.CONSTANTE_CHAINE_VIDE + _nItemsPerPage );
        model.put( MARK_POSITION_CURSEUR, request.getParameter( OdsParameters.ODJ_POSITION_CURSEUR ) );

        return model;
    }

    /**
     * Cre une expression usuelle.
     * Si le champ obligatoire est renseign,
     * la mthode retourne l'url de gestion des expressions usuelles.
     * Sinon la mthode retourne l'url affichant les messages d'erreur
     * @param request la requte HTTP
     * @return une url
     */
    public String doCreationExpression( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        if ( !requiredField( request ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            Object[] tabRequiredFields = 
                {
                    I18nService.getLocalizedString( requiredField( request ), request.getLocale(  ) ),
                };

            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED,
                tabRequiredFields, AdminMessage.TYPE_STOP );
        }

        GExpressionUsuelle expression = newExpressionUsuelleInstance(  );

        String strExpression = request.getParameter( OdsParameters.EXPRESSION ).trim(  );

        if ( strExpression.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            return AdminMessageService.getMessageUrl( request, Messages.MANDATORY_FIELDS, AdminMessage.TYPE_STOP );
        }

        expression.setExpression( strExpression );

        _expressionUsuelleHome.create( expression, plugin );

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * Modifie une expression usuelle.
     * Si le champ obligatoire est renseign,
     * la mthode retourne l'url de gestion des expressions usuelles.
     * Sinon la mthode retourne l'url affichant les messages d'erreur
     * @param request la requte HTTP
     * @return une url
     */
    public String doModificationExpression( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        if ( !requiredField( request ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            Object[] tabRequiredFields = 
                {
                    I18nService.getLocalizedString( requiredField( request ), request.getLocale(  ) ),
                };

            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED,
                tabRequiredFields, AdminMessage.TYPE_STOP );
        }

        int nIdExpression = OdsUtils.getIntegerParameter( OdsParameters.ID_EXPRESSION, request );
        GExpressionUsuelle expression = _expressionUsuelleHome.findByPrimaryKey( nIdExpression, plugin );

        if ( expression == null )
        {
            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                AdminMessage.TYPE_ERROR );
        }

        String strExpression = request.getParameter( OdsParameters.EXPRESSION ).trim(  );

        if ( strExpression.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            return AdminMessageService.getMessageUrl( request, Messages.MANDATORY_FIELDS, AdminMessage.TYPE_STOP );
        }

        expression.setExpression( strExpression );

        _expressionUsuelleHome.update( expression, plugin );

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * Supprime une expression usuelle.
     * @param request la requte HTTP
     * @return url de gestion des expressions usuelles
     */
    public String doSuppressionExpression( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        if ( request.getParameter( OdsParameters.ID_EXPRESSION ) == null )
        {
            return OdsConstants.CONSTANTE_CHAINE_VIDE;
        }

        int nIdExpression = OdsUtils.getIntegerParameter( OdsParameters.ID_EXPRESSION, request );

        GExpressionUsuelle expression = newExpressionUsuelleInstance(  );
        expression.setIdExpression( nIdExpression );

        try
        {
            _expressionUsuelleHome.remove( expression, plugin );
        }
        catch ( AppException ae )
        {
            if ( ae.getInitialException(  ) instanceof SQLException )
            {
                return AdminMessageService.getMessageUrl( request, MESSAGE_CANNOT_DELETE_EXPRESSION,
                    AdminMessage.TYPE_STOP );
            }
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * Indique si la requte est valide, c'est  dire que tous les champs obligatoires ont t remplis.
     * @param request la requte HTTP
     * @return true si la requte est valide, false sinon
     */
    private String requiredField( HttpServletRequest request )
    {
        if ( ( request.getParameter( OdsParameters.EXPRESSION ) == null ) ||
                request.getParameter( OdsParameters.EXPRESSION ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            return FIELD_EXPRESSION;
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    protected abstract GExpressionUsuelle newExpressionUsuelleInstance(  );
}
