/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.fichier;

import fr.paris.lutece.plugins.ods.business.fichier.FichierPhysiqueHome;
import fr.paris.lutece.plugins.ods.business.fichier.IFichierHome;
import fr.paris.lutece.plugins.ods.business.historique.HistoriqueHome;
import fr.paris.lutece.plugins.ods.business.seance.ISeanceHome;
import fr.paris.lutece.plugins.ods.dto.IBusinessItem;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.FichierPhysique;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter;
import fr.paris.lutece.plugins.ods.dto.fichier.ITypeDocument;
import fr.paris.lutece.plugins.ods.dto.historique.Historique;
import fr.paris.lutece.plugins.ods.dto.horodatage.HorodatageActionEnum;
import fr.paris.lutece.plugins.ods.dto.horodatage.IEntiteHorodatage;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur;
import fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.typeentite.ITypeEntite;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.service.horodatage.IHorodatageService;
import fr.paris.lutece.plugins.ods.service.horodatage.IHorodatageServiceManaged;
import fr.paris.lutece.plugins.ods.service.notification.INotificationService;
import fr.paris.lutece.plugins.ods.service.search.ISearchService;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.commons.ShaUtils;
import fr.paris.lutece.plugins.ods.utils.commons.StringUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.service.util.AppPropertiesService;
import fr.paris.lutece.portal.web.constants.Messages;
import fr.paris.lutece.portal.web.upload.MultipartHttpServletRequest;
import fr.paris.lutece.util.ReferenceList;
import fr.paris.lutece.util.html.Paginator;
import fr.paris.lutece.util.url.UrlItem;

import org.apache.commons.fileupload.FileItem;

import org.springframework.beans.factory.annotation.Autowired;

import java.sql.SQLException;
import java.sql.Timestamp;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.servlet.http.HttpServletRequest;


public abstract class AbstractFichierService<GSeance extends ISeance, GSeanceHome extends ISeanceHome<GSeance, GSeanceFilter, GFichier, GElu, GVoeuAmendement, GPDD>, GFichier extends IFichier<GSeance, GFichier>, GFichierFilter extends IFichierFilter, GTypeDocument extends ITypeDocument, GSeanceFilter extends ISeanceFilter, GFichierHome extends IFichierHome<GFichier, GFichierFilter, GTypeDocument, GSeance>, GElu extends IElu, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GRequeteUtilisateur extends IRequeteUtilisateur<GSeance, GSeanceFilter>, GResultatRequete extends IResultatRequete<GSeance, GSeanceFilter, GRequeteUtilisateur, GElu, GFichier, GVoeuAmendement, GPDD>>
    implements IFichierService<GSeance, GFichier>, IHorodatageServiceManaged
{
    private static final String PROPERTY_URL_DO_SUPPRESSION_FICHIER_JSP = "jsp/admin/plugins/ods/fichiers/DoSuppressionFichier.jsp";
    private static final String PROPERTY_URL_DO_SUPPRESSION_FICHIER_GESTION_AVAL_JSP = "jsp/admin/plugins/ods/fichiers/DoSuppressionFichierGestionAval.jsp";
    private static final String PROPERTY_LABEL_TYPE_DOCUMENT = "ods.creationfichier.label.typededocument";
    private static final String PROPERTY_LABEL_INTITULE = "ods.creationfichier.label.intitule";
    private static final String PROPERTY_LABEL_FICHIER = "ods.creationfichier.label.fichier";
    private static final String PROPERTY_LABEL_FICHIER_NO_PDF = "ods.creationfichier.label.fichier.noPDF";
    private static final String MARK_LISTE_FICHIERS = "liste_fichiers";
    private static final String MARK_LISTE_HISTORIQUES = "liste_historiques";
    private static final String MARK_LISTE_FICHIERS_SIZE_TOTAL = "liste_fichiers_size_total";
    private static final String MARK_URL_RETOUR = "url_retour";
    private static final String MESSAGE_CANNOT_DELETE_FICHIER = "ods.message.cannotDeleteFichier";
    private static final String MESSAGE_CONFIRM_DELETE_FICHIER = "ods.message.confirmDeleteFichier";

    // Variable de session pour la sance
    protected GSeance _seance;

    // liste des seances pour la gestion aval
    private List<GSeance> _listSeances;
    private int _nIdSeanceSelected;

    // Referencelist en vue de stocker l'ensemble des items reprsentant les types de documents
    protected ReferenceList _referenceTypeDocumentList;

    // Referencelist en vue de stocker l'ensemble des items reprsentant les types de documents pour la recherche
    protected ReferenceList _referenceTypeDocumentListRecherche;
    protected int _nIdTypeDocumentSelected;

    // Referencelist en vue de stocker l'ensemble des items reprsentant les types de publication
    private ReferenceList _referencePublicationList;
    private int _nIdPublicationSelected;

    // Gere la page du Paginator
    private String _strCurrentPageIndex;

    //Gere le nombre d'item par page
    private int _nItemsPerPage;
    private int _nDefaultItemsPerPage;
    @Autowired
    protected GFichierHome _fichierHome;
    @Autowired
    private FichierPhysiqueHome _fichierPhysiqueHome;
    @Autowired
    private IHorodatageService _horodatageService;
    @Autowired
    protected INotificationService<GSeance, GFichier> _notificationService;
    @Autowired
    private ISearchService<GSeance, GSeanceFilter, GRequeteUtilisateur, GResultatRequete, GElu, GFichier, GVoeuAmendement, GPDD> _searchService;
    @Autowired
    private HistoriqueHome _historiqueHome;
    @Autowired
    private GSeanceHome _seanceHome;

    /**
     * Constructor.
     */
    public AbstractFichierService(  )
    {
        _nDefaultItemsPerPage = AppPropertiesService.getPropertyInt( OdsProperties.PROPERTY_FICHIERS_PER_PAGE, 10 );
        _nIdSeanceSelected = -1;
        _nIdTypeDocumentSelected = -1;
        _nIdPublicationSelected = -1;
    }

    protected abstract void getFichiersListSpec( GFichierFilter filter, boolean bIsGestionAval );

    protected abstract boolean estTypeNotification( GFichier fichier );

    protected abstract void doPublicationFichierSpec( HttpServletRequest request, GFichier fichier );

    protected abstract String getParameterPluginName(  );

    /**
     * Permet d'initialiser la prochaine seance
     */
    public void setProchaineSeance(  )
    {
        Plugin plugin = getPlugin(  );
        _seance = _seanceHome.getProchaineSeance( plugin );

        if ( _seance != null )
        {
            _nIdSeanceSelected = _seance.getIdSeance(  );
        }
    }

    /**
     * Permet d'initialiser la dernire seance
     * @param request la requete Http
     */
    public void setAllSeances( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        // la liste contient toutes les seances passes
        _listSeances = _seanceHome.findOldSeance( plugin );

        int nIdSeance = OdsUtils.getIntegerParameter( OdsParameters.ID_SEANCE, request );

        if ( ( request.getParameter( OdsParameters.ID_SEANCE ) != null ) &&
                !request.getParameter( OdsParameters.ID_SEANCE ).trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            if ( nIdSeance > 0 )
            {
                _seance = _seanceHome.findByPrimaryKey( nIdSeance, plugin );
            }
            else
            {
                _seance = _seanceHome.getDerniereSeance( plugin );
            }
        }
        else
        {
            _seance = _seanceHome.getDerniereSeance( plugin );
        }

        if ( _seance != null )
        {
            _nIdSeanceSelected = _seance.getIdSeance(  );
        }
        else
        {
            _nIdSeanceSelected = -1;
        }
    }

    /**
     * Permet d'initialiser la liste des diffrents types de documents
     * @param request la requte http
     * @param bIsGestionAval TRUE si on est dans la gestion aval, FALSE sinon
     */
    public void setAllTypeDocuments( HttpServletRequest request, boolean bIsGestionAval )
    {
        Plugin plugin = getPlugin(  );

        List<GTypeDocument> typeDocuments;

        if ( bIsGestionAval )
        {
            typeDocuments = _fichierHome.findTypeDocumentsAvalList( plugin );
        }
        else
        {
            typeDocuments = _fichierHome.findTypeDocumentsProchaineSeanceList( plugin );
        }

        _referenceTypeDocumentList = new ReferenceList(  );
        //ajout dans la referenceList de l'item chaine vide
        _referenceTypeDocumentList.addItem( -1, OdsConstants.CONSTANTE_CHAINE_VIDE );
        _referenceTypeDocumentListRecherche = new ReferenceList(  );
        _referenceTypeDocumentListRecherche.addItem( -1, OdsConstants.CONSTANTE_CHAINE_VIDE );

        //ajout dans la referenceList de l'ensemble des items reprsentant les types de documents
        for ( GTypeDocument typeDocument : typeDocuments )
        {
            _referenceTypeDocumentList.addItem( typeDocument.getId(  ), typeDocument.getLibelle(  ) );
            _referenceTypeDocumentListRecherche.addItem( typeDocument.getId(  ), typeDocument.getLibelle(  ) );
        }

        if ( bIsGestionAval )
        {
            // on ajoute les types de document de la prochaine seance
            List<GTypeDocument> typeDocumentsProchaineSeance = _fichierHome.findTypeDocumentsProchaineSeanceList( plugin );

            for ( GTypeDocument typeDocument : typeDocumentsProchaineSeance )
            {
                _referenceTypeDocumentListRecherche.addItem( typeDocument.getId(  ), typeDocument.getLibelle(  ) );
            }
        }

        int nIdTypeDocument = -1;

        if ( request.getParameter( OdsParameters.ID_TYPE_DOCUMENT ) != null )
        {
            try
            {
                nIdTypeDocument = Integer.parseInt( request.getParameter( OdsParameters.ID_TYPE_DOCUMENT ).trim(  ) );
            }
            catch ( NumberFormatException nfe )
            {
                AppLogService.error( nfe );
            }

            _nIdTypeDocumentSelected = nIdTypeDocument;
        }
    }

    /**
     * Permet d'initialiser la liste des choix de publications
     * Les choix possibles: OUI, NON ou Pas d'importance
     * @param request la requte http
     */
    public void setAllPublication( HttpServletRequest request )
    {
        _referencePublicationList = new ReferenceList(  );

        //ajout dans la referenceList de l'item chaine vide
        _referencePublicationList.addItem( -1, OdsConstants.CONSTANTE_CHAINE_VIDE );
        _referencePublicationList.addItem( 0,
            I18nService.getLocalizedString( OdsConstants.CONSTANTE_NON, request.getLocale(  ) ) );
        _referencePublicationList.addItem( 1,
            I18nService.getLocalizedString( OdsConstants.CONSTANTE_OUI, request.getLocale(  ) ) );

        if ( request.getParameter( OdsParameters.ID_PUBLICATION ) != null )
        {
            int nIdPublication = OdsUtils.getIntegerParameter( OdsParameters.ID_PUBLICATION, request );

            _nIdPublicationSelected = nIdPublication;
        }
    }

    /**
     * Retourne la map contenant les lments utiles au template HTML de la liste
     * @param request la requte http
     * @param bIsGestionAval si on est dans la gestion aval ou pas
     * @param strUrlReturn l'URL de retour
     * @return model la map contenant les lments utiles au template HTML de la liste
     */
    public HashMap<String, Object> getFichiersList( HttpServletRequest request, boolean bIsGestionAval,
        String strUrlReturn )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        _strCurrentPageIndex = Paginator.getPageIndex( request, Paginator.PARAMETER_PAGE_INDEX, _strCurrentPageIndex );
        _nItemsPerPage = Paginator.getItemsPerPage( request, Paginator.PARAMETER_ITEMS_PER_PAGE, _nItemsPerPage,
                _nDefaultItemsPerPage );

        GFichierFilter filter = newFichierFilterInstance(  );
        List<GFichier> fichiers = new ArrayList<GFichier>(  );

        // on recupre la liste des fichiers correspondants au filtre
        if ( _seance != null )
        {
            filter.setIdSeance( _seance.getIdSeance(  ) );
            filter.setIdTypeDocument( _nIdTypeDocumentSelected );
            filter.setPublication( _nIdPublicationSelected );

            getFichiersListSpec( filter, bIsGestionAval );

            fichiers = _fichierHome.findByFilter( filter, plugin );
        }

        setAllTypeDocuments( request, bIsGestionAval );

        Paginator paginator = new Paginator( fichiers, _nItemsPerPage, strUrlReturn, Paginator.PARAMETER_PAGE_INDEX,
                _strCurrentPageIndex );

        // Ajouts des lments ncessaire dans la Hashmap
        if ( bIsGestionAval )
        {
            model.put( OdsMarks.MARK_LISTE_SEANCES, _listSeances );
            model.put( OdsMarks.MARK_ID_SEANCE_SELECTED, _nIdSeanceSelected );
        }

        model.put( OdsMarks.MARK_GESTION_AVAL, bIsGestionAval );
        model.put( MARK_LISTE_FICHIERS, paginator.getPageItems(  ) );
        model.put( MARK_LISTE_FICHIERS_SIZE_TOTAL, fichiers.size(  ) );
        model.put( OdsMarks.MARK_PAGINATOR, paginator );
        model.put( OdsMarks.MARK_NB_ITEMS_PER_PAGE, OdsConstants.CONSTANTE_CHAINE_VIDE + _nItemsPerPage );
        model.put( OdsMarks.MARK_PROCHAINE_SEANCE, _seance );
        model.put( OdsMarks.MARK_LISTE_TYPE_DOCUMENT, _referenceTypeDocumentListRecherche );
        model.put( OdsMarks.MARK_ID_TYPE_DOCUMENT_SELECTED, _nIdTypeDocumentSelected );

        ReferenceList refListPublie = new ReferenceList(  );
        initRefListPublie( refListPublie, request );
        model.put( OdsMarks.MARK_LISTE_PUBLIE, refListPublie );
        model.put( OdsMarks.MARK_ID_PUBLICATION_SELECTED, _nIdPublicationSelected );

        return model;
    }

    /**
     * retourne la map contenant les lments utiles au template HTML de creation
     * @param request la requte http
     * @param bIsGestionAval TRUE si on est dans la gestion aval, FALSE sinon
     * @return model la map contenant les lments utiles au template HTML de creation
     */
    public HashMap<String, Object> getCreationFichier( HttpServletRequest request, boolean bIsGestionAval )
    {
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        if ( bIsGestionAval )
        {
            model.put( OdsMarks.MARK_LISTE_SEANCES, _listSeances );

            if ( _seance != null )
            {
                model.put( OdsMarks.MARK_ID_SEANCE_SELECTED, _seance.getIdSeance(  ) );
            }
            else
            {
                model.put( OdsMarks.MARK_ID_SEANCE_SELECTED, _nIdSeanceSelected );
            }
        }

        model.put( OdsMarks.MARK_PROCHAINE_SEANCE, _seance );

        /*
         * Dplac dans CP/MA
        model.put( OdsMarks.MARK_LISTE_TYPE_DOCUMENT, _referenceTypeDocumentList );*/
        model.put( OdsMarks.MARK_GESTION_AVAL, bIsGestionAval );

        getCreationFichierSpec( request, model );

        return model;
    }

    /**
     * retourne la map contenant les lments utiles au template HTML de creation
     * @param request la requte http
     * @return model la map contenant les lments utiles au template HTML de creation
     */
    public HashMap<String, Object> getCreationFichierForPDD( HttpServletRequest request, int nIdTypeDocument,
        String strUrlRetour )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        String idPDD = request.getParameter( OdsParameters.ID_PDD );
        String idSeance = request.getParameter( OdsParameters.ID_SEANCE );

        ITypeDocument typePieceAnnexe = _fichierHome.findTypeDocumentsById( nIdTypeDocument, plugin );

        int nIdSeance = -1;

        if ( idSeance != null )
        {
            try
            {
                nIdSeance = Integer.parseInt( idSeance );
            }
            catch ( NumberFormatException e )
            {
                AppLogService.error( e );
            }
        }

        if ( nIdSeance != -1 )
        {
            _seance = _seanceHome.findByPrimaryKey( nIdSeance, getPlugin(  ) );
        }
        else
        {
            _seance = _seanceHome.getProchaineSeance( getPlugin(  ) );
        }

        model.put( OdsMarks.MARK_PROCHAINE_SEANCE, _seance );
        model.put( OdsMarks.MARK_LISTE_TYPE_DOCUMENT, typePieceAnnexe );
        model.put( OdsParameters.ID_PDD, idPDD );
        model.put( MARK_URL_RETOUR, strUrlRetour );

        return model;
    }

    /**
     * Retourne la map contenant les lments utiles au template HTML de modificationr
     * @param request la requte http
     * @param bIsGestionAval TRUE si on est dans la gestion aval, FALSE sinon
     * @return model la map contenant les lments utiles au template HTML de modification
     */
    public HashMap<String, Object> getModificationFichier( HttpServletRequest request, boolean bIsGestionAval )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        int nIdFichier = OdsUtils.getIntegerParameter( OdsParameters.ID_FICHIER, request );
        GFichier fichier = _fichierHome.findByPrimaryKey( nIdFichier, plugin );
        OdsUtils.testNullObject( fichier, request );

        model.put( OdsMarks.MARK_PROCHAINE_SEANCE, _seance );
        model.put( OdsMarks.MARK_FICHIER, fichier );

        // on vrifie que la liste contient le type de document sinon on l'ajoute
        /*
         * Dplac dans CP/MA  (getModificationFichierSpec)
        int nIdTypeDocumentFichier = fichier.getTypdeDocument(  ).getId(  );
        
        if ( bIsGestionAval && ( nIdTypeDocumentFichier != TypeDocumentEnum.COMPTE_RENDU.getId(  ) ) &&
                ( nIdTypeDocumentFichier != TypeDocumentEnum.LISTE_DE_DESIGNATION.getId(  ) ) &&
                ( nIdTypeDocumentFichier != TypeDocumentEnum.DOCUMENT_LIBRE.getId(  ) ) &&
                ( nIdTypeDocumentFichier != TypeDocumentEnum.BMO_DEBATS.getId(  ) ) &&
                ( nIdTypeDocumentFichier != TypeDocumentEnum.BDO_DEBATS.getId(  ) ) &&
                ( nIdTypeDocumentFichier != TypeDocumentEnum.COMPTE_RENDU_SOMMAIRE_CM.getId(  ) ) &&
                ( nIdTypeDocumentFichier != TypeDocumentEnum.COMPTE_RENDU_SOMMAIRE_CG.getId(  ) ) )
        {
            ReferenceList refListTypeDocumentModification = _referenceTypeDocumentList;
            refListTypeDocumentModification.addItem( fichier.getTypdeDocument(  ).getId(  ),
                fichier.getTypdeDocument(  ).getLibelle(  ) );
            model.put( OdsMarks.MARK_LISTE_TYPE_DOCUMENT, refListTypeDocumentModification );
        }
        else
        {
            model.put( OdsMarks.MARK_LISTE_TYPE_DOCUMENT, _referenceTypeDocumentList );
        }*/

        // on ajoute l'id du type de document slectionn ou l'id par dfaut
        if ( fichier.getTypdeDocument(  ) != null )
        {
            model.put( OdsMarks.MARK_ID_TYPE_DOCUMENT_SELECTED, fichier.getTypdeDocument(  ).getId(  ) );
        }
        else
        {
            model.put( OdsMarks.MARK_ID_TYPE_DOCUMENT_SELECTED, _nIdTypeDocumentSelected );
        }

        if ( bIsGestionAval )
        {
            model.put( OdsMarks.MARK_LISTE_SEANCES, _listSeances );

            if ( fichier.getSeance(  ) != null )
            {
                model.put( OdsMarks.MARK_ID_SEANCE_SELECTED, fichier.getSeance(  ).getIdSeance(  ) );
            }
            else
            {
                model.put( OdsMarks.MARK_ID_SEANCE_SELECTED, _nIdSeanceSelected );
            }
        }

        model.put( OdsMarks.MARK_GESTION_AVAL, bIsGestionAval );

        getModificationFichierSpec( request, fichier, model, bIsGestionAval );

        return model;
    }

    /**
     * retourne la map contenant les lments utiles au template HTML de modification
     * @param request la requte http
     * @return model la map contenant les lments utiles au template HTML de modification
     */
    public HashMap<String, Object> getModificationFichierForPDD( HttpServletRequest request, int nIdTypeDocument,
        String strUrlRetour )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        int nIdFichier = OdsUtils.getIntegerParameter( OdsParameters.ID_FICHIER, request );
        GFichier fichier = _fichierHome.findByPrimaryKey( nIdFichier, plugin );
        OdsUtils.testNullObject( fichier, request );

        ITypeDocument typePieceAnnexe = _fichierHome.findTypeDocumentsById( nIdTypeDocument, plugin );

        if ( _seance == null )
        {
            _seance = _seanceHome.getProchaineSeance( plugin );
        }

        int nIdPDD = OdsUtils.getIntegerParameter( OdsParameters.IDPDD, request );
        model.put( OdsMarks.MARK_PROCHAINE_SEANCE, _seance );
        model.put( OdsMarks.MARK_FICHIER, fichier );
        model.put( OdsMarks.MARK_LISTE_TYPE_DOCUMENT, typePieceAnnexe );
        model.put( OdsParameters.ID_PDD, nIdPDD );
        model.put( MARK_URL_RETOUR, strUrlRetour );

        return model;
    }

    /**
     * retourne une demande de confirmation pour la suppression du fichier choisie
     * @param request la requte http
     * @param bIsGestionAval TRUE si on est dans la gestion aval, FALSE sinon
     * @return la page de confirmation de suppression d'un fichier
     */
    public String getSuppressionFichier( HttpServletRequest request, boolean bIsGestionAval )
    {
        Plugin plugin = getPlugin(  );

        if ( request.getParameter( OdsParameters.ID_FICHIER ) != null )
        {
            String strIdFichier = request.getParameter( OdsParameters.ID_FICHIER );
            int nIdFichier = -1;

            try
            {
                nIdFichier = Integer.parseInt( strIdFichier.trim(  ) );
            }
            catch ( NumberFormatException nfe )
            {
                AppLogService.error( nfe );
            }

            GFichier fichier = _fichierHome.findByPrimaryKey( nIdFichier, plugin );

            if ( fichier == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            UrlItem url;

            if ( bIsGestionAval )
            {
                url = new UrlItem( PROPERTY_URL_DO_SUPPRESSION_FICHIER_GESTION_AVAL_JSP );
            }
            else
            {
                url = new UrlItem( PROPERTY_URL_DO_SUPPRESSION_FICHIER_JSP );
            }

            url.addParameter( OdsParameters.ID_FICHIER, strIdFichier );

            return AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRM_DELETE_FICHIER, url.getUrl(  ),
                AdminMessage.TYPE_CONFIRMATION );
        }
        else
        {
            return OdsConstants.CONSTANTE_CHAINE_VIDE;
        }
    }

    /**
     * retourne la map contenant les lments utiles au template HTML de l'historique
     * @param request la requte http
     * @return model la map contenant les lments utiles au template HTML de l'historique
     */
    public HashMap<String, Object> getHistoriqueFichier( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        int nIdFichier = OdsUtils.getIntegerParameter( OdsParameters.ID_FICHIER, request );
        GFichier fichier = _fichierHome.findByPrimaryKey( nIdFichier, plugin );
        OdsUtils.testNullObject( fichier, request );

        List<Historique> historiques = _historiqueHome.getHistoriqueList( fichier, plugin );

        model.put( MARK_LISTE_HISTORIQUES, historiques );

        return model;
    }

    /**
     * Creation d'un fichier
     * si tous les champs obligatoires du formulaire de cration d'un fichier ont t renseign,<BR>
     * la mthode retourne l'url de gestion des fichiers,<BR>
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request la requte http
     * @param bNoReinit FALSE si on reinitialise, TRUE sinon
     * @param strReturnDefault l'URL de retour par dfaut
     * @return l'url de creation d'un fichier
     */
    public String doCreationFichier( HttpServletRequest request, boolean bNoReinit, String strReturnDefault )
    {
        Plugin plugin = getPlugin(  );

        List<String> champsNonSaisie = new ArrayList<String>(  );

        if ( isValideForme( champsNonSaisie, request ) )
        {
            MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
            int nCreationIdTypeDocument = -1;
            FileItem fileItem = null;
            boolean enLigne = false;

            int nModificationIdSeance = -1;

            if ( request.getParameter( OdsParameters.ID_SEANCE ) != null )
            {
                try
                {
                    nModificationIdSeance = Integer.parseInt( request.getParameter( OdsParameters.ID_SEANCE ).trim(  ) );
                    nCreationIdTypeDocument = Integer.parseInt( request.getParameter( OdsParameters.ID_TYPE_DOCUMENT ) );
                }
                catch ( NumberFormatException nfe )
                {
                    AppLogService.error( nfe );
                }
            }

            String strCreationIntitule = request.getParameter( OdsParameters.FICHIER_INTITULE );
            String strCreationFichierTxtRecherche = request.getParameter( OdsParameters.FICHIER_TXT_RECHERCHE );

            fileItem = multipartRequest.getFile( OdsParameters.FICHIER_PHYSIQUE );

            GFichier newFichier = newFichierInstance(  );
            FichierPhysique newFichierPhysique = new FichierPhysique(  );

            // Creation de l'objet reprsentant le fichier physique
            if ( ( fileItem != null ) && ( fileItem.getName(  ) != null ) &&
                    !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItem.getName(  ) ) )
            {
                newFichierPhysique.setDonnees( fileItem.get(  ) );
                newFichierPhysique.setIdFichier( _fichierPhysiqueHome.create( newFichierPhysique, plugin ) );
                newFichier.setFichier( newFichierPhysique );
                newFichier.setNom( StringUtils.getNameCourt( fileItem ) );

                String strExtension = fileItem.getName(  ).substring( fileItem.getName(  ).length(  ) - 3 ).toUpperCase(  );

                if ( ( strExtension != null ) && ( strExtension.length(  ) == 3 ) &&
                        OdsConstants.CONSTANTE_PDF.equals( strExtension ) )
                {
                    newFichier.setExtension( strExtension );
                }
                else
                {
                    return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_NO_PDF,
                        AdminMessage.TYPE_STOP );
                }

                newFichier.setTaille( (int) fileItem.getSize(  ) );
            }
            else
            {
                Object[] messagesArgs = { I18nService.getLocalizedString( PROPERTY_LABEL_FICHIER, request.getLocale(  ) ) };

                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED,
                    messagesArgs, AdminMessage.TYPE_STOP );
            }

            // Creation de l'objet reprsentant le Fichier            
            if ( nModificationIdSeance > 0 )
            {
                GSeance seance = _seanceHome.findByPrimaryKey( nModificationIdSeance, plugin );
                newFichier.setSeance( seance );
            }

            if ( nCreationIdTypeDocument > 0 )
            {
                newFichier.setTypdeDocument( _fichierHome.findTypeDocumentsById( nCreationIdTypeDocument, plugin ) );
            }

            newFichier.setIntitule( strCreationIntitule );
            newFichier.setTexte( strCreationFichierTxtRecherche );
            newFichier.setEnLigne( enLigne );
            newFichier.setVersion( -1 );

            _fichierHome.create( newFichier, plugin );

            if ( newFichier.getSeance(  ) != null )
            {
                ISeance seanceByFichier = newFichier.getSeance(  );
                ISeance nextSeance = _seanceHome.getProchaineSeance( plugin );

                if ( ( nextSeance != null ) && !seanceByFichier.equals( nextSeance ) )
                {
                    _searchService.addObjectToIndex( newFichier, true, plugin );
                }
            }

            String strReturn = strReturnDefault;

            if ( bNoReinit )
            {
                strReturn += ( OdsConstants.CONSTANTE_POINT_INTERROGATION + getParameterPluginName(  ) +
                OdsConstants.CONSTANTE_ET + OdsParameters.ID_SEANCE + OdsConstants.CONSTANTE_EGAL +
                newFichier.getSeance(  ).getIdSeance(  ) );
            }

            return strReturn;
        }
        else
        {
            String strChamp = ( champsNonSaisie.get( 0 ) != null ) ? champsNonSaisie.get( 0 ) : Messages.MANDATORY_FIELDS;
            Object[] messagesArgs = { I18nService.getLocalizedString( strChamp, request.getLocale(  ) ) };

            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED,
                messagesArgs, AdminMessage.TYPE_STOP );
        }
    }

    /**
     * Modifie le fichier et le fichier physique
     * Si le fichie est en ligne, alors la version est incrment de 1 et ajout d'un historique
     * si tous les champs obligatoires du formulaire de modification d'un fichier ont t renseign,<BR>
     * la mthode retourne l'url de gestion des fichiers,<BR>
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request la requte http
     * @param bNoReinit FALSE si on reinitialise, TRUE sinon
     * @param strReturnDefault l'URL de retour par dfaut
     * @return l'url de modification d'un fichier
     */
    public String doModificationFichier( HttpServletRequest request, boolean bNoReinit, String strReturnDefault )
    {
        Plugin plugin = getPlugin(  );

        Timestamp dateForVersion = new Timestamp( System.currentTimeMillis(  ) );
        int nIdFichier = OdsUtils.getIntegerParameter( OdsParameters.ID_FICHIER, request );
        int nModificationIdTypeDocument = OdsUtils.getIntegerParameter( OdsParameters.ID_TYPE_DOCUMENT, request );

        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;

        if ( ( nIdFichier != -1 ) && ( nModificationIdTypeDocument != -1 ) )
        {
            GFichier fichier = _fichierHome.findByPrimaryKey( nIdFichier, plugin );

            if ( fichier == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            // recupration des modifications
            int nModificationIdSeance = OdsUtils.getIntegerParameter( OdsParameters.ID_SEANCE, request );

            String strModificationIntitule = request.getParameter( OdsParameters.FICHIER_INTITULE );

            if ( ( strModificationIntitule == null ) ||
                    OdsConstants.CONSTANTE_CHAINE_VIDE.equals( strModificationIntitule ) )
            {
                Object[] messagesArgs = 
                    {
                        I18nService.getLocalizedString( PROPERTY_LABEL_INTITULE, request.getLocale(  ) ),
                    };

                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED,
                    messagesArgs, AdminMessage.TYPE_STOP );
            }

            String strModificationFichierTxtRecherche = request.getParameter( OdsParameters.FICHIER_TXT_RECHERCHE );
            FileItem fileItem = multipartRequest.getFile( OdsParameters.FICHIER_PHYSIQUE );

            if ( ( fileItem != null ) && ( fileItem.getName(  ) != null ) &&
                    !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItem.getName(  ) ) )
            {
                FichierPhysique fichierPhysique = new FichierPhysique(  );
                fichierPhysique.setIdFichier( fichier.getFichier(  ).getIdFichier(  ) );

                // Modification de l'objet reprsentant le fichier physique
                fichierPhysique.setDonnees( fileItem.get(  ) );
                _fichierPhysiqueHome.update( fichierPhysique, plugin );
                fichier.setFichier( fichierPhysique );
                fichier.setNom( StringUtils.getNameCourt( fileItem ) );

                String strExtension = fileItem.getName(  ).substring( fileItem.getName(  ).length(  ) - 3 ).toUpperCase(  );

                if ( ( strExtension != null ) && ( strExtension.length(  ) == 3 ) &&
                        OdsConstants.CONSTANTE_PDF.equals( strExtension ) )
                {
                    fichier.setExtension( strExtension );
                }
                else
                {
                    return AdminMessageService.getMessageUrl( request, PROPERTY_LABEL_FICHIER_NO_PDF,
                        AdminMessage.TYPE_STOP );
                }

                fichier.setTaille( (int) fileItem.getSize(  ) );
            }

            // Modification de l'objet reprsentant le Fichier
            fichier.setTypdeDocument( _fichierHome.findTypeDocumentsById( nModificationIdTypeDocument, plugin ) );

            if ( nModificationIdSeance > 0 )
            {
                GSeance seance = _seanceHome.findByPrimaryKey( nModificationIdSeance, plugin );
                fichier.setSeance( seance );
            }

            fichier.setIntitule( strModificationIntitule );
            fichier.setTexte( strModificationFichierTxtRecherche );

            if ( fichier.getEnLigne(  ) )
            {
                // on incremente le numero de version
                fichier.setVersion( fichier.getVersion(  ) + 1 );
                fichier.setDatePublication( dateForVersion );

                //Ajout de l'historique
                Historique historique = new Historique(  );
                historique.setVersion( fichier.getVersion(  ) );
                historique.setDatePublication( dateForVersion );
                historique.setIdEntite( fichier.getId(  ) );
                historique.setIdTypeEntite( ITypeEntite.FICHIERS );
                _historiqueHome.create( historique, plugin );
            }

            if ( ( fichier != null ) && ( fichier.getSeance(  ) != null ) )
            {
                ISeance seanceByFichier = fichier.getSeance(  );
                ISeance nextSeance = _seanceHome.getProchaineSeance( plugin );

                if ( ( nextSeance != null ) && !seanceByFichier.equals( nextSeance ) &&
                        ( ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.DOCUMENT_LIBRE ) ||
                        ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.COMPTE_RENDU ) ||
                        ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.LISTE_DE_DESIGNATION ) ) )
                {
                    _searchService.updateObjectInIndex( fichier, true, plugin );
                }
            }

            _fichierHome.update( fichier, plugin );

            if ( fichier.getEnLigne(  ) )
            {
                _horodatageService.tracePublication( fichier, HorodatageActionEnum.MODIFICATION );
            }

            String strReturn = strReturnDefault;

            if ( bNoReinit )
            {
                strReturn += ( OdsConstants.CONSTANTE_POINT_INTERROGATION + getParameterPluginName(  ) +
                OdsConstants.CONSTANTE_ET + OdsParameters.ID_SEANCE + OdsConstants.CONSTANTE_EGAL +
                fichier.getSeance(  ).getIdSeance(  ) );
            }

            return strReturn;
        }
        else
        {
            Object[] messagesArgs = 
                {
                    I18nService.getLocalizedString( PROPERTY_LABEL_TYPE_DOCUMENT, request.getLocale(  ) ),
                };

            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED,
                messagesArgs, AdminMessage.TYPE_STOP );
        }
    }

    /**
     * Publication d'un fichier
     * Si "OdsParameters.FICHIER_IS_PUBLIER" gale  TRUE, alors la mthode <b>publie</b> le fichier et incrmente la version<BR>
     *                 Dans ce cas, un historique est cre
     * Sinon la mthode change le statut enLigne d'un fichier a FALSE
     * @param request la requte http
     * @param bNoReinit FALSE si on reinitialise, TRUE sinon
     * @param strUrlReturn l'URL de retour
     * @return l'url
     */
    public String doPublicationFichier( HttpServletRequest request, boolean bNoReinit, String strUrlReturn )
    {
        Plugin plugin = getPlugin(  );

        Timestamp dateForVersion = new Timestamp( System.currentTimeMillis(  ) );

        String strIsPublier = request.getParameter( OdsParameters.FICHIER_IS_PUBLIER );
        boolean bEstPublication = Boolean.parseBoolean( OdsConstants.CONSTANTE_CHAINE_VIDE + strIsPublier );

        int nIdFichier = OdsUtils.getIntegerParameter( OdsParameters.ID_FICHIER, request );

        if ( nIdFichier != -1 )
        {
            GFichier fichier = _fichierHome.findByPrimaryKey( nIdFichier, plugin );
            OdsUtils.testNullObject( fichier, request );

            if ( !( ( fichier.getEnLigne(  ) && bEstPublication ) || ( !fichier.getEnLigne(  ) && !bEstPublication ) ) )
            {
                if ( bEstPublication && ( fichier.getVersion(  ) < 1 ) )
                {
                    fichier.setVersion( 0 );
                }

                _fichierHome.publication( fichier.getId(  ), dateForVersion, fichier.getVersion(  ), bEstPublication,
                    plugin );
                fichier = _fichierHome.findByPrimaryKey( nIdFichier, plugin );

                if ( bEstPublication )
                {
                    Historique historique = new Historique(  );
                    historique.setVersion( fichier.getVersion(  ) );
                    historique.setDatePublication( dateForVersion );
                    historique.setIdEntite( fichier.getId(  ) );
                    historique.setIdTypeEntite( ITypeEntite.FICHIERS );
                    _historiqueHome.create( historique, plugin );
                }

                if ( fichier.getSeance(  ) != null )
                {
                    GSeance seance = _seanceHome.findByPrimaryKey( fichier.getSeance(  ).getIdSeance(  ), plugin );
                    fichier.setSeance( seance );
                }

                // on horodate la (de)publication
                HorodatageActionEnum action = ( bEstPublication ) ? HorodatageActionEnum.PUBLICATION
                                                                  : HorodatageActionEnum.DE_PUBLICATION;
                _horodatageService.tracePublication( fichier, action );

                String strReturn = strUrlReturn;

                if ( bNoReinit )
                {
                    strReturn += ( OdsConstants.CONSTANTE_POINT_INTERROGATION + getParameterPluginName(  ) +
                    OdsConstants.CONSTANTE_ET + OdsParameters.ID_SEANCE + OdsConstants.CONSTANTE_EGAL +
                    fichier.getSeance(  ).getIdSeance(  ) );
                }

                if ( bEstPublication && estTypeNotification( fichier ) )
                {
                    doPublicationFichierSpec( request, fichier );
                }

                return strReturn;
            }
        }

        String strReturn = strUrlReturn;

        if ( bNoReinit )
        {
            strReturn += ( OdsConstants.CONSTANTE_POINT_INTERROGATION + getParameterPluginName(  ) +
            OdsConstants.CONSTANTE_ET + OdsParameters.ID_SEANCE + OdsConstants.CONSTANTE_EGAL +
            _seance.getIdSeance(  ) );
        }

        return strReturn;
    }

    /**
     * supprime le fichier choisi
     * @param request la requte http
     * @param bNoReinit FALSE si on reinitialise, TRUE sinon
     * @param strReturnDefault l'URL de retour par dfaut
     * @return l'url de suppression de fichier
     */
    public String doSuppressionFichier( HttpServletRequest request, boolean bNoReinit, String strReturnDefault )
    {
        Plugin plugin = getPlugin(  );

        String strReturn = null;

        int nIdFichier = OdsUtils.getIntegerParameter( OdsParameters.ID_FICHIER, request );

        if ( nIdFichier != -1 )
        {
            GFichier fichier = _fichierHome.findByPrimaryKey( nIdFichier, plugin );

            if ( fichier == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            FichierPhysique fichierPhysique = _fichierPhysiqueHome.findByPrimaryKey( fichier.getFichier(  )
                                                                                            .getIdFichier(  ), plugin );

            try
            {
                _historiqueHome.remove( fichier, plugin );

                if ( ( fichier != null ) && ( fichier.getSeance(  ) != null ) )
                {
                    ISeance seanceByFichier = fichier.getSeance(  );
                    ISeance nextSeance = _seanceHome.getProchaineSeance( plugin );

                    if ( ( nextSeance != null ) && !seanceByFichier.equals( nextSeance ) &&
                            ( ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.DOCUMENT_LIBRE ) ||
                            ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.COMPTE_RENDU ) ||
                            ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.LISTE_DE_DESIGNATION ) ) )
                    {
                        _searchService.deleteObjectFromIndex( fichier, true, plugin );
                    }
                }

                _fichierHome.remove( fichier, plugin );
                _fichierPhysiqueHome.remove( fichierPhysique, plugin );
            }
            catch ( AppException ae )
            {
                if ( ae.getInitialException(  ) instanceof SQLException )
                {
                    strReturn = AdminMessageService.getMessageUrl( request, MESSAGE_CANNOT_DELETE_FICHIER,
                            AdminMessage.TYPE_STOP );
                }

                if ( strReturn == null )
                {
                    strReturn = strReturnDefault;

                    if ( bNoReinit )
                    {
                        strReturn += ( OdsConstants.CONSTANTE_POINT_INTERROGATION + getParameterPluginName(  ) +
                        OdsConstants.CONSTANTE_ET + OdsParameters.ID_SEANCE + OdsConstants.CONSTANTE_EGAL +
                        _seance.getIdSeance(  ) );
                    }
                }
            }

            strReturn = strReturnDefault;

            if ( bNoReinit )
            {
                strReturn += ( OdsConstants.CONSTANTE_POINT_INTERROGATION + getParameterPluginName(  ) +
                OdsConstants.CONSTANTE_ET + OdsParameters.ID_SEANCE + OdsConstants.CONSTANTE_EGAL +
                fichier.getSeance(  ).getIdSeance(  ) );
            }
        }
        else
        {
            strReturn = strReturnDefault;

            if ( bNoReinit )
            {
                strReturn += ( OdsConstants.CONSTANTE_POINT_INTERROGATION + getParameterPluginName(  ) +
                OdsConstants.CONSTANTE_ET + OdsParameters.ID_SEANCE + OdsConstants.CONSTANTE_EGAL +
                _seance.getIdSeance(  ) );
            }
        }

        return strReturn;
    }

    /**
     * Tlecharge un fichier.
     * Renvoie un flux de bytes du fichier dans l'objet HttpServletResponse response
     * @param request la requte http
     * @return fichierTelecharger le fichier  tlcharger
     */
    public GFichier doDownloadFichier( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        GFichier fichierTelecharger = null;

        String strIdFichier = request.getParameter( OdsParameters.ID_FICHIER );

        if ( strIdFichier != null )
        {
            int nIdFichier = -1;

            try
            {
                nIdFichier = Integer.parseInt( strIdFichier );
            }
            catch ( NumberFormatException nfe )
            {
                AppLogService.error( nfe );
            }

            fichierTelecharger = _fichierHome.findByPrimaryKey( nIdFichier, plugin );

            if ( fichierTelecharger != null )
            {
                FichierPhysique fichierPhysique = _fichierPhysiqueHome.findByPrimaryKey( fichierTelecharger.getFichier(  )
                                                                                                           .getIdFichier(  ),
                        plugin );
                fichierTelecharger.setFichier( fichierPhysique );
            }
        }

        return fichierTelecharger;
    }

    /**
     *
     * @param request
     * @return
     */
    public boolean isPublier( HttpServletRequest request )
    {
        String strIsPublier = request.getParameter( OdsParameters.FICHIER_IS_PUBLIER );

        return Boolean.parseBoolean( OdsConstants.CONSTANTE_CHAINE_VIDE + strIsPublier );
    }

    /**
     * Retourne true si le VA est publi, false sinon
     * @param nIdVA l'id du VA
     * @return true si le VA est publi, false sinon
     */
    public boolean isPublier( int nIdFichier )
    {
        Plugin plugin = getPlugin(  );

        GFichier fichier = _fichierHome.findByPrimaryKey( nIdFichier, plugin );

        return fichier.getEnLigne(  );
    }

    /**
     * methode qui test si dans le formulaire de saisi d'un fichier,l'ensemble des critres obligatoires ont t renseign
     * @param request la requte http
     * @param champsNonSaisie List<String>
     * @return VRAI si le formulaire envoy est valide
     */
    private boolean isValideForme( List<String> champsNonSaisie, HttpServletRequest request )
    {
        boolean isValid = true;

        if ( OdsUtils.getIntegerParameter( OdsParameters.ID_TYPE_DOCUMENT, request ) == -1 )
        {
            champsNonSaisie.add( PROPERTY_LABEL_TYPE_DOCUMENT );
            isValid = false;
        }

        String strCreationIntitule = request.getParameter( OdsParameters.FICHIER_INTITULE );

        if ( ( strCreationIntitule == null ) || OdsConstants.CONSTANTE_CHAINE_VIDE.equals( strCreationIntitule ) )
        {
            champsNonSaisie.add( PROPERTY_LABEL_INTITULE );
            isValid = false;
        }

        return isValid;
    }

    @SuppressWarnings( "unchecked" )
    public ISeance getSeance( IBusinessItem entite )
    {
        Plugin plugin = getPlugin(  );

        GFichier fichier = (GFichier) entite;

        ISeance seance = fichier.getSeance(  );

        if ( seance == null )
        {
            seance = _seanceHome.getProchaineSeance( plugin );
        }

        return seance;
    }

    @SuppressWarnings( "unchecked" )
    public String getSignature( IEntiteHorodatage entite )
    {
        Plugin plugin = getPlugin(  );

        String signature = OdsConstants.CONSTANTE_CHAINE_VIDE;

        GFichier fichier = (GFichier) entite;
        fichier.setFichier( _fichierPhysiqueHome.findByPrimaryKey( fichier.getFichier(  ).getIdFichier(  ), plugin ) );

        if ( fichier.getFichier(  ) != null )
        {
            byte[] donnees = fichier.getFichier(  ).getDonnees(  );
            signature = ShaUtils.toSha224String( donnees );

            if ( signature == null )
            {
                signature = "error_" + fichier.getNom(  );
                AppLogService.error( "Erreur lors de la recuperation de la signature SHA 224 du fichier : " +
                    fichier.getNom(  ) );
            }
        }

        return signature;
    }

    public String getIdParameter(  )
    {
        return null;
    }

    public int getIndexerAction( IBusinessItem item )
    {
        return item.getTypeEntite(  );
    }

    public IBusinessItem createItemToDeleteInIndexer( int nIdItem )
    {
        GFichier fichier = newFichierInstance(  );
        fichier.setId( nIdItem );

        return fichier;
    }

    /**
    * Permet d'initialiser la liste des choix de publications
    * Les choix possibles: OUI,NON
    * @param refList la liste pour le menu droulant
    */
    protected void initRefListPublie( ReferenceList refList, HttpServletRequest request )
    {
        refList.addItem( -1, OdsConstants.CONSTANTE_CHAINE_VIDE );
        refList.addItem( 1, I18nService.getLocalizedString( OdsConstants.CONSTANTE_OUI, request.getLocale(  ) ) );
        refList.addItem( 0, I18nService.getLocalizedString( OdsConstants.CONSTANTE_NON, request.getLocale(  ) ) );
    }

    protected abstract GFichierFilter newFichierFilterInstance(  );

    /**
     * Cre un objet IFichier
     * @return une instance de GFichier
     */
    protected abstract GFichier newFichierInstance(  );

    /**
     * Cre un lment ITypeDocument
     * @return une instance de GTypeDocument
     */
    protected abstract GTypeDocument getTypeDocument(  );

    /**
     * Rajoute des informations supplmentaires dans le model pour la modification.
     * Doit ajouter la liste des types de documents  crer.
     * @param request HttpServletRequest
     * @param fichier le fichier
     * @param model le model
     * @param bIsGestionAval <b>true</b> si gestion aval, <b>false</b> sinon.
     */
    protected abstract void getModificationFichierSpec( HttpServletRequest request, GFichier fichier,
        HashMap<String, Object> model, boolean bIsGestionAval );

    /**
     * Rajoute des informations supplmentaires dans le model pour la cration
     * @param request HttpServletRequest
     * @param model le model
     */
    protected abstract void getCreationFichierSpec( HttpServletRequest request, HashMap<String, Object> model );

    /**
     * Retourne le plugin
     * @return le plugin
     */
    protected abstract Plugin getPlugin(  );
}
