/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.fichier;

import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.service.IODSService;

import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;


/**
 *
 * IFichierService
 *
 */
public interface IFichierService<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>>
    extends IODSService
{
    /**
     * Cre une nouvelle instance d'IFichier
     * @return une nouvelle instance d'IFI
     */
    GFichier newFichier(  );

    /**
     * Permet d'initialiser la prochaine seance
     */
    void setProchaineSeance(  );

    /**
     * Permet d'initialiser la dernire seance
     * @param request la requete Http
     */
    void setAllSeances( HttpServletRequest request );

    /**
     * Permet d'initialiser la liste des diffrents types de documents
     * @param request la requte http
     * @param bIsGestionAval TRUE si on est dans la gestion aval, FALSE sinon
     */
    void setAllTypeDocuments( HttpServletRequest request, boolean bIsGestionAval );

    /**
     * Permet d'initialiser la liste des choix de publications
     * Les choix possibles: OUI, NON ou Pas d'importance
     * @param request la requte http
     */
    void setAllPublication( HttpServletRequest request );

    /**
     * Retourne la map contenant les lments utiles au template HTML de la liste
     * @param request la requte http
     * @param bIsGestionAval si on est dans la gestion aval ou pas
     * @param strUrlReturn l'URL de retour
     * @return model la map contenant les lments utiles au template HTML de la liste
     */
    HashMap<String, Object> getFichiersList( HttpServletRequest request, boolean bIsGestionAval, String strUrlReturn );

    /**
     * retourne la map contenant les lments utiles au template HTML de creation
     * @param request la requte http
     * @param bIsGestionAval TRUE si on est dans la gestion aval, FALSE sinon
     * @return model la map contenant les lments utiles au template HTML de creation
     */
    HashMap<String, Object> getCreationFichier( HttpServletRequest request, boolean bIsGestionAval );

    /**
     * retourne la map contenant les lments utiles au template HTML de creation
     * @param request la requte http
      * @return model la map contenant les lments utiles au template HTML de creation
     */
    HashMap<String, Object> getCreationFichierForPDD( HttpServletRequest request, int nIdTypeDocument,
        String strUrlRetour );

    /**
     * Retourne la map contenant les lments utiles au template HTML de modificationr
     * @param request la requte http
     * @param bIsGestionAval TRUE si on est dans la gestion aval, FALSE sinon
     * @return model la map contenant les lments utiles au template HTML de modification
     */
    HashMap<String, Object> getModificationFichier( HttpServletRequest request, boolean bIsGestionAval );

    /**
     * retourne la map contenant les lments utiles au template HTML de modification
     * @param request la requte http
     * @return model la map contenant les lments utiles au template HTML de modification
     */
    HashMap<String, Object> getModificationFichierForPDD( HttpServletRequest request, int nIdTypeDocument,
        String strUrlRetour );

    /**
     * retourne une demande de confirmation pour la suppression du fichier choisie
     * @param request la requte http
     * @param bIsGestionAval TRUE si on est dans la gestion aval, FALSE sinon
     * @return la page de confirmation de suppression d'un fichier
     */
    String getSuppressionFichier( HttpServletRequest request, boolean bIsGestionAval );

    /**
     * retourne la map contenant les lments utiles au template HTML de l'historique
     * @param request la requte http
     * @return model la map contenant les lments utiles au template HTML de l'historique
     */
    HashMap<String, Object> getHistoriqueFichier( HttpServletRequest request );

    /**
     * Creation d'un fichier
     * si tous les champs obligatoires du formulaire de cration d'un fichier ont t renseign,<BR>
     * la mthode retourne l'url de gestion des fichiers,<BR>
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request la requte http
     * @param bNoReinit FALSE si on reinitialise, TRUE sinon
     * @param strReturnDefault l'URL de retour par dfaut
     * @return l'url de creation d'un fichier
     */
    String doCreationFichier( HttpServletRequest request, boolean bNoReinit, String strReturnDefault );

    /**
     * Modifie le fichier et le fichier physique
     * Si le fichie est en ligne, alors la version est incrment de 1 et ajout d'un historique
     * si tous les champs obligatoires du formulaire de modification d'un fichier ont t renseign,<BR>
     * la mthode retourne l'url de gestion des fichiers,<BR>
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request la requte http
     * @param bNoReinit FALSE si on reinitialise, TRUE sinon
     * @param strReturnDefault l'URL de retour par dfaut
     * @return l'url de modification d'un fichier
     */
    String doModificationFichier( HttpServletRequest request, boolean bNoReinit, String strReturnDefault );

    /**
     * Publication d'un fichier
     * Si "OdsParameters.FICHIER_IS_PUBLIER" gale  TRUE, alors la mthode <b>publie</b> le fichier et incrmente la version<BR>
     *                 Dans ce cas, un historique est cre
     * Sinon la mthode change le statut enLigne d'un fichier a FALSE
     * @param request la requte http
     * @param bNoReinit FALSE si on reinitialise, TRUE sinon
     * @param strUrlReturn l'URL de retour
     * @return l'url
     */
    String doPublicationFichier( HttpServletRequest request, boolean bNoReinit, String strUrlReturn );

    /**
     * supprime le fichier choisi
     * @param request la requte http
     * @param bNoReinit FALSE si on reinitialise, TRUE sinon
     * @param strReturnDefault l'URL de retour par dfaut
     * @return l'url de suppression de fichier
     */
    String doSuppressionFichier( HttpServletRequest request, boolean bNoReinit, String strReturnDefault );

    /**
     * Tlecharge un fichier.
     * Renvoie un flux de bytes du fichier dans l'objet HttpServletResponse response
     * @param request la requte http
     * @return fichierTelecharger le fichier  tlcharger
     */
    GFichier doDownloadFichier( HttpServletRequest request );

    /**
     *
     * @param request
     * @return
     */
    boolean isPublier( HttpServletRequest request );

    /**
     * Retourne true si le VA est publi, false sinon
     * @param nIdVA l'id du VA
     * @return true si le VA est publi, false sinon
     */
    boolean isPublier( int nIdFichier );
}
