/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.formationconseil;

import fr.paris.lutece.plugins.ods.business.formationconseil.FormationConseilHome;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.util.ReferenceList;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;


public abstract class AbstractFormationConseilService implements IFormationConseilService
{
    @Autowired
    private FormationConseilHome _formationConseilHome;

    protected abstract Plugin getPlugin(  );

    /**
     * Methode qui initialise la list du format de conseil
     * @return ReferenceList liste formation conseil
     */
    public ReferenceList initRefListFormationConseil( boolean bIsItemVideEnabled )
    {
        Plugin plugin = getPlugin(  );

        ReferenceList refListFormationConseil = new ReferenceList(  );
        List<FormationConseil> listFormationConseil = _formationConseilHome.findFormationConseilList( plugin );

        if ( bIsItemVideEnabled )
        {
            refListFormationConseil.addItem( -1, OdsConstants.CONSTANTE_CHAINE_VIDE );
        }

        for ( FormationConseil formationConseil : listFormationConseil )
        {
            refListFormationConseil.addItem( formationConseil.getIdFormationConseil(  ), formationConseil.getLibelle(  ) );
        }

        return refListFormationConseil;
    }
}
