/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.groupepolitique;

import fr.paris.lutece.plugins.ods.business.groupepolitique.IGroupePolitiqueHome;
import fr.paris.lutece.plugins.ods.dto.groupepolitique.GroupePolitique;
import fr.paris.lutece.plugins.ods.dto.groupepolitique.IGroupePolitique;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.web.constants.Messages;
import fr.paris.lutece.util.ReferenceList;
import fr.paris.lutece.util.url.UrlItem;

import org.springframework.beans.factory.annotation.Autowired;

import java.sql.SQLException;

import java.util.HashMap;
import java.util.List;

import javax.servlet.http.HttpServletRequest;


public abstract class AbstractGroupePolitiqueService implements IGroupePolitiqueService
{
    private static final String MARK_LISTE_GROUPES = "liste_groupes";
    private static final String MARK_GROUPE = "groupe";
    private static final String MESSAGE_CONFIRMDELETEGROUPE = "ods.message.confirmDeleteGroupe";
    private static final String MESSAGE_CANNOT_DELETE_GROUPE = "ods.message.cannotDeleteGroupe";
    private static final String JSP_URL_DO_SUPPRESSION_GROUPE_JSP = "jsp/admin/plugins/ods/groupepolitique/DoSuppressionGroupe.jsp";
    private static final String FIELD_NOM_COMPLET = "ods.creationgroupe.label.nom_complet";
    private static final String FIELD_NOM_GROUPE = "ods.creationgroupe.label.nom_groupe";
    @Autowired
    private IGroupePolitiqueHome _groupePolitiqueHome;

    protected abstract Plugin getPlugin(  );

    /**
     * retourne l'interface de gestion des groupes politiques.
     * @param request la requete HTTP
     * @return   interface de gestion des groupes politiques
     */
    public HashMap<String, Object> getGroupeList( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        List<IGroupePolitique> groupes = _groupePolitiqueHome.findGroupePolitiqueList( plugin );
        model.put( MARK_LISTE_GROUPES, groupes );

        return model;
    }

    /**
     * cre un groupe politique.
     * si tous les champs obligatoires du formulaire de cration d'un groupe politique ont t renseign,
     * la mthode retourne l'url de gestion des groupes poliques,
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request request
     * @return url url de retour
     */
    public String doCreationGroupe( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        String strChamp = requiredField( request );

        if ( !strChamp.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            Object[] message = { I18nService.getLocalizedString( strChamp, request.getLocale(  ) ) };

            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED, message,
                AdminMessage.TYPE_STOP );
        }

        GroupePolitique groupe = new GroupePolitique(  );

        String strNomCourt = request.getParameter( OdsParameters.NOM_GROUPE ).trim(  );
        groupe.setNomGroupe( strNomCourt );

        String strNomLong = request.getParameter( OdsParameters.NOM_COMPLET ).trim(  );
        groupe.setNomComplet( strNomLong );

        String strActif = request.getParameter( OdsParameters.ACTIF );
        boolean bActif;

        if ( null == strActif )
        {
            bActif = false;
        }
        else
        {
            bActif = true;
        }

        groupe.setActif( bActif );

        _groupePolitiqueHome.create( groupe, plugin );

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * retourne le formulaire de modification d'un groupe politique  pr remplis avec  les critres  du groupe politique choisi
     * @param request la requete HTTP
     * @return formulaire de modification d'un groupe politique
     */
    public HashMap<String, Object> getModificationGroupe( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        if ( request.getParameter( OdsParameters.ID_GROUPE ) == null )
        {
            return null;
        }

        int nIdGroupePolitique = OdsUtils.getIntegerParameter( OdsParameters.ID_GROUPE, request );
        IGroupePolitique groupe = _groupePolitiqueHome.findByPrimaryKey( nIdGroupePolitique, plugin );
        OdsUtils.testNullObject( groupe, request );

        model.put( MARK_GROUPE, groupe );
        model.put( OdsMarks.MARK_ACTIF, groupe.isActif(  ) );

        return model;
    }

    /**
     * Modifie un groupe politique.
     * si tous les champs obligatoires du formulaire de modification d'un groupe politique ont t renseign,
     * la mthode retourne l'url de gestion des groupes poliques,
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request request
     * @return url url de retour
     */
    public String doModificationGroupe( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        String strChamp = requiredField( request );

        if ( !strChamp.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            Object[] message = { I18nService.getLocalizedString( strChamp, request.getLocale(  ) ) };

            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED, message,
                AdminMessage.TYPE_STOP );
        }

        int nIdGroupe = OdsUtils.getIntegerParameter( OdsParameters.ID_GROUPE, request );

        if ( nIdGroupe == -1 )
        {
            return AdminMessageService.getMessageUrl( request, Messages.MANDATORY_FIELDS, AdminMessage.TYPE_STOP );
        }

        IGroupePolitique groupe = _groupePolitiqueHome.findByPrimaryKey( nIdGroupe, plugin );

        if ( groupe == null )
        {
            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                AdminMessage.TYPE_ERROR );
        }

        String strNomGroupe = request.getParameter( OdsParameters.NOM_GROUPE ).trim(  );
        groupe.setNomGroupe( strNomGroupe );

        String strNomComplet = request.getParameter( OdsParameters.NOM_COMPLET ).trim(  );
        groupe.setNomComplet( strNomComplet );

        String strActif = request.getParameter( OdsParameters.ACTIF );
        boolean bActif;

        if ( null == strActif )
        {
            bActif = false;
        }
        else
        {
            bActif = true;
        }

        groupe.setActif( bActif );

        _groupePolitiqueHome.update( groupe, plugin );

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * retourne une demande de confirmation pour la suppression du  groupe politique  choisi
     * @param request la requete HTTP
     * @return interface de confirmation de suppression
     */
    public String getSuppressionGroupe( HttpServletRequest request )
    {
        if ( request.getParameter( OdsParameters.ID_GROUPE ) == null )
        {
            return OdsConstants.CONSTANTE_CHAINE_VIDE;
        }

        String strIdGroupe = request.getParameter( OdsParameters.ID_GROUPE );
        UrlItem url = new UrlItem( JSP_URL_DO_SUPPRESSION_GROUPE_JSP );
        url.addParameter( OdsParameters.ID_GROUPE, strIdGroupe );

        return AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRMDELETEGROUPE, url.getUrl(  ),
            AdminMessage.TYPE_CONFIRMATION );
    }

    /**
     * supprime le groupe politique choisi
     * @param request la requete HTTP
     * @return url de gestion des groupes poliques
     */
    public String doSuppressionGroupe( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        if ( request.getParameter( OdsParameters.ID_GROUPE ) == null )
        {
            return OdsConstants.CONSTANTE_CHAINE_VIDE;
        }

        int nIdGroupe = OdsUtils.getIntegerParameter( OdsParameters.ID_GROUPE, request );

        GroupePolitique groupe = new GroupePolitique(  );
        groupe.setIdGroupe( nIdGroupe );

        try
        {
            _groupePolitiqueHome.remove( groupe, plugin );
        }
        catch ( AppException ae )
        {
            AppLogService.error( ae );

            if ( ae.getInitialException(  ) instanceof SQLException )
            {
                return AdminMessageService.getMessageUrl( request, MESSAGE_CANNOT_DELETE_GROUPE, AdminMessage.TYPE_STOP );
            }
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * Permet de rcuprer la liste des commissions
     * @return ReferenceList La liste des commissions disponibles
     */
    public ReferenceList initRefListGroupeDepositaire( boolean bIsGestionAval )
    {
        Plugin plugin = getPlugin(  );

        ReferenceList refListGroupeDepositaire = new ReferenceList(  );
        List<IGroupePolitique> groupePolitiques;

        if ( bIsGestionAval )
        {
            groupePolitiques = _groupePolitiqueHome.findGroupePolitiqueList( plugin );
        }
        else
        {
            groupePolitiques = _groupePolitiqueHome.findGroupesActifs( plugin );
        }

        // ajout dans la referenceList de l'item chaine vide
        refListGroupeDepositaire.addItem( OdsConstants.FILTER_ALL, OdsConstants.CONSTANTE_CHAINE_VIDE );

        // ajout dans la referenceList de l'ensemble des items
        // reprsentant les categories de deliberations
        for ( IGroupePolitique groupe : groupePolitiques )
        {
            refListGroupeDepositaire.addItem( groupe.getIdGroupe(  ), groupe.getNomGroupe(  ) );
        }

        return refListGroupeDepositaire;
    }

    /**
     * methode qui test si  dans le formulaire de saisi d'un groupe politique  ,l'ensemble des critres obligatoires ont t renseign
     * @param request  la requete HTTP
     * @return l'ensemble des critres obligatoires qui n'ont pas t renseign
     */
    private String requiredField( HttpServletRequest request )
    {
        String strField = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( ( null == request.getParameter( OdsParameters.NOM_GROUPE ) ) ||
                request.getParameter( OdsParameters.NOM_GROUPE ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strField = FIELD_NOM_GROUPE;
        }
        else if ( ( null == request.getParameter( OdsParameters.NOM_COMPLET ) ) ||
                request.getParameter( OdsParameters.NOM_COMPLET ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strField = FIELD_NOM_COMPLET;
        }

        return strField;
    }
}
