/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.groupepolitique;

import fr.paris.lutece.plugins.ods.service.IODSService;
import fr.paris.lutece.util.ReferenceList;

import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;


public interface IGroupePolitiqueService extends IODSService
{
    /**
     * Permet de rcuprer la liste des commissions
     * @return ReferenceList La liste des commissions disponibles
     */
    ReferenceList initRefListGroupeDepositaire( boolean bIsGestionAval );

    /**
    * retourne l'interface de gestion des groupes politiques.
    * @param request la requete HTTP
    * @return   interface de gestion des groupes politiques
    */
    HashMap<String, Object> getGroupeList( HttpServletRequest request );

    /**
     * cre un groupe politique.
     * si tous les champs obligatoires du formulaire de cration d'un groupe politique ont t renseign,
     * la mthode retourne l'url de gestion des groupes poliques,
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request request
     * @return url url de retour
     */
    String doCreationGroupe( HttpServletRequest request );

    /**
     * retourne le formulaire de modification d'un groupe politique  pr remplis avec  les critres  du groupe politique choisi
     * @param request la requete HTTP
     * @return formulaire de modification d'un groupe politique
     */
    HashMap<String, Object> getModificationGroupe( HttpServletRequest request );

    /**
     * Modifie un groupe politique.
     * si tous les champs obligatoires du formulaire de modification d'un groupe politique ont t renseign,
     * la mthode retourne l'url de gestion des groupes poliques,
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request request
     * @return url url de retour
     */
    String doModificationGroupe( HttpServletRequest request );

    /**
     * retourne une demande de confirmation pour la suppression du  groupe politique  choisi
     * @param request la requete HTTP
     * @return interface de confirmation de suppression
     */
    String getSuppressionGroupe( HttpServletRequest request );

    /**
     * supprime le groupe politique choisi
     * @param request la requete HTTP
     * @return url de gestion des groupes poliques
     */
    String doSuppressionGroupe( HttpServletRequest request );
}
