/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.horodatage;

import au.com.bytecode.opencsv.CSVWriter;

import fr.paris.lutece.plugins.ods.business.horodatage.HorodatageHome;
import fr.paris.lutece.plugins.ods.dto.horodatage.HorodatageActionEnum;
import fr.paris.lutece.plugins.ods.dto.horodatage.IEntiteHorodatage;
import fr.paris.lutece.plugins.ods.dto.horodatage.ITrace;
import fr.paris.lutece.plugins.ods.dto.horodatage.TraceNotification;
import fr.paris.lutece.plugins.ods.dto.horodatage.TracePublication;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.utilisateur.Utilisateur;
import fr.paris.lutece.plugins.ods.utils.business.HorodatageUtils;
import fr.paris.lutece.plugins.ods.utils.commons.DateUtils;
import fr.paris.lutece.plugins.ods.utils.commons.StringUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppLogService;

import org.springframework.beans.factory.annotation.Autowired;

import java.io.IOException;
import java.io.StringWriter;

import java.sql.Timestamp;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;


/**
 *
 * HorodatageService
 *
 */
public abstract class AbstractHorodatageService implements IHorodatageService
{
    private static final String MARK_DATE_START = "date_start";
    private static final String MARK_DATE_END = "date_end";
    private static final String MARK_TYPE_HORODATAGE = "type_horodatage";
    private static final String MESSAGE_ERROR_DATE = "ods.horodatage.message.errorDate";

    /* Variables de session */
    private Timestamp _dateStart;
    private Timestamp _dateEnd;
    private String _typeHorodatage;
    @Autowired
    private HorodatageHome _horodatageHome;
    private Map<String, IHorodatageServiceManaged> _mappingService;

    protected abstract Plugin getPlugin(  );

    public Map<String, IHorodatageServiceManaged> getMappingService(  )
    {
        return _mappingService;
    }

    public void setMappingService( Map<String, IHorodatageServiceManaged> mappingService )
    {
        this._mappingService = mappingService;
    }

    /**
     * Permet de tracer la publication ou la dpublication d'un pdd
     *
     * @param pdd PDD
     * @param horodatageAction HorodatageActionEnum
     * @param plugin le plugin actif
     * @return id qui a ete gnr
     */
    public int tracePublication( IEntiteHorodatage entiteHorodatage, HorodatageActionEnum horodatageAction )
    {
        Plugin plugin = getPlugin(  );

        String strTypeEntite = entiteHorodatage.getFamilleDocumentEnum(  );

        ISeance seance = _mappingService.get( strTypeEntite ).getSeance( entiteHorodatage );

        Timestamp dateSeance = null;

        if ( ( seance != null ) )
        {
            dateSeance = seance.getDateSeance(  );
        }

        String signature = _mappingService.get( strTypeEntite ).getSignature( entiteHorodatage );

        return _horodatageHome.createTracePublication( entiteHorodatage, dateSeance, horodatageAction, signature, plugin );
    }

    /**
     * Permet de tracer l'envoie de la notification
     *
     * @param dateEnvoi Date d'envoie de la notification
     * @param objet Objet de la notofication
     * @param contenu Contenu de la notification
     * @param user L'utilisateur qui envoie la notification
     * @param mails Liste contenant les mails des destinataires spares par des ;
     * @param typeNotif le type de notification (F = Fichier, P = PDD, R = Requetes personnelles)
     * @param plugin le plugin actif
     */
    public void traceNotification( Timestamp dateEnvoi, String objet, String contenu, Utilisateur user, String mails,
        String typeNotif, Plugin plugin )
    {
        String signature = HorodatageUtils.getSignature( contenu );
        String idDestinataire = user.getIdUtilisateur(  );
        int idTraceNotification = _horodatageHome.createTraceNotification( objet, contenu, signature, typeNotif, plugin );

        _horodatageHome.createTraceNotificationDestinaire( idTraceNotification, idDestinataire, dateEnvoi, mails, plugin );
    }

    /**
     * Permet de tracer l'envoi de la notification du message uniquement
     *
     * @param objet Objet de la notification
     * @param contenu Contenu de la notification
     * @param signature la signature md5 du contenu du message
     * @param typeNotif le type de notification (F = Fichier, P = PDD, R = Requetes personnelles)
     * @param plugin le plugin actif
     * @return l'identifiant du message notifi
     */
    public int traceNotificationMessageOnly( String objet, String contenu, String signature, String typeNotif,
        Plugin plugin )
    {
        return _horodatageHome.createTraceNotification( objet, contenu, signature, typeNotif, plugin );
    }

    /**
     * Permet de tracer l'envoi de la notification  l'utilisateur d'un message notifi
     *
     * @param idTraceNotification identifiant du message notifi
     * @param idDestinataire identifiant de l'utilisateur
     * @param dateEnvoi date d'envoi du message
     * @param mails List contenant les mails des destinataires
     * @param plugin le plugin actif
     */
    public void traceNotificationUtilisateur( int idTraceNotification, String idDestinataire, Timestamp dateEnvoi,
        String mails, Plugin plugin )
    {
        _horodatageHome.createTraceNotificationDestinaire( idTraceNotification, idDestinataire, dateEnvoi, mails, plugin );
    }

    /**
     * Retourne la date de na dernire notification
     * @param plugin plugin
     * @return la date de na dernire notification
     */
    public Timestamp getLastNotification( Plugin plugin )
    {
        return _horodatageHome.findLastNotification( plugin );
    }

    /**
     * Retourne l'interface de gestion de l'horodatage
     *
     * @param request le requte HTTP
     * @return l'interface de gestion des directions
     */
    public HashMap<String, Object> getHorodatage( HttpServletRequest request )
    {
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        model.put( MARK_DATE_START, _dateStart );
        model.put( MARK_DATE_END, _dateEnd );

        return model;
    }

    /**
     * Permet de renvoyer le flux de donnes du fichier a telecharger
     *
     * @param request HttpServletRequest
     * @return Le flux de donnes du fichier a telecharger
     */
    public String doDownloadCSV( HttpServletRequest request )
    {
        String dateStart = request.getParameter( MARK_DATE_START );
        String dateEnd = request.getParameter( MARK_DATE_END );
        _typeHorodatage = request.getParameter( MARK_TYPE_HORODATAGE );
        _dateStart = DateUtils.getDate( dateStart, true );
        _dateEnd = DateUtils.getDate( dateEnd, false );

        // Cas d'erreur
        if ( ( _dateStart == null ) || ( _dateEnd == null ) || ( _dateStart.after( _dateEnd ) ) )
        {
            return AdminMessageService.getMessageUrl( request, MESSAGE_ERROR_DATE, AdminMessage.TYPE_STOP );
        }

        if ( _typeHorodatage == null )
        {
            return null;
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * Permet de construire le fichier CSV
     *
     * @return Tableau de byte reprsentant le fichier  tlcharger
     */
    public byte[] getFichier(  )
    {
        Plugin plugin = getPlugin(  );
        List<ITrace> traces = _horodatageHome.findAllTrace( plugin, _typeHorodatage, _dateStart, _dateEnd );

        //on cr les titres des colonnes du CSV
        List<String[]> listToCSVWriter = new ArrayList<String[]>(  );
        HorodatageUtils.writeColonnes( listToCSVWriter, _typeHorodatage );

        for ( ITrace trace : traces )
        {
            List<String> listField = new ArrayList<String>(  );
            HorodatageUtils.writeTrace( listField, trace );

            if ( _typeHorodatage.equals( OdsConstants.CONSTANTE_PUBLICATION ) )
            {
                HorodatageUtils.writeTracePublication( listField, (TracePublication) trace );
            }
            else if ( _typeHorodatage.equals( OdsConstants.CONSTANTE_NOTIFICATION ) )
            {
                HorodatageUtils.writeTraceNotification( listField, (TraceNotification) trace );
            }

            //On insre la ligne nouvellement cr dans la liste des lignes du CSV
            listToCSVWriter.add( StringUtils.transformListToTab( listField ) );
        }

        String strCsvSeparator = OdsConstants.CONSTANTE_POINT_VIRGULE;
        StringWriter strWriter = new StringWriter(  );

        try
        {
            CSVWriter csvWriter = new CSVWriter( strWriter, strCsvSeparator.toCharArray(  )[0] );
            csvWriter.writeAll( listToCSVWriter );

            byte[] byteFileOutPut = strWriter.toString(  ).getBytes(  );

            csvWriter.close(  );
            strWriter.close(  );

            return byteFileOutPut;
        }
        catch ( IOException e )
        {
            AppLogService.error( e );

            return null;
        }
    }
}
