/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.indexer;

import fr.paris.lutece.plugins.ods.business.indexer.IndexerHome;
import fr.paris.lutece.plugins.ods.dto.indexer.Indexer;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.util.url.UrlItem;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;


public abstract class AbstractIndexerService implements IIndexerService
{
    private static final String PARAM_BOUTON_ACTION = "action";
    private static final String PARAM_ID_INDEXER = "id_indexer";
    private static final String MARK_LISTE_INDEXERS = "liste_indexers";
    private static final String JSP_DO_DEMANDE_INDEXATION = "jsp/admin/plugins/ods/indexer/DoDemandeIndexation.jsp";
    private static final String MESSAGE_CONFIRM_DEMANDE = "ods.demandeindexation.message.confirmDemande";
    private static final String MESSAGE_CONFIRM_ANNULATION = "ods.demandeindexation.message.confirmAnnulation";
    private static final String MESSAGE_DEMANDE = "ods.listeindexer.button.demande";
    @Autowired
    private IndexerHome _indexerHome;

    protected abstract Plugin getPlugin(  );

    /**
     * Retourne la page de gestion des indexers
     * @param request la requte Http
     * @return la liste des indexers
     */
    public HashMap<String, Object> getListeIndexer( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        model.put( MARK_LISTE_INDEXERS, _indexerHome.findAllIndexers( plugin ) );

        return model;
    }

    /**
     * Retourne la page de confirmation de demande d'indexation
     * @param request la requte Http
     * @return un message de confirmation
     */
    public String getDemandeIndexation( HttpServletRequest request )
    {
        String strAction = request.getParameter( PARAM_BOUTON_ACTION );
        String strIdIndexer = request.getParameter( PARAM_ID_INDEXER );
        UrlItem url = new UrlItem( JSP_DO_DEMANDE_INDEXATION );
        url.addParameter( PARAM_ID_INDEXER, strIdIndexer );
        url.addParameter( PARAM_BOUTON_ACTION, strAction );

        String strConfirm;

        if ( strAction.equals( I18nService.getLocalizedString( MESSAGE_DEMANDE, request.getLocale(  ) ) ) )
        {
            strConfirm = MESSAGE_CONFIRM_DEMANDE;
        }
        else
        {
            strConfirm = MESSAGE_CONFIRM_ANNULATION;
        }

        return AdminMessageService.getMessageUrl( request, strConfirm, url.getUrl(  ), AdminMessage.TYPE_CONFIRMATION );
    }

    /**
     * Demande ou annule la demande d'indexation complte
     * @param request la erqute Http
     * @return la page de gestion des inedexers
     */
    public void doDemandeIndexation( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        String strAction = request.getParameter( PARAM_BOUTON_ACTION );
        String strIdIndexer = request.getParameter( PARAM_ID_INDEXER );
        int nIdIndexer;

        try
        {
            nIdIndexer = Integer.parseInt( strIdIndexer );
        }
        catch ( NumberFormatException nfe )
        {
            AppLogService.error( nfe );

            return;
        }

        Indexer indexer = _indexerHome.findByPrimaryKey( nIdIndexer, plugin );

        if ( strAction.equals( I18nService.getLocalizedString( MESSAGE_DEMANDE, request.getLocale(  ) ) ) )
        {
            indexer.setIndexationComplete( true );
        }
        else
        {
            indexer.setIndexationComplete( false );
        }

        _indexerHome.update( indexer, plugin );
    }
}
