/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.listemembrepresent;

import fr.paris.lutece.plugins.ods.business.acte.ActeHome;
import fr.paris.lutece.plugins.ods.business.acte.NatureActeHome;
import fr.paris.lutece.plugins.ods.business.acte.StatutActeHome;
import fr.paris.lutece.plugins.ods.business.autremodele.AutreModeleHome;
import fr.paris.lutece.plugins.ods.business.categoriedeliberation.ICategorieDeliberationHome;
import fr.paris.lutece.plugins.ods.business.elu.IEluHome;
import fr.paris.lutece.plugins.ods.business.fichier.FichierPhysiqueHome;
import fr.paris.lutece.plugins.ods.business.fichier.IFichierHome;
import fr.paris.lutece.plugins.ods.business.formationconseil.FormationConseilHome;
import fr.paris.lutece.plugins.ods.business.historique.HistoriqueHome;
import fr.paris.lutece.plugins.ods.business.listemembrepresent.IListeMembrePresentHome;
import fr.paris.lutece.plugins.ods.business.ordredujour.generated.entete.Entete;
import fr.paris.lutece.plugins.ods.business.ordredujour.generated.pieddepage.PiedDePage;
import fr.paris.lutece.plugins.ods.business.seance.ISeanceHome;
import fr.paris.lutece.plugins.ods.dto.IBusinessItem;
import fr.paris.lutece.plugins.ods.dto.acte.Acte;
import fr.paris.lutece.plugins.ods.dto.acte.IActeFilter;
import fr.paris.lutece.plugins.ods.dto.acte.NatureActe;
import fr.paris.lutece.plugins.ods.dto.acte.NatureActeEnum;
import fr.paris.lutece.plugins.ods.dto.acte.StatutActe;
import fr.paris.lutece.plugins.ods.dto.acte.StatutActeEnum;
import fr.paris.lutece.plugins.ods.dto.autremodele.AutreModeleFilter;
import fr.paris.lutece.plugins.ods.dto.autremodele.IAutreModele;
import fr.paris.lutece.plugins.ods.dto.autremodele.TypeAutreModeleEnum;
import fr.paris.lutece.plugins.ods.dto.categoriedeliberation.CategorieDeliberation;
import fr.paris.lutece.plugins.ods.dto.certificataffichage.IEntiteCertificatAffichage;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.FichierPhysique;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter;
import fr.paris.lutece.plugins.ods.dto.fichier.ITypeDocument;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.historique.Historique;
import fr.paris.lutece.plugins.ods.dto.horodatage.HorodatageActionEnum;
import fr.paris.lutece.plugins.ods.dto.horodatage.IEntiteHorodatage;
import fr.paris.lutece.plugins.ods.dto.listemembrepresent.ListeMembrePresent;
import fr.paris.lutece.plugins.ods.dto.listemembrepresent.ListeMembrePresentFilter;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur;
import fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.typeentite.ITypeEntite;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.service.acte.IActeService;
import fr.paris.lutece.plugins.ods.service.categoriedeliberation.ICategorieDeliberationService;
import fr.paris.lutece.plugins.ods.service.certificataffichage.ICertificatAffichageServiceManaged;
import fr.paris.lutece.plugins.ods.service.direction.IDirectionService;
import fr.paris.lutece.plugins.ods.service.formationconseil.IFormationConseilService;
import fr.paris.lutece.plugins.ods.service.horodatage.IHorodatageService;
import fr.paris.lutece.plugins.ods.service.horodatage.IHorodatageServiceManaged;
import fr.paris.lutece.plugins.ods.service.manager.DocumentManager;
import fr.paris.lutece.plugins.ods.service.search.ISearchService;
import fr.paris.lutece.plugins.ods.utils.business.ModeleUtils;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.commons.DateUtils;
import fr.paris.lutece.plugins.ods.utils.commons.ShaUtils;
import fr.paris.lutece.plugins.ods.utils.commons.StringUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.service.admin.AccessDeniedException;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.service.util.AppPathService;
import fr.paris.lutece.portal.service.util.AppPropertiesService;
import fr.paris.lutece.util.url.UrlItem;

import net.sf.jcopist.service.exception.ServerException;
import net.sf.jcopist.service.exception.UnavailableException;

import org.springframework.beans.factory.annotation.Autowired;

import java.io.IOException;

import java.sql.Timestamp;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;

import javax.servlet.http.HttpServletRequest;


public abstract class AbstractListeMembrePresentService<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>, GActeFilter extends IActeFilter<GFichier, GSeance>, GElu extends IElu, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GSeanceFilter extends ISeanceFilter, GRequeteUtilisateur extends IRequeteUtilisateur<GSeance, GSeanceFilter>, GResultatRequete extends IResultatRequete<GSeance, GSeanceFilter, GRequeteUtilisateur, GElu, GFichier, GVoeuAmendement, GPDD>>
    implements IListeMembrePresentService<GSeance, GFichier>, IHorodatageServiceManaged,
        ICertificatAffichageServiceManaged<GSeance, GFichier>
{
    protected static final String JSP_URL_GET_MODIFICATION_MEMBRE_FOR_ACTE = "jsp/admin/plugins/ods/listemembrepresent/ModificationListeMembrePresentForActe.jsp";
    protected static final String JSP_URL_LISTE_MEMBRE_PRESENT = "jsp/admin/plugins/ods/listemembrepresent/AdminListeMembrePresentNoReinit.jsp";
    protected static final String JSP_URL_GET_VISUALISATION_MEMBRE_FOR_ACTE = "jsp/admin/plugins/ods/listemembrepresent/VisualisationListeMembrePresentForActe.jsp";
    protected static final String JSP_URL_DO_SUPPRESSION_LISTE_MEMBRE_PRESENT = "jsp/admin/plugins/ods/listemembrepresent/DoSuppressionListeMembrePresentGestionAval.jsp";
    protected static final String JSP_URL_GET_MODIFICATION_LISTE_MEMBRE_PRESENT = "jsp/admin/plugins/ods/listemembrepresent/ModificationListeMembrePresentGestionAval.jsp";
    protected static final String JSP_URL_FINALISATION_ACTE = "jsp/admin/plugins/ods/listemembrepresent/FinalisationActe.jsp";
    protected static final String PROPERTY_LABEL_SEANCE = "ods.creationlistemembrepresent.label.seance";
    protected static final String PROPERTY_LABEL_INTITULE = "ods.creationlistemembrepresent.label.intitule";
    protected static final String PROPERTY_LABEL_FORMATION_CONSEIL = "ods.creationlistemembrepresent.label.formationConseil";
    protected static final String PROPERTY_NOM_FICHIER_PDF = "ods.listemembrepresent.fichierPDF.nom";
    protected static final String PROPERTY_MESSAGE_NO_MODELE = "ods.listemembrepresent.error.noModele";
    protected static final String PROPERTY_MESSAGE_NO_ENTETE_PIED = "ods.listemembrepresent.error.noEntetePied";
    protected static final String PROPERTY_LABEL_DU = "ods.listemembrepresent.entete.label.du";
    protected static final String PROPERTY_LABEL_DES = "ods.listemembrepresent.entete.label.des";
    protected static final String PROPERTY_LABEL_ET = "ods.listemembrepresent.entete.label.et";
    protected static final String PROPERTY_LABEL_A = "ods.listemembrepresent.entete.label.a";
    protected static final String PROPERTY_LABEL_LE = "ods.listemembrepresent.entete.label.le";
    protected static final String PROPERTY_LABEL_LES = "ods.listemembrepresent.entete.label.les";
    protected static final String PROPERTY_LABEL_DATE_TRANSMISSION_CTRL_LEGALITE = "ods.creationlistemembrepresent.label.dateTransmissionControleLegalite";
    protected static final String MARK_LISTE_HISTORIQUES = "liste_historiques";
    protected static final String MARK_LISTE_MEMBRE_PRESENT_SIZE_TOTAL = "liste_membre_present_size_total";
    protected static final String MARK_LISTE_MEMBRE_PRESENT = "liste_membre_present";
    protected static final String MARK_LISTE_ELUS = "liste_elus";
    protected static final String MARK_LISTE_ELUS_PRESENTS = "liste_elus_presents";
    protected static final String MARK_LISTE_ELUS_SIZE_TOTAL = "liste_elus_size_total";
    protected static final String MARK_LISTE_SEANCES = "liste_seance";
    protected static final String MARK_ID_SEANCE_SELECTED = "id_seance_selected";
    protected static final String MARK_DATE_TRANSMISSION_DEFAUT = "date_transmission_defaut";
    protected static final String MARK_LISTE_CATEGORIE_DELIBERATION = "liste_categorie_deliberation";
    protected static final String MARK_ID_ACTE = "id_acte";
    protected static final String MARK_ACTE = "acte";
    protected static final String MARK_ID_CATEGORIE_DELIBERATION_SELECTED = "id_categorie_deliberation_selected";
    protected static final String MARK_LISTE_NATURE_DOCUMENT = "liste_nature_document";
    protected static final String MARK_LISTE_DIRECTION = "liste_direction";
    protected static final String MARK_STATUT_ACTE_SELECTED = "statut_acte_selected";
    protected static final String MESSAGE_CONFIRM_DELETE_LISTE_MEMBRE_PRESENT = "ods.message.confirmDeleteListeMembresPresents";
    protected static final String MESSAGE_CONFIRM_DELETE_LISTE_MEMBRE_PRESENT_WITH_ACTES_ANNULES = "ods.message.confirmDeleteListeMembresPresentsWithActes";
    protected static final String MESSAGE_LISTE_ALREADY_EXIST = "ods.listemembrepresent.message.listeAlreadyExist";
    protected static final String MESSAGE_PROBLEME_CONVERSION_JCOPIST = "ods.odj.message.problemJCopist";
    protected static final String MESSAGE_ERROR_DELETE_LISTE_MEMBRE_PRESENT = "ods.message.errorDeleteListeMembresPresents";
    protected static final String XML_LISTE_MEMBRE_PRESENT_OPEN = "<MembrePresent>";
    protected static final String XML_LISTE_MEMBRE_PRESENT_CLOSE = "</MembrePresent>";
    protected static final String XML_LISTE_ELUS_EXCUSES_OPEN = "<listeElusExcuses>";
    protected static final String XML_LISTE_ELUS_EXCUSES_CLOSE = "</listeElusExcuses>";
    protected static final String PROPERTY_CODE_DELIBERATION_DEFAUT = "34";
    protected static final int DEFAULT_CODE_DELIBERATION = 34;
    protected static final String CONSTANTE_PREPARER_ACTE = "preparer_acte";
    protected static final String CONSTANTE_LISTE_MEMBRES_PRESENTS = "liste_membres_presents";

    // Variable de session pour la sance
    protected int _nIdSeanceSelected = -1;
    protected int _nIdFormationConseilSelected = -1;
    protected int _nStatutActeSelected = -1;

    //gre si il ya eu des changements dans un une liste de membres prsents avant enregistrement
    protected boolean _bHasChanged;

    //variable de session permettant de savoir si le retour ne doit pas ce faire sur l'interface de gestion des vas 
    protected boolean _bUrlRetour;

    //variable de session stockant l'url de retour si le retour ne ce fait pas sur l'interface de gestion des vas 
    protected String _strUrlRetour;

    //variable de session permettant de savoir si le retour se fait sur la gestion des listes (0) ou sur le suivi des actes (1)
    protected int _nTypeUrlRetour;

    //variable de session permettant de savoir si l'on vient de la gestion des actes
    protected boolean _bComeFromActe;
    @Autowired
    protected IHorodatageService _horodatageService;
    @Autowired
    protected ISearchService<GSeance, GSeanceFilter, GRequeteUtilisateur, GResultatRequete, GElu, GFichier, GVoeuAmendement, GPDD> _searchService;
    @Autowired
    protected IListeMembrePresentHome<GSeance, GFichier, GActeFilter> _listeMembrePresentHome;
    @Autowired
    protected ISeanceHome<GSeance, ISeanceFilter, GFichier, GElu, GVoeuAmendement, GPDD> _seanceHome;
    @Autowired
    protected IActeService<GSeance, GFichier, GActeFilter> _acteService;
    @Autowired
    protected IFichierHome<GFichier, IFichierFilter, ITypeDocument, GSeance> _fichierHome;
    @Autowired
    protected HistoriqueHome _historiqueHome;
    @Autowired
    protected IEluHome<IElu> _eluHome;
    @Autowired
    protected ActeHome<GSeance, GFichier, GActeFilter> _acteHome;
    @Autowired
    protected StatutActeHome _statutActeHome;
    @Autowired
    protected NatureActeHome _natureActeHome;
    @Autowired
    protected FichierPhysiqueHome _fichierPhysiqueHome;
    @Autowired
    protected AutreModeleHome<IAutreModele> _autreModeleHome;
    @Autowired
    protected ICategorieDeliberationHome _categorieDeliberationHome;
    @Autowired
    protected ICategorieDeliberationService _categorieDeliberationService;
    @Autowired
    protected IDirectionService _directionService;
    @Autowired
    protected IFormationConseilService _formationConseilService;
    @Autowired
    protected FormationConseilHome _formationConseilHome;

    protected abstract GFichier newFichierInstance(  );

    protected abstract String getParameterPluginName(  );

    protected abstract GSeance newSeanceInstance(  );

    protected abstract void getModificationListeMembrePresentSpecEnd( HashMap<String, Object> model,
        HttpServletRequest request );

    protected abstract void getModificationListeMembrePresentSpec( HttpServletRequest request,
        ListeMembrePresent<GSeance, GFichier> liste );

    protected abstract void getAdminListeMembrePresentSpec( HashMap<String, Object> model, HttpServletRequest request );

    protected abstract void getCreationListeMembrePresentSpec( HashMap<String, Object> model, HttpServletRequest request );

    protected abstract int getFormationConseil( HttpServletRequest request );

    protected abstract void doGenerePDFEnteteSpec( HashMap<Integer, Object> xmlParamsEntete,
        ListeMembrePresent<GSeance, GFichier> listeMembrePresent, HttpServletRequest request );

    protected abstract Plugin getPlugin(  );

    /**
     * Reinitialise tous les attributs de session
     * @param request requete HTTP
     * @throws AccessDeniedException   AccessDeniedException
     */
    public void reinitSession( HttpServletRequest request )
        throws AccessDeniedException
    {
        Plugin plugin = getPlugin(  );

        _bHasChanged = false;
        _nIdFormationConseilSelected = -1;

        ISeance seance = _seanceHome.getDerniereSeance( plugin );

        if ( seance != null )
        {
            _nIdSeanceSelected = seance.getIdSeance(  );
        }

        _nStatutActeSelected = -1;
        _bUrlRetour = false;
        _strUrlRetour = null;
        _nTypeUrlRetour = -1;
        _bComeFromActe = false;
    }

    /**
     * Permet d'initialiser l'url de retour
     * @param strUrl url de retour
     * @param request la requte http
     */
    public void initUrlRetour( String strUrl, HttpServletRequest request )
    {
        _strUrlRetour = strUrl;
        _bUrlRetour = true;

        if ( _strUrlRetour.equals( _acteService.getUrlGestionActe(  ) ) )
        {
            _nTypeUrlRetour = 1;
        }
        else
        {
            _nTypeUrlRetour = 0;
        }

        _strUrlRetour += OdsConstants.CONSTANTE_POINT_INTERROGATION;

        if ( request.getParameter( OdsParameters.ID_LISTE_MEMBRE_PRESENT ) != null )
        {
            _strUrlRetour += ( OdsParameters.ID_LISTE_MEMBRE_PRESENT + OdsConstants.CONSTANTE_EGAL +
            request.getParameter( OdsParameters.ID_LISTE_MEMBRE_PRESENT ) + OdsConstants.CONSTANTE_ET );
        }

        _strUrlRetour += getParameterPluginName(  );
    }

    public void comeFromActe(  )
    {
        _bComeFromActe = true;
    }

    /**
     * Retourne l'interface de gestion de la liste des membres prsents.
     * @param request la requte http
     * @return la page affichant la liste des membres prsents
     */
    public HashMap<String, Object> getAdminListeMembrePresent( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        if ( ( request.getParameter( OdsParameters.ID_SEANCE ) != null ) )
        {
            _nIdSeanceSelected = OdsUtils.getIntegerParameter( OdsParameters.ID_SEANCE, request );
        }

        int nIdFormationConseil = getFormationConseil( request );

        if ( nIdFormationConseil != -1 )
        {
            _nIdFormationConseilSelected = nIdFormationConseil;
        }

        if ( request.getParameter( OdsParameters.STATUT_ACTE ) != null )
        {
            _nStatutActeSelected = OdsUtils.getIntegerParameter( OdsParameters.STATUT_ACTE, request );
        }

        ListeMembrePresentFilter filter = new ListeMembrePresentFilter(  );
        filter.setIdSeance( _nIdSeanceSelected );
        filter.setIdFormationConseil( _nIdFormationConseilSelected );
        filter.setStatutActe( _nStatutActeSelected );

        List<ListeMembrePresent<GSeance, GFichier>> listeMembrePresent = _listeMembrePresentHome.findByFilter( filter,
                plugin );

        // Ajout des lments ncessaire dans la Hashmap
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        model.put( MARK_LISTE_SEANCES, _seanceHome.findOldSeance( plugin ) );
        model.put( MARK_ID_SEANCE_SELECTED, _nIdSeanceSelected );

        // Rcupration de l'identifiant de la sance  partir de laquelle les crations d'acte sont autorises
        int nIdFirstSeanceForActe = AppPropertiesService.getPropertyInt( OdsProperties.PROPERTY_CREATION_ID_SEANCE, 1 );
        model.put( OdsMarks.MARK_ACTE_CREATION_ID_SEANCE, nIdFirstSeanceForActe );

        model.put( MARK_LISTE_MEMBRE_PRESENT_SIZE_TOTAL, listeMembrePresent.size(  ) );
        model.put( MARK_LISTE_MEMBRE_PRESENT, listeMembrePresent );
        model.put( MARK_STATUT_ACTE_SELECTED, _nStatutActeSelected );

        getAdminListeMembrePresentSpec( model, request );

        return model;
    }

    /**
     * Publication ou dpublication  d'une liste de membre prsent
     *
     * @param request la requte http
     * @return l'url
     */
    public String doChangementEtatListeMembrePresent( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        Timestamp dateForVersion = DateUtils.getCurrentDate(  );
        int nIdListeMembrePresent = OdsUtils.getIntegerParameter( OdsParameters.ID_LISTE_MEMBRE_PRESENT, request );
        String strIsPublier = request.getParameter( OdsParameters.DELIBERATION_IS_PUBLIER );
        boolean bEstPublication = Boolean.parseBoolean( OdsConstants.CONSTANTE_CHAINE_VIDE + strIsPublier );

        if ( nIdListeMembrePresent != -1 )
        {
            ListeMembrePresent<GSeance, GFichier> listeMembrePresent = _listeMembrePresentHome.findByPrimaryKey( nIdListeMembrePresent,
                    plugin );
            OdsUtils.testNullObject( listeMembrePresent, request );

            if ( !( ( listeMembrePresent.getEnLigne(  ) && bEstPublication ) ||
                    ( !listeMembrePresent.getEnLigne(  ) && !bEstPublication ) ) )
            {
                if ( bEstPublication && ( listeMembrePresent.getVersion(  ) < 1 ) )
                {
                    listeMembrePresent.setVersion( 0 );
                }

                // On publie la liste des membres prsents ainsi que le fichier associ
                _listeMembrePresentHome.publication( listeMembrePresent.getId(  ), dateForVersion,
                    listeMembrePresent.getVersion(  ), bEstPublication, plugin );

                // On publie les fichiers associs
                _fichierHome.publication( listeMembrePresent.getFichierGenere(  ).getId(  ), dateForVersion,
                    listeMembrePresent.getVersion(  ), bEstPublication, plugin );

                if ( bEstPublication )
                {
                    Historique historique = new Historique(  );
                    historique.setVersion( listeMembrePresent.getVersion(  ) + 1 );
                    historique.setDatePublication( dateForVersion );
                    historique.setIdEntite( listeMembrePresent.getId(  ) );
                    historique.setIdTypeEntite( ITypeEntite.LISTE_DES_MEMBRES_PRESENTS );
                    _historiqueHome.create( historique, plugin );
                }

                // On horodate la (de)publication
                HorodatageActionEnum action = ( bEstPublication ) ? HorodatageActionEnum.PUBLICATION
                                                                  : HorodatageActionEnum.DE_PUBLICATION;
                _horodatageService.tracePublication( listeMembrePresent, action );
            }
        }
        else
        {
            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                AdminMessage.TYPE_STOP );
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * retourne le formulaire de cration d'une liste de membre prsent
     * @param request la requte http
     * @return la page de cration d'une liste de membre prsent
     */
    public HashMap<String, Object> getCreationListeMembrePresent( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        HashMap<String, Object> model = new HashMap<String, Object>(  );
        List<IElu> listElus = _eluHome.findElusActifs( plugin );

        Timestamp tsDateTransmissionDefaut = _seanceHome.findByPrimaryKey( _nIdSeanceSelected, plugin ).getDateSeance(  );

        model.put( MARK_LISTE_SEANCES, _seanceHome.findOldSeance( plugin ) );
        model.put( MARK_ID_SEANCE_SELECTED, _nIdSeanceSelected );
        model.put( MARK_DATE_TRANSMISSION_DEFAUT, tsDateTransmissionDefaut );
        model.put( MARK_LISTE_ELUS_SIZE_TOTAL, listElus.size(  ) );
        model.put( MARK_LISTE_ELUS, listElus );

        getCreationListeMembrePresentSpec( model, request );

        return model;
    }

    /**
     * Permet de confirmer la finalisation d'un acte
     *
     * @param request
     *            la requete HTTP
     * @return strUrl url de retour
     */
    public String doFinaliserActe( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        int nIdListeMembre = OdsUtils.getIntegerParameter( OdsParameters.ID_LISTE_MEMBRE_PRESENT, request );
        ListeMembrePresent<GSeance, GFichier> listeMembrePresent = _listeMembrePresentHome.findByPrimaryKey( nIdListeMembre,
                plugin );
        OdsUtils.testNullObject( listeMembrePresent, request );

        String strReturn;

        Acte<GSeance, GFichier> acteCourant = listeMembrePresent.getActeCourant(  );

        // Si on a cliqu sur le bouton "Annuler"
        if ( request.getParameter( OdsParameters.CONSTANTE_ACTION_CANCEL ) != null )
        {
            if ( _nTypeUrlRetour == 1 )
            {
                strReturn = AppPathService.getBaseUrl( request ) + JSP_URL_GET_MODIFICATION_MEMBRE_FOR_ACTE +
                    OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_LISTE_MEMBRE_PRESENT +
                    OdsConstants.CONSTANTE_EGAL + nIdListeMembre;
            }
            else
            {
                strReturn = AppPathService.getBaseUrl( request ) + JSP_URL_GET_MODIFICATION_LISTE_MEMBRE_PRESENT +
                    OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_LISTE_MEMBRE_PRESENT +
                    OdsConstants.CONSTANTE_EGAL + nIdListeMembre + OdsConstants.CONSTANTE_ET + CONSTANTE_PREPARER_ACTE +
                    OdsConstants.CONSTANTE_EGAL + CONSTANTE_PREPARER_ACTE;
            }
        }
        else
        {
            String strActe = request.getParameter( OdsParameters.CHECKBOX_DELETE );

            if ( strActe != null )
            {
                acteCourant.setStatutActe( StatutActeEnum.A_TRANSMETTRE.getStatutOnlyWidthId(  ) );
                acteCourant.setIdTransmission( 0 );
                _acteHome.update( acteCourant, plugin );

                if ( _bUrlRetour )
                {
                    strReturn = AppPathService.getBaseUrl( request ) + _strUrlRetour;
                    _bUrlRetour = false;
                    _strUrlRetour = null;
                }
                else
                {
                    strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;
                }
            }
            else
            {
                strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_CHECKBOX,
                        AdminMessage.TYPE_STOP );
            }
        }

        return strReturn;
    }

    /**
     * retourne le formulaire de confirmation d'envoi manuel de l'acte
     * @param request la requte http
     * @return la page de confirmation
     */
    public HashMap<String, Object> getConfirmFinaliser( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        int nIdListeMembre = OdsUtils.getIntegerParameter( OdsParameters.ID_LISTE_MEMBRE_PRESENT, request );
        ListeMembrePresent<GSeance, GFichier> listeMembrePresent = _listeMembrePresentHome.findByPrimaryKey( nIdListeMembre,
                plugin );

        model.put( MARK_LISTE_MEMBRE_PRESENT, listeMembrePresent );

        return model;
    }

    /**
     * retourne le formulaire de modificationd'une liste de membre prsent
     * @param request la requte http
     * @param
     * @return la page de modification d'une liste de membre prsent
     */
    public HashMap<String, Object> getModificationListeMembrePresent( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        int nIdListeMembre = OdsUtils.getIntegerParameter( OdsParameters.ID_LISTE_MEMBRE_PRESENT, request );
        ListeMembrePresent<GSeance, GFichier> listeMembrePresent = _listeMembrePresentHome.findByPrimaryKey( nIdListeMembre,
                plugin );
        OdsUtils.testNullObject( listeMembrePresent, request );

        List<IElu> listElus = _eluHome.findElusActifs( plugin );

        getModificationListeMembrePresentSpec( request, listeMembrePresent );

        if ( request.getParameter( OdsParameters.PREPARER_ACTE ) != null )
        {
            Acte<GSeance, GFichier> acteCourant = new Acte<GSeance, GFichier>(  );

            acteCourant.setIdTypeEntite( ITypeEntite.LISTE_DES_MEMBRES_PRESENTS );

            NatureActe natureActe = new NatureActe(  );
            natureActe.setIdNatureDocument( NatureActeEnum.COMMUNICATION.getId(  ) );
            acteCourant.setNatureDocument( natureActe );

            StatutActe statut = _statutActeHome.findByPrimaryKey( StatutActeEnum.A_PREPARER.getId(  ), plugin );
            acteCourant.setStatutActe( statut );
            acteCourant.setReference( listeMembrePresent.getIntitule(  ) );

            CategorieDeliberation categorie = _categorieDeliberationHome.findCategorieByCode( DEFAULT_CODE_DELIBERATION,
                    plugin );
            acteCourant.setCategorie( categorie );

            acteCourant.setSeance( listeMembrePresent.getSeance(  ) );
            acteCourant.setFormationConseil( listeMembrePresent.getFormationConseil(  ) );
            acteCourant.setIdEntite( listeMembrePresent.getId(  ) );
            acteCourant.setDeliberationFinale( listeMembrePresent.getFichierGenere(  ) );
            acteCourant.setPoids( acteCourant.getDeliberationFinale(  ).getTaille(  ) );

            int nMaxPoids = AppPropertiesService.getPropertyInt( OdsProperties.PROPERTY_POIDS_MAX_ACTE,
                    OdsConstants.DEFAULT_POIDS_MAX_ACTE );

            if ( acteCourant.getPoids(  ) > nMaxPoids )
            {
                acteCourant.setIdTransmission( 0 );
            }
            else
            {
                acteCourant.setIdTransmission( 1 );
            }

            listeMembrePresent.setActeCourant( acteCourant );
        }

        HashMap<String, Object> model = new HashMap<String, Object>(  );
        model.put( MARK_LISTE_SEANCES, _seanceHome.findOldSeance( plugin ) );

        // Rcupration de l'identifiant de la sance  partir de laquelle les crations d'acte sont autorises
        int nIdFirstSeanceForActe = AppPropertiesService.getPropertyInt( OdsProperties.PROPERTY_CREATION_ID_SEANCE, 1 );
        model.put( OdsMarks.MARK_ACTE_CREATION_ID_SEANCE, nIdFirstSeanceForActe );

        model.put( MARK_LISTE_MEMBRE_PRESENT, listeMembrePresent );
        model.put( MARK_LISTE_ELUS_SIZE_TOTAL, listElus.size(  ) );
        model.put( MARK_LISTE_ELUS, listElus );
        model.put( MARK_LISTE_ELUS_PRESENTS, _listeMembrePresentHome.getListeIdsElusPresents( nIdListeMembre, plugin ) );
        model.put( MARK_LISTE_CATEGORIE_DELIBERATION,
            _categorieDeliberationService.initRefListCategorieDeliberation( plugin, false ) );

        int nCodeCategorieDefaut = AppPropertiesService.getPropertyInt( PROPERTY_CODE_DELIBERATION_DEFAUT,
                DEFAULT_CODE_DELIBERATION );
        int nIdCategorieDefaut = _categorieDeliberationHome.findIdCategorieByCode( nCodeCategorieDefaut, plugin );
        model.put( MARK_ID_CATEGORIE_DELIBERATION_SELECTED, nIdCategorieDefaut );
        model.put( MARK_LISTE_DIRECTION, _directionService.initRefListDirection( true, false ) );

        // Rcupration des natures de document       
        model.put( MARK_LISTE_NATURE_DOCUMENT, _acteService.getRefListNatureActe(  ) );

        // Si l'on a cliqu sur visualiser l'acte  partir de la liste de suivi des actes
        int nIdActe = OdsUtils.getIntegerParameter( MARK_ID_ACTE, request );

        if ( nIdActe != -1 )
        {
            model.put( MARK_ID_ACTE, nIdActe );

            Acte<GSeance, GFichier> acte = _acteHome.findByPrimaryKey( nIdActe, plugin );
            model.put( MARK_ACTE, acte );
        }

        getModificationListeMembrePresentSpecEnd( model, request );

        return model;
    }

    /**
     * Retourne true si la liste des membres prsents est publie, false sinon
     * @param nIdListeMembrePresent l'id de la liste des membres prsents
     * @return true si la liste des membres prsents est publie, false sinon
     */
    public boolean isPublier( int nIdListeMembrePresent )
    {
        Plugin plugin = getPlugin(  );

        ListeMembrePresent<GSeance, GFichier> listeMembrePresent = _listeMembrePresentHome.findByPrimaryKey( nIdListeMembrePresent,
                plugin );

        return listeMembrePresent.getEnLigne(  );
    }

    /**
     * retourne l'ecran de visualisation de l'historique des versions d'une liste de membre prsent
     * @param request la requte http
     * @return la page de l'historique des membres prsent
     */
    public HashMap<String, Object> getHistoriqueListeMembrePresent( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        int nIdListeMembre = OdsUtils.getIntegerParameter( OdsParameters.ID_LISTE_MEMBRE_PRESENT, request );

        ListeMembrePresent<GSeance, GFichier> listeMembrePresent = _listeMembrePresentHome.findByPrimaryKey( nIdListeMembre,
                plugin );
        OdsUtils.testNullObject( listeMembrePresent, request );

        List<Historique> historiques = _historiqueHome.getHistoriqueList( listeMembrePresent, plugin );

        HashMap<String, Object> model = new HashMap<String, Object>(  );
        model.put( MARK_LISTE_HISTORIQUES, historiques );

        return model;
    }

    /**
     * retourne une demande de confirmation pour la suppression de la liste des membres prsents
     * @param request la requte http
     * @return la page de confirmation de suppression de la liste des membres prsents
     */
    public String getSuppressionListeMembrePresent( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        // On vrifie qu'il n'y a pas d'actes rattachs  la liste de membres
        int nIdListeMembrePresent = OdsUtils.getIntegerParameter( OdsParameters.ID_LISTE_MEMBRE_PRESENT, request );
        ListeMembrePresent<GSeance, GFichier> listeMembrePresent = _listeMembrePresentHome.findByPrimaryKey( nIdListeMembrePresent,
                plugin );

        String strIdListeMembrePresent = request.getParameter( OdsParameters.ID_LISTE_MEMBRE_PRESENT );
        UrlItem url = new UrlItem( JSP_URL_DO_SUPPRESSION_LISTE_MEMBRE_PRESENT );
        url.addParameter( OdsParameters.ID_LISTE_MEMBRE_PRESENT, strIdListeMembrePresent );

        if ( listeMembrePresent.getActeCourant(  ) != null )
        {
            return AdminMessageService.getMessageUrl( request, MESSAGE_ERROR_DELETE_LISTE_MEMBRE_PRESENT,
                AdminMessage.TYPE_STOP );
        }
        else if ( ( listeMembrePresent.getActes(  ) != null ) && !listeMembrePresent.getActes(  ).isEmpty(  ) )
        {
            return AdminMessageService.getMessageUrl( request,
                MESSAGE_CONFIRM_DELETE_LISTE_MEMBRE_PRESENT_WITH_ACTES_ANNULES, url.getUrl(  ),
                AdminMessage.TYPE_CONFIRMATION );
        }

        return AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRM_DELETE_LISTE_MEMBRE_PRESENT, url.getUrl(  ),
            AdminMessage.TYPE_CONFIRMATION );
    }

    /**
     * supprime la liste de membre prsent choisie
     * @param request la requte http
     * @return l'url de suppression de la dlibration de dsignation
     */
    public String doSuppressionListeMembrePresent( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        int nIdListeMembre = OdsUtils.getIntegerParameter( OdsParameters.ID_LISTE_MEMBRE_PRESENT, request );
        ListeMembrePresent<GSeance, GFichier> listeMembrePresent = _listeMembrePresentHome.findByPrimaryKey( nIdListeMembre,
                plugin );

        if ( listeMembrePresent == null )
        {
            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                AdminMessage.TYPE_ERROR );
        }

        GFichier fichierGenere = listeMembrePresent.getFichierGenere(  );
        _listeMembrePresentHome.removeAllElus( nIdListeMembre, plugin );

        _historiqueHome.remove( listeMembrePresent, plugin );
        _listeMembrePresentHome.remove( nIdListeMembre, plugin );

        // Suppression des actes rattachs qui sont annuls
        if ( ( listeMembrePresent.getActes(  ) != null ) && !listeMembrePresent.getActes(  ).isEmpty(  ) )
        {
            for ( Acte<GSeance, GFichier> acte : listeMembrePresent.getActes(  ) )
            {
                acte = _acteHome.findByPrimaryKey( acte.getId(  ), plugin );

                if ( acte.getDeliberationFinale(  ) != null )
                {
                    GFichier fichier = _fichierHome.findByPrimaryKey( acte.getDeliberationFinale(  ).getId(  ), plugin );
                    _acteHome.remove( acte, plugin );
                    _historiqueHome.remove( fichier, plugin );
                    OdsUtils.testNullObject( fichier, request );

                    if ( fichier.getSeance(  ) != null )
                    {
                        ISeance seanceByFichier = fichier.getSeance(  );
                        ISeance nextSeance = _seanceHome.getProchaineSeance( plugin );

                        if ( ( nextSeance != null ) && !seanceByFichier.equals( nextSeance ) &&
                                ( ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.DOCUMENT_LIBRE ) ||
                                ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.COMPTE_RENDU ) ||
                                ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.LISTE_DE_DESIGNATION ) ) )
                        {
                            _searchService.deleteObjectFromIndex( fichier, true, plugin );
                        }
                    }

                    _fichierHome.remove( fichier, plugin );
                    _fichierPhysiqueHome.remove( fichier.getFichier(  ), plugin );
                }
                else
                {
                    _acteHome.remove( acte, plugin );

                    // Suppression du fichier gnr
                    if ( fichierGenere != null )
                    {
                        fichierGenere = _fichierHome.findByPrimaryKey( fichierGenere.getId(  ), plugin );

                        FichierPhysique fichierPhysiqueGenere = _fichierPhysiqueHome.findByPrimaryKey( fichierGenere.getFichier(  )
                                                                                                                    .getIdFichier(  ),
                                plugin );
                        _historiqueHome.remove( fichierGenere, plugin );

                        if ( fichierGenere.getSeance(  ) != null )
                        {
                            ISeance seanceByFichier = fichierGenere.getSeance(  );
                            ISeance nextSeance = _seanceHome.getProchaineSeance( plugin );

                            if ( ( nextSeance != null ) && !seanceByFichier.equals( nextSeance ) &&
                                    ( ( fichierGenere.getTypdeDocument(  ).getId(  ) == ITypeDocument.DOCUMENT_LIBRE ) ||
                                    ( fichierGenere.getTypdeDocument(  ).getId(  ) == ITypeDocument.COMPTE_RENDU ) ||
                                    ( fichierGenere.getTypdeDocument(  ).getId(  ) == ITypeDocument.LISTE_DE_DESIGNATION ) ) )
                            {
                                _searchService.deleteObjectFromIndex( fichierGenere, true, plugin );
                            }
                        }

                        _fichierHome.remove( fichierGenere, plugin );
                        _fichierPhysiqueHome.remove( fichierPhysiqueGenere, plugin );
                    }
                }
            }
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * Modifie la dlibration de dsignation
     * Si la dlibration est en ligne, alors la version est incrmente de 1 et ajout d'un historique
     * si tous les champs obligatoires du formulaire de modification d'une dlibration ont t renseigns,<BR>
     * la mthode retourne l'url de gestion des dlibrations,<BR>
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request la requte http
     * @return l'url de modification d'une dlibration
     */
    public String doModificationListeMembrePresent( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;
        _bHasChanged = false;

        // Si on a cliqu sur le bouton "Annuler"
        if ( request.getParameter( OdsParameters.CONSTANTE_ACTION_CANCEL ) != null )
        {
            if ( _bUrlRetour )
            {
                strReturn = AppPathService.getBaseUrl( request ) + _strUrlRetour;
                _bUrlRetour = false;
                _strUrlRetour = null;
            }
            else
            {
                strReturn = AppPathService.getBaseUrl( request ) + JSP_URL_LISTE_MEMBRE_PRESENT;
            }

            return strReturn;
        }

        int nIdListeMembrePresent = OdsUtils.getIntegerParameter( OdsParameters.ID_LISTE_MEMBRE_PRESENT, request );

        if ( nIdListeMembrePresent != -1 )
        {
            int nIdFormationConseil = getFormationConseil( request );
            Timestamp dateCreation = DateUtils.getCurrentDate(  );
            ListeMembrePresent<GSeance, GFichier> listeMembrePresent = _listeMembrePresentHome.findByPrimaryKey( nIdListeMembrePresent,
                    plugin );

            // Si Preparer acte est cliqu, on recharge la page avec les champs de prparation de l'acte
            if ( request.getParameter( OdsParameters.PREPARER_ACTE_BUTTON ) != null )
            {
                if ( ( listeMembrePresent.getActeCourant(  ) == null ) ||
                        ( ( listeMembrePresent.getActeCourant(  ) != null ) &&
                        ( listeMembrePresent.getActeCourant(  ).getStatutActe(  ).getIdStatut(  ) != StatutActeEnum.A_PREPARER.getId(  ) ) ) )
                {
                    if ( _nTypeUrlRetour == 1 )
                    {
                        strReturn = AppPathService.getBaseUrl( request ) + JSP_URL_GET_MODIFICATION_MEMBRE_FOR_ACTE;
                    }
                    else
                    {
                        strReturn = AppPathService.getBaseUrl( request ) +
                            JSP_URL_GET_MODIFICATION_LISTE_MEMBRE_PRESENT;
                    }

                    strReturn += ( OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_LISTE_MEMBRE_PRESENT +
                    OdsConstants.CONSTANTE_EGAL + nIdListeMembrePresent );
                    strReturn += ( OdsConstants.CONSTANTE_ET + CONSTANTE_PREPARER_ACTE + OdsConstants.CONSTANTE_EGAL +
                    CONSTANTE_PREPARER_ACTE );

                    return strReturn;
                }

                if ( _nTypeUrlRetour == 1 )
                {
                    strReturn = AppPathService.getBaseUrl( request ) + JSP_URL_GET_MODIFICATION_MEMBRE_FOR_ACTE;
                }
                else
                {
                    strReturn = AppPathService.getBaseUrl( request ) + JSP_URL_GET_MODIFICATION_LISTE_MEMBRE_PRESENT;
                }

                strReturn += ( OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_LISTE_MEMBRE_PRESENT +
                OdsConstants.CONSTANTE_EGAL + nIdListeMembrePresent );

                return strReturn;
            }

            String strError = getListeMembrePresentData( request, listeMembrePresent );

            if ( strError != null )
            {
                return strError;
            }

            // On verifie qu'une liste de membres de mme sance et mme formation n'existe pas
            boolean bIsAlreadyExist = _listeMembrePresentHome.isAlreadyExist( listeMembrePresent, plugin );

            if ( bIsAlreadyExist )
            {
                return AdminMessageService.getMessageUrl( request, MESSAGE_LISTE_ALREADY_EXIST, AdminMessage.TYPE_STOP );
            }

            int nMaxPoids = AppPropertiesService.getPropertyInt( OdsProperties.PROPERTY_POIDS_MAX_ACTE,
                    OdsConstants.DEFAULT_POIDS_MAX_ACTE );

            Acte<GSeance, GFichier> acteCourant = listeMembrePresent.getActeCourant(  );

            if ( request.getParameter( OdsParameters.PREPARER_ACTE ) != null )
            {
                acteCourant = new Acte<GSeance, GFichier>(  );
                acteCourant.setStatutActe( StatutActeEnum.A_PREPARER.getStatutOnlyWidthId(  ) );
            }

            if ( ( request.getParameter( MARK_ID_ACTE ) == null ) && ( acteCourant != null ) &&
                    ( ( acteCourant.getStatutActe(  ).getIdStatut(  ) == StatutActeEnum.A_PREPARER.getId(  ) ) ||
                    ( acteCourant.getStatutActe(  ).getIdStatut(  ) == StatutActeEnum.EN_PREPARATION.getId(  ) ) ) )
            {
                acteCourant.setDeliberationFinale( listeMembrePresent.getFichierGenere(  ) );
                acteCourant.setPoids( listeMembrePresent.getFichierGenere(  ).getTaille(  ) );

                int nIdModeTransmission = Integer.parseInt( request.getParameter( OdsParameters.MODE_TRANSMISSION ) );

                if ( ( nIdModeTransmission == 1 ) && ( acteCourant.getPoids(  ) > nMaxPoids ) )
                {
                    // Mode de transmission automatique, mais le poids est suprieur
                    return AdminMessageService.getMessageUrl( request,
                        OdsProperties.PROPERTY_MESSAGE_MODE_TRANSMISSION_ERROR, AdminMessage.TYPE_STOP );
                }

                acteCourant.setIdTransmission( nIdModeTransmission );

                // Ajout de la catgorie
                CategorieDeliberation categorie = _categorieDeliberationHome.findByPrimaryKey( Integer.parseInt( 
                            request.getParameter( OdsParameters.ID_CATEGORIE ) ), plugin );
                acteCourant.setCategorie( categorie );
                acteCourant.setReference( listeMembrePresent.getIntitule(  ) );

                int natureActeID = Integer.parseInt( request.getParameter( OdsParameters.NATURE_DOCUMENT ) );
                acteCourant.setNatureDocument( _natureActeHome.findByPrimaryKey( natureActeID, plugin ) );
                acteCourant.setIdEntite( listeMembrePresent.getId(  ) );
                acteCourant.setIdTypeEntite( ITypeEntite.LISTE_DES_MEMBRES_PRESENTS );
                acteCourant.setStatutActe( StatutActeEnum.EN_PREPARATION.getStatutOnlyWidthId(  ) );
                acteCourant.setSeance( listeMembrePresent.getSeance(  ) );
                acteCourant.setFormationConseil( _formationConseilHome.findByPrimaryKey( nIdFormationConseil, plugin ) );
                acteCourant.setEnLigne( false );
                listeMembrePresent.setActeCourant( acteCourant );

                if ( request.getParameter( OdsParameters.PREPARER_ACTE ) != null )
                {
                    _acteHome.create( acteCourant, plugin );
                }
                else
                {
                    _acteHome.update( acteCourant, plugin );
                }
            }

            // Finalisation de l'acte (si bouton finaliser cliqu)
            if ( request.getParameter( OdsParameters.FINALISER_ACTE ) != null )
            {
                if ( ( acteCourant.getIdTransmission(  ) == 1 ) && ( acteCourant.getPoids(  ) > nMaxPoids ) )
                {
                    strReturn = AppPathService.getBaseUrl( request ) + JSP_URL_FINALISATION_ACTE +
                        OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_LISTE_MEMBRE_PRESENT +
                        OdsConstants.CONSTANTE_EGAL + nIdListeMembrePresent + OdsConstants.CONSTANTE_ET +
                        getParameterPluginName(  );

                    return strReturn;
                }
                else
                {
                    acteCourant.setStatutActe( StatutActeEnum.A_TRANSMETTRE.getStatutOnlyWidthId(  ) );
                    _acteHome.update( acteCourant, plugin );

                    if ( _bUrlRetour )
                    {
                        strReturn = AppPathService.getBaseUrl( request ) + _strUrlRetour;
                        _bUrlRetour = false;
                        _strUrlRetour = null;
                    }

                    return strReturn;
                }
            }

            if ( listeMembrePresent.getEnLigne(  ) && _bHasChanged )
            {
                listeMembrePresent.setVersion( listeMembrePresent.getVersion(  ) + 1 );

                // Gestion de l'historique de la liste des membres presents
                Historique historique = new Historique(  );
                historique.setVersion( listeMembrePresent.getVersion(  ) );
                historique.setDatePublication( dateCreation );
                historique.setIdEntite( listeMembrePresent.getId(  ) );
                historique.setIdTypeEntite( ITypeEntite.LISTE_DES_MEMBRES_PRESENTS );
                _historiqueHome.create( historique, plugin );
            }

            // On met  jour la liste des membres prsents
            _listeMembrePresentHome.update( listeMembrePresent, plugin );

            // On supprime les lus prsents
            _listeMembrePresentHome.removeAllElus( listeMembrePresent.getId(  ), plugin );

            // On rcupre de la liste des Id d'lus  ajouter  la liste des membres prsents
            String[] strNewListeElus = request.getParameterValues( OdsParameters.LISTE_ELU_CHECKED );

            if ( strNewListeElus != null )
            {
                for ( String strNewElus : strNewListeElus )
                {
                    int nIdNewElu = StringUtils.string2id( strNewElus );

                    if ( nIdNewElu != -1 )
                    {
                        //On ajoute l'lu  la liste
                        _listeMembrePresentHome.insertElus( nIdNewElu, listeMembrePresent.getId(  ), plugin );
                    }
                }
            }

            // Gnration du fichier PDF
            strError = doGenerePDF( listeMembrePresent, request );

            if ( strError != null )
            {
                return strError;
            }

            // Si Appliquer est cliqu, on enregistre et recharge la page
            if ( request.getParameter( OdsParameters.CONSTANTE_ACTION_APPLY ) != null )
            {
                if ( request.getParameter( MARK_ID_ACTE ) != null )
                {
                    strReturn = AppPathService.getBaseUrl( request ) + JSP_URL_GET_VISUALISATION_MEMBRE_FOR_ACTE +
                        OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_LISTE_MEMBRE_PRESENT +
                        OdsConstants.CONSTANTE_EGAL + nIdListeMembrePresent;

                    int nIdActe = OdsUtils.getIntegerParameter( MARK_ID_ACTE, request );
                    strReturn += ( OdsConstants.CONSTANTE_ET + OdsParameters.ID_ACTE + OdsConstants.CONSTANTE_EGAL +
                    nIdActe );
                }
                else
                {
                    if ( _nTypeUrlRetour == 1 )
                    {
                        strReturn = AppPathService.getBaseUrl( request ) + JSP_URL_GET_MODIFICATION_MEMBRE_FOR_ACTE;
                    }
                    else
                    {
                        strReturn = AppPathService.getBaseUrl( request ) +
                            JSP_URL_GET_MODIFICATION_LISTE_MEMBRE_PRESENT;
                    }

                    strReturn += ( OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_LISTE_MEMBRE_PRESENT +
                    OdsConstants.CONSTANTE_EGAL + nIdListeMembrePresent );
                }

                return strReturn;
            }
        }
        else
        {
            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                AdminMessage.TYPE_STOP );
        }

        if ( _bUrlRetour )
        {
            strReturn = AppPathService.getBaseUrl( request ) + _strUrlRetour;
            _bUrlRetour = false;
            _strUrlRetour = null;
        }

        return strReturn;
    }

    /**
     * Creation d'une liste de membre prsent
     * si tous les champs obligatoires du formulaire de cration d'une liste de membre prsent ont t renseign,<BR>
     * la mthode retourne l'url de gestion des dlibrations,<BR>
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request la requte http
     * @return l'url de creation d'une dlibration de dsignation
     */
    public String doCreationListeMembrePresent( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        // Si on a cliqu sur le bouton "Annuler"
        if ( request.getParameter( OdsParameters.CONSTANTE_ACTION_CANCEL ) != null )
        {
            return AppPathService.getBaseUrl( request ) + JSP_URL_LISTE_MEMBRE_PRESENT;
        }

        ListeMembrePresent<GSeance, GFichier> listeMembrePresent = new ListeMembrePresent<GSeance, GFichier>(  );
        String strError = getListeMembrePresentData( request, listeMembrePresent );

        if ( strError != null )
        {
            return strError;
        }

        // On vrifie qu'une liste de membres de mme sance et mme formation n'existe pas
        boolean bIsAlreadyExist = _listeMembrePresentHome.isAlreadyExist( listeMembrePresent, plugin );

        if ( bIsAlreadyExist )
        {
            return AdminMessageService.getMessageUrl( request, MESSAGE_LISTE_ALREADY_EXIST, AdminMessage.TYPE_STOP );
        }

        listeMembrePresent.setEnLigne( false );

        // Cration de la liste des membres prsents
        int nIdListe = _listeMembrePresentHome.create( listeMembrePresent, plugin );

        // Rcupration de la liste des Id d'lus  ajouter  la liste des membres prsents
        String[] strListeElus = request.getParameterValues( OdsParameters.LISTE_ELU_CHECKED );

        if ( strListeElus != null )
        {
            for ( String strElus : strListeElus )
            {
                int nIdElu = StringUtils.string2id( strElus );

                if ( nIdElu != -1 )
                {
                    _listeMembrePresentHome.insertElus( nIdElu, nIdListe, plugin );
                }
            }
        }

        // Gnration du fichier PDF
        strError = doGenerePDF( listeMembrePresent, request );

        if ( strError != null )
        {
            _listeMembrePresentHome.removeAllElus( nIdListe, plugin );
            _listeMembrePresentHome.remove( nIdListe, plugin );

            return strError;
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * Rcupre un fichier  partir de son identifiant
     * @return le fichier
     */
    public GFichier getFichier( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        int nIdFichier = OdsUtils.getIntegerParameter( OdsParameters.ID_FICHIER, request );

        GFichier fichier = _fichierHome.findByPrimaryKey( nIdFichier, plugin );
        FichierPhysique fichierPhysique = _fichierPhysiqueHome.findByPrimaryKey( fichier.getFichier(  ).getIdFichier(  ),
                plugin );

        fichier.setFichier( fichierPhysique );

        return fichier;
    }

    /**
     * recupere dans la requete les valeurs saisies
     * retourne null si aucune erreur n'est survenue
     * @param request the request
     * @param listeMembrePresent la liste des membres prsents
    
     * @return null if there is no error or else return the error page url
     */
    private String getListeMembrePresentData( HttpServletRequest request,
        ListeMembrePresent<GSeance, GFichier> listeMembrePresent )
    {
        String strFieldError = OdsConstants.CONSTANTE_CHAINE_VIDE;

        int nIdSeance = OdsUtils.getIntegerParameter( OdsParameters.ID_SEANCE, request );
        int nIdFormationConseil = getFormationConseil( request );
        String strIntitule = request.getParameter( OdsParameters.INTITULE ).trim(  );
        String strDateTransmissionControleLegalite = request.getParameter( OdsParameters.DATE_TRANSMISSION_CONTROLE_LEGALITE )
                                                            .trim(  );

        if ( nIdSeance == -1 )
        {
            strFieldError = PROPERTY_LABEL_SEANCE;
        }
        else if ( nIdFormationConseil == -1 )
        {
            strFieldError = PROPERTY_LABEL_FORMATION_CONSEIL;
        }
        else if ( ( strIntitule == null ) || strIntitule.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strFieldError = PROPERTY_LABEL_INTITULE;
        }
        else if ( ( strDateTransmissionControleLegalite == null ) ||
                strDateTransmissionControleLegalite.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strFieldError = PROPERTY_LABEL_DATE_TRANSMISSION_CTRL_LEGALITE;
        }

        if ( !strFieldError.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            Object[] tabRequiredFields = { I18nService.getLocalizedString( strFieldError, request.getLocale(  ) ) };

            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED,
                tabRequiredFields, AdminMessage.TYPE_STOP );
        }

        Timestamp tsDateTransmissionControleLegalite = DateUtils.getDate( strDateTransmissionControleLegalite, true );

        if ( tsDateTransmissionControleLegalite == null )
        {
            Object[] messagesArgs = 
                {
                    I18nService.getLocalizedString( PROPERTY_LABEL_DATE_TRANSMISSION_CTRL_LEGALITE,
                        request.getLocale(  ) ),
                };

            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_DATE_FORMAT_ERROR,
                messagesArgs, AdminMessage.TYPE_ERROR );
        }

        if ( ( listeMembrePresent.getSeance(  ) != null ) &&
                ( listeMembrePresent.getSeance(  ).getIdSeance(  ) != nIdSeance ) )
        {
            _bHasChanged = true;
        }
        else if ( ( listeMembrePresent.getFormationConseil(  ) != null ) &&
                ( listeMembrePresent.getFormationConseil(  ).getIdFormationConseil(  ) != nIdFormationConseil ) )
        {
            _bHasChanged = true;
        }
        else if ( ( listeMembrePresent.getIntitule(  ) != null ) &&
                !listeMembrePresent.getIntitule(  ).equals( strIntitule ) )
        {
            _bHasChanged = true;
        }
        else if ( ( listeMembrePresent.getDateTransmissionControleLegalite(  ) != null ) &&
                !listeMembrePresent.getDateTransmissionControleLegalite(  ).equals( tsDateTransmissionControleLegalite ) )
        {
            _bHasChanged = true;
        }

        GSeance seance = newSeanceInstance(  );
        seance.setIdSeance( nIdSeance );
        listeMembrePresent.setSeance( seance );

        FormationConseil formationConseil = new FormationConseil(  );
        formationConseil.setIdFormationConseil( nIdFormationConseil );
        listeMembrePresent.setFormationConseil( formationConseil );

        listeMembrePresent.setIntitule( strIntitule );
        listeMembrePresent.setDateTransmissionControleLegalite( tsDateTransmissionControleLegalite );

        return null;
    }

    /**
     * Gnre le fichier PDF correspondant  la liste des membres prsents
     * @param listeMembrePresent la liste des membres prsents
     * @param request HttpServletRequest request
     * @return null si tout s'est bien pass, un message d'erreur sinon
     */
    private String doGenerePDF( ListeMembrePresent<GSeance, GFichier> listeMembrePresent, HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        if ( !DocumentManager.checkJCopist(  ) )
        {
            return AdminMessageService.getMessageUrl( request, MESSAGE_PROBLEME_CONVERSION_JCOPIST,
                AdminMessage.TYPE_ERROR );
        }

        listeMembrePresent.setSeance( _seanceHome.findByPrimaryKey( listeMembrePresent.getSeance(  ).getIdSeance(  ),
                plugin ) );
        listeMembrePresent.setFormationConseil( _formationConseilHome.findByPrimaryKey( 
                listeMembrePresent.getFormationConseil(  ).getIdFormationConseil(  ), plugin ) );

        List<IElu> listElus = _eluHome.findElusActifs( plugin );
        List<IElu> listElusPresents = new ArrayList<IElu>(  );
        List<Integer> listIdElu = _listeMembrePresentHome.getListeIdsElusPresents( listeMembrePresent.getId(  ), plugin );

        for ( int idElu : listIdElu )
        {
            IElu elu = _eluHome.findByPrimaryKey( idElu, plugin );
            listElusPresents.add( elu );
            listElus.remove( elu );
        }

        listeMembrePresent.setElusPresents( listElusPresents );

        /* Rcupration du modle du certificat pour la gnration PDF */
        AutreModeleFilter filterAutreModele = new AutreModeleFilter(  );
        filterAutreModele.setIdType( TypeAutreModeleEnum.LISTE_MEMBRES_PRESENTS.getId(  ) );

        List<IAutreModele> listAutreModele = _autreModeleHome.findByFilter( filterAutreModele, plugin );

        if ( ( listAutreModele != null ) && listAutreModele.isEmpty(  ) )
        {
            return AdminMessageService.getMessageUrl( request, PROPERTY_MESSAGE_NO_MODELE, AdminMessage.TYPE_ERROR );
        }
        else
        {
            IAutreModele autreModele = listAutreModele.get( 0 );

            if ( autreModele != null )
            {
                //On rcupre l'entte et le pied de page
                Entete entete = ModeleUtils.getEntete( autreModele.getEnteteDocument(  ).getDonnees(  ) );
                PiedDePage piedDePage = ModeleUtils.getPiedDePage( autreModele.getPiedDocument(  ).getDonnees(  ) );

                HashMap<Integer, Object> xmlParamsEntete = new HashMap<Integer, Object>(  );
                HashMap<Integer, Object> xmlParamsPiedDePage = new HashMap<Integer, Object>(  );

                Timestamp dateCreation = DateUtils.getCurrentDate(  );

                //On genere le flux XML de l'entte
                xmlParamsEntete.put( ModeleUtils.XML_PARAMETER_CODE_DATE_TRANSMISSION_CTRL,
                    DateUtils.getDate( listeMembrePresent.getDateTransmissionControleLegalite(  ),
                        OdsConstants.CONSTANTE_PATTERN_DATE_LONG ) );
                xmlParamsEntete.put( ModeleUtils.XML_PARAMETER_CODE_FORMATION_CONSEIL,
                    listeMembrePresent.getFormationConseil(  ).getLibelle(  ) );

                doGenerePDFEnteteSpec( xmlParamsEntete, listeMembrePresent, request );

                String strXmlEntete = ModeleUtils.getXmlEntete( entete, xmlParamsEntete );

                //On genere le flux XML du pied de page
                String strNomSecretaireGenerale = AppPropertiesService.getProperty( OdsProperties.PROPERTY_NOM_SECRETAIRE_GENERALE );
                xmlParamsPiedDePage.put( ModeleUtils.XML_PARAMETER_CODE_NOM_SECRETAIRE_GENERAL, strNomSecretaireGenerale );

                String strXmlPiedDePage = ModeleUtils.getXmlPiedDePage( piedDePage, xmlParamsPiedDePage );

                StringBuffer sbXml = new StringBuffer( 68 * 1024 );

                sbXml.append( XML_LISTE_MEMBRE_PRESENT_OPEN );
                sbXml.append( strXmlEntete );
                sbXml.append( strXmlPiedDePage );
                sbXml.append( listeMembrePresent.getXml(  ) );
                sbXml.append( XML_LISTE_ELUS_EXCUSES_OPEN );

                for ( IElu elu : listElus )
                {
                    sbXml.append( elu.getXml(  ) );
                }

                sbXml.append( XML_LISTE_ELUS_EXCUSES_CLOSE );
                sbXml.append( OdsConstants.XML_SECRETAIRE_GENERALE_OPEN + strNomSecretaireGenerale +
                    OdsConstants.XML_SECRETAIRE_GENERALE_CLOSE );
                sbXml.append( XML_LISTE_MEMBRE_PRESENT_CLOSE );

                try
                {
                    byte[] xml = ( OdsConstants.CONSTANTE_ENTETE_XML + sbXml.toString(  ) ).getBytes( OdsConstants.UTF8 );
                    byte[] bConvertedDoc = DocumentManager.convert( DocumentManager.PDF_CONVERSION,
                            CONSTANTE_LISTE_MEMBRES_PRESENTS, xml );

                    if ( ( bConvertedDoc != null ) && ( bConvertedDoc.length > 0 ) )
                    {
                        GFichier fichier;
                        FichierPhysique fichierPhysique;

                        if ( listeMembrePresent.getFichierGenere(  ) != null )
                        {
                            fichier = listeMembrePresent.getFichierGenere(  );
                            fichier = _fichierHome.findByPrimaryKey( fichier.getId(  ), plugin );
                            fichierPhysique = _fichierPhysiqueHome.findByPrimaryKey( fichier.getFichier(  )
                                                                                            .getIdFichier(  ), plugin );

                            fichierPhysique.setDonnees( bConvertedDoc );
                            _fichierPhysiqueHome.update( fichierPhysique, plugin );

                            fichier.setFichier( fichierPhysique );
                            fichier.setTaille( fichierPhysique.getTaille(  ) );

                            if ( fichier.getSeance(  ) != null )
                            {
                                ISeance seanceByFichier = fichier.getSeance(  );
                                ISeance nextSeance = _seanceHome.getProchaineSeance( plugin );

                                if ( ( ( nextSeance != null ) && !seanceByFichier.equals( nextSeance ) &&
                                        ( ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.DOCUMENT_LIBRE ) ||
                                        ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.COMPTE_RENDU ) ) ) ||
                                        ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.LISTE_DE_DESIGNATION ) )
                                {
                                    _searchService.updateObjectInIndex( fichier, true, plugin );
                                }
                            }

                            _fichierHome.update( fichier, plugin );
                        }
                        else
                        {
                            fichier = newFichierInstance(  );
                            fichierPhysique = new FichierPhysique(  );

                            fichierPhysique.setDonnees( bConvertedDoc );
                            fichierPhysique.setIdFichier( _fichierPhysiqueHome.create( fichierPhysique, plugin ) );

                            fichier.setFichier( fichierPhysique );
                            fichier.setSeance( listeMembrePresent.getSeance(  ) );
                            fichier.setFormationConseil( listeMembrePresent.getFormationConseil(  ) );
                            fichier.setTypdeDocument( _fichierHome.findTypeDocumentsById( 
                                    ITypeDocument.LISTE_MEMBRE_PRESENT, plugin ) );
                            fichier.setIntitule( listeMembrePresent.getIntitule(  ) );
                            fichier.setNom( I18nService.getLocalizedString( PROPERTY_NOM_FICHIER_PDF,
                                    request.getLocale(  ) ) );
                            fichier.setExtension( OdsConstants.CONSTANTE_PDF );
                            fichier.setEnLigne( false );
                            fichier.setVersion( 1 );
                            fichier.setTaille( fichierPhysique.getTaille(  ) );
                            fichier.setId( _fichierHome.create( fichier, plugin ) );

                            if ( fichier.getSeance(  ) != null )
                            {
                                ISeance seanceByFichier = fichier.getSeance(  );
                                ISeance nextSeance = _seanceHome.getProchaineSeance( plugin );

                                if ( ( nextSeance != null ) && !seanceByFichier.equals( nextSeance ) &&
                                        ( ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.DOCUMENT_LIBRE ) ||
                                        ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.COMPTE_RENDU ) ||
                                        ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.LISTE_DE_DESIGNATION ) ) )
                                {
                                    _searchService.addObjectToIndex( fichier, true, plugin );
                                }
                            }
                        }

                        listeMembrePresent.setFichierGenere( fichier );
                        _listeMembrePresentHome.update( listeMembrePresent, plugin );

                        // Ajout de l'historique
                        Historique historique = new Historique(  );
                        historique.setVersion( 1 );
                        historique.setDatePublication( dateCreation );
                        historique.setIdEntite( fichier.getId(  ) );
                        historique.setIdTypeEntite( ITypeEntite.FICHIERS );
                        _historiqueHome.create( historique, plugin );
                    }
                }
                catch ( ServerException e )
                {
                    AppLogService.error( "doGenerePDF - ServerException : " + e );

                    return AdminMessageService.getMessageUrl( request, MESSAGE_PROBLEME_CONVERSION_JCOPIST,
                        AdminMessage.TYPE_ERROR );
                }
                catch ( IOException e )
                {
                    AppLogService.error( "doGenerePDF - IOException : " + e );

                    return AdminMessageService.getMessageUrl( request, MESSAGE_PROBLEME_CONVERSION_JCOPIST,
                        AdminMessage.TYPE_ERROR );
                }
                catch ( UnavailableException e )
                {
                    AppLogService.error( "doGenerePDF - UnavailableException : " + e );

                    return AdminMessageService.getMessageUrl( request, MESSAGE_PROBLEME_CONVERSION_JCOPIST,
                        AdminMessage.TYPE_ERROR );
                }
            }
        }

        return null;
    }

    public String getJspUrlGetModificationForActe( HttpServletRequest request, int nIdEntite, int nIdActe )
    {
        return AppPathService.getBaseUrl( request ) + JSP_URL_GET_MODIFICATION_MEMBRE_FOR_ACTE +
        OdsConstants.CONSTANTE_POINT_INTERROGATION + getParameterPluginName(  ) + OdsConstants.CONSTANTE_ET +
        getIdParameter(  ) + OdsConstants.CONSTANTE_EGAL + nIdEntite;
    }

    public String getIdParameter(  )
    {
        return OdsParameters.ID_LISTE_MEMBRE_PRESENT;
    }

    public void doPublicationActe( Acte<GSeance, GFichier> acte, boolean bIsPublie, Timestamp dateForVersion )
    {
        Plugin plugin = getPlugin(  );

        ListeMembrePresent<GSeance, GFichier> listeMembrePresent = _listeMembrePresentHome.findByPrimaryKey( acte.getIdEntite(  ),
                plugin );

        if ( !( ( listeMembrePresent.getEnLigne(  ) && bIsPublie ) ||
                ( !listeMembrePresent.getEnLigne(  ) && !bIsPublie ) ) )
        {
            if ( bIsPublie && ( listeMembrePresent.getVersion(  ) < 1 ) )
            {
                listeMembrePresent.setVersion( 0 );
            }

            //On publie la liste des membres prsents ainsi que le fichier associ
            _listeMembrePresentHome.publication( listeMembrePresent.getId(  ), dateForVersion,
                listeMembrePresent.getVersion(  ), bIsPublie, plugin );

            //On publie les fichiers associs
            _fichierHome.publication( listeMembrePresent.getFichierGenere(  ).getId(  ), dateForVersion,
                listeMembrePresent.getVersion(  ), bIsPublie, plugin );

            if ( bIsPublie )
            {
                Historique historique = new Historique(  );
                historique.setVersion( listeMembrePresent.getVersion(  ) + 1 );
                historique.setDatePublication( dateForVersion );
                historique.setIdEntite( listeMembrePresent.getId(  ) );
                historique.setIdTypeEntite( ITypeEntite.LISTE_DES_MEMBRES_PRESENTS );
                _historiqueHome.create( historique, plugin );
            }

            // on horodate la (de)publication
            HorodatageActionEnum action = ( bIsPublie ) ? HorodatageActionEnum.PUBLICATION
                                                        : HorodatageActionEnum.DE_PUBLICATION;
            _horodatageService.tracePublication( listeMembrePresent, action );
        }
    }

    @SuppressWarnings( "unchecked" )
    public ISeance getSeance( IBusinessItem entite )
    {
        Plugin plugin = getPlugin(  );

        ListeMembrePresent<GSeance, GFichier> listeMembrePresent = (ListeMembrePresent<GSeance, GFichier>) entite;
        ISeance seance = _seanceHome.findByPrimaryKey( listeMembrePresent.getSeance(  ).getIdSeance(  ), plugin );

        return seance;
    }

    /**
     * Retourne la signature numriques d'une liste de membre present
     *
     * @param listeMembrePresent
     *            liste de membre present
     * @return la signature
     */
    @SuppressWarnings( "unchecked" )
    public String getSignature( IEntiteHorodatage entite )
    {
        Plugin plugin = getPlugin(  );
        String signature = OdsConstants.CONSTANTE_CHAINE_VIDE;

        ListeMembrePresent<GSeance, GFichier> listeMembrePresent = (ListeMembrePresent<GSeance, GFichier>) entite;

        GFichier fichierGenere = listeMembrePresent.getFichierGenere(  );
        fichierGenere.setFichier( _fichierPhysiqueHome.findByPrimaryKey( 
                listeMembrePresent.getFichierGenere(  ).getFichier(  ).getIdFichier(  ), plugin ) );
        listeMembrePresent.setFichierGenere( fichierGenere );

        if ( ( listeMembrePresent.getFichierGenere(  ) != null ) &&
                ( listeMembrePresent.getFichierGenere(  ).getFichier(  ) != null ) )
        {
            byte[] donnees = listeMembrePresent.getFichierGenere(  ).getFichier(  ).getDonnees(  );
            signature = ShaUtils.toSha224String( donnees );

            if ( signature == null )
            {
                signature = "error_" + listeMembrePresent.getFichierGenere(  ).getNom(  );
                AppLogService.error( "Erreur lors de la recuperation de la signature SHA 224 du fichier : " +
                    listeMembrePresent.getFichierGenere(  ).getNom(  ) );
            }
        }

        return signature;
    }

    public IEntiteCertificatAffichage getEntite( Acte<GSeance, GFichier> acte )
    {
        Plugin plugin = getPlugin(  );

        ListeMembrePresent<GSeance, GFichier> listeMembrePresent = _listeMembrePresentHome.findByPrimaryKey( acte.getIdEntite(  ),
                plugin );

        return listeMembrePresent;
    }

    public Comparator<IBusinessItem> getComparatorClass(  )
    {
        return null;
    }

    public String getMarkForCertificat( int nIdTypeEntite )
    {
        return OdsMarks.MARK_LISTE_MEMBRES_PRESENTS;
    }

    /**
     * Renvoie l'objet de l'acte utilis lors de l'envoi vers la plate-forme
     *                         Intitule
     * @param acte l'acte que l'on va transmettre
     * @return strObjet l'objet de l'acte
     */
    public String getObjetActe( Acte<GSeance, GFichier> acte )
    {
        Plugin plugin = getPlugin(  );
        String strObjet = OdsConstants.CONSTANTE_CHAINE_VIDE;

        ListeMembrePresent<GSeance, GFichier> listeMembrePresent = _listeMembrePresentHome.findByPrimaryKey( acte.getIdEntite(  ),
                plugin );

        if ( listeMembrePresent != null )
        {
            strObjet = listeMembrePresent.getIntitule(  );
        }

        return strObjet;
    }
}
