/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.manager;

import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.spring.SpringContextService;

import net.sf.jcopist.service.FileData;
import net.sf.jcopist.service.IDocumentService;
import net.sf.jcopist.service.ITemplateService;
import net.sf.jcopist.service.Job;
import net.sf.jcopist.service.Result;
import net.sf.jcopist.service.exception.ServerException;
import net.sf.jcopist.service.exception.UnavailableException;

import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;


/**
 * Classe qui a pour responsabilit de faire le proxy entre l'application ODS
 * et l'outil utilis pour gnr les documents PDF
 */
public final class DocumentManager
{
    public static final String PDF_CONVERSION = "pdf";
    public static final String CONSTANTE_ODT = "odt";
    public static final String PDF_WORD = "doc";

    /**
     * Constructeur vide
     */
    private DocumentManager(  )
    {
    }

    /**
     * Cette mthode permet de convertir un fichier XML en un fichier PDF
     *
     * @param strFormatConversion Format de la conversion tel que PDF ou DOC
     * @param strNameTemplate Nom de la template utilis
     * @param xml fichier XML
     * @throws UnavailableException UnavailableException
     * @throws ServerException ServerException
     * @throws IOException IOException
     * @return le pdf gnr
     */
    public static byte[] convert( String strFormatConversion, String strNameTemplate, byte[] xml )
        throws UnavailableException, ServerException, IOException
    {
        IDocumentService documentService = (IDocumentService) SpringContextService.getPluginBean( OdsConstants.CONSTANTE_PLUGIN_ODS,
                "documentService" );

        byte[] bConvertedDoc = null;

        /* Create a job */
        Job job = new Job(  );
        job.setDestFormat( strFormatConversion );
        job.setTemplateName( strNameTemplate );
        job.setXmlData( xml );

        /* Call the document service */
        Result result = null;

        result = documentService.process( job );

        if ( ( null != result ) && ( null != result.getFiles(  ) ) )
        {
            /* Save the result files */
            FileData[] fileDatas = result.getFiles(  );

            if ( ( fileDatas != null ) && ( fileDatas.length > 0 ) )
            {
                bConvertedDoc = fileDatas[0].getBinaryData(  );
            }
        }

        return bConvertedDoc;
    }

    /**
     * Deploie une nouvelle template
     * @param pathTemplate le path de la template
     * @param nameTemplate le nom de la template
     * @throws Exception if an error occurs.
     */
    public static void deployTemplateService( String pathTemplate, String nameTemplate )
        throws Exception
    {
        ITemplateService templateService = (ITemplateService) SpringContextService.getPluginBean( OdsConstants.CONSTANTE_PLUGIN_ODS,
                "templateService" );

        final String TEMPLATE_ESSAI_PATH = pathTemplate;
        final String TEMPLATE_ESSAI_NAME = nameTemplate;

        Map<String, String> templateMap = new HashMap<String, String>(  );
        templateMap.put( TEMPLATE_ESSAI_NAME, TEMPLATE_ESSAI_PATH );

        //for ( String templateName : templateMap.keySet(  ) )
        for ( Entry<String, String> entryTemplate : templateMap.entrySet(  ) )
        {
            // Read the template file data
            //File templateFile = new File( templateMap.get( templateName ) );
            File templateFile = new File( entryTemplate.getValue(  ) );
            InputStream inStream = new BufferedInputStream( new FileInputStream( templateFile ) );
            ByteArrayOutputStream outStream = new ByteArrayOutputStream(  );
            int c;

            while ( ( c = inStream.read(  ) ) >= 0 )
            {
                outStream.write( c );
            }

            // Register the template
            templateService.register( entryTemplate.getValue(  ), CONSTANTE_ODT, outStream.toByteArray(  ) );

            inStream.close(  );
        }
    }

    /**
     * Teste la connexion au serveur JCopist.
     * @return TRUE si le test s'est bien droul, FALSE sinon
     */
    public static boolean checkJCopist(  )
    {
        boolean bResult = true;

        /* On instancie une nouvelle connexion au service JCopist */
        try
        {
            IDocumentService documentService = (IDocumentService) SpringContextService.getPluginBean( OdsConstants.CONSTANTE_PLUGIN_ODS,
                    "documentService" );
            documentService.getPrintersNameList(  );
        }
        catch ( Exception e )
        {
            bResult = false;
        }

        return bResult;
    }
}
