/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.modeleordredujour;

import fr.paris.lutece.plugins.ods.business.fichier.FichierPhysiqueHome;
import fr.paris.lutece.plugins.ods.business.modeleordredujour.IModeleOrdreDuJourHome;
import fr.paris.lutece.plugins.ods.business.ordredujour.TypeOrdreDuJourHome;
import fr.paris.lutece.plugins.ods.dto.fichier.FichierPhysique;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.modeleordredujour.IModeleOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter;
import fr.paris.lutece.plugins.ods.dto.ordredujour.TypeOrdreDuJour;
import fr.paris.lutece.plugins.ods.service.formationconseil.IFormationConseilService;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.web.upload.MultipartHttpServletRequest;
import fr.paris.lutece.util.ReferenceList;
import fr.paris.lutece.util.url.UrlItem;

import org.apache.commons.fileupload.FileItem;

import org.springframework.beans.factory.annotation.Autowired;

import java.sql.SQLException;

import java.util.HashMap;
import java.util.List;

import javax.servlet.http.HttpServletRequest;


public abstract class AbstractModeleOrdreDuJourService<GOrdreDuJourFilter extends IOrdreDuJourFilter, GModeleOrdreDuJour extends IModeleOrdreDuJour, GModeleOrdreDuJourHome extends IModeleOrdreDuJourHome<GOrdreDuJourFilter, GModeleOrdreDuJour>>
    implements IModeleOrdreDuJourService
{
    private static final String PROPERTY_LABEL_TITRE = "ods.modele.field.titre";
    private static final String PROPERTY_LABEL_TYPE = "ods.modele.field.type";
    private static final String PROPERTY_LABEL_FORMATION = "ods.modele.field.formation";
    private static final String MARK_MODELE = "modele";
    private static final String MARK_LIST_MODELE = "liste_modele";
    private static final String MARK_LISTE_FORMATION_CONSEIL = "liste_formation_conseil";
    private static final String MARK_LISTE_TYPE = "liste_type";
    protected static final String MESSAGE_MODELE_ALREADY_EXIST = "ods.modele.message.existeDeja";
    private static final String MESSAGE_MODELE_ENTETE_NO_XML = "ods.modele.message.enteteNoXml";
    private static final String MESSAGE_MODELE_PIED_NO_XML = "ods.modele.message.piedNoXml";
    private static final String MESSAGE_CONFIRME_DELETE_MODELE = "ods.modele.message.confirmeSuppression";
    private static final String MESSAGE_CANNOT_DELETE_MODELE = "ods.modele.message.suppressionImpossible";
    private static final String JSP_URL_DO_SUPPRESSION_MODELE_JSP = "jsp/admin/plugins/ods/modeleordredujour/DoSuppressionModele.jsp";
    protected static final String MARK_COMMISSION_DISPLAY = "commission_display";
    protected static final String MARK_FORMATION_CONSEIL_DISPLAY = "formation_conseil_display";
    @Autowired
    protected GModeleOrdreDuJourHome _modeleOrdreDuJourHome;
    @Autowired
    private FichierPhysiqueHome _fichierPhysiqueHome;
    @Autowired
    private IFormationConseilService _formationConseilService;
    @Autowired
    private TypeOrdreDuJourHome _typeOrdreDuJourHome;

    protected abstract void getModeleOdjListSpec( HttpServletRequest request, HashMap<String, Object> model );

    protected abstract void getModificationModeleOdjSpec( HttpServletRequest request, HashMap<String, Object> model );

    protected abstract void getCreationModeleOdjSpec( HttpServletRequest request, HashMap<String, Object> model );

    protected abstract Plugin getPlugin(  );

    /**
     * Rcupre la page de gestion des modles d'ordre du jour
     *@param request la requete
     * @return la page de gestion des modeles
     */
    public HashMap<String, Object> getModelesOdjList( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        HashMap<String, Object> model = new HashMap<String, Object>(  );
        List<GModeleOrdreDuJour> modeleOrdreDuJours = _modeleOrdreDuJourHome.findModeleOrdreDuJourList( plugin );

        getModeleOdjListSpec( request, model );

        model.put( MARK_LIST_MODELE, modeleOrdreDuJours );

        return model;
    }

    /**
     * retourne l'interface de creation  d'un modle d'ordre du jour
     *@param request la requete
     *@return la page de creation d'un modele d'ordre du jour
     */
    public HashMap<String, Object> getCreationModeleOdj( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        ReferenceList referenceListFormation = _formationConseilService.initRefListFormationConseil( true );
        ReferenceList referenceListType = initRefListType( plugin );

        getCreationModeleOdjSpec( request, model );

        model.put( MARK_LISTE_FORMATION_CONSEIL, referenceListFormation );
        model.put( MARK_LISTE_TYPE, referenceListType );

        return model;
    }

    /**
     * retourne l'interface de modification du modle d'ordre du jour selectionn
     *@param request la requete
     *@return l'interface de modification du modle d'ordre du jour selectionn
     */
    public HashMap<String, Object> getModificationModeleOdj( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        int nIdModele = OdsUtils.getIntegerParameter( OdsParameters.ID_MODELE, request );
        GModeleOrdreDuJour modeleOrdreDuJour = _modeleOrdreDuJourHome.findByPrimaryKey( nIdModele, plugin );
        OdsUtils.testNullObject( modeleOrdreDuJour, request );

        ReferenceList referenceListFormation = _formationConseilService.initRefListFormationConseil( true );
        ReferenceList referenceListType = initRefListType( plugin );

        getModificationModeleOdjSpec( request, model );

        model.put( MARK_MODELE, modeleOrdreDuJour );
        model.put( MARK_LISTE_FORMATION_CONSEIL, referenceListFormation );
        model.put( MARK_LISTE_TYPE, referenceListType );

        return model;
    }

    /**
     * cre un modele d'ordre du jour apartir des parametres contenus dans la requete
     *@param request la requete
     *@return retour sur l'interface de gestion des modeles d'ordre du jour
     */
    public String doCreationModeleOdj( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        String strReturn = null;

        GModeleOrdreDuJour modeleOrdreDuJour = newModeleOrdreDuJourInstance(  );
        modeleOrdreDuJour.setIdModele( -1 );

        String strMessage = testField( request, modeleOrdreDuJour );

        if ( !strMessage.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = doTraitementMessage( strMessage, request );

            if ( ( strReturn == null ) || strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                if ( strMessage.equals( MESSAGE_MODELE_ALREADY_EXIST ) )
                {
                    strReturn = AdminMessageService.getMessageUrl( request, strMessage, AdminMessage.TYPE_STOP );
                }
                else if ( strMessage.equals( OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND ) )
                {
                    strReturn = AdminMessageService.getMessageUrl( request,
                            OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND, AdminMessage.TYPE_ERROR );
                }
                else
                {
                    Object[] tabRequiredFields = { I18nService.getLocalizedString( strMessage, request.getLocale(  ) ) };

                    strReturn = AdminMessageService.getMessageUrl( request,
                            OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED, tabRequiredFields, AdminMessage.TYPE_STOP );
                }
            }
        }
        else
        {
            MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
            FileItem fileItemEnteteDocument = multipartRequest.getFile( OdsParameters.ENTETE_DOCUMENT );
            FileItem fileItemPiedDocument = multipartRequest.getFile( OdsParameters.PIED_DOCUMENT );

            if ( ( null != fileItemEnteteDocument ) && ( null != ( fileItemEnteteDocument.getName(  ) ) ) &&
                    !( OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItemEnteteDocument.getName(  ) ) ) )
            {
                if ( !isValideFile( fileItemEnteteDocument ) )
                {
                    strReturn = AdminMessageService.getMessageUrl( request, MESSAGE_MODELE_ENTETE_NO_XML,
                            AdminMessage.TYPE_STOP );
                }
                else
                {
                    modeleOrdreDuJour.setEnteteDocument( doCreationFichierPhysique( fileItemEnteteDocument ) );
                }
            }

            if ( ( strReturn == null ) && ( null != fileItemPiedDocument ) &&
                    ( null != fileItemPiedDocument.getName(  ) ) &&
                    !( OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItemPiedDocument.getName(  ) ) ) )
            {
                if ( !isValideFile( fileItemPiedDocument ) )
                {
                    strReturn = AdminMessageService.getMessageUrl( request, MESSAGE_MODELE_PIED_NO_XML,
                            AdminMessage.TYPE_STOP );
                }
                else
                {
                    modeleOrdreDuJour.setPiedDocument( doCreationFichierPhysique( fileItemPiedDocument ) );
                }
            }

            if ( strReturn == null )
            {
                _modeleOrdreDuJourHome.create( modeleOrdreDuJour, plugin );

                strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;
            }
        }

        return strReturn;
    }

    /**
     * modifie  un modele d'ordre du jour a partir des parametres contenus dans la requete
     *@param request la requete
     *@return retour sur l'interface de gestion des modeles d'ordre du jour
     */
    public String doModificationModeleOdj( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        if ( request.getParameter( OdsParameters.ID_MODELE ) != null )
        {
            int nIdModele = OdsUtils.getIntegerParameter( OdsParameters.ID_MODELE, request );

            GModeleOrdreDuJour modeleOrdreDuJour = _modeleOrdreDuJourHome.findByPrimaryKey( nIdModele, plugin );

            if ( modeleOrdreDuJour == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            String strMessage = testField( request, modeleOrdreDuJour );

            if ( !strMessage.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                String strReturn = doTraitementMessage( strMessage, request );

                if ( ( strReturn == null ) || strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                {
                    if ( strMessage.equals( MESSAGE_MODELE_ALREADY_EXIST ) )
                    {
                        strReturn = AdminMessageService.getMessageUrl( request, strMessage, AdminMessage.TYPE_STOP );
                    }
                    else if ( strMessage.equals( OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND ) )
                    {
                        strReturn = AdminMessageService.getMessageUrl( request,
                                OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND, AdminMessage.TYPE_ERROR );
                    }
                    else
                    {
                        Object[] tabRequiredFields = 
                            {
                                I18nService.getLocalizedString( strMessage, request.getLocale(  ) ),
                            };

                        strReturn = AdminMessageService.getMessageUrl( request,
                                OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED, tabRequiredFields, AdminMessage.TYPE_STOP );
                    }
                }

                return strReturn;
            }
            else
            {
                MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
                FileItem fileItemEnteteDocument = multipartRequest.getFile( OdsParameters.ENTETE_DOCUMENT );
                FileItem fileItemPiedDocument = multipartRequest.getFile( OdsParameters.PIED_DOCUMENT );

                String strReturn = null;

                if ( ( null != fileItemEnteteDocument ) && ( null != ( fileItemEnteteDocument.getName(  ) ) ) &&
                        !( OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItemEnteteDocument.getName(  ) ) ) &&
                        !isValideFile( fileItemEnteteDocument ) )
                {
                    strReturn = AdminMessageService.getMessageUrl( request, MESSAGE_MODELE_ENTETE_NO_XML,
                            AdminMessage.TYPE_STOP );
                }
                else if ( ( null != fileItemEnteteDocument ) && ( null != ( fileItemEnteteDocument.getName(  ) ) ) &&
                        !( OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItemEnteteDocument.getName(  ) ) ) &&
                        ( null == modeleOrdreDuJour.getEnteteDocument(  ) ) )
                {
                    modeleOrdreDuJour.setEnteteDocument( doCreationFichierPhysique( fileItemEnteteDocument ) );
                }
                else if ( ( null != fileItemEnteteDocument ) && ( null != ( fileItemEnteteDocument.getName(  ) ) ) &&
                        !( OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItemEnteteDocument.getName(  ) ) ) )
                {
                    FichierPhysique fichierEntete = modeleOrdreDuJour.getEnteteDocument(  );
                    doModificationFichierPhysique( fileItemEnteteDocument, fichierEntete );
                }

                if ( ( strReturn == null ) && ( null != fileItemPiedDocument ) &&
                        ( null != fileItemPiedDocument.getName(  ) ) &&
                        !( OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItemPiedDocument.getName(  ) ) ) &&
                        ( !isValideFile( fileItemPiedDocument ) ) )
                {
                    strReturn = AdminMessageService.getMessageUrl( request, MESSAGE_MODELE_PIED_NO_XML,
                            AdminMessage.TYPE_STOP );
                }
                else if ( ( strReturn == null ) && ( null != fileItemPiedDocument ) &&
                        ( null != fileItemPiedDocument.getName(  ) ) &&
                        !( OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItemPiedDocument.getName(  ) ) ) &&
                        ( null == modeleOrdreDuJour.getPiedDocument(  ) ) )
                {
                    modeleOrdreDuJour.setPiedDocument( doCreationFichierPhysique( fileItemPiedDocument ) );
                }
                else if ( ( strReturn == null ) && ( null != fileItemPiedDocument ) &&
                        ( null != fileItemPiedDocument.getName(  ) ) &&
                        !( OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItemPiedDocument.getName(  ) ) ) )
                {
                    FichierPhysique fichierPiedDocument = modeleOrdreDuJour.getPiedDocument(  );
                    doModificationFichierPhysique( fileItemPiedDocument, fichierPiedDocument );
                }

                if ( strReturn != null )
                {
                    return strReturn;
                }
            }

            _modeleOrdreDuJourHome.update( modeleOrdreDuJour, plugin );
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    protected abstract String doTraitementMessage( String strMessage, HttpServletRequest request );

    /**
     * Retourne une demande pour la suppression d'un modele choisi
     * @param request la requete HTTP
     * @return interface de confirmation de suppression
     */
    public String getSuppressionModele( HttpServletRequest request )
    {
        if ( request.getParameter( OdsParameters.ID_MODELE ) == null )
        {
            return OdsConstants.CONSTANTE_CHAINE_VIDE;
        }

        String strIdModele = request.getParameter( OdsParameters.ID_MODELE );
        UrlItem url = new UrlItem( JSP_URL_DO_SUPPRESSION_MODELE_JSP );
        url.addParameter( OdsParameters.ID_MODELE, strIdModele );

        return AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRME_DELETE_MODELE, url.getUrl(  ),
            AdminMessage.TYPE_CONFIRMATION );
    }

    /**
     * supprime l'lu choisi
     * @param request la requete HTTP
     * @return url de gestion des groupes poliques
     */
    public String doSuppressionModele( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        if ( null != request.getParameter( OdsParameters.ID_MODELE ) )
        {
            int nIdModele = OdsUtils.getIntegerParameter( OdsParameters.ID_MODELE, request );

            GModeleOrdreDuJour modeleOrdreDuJour = _modeleOrdreDuJourHome.findByPrimaryKey( nIdModele, plugin );

            if ( modeleOrdreDuJour == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            try
            {
                // Suppression du modle
                _modeleOrdreDuJourHome.remove( modeleOrdreDuJour, plugin );

                if ( null != modeleOrdreDuJour.getEnteteDocument(  ) )
                {
                    _fichierPhysiqueHome.remove( modeleOrdreDuJour.getEnteteDocument(  ), plugin );
                }

                if ( null != modeleOrdreDuJour.getPiedDocument(  ) )
                {
                    _fichierPhysiqueHome.remove( modeleOrdreDuJour.getPiedDocument(  ), plugin );
                }
            }
            catch ( AppException ae )
            {
                AppLogService.error( ae );

                if ( ae.getInitialException(  ) instanceof SQLException )
                {
                    return AdminMessageService.getMessageUrl( request, MESSAGE_CANNOT_DELETE_MODELE,
                        AdminMessage.TYPE_STOP );
                }
            }
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * Rcupre un fichier  partir de son identifiant
     * @return le fichier
     */
    public FichierPhysique getFichier( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        String strIdFichier = request.getParameter( OdsParameters.ID_FICHIER );
        FichierPhysique fichierPhysique = null;

        if ( strIdFichier != null )
        {
            try
            {
                int nIdFichier = Integer.parseInt( strIdFichier );
                fichierPhysique = _fichierPhysiqueHome.findByPrimaryKey( nIdFichier, plugin );
            }

            catch ( NumberFormatException ne )
            {
                AppLogService.error( ne );
            }
        }

        return fichierPhysique;
    }

    /**
     * Permet d'initialiser la liste des choix de type de conseil
     * @param plugin le plugin
     * @return la liste des types d'ordre du jour
     */
    private ReferenceList initRefListType( Plugin plugin )
    {
        ReferenceList refListType = new ReferenceList(  );
        List<TypeOrdreDuJour> listType = _typeOrdreDuJourHome.findTypeOrdreDuJourList( plugin );
        // Ajout dans la referenceList de l'item chaine vide
        refListType.addItem( -1, OdsConstants.CONSTANTE_CHAINE_VIDE );

        // Ajout dans la referenceList de l'ensemble des items reprsentant les groupes politiques
        for ( TypeOrdreDuJour type : listType )
        {
            refListType.addItem( type.getIdTypeOrdreDuJour(  ), OdsConstants.CONSTANTE_CHAINE_VIDE +
                type.getLibelle(  ) );
        }

        return refListType;
    }

    /**
     * Retourne le premier champ requis manquant s'il y'en a .
     * Retourne un message d'erreur si les parametres saisis ne conviennent pas
     * @param request  la requete HTTP
     * @param modeleOrdreDuJour le modele d'ordre du jour
     * @return boolean retourne true si l'ensemble des critres obligatoires ont t renseign
     */
    private String testField( HttpServletRequest request, GModeleOrdreDuJour modeleOrdreDuJour )
    {
        Plugin plugin = getPlugin(  );
        String strRequiredField = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( isValideForme( request ) )
        {
            FormationConseil formationConseil = new FormationConseil(  );
            TypeOrdreDuJour typeOrdreDuJour = new TypeOrdreDuJour(  );
            int nIdFormationConseil;
            int nIdType;

            String strTitre = request.getParameter( OdsParameters.TITRE ).trim(  );

            try
            {
                nIdFormationConseil = Integer.parseInt( request.getParameter( OdsParameters.ID_FORMATION_CONSEIL ) );
                nIdType = Integer.parseInt( request.getParameter( OdsParameters.ID_TYPE_ORDRE_DU_JOUR ) );
            }
            catch ( NumberFormatException ne )
            {
                AppLogService.error( ne );

                return strRequiredField;
            }

            if ( strTitre.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                strRequiredField = PROPERTY_LABEL_TITRE;
            }
            else if ( nIdType == -1 )
            {
                strRequiredField = PROPERTY_LABEL_TYPE;
            }
            else if ( nIdFormationConseil == -1 )
            {
                strRequiredField = PROPERTY_LABEL_FORMATION;
            }

            // On insre les paramtres dans l'objet pass en paramtre
            modeleOrdreDuJour.setTitre( strTitre );
            formationConseil.setIdFormationConseil( nIdFormationConseil );
            modeleOrdreDuJour.setFormationConseil( formationConseil );

            typeOrdreDuJour.setIdTypeOrdreDuJour( nIdType );
            modeleOrdreDuJour.setTypeOrdreDuJour( typeOrdreDuJour );

            try
            {
                if ( strRequiredField.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                {
                    strRequiredField = testFieldSpec( request, modeleOrdreDuJour, nIdType, nIdFormationConseil, plugin );
                }
            }
            catch ( NumberFormatException ne )
            {
                AppLogService.error( ne );

                return strRequiredField;
            }
        }
        else
        {
            strRequiredField = PROPERTY_LABEL_TITRE;
        }

        return strRequiredField;
    }

    /**
     * Verifie et remplit les champs. Doit vrifier si le modle n'existe pas dj
     * @param request HttpServletRequest
     * @param modeleOrdreDuJour le modele
     * @return chaine vide si aucune erreur, message si erreur
     * @throws NumberFormatException si format nombre incorrect
     */
    protected abstract String testFieldSpec( HttpServletRequest request, GModeleOrdreDuJour modeleOrdreDuJour,
        int nIdType, int nIdFormationConseil, Plugin plugin )
        throws NumberFormatException;

    /**
     * methode qui test la validit  du formulaire de saisi ou de modification d'un MODJ,
     ** @param request  la requete HTTP
     * @return boolean
     */
    private boolean isValideForme( HttpServletRequest request )
    {
        if ( ( null != request.getParameter( OdsParameters.TITRE ) ) &&
                ( ( null != request.getParameter( OdsParameters.ID_TYPE_ORDRE_DU_JOUR ) ) &&
                ( null != request.getParameter( OdsParameters.ID_FORMATION_CONSEIL ) ) ) )
        {
            return true;
        }

        return false;
    }

    /**
     * test si le fichier fourni est valide
     * @param fileItem le fichier a tester
     * @return true si le fichier est valide  ont
     */
    private boolean isValideFile( FileItem fileItem )
    {
        boolean nIsFileGood = true;
        String extensionFile = fileItem.getName(  ).substring( fileItem.getName(  ).length(  ) - 3 ).toUpperCase(  );

        if ( !OdsConstants.CONSTANTE_XML.equals( extensionFile ) )
        {
            nIsFileGood = false;
        }

        return nIsFileGood;
    }

    /**
     * Mthode qui cre le  fichier rcupr de la requette
     * @param fileItem le fichier associ au voeu(rcupr de la requette )
     * @return Fichier cr
     */
    private FichierPhysique doCreationFichierPhysique( FileItem fileItem )
    {
        Plugin plugin = getPlugin(  );

        FichierPhysique newFichierPhysique = new FichierPhysique(  );
        // Cration de l'objet reprsentant le fichier physique
        newFichierPhysique.setDonnees( fileItem.get(  ) );
        newFichierPhysique.setIdFichier( _fichierPhysiqueHome.create( newFichierPhysique, plugin ) );

        return newFichierPhysique;
    }

    /**
     * Modifie le fichier et le fichier physique li au voeuAmendement
     * @param fileItem le fichier
     * @param fichierPhysique le fichier physique
     */
    private void doModificationFichierPhysique( FileItem fileItem, FichierPhysique fichierPhysique )
    {
        Plugin plugin = getPlugin(  );

        // Modification de l'objet reprsentant le fichier physique
        fichierPhysique.setDonnees( fileItem.get(  ) );
        _fichierPhysiqueHome.update( fichierPhysique, plugin );
    }

    protected abstract GModeleOrdreDuJour newModeleOrdreDuJourInstance(  );
}
