/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.modeleordredujour;

import fr.paris.lutece.plugins.ods.dto.fichier.FichierPhysique;
import fr.paris.lutece.plugins.ods.service.IODSService;

import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;


public interface IModeleOrdreDuJourService extends IODSService
{
    /**
    * Rcupre la page de gestion des modles d'ordre du jour
    *@param request la requete
    * @return la page de gestion des modeles
    */
    HashMap<String, Object> getModelesOdjList( HttpServletRequest request );

    /**
     * retourne l'interface de creation  d'un modle d'ordre du jour
     *@param request la requete
     *@return la page de creation d'un modele d'ordre du jour
     */
    HashMap<String, Object> getCreationModeleOdj( HttpServletRequest request );

    /**
     * retourne l'interface de modification du modle d'ordre du jour selectionn
     *@param request la requete
     *@return l'interface de modification du modle d'ordre du jour selectionn
     */
    HashMap<String, Object> getModificationModeleOdj( HttpServletRequest request );

    /**
     * cre un modele d'ordre du jour apartir des parametres contenus dans la requete
     *@param request la requete
     *@return retour sur l'interface de gestion des modeles d'ordre du jour
     */
    String doCreationModeleOdj( HttpServletRequest request );

    /**
     * modifie  un modele d'ordre du jour a partir des parametres contenus dans la requete
     *@param request la requete
     *@return retour sur l'interface de gestion des modeles d'ordre du jour
     */
    String doModificationModeleOdj( HttpServletRequest request );

    /**
     * Retourne une demande pour la suppression d'un modele choisi
     * @param request la requete HTTP
     * @return interface de confirmation de suppression
     */
    String getSuppressionModele( HttpServletRequest request );

    /**
     * supprime l'lu choisi
     * @param request la requete HTTP
     * @return url de gestion des groupes poliques
     */
    String doSuppressionModele( HttpServletRequest request );

    /**
     * Rcupre un fichier  partir de son identifiant
     * @return le fichier
     */
    FichierPhysique getFichier( HttpServletRequest request );
}
