/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.notification;

import fr.paris.lutece.plugins.ods.business.pdd.IPDDHome;
import fr.paris.lutece.plugins.ods.business.seance.ISeanceHome;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.pdd.IArrondissement;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDDFilter;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.utilisateur.Utilisateur;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendementFilter;
import fr.paris.lutece.plugins.ods.service.horodatage.IHorodatageService;
import fr.paris.lutece.plugins.ods.service.utilisateur.IUtilisateurService;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.service.util.AppPropertiesService;

import org.springframework.beans.factory.annotation.Autowired;

import java.sql.Timestamp;

import java.util.Collection;

import javax.servlet.http.HttpServletRequest;


public abstract class AbstractNotificationService<GSeance extends ISeance, GSeanceFilter extends ISeanceFilter, GSeanceHome extends ISeanceHome<GSeance, GSeanceFilter, GFichier, GElu, GVoeuAmendement, GPDD>, GFichier extends IFichier<GSeance, GFichier>, GElu extends IElu, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GPDDFilter extends IPDDFilter, GVoeuAmendementFilter extends IVoeuAmendementFilter, GArrondissement extends IArrondissement>
    implements INotificationService<GSeance, GFichier>
{
    protected static final String MARK_LISTE_PDDS = "liste_pdds";
    protected static final String MARK_LIST_RESULTATS = "liste_resultats";
    protected static final String MARK_TITLE_MAIL = "title_mail";
    protected static final String MARK_ODS = "type_ods";
    protected static final String TEMPLATE_NOTIFICATION_FICHIER = "admin/plugins/ods/notification/notification_nouveau_fichier.html";
    protected static final String SEPARATOR_MAIL_LIST = AppPropertiesService.getProperty( "mail.list.separator" );
    protected static final String TYPE_REQUETE_SIMPLE = "simple";
    @Autowired
    protected IHorodatageService _horodatageService;
    @Autowired
    protected GSeanceHome _seanceHome;
    @Autowired
    protected IUtilisateurService _utilisateurService;
    @Autowired
    protected IPDDHome<GPDD, GPDDFilter, GVoeuAmendementFilter, GSeance, GElu, GVoeuAmendement, GFichier, GArrondissement> _projetDeliberationHome;

    protected abstract Plugin getPluginInstance(  );

    protected abstract void notifierPddUtilisateurs( Collection<Utilisateur> listUtilisateurs, String strType,
        Timestamp tsDateDerniereNotification, Plugin plugin );

    protected abstract void notifierUtilisateurs( Collection<Utilisateur> listUtilisateurs,
        Timestamp tsDateDerniereNotification );

    protected abstract boolean isProchaineSeanceExist(  );

    protected abstract String getTitleMail(  );

    /**
     * Notifie les requetes dont les rsultats ont t modifis aux utilisateurs
     * @param listeUtilisateurs la liste des utilisateurs
     * @param tsDateDerniereNotification la date de la dernire notification
     */
    protected abstract void notificationRequetesUtilisateurs( Collection<Utilisateur> listeUtilisateurs,
        Timestamp tsDateDerniereNotification );

    /**
     * Lance l'execution du traitement.
     *
     * @param request la requte HTTP
     * @return 0 si OK, 10 sinon
     */
    public String doNotification( HttpServletRequest request )
    {
        /*if ( isProchaineSeanceExist(  ) )
        {
            run(  );
        }
        else
        {
            AppLogService.error( "Notification : Aucune prochaine sance n'existe" );
        
            return OdsConstants.RESULT_KO;
        }*/
        run(  );

        return OdsConstants.RESULT_OK;
    }

    /**
     * Lance la notification.
     *
     * @param plugin le plugin
     */
    private void run(  )
    {
        AppLogService.info( "odsNotification - start of process." );

        Plugin plugin = getPluginInstance(  );

        /*
         * On rcupre la date de dernire excution du dmon de notification.
         */
        Timestamp tsDateDerniereNotification = _horodatageService.getLastNotification( plugin );

        /*
         * On rcupre la liste de tous les utilisateurs
         */
        Collection<Utilisateur> listeUtilisateurs = _utilisateurService.getUserFrontList( plugin );

        /*
         * Notification des nouveaux projets et propositions de dlibration de la prochaine seance.
         */
        if ( isProchaineSeanceExist(  ) )
        {
            notifierUtilisateurs( listeUtilisateurs, tsDateDerniereNotification );

            /*notifierPddUtilisateurs( listeUtilisateurs, IPDD.CONSTANTE_TYPE_PROJET, tsDateDerniereNotification, plugin );
            notifierPddUtilisateurs( listeUtilisateurs, IPDD.CONSTANTE_TYPE_PROP, tsDateDerniereNotification, plugin );*/
        }
        else
        {
            AppLogService.error( "Notification : Aucune prochaine sance n'existe - Pas de notification des PDDs" );
        }

        /*
         * Notification des nouveaux PDD, voeux et amendements de la prochaine sance
         * rpondants aux requtes personnelles.
         */
        notificationRequetesUtilisateurs( listeUtilisateurs, tsDateDerniereNotification );

        AppLogService.info( "odsNotification - end of process." );
    }

    /**
     * Construit la liste des adresses mails d'un utilisateur
     * @param user l'utilisateur concern
     * @return listeAdressesMails la liste des adresses mails de l'utilisateur
     */
    protected String getAddressesMails( Utilisateur user )
    {
        // Si l'utilisateur a des adresses mails en copie on les ajoute  l'adresse principale 
        // en le sparant par un point-virgule dans l'adresse des destinataires
        String listeAdressesMails = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( user.getMail(  ) != null )
        {
            listeAdressesMails += user.getMail(  ).trim(  );
        }

        if ( user.getMailCopie1(  ) != null )
        {
            if ( !listeAdressesMails.trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                listeAdressesMails += ( SEPARATOR_MAIL_LIST );
            }

            listeAdressesMails += ( user.getMailCopie1(  ) );
        }

        if ( user.getMailCopie2(  ) != null )
        {
            if ( !listeAdressesMails.trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                listeAdressesMails += ( SEPARATOR_MAIL_LIST );
            }

            listeAdressesMails += ( user.getMailCopie2(  ) );
        }

        return listeAdressesMails;
    }
}
