/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.ordredujour;

import fr.paris.lutece.plugins.ods.business.elu.IEluHome;
import fr.paris.lutece.plugins.ods.business.expression.IExpressionUsuelleHome;
import fr.paris.lutece.plugins.ods.business.fichier.FichierPhysiqueHome;
import fr.paris.lutece.plugins.ods.business.fichier.IFichierHome;
import fr.paris.lutece.plugins.ods.business.formationconseil.FormationConseilHome;
import fr.paris.lutece.plugins.ods.business.historique.HistoriqueHome;
import fr.paris.lutece.plugins.ods.business.modeleordredujour.IModeleOrdreDuJourHome;
import fr.paris.lutece.plugins.ods.business.ordredujour.IEntreeOrdreDuJourHome;
import fr.paris.lutece.plugins.ods.business.ordredujour.IOrdreDuJourHome;
import fr.paris.lutece.plugins.ods.business.ordredujour.generated.entete.Entete;
import fr.paris.lutece.plugins.ods.business.ordredujour.generated.pieddepage.PiedDePage;
import fr.paris.lutece.plugins.ods.business.pdd.IPDDHome;
import fr.paris.lutece.plugins.ods.business.seance.ISeanceHome;
import fr.paris.lutece.plugins.ods.business.voeuamendement.IVoeuAmendementHome;
import fr.paris.lutece.plugins.ods.dto.IDepositaire;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.expression.IExpressionUsuelle;
import fr.paris.lutece.plugins.ods.dto.fichier.FichierPhysique;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter;
import fr.paris.lutece.plugins.ods.dto.fichier.ITypeDocument;
import fr.paris.lutece.plugins.ods.dto.groupepolitique.GroupePolitique;
import fr.paris.lutece.plugins.ods.dto.modeleordredujour.IModeleOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter;
import fr.paris.lutece.plugins.ods.dto.pdd.IArrondissement;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDDFilter;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.typeentite.ITypeEntite;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendementFilter;
import fr.paris.lutece.plugins.ods.service.formationconseil.IFormationConseilService;
import fr.paris.lutece.plugins.ods.service.manager.DocumentManager;
import fr.paris.lutece.plugins.ods.service.notification.INotificationService;
import fr.paris.lutece.plugins.ods.service.search.ISearchService;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.business.OrdreDuJourUtils;
import fr.paris.lutece.plugins.ods.utils.commons.DateUtils;
import fr.paris.lutece.plugins.ods.utils.commons.StringUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.template.AppTemplateService;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.service.util.AppPathService;
import fr.paris.lutece.portal.service.util.AppPropertiesService;
import fr.paris.lutece.portal.web.constants.Messages;
import fr.paris.lutece.util.ReferenceList;
import fr.paris.lutece.util.html.HtmlTemplate;
import fr.paris.lutece.util.url.UrlItem;

import net.sf.jcopist.service.exception.ServerException;
import net.sf.jcopist.service.exception.UnavailableException;

import org.springframework.beans.factory.BeanCreationException;
import org.springframework.beans.factory.annotation.Autowired;

import org.springframework.remoting.RemoteConnectFailureException;

import java.io.IOException;
import java.io.UnsupportedEncodingException;

import java.sql.SQLException;
import java.sql.Timestamp;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Map.Entry;

import javax.servlet.http.HttpServletRequest;

import javax.xml.bind.JAXBException;


public abstract class AbstractOrdreDuJourService<GOrdreDuJour extends IOrdreDuJour<GEntreeOrdreDuJour, GSeance, GFichier, GElu, GOrdreDuJour>, GEntreeOrdreDuJour extends IEntreeOrdreDuJour<GElu, GSeance, GFichier, GEntreeOrdreDuJour, GOrdreDuJour>, GOrdreDuJourFilter extends IOrdreDuJourFilter, GEntreeOrdreDuJourHome extends IEntreeOrdreDuJourHome<GOrdreDuJour, GEntreeOrdreDuJour, GElu, GSeance, GFichier>, GOrdreDuJourHome extends IOrdreDuJourHome<GOrdreDuJour, GEntreeOrdreDuJour, GOrdreDuJourFilter, GSeance, GElu, GFichier>, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GVoeuAmendementFilter extends IVoeuAmendementFilter, GSeance extends ISeance, GElu extends IElu, GModeleOrdreDuJour extends IModeleOrdreDuJour, GFichier extends IFichier<GSeance, GFichier>, GFichierFilter extends IFichierFilter, GSeanceFilter extends ISeanceFilter, GOrdreDuJourServiceManaged extends IOrdreDuJourServiceManaged<GEntreeOrdreDuJour, GOrdreDuJour, GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GPDDFilter extends IPDDFilter, GArrondissement extends IArrondissement>
    implements IOrdreDuJourService<GOrdreDuJour, GEntreeOrdreDuJour, GOrdreDuJourFilter, GSeance, GElu, GFichier>
{
    private static final String JSP_SELECTION_EXPRESSION = "jsp/admin/plugins/ods/expression/ExpressionsForOdj.jsp";
    private static final String JSP_SELECTION_PROPOSITION = "jsp/admin/plugins/ods/projetdeliberation/PropositionDeliberationForOdjNoReinit.jsp";
    private static final String JSP_SELECTION_PROJET = "jsp/admin/plugins/ods/projetdeliberation/ProjetDeliberationForOdjNoReinit.jsp";
    private static final String JSP_SELECTION_VOEU = "jsp/admin/plugins/ods/voeuamendement/VoeuNonRattacheForOdj.jsp";
    protected static final String JSP_SELECTION_PROPOSITIONS = "jsp/admin/plugins/ods/projetdeliberation/PropositionsDeliberationForOdj.jsp";
    protected static final String JSP_SELECTION_PROJETS = "jsp/admin/plugins/ods/projetdeliberation/ProjetsDeliberationForOdj.jsp";
    private static final String JSP_SELECTION_VOEU_RETOUR = "jsp/admin/plugins/ods/ordredujour/DoSelectionVoeuAmendement.jsp";
    private static final String JSP_SELECTION_ENTREE_ORDRE_DU_JOUR = "jsp/admin/plugins/ods/ordredujour/SelectionEntreeOrdreDuJour.jsp";
    private static final String JSP_AJOUT_VOEU = "jsp/admin/plugins/ods/voeuamendement/CreationVoeuNonRattacheForOdj.jsp";
    private static final String JSP_AJOUT_VOEU_AVAL = "jsp/admin/plugins/ods/voeuamendement/CreationVoeuNonRattacheForOdjAval.jsp";
    private static final String JSP_DO_SUPPRESSION_ENTREE_JSP = "jsp/admin/plugins/ods/ordredujour/DoSuppressionEntreeOrdreDuJour.jsp";
    private static final String JSP_DO_SUPPRESSION_LISTE_ENTREE = "jsp/admin/plugins/ods/ordredujour/DoSuppressionListeEntreeOrdreDuJour.jsp";
    private static final String JSP_URL_DO_SUPPRESSION_ODJ = "jsp/admin/plugins/ods/ordredujour/DoSuppressionOrdreDuJour.jsp";
    private static final String JSP_MODIFICATION_ORDRE_DU_JOUR = "jsp/admin/plugins/ods/ordredujour/ModificationOrdreDuJour.jsp";
    private static final String JSP_MODIFICATION_ENTETE_PIED_DE_PAGE = "jsp/admin/plugins/ods/ordredujour/ModificationEntetePiedPage.jsp";
    private static final String JSP_DO_SUPPRESSION_ENTREE_ENTETE_PIED_DE_PAGE = "jsp/admin/plugins/ods/ordredujour/DoSuppressionEntreeEntetePiedPage.jsp";
    private static final String JSP_URL_DO_PUBLICATION_ODJ = "jsp/admin/plugins/ods/ordredujour/DoPublicationOrdreDuJour.jsp";
    private static final String JSP_URL_DO_DEPUBLICATION_ODJ = "jsp/admin/plugins/ods/ordredujour/DoDepublicationOrdreDuJour.jsp";
    protected static final String JSP_LISTE_ORDRE_DU_JOUR = "jsp/admin/plugins/ods/ordredujour/Ordresdujour.jsp";
    protected static final String JSP_LISTE_ORDRE_DU_JOUR_AVAL = "jsp/admin/plugins/ods/ordredujour/OrdresDuJourGestionAval.jsp";
    protected static final String PROPERTY_VA = "ods.odj.label.amendementEtVoeux";
    private static final String PROPERTY_ODJ_GROUPE_DE_RAPPORTEURS_STYLE = "ods.odj.groupeDeRapporteurs.style";
    private static final String PROPERTY_LABEL_INTITULE = "ods.odj.modificationOdj.label.intitule";
    protected static final String PROPERTY_COMMISSION = "ods.odj.creation.label.commission";
    protected static final String PROPERTY_ERE = "ods.all.label.ere";
    protected static final String PROPERTY_EME = "ods.all.label.eme";
    private static final String PROPERTY_ODJ_NOM_TEMPLATE = "ods.odj.nomtemplate";
    private static final String PROPERTY_LABEL_EXECUTIF = "ods.liasse.label.executif";
    private static final String PROPERTY_GENERATION_ODJ_TMP_DIRECTORY = "ods.genereODJ.tmp.directory";
    protected static final String CONSTANTE_STYLE_H2 = "2";
    private static final String CONSTANTE_STYLE_H4 = "4";
    private static final String CONSTANTE_STYLE = "ods.odj.creation.label.style";
    private static final String CONSTANTE_AUCUN_STYLE = "ods.odj.creation.label.aucun";
    private static final String CONSTANTE_ENTETE = "entete";
    private static final String CONSTANTE_PIED_DE_PAGE = "piedpage";
    protected static final String CONSTANTE_NOUVEL_ODJ = "ods.odj.liste.label.nouvelOdj";
    protected static final String CONSTANTE_SESSION = "session";
    protected static final String CONSTANTE_APPLIQUER = "appliquer";
    protected static final String CONSTANTE_ANNULER = "annuler";
    protected static final String CONSTANTE_CANCEL = "cancel";
    private static final String CONSTANTE_ENREGISTRER = "enregistrer";
    private static final String CONSTANTE_AJOUTER_RAPPORTEUR = "ajouter_rapporteur";
    private static final String CONSTANTE_AJOUTER_VOEU = "ajouter_voeu";
    private static final String CONSTANTE_AJOUTER_AVANT_SELECTION = "ajouter_avant_selection";
    private static final String CONSTANTE_AJOUTER_APRES_SELECTION = "ajouter_apres_selection";
    private static final String CONSTANTE_DEPLACER_SELECTION = "deplacer_selection";
    private static final String CONSTANTE_SUPPRIMER_SELECTION = "supprimer_selection";
    private static final String CONSTANTE_SELECTION_VOEU = "selectionner_voeu";
    private static final String CONSTANTE_AJOUTER_EXPRESSION = "ajouter_expression";
    private static final String CONSTANTE_DESELECTION_PDD = "deselectionner_pdd";
    private static final String CONSTANTE_DESELECTION_VOEU = "deselectionner_voeu";
    protected static final String CONSTANTE_TYPE_SUPPRESSION = "type_suppression";
    private static final String CONSTANTE_AJOUT_ENTREE = "ajoutEntree";
    private static final String CONSTANTE_RESET_ENTETE_PIED_DE_PAGE = "resetEntetePiedPage";
    private static final String CONSTANTE_TYPE_SUPPRESSION_ELU = "elu";
    protected static final String CONSTANTE_G = "-G";
    protected static final String CONSTANTE_EXTENSION_DOC = ".doc";
    private static final String CONSTANTE_ANCHOR_ENTREE_ODJ = "entree";
    private static final String MARK_ENTREE_ODJ = "entree_odj";
    protected static final String MARK_ID_FORMATION_CONSEIL_SELECTED = "id_formation_conseil_selected";
    private static final String MARK_ID_ODJ_SELECTED = "id_odj_selected";
    protected static final String MARK_LISTE_FORMATION_CONSEIL = "liste_formation_conseil";
    protected static final String MARK_LISTE_ORDRE_DU_JOUR = "liste_ordre_du_jour";
    protected static final String MARK_LISTE_RAPPORTEUR = "liste_rapporteur";
    private static final String MARK_LISTE_ID_NOUVELLE_ENTREE = "liste_nouvelle_entree";
    private static final String MARK_LISTE_SEANCES = "liste_seances";
    private static final String MARK_LISTE_STYLE = "liste_style";
    private static final String MARK_ID_SEANCE_SELECTED = "id_seance_selected";
    private static final String MARK_TEXT_SUPPRESSION = "text_suppression";
    private static final String MARK_SUPPRESSION = "suppression";
    private static final String MARK_URL = "url";
    private static final String MARK_ENTETE = "isEntete";
    private static final String MARK_PIED_DE_PAGE = "isPiedPage";
    protected static final String MARK_IS_FORMATION_CONSEIL_DISPLAY = "is_formation_conseil_display";
    private static final String MESSAGE_CONFIRME_DELETE_ENTREE_ENTETE = "ods.odj.EntetePiedPage.message.confirmeSuppressionEntete";
    private static final String MESSAGE_CONFIRME_DELETE_ENTREE_PIED_DE_PAGE = "ods.odj.EntetePiedPage.message.confirmeSuppressionPiedPage";
    private static final String MESSAGE_CONFIRME_DELETE_ENTREE = "ods.entreeodj.message.confirmeSuppression";
    private static final String MESSAGE_CONFIRME_DELETE_LISTE_ENTREE = "ods.entreeodj.message.confirmeSuppressionListe";
    private static final String MESSAGE_CONFIRME_DELETE_ODJ = "ods.odj.message.confirmeSuppression";
    private static final String MESSAGE_DELETE_ODJ = "ods.odj.message.suppression";
    private static final String MESSAGE_CONFIRMATION_TO_DELETE = "ods.odj.message.confirmationSuppression";
    private static final String MESSAGE_CONFIRMATION_PUBLICATION = "ods.odj.message.confirmPublication";
    private static final String MESSAGE_CONFIRMATION_DEPUBLICATION = "ods.odj.message.confirmDepublication";
    private static final String MESSAGE_CONFIRMATION_NOUVELLE_PUBLICATION = "ods.odj.message.confirmNouvellePublication";
    protected static final String MESSAGE_CANNOT_DELETE_ENTREE = "ods.entreeodj.message.suppressionImpossible";
    protected static final String MESSAGE_CANNOT_DEPUBLIER = "ods.odj.message.cannotDepublier";
    private static final String MESSAGE_ALREADY_DEPUBLIER = "ods.odj.message.alreadyDepublier";
    private static final String MESSAGE_PAS_DE_PROJET_CHOISIE = "ods.voeuamendement.message.pasdeprojetchoisie";
    private static final String MESSAGE_PAS_DE_PROPOSITION_CHOISIE = "ods.voeuamendement.message.pasdepropositionchoisie";
    private static final String MESSAGE_PAS_DE_PDD_CHOISI = "ods.entreeodj.message.pasdepddchoisi";
    private static final String MESSAGE_PAS_DE_VOEU_CHOISI = "ods.entreeodj.message.pasdevoeuchoisi";
    private static final String MESSAGE_PAS_EXPRESSION_CHOISI = "ods.entreeodj.message.pasexpressionchoisi";
    protected static final String MESSAGE_PAS_ENTREE_CHOISIE = "ods.odj.message.pasentreechoisie";
    private static final String MESSAGE_PROBLEME_DE_CREATION_ODJ_PDF = "ods.odj.message.problemeCreationOdjPdf";
    private static final String MESSAGE_NO_ENTREE_SELECTED = "ods.odj.message.noEntreeSelected";
    private static final String MESSAGE_AUCUNE_ENTREE = "ods.odj.message.aucuneEntree";
    private static final String MESSAGE_ODJ_DEJA_PUBLIE = "ods.odj.message.dejaPublie";
    private static final String MESSAGE_ODJ_DEJA_DEPUBLIE = "ods.odj.message.dejaDepublie";
    private static final String MESSAGE_CSV_SEANCE = "ods.liasse.csv.colonne.seance";
    private static final String MESSAGE_CSV_REFERENCE = "ods.liasse.csv.colonne.reference";
    private static final String MESSAGE_CSV_TYPE = "ods.liasse.csv.colonne.type";
    private static final String MESSAGE_CSV_ENTITE_DEPOSITAIRE = "ods.liasse.csv.colonne.entite_depositaire";
    private static final String MESSAGE_CSV_RAPPORTEUR = "ods.liasse.csv.colonne.rapporteur";
    private static final String MESSAGE_CSV_OBJET = "ods.liasse.csv.colonne.objet";
    private static final String MESSAGE_CSV_ELU_DEPOSITAIRE = "ods.liasse.csv.colonne.elu_depositaire";
    protected static final String MESSAGE_EXPORT_CSV_CONSEIL = "ods.odj.exportcsv.colonne.conseil";
    private static final String DEFAULT_ODJ_NOM_TEMPLATE = "ordre_du_jour";
    private static final String PARAM_CHECKBOX_DELETE = "checkbox_delete";

    /*
     * ********************* Variables de session *********************
     */
    protected int _nIdFormationConseilSelected = -1;
    private int _nIdOdjSelected = -1;
    protected int _nIdSeanceSelected = -1;
    protected GEntreeOrdreDuJour _entreeOrdreDuJourBean;
    private int _nIdPositionInsert = -1;
    protected String[] _strTabIdSelect;
    private List<Integer> _listIdNouvelleEntree;

    /*
     * ********************* Injection SPRING *********************
     */
    @Autowired
    protected INotificationService<GSeance, GFichier> _notificationService;
    @Autowired
    protected IFormationConseilService _formationConseilService;
    @Autowired
    protected FormationConseilHome _formationConseilHome;
    @Autowired
    protected ISearchService<?, ?, ?, ?, ?, ?, ?, ?> _searchService;
    @Autowired
    protected GOrdreDuJourHome _ordreDuJourHome;
    @Autowired
    private IEluHome<GElu> _eluHome;
    @Autowired
    private IExpressionUsuelleHome<IExpressionUsuelle> _expressionUsuelleHome;
    @Autowired
    protected IFichierHome<GFichier, GFichierFilter, ITypeDocument, GSeance> _fichierHome;
    @Autowired
    protected FichierPhysiqueHome _fichierPhysiqueHome;
    @Autowired
    protected IVoeuAmendementHome<GVoeuAmendement, GVoeuAmendementFilter, GSeance, GElu, GFichier, GPDD> _voeuAmendementHome;
    @Autowired
    protected HistoriqueHome _historiqueHome;
    @Autowired
    protected IModeleOrdreDuJourHome<GOrdreDuJourFilter, GModeleOrdreDuJour> _modeleOrdreDuJourHome;
    @Autowired
    protected GEntreeOrdreDuJourHome _entreeOrdreDuJourHome;
    @Autowired
    protected ISeanceHome<GSeance, GSeanceFilter, GFichier, GElu, GVoeuAmendement, GPDD> _seanceHome;
    @Autowired
    private IPDDHome<GPDD, GPDDFilter, GVoeuAmendementFilter, GSeance, GElu, GVoeuAmendement, GFichier, GArrondissement> _projetDeliberationHome;
    protected Map<Integer, GOrdreDuJourServiceManaged> _mappingService;

    public Map<Integer, GOrdreDuJourServiceManaged> getMappingService(  )
    {
        return _mappingService;
    }

    public void setMappingService( Map<Integer, GOrdreDuJourServiceManaged> mappingService )
    {
        this._mappingService = mappingService;
    }

    public void initSession(  )
    {
        _nIdSeanceSelected = -1;
    }

    /*
     * ********************* Methodes abstraites *********************
     */
    protected abstract Plugin getPluginSpec(  );

    protected abstract GEntreeOrdreDuJour getEntreeOrdreDuJour(  );

    protected abstract GOrdreDuJour createNewOdj(  );

    protected abstract GOrdreDuJourFilter createNewOdjFilter(  );

    protected abstract void isCommissionDisplayed( HashMap<String, Object> model );

    protected abstract void getModificationOrdreDuJourSpec( HttpServletRequest request, GOrdreDuJour ordreDuJour,
        GSeance seance, HashMap<String, Object> model );

    protected abstract String setXmlEnteteSpec( Entete entete, GSeance seance );

    protected abstract String setXmlPiedPageSpec( PiedDePage piedPage, GSeance seance );

    protected abstract Entete getEnteteFromModeleSpec( GOrdreDuJour ordreDuJour );

    protected abstract PiedDePage getPiedDePageFromModeleSpec( GOrdreDuJour ordreDuJour );

    protected abstract void genererFrontHtmlSpec( GOrdreDuJour ordreDuJour, HashMap<String, Object> model );

    protected abstract String genererFrontHtmlTemplateSpec( GOrdreDuJour ordreDuJour );

    protected abstract void getInitEntreeOrdreDuJourSpec( HashMap<String, Object> model,
        GOrdreDuJourServiceManaged service, Locale locale );

    protected abstract String doTraitementEntreeOrdreDuJourSpec( HttpServletRequest request, boolean bIsForCreation,
        String strUrlRetour );

    protected abstract String doTraitementEntreeCommissionSpec( HttpServletRequest request, boolean bIsForCreation );

    protected abstract String doTraitementEntreeOrdreDuJourEluSpec( HttpServletRequest request, GElu elu );

    protected abstract void doModificationOrdreDuJourSpec( HttpServletRequest request, GOrdreDuJour ordreDuJour );

    protected abstract void doClasseEntree( Locale locale, GOrdreDuJour ordreDuJour );

    protected abstract void insertRuptureInTheOdjList( GOrdreDuJour ordreDuJour, boolean bIsSauvBase, Locale locale );

    protected abstract int getPositionNewEntree( GEntreeOrdreDuJour entree, GOrdreDuJour ordreDuJour );

    protected abstract void createSauvegardeOdjSpec( GEntreeOrdreDuJour entree );

    protected abstract String doPublicationSpec( HttpServletRequest request, GFichier newFichier,
        GOrdreDuJour ordreDuJour, Timestamp dateForVersion );

    protected abstract void doPublicationNotificationSpec( HttpServletRequest request, GOrdreDuJour ordreDuJour );

    protected abstract void doPublicationExterneSpec( GOrdreDuJour ordreDuJour );

    protected abstract void doDepublicationExterneSpec( GOrdreDuJour ordreDuJour );

    protected abstract String getDepublicationOrdreDuJourSpec( HttpServletRequest request, GOrdreDuJour ordreDuJour );

    protected abstract void doNumeroteVaSpec( GOrdreDuJour ordreDuJour );

    protected abstract String doGenereODJSpec( HttpServletRequest request, String strTmpPath, boolean bIsConversionOK );

    protected abstract StringBuffer insertEnteteInThePdfLiasseSpec( Locale locale, GOrdreDuJour ordreDuJour );

    protected abstract String getLibelleSeance( GSeance seance );

    protected abstract void setEntreeOrdreDuJourSpec( HttpServletRequest request );

    protected abstract void setEntreeDataSpec( GEntreeOrdreDuJour newEntree, GEntreeOrdreDuJour entreeOrdreDuJour );

    protected abstract void setEntreeOtherDataSpec( GEntreeOrdreDuJour newEntree, GEntreeOrdreDuJour entreeOrdreDuJour );

    protected abstract StringBuffer initRefListRapporteurSpec( GElu elu, List<GElu> listRapporteur, Locale locale );

    protected abstract String getLibelleGroupeRapporteursSpec( List<GElu> rapporteurs, Locale locale );

    protected abstract int getTypeEntreeRapporteurs(  );

    protected abstract void getOrdreDuJourGestionAvalList( HashMap<String, Object> model,
        List<GOrdreDuJour> listOrdreDuJour );

    protected abstract String doTraitementEntreeOdjSpec( HttpServletRequest request, boolean bIsEntite, String strIdOdj );

    protected abstract String doTraitementOdjSpec( HttpServletRequest request, String strAction, int nIdOdj );

    protected abstract void doSelectionPDDSpec( GPDD pdd, GSeance seance );

    protected abstract String getParameterPluginName(  );

    /**
     * Retourne l'URL de selection d'une expression usuelle
     * @return l'URL de selection d'une expression usuelle
     */
    public String getUrlSelectionExpression(  )
    {
        return JSP_SELECTION_EXPRESSION;
    }

    /**
     * methode qui initialise la liste des lus  afficher en fonction de
     * l'entre d'ordre du jour passe en parametre pass en parametre
     *
     * @param entree
     *            entree
     * @param plugin
     *            Plugin
     * @return ReferenceList la liste des diffrents elus a afficher
     */
    private ReferenceList initRefListRapporteur( HttpServletRequest request, List<GElu> listRapporteur, Plugin plugin )
    {
        List<GElu> listElu = _eluHome.findElusActifs( plugin );

        ReferenceList refListElu = new ReferenceList(  );
        refListElu.addItem( -1, OdsConstants.CONSTANTE_CHAINE_VIDE );

        for ( GElu elu : listElu )
        {
            StringBuffer strLibelleElu = initRefListRapporteurSpec( elu, listRapporteur, request.getLocale(  ) );

            if ( !strLibelleElu.toString(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                refListElu.addItem( elu.getIdElu(  ), strLibelleElu.toString(  ) );
            }
        }

        return refListElu;
    }

    /**
     * methode qui initialise la liste des styles  afficher pour
     * l'cran de modification des enttes et pied de page
     *
     * @return ReferenceList la liste des styles  afficher
     */
    private ReferenceList initRefListStyle( HttpServletRequest request )
    {
        ReferenceList refListStyle = new ReferenceList(  );
        refListStyle.addItem( -1, I18nService.getLocalizedString( CONSTANTE_AUCUN_STYLE, request.getLocale(  ) ) );

        String strStyle = I18nService.getLocalizedString( CONSTANTE_STYLE, request.getLocale(  ) );

        for ( int i = 1; i < 6; i++ )
        {
            refListStyle.addItem( i, strStyle + OdsConstants.CONSTANTE_ESPACE + i );
        }

        return refListStyle;
    }

    /**
     * methode qui test si l'elu d'id nkey se trouve dans la liste des
     * rapporteurs listRapporteurs
     *
     * @param listRapporteurs
     *            la liste des rapporteurs de l'entre en cours de traitement
     * @param nKey
     *            id de l'lu
     * @return true si l'elu se trouve dans la liste des rapporteurs,false sinon
     */
    protected Boolean isInTheEluList( int nKey, List<GElu> listRapporteurs )
    {
        for ( GElu elu : listRapporteurs )
        {
            if ( elu.getIdElu(  ) == nKey )
            {
                return true;
            }
        }

        return false;
    }

    /**
     * mthode qui recupre les parametres fournis par l'utilisateur en vue de
     * les insrer dans l'entree d'ordre du jour stock en session
     *
     * @param request
     *            request
     * @return retourne true si aucune erreur ne c'est produite
     */
    private boolean setEntreeOrdreDuJour( HttpServletRequest request )
    {
        _entreeOrdreDuJourBean.setReference( request.getParameter( OdsParameters.REFERENCE ) );
        _entreeOrdreDuJourBean.setObjet( request.getParameter( OdsParameters.OBJET ) );

        if ( request.getParameter( OdsParameters.STYLE ) != null )
        {
            String strStyle = request.getParameter( OdsParameters.STYLE ).trim(  );

            if ( !strStyle.equals( OdsConstants.FILTER_ALL ) )
            {
                _entreeOrdreDuJourBean.setStyle( strStyle );
            }
            else
            {
                _entreeOrdreDuJourBean.setStyle( null );
            }
        }

        setEntreeOrdreDuJourSpec( request );

        return true;
    }

    /**
     * retourne l'url de modification de l'ordre du jour d'id nIdOdj
     *
     * @param request
     *            request
     * @param nIdOdj
     *            l'id de l'ordre du jour
     * @return l'url de modification de l'ordre du jour d'id nIdOdj
     */
    protected String getJspModificationOdj( HttpServletRequest request, int nIdOdj )
    {
        return AppPathService.getBaseUrl( request ) + JSP_MODIFICATION_ORDRE_DU_JOUR +
        OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_ODJ + OdsConstants.CONSTANTE_EGAL + nIdOdj;
    }

    /**
     * retourne l'url de modification de l'entte ou pied de page de l'ordre du jour d'id nIdOdj
     *
     * @param request
     *            request
     * @param nIdOdj
     *            l'id de l'ordre du jour
     * @param isEntete
     *                           true si c'est une entte , false si c'est un pied de page
     * @return l'url de modification de l'entte ou pied de page de l'ordre du jour d'id nIdOdj
     */
    private String getJspModificationEntetePiedPage( HttpServletRequest request, int nIdOdj, boolean isEntete )
    {
        String strAction = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( isEntete )
        {
            strAction = CONSTANTE_ENTETE;
        }
        else
        {
            strAction = CONSTANTE_PIED_DE_PAGE;
        }

        return AppPathService.getBaseUrl( request ) + JSP_MODIFICATION_ENTETE_PIED_DE_PAGE +
        OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_ODJ + OdsConstants.CONSTANTE_EGAL + nIdOdj +
        OdsConstants.CONSTANTE_ET + OdsParameters.ACTION + OdsConstants.CONSTANTE_EGAL + strAction;
    }

    /**
     * Ajoute a l'url passe en paramtre l'ancre entree ordre du jour
     *
     * @param strUrl l'url
     * @param nIdEntreeOdj l'id de l'entree d'ordre du jour
     *  @return l'url + ancre permettant de retourner au niveau de l'entre d'ordre du jour voulue
     */
    private String addUrlAnchorEntreeOdj( String strUrl, int nIdEntreeOdj )
    {
        StringBuffer stringBuffer = new StringBuffer(  );
        stringBuffer.append( strUrl );
        stringBuffer.append( OdsConstants.CONSTANTE_DIESE );
        stringBuffer.append( CONSTANTE_ANCHOR_ENTREE_ODJ );
        stringBuffer.append( nIdEntreeOdj );

        return stringBuffer.toString(  );
    }

    /**
     * methode qui supprime l'elu d'id nkey de la liste des rapporteurs elus
     *
     * @param elus
     *            la liste des rapporteurs de l'entre en cours de traitement
     * @param nKey
     *            id de l'lu
     */
    private void removeEluInTheList( int nKey, List<GElu> elus )
    {
        GElu eluToRemove = null;

        for ( GElu elu : elus )
        {
            if ( elu.getIdElu(  ) == nKey )
            {
                eluToRemove = elu;

                break;
            }
        }

        if ( eluToRemove != null )
        {
            elus.remove( eluToRemove );
        }
    }

    /**
     * supprime l'entre d'id nkey de la liste des entres d'ordre du jour
     *
     * @param nKey
     *            id de l'entre  supprimer
     * @param entrees
     *            la liste des entes
     */
    protected void removeEntreeInTheList( int nKey, List<GEntreeOrdreDuJour> entrees )
    {
        GEntreeOrdreDuJour entreeToRemove = null;

        for ( GEntreeOrdreDuJour entree : entrees )
        {
            if ( entree.getIdEntreeOrdreDuJour(  ) == nKey )
            {
                entreeToRemove = entree;

                break;
            }
        }

        if ( entreeToRemove != null )
        {
            entrees.remove( entreeToRemove );
        }
    }

    /**
     * Retourne l'URL de selection des voeux pour l'ordredujour
     * @return l'URL de selection des voeux pour l'ordredujour
     */
    public String getUrlSelectionVoeu(  )
    {
        return JSP_SELECTION_VOEU;
    }

    /**
     * Retourne l'URL de selection des voeux pour l'ordredujour
     * @return l'URL de selection des voeux pour l'ordredujour
     */
    public String getUrlSelectionVoeuRetour(  )
    {
        return JSP_SELECTION_VOEU_RETOUR;
    }

    /**
     *
     * @param ordreDuJour l'odj
     * @param nIdTypeOdj le type d'odj
     * @return TRUE si l'odj est un odj de commission, FALSE sinon
     */
    public boolean isOrdreDuJourCommission( HttpServletRequest request )
    {
        return false;
    }

    public String getPageTitleCreationEntreeODJ(  )
    {
        GOrdreDuJourServiceManaged service = _mappingService.get( _entreeOrdreDuJourBean.getIdTypeEntite(  ) );

        return service.getPageTitleCreationEntreeODJ(  );
    }

    public String getPageTitleModificationEntreeODJ(  )
    {
        GOrdreDuJourServiceManaged service = _mappingService.get( _entreeOrdreDuJourBean.getIdTypeEntite(  ) );

        return service.getPageTitleModificationEntreeODJ(  );
    }

    /**
     * Retourne la map contenant les lments utiles au template HTML de la liste
     *
     * @param request la requte Http
     * @return model la map contenant les lments utiles au template HTML de la liste
     */
    public abstract HashMap<String, Object> getOrdreDuJourList( HttpServletRequest request );

    /**
     * Retourne la map contenant les lments utiles au template HTML de la liste
     *
     * @param request la requte Http
     * @return model la map contenant les lments utiles au template HTML de la liste
     */
    public HashMap<String, Object> getOrdreDuJourGestionAvalList( HttpServletRequest request )
    {
        Plugin plugin = getPluginSpec(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        List<GSeance> listSeances = _seanceHome.findOldSeance( plugin );

        if ( listSeances != null )
        {
            // initialisation du filtre
            GOrdreDuJourFilter ordreDuJourFilter = createNewOdjFilter(  );

            int nIdSeanceSelected = OdsUtils.getIntegerParameter( OdsParameters.ID_SEANCE, request );

            if ( nIdSeanceSelected != -1 )
            {
                _nIdSeanceSelected = nIdSeanceSelected;
            }

            if ( _nIdSeanceSelected == -1 )
            {
                GSeance derniereSeance = _seanceHome.getDerniereSeance( plugin );

                if ( derniereSeance != null )
                {
                    ordreDuJourFilter.setIdSeance( derniereSeance.getIdSeance(  ) );
                    _nIdSeanceSelected = derniereSeance.getIdSeance(  );
                }
            }
            else
            {
                ordreDuJourFilter.setIdSeance( _nIdSeanceSelected );
            }

            ordreDuJourFilter.setIdSauvegarde( 0 );
            ordreDuJourFilter.setIdFormationConseil( getIdFormationConseil( request ) );

            List<GOrdreDuJour> listOrdreDuJour = _ordreDuJourHome.findOrdreDuJourList( plugin, ordreDuJourFilter, false );

            model.put( MARK_LISTE_SEANCES, listSeances );
            model.put( MARK_ID_SEANCE_SELECTED, _nIdSeanceSelected );
            model.put( MARK_ID_ODJ_SELECTED, _nIdOdjSelected );

            getOrdreDuJourGestionAvalList( model, listOrdreDuJour );
        }

        return model;
    }

    /**
     * retourne l'id du type de conseil slectionn si le parametre SESSION est
     * pass dans la requette on renvoie l'id du type de conseil stock en
     * session
     *
     * @param request
     *            request
     * @return id du type de conseil slectionn
     */
    public int getIdFormationConseil( HttpServletRequest request )
    {
        int nIdFormationConseil = -1;

        if ( request.getParameter( OdsParameters.ID_FORMATION_CONSEIL ) != null )
        {
            try
            {
                nIdFormationConseil = Integer.parseInt( ( request.getParameter( OdsParameters.ID_FORMATION_CONSEIL ) ) );
                _nIdFormationConseilSelected = nIdFormationConseil;
            }
            catch ( NumberFormatException ne )
            {
                AppLogService.error( ne );
                nIdFormationConseil = _nIdFormationConseilSelected;
            }
        }
        else
        {
            if ( null != request.getParameter( CONSTANTE_SESSION ) )
            {
                nIdFormationConseil = _nIdFormationConseilSelected;
            }
            else
            {
                // rinitialisation de la variable de session
                _nIdFormationConseilSelected = -1;
            }
        }

        return nIdFormationConseil;
    }

    /**
     * Retourne la map contenant les lments utiles au template HTML de modification
     *
     * @param request la requte Http
     * @return model la map contenant les lments utiles au template HTML de modification
     */
    public HashMap<String, Object> getModificationOrdreDuJour( HttpServletRequest request )
    {
        Plugin plugin = getPluginSpec(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        if ( request.getParameter( OdsParameters.ID_ODJ ) != null )
        {
            int nIdOdj = OdsUtils.getIntegerParameter( OdsParameters.ID_ODJ, request );
            GOrdreDuJour ordreDuJour = _ordreDuJourHome.findByPrimaryKey( nIdOdj, plugin );
            OdsUtils.testNullObject( ordreDuJour, request );

            GSeance seance = _seanceHome.findByPrimaryKey( ordreDuJour.getSeance(  ).getIdSeance(  ), plugin );
            ordreDuJour.setSeance( seance );
            model.put( OdsMarks.MARK_SEANCE, seance );

            // rcupration de la liste des entres associes  l'odj
            List<GEntreeOrdreDuJour> listEntreeOrdreDuJour = _entreeOrdreDuJourHome.findEntreesListByIdOrdreDuJour( ordreDuJour.getIdOrdreDuJour(  ),
                    plugin );
            ordreDuJour.setListEntrees( listEntreeOrdreDuJour );

            if ( ( _listIdNouvelleEntree != null ) && ( _listIdNouvelleEntree.size(  ) != 0 ) )
            {
                model.put( MARK_LISTE_ID_NOUVELLE_ENTREE, _listIdNouvelleEntree );
                _listIdNouvelleEntree = null;
            }

            getModificationOrdreDuJourSpec( request, ordreDuJour, seance, model );

            model.put( IOrdreDuJour.MARK_ODJ, ordreDuJour );
            model.put( OdsMarks.MARK_URL_WEBAPP, AppPathService.getBaseUrl( request ) );

            return model;
        }

        return null;
    }

    /**
     * Retourne la map contenant les lments utiles au template HTML de modification de l'entete et pied de page
     *
     * @param request la requte Http
     * @return model la map contenant les lments utiles au template HTML de modification de l'entete et pied de page
     */
    public HashMap<String, Object> getModificationEntetePiedPage( HttpServletRequest request )
    {
        Plugin plugin = getPluginSpec(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        GSeance seance = _seanceHome.getProchaineSeance( plugin );
        String strAction = request.getParameter( OdsParameters.ACTION );

        if ( request.getParameter( OdsParameters.ID_ODJ ) != null )
        {
            int nIdOdj = OdsUtils.getIntegerParameter( OdsParameters.ID_ODJ, request );

            GOrdreDuJour ordreDuJour = _ordreDuJourHome.findByPrimaryKey( nIdOdj, plugin );
            OdsUtils.testNullObject( ordreDuJour, request );

            if ( ( strAction != null ) && strAction.equals( CONSTANTE_ENTETE ) )
            {
                Entete entete = OrdreDuJourUtils.getEntete( ordreDuJour, plugin );
                model.put( MARK_ENTETE, "1" );
                model.put( OrdreDuJourUtils.TAG_ENTETE, entete );
            }
            else if ( ( strAction != null ) && strAction.equals( CONSTANTE_PIED_DE_PAGE ) )
            {
                model.put( MARK_PIED_DE_PAGE, "1" );

                PiedDePage piedPage = OrdreDuJourUtils.getPiedDePage( ordreDuJour, plugin );
                model.put( OrdreDuJourUtils.TAG_PIED_DE_PAGE, piedPage );
            }

            ReferenceList refListStyle = initRefListStyle( request );

            model.put( MARK_LISTE_STYLE, refListStyle );
            model.put( OdsMarks.MARK_SEANCE, seance );
            model.put( IOrdreDuJour.MARK_ODJ, ordreDuJour );

            isCommissionDisplayed( model );

            return model;
        }

        return null;
    }

    /**
     * Retourne l'interface de confirmation de suppression d'une entre d'une entte
     *                                 ou d'un pied de page
     *
     * @param request
     *                    la requte Http
     * @return 'interface de confirmation de suppression d'une entre d'une entte
     *                                 ou d'un pied de page
     */
    public String getSuppressionEntreeEntetePiedPage( HttpServletRequest request )
    {
        if ( ( request.getParameter( OdsParameters.ID_ODJ ) == null ) &&
                ( request.getParameter( OdsParameters.POSITION_ENTREE ) == null ) &&
                ( request.getParameter( OdsParameters.ACTION ) == null ) )
        {
            return OdsConstants.CONSTANTE_CHAINE_VIDE;
        }

        String strAction = request.getParameter( OdsParameters.ACTION );
        String strIdODJ = request.getParameter( OdsParameters.ID_ODJ );
        String strPosition = request.getParameter( OdsParameters.POSITION_ENTREE );

        UrlItem url = new UrlItem( JSP_DO_SUPPRESSION_ENTREE_ENTETE_PIED_DE_PAGE );
        url.addParameter( OdsParameters.ID_ODJ, strIdODJ );
        url.addParameter( OdsParameters.POSITION_ENTREE, strPosition );
        url.addParameter( OdsParameters.ACTION, strAction );

        if ( strAction.equals( CONSTANTE_ENTETE ) )
        {
            return AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRME_DELETE_ENTREE_ENTETE, url.getUrl(  ),
                AdminMessage.TYPE_CONFIRMATION );
        }
        else
        {
            return AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRME_DELETE_ENTREE_PIED_DE_PAGE,
                url.getUrl(  ), AdminMessage.TYPE_CONFIRMATION );
        }
    }

    /**
     * Retourne la map contenant les lments utiles au template HTML de la suppression de l'ordre du jour
     *
     * @param request la requte Http
     * @return model la map contenant les lments utiles au template HTML de la suppression de l'ordre du jour
     */
    public HashMap<String, String> getSuppressionOrdreDuJour( HttpServletRequest request )
    {
        HashMap<String, String> model = new HashMap<String, String>(  );

        if ( request.getParameter( OdsParameters.ID_ODJ ) == null )
        {
            return null;
        }

        String strIdOdj = request.getParameter( OdsParameters.ID_ODJ );
        UrlItem url = new UrlItem( JSP_URL_DO_SUPPRESSION_ODJ );
        url.addParameter( OdsParameters.ID_ODJ, strIdOdj );

        String strMessageSuppression = I18nService.getLocalizedString( MESSAGE_CONFIRME_DELETE_ODJ,
                request.getLocale(  ) );
        String strSuppression = I18nService.getLocalizedString( MESSAGE_DELETE_ODJ, request.getLocale(  ) );

        model.put( MARK_TEXT_SUPPRESSION, strMessageSuppression );
        model.put( MARK_SUPPRESSION, strSuppression );
        model.put( MARK_URL, url.getUrl(  ) );

        return model;
    }

    /**
     * Retourne l'interface de confirmation de publication si au moins un pdd
     * rattach  l'ordre du jour n'est pas publi sinon redirection vers le jsp
     * de publication
     *
     * @param request
     *            la requte Http
     * @return l'interface de confirmation de publication si au moins un pdd
     *         rattach  l'ordre du jour n'est pas publi sinon redirection
     *         vers le jsp de publication
     */
    public String getPublicationOrdreDuJour( HttpServletRequest request )
    {
        Plugin plugin = getPluginSpec(  );

        if ( request.getParameter( OdsParameters.ID_ODJ ) != null )
        {
            GOrdreDuJour ordreDuJour;

            try
            {
                ordreDuJour = _ordreDuJourHome.findByPrimaryKey( Integer.parseInt( request.getParameter( 
                                OdsParameters.ID_ODJ ) ), plugin );
            }
            catch ( NumberFormatException ne )
            {
                AppLogService.error( ne );

                return OdsConstants.CONSTANTE_CHAINE_VIDE;
            }

            if ( !ordreDuJour.getPublie(  ) )
            {
                List<GEntreeOrdreDuJour> entreeOrdreDuJours = _entreeOrdreDuJourHome.findEntreesListByIdOrdreDuJour( ordreDuJour.getIdOrdreDuJour(  ),
                        plugin );

                UrlItem url = new UrlItem( JSP_URL_DO_PUBLICATION_ODJ );
                url.addParameter( OdsParameters.ID_ODJ, ordreDuJour.getIdOrdreDuJour(  ) );

                for ( GEntreeOrdreDuJour entreeOrdreDuJour : entreeOrdreDuJours )
                {
                    GOrdreDuJourServiceManaged service = _mappingService.get( entreeOrdreDuJour.getIdTypeEntite(  ) );
                    String strReturn = service.getPublicationOrdreDuJour( entreeOrdreDuJour, ordreDuJour );

                    if ( ( strReturn != null ) && !strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                    {
                        return AdminMessageService.getMessageUrl( request, strReturn, url.getUrl(  ),
                            AdminMessage.TYPE_CONFIRMATION );
                    }
                }

                // Sinon on affiche un message de confirmation
                return AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRMATION_PUBLICATION, url.getUrl(  ),
                    AdminMessage.TYPE_CONFIRMATION );
            }
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * Retourne l'interface de confirmation de depublication
     *
     * @param request
     *            la requte Http
     * @return l'interface de confirmation de publication si au moins un pdd
     *         rattach  l'ordre du jour n'est pas publi sinon redirection
     *         vers le jsp de publication
     */
    public String getDepublicationOrdreDuJour( HttpServletRequest request )
    {
        Plugin plugin = getPluginSpec(  );

        if ( request.getParameter( OdsParameters.ID_ODJ ) != null )
        {
            GOrdreDuJour ordreDuJour;

            try
            {
                ordreDuJour = _ordreDuJourHome.findByPrimaryKey( Integer.parseInt( request.getParameter( 
                                OdsParameters.ID_ODJ ) ), plugin );
            }
            catch ( NumberFormatException ne )
            {
                AppLogService.error( ne );

                return OdsConstants.CONSTANTE_CHAINE_VIDE;
            }

            String strReturn = getDepublicationOrdreDuJourSpec( request, ordreDuJour );

            if ( !strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                return strReturn;
            }

            if ( ordreDuJour.getPublie(  ) )
            {
                UrlItem url = new UrlItem( JSP_URL_DO_DEPUBLICATION_ODJ );
                url.addParameter( OdsParameters.ID_ODJ, ordreDuJour.getIdOrdreDuJour(  ) );

                /* On affiche un message de confirmation */
                return AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRMATION_DEPUBLICATION, url.getUrl(  ),
                    AdminMessage.TYPE_CONFIRMATION );
            }
            else
            {
                /* L'ordre du jour est dj dpubli */
                return AdminMessageService.getMessageUrl( request, MESSAGE_ALREADY_DEPUBLIER, AdminMessage.TYPE_STOP );
            }
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * Retourne la map contenant les lments utiles au template HTML de la cration d'une entre
     *
     * @param request la requte Http
     * @return model la map contenant les lments utiles au template HTML de la cration d'une entre
     */
    public HashMap<String, Object> getCreationEntreeOrdreDuJour( HttpServletRequest request )
    {
        Plugin plugin = getPluginSpec(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        if ( ( null == request.getParameter( CONSTANTE_SESSION ) ) &&
                ( null != request.getParameter( OdsParameters.TYPE ) ) &&
                ( null != request.getParameter( OdsParameters.ID_ODJ ) ) )
        {
            // initialisation de l'objet en session
            _entreeOrdreDuJourBean = getEntreeOrdreDuJour(  );
            _entreeOrdreDuJourBean.setIdEntreeOrdreDuJour( -1 );

            int nTypeCreation = OdsUtils.getIntegerParameter( OdsParameters.TYPE, request );
            _entreeOrdreDuJourBean.setIdTypeEntite( nTypeCreation );

            try
            {
                // rcupration de l'ordre du jour
                int nIdOdj = Integer.parseInt( request.getParameter( OdsParameters.ID_ODJ ) );
                GOrdreDuJour ordreDuJour = _ordreDuJourHome.findByPrimaryKey( nIdOdj, plugin );
                _entreeOrdreDuJourBean.setOrdreDuJour( ordreDuJour );
            }
            catch ( NumberFormatException ne )
            {
                // probleme de parametres
                AppLogService.error( ne );

                return null;
            }
        }

        getInitEntreeOrdreDuJour( request, model );

        model.put( MARK_ENTREE_ODJ, _entreeOrdreDuJourBean );

        //Initialisation des styles
        ReferenceList refListStyle = initRefListStyle( request );
        model.put( MARK_LISTE_STYLE, refListStyle );

        return model;
    }

    /**
     * Retourne la map contenant les lments utiles au template HTML de la modification d'une entre
     *
     * @param request la requte Http
     * @return model la map contenant les lments utiles au template HTML de la modification d'une entre
     */
    public HashMap<String, Object> getModificationEntreeOrdreDuJour( HttpServletRequest request )
    {
        Plugin plugin = getPluginSpec(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        if ( ( request.getParameter( CONSTANTE_SESSION ) == null ) &&
                ( request.getParameter( OdsParameters.ID_ENTREE ) != null ) )
        {
            // Rcupration de l'objet en base
            int nIdEntreeOdj = OdsUtils.getIntegerParameter( OdsParameters.ID_ENTREE, request );
            _entreeOrdreDuJourBean = _entreeOrdreDuJourHome.findByPrimaryKey( nIdEntreeOdj, plugin );
            OdsUtils.testNullObject( _entreeOrdreDuJourBean, request );

            // Rcupration de l'ordre du jour
            GOrdreDuJour ordreDuJour = _ordreDuJourHome.findByPrimaryKey( _entreeOrdreDuJourBean.getOrdreDuJour(  )
                                                                                                .getIdOrdreDuJour(  ),
                    plugin );
            OdsUtils.testNullObject( ordreDuJour, request );
            _entreeOrdreDuJourBean.setOrdreDuJour( ordreDuJour );
        }

        getInitEntreeOrdreDuJour( request, model );

        model.put( MARK_ENTREE_ODJ, _entreeOrdreDuJourBean );

        //Initialisation des styles
        ReferenceList refListStyle = initRefListStyle( request );
        model.put( MARK_LISTE_STYLE, refListStyle );

        return model;
    }

    /**
     * Retourne l'interface de confirmation de suppression d'une entre d'ordre
     * du jour
     *
     * @param request
     *            la requte Http
     * @return l'interface de confirmation de suppression d'une entre d'ordre
     *         du jour
     */
    public String getSuppressionEntreeOrdreDuJour( HttpServletRequest request )
    {
        if ( request.getParameter( OdsParameters.ID_ENTREE ) == null )
        {
            return OdsConstants.CONSTANTE_CHAINE_VIDE;
        }

        String strIdEntree = request.getParameter( OdsParameters.ID_ENTREE );
        UrlItem url = new UrlItem( JSP_DO_SUPPRESSION_ENTREE_JSP );
        url.addParameter( OdsParameters.ID_ENTREE, strIdEntree );

        return AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRME_DELETE_ENTREE, url.getUrl(  ),
            AdminMessage.TYPE_CONFIRMATION );
    }

    /**
     * Retourne la map contenant les lments utiles au template HTML de slection
     *
     * @param request la requte Http
     * @return model la map contenant les lments utiles au template HTML de slection
     */
    public HashMap<String, Object> getSelectionEntreeOrdreDuJour( HttpServletRequest request )
    {
        Plugin plugin = getPluginSpec(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        if ( request.getParameter( OdsParameters.ID_ODJ ) != null )
        {
            int nIdOdj = OdsUtils.getIntegerParameter( OdsParameters.ID_ODJ, request );

            GOrdreDuJour ordreDuJour = _ordreDuJourHome.findByPrimaryKey( nIdOdj, plugin );
            OdsUtils.testNullObject( ordreDuJour, request );

            // rcupration de la liste des entres associes  l'odj
            List<GEntreeOrdreDuJour> listEntreeOrdreDuJour = _entreeOrdreDuJourHome.findEntreesListByIdOrdreDuJour( ordreDuJour.getIdOrdreDuJour(  ),
                    plugin );
            List<GEntreeOrdreDuJour> listEntreeOrdreDuJourSelection = new ArrayList<GEntreeOrdreDuJour>(  );

            // Les vux et amendements rattachs  un PDD ne sont affichs que
            // si lordre du jour est dfinitif.
            // on recupere les voeux et amendements associs au pdd
            for ( GEntreeOrdreDuJour entreeOrdreDuJour : listEntreeOrdreDuJour )
            {
                boolean isSelect = false;

                if ( _strTabIdSelect != null )
                {
                    int nIdEntreeDeplace;

                    for ( String strIdSelect : _strTabIdSelect )
                    {
                        try
                        {
                            nIdEntreeDeplace = Integer.parseInt( strIdSelect );

                            if ( nIdEntreeDeplace == entreeOrdreDuJour.getIdEntreeOrdreDuJour(  ) )
                            {
                                isSelect = true;
                            }
                        }
                        catch ( NumberFormatException ne )
                        {
                            AppLogService.error( ne );

                            return null;
                        }
                    }
                }

                if ( !isSelect )
                {
                    listEntreeOrdreDuJourSelection.add( entreeOrdreDuJour );
                }
            }

            ordreDuJour.setListEntrees( listEntreeOrdreDuJourSelection );
            model.put( IOrdreDuJour.MARK_ODJ, ordreDuJour );

            return model;
        }

        return null;
    }

    /**
     * Effectue la cration d'un ordre du jour  partir des informations
     * contenues dans la requte
     *
     * @param request
     *            la requte Http
     * @return l'url de gestion des ordres du jour si la requte est valide
     */
    public abstract String doCreationOrdreDuJour( HttpServletRequest request );

    /**
     * Effectue la modification d'un ordre du jour  partir des informations
     * contenues dans la requte
     *
     * @param request
     *            la requte Http
     * @return l'url de gestion des ordres du jour
     */
    public String doModificationOrdreDuJour( HttpServletRequest request )
    {
        Plugin plugin = getPluginSpec(  );

        int nIdOdj = OdsUtils.getIntegerParameter( OdsParameters.ID_ODJ, request );

        if ( ( request.getParameter( CONSTANTE_ANNULER ) == null ) && ( nIdOdj != -1 ) )
        {
            GOrdreDuJour ordreDuJour = _ordreDuJourHome.findByPrimaryKey( nIdOdj, plugin );

            if ( ordreDuJour == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            if ( ( null != request.getParameter( OdsParameters.TITRE ) ) &&
                    !request.getParameter( OdsParameters.TITRE ).trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                ordreDuJour.setIntitule( request.getParameter( OdsParameters.TITRE ) );
            }
            else
            {
                Object[] messagesArgs = 
                    {
                        I18nService.getLocalizedString( PROPERTY_LABEL_INTITULE, request.getLocale(  ) ),
                    };

                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED,
                    messagesArgs, AdminMessage.TYPE_STOP );
            }

            doModificationOrdreDuJourSpec( request, ordreDuJour );

            _ordreDuJourHome.update( ordreDuJour, plugin );

            if ( null != request.getParameter( CONSTANTE_APPLIQUER ) )
            {
                return getJspModificationOdj( request, nIdOdj );
            }
        }

        return getListeOrdreDuJourUrl( request, nIdOdj );
    }

    protected String getListeOrdreDuJourUrl( HttpServletRequest request, int nIdOdj )
    {
        return AppPathService.getBaseUrl( request ) + JSP_LISTE_ORDRE_DU_JOUR +
        OdsConstants.CONSTANTE_POINT_INTERROGATION + CONSTANTE_SESSION + OdsConstants.CONSTANTE_EGAL +
        CONSTANTE_SESSION;
    }

    /**
     * Effectue la modification de l'entte ou du pied de page d'un ODJ
     *
     * @param request
     *            la requte Http
     * @return l'url de modification de l'ordre du jour
     */
    public String doModificationEntetePiedPage( HttpServletRequest request )
    {
        Plugin plugin = getPluginSpec(  );

        int nIdOdj = OdsUtils.getIntegerParameter( OdsParameters.ID_ODJ, request );
        String strAction = request.getParameter( OdsParameters.ACTION );

        if ( nIdOdj != -1 )
        {
            GOrdreDuJour ordreDuJour = _ordreDuJourHome.findByPrimaryKey( nIdOdj, plugin );

            if ( ordreDuJour == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            if ( ( request.getParameter( CONSTANTE_AJOUT_ENTREE ) != null ) )
            {
                //On va ajouter une entre  la fin dans l'entte ou pied de page
                if ( ( strAction != null ) && strAction.equals( CONSTANTE_ENTETE ) )
                {
                    ajoutEntreeEntetePiedPage( ordreDuJour, plugin, true );

                    return getJspModificationEntetePiedPage( request, nIdOdj, true );
                }
                else if ( ( strAction != null ) && strAction.equals( CONSTANTE_PIED_DE_PAGE ) )
                {
                    ajoutEntreeEntetePiedPage( ordreDuJour, plugin, false );

                    return getJspModificationEntetePiedPage( request, nIdOdj, false );
                }
            }
            else if ( ( request.getParameter( CONSTANTE_RESET_ENTETE_PIED_DE_PAGE ) != null ) && ( nIdOdj != -1 ) )
            {
                //On va rinitialiser l'entte ou le pied de page avec celui du modle d'ordre du jour
                if ( ( strAction != null ) && strAction.equals( CONSTANTE_ENTETE ) )
                {
                    if ( restaureEntetePiedPage( ordreDuJour, plugin, true ) )
                    {
                        return getJspModificationEntetePiedPage( request, nIdOdj, true );
                    }
                    else
                    {
                        //le modle de l'odj n'existe pas
                        return AdminMessageService.getMessageUrl( request,
                            OdsProperties.PROPERTY_MESSAGE_MODELE_ODJ_NOT_FOUND, AdminMessage.TYPE_STOP );
                    }
                }
                else if ( ( strAction != null ) && strAction.equals( CONSTANTE_PIED_DE_PAGE ) )
                {
                    if ( restaureEntetePiedPage( ordreDuJour, plugin, false ) )
                    {
                        return getJspModificationEntetePiedPage( request, nIdOdj, false );
                    }
                    else
                    {
                        //le modle de l'odj n'existe pas
                        return AdminMessageService.getMessageUrl( request,
                            OdsProperties.PROPERTY_MESSAGE_MODELE_ODJ_NOT_FOUND, AdminMessage.TYPE_STOP );
                    }
                }
            }
            else if ( ( request.getParameter( CONSTANTE_ANNULER ) == null ) && ( nIdOdj != -1 ) )
            {
                //On enregistre les modifications ralises sur l'entte ou le pied de page
                GSeance seance = _seanceHome.getProchaineSeance( plugin );

                //rcupration des valeurs
                String[] strTexteLibres = request.getParameterValues( OdsParameters.TEXTE_LIBRE );
                String[] strStyles = request.getParameterValues( OdsParameters.ID_STYLE );

                if ( ( strTexteLibres != null ) && ( strStyles != null ) )
                {
                    int nLength = strTexteLibres.length;

                    if ( ( strAction != null ) && strAction.equals( CONSTANTE_ENTETE ) )
                    {
                        //Cas de l'entte
                        Entete entete = new Entete(  );

                        for ( int i = 0; i < nLength; i++ )
                        {
                            Entete.EntreeEntete entree = new Entete.EntreeEntete(  );
                            entree.setTexteLibre( strTexteLibres[i] );
                            entree.setStyle( strStyles[i] );

                            entete.getEntreeEntete(  ).add( entree );
                        }

                        ordreDuJour.setXmlEntete( setXmlEnteteSpec( entete, seance ) );
                        _ordreDuJourHome.update( ordreDuJour, plugin );
                    }
                    else if ( ( strAction != null ) && strAction.equals( CONSTANTE_PIED_DE_PAGE ) )
                    {
                        //Cas du pied de page
                        PiedDePage piedDePage = new PiedDePage(  );

                        for ( int i = 0; i < nLength; i++ )
                        {
                            PiedDePage.EntreePiedDePage entree = new PiedDePage.EntreePiedDePage(  );
                            entree.setTexteLibre( strTexteLibres[i] );
                            entree.setStyle( strStyles[i] );

                            piedDePage.getEntreePiedDePage(  ).add( entree );
                        }

                        ordreDuJour.setXmlPiedDePage( setXmlPiedPageSpec( piedDePage, seance ) );
                        _ordreDuJourHome.update( ordreDuJour, plugin );
                    }
                }
            }

            //Si l'on a cliquer sur "Appliquer" on reste sur la page de modification des enttes et pied de page
            if ( null != request.getParameter( CONSTANTE_APPLIQUER ) )
            {
                if ( ( strAction != null ) && strAction.equals( CONSTANTE_ENTETE ) )
                {
                    return getJspModificationEntetePiedPage( request, nIdOdj, true );
                }
                else if ( ( strAction != null ) && strAction.equals( CONSTANTE_PIED_DE_PAGE ) )
                {
                    return getJspModificationEntetePiedPage( request, nIdOdj, false );
                }
                else
                {
                    return getJspModificationOdj( request, nIdOdj );
                }
            }

            // Sinon on revient sur la page de modification de l'ordre du jour
            // Cas du bouton "Annuler" ou cas du bouton "Enregistrer"
            else
            {
                return getJspModificationOdj( request, nIdOdj );
            }
        }
        else
        {
            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                AdminMessage.TYPE_ERROR );
        }
    }

    /**
     * Effectue la suppression d'une entre d'entte ou pied de page d'un ordre du jour  partir des
     * informations contenues dans la requte
     *
     * @param request
     *            la requte Http
     * @return l'url de modification d'entte ou pied de page de l' ordre du jour
     */
    public String doSuppressionEntreeEntetePiedPage( HttpServletRequest request )
    {
        Plugin plugin = getPluginSpec(  );

        if ( null != request.getParameter( OdsParameters.ID_ODJ ) )
        {
            int nIdODJ = OdsUtils.getIntegerParameter( OdsParameters.ID_ODJ, request );
            GOrdreDuJour ordreDuJour = _ordreDuJourHome.findByPrimaryKey( nIdODJ, plugin );

            if ( ordreDuJour == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            if ( null != request.getParameter( OdsParameters.POSITION_ENTREE ) )
            {
                int nPosition = OdsUtils.getIntegerParameter( OdsParameters.POSITION_ENTREE, request );
                GSeance seance = _seanceHome.getProchaineSeance( plugin );

                String strAction = request.getParameter( OdsParameters.ACTION );

                if ( ( strAction != null ) && strAction.equals( CONSTANTE_ENTETE ) )
                {
                    Entete entete = OrdreDuJourUtils.getEntete( ordreDuJour, plugin );
                    entete.getEntreeEntete(  ).remove( nPosition - 1 );

                    ordreDuJour.setXmlEntete( setXmlEnteteSpec( entete, seance ) );
                    _ordreDuJourHome.update( ordreDuJour, plugin );

                    return getJspModificationEntetePiedPage( request, nIdODJ, true );
                }
                else if ( ( strAction != null ) && strAction.equals( CONSTANTE_PIED_DE_PAGE ) )
                {
                    PiedDePage piedPage = OrdreDuJourUtils.getPiedDePage( ordreDuJour, plugin );
                    piedPage.getEntreePiedDePage(  ).remove( nPosition - 1 );

                    ordreDuJour.setXmlPiedDePage( setXmlPiedPageSpec( piedPage, seance ) );
                    _ordreDuJourHome.update( ordreDuJour, plugin );

                    return getJspModificationEntetePiedPage( request, nIdODJ, false );
                }
            }
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * Retourne l'ordre du jour sous format PDF ou Word
     * @param request la requete Http
     * @return l'ordre du jour
     */
    public GOrdreDuJour doVisualisationODJ( HttpServletRequest request )
    {
        Plugin plugin = getPluginSpec(  );
        GOrdreDuJour ordreDuJour = null;

        int nIdOdj = OdsUtils.getIntegerParameter( OdsParameters.ID_ODJ, request );
        String strTypeConversion = request.getParameter( OdsParameters.FORMAT );

        if ( ( nIdOdj != -1 ) && ( strTypeConversion != null ) )
        {
            ordreDuJour = _ordreDuJourHome.findByPrimaryKey( nIdOdj, plugin );
            OdsUtils.testNullObject( ordreDuJour, request );

            doClasseEntree( request.getLocale(  ), ordreDuJour );

            GSeance seance = _seanceHome.findByPrimaryKey( ordreDuJour.getSeance(  ).getIdSeance(  ), plugin );
            ordreDuJour.setSeance( seance );

            Entete entete = OrdreDuJourUtils.getEntete( ordreDuJour, plugin );
            PiedDePage piedDePage = OrdreDuJourUtils.getPiedDePage( ordreDuJour, plugin );

            if ( entete != null )
            {
                ordreDuJour.setXmlEntete( setXmlEnteteSpec( entete, seance ) );
            }

            if ( piedDePage != null )
            {
                ordreDuJour.setXmlPiedDePage( setXmlPiedPageSpec( piedDePage, seance ) );
            }
        }

        return ordreDuJour;
    }

    /**
     * Retourne l'ordre du jour sous format csv
     * @param request la requete Http
     * @return listToCSVWriter le tableau utile pour la gnration CSV
     */
    public List<String[]> doVisualisationODJcsv( HttpServletRequest request )
    {
        Plugin plugin = getPluginSpec(  );
        GOrdreDuJour ordreDuJour = null;

        int nIdOdj = OdsUtils.getIntegerParameter( OdsParameters.ID_ODJ, request );

        if ( nIdOdj != -1 )
        {
            ordreDuJour = _ordreDuJourHome.findByPrimaryKey( nIdOdj, plugin );

            if ( ordreDuJour != null )
            {
                //on cr les titres des colonnes du CSV
                List<String[]> listToCSVWriter = new ArrayList<String[]>(  );
                String[] header = 
                    {
                        I18nService.getLocalizedString( MESSAGE_CSV_SEANCE, request.getLocale(  ) ),
                        I18nService.getLocalizedString( MESSAGE_CSV_REFERENCE, request.getLocale(  ) ),
                        I18nService.getLocalizedString( MESSAGE_CSV_ENTITE_DEPOSITAIRE, request.getLocale(  ) ),
                        I18nService.getLocalizedString( MESSAGE_CSV_ELU_DEPOSITAIRE, request.getLocale(  ) ),
                        I18nService.getLocalizedString( MESSAGE_CSV_OBJET, request.getLocale(  ) ),
                        I18nService.getLocalizedString( MESSAGE_CSV_TYPE, request.getLocale(  ) ),
                        I18nService.getLocalizedString( MESSAGE_CSV_RAPPORTEUR, request.getLocale(  ) ),
                    };
                listToCSVWriter.add( header );

                //on recupere la liste des entres en base
                List<GEntreeOrdreDuJour> listEntrees = _entreeOrdreDuJourHome.findEntreesListByIdOrdreDuJour( ordreDuJour.getIdOrdreDuJour(  ),
                        plugin );
                ordreDuJour.setListEntrees( listEntrees );

                GSeance seance = _seanceHome.findByPrimaryKey( ordreDuJour.getSeance(  ).getIdSeance(  ), plugin );
                ordreDuJour.setSeance( seance );

                String strSeance = getLibelleSeance( ordreDuJour.getSeance(  ) );

                for ( GEntreeOrdreDuJour entree : ordreDuJour.getListEntrees(  ) )
                {
                    GOrdreDuJourServiceManaged service = _mappingService.get( entree.getIdTypeEntite(  ) );
                    boolean bIsForCsv = service.isForCsv(  );

                    if ( bIsForCsv && ( entree.getEntite(  ) != null ) )
                    {
                        addEntreeCsvLine( entree, strSeance, listToCSVWriter, request.getLocale(  ) );
                    }
                }

                return listToCSVWriter;
            }
        }

        return null;
    }

    /**
     * genre les odj de la prochaine sance au format word et les enregistrer sur le File System
     * @param request la requte HTTP
     * @return 0 si OK, 10 sinon
     * @throws IOException IOException
     */
    public String doGenereODJ( HttpServletRequest request )
        throws IOException
    {
        boolean bConversionIsOK = true;

        String strAbsoluteTmpDirectory = AppPropertiesService.getProperty( PROPERTY_GENERATION_ODJ_TMP_DIRECTORY );

        if ( strAbsoluteTmpDirectory == null )
        {
            AppLogService.error( "Generation ODJ : Temp directory was not found" );

            return OdsConstants.RESULT_KO;
        }

        String strReturn = doGenereODJSpec( request, strAbsoluteTmpDirectory, bConversionIsOK );

        if ( strReturn.equals( OdsConstants.RESULT_KO ) )
        {
            return strReturn;
        }

        if ( !bConversionIsOK )
        {
            AppLogService.error( "Generation ODJ : Error during the Word conversion or the zip creation" );

            return OdsConstants.RESULT_KO;
        }

        return OdsConstants.RESULT_OK;
    }

    /**
     * Effectue la suppression d'un ordre du jour  partir des informations
     * contenues dans la requte
     *
     * @param request
     *            la requte Http
     * @return l'url de gestion des ordres du jour
     */
    public String doSuppressionOrdreDuJour( HttpServletRequest request )
    {
        Plugin plugin = getPluginSpec(  );

        String strConfirmationSuppression = request.getParameter( PARAM_CHECKBOX_DELETE );

        if ( strConfirmationSuppression == null )
        {
            return AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRMATION_TO_DELETE, AdminMessage.TYPE_WARNING );
        }
        else
        {
            int nIdOdj = OdsUtils.getIntegerParameter( OdsParameters.ID_ODJ, request );

            if ( nIdOdj != -1 )
            {
                GOrdreDuJour ordreDuJour = _ordreDuJourHome.findByPrimaryKey( nIdOdj, plugin );

                if ( ordreDuJour == null )
                {
                    return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                        AdminMessage.TYPE_ERROR );
                }

                if ( !ordreDuJour.getPublie(  ) )
                {
                    List<GEntreeOrdreDuJour> entreeOrdreDuJours = _entreeOrdreDuJourHome.findEntreesListByIdOrdreDuJour( nIdOdj,
                            plugin );

                    try
                    {
                        // suppression de toutes les entres d'ordre du jour
                        for ( GEntreeOrdreDuJour entreeOrdreDuJour : entreeOrdreDuJours )
                        {
                            GOrdreDuJourServiceManaged service = _mappingService.get( entreeOrdreDuJour.getIdTypeEntite(  ) );
                            service.deleteEntiteForODJ( entreeOrdreDuJour.getIdEntite(  ) );

                            // suppression des entrees lies a l'ordre du jour
                            _entreeOrdreDuJourHome.remove( entreeOrdreDuJour, plugin );
                        }

                        // suppression de l'ordre du jour
                        _ordreDuJourHome.remove( ordreDuJour.getIdOrdreDuJour(  ), plugin );

                        //si l'ordre du jour possede une sauvegarde on la supprime aussi
                        if ( ordreDuJour.getOrdreDuJourSauveGarde(  ) != null )
                        {
                            entreeOrdreDuJours = _entreeOrdreDuJourHome.findEntreesListByIdOrdreDuJour( ordreDuJour.getOrdreDuJourSauveGarde(  )
                                                                                                                   .getIdOrdreDuJour(  ),
                                    plugin );

                            // suppression de toutes les entres de la sauvegarde d'ordre du jour 
                            for ( GEntreeOrdreDuJour entreeOrdreDuJour : entreeOrdreDuJours )
                            {
                                GOrdreDuJourServiceManaged service = _mappingService.get( entreeOrdreDuJour.getIdTypeEntite(  ) );
                                service.deleteEntiteForODJ( entreeOrdreDuJour.getIdEntite(  ) );

                                // suppression des entrees lies a l'ordre du jour
                                _entreeOrdreDuJourHome.remove( entreeOrdreDuJour, plugin );
                            }

                            //suppression de la sauvegarde d'ordre du jour
                            _ordreDuJourHome.remove( ordreDuJour.getOrdreDuJourSauveGarde(  ).getIdOrdreDuJour(  ),
                                plugin );
                        }
                    }
                    catch ( AppException ae )
                    {
                        AppLogService.error( ae );

                        if ( ae.getInitialException(  ) instanceof SQLException )
                        {
                            return AdminMessageService.getMessageUrl( request, MESSAGE_CANNOT_DELETE_ENTREE,
                                AdminMessage.TYPE_STOP );
                        }
                    }
                }
            }

            return getListeOrdreDuJourUrl( request, nIdOdj );
        }
    }

    /**
     * Effectue la publication d'un ordre du jour  partir de l'id de l'odj
     * contenu dans la requette
     *
     * @param request
     *            la requte Http
     * @return l'url de gestion des ordres du jour si la requte est valide, un
     *         message d'erreur sinon
     */
    public String doPublicationOrdreDuJour( HttpServletRequest request )
    {
        Plugin plugin = getPluginSpec(  );

        int nIdOdj = OdsUtils.getIntegerParameter( OdsParameters.ID_ODJ, request );

        if ( nIdOdj != -1 )
        {
            Timestamp dateForVersion = new Timestamp( System.currentTimeMillis(  ) );
            GOrdreDuJour ordreDuJour = _ordreDuJourHome.findByPrimaryKey( nIdOdj, plugin );

            if ( ordreDuJour == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            if ( ordreDuJour.getPublie(  ) )
            {
                return AdminMessageService.getMessageUrl( request, MESSAGE_ODJ_DEJA_PUBLIE, AdminMessage.TYPE_STOP );
            }

            if ( ( ordreDuJour.getDateDebutPublication(  ) != null ) &&
                    ( request.getParameter( OdsParameters.FORCE_PUBLICATION ) == null ) )
            {
                UrlItem url = new UrlItem( JSP_URL_DO_PUBLICATION_ODJ );
                url.addParameter( OdsParameters.FORCE_PUBLICATION, OdsParameters.FORCE_PUBLICATION );
                url.addParameter( OdsParameters.ID_ODJ, ordreDuJour.getIdOrdreDuJour(  ) );

                Object[] tabDateDebutPublication = 
                    {
                        DateUtils.getDate( ordreDuJour.getDateDebutPublication(  ),
                            OdsConstants.CONSTANTE_PATTERN_DATE_LONG ),
                        DateUtils.getDate( ordreDuJour.getDateDebutPublication(  ), OdsConstants.CONSTANTE_PATTERN_TIME ),
                    };

                return AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRMATION_NOUVELLE_PUBLICATION,
                    tabDateDebutPublication, url.getUrl(  ), AdminMessage.TYPE_CONFIRMATION );
            }

            _ordreDuJourHome.updateDateDebutPublication( ordreDuJour.getIdOrdreDuJour(  ), dateForVersion, plugin );

            doClasseEntree( request.getLocale(  ), ordreDuJour );

            GSeance seance = _seanceHome.findByPrimaryKey( ordreDuJour.getSeance(  ).getIdSeance(  ), plugin );
            ordreDuJour.setSeance( seance );

            if ( ordreDuJour.getXmlEntete(  ) == null )
            {
                Entete entete = getEnteteFromModeleSpec( ordreDuJour );

                if ( entete != null )
                {
                    ordreDuJour.setXmlEntete( setXmlEnteteSpec( entete, seance ) );
                }
            }

            if ( ordreDuJour.getXmlPiedDePage(  ) == null )
            {
                PiedDePage piedPage = getPiedDePageFromModeleSpec( ordreDuJour );

                if ( piedPage != null )
                {
                    ordreDuJour.setXmlPiedDePage( setXmlPiedPageSpec( piedPage, seance ) );
                }
            }

            ordreDuJour.setXmlPublication( ordreDuJour.getXml(  ) );

            // creation du fichier pdf de l'ordre du jour
            GFichier newFichier = newFichierInstance(  );
            FichierPhysique newFichierPhysique = new FichierPhysique(  );

            // Creation de l'objet reprsentant le fichier physique
            // recuperation du xml de l'ordre du jour a
            try
            {
                byte[] tabXmlOdj = ( OdsConstants.CONSTANTE_ENTETE_XML + ordreDuJour.getXmlPublication(  ) ).getBytes( OdsConstants.UTF8 );

                String nomTemplate = AppPropertiesService.getProperty( PROPERTY_ODJ_NOM_TEMPLATE,
                        DEFAULT_ODJ_NOM_TEMPLATE );

                tabXmlOdj = DocumentManager.convert( DocumentManager.PDF_CONVERSION, nomTemplate, tabXmlOdj );

                newFichierPhysique.setDonnees( tabXmlOdj );
                newFichierPhysique.setIdFichier( _fichierPhysiqueHome.create( newFichierPhysique, plugin ) );
                newFichier.setFichier( newFichierPhysique );
                newFichier.setIntitule( ordreDuJour.getIntitule(  ) );
                newFichier.setNom( ordreDuJour.getIntitule(  ) );
                newFichier.setDatePublication( dateForVersion );
                newFichier.setExtension( DocumentManager.PDF_CONVERSION );
                newFichier.setTaille( tabXmlOdj.length );
                // Creation de l'objet reprsentant le Fichier
                newFichier.setSeance( ordreDuJour.getSeance(  ) );
                newFichier.setTypdeDocument( _fichierHome.findTypeDocumentsById( ITypeDocument.ORDRE_DU_JOUR, plugin ) );

                newFichier.setEnLigne( true );
                newFichier.setVersion( 1 );

                String strReturn = doPublicationSpec( request, newFichier, ordreDuJour, dateForVersion );

                if ( !strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                {
                    return strReturn;
                }

                newFichier.setId( _fichierHome.create( newFichier, plugin ) );

                if ( newFichier.getSeance(  ) != null )
                {
                    ISeance seanceByFichier = newFichier.getSeance(  );
                    GSeance nextSeance = _seanceHome.getProchaineSeance( plugin );

                    if ( ( nextSeance != null ) && !seanceByFichier.equals( nextSeance ) &&
                            ( ( newFichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.DOCUMENT_LIBRE ) ||
                            ( newFichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.COMPTE_RENDU ) ||
                            ( newFichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.LISTE_DE_DESIGNATION ) ) )
                    {
                        _searchService.addObjectToIndex( newFichier, true, plugin );
                    }
                }

                //Validation de la numrotation
                for ( GEntreeOrdreDuJour entree : ordreDuJour.getListEntrees(  ) )
                {
                    GOrdreDuJourServiceManaged service = _mappingService.get( entree.getIdTypeEntite(  ) );
                    service.validateNumerotation( entree );
                }

                ordreDuJour.setFichier( newFichier );
                dateForVersion = new Timestamp( System.currentTimeMillis(  ) );
                ordreDuJour.setPublie( true );
                ordreDuJour.setDatePublication( dateForVersion );

                _ordreDuJourHome.update( ordreDuJour, plugin );

                doPublicationNotificationSpec( request, ordreDuJour );
                doPublicationExterneSpec( ordreDuJour );

                return getListeOrdreDuJourUrl( request, nIdOdj );
            }
            catch ( UnsupportedEncodingException uee )
            {
                AppLogService.error( uee );
            }
            catch ( RemoteConnectFailureException e )
            {
                AppLogService.error( e );
            }
            catch ( BeanCreationException be )
            {
                AppLogService.error( be );
            }
            catch ( IllegalArgumentException iae )
            {
                AppLogService.error( iae );
            }
            catch ( UnavailableException ue )
            {
                AppLogService.error( ue );
            }
            catch ( ServerException se )
            {
                AppLogService.error( se );
            }
            catch ( IOException ioe )
            {
                AppLogService.error( ioe );
            }

            return AdminMessageService.getMessageUrl( request, MESSAGE_PROBLEME_DE_CREATION_ODJ_PDF,
                AdminMessage.TYPE_STOP );
        }

        return getListeOrdreDuJourUrl( request, nIdOdj );
    }

    /**
     * Effectue la dpublication d'un ordre du jour  partir de l'id de l'odj
     * contenu dans la requette
     *
     * @param request
     *            la requte Http
     * @return l'url de gestion des ordres du jour si la requte est valide, un
     *         message d'erreur sinon
     */
    public String doDepublicationOrdreDuJour( HttpServletRequest request )
    {
        Plugin plugin = getPluginSpec(  );

        int nIdOdj = OdsUtils.getIntegerParameter( OdsParameters.ID_ODJ, request );

        if ( nIdOdj != -1 )
        {
            GOrdreDuJour ordreDuJour = _ordreDuJourHome.findByPrimaryKey( nIdOdj, plugin );

            if ( ordreDuJour == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            if ( !ordreDuJour.getPublie(  ) )
            {
                return AdminMessageService.getMessageUrl( request, MESSAGE_ODJ_DEJA_DEPUBLIE, AdminMessage.TYPE_STOP );
            }

            /* Suppression du fichier PDF gnr lors de la publication */
            GFichier fichier = ordreDuJour.getFichier(  );
            fichier = _fichierHome.findByPrimaryKey( fichier.getId(  ), plugin );

            FichierPhysique fichierPhysique = fichier.getFichier(  );

            _historiqueHome.remove( fichier, plugin );

            if ( fichier.getSeance(  ) != null )
            {
                ISeance seanceByFichier = fichier.getSeance(  );
                GSeance nextSeance = _seanceHome.getProchaineSeance( plugin );

                if ( ( nextSeance != null ) && !seanceByFichier.equals( nextSeance ) &&
                        ( ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.DOCUMENT_LIBRE ) ||
                        ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.COMPTE_RENDU ) ||
                        ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.LISTE_DE_DESIGNATION ) ) )
                {
                    _searchService.deleteObjectFromIndex( fichier, true, plugin );
                }
            }

            _fichierHome.remove( fichier, plugin );
            _fichierPhysiqueHome.remove( fichierPhysique, plugin );

            /* Mise  jour de l'ordre du jour */
            ordreDuJour.setPublie( false );
            ordreDuJour.setFichier( null );
            _ordreDuJourHome.update( ordreDuJour, plugin );

            /* Suppression de la liasse des VAs correspondant  l'ordre du jour */
            GFichierFilter fichierFilter = newFichierFilterInstance(  );
            fichierFilter.setIdSeance( ordreDuJour.getSeance(  ).getIdSeance(  ) );
            fichierFilter.setIdFormationConseil( ordreDuJour.getFormationConseil(  ).getIdFormationConseil(  ) );
            fichierFilter.setIdTypeDocument( ITypeDocument.LIASSE_DES_VA );

            List<GFichier> liasses = _fichierHome.findByFilter( fichierFilter, plugin );

            if ( ( liasses != null ) && ( liasses.size(  ) > 0 ) )
            {
                /* Si l'on est dans un tat cohrent, il n'y a qu'une seule liasse de VAs
                   par sance et par formation conseil */
                GFichier liasse = liasses.get( 0 );
                FichierPhysique liassePhysique = liasse.getFichier(  );
                _historiqueHome.remove( liasse, plugin );

                if ( liasse.getSeance(  ) != null )
                {
                    ISeance seanceByFichier = liasse.getSeance(  );
                    GSeance nextSeance = _seanceHome.getProchaineSeance( plugin );

                    if ( ( nextSeance != null ) && !seanceByFichier.equals( nextSeance ) &&
                            ( ( liasse.getTypdeDocument(  ).getId(  ) == ITypeDocument.DOCUMENT_LIBRE ) ||
                            ( liasse.getTypdeDocument(  ).getId(  ) == ITypeDocument.COMPTE_RENDU ) ||
                            ( liasse.getTypdeDocument(  ).getId(  ) == ITypeDocument.LISTE_DE_DESIGNATION ) ) )
                    {
                        _searchService.deleteObjectFromIndex( liasse, true, plugin );
                    }
                }

                _fichierHome.remove( liasse, plugin );
                _fichierPhysiqueHome.remove( liassePhysique, plugin );
            }

            doDepublicationExterneSpec( ordreDuJour );
        }

        return getListeOrdreDuJourUrl( request, nIdOdj );
    }

    /**
     * Effectue l'ensemble des traitements necessaire lors de la cration d'une
     * entre de l'ordre du jour  partir des informations contenues dans la
     * requte creation d'une nouvelle entre ,ajout d'un rapporteur ,d'une
     * commission  l'entre en cours de creation.....
     *
     * @param request
     *            la requte Http
     * @return en fonction des diffrents traitements retour vers l'url de
     *         modification d'un ordre du jour ,de creation de l'entre en cours
     *         de creation...
     */
    public String doCreationEntreeOrdreDuJour( String strReturnUrl, HttpServletRequest request )
    {
        return doTraitementEntreeOrdreDuJour( strReturnUrl, request, true );
    }

    /**
     * Effectue l'ensemble des traitements necessaire lors d'une modification
     * d'une entre d'ordre du jour  partir des informations contenues dans la
     * requte modification d'une entre ,ajout d'un rapporteur ,d'une
     * commission  l'entre en cours de modification.....
     *
     * @param request la requte Http
     * @return en fonction des diffrents traitements retour vers l'url de
     *         modification d'un ordre du jour ,de creation de l'entre en cours
     *         de creation...
     */
    public String doModificationEntreeOrdreDuJour( String strReturnUrl, HttpServletRequest request )
    {
        return doTraitementEntreeOrdreDuJour( strReturnUrl, request, false );
    }

    /**
     * Effectue l'ensemble des traitements necessaire lors d'une modification
     * d'une entre d'ordre du jour  partir des informations contenues dans la
     * requte modification d'une entre ,ajout d'un rapporteur ,d'une
     * commission  l'entre en cours de modification.....
     * @param strBaseUrl l'url de la jsp de destination
     * @param request la requte Http
     * @return en fonction des diffrents traitements retour vers l'url de
     *         modification d'un ordre du jour ,de creation de l'entre en cours
     *         de creation...
     */
    public String doTraitementEntreeOrdreDuJour( String strBaseUrl, HttpServletRequest request, boolean bIsForCreation )
    {
        Plugin plugin = getPluginSpec(  );

        String strUrlRetour = strBaseUrl;

        List<String> listErreurs = new ArrayList<String>(  );

        if ( request.getParameter( CONSTANTE_ANNULER ) == null )
        {
            if ( _entreeOrdreDuJourBean == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            if ( ( request.getParameter( OdsParameters.FLAG ) != null ) || setEntreeOrdreDuJour( request ) )
            {
                // si la rcupration des parametres du formulaire n'a pas pos de problme
                if ( ( request.getParameter( OdsParameters.FLAG ) != null ) ||
                        ( request.getParameter( CONSTANTE_ENREGISTRER ) != null ) )
                {
                    GOrdreDuJourServiceManaged service = _mappingService.get( _entreeOrdreDuJourBean.getIdTypeEntite(  ) );
                    boolean bIsFieldRequired = service.isFieldRequired( _entreeOrdreDuJourBean, listErreurs );

                    /* si l'utilisateur click sur le bouton enregistrer
                       on test les parametres saisis en utilisant l'objet
                       EntreeOrdreDuJour en session
                    */
                    if ( ( request.getParameter( OdsParameters.FLAG ) == null ) && !bIsFieldRequired )
                    {
                        String strChamp = ( listErreurs.get( 0 ) != null ) ? listErreurs.get( 0 )
                                                                           : OdsConstants.CONSTANTE_CHAINE_VIDE;
                        Object[] messagesArgs = { I18nService.getLocalizedString( strChamp, request.getLocale(  ) ) };

                        return AdminMessageService.getMessageUrl( request,
                            OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED, messagesArgs, AdminMessage.TYPE_STOP );
                    }
                    else
                    {
                        boolean bIsNoVoeuSelected = service.bIsNoEntitySelected( _entreeOrdreDuJourBean );

                        if ( bIsNoVoeuSelected )
                        {
                            return AdminMessageService.getMessageUrl( request, service.getErrorNoEntitySelected(  ),
                                AdminMessage.TYPE_STOP );
                        }

                        try
                        {
                            if ( bIsForCreation )
                            {
                                int nIdEntite = service.createEntiteForODJ( _entreeOrdreDuJourBean );

                                if ( nIdEntite != -1 )
                                {
                                    _entreeOrdreDuJourBean.setIdEntite( nIdEntite );
                                }

                                int nIdEntree = _entreeOrdreDuJourHome.create( _entreeOrdreDuJourBean, plugin );
                                _entreeOrdreDuJourBean.setIdEntreeOrdreDuJour( nIdEntree );
                            }
                            else
                            {
                                service.updateEntiteForODJ( _entreeOrdreDuJourBean );
                                _entreeOrdreDuJourHome.update( _entreeOrdreDuJourBean, plugin );
                            }
                        }
                        catch ( AppException ae )
                        {
                            if ( ae.getInitialException(  ) instanceof SQLException )
                            {
                                return AdminMessageService.getMessageUrl( request,
                                    OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND, AdminMessage.TYPE_ERROR );
                            }
                        }

                        String strReturn = doTraitementEntreeCommissionSpec( request, bIsForCreation );

                        if ( !strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                        {
                            return strReturn;
                        }

                        // on ajoute a la liste des nouvelles entres l'id de la nouvelle entre
                        if ( _listIdNouvelleEntree == null )
                        {
                            _listIdNouvelleEntree = new ArrayList<Integer>(  );
                        }

                        _listIdNouvelleEntree.add( _entreeOrdreDuJourBean.getIdEntreeOrdreDuJour(  ) );

                        if ( !bIsForCreation )
                        {
                            // suppression des rapporteurs
                            _entreeOrdreDuJourHome.deleteRapporteur( _entreeOrdreDuJourBean.getIdEntreeOrdreDuJour(  ),
                                plugin );
                        }

                        if ( _entreeOrdreDuJourBean.getElus(  ) != null )
                        {
                            // insertion des rapporteurs
                            for ( GElu elu : _entreeOrdreDuJourBean.getElus(  ) )
                            {
                                GElu e = _eluHome.findByPrimaryKey( elu.getIdElu(  ), plugin );

                                if ( e == null )
                                {
                                    return AdminMessageService.getMessageUrl( request,
                                        OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND, AdminMessage.TYPE_ERROR );
                                }

                                _entreeOrdreDuJourHome.insertRapporteur( _entreeOrdreDuJourBean.getIdEntreeOrdreDuJour(  ),
                                    elu, plugin );
                            }
                        }

                        /*        on recupere la nouvelle entre en base
                            si on est en mode automatique on aplique
                                l'algorithme de classement automatique
                                sinon on recupere la position ou doit etre insr la
                                nouvelle entre
                        */
                        _entreeOrdreDuJourBean = _entreeOrdreDuJourHome.findByPrimaryKey( _entreeOrdreDuJourBean.getIdEntreeOrdreDuJour(  ),
                                plugin );

                        if ( _entreeOrdreDuJourBean == null )
                        {
                            return AdminMessageService.getMessageUrl( request,
                                OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND, AdminMessage.TYPE_ERROR );
                        }

                        if ( ( _entreeOrdreDuJourBean != null ) && ( _entreeOrdreDuJourBean.getOrdreDuJour(  ) != null ) )
                        {
                            GOrdreDuJour ordreDuJour = _ordreDuJourHome.findByPrimaryKey( _entreeOrdreDuJourBean.getOrdreDuJour(  )
                                                                                                                .getIdOrdreDuJour(  ),
                                    plugin );

                            if ( ordreDuJour == null )
                            {
                                return AdminMessageService.getMessageUrl( request,
                                    OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND, AdminMessage.TYPE_ERROR );
                            }

                            _entreeOrdreDuJourBean.setOrdreDuJour( ordreDuJour );

                            if ( !bIsForCreation )
                            {
                                boolean bIsForModeClassement = service.isForModeClassement(  );

                                if ( ordreDuJour.getModeClassement(  ).equals( OdsConstants.CONSTANTE_TYPE_AUTOMATIQUE ) &&
                                        bIsForModeClassement )
                                {
                                    // Mode automatique
                                    classeEntree( _entreeOrdreDuJourBean );
                                }
                            }
                            else
                            {
                                if ( ordreDuJour.getModeClassement(  ).equals( OdsConstants.CONSTANTE_TYPE_AUTOMATIQUE ) )
                                {
                                    // Mode automatique
                                    classeEntree( _entreeOrdreDuJourBean );
                                }
                                else
                                {
                                    // Mode manuel
                                    List<GEntreeOrdreDuJour> listEntrees = _entreeOrdreDuJourHome.findEntreesListByIdOrdreDuJour( ordreDuJour.getIdOrdreDuJour(  ),
                                            plugin );

                                    // on supprime la derniere entre insre de la
                                    // liste pour l'insrer a la place slectionn
                                    // par l'utilisateur
                                    if ( _nIdPositionInsert != -1 )
                                    {
                                        listEntrees.remove( listEntrees.size(  ) - 1 );
                                        listEntrees.add( _nIdPositionInsert, _entreeOrdreDuJourBean );

                                        // on update les modifications
                                        int nNumeroOrdre = 1;

                                        for ( GEntreeOrdreDuJour entree : listEntrees )
                                        {
                                            entree.setNumeroOrdre( nNumeroOrdre );
                                            _entreeOrdreDuJourHome.update( entree, plugin );
                                            nNumeroOrdre++;
                                        }
                                    }
                                }
                            }

                            // on retourne sur l'interface de modification de l'ODJ concern
                            strUrlRetour = addUrlAnchorEntreeOdj( getJspModificationOdj( request,
                                        _entreeOrdreDuJourBean.getOrdreDuJour(  ).getIdOrdreDuJour(  ) ),
                                    _entreeOrdreDuJourBean.getIdEntreeOrdreDuJour(  ) );
                        }
                    }
                }
                else if ( null != request.getParameter( CONSTANTE_AJOUTER_RAPPORTEUR ) )
                {
                    // Si l'utilisateur click sur le bouton "Ajouter un rapporteur"
                    // on insere l'lu dans l'objet EntreeOrdreDuJour en session
                    int nIdElu = OdsUtils.getIntegerParameter( OdsParameters.ID_ELU, request );

                    if ( nIdElu != -1 )
                    {
                        // si la liste des lus de l'entre en session est null
                        // on l'initialise
                        if ( _entreeOrdreDuJourBean.getElus(  ) == null )
                        {
                            _entreeOrdreDuJourBean.setElus( new ArrayList<GElu>(  ) );
                        }

                        if ( ( nIdElu != -1 ) && !isInTheEluList( nIdElu, _entreeOrdreDuJourBean.getElus(  ) ) )
                        {
                            GElu elu = _eluHome.findByPrimaryKey( nIdElu, plugin );

                            if ( elu == null )
                            {
                                return AdminMessageService.getMessageUrl( request,
                                    OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND, AdminMessage.TYPE_ERROR );
                            }

                            String strReturn = doTraitementEntreeOrdreDuJourEluSpec( request, elu );

                            if ( !strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                            {
                                return strReturn;
                            }

                            _entreeOrdreDuJourBean.getElus(  ).add( elu );
                        }
                    }

                    strUrlRetour = AppPathService.getBaseUrl( request ) + strUrlRetour +
                        OdsConstants.CONSTANTE_POINT_INTERROGATION + CONSTANTE_SESSION + OdsConstants.CONSTANTE_EGAL +
                        CONSTANTE_SESSION;
                }
                else if ( ( request.getParameter( CONSTANTE_TYPE_SUPPRESSION ) != null ) &&
                        request.getParameter( CONSTANTE_TYPE_SUPPRESSION ).equals( CONSTANTE_TYPE_SUPPRESSION_ELU ) &&
                        ( request.getParameter( OdsParameters.ID_ELU_TO_REMOVE ) != null ) )
                {
                    /*        Si l'utilisateur click sur le bouton "Suppression d'un rapporteur"
                            on supprimme l'lu selectionn dans l'objet EntreeOrdreDuJour en session
                    */
                    int nIdEluToRemove = OdsUtils.getIntegerParameter( OdsParameters.ID_ELU_TO_REMOVE, request );

                    if ( nIdEluToRemove != -1 )
                    {
                        // si la liste des lus de l'entre en session est null
                        // on l'initialise
                        if ( _entreeOrdreDuJourBean.getElus(  ) == null )
                        {
                            _entreeOrdreDuJourBean.setElus( new ArrayList<GElu>(  ) );
                        }

                        removeEluInTheList( nIdEluToRemove, _entreeOrdreDuJourBean.getElus(  ) );
                    }

                    strUrlRetour = AppPathService.getBaseUrl( request ) + strUrlRetour +
                        OdsConstants.CONSTANTE_POINT_INTERROGATION + CONSTANTE_SESSION + OdsConstants.CONSTANTE_EGAL +
                        CONSTANTE_SESSION;
                }
                else if ( request.getParameter( CONSTANTE_DESELECTION_PDD ) != null )
                {
                    // l'utilisateur click sur le bouton supprimer une
                    // proposition
                    // ou un projet de deliberation
                    _entreeOrdreDuJourBean.setReference( OdsConstants.CONSTANTE_CHAINE_VIDE );
                    _entreeOrdreDuJourBean.setObjet( OdsConstants.CONSTANTE_CHAINE_VIDE );
                    _entreeOrdreDuJourBean.setEntite( null );

                    strUrlRetour = AppPathService.getBaseUrl( request ) + strUrlRetour +
                        OdsConstants.CONSTANTE_POINT_INTERROGATION + CONSTANTE_SESSION + OdsConstants.CONSTANTE_EGAL +
                        CONSTANTE_SESSION;
                }
                else if ( request.getParameter( CONSTANTE_SELECTION_VOEU ) != null )
                {
                    // l'utilisateur click sur le bouton ajouter un voeu
                    strUrlRetour = AppPathService.getBaseUrl( request ) + JSP_SELECTION_VOEU +
                        OdsConstants.CONSTANTE_POINT_INTERROGATION + getParameterPluginName(  ) +
                        OdsConstants.CONSTANTE_ET + OdsParameters.ID_ODJ + OdsConstants.CONSTANTE_EGAL +
                        _entreeOrdreDuJourBean.getOrdreDuJour(  ).getIdOrdreDuJour(  );
                }
                else if ( request.getParameter( CONSTANTE_AJOUTER_VOEU ) != null )
                {
                    String strIsGestionAval = request.getParameter( OdsMarks.MARK_GESTION_AVAL );

                    if ( ( strIsGestionAval != null ) && strIsGestionAval.equals( "1" ) )
                    {
                        // l'utilisateur click sur le bouton ajouter un voeu en gestion aval
                        strUrlRetour = AppPathService.getBaseUrl( request ) + JSP_AJOUT_VOEU_AVAL +
                            OdsConstants.CONSTANTE_POINT_INTERROGATION + getParameterPluginName(  ) +
                            OdsConstants.CONSTANTE_ET + OdsParameters.ID_ODJ + OdsConstants.CONSTANTE_EGAL +
                            _entreeOrdreDuJourBean.getOrdreDuJour(  ).getIdOrdreDuJour(  );
                    }
                    else
                    {
                        // l'utilisateur click sur le bouton ajouter un voeu
                        strUrlRetour = AppPathService.getBaseUrl( request ) + JSP_AJOUT_VOEU +
                            OdsConstants.CONSTANTE_POINT_INTERROGATION + getParameterPluginName(  ) +
                            OdsConstants.CONSTANTE_ET + OdsParameters.ID_ODJ + OdsConstants.CONSTANTE_EGAL +
                            _entreeOrdreDuJourBean.getOrdreDuJour(  ).getIdOrdreDuJour(  );
                    }
                }
                else if ( request.getParameter( CONSTANTE_DESELECTION_VOEU ) != null )
                {
                    // l'utilisateur click sur le bouton supprimer un voeu
                    _entreeOrdreDuJourBean.setReference( OdsConstants.CONSTANTE_CHAINE_VIDE );
                    _entreeOrdreDuJourBean.setObjet( OdsConstants.CONSTANTE_CHAINE_VIDE );
                    _entreeOrdreDuJourBean.setEntite( null );

                    strUrlRetour = AppPathService.getBaseUrl( request ) + strUrlRetour +
                        OdsConstants.CONSTANTE_POINT_INTERROGATION + CONSTANTE_SESSION + OdsConstants.CONSTANTE_EGAL +
                        CONSTANTE_SESSION;
                }
                else if ( request.getParameter( CONSTANTE_AJOUTER_EXPRESSION ) != null )
                {
                    // l'utilisateur click sur le bouton inserer une expression
                    // usuelle
                    strUrlRetour = AppPathService.getBaseUrl( request ) + JSP_SELECTION_EXPRESSION +
                        OdsConstants.CONSTANTE_POINT_INTERROGATION + getParameterPluginName(  );

                    if ( ( request.getParameter( OdsParameters.ODJ_POSITION_CURSEUR ) != null ) &&
                            ( request.getParameter( OdsParameters.ODJ_POSITION_CURSEUR ).trim(  ).length(  ) > 0 ) )
                    {
                        strUrlRetour += ( OdsConstants.CONSTANTE_ET + "id_position_curseur=" +
                        request.getParameter( "id_position_curseur" ) );
                    }
                }
                else
                {
                    strUrlRetour = doTraitementEntreeOrdreDuJourSpec( request, bIsForCreation, strUrlRetour );
                }
            }
        }
        else
        {
            /*
             *  si l'utilisateur click sur le bouton annuler on retourne sur
             *  l'interface de modification de l'ordre du jour prcdemment slectionn
             *  on ajoute a la liste entres modifi l'id de la l'entre pour affichage
             */
            if ( ( _entreeOrdreDuJourBean != null ) && ( _entreeOrdreDuJourBean.getOrdreDuJour(  ) != null ) )
            {
                strUrlRetour = addUrlAnchorEntreeOdj( getJspModificationOdj( request,
                            _entreeOrdreDuJourBean.getOrdreDuJour(  ).getIdOrdreDuJour(  ) ),
                        _entreeOrdreDuJourBean.getIdEntreeOrdreDuJour(  ) );
            }
        }

        return strUrlRetour;
    }

    /**
     * Effectue la suppression d'une entre ordre du jour  partir des
     * informations contenues dans la requte
     *
     * @param request
     *            la requte Http
     * @return l'url de modification de l' ordre du jour en cours de
     *         modification
     */
    public String doSuppressionEntreeOrdreDuJour( HttpServletRequest request )
    {
        Plugin plugin = getPluginSpec(  );

        if ( null != request.getParameter( OdsParameters.ID_ENTREE ) )
        {
            int nIdEntree = OdsUtils.getIntegerParameter( OdsParameters.ID_ENTREE, request );
            GEntreeOrdreDuJour entreeOrdreDuJour = _entreeOrdreDuJourHome.findByPrimaryKey( nIdEntree, plugin );

            if ( entreeOrdreDuJour == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            try
            {
                GOrdreDuJourServiceManaged service = _mappingService.get( entreeOrdreDuJour.getIdTypeEntite(  ) );
                service.deleteEntiteForODJ( entreeOrdreDuJour.getIdEntite(  ) );
                // suppression de l'entree
                _entreeOrdreDuJourHome.remove( entreeOrdreDuJour, plugin );

                // on update les modifications
                int nNumeroOrdre = 1;
                List<GEntreeOrdreDuJour> listEntrees = _entreeOrdreDuJourHome.findEntreesListByIdOrdreDuJour( entreeOrdreDuJour.getOrdreDuJour(  )
                                                                                                                               .getIdOrdreDuJour(  ),
                        plugin );

                for ( GEntreeOrdreDuJour entree : listEntrees )
                {
                    entree.setNumeroOrdre( nNumeroOrdre );
                    _entreeOrdreDuJourHome.update( entree, plugin );
                    nNumeroOrdre++;
                }

                // on retourn sur l'inteface de modification
                return getJspModificationOdj( request, entreeOrdreDuJour.getOrdreDuJour(  ).getIdOrdreDuJour(  ) );
            }
            catch ( AppException ae )
            {
                AppLogService.error( ae );

                if ( ae.getInitialException(  ) instanceof SQLException )
                {
                    return AdminMessageService.getMessageUrl( request, MESSAGE_CANNOT_DELETE_ENTREE,
                        AdminMessage.TYPE_STOP );
                }
            }
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * Effectue la suppression de le liste des entes d'ordre du jour
     * selectionnes par l'utisateur
     *
     * @param request
     *            la requte Http
     * @return l'url de modification de l' ordre du jour en cours de
     *         modification
     */
    public String doSuppressionListeEntree( HttpServletRequest request )
    {
        Plugin plugin = getPluginSpec(  );

        if ( ( _strTabIdSelect != null ) && ( _strTabIdSelect.length != 0 ) &&
                ( request.getParameter( OdsParameters.ID_ODJ ) != null ) )
        {
            for ( String strIdEntre : _strTabIdSelect )
            {
                try
                {
                    int nIdEntree = Integer.parseInt( strIdEntre );
                    GEntreeOrdreDuJour entreeOrdreDuJour = _entreeOrdreDuJourHome.findByPrimaryKey( nIdEntree, plugin );

                    if ( entreeOrdreDuJour == null )
                    {
                        return AdminMessageService.getMessageUrl( request,
                            OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND, AdminMessage.TYPE_ERROR );
                    }

                    GOrdreDuJourServiceManaged service = _mappingService.get( entreeOrdreDuJour.getIdTypeEntite(  ) );
                    service.deleteEntiteForODJ( entreeOrdreDuJour.getIdEntite(  ) );

                    _entreeOrdreDuJourHome.remove( entreeOrdreDuJour, plugin );
                }
                catch ( NumberFormatException ne )
                {
                    return OdsConstants.CONSTANTE_CHAINE_VIDE;
                }
            }

            int nIdOdj = OdsUtils.getIntegerParameter( OdsParameters.ID_ODJ, request );

            if ( nIdOdj != -1 )
            {
                // on update les modifications
                int nNumeroOrdre = 1;
                List<GEntreeOrdreDuJour> listEntrees = _entreeOrdreDuJourHome.findEntreesListByIdOrdreDuJour( nIdOdj,
                        plugin );

                for ( GEntreeOrdreDuJour entree : listEntrees )
                {
                    entree.setNumeroOrdre( nNumeroOrdre );
                    _entreeOrdreDuJourHome.update( entree, plugin );
                    nNumeroOrdre++;
                }

                // on retourne sur l'intreface de modification de l' ODJ concern
                return getJspModificationOdj( request, nIdOdj );
            }
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * insere le PDD selectionn dans l'entre d'ordre du jour en session
     *
     * @param request la requte Http
     * @return retourne sur l'entre d'ordre du jour en cours de traitement
     */
    public String doSelectionPDD( HttpServletRequest request )
    {
        Plugin plugin = getPluginSpec(  );

        // Si l'action effectue est "Annuler" on en fait rien.
        if ( ( request.getParameter( OdsParameters.A_FAIRE ) != null ) &&
                !request.getParameter( OdsParameters.A_FAIRE ).equals( CONSTANTE_CANCEL ) )
        {
            // Si l'id du pdd n'est pas donn en paramtre message
            if ( ( request.getParameter( OdsParameters.ID_PDD ) == null ) ||
                    request.getParameter( OdsParameters.ID_PDD ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                return AdminMessageService.getMessageUrl( request, MESSAGE_PAS_DE_PDD_CHOISI, AdminMessage.TYPE_STOP );
            }

            // On rcupre l'id du pdd et si il est invalide => retour  la case
            // dpart
            int nIdPDD = OdsUtils.getIntegerParameter( OdsParameters.ID_PDD, request );

            if ( nIdPDD == -1 )
            {
                return OdsConstants.CONSTANTE_CHAINE_VIDE;
            }

            // On insert dans l'objet entreeOrdreDuJour en session le pdd dont
            // on a rcupr l'id
            GPDD pddSelected = _projetDeliberationHome.findByPrimaryKey( nIdPDD, plugin );

            if ( pddSelected == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            doSelectionPDDSpec( pddSelected, _entreeOrdreDuJourBean.getOrdreDuJour(  ).getSeance(  ) );
            _entreeOrdreDuJourBean.setIdEntite( pddSelected.getId(  ) );
            _entreeOrdreDuJourBean.setEntite( pddSelected );

            // Les champs de saisie de la rfrence et de lobjet affichs sont
            // valoriss avec la rfrence
            // et lobjet du PDD slectionn.
            _entreeOrdreDuJourBean.setReference( pddSelected.getReference(  ) );
            _entreeOrdreDuJourBean.setObjet( pddSelected.getObjet(  ) );
        }

        if ( _entreeOrdreDuJourBean.getIdEntreeOrdreDuJour(  ) == -1 )
        {
            // Si l'id de l'entree en session =-1 retour sur l'interface de
            // creation
            return AppPathService.getBaseUrl( request ) + JSP_CREATION_ENTREE_ORDRE_DU_JOUR +
            OdsConstants.CONSTANTE_POINT_INTERROGATION + CONSTANTE_SESSION + OdsConstants.CONSTANTE_EGAL +
            CONSTANTE_SESSION;
        }
        else
        {
            // retour sur l'interface de modification
            return AppPathService.getBaseUrl( request ) + JSP_MODIFICATION_ENTREE_ORDRE_DU_JOUR +
            OdsConstants.CONSTANTE_POINT_INTERROGATION + CONSTANTE_SESSION + OdsConstants.CONSTANTE_EGAL +
            CONSTANTE_SESSION;
        }
    }

    /**
     * methode qui cre pour chaque Pdds selectionns une entre d'ordre du jour
     *
     * @param request
     *            la requte Http
     * @return retour sur l'ordre du jour en cours de modification
     */
    public String doSelectionPDDs( HttpServletRequest request )
    {
        Plugin plugin = getPluginSpec(  );

        int nIdEntree = OdsUtils.getIntegerParameter( OdsParameters.ID_ENTREE, request );

        if ( nIdEntree != -1 )
        {
            GEntreeOrdreDuJour entreeOrdreDuJour = _entreeOrdreDuJourHome.findByPrimaryKey( nIdEntree, plugin );

            if ( entreeOrdreDuJour == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            if ( request.getParameter( CONSTANTE_ANNULER ) == null )
            {
                // recupration de la liste des Id pdds  ajouter au va
                String[] strListePdd = request.getParameterValues( OdsParameters.LISTE_PDD_CHECKED );

                if ( ( strListePdd != null ) && ( strListePdd.length != 0 ) )
                {
                    // on recupere l'entre d'ordre du jour dans le but de
                    // recuperer
                    // son groupe de commissions et son groupe de rapporteurs
                    GEntreeOrdreDuJour newEntree;
                    IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD> pdd;
                    List<GEntreeOrdreDuJour> listEntreeATrier = new ArrayList<GEntreeOrdreDuJour>(  );

                    for ( String strIdPdd : strListePdd )
                    {
                        int nIdPdd;

                        try
                        {
                            nIdPdd = Integer.parseInt( strIdPdd );
                            pdd = _projetDeliberationHome.findByPrimaryKey( nIdPdd, plugin );
                        }
                        catch ( NumberFormatException ne )
                        {
                            return OdsConstants.CONSTANTE_CHAINE_VIDE;
                        }

                        newEntree = getEntreeOrdreDuJour(  );
                        newEntree.setIdTypeEntite( ITypeEntite.PDD );
                        newEntree.setIdEntite( pdd.getId(  ) );
                        newEntree.setEntite( pdd );
                        newEntree.setReference( pdd.getReference(  ) );
                        newEntree.setObjet( pdd.getObjet(  ) );
                        newEntree.setOrdreDuJour( entreeOrdreDuJour.getOrdreDuJour(  ) );
                        newEntree.setIdEntreeOrdreDuJour( _entreeOrdreDuJourHome.create( newEntree, plugin ) );

                        setEntreeDataSpec( newEntree, entreeOrdreDuJour );

                        listEntreeATrier.add( newEntree );

                        if ( _listIdNouvelleEntree == null )
                        {
                            _listIdNouvelleEntree = new ArrayList<Integer>(  );
                        }

                        // on ajoute a la liste des nouvelles entres l'id de la
                        // nouvelle entre
                        _listIdNouvelleEntree.add( newEntree.getIdEntreeOrdreDuJour(  ) );
                    }

                    // Les commissions et les rapporteurs des entres qui sont
                    // cres sont celles du groupe choisi.
                    for ( GEntreeOrdreDuJour entree : listEntreeATrier )
                    {
                        entree.setElus( entreeOrdreDuJour.getElus(  ) );

                        setEntreeOtherDataSpec( entree, entreeOrdreDuJour );

                        for ( GElu elu : entree.getElus(  ) )
                        {
                            _entreeOrdreDuJourHome.insertRapporteur( entree.getIdEntreeOrdreDuJour(  ), elu, plugin );
                        }
                    }

                    // on reclasse l'ordre du jour en mode automatique
                    classeListEntrees( listEntreeATrier );
                }
                else
                {
                    String strMessage = null;

                    if ( ( null != request.getParameter( OdsParameters.TYPE ) ) &&
                            request.getParameter( OdsParameters.TYPE ).equals( IPDD.CONSTANTE_TYPE_PROJET ) )
                    {
                        strMessage = MESSAGE_PAS_DE_PROJET_CHOISIE;
                    }
                    else
                    {
                        strMessage = MESSAGE_PAS_DE_PROPOSITION_CHOISIE;
                    }

                    return AdminMessageService.getMessageUrl( request, strMessage, AdminMessage.TYPE_STOP );
                }
            }
            else
            {
                if ( _listIdNouvelleEntree == null )
                {
                    _listIdNouvelleEntree = new ArrayList<Integer>(  );
                }

                _listIdNouvelleEntree.add( entreeOrdreDuJour.getIdEntreeOrdreDuJour(  ) );
            }

            return getJspModificationOdj( request, entreeOrdreDuJour.getOrdreDuJour(  ).getIdOrdreDuJour(  ) );
        }
        else
        {
            return OdsConstants.CONSTANTE_CHAINE_VIDE;
        }
    }

    /**
     * insere une expression usuelle dans l'entre d'ordre du jour en session
     *
     * @param request
     *            la requte Http
     * @return l'url de traitement de l'entre d'ordre du jour
     */
    public String doSelectionExpression( HttpServletRequest request )
    {
        Plugin plugin = getPluginSpec(  );

        // Si l'action effectue est "Annuler" on en fait rien.
        int nIndexCurseur = OdsUtils.getIntegerParameter( OdsParameters.ODJ_POSITION_CURSEUR, request );

        if ( ( request.getParameter( CONSTANTE_ANNULER ) == null ) )
        {
            // Si l'id du de l'expression n'est pas donn en paramtre => Retour
            //  la case dpart
            if ( ( request.getParameter( OdsParameters.ID_EXPRESSION ) == null ) ||
                    request.getParameter( OdsParameters.ID_EXPRESSION ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                return AdminMessageService.getMessageUrl( request, MESSAGE_PAS_EXPRESSION_CHOISI, AdminMessage.TYPE_STOP );
            }

            // On rcupre l'id du pdd et si il est invalide => retour  la case
            // dpart
            int nIdExpression = -1;

            try
            {
                nIdExpression = Integer.parseInt( request.getParameter( OdsParameters.ID_EXPRESSION ) );
            }
            catch ( NumberFormatException ne )
            {
                return OdsConstants.CONSTANTE_CHAINE_VIDE;
            }

            if ( nIdExpression == -1 )
            {
                return OdsConstants.CONSTANTE_CHAINE_VIDE;
            }

            IExpressionUsuelle expressionSelected = _expressionUsuelleHome.findByPrimaryKey( nIdExpression, plugin );

            if ( expressionSelected == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            // Le texte de lexpression est copi dans lobjet,  la suite du
            // texte dj saisi le cas chant
            if ( nIndexCurseur == -1 )
            {
                String strObjet = ( _entreeOrdreDuJourBean.getObjet(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                    ? expressionSelected.getExpression(  )
                    : ( _entreeOrdreDuJourBean.getObjet(  ) + OdsConstants.CONSTANTE_ESPACE +
                    expressionSelected.getExpression(  ) );
                _entreeOrdreDuJourBean.setObjet( strObjet );
            }
            else
            {
                String strObjet = _entreeOrdreDuJourBean.getObjet(  ).substring( 0, nIndexCurseur ) +
                    expressionSelected.getExpression(  ) +
                    _entreeOrdreDuJourBean.getObjet(  )
                                          .substring( nIndexCurseur, _entreeOrdreDuJourBean.getObjet(  ).length(  ) );
                _entreeOrdreDuJourBean.setObjet( strObjet );
            }
        }

        if ( _entreeOrdreDuJourBean.getIdEntreeOrdreDuJour(  ) == -1 )
        {
            // Si l'id de l'entree en session =-1 retour sur l'interface de
            // creation
            return AppPathService.getBaseUrl( request ) + JSP_CREATION_ENTREE_ORDRE_DU_JOUR +
            OdsConstants.CONSTANTE_POINT_INTERROGATION + CONSTANTE_SESSION + OdsConstants.CONSTANTE_EGAL +
            CONSTANTE_SESSION;
        }
        else
        {
            // retour sur l'interface de modification
            return AppPathService.getBaseUrl( request ) + JSP_MODIFICATION_ENTREE_ORDRE_DU_JOUR +
            OdsConstants.CONSTANTE_POINT_INTERROGATION + CONSTANTE_SESSION + OdsConstants.CONSTANTE_EGAL +
            CONSTANTE_SESSION;
        }
    }

    /**
     * Insre un voeu dans l'entre de l'ordre du jour
     *
     * @param request la requte Http
     * @return l'url de traitement de l'entre d'ordre du jour
     */
    public String doSelectionVoeuAmendement( HttpServletRequest request )
    {
        Plugin plugin = getPluginSpec(  );

        // Si l'action effectue est "Annuler" on en fait rien.
        if ( ( ( request.getParameter( OdsParameters.A_FAIRE ) != null ) &&
                !request.getParameter( OdsParameters.A_FAIRE ).equals( CONSTANTE_CANCEL ) ) ||
                ( request.getParameter( OdsParameters.A_FAIRE ) == null ) )
        {
            // Si l'id de l'amendement n'est pas donn en paramtre => Retour 
            // la case dpart
            if ( ( request.getParameter( OdsParameters.ID_VOEUAMENDEMENT ) == null ) ||
                    request.getParameter( OdsParameters.ID_VOEUAMENDEMENT ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                return AdminMessageService.getMessageUrl( request, MESSAGE_PAS_DE_VOEU_CHOISI, AdminMessage.TYPE_STOP );
            }

            // On rcupre l'id de l'amendement et si il est invalide => retour
            //  la case dpart
            int nIdVA = OdsUtils.getIntegerParameter( OdsParameters.ID_VOEUAMENDEMENT, request );

            if ( nIdVA == -1 )
            {
                return AdminMessageService.getMessageUrl( request, Messages.MANDATORY_FIELDS, AdminMessage.TYPE_STOP );
            }

            // On met en session l'amendement dont on a rcupr l'id
            GVoeuAmendement va = _voeuAmendementHome.findByPrimaryKey( nIdVA, plugin );

            if ( va == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            _entreeOrdreDuJourBean.setIdEntite( va.getId(  ) );
            _entreeOrdreDuJourBean.setEntite( va );
        }

        if ( _entreeOrdreDuJourBean.getIdEntreeOrdreDuJour(  ) == -1 )
        {
            // Si l'id de l'entree en session =-1 retour sur l'interface de
            // creation
            return AppPathService.getBaseUrl( request ) + JSP_CREATION_ENTREE_ORDRE_DU_JOUR +
            OdsConstants.CONSTANTE_POINT_INTERROGATION + CONSTANTE_SESSION + OdsConstants.CONSTANTE_EGAL +
            CONSTANTE_SESSION;
        }
        else
        {
            // retour sur l'interface de modification
            return AppPathService.getBaseUrl( request ) + JSP_MODIFICATION_ENTREE_ORDRE_DU_JOUR +
            OdsConstants.CONSTANTE_POINT_INTERROGATION + CONSTANTE_SESSION + OdsConstants.CONSTANTE_EGAL +
            CONSTANTE_SESSION;
        }
    }

    /**
     * methode qui cre pour chaque voeux selectionns une entre d'ordre du
     * jour
     *
     * @param request
     *            la requte Http
     * @return retour sur l'ordre du jour en cours de modification
     */
    public String doSelectionVoeuxAmendements( HttpServletRequest request )
    {
        Plugin plugin = getPluginSpec(  );

        if ( null != request.getParameter( OdsParameters.ID_ENTREE ) )
        {
            int nIdEntree = OdsUtils.getIntegerParameter( OdsParameters.ID_ENTREE, request );
            GEntreeOrdreDuJour entreeOrdreDuJour = _entreeOrdreDuJourHome.findByPrimaryKey( nIdEntree, plugin );

            if ( entreeOrdreDuJour == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            if ( request.getParameter( CONSTANTE_ANNULER ) == null )
            {
                // recupration de la liste des Id pdds  ajouter au va
                String[] strListeVoeu = request.getParameterValues( OdsParameters.LISTE_VA_CHECKED );

                if ( ( strListeVoeu != null ) && ( strListeVoeu.length != 0 ) )
                {
                    // on recupere l'entre d'ordre du jour dans le but de
                    // recuperer
                    // son groupe de commissions et son groupe de rapporteurs
                    GEntreeOrdreDuJour newEntree;
                    GVoeuAmendement voeuAmendement;
                    List<GEntreeOrdreDuJour> listEntreeATrier = new ArrayList<GEntreeOrdreDuJour>(  );

                    for ( String strIdVoeu : strListeVoeu )
                    {
                        int nIdVa;

                        try
                        {
                            nIdVa = Integer.parseInt( strIdVoeu );
                            voeuAmendement = _voeuAmendementHome.findByPrimaryKey( nIdVa, plugin );
                        }
                        catch ( NumberFormatException ne )
                        {
                            return OdsConstants.CONSTANTE_CHAINE_VIDE;
                        }

                        newEntree = getEntreeOrdreDuJour(  );
                        newEntree.setIdTypeEntite( ITypeEntite.VOEUX );
                        newEntree.setIdEntite( voeuAmendement.getId(  ) );
                        newEntree.setEntite( voeuAmendement );
                        newEntree.setReference( voeuAmendement.getReferenceComplete(  ) );
                        newEntree.setOrdreDuJour( entreeOrdreDuJour.getOrdreDuJour(  ) );
                        newEntree.setIdEntreeOrdreDuJour( _entreeOrdreDuJourHome.create( newEntree, plugin ) );

                        setEntreeDataSpec( newEntree, entreeOrdreDuJour );

                        if ( _listIdNouvelleEntree == null )
                        {
                            _listIdNouvelleEntree = new ArrayList<Integer>(  );
                        }

                        // on ajoute a la liste des nouvelles entres l'id de la
                        // nouvelle entre
                        _listIdNouvelleEntree.add( newEntree.getIdEntreeOrdreDuJour(  ) );

                        listEntreeATrier.add( newEntree );
                    }

                    // Les commissions et les rapporteurs des entres qui sont
                    // cres sont celles du groupe choisi.
                    for ( GEntreeOrdreDuJour entree : listEntreeATrier )
                    {
                        entree.setElus( entreeOrdreDuJour.getElus(  ) );

                        setEntreeOtherDataSpec( entree, entreeOrdreDuJour );

                        for ( GElu elu : entree.getElus(  ) )
                        {
                            _entreeOrdreDuJourHome.insertRapporteur( entree.getIdEntreeOrdreDuJour(  ), elu, plugin );
                        }
                    }

                    // on reclasse l'ordre du jour en mode automatique
                    classeListEntrees( listEntreeATrier );
                }
                else
                {
                    return AdminMessageService.getMessageUrl( request, MESSAGE_PAS_DE_VOEU_CHOISI,
                        AdminMessage.TYPE_STOP );
                }
            }

            else
            {
                if ( _listIdNouvelleEntree == null )
                {
                    _listIdNouvelleEntree = new ArrayList<Integer>(  );
                }

                _listIdNouvelleEntree.add( entreeOrdreDuJour.getIdEntreeOrdreDuJour(  ) );
            }

            return getJspModificationOdj( request, entreeOrdreDuJour.getOrdreDuJour(  ).getIdOrdreDuJour(  ) );
        }
        else
        {
            return OdsConstants.CONSTANTE_CHAINE_VIDE;
        }
    }

    /**
     * methode qui gere l'ensemble des traitements effectus sur l'ordre du jour en paramtre de la request
     *
     * @param request request
     * @return l'url de modification de l'ordre du jour en cours de traitement
     */
    public String doTraitementOdj( HttpServletRequest request, String strReturnDefault )
    {
        Plugin plugin = getPluginSpec(  );
        String strUrlRetour = strReturnDefault;

        if ( ( request.getParameter( OdsParameters.ID_ODJ ) != null ) &&
                ( request.getParameter( OdsParameters.A_FAIRE ) != null ) )
        {
            int nIdOdj = OdsUtils.getIntegerParameter( OdsParameters.ID_ODJ, request );
            String strAction = request.getParameter( OdsParameters.A_FAIRE );

            List<GEntreeOrdreDuJour> listEntrees = _entreeOrdreDuJourHome.findEntreesListByIdOrdreDuJour( nIdOdj, plugin );

            if ( ( listEntrees == null ) || listEntrees.isEmpty(  ) )
            {
                _strTabIdSelect = null;
                strUrlRetour = AdminMessageService.getMessageUrl( request, MESSAGE_AUCUNE_ENTREE, AdminMessage.TYPE_STOP );
            }
            else
            {
                _strTabIdSelect = new String[listEntrees.size(  )];

                for ( int i = 0; i < listEntrees.size(  ); i++ )
                {
                    _strTabIdSelect[i] = OdsConstants.CONSTANTE_CHAINE_VIDE +
                        listEntrees.get( i ).getIdEntreeOrdreDuJour(  );
                }
            }

            String strReturn = doTraitementOdjSpec( request, strAction, nIdOdj );

            if ( !strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                strUrlRetour = strReturn;
            }
        }

        return strUrlRetour;
    }

    /**
     * methode qui gere l'ensemble des traitements effectus sur la liste des
     * entres d'ordre du jour en cours de traitement
     *
     * @param request
     *            request
     * @return l'url de modification de l'ordre du jour en cours de traitement
     */
    public String doTraitementEntreeOdj( HttpServletRequest request, String strReturnDefault )
    {
        Plugin plugin = getPluginSpec(  );
        String strUrlRetour = strReturnDefault;

        if ( request.getParameter( OdsParameters.ID_ODJ ) != null )
        {
            String strIdOdj = request.getParameter( OdsParameters.ID_ODJ );

            boolean bIsEntite = false;

            for ( Entry<Integer, GOrdreDuJourServiceManaged> entry : _mappingService.entrySet(  ) )
            {
                GOrdreDuJourServiceManaged serviceManaged = entry.getValue(  );

                if ( request.getParameter( serviceManaged.getAddParameter(  ) ) != null )
                {
                    strUrlRetour = AppPathService.getBaseUrl( request ) + JSP_CREATION_ENTREE_ORDRE_DU_JOUR +
                        OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.TYPE + OdsConstants.CONSTANTE_EGAL +
                        entry.getKey(  ) + OdsConstants.CONSTANTE_ET + OdsParameters.ID_ODJ +
                        OdsConstants.CONSTANTE_EGAL + strIdOdj;

                    bIsEntite = true;

                    break;
                }
            }

            if ( ( request.getParameter( CONSTANTE_AJOUTER_AVANT_SELECTION ) != null ) ||
                    ( request.getParameter( CONSTANTE_AJOUTER_APRES_SELECTION ) != null ) )
            {
                // demande de creation d'une entre d'ordre du jour avant ou
                // aprs
                // l'entre d'ordre du jour selectionne
                _nIdPositionInsert = -1;

                // recupration de la liste des entres selectionnes
                String[] strListEntree = request.getParameterValues( OdsParameters.LISTE_ENTREE_CHECKED );

                if ( ( request.getParameter( OdsParameters.TYPE ) != null ) &&
                        ( !request.getParameter( OdsParameters.TYPE ).trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) ) &&
                        ( !request.getParameter( OdsParameters.TYPE ).equals( OdsConstants.FILTER_ALL ) ) )
                {
                    if ( ( strListEntree != null ) && ( strListEntree.length != 0 ) )
                    {
                        try
                        {
                            int nIdEntree;
                            GEntreeOrdreDuJour entree;

                            if ( request.getParameter( CONSTANTE_AJOUTER_AVANT_SELECTION ) != null )
                            {
                                nIdEntree = Integer.parseInt( strListEntree[0] );
                                entree = _entreeOrdreDuJourHome.findByPrimaryKey( nIdEntree, plugin );

                                if ( entree == null )
                                {
                                    return AdminMessageService.getMessageUrl( request,
                                        OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND, AdminMessage.TYPE_ERROR );
                                }

                                _nIdPositionInsert = entree.getNumeroOrdre(  ) - 1;
                            }
                            else
                            {
                                nIdEntree = Integer.parseInt( strListEntree[strListEntree.length - 1] );
                                entree = _entreeOrdreDuJourHome.findByPrimaryKey( nIdEntree, plugin );

                                if ( entree == null )
                                {
                                    return AdminMessageService.getMessageUrl( request,
                                        OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND, AdminMessage.TYPE_ERROR );
                                }

                                _nIdPositionInsert = entree.getNumeroOrdre(  );
                            }

                            String strType = request.getParameter( OdsParameters.TYPE );
                            strUrlRetour = AppPathService.getBaseUrl( request ) + JSP_CREATION_ENTREE_ORDRE_DU_JOUR +
                                OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.TYPE +
                                OdsConstants.CONSTANTE_EGAL + strType + OdsConstants.CONSTANTE_ET +
                                OdsParameters.ID_ODJ + OdsConstants.CONSTANTE_EGAL + strIdOdj;
                        }
                        catch ( NumberFormatException ne )
                        {
                            AppLogService.error( ne );

                            return OdsConstants.CONSTANTE_CHAINE_VIDE;
                        }
                    }
                    else
                    {
                        //pas d'entre choisie
                        try
                        {
                            int nIdOdj = Integer.parseInt( strIdOdj );

                            if ( _entreeOrdreDuJourHome.findCountEntreesListByIdOrdreDuJour( nIdOdj, plugin ) == 0 )
                            {
                                //pas d'entree dans la liste des entrees d'ordre du jour on insere 
                                //la nouvelle entree en premiere position 
                                _nIdPositionInsert = 0;

                                String strType = request.getParameter( OdsParameters.TYPE );
                                strUrlRetour = AppPathService.getBaseUrl( request ) +
                                    JSP_CREATION_ENTREE_ORDRE_DU_JOUR + OdsConstants.CONSTANTE_POINT_INTERROGATION +
                                    OdsParameters.TYPE + OdsConstants.CONSTANTE_EGAL + strType +
                                    OdsConstants.CONSTANTE_ET + OdsParameters.ID_ODJ + OdsConstants.CONSTANTE_EGAL +
                                    strIdOdj;
                            }
                            else
                            { //message pas d'entree choisie
                                strUrlRetour = AdminMessageService.getMessageUrl( request, MESSAGE_PAS_ENTREE_CHOISIE,
                                        AdminMessage.TYPE_STOP );
                            }
                        }
                        catch ( NumberFormatException ne )
                        {
                            AppLogService.error( ne );

                            return OdsConstants.CONSTANTE_CHAINE_VIDE;
                        }
                    }
                }
                else
                {
                    //pas de type d'entr choisi
                    // on retourne sur l'interface de modification de l'
                    // ODJ concern
                    try
                    {
                        int nIdOdj = Integer.parseInt( strIdOdj );
                        strUrlRetour = getJspModificationOdj( request, nIdOdj );
                    }
                    catch ( NumberFormatException ne )
                    {
                        AppLogService.error( ne );
                    }
                }
            }
            else if ( request.getParameter( CONSTANTE_SUPPRIMER_SELECTION ) != null )
            {
                // demande de suppression des entes selectionnes
                // recupration de la liste des entres  supprimer
                String[] strListEntree = request.getParameterValues( OdsParameters.LISTE_ENTREE_CHECKED );

                if ( ( strListEntree != null ) && ( strListEntree.length != 0 ) )
                {
                    _strTabIdSelect = strListEntree;

                    UrlItem url = new UrlItem( JSP_DO_SUPPRESSION_LISTE_ENTREE );
                    url.addParameter( OdsParameters.ID_ODJ, strIdOdj );
                    strUrlRetour = AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRME_DELETE_LISTE_ENTREE,
                            url.getUrl(  ), AdminMessage.TYPE_CONFIRMATION );
                }

                else
                {
                    strUrlRetour = AdminMessageService.getMessageUrl( request, MESSAGE_PAS_ENTREE_CHOISIE,
                            AdminMessage.TYPE_STOP );
                }
            }
            else if ( request.getParameter( CONSTANTE_DEPLACER_SELECTION ) != null )
            {
                // recupration de la liste des entres  deplacer
                String[] strListEntree = request.getParameterValues( OdsParameters.LISTE_ENTREE_CHECKED );

                if ( ( strListEntree != null ) && ( strListEntree.length != 0 ) )
                {
                    try
                    {
                        int nIdOdj = Integer.parseInt( strIdOdj );
                        List<GEntreeOrdreDuJour> listEntree = _entreeOrdreDuJourHome.findEntreesListByIdOrdreDuJour( nIdOdj,
                                plugin );

                        if ( ( listEntree.size(  ) <= 1 ) || ( strListEntree.length == listEntree.size(  ) ) )
                        {
                            // si il n'y a qu'une entre dans la liste ou si la selection concerne toutes les entres
                            // on retourne directement  l'interface de modification
                            strUrlRetour = getJspModificationOdj( request, nIdOdj );
                        }
                        else
                        {
                            _strTabIdSelect = strListEntree;
                            strUrlRetour = AppPathService.getBaseUrl( request ) + JSP_SELECTION_ENTREE_ORDRE_DU_JOUR +
                                OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_ODJ +
                                OdsConstants.CONSTANTE_EGAL + strIdOdj;
                        }
                    }
                    catch ( NumberFormatException ne )
                    {
                        AppLogService.error( ne );

                        return OdsConstants.CONSTANTE_CHAINE_VIDE;
                    }
                }

                else
                {
                    strUrlRetour = AdminMessageService.getMessageUrl( request, MESSAGE_PAS_ENTREE_CHOISIE,
                            AdminMessage.TYPE_STOP );
                }
            }
            else
            {
                String strReturn = doTraitementEntreeOdjSpec( request, bIsEntite, strIdOdj );

                if ( !strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                {
                    strUrlRetour = strReturn;
                }
            }
        }

        return strUrlRetour;
    }

    /**
     * Numerote les voeux et amendements de l'ordre du jour definitif
     *
     * @param request
     *            request
     * @return l'url de modification de l'ordre du jour en cours de traitement
     */
    public String doNumeroteVa( HttpServletRequest request )
    {
        Plugin plugin = getPluginSpec(  );

        if ( request.getParameter( OdsParameters.ID_ODJ ) != null )
        {
            int nIdOdj = OdsUtils.getIntegerParameter( OdsParameters.ID_ODJ, request );

            // recuperation de l'ordre du jour
            GOrdreDuJour ordreDuJour = _ordreDuJourHome.findByPrimaryKey( nIdOdj, plugin );

            if ( ordreDuJour == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            doNumeroteVaSpec( ordreDuJour );

            return getJspModificationOdj( request, nIdOdj );
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * Deplace la liste des entres selectionnes  la position choisie par
     * l'utilisateur
     *
     * @param request
     *            request
     * @return l'url de modification de l'ordre du jour en cours de traitement
     */
    public String doDeplaceListeEntree( HttpServletRequest request )
    {
        Plugin plugin = getPluginSpec(  );

        if ( ( _strTabIdSelect != null ) && ( _strTabIdSelect.length != 0 ) )
        {
            List<GEntreeOrdreDuJour> listEntreeOrdreDuJour;
            List<GEntreeOrdreDuJour> listEntreeADeplacer;
            int nIdEntreeSelect = -1;
            int nIdEntreeDeplace = -1;
            int nIdOdj = -1;

            if ( ( request.getParameter( CONSTANTE_ANNULER ) != null ) ||
                    ( ( request.getParameter( OdsParameters.ID_ODJ ) != null ) &&
                    ( request.getParameter( OdsParameters.ID_ENTREE ) != null ) &&
                    ( request.getParameter( OdsParameters.TYPE ) != null ) ) )
            {
                nIdOdj = OdsUtils.getIntegerParameter( OdsParameters.ID_ODJ, request );

                if ( nIdOdj == -1 )
                {
                    return OdsConstants.CONSTANTE_CHAINE_VIDE;
                }

                if ( request.getParameter( CONSTANTE_ANNULER ) == null )
                {
                    nIdEntreeSelect = OdsUtils.getIntegerParameter( OdsParameters.ID_ENTREE, request );

                    listEntreeOrdreDuJour = _entreeOrdreDuJourHome.findEntreesListByIdOrdreDuJour( nIdOdj, plugin );
                    listEntreeADeplacer = new ArrayList<GEntreeOrdreDuJour>(  );

                    for ( String strIdEntree : _strTabIdSelect )
                    {
                        try
                        {
                            nIdEntreeDeplace = Integer.parseInt( strIdEntree );
                        }
                        catch ( NumberFormatException ne )
                        {
                            AppLogService.error( ne );

                            return OdsConstants.CONSTANTE_CHAINE_VIDE;
                        }

                        for ( GEntreeOrdreDuJour entree : listEntreeOrdreDuJour )
                        {
                            if ( entree.getIdEntreeOrdreDuJour(  ) == nIdEntreeDeplace )
                            {
                                listEntreeADeplacer.add( entree );
                            }
                        }
                    }

                    // on supprime les entres avant de les inserer a la
                    // position selectionn
                    listEntreeOrdreDuJour.removeAll( listEntreeADeplacer );

                    // on determine l'index de l'entre selectionn
                    int nIndex = 0;

                    for ( GEntreeOrdreDuJour entree : listEntreeOrdreDuJour )
                    {
                        if ( entree.getIdEntreeOrdreDuJour(  ) == nIdEntreeSelect )
                        {
                            break;
                        }

                        nIndex++;
                    }

                    if ( request.getParameter( OdsParameters.TYPE ).equals( CONSTANTE_AJOUTER_APRES_SELECTION ) )
                    {
                        nIndex++;
                    }

                    //on ajoute a la liste des nouvelles entres l'id de la
                    // nouvelle entre
                    if ( _listIdNouvelleEntree == null )
                    {
                        _listIdNouvelleEntree = new ArrayList<Integer>(  );
                    }

                    for ( GEntreeOrdreDuJour entree : listEntreeADeplacer )
                    {
                        _listIdNouvelleEntree.add( entree.getIdEntreeOrdreDuJour(  ) );

                        listEntreeOrdreDuJour.add( nIndex, entree );
                        nIndex++;
                    }

                    // mise a joure en base des numero d'ordre des entres
                    int nNumeroOrdre = 1;

                    for ( GEntreeOrdreDuJour entree : listEntreeOrdreDuJour )
                    {
                        entree.setNumeroOrdre( nNumeroOrdre );
                        _entreeOrdreDuJourHome.update( entree, plugin );
                        nNumeroOrdre++;
                    }
                }

                return getJspModificationOdj( request, nIdOdj );
            }
            else
            {
                return AdminMessageService.getMessageUrl( request, MESSAGE_NO_ENTREE_SELECTED, AdminMessage.TYPE_STOP );
            }
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * Retourne l'en-tte du modle d'ordre du jour pass en paramtre
     * @param ordreDuJour l'ordre du jour
     * @param plugin le plugin
     * @return l'en-tte de l'ordre du jour pass en paramtre
     */
    public Entete getEnteteFromModele( GOrdreDuJour ordreDuJour, GOrdreDuJourFilter ordreDuJourFilter )
    {
        Plugin plugin = getPluginSpec(  );

        Entete entete = null;

        if ( ordreDuJour.getFormationConseil(  ) != null )
        {
            ordreDuJourFilter.setIdFormationConseil( ordreDuJour.getFormationConseil(  ).getIdFormationConseil(  ) );
        }

        if ( ordreDuJour.getTypeOrdreDuJour(  ) != null )
        {
            ordreDuJourFilter.setIdType( ordreDuJour.getTypeOrdreDuJour(  ).getIdTypeOrdreDuJour(  ) );
        }

        ordreDuJourFilter.setIdSauvegarde( 0 );

        GModeleOrdreDuJour modeleOrdreDuJour = _modeleOrdreDuJourHome.findByFilter( ordreDuJourFilter, plugin );

        if ( ( modeleOrdreDuJour != null ) && ( modeleOrdreDuJour.getEnteteDocument(  ) != null ) )
        {
            FichierPhysique enteteFichierPhysique = modeleOrdreDuJour.getEnteteDocument(  );
            enteteFichierPhysique = _fichierPhysiqueHome.findByPrimaryKey( enteteFichierPhysique.getIdFichier(  ),
                    plugin );

            if ( enteteFichierPhysique.getDonnees(  ) != null )
            {
                try
                {
                    entete = OrdreDuJourUtils.analyzeEnteteXML( enteteFichierPhysique.getDonnees(  ) );
                }

                catch ( JAXBException e )
                {
                    AppLogService.error( "getEnteteFromModele - " + e.getClass(  ) + " : " + e.getMessage(  ) );
                }

                catch ( Exception e )
                {
                    AppLogService.error( "getEnteteFromModele - " + e.getClass(  ) + " : " + e.getMessage(  ) );
                }
            }
        }

        return entete;
    }

    /**
     * Retourne le pied de page du modle d'ordre du jour pass en paramtre
     * @param ordreDuJour l'ordre du jour
     * @param plugin le plugin
     * @return le pied de page de l'ordre du jour pass en paramtre
     */
    public PiedDePage getPiedDePageFromModele( GOrdreDuJour ordreDuJour, GOrdreDuJourFilter ordreDuJourFilter )
    {
        Plugin plugin = getPluginSpec(  );

        PiedDePage piedDePage = null;

        if ( ordreDuJour.getFormationConseil(  ) != null )
        {
            ordreDuJourFilter.setIdFormationConseil( ordreDuJour.getFormationConseil(  ).getIdFormationConseil(  ) );
        }

        if ( ordreDuJour.getTypeOrdreDuJour(  ) != null )
        {
            ordreDuJourFilter.setIdType( ordreDuJour.getTypeOrdreDuJour(  ).getIdTypeOrdreDuJour(  ) );
        }

        ordreDuJourFilter.setIdSauvegarde( 0 );

        GModeleOrdreDuJour modeleOrdreDuJour = _modeleOrdreDuJourHome.findByFilter( ordreDuJourFilter, plugin );

        if ( ( modeleOrdreDuJour != null ) && ( modeleOrdreDuJour.getPiedDocument(  ) != null ) )
        {
            FichierPhysique piedDePageFichierPhysique = modeleOrdreDuJour.getPiedDocument(  );
            piedDePageFichierPhysique = _fichierPhysiqueHome.findByPrimaryKey( piedDePageFichierPhysique.getIdFichier(  ),
                    plugin );

            if ( piedDePageFichierPhysique.getDonnees(  ) != null )
            {
                try
                {
                    piedDePage = OrdreDuJourUtils.analyzePiedDePageXML( piedDePageFichierPhysique.getDonnees(  ) );
                }
                catch ( JAXBException e )
                {
                    AppLogService.error( e );
                }

                catch ( Exception e )
                {
                    AppLogService.error( e );
                }
            }
        }

        return piedDePage;
    }

    /**
     * Ajoute une entre dans l'entte ou le pied de page de l'ordre du jour pass en paramtre
     *
     * @param odj l'ordre du jour
     * @param plugin le plugin
     * @param isEntete true si l'entre est  ajouter dans une entte, false si c'est dans un pied de page
     */
    public void ajoutEntreeEntetePiedPage( GOrdreDuJour odj, Plugin plugin, boolean isEntete )
    {
        GSeance seance = _seanceHome.getProchaineSeance( plugin );

        if ( isEntete )
        {
            Entete entete = OrdreDuJourUtils.getEntete( odj, plugin );

            if ( entete != null )
            {
                entete.getEntreeEntete(  ).add( new Entete.EntreeEntete(  ) );
                odj.setXmlEntete( setXmlEnteteSpec( entete, seance ) );

                _ordreDuJourHome.update( odj, plugin );
            }
        }
        else
        {
            PiedDePage piedPage = OrdreDuJourUtils.getPiedDePage( odj, plugin );

            if ( piedPage != null )
            {
                piedPage.getEntreePiedDePage(  ).add( new PiedDePage.EntreePiedDePage(  ) );
                odj.setXmlPiedDePage( setXmlPiedPageSpec( piedPage, seance ) );

                _ordreDuJourHome.update( odj, plugin );
            }
        }
    }

    /**
     * Restaure l'entte ou le pied de page de l'ordre du jour pass en paramtre avec celui du modle
     *
     * @param odj l'ordre du jour
     * @param plugin le plugin
     * @param isEntete true si l'entre est  ajouter dans une entte, false si c'est dans un pied de page
     * @return true si la restauration s'est bien passe, false sinon
     */
    public boolean restaureEntetePiedPage( GOrdreDuJour odj, Plugin plugin, boolean isEntete )
    {
        GSeance seance = _seanceHome.getProchaineSeance( plugin );
        boolean bIisOK;

        if ( isEntete )
        {
            Entete entete = getEnteteFromModeleSpec( odj );

            if ( entete != null )
            {
                odj.setXmlEntete( setXmlEnteteSpec( entete, seance ) );
                _ordreDuJourHome.update( odj, plugin );
                bIisOK = true;
            }
            else
            {
                bIisOK = false;
            }
        }
        else
        {
            PiedDePage piedPage = getPiedDePageFromModeleSpec( odj );

            if ( piedPage != null )
            {
                odj.setXmlPiedDePage( setXmlPiedPageSpec( piedPage, seance ) );
                _ordreDuJourHome.update( odj, plugin );
                bIisOK = true;
            }
            else
            {
                bIisOK = false;
            }
        }

        return bIisOK;
    }

    /**
     * Gnre le code HTML pour le Front-Office
     * @param nOdjId l'id de l'odj
     * @param plugin le plugin
     * @param locale la locale
     * @param model le modle pour la template
     * @return le code HTML gnr
     */
    public String getFrontHtml( int nOdjId, Plugin plugin, Locale locale, HashMap<String, Object> model )
    {
        return genererFrontHtml( nOdjId, plugin, locale, model );
    }

    /**
     * Gnre le code HTML pour le Front-Office
     * @param nOdjId l'id de l'odj
     * @param plugin le plugin
     * @param locale la locale
     * @param model le modle pour la template
     * @param bIsFront true si c'est pour le Front, false sinon
     * @return le code HTML gnr
     */
    private String genererFrontHtml( int nOdjId, Plugin plugin, Locale locale, HashMap<String, Object> model )
    {
        GOrdreDuJour ordreDuJour = _ordreDuJourHome.findByPrimaryKey( nOdjId, plugin );

        if ( ordreDuJour != null )
        {
            GSeance seance = _seanceHome.findByPrimaryKey( ordreDuJour.getSeance(  ).getIdSeance(  ), plugin );
            ordreDuJour.setSeance( seance );

            genererFrontHtmlSpec( ordreDuJour, model );
            model.put( IOrdreDuJour.MARK_ODJ, ordreDuJour );
        }

        String sTemplateName = genererFrontHtmlTemplateSpec( ordreDuJour );
        HtmlTemplate template = AppTemplateService.getTemplate( sTemplateName, locale, model );

        return template.getHtml(  );
    }

    /**
     * retourne le libelle de l'lu   insrer dans le PDF
     * @param elu l'lu
     * @param locale la locale
     * @return le libelle de l'lu   insrer dans le PDF
     */
    protected String getLibelleEluPdf( GElu elu, Locale locale )
    {
        StringBuffer strObjet = new StringBuffer(  );
        strObjet.append( elu.getCivilite(  ) );
        strObjet.append( OdsConstants.CONSTANTE_ESPACE );
        strObjet.append( elu.getPrenomElu(  ) );
        strObjet.append( OdsConstants.CONSTANTE_ESPACE );
        strObjet.append( elu.getNomElu(  ) );
        strObjet.append( OdsConstants.CONSTANTE_ESPACE );

        getLibelleEluPdfSpec( elu, strObjet, locale );

        return strObjet.toString(  );
    }

    protected abstract void getLibelleEluPdfSpec( GElu elu, StringBuffer buffer, Locale locale );

    /**
     * retourne l'odj gnr en Word
     * @param ordreDuJour l'odj  gnrer en Word
     * @param plugin le plugin
     * @param locale la locale
     * @return l'odj gnr en Word
     */
    public byte[] genereODJInWord( GOrdreDuJour ordreDuJour, Plugin plugin, Locale locale )
    {
        byte[] bConvertedDoc;

        if ( !DocumentManager.checkJCopist(  ) )
        {
            return null;
        }

        if ( ordreDuJour == null )
        {
            return null;
        }

        doClasseEntree( locale, ordreDuJour );

        GSeance seance = _seanceHome.findByPrimaryKey( ordreDuJour.getSeance(  ).getIdSeance(  ), plugin );
        ordreDuJour.setSeance( seance );

        try
        {
            Entete entete = OrdreDuJourUtils.getEntete( ordreDuJour, plugin );
            PiedDePage piedDePage = OrdreDuJourUtils.getPiedDePage( ordreDuJour, plugin );

            if ( entete != null )
            {
                ordreDuJour.setXmlEntete( setXmlEnteteSpec( entete, seance ) );
            }

            if ( piedDePage != null )
            {
                ordreDuJour.setXmlPiedDePage( setXmlPiedPageSpec( piedDePage, seance ) );
            }

            String strXml = ordreDuJour.getXml(  );

            byte[] xml = ( OdsConstants.CONSTANTE_ENTETE_XML + strXml ).getBytes( OdsConstants.UTF8 );
            bConvertedDoc = DocumentManager.convert( DocumentManager.PDF_WORD, OdsConstants.CONSTANTE_TEMPLATE_XML_ODJ,
                    xml );

            return bConvertedDoc;
        }
        catch ( Exception e )
        {
            AppLogService.error( "Creation Word de l'ODJ : " + e.getMessage(  ) );

            return null;
        }
    }

    /**
     * Permet de savoir si il s'agit d'une demande de slection simple de PDD pour Ordre du Jour
     * @param strUrlJspAppelant l'url de la JSP source.
     * @return TRUE, si c'est pour le traitement des ordres du jour, FALSE sinon
     */
    public boolean estPourSelectionSimpleOrdreDuJour( String strUrlJspAppelant )
    {
        boolean bRetour = false;

        if ( strUrlJspAppelant.equals( JSP_SELECTION_PROJET ) || strUrlJspAppelant.equals( JSP_SELECTION_PROPOSITION ) )
        {
            bRetour = true;
        }

        return bRetour;
    }

    /**
     * Permet de savoir si il s'agit d'une demande de slection multiple de PDD pour Ordre du Jour
     * @param strUrlJspAppelant l'url de la JSP source.
     * @return TRUE, si c'est pour le traitement des entres d'Ordre du jour, FALSE sinon
     */
    public boolean estPourSelectionMultipleOrdreDuJour( String strUrlJspAppelant )
    {
        boolean bRetour = false;

        if ( strUrlJspAppelant.equals( JSP_SELECTION_PROJETS ) ||
                strUrlJspAppelant.equals( JSP_SELECTION_PROPOSITIONS ) )
        {
            bRetour = true;
        }

        return bRetour;
    }

    /**
     * cre une entre de type texte a partir de la liste des rapporteurs
     * passes en parametre
     *
     * @param rapporteurs la liste des rapporteurs  transformer en entre d'ordre du jour
     * @param ordreDuJour l'ordre du jour en cours de traitement
     * @param locale la locale
     * @return l'entre d'ordre du jour cre
     */
    protected GEntreeOrdreDuJour createEntreerapporteurs( List<GElu> rapporteurs, GOrdreDuJour ordreDuJour,
        Locale locale )
    {
        GEntreeOrdreDuJour newEntreeOdj = getEntreeOrdreDuJour(  );
        newEntreeOdj.setOrdreDuJour( ordreDuJour );
        newEntreeOdj.setIdTypeEntite( getTypeEntreeRapporteurs(  ) );

        if ( AppPropertiesService.getProperty( PROPERTY_ODJ_GROUPE_DE_RAPPORTEURS_STYLE ) != null )
        {
            newEntreeOdj.setStyle( AppPropertiesService.getProperty( PROPERTY_ODJ_GROUPE_DE_RAPPORTEURS_STYLE ) );
        }
        else
        {
            newEntreeOdj.setStyle( CONSTANTE_STYLE_H4 );
        }

        newEntreeOdj.setObjet( getLibelleGroupeRapporteursSpec( rapporteurs, locale ) );

        return newEntreeOdj;
    }

    /**
     * ajoute une ligne de VA dans le CSV gnr
     * @param va le VA  ajouter
     * @param strSeance la sance en chaine de caractres
     * @return un tableau contenant tous les lments de la ligne du CSV
     */
    @SuppressWarnings( "unchecked" )
    protected String[] addVACsvLine( GVoeuAmendement va, String strSeance, Locale locale )
    {
        Plugin plugin = getPluginSpec(  );

        List<String> listField = new ArrayList<String>(  );

        listField.add( strSeance );
        listField.add( va.getReferenceComplete(  ) );

        // Dpos Par
        String strElus = OdsConstants.CONSTANTE_CHAINE_VIDE;
        String strGroupe = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( va.getDeposeExecutif(  ) )
        {
            listField.add( I18nService.getLocalizedString( PROPERTY_LABEL_EXECUTIF, locale ) );
        }
        else
        {
            if ( ( va.getDepositaires(  ) != null ) && ( va.getDepositaires(  ).size(  ) > 0 ) )
            {
                for ( IDepositaire depositaire : va.getDepositaires(  ) )
                {
                    if ( depositaire.getType(  ) == IDepositaire.DEPOSITAIRE_TYPE_ELU )
                    {
                        String strGpeTemp = ( (GElu) depositaire ).getGroupe(  ).getNomGroupe(  );

                        if ( !strGroupe.contains( strGpeTemp ) )
                        {
                            strGroupe += ( strGpeTemp + OdsConstants.CONSTANTE_VIRGULE + OdsConstants.CONSTANTE_ESPACE );
                        }

                        if ( ( ( (GElu) depositaire ).getPrenomElu(  ) == null ) ||
                                ( ( (GElu) depositaire ).getNomElu(  ) == null ) )
                        {
                            depositaire = _eluHome.findByPrimaryKey( ( (GElu) depositaire ).getIdElu(  ), plugin );
                        }

                        strElus += ( ( (GElu) depositaire ).getPrenomElu(  ) + OdsConstants.CONSTANTE_ESPACE +
                        ( (GElu) depositaire ).getNomElu(  ) + OdsConstants.CONSTANTE_VIRGULE +
                        OdsConstants.CONSTANTE_ESPACE );
                    }

                    if ( depositaire.getType(  ) == IDepositaire.DEPOSITAIRE_TYPE_GROUPE )
                    {
                        String strGpeTemp = ( (GroupePolitique) depositaire ).getNomGroupe(  );

                        if ( !strGroupe.contains( strGpeTemp ) )
                        {
                            strGroupe += ( strGpeTemp + OdsConstants.CONSTANTE_VIRGULE + OdsConstants.CONSTANTE_ESPACE );
                        }
                    }
                }

                if ( ( strElus.length(  ) > 2 ) &&
                        strElus.substring( strElus.length(  ) - 2, strElus.length(  ) - 1 )
                                   .contains( OdsConstants.CONSTANTE_VIRGULE ) )
                {
                    strElus = strElus.substring( 0, strElus.length(  ) - 2 );
                }

                if ( ( strGroupe.length(  ) > 2 ) &&
                        strGroupe.substring( strGroupe.length(  ) - 2, strGroupe.length(  ) - 1 )
                                     .contains( OdsConstants.CONSTANTE_VIRGULE ) )
                {
                    strGroupe = strGroupe.substring( 0, strGroupe.length(  ) - 2 );
                }
            }

            listField.add( strGroupe );
        }

        listField.add( strElus );
        listField.add( StringUtils.substituteSpecialCaractersForCSV( va.getObjet(  ).trim(  ) ) );
        listField.add( va.getType(  ) );
        listField.add( OdsConstants.CONSTANTE_CHAINE_VIDE );

        return StringUtils.transformListToTab( listField );
    }

    /**
     * Ajoute une ligne d'entre dans le CSV
     * @param entree l'entre  insrer
     * @param strSeance la sance
     * @param listToCSVWriter la liste contenant les lignes du CSV
     */
    @SuppressWarnings( "unchecked" )
    protected void addEntreeCsvLine( GEntreeOrdreDuJour entree, String strSeance, List<String[]> listToCSVWriter,
        Locale locale )
    {
        Plugin plugin = getPluginSpec(  );

        List<String> listField = new ArrayList<String>(  );
        listField.add( strSeance );

        if ( ( entree.getIdTypeEntite(  ) == ITypeEntite.VOEUX ) && ( entree.getEntite(  ) != null ) )
        {
            listField.add( ( (GVoeuAmendement) entree.getEntite(  ) ).getReferenceComplete(  ) );
        }
        else if ( entree.getReference(  ) != null )
        {
            if ( ( (GPDD) entree.getEntite(  ) ).getTypePdd(  ).equals( OdsConstants.CONSTANTE_ENTREE_TYPE_PROJET ) &&
                    ( ( (GPDD) entree.getEntite(  ) ).getFormationConseil(  ).getIdFormationConseil(  ) == 1 ) )
            {
                //On ajoute  "-G"  la rfrence, si l'entre est un projet et le conseil est gnral (idFormationConseil = 1)
                listField.add( entree.getReference(  ) + CONSTANTE_G );
            }
            else
            {
                listField.add( entree.getReference(  ) );
            }
        }

        //La colonne "Entit dpositaire" est toujours vide pour les PDDs
        listField.add( OdsConstants.CONSTANTE_CHAINE_VIDE );
        //La colonne "Elu dpositaire" est toujours vide pour les PDDs
        listField.add( OdsConstants.CONSTANTE_CHAINE_VIDE );

        if ( ( entree.getIdTypeEntite(  ) == ITypeEntite.VOEUX ) && ( entree.getEntite(  ) != null ) )
        {
            listField.add( StringUtils.substituteSpecialCaractersForCSV( 
                    ( (GVoeuAmendement) entree.getEntite(  ) ).getObjet(  ).trim(  ) ) );
        }
        else if ( entree.getIdTypeEntite(  ) == ITypeEntite.PDD )
        {
            listField.add( StringUtils.substituteSpecialCaractersForCSV( entree.getObjet(  ).trim(  ) ) );
        }

        if ( entree.getIdTypeEntite(  ) == ITypeEntite.PDD )
        {
            GPDD pdd = (GPDD) entree.getEntite(  );

            if ( ( pdd != null ) && pdd.isDelegationsServices(  ) )
            {
                listField.add( IPDD.CONSTANTE_TYPE_DSP );
            }
            else if ( pdd != null )
            {
                listField.add( pdd.getTypePdd(  ) );
            }
            else
            {
                listField.add( IPDD.CONSTANTE_TYPE_PROJET );
            }
        }
        else
        {
            GOrdreDuJourServiceManaged service = _mappingService.get( entree.getIdTypeEntite(  ) );
            String strTypeEntree = service.getTypeEntree(  );

            listField.add( strTypeEntree );
        }

        listField.add( getLibelleGroupeRapporteursSpec( entree.getElus(  ), locale ) );

        //On insre la ligne nouvellement cr dans la liste des lignes du CSV
        listToCSVWriter.add( StringUtils.transformListToTab( listField ) );

        //Si le PDD est rattach  des VAs, on insre cas VAs.
        if ( ( entree.getIdTypeEntite(  ) == ITypeEntite.PDD ) && ( entree.getEntite(  ) != null ) )
        {
            List<GVoeuAmendement> vaRattaches = ( (GPDD) entree.getEntite(  ) ).getVoeuxAmendements(  );

            for ( GVoeuAmendement va : vaRattaches )
            {
                va = _voeuAmendementHome.findByPrimaryKey( va.getId(  ), plugin );
                listToCSVWriter.add( addVACsvLine( va, strSeance, locale ) );
            }
        }
    }

    /**
     * methode qui classe automatiquement la nouvelle entre fournie en
     * parametre dans la liste des entres de l'ordre du jour
     *
     * @param newEntree l'entre a classer dans la liste des entres
     */
    protected void classeEntree( GEntreeOrdreDuJour newEntree )
    {
        Plugin plugin = getPluginSpec(  );

        // initialistaion du numero d'ordre de la premiere entre dans la liste
        // a 1
        int nCpt = 1;

        // on recupere la liste des entes de l'ordre du jour
        GOrdreDuJour ordreDuJour = (GOrdreDuJour) newEntree.getOrdreDuJour(  );
        ordreDuJour.setListEntrees( _entreeOrdreDuJourHome.findEntreesListByIdOrdreDuJour( 
                ordreDuJour.getIdOrdreDuJour(  ), plugin ) );
        // on supprime l'entre  positionner de la liste des entres
        removeEntreeInTheList( newEntree.getIdEntreeOrdreDuJour(  ), ordreDuJour.getListEntrees(  ) );

        int nNewEntreePosition = getPositionNewEntree( newEntree, ordreDuJour );
        // on insere la nouvelle entre dans la liste des entres
        ordreDuJour.getListEntrees(  ).add( nNewEntreePosition, newEntree );

        for ( GEntreeOrdreDuJour entree : ordreDuJour.getListEntrees(  ) )
        {
            entree.setNumeroOrdre( nCpt );
            _entreeOrdreDuJourHome.update( entree, plugin );
            nCpt++;
        }
    }

    /**
     * methode qui classe automatiquement la liste des entres fournies en
     * parametre dans la liste des entres de l'ordre du jour
     *
     * @param entrees
     *            la liste des entres a classer dans la liste des entres
     */
    protected void classeListEntrees( List<GEntreeOrdreDuJour> entrees )
    {
        Plugin plugin = getPluginSpec(  );
        GOrdreDuJour ordreDuJour = null;

        if ( entrees.size(  ) != 0 )
        {
            ordreDuJour = (GOrdreDuJour) entrees.get( 0 ).getOrdreDuJour(  );
        }

        OdsUtils.testNullObject( ordreDuJour );
        ordreDuJour = _ordreDuJourHome.findByPrimaryKey( ordreDuJour.getIdOrdreDuJour(  ), plugin );
        ordreDuJour.setListEntrees( _entreeOrdreDuJourHome.findEntreesListByIdOrdreDuJour( 
                ordreDuJour.getIdOrdreDuJour(  ), plugin ) );

        int nCpt = 1;

        for ( GEntreeOrdreDuJour newEntree : entrees )
        {
            // on supprime l'entre  positionner de la liste des entres
            removeEntreeInTheList( newEntree.getIdEntreeOrdreDuJour(  ), ordreDuJour.getListEntrees(  ) );

            int nNewEntreePosition = getPositionNewEntree( newEntree, ordreDuJour );
            // on insere la nouvelle entre dans la liste des entres
            ordreDuJour.getListEntrees(  ).add( nNewEntreePosition, newEntree );
        }

        // update sur la liste des entres
        for ( GEntreeOrdreDuJour entree : ordreDuJour.getListEntrees(  ) )
        {
            entree.setNumeroOrdre( nCpt );
            _entreeOrdreDuJourHome.update( entree, plugin );
            nCpt++;
        }
    }

    /**
     * cre une sauvegarde de l'ordre du jour et l'insere dans l'ordre du jour
     * pass en parametre
     *
     * @param ordreDuJour
     *            l'ordre du jour a sauvegarder
     * @param plugin
     *            plugin
     */
    protected void createSauvegardeOdj( GOrdreDuJour ordreDuJour, Plugin plugin )
    {
        int nIdOrdreDuJour = ordreDuJour.getIdOrdreDuJour(  );

        // Une sauvegarde de lordre du jour est cre
        GOrdreDuJour ordreDuJourSauveGarde = createNewOdj(  );
        // parametre estsauvegarde = true juste pour la creation de la sauvegarde
        ordreDuJour.setEstSauvegarde( true );
        ordreDuJourSauveGarde.setIdOrdreDuJour( _ordreDuJourHome.create( ordreDuJour, plugin ) );
        // on update l'ordre du jour
        ordreDuJour.setEstSauvegarde( false );
        ordreDuJour.setIdOrdreDuJour( nIdOrdreDuJour );
        ordreDuJour.setModeClassement( OdsConstants.CONSTANTE_TYPE_MANUEL );
        ordreDuJour.setOrdreDuJourSauveGarde( ordreDuJourSauveGarde );
        _ordreDuJourHome.update( ordreDuJour, plugin );

        List<GEntreeOrdreDuJour> listEntreOrdreDuJour = _entreeOrdreDuJourHome.findEntreesListByIdOrdreDuJour( ordreDuJour.getIdOrdreDuJour(  ),
                plugin );
        int nIdNewEntree;

        for ( GEntreeOrdreDuJour entree : listEntreOrdreDuJour )
        {
            entree.setOrdreDuJour( ordreDuJourSauveGarde );
            nIdNewEntree = _entreeOrdreDuJourHome.create( entree, plugin );
            entree.setIdEntreeOrdreDuJour( nIdNewEntree );

            createSauvegardeOdjSpec( entree );

            if ( entree.getElus(  ) != null )
            {
                // insertion des rapporteurs
                for ( GElu elu : entree.getElus(  ) )
                {
                    _entreeOrdreDuJourHome.insertRapporteur( nIdNewEntree, elu, plugin );
                }
            }
        }
    }

    protected void getInitEntreeOrdreDuJour( HttpServletRequest request, HashMap<String, Object> model )
    {
        if ( ( _entreeOrdreDuJourBean != null ) && ( _entreeOrdreDuJourBean.getIdTypeEntite(  ) != -1 ) )
        {
            GOrdreDuJourServiceManaged service = _mappingService.get( _entreeOrdreDuJourBean.getIdTypeEntite(  ) );

            if ( service.isRapporteurEnabled(  ) )
            {
                ReferenceList refListRapporteur = initRefListRapporteur( request, _entreeOrdreDuJourBean.getElus(  ),
                        getPluginSpec(  ) );
                model.put( MARK_LISTE_RAPPORTEUR, refListRapporteur );
            }

            getInitEntreeOrdreDuJourSpec( model, service, request.getLocale(  ) );
        }
    }

    protected abstract GFichier newFichierInstance(  );

    protected abstract GFichierFilter newFichierFilterInstance(  );
}
