/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.ordredujour;

import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.service.IODSService;
import fr.paris.lutece.portal.service.plugin.Plugin;

import java.io.IOException;

import java.util.HashMap;
import java.util.List;
import java.util.Locale;

import javax.servlet.http.HttpServletRequest;


public interface IOrdreDuJourService<GOrdreDuJour extends IOrdreDuJour<GEntreeOrdreDuJour, GSeance, GFichier, GElu, GOrdreDuJour>, GEntreeOrdreDuJour extends IEntreeOrdreDuJour<GElu, GSeance, GFichier, GEntreeOrdreDuJour, GOrdreDuJour>, GOrdreDuJourFilter extends IOrdreDuJourFilter, GSeance extends ISeance, GElu extends IElu, GFichier extends IFichier<GSeance, GFichier>>
    extends IODSService
{
    String JSP_CREATION_ENTREE_ORDRE_DU_JOUR = "jsp/admin/plugins/ods/ordredujour/CreationEntreeOrdreDuJour.jsp";
    String JSP_MODIFICATION_ENTREE_ORDRE_DU_JOUR = "jsp/admin/plugins/ods/ordredujour/ModificationEntreeOrdreDuJour.jsp";

    /**
     * Initialise les variables de session
     */
    void initSession(  );

    /**
     * Retourne l'URL de selection d'une expression usuelle
     * @return l'URL de selection d'une expression usuelle
     */
    String getUrlSelectionExpression(  );

    /**
     * Retourne l'URL de selection des voeux pour l'ordredujour
     * @return l'URL de selection des voeux pour l'ordredujour
     */
    String getUrlSelectionVoeu(  );

    /**
     * Retourne l'URL de selection des voeux pour l'ordredujour
     * @return l'URL de selection des voeux pour l'ordredujour
     */
    String getUrlSelectionVoeuRetour(  );

    /**
     * Retourne la map contenant les lments utiles au template HTML de la liste
     *
     * @param request la requte Http
     * @return model la map contenant les lments utiles au template HTML de la liste
     */
    HashMap<String, Object> getOrdreDuJourList( HttpServletRequest request );

    /**
     * Retourne la map contenant les lments utiles au template HTML de la liste
     *
     * @param request la requte Http
     * @return model la map contenant les lments utiles au template HTML de la liste
     */
    HashMap<String, Object> getOrdreDuJourGestionAvalList( HttpServletRequest request );

    /**
     * Retourne la map contenant les lments utiles au template HTML de modification
     *
     * @param request la requte Http
     * @return model la map contenant les lments utiles au template HTML de modification
     */
    HashMap<String, Object> getModificationOrdreDuJour( HttpServletRequest request );

    /**
     * Retourne la map contenant les lments utiles au template HTML de la suppression de l'ordre du jour
     *
     * @param request la requte Http
     * @return model la map contenant les lments utiles au template HTML de la suppression de l'ordre du jour
     */
    HashMap<String, String> getSuppressionOrdreDuJour( HttpServletRequest request );

    /**
     * Retourne la map contenant les lments utiles au template HTML de modification de l'entete et pied de page
     *
     * @param request la requte Http
     * @return model la map contenant les lments utiles au template HTML de modification de l'entete et pied de page
     */
    HashMap<String, Object> getModificationEntetePiedPage( HttpServletRequest request );

    /**
     * Retourne l'interface de confirmation de suppression d'une entre d'une entte
     *                                 ou d'un pied de page
     *
     * @param request
     *                    la requte Http
     * @return 'interface de confirmation de suppression d'une entre d'une entte
     *                                 ou d'un pied de page
     */
    String getSuppressionEntreeEntetePiedPage( HttpServletRequest request );

    /**
     * Retourne l'interface de confirmation de publication si au moins un pdd
     * rattach  l'ordre du jour n'est pas publi sinon redirection vers le jsp
     * de publication
     *
     * @param request
     *            la requte Http
     * @return l'interface de confirmation de publication si au moins un pdd
     *         rattach  l'ordre du jour n'est pas publi sinon redirection
     *         vers le jsp de publication
     */
    String getPublicationOrdreDuJour( HttpServletRequest request );

    /**
     * Retourne l'interface de confirmation de depublication
     *
     * @param request
     *            la requte Http
     * @return l'interface de confirmation de publication si au moins un pdd
     *         rattach  l'ordre du jour n'est pas publi sinon redirection
     *         vers le jsp de publication
     */
    String getDepublicationOrdreDuJour( HttpServletRequest request );

    /**
     * Retourne la map contenant les lments utiles au template HTML de la cration d'une entre
     *
     * @param request la requte Http
     * @return model la map contenant les lments utiles au template HTML de la cration d'une entre
     */
    HashMap<String, Object> getCreationEntreeOrdreDuJour( HttpServletRequest request );

    /**
     * Retourne la map contenant les lments utiles au template HTML de la modification d'une entre
     *
     * @param request la requte Http
     * @return model la map contenant les lments utiles au template HTML de la modification d'une entre
     */
    HashMap<String, Object> getModificationEntreeOrdreDuJour( HttpServletRequest request );

    /**
     * Retourne l'interface de confirmation de suppression d'une entre d'ordre
     * du jour
     *
     * @param request
     *            la requte Http
     * @return l'interface de confirmation de suppression d'une entre d'ordre
     *         du jour
     */
    String getSuppressionEntreeOrdreDuJour( HttpServletRequest request );

    /**
     * Retourne la map contenant les lments utiles au template HTML de slection
     *
     * @param request la requte Http
     * @return model la map contenant les lments utiles au template HTML de slection
     */
    HashMap<String, Object> getSelectionEntreeOrdreDuJour( HttpServletRequest request );

    /**
     * Effectue l'ensemble des traitements necessaire lors de la cration d'une
     * entre de l'ordre du jour  partir des informations contenues dans la
     * requte creation d'une nouvelle entre ,ajout d'un rapporteur ,d'une
     * commission  l'entre en cours de creation.....
     *
     * @param request
     *            la requte Http
     * @return en fonction des diffrents traitements retour vers l'url de
     *         modification d'un ordre du jour ,de creation de l'entre en cours
     *         de creation...
     */
    String doCreationEntreeOrdreDuJour( String strReturnUrl, HttpServletRequest request );

    /**
     * Effectue la cration d'un ordre du jour  partir des informations
     * contenues dans la requte
     *
     * @param request
     *            la requte Http
     * @return l'url de gestion des ordres du jour si la requte est valide
     */
    String doCreationOrdreDuJour( HttpServletRequest request );

    /**
     * Effectue la modification d'un ordre du jour  partir des informations
     * contenues dans la requte
     *
     * @param request
     *            la requte Http
     * @return l'url de gestion des ordres du jour
     */
    String doModificationOrdreDuJour( HttpServletRequest request );

    /**
     * Effectue la modification de l'entte ou du pied de page d'un ODJ
     *
     * @param request
     *            la requte Http
     * @return l'url de modification de l'ordre du jour
     */
    String doModificationEntetePiedPage( HttpServletRequest request );

    /**
     * Effectue l'ensemble des traitements necessaire lors d'une modification
     * d'une entre d'ordre du jour  partir des informations contenues dans la
     * requte modification d'une entre ,ajout d'un rapporteur ,d'une
     * commission  l'entre en cours de modification.....
     *
     * @param request la requte Http
     * @return en fonction des diffrents traitements retour vers l'url de
     *         modification d'un ordre du jour ,de creation de l'entre en cours
     *         de creation...
     */
    String doModificationEntreeOrdreDuJour( String strReturnUrl, HttpServletRequest request );

    /**
     * Effectue la suppression d'une entre ordre du jour  partir des
     * informations contenues dans la requte
     *
     * @param request
     *            la requte Http
     * @return l'url de modification de l' ordre du jour en cours de
     *         modification
     */
    String doSuppressionEntreeOrdreDuJour( HttpServletRequest request );

    /**
     * Effectue la suppression d'une entre d'entte ou pied de page d'un ordre du jour  partir des
     * informations contenues dans la requte
     *
     * @param request
     *            la requte Http
     * @return l'url de modification d'entte ou pied de page de l' ordre du jour
     */
    String doSuppressionEntreeEntetePiedPage( HttpServletRequest request );

    /**
     * Effectue la suppression de le liste des entes d'ordre du jour
     * selectionnes par l'utisateur
     *
     * @param request
     *            la requte Http
     * @return l'url de modification de l' ordre du jour en cours de
     *         modification
     */
    String doSuppressionListeEntree( HttpServletRequest request );

    /**
     * insere le PDD selectionn dans l'entre d'ordre du jour en session
     *
     * @param request la requte Http
     * @return retourne sur l'entre d'ordre du jour en cours de traitement
     */
    String doSelectionPDD( HttpServletRequest request );

    /**
     * methode qui cre pour chaque Pdds selectionns une entre d'ordre du jour
     *
     * @param request
     *            la requte Http
     * @return retour sur l'ordre du jour en cours de modification
     */
    String doSelectionPDDs( HttpServletRequest request );

    /**
     * insere une expression usuelle dans l'entre d'ordre du jour en session
     *
     * @param request
     *            la requte Http
     * @return l'url de traitement de l'entre d'ordre du jour
     */
    String doSelectionExpression( HttpServletRequest request );

    /**
     * Insre un voeu dans l'entre de l'ordre du jour
     *
     * @param request la requte Http
     * @return l'url de traitement de l'entre d'ordre du jour
     */
    String doSelectionVoeuAmendement( HttpServletRequest request );

    /**
     * methode qui cre pour chaque voeux selectionns une entre d'ordre du
     * jour
     *
     * @param request
     *            la requte Http
     * @return retour sur l'ordre du jour en cours de modification
     */
    String doSelectionVoeuxAmendements( HttpServletRequest request );

    /**
     * methode qui gere l'ensemble des traitements effectus sur l'ordre du jour en paramtre de la request
     *
     * @param request request
     * @return l'url de modification de l'ordre du jour en cours de traitement
     */
    String doTraitementOdj( HttpServletRequest request, String strReturnDefault );

    /**
     * methode qui gere l'ensemble des traitements effectus sur la liste des
     * entres d'ordre du jour en cours de traitement
     *
     * @param request
     *            request
     * @return l'url de modification de l'ordre du jour en cours de traitement
     */
    String doTraitementEntreeOdj( HttpServletRequest request, String strReturnDefault );

    /**
     * Retourne l'ordre du jour sous format PDF ou Word
     * @param request la requete Http
     * @return l'ordre du jour
     */
    GOrdreDuJour doVisualisationODJ( HttpServletRequest request );

    /**
     * Retourne l'ordre du jour sous format csv
     * @param request la requete Http
     * @return listToCSVWriter le tableau utile pour la gnration CSV
     */
    List<String[]> doVisualisationODJcsv( HttpServletRequest request );

    /**
     * genre les odj de la prochaine sance au format word et les enregistrer sur le File System
     * @param request la requte HTTP
     * @return 0 si OK, 10 sinon
     * @throws IOException IOException
     */
    String doGenereODJ( HttpServletRequest request ) throws IOException;

    /**
     * Effectue la suppression d'un ordre du jour  partir des informations
     * contenues dans la requte
     *
     * @param request
     *            la requte Http
     * @return l'url de gestion des ordres du jour
     */
    String doSuppressionOrdreDuJour( HttpServletRequest request );

    /**
     * Effectue la publication d'un ordre du jour  partir de l'id de l'odj
     * contenu dans la requette
     *
     * @param request
     *            la requte Http
     * @return l'url de gestion des ordres du jour si la requte est valide, un
     *         message d'erreur sinon
     */
    String doPublicationOrdreDuJour( HttpServletRequest request );

    /**
     * Effectue la dpublication d'un ordre du jour  partir de l'id de l'odj
     * contenu dans la requette
     *
     * @param request
     *            la requte Http
     * @return l'url de gestion des ordres du jour si la requte est valide, un
     *         message d'erreur sinon
     */
    String doDepublicationOrdreDuJour( HttpServletRequest request );

    /**
     * Numerote les voeux et amendements de l'ordre du jour definitif
     *
     * @param request
     *            request
     * @return l'url de modification de l'ordre du jour en cours de traitement
     */
    String doNumeroteVa( HttpServletRequest request );

    /**
     * Deplace la liste des entres selectionnes  la position choisie par
     * l'utilisateur
     *
     * @param request
     *            request
     * @return l'url de modification de l'ordre du jour en cours de traitement
     */
    String doDeplaceListeEntree( HttpServletRequest request );

    /**
     * Gnre le code HTML pour le Front-Office
     * @param nOdjId l'id de l'odj
     * @param plugin le plugin
     * @param locale la locale
     * @param model le modle pour la template
     * @return le code HTML gnr
     */
    String getFrontHtml( int nOdjId, Plugin plugin, Locale locale, HashMap<String, Object> model );

    /**
     * Permet de savoir si il s'agit d'une demande de slection simple de PDD pour Ordre du Jour
     * @param strUrlJspAppelant l'url de la JSP source.
     * @return TRUE, si c'est pour le traitement des ordres du jour, FALSE sinon
     */
    boolean estPourSelectionSimpleOrdreDuJour( String strUrlJspAppelant );

    /**
     * Permet de savoir si il s'agit d'une demande de slection multiple de PDD pour Ordre du Jour
     * @param strUrlJspAppelant l'url de la JSP source.
     * @return TRUE, si c'est pour le traitement des entres d'Ordre du jour, FALSE sinon
     */
    boolean estPourSelectionMultipleOrdreDuJour( String strUrlJspAppelant );

    String getPageTitleCreationEntreeODJ(  );

    String getPageTitleModificationEntreeODJ(  );
}
