/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.pdd;

import fr.paris.lutece.plugins.ods.business.acte.ActeHome;
import fr.paris.lutece.plugins.ods.business.acte.DocumentActeHome;
import fr.paris.lutece.plugins.ods.business.acte.StatutActeHome;
import fr.paris.lutece.plugins.ods.business.categoriedeliberation.CategorieDeliberationHome;
import fr.paris.lutece.plugins.ods.business.direction.IDirectionHome;
import fr.paris.lutece.plugins.ods.business.fichier.FichierPhysiqueHome;
import fr.paris.lutece.plugins.ods.business.fichier.IFichierHome;
import fr.paris.lutece.plugins.ods.business.formationconseil.FormationConseilHome;
import fr.paris.lutece.plugins.ods.business.groupepolitique.GroupePolitiqueHome;
import fr.paris.lutece.plugins.ods.business.historique.HistoriqueHome;
import fr.paris.lutece.plugins.ods.business.ordredujour.IEntreeOrdreDuJourHome;
import fr.paris.lutece.plugins.ods.business.ordredujour.IOrdreDuJourHome;
import fr.paris.lutece.plugins.ods.business.panier.PanierHome;
import fr.paris.lutece.plugins.ods.business.pdd.IPDDHome;
import fr.paris.lutece.plugins.ods.business.seance.ISeanceHome;
import fr.paris.lutece.plugins.ods.business.statut.StatutHome;
import fr.paris.lutece.plugins.ods.business.voeuamendement.IVoeuAmendementHome;
import fr.paris.lutece.plugins.ods.dto.acte.Acte;
import fr.paris.lutece.plugins.ods.dto.acte.IActeFilter;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter;
import fr.paris.lutece.plugins.ods.dto.fichier.ITypeDocument;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter;
import fr.paris.lutece.plugins.ods.dto.pdd.IArrondissement;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDDFilter;
import fr.paris.lutece.plugins.ods.dto.pdd.IPddFormBean;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendementFilter;
import fr.paris.lutece.plugins.ods.service.acte.IActeServiceManaged;
import fr.paris.lutece.plugins.ods.service.categoriedeliberation.ICategorieDeliberationService;
import fr.paris.lutece.plugins.ods.service.direction.IDirectionService;
import fr.paris.lutece.plugins.ods.service.formationconseil.IFormationConseilService;
import fr.paris.lutece.plugins.ods.service.groupepolitique.IGroupePolitiqueService;
import fr.paris.lutece.plugins.ods.service.horodatage.IHorodatageService;
import fr.paris.lutece.plugins.ods.service.ordredujour.IOrdreDuJourService;
import fr.paris.lutece.plugins.ods.service.pdd.declinaison.IDeclinaisonPDDService;
import fr.paris.lutece.plugins.ods.service.pdd.modegestion.IModeGestionPDD;
import fr.paris.lutece.plugins.ods.service.search.ISearchService;
import fr.paris.lutece.plugins.ods.service.voeuamendement.IVoeuAmendementService;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.portal.business.user.AdminUser;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppPathService;

import org.springframework.beans.factory.annotation.Autowired;

import java.sql.Timestamp;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;


public abstract class AbstractProjetDeDeliberationService<GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>, GEntreeOrdreDuJour extends IEntreeOrdreDuJour<GElu, GSeance, GFichier, GEntreeOrdreDuJour, GOrdreDuJour>, GOrdreDuJour extends IOrdreDuJour<GEntreeOrdreDuJour, GSeance, GFichier, GElu, GOrdreDuJour>, GSeance extends ISeance, GElu extends IElu, GPdd extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>, GOrdreDuJourFilter extends IOrdreDuJourFilter, GOrdreDuJourHome extends IOrdreDuJourHome<GOrdreDuJour, GEntreeOrdreDuJour, GOrdreDuJourFilter, GSeance, GElu, GFichier>, GOrdreDuJourService extends IOrdreDuJourService<GOrdreDuJour, GEntreeOrdreDuJour, GOrdreDuJourFilter, GSeance, GElu, GFichier>, GVoeuAmendementService extends IVoeuAmendementService<GVoeuAmendement, GOrdreDuJour, GEntreeOrdreDuJour, GSeance, GElu, GFichier, GPdd>, GVoeuAmendementFilter extends IVoeuAmendementFilter, GVoeuAmendementHome extends IVoeuAmendementHome<GVoeuAmendement, GVoeuAmendementFilter, GSeance, GElu, GFichier, GPdd>, GEntreeOrdreDuJourHome extends IEntreeOrdreDuJourHome<GOrdreDuJour, GEntreeOrdreDuJour, GElu, GSeance, GFichier>, GSeanceHome extends ISeanceHome<GSeance, GSeanceFilter, GFichier, GElu, GVoeuAmendement, GPdd>, GFichier extends IFichier<GSeance, GFichier>, GSeanceFilter extends ISeanceFilter, GPDDFormBean extends IPddFormBean<GSeance, GFichier, GElu, GVoeuAmendement, GPdd, GArrondissement>, GPDDFilter extends IPDDFilter, GArrondissement extends IArrondissement, GActeFilter extends IActeFilter<GFichier, GSeance>>
    implements IProjetDeDeliberationService<GOrdreDuJour, GEntreeOrdreDuJour, GVoeuAmendement, GSeance, GElu, GPdd, GFichier, GPDDFormBean, GPDDFilter, GArrondissement>,
        IActeServiceManaged<GSeance, GFichier>
{
    public static final String MARK_ID_DIRECTION_SELECTED = "id_direction_selected";
    private static final String JSP_URL_MODIFICATION_ACTE_PDD = "jsp/admin/plugins/ods/acte/ModificationActePDD.jsp";
    protected static final String JSP_DO_APPLIQUER_MODE_MANUEL = "jsp/admin/plugins/ods/projetdeliberation/DoAppliquerModeManuel.jsp";
    @Autowired
    protected GOrdreDuJourHome _ordreDuJourHome;
    @Autowired
    protected IPDDHome<GPdd, GPDDFilter, GVoeuAmendementFilter, GSeance, GElu, GVoeuAmendement, GFichier, GArrondissement> _projetDeliberationHome;
    @Autowired
    protected ISearchService<?, ?, ?, ?, ?, ?, ?, ?> _searchService;
    @Autowired
    protected IHorodatageService _horodatageService;
    @Autowired
    protected ActeHome<GSeance, GFichier, GActeFilter> _acteHome;
    @Autowired
    protected StatutActeHome _statutActeHome;
    @Autowired
    protected DocumentActeHome<GFichier, IFichierFilter, ITypeDocument, GSeance> _documentActeHome;
    @Autowired
    protected CategorieDeliberationHome _categorieDeliberationHome;
    @Autowired
    protected ICategorieDeliberationService _categorieDeliberationService;
    @Autowired
    protected IDirectionHome _directionHome;
    @Autowired
    protected IDirectionService _directionService;
    @Autowired
    protected IFichierHome<GFichier, IFichierFilter, ITypeDocument, GSeance> _fichierHome;
    @Autowired
    protected FichierPhysiqueHome _fichierPhysiqueHome;
    @Autowired
    protected FormationConseilHome _formationConseilHome;
    @Autowired
    protected IFormationConseilService _formationConseilService;
    @Autowired
    protected GroupePolitiqueHome _groupePolitiqueHome;
    @Autowired
    protected IGroupePolitiqueService _groupePolitiqueService;
    @Autowired
    protected GOrdreDuJourService _ordreDuJourService;
    @Autowired
    protected GVoeuAmendementService _voeuAmendementService;
    @Autowired
    protected HistoriqueHome _historiqueHome;
    @Autowired
    protected GVoeuAmendementHome _voeuAmendementHome;
    @Autowired
    protected GEntreeOrdreDuJourHome _entreeOrdreDuJourHome;
    @Autowired
    protected StatutHome _statutHome;
    @Autowired
    protected GSeanceHome _seanceHome;
    @Autowired
    protected PanierHome _panierHome;
    protected Map<Integer, IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd>> _declinaisonPddServiceMapping;
    protected Map<Integer, IModeGestionPDD<GPDDFilter, GPDDFormBean, GArrondissement, GVoeuAmendement, GSeance, GElu, GPdd, GFichier>> _modeGestionPddServiceMapping;
    protected IModeGestionPDD<GPDDFilter, GPDDFormBean, GArrondissement, GVoeuAmendement, GSeance, GElu, GPdd, GFichier> _modeGestion;
    protected IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> _declinaisonPDD;
    protected GPDDFormBean _pddFormBean;
    protected boolean _bComeFromActe;

    protected abstract String getParameterPluginName(  );

    protected abstract Plugin getPlugin(  );

    public void comeFromActe(  )
    {
        _bComeFromActe = true;
    }

    public abstract void classeVA( GVoeuAmendement voeuamendement, GPdd pdd );

    public void doAjoutArrondissement( HttpServletRequest request )
    {
        // TODO Auto-generated method stub
    }

    public void doAjoutDirectionCoEmetrice( HttpServletRequest request )
    {
        // TODO Auto-generated method stub
    }

    public String doAjoutVoeuAmendement( HttpServletRequest request )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public String doAppliquerModeManuel( HttpServletRequest request )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public String doChangeNumOrdreVoeuxAmendements( HttpServletRequest request, boolean isForAmandement )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public String doCreationPDD( HttpServletRequest request )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public String doCreationPieceAnnexe( HttpServletRequest request )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public void doDeleteArrondissement( HttpServletRequest request )
    {
        // TODO Auto-generated method stub
    }

    public void doDeleteDirectionCoEmetrice( HttpServletRequest request )
    {
        // TODO Auto-generated method stub
    }

    public GFichier doDowloadFichier( HttpServletRequest request )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public String doGlisserDeposer( HttpServletRequest request )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public String doModificationPDD( HttpServletRequest request, boolean bIsAuthorized )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public String doModificationPieceAnnexe( HttpServletRequest request )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public HashMap<String, Object> doPublicationPDD( HttpServletRequest request )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public String doSuppressionPDD( HttpServletRequest request )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public String doSuppressionPieceAnnexe( HttpServletRequest request )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public String getConfirmActiveModeManuel( HttpServletRequest request )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public String getCreationPageTitleProperty(  )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public HashMap<String, Object> getHistoriquePDD( HttpServletRequest request )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public HashMap<String, Object> getHistoriquePieceAnnexe( HttpServletRequest request )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public Map<Integer, IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd>> getDeclinaisonMappingService(  )
    {
        return _declinaisonPddServiceMapping;
    }

    public Map<Integer, IModeGestionPDD<GPDDFilter, GPDDFormBean, GArrondissement, GVoeuAmendement, GSeance, GElu, GPdd, GFichier>> getModeGestionMappingService(  )
    {
        return _modeGestionPddServiceMapping;
    }

    public HashMap<String, Object> getModificationCompleteProjetDeDeliberation( HttpServletRequest request )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public String getModificationPageTitleProperty(  )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public HashMap<String, Object> getPDD( HttpServletRequest request )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public HashMap<String, Object> getPDDList( HttpServletRequest request )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public HashMap<String, Object> getPDDListMultiSelect( HttpServletRequest request, String strUrlJSP,
        String strUrlRetour, boolean estGestionAval )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public String getPDDListPageTitleProperty(  )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public HashMap<String, Object> getPDDListSelect( HttpServletRequest request, String strUrlJSP, String strUrlRetour )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public String getSuppressionPDD( HttpServletRequest request )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public String getSuppressionPieceAnnexe( HttpServletRequest request )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public String getTemplateList(  )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public String getTemplateListMultiSelect(  )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public String getTemplateListSelect(  )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public String init( HttpServletRequest request )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public void initFilter( HttpServletRequest request, boolean isGestionAval )
    {
        // TODO Auto-generated method stub
    }

    public void initUrlRetour( String strUrl, HttpServletRequest request )
    {
        // TODO Auto-generated method stub
    }

    public boolean isAuthorized( String strPermission, AdminUser user )
    {
        // TODO Auto-generated method stub
        return false;
    }

    public boolean isFonctionnaliteProjetDeDeliberation(  )
    {
        // TODO Auto-generated method stub
        return false;
    }

    public boolean isGestionAval(  )
    {
        // TODO Auto-generated method stub
        return false;
    }

    public boolean isPublier( int nIdPDD )
    {
        // TODO Auto-generated method stub
        return false;
    }

    public HashMap<String, Object> returnEcran( HttpServletRequest request, String strPDDEcran, String strAnnee,
        String strNumero, String strObjet, String strPieceManuel, String strDelegationServicePublic )
    {
        // TODO Auto-generated method stub
        return null;
    }

    public void setFonctionnalite( boolean isFonctionnaliteProjetDeDeliberation )
    {
        // TODO Auto-generated method stub
    }

    public void setGestionAval( boolean bGestionAval )
    {
        // TODO Auto-generated method stub
    }

    public void setDeclinaisonMappingService( 
        Map<Integer, IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd>> mappingService )
    {
        _declinaisonPddServiceMapping = mappingService;
    }

    public void setModeGestionMappingService( 
        Map<Integer, IModeGestionPDD<GPDDFilter, GPDDFormBean, GArrondissement, GVoeuAmendement, GSeance, GElu, GPdd, GFichier>> mappingService )
    {
        _modeGestionPddServiceMapping = mappingService;
    }

    public void updateNumeroOrdreVA( GPdd pdd, int nCompteur, GVoeuAmendement voeuamendement, boolean bToADD )
    {
        // TODO Auto-generated method stub
    }

    public void setDeclinaison( int nIdTypeDeclinaison )
    {
        _declinaisonPDD = getDeclinaisonMappingService(  ).get( nIdTypeDeclinaison );
    }

    public void setModeGestion( int nIdModeGestion )
    {
        _modeGestion = getModeGestionMappingService(  ).get( nIdModeGestion );
    }

    public String getVisualisationPageTitleProperty(  )
    {
        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    public String getJspUrlGetModificationForActe( HttpServletRequest request, int nIdEntite, int nIdActe )
    {
        String strReturn = AppPathService.getBaseUrl( request ) + JSP_URL_MODIFICATION_ACTE_PDD +
            OdsConstants.CONSTANTE_POINT_INTERROGATION + getParameterPluginName(  ) + OdsConstants.CONSTANTE_ET +
            OdsParameters.ID_ACTE + OdsConstants.CONSTANTE_EGAL + nIdActe;

        int nIdPDD = OdsUtils.getIntegerParameter( OdsParameters.ID_PDD, request );

        if ( nIdPDD != -1 )
        {
            strReturn += ( OdsConstants.CONSTANTE_ET + getIdParameter(  ) + OdsConstants.CONSTANTE_EGAL + nIdPDD );
        }

        return strReturn;
    }

    public String getIdParameter(  )
    {
        return OdsParameters.ID_PDD;
    }

    public void doPublicationActe( Acte<GSeance, GFichier> acte, boolean bIsPublie, Timestamp dateForVersion )
    {
        //TODO
    }

    /**
     * Renvoie l'objet de l'acte utilis lors de l'envoi vers la plate-forme
     *                         Anne Direction Numro Objet
     * @param acte l'acte que l'on va transmettre
     * @return strObjet l'objet de l'acte
     */
    public String getObjetActe( Acte<GSeance, GFichier> acte )
    {
        Plugin plugin = getPlugin(  );
        String strObjet = OdsConstants.CONSTANTE_CHAINE_VIDE;

        GPdd pdd = _projetDeliberationHome.findByPrimaryKey( acte.getIdEntite(  ), plugin );

        if ( pdd != null )
        {
            strObjet = pdd.getReference(  ) + OdsConstants.CONSTANTE_ESPACE + pdd.getObjet(  );
        }

        return strObjet;
    }
}
