/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.pdd;

import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.pdd.IArrondissement;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDDFilter;
import fr.paris.lutece.plugins.ods.dto.pdd.IPddFormBean;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.service.IODSService;
import fr.paris.lutece.plugins.ods.service.pdd.declinaison.IDeclinaisonPDDService;
import fr.paris.lutece.plugins.ods.service.pdd.modegestion.IModeGestionPDD;
import fr.paris.lutece.portal.business.user.AdminUser;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;


public interface IProjetDeDeliberationService<GOrdreDuJour extends IOrdreDuJour<GEntreeOrdreDuJour, GSeance, GFichier, GElu, GOrdreDuJour>, GEntreeOrdreDuJour extends IEntreeOrdreDuJour<GElu, GSeance, GFichier, GEntreeOrdreDuJour, GOrdreDuJour>, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>, GSeance extends ISeance, GElu extends IElu, GPdd extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>, GFichier extends IFichier<GSeance, GFichier>, GPDDFormBean extends IPddFormBean<GSeance, GFichier, GElu, GVoeuAmendement, GPdd, GArrondissement>, GPDDFilter extends IPDDFilter, GArrondissement extends IArrondissement>
    extends IODSService
{
    /**
     * Retourne true si le VA est publi, false sinon
     * @param nIdVA l'id du VA
     * @return true si le VA est publi, false sinon
     */
    boolean isPublier( int nIdPDD );

    /**
     * Initialisation
     * @param request la requte HTTP
     * @return strReturn le type de PDD : projet ou proposition
     */
    String init( HttpServletRequest request );

    /**
     * Permet d'initialiser l'url de retour
     * @param strUrl url de retour
     * @param request la requte http
     */
    void initUrlRetour( String strUrl, HttpServletRequest request );

    /**
     * Initialise tous les filtres
     * @param request la requete Http
     * @param isGestionAval si on est dans la gestion aval ou pas
     */
    void initFilter( HttpServletRequest request, boolean isGestionAval );

    /**
     * @return TRUE si c'est en gestion aval, FALSE sinon
     */
    boolean isGestionAval(  );

    /**
     * @param isFonctionnaliteProjetDeDeliberation TRUE si c'est un projet, FALSE sinon
     */
    void setFonctionnalite( boolean isFonctionnaliteProjetDeDeliberation );

    /**
     * @return _bFonctionnaliteProjetDeDeliberation boolean
     */
    boolean isFonctionnaliteProjetDeDeliberation(  );

    /**
     *
     * @param bGestionAval TRUE si on est dans le contexte de la gestion aval, FALSE sinon
     */
    void setGestionAval( boolean bGestionAval );

    /**
     * Retourne la map contenant les lments utiles au template HTML de la liste
     *
     * @param request la requte HTTP
     * @return model la map contenant les lments utiles au template HTML de la liste
     */
    HashMap<String, Object> getPDDList( HttpServletRequest request );

    /**
    *
    * @param la map contenant les lments utiles au template HTML de la liste
    * @param strUrlJSP l'url de la JSP appelant la methode
    * @param strUrlRetour l'url de la JSP de retour
    * @return model la map contenant les lments utiles au template HTML de la liste
    */
    HashMap<String, Object> getPDDListSelect( HttpServletRequest request, String strUrlJSP, String strUrlRetour );

    /**
     * Retourne la map contenant les lments utiles au template HTML de la liste
     * @param request la requte Http
     * @param strUrlJSP l'url appelant cette mthode
     * @param strUrlRetour l'url de retour du formulaire affich
     * @param estGestionAval TRUE si l'on est en gestion aval, FALSE sinon
     * @return model la map contenant les lments utiles au template HTML de la liste
     */
    HashMap<String, Object> getPDDListMultiSelect( HttpServletRequest request, String strUrlJSP, String strUrlRetour,
        boolean estGestionAval );

    /**
     * retourne la map contenant les lments utiles au template HTML
     *
     * @param request la requte HTTP
     * @return model la map contenant les lments utiles au template HTML
     */
    HashMap<String, Object> getPDD( HttpServletRequest request );

    /**
     * retourne la map contenant les lments utiles au template HTML de modification
     *
     * @param request la requte HTTP
     * @return model la map contenant les lments utiles au template HTML de modification
     */
    HashMap<String, Object> getModificationCompleteProjetDeDeliberation( HttpServletRequest request );

    /**
     * retourne la map contenant les lments utiles au template HTML de l'historique
     *
     * @param request
     *            la requte HTTP
     * @return model la map contenant les lments utiles au template HTML de l'historique
     */
    HashMap<String, Object> getHistoriquePDD( HttpServletRequest request );

    /**
     * retourne la map contenant les lments utiles au template HTML de l'historique
     *
     * @param request
     *            la requte HTTP
     * @return model la map contenant les lments utiles au template HTML de l'historique
     */
    HashMap<String, Object> getHistoriquePieceAnnexe( HttpServletRequest request );

    /**
     * retourne une demande de confirmation pour la suppression du pdd choisie
     *
     * @param request la requte HTTP
     * @return String template
     */
    String getSuppressionPDD( HttpServletRequest request );

    /**
     * retourne une demande de confirmation pour la suppression de la piece
     * annexe choisie
     *
     * @param request
     *            la requte HTTP
     * @return String template
     */
    String getSuppressionPieceAnnexe( HttpServletRequest request );

    /**
     * methode qui demande une confirmation de l'utilisateur avant de basculer le classement des vas du pdd en mode maunuel
     *
     * @param request request
     * @return l'url de traitement
     */
    String getConfirmActiveModeManuel( HttpServletRequest request );

    /**
     * Creation d'un fichier si tous les champs obligatoires du formulaire de
     * cration d'un fichier ont ete renseigne,<BR>
     * la mthode retourne l'url de gestion des fichiers,<BR>
     * sinon l'url retourne est celle de l'interface spcifiant les messages
     * d'erreurs.
     *
     * @param request la requte HTTP
     * @return String template
     */
    String doCreationPieceAnnexe( HttpServletRequest request );

    /**
     * Modifie la piece annexe et le fichier physique. Si le fichie est en ligne,
     * alors la version est incrment de 1 et ajout d'un historique si tous les
     * champs obligatoires du formulaire de modification d'un fichier ont t
     * renseign,<BR>
     * la mthode retourne l'url de gestion des fichiers,<BR>
     * sinon l'url retourne est celle de l'interface spcifiant les messages
     * d'erreurs.
     *
     * @param request
     *            la requte HTTP
     * @return String template
     */
    String doModificationPieceAnnexe( HttpServletRequest request );

    /**
     * Retourne la map contenant les lments utiles au template HTML de publication
     *
     * @param request la requte HTTP
     * @return model la map contenant les lments utiles au template HTML de publication
     */
    HashMap<String, Object> doPublicationPDD( HttpServletRequest request );

    /**
     * supprime le pdd choisi
     *
     * @param request
     *            la requte HTTP
     * @return String template
     */
    String doSuppressionPDD( HttpServletRequest request );

    /**
     * supprime la piece annexe choisie
     *
     * @param request
     *            la requte HTTP
     * @return String template
     */
    String doSuppressionPieceAnnexe( HttpServletRequest request );

    /**
     * Permet de changer l'ordre des voeux Amandements rattachs
     *
     * @param isForAmandement
     *            TRUE pour traiter les amendements, FALSE pour les voeux
     *            rattachs
     * @param request
     *            la requte HTTP
     * @return String template
     */
    String doChangeNumOrdreVoeuxAmendements( HttpServletRequest request, boolean isForAmandement );

    /**
     * Ajout d'un Voeu/Amendement au PDD
     *
     * @param request la requte HTTP
     * @return String template
     */
    String doAjoutVoeuAmendement( HttpServletRequest request );

    /**
     * methode qui bascule le pdd en mode de classement manuel.
     *
     * @param request
     *            request
     * @return l'url de modification du pdd en cours de traitement
     */
    String doAppliquerModeManuel( HttpServletRequest request );

    /**
     * methode qui gre le glisser dposer des vas.
     *
     * @param request
     *            request
     * @return l'url de modification du pdd en cours de traitement
     */
    String doGlisserDeposer( HttpServletRequest request );

    /**
     * Permet d'ajouter une direction coemetrice au pdd qui est en cours de
     * creation ou de modification
     *
     * @param request
     *            la requte HTTP
     * @return String template
     */
    void doAjoutDirectionCoEmetrice( HttpServletRequest request );

    /**
     * Permet de supprimer une direction coemetrice au pdd qui est en cours de
     * creation ou de modification
     *
     * @param request
     *            la requte HTTP
     * @return String template
     */
    void doDeleteDirectionCoEmetrice( HttpServletRequest request );

    /**
     * Permet d'ajouter un arrondissement au pdd qui est en cours de creation ou
     * de modification
     *
     * @param request
     *            la requte HTTP
     * @return String template
     */
    void doAjoutArrondissement( HttpServletRequest request );

    /**
     * Permet de supprimer un arrondissement au pdd qui est en cours de creation
     * ou de modification
     *
     * @param request
     *            la requte HTTP
     * @return String template
     */
    void doDeleteArrondissement( HttpServletRequest request );

    /**
     * Mthode qui effectue la cration.
     * Si retourList = TRUE alors on enregistre les donnes et on revient sur la liste des pdd.
     * Si retourList = FALSE alors on enregistre les donnes mais redirige sur l'ecran de modification.
     *
     * @param request la requte HTTP
     * @return String la template  afficher
     */
    String doCreationPDD( HttpServletRequest request );

    /**
     * Methode qui effectue les modifications Si retourList = TRUE alors on
     * enregistre les donnes et on revient sur la liste des pdd Si retourList =
     * FALSE alors on enregistre les donnes mais on reste sur l'ecran de
     * modification
     *
     * @param request la requte HTTP
     * @param bIsAuthorized TRUE si l'utilisateur est autoris, FALSE sinon
     * @return String template
     */
    String doModificationPDD( HttpServletRequest request, boolean bIsAuthorized );

    /**
     * Renvoie le fichier.
     *
     * @param request la requte HTTP
     */
    GFichier doDowloadFichier( HttpServletRequest request );

    /**
     * Retourne l'ecran sur lequel on effectue les modifications Si strPDDEcran =
     * MARK_PDD_ECRAN_MODIFICATION alors on revient sur l'ecran de modification
     * Si strPDDEcran = MARK_PDD_ECRAN_CREATION alors on revient sur l'ecran de
     * creation
     *
     * La methode passe alors tous les paramtres ncessaire aux ecrans sur
     * lequel on est redirig
     *
     * @param strPDDEcran identifiant du pdd donn par el formulaire
     * @param strAnnee anne du pdd donne par le formulaire
     * @param strNumero numero du pdd donn par le formulaire
     * @param strObjet objet du pdd donn par le formulaire
     * @param strPieceManuel piece manuelle donn par le formulaire
     * @param strDelegationServicePublic delegation de service public donn par le forulaire
     * @param request la requte HTTP
     * @return String template
     */
    HashMap<String, Object> returnEcran( HttpServletRequest request, String strPDDEcran, String strAnnee,
        String strNumero, String strObjet, String strPieceManuel, String strDelegationServicePublic );

    /**
     * methode qui classe automatiquement le nouveau amendement fourni
     *
     * @param voeuamendement le voeuamendement  insrer
     * @param pdd le pdd dans lequel on veut insrer l'amendement
     */
    void classeVA( GVoeuAmendement voeuamendement, GPdd pdd );

    /*
     * Mthode qui permet de modifier les numros d'ordre des vas du pdd
     * @param pdd le pdd
     * @param voeuamendement le va  insrer
     * @param nCompteur le numro d'insertion du va dans le pdd
     * @param bToADD true si l'on ajouter un va, false si on le supprimer
     */

    //void updateNumeroOrdreVA( GPdd pdd, int nCompteur, GVoeuAmendement voeuamendement, boolean bToADD );
    boolean isAuthorized( String strPermission, AdminUser user );

    String getCreationPageTitleProperty(  );

    String getModificationPageTitleProperty(  );

    String getPDDListPageTitleProperty(  );

    Map<Integer, IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd>> getDeclinaisonMappingService(  );

    void setDeclinaisonMappingService( 
        Map<Integer, IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd>> mappingService );

    Map<Integer, IModeGestionPDD<GPDDFilter, GPDDFormBean, GArrondissement, GVoeuAmendement, GSeance, GElu, GPdd, GFichier>> getModeGestionMappingService(  );

    void setModeGestionMappingService( 
        Map<Integer, IModeGestionPDD<GPDDFilter, GPDDFormBean, GArrondissement, GVoeuAmendement, GSeance, GElu, GPdd, GFichier>> mappingService );

    String getTemplateList(  );

    String getTemplateListSelect(  );

    String getTemplateListMultiSelect(  );

    void setDeclinaison( int nIdTypeDeclinaison );

    void setModeGestion( int nIdModeGestion );

    String getPageTitleListSelect(  );

    String getRight(  );

    HashMap<String, Object> getVisualisationProjetDeDeliberation( HttpServletRequest request, boolean isForSuivi );

    String getVisualisationPageTitleProperty(  );

    String getAccuseReceptionPageTitleProperty(  );

    String getType(  );
}
