/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.pdd.declinaison;

import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.pdd.IArrondissement;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDDFilter;
import fr.paris.lutece.plugins.ods.dto.pdd.IPddFormBean;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.service.pdd.ProjetDeDeliberationServiceDTO;
import fr.paris.lutece.portal.business.user.AdminUser;
import fr.paris.lutece.portal.service.plugin.Plugin;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;


/**
 * Interface representant les differents types de PDD (Projet, Proposition)
 *
 */
public interface IDeclinaisonPDDService<GPDDFilter extends IPDDFilter, GPDDFormBean extends IPddFormBean<GSeance, GFichier, GElu, GVoeuAmendement, GPdd, GArrondissement>, GArrondissement extends IArrondissement, GSeance extends ISeance, GElu extends IElu, GFichier extends IFichier<GSeance, GFichier>, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>, GPdd extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>>
{
    void getPDDList( HttpServletRequest request, Map<String, Object> model,
        ProjetDeDeliberationServiceDTO<GArrondissement> pddDTO );

    void getPDDListAmont( GPDDFilter filter, int nIdPublicationSelected );

    int getIdTypeEntite(  );

    GPDDFormBean getFormBean(  );

    /**
     *
     * Debut AJout MA
     */

    //remplasse dans MA getPDDList
    void getPDDListGestionAval( HttpServletRequest request, Map<String, Object> model,
        ProjetDeDeliberationServiceDTO<GArrondissement> pddDTO, GPDDFilter filter );

    //	remplasse dans MA getPDDListAmont
    void getPDDListGestionAmont( HttpServletRequest request, Map<String, Object> model,
        ProjetDeDeliberationServiceDTO<GArrondissement> pddDTO, GPDDFilter filter );

    GPDDFormBean getFormBeanGestionAmont(  );

    GPDDFormBean getFormBeanGestionAval(  );

    boolean isAuthorizedGestionAval( String strPermission, AdminUser user );

    boolean isAuthorizedGestionAmont( String strPermission, AdminUser user );

    String getCreationPageTitlePropertyGestionAval(  );

    String getCreationPageTitlePropertyGestionAmont(  );

    String getModificationPageTitlePropertyGestionAval(  );

    String getModificationPageTitlePropertyGestionAmont(  );

    String getPDDListPageTitlePropertyGestionAval(  );

    String getPDDListPageTitlePropertyGestionAmont(  );

    void getCreationGestionAval( HttpServletRequest request, Map<String, Object> model,
        ProjetDeDeliberationServiceDTO<GArrondissement> pddDTO, GPDDFormBean pddFormBean );

    void getCreationGestionAmont( HttpServletRequest request, Map<String, Object> model,
        ProjetDeDeliberationServiceDTO<GArrondissement> pddDTO, GPDDFormBean pddFormBean );

    void getModificationGestionAval( HttpServletRequest request, Map<String, Object> model,
        ProjetDeDeliberationServiceDTO<GArrondissement> pddDTO, GPDDFormBean pddFormBean );

    void getModificationGestionAmont( HttpServletRequest request, Map<String, Object> model,
        ProjetDeDeliberationServiceDTO<GArrondissement> pddDTO, GPDDFormBean pddFormBean );

    String getMessageGestionAval( String strMessageKey );

    String getMessageGestionAmont( String strMessageKey );

    boolean majPDDGestionAval( GPdd pdd, ProjetDeDeliberationServiceDTO<GArrondissement> pddDTO,
        GPDDFormBean pddFormBean, Plugin plugin );

    boolean majPDDGestionAmont( GPdd pdd, ProjetDeDeliberationServiceDTO<GArrondissement> pddDTO,
        GPDDFormBean pddFormBean, Plugin plugin );

    String getTemplateListGestionAval(  );

    String getTemplateListGestionAmont(  );

    String getTemplateListSelectGestionAval(  );

    String getTemplateListSelectGestionAmont(  );

    String getTemplateListMultiSelectGestionAval(  );

    String getTemplateListMultiSelectGestionAmont(  );

    String getType(  );

    String getJSPListGestionAval(  );

    String getJSPListGestionAmont(  );

    String getJSPModificationGestionAval(  );

    String getJSPModificationGestionAmont(  );

    String getJSPDoSuppressionGestionAval(  );

    String getJSPDoSuppressionGestionAmont(  );

    String getJSPDoSuppressionPieceAnnexeGestionAval(  );

    String getJSPDoSuppressionPieceAnnexeGestionAmont(  );

    String getSelectionForODJPageTitlePropertyGestionAmont(  );

    String getSelectionForODJPageTitlePropertyGestionAval(  );

    boolean isMultipleArrondissements(  );

    String getRightGestionAmont(  );

    String getRightGestionAval(  );

    IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> getVisualisationService(  );

    IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> getVisualisationSuiviService(  );

    String getVisualisationPageTitleProperty(  );

    GPDDFormBean getFormBeanVisualisation(  );

    GPDDFormBean getFormBeanVisualisationSuivi(  );

    GPDDFormBean getFormBeanVisualisationSuiviAval(  );

    IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> getAccuseReceptionService(  );

    String getAccuseReceptionPageTitleProperty(  );

    boolean isSeancePrevisModify( GPdd pddUpdated, GPdd pddForm );

    int getTypePieceAnnexe(  );

    /**
     *
     * FIN AJout MA
     */
}
