/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.pdd.modegestion;

import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.pdd.IArrondissement;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDDFilter;
import fr.paris.lutece.plugins.ods.dto.pdd.IPddFormBean;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.service.pdd.ProjetDeDeliberationServiceDTO;
import fr.paris.lutece.plugins.ods.service.pdd.declinaison.IDeclinaisonPDDService;
import fr.paris.lutece.portal.business.user.AdminUser;
import fr.paris.lutece.portal.service.plugin.Plugin;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;


/**
 * Interface representant les modes de gestion amont/aval
 *
 */
public interface IModeGestionPDD<GPDDFilter extends IPDDFilter, GPDDFormBean extends IPddFormBean<GSeance, GFichier, GElu, GVoeuAmendement, GPdd, GArrondissement>, GArrondissement extends IArrondissement, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>, GSeance extends ISeance, GElu extends IElu, GPdd extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>, GFichier extends IFichier<GSeance, GFichier>>
{
    /**
     *
     * DEBUT AJOUT MA
     */
    String MESSAGE_KEY_CONFIRM_DELETE = "CONFIRM_DELETE";

    void getPDDList( HttpServletRequest request, Map<String, Object> model,
        ProjetDeDeliberationServiceDTO<GArrondissement> pddDTO, GPDDFilter filter,
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD );

    void getModificationCompleteProjetDeDeliberation( HttpServletRequest request, Map<String, Object> model,
        ProjetDeDeliberationServiceDTO<GArrondissement> pddDTO,
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD,
        GPDDFormBean pddFormBean );

    void getModificationCompleteProjetDeDeliberationStatuts( HttpServletRequest request, Map<String, Object> model,
        ProjetDeDeliberationServiceDTO<GArrondissement> pddDTO,
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD,
        GPDDFormBean pddFormBean, GPdd pdd );

    void returnEcran( HttpServletRequest request, Map<String, Object> model,
        ProjetDeDeliberationServiceDTO<GArrondissement> pddDTO,
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD,
        GPdd pdd );

    String doModificationPDD( HttpServletRequest request, ProjetDeDeliberationServiceDTO<GArrondissement> pddDTO,
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD,
        GPdd pddUpdated, GPDDFormBean pddFormBean, boolean retourList, boolean bIsAuthorized );

    String doCreationPDD( HttpServletRequest request, ProjetDeDeliberationServiceDTO<GArrondissement> pddDTO,
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD,
        GPDDFormBean pddFormBean, boolean retourList );

    void doAjoutDirectionCoEmetrice( String strIdDirectionCoEmetrice, String strPddRefCoEmetrice,
        ProjetDeDeliberationServiceDTO<GArrondissement> pddDTO );

    boolean isAuthorized( String strPermission, AdminUser user,
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD );

    String getCreationPageTitleProperty( 
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD );

    String getModificationPageTitleProperty( 
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD );

    String getPDDListPageTitleProperty( 
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD );

    GPDDFormBean getFormBean( 
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD );

    void getCreation( HttpServletRequest request, Map<String, Object> model,
        ProjetDeDeliberationServiceDTO<GArrondissement> pddDTO,
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD,
        GPDDFormBean pddFormBean );

    void getModification( HttpServletRequest request, Map<String, Object> model,
        ProjetDeDeliberationServiceDTO<GArrondissement> pddDTO,
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD,
        GPDDFormBean pddFormBean );

    String getMessage( String strMessageKey,
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD );

    String getTemplateList( 
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD );

    String getTemplateListSelect( 
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD );

    String getTemplateListMultiSelect( 
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD );

    String getJSPList( 
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD );

    String getJSPModification( 
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD );

    String getJSPDoSuppression( 
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD );

    String getJSPDoSuppressionPieceAnnexe( 
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD );

    String getPageTitleListSelect( 
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD );

    String getRight( 
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD );

    IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> getVisualisationService( 
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD );

    String getVisualisationPageTitleProperty( 
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD );

    IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> getAccuseReceptionService( 
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD );

    String getAccuseReceptionPageTitleProperty( 
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD );

    GPDDFormBean getVisualisationSuiviFormBean( 
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD );

    IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> getVisualisationSuiviService( 
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD );

    int doAjoutVoeuAmendement( HttpServletRequest request,
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD );

    String doValidationPDDSGCP( HttpServletRequest request,
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD );

    String getType( 
        IDeclinaisonPDDService<GPDDFilter, GPDDFormBean, GArrondissement, GSeance, GElu, GFichier, GVoeuAmendement, GPdd> declinaisonPDD );

    /**
     * Supprime un VA de la liste des VA du PDD
     * @param pdd le pdd
     * @param isForAmandement true si pour amendements, false sinon
     * @param nIndex l'index  supprimer
     * @param strIdVA l'identifiant du VA  supprimer
     * @param plugin le plugin
     */
    void doRemoveVA( GPdd pdd, boolean isForAmandement, int nIndex, String strIdVA, Plugin plugin );

    /**
     * Retourne la liste des Voeux, Amendements, ou VoeuxAmendements
     * @param pdd le pdd
     * @param isForAmandement true si pour amendement, false sinon
     * @return la liste des Voeux, Amendements, ou VoeuxAmendements
     */
    List<GVoeuAmendement> getVADeplacement( GPdd pdd, boolean isForAmandement );

    /**
     *
     * FIN AJOUT MA
     */
}
