/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.seance;

import fr.paris.lutece.plugins.ods.business.indexer.IndexerHome;
import fr.paris.lutece.plugins.ods.business.seance.ISeanceHome;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.indexer.Indexer;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.commons.DateUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.service.util.AppPathService;
import fr.paris.lutece.util.url.UrlItem;

import org.springframework.beans.factory.annotation.Autowired;

import java.sql.SQLException;
import java.sql.Timestamp;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.servlet.http.HttpServletRequest;


public abstract class AbstractSeanceService<GSeance extends ISeance, GSeanceFilter extends ISeanceFilter, GFichier extends IFichier<GSeance, GFichier>, GElu extends IElu, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>, GPdd extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>>
    implements ISeanceService
{
    private static final String URL_CREATION_SEANCE_JSP = "jsp/admin/plugins/ods/seance/CreationSeance.jsp";
    private static final String URL_DO_SUPPRESSION_SEANCE_JSP = "jsp/admin/plugins/ods/seance/DoSuppressionSeance.jsp";
    private static final String MESSAGE_CONFIRM_DELETE_SEANCE = "ods.message.confirmDeleteSuppression";
    private static final String MESSAGE_CANNOT_DELETE_SEANCE = "ods.message.cannotDeleteSeance";
    protected static final String PROPERTY_LABEL_DATE_DEBUT = "ods.listeseances.label.start";
    protected static final String PROPERTY_LABEL_DATE_FIN = "ods.listeseances.label.end";
    private static final String MARK_FILTER_ERROR = "filter_error";
    protected static final String MESSAGE_CANNOT_CREATE_SEANCE = "ods.message.cannotCreateSeance";
    protected static final String MARK_LISTE_SEANCES = "liste_seances";
    protected static final String MARK_SEANCE_CREATING_OR_UPDATING = "seanceBean";
    @Autowired
    protected ISeanceHome<GSeance, GSeanceFilter, GFichier, GElu, GVoeuAmendement, GPdd> _seanceHome;
    @Autowired
    private IndexerHome _indexerHome;

    protected abstract void getCreateSeanceSpec( HashMap<String, Object> model );

    protected abstract void getModificationSeanceSpec( GSeance seance, HashMap<String, Object> model );

    protected abstract int doCreateSeanceSpec( HttpServletRequest request );

    protected abstract String doCreateAndRegisterSeanceSpec( HttpServletRequest request, List<String> champsNonSaisie );

    protected abstract String doUpdateAndRegisterSeanceSpec( HttpServletRequest request, List<String> champsNonSaisie );

    protected abstract int isValidForm( List<String> champsNonSaisie, HttpServletRequest request, boolean bIsForSearch );

    protected abstract List<GSeance> applySeanceFilterSpec( Timestamp dSeanceDateBeginFilter,
        Timestamp dSeanceDateEndFilter );

    protected abstract Plugin getPlugin(  );

    /**
    * Retourne la map contenant les lments utiles au template HTML de la liste
    * Seuls les sances dont la date de debut est suprieure ou gale  la date du jour seront affichs
    *
    * @param request la requete HTTP
    * @return model la map contenant les lments utiles au template HTML de la liste
    */
    public HashMap<String, Object> getSeanceList( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        HashMap<String, Object> model = new HashMap<String, Object>(  );

        List<GSeance> seances = _seanceHome.findSeanceList( plugin );
        model.put( MARK_LISTE_SEANCES, seances );

        return model;
    }

    /**
    * Retourne la map contenant les lments utiles au template HTML de la liste
    * Seuls les sances dont la date de dbut est comprise entre les deux dates du filtre seront affichs
    * @param request request la requete HTTP
    * @return model la map contenant les lments utiles au template HTML de la liste
    */
    public HashMap<String, Object> applySeanceFilter( HttpServletRequest request )
    {
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        String strSeanceBegin = request.getParameter( OdsParameters.SEANCE_BEGIN );
        String strSeanceEnd = request.getParameter( OdsParameters.SEANCE_END );

        List<String> champsNonSaisie = new ArrayList<String>(  );
        int result = isValidFormSearch( champsNonSaisie, request );

        if ( ( result == 0 ) &&
                ( DateUtils.getDate( strSeanceBegin.trim(  ), true )
                               .compareTo( DateUtils.getDate( strSeanceEnd.trim(  ), true ) ) <= 0 ) &&
                ( DateUtils.getDate( strSeanceEnd.trim(  ), false ).compareTo( DateUtils.getCurrentDate(  ) ) >= 0 ) )
        {
            Timestamp dSeanceDateBeginFilter = DateUtils.getDate( strSeanceBegin, true );
            Timestamp dSeanceDateEndFilter = DateUtils.getDate( strSeanceEnd, false );

            List<GSeance> seances = applySeanceFilterSpec( dSeanceDateBeginFilter, dSeanceDateEndFilter );

            model.put( MARK_LISTE_SEANCES, seances );

            model.put( OdsParameters.SEANCE_BEGIN, strSeanceBegin );
            model.put( OdsParameters.SEANCE_END, strSeanceEnd );
        }
        else
        {
            model.put( MARK_FILTER_ERROR, OdsConstants.CONSTANTE_CHAINE_VIDE );
        }

        return model;
    }

    /**
     * methode qui test si dans le formulaire de saisi d'une sance,l'ensemble des critres obligatoires ont t renseign
     * @param champsNonSaisie List<String> gere les champs non  obligatoires non renseigns
     * @param request la requete HTTP
     * @return int 1 si la date de debut ou 2 pour la date de fin (champs non correct) et 0 si tout est OK
     */
    private int isValidFormSearch( List<String> champsNonSaisie, HttpServletRequest request )
    {
        String strSeanceBegin = request.getParameter( OdsParameters.SEANCE_BEGIN );
        String strSeanceEnd = request.getParameter( OdsParameters.SEANCE_END );

        int result = 0;

        if ( ( ( strSeanceBegin != null ) && !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( strSeanceBegin.trim(  ) ) ) &&
                ( DateUtils.getDate( strSeanceBegin.trim(  ), true ) != null ) )
        {
            result = 0;
        }
        else
        {
            champsNonSaisie.add( PROPERTY_LABEL_DATE_DEBUT );

            return 1;
        }

        if ( ( ( strSeanceEnd != null ) && !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( strSeanceEnd.trim(  ) ) ) &&
                ( DateUtils.getDate( strSeanceEnd.trim(  ), true ) != null ) )
        {
            result = 0;
        }
        else
        {
            champsNonSaisie.add( PROPERTY_LABEL_DATE_FIN );

            return 2;
        }

        return result;
    }

    /**
    * retourne la map contenant les lments utiles au template HTML de la cration
    * @param request la requete HTTP
    * @return model la map contenant les lments utiles au template HTML de la cration
    */
    public HashMap<String, Object> getCreateSeance( HttpServletRequest request )
    {
        HashMap<String, Object> model = new HashMap<String, Object>(  );
        getCreateSeanceSpec( model );

        return model;
    }

    /**
     * retourne la map contenant les lments utiles au template HTML de la modification
     * @param request la requete HTTP
     * @return model la map contenant les lments utiles au template HTML de la modification
     */
    public HashMap<String, Object> getModificationSeance( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        int nIdSeance = OdsUtils.getIntegerParameter( OdsParameters.ID_SEANCE, request );

        if ( nIdSeance != -1 )
        {
            GSeance seance = _seanceHome.findByPrimaryKey( nIdSeance, plugin );
            OdsUtils.testNullObject( seance, request );

            getModificationSeanceSpec( seance, model );
        }
        else
        {
            return null;
        }

        return model;
    }

    /**
     * retourne une demande de confirmation pour la suppression de la sance choisie
     * @param request la requete HTTP
     * @return String Template de confirmation de suppression des seances
     */
    public String getSupressionSeance( HttpServletRequest request )
    {
        if ( request.getParameter( OdsParameters.ID_SEANCE ) != null )
        {
            String strIdSeance = request.getParameter( OdsParameters.ID_SEANCE );
            UrlItem url = new UrlItem( URL_DO_SUPPRESSION_SEANCE_JSP );
            url.addParameter( OdsParameters.ID_SEANCE, strIdSeance );

            return AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRM_DELETE_SEANCE, url.getUrl(  ),
                AdminMessage.TYPE_CONFIRMATION );
        }
        else
        {
            return OdsConstants.CONSTANTE_CHAINE_VIDE;
        }
    }

    /**
     * Permet de calculer les dates automatiquement en fonction de la date de dbut et de fin de sance
     * Ne crer pas la sance.
     * @param request La requete HTTP
     * @return String template de creation de seance
     */
    public String doCreateSeance( HttpServletRequest request )
    {
        List<String> champsNonSaisie = new ArrayList<String>(  );
        int result = isValidForm( champsNonSaisie, request, false );

        if ( result == 0 )
        {
            result = doCreateSeanceSpec( request );
        }

        if ( ( result == 1 ) || ( result == 2 ) )
        {
            Object[] messagesArgs = { I18nService.getLocalizedString( champsNonSaisie.get( 0 ), request.getLocale(  ) ) };

            return AdminMessageService.getMessageUrl( request, MESSAGE_CANNOT_CREATE_SEANCE, messagesArgs,
                AdminMessage.TYPE_STOP );
        }
        else
        {
            return AppPathService.getBaseUrl( request ) + URL_CREATION_SEANCE_JSP;
        }
    }

    /**
     * Retourne la map contenant les lments utiles au template HTML de la modification
     *
     * @param request la requete HTTP
     * @return  model la map contenant les lments utiles au template HTML de la modification
     */
    public abstract HashMap<String, Object> doModificationSeance( HttpServletRequest request );

    /**
     * cre une sance
     * si tous les champs obligatoires du formulaire de cration d'une sance ont t renseign,<BR>
     * la mthode retourne l'url de gestion des sances,<BR>
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request La requete HTTP
     * @return String template liste des seances
     */
    public String doCreateAndRegisterSeance( HttpServletRequest request )
    {
        List<String> champsNonSaisie = new ArrayList<String>(  );
        int result = isValidForm( champsNonSaisie, request, false );

        if ( result == 0 )
        {
            return doCreateAndRegisterSeanceSpec( request, champsNonSaisie );
        }
        else
        {
            Object[] messagesArgs = { I18nService.getLocalizedString( champsNonSaisie.get( 0 ), request.getLocale(  ) ) };

            return AdminMessageService.getMessageUrl( request, MESSAGE_CANNOT_CREATE_SEANCE, messagesArgs,
                AdminMessage.TYPE_STOP );
        }
    }

    /**
     * Modifie une sance
     * si tous les champs obligatoires du formulaire de modification d'une sance ont t renseign,<BR>
     * la mthode retourne l'url de gestion des sances,<BR>
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request La requete HTTP
     * @return String template liste des seances
     */
    public String doUpdateAndRegisterSeance( HttpServletRequest request )
    {
        String strReturn = null;

        List<String> champsNonSaisie = new ArrayList<String>(  );
        int result = isValidForm( champsNonSaisie, request, false );

        if ( result == 0 )
        {
            strReturn = doUpdateAndRegisterSeanceSpec( request, champsNonSaisie );
        }

        if ( strReturn == null )
        {
            Object[] messagesArgs = { I18nService.getLocalizedString( champsNonSaisie.get( 0 ), request.getLocale(  ) ) };

            strReturn = AdminMessageService.getMessageUrl( request, MESSAGE_CANNOT_CREATE_SEANCE, messagesArgs,
                    AdminMessage.TYPE_STOP );
        }

        return strReturn;
    }

    /**
     * Supprime la sance choisi
     * @param request la requete HTTP
     * @return String template de liste des seances
     */
    public String doSupressionSeance( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        int nIdSeance = OdsUtils.getIntegerParameter( OdsParameters.ID_SEANCE, request );

        if ( nIdSeance != -1 )
        {
            try
            {
                /* On met  jour les indexer qui pointe vers cette sance */
                GSeance derniereSeance = _seanceHome.getDerniereSeance( plugin );
                List<Indexer> listIndexer = _indexerHome.findBySeance( nIdSeance, plugin );

                for ( Indexer index : listIndexer )
                {
                    index.setIdSeance( ( derniereSeance == null ) ? ( -1 ) : derniereSeance.getIdSeance(  ) );
                    _indexerHome.update( index, plugin );
                }

                _seanceHome.remove( nIdSeance, plugin );
            }
            catch ( AppException ae )
            {
                if ( ae.getInitialException(  ) instanceof SQLException )
                {
                    return AdminMessageService.getMessageUrl( request, MESSAGE_CANNOT_DELETE_SEANCE,
                        AdminMessage.TYPE_STOP );
                }
                else
                {
                    AppLogService.error( ae );
                }

                throw ae;
            }
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * rinitialise l'objet seance stock en session
     * @return String template liste des seances
     */
    public abstract void doResetSeance(  );
}
