/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.seance;

import fr.paris.lutece.plugins.ods.service.IODSService;

import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;


public interface ISeanceService extends IODSService
{
    /**
    * Retourne la map contenant les lments utiles au template HTML de la liste
    * Seuls les sances dont la date de debut est suprieure ou gale  la date du jour seront affichs
    *
    * @param request la requete HTTP
    * @return model la map contenant les lments utiles au template HTML de la liste
    */
    HashMap<String, Object> getSeanceList( HttpServletRequest request );

    /**
    * Retourne la map contenant les lments utiles au template HTML de la liste
    * Seuls les sances dont la date de dbut est comprise entre les deux dates du filtre seront affichs
    * @param request request la requete HTTP
    * @return model la map contenant les lments utiles au template HTML de la liste
    */
    HashMap<String, Object> applySeanceFilter( HttpServletRequest request );

    /**
    * retourne la map contenant les lments utiles au template HTML de la cration
    * @param request la requete HTTP
    * @return model la map contenant les lments utiles au template HTML de la cration
    */
    HashMap<String, Object> getCreateSeance( HttpServletRequest request );

    /**
     * retourne la map contenant les lments utiles au template HTML de la modification
     * @param request la requete HTTP
     * @return model la map contenant les lments utiles au template HTML de la modification
     */
    HashMap<String, Object> getModificationSeance( HttpServletRequest request );

    /**
     * retourne une demande de confirmation pour la suppression de la sance choisie
     * @param request la requete HTTP
     * @return String Template de confirmation de suppression des seances
     */
    String getSupressionSeance( HttpServletRequest request );

    /**
     * Permet de calculer les dates automatiquement en fonction de la date de dbut et de fin de sance
     * Ne crer pas la sance.
     * @param request La requete HTTP
     * @return String template de creation de seance
     */
    String doCreateSeance( HttpServletRequest request );

    /**
     * Retourne la map contenant les lments utiles au template HTML de la modification
     *
     * @param request la requete HTTP
     * @return  model la map contenant les lments utiles au template HTML de la modification
     */
    HashMap<String, Object> doModificationSeance( HttpServletRequest request );

    /**
     * cre une sance
     * si tous les champs obligatoires du formulaire de cration d'une sance ont t renseign,<BR>
     * la mthode retourne l'url de gestion des sances,<BR>
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request La requete HTTP
     * @return String template liste des seances
     */
    String doCreateAndRegisterSeance( HttpServletRequest request );

    /**
     * Modifie une sance
     * si tous les champs obligatoires du formulaire de modification d'une sance ont t renseign,<BR>
     * la mthode retourne l'url de gestion des sances,<BR>
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request La requete HTTP
     * @return String template liste des seances
     */
    String doUpdateAndRegisterSeance( HttpServletRequest request );

    /**
     * rinitialise l'objet seance stock en session
     * @return String template liste des seances
     */
    void doResetSeance(  );

    /**
     * Supprime la sance choisi
     * @param request la requete HTTP
     * @return String template de liste des seances
     */
    String doSupressionSeance( HttpServletRequest request );
}
