/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.search;

import fr.paris.lutece.plugins.ods.business.acte.ActeHome;
import fr.paris.lutece.plugins.ods.business.fichier.FichierPhysiqueHome;
import fr.paris.lutece.plugins.ods.business.fichier.IFichierHome;
import fr.paris.lutece.plugins.ods.business.search.ISearchDAO;
import fr.paris.lutece.plugins.ods.business.search.ISearchHome;
import fr.paris.lutece.plugins.ods.dto.IBusinessItem;
import fr.paris.lutece.plugins.ods.dto.acte.Acte;
import fr.paris.lutece.plugins.ods.dto.acte.IActeFilter;
import fr.paris.lutece.plugins.ods.dto.arrete.Arrete;
import fr.paris.lutece.plugins.ods.dto.deliberationdesignation.DeliberationDesignation;
import fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.FichierPhysique;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter;
import fr.paris.lutece.plugins.ods.dto.fichier.ITypeDocument;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur;
import fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.message.SiteMessageException;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.plugin.PluginService;
import fr.paris.lutece.portal.service.search.SearchIndexer;
import fr.paris.lutece.portal.service.spring.SpringContextService;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.service.util.AppPropertiesService;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.index.IndexWriter;

import org.pdfbox.exceptions.CryptographyException;
import org.pdfbox.exceptions.InvalidPasswordException;

import org.pdfbox.pdmodel.PDDocument;

import org.pdfbox.util.PDFTextStripper;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.StringWriter;

import java.util.ArrayList;
import java.util.List;


/**
 * ODSSearchIndexer : permet de rcuprer les documents pour indexer/supprimer dans/de l'index
 */
public abstract class AbstractODSSearchIndexer<GSeance extends ISeance, GSeanceFilter extends ISeanceFilter, GElu extends IElu, GFichier extends IFichier<GSeance, GFichier>, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GRequeteUtilisateur extends IRequeteUtilisateur<GSeance, GSeanceFilter>, GResultatRequete extends IResultatRequete<GSeance, GSeanceFilter, GRequeteUtilisateur, GElu, GFichier, GVoeuAmendement, GPDD>, GSearchDAO extends ISearchDAO<GFichier, GSeance, GSeanceFilter, GRequeteUtilisateur, GElu, GVoeuAmendement, GPDD>, GFichierFilter extends IFichierFilter, GTypeDocument extends ITypeDocument, GActeFilter extends IActeFilter<GFichier, GSeance>>
    implements SearchIndexer, IODSSearchIndexer
{
    private static final String PROPERTY_INDEXER_NAME = "ods.searchindexer.name";
    private static final String INDEXER_DESCRIPTION = "ods.odssearchindexer.description";
    private static final String INDEXER_VERSION = "1.0.0";
    private static final String PROPERTY_INDEXER_ENABLE = "ods.searchindexer.enable";
    private static final String VA_TYPE_A = "A";
    private static final String VA_TYPE_LR = "LR";
    protected boolean _bArchiveIndexer;
    private ISearchService<GSeance, GSeanceFilter, GRequeteUtilisateur, GResultatRequete, GElu, GFichier, GVoeuAmendement, GPDD> _searchService =
        ( (ISearchService<GSeance, GSeanceFilter, GRequeteUtilisateur, GResultatRequete, GElu, GFichier, GVoeuAmendement, GPDD>) SpringContextService.getPluginBean( OdsConstants.CONSTANTE_PLUGIN_ODS,
            "searchService" ) );
    protected ISearchHome<GFichier, GSeance, GSeanceFilter, GRequeteUtilisateur, GSearchDAO, GElu, GVoeuAmendement, GPDD> _searchHome =
        ( (ISearchHome<GFichier, GSeance, GSeanceFilter, GRequeteUtilisateur, GSearchDAO, GElu, GVoeuAmendement, GPDD>) SpringContextService.getPluginBean( OdsConstants.CONSTANTE_PLUGIN_ODS,
            "searchHome" ) );
    protected IFichierHome<GFichier, GFichierFilter, GTypeDocument, GSeance> _fichierHome = ( (IFichierHome<GFichier, GFichierFilter, GTypeDocument, GSeance>) SpringContextService.getPluginBean( OdsConstants.CONSTANTE_PLUGIN_ODS,
            "fichierHome" ) );
    protected FichierPhysiqueHome _fichierPhysiqueHome = ( (FichierPhysiqueHome) SpringContextService.getPluginBean( OdsConstants.CONSTANTE_PLUGIN_ODS,
            "fichierPhysiqueHome" ) );
    protected ActeHome<GSeance, GFichier, GActeFilter> _acteHome = ( (ActeHome<GSeance, GFichier, GActeFilter>) SpringContextService.getPluginBean( OdsConstants.CONSTANTE_PLUGIN_ODS,
            "acteHome" ) );

    /**
     * Constructeur: rcupre tous les documents  indexer, soit pour les archives soit pour la prochaine sance
     * @param bArchiveIndexer le SearchIndexer agit sur les archives ou la prochaine sance
     */
    public AbstractODSSearchIndexer( boolean bArchiveIndexer )
    {
        _bArchiveIndexer = bArchiveIndexer;
    }

    protected abstract void indexAllDocumentsSpec( GSeance prochaineSeance, Plugin plugin, IndexWriter indexWriter );

    protected abstract void getDocumentsToIndexSpec( IndexWriter indexWriter, Object object );

    protected abstract String getTypePDDSpec( GPDD pdd );

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.search.IODSSearchIndexerr#getDocuments(org.apache.lucene.index.IndexWriter)
         */
    public abstract void getDocuments( IndexWriter indexWriter )
        throws IOException, InterruptedException;

    /**
    * Retourne la liste des documents Lucene crs  partir des PDD de la base
    * @param prochaineSeance la prochaine sance
    * @param plugin le plugin ODS
    * @param indexWriter l'index
    * @return la liste des documents Lucene crs  partir des PDD de la base
    */
    protected List<Document> indexAllDocumentsPDDs( GSeance prochaineSeance, Plugin plugin, IndexWriter indexWriter )
    {
        List<Document> listDocs = new ArrayList<Document>(  );
        List<GPDD> listPdds = _searchHome.findPDDsAIndexer( _bArchiveIndexer, prochaineSeance, plugin );

        for ( GPDD pdd : listPdds )
        {
            indexPDDDocuments( pdd, indexWriter );
        }

        return listDocs;
    }

    /**
     * Retourne la liste des documents Lucene crs  partir des dlibrations de dsignation de la base
     * @param plugin le plugin ODS
     * @param indexWriter l'index
     * @return la liste des documents Lucene crs  partir des dlibrations de dsignation de la base
     */
    protected List<Document> indexAllDocumentsDeliberationDesignation( Plugin plugin, IndexWriter indexWriter )
    {
        List<Document> listDocs = new ArrayList<Document>(  );
        List<DeliberationDesignation<GSeance, GFichier>> listDeliberations = _searchHome.findDeliberationDesignationAIndexer( plugin );

        for ( DeliberationDesignation<GSeance, GFichier> deliberation : listDeliberations )
        {
            indexDeliberationDocument( deliberation, indexWriter );
        }

        return listDocs;
    }

    /**
     * Indexe la liste des documents Lucene cr  partir des VoeuAmendement de la base
     * @param prochaineSeance la prochaine sance
     * @param plugin le plugin ODS
     * @param indexWriter l'index
     */
    protected void indexAllDocumentsVoeuAmendement( GSeance prochaineSeance, Plugin plugin, IndexWriter indexWriter )
    {
        List<GVoeuAmendement> listVoeuAmendements = _searchHome.findVAAIndexer( _bArchiveIndexer, prochaineSeance,
                plugin );

        for ( GVoeuAmendement va : listVoeuAmendements )
        {
            indexVADocument( va, indexWriter );
        }
    }

    /**
     * Indexe la liste de tous les fichiers  indexer dans les archives
     * @param plugin le plugin ODS
     * @param indexWriter l'index
     */
    protected void indexAllDocumentsFichier( Plugin plugin, IndexWriter indexWriter )
    {
        List<GFichier> listFichiers = _searchHome.findFichiersAIndexer( plugin );

        for ( GFichier fichier : listFichiers )
        {
            indexFichierDocument( fichier, indexWriter );
        }
    }

    /* ******************** */
    /* CHANGEMENT DE SEANCE */
    /* ******************** */

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.search.IODSSearchIndexerr#getDocumentsAArchiver(int, org.apache.lucene.index.IndexWriter)
         */
    public void getDocumentsAArchiver( int nIdAncienneSeance, IndexWriter indexWriter )
    {
        Plugin plugin = PluginService.getPlugin( OdsConstants.CONSTANTE_PLUGIN_ODS );

        indexAllDocumentsPDDsAArchiver( nIdAncienneSeance, plugin, indexWriter );
        indexAllDocumentsVAsAArchiver( nIdAncienneSeance, plugin, indexWriter );
        indexAllDocumentsFichiersAArchiver( nIdAncienneSeance, plugin, indexWriter );
    }

    /**
     * Indexe la liste des documents Lucene correspondant aux PDDs  indexer dans les archives
     * @param nIdAncienneSeance l'id de la dernire sance
     * @param plugin le plugin
     * @param indexWriter l'index
     */
    private void indexAllDocumentsPDDsAArchiver( int nIdAncienneSeance, Plugin plugin, IndexWriter indexWriter )
    {
        List<GPDD> listPdds = _searchHome.findPDDsChangementSeance( nIdAncienneSeance, plugin );

        for ( GPDD pdd : listPdds )
        {
            indexPDDDocuments( pdd, indexWriter );
        }
    }

    /**
     * Indexe la liste des documents Lucene correspondant aux VAs  ajouter dans l'index des archives
     * @param nIdAncienneSeance l'id de la dernire sance
     * @param plugin le plugin
     * @param indexWriter l'index
     */
    protected void indexAllDocumentsVAsAArchiver( int nIdAncienneSeance, Plugin plugin, IndexWriter indexWriter )
    {
        List<GVoeuAmendement> listVoeuAmendements = _searchHome.findVAChangementSeance( nIdAncienneSeance, plugin );

        for ( GVoeuAmendement va : listVoeuAmendements )
        {
            indexVADocument( va, indexWriter );
        }
    }

    /**
     * Indexe la liste des documents Lucene correspondant aux "autres documents" de sance
     * @param nIdAncienneSeance l'id de la dernire sance passe
     * @param plugin le plugin
     * @param indexWriter l'index
     */
    protected void indexAllDocumentsFichiersAArchiver( int nIdAncienneSeance, Plugin plugin, IndexWriter indexWriter )
    {
        List<GFichier> listFichiers = _searchHome.findFichiersChangementSeance( nIdAncienneSeance, plugin );

        for ( GFichier fichier : listFichiers )
        {
            indexFichierDocument( fichier, indexWriter );
        }
    }

    /* ****************************************************** */
    /* CREATION DES DOCUMENTS LUCENE A PARTIR DES OBJETS JAVA */
    /* ****************************************************** */

    /**
     * Indexe la liste des documents du PDD, c'est  dire le PDD lui-mme et tous ses fichiers
     * @param pdd l'objet pdd
     * @param indexWriter l'index
     */
    protected void indexPDDDocuments( GPDD pdd, IndexWriter indexWriter )
    {
        Plugin plugin = PluginService.getPlugin( OdsConstants.CONSTANTE_PLUGIN_ODS );

        StringBuffer sbPddContentToIndex = new StringBuffer(  );
        String strTypeObjet = getTypePDDSpec( pdd );

        Document docPdd = new Document(  );
        docPdd.add( new Field( OdsConstants.FIELD_ID_OBJET, strTypeObjet + " " + pdd.getId(  ), Field.Store.YES,
                Field.Index.UN_TOKENIZED ) );
        docPdd.add( new Field( OdsConstants.FIELD_TYPE_OBJET, strTypeObjet, Field.Store.YES, Field.Index.UN_TOKENIZED ) );
        docPdd.add( new Field( OdsConstants.FIELD_ODS_OBJECT_ID, Integer.toString( pdd.getId(  ) ), Field.Store.YES,
                Field.Index.UN_TOKENIZED ) );

        if ( ( pdd.getReference(  ) != null ) && !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( pdd.getReference(  ) ) )
        {
            sbPddContentToIndex.append( pdd.getReference(  ) );
        }

        if ( ( pdd.getObjet(  ) != null ) && !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( pdd.getObjet(  ) ) )
        {
            sbPddContentToIndex.append( " " ).append( pdd.getObjet(  ) ).append( " " );
        }

        GFichierFilter fichierFilter = _fichierHome.newFichierFilterInstance(  );
        fichierFilter.setPDD( pdd.getId(  ) );

        List<GFichier> fichiers = _fichierHome.findByFilter( fichierFilter, plugin );

        for ( GFichier fichier : fichiers )
        {
            FichierPhysique fichierPhysique = _fichierPhysiqueHome.findByPrimaryKey( fichier.getFichier(  )
                                                                                            .getIdFichier(  ), plugin );
            Document docFichierPdd = new Document(  );
            docFichierPdd.add( new Field( OdsConstants.FIELD_ID_OBJET,
                    OdsConstants.TYPE_FICHIER_PDD + " " + fichier.getId(  ), Field.Store.YES, Field.Index.UN_TOKENIZED ) );
            docFichierPdd.add( new Field( OdsConstants.FIELD_TYPE_OBJET, OdsConstants.TYPE_FICHIER_PDD + pdd.getId(  ),
                    Field.Store.YES, Field.Index.UN_TOKENIZED ) );
            docFichierPdd.add( new Field( OdsConstants.FIELD_ODS_OBJECT_ID, Integer.toString( fichier.getId(  ) ),
                    Field.Store.YES, Field.Index.UN_TOKENIZED ) );

            String strContentFichier = OdsConstants.CONSTANTE_CHAINE_VIDE;

            try
            {
                strContentFichier += getContent( fichierPhysique.getDonnees(  ), fichier.getId(  ),
                    fichierPhysique.getIdFichier(  ) );
            }
            catch ( Exception e )
            {
                AppLogService.error( "Error getting content from file: id = " + fichierPhysique.getIdFichier(  ) );
            }

            if ( fichier.getTexte(  ) != null )
            {
                strContentFichier += ( " " + fichier.getTexte(  ) );
            }

            if ( strContentFichier.length(  ) != 0 )
            {
                docFichierPdd.add( new Field( OdsConstants.FIELD_CONTENT, strContentFichier.toLowerCase(  ),
                        Field.Store.NO, Field.Index.TOKENIZED ) );

                try
                {
                    indexWriter.addDocument( docFichierPdd );
                }
                catch ( CorruptIndexException e )
                {
                    AppLogService.error( "Error while indexing file : " + e.getMessage(  ) );
                }
                catch ( IOException e )
                {
                    AppLogService.error( "Error while indexing file : " + e.getMessage(  ) );
                }

                sbPddContentToIndex.append( strContentFichier );
            }
        }

        if ( sbPddContentToIndex.length(  ) > 0 )
        {
            docPdd.add( new Field( OdsConstants.FIELD_CONTENT, sbPddContentToIndex.toString(  ).toLowerCase(  ),
                    Field.Store.NO, Field.Index.TOKENIZED ) );
        }

        try
        {
            indexWriter.addDocument( docPdd );
        }
        catch ( CorruptIndexException e )
        {
            AppLogService.error( "Error while indexing file : " + e.getMessage(  ) );
        }
        catch ( IOException e )
        {
            AppLogService.error( "Error while indexing file : " + e.getMessage(  ) );
        }
    }

    /**
    * Indexe le document correspondant au voeu ou amendement
    * @param va le voeu ou amendement
    * @param indexWriter l'index
    */
    protected void indexVADocument( GVoeuAmendement va, IndexWriter indexWriter )
    {
        Plugin plugin = PluginService.getPlugin( OdsConstants.CONSTANTE_PLUGIN_ODS );

        Document doc = new Document(  );
        StringBuffer sbVAContentToIndex = new StringBuffer(  );

        String strTypeVA;

        if ( ( va.getType(  ).equals( VA_TYPE_A ) ) || ( va.getType(  ).equals( VA_TYPE_LR ) ) )
        {
            strTypeVA = OdsConstants.TYPE_AMENDEMENT;
        }
        else
        {
            strTypeVA = OdsConstants.TYPE_VOEU;
        }

        doc.add( new Field( OdsConstants.FIELD_ID_OBJET, strTypeVA + OdsConstants.CONSTANTE_ESPACE + va.getId(  ),
                Field.Store.YES, Field.Index.UN_TOKENIZED ) );
        doc.add( new Field( OdsConstants.FIELD_TYPE_OBJET, strTypeVA, Field.Store.YES, Field.Index.UN_TOKENIZED ) );
        doc.add( new Field( OdsConstants.FIELD_ODS_OBJECT_ID, Integer.toString( va.getId(  ) ), Field.Store.YES,
                Field.Index.UN_TOKENIZED ) );

        if ( ( va.getReferenceCompleteFront(  ) != null ) &&
                !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( va.getReferenceCompleteFront(  ) ) )
        {
            sbVAContentToIndex.append( va.getReferenceCompleteFront(  ) );
        }

        if ( ( va.getObjet(  ) != null ) && !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( va.getObjet(  ) ) )
        {
            sbVAContentToIndex.append( OdsConstants.CONSTANTE_ESPACE ).append( va.getObjet(  ) );
        }

        GFichier fichier = _fichierHome.findByPrimaryKey( va.getFichier(  ).getId(  ), plugin );

        FichierPhysique fichierPhysique = _fichierPhysiqueHome.findByPrimaryKey( fichier.getFichier(  ).getIdFichier(  ),
                plugin );

        try
        {
            sbVAContentToIndex.append( OdsConstants.CONSTANTE_ESPACE )
                              .append( getContent( fichierPhysique.getDonnees(  ), fichier.getId(  ),
                    fichierPhysique.getIdFichier(  ) ) );
        }
        catch ( Exception e )
        {
            AppLogService.error( "Error getting content from file: id = " + fichierPhysique.getIdFichier(  ) );
        }

        if ( fichier.getTexte(  ) != null )
        {
            sbVAContentToIndex.append( OdsConstants.CONSTANTE_ESPACE ).append( fichier.getTexte(  ) );
        }

        if ( sbVAContentToIndex.length(  ) > 0 )
        {
            doc.add( new Field( OdsConstants.FIELD_CONTENT, sbVAContentToIndex.toString(  ).toLowerCase(  ),
                    Field.Store.NO, Field.Index.TOKENIZED ) );
        }

        try
        {
            indexWriter.addDocument( doc );
        }
        catch ( CorruptIndexException e )
        {
            AppLogService.error( "Error while indexing file : " + e.getMessage(  ) );
        }
        catch ( IOException e )
        {
            AppLogService.error( "Error while indexing file : " + e.getMessage(  ) );
        }
    }

    /**
     * Indexe le document Lucene correspondant au fichier
     * @param fichier le fichier
     * @param indexWriter l'index
     */
    protected void indexFichierDocument( GFichier fichier, IndexWriter indexWriter )
    {
        Plugin plugin = PluginService.getPlugin( OdsConstants.CONSTANTE_PLUGIN_ODS );

        Document doc = new Document(  );
        doc.add( new Field( OdsConstants.FIELD_ID_OBJET, OdsConstants.TYPE_AUTRE_DOC + " " + fichier.getId(  ),
                Field.Store.YES, Field.Index.UN_TOKENIZED ) );
        doc.add( new Field( OdsConstants.FIELD_TYPE_OBJET, OdsConstants.TYPE_AUTRE_DOC, Field.Store.YES,
                Field.Index.UN_TOKENIZED ) );
        doc.add( new Field( OdsConstants.FIELD_ODS_OBJECT_ID, Integer.toString( fichier.getId(  ) ), Field.Store.YES,
                Field.Index.UN_TOKENIZED ) );

        FichierPhysique fichierPhysique = _fichierPhysiqueHome.findByPrimaryKey( fichier.getFichier(  ).getIdFichier(  ),
                plugin );

        String strContent = getContent( fichierPhysique.getDonnees(  ), fichier.getId(  ),
                fichierPhysique.getIdFichier(  ) );

        if ( fichier.getTexte(  ) != null )
        {
            strContent += fichier.getTexte(  );
        }

        doc.add( new Field( OdsConstants.FIELD_CONTENT, strContent.toLowerCase(  ), Field.Store.NO,
                Field.Index.TOKENIZED ) );

        try
        {
            indexWriter.addDocument( doc );
        }
        catch ( CorruptIndexException e )
        {
            AppLogService.error( "Error while indexing file : " + e.getMessage(  ) );
        }
        catch ( IOException e )
        {
            AppLogService.error( "Error while indexing file : " + e.getMessage(  ) );
        }
    }

    /**
     * Indexe le document Lucene correspondant  l'arret
     * @param arrete l'arrete
     * @param indexWriter l'index
     */
    protected void indexArreteDocument( Arrete<GSeance, GFichier> arrete, IndexWriter indexWriter )
    {
        Plugin plugin = PluginService.getPlugin( OdsConstants.CONSTANTE_PLUGIN_ODS );

        Document doc = new Document(  );
        StringBuffer sbArreteContentToIndex = new StringBuffer(  );

        doc.add( new Field( OdsConstants.FIELD_ID_OBJET, OdsConstants.TYPE_ARRETE + " " + arrete.getId(  ),
                Field.Store.YES, Field.Index.UN_TOKENIZED ) );
        doc.add( new Field( OdsConstants.FIELD_TYPE_OBJET, OdsConstants.TYPE_ARRETE, Field.Store.YES,
                Field.Index.UN_TOKENIZED ) );
        doc.add( new Field( OdsConstants.FIELD_ODS_OBJECT_ID, Integer.toString( arrete.getId(  ) ), Field.Store.YES,
                Field.Index.UN_TOKENIZED ) );

        if ( ( arrete.getIntitule(  ) != null ) &&
                !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( arrete.getIntitule(  ) ) )
        {
            sbArreteContentToIndex.append( arrete.getIntitule(  ) );
        }

        GFichier fichier = _fichierHome.findByPrimaryKey( arrete.getFichierOr(  ).getId(  ), plugin );

        FichierPhysique fichierPhysique = _fichierPhysiqueHome.findByPrimaryKey( fichier.getFichier(  ).getIdFichier(  ),
                plugin );

        try
        {
            sbArreteContentToIndex.append( OdsConstants.CONSTANTE_ESPACE )
                                  .append( getContent( fichierPhysique.getDonnees(  ), fichier.getId(  ),
                    fichierPhysique.getIdFichier(  ) ) );
        }
        catch ( Exception e )
        {
            AppLogService.error( "Error getting content from file: id = " + fichierPhysique.getIdFichier(  ) );
        }

        if ( fichier.getTexte(  ) != null )
        {
            sbArreteContentToIndex.append( OdsConstants.CONSTANTE_ESPACE ).append( fichier.getTexte(  ) );
        }

        if ( sbArreteContentToIndex.length(  ) > 0 )
        {
            doc.add( new Field( OdsConstants.FIELD_CONTENT, sbArreteContentToIndex.toString(  ).toLowerCase(  ),
                    Field.Store.NO, Field.Index.TOKENIZED ) );
        }

        try
        {
            indexWriter.addDocument( doc );
        }
        catch ( CorruptIndexException e )
        {
            AppLogService.error( "Error while indexing file : " + e.getMessage(  ) );
        }
        catch ( IOException e )
        {
            AppLogService.error( "Error while indexing file : " + e.getMessage(  ) );
        }
    }

    /**
     * Indexe le document Lucene correspondant  la dlibration de dsignation
     * @param deliberation la dlibration
     * @param indexWriter l'index
     */
    protected void indexDeliberationDocument( IDeliberationDesignation<GSeance, GFichier> deliberation,
        IndexWriter indexWriter )
    {
        Plugin plugin = PluginService.getPlugin( OdsConstants.CONSTANTE_PLUGIN_ODS );

        Document doc = new Document(  );
        StringBuffer sbDeliberationContentToIndex = new StringBuffer(  );

        doc.add( new Field( OdsConstants.FIELD_ID_OBJET,
                OdsConstants.TYPE_DELIBERATION_DESIGNATION + " " + deliberation.getId(  ), Field.Store.YES,
                Field.Index.UN_TOKENIZED ) );
        doc.add( new Field( OdsConstants.FIELD_TYPE_OBJET, OdsConstants.TYPE_DELIBERATION_DESIGNATION, Field.Store.YES,
                Field.Index.UN_TOKENIZED ) );
        doc.add( new Field( OdsConstants.FIELD_ODS_OBJECT_ID, Integer.toString( deliberation.getId(  ) ),
                Field.Store.YES, Field.Index.UN_TOKENIZED ) );

        if ( ( deliberation.getIntitule(  ) != null ) &&
                !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( deliberation.getIntitule(  ) ) )
        {
            sbDeliberationContentToIndex.append( deliberation.getIntitule(  ) );
        }

        if ( deliberation.getActeCourant(  ) != null )
        {
            Acte<GSeance, GFichier> acteCourant = _acteHome.findByPrimaryKey( deliberation.getActeCourant(  ).getId(  ),
                    plugin );
            GFichier fichier = _fichierHome.findByPrimaryKey( acteCourant.getDeliberationFinale(  ).getId(  ), plugin );

            FichierPhysique fichierPhysique = _fichierPhysiqueHome.findByPrimaryKey( fichier.getFichier(  )
                                                                                            .getIdFichier(  ), plugin );

            try
            {
                sbDeliberationContentToIndex.append( OdsConstants.CONSTANTE_ESPACE ).append( fichier.getNom(  ) )
                                            .append( OdsConstants.CONSTANTE_ESPACE )
                                            .append( getContent( fichierPhysique.getDonnees(  ), fichier.getId(  ),
                        fichierPhysique.getIdFichier(  ) ) );
            }
            catch ( Exception e )
            {
                AppLogService.error( "Error getting content from file: id = " + fichierPhysique.getIdFichier(  ) );
            }

            if ( fichier.getTexte(  ) != null )
            {
                sbDeliberationContentToIndex.append( OdsConstants.CONSTANTE_ESPACE ).append( fichier.getTexte(  ) );
            }
        }

        if ( sbDeliberationContentToIndex.length(  ) > 0 )
        {
            doc.add( new Field( OdsConstants.FIELD_CONTENT, sbDeliberationContentToIndex.toString(  ).toLowerCase(  ),
                    Field.Store.NO, Field.Index.TOKENIZED ) );
        }

        try
        {
            indexWriter.addDocument( doc );
        }
        catch ( CorruptIndexException e )
        {
            AppLogService.error( "Error while indexing file : " + e.getMessage(  ) );
        }
        catch ( IOException e )
        {
            AppLogService.error( "Error while indexing file : " + e.getMessage(  ) );
        }
    }

    /* *********************** */
    /* INDEXATION INCREMENTALE */
    /* *********************** */

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.search.IODSSearchIndexerr#getDocumentsToRemove()
         */
    public List<Document> getDocumentsToRemove(  )
    {
        Plugin plugin = PluginService.getPlugin( OdsConstants.CONSTANTE_PLUGIN_ODS );

        List<Document> listDocs = new ArrayList<Document>(  );
        Document doc;
        List<IBusinessItem> listObjets = _searchService.getDeletedObjects( _bArchiveIndexer, plugin );
        listObjets.addAll( _searchService.getUpdatedObjects( false, _bArchiveIndexer, plugin ) );

        for ( Object object : listObjets )
        {
            if ( ( object instanceof IPDD<?, ?, ?, ?, ?> ) && ( object != null ) )
            {
                String strTypeObjet = getTypePDDSpec( (GPDD) object );

                doc = new Document(  );
                doc.add( new Field( OdsConstants.FIELD_ID_OBJET,
                        strTypeObjet + OdsConstants.CONSTANTE_ESPACE + ( (IPDD) object ).getId(  ), Field.Store.YES,
                        Field.Index.UN_TOKENIZED ) );
                listDocs.add( doc );
                //listDocs.addAll( getListeDocumentsFichiersDuPddSuppression( ( (IPDD) object ).getId(  ) ) );
                listDocs.addAll( getListeDocumentsFichiersDuPddSuppression(  ) );
            }
            else if ( ( object instanceof IVoeuAmendement<?, ?, ?, ?, ?> ) && ( object != null ) )
            {
                String strTypeVA = ( (GVoeuAmendement) object ).getType(  );
                String strTypeDocVA;

                if ( strTypeVA.equals( VA_TYPE_A ) || strTypeVA.equals( VA_TYPE_LR ) )
                {
                    strTypeDocVA = OdsConstants.TYPE_AMENDEMENT;
                }
                else
                {
                    strTypeDocVA = OdsConstants.TYPE_VOEU;
                }

                doc = new Document(  );
                doc.add( new Field( OdsConstants.FIELD_ID_OBJET,
                        strTypeDocVA + OdsConstants.CONSTANTE_ESPACE + ( (GVoeuAmendement) object ).getId(  ),
                        Field.Store.YES, Field.Index.UN_TOKENIZED ) );
                listDocs.add( doc );
            }
            else if ( ( object instanceof IFichier<?, ?> ) && ( object != null ) )
            {
                doc = new Document(  );
                doc.add( new Field( OdsConstants.FIELD_ID_OBJET,
                        OdsConstants.TYPE_AUTRE_DOC + OdsConstants.CONSTANTE_ESPACE + ( (GFichier) object ).getId(  ),
                        Field.Store.YES, Field.Index.UN_TOKENIZED ) );
                listDocs.add( doc );
            }
            else if ( ( object instanceof Arrete<?, ?> ) && ( object != null ) )
            {
                doc = new Document(  );
                doc.add( new Field( OdsConstants.FIELD_ID_OBJET,
                        OdsConstants.TYPE_ARRETE + OdsConstants.CONSTANTE_ESPACE +
                        ( (Arrete<GSeance, GFichier>) object ).getId(  ), Field.Store.YES, Field.Index.UN_TOKENIZED ) );
                listDocs.add( doc );
            }
            else if ( ( object instanceof IDeliberationDesignation<?, ?> ) && ( object != null ) )
            {
                doc = new Document(  );
                doc.add( new Field( OdsConstants.FIELD_ID_OBJET,
                        OdsConstants.TYPE_DELIBERATION_DESIGNATION + OdsConstants.CONSTANTE_ESPACE +
                        ( (IDeliberationDesignation<GSeance, GFichier>) object ).getId(  ), Field.Store.YES,
                        Field.Index.UN_TOKENIZED ) );
                listDocs.add( doc );
            }
            else
            {
                getDocumentsToRemoveSpec( listDocs, object );
            }
        }

        return listDocs;
    }

    protected abstract void getDocumentsToRemoveSpec( List<Document> listDocs, Object object );

    /**
    * Retourne la liste des documents Lucene de type fichier du PDD  supprimer
    * @return une liste de documents
    */
    private List<Document> getListeDocumentsFichiersDuPddSuppression(  )
    {
        // @param nIdPdd l'id du PDD supprim ou modifi
        // OLD METHOD : private List<Document> getListeDocumentsFichiersDuPddSuppression( int nIdPdd )
        //Plugin plugin = PluginService.getPlugin( OdsConstants.CONSTANTE_PLUGIN_ODS );
        List<Document> listDocs = new ArrayList<Document>(  );

        /*NE PEUT PAS FONCTIONNER les fichiers tant dj supprims il ne sont plus dans la base
         * de plus le doc n'est pas ajout  la liste
         *  List<IFichier> listFichiersDuPdd = searchHome.findFichiersDuPdd( null, nIdPdd, plugin );
        
         for ( IFichier fichier : listFichiersDuPdd )
         {
             Document doc = new Document(  );
             doc.add( new Field( OdsConstants.FIELD_ID_OBJET,
                     OdsConstants.TYPE_FICHIER_PDD + OdsConstants.CONSTANTE_ESPACE + fichier.getId(  ), Field.Store.YES,
                     Field.Index.UN_TOKENIZED ) );
         }*/
        return listDocs;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.search.IODSSearchIndexerr#getDocumentsToIndex(org.apache.lucene.index.IndexWriter)
         */
    public void getDocumentsToIndex( IndexWriter indexWriter )
    {
        Plugin plugin = PluginService.getPlugin( OdsConstants.CONSTANTE_PLUGIN_ODS );

        //List<Document> listDocs = new ArrayList<Document>(  );
        List<IBusinessItem> listObjets = _searchService.getAddedObjects( _bArchiveIndexer, plugin );
        listObjets.addAll( _searchService.getUpdatedObjects( true, _bArchiveIndexer, plugin ) );

        for ( Object object : listObjets )
        {
            if ( object instanceof IPDD<?, ?, ?, ?, ?> )
            {
                GPDD pdd = (GPDD) object;
                indexPDDDocuments( pdd, indexWriter );

                /*for ( Document docToIndex : listDocs )
                {
                    try
                    {
                        indexWriter.addDocument( docToIndex );
                    }
                    catch ( IOException ioe )
                    {
                        AppLogService.error( "Error during indexing documents : " + ioe.getMessage(  ) );
                    }
                }*/
            }
            else if ( object instanceof IVoeuAmendement<?, ?, ?, ?, ?> )
            {
                GVoeuAmendement va = (GVoeuAmendement) object;
                indexVADocument( va, indexWriter );
            }
            else if ( object instanceof IFichier<?, ?> )
            {
                GFichier fichier = (GFichier) object;
                indexFichierDocument( fichier, indexWriter );
            }
            else if ( object instanceof Arrete<?, ?> )
            {
                Arrete<GSeance, GFichier> arrete = (Arrete<GSeance, GFichier>) object;
                indexArreteDocument( arrete, indexWriter );
            }
            else if ( object instanceof IDeliberationDesignation<?, ?> )
            {
                IDeliberationDesignation<GSeance, GFichier> deliberation = (IDeliberationDesignation<GSeance, GFichier>) object;
                indexDeliberationDocument( deliberation, indexWriter );
            }
            else
            {
                getDocumentsToIndexSpec( indexWriter, object );
            }
        }
    }

    /* *************************************************************** */
    /* RECUPERATION DU CONTENU D'UN FICHIER PHYSIQUE AVEC L'API PDFBOX */
    /* *************************************************************** */

    /**
     * Retourne le contenu du fichier PDF
     * @param byteArray le contenu du fichier PDF
     * @param nIdFichier l'identifiant du fichier
     * @param nIdFichierPhysique l'identifiant du fichier physique
     * @return le contenu du fichier
     */
    protected String getContent( byte[] byteArray, int nIdFichier, int nIdFichierPhysique )
    {
        String strContent = OdsConstants.CONSTANTE_CHAINE_VIDE;
        PDDocument pdfDocument = null;

        try
        {
            pdfDocument = PDDocument.load( new ByteArrayInputStream( byteArray ) );

            if ( pdfDocument.isEncrypted(  ) )
            {
                pdfDocument.decrypt( OdsConstants.CONSTANTE_CHAINE_VIDE );
            }

            StringWriter writer = new StringWriter(  );
            PDFTextStripper stripper = new PDFTextStripper(  );
            stripper.writeText( pdfDocument, writer );
            strContent = writer.getBuffer(  ).toString(  );
        }
        catch ( CryptographyException e )
        {
            AppLogService.error( "Erreur rcupration pdfDocument:\nfichier.id = " + nIdFichier +
                "\nfichierPhysique.id = " + nIdFichierPhysique + "\nErreur: " + e.getClass(  ) );
        }
        catch ( IOException e )
        {
            AppLogService.error( "Erreur rcupration pdfDocument:\nfichier.id = " + nIdFichier +
                "\nfichierPhysique.id = " + nIdFichierPhysique + "\nErreur: " + e.getClass(  ) );
        }
        catch ( InvalidPasswordException e )
        {
            AppLogService.error( "Erreur rcupration pdfDocument:\nfichier.id = " + nIdFichier +
                "\nfichierPhysique.id = " + nIdFichierPhysique + "\nErreur: " + e.getClass(  ) );
        }
        finally
        {
            if ( pdfDocument != null )
            {
                try
                {
                    pdfDocument.close(  );
                }
                catch ( IOException e )
                {
                    AppLogService.error( "Erreur fermeture pdfDocument:\nfichier.id = " + nIdFichier +
                        "\nfichierPhysique.id = " + nIdFichierPhysique + "\nErreur: " + e.getClass(  ) );
                }
            }
        }

        return strContent;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.search.IODSSearchIndexerr#getDescription()
         */
    public String getDescription(  )
    {
        return INDEXER_DESCRIPTION;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.search.IODSSearchIndexerr#getName()
         */
    public String getName(  )
    {
        return AppPropertiesService.getProperty( PROPERTY_INDEXER_NAME );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.search.IODSSearchIndexerr#getVersion()
         */
    public String getVersion(  )
    {
        return INDEXER_VERSION;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.search.IODSSearchIndexerr#isEnable()
         */
    public boolean isEnable(  )
    {
        String strEnable = AppPropertiesService.getProperty( PROPERTY_INDEXER_ENABLE, "true" );

        return ( strEnable.equalsIgnoreCase( "true" ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.search.IODSSearchIndexerr#getDocuments()
         */
    public List<Document> getDocuments(  ) throws IOException, InterruptedException, SiteMessageException
    {
        return null;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.search.IODSSearchIndexerr#getDocuments(java.lang.String)
         */
    public List<Document> getDocuments( String arg0 ) throws IOException, InterruptedException, SiteMessageException
    {
        // TODO Auto-generated method stub
        return null;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.search.IODSSearchIndexerr#indexDocuments()
         */
    public void indexDocuments(  ) throws IOException, InterruptedException, SiteMessageException
    {
        // TODO Auto-generated method stub
    }
}
