/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.search;

import fr.paris.lutece.plugins.ods.dto.arrete.Arrete;
import fr.paris.lutece.plugins.ods.dto.deliberationdesignation.DeliberationDesignation;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.service.IODSService;
import fr.paris.lutece.portal.service.plugin.Plugin;

import java.util.List;


public interface IDocSearchService<GSeance extends ISeance, GSeanceFilter extends ISeanceFilter, GRequeteUtilisateur extends IRequeteUtilisateur<GSeance, GSeanceFilter>, GElu extends IElu, GFichier extends IFichier<GSeance, GFichier>, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>>
    extends IODSService
{
    /**
    * Retourne la liste des projets correspondant aux critres de recherche
    * @param requete la requte de l'utilisateur
    * @param prochaineSeance la prochaine sance
    * @param listIdsProjets la liste des projets retrouvs dans l'index contenant le(s) mot(s) cl recherchs
    * @param listSeances la liste des sances pour les fiches de sance
    * @param plugin le plugin
    * @return une liste de PDDs de type projet
    */
    List<GPDD> getProjets( GRequeteUtilisateur requete, GSeance prochaineSeance, List<String> listIdsProjets,
        List<GSeance> listSeances, Plugin plugin );

    /**
     * Retourne la liste des propositions correspondant aux critres de recherche
     * @param requete la requte de l'utilisateur
     * @param prochaineSeance la prochaine sance
     * @param listIdsPropositions la liste des propositions retrouves dans l'index contenant le(s) mot(s) cl recherchs
     * @param listSeances la liste des sances pour les fiches de sance
     * @param plugin le plugin
     * @return une liste de PDDs de type proposition
     */
    List<GPDD> getPropositions( GRequeteUtilisateur requete, GSeance prochaineSeance, List<String> listIdsPropositions,
        List<GSeance> listSeances, Plugin plugin );

    /**
     * La liste des amendements correspondant aux critres de recherche
     * @param requete la requte de l'utilisateur
     * @param prochaineSeance la prochaine sance
     * @param listIdsAmendements la liste des amendements retrouvs dans l'index
     * @param listSeances la liste des sances pour les fiches de sance
     * @param plugin le plugin
     * @return une liste de VA de type Amendement ou Lettre Rectificative
     */
    List<GVoeuAmendement> getAmendements( GRequeteUtilisateur requete, GSeance prochaineSeance,
        List<String> listIdsAmendements, List<GSeance> listSeances, Plugin plugin );

    /**
     * La liste des voeux correspondant aux critres de recherche
     * @param requete la requte de l'utilisateur
     * @param prochaineSeance la prochaine sance
     * @param listIdsVoeux la liste des voeux retrouvs dans l'index
     * @param listSeances la liste des sances pour les fiches de sance
     * @param plugin le plugin
     * @return une liste de VA de type Voeu ou Voeu Non Rattach
     */
    List<GVoeuAmendement> getVoeux( GRequeteUtilisateur requete, GSeance prochaineSeance, List<String> listIdsVoeux,
        List<GSeance> listSeances, Plugin plugin );

    /**
     * Retourne la liste des autres documents de sance corrspondant aux critres de recherche
     * @param requete la requte de l'utilisateur
     * @param listIdsFichiers la liste des fichiers indexs correspondant aux critres de recherche
     * @param listSeances la liste des sances pour les fiches de sance
     * @param plugin le plugin
     * @return une liste de fichiers
     */
    List<GFichier> getAutresDocuments( GRequeteUtilisateur requete, List<String> listIdsFichiers,
        List<GSeance> listSeances, Plugin plugin );

    /**
     * Retourne la liste des arrets de sance corrspondant aux critres de recherche
     * @param requete la requte de l'utilisateur
     * @param listIdsArretes la liste des arretes indexs correspondant aux critres de recherche
     * @param listSeances la liste des sances pour les fiches de sance
     * @param plugin le plugin
     * @return une liste d'arretes
     */
    List<Arrete<GSeance, GFichier>> getArretes( GRequeteUtilisateur requete, List<String> listIdsArretes,
        List<GSeance> listSeances, Plugin plugin );

    /**
     * Retourne la liste des dlibration de dsignation de sance corrspondant aux critres de recherche
     * @param requete la requte de l'utilisateur
     * @param listIdsDeliberationsDesignation la liste des dlibration de dsigantion indexes correspondant aux critres de recherche
     * @param listSeances la liste des sances pour les fiches de sance
     * @param plugin le plugin
     * @return une liste de dlibration de dsignation
     */
    List<DeliberationDesignation<GSeance, GFichier>> getDeliberationsDesignation( GRequeteUtilisateur requete,
        List<String> listIdsDeliberationsDesignation, List<GSeance> listSeances, Plugin plugin );

    //////////////////////////////////////////////////////////////////////////////////////////////////
    //																								//
    //											Sances												//
    //																								//
    //////////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * Retourne la liste des sances comprises entre les 2 dates de la requte
     * @param requete la requete utilisateur
     * @param plugin le plugin
     * @return la liste des sances
     */
    List<GSeance> getSeances( GRequeteUtilisateur requete, Plugin plugin );
}
