/*
 * Copyright (c) 2002-2008, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.search;

import fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;

import java.util.List;


public interface IIndexedDocSearchService<GSeance extends ISeance, GSeanceFilter extends ISeanceFilter, GRequeteUtilisateur extends IRequeteUtilisateur<GSeance, GSeanceFilter>>
{
    /**
     * Retourne la liste des ids de projets retrouvs dans l'index avec le mot-cl recherch
     * @param requete la requte utilisateur
     * @return une liste d'ids de projets
     */
     List<String> getListeIdsProjets( GRequeteUtilisateur requete );

    /**
     * Retourne la liste des ids de propositions retrouvs dans l'index avec le mot-cl recherch
     * @param requete la requte utilisateur
     * @return une liste d'ids de propositions
     */
     List<String> getListeIdsPropositions( GRequeteUtilisateur requete );

    /**
     * Retourne la liste des ids d'amendements retrouvs dans l'index avec le mot-cl recherch
     * @param requete la requte utilisateur
     * @return une liste d'ids dd'amendements
     */
     List<String> getListeIdsAmendements( GRequeteUtilisateur requete );

    /**
     * Retourne la liste des ids de voeux retrouvs dans l'index avec le mot-cl recherch
     * @param requete la requte utilisateur
     * @return une liste d'ids de voeux
     */
     List<String> getListeIdsVoeux( GRequeteUtilisateur requete );

    /**
     * Retourne la liste des ids de fichiers rattachs  un pdd retrouvs dans l'index avec le mot-cl recherch
     * @param requete la requte utilisateur
     * @param nIdPdd l'id du pdd contenant les fichiers recherchs
     * @return une liste d'ids de fichiers rattachs  un pdd
     */
     List<String> getListeIdsFichiersPdd( GRequeteUtilisateur requete, int nIdPdd );

    /**
     * Retourne la liste des ids de fichiers de sance retrouvs dans l'index avec le mot-cl recherch
     * @param requete la requte utilisateur
     * @return une liste d'ids de fichiers de sance
     */
     List<String> getListeIdsAutresFichiers( GRequeteUtilisateur requete );

    /**
     * Retourne la liste des ids d'arrets retrouvs dans l'index avec le mot-cl recherch
     * @param requete la requte utilisateur
     * @return une liste d'ids d'arrts
     */
     List<String> getListeIdsArretes( GRequeteUtilisateur requete );

    /**
     * Retourne la liste des ids de dlibration de dsignation retrouves dans l'index avec le mot-cl recherch
     * @param requete la requte utilisateur
     * @return une liste d'ids de dlibration de dsignation
     */
     List<String> getListeIdsDeliberationsDesignation( GRequeteUtilisateur requete );
}
