/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.search;

import fr.paris.lutece.portal.service.message.SiteMessageException;

import org.apache.lucene.document.Document;
import org.apache.lucene.index.IndexWriter;

import java.io.IOException;

import java.util.List;


public interface IODSSearchIndexer
{
    /**
     * Retourne tous les documents  indexer, autrement dit la liste des documents pour l'initialisation de l'index<br/>
     * Par dfaut cette mthode renvoie les documents de la prochaine sance du conseil
     * @param indexWriter l'index
     * @throws IOException exception hrite de fr.paris.lutece.portal.service.search.SearchIndexer
     * @throws InterruptedException exception hrite de fr.paris.lutece.portal.service.search.SearchIndexer
     */
    void getDocuments( IndexWriter indexWriter ) throws IOException, InterruptedException;

    /**
     * Retourne tous les documents  indexer pour une sance
     * @param nIdAncienneSeance l'id de la dernire sance passe
     * @param indexWriter l'index
     */
    void getDocumentsAArchiver( int nIdAncienneSeance, IndexWriter indexWriter );

    /**
     * Retourne la liste des documents supprims ou modifis  supprimer de l'index
     * @return une liste de documents
     */
    List<Document> getDocumentsToRemove(  );

    /**
     * Indexe la liste des documents ajouts ou modifis  ajouter dans l'index
     * @param indexWriter l'index
     */
    void getDocumentsToIndex( IndexWriter indexWriter );

    /**
     * Retourne la description de l'indexer
     * @return la description
     */
    String getDescription(  );

    /**
     * Retourne le nom de l'indexer
     * @return le nom de l'indexer
     */
    String getName(  );

    /**
     * Retourne la version de l'indexer
     * @return la version de l'indexer
     */
    String getVersion(  );

    /**
     * Indique si l'indexer est actif ou non
     * @return <b>true</b> si l'indexer est actif, <b>false</b> sinon
     */
    boolean isEnable(  );

    /**
     * Rcupre la liste des documents
     * @return la liste des documents
     * @throws IOException IOException
     * @throws InterruptedException InterruptedException
     * @throws SiteMessageException SiteMessageException
     */
    List<Document> getDocuments(  ) throws IOException, InterruptedException, SiteMessageException;

    List<Document> getDocuments( String arg0 ) throws IOException, InterruptedException, SiteMessageException;

    void indexDocuments(  ) throws IOException, InterruptedException, SiteMessageException;
}
