/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.search;

import fr.paris.lutece.plugins.ods.dto.IBusinessItem;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur;
import fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.service.IODSService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;

import java.util.List;


public interface ISearchService<GSeance extends ISeance, GSeanceFilter extends ISeanceFilter, GRequeteUtilisateur extends IRequeteUtilisateur<GSeance, GSeanceFilter>, GResultatRequete extends IResultatRequete<GSeance, GSeanceFilter, GRequeteUtilisateur, GElu, GFichier, GVoeuAmendement, GPDD>, GElu extends IElu, GFichier extends IFichier<GSeance, GFichier>, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>>
    extends IODSService
{
    /**
     * Pour chaque indexer, cette mthode insre une nouvelle action de cration dans l'index
     * @param item l'objet  indexer
     * @param bArchive indique dans quel index il faut ajouter l'objet
     * @param plugin le plugin
     */
    void addObjectToIndex( IBusinessItem item, boolean bArchive, Plugin plugin );

    /**
     * Pour chaque indexer, cette mthode insre une nouvelle action de modification de l'index
     * @param object l'objet  modifier dans l'index
     * @param bArchive indique dans quel index il faut modifier l'objet
     * @param plugin le plugin
     */
    void updateObjectInIndex( IBusinessItem object, boolean bArchive, Plugin plugin );

    /**
     * Pour chaque indexer, cette mthode insre une nouvelle action de suppression dans l'index
     * @param item l'objet  supprimer de l'index
     * @param bArchive indique dans quel index il faut supprimer l'objet
     * @param plugin plugin
     */
    void deleteObjectFromIndex( IBusinessItem object, boolean bArchive, Plugin plugin );

    /**
     * Retourne la liste des objets ajouts depuis la dernire indexation
     * @param bArchive indique sur quel index oprer (archive ou prochaine sance)
     * @param plugin le plugin
     * @return la liste de fichiers ajouts depuis la dernire indexation
     * @throws  AppException  AppException
     */
    List<IBusinessItem> getAddedObjects( boolean bArchive, Plugin plugin )
        throws AppException;

    /**
     * Retourne la liste des objets modifis depuis la dernire indexation
     * @param bDelete indique si l'on doit supprimer l'action aprs l'avoir lue
     * @param bArchive indique sur quel index oprer (archive ou prochaine sance)
     * @param plugin le plugin
     * @return la liste des objets modifis depuis la dernire indexation
     * @throws AppException AppException
     */
    List<IBusinessItem> getUpdatedObjects( boolean bDelete, boolean bArchive, Plugin plugin )
        throws AppException;

    /**
     * Retourne la liste des objets supprims depuis la dernire indexation
     * @param bArchive indique sur quel index oprer (archive ou prochaine sance)
     * @param plugin le plugin
     * @return la liste des objets supprims depuis la dernire indexation
     * @throws AppException AppException
     */
    List<IBusinessItem> getDeletedObjects( boolean bArchive, Plugin plugin )
        throws AppException;

    /**
     * Retourne la liste des PDD recherchs par rfrence
     * @param requete la requte utilisateur
     * @param prochaineSeance la prochaine sance
     * @param plugin le plugin
     * @return la liste des PDD dont la rfrence contient la chane passe dans la requte
     */
    GResultatRequete getPDDByReference( GRequeteUtilisateur requete, GSeance prochaineSeance, Plugin plugin );

    /**
     * Retourne le rsultat de la recherche "simple", c'est  dire  partir du bandeau haut.<br>
     * Les tests de validit de la requte ont dj t effectus, ainsi on sait qu'il y a au moins un mot-cl
     * @param requete la requte de l'utilisateur
     * @param prochaineSeance la prochaine sance
     * @param bIsLimited true si le nombre est limit, false sinon
     * @param plugin le plugin
     * @return le rsultat de la requte
     */
    GResultatRequete getIndexedObjects( GRequeteUtilisateur requete, GSeance prochaineSeance, boolean bIsLimited,
        Plugin plugin );

    /**
     * Retourne le rsultat de la recherche multi-critres
     * @param requete la requte de l'utilisateur
     * @param prochaineSeance la prochaine sance
     * @param bIsLimited true si le nombre est limit, false sinon
     * @param plugin le plugin
     * @return le rsultat de la recherche
     */
    GResultatRequete getAllObjectsByCriterias( GRequeteUtilisateur requete, GSeance prochaineSeance,
        boolean bIsLimited, Plugin plugin );
}
