/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.search;

import fr.paris.lutece.plugins.ods.business.indexer.IndexerHome;
import fr.paris.lutece.plugins.ods.business.indexer.action.IndexerActionHome;
import fr.paris.lutece.plugins.ods.business.seance.ISeanceHome;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.indexer.Indexer;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.daemon.Daemon;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.plugin.PluginService;
import fr.paris.lutece.portal.service.spring.SpringContextService;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.service.util.AppPropertiesService;

import java.util.ArrayList;
import java.util.List;


/**
 * Daemon d'indexation: lance  intervalle rgulier l'indexation incrmentale.
 * A la premire xcution le daemon cre l'index sur le serveur; cet index sera unique par machine serveur
 * mme en cas de plusieurs serveurs Tomcat sur une seule et mme machine.
 *
 */
public class SearchIndexerDaemon<GSeance extends ISeance, GSeanceFilter extends ISeanceFilter, GElu extends IElu, GFichier extends IFichier<GSeance, GFichier>, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>>
    extends Daemon
{
    protected static final String PROPERTY_INDEXER_NAME = "ods.searchindexer.name";
    protected int _nIdSeance = -1;
    protected IndexerHome _indexerHome = ( (IndexerHome) SpringContextService.getPluginBean( OdsConstants.CONSTANTE_PLUGIN_ODS,
            "indexerHome" ) );
    protected IndexerActionHome _indexerActionHome = ( (IndexerActionHome) SpringContextService.getPluginBean( OdsConstants.CONSTANTE_PLUGIN_ODS,
            "indexerActionHome" ) );
    private ISeanceHome<GSeance, GSeanceFilter, GFichier, GElu, GVoeuAmendement, GPDD> _seanceHome = ( (ISeanceHome<GSeance, GSeanceFilter, GFichier, GElu, GVoeuAmendement, GPDD>) SpringContextService.getPluginBean( OdsConstants.CONSTANTE_PLUGIN_ODS,
            "seanceHome" ) );

    /**
     * Lance l'indexation  intervalle rgulier
     */
    public void run(  )
    {
        Plugin plugin = PluginService.getPlugin( OdsConstants.CONSTANTE_PLUGIN_ODS );

        List<GSeance> listSeancesPassees = _seanceHome.findOldSeance( plugin );
        List<GSeance> listSeancesFutures = _seanceHome.findSeanceList( plugin );
        List<GSeance> listAllSeances = new ArrayList<GSeance>(  );
        ISeance prochaineSeance = _seanceHome.getProchaineSeance( plugin );
        String strIndexerName = AppPropertiesService.getProperty( PROPERTY_INDEXER_NAME );

        if ( ( listSeancesPassees != null ) && !listSeancesPassees.isEmpty(  ) )
        {
            listAllSeances.addAll( listSeancesPassees );
        }

        if ( ( listSeancesFutures != null ) && !listSeancesFutures.isEmpty(  ) )
        {
            listAllSeances.addAll( listSeancesFutures );
        }

        if ( ( !listAllSeances.isEmpty(  ) ) && ( strIndexerName != null ) &&
                ( strIndexerName != OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            Indexer indexer = _indexerHome.findByName( strIndexerName, plugin );

            if ( indexer != null )
            {
                _nIdSeance = indexer.getIdSeance(  );
            }

            if ( ( indexer == null ) || ( ( _nIdSeance == -1 ) && ( prochaineSeance != null ) ) ||
                    indexer.isIndexationComplete(  ) )
            {
                // indexer == null ==> la webapp n'a jamais t dmarre
                // indexer.getIdSeance == -1 && prochaineSeance != null ==> l'indexer n'a jamais t lanc et il existe une prochaine sance
                // indexer.isIndexationComplete ==> demande explicite de la part des administrateurs via l'interface
                _indexerActionHome.removeForIndexer( _indexerHome.findByName( strIndexerName, plugin ), plugin );

                if ( indexer == null )
                {
                    // inscription dans la liste des indexer ecrivant sur l'index + init
                    indexer = new Indexer(  );
                    indexer.setNomIndexer( strIndexerName );
                    indexer.setIndexationComplete( false );

                    if ( prochaineSeance != null )
                    {
                        indexer.setIdSeance( prochaineSeance.getIdSeance(  ) );
                    }

                    _indexerHome.create( indexer, plugin );
                }

                if ( prochaineSeance != null )
                {
                    IndexationServiceProchaineSeance.getInstance(  ).initIndex(  );
                    indexer.setIdSeance( prochaineSeance.getIdSeance(  ) );
                    _indexerHome.update( indexer, plugin );
                }

                if ( ( listSeancesPassees != null ) && !listSeancesPassees.isEmpty(  ) )
                {
                    IndexationServiceArchive.getInstance(  ).initIndex(  );

                    if ( prochaineSeance == null )
                    {
                        indexer.setIdSeance( ( _seanceHome.getDerniereSeance( plugin ) ).getIdSeance(  ) );
                        _indexerHome.update( indexer, plugin );
                    }
                }

                indexer.setIndexationComplete( false );
                _indexerHome.update( indexer, plugin );
            }
            else
            {
                if ( prochaineSeance == null )
                {
                    if ( _nIdSeance != -1 )
                    {
                        changeSeance( null, indexer, plugin );
                    }
                }
                else
                {
                    if ( prochaineSeance.getIdSeance(  ) != _nIdSeance )
                    {
                        changeSeance( prochaineSeance, indexer, plugin );
                    }
                    else
                    {
                        try
                        {
                            IndexationServiceProchaineSeance.getInstance(  ).processIndexing(  );
                        }
                        catch ( AppException ae )
                        {
                            AppLogService.error( ae.getMessage(  ) );
                        }
                    }
                }

                if ( ( listSeancesPassees != null ) && !listSeancesPassees.isEmpty(  ) )
                {
                    try
                    {
                        IndexationServiceArchive.getInstance(  ).processIndexing(  );
                    }
                    catch ( AppException ae )
                    {
                        AppLogService.error( ae.getMessage(  ) );
                    }
                }
            }
        }
    }

    /**
     * Fonction qui ajoute les documents de la sance passe  l'index des archives,
     * et rinitialise l'index prochaine sance
     * @param prochaineSeance la prochaine sance
     * @param indexer l'indexer
     * @param plugin le plugin
     */
    protected void changeSeance( ISeance prochaineSeance, Indexer indexer, Plugin plugin )
    {
        _indexerActionHome.removeForIndexer( indexer, plugin );

        if ( prochaineSeance != null )
        {
            try
            {
                IndexationServiceProchaineSeance.getInstance(  ).initIndex(  );
            }
            catch ( AppException ae )
            {
                AppLogService.error( ae.getMessage(  ) );
            }
        }
        else
        {
            IndexationServiceProchaineSeance.getInstance(  ).viderIndex(  );
        }

        try
        {
            IndexationServiceArchive.getInstance(  ).changeSeance( _nIdSeance );
        }
        catch ( AppException ae )
        {
            AppLogService.error( ae.getMessage(  ) );
        }

        if ( prochaineSeance != null )
        {
            _nIdSeance = prochaineSeance.getIdSeance(  );
            indexer.setIdSeance( _nIdSeance );
            _indexerHome.update( indexer, plugin );
        }
    }
}
