/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.textelibre;

import fr.paris.lutece.plugins.ods.business.textelibre.TexteLibreHome;
import fr.paris.lutece.plugins.ods.dto.IBusinessItem;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.textelibre.TexteLibre;
import fr.paris.lutece.plugins.ods.dto.typeentite.ITypeEntite;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.plugin.Plugin;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;
import java.util.List;
import java.util.Locale;


public abstract class AbstractTexteLibreService<GOrdreDuJour extends IOrdreDuJour<GEntreeOrdreDuJour, GSeance, GFichier, GElu, GOrdreDuJour>, GEntreeOrdreDuJour extends IEntreeOrdreDuJour<GElu, GSeance, GFichier, GEntreeOrdreDuJour, GOrdreDuJour>, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>, GSeance extends ISeance, GElu extends IElu, GFichier extends IFichier<GSeance, GFichier>, GPdd extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>>
    implements ITexteLibreService<GOrdreDuJour, GEntreeOrdreDuJour, GVoeuAmendement, GSeance, GElu, GFichier, GPdd>
{
    private static final String CONSTANTE_AJOUTER_TEXTE = "ajouter_texte";
    private static final String PROPERTY_LIBELLE_CREATION_ENTREE_ODJ = "ods.odj.modificationOdj.select.value.texte";
    private static final String PROPERTY_TITLE_CREATION_ENTREE_ODJ = "ods.odj.creation.label.creationEntreeTexte";
    private static final String PROPERTY_TITLE_MODIFICATION_ENTREE_ODJ = "ods.odj.modification.label.creationEntreeTexte";
    @Autowired
    private TexteLibreHome _texteLibreHome;

    protected abstract Plugin getPlugin(  );

    /**
         *
         */
    public String getTypeEntree(  )
    {
        return OdsConstants.CONSTANTE_ENTREE_TYPE_TEXTE;
    }

    /**
     *
     */
    public boolean isForCsv(  )
    {
        return false;
    }

    /**
     *
     */
    public boolean isOtherDataEnabled(  )
    {
        return false;
    }

    /**
     *
     */
    public boolean isRapporteurEnabled(  )
    {
        return false;
    }

    /**
     *
     */
    public boolean isForModeClassement(  )
    {
        return false;
    }

    /**
    * test si l'ensemble des parametres contenus dans l'objet EntreeOrdreDujour
    * en session permettent une insertion ou une modification de l'entre en
    * cours de traitement.
    *
    * @param listErreurs
    *            la liste des erreurs
    * @return true si aucune erreur n'est constat
    */
    public boolean isFieldRequired( GEntreeOrdreDuJour entree, List<String> listErreurs )
    {
        boolean bIsFieldRequired = true;

        if ( ( entree.getObjet(  ) == null ) ||
                entree.getObjet(  ).trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            listErreurs.add( PROPERTY_LIBELLE_CREATION_ENTREE_ODJ );
            bIsFieldRequired = false;
        }

        return bIsFieldRequired;
    }

    /**
     *
     */
    public boolean isForPositionNewEntree( GEntreeOrdreDuJour entree )
    {
        if ( entree.getIdTypeEntite(  ) != ITypeEntite.TEXTE_LIBRE )
        {
            return true;
        }

        return false;
    }

    /**
     *
     */
    public boolean bIsNoEntitySelected( GEntreeOrdreDuJour entree )
    {
        return false;
    }

    /**
     *
     */
    public String getErrorNoEntitySelected(  )
    {
        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     *
     */
    public void validateNumerotation( GEntreeOrdreDuJour entree )
    {
        // Aucune action
    }

    /**
     *
     */
    public String getIdParameter(  )
    {
        return null;
    }

    /**
     *
     */
    public ISeance getSeance( IBusinessItem entite )
    {
        return null;
    }

    /**
     *
     */
    public String getAddParameter(  )
    {
        return CONSTANTE_AJOUTER_TEXTE;
    }

    /**
     *
     */
    public String getPublicationOrdreDuJour( GEntreeOrdreDuJour entree, GOrdreDuJour ordreDuJour )
    {
        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    public int createEntiteForODJ( GEntreeOrdreDuJour entree )
    {
        Plugin plugin = getPlugin(  );

        TexteLibre texteLibre = new TexteLibre(  );
        texteLibre.setLibelle( entree.getObjet(  ) );

        return _texteLibreHome.create( texteLibre, plugin );
    }

    public void deleteEntiteForODJ( int nIdEntite )
    {
        Plugin plugin = getPlugin(  );

        TexteLibre texteLibre = new TexteLibre(  );
        texteLibre.setId( nIdEntite );

        _texteLibreHome.remove( texteLibre, plugin );
    }

    public void updateEntiteForODJ( GEntreeOrdreDuJour entree )
    {
        Plugin plugin = getPlugin(  );

        TexteLibre texteLibre = new TexteLibre(  );
        texteLibre.setId( entree.getIdEntite(  ) );
        texteLibre.setLibelle( entree.getObjet(  ) );

        _texteLibreHome.update( texteLibre, plugin );
    }

    public String getLibelleCreationEntree( Locale locale )
    {
        return I18nService.getLocalizedString( PROPERTY_LIBELLE_CREATION_ENTREE_ODJ, locale );
    }

    public String getPageTitleCreationEntreeODJ(  )
    {
        return PROPERTY_TITLE_CREATION_ENTREE_ODJ;
    }

    public String getPageTitleModificationEntreeODJ(  )
    {
        return PROPERTY_TITLE_MODIFICATION_ENTREE_ODJ;
    }

    public void initEntreeOrdreDuJourSpec( HashMap<String, Object> model, Locale locale )
    {
        //Aucune Action		
    }
}
