/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.utilisateur;

import fr.paris.lutece.plugins.ods.dto.utilisateur.Utilisateur;
import fr.paris.lutece.plugins.ods.service.IODSService;
import fr.paris.lutece.portal.service.plugin.Plugin;

import java.util.Collection;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;


public interface IUtilisateurService extends IODSService
{
    /**
    * Effectue les traitements pour la gestion des utilisateurs sur le front
    * @param model la table de hashage qui va tre envoy  la template freemarker
    * @param request la requte HTTP
    * @param plugin le plugin
    */
    void gestionUtilisateur( Map<String, Object> model, HttpServletRequest request, Plugin plugin );

    /**
     * Retourne le nom de l'utilisateur connect
     * @param request la requete
     * @return le nom de l'utilisateur connect
     */
    String getGivenName( HttpServletRequest request );

    /**
     * Rcupre l'utilisateur courant et le cre si il n'existe pas.
     * @param request la requte Http
     * @param plugin la Plugin
     * @return l'utilisateur courant
     */
    Utilisateur getUtilisateurCourant( HttpServletRequest request, Plugin plugin );

    /**
     * retourne la liste des utilisateurs du front
     * @param plugin plugin
     * @return la liste des utilisateurs
     */
    Collection<Utilisateur> getUserFrontList( Plugin plugin );
}
