/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.voeuamendement;

import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.service.IODSService;
import fr.paris.lutece.plugins.ods.service.acte.IActeServiceManaged;
import fr.paris.lutece.plugins.ods.service.certificataffichage.ICertificatAffichageServiceManaged;
import fr.paris.lutece.plugins.ods.service.horodatage.IHorodatageServiceManaged;
import fr.paris.lutece.plugins.ods.service.ordredujour.IOrdreDuJourServiceManaged;
import fr.paris.lutece.portal.business.user.AdminUser;
import fr.paris.lutece.portal.service.plugin.Plugin;

import java.util.HashMap;
import java.util.List;

import javax.servlet.http.HttpServletRequest;


/**
 * IVoeuAmendementService
 */
public interface IVoeuAmendementService<GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>, GOrdreDuJour extends IOrdreDuJour<GEntreeOrdreDuJour, GSeance, GFichier, GElu, GOrdreDuJour>, GEntreeOrdreDuJour extends IEntreeOrdreDuJour<GElu, GSeance, GFichier, GEntreeOrdreDuJour, GOrdreDuJour>, GSeance extends ISeance, GElu extends IElu, GFichier extends IFichier<GSeance, GFichier>, GPdd extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>>
    extends IODSService, IActeServiceManaged<GSeance, GFichier>, IHorodatageServiceManaged,
        ICertificatAffichageServiceManaged<GSeance, GFichier>,
        IOrdreDuJourServiceManaged<GEntreeOrdreDuJour, GOrdreDuJour, GVoeuAmendement, GSeance, GElu, GFichier, GPdd>
{
    /**
     * Retourne l'URL de retour pour un voeu rattach
     * @return l'URL de retour pour un voeu rattach
     */
    String getURLRetourVoeuRattache(  );

    /**
     * Retourne l'URL de retour pour un amendement
     * @return l'URL de retour pour un amendement
     */
    String getURLRetourAmendement(  );

    /**
    * Appel  la mthode init() de PluginAdminPageJspBean (initialise les
    * variables de session si elles n'ont pas t auparavant initialises)
    * @param request Requte HTTP
    * @param bIsGestionAval TRUE si gestion aval, FALSE sinon
    */
    void init( HttpServletRequest request, boolean bIsGestionAval );

    /**
         * Rinitialise tous les attributs de session
         * @param request Requte HTTP
         * @param bIsGestionAval TRUE si gestion aval, FALSE sinon
         */
    void reinitSession( HttpServletRequest request, boolean bIsGestionAval );

    /**
     * Permet d'initialiser l'url de retour
     * @param strUrl url de retour
     * @param request la requte http
     */
    void initUrlRetour( String strUrl, HttpServletRequest request );

    /**
     * Retourne true si le VA est publi, false sinon
     * @param nIdVA l'id du VA
     * @return true si le VA est publi, false sinon
     */
    boolean isPublier( int nIdVA );

    /**
     * Retourne le type du va en session
     * @return le type du va en session
     */
    String getTypeVa(  );

    /**
     * @return _bIsGestionAval boolean
     */
    boolean isGestionAval(  );

    /**
     * @return _b boolean
     */
    boolean isUrLRetour(  );

    /**
     * affecte la variable bUrlRetour retour
     * @param bUrlRetour =true retour a l'interface de gestion de jsp _strUrlRetour
     */
    void setBurlRetour( boolean bUrlRetour );

    /**
     * affecte  l'url de retour
     * @param strUrl url de retour
     */
    void setUrlRetour( String strUrl );

    /**
     * retourne l'url de retour
     * @return l'url de retour
     */
    String getUrlRetour(  );

    /**
     * Retourne l'interface de gestion des amendements, des voeux rattachs
     * ou des voeux non rattachs en fonction du type demand
     * @param request Requte HTTP
     * @param strType (amendement, voeu rattach, voeu non rattach)
     * @return Interface de gestion des VA
     */
    HashMap<String, Object> getVoeuAmendementList( HttpServletRequest request, String strType, String strReturnDefault );

    /**
     * retourne la map contenant les lments utiles au template HTML de la liste
     * @param request la requete HTTP
     * @param strType (amendement,voeu rattache,voeu non rattache)
     * @param strUrlJSP l'url du JSP
     * @param strUrlRetour l'url de retour
     * @return model la map contenant les lments utiles au template HTML de la liste
     */
    HashMap<String, Object> getVoeuAmendementList( HttpServletRequest request, String strType, String strUrlJSP,
        String strUrlRetour );

    /**
     * Retourne la map contenant les lments utiles au template HTML de la liste
     * @param request la requete
     * @param strType le type de VA
     * @param doReinitVA doit-on reinitialiser le VA en session
     * @return model la map contenant les lments utiles au template HTML de la liste
     */
    HashMap<String, Object> getCreationVoeuAmendement( HttpServletRequest request, String strType, boolean doReinitVA );

    /**
     * Retourne la map contenant les lments utiles au template HTML de modification
     * @param request request
     * @param strReturnDefault l'URL de retour par dfaut
     * @return model la map contenant les lments utiles au template HTML de modification
     */
    HashMap<String, Object> getModificationVoeuAmendement( HttpServletRequest request, String strReturnDefault );

    /**
     * retourne une demande pour la suppression du VA choisi
     * @param request la requete HTTP
     * @return interface de confirmation de suppression
     */
    String getSuppressionVoeuAmendement( HttpServletRequest request );

    /**
     * retourne un avertissement pour signaler que le VA n'est associ  aucun PDD
     * @param request la requete HTTP
     * @param strReturnDefault l'URL de retour par dfaut
     * @return interface de confirmation de suppression
     */
    String getAvertissementEnregistrementSansPDD( HttpServletRequest request, String strReturnDefault );

    /**
     * retourne la map contenant les lments utiles au template HTML de la liste pour l'ODJ
     * @param request la requete HTTP
     * @return model la map contenant les lments utiles au template HTML de la liste pour l'ODJ
     */
    HashMap<String, Object> getVoeuListForOdj( HttpServletRequest request );

    /**
     * retourne la map contenant les lments utiles au template HTML de l'historique
     * @param request la requete
     * @return model la map contenant les lments utiles au template HTML de l'historique
     */
    HashMap<String, Object> getHistoriqueVoeuAmendement( HttpServletRequest request );

    /**
     * retourne la map contenant les lments utiles au template HTML de confirmation de finalisation
     * @param request la requte http
     * @return model la map contenant les lments utiles au template HTML de confirmation de finalisation
     */
    HashMap<String, Object> getConfirmFinaliser( HttpServletRequest request );

    /**
     * Methode qui effectue la creation d'un amendement, d'un voeu rattach ou d'un voeu non rattach
     * en fonction du type de cration demand.
     * Redirection vers l'interface d'administration(amendement,voeu rattach ou voeu non rattach)
     * @param request request
     * @param strType type de cration demand
     * @param strReturnDefault l'URL de retour par dfaut
     * @return url l'url
     */
    String doCreationVoeuAmendement( HttpServletRequest request, String strType, String strReturnDefault );

    /**
     * Methode qui effectue la modification d'un amendement, d'un voeu rattach
     * en fonction du type du VoeuAmendement prsent en session.
     * Redirection vers l'interface d'administration(amendement,voeu non rattach,voeu rattach)
     * @param request request
     * @param strReturnDefault l'URL de retour par dfaut
     * @param user l'utilisateur courant
     * @return template
     */
    String doModificationVoeuAmendement( HttpServletRequest request, String strReturnDefault, AdminUser user );

    /**
     * Methode qui effectue la modification d'un amendement, d'un voeu rattach
     * en fonction du type du VoeuAmendement prsent en session.
     * Redirection vers l'interface d'administration des statuts VA
     * @param request request
     * @param strReturnDefault l'URL de retour par dfaut
     * @return template
     */
    String doModificationStatutVA( HttpServletRequest request, String strReturnDefault );

    /**
     * supprime le voeuAmendement choisi
     * @param request la requete HTTP
     * @param strReturnDefault l'URL de retour par dfaut
     * @return url de gestion des voeux amendements
     */
    String doSuppressionVoeuAmendement( HttpServletRequest request, String strReturnDefault );

    /**
     * mthode qui gre la publication du voeu amendement
     * @param request request
     * @param strType le type de VA
     * @param strReturnDefault l'URL de retour par dfaut
     * @return interface d'administration
     */
    String doPublicationVoeuAmendement( HttpServletRequest request, String strType, String strReturnDefault );

    /**
     * methode qui ajoute un dpositaire dans la liste des dpositaires du VA .
     * Retour sur l'interface de cration d'un amendement,voeu non rattach ou
     * voeu rattach en fonction du type du VA
     * @param request la requete
     * @param strReturnDefault l'URL de retour par dfaut
     * @return url l'interface de cration d'un va
     */
    String doAddDepositaireCreation( HttpServletRequest request, String strReturnDefault );

    /**
     * methode qui ajoute un dpositaire dans la liste des dpositaires du VA .
     * Retour sur l'interface de cration d'un amendement,voeu non rattach ou
     * voeu rattach en fonction du type du VA
     * @param request la requete
     * @param strReturnDefault l'URL de retour par dfaut
     * @return l'url de modification d'un va
     */
    String doAddDepositaireModification( HttpServletRequest request, String strReturnDefault );

    /**
     * methode qui supprime  un dpositaire  en dans la liste des dpositaires du VA .
     * Retour sur l'interface de cration d'un amendement,voeu non rattach ou
     * voeu rattach en fonction du type du VA
     * @param request la requete
     * @param strReturnDefault l'URL de retour par dfaut
     * @return url de l'interface de cration d'un VA
     */
    String doRemoveDepositaireCreation( HttpServletRequest request, String strReturnDefault );

    /**
     * methode qui supprime  un dpositaire en dans la liste des dpositaires du VA .
     * Retour sur l'interface de modification d'un amendement,voeu non rattach ou
     * voeu rattach en fonction du type du VA
     * @param request la requete
     * @param strReturnDefault l'URL de retour par dfaut
     * @return l'url de modification d'un va
     */
    String doRemoveDepositaireModification( HttpServletRequest request, String strReturnDefault );

    /**
     * methode qui ajoutes  les  Pdds selectionns par l'utilisateur dans la liste des pdds du VA .
     * Retour sur l'interface de modification d'un amendement,ou
     * voeu rattach en fonction du type du VA
     * @param request la requete
     * @param strReturnDefault l'URL de retour par dfaut
     * @return l'url de modification d'un VA
     */
    String doAddPdds( HttpServletRequest request, String strReturnDefault );

    /**
     * methode qui supprime  un Pdd dans la liste des pdds du VA .
     * Retour sur l'interface de modification d'un amendement,ou
     * voeu rattach en fonction du type du VA
     * @param request la requete
     * @param strReturnDefault l'URL de retour par dfaut
     * @return l'url de modificaion d'un VA
     */
    String doRemovePdd( HttpServletRequest request, String strReturnDefault );

    /**
     * rinitialise le voeuAmendement en session
     * @param request la requete
     * @param strReturnDefault l'URL de retour par dfaut
     * @return l'url en fonction di type du VA
     */
    String doResetVoeuAmendement( HttpServletRequest request, String strReturnDefault );

    /**
     * Modifie le statut selectionn pour lui attribu le statut demand
     * @param request la requte
     * @return l'url de retour
     */
    String doAssignStatut( HttpServletRequest request );

    /**
     * Supprime le parent du voeu/amendement.
     * @param request la requete
     * @param strAction marqueur de l'action  effecuer (ajout ou suppression)
     * @param strReturn marqueur sur la page de retour (cration ou modification)
     * @param strReturnDefault l'URL de retour par dfaut
     * @return la page de cration ou de modification
     */
    String doAddRemoveParent( HttpServletRequest request, String strAction, String strReturn, String strReturnDefault );

    /**
     * Tlecharge un fichier.
     * Renvoie un flux de bytes du fichier dans l'objet HttpServletResponse response
     *
     * @param request la requete
     * @return fichierTelecharger le fichier  tlcharger
     */
    GFichier doDownloadFichier( HttpServletRequest request );

    /**
     * Permet de confirmer la finalisation d'un acte
     *
     * @param request la requete HTTP
     * @param strReturnDefault l'URL de retour par dfaut
     * @return strUrl url de retour
     */
    String doFinaliserActe( HttpServletRequest request, String strReturnDefault );

    /**
    *
    * retourne la liste des voeux amendements descendant du voeu amendement d'id nIdVoeuAmendement
    * @param nIdVoeuAmendement l'id du voeu amendement
    * @param plugin yhe plugin
    * @param listVaDescendant la liste des VA descendants
    */
    void getVoeuxAmendementsDescendant( int nIdVoeuAmendement, Plugin plugin, List<GVoeuAmendement> listVaDescendant );

    /**
     * Permet de savoir si il s'agit d'une demande de slection de PDD pour Voeu/Amendement
     * @param strUrlJspAppelant l'url de la JSP source.
     * @return TRUE, si c'est pour le traitement des Voeux/amendement, FALSE sinon
     */
    boolean estPourSelectionVoeuAmendement( String strUrlJspAppelant );
}
