/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.xpage.accueil;

import fr.paris.lutece.plugins.ods.business.pdd.IPDDHome;
import fr.paris.lutece.plugins.ods.business.seance.ISeanceHome;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.pdd.IArrondissement;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDDFilter;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendementFilter;
import fr.paris.lutece.plugins.ods.service.utilisateur.IUtilisateurService;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.portal.business.role.Role;
import fr.paris.lutece.portal.business.role.RoleHome;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.security.LuteceUser;
import fr.paris.lutece.portal.service.security.SecurityService;
import fr.paris.lutece.portal.service.security.UserNotSignedException;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.service.util.AppPathService;
import fr.paris.lutece.util.ReferenceList;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;


public abstract class AbstractAccueilAppService<GSeance extends ISeance, GSeanceFilter extends ISeanceFilter, GFichier extends IFichier<GSeance, GFichier>, GElu extends IElu, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>, GPdd extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>, GArrondissement extends IArrondissement>
    implements IAccueilAppService
{
    private static final String MARK_URL_PDD_LIST_NEW = "url_pdd_list_new";
    private static final String MARK_VALUE_URL_PDD_LIST_NEW = "ods_pdd_list_new";
    private static final String MARK_URL_CALENDRIER = "url_calendrier";
    private static final String MARK_VALUE_URL_CALENDRIER = "ods_calendrier";
    private static final String MARK_URL_RECHERCHE_RESULTATS = "url_recherche_resultats";
    private static final String MARK_VALUE_URL_RECHERCHE_RESULTATS = "ods_recherche";
    private static final String MARK_URL_RECHERCHE_ARCHIVE_RESULTATS = "url_recherche_archive_resultats";
    private static final String MARK_VALUE_URL_RECHERCHE_ARCHIVE_RESULTATS = "ods_recherche_archives";
    private static final String MARK_URL_CRITERES = "url_mes_criteres";
    private static final String MARK_VALUE_URL_CRITERES = "ods_mes_criteres";
    protected static final String MARK_LISTE_REQUETES = "liste_requetes";
    private static final String MARK_NBR_PROJETS = "nbr_projets";
    private static final String MARK_NBR_PROPOSITIONS = "nbr_propositions";
    private static final String MARK_NBR_NOUVEAUX_PROJETS = "nbr_nouveaux_projets";
    private static final String MARK_NBR_NOUVEAUX_PROPOSITIONS = "nbr_nouveaux_propositions";
    @Autowired
    private IUtilisateurService _utilisateurService;
    @Autowired
    private ISeanceHome<GSeance, GSeanceFilter, GFichier, GElu, GVoeuAmendement, GPdd> _seanceHome;
    @Autowired
    private IPDDHome<GPdd, IPDDFilter, IVoeuAmendementFilter, GSeance, GElu, GVoeuAmendement, GFichier, GArrondissement> _projetDeliberationHome;

    protected abstract void getPageSpec( HashMap<String, Object> model, HttpServletRequest request, int nMode,
        Plugin plugin );

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.xpage.accueil.IAccueilAppService#getPage(javax.servlet.http.HttpServletRequest, int, fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public HashMap<String, Object> getPage( HttpServletRequest request, int nMode, Plugin plugin )
    {
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        // Met  jour la liste des utilisateurs et gre la session de l'utilisateur
        _utilisateurService.gestionUtilisateur( model, request, plugin );

        String strPortalUrl = AppPathService.getPortalUrl(  ) + OdsConstants.CONSTANTE_POINT_INTERROGATION + "page=";

        // On met dans la Hashmap les liens vers les pages accessibles depuis la page d'acceuil
        model.put( MARK_URL_PDD_LIST_NEW, strPortalUrl + MARK_VALUE_URL_PDD_LIST_NEW );
        model.put( MARK_URL_CALENDRIER, strPortalUrl + MARK_VALUE_URL_CALENDRIER );
        model.put( MARK_URL_RECHERCHE_RESULTATS, strPortalUrl + MARK_VALUE_URL_RECHERCHE_RESULTATS );
        model.put( MARK_URL_RECHERCHE_ARCHIVE_RESULTATS, strPortalUrl + MARK_VALUE_URL_RECHERCHE_ARCHIVE_RESULTATS );
        model.put( MARK_URL_CRITERES, strPortalUrl + MARK_VALUE_URL_CRITERES );

        // On rcupre le nombre de projets et du nombre de propositions nouvellements crs
        ISeance seance = _seanceHome.getProchaineSeance( plugin );
        int nNbrProjets = 0;
        int nNbrPropositions = 0;
        int nNbrNouveauxProjets = 0;
        int nNbrNouveauxPropositions = 0;

        // On rcupre le nombre de Projets et de Propositions de dlibration de la prochaine sance
        // et on ajoute ces donnes dans la Hashmap
        if ( seance != null )
        {
            nNbrProjets = getNombrePddsPublies( seance.getIdSeance(  ), true, false, request, plugin );
            nNbrPropositions = getNombrePddsPublies( seance.getIdSeance(  ), false, false, request, plugin );
            nNbrNouveauxProjets = getNombrePddsPublies( seance.getIdSeance(  ), true, true, request, plugin );
            nNbrNouveauxPropositions = getNombrePddsPublies( seance.getIdSeance(  ), false, true, request, plugin );
        }

        model.put( MARK_NBR_PROJETS, nNbrProjets );
        model.put( MARK_NBR_PROPOSITIONS, nNbrPropositions );
        model.put( MARK_NBR_NOUVEAUX_PROJETS, nNbrNouveauxProjets );
        model.put( MARK_NBR_NOUVEAUX_PROPOSITIONS, nNbrNouveauxPropositions );

        ISeance seanceEnCours = _seanceHome.getSeanceEnCours( plugin );

        if ( seanceEnCours != null )
        {
            model.put( OdsMarks.MARK_ID_SEANCE_EN_COURS, seanceEnCours.getIdSeance(  ) );
        }

        if ( SecurityService.isAuthenticationEnable(  ) )
        {
            LuteceUser luteceUser = null;

            try
            {
                luteceUser = SecurityService.getInstance(  ).getRemoteUser( request );
            }
            catch ( UserNotSignedException e )
            {
                AppLogService.error( e.getMessage(  ) );
            }

            if ( luteceUser != null )
            {
                String[] userRoles = luteceUser.getRoles(  );

                ReferenceList refListRoles = new ReferenceList(  );

                if ( ( userRoles != null ) && ( userRoles.length > 0 ) )
                {
                    for ( String userRole : userRoles )
                    {
                        Role role = RoleHome.findByPrimaryKey( userRole );

                        refListRoles.addItem( role.getRole(  ), role.getRoleDescription(  ) );
                    }
                }

                model.put( "workgroups", refListRoles );

                if ( refListRoles.size(  ) == 1 )
                {
                    model.put( "workgroup_disabled", true );

                    //SelectMairieService.doSelectMairie( request, item.getCode() );
                }

                if ( request != null )
                {
                    String strWorkgroupSelected = (String) request.getSession(  ).getAttribute( "user_workgroup" );

                    if ( strWorkgroupSelected != null )
                    {
                        model.put( "selected_workgroup", strWorkgroupSelected.toUpperCase(  ) );
                    }
                }
            }
        }

        getPageSpec( model, request, nMode, plugin );

        return model;
    }

    /**
     * Get the ProjetDeliberationHome
     * @return The ProjetDeliberationHome
     */
    public IPDDHome<GPdd, IPDDFilter, IVoeuAmendementFilter, GSeance, GElu, GVoeuAmendement, GFichier, GArrondissement> getProjetDeliberationHome(  )
    {
        return _projetDeliberationHome;
    }
}
