/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.xpage.panier;

import fr.paris.lutece.plugins.ods.business.arrete.ArreteHome;
import fr.paris.lutece.plugins.ods.business.deliberationdesignation.IDeliberationDesignationHome;
import fr.paris.lutece.plugins.ods.business.fichier.IFichierHome;
import fr.paris.lutece.plugins.ods.business.panier.PanierHome;
import fr.paris.lutece.plugins.ods.business.seance.ISeanceHome;
import fr.paris.lutece.plugins.ods.business.voeuamendement.IVoeuAmendementHome;
import fr.paris.lutece.plugins.ods.dto.IDesignation;
import fr.paris.lutece.plugins.ods.dto.acte.IActeFilter;
import fr.paris.lutece.plugins.ods.dto.arrete.Arrete;
import fr.paris.lutece.plugins.ods.dto.deliberationdesignation.IDeliberationDesignation;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter;
import fr.paris.lutece.plugins.ods.dto.fichier.ITypeDocument;
import fr.paris.lutece.plugins.ods.dto.panier.Panier;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.utilisateur.Utilisateur;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendementFilter;
import fr.paris.lutece.plugins.ods.service.manager.GenerateZipManager;
import fr.paris.lutece.plugins.ods.service.utilisateur.IUtilisateurService;
import fr.paris.lutece.plugins.ods.utils.commons.StringUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.service.util.AppPathService;
import fr.paris.lutece.portal.service.util.AppPropertiesService;
import fr.paris.lutece.portal.web.LocalVariables;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Properties;

import javax.jms.JMSException;
import javax.jms.Queue;
import javax.jms.QueueConnectionFactory;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


public abstract class AbstractPanierAppService<GProduit extends IProduit<GSeance, GFichier, GElu, GVoeuAmendement, GPdd>, GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>, GSeanceFilter extends ISeanceFilter, GElu extends IElu, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>, GPdd extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>, GVoeuAmendementFilter extends IVoeuAmendementFilter, GActeFilter extends IActeFilter<GFichier, GSeance>, GDeliberationDesignation extends IDeliberationDesignation<GSeance, GFichier>>
    implements IPanierAppService
{
    protected static final String PROPERTY_TELECHARGEMENT_SERVLET_PANIER = "ods.telechargement.servlet.panier";
    private static final String PROPERTY_TELECHARGEMENT_COOKIE_DOMAIN = "ods.telechargement.cookie.domain";
    private static final String MARK_TAILLE_TOTAL_MAX = "tailleTotalMax";
    private static final String MARK_FICHIERS_NON_PUBLIES = "fichiers_non_publies";
    private static final String MARK_ERREUR_JMS = "erreur_jms";
    private static final String MARK_LISTE_PRODUIT = "liste_produits";
    private static final String MARK_LISTE_ZIP = "liste_zip";
    private static final String MARK_SELECTION = "selection";
    private static final String MARK_DO_ADD_SELECTION_PANIER = "addSelectionPanier";
    private static final String MARK_DO_DELETE_SELECTION_PANIER = "deleteSelectionPanier";
    private static final String MARK_DO_DOWNLOAD_SELECTION_PANIER = "downloadSelectionPanier";
    private static final String MARK_SELECTION_ELEMENTS_PANIER_TO_DOWNLOAD = "idElementPanier";
    private static final String MARK_URLBASE = "urlBase";
    private static final String MARK_URL_WEBAPP_SERVLET_TELECHARGEMENT = "urlWebappServletTelechargement";
    private static final String MARK_URL_DOWNLOAD_PANIER = "url_download_panier";
    private static final String PROPERTY_TAILLE_TOTAL_MAX = "ods.taille_total_max_telechargement";
    private static final String CONSTANT_XPAGE_CONFIRMATION = "jsp/site/RunStandaloneApp.jsp?page=ods_confirmation_telechargement";
    private static final String CONSTANT_ERROR_MESSAGE_SEPARATOR = " : ";
    private static final int DEFAULT_PROPERTY_TAILLE_TOTAL_MAX = 22020096;
    @Autowired
    private IUtilisateurService _utilisateurService;
    @Autowired
    private ISeanceHome<GSeance, GSeanceFilter, GFichier, GElu, GVoeuAmendement, GPdd> _seanceHome;
    @Autowired
    private IVoeuAmendementHome<GVoeuAmendement, GVoeuAmendementFilter, GSeance, GElu, GFichier, GPdd> _voeuAmendementHome;
    @Autowired
    private IFichierHome<GFichier, IFichierFilter, ITypeDocument, GSeance> _fichierHome;
    @Autowired
    private IDeliberationDesignationHome<GSeance, GFichier, GDeliberationDesignation, GActeFilter> _deliberationDesignationHome;
    @Autowired
    private ArreteHome<GSeance, GFichier> _arreteHome;
    @Autowired
    private PanierHome _panierHome;

    protected abstract GProduit newProduitInstance(  );

    /**
     * Renvoie true si le fichier a t trait, false sinon.
     * @param plugin
     * @param idUser
     * @param produits
     * @return
     */
    protected abstract boolean refreshElementsPanierSpec( Plugin plugin, String idUser, List<GProduit> produits,
        GFichier fichier, GProduit produit );

    /**
     * renvoie la page.
     * @param request le requte http
     * @param nMode le mode
     * @param plugin le Plugin actif
     * @return la page
     */
    public HashMap<String, Object> getPage( HttpServletRequest request, int nMode, Plugin plugin )
    {
        /* Variables de session */
        List<GProduit> produits = new ArrayList<GProduit>(  );
        List<Panier> archives = new ArrayList<Panier>(  );

        HashMap<String, Object> model = new HashMap<String, Object>(  );

        /* On met  jour la liste des utilisateurs et gre la session de l'utilisateur */
        _utilisateurService.gestionUtilisateur( model, request, plugin );

        /* Rcupre l'id de l'utilisateur connect. */
        String idUser = _utilisateurService.getUtilisateurCourant( request, plugin ).getIdUtilisateur(  );

        /* Rcupre les valeurs des paramtres passs  la requte */
        String doWhat = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( request.getParameter( MARK_DO_ADD_SELECTION_PANIER ) != null )
        {
            doWhat = MARK_DO_ADD_SELECTION_PANIER;
        }
        else if ( request.getParameter( MARK_DO_DELETE_SELECTION_PANIER ) != null )
        {
            doWhat = MARK_DO_DELETE_SELECTION_PANIER;
        }
        else if ( request.getParameter( MARK_DO_DOWNLOAD_SELECTION_PANIER ) != null )
        {
            doWhat = MARK_DO_DOWNLOAD_SELECTION_PANIER;
        }

        /* Tableau reprsentant les ids des documents  tlcharger */
        String[] ids = null;

        /* Permet de mettre  jour la liste en session contenant les lments du paniers de l'utilisateur connct */
        refreshElementsPanier( plugin, idUser, produits );

        String dateAjout = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( ( request.getParameter( OdsParameters.DATE_AJOUT ) != null ) &&
                ( request.getParameter( OdsParameters.DATE_AJOUT ).trim(  ).length(  ) > 0 ) )
        {
            dateAjout = request.getParameter( OdsParameters.DATE_AJOUT );
        }
        else
        {
            long lDateAjout = ( new Date(  ) ).getTime(  );
            dateAjout = Long.toString( lDateAjout );
        }

        /* Test pour savoir si l'on doit ajouter ou enlever les lments du panier qui ont t slctionns */
        if ( ( doWhat != null ) && !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( doWhat ) )
        {
            if ( doWhat.equals( MARK_DO_ADD_SELECTION_PANIER ) )
            {
                ids = request.getParameterValues( MARK_SELECTION );
                addSelectionPanier( request, plugin, ids, idUser, produits );
            }
            else if ( doWhat.equals( MARK_DO_DELETE_SELECTION_PANIER ) )
            {
                ids = request.getParameterValues( OdsParameters.ID_ELEMENT_PANIER );
                removeSelectionPanier( plugin, ids, idUser, produits );
            }
            else if ( doWhat.equals( MARK_DO_DOWNLOAD_SELECTION_PANIER ) )
            {
                ids = request.getParameterValues( OdsParameters.ID_ELEMENT_PANIER );
                downloadSelectionPanier( request, dateAjout, plugin );
            }
        }

        /* Permet de mettre  jour la liste en session contenant les archives tlcharges de l'utilisateur connect */
        refreshZips( plugin, idUser, archives );

        /* Cration du cookie pour attester de l'authentification sur le serveur de tlchargement */
        HttpServletResponse response = LocalVariables.getResponse(  );
        Cookie[] cookies = request.getCookies(  );
        String strGuid = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( cookies != null )
        {
            for ( Cookie c : cookies )
            {
                if ( ( c != null ) && ( c.getName(  ) != null ) &&
                        c.getName(  )
                             .equals( AppPropertiesService.getProperty( OdsProperties.PROPERTY_COOKIE_WSSO_NAME_SOURCE ) ) )
                {
                    strGuid = c.getValue(  );

                    break;
                }
            }
        }

        Cookie newCookie = new Cookie( AppPropertiesService.getProperty( 
                    OdsProperties.PROPERTY_COOKIE_WSSO_NAME_DESTINATION ), strGuid );
        newCookie.setDomain( AppPropertiesService.getProperty( PROPERTY_TELECHARGEMENT_COOKIE_DOMAIN ) );
        newCookie.setPath( OdsConstants.CONSTANTE_SLASH );
        response.addCookie( newCookie );

        String url = getUrlDownloadPanier(  );

        model.put( MARK_URL_WEBAPP_SERVLET_TELECHARGEMENT,
            AppPropertiesService.getProperty( OdsProperties.PROPERTY_TELECHARGEMENT_HOST ) +
            OdsConstants.CONSTANTE_SLASH +
            AppPropertiesService.getProperty( OdsProperties.PROPERTY_TELECHARGEMENT_WEBAPP ) );
        model.put( MARK_TAILLE_TOTAL_MAX,
            AppPropertiesService.getPropertyInt( PROPERTY_TAILLE_TOTAL_MAX, DEFAULT_PROPERTY_TAILLE_TOTAL_MAX ) );
        model.put( MARK_LISTE_PRODUIT, produits );
        model.put( MARK_LISTE_ZIP, archives );
        model.put( MARK_URLBASE, AppPathService.getBaseUrl( request ) + CONSTANT_XPAGE_CONFIRMATION );
        model.put( OdsMarks.MARK_URL_WEBAPP, AppPathService.getBaseUrl( request ) );
        model.put( MARK_URL_DOWNLOAD_PANIER, url );

        if ( request.getParameter( MARK_FICHIERS_NON_PUBLIES ) != null )
        {
            model.put( MARK_FICHIERS_NON_PUBLIES, true );
        }

        //On rcupre la sance en cours si elle existe
        ISeance seanceEnCours = _seanceHome.getSeanceEnCours( plugin );

        if ( seanceEnCours != null )
        {
            model.put( OdsMarks.MARK_ID_SEANCE_EN_COURS, seanceEnCours.getIdSeance(  ) );
        }

        return model;
    }

    /**
     * Cette mthode permet de lancer la gnration de l'archive contenant les lments du panier slctionns
     *
     * @param request le requte http
     * @param dateAjout la date d'ajout de l'objet dans le panier sous la forme d'une chaine de caractres
     */
    private void downloadSelectionPanier( HttpServletRequest request, String dateAjout, Plugin plugin )
    {
        /* On rcupre les paramtres envoys dans la requte */
        String[] ids = request.getParameterValues( MARK_SELECTION_ELEMENTS_PANIER_TO_DOWNLOAD );

        /* On rcupre l'utilisateur courant */
        Utilisateur user = _utilisateurService.getUtilisateurCourant( request, plugin );

        if ( !checkJoram(  ) )
        {
            request.getSession(  ).setAttribute( MARK_ERREUR_JMS, "joram" );
        }
        else
        {
            try
            {
                /* Si la liste de fichiers n'est pas nulle ou vide on gnre le zip contenant tous ces fichiers */
                if ( ( ids != null ) && ( ids.length > 0 ) )
                {
                    GFichier fichier;

                    for ( int i = 0; i < ids.length; i++ )
                    {
                        //fichier = null;
                        if ( ids[i] != null )
                        {
                            int nIdFichier = -1;

                            try
                            {
                                nIdFichier = Integer.parseInt( ids[i] );
                            }
                            catch ( NumberFormatException nfe )
                            {
                                AppLogService.error( "downloadSelectionPanier - " + nfe.getClass(  ) +
                                    CONSTANT_ERROR_MESSAGE_SEPARATOR + nfe.getMessage(  ) );
                            }

                            fichier = _fichierHome.findByPrimaryKey( nIdFichier, plugin );

                            if ( ( fichier != null ) && !fichier.getEnLigne(  ) )
                            {
                                break;
                            }
                        }
                    }

                    GenerateZipManager.generateZIP( request, ids, plugin, user.getIdUtilisateur(  ), dateAjout, 1 );
                }
            }
            catch ( NamingException e1 )
            {
                AppLogService.error( e1 );
                request.getSession(  ).setAttribute( MARK_ERREUR_JMS, "naming" );
            }
            catch ( JMSException e1 )
            {
                AppLogService.error( e1 );
                request.getSession(  ).setAttribute( MARK_ERREUR_JMS, "jms" );
            }
        }
    }

    /**
    * Cette mthode permet de rcuprer les zip de l'utilisateur connect
    *
    * @param plugin le Plugin actif
    * @param idUser Id utilisateur connect
     * @param archives les archives
    */
    private void refreshZips( Plugin plugin, String idUser, List<Panier> archives )
    {
        List<Panier> listArchives = _panierHome.findZipByIdUser( idUser, plugin );

        if ( ( listArchives != null ) && !listArchives.isEmpty(  ) )
        {
            archives.addAll( listArchives );
        }
    }

    /**
     *
     * Cette mthode permet de rcuprer les lments du panier de l'utilisateur connect
     *
     * @param plugin le Plugin actif
     * @param idUser Id utilisateur connect
     * @param produits la liste des produits du panier
     */
    @SuppressWarnings( "unchecked" )
    private void refreshElementsPanier( Plugin plugin, String idUser, List<GProduit> produits )
    {
        // On rcupre la liste et le nombre des identifiants de fichiers de l'utilisateur
        String[] ids = _panierHome.findElementPanierByIdUser( idUser, plugin );
        int lIds = ( ids != null ) ? ids.length : 0;

        for ( int i = 0; i < lIds; i++ )
        {
            if ( ( ids[i] != null ) && !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( ids[i] ) )
            {
                GProduit produit = newProduitInstance(  );
                List<GPdd> refPdds = new ArrayList<GPdd>(  );

                // On rcupre le fichier identifi
                int idFichier = -1;

                try
                {
                    idFichier = Integer.parseInt( ids[i] );
                }
                catch ( NumberFormatException nfe )
                {
                    AppLogService.error( "refreshElementsPanier - " + nfe.getClass(  ) +
                        CONSTANT_ERROR_MESSAGE_SEPARATOR + nfe.getMessage(  ) );
                }

                GFichier fichier = _fichierHome.findByPrimaryKey( idFichier, plugin );

                if ( fichier != null )
                {
                    produit.setFichier( fichier );

                    // Si le fichier est un voeu ou un amendement, on recupre la liste des PDDs 
                    // que l'on ajoute au panier en tant que rfrence
                    if ( ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.AMENDEMENT ) ||
                            ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.VOEU ) ||
                            ( ( fichier.getPDD(  ) == null ) &&
                            ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.DELIBERATION ) ) )
                    {
                        GVoeuAmendement voeuAmendement;

                        if ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.DELIBERATION )
                        {
                            voeuAmendement = _voeuAmendementHome.findByDeliberation( fichier, plugin );
                        }
                        else
                        {
                            voeuAmendement = _voeuAmendementHome.findByTexteInitial( fichier, plugin );
                        }

                        if ( voeuAmendement != null )
                        {
                            produit.setVoeuAmendement( voeuAmendement );

                            List<GPdd> pdds = voeuAmendement.getPdds(  );

                            for ( GPdd pdd : pdds )
                            {
                                refPdds.add( pdd );
                            }
                        }
                    }
                    else if ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.ARRETE )
                    {
                        Arrete<GSeance, GFichier> arrete = _arreteHome.findByFichier( fichier.getId(  ), plugin );
                        produit.setDesignation( (IDesignation<GSeance, GFichier>) arrete );
                    }
                    else if ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.DELIBERATION_DESIGNATION )
                    {
                        GDeliberationDesignation deliberation = _deliberationDesignationHome.findByFichier( fichier.getId(  ),
                                plugin );
                        produit.setDesignation( (IDesignation<GSeance, GFichier>) deliberation );
                    }
                    else if ( !refreshElementsPanierSpec( plugin, idUser, produits, fichier, produit ) )
                    {
                        refPdds.add( (GPdd) fichier.getPDD(  ) );
                    }

                    produit.setRefPdds( refPdds );
                    produits.add( produit );
                }
            }
        }
    }

    /**
     * Permet de supprimer les lments du panier slectionn par l'utilisateur
     *
     * @param plugin le Plugin actif
     * @param ids tableau des id des documents qui doivent tre tlcharg
     * @param idUser id de l'utilisateur
     * @param produits la liste des produits du panier
     */
    private void removeSelectionPanier( Plugin plugin, String[] ids, String idUser, List<GProduit> produits )
    {
        int lIds = ( ids != null ) ? ids.length : 0;

        // Pour chaque fichier slectionn, on le supprime dans la base
        for ( int i = 0; i < lIds; i++ )
        {
            int nIdFichier = StringUtils.string2id( ids[i] );

            if ( nIdFichier != -1 )
            {
                int indexProduit = getIndexProduitByIdFichier( nIdFichier, produits );

                if ( indexProduit != -1 )
                {
                    GProduit p = produits.get( indexProduit );
                    _panierHome.removeElementPanier( idUser, p.getFichier(  ).getId(  ), plugin );
                    produits.remove( indexProduit );
                }
            }
        }
    }

    /**
     * Retourne l'objet Produit qui correspond au fichier pass en paramtre
     *
     * @param idFichier id du fichier
     * @param produits la liste des produits du panier
     * @return l'objet Produit ou -1 si aucun Produit ne correspond au fichier
     */
    private int getIndexProduitByIdFichier( int idFichier, List<GProduit> produits )
    {
        int nProduitSize = produits.size(  );

        // Pour chaque Produit du panier, si l'identifiant dfdu fichier correspond  l'identifiant
        // du fichier que l'on recherche, on retourne le Produit
        for ( int i = 0; i < nProduitSize; i++ )
        {
            GProduit p = produits.get( i );

            if ( ( p.getFichier(  ) != null ) && ( p.getFichier(  ).getId(  ) == idFichier ) )
            {
                return i;
            }
        }

        return -1;
    }

    /**
     * Permet d'ajouter les lments du panier slectionn par l'utilisateur
     *
     * @param request le requte http
     * @param plugin le Plugin actif
     * @param ids tableau des id des documents qui doivent tre tlcharg
     * @param idUser id de l'utilisateur
     * @param produits la liste des produits du panier
     */
    @SuppressWarnings( "unchecked" )
    private void addSelectionPanier( HttpServletRequest request, Plugin plugin, String[] ids, String idUser,
        List<GProduit> produits )
    {
        int lIds = ( ids != null ) ? ids.length : 0;

        // Pour chaque lment slectionn, si le fichier n'est pas dj dans le panier,
        // alors on ajoute l'lment slectionn au panier dans la base
        for ( int i = 0; i < lIds; i++ )
        {
            int idFichier = -1;

            try
            {
                idFichier = Integer.parseInt( ids[i] );
            }
            catch ( NumberFormatException nfe )
            {
                AppLogService.error( "addSelectionPanier - " + nfe.getClass(  ) + CONSTANT_ERROR_MESSAGE_SEPARATOR +
                    nfe.getMessage(  ) );
            }

            int nIndexProduit = getIndexProduitByIdFichier( idFichier, produits );

            if ( ( ids[i] != null ) && !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( ids[i] ) && ( nIndexProduit == -1 ) )
            {
                GFichier fichier = _fichierHome.findByPrimaryKey( idFichier, plugin );

                if ( fichier != null )
                {
                    _utilisateurService.getUtilisateurCourant( request, plugin );
                    _panierHome.addElementPanier( idUser, idFichier, plugin );

                    /* Ajout du produits dans la liste des produits */
                    GProduit p = newProduitInstance(  );
                    List<GPdd> refPdds = new ArrayList<GPdd>(  );

                    p.setFichier( fichier );

                    if ( ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.AMENDEMENT ) ||
                            ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.VOEU ) ||
                            ( ( fichier.getPDD(  ) == null ) &&
                            ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.DELIBERATION ) ) )
                    {
                        GVoeuAmendement voeuAmendement;

                        if ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.DELIBERATION )
                        {
                            voeuAmendement = _voeuAmendementHome.findByDeliberation( fichier, plugin );
                        }
                        else
                        {
                            voeuAmendement = _voeuAmendementHome.findByTexteInitial( fichier, plugin );
                        }

                        if ( voeuAmendement != null )
                        {
                            p.setVoeuAmendement( voeuAmendement );

                            List<GPdd> pdds = voeuAmendement.getPdds(  );

                            for ( GPdd pdd : pdds )
                            {
                                refPdds.add( pdd );
                            }
                        }
                    }
                    else if ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.ARRETE )
                    {
                        Arrete<GSeance, GFichier> arrete = _arreteHome.findByFichier( fichier.getId(  ), plugin );
                        p.setDesignation( (IDesignation<GSeance, GFichier>) arrete );
                    }
                    else if ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.DELIBERATION_DESIGNATION )
                    {
                        GDeliberationDesignation deliberation = _deliberationDesignationHome.findByFichier( fichier.getId(  ),
                                plugin );
                        p.setDesignation( (IDesignation<GSeance, GFichier>) deliberation );
                    }
                    else
                    {
                        refPdds.add( (GPdd) fichier.getPDD(  ) );
                    }

                    p.setRefPdds( refPdds );
                    produits.add( p );
                }
            }
        }
    }

    /**
     * Teste la connexion au serveur JORAM.
     * @return TRUE si le test s'est bien droul, FALSE sinon
     */
    private static boolean checkJoram(  )
    {
        boolean bResult = true;

        try
        {
            Properties props = new Properties(  );
            props.setProperty( OdsProperties.PROPERTY_JNDI_INITIAL_NAME,
                AppPropertiesService.getProperty( OdsProperties.PROPERTY_JNDI_INITIAL,
                    OdsProperties.DEFAULT_PROPERTY_JNDI_INITIAL ) );
            props.setProperty( OdsProperties.PROPERTY_JNDI_HOST_NAME,
                AppPropertiesService.getProperty( OdsProperties.PROPERTY_JNDI_HOST ) );
            props.setProperty( OdsProperties.PROPERTY_JNDI_PORT_NAME,
                AppPropertiesService.getProperty( OdsProperties.PROPERTY_JNDI_PORT ) );

            Context ictx = new InitialContext( props );

            /* Initialisation de la queue */
            Queue queue = (Queue) ictx.lookup( AppPropertiesService.getProperty( 
                        OdsProperties.PROPERTY_JNDI_QUEUE_NAME, OdsProperties.DEFAULT_PROPERTY_JNDI_QUEUE_NAME ) );

            if ( queue == null )
            {
                AppLogService.error( "Panier : checkJoram - Queue null" );
                bResult = false;

                return bResult;
            }

            /* Initialisation de la connexion  la queue */
            QueueConnectionFactory qcf = (QueueConnectionFactory) ictx.lookup( AppPropertiesService.getProperty( 
                        OdsProperties.PROPERTY_JNDI_CONNECTION_FACTORY,
                        OdsProperties.DEFAULT_PROPERTY_JNDI_CONNECTION_FACTORY ) );

            if ( qcf == null )
            {
                AppLogService.error( "Panier : checkJoram - QueueConnectionFactory null" );
                bResult = false;

                return bResult;
            }

            ictx.close(  );
        }
        catch ( Exception e )
        {
            AppLogService.error( "Panier : checkJoram - " + e.getClass(  ) + CONSTANT_ERROR_MESSAGE_SEPARATOR +
                e.getMessage(  ) );
            bResult = false;
        }

        return bResult;
    }

    /**
     * Retourne l'URL de tlchargement du panier
     * @return l'URL de tlchargement du panier
     */
    public abstract String getUrlDownloadPanier(  );

    /*
     * (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.service.xpage.panier.IPanierAppService#getPropertyPageTitle()
     */
    public abstract String getPropertyPageTitle(  );

    /*
     * (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.service.xpage.panier.IPanierAppService#getPropertyPathLabel()
     */
    public abstract String getPropertyPathLabel(  );

    /*
     * (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.service.xpage.panier.IPanierAppService#getTemplate()
     */
    public abstract String getTemplate(  );
}
