/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.xpage.panier;

import fr.paris.lutece.plugins.ods.business.panier.PanierHome;
import fr.paris.lutece.plugins.ods.business.seance.ISeanceHome;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.panier.Panier;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.service.utilisateur.IUtilisateurService;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppPathService;
import fr.paris.lutece.portal.service.util.AppPropertiesService;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;


public abstract class AbstractPanierConfirmationAppService<GSeance extends ISeance, GSeanceFilter extends ISeanceFilter, GFichier extends IFichier<GSeance, GFichier>, GElu extends IElu, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>, GPdd extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>>
    implements IPanierConfirmationAppService
{
    private static final String MARK_ZIP_IS_FINISHED = "isZipFinished";
    private static final String MARK_ZIP_ID_DOWNLOAD = "idToDownload";
    private static final String MARK_URL_IMAGE = "urlImage";
    private static final String MARK_TAILLE_TOTAL_MAX = "tailleTotalMax";
    private static final String MARK_URL_DOWNLOAD_PANIER = "url_download_panier";
    private static final String MARK_ERREUR_JMS = "erreur_jms";
    private static final String PROPERTY_TAILLE_TOTAL_MAX = "ods.taille_total_max_telechargement";
    private static final int DEFAULT_PROPERTY_TAILLE_TOTAL_MAX = 22020096;
    private static final String CONSTANTE_DATEAJOUT_NULL = "null";
    private static final String URL_IMAGE = "images/local/skin/plugins/ods/dot3_progress.gif";
    @Autowired
    private PanierHome _panierHome;
    @Autowired
    private IUtilisateurService _utilisateurService;
    @Autowired
    private ISeanceHome<GSeance, GSeanceFilter, GFichier, GElu, GVoeuAmendement, GPdd> _seanceHome;
    @Autowired
    private IPanierAppService _panierAppService;

    /**
     * renvoie la page.
     * @param request le requte http
     * @param nMode le mode
     * @param plugin le Plugin actif
     * @return la page
     */
    public HashMap<String, Object> getPage( HttpServletRequest request, int nMode, Plugin plugin )
    {
        HashMap<String, Object> model = new HashMap<String, Object>(  );
        _utilisateurService.gestionUtilisateur( model, request, plugin );

        // Rcupre la date de la demande de tlchargment du panier
        String dateAjout = request.getParameter( OdsMarks.MARK_DATE_AJOUT );

        // Gestion en cas d'erreur de connexion avec JORAM
        if ( request.getSession(  ).getAttribute( MARK_ERREUR_JMS ) != null )
        {
            model.put( MARK_ERREUR_JMS, OdsConstants.CONSTANTE_CHAINE_VIDE );
            request.getSession(  ).removeAttribute( MARK_ERREUR_JMS );
        }

        // On rcupre la sance en cours si elle existe
        ISeance seanceEnCours = _seanceHome.getSeanceEnCours( plugin );

        if ( seanceEnCours != null )
        {
            model.put( OdsMarks.MARK_ID_SEANCE_EN_COURS, seanceEnCours.getIdSeance(  ) );
        }

        // Si la date d'ajout est correcte alors on vrifie si l'archive est prsente dans la base de donnes
        if ( ( dateAjout != null ) && !CONSTANTE_DATEAJOUT_NULL.equals( dateAjout ) )
        {
            Panier panier = _panierHome.findZipByDateAjout( dateAjout, plugin );

            if ( panier != null )
            {
                model.put( MARK_ZIP_IS_FINISHED, true );
                model.put( MARK_ZIP_ID_DOWNLOAD, panier.getIdToDownload(  ) );
            }
            else
            {
                model.put( MARK_ZIP_IS_FINISHED, false );
            }

            model.put( MARK_URL_IMAGE, AppPathService.getBaseUrl( request ) + URL_IMAGE );

            // Gestion du lien externe vers la webapp de telechargement du panier
            String url = _panierAppService.getUrlDownloadPanier(  );
            model.put( MARK_URL_DOWNLOAD_PANIER, url );
        }
        else
        {
            // Sinon on indique  l'utilisateur qu'une erreur a eu lieu
            if ( dateAjout != null )
            {
                model.put( MARK_TAILLE_TOTAL_MAX,
                    AppPropertiesService.getPropertyInt( PROPERTY_TAILLE_TOTAL_MAX, DEFAULT_PROPERTY_TAILLE_TOTAL_MAX ) / 10000 );
            }
        }

        return model;
    }
}
