/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.xpage.pdd;

import fr.paris.lutece.plugins.ods.business.pdd.IPDDHome;
import fr.paris.lutece.plugins.ods.business.seance.ISeanceHome;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.pdd.IArrondissement;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDDFilter;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.utilisateur.Utilisateur;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendementFilter;
import fr.paris.lutece.plugins.ods.service.utilisateur.IUtilisateurService;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.portal.service.plugin.Plugin;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;
import java.util.List;

import javax.servlet.http.HttpServletRequest;


public abstract class AbstractPddListNewAppService<GSeance extends ISeance, GSeanceFilter extends ISeanceFilter, GFichier extends IFichier<GSeance, GFichier>, GElu extends IElu, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>, GPdd extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>, GPDDFilter extends IPDDFilter, GVoeuAmendementFilter extends IVoeuAmendementFilter, GArrondissement extends IArrondissement>
    implements IPddListNewAppService
{
    private static final String MARK_LISTE_NOUVEAUX_PDDS = "liste_pdds_nouveaux";
    @Autowired
    private IUtilisateurService _utilisateurService;
    @Autowired
    private ISeanceHome<GSeance, GSeanceFilter, GFichier, GElu, GVoeuAmendement, GPdd> _seanceHome;
    @Autowired
    private IPDDHome<GPdd, GPDDFilter, GVoeuAmendementFilter, GSeance, GElu, GVoeuAmendement, GFichier, GArrondissement> _projetDeliberationHome;

    protected abstract GPDDFilter getPDDFilter(  );

    protected abstract void setFilterFormationConseilSpec( GPDDFilter filter );

    /**
     * @param request la requte HTTP
     * @param nMode le mode
     * @param plugin le plugin
     * @return XPage template
     */
    public HashMap<String, Object> getPage( HttpServletRequest request, int nMode, Plugin plugin )
    {
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        // On met  jour la liste des utilisateurs et gre la session de l'utilisateur
        _utilisateurService.gestionUtilisateur( model, request, plugin );

        // On rcupre la liste des Pdds inscrits  l'ordre du jour de rfrence
        GSeance seance = _seanceHome.getProchaineSeance( plugin );

        List<GPdd> nouveauxPDD = getNouveauxPddsPublies( request, seance, plugin );

        if ( nouveauxPDD != null )
        {
            model.put( MARK_LISTE_NOUVEAUX_PDDS, nouveauxPDD );
        }

        //On rcupre la sance en cours si elle existe
        GSeance seanceEnCours = _seanceHome.getSeanceEnCours( plugin );

        if ( seanceEnCours != null )
        {
            model.put( OdsMarks.MARK_ID_SEANCE_EN_COURS, seanceEnCours.getIdSeance(  ) );
        }

        return model;
    }

    /**
     * Recuperation de la liste des pdds publis aprs la date de dernire connexion
     * de l'utilisateur courant
     * @param request la requte Http
     * @param seance la sance concerne
     * @param plugin le plugin ODS
     * @return la liste des Pdds publis aprs la date de dernire connexion de l'utilisateur courante
     */
    private List<GPdd> getNouveauxPddsPublies( HttpServletRequest request, GSeance seance, Plugin plugin )
    {
        Utilisateur utilisateur = (Utilisateur) request.getSession(  ).getAttribute( OdsMarks.MARK_UTILISATEUR );

        boolean isPremiereConnexion = false;

        if ( ( request.getSession(  ).getAttribute( OdsMarks.MARK_PREMIERE_CONNEXION ) != null ) &&
                ( (Boolean) request.getSession(  ).getAttribute( OdsMarks.MARK_PREMIERE_CONNEXION ) ) )
        {
            isPremiereConnexion = true;
        }

        if ( seance != null )
        {
            GPDDFilter filter = getPDDFilter(  );

            if ( ( utilisateur != null ) && !isPremiereConnexion )
            {
                filter.setDatePublication( utilisateur.getDerniereConnexion(  ) );
            }

            filter.setPublication( 1 );
            filter.setProchaineSeance( seance.getIdSeance(  ) );
            setFilterFormationConseilSpec( filter );

            //filter.setIdFormationConseil( SelectArrondissementService.getArrondissement(  ) );

            // On rcupre la liste des Pdd mis en ligne aprs la date de dernire connexion de l'utilisateur
            return _projetDeliberationHome.findByFilter( true, filter, plugin );
        }

        return null;
    }
}
