/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.xpage.profil;

import fr.paris.lutece.plugins.ods.business.seance.ISeanceHome;
import fr.paris.lutece.plugins.ods.business.utilisateur.UtilisateurHome;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.utilisateur.Utilisateur;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.service.utilisateur.IUtilisateurService;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppPathService;
import fr.paris.lutece.portal.service.util.AppPropertiesService;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import javax.servlet.http.HttpServletRequest;


public abstract class AbstractProfilAppService<GSeance extends ISeance, GSeanceFilter extends ISeanceFilter, GFichier extends IFichier<GSeance, GFichier>, GElu extends IElu, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>, GPdd extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>>
    implements IProfilAppService
{
    private static final String PROPERTY_URL_PASSWORD = "ods.profil.urlPassword";
    private static final String MARK_UTILISATEUR = "utilisateur";
    private static final String MARK_PASSWORD = "password";
    private static final String MARK_ENREGISTREMENT = "enregistrement";
    private static final String CONSTANTE_ENREGISTRER = "enregistrer";
    private static final String CONSTANTE_PATTERN_TYPE_MAIL = "^[a-zA-Z0-9\\.\\-\\_]+@([a-zA-Z0-9\\-\\_\\.]+\\.)+([a-zA-Z]{2,4})$";
    @Autowired
    private IUtilisateurService _utilisateurService;
    @Autowired
    private UtilisateurHome _utilisateurHome;
    @Autowired
    private ISeanceHome<GSeance, GSeanceFilter, GFichier, GElu, GVoeuAmendement, GPdd> _seanceHome;

    /**
     * @param request la requte HTTP
     * @param nMode le mode
     * @param plugin le plugin
     * @return XPage template
     */
    public HashMap<String, Object> getPage( HttpServletRequest request, int nMode, Plugin plugin )
    {
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        // On met  jour la liste des utilisateurs et gre la session de l'utilisateur
        _utilisateurService.gestionUtilisateur( model, request, plugin );

        boolean bEnregistrement = true;

        // On rcupre l'utilisateur courant
        Utilisateur user = _utilisateurService.getUtilisateurCourant( request, plugin );

        // Si on affiche cette page suite  une requte d'enregistrement,
        // alors on sauvegarde en base les adresses mail de copie.
        if ( request.getParameter( CONSTANTE_ENREGISTRER ) != null )
        {
            if ( ( request.getParameter( OdsParameters.MAIL_COPIE_1 ) != null ) &&
                    !request.getParameter( OdsParameters.MAIL_COPIE_1 ).trim(  )
                                .equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                if ( isMatchesPatternMail( request.getParameter( OdsParameters.MAIL_COPIE_1 ).trim(  ) ) )
                {
                    user.setMailCopie1( request.getParameter( OdsParameters.MAIL_COPIE_1 ).trim(  ) );
                }
                else
                {
                    bEnregistrement = false;
                }
            }
            else
            {
                user.setMailCopie1( null );
            }

            if ( ( request.getParameter( OdsParameters.MAIL_COPIE_2 ) != null ) &&
                    !request.getParameter( OdsParameters.MAIL_COPIE_2 ).trim(  )
                                .equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                if ( isMatchesPatternMail( request.getParameter( OdsParameters.MAIL_COPIE_2 ).trim(  ) ) )
                {
                    user.setMailCopie2( request.getParameter( OdsParameters.MAIL_COPIE_2 ).trim(  ) );
                }
                else
                {
                    bEnregistrement = false;
                }
            }
            else
            {
                user.setMailCopie2( null );
            }

            _utilisateurHome.update( user, plugin );
            model.put( MARK_ENREGISTREMENT, bEnregistrement );
        }

        //On rcupre la sance en cours si elle existe
        ISeance seanceEnCours = _seanceHome.getSeanceEnCours( plugin );

        if ( seanceEnCours != null )
        {
            model.put( OdsMarks.MARK_ID_SEANCE_EN_COURS, seanceEnCours.getIdSeance(  ) );
        }

        model.put( MARK_UTILISATEUR, user );

        String strUrlPassword = AppPropertiesService.getProperty( PROPERTY_URL_PASSWORD );
        model.put( MARK_PASSWORD, AppPathService.getBaseUrl( request ) + strUrlPassword );

        return model;
    }

    /**
     * return true if the value in parameter verify regular expression
     * @param strValueToTest the value to test
     * @return true if the value in parameter verify regular expression
     */
    private boolean isMatchesPatternMail( String strValueToTest )
    {
        Pattern pattern = null;

        try
        {
            pattern = Pattern.compile( CONSTANTE_PATTERN_TYPE_MAIL );
        }
        catch ( PatternSyntaxException exception )
        {
            return false;
        }

        Matcher controler = pattern.matcher( strValueToTest );

        return controler.matches(  );
    }
}
