/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.xpage.seance;

import fr.paris.lutece.plugins.ods.business.fichier.IFichierHome;
import fr.paris.lutece.plugins.ods.business.ordredujour.IOrdreDuJourHome;
import fr.paris.lutece.plugins.ods.business.seance.ISeanceHome;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter;
import fr.paris.lutece.plugins.ods.dto.fichier.ITypeDocument;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.service.utilisateur.IUtilisateurService;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppPropertiesService;
import fr.paris.lutece.portal.web.LocalVariables;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


public abstract class AbstractProchainConseilAppService<GSeance extends ISeance, GSeanceFilter extends ISeanceFilter, GFichier extends IFichier<GSeance, GFichier>, GElu extends IElu, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>, GPdd extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>, GFichierFilter extends IFichierFilter, GTypeDocument extends ITypeDocument, GOrdreDuJour extends IOrdreDuJour<GEntreeOrdreDuJour, GSeance, GFichier, GElu, GOrdreDuJour>, GEntreeOrdreDuJour extends IEntreeOrdreDuJour<GElu, GSeance, GFichier, GEntreeOrdreDuJour, GOrdreDuJour>, GOrdreDuJourFilter extends IOrdreDuJourFilter>
    implements IProchainConseilAppService
{
    protected static final String PROPERTY_TELECHARGEMENT_SERVLET_DOCUMENTS_SEANCE = "ods.telechargement.servlet.documentsSeance";
    protected static final String PROPERTY_TELECHARGEMENT_COOKIE_DOMAIN = "ods.telechargement.cookie.domain";
    protected static final String MARK_URL_DOWNLOAD_DOCUMENTS_SEANCE = "url_download_documents_seance";
    @Autowired
    private IUtilisateurService _utilisateurService;
    @Autowired
    protected ISeanceHome<GSeance, GSeanceFilter, GFichier, GElu, GVoeuAmendement, GPdd> _seanceHome;
    @Autowired
    protected IOrdreDuJourHome<GOrdreDuJour, GEntreeOrdreDuJour, GOrdreDuJourFilter, GSeance, GElu, GFichier> _ordreDuJourHome;
    @Autowired
    protected IFichierHome<GFichier, GFichierFilter, GTypeDocument, GSeance> _fichierHome;

    protected abstract void addCookieSpec( HttpServletRequest request, HttpServletResponse response );

    /**
     * @param request la requte HTTP
     * @param nMode le mode
     * @param plugin le plugin
     * @return XPage template
     */
    public HashMap<String, Object> getPage( HttpServletRequest request, int nMode, Plugin plugin )
    {
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        _utilisateurService.gestionUtilisateur( model, request, plugin );

        /* Cration du cookie pour attester de l'authentification sur le serveur de tlchargement */
        HttpServletResponse response = LocalVariables.getResponse(  );
        Cookie[] cookies = request.getCookies(  );
        String strGuid = "";

        if ( cookies != null )
        {
            for ( Cookie c : cookies )
            {
                if ( ( c != null ) && ( c.getName(  ) != null ) &&
                        c.getName(  )
                             .equals( AppPropertiesService.getProperty( OdsProperties.PROPERTY_COOKIE_WSSO_NAME_SOURCE ) ) )
                {
                    strGuid = c.getValue(  );

                    break;
                }
            }
        }

        Cookie newCookie = new Cookie( AppPropertiesService.getProperty( 
                    OdsProperties.PROPERTY_COOKIE_WSSO_NAME_DESTINATION ), strGuid );
        newCookie.setDomain( AppPropertiesService.getProperty( PROPERTY_TELECHARGEMENT_COOKIE_DOMAIN ) );
        newCookie.setPath( OdsConstants.PATH_SEPARATOR );
        response.addCookie( newCookie );

        addCookieSpec( request, response );

        String url = AppPropertiesService.getProperty( OdsProperties.PROPERTY_TELECHARGEMENT_HOST ) +
            OdsConstants.PATH_SEPARATOR +
            AppPropertiesService.getProperty( OdsProperties.PROPERTY_TELECHARGEMENT_WEBAPP ) +
            OdsConstants.PATH_SEPARATOR +
            AppPropertiesService.getProperty( PROPERTY_TELECHARGEMENT_SERVLET_DOCUMENTS_SEANCE );

        //On rcupre la sance en cours si elle existe
        ISeance seanceEnCours = _seanceHome.getSeanceEnCours( plugin );

        if ( seanceEnCours != null )
        {
            model.put( OdsMarks.MARK_ID_SEANCE_EN_COURS, seanceEnCours.getIdSeance(  ) );
        }

        model.put( MARK_URL_DOWNLOAD_DOCUMENTS_SEANCE, url );

        return model;
    }
}
