/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.xpage.search;

import fr.paris.lutece.plugins.ods.business.requete.IRequeteUtilisateurHome;
import fr.paris.lutece.plugins.ods.business.seance.ISeanceHome;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.utilisateur.Utilisateur;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.service.utilisateur.IUtilisateurService;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.security.SecurityService;
import fr.paris.lutece.portal.service.util.AppPathService;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;
import java.util.List;

import javax.servlet.http.HttpServletRequest;


public abstract class AbstractCriteresDeRechercheAppService<GSeance extends ISeance, GSeanceFilter extends ISeanceFilter, GRequeteUtilisateur extends IRequeteUtilisateur<GSeance, GSeanceFilter>, GFichier extends IFichier<GSeance, GFichier>, GElu extends IElu, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>, GPdd extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>>
    implements ICriteresDeRechercheAppService<GSeance, GSeanceFilter, GRequeteUtilisateur>
{
    private static final String PARAM_SAVE_CRITERE = "save";
    private static final String PARAM_VALUE_NEXT = "next";
    private static final String PARAM_REQUETE = "id_requete";
    private static final String PARAM_TODO = "a_faire";
    private static final String MARK_ERREUR_MESSAGE = "message";
    private static final String MARK_URL_RECHERCHE_RESULTATS = "url_recherche_resultats";
    private static final String MARK_VALUE_URL_RECHERCHE_RESULTATS = "?page=ods_recherche";
    private static final String MARK_URL_RECHERCHE_ARCHIVES_RESULTATS = "url_recherche_archives_resultats";
    private static final String MARK_VALUE_URL_RECHERCHE_ARCHIVES_RESULTATS = "?page=ods_recherche_archives";
    private static final String MARK_URL_FORMULAIRE = "url_formulaire";
    private static final String MARK_VALUE_URL_FORMULAIRE = "?page=ods_recherche";
    private static final String MARK_URL_PROFIL = "url_profil";
    private static final String MARK_VALUE_URL_PROFIL = "?page=ods_mon_profil";
    protected static final String MARK_LISTE_REQUETES = "liste_requetes";
    private static final String MARK_UTILISATEUR = "utilisateur";
    private static final String MARK_REQUETE_UTILISATEUR = "user_search";
    private static final String MARK_IS_ARCHIVE = "isArchive";
    private static final String CONSTANTE_ALERT = "alert";
    private static final String CONSTANTE_NOTIFIE = "notifie";
    @Autowired
    private IUtilisateurService _utilisateurService;
    @Autowired
    private ISeanceHome<GSeance, GSeanceFilter, GFichier, GElu, GVoeuAmendement, GPdd> _seanceHome;
    @Autowired
    private IRequeteUtilisateurHome<GSeance, GSeanceFilter, GRequeteUtilisateur> _requeteUtilisateurHome;

    /**
     * Donnes spcifique  placer dans le model. Doit galement rcuprer la liste de requete utilisateurs
     * @param model le model
     * @param request Http request
     * @param nMode le mode
     * @param plugin le plugin
     */
    protected abstract void getPageSpec( HashMap<String, Object> model, HttpServletRequest request, int nMode,
        Plugin plugin );

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.xpage.search.ICriteresDeRechercheAppService#getPage(javax.servlet.http.HttpServletRequest, int, fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public HashMap<String, Object> getPage( HttpServletRequest request, int nMode, Plugin plugin )
    {
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        // Met  jour la liste des utilisateurs et gre la session de l'utilisateur
        _utilisateurService.gestionUtilisateur( model, request, plugin );

        // Ce paramtre vient de cette mme page qui se rappelle.
        String strToDo = request.getParameter( PARAM_TODO );

        if ( strToDo != null )
        {
            // On rcupre la requte  partir de son identifiant
            int nIdRequete;

            try
            {
                nIdRequete = Integer.parseInt( request.getParameter( PARAM_REQUETE ) );
            }
            catch ( NumberFormatException nfe )
            {
                nIdRequete = -1;
            }

            GRequeteUtilisateur requete = _requeteUtilisateurHome.findByPrimaryKey( nIdRequete, plugin );

            // Si la requte n'est pas nulle mais qu'elle n'appartient pas  l'utilisateur courant, 
            // alors on affiche un message d'erreur
            if ( ( requete == null ) ||
                    !requete.getUserName(  )
                                .equals( SecurityService.getInstance(  ).getRegisteredUser( request ).getName(  ) ) )
            {
                return null;
            }

            // Si l'action demande est d'tre alert, on met  jour la requte en base
            if ( strToDo.equals( CONSTANTE_ALERT ) )
            {
                requete.setNotifie( Boolean.parseBoolean( request.getParameter( CONSTANTE_NOTIFIE ) ) );
                _requeteUtilisateurHome.store( requete, plugin );
            }

            // Sinon l'action demande est une action de suppression et donc on supprime la requte de la base
            else
            {
                _requeteUtilisateurHome.remove( requete, plugin );
            }
        }

        String strPortalUrl = AppPathService.getPortalUrl(  );

        model.put( MARK_URL_RECHERCHE_RESULTATS, strPortalUrl + MARK_VALUE_URL_RECHERCHE_RESULTATS );
        model.put( MARK_URL_RECHERCHE_ARCHIVES_RESULTATS, strPortalUrl + MARK_VALUE_URL_RECHERCHE_ARCHIVES_RESULTATS );

        List<GRequeteUtilisateur> listeRequetes = _requeteUtilisateurHome.findByUser( SecurityService.getInstance(  )
                                                                                                     .getRegisteredUser( request )
                                                                                                     .getName(  ),
                plugin );

        for ( GRequeteUtilisateur requete : listeRequetes )
        {
            requete.toHtml( plugin, request.getLocale(  ) );
        }

        model.put( MARK_LISTE_REQUETES, listeRequetes );
        model.put( MARK_URL_FORMULAIRE, strPortalUrl + MARK_VALUE_URL_FORMULAIRE );
        model.put( MARK_URL_PROFIL, strPortalUrl + MARK_VALUE_URL_PROFIL );

        // On rcupre l'utilisateur courant
        Utilisateur user = _utilisateurService.getUtilisateurCourant( request, plugin );
        model.put( MARK_UTILISATEUR, user );

        // On rcupre la sance en cours si elle existe
        ISeance seanceEnCours = _seanceHome.getSeanceEnCours( plugin );

        if ( seanceEnCours != null )
        {
            model.put( OdsMarks.MARK_ID_SEANCE_EN_COURS, seanceEnCours.getIdSeance(  ) );
        }

        getPageSpec( model, request, nMode, plugin );

        return model;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.xpage.search.ICriteresDeRechercheAppService#getErrorPage(javax.servlet.http.HttpServletRequest, fr.paris.lutece.plugins.ods.dto.requete.RequeteUtilisateur, java.lang.String, fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public HashMap<String, Object> getErrorPage( HttpServletRequest request, GRequeteUtilisateur requeteUtilisateur,
        String strErrorMessage, Plugin plugin )
    {
        HashMap<String, Object> model = new HashMap<String, Object>(  );
        _utilisateurService.gestionUtilisateur( model, request, plugin );

        model.put( MARK_ERREUR_MESSAGE, strErrorMessage );
        model.put( OdsMarks.MARK_USER_GIVEN_NAME, _utilisateurService.getGivenName( request ) );

        if ( requeteUtilisateur != null )
        {
            model.put( MARK_REQUETE_UTILISATEUR, requeteUtilisateur );
        }

        String strSave = request.getParameter( PARAM_SAVE_CRITERE );

        if ( ( strSave != null ) && strSave.equals( PARAM_VALUE_NEXT ) )
        {
            model.put( MARK_IS_ARCHIVE, false );
        }
        else
        {
            model.put( MARK_IS_ARCHIVE, true );
        }

        return model;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.xpage.search.ICriteresDeRechercheAppService#saveCriteria(fr.paris.lutece.plugins.ods.dto.requete.RequeteUtilisateur, fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public void saveCriteria( GRequeteUtilisateur requeteUtilisateur, Plugin plugin )
    {
        _requeteUtilisateurHome.create( requeteUtilisateur, plugin );
    }
}
