/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.xpage.search;

import fr.paris.lutece.plugins.ods.business.categoriedeliberation.CategorieDeliberationHome;
import fr.paris.lutece.plugins.ods.business.direction.IDirectionHome;
import fr.paris.lutece.plugins.ods.business.elu.IEluHome;
import fr.paris.lutece.plugins.ods.business.formationconseil.FormationConseilHome;
import fr.paris.lutece.plugins.ods.business.groupepolitique.GroupePolitiqueHome;
import fr.paris.lutece.plugins.ods.business.requete.IRequeteUtilisateurHome;
import fr.paris.lutece.plugins.ods.business.seance.ISeanceHome;
import fr.paris.lutece.plugins.ods.business.statut.StatutHome;
import fr.paris.lutece.plugins.ods.dto.categoriedeliberation.CategorieDeliberation;
import fr.paris.lutece.plugins.ods.dto.direction.Direction;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.groupepolitique.IGroupePolitique;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur;
import fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.statut.IStatutFilter;
import fr.paris.lutece.plugins.ods.dto.statut.Statut;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.service.search.ISearchService;
import fr.paris.lutece.plugins.ods.service.utilisateur.IUtilisateurService;
import fr.paris.lutece.portal.service.plugin.Plugin;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;
import java.util.List;

import javax.servlet.http.HttpServletRequest;


public abstract class AbstractRechercheArchivesAppService<GSeance extends ISeance, GSeanceFilter extends ISeanceFilter, GRequeteUtilisateur extends IRequeteUtilisateur<GSeance, GSeanceFilter>, GResultatRequete extends IResultatRequete<GSeance, GSeanceFilter, GRequeteUtilisateur, GElu, GFichier, GVoeuAmendement, GPDD>, GFichier extends IFichier<GSeance, GFichier>, GElu extends IElu, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>>
    implements IRechercheArchivesAppService<GSeance, GSeanceFilter, GRequeteUtilisateur, GResultatRequete, GFichier, GElu, GVoeuAmendement, GPDD>
{
    @Autowired
    private IUtilisateurService _utilisateurService;
    @Autowired
    private ISeanceHome<GSeance, GSeanceFilter, GFichier, GElu, GVoeuAmendement, GPDD> _seanceHome;
    @Autowired
    private StatutHome _statutHome;
    @Autowired
    private IRequeteUtilisateurHome<GSeance, GSeanceFilter, GRequeteUtilisateur> _requeteUtilisateurHome;
    @Autowired
    private GroupePolitiqueHome _groupePolitiqueHome;
    @Autowired
    private FormationConseilHome _formationConseilHome;
    @Autowired
    private IEluHome<IElu> _eluHome;
    @Autowired
    private IDirectionHome _directionHome;
    @Autowired
    private CategorieDeliberationHome _categorieDeliberationHome;

    protected abstract ISearchService<GSeance, GSeanceFilter, GRequeteUtilisateur, GResultatRequete, GElu, GFichier, GVoeuAmendement, GPDD> getSearchService(  );

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheArchivesAppService#gestionUtilisateur(java.util.HashMap, javax.servlet.http.HttpServletRequest, fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public void gestionUtilisateur( HashMap<String, Object> model, HttpServletRequest request, Plugin plugin )
    {
        _utilisateurService.gestionUtilisateur( model, request, plugin );
    }

    /*
     * (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheArchivesAppService#getProchaineSeance(fr.paris.lutece.portal.service.plugin.Plugin)
     */
    public GSeance getProchaineSeance( Plugin plugin )
    {
        return _seanceHome.getProchaineSeance( plugin );
    }

    /*
     * (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheArchivesAppService#getDerniereSeance(fr.paris.lutece.portal.service.plugin.Plugin)
     */
    public GSeance getDerniereSeance( Plugin plugin )
    {
        return _seanceHome.getDerniereSeance( plugin );
    }

    /*
     * (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheArchivesAppService#getSeanceEnCours(fr.paris.lutece.portal.service.plugin.Plugin)
     */
    public GSeance getSeanceEnCours( Plugin plugin )
    {
        return _seanceHome.getSeanceEnCours( plugin );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheArchivesAppService#findDirectionList(fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public List<Direction> findDirectionList( Plugin plugin )
    {
        return _directionHome.findDirectionList( plugin );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheArchivesAppService#findCategorieDeliberationList(fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public List<CategorieDeliberation> findCategorieDeliberationList( Plugin plugin )
    {
        return _categorieDeliberationHome.findCategorieDeliberationList( plugin );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheArchivesAppService#findEluList(fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public List<IElu> findEluList( Plugin plugin )
    {
        return _eluHome.findEluList( plugin );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheArchivesAppService#findFormationConseilList(fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public List<FormationConseil> findFormationConseilList( Plugin plugin )
    {
        return _formationConseilHome.findFormationConseilList( plugin );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheArchivesAppService#findGroupePolitiqueList(fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public List<IGroupePolitique> findGroupePolitiqueList( Plugin plugin )
    {
        return _groupePolitiqueHome.findGroupePolitiqueList( plugin );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheArchivesAppService#findAllRapporteurs(fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public List<IElu> findAllRapporteurs( Plugin plugin )
    {
        return _eluHome.findAllRapporteurs( plugin );
    }

    /*
     * (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheArchivesAppService#findStatutWithFilterList(fr.paris.lutece.plugins.ods.dto.statut.IStatutFilter, fr.paris.lutece.portal.service.plugin.Plugin)
     */
    public List<Statut> findStatutWithFilterList( IStatutFilter filter, Plugin plugin )
    {
        return _statutHome.findStatutWithFilterList( filter, plugin );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheArchivesAppService#findRequeteUtilisateurByPrimaryKey(int, fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public GRequeteUtilisateur findRequeteUtilisateurByPrimaryKey( int nKey, Plugin plugin )
    {
        return _requeteUtilisateurHome.findByPrimaryKey( nKey, plugin );
    }

    /*
     * (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheArchivesAppService#getPDDByReference(fr.paris.lutece.plugins.ods.dto.requete.RequeteUtilisateur, fr.paris.lutece.plugins.ods.dto.seance.ISeance, fr.paris.lutece.portal.service.plugin.Plugin)
     */
    public GResultatRequete getPDDByReference( GRequeteUtilisateur requete, GSeance prochaineSeance, Plugin plugin )
    {
        return getSearchService(  ).getPDDByReference( requete, prochaineSeance, plugin );
    }

    /*
     * (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheArchivesAppService#getAllObjectsByCriterias(fr.paris.lutece.plugins.ods.dto.requete.RequeteUtilisateur, fr.paris.lutece.plugins.ods.dto.seance.ISeance, boolean, fr.paris.lutece.portal.service.plugin.Plugin)
     */
    public GResultatRequete getAllObjectsByCriterias( GRequeteUtilisateur requete, GSeance prochaineSeance,
        boolean bIsLimited, Plugin plugin )
    {
        return getSearchService(  ).getAllObjectsByCriterias( requete, prochaineSeance, bIsLimited, plugin );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheArchivesAppService#findFormationConseilByPrimaryKey(int, fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public FormationConseil findFormationConseilByPrimaryKey( int nKey, Plugin plugin )
    {
        return _formationConseilHome.findByPrimaryKey( nKey, plugin );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheArchivesAppService#findEluByPrimaryKey(int, fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public IElu findEluByPrimaryKey( int nKey, Plugin plugin )
    {
        return _eluHome.findByPrimaryKey( nKey, plugin );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheArchivesAppService#findDirectionByPrimaryKey(int, fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public Direction findDirectionByPrimaryKey( int nKey, Plugin plugin )
    {
        return _directionHome.findByPrimaryKey( nKey, plugin );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheArchivesAppService#findCategorieDeliberationByPrimaryKey(int, fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public CategorieDeliberation findCategorieDeliberationByPrimaryKey( int nKey, Plugin plugin )
    {
        return _categorieDeliberationHome.findByPrimaryKey( nKey, plugin );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheArchivesAppService#findGroupePolitiqueByPrimaryKey(int, fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public IGroupePolitique findGroupePolitiqueByPrimaryKey( int nKey, Plugin plugin )
    {
        return _groupePolitiqueHome.findByPrimaryKey( nKey, plugin );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheArchivesAppService#findStatutByPrimaryKey(int, fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public Statut findStatutByPrimaryKey( int nKey, Plugin plugin )
    {
        return _statutHome.findByPrimaryKey( nKey, plugin );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheArchivesAppService#findElusRapporteursActifs(fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public List<IElu> findElusRapporteursActifs( Plugin plugin )
    {
        return _eluHome.findElusRapporteursActifs( plugin );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheArchivesAppService#findAllDirectionsActives(fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public List<Direction> findAllDirectionsActives( Plugin plugin )
    {
        return _directionHome.findAllDirectionsActives( plugin );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheArchivesAppService#findAllActivesOrderByLibelle(fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public List<CategorieDeliberation> findAllActivesOrderByLibelle( Plugin plugin )
    {
        return _categorieDeliberationHome.findAllActivesOrderByLibelle( plugin );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheArchivesAppService#findGroupesActifs(fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public List<IGroupePolitique> findGroupesActifs( Plugin plugin )
    {
        return _groupePolitiqueHome.findGroupesActifs( plugin );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheArchivesAppService#findElusActifs(fr.paris.lutece.portal.service.plugin.Plugin)
         */
    public List<IElu> findElusActifs( Plugin plugin )
    {
        return _eluHome.findElusActifs( plugin );
    }
}
