/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.xpage.search;

import fr.paris.lutece.plugins.ods.dto.categoriedeliberation.CategorieDeliberation;
import fr.paris.lutece.plugins.ods.dto.direction.Direction;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.formationconseil.FormationConseil;
import fr.paris.lutece.plugins.ods.dto.groupepolitique.IGroupePolitique;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur;
import fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.portal.service.plugin.Plugin;

import java.util.HashMap;
import java.util.List;

import javax.servlet.http.HttpServletRequest;


/**
 *
 * @author eLyaet
 *
 * @param <GRequeteUtilisateur>
 */
public interface IRechercheAppService<GSeance extends ISeance, GSeanceFilter extends ISeanceFilter, GRequeteUtilisateur extends IRequeteUtilisateur<GSeance, GSeanceFilter>, GElu extends IElu, GFichier extends IFichier<GSeance, GFichier>, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>, GPdd extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>>
{
    /**
     *
     * @param plugin
     * @return
     */
    GSeance getProchaineSeance( Plugin plugin );

    /**
     *
     * @param model
     * @param request
     * @param plugin
     */
    void gestionUtilisateur( HashMap<String, Object> model, HttpServletRequest request, Plugin plugin );

    /**
     *
     * @param plugin
     * @return
     */
    ISeance getSeanceEnCours( Plugin plugin );

    /**
     *
     * @param plugin
     * @return
     */
    List<Direction> findAllDirectionsActives( Plugin plugin );

    /**
     *
     * @param plugin
     * @return
     */
    List<CategorieDeliberation> findAllActivesOrderByLibelle( Plugin plugin );

    /**
     *
     * @param plugin
     * @return
     */
    List<IGroupePolitique> findGroupesActifs( Plugin plugin );

    /**
     *
     * @param plugin
     * @return
     */
    List<IElu> findElusActifs( Plugin plugin );

    /**
     *
     * @param plugin
     * @return
     */
    List<FormationConseil> findFormationConseilList( Plugin plugin );

    /**
     *
     * @param plugin
     * @return
     */
    List<IElu> findElusRapporteursActifs( Plugin plugin );

    /**
     *
     * @param nKey
     * @param plugin
     * @return
     */
    GRequeteUtilisateur findRequeteUtilisateurByPrimaryKey( int nKey, Plugin plugin );

    /**
     *
     * @param requete
     * @param prochaineSeance
     * @param plugin
     * @return
     */
    IResultatRequete<GSeance, GSeanceFilter, GRequeteUtilisateur, GElu, GFichier, GVoeuAmendement, GPdd> getPDDByReference( 
        GRequeteUtilisateur requete, GSeance prochaineSeance, Plugin plugin );

    /**
     *
     * @param requete
     * @param prochaineSeance
     * @param bIsLimited
     * @param plugin
     * @return
     */
    IResultatRequete<GSeance, GSeanceFilter, GRequeteUtilisateur, GElu, GFichier, GVoeuAmendement, GPdd> getAllObjectsByCriterias( 
        GRequeteUtilisateur requete, GSeance prochaineSeance, boolean bIsLimited, Plugin plugin );

    /**
     *
     * @param seance
     * @param bInscrit
     * @param request
     */
    List<Integer> getIdPddsInscritsOdj( GSeance seance, boolean bInscrit, HttpServletRequest request );

    /**
     *
     * @param requete
     * @param prochaineSeance
     * @param bIsLimited
     * @param plugin
     * @return
     */
    IResultatRequete<GSeance, GSeanceFilter, GRequeteUtilisateur, GElu, GFichier, GVoeuAmendement, GPdd> getIndexedObjects( 
        GRequeteUtilisateur requete, GSeance prochaineSeance, boolean bIsLimited, Plugin plugin );

    /**
     *
     * @param nKey
     * @param plugin
     * @return
     */
    FormationConseil findFormationConseilByPrimaryKey( int nKey, Plugin plugin );

    /**
     *
     * @param nKey
     * @param plugin
     * @return
     */
    IElu findEluByPrimaryKey( int nKey, Plugin plugin );

    /**
     *
     * @param nKey
     * @param plugin
     * @return
     */
    Direction findDirectionByPrimaryKey( int nKey, Plugin plugin );

    /**
     *
     * @param nKey
     * @param plugin
     * @return
     */
    CategorieDeliberation findCategorieDeliberationByPrimaryKey( int nKey, Plugin plugin );

    /**
     *
     * @param nKey
     * @param plugin
     * @return
     */
    IGroupePolitique findGroupePolitiqueByPrimaryKey( int nKey, Plugin plugin );

    /**
     *
     * @return
     */
    String getPropertyPageTitle(  );

    /**
     *
     * @return
     */
    String getPropertyPageResultatTitle(  );

    /**
     *
     * @return
     */
    String getPropertyPageFormulaireTitle(  );

    /**
     *
     * @return
     */
    String getPropertyPageErreurTitle(  );

    /**
     *
     * @return
     */
    String getPropertyPathLabel(  );

    /**
     *
     * @return
     */
    String getTemplate(  );

    /**
     *
     * @return
     */
    String getTemplateRecherche(  );

    /**
     *
     * @return
     */
    String getTemplateRechercheResultats(  );

    /**
     *
     * @return
     */
    String getTemplateRequeteNonValide(  );
}
