/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.ui.field;


/**
 * Classe abstraite pour la mise en oeuvre du pattern Decorator.
 *
 * Permet d'etendre les fonctinonalite d'affichage d'un champ de base par des controles supplementaires.
 * Par exemple : pour l'ajout d'un bouton de type "Recalculer" lie a un champ date
 *
 * @param <T> le type de la valeur manipule par le champ
 */
public abstract class AbstractFieldDecorator<T> extends AbstractField<T>
{
    protected Field _field;

    protected AbstractFieldDecorator(  )
    {
    }

    /**
     * Constructeur du Decorator
     *
     * @param field
     */
    public AbstractFieldDecorator( Field field )
    {
        this._field = field;
    }

    /* (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.ui.field.AbstractField#getId()
     */
    public String getId(  )
    {
        return _field.getId(  );
    }

    public boolean isReadonly(  )
    {
        return _field.isReadonly(  );
    }
}
