/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.ui.field;

import fr.paris.lutece.util.ReferenceList;

import java.util.HashMap;


/**
 * Classe pour l'implementation des combo box
 *
 * @param <T> le type des cles de la combo box
 */
@Template( templatePath = "admin/plugins/ods/ui/combo_list.html" )
public class ComboListField<T> extends AbstractField<T>
{
    private ReferenceList _values;

    /**
     * Permet l'initialisation de l'element selectionne dans la liste.
     * La recuperation est faite par beanutils
     *
     * @param id l'identiant du champ dans le bean
     * @param o le bean
     */
    public ComboListField( String id, Object o )
    {
        init( id, o );
    }

    /**
     * Permet d'initialiser la liste des choix possibles de la combo
     *
     * @param values une referenceList
     */
    public void fill( ReferenceList values )
    {
        this._values = values;
    }

    public String render(  )
    {
        HashMap<String, Object> model = new HashMap<String, Object>(  );
        model.put( "values", _values );
        model.put( "id", _id );
        model.put( "selected_value", _value.toString(  ) );

        return renderHtml( model );
    }
}
