/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.ui.field;

import fr.paris.lutece.portal.service.i18n.I18nService;

import java.util.Collections;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;


/**
 * Classe de type Decorator permettant d'assigner un label a un autre champ
 *
 */
@Template( templatePath = "admin/plugins/ods/ui/label.html" )
public class LabelField extends AbstractField<Void>
{
    private static final String JS_TEMPLATE_MAIN = "admin/plugins/ods/ui/label_js.html";
    private Field _field;
    private String _label;

    public LabelField( String label )
    {
        this._label = label;
    }

    public LabelField( String label, Field field )
    {
        this._label = label;
        this._field = field;
    }

    public String render(  )
    {
        // TODO Auto-generated method stub
        HashMap<String, Object> model = new HashMap<String, Object>(  );
        model.put( "id", _field.getId(  ) );
        model.put( "label", I18nService.getLocalizedString( _label, Locale.FRANCE ) );
        model.put( "field", _field.render(  ) );

        return renderHtml( model );
    }

    public String getId(  )
    {
        return _field.getId(  );
    }

    public String renderJS(  )
    {
        // TODO Auto-generated method stub
        HashMap<String, Object> model = new HashMap<String, Object>(  );
        model.put( "field", _field.renderJS(  ) );

        return renderJS( model );
    }

    @Override
    protected String getJSTemplatePath(  )
    {
        // TODO Auto-generated method stub
        return JS_TEMPLATE_MAIN;
    }

    @Override
    public void setReadonly( boolean isReadonly )
    {
        _field.setReadonly( isReadonly );
    }

    @Override
    public boolean isReadonly(  )
    {
        return _field.isReadonly(  );
    }

    @Override
    public Map<String, Object> getAttributes(  )
    {
        if ( _field != null )
        {
            return _field.getAttributes(  );
        }
        else
        {
            return Collections.EMPTY_MAP;
        }
    }

    @Override
    public void setRequired( boolean isRequired )
    {
        if ( ( _field != null ) && ( _field.getAttributes(  ) != null ) )
        {
            _field.getAttributes(  ).put( ATTRIBUTE_REQUIRED, isRequired );
        }
    }

    public void setJsEventMap( Map<JsEventEnum, String> jsEventMap )
    {
        if ( ( _field != null ) && _field instanceof AbstractField<?> )
        {
            ( (AbstractField<?>) _field ).setJsEventMap( jsEventMap );
        }
    }
}
