/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.ui.field;

import fr.paris.lutece.portal.service.template.AppTemplateService;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.util.html.HtmlTemplate;

import java.util.Locale;
import java.util.Map;


/**
 * Classe permettant de brancher une gestion plus classique des templates freemarker.
 *
 * TemplateField se base principalement sur un model et un template freemarker passee a l'instanciation.
 *
 */
public class TemplateField implements Field
{
    private String _id;
    private String _templatePath;
    private Map<String, Object> _model;
    private String _templateJSPath;
    private Map<String, Object> _jsModel;
    private boolean _readonly;

    public TemplateField( String id, String templatePath, Map<String, Object> model, String templateJSPath,
        Map<String, Object> jsModel )
    {
        this._id = id;
        this._templatePath = templatePath;
        this._model = model;
        this._templateJSPath = templateJSPath;
        this._jsModel = jsModel;
    }

    public String getId(  )
    {
        // TODO Auto-generated method stub
        return _id;
    }

    public String render(  )
    {
        // TODO Auto-generated method stub
        if ( _templatePath == null )
        {
            return null;
        }

        try
        {
            HtmlTemplate htmlTemplate = AppTemplateService.getTemplate( _templatePath, Locale.FRENCH, getModel( _model ) );

            return htmlTemplate.getHtml(  );
        }
        catch ( Exception e )
        {
            throw ( new AppException( "Render error on template field : " + getId(  ), e ) );
        }
    }

    public String renderJS(  )
    {
        // TODO Auto-generated method stub
        if ( _templateJSPath == null )
        {
            return null;
        }

        HtmlTemplate htmlTemplate = AppTemplateService.getTemplate( _templateJSPath, Locale.FRENCH, getModel( _jsModel ) );

        return htmlTemplate.getHtml(  );
    }

    public boolean isReadonly(  )
    {
        // TODO Auto-generated method stub
        return _readonly;
    }

    public void setReadonly( boolean isReadonly )
    {
        // TODO Auto-generated method stub
        _readonly = isReadonly;
    }

    public Map<String, Object> getAttributes(  )
    {
        // TODO Auto-generated method stub
        return null;
    }

    /**
     * Methode permettant d'ajouter le parametre readonly  un modele freemarker
     * @param model si model vaut null, alors l'attribut n'est pas ajout est la valeur null est retournee
     * @return
     */
    private Map<String, Object> getModel( Map<String, Object> model )
    {
        if ( model != null )
        {
            model.put( "readonly", _readonly );
        }

        return model;
    }

    public void setRequired( boolean isRequired )
    {
        // TODO Auto-generated method stub
        if ( _model != null )
        {
            _model.put( "required", isRequired );
        }
    }
}
