/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.ui.form;

import fr.paris.lutece.plugins.ods.ui.field.AbstractField;
import fr.paris.lutece.plugins.ods.ui.field.Field;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.web.upload.MultipartHttpServletRequest;
import fr.paris.lutece.util.ReferenceList;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.ConvertUtils;
import org.apache.commons.fileupload.FileItem;

import java.lang.reflect.InvocationTargetException;

import java.sql.Timestamp;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;


/**
 * Regroupement des fonctionnalites communes aux formulaires
 *
 *
 */
public abstract class AbstractFormBean implements FormBean
{
    protected static final int PROPERTY_MANQUANT = 0;
    protected static final int PROPERTY_EXTENSION = 1;
    private String _aFaire;
    protected String _formName;
    protected String _defaultAction;
    protected Map<String, ReferenceList> _mapReferentiel;

    /* (non-Javadoc)
     * @see fr.paris.lutece.plugins.ods.ui.form.FormBean#fillFromRequest(javax.servlet.http.HttpServletRequest)
     */
    public void fillFromRequest( HttpServletRequest request )
    {
        // TODO Auto-generated method stub
        try
        {
            ConvertUtils.register( new TimestampConverter(  ), Timestamp.class );
            BeanUtils.populate( this, request.getParameterMap(  ) );

            if ( request instanceof MultipartHttpServletRequest )
            {
                BeanUtils.populate( this, ( (MultipartHttpServletRequest) request ).getFileMap(  ) );
            }
        }
        catch ( IllegalAccessException e )
        {
            AppLogService.error( e.getMessage(  ) );
        }
        catch ( InvocationTargetException e )
        {
            AppLogService.error( e.getMessage(  ) );
        }
    }

    protected ReferenceList toReferenceList( List<?> list, String key, String value, boolean isInt )
    {
        return ReferenceList.convert( list, key, value, isInt );
    }

    public void setAfaire( String aFaire )
    {
        this._aFaire = aFaire;
    }

    public String getAfaire(  )
    {
        return _aFaire;
    }

    public void testAOP(  )
    {
        AppLogService.info( "Test AOP" );
    }

    protected boolean isEmptyOrEquals( Object o1, Object o2 )
    {
        return isEmpty( o1 ) || o1.equals( o2 );
    }

    protected boolean isEmpty( Object o1 )
    {
        return ( o1 == null ) || OdsConstants.CONSTANTE_CHAINE_VIDE.equals( o1 );
    }

    public void setFormName( String formName )
    {
        this._formName = formName;
    }

    public String getFormName(  )
    {
        return _formName;
    }

    public void setDefaultAction( String defaultAction )
    {
        this._defaultAction = defaultAction;
    }

    public String getDefaultAction(  )
    {
        return _defaultAction;
    }

    public void setReferentiel( Map<String, ReferenceList> mapReferentiel )
    {
        this._mapReferentiel = mapReferentiel;
    }

    protected ReferenceList getReferenceList( String key )
    {
        ReferenceList result = _mapReferentiel.get( key );

        if ( result == null )
        {
            result = new ReferenceList(  );
        }

        return result;
    }

    /**
     * Permet d'ajouter un ou plusieurs champs (Field) apres le champ d'identifiant wantedId dans la liste list
     *
     * @param list
     * @param wantedId
     * @param addedFields
     */
    protected void addAfter( List<Field> list, final String wantedId, Field... addedFields )
    {
        Field f = new AbstractField<Void>(  )
            {
                @Override
                public String getId(  )
                {
                    return wantedId;
                }

                public String render(  )
                {
                    // TODO Auto-generated method stub
                    return null;
                }
            };

        int insertedPosition = list.indexOf( f );

        for ( Field insertedField : addedFields )
        {
            insertedPosition++;
            list.add( insertedPosition, insertedField );
        }
    }

    /**
     * Permet d'ajouter une liste de champs (Field) apres le champ d'identifiant wantedId dans la liste list
     *
     * @param list
     * @param wantedId
     * @param addedFields
     */
    protected void addAllAfter( List<Field> list, final String wantedId, List<Field> addedFields )
    {
        addAfter( list, wantedId, addedFields.toArray( new Field[addedFields.size(  )] ) );
    }

    protected void validateFile( FileItem file, Map<String, String> errorMap, List<String> champsNonSaisis,
        String[] messages )
    {
        if ( ( file == null ) || ( file.getName(  ) == null ) ||
                OdsConstants.CONSTANTE_CHAINE_VIDE.equals( file.getName(  ) ) )
        {
            champsNonSaisis.add( messages[PROPERTY_MANQUANT] );
        }
        else if ( ( file.getName(  ).length(  ) <= 3 ) ||
                !OdsConstants.CONSTANTE_PDF.equals( file.getName(  ).substring( file.getName(  ).length(  ) - 3 )
                                                            .toUpperCase(  ) ) )
        {
            champsNonSaisis.add( messages[PROPERTY_EXTENSION] );
        }
    }

    protected abstract List<String> getReadonlyFields(  );

    /**
     * Permet de rechercher le champ d'identifiant wantedId dans la liste list
     *
     * @param list
     * @param wantedId
     */
    protected Field find( List<Field> list, final String wantedId )
    {
        Field f = new AbstractField<Void>(  )
            {
                @Override
                public String getId(  )
                {
                    return wantedId;
                }

                public String render(  )
                {
                    // TODO Auto-generated method stub
                    return null;
                }
            };

        int insertedPosition = list.indexOf( f );

        return ( insertedPosition < 0 ) ? null : list.get( insertedPosition );
    }
}
