/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.utils.business;

import com.lowagie.text.DocumentException;
import com.lowagie.text.pdf.PRAcroForm;
import com.lowagie.text.pdf.PdfCopy;
import com.lowagie.text.pdf.PdfImportedPage;
import com.lowagie.text.pdf.PdfReader;

import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.util.xml.XmlUtil;

import java.io.IOException;
import java.io.OutputStream;

import java.util.Locale;

import javax.servlet.http.HttpServletRequest;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;


/**
 * Classe utilitaire
 *
 */
public final class OdsUtils
{
    /**
     * Constructeur vide
     */
    private OdsUtils(  )
    {
        // rien
    }

    /**
     *
     * @param strXmlBuffer buffer
     * @param strTag tag gnr dans le XML
     * @param strValue Valeur de la balise
     */
    public static void addElement( StringBuffer strXmlBuffer, String strTag, String strValue )
    {
        if ( ( strValue != null ) && !strValue.equals( "null" ) &&
                !strValue.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            XmlUtil.addElementHtml( strXmlBuffer, strTag, strValue );
        }
        else
        {
            XmlUtil.addElementHtml( strXmlBuffer, strTag, OdsConstants.CONSTANTE_CHAINE_VIDE );
        }
    }

    /**
     *
     * @param strXmlBuffer buffer
     * @param strTag tag gnr dans le XML
     * @param strValue Valeur de la balise
     */
    public static void addElement( StringBuffer strXmlBuffer, String strTag, int strValue )
    {
        XmlUtil.addElement( strXmlBuffer, strTag, strValue );
    }

    /**
     * Renvoie l'entier dsign par le paramtre donn dans la requte
     * @param strParameterName le nom du paramtre ( OdsParameters )
     * @param request la requete Http
     * @return l'entier dsign par le paramtre donn dans la requte
     */
    public static int getIntegerParameter( String strParameterName, HttpServletRequest request )
    {
        int nRetour = -1;

        if ( ( request != null ) && ( strParameterName != null ) &&
                !strParameterName.trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) &&
                ( request.getParameter( strParameterName ) != null ) &&
                !request.getParameter( strParameterName ).trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            try
            {
                nRetour = Integer.parseInt( request.getParameter( strParameterName ) );
            }
            catch ( NumberFormatException nfe )
            {
                AppLogService.error( nfe );
            }
        }

        return nRetour;
    }

    /**
     * Renvoie l'entier dsign par la chaine de caractres
     * @param strInteger l'entier sous forme de chaine de caractres
     * @return l'entier dsign par la chaine de caractres
     */
    public static int getInteger( String strInteger )
    {
        int nRetour = -1;

        if ( ( strInteger != null ) && !strInteger.trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            try
            {
                nRetour = Integer.parseInt( strInteger );
            }
            catch ( NumberFormatException nfe )
            {
                AppLogService.error( nfe );
            }
        }

        return nRetour;
    }

    /**
     * Teste si un objet vaut NULL et renvoie une exception si c'est le cas
     * @param o l'objet  tester
     * @param request la requte Http
     */
    public static void testNullObject( Object o, HttpServletRequest request )
    {
        if ( o == null )
        {
            throw new AppException( I18nService.getLocalizedString( OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    request.getLocale(  ) ) );
        }
    }

    /**
     * Teste si un objet vaut NULL et renvoie une exception si c'est le cas.
     * L'exception est renvoye en Francais.
     * @param o l'objet  tester
     */
    public static void testNullObject( Object o )
    {
        if ( o == null )
        {
            throw new AppException( I18nService.getLocalizedString( OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    Locale.FRANCE ) );
        }
    }

    /**
     * Retourne la valeur d'un parametre de requete.
     *
     * Cette methode est  utiliser lors de la rcupration des paramtres pour un filtre :
     * elle retounre la valeur all si le paramtre n'est pas renseign
     * @param key
     * @param request
     * @return
     */
    public static String getFilterParameter( String key, HttpServletRequest request )
    {
        String strValue = request.getParameter( key );

        if ( ( strValue == null ) || OdsConstants.FILTER_ALL.equals( strValue ) )
        {
            // TODO externaliser la constante all
            strValue = "all";
        }

        return strValue;
    }

    /**
     * Teste si un parametre est prsent en request
     * @param key
     * @param request
     * @return true si le parametre est prsent, false sinon
     */
    public static boolean isParameterInRequest( String key, HttpServletRequest request )
    {
        return request.getParameter( key ) != null;
    }

    /**
     * Retourne la valeur dans la request si elle est prsente.
     * Retourne la valeur dans la session sinon.
     * @param id
     * @param sessionVariable
     * @param request
     * @return
     */
    public static int getParameter( String id, int sessionVariable, HttpServletRequest request )
    {
        if ( OdsUtils.isParameterInRequest( id, request ) )
        {
            return OdsUtils.getIntegerParameter( id, request );
        }

        return sessionVariable;
    }

    /**
     * Retourne la valeur dans la request si elle est prsente.
     * Retourne la valeur dans la session sinon.
     * @param id
     * @param sessionVariable
     * @param request
     * @return
     */
    public static String getParameter( String id, String sessionVariable, HttpServletRequest request )
    {
        if ( OdsUtils.isParameterInRequest( id, request ) )
        {
            return OdsUtils.getFilterParameter( id, request );
        }

        return sessionVariable;
    }

    /**
     * genere un fichier xml correspondant a la representation de l'objet pass en parametre.
     * @param object l'objet a partir du quel doit etre gnr le fichier xml
     * @param strPackage le package necessaire a la generation du fichier xml
     * @param outputStream outputstream dans lequel le fichier xml doit etre gnr
     * @throws JAXBException jAXBException
     * @throws IOException iOException
     */
    public static void marshal( Object object, String strPackage, OutputStream outputStream )
        throws JAXBException, IOException
    {
        /* create a JAXBContext capable of handling classes generated into */
        JAXBContext jc = JAXBContext.newInstance( strPackage );

        /* create an marshaller */
        Marshaller marshaller = jc.createMarshaller(  );

        /* unmarshal an invalid po instance document into a tree of Java
         * content objects composed of classes from the primer.po package */

        //creation de la liste des directions
        marshaller.marshal( object, outputStream );
        outputStream.close(  );
    }

    /**
     * cette methode ajoute a la suite du fichier pdfWriter le fichier pdfReader
     * @param pdfReader le fichier  concatener au fichier pdfWriter
     * @param pdfWriter le fichier dans lequel on ajoute le contenu du fichier pdfReader
     * @param nPageOffset le nombre de page du fichier pdfWriter
     * @return le nombre de page du fichier pdfWriter apres concatenation
     * @throws DocumentException exception sur les documents
     * @throws IOException exception sur les entres sorties
     */
    public static int concatenePdfReader( PdfReader pdfReader, PdfCopy pdfWriter, int nPageOffset )
        throws DocumentException, IOException
    {
        // we retrieve the total number of pages
        int n = pdfReader.getNumberOfPages(  );

        //on ajoute le contenu de pdfReader au contenu de pdfWriter
        PdfImportedPage page;

        for ( int i = 0; i < n; )
        {
            ++i;
            page = pdfWriter.getImportedPage( pdfReader, i );
            pdfWriter.addPage( page );
        }

        PRAcroForm form = pdfReader.getAcroForm(  );

        if ( form != null )
        {
            pdfWriter.copyAcroForm( pdfReader );
        }

        return ( nPageOffset + n );
    }
}
