/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.utils.commons;

import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;

import java.io.IOException;
import java.io.OutputStream;

import java.net.SocketException;


public final class FTPUtils
{
    /**
     * Constructeur vide
     */
    private FTPUtils(  )
    {
        // rien
    }

    /**
    * Test si l'ensemble des parametres de configarations ncessaires au transfert
    * vers ODS CP(parametres FTP,configuration du  repertoire locale...)
    * ont t renseigns dans le fichier ods-ma.properties
    * @param strHost strHost
    * @param strUserName strUserName
    * @param strPassword strPassword
    * @param strCurentDirectory strCurentDirectory
    * @param strLockDirectory strLockDirectory
    * @param strLockFilePublication strLockFilePublication
    * @param strLockFileTransfert strLockFileTransfert
    * @param strLocalDirectory strLocalDirectory
    * @return  true si l'ensemble des parametres ont t renseign, false sinon
    */
    public static boolean allFtpParametersRequiered( String strHost, String strUserName, String strPassword,
        String strCurentDirectory, String strLockFilePublication, String strLockFileTransfert, String strLocalDirectory )
    {
        boolean bFtpParametersRequiered = true;

        if ( strHost == null )
        {
            bFtpParametersRequiered = false;
        }

        if ( strUserName == null )
        {
            bFtpParametersRequiered = false;
        }

        if ( strPassword == null )
        {
            bFtpParametersRequiered = false;
        }

        if ( strCurentDirectory == null )
        {
            bFtpParametersRequiered = false;
        }

        if ( strLockFilePublication == null )
        {
            bFtpParametersRequiered = false;
        }

        if ( strLockFileTransfert == null )
        {
            bFtpParametersRequiered = false;
        }

        if ( strLocalDirectory == null )
        {
            bFtpParametersRequiered = false;
        }

        return bFtpParametersRequiered;
    }

    /**
    *
    * @param strHost strHost
    * @param strUserName strUserName
    * @param strPassword strPassword
    * @return FTPClient
    * @throws SocketException socketException
    * @throws IOException iOException
    */
    public static FTPClient ftpConnexion( String strHost, String strUserName, String strPassword )
        throws SocketException, IOException
    {
        FTPClient ftpClient = new FTPClient(  );
        ftpClient.connect( strHost );
        ftpClient.login( strUserName, strPassword );
        ftpClient.printWorkingDirectory(  );
        ftpClient.enterLocalPassiveMode(  );
        ftpClient.setFileType( FTP.BINARY_FILE_TYPE );

        return ftpClient;
    }

    /**
     * Test si un verrou empechant le transfert est present dans le repertoire strDirLock
     * @param ftpClient ftpClient
     * @param strDirLock le chemin  pour acceder du fichier verrou
     * @param strFileLock le nom du  fichier verrou
     * @return true si un verrou a t pos, false sinon
     * @throws IOException iOException
     */
    public static boolean isLockPublication( FTPClient ftpClient, String strDirLock, String strFileLock )
        throws IOException
    {
        ftpClient.changeWorkingDirectory( strDirLock );

        FTPFile[] fileList = ftpClient.listFiles(  );

        if ( fileList != null )
        {
            for ( int i = 0; i < fileList.length; i++ )
            {
                if ( fileList[i].getName(  ).equals( strFileLock ) )
                {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Positionne le verrou ods dans le repertoire strDirLock
     * @param ftpClient ftpClient
     * @param strDirLock le chemin  pour acceder du fichier verrou
     * @param strFileLock le nom du  fichier verrou
     * @throws IOException IOException
     */
    public static boolean setLockOds( FTPClient ftpClient, String strDirLock, String strFileLock )
        throws IOException
    {
        ftpClient.changeWorkingDirectory( strDirLock );

        OutputStream outputStream = ftpClient.storeFileStream( strFileLock );

        if ( outputStream != null )
        {
            outputStream.close(  );

            return false;
        }

        return true;
    }

    /**
     * Supprime  le verrou ods dans le repertoire strDirLock
     * @param ftpClient ftpClient
     * @param strDirLock le chemin  pour acceder du fichier verrou
     * @param strFileLock le nom du  fichier verrou
     * @return true si la suppression du verrou a bien t effectu, false sinon
     * @throws IOException
     * @throws SocketException
     * @throws IOException
     */
    public static boolean removeLockOds( FTPClient ftpClient, String strDirLock, String strFileLock, String strHost,
        String strUserName, String strPassword ) throws SocketException, IOException
    {
        boolean bResult = false;

        FTPClient newFtpCLient = ftpClient;

        if ( ( ftpClient == null ) || !ftpClient.isConnected(  ) )
        {
            newFtpCLient = FTPUtils.ftpConnexion( strHost, strUserName, strPassword );
        }

        if ( ( newFtpCLient != null ) && newFtpCLient.isConnected(  ) )
        {
            bResult = newFtpCLient.changeWorkingDirectory( strDirLock );
            bResult = newFtpCLient.deleteFile( strFileLock );
            newFtpCLient.disconnect(  );
        }

        return bResult;
    }
}
