/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.utils.commons;

import de.schlichtherle.io.File;
import de.schlichtherle.io.FileInputStream;

import de.schlichtherle.util.zip.ZipEntry;
import de.schlichtherle.util.zip.ZipOutputStream;

import fr.paris.lutece.portal.service.util.AppLogService;

import java.io.IOException;
import java.io.OutputStream;


public final class FileUtils
{
    /**
     * Constructeur vide
     */
    private FileUtils(  )
    {
        // rien
    }

    /**
     * Cette mthode supprime les fichiers d'un rpertoire
     * @param strPath le rpertoire
     */
    public static void deleteDirectory( String strPath, boolean bIsDeleteDirectory )
    {
        File path = new File( strPath );

        if ( path.exists(  ) )
        {
            File[] files = (File[]) path.listFiles(  );

            for ( File fFile : files )
            {
                if ( fFile.isDirectory(  ) )
                {
                    deleteDirectory( fFile.getAbsolutePath(  ), bIsDeleteDirectory );
                }
                else
                {
                    fFile.delete(  );
                }
            }

            if ( bIsDeleteDirectory )
            {
                path.delete(  );
            }
        }
    }

    /**
     * Permet de crer un zip  partir d'un dossier de fichier
     * @param dossier Rpertoire contenant les fichiers  archiver
     * @param out Le flux
     */
    public static void createZip( String dossier, OutputStream out )
    {
        File tmpDir = new File( dossier );

        try
        {
            ZipOutputStream zout = new ZipOutputStream( out, "IBM437" );
            compressProfondeur( (File[]) tmpDir.listFiles(  ), zout, dossier );

            zout.finish(  );
            zout.close(  );
        }
        catch ( Exception e )
        {
            AppLogService.error( "Creation Zip  : " + e.getMessage(  ) );
        }
    }

    /**
      * compresse un fichier
      *
      * @param srcs le fichier a compresser
      * @param zout le rsultat de la compression
      * @param dossier la destination du fichier decompress
      * @throws IOException IOException
      */
    public static void compressProfondeur( File[] srcs, ZipOutputStream zout, String dossier )
        throws IOException
    {
        for ( File fFile : srcs )
        {
            String strEntryPath = fFile.getPath(  ).substring( dossier.length(  ) + 1, fFile.getPath(  ).length(  ) );

            if ( fFile.isDirectory(  ) )
            {
                compressProfondeur( (File[]) fFile.listFiles(  ), zout, dossier );
            }
            else
            {
                ZipEntry ze = new ZipEntry( strEntryPath );
                ze.setSize( fFile.length(  ) );
                zout.putNextEntry( ze );

                FileInputStream in = new FileInputStream( fFile );
                byte[] bTab = new byte[4096];
                int nLu = -1;

                do
                {
                    nLu = in.read( bTab );

                    if ( nLu > 0 )
                    {
                        zout.write( bTab, 0, nLu );
                    }
                }
                while ( nLu > 0 );

                in.close(  );
                zout.closeEntry(  );
            }
        }
    }
}
