/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.utils.commons;

import fr.paris.lutece.portal.service.util.AppLogService;

import org.bouncycastle.jce.provider.JDKMessageDigest;

import org.bouncycastle.util.encoders.Hex;


/**
 * Classe utilitaire
 *
 */
public final class ShaUtils
{
    /**
     * Constructeur vide
     */
    private ShaUtils(  )
    {
    }

    /**
     * Gnration de la signature SHA 224 correspondant au tableau de byte pass en paramtre
     * @param input le tableau de byte  signer
     * @return Une chane de caractres reprsentant la signature
     */
    public static String toSha224String( byte[] input )
    {
        String strResult = null;

        try
        {
            JDKMessageDigest.SHA224 digest = new JDKMessageDigest.SHA224(  );
            digest.engineUpdate( input, 0, input.length );

            byte[] empreinte = Hex.encode( digest.engineDigest(  ) );
            strResult = new String( empreinte );
        }
        catch ( Exception e )
        {
            AppLogService.error( e );
        }

        return strResult;
    }

    /**
     * Gnration de la signature SHA 224 correspondant  la chane de caractres passe en paramtre
     * @param strInput la chaine  signer
     * @return Une chane de caractres reprsentant la signature
     */
    public static String toSha224String( String strInput )
    {
        String strResult = null;

        try
        {
            JDKMessageDigest.SHA224 digest = new JDKMessageDigest.SHA224(  );
            digest.engineUpdate( strInput.getBytes(  ), 0, strInput.getBytes(  ).length );

            byte[] empreinte = Hex.encode( digest.engineDigest(  ) );
            strResult = new String( empreinte );
        }
        catch ( Exception e )
        {
            AppLogService.error( e );
        }

        return strResult;
    }
}
