/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.utils.commons;

import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.util.string.StringUtil;

import org.apache.commons.fileupload.FileItem;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;


/**
 * Classe utilitaire
 *
 */
public final class StringUtils
{
    private static final String CONSTANTE_SPLIT_CHAINE = ",| ";

    /**
     * Constructeur vide
     *
     */
    private StringUtils(  )
    {
    }

    /**
     * Compare deux chaine de caractre en considrant les suites de caractres numriques
    * comme des nombres et non comme une chane de caractre ascii
    * @param chaine1 premire chane  comparer
    * @param chaine2 deuxime chane  comparer
    * @return entier : < 0 si chaine1<chaine2
    *                   0 si les deux chanes sont gales
    *                   > 0 si chaine2>chaine1
     */
    public static int compare( String chaine1, String chaine2 )
    {
        String strChaine1LC = chaine1.toLowerCase(  );
        String strChaine2LC = chaine2.toLowerCase(  );

        int chaine1FirstIndex = 0;
        int chaine2FirstIndex = 0;

        int compare = 0;

        String chaine1Block = strChaine1LC;
        String chaine2Block = strChaine2LC;

        boolean stop = false;

        while ( ( compare == 0 ) && !stop )
        {
            int endIndex1 = 0;

            for ( int i = chaine1FirstIndex; i < strChaine1LC.length(  ); i++ )
            {
                char currentChar = strChaine1LC.charAt( i );
                endIndex1 = i + 1;

                if ( i < ( strChaine1LC.length(  ) - 1 ) )
                {
                    char nextChar = strChaine1LC.charAt( i + 1 );

                    if ( Character.getType( currentChar ) != Character.getType( nextChar ) )
                    {
                        break;
                    }
                }
            }

            if ( endIndex1 == strChaine1LC.length(  ) )
            {
                stop = true;
            }

            chaine1Block = strChaine1LC.substring( chaine1FirstIndex, endIndex1 );
            chaine1FirstIndex = endIndex1;

            int endIndex2 = 0;

            for ( int i = chaine2FirstIndex; i < strChaine2LC.length(  ); i++ )
            {
                char currentChar = strChaine2LC.charAt( i );

                endIndex2 = i + 1;

                if ( i < ( strChaine2LC.length(  ) - 1 ) )
                {
                    char nextChar = strChaine2LC.charAt( i + 1 );
                    endIndex2 = i + 1;

                    if ( Character.getType( currentChar ) != Character.getType( nextChar ) )
                    {
                        break;
                    }
                }
            }

            if ( endIndex2 == strChaine2LC.length(  ) )
            {
                stop = true;
            }

            chaine2Block = strChaine2LC.substring( chaine2FirstIndex, endIndex2 );
            chaine2FirstIndex = endIndex2;

            if ( ( chaine1Block.trim(  ).length(  ) > 0 ) && ( chaine2Block.trim(  ).length(  ) > 0 ) &&
                    org.apache.commons.lang.StringUtils.isNumeric( chaine1Block ) &&
                    org.apache.commons.lang.StringUtils.isNumeric( chaine2Block ) &&
                    (  // Vrifie que les nombres ne dpassent pas la valeur maximale d'un int (2147483647)
                Long.parseLong( chaine1Block.trim(  ) ) <= Integer.MAX_VALUE ) &&
                    ( Long.parseLong( chaine2Block.trim(  ) ) <= Integer.MAX_VALUE ) )
            {
                compare = Integer.parseInt( chaine1Block ) - Integer.parseInt( chaine2Block );
            }
            else
            {
                compare = chaine1Block.compareTo( chaine2Block );
            }
        }

        return compare;
    }

    /**
     * Remplace de la chaine strSource, les caractres spciaux du type ('\t','\r') qui ne sont pas
     * interprts correctement en CSV
     * @param strSource le chaine source
     * @return strResult la nouvelle chaine sans ces caractres spciaux
     */
    public static String substituteSpecialCaractersForCSV( String strSource )
    {
        String strResult = strSource;

        //strResult = StringUtil.substitute(strResult, OdsConstants.CONSTANTE_CHAINE_VIDE, OdsConstants.CONSTANTE_GUILLEMET);
        strResult = StringUtil.substitute( strResult, OdsConstants.CONSTANTE_CHAINE_VIDE,
                OdsConstants.CONSTANTE_CARRIAGE_RETURN );
        strResult = StringUtil.substitute( strResult, OdsConstants.CONSTANTE_TABULATION, OdsConstants.CONSTANTE_TAB );

        return strResult;
    }

    /**
     * Transforme une liste de String en tableau de String
     * @param listField la liste de String contenant les lments  insrer dans le tableau de String
     * @return line le tableau de String cr  parti de la liste
     */
    public static String[] transformListToTab( List<String> listField )
    {
        int i = 0;
        String[] line = new String[listField.size(  )];

        for ( String strField : listField )
        {
            if ( strField != null )
            {
                line[i] = strField;
            }
            else
            {
                line[i] = OdsConstants.CONSTANTE_CHAINE_VIDE;
            }

            i++;
        }

        return line;
    }

    /**
     * Retourne le nom court du fichier contenu dans la methode getName() de l'objet FileItem
     * Par exemple, C:\temp\fichier.txt est le path du fichier.
     * La methode retourne "fichier"
     * @param fileItem Fichier tlcharger
     * @return String le nom court du fichier
     */
    public static String getNameCourt( FileItem fileItem )
    {
        String nomCourt = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( fileItem != null )
        {
            String nomLong = fileItem.getName(  );

            if ( nomLong != null )
            {
                int iIndexOfSeparator = -1;
                iIndexOfSeparator = nomLong.lastIndexOf( "/" );

                if ( iIndexOfSeparator >= 0 )
                {
                    nomCourt = nomLong.substring( iIndexOfSeparator + 1, nomLong.lastIndexOf( '.' ) );
                }
                else
                {
                    iIndexOfSeparator = nomLong.lastIndexOf( "\\" );

                    if ( iIndexOfSeparator >= 0 )
                    {
                        nomCourt = nomLong.substring( iIndexOfSeparator + 1, nomLong.lastIndexOf( '.' ) );
                    }
                    else
                    {
                        nomCourt = nomLong.substring( 0, nomLong.indexOf( '.' ) );
                    }
                }
            }
        }

        return nomCourt;
    }

    /**
     * Renvoie l'entier dsign par le paramtre donn dans la requte
     * @param strParameterName le nom du paramtre ( OdsParameters )
     * @return l'entier dsign par le paramtre donn dans la requte
     */
    public static int string2id( String strParameterName )
    {
        int nRetour = -1;

        if ( ( strParameterName != null ) && !strParameterName.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            try
            {
                nRetour = Integer.parseInt( strParameterName );
            }
            catch ( NumberFormatException nfe )
            {
                AppLogService.error( nfe );
            }
        }

        return nRetour;
    }

    /**
     * Dcoupe une chaine et la transforme en liste.
     * Le "split" se fait sur la valeur <b>&quote;,| &quote;</b>.
     * @param strToSplit la chaine a decouper
     * @return une liste vide si la chaine est <b>null</b> ou <b>vide</b>,
     */
    public static List<String> splitStringToList( String strToSplit )
    {
        if ( ( strToSplit == null ) || ( strToSplit.length(  ) == 0 ) )
        {
            return Collections.emptyList(  );
        }

        String[] stringArray = strToSplit.split( CONSTANTE_SPLIT_CHAINE );

        if ( stringArray == null )
        {
            return Collections.emptyList(  );
        }

        return Arrays.asList( stringArray );
    }
}
