/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web;

import fr.paris.lutece.plugins.ods.business.fichier.FichierPhysiqueHome;
import fr.paris.lutece.plugins.ods.business.fichier.IFichierHome;
import fr.paris.lutece.plugins.ods.dto.fichier.FichierPhysique;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter;
import fr.paris.lutece.plugins.ods.dto.fichier.ITypeDocument;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.portal.service.util.AppLogService;

import org.springframework.beans.factory.annotation.Autowired;

import java.io.IOException;
import java.io.OutputStream;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


/**
 * Cette classe permet de telecharger un fichier
 *
 */
public abstract class AbstractDownloadFile<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>, GFichierFilter extends IFichierFilter, GTypeDocument extends ITypeDocument>
    implements IDownloadFile
{
    private static final String PROPERTY_ERROR_MESSAGE = "ods.message.errorDownloadFile";
    @Autowired
    private IFichierHome<GFichier, GFichierFilter, GTypeDocument, GSeance> _fichierHome;
    @Autowired
    private FichierPhysiqueHome _fichierPhysiqueHome;

    protected abstract Plugin getPlugin(  );

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.IDownloadFile#doDowloadFichier(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
         */
    public void doDowloadFichier( HttpServletRequest request, HttpServletResponse response )
        throws Exception
    {
        int nIdFichier = OdsUtils.getIntegerParameter( OdsParameters.ID_FICHIER, request );

        if ( nIdFichier != -1 )
        {
            Plugin plugin = getPlugin(  );

            GFichier fichierTelecharger = _fichierHome.findByPrimaryKey( nIdFichier, plugin );

            if ( ( fichierTelecharger != null ) && fichierTelecharger.getEnLigne(  ) )
            {
                FichierPhysique fichierPhysique = _fichierPhysiqueHome.findByPrimaryKey( fichierTelecharger.getFichier(  )
                                                                                                           .getIdFichier(  ),
                        plugin );

                if ( fichierPhysique != null )
                {
                    try
                    {
                        String strFileName = fichierTelecharger.getNom(  ) + "." + fichierTelecharger.getExtension(  );

                        response.setHeader( "Content-disposition", "inline; filename=\"" + strFileName + "\"" );
                        response.setContentType( "application/pdf" );
                        response.setHeader( "Pragma", "public" );
                        response.setHeader( "Expires", "0" );
                        response.setHeader( "Cache-Control", "must-revalidate,post-check=0,pre-check=0" );
                        response.addHeader( "Accept-Ranges", "bytes" );
                        response.setContentLength( (int) fichierPhysique.getDonnees(  ).length );

                        OutputStream os = response.getOutputStream(  );
                        os.write( fichierPhysique.getDonnees(  ) );
                        os.flush(  );
                        os.close(  );
                    }
                    catch ( IOException e )
                    {
                        throw new AppException( I18nService.getLocalizedString( PROPERTY_ERROR_MESSAGE,
                                request.getLocale(  ) ) );
                    }
                }
                else
                {
                    AppLogService.error( 
                        "fr.paris.lutece.plugins.ods.web.DownloadFile : le fichier physique n'existe pas" );
                    throw new AppException( I18nService.getLocalizedString( PROPERTY_ERROR_MESSAGE,
                            request.getLocale(  ) ) );
                }
            }
            else
            {
                AppLogService.error( 
                    "fr.paris.lutece.plugins.ods.web.DownloadFile : le fichier n'existe pas ou n'est pas publie" );
                throw new AppException( I18nService.getLocalizedString( PROPERTY_ERROR_MESSAGE, request.getLocale(  ) ) );
            }
        }
    }
}
