/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web;

import fr.paris.lutece.plugins.ods.business.odscheckapp.generated.checkapp.Apptest;
import fr.paris.lutece.plugins.ods.business.odscheckapp.generated.checkapp.ModuleType;
import fr.paris.lutece.plugins.ods.business.odscheckapp.generated.checkapp.TestsType;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.database.AppConnectionService;
import fr.paris.lutece.portal.service.spring.SpringContextService;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.service.util.AppPropertiesService;
import fr.paris.lutece.portal.web.admin.PluginAdminPageJspBean;
import fr.paris.lutece.util.pool.service.ConnectionService;

import net.sf.jcopist.service.IDocumentService;

import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;

import java.sql.Connection;

import java.util.Collection;
import java.util.Properties;

import javax.jms.Queue;
import javax.jms.QueueConnectionFactory;

import javax.naming.Context;
import javax.naming.InitialContext;

import javax.servlet.http.HttpServletResponse;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;


/**
 * CheckJspBean.
 * Classe permettant de tester les diffrentes applications
 * auxquelles se connecte le plugin ODS via le front-office et/ou le back-office
 */
public class CheckJspBean extends PluginAdminPageJspBean
{
    private static final long serialVersionUID = 7896248673638257863L;
    private static final String CONSTANTE_PACKAGE_CHECKAPP = "fr.paris.lutece.plugins.ods.business.odscheckapp.generated.checkapp";
    private static final String CONSTANTE_ERROR_MESSAGE_SEPARATOR = " : ";

    /**
     * Teste les applications auxquelles se connecte le plugin ODS : Bases de donnes, Joram, JCopist.
     * @param response la rponse HTTP
     * @return un XML reprsentant les rsultats des tests
     */
    public static String test( HttpServletResponse response )
    {
        String strResult = null;

        response.addHeader( "Content-Disposition", "inline" );
        response.addHeader( "Content-Type", "text/xml" );

        /*
         * On rcupre les proprits permettant de savoir si on teste ou pas une application :
         * 1 = OUI
         * 0 = NON
         */
        int bCheckDB = AppPropertiesService.getPropertyInt( "ods.check.databases", 1 );
        int bCheckJoram = AppPropertiesService.getPropertyInt( "ods.check.joram", 1 );
        int bCheckJCopist = AppPropertiesService.getPropertyInt( "ods.check.jcopist", 1 );

        Apptest appTest = new Apptest(  );
        appTest.setName( OdsConstants.CONSTANTE_PLUGIN_ODS );

        TestsType tests = new TestsType(  );

        boolean bOdsState = true;

        /* Test des bases de donnes */
        if ( bCheckDB == 1 )
        {
            bOdsState = checkDatabases( tests ) && bOdsState;
        }

        /* Test de Joram */
        if ( bCheckJoram == 1 )
        {
            bOdsState = checkJoram( tests ) && bOdsState;
        }

        /* Test de JCopist */
        if ( bCheckJCopist == 1 )
        {
            bOdsState = checkJCopist( tests ) && bOdsState;
        }

        appTest.setTests( tests );

        /* Si tous les tests sont bons on affecte la valeur "ok"  l'tat du plugin */
        if ( bOdsState )
        {
            appTest.setState( "ok" );
        }

        /* Sinon on affecte la valeur "error" */
        else
        {
            appTest.setState( "error" );
        }

        /* Gnration du XML avec JAXB */
        StringWriter writer = new StringWriter(  );

        try
        {
            marshal( appTest, CONSTANTE_PACKAGE_CHECKAPP, writer );
            strResult = writer.toString(  );
        }
        catch ( IOException ioe )
        {
            AppLogService.error( "ODS Check App : Erreur lors de la fermeture du Writer XML" );
        }

        return strResult;
    }

    /**
     * Teste les pool de bases de donnes.
     * @param tests l'objet TestsType que l'on complte
     * @return TRUE si le test s'est bien droul, FALSE sinon
     */
    private static boolean checkDatabases( TestsType tests )
    {
        boolean bResult = true;

        /* On rcupre la liste des pools de connexions aux bases de donnes */
        Collection<ConnectionService> listPool = AppConnectionService.getPoolManager(  ).getPools(  );

        for ( ConnectionService cs : listPool )
        {
            String strPoolName = cs.getPoolName(  );
            Connection con = cs.getConnection(  );

            ModuleType module = new ModuleType(  );
            module.setName( "Pool Database : " + strPoolName );

            /* Pour chaque pool de connexion on teste si la connexion est null ou pas */
            try
            {
                if ( con != null )
                {
                    cs.freeConnection( con );
                }

                /* Ajout de l'tat et du message "ok" */
                appendOK( module );
            }
            catch ( Exception e )
            {
                /* Ajout de l'tat "error" et du message d'erreur */
                appendError( module, e.getClass(  ).toString(  ) + CONSTANTE_ERROR_MESSAGE_SEPARATOR +
                    e.getMessage(  ) );
                bResult = false;
            }

            tests.getModule(  ).add( module );
        }

        return bResult;
    }

    /**
     * Teste la connexion au serveur JORAM.
     * @param tests l'objet TestsType que l'on complte
     * @return TRUE si le test s'est bien droul, FALSE sinon
     */
    private static boolean checkJoram( TestsType tests )
    {
        boolean bResult = true;

        /* Proprits de connexion  JORAM */
        String strPropertyJndiInitial = "ods.java.naming.factory.initial";
        String strDefaultPropertyJndiInitial = "fr.dyade.aaa.jndi2.client.NamingContextFactory";
        String strPropertyJndiHost = "ods.java.naming.factory.host";
        String strPropertyJndiPort = "ods.java.naming.factory.port";
        String strPropertyQueueName = "ods.queue_name";
        String strDefaultPropertyQueueName = "queue";
        String strPropertyConnectionFactory = "ods.connection_factory";
        String strDefaultPropertyConnectionFactory = "qcf";

        ModuleType module = new ModuleType(  );
        module.setName( "JORAM" );

        try
        {
            Properties props = new Properties(  );
            props.setProperty( strPropertyJndiInitial,
                AppPropertiesService.getProperty( strPropertyJndiInitial, strDefaultPropertyJndiInitial ) );
            props.setProperty( strPropertyJndiHost, AppPropertiesService.getProperty( strPropertyJndiHost ) );
            props.setProperty( strPropertyJndiPort, AppPropertiesService.getProperty( strPropertyJndiPort ) );

            Context ictx = new InitialContext( props );

            /* Initialisation de la queue */
            Queue queue = (Queue) ictx.lookup( AppPropertiesService.getProperty( strPropertyQueueName,
                        strDefaultPropertyQueueName ) );

            if ( queue == null )
            {
                /* Ajout de l'tat "error" et du message d'erreur si la queue est null */
                appendError( module, "Queue null" );
                bResult = false;

                return bResult;
            }

            /* Initialisation de la connexion  la queue */
            QueueConnectionFactory qcf = (QueueConnectionFactory) ictx.lookup( AppPropertiesService.getProperty( 
                        strPropertyConnectionFactory, strDefaultPropertyConnectionFactory ) );

            if ( qcf == null )
            {
                /* Ajout de l'tat "error" et du message d'erreur si la connexion est null */
                appendError( module, "QueueConnectionFactory null" );
                bResult = false;

                return bResult;
            }

            ictx.close(  );

            /* Ajout de l'tat et du message "ok" */
            appendOK( module );
        }
        catch ( Exception e )
        {
            /* Ajout de l'tat "error" et du message d'erreur */
            appendError( module, e.getClass(  ).toString(  ) + CONSTANTE_ERROR_MESSAGE_SEPARATOR + e.getMessage(  ) );
            bResult = false;
        }

        tests.getModule(  ).add( module );

        return bResult;
    }

    /**
     * Teste la connexion au serveur JCopist.
     * @param tests l'objet TestsType que l'on complte
     * @return TRUE si le test s'est bien droul, FALSE sinon
     */
    private static boolean checkJCopist( TestsType tests )
    {
        boolean bResult = true;

        ModuleType module = new ModuleType(  );
        module.setName( "JCopist" );

        /* On instancie une nouvelle connexion au service JCopist */
        try
        {
            IDocumentService documentService = (IDocumentService) SpringContextService.getPluginBean( OdsConstants.CONSTANTE_PLUGIN_ODS,
                    "documentService" );
            documentService.getPrintersNameList(  );

            /* Ajout de l'tat et du message "ok" */
            appendOK( module );
        }
        catch ( Exception e )
        {
            /* Ajout de l'tat "error" et du message d'erreur */
            appendError( module, e.getClass(  ).toString(  ) + CONSTANTE_ERROR_MESSAGE_SEPARATOR + e.getMessage(  ) );
            bResult = false;
        }

        tests.getModule(  ).add( module );

        return bResult;
    }

    /**
     * Ajout l'tat et le message "ok" au module dcrit pour le XML.
     * @param module le module  complter
     */
    private static void appendOK( ModuleType module )
    {
        module.setState( "ok" );
        module.setMessage( "ok" );
    }

    /**
     * Ajout l'tat "error" et le message d'erreur donn en paramtre.
     * @param module le module  complter
     * @param strMessage le message d'erreur dcrivant l'erreur
     */
    private static void appendError( ModuleType module, String strMessage )
    {
        module.setState( "error" );
        module.setMessage( strMessage );
    }

    /**
     * Gnre le code XML reprsentant les rsultats des tests  partir de l'objet
     * AppTest via l'utilisation de JAXB.
     * @param object l'objet AppTest  gnrer en XML
     * @param strPackage le package o se trouvent les classes JAXB
     * @param writer le Writer dans lequel est stock le XML
     * @throws IOException lorsqu'une erreur survient pendant que l'on manipule le writer
     */
    private static void marshal( Object object, String strPackage, Writer writer )
        throws IOException
    {
        try
        {
            /* create a JAXBContext capable of handling classes generated into */
            JAXBContext jc = JAXBContext.newInstance( strPackage );

            /* create an marshaller */
            Marshaller marshaller = jc.createMarshaller(  );

            /* Gnration du XML */
            marshaller.marshal( object, writer );
        }
        catch ( JAXBException je )
        {
            writer.flush(  );
            writer.write( 
                "<apptest><name>ODS</name><state>ODS Check App : Erreur lors de la gnration du XML</state><tests/></apptest>" );
            AppLogService.error( "ODS Check App : Erreur lors de la gnration du XML" );
            AppLogService.error( je.getClass(  ) + CONSTANTE_ERROR_MESSAGE_SEPARATOR + je.getMessage(  ) );
        }
        finally
        {
            try
            {
                writer.close(  );
            }
            catch ( IOException ioe )
            {
                AppLogService.error( "ODS Check App : Erreur lors de la fermeture du Writer XML" );
            }
        }
    }
}
