/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web.acte;

import fr.paris.lutece.plugins.ods.dto.acte.IActeFilter;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.service.acte.IActeService;
import fr.paris.lutece.plugins.ods.service.role.ActesResourceIdService;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.business.rbac.RBAC;
import fr.paris.lutece.portal.service.admin.AccessDeniedException;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.rbac.RBACService;
import fr.paris.lutece.portal.service.template.AppTemplateService;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.web.admin.PluginAdminPageJspBean;
import fr.paris.lutece.util.html.HtmlTemplate;

import java.io.ByteArrayOutputStream;
import java.io.IOException;

import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


public abstract class AbstractActeJspBean<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>, GActeFilter extends IActeFilter<GFichier, GSeance>, GActeService extends IActeService<GSeance, GFichier, GActeFilter>>
    extends PluginAdminPageJspBean implements IActeJspBean
{
    private static final long serialVersionUID = 6177352616157957583L;
    private static final String TEMPLATE_CONFIRMATION_SUPPRESSION_ACTE = "admin/plugins/ods/acte/confirmation_suppression_acte.html";
    private static final String TEMPLATE_ANNULATION_ACTE = "admin/plugins/ods/acte/annulation_acte.html";
    private static final String TEMPLATE_CONFIRMATION_FINAL = "admin/plugins/ods/acte/confirmation_finaliser_acte.html";
    private static final String TEMPLATE_CONFIRMATION_RECEPTION = "admin/plugins/ods/acte/confirmation_reception_acte.html";
    private static final String TEMPLATE_CONFIRMATION_REJET = "admin/plugins/ods/acte/confirmation_rejeter_acte.html";
    private static final String TEMPLATE_CONFIRMATION_ERREUR_TRANSMISSION = "admin/plugins/ods/acte/confirmation_erreurTransmission_acte.html";
    private static final String PROPERTY_PAGE_PREPARATION_ACTE = "ods.acte.preparation.page.title";
    private static final String PROPERTY_PAGE_MODIFICATION_ACTE = "ods.acte.modification.page.title";
    private static final String PROPERTY_PAGE_SUPPRESSION_ACTE = "ods.acte.suppression.page.title";
    private static final String PROPERTY_PAGE_VISUALISATION_ACTE = "ods.acte.visualisation.page.title";
    protected static final String MARK_PERMISSION_ERREUR_TRANSMISSION = "permission_erreur_transmission";

    protected abstract String getActeListTemplate(  );

    protected abstract String getCreationActeTemplate(  );

    protected abstract String getModificationActeTemplate(  );

    protected abstract String getVisualisationActeTemplate(  );

    public abstract GActeService getActeService(  );

    public String getRight(  )
    {
        return getActeService(  ).getRight(  );
    }

    /**
     * Initialisation du PDD
     * @param request la requte HTTP
     */
    public void initPDD( HttpServletRequest request )
    {
        String strTypePDD = getActeService(  ).initPDD( request );

        if ( ( strTypePDD != null ) && !strTypePDD.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            try
            {
                init( request, strTypePDD );
            }
            catch ( AccessDeniedException ade )
            {
                AppLogService.error( ade );
            }
        }
    }

    /**
     * retourne l'interface de gestion des actes
     * @param request la requete HTTP
     * @return  String url de l'interface de gestion des actes
     */
    public String getActeList( HttpServletRequest request )
    {
        HashMap<String, Object> model = getActeService(  ).getActeList( request );

        if ( RBACService.isAuthorized( ActesResourceIdService.ACTES, RBAC.WILDCARD_RESOURCES_ID,
                    ActesResourceIdService.PERMISSION_ERREUR_TRANSMISSION, getUser(  ) ) )
        {
            //Si l'utilisateur est autoris  mettre en "Erreur de transmission" les actes, on lui affiche l'icone
            model.put( MARK_PERMISSION_ERREUR_TRANSMISSION, true );
        }

        HtmlTemplate template = AppTemplateService.getTemplate( getActeListTemplate(  ), getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Retourne l'ecran de prparation d'acte
     *
     * @param request la requete HTTP
     * @return String template
     */
    public String getPreparationActe( HttpServletRequest request )
    {
        HtmlTemplate template;

        if ( RBACService.isAuthorized( ActesResourceIdService.ACTES, RBAC.WILDCARD_RESOURCES_ID,
                    ActesResourceIdService.PERMISSION_CREATION, getUser(  ) ) )
        {
            setPageTitleProperty( PROPERTY_PAGE_PREPARATION_ACTE );

            HashMap<String, Object> model = getActeService(  ).getPreparationActe( request );
            template = AppTemplateService.getTemplate( getCreationActeTemplate(  ), getLocale(  ), model );
        }
        else
        {
            template = AppTemplateService.getTemplate( OdsProperties.TEMPLATE_PROFIL_ERROR, getLocale(  ) );
        }

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Retourne l'ecran de visualisation de l'acte suivant le type d'entit
     *
     * @param request la requete HTTP
     * @return String template
     */
    public String getModificationActe( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_PAGE_MODIFICATION_ACTE );

        return getActeService(  ).getModificationActe( request );
    }

    /**
     * Retourne l'ecran de modififcation de l'acte lorsque l'on est sur l'cran de suivi des actes
     *
     * @param request la requete HTTP
     * @return String template
     */
    public String getModificationActePDD( HttpServletRequest request )
    {
        HtmlTemplate template;

        if ( RBACService.isAuthorized( ActesResourceIdService.ACTES, RBAC.WILDCARD_RESOURCES_ID,
                    ActesResourceIdService.PERMISSION_MISE_A_JOUR, getUser(  ) ) )
        {
            setPageTitleProperty( PROPERTY_PAGE_MODIFICATION_ACTE );

            HashMap<String, Object> model = getActeService(  ).getModificationActePDD( request );
            template = AppTemplateService.getTemplate( getModificationActeTemplate(  ), getLocale(  ), model );
        }
        else
        {
            template = AppTemplateService.getTemplate( OdsProperties.TEMPLATE_PROFIL_ERROR, getLocale(  ) );
        }

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Retourne l'ecran de visualisation d'acte pour un pdd
     *
     * @param request la requete HTTP
     * @return String template
     */
    public String getVisualisationActe( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_PAGE_VISUALISATION_ACTE );

        HashMap<String, Object> model = getActeService(  ).getVisualisationActe( request );
        HtmlTemplate template = AppTemplateService.getTemplate( getVisualisationActeTemplate(  ), getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Retourne l'ecran de confirmation de finalisation
     * @param request la requete HTTP
     * @return String template
     */
    public String getConfirmFinaliser( HttpServletRequest request )
    {
        HashMap<String, Object> model = getActeService(  ).getConfirmFinaliser( request );
        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_CONFIRMATION_FINAL, getLocale(  ), model );

        return template.getHtml(  );
    }

    /**
     * Retourne l'ecran de confirmation de rejet
     *
     * @param request la requete HTTP
     * @return String template
     */
    public String getConfirmRejeter( HttpServletRequest request )
    {
        HashMap<String, String> model = getActeService(  ).getConfirmRejeter( request );
        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_CONFIRMATION_REJET, getLocale(  ), model );

        return template.getHtml(  );
    }

    /**
     * Retourne l'ecran de confirmation de reception de l'acte
     *
     * @param request la requete HTTP
     * @return String template
     */
    public String getReceptionActe( HttpServletRequest request )
    {
        HashMap<String, Object> model = getActeService(  ).getReceptionActe( request );
        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_CONFIRMATION_RECEPTION, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Retourne l'interface de confirmation de la suppression de l'acte
     *
     * @param request
     *            la requte Http
     * @return l'interface de confirmation de la suppression de l'acte
     */
    public String getSuppressionActe( HttpServletRequest request )
    {
        HtmlTemplate template;

        if ( RBACService.isAuthorized( ActesResourceIdService.ACTES, RBAC.WILDCARD_RESOURCES_ID,
                    ActesResourceIdService.PERMISSION_SUPPRESSION, getUser(  ) ) )
        {
            setPageTitleProperty( PROPERTY_PAGE_SUPPRESSION_ACTE );

            HashMap<String, String> model = getActeService(  ).getSuppressionActe( request );
            template = AppTemplateService.getTemplate( TEMPLATE_CONFIRMATION_SUPPRESSION_ACTE, getLocale(  ), model );
        }
        else
        {
            template = AppTemplateService.getTemplate( OdsProperties.TEMPLATE_PROFIL_ERROR, getLocale(  ) );
        }

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Retourne l'ecran d'annulation de l'acte
     *
     * @param request la requete HTTP
     * @return String template
     */
    public String getAnnulationActe( HttpServletRequest request )
    {
        HashMap<String, Object> model = getActeService(  ).getAnnulationActe( request );
        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_ANNULATION_ACTE, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Cr la modification de l'acte
     *
     * @param request la requte HTTP
     * @return String l'URL de retour
     */
    public String doPreparationActe( HttpServletRequest request )
    {
        String strUrl = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( RBACService.isAuthorized( ActesResourceIdService.ACTES, RBAC.WILDCARD_RESOURCES_ID,
                    ActesResourceIdService.PERMISSION_CREATION, getUser(  ) ) )
        {
            strUrl = getActeService(  ).doPreparationActe( request, getLocale(  ) );
        }
        else
        {
            strUrl = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }

        return strUrl;
    }

    /**
     * Modifie l'acte
     *
     * @param request la requete HTTP
     * @param nTypeRetour l'id du type de retour
     * @return String l'URL de destination
     */
    public String doModificationActe( HttpServletRequest request, int nTypeRetour )
    {
        String strUrl = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( RBACService.isAuthorized( ActesResourceIdService.ACTES, RBAC.WILDCARD_RESOURCES_ID,
                    ActesResourceIdService.PERMISSION_MISE_A_JOUR, getUser(  ) ) )
        {
            strUrl = getActeService(  ).doModificationActe( request, nTypeRetour );
        }
        else
        {
            strUrl = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }

        return strUrl;
    }

    /**
     * Permet de finaliser un acte
     * @param request la requte HTTP
     * @return strUrl url de retour
     */
    public String doFinaliserActe( HttpServletRequest request )
    {
        return getActeService(  ).doFinaliserActe( request );
    }

    /**
     * Transmet l'acte
     * @param request la requete
     * @return String la page de retour
     */
    public String doTransmissionActe( HttpServletRequest request )
    {
        String strUrl = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( RBACService.isAuthorized( ActesResourceIdService.ACTES, RBAC.WILDCARD_RESOURCES_ID,
                    ActesResourceIdService.PERMISSION_TRANSMISSION, getUser(  ) ) )
        {
            strUrl = getActeService(  ).doTransmissionActe( request );
        }
        else
        {
            strUrl = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }

        return strUrl;
    }

    /**
     * Permet de transmettre manuellement un acte
     * @param request la requte HTTP
     * @return strUrl url de retour
     */
    public String doTransmissionManuelleActe( HttpServletRequest request )
    {
        String strUrl = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( RBACService.isAuthorized( ActesResourceIdService.ACTES, RBAC.WILDCARD_RESOURCES_ID,
                    ActesResourceIdService.PERMISSION_TRANSMISSION, getUser(  ) ) )
        {
            strUrl = getActeService(  ).doTransmissionManuelleActe( request );
        }
        else
        {
            strUrl = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }

        return strUrl;
    }

    /**
     * Permet de retourner  la liste des actes
     * @param request la requte HTTP
     * @return strUrl url de retour
     */
    public String doVisualiserActe( HttpServletRequest request )
    {
        return getActeService(  ).doVisualiserActe( request );
    }

    /**
     * Permet de retourner  la liste de suivi des actes
     * @param request la requte HTTP
     * @return strUrl url de retour
     */
    public String doVisualiserActeForSuiviActe( HttpServletRequest request )
    {
        return getActeService(  ).doVisualiserActeForSuiviActe( request );
    }

    /**
     * Permet de receptionner un acte (si transmis manuellement)
     * @param request la requte HTTP
     * @return strUrl url de retour
     */
    public String doReceptionActe( HttpServletRequest request )
    {
        return getActeService(  ).doReceptionActe( request );
    }

    /**
     * Permet de rejeter un acte
     * @param request la requte HTTP
     * @return strUrl url de retour
     */
    public String doRejeterActe( HttpServletRequest request )
    {
        return getActeService(  ).doRejeterActe( request );
    }

    /**
     * Permet d'annuler un acte
     * @param request la requte HTTP
     * @return strUrl url de retour
     */
    public String doAnnulerActe( HttpServletRequest request )
    {
        return getActeService(  ).doAnnulerActe( request );
    }

    /**
     * Permet de cloner un acte
     * @param request la requte HTTP
     * @return strUrl url de retour
     */
    public String doClonerActe( HttpServletRequest request )
    {
        return getActeService(  ).doClonerActe( request );
    }

    /**
     * Effectue la suppression d'acte  partir des informations
     * contenues dans la requte
     *
     * @param request
     *            la requte Http
     * @return l'url de gestion des actes
     */
    public String doSuppressionActe( HttpServletRequest request )
    {
        String strUrl = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( RBACService.isAuthorized( ActesResourceIdService.ACTES, RBAC.WILDCARD_RESOURCES_ID,
                    ActesResourceIdService.PERMISSION_SUPPRESSION, getUser(  ) ) )
        {
            strUrl = getActeService(  ).doSuppressionActe( request );
        }
        else
        {
            strUrl = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }

        return strUrl;
    }

    /**
     * Retourne l'ecran de confirmation de l'erreur de transmission si la personne est autorisee
     * sinon retourne un msg d'erreur
     * @param request la requete HTTP
     * @return String template
     */
    public String getErreurTransmissionActe( HttpServletRequest request )
    {
        if ( !RBACService.isAuthorized( ActesResourceIdService.ACTES, RBAC.WILDCARD_RESOURCES_ID,
                    ActesResourceIdService.PERMISSION_ERREUR_TRANSMISSION, getUser(  ) ) )
        {
            //Si l'utilisateur n'est pas autoris  mettre en "Erreur de transmission" les actes, on lui affiche un msg d'erreur
            return AppTemplateService.getTemplate( OdsProperties.TEMPLATE_PROFIL_ERROR, getLocale(  ) ).getHtml(  );
        }
        else
        {
            HashMap<String, Object> model = getActeService(  ).getErreurTransmissionActe( request );

            return AppTemplateService.getTemplate( TEMPLATE_CONFIRMATION_ERREUR_TRANSMISSION, getLocale(  ), model )
                                     .getHtml(  );
        }
    }

    /**
     * Permet de passer l'acte a l'tat "Erreur de transmission"
     *
     * @param request
     *            la requte HTTP
     * @return strUrl url de retour
     */
    public String doErreurTransmissionActe( HttpServletRequest request )
    {
        String strUrl = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( !RBACService.isAuthorized( ActesResourceIdService.ACTES, RBAC.WILDCARD_RESOURCES_ID,
                    ActesResourceIdService.PERMISSION_ERREUR_TRANSMISSION, getUser(  ) ) )
        {
            //Si l'utilisateur n'est pas autoris  mettre en "Erreur de transmission" les actes, on lui affiche un msg d'erreur
            strUrl = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }
        else
        {
            strUrl = getActeService(  ).doErreurTransmissionActe( request );
        }

        return strUrl;
    }

    /**
     * Telecharge un fichier. Renvoie un flux de bytes du fichier dans l'objet
     * HttpServletResponse response
     *
     * @param request
     *            la requete HTTP
     * @param response
     *            la reponse HTTP
     */
    public void doDownloadFichier( HttpServletRequest request, HttpServletResponse response )
    {
        GFichier fichierTelecharger = getActeService(  ).doDownloadFichier( request );

        if ( fichierTelecharger != null )
        {
            try
            {
                String strFileName = fichierTelecharger.getNom(  ) + "." + fichierTelecharger.getExtension(  );

                ByteArrayOutputStream ba = new ByteArrayOutputStream(  );
                ba.write( fichierTelecharger.getFichier(  ).getDonnees(  ) );
                response.setContentLength( ba.size(  ) );
                response.setHeader( "Content-Disposition", "attachment ;filename=\"" + strFileName + "\"" );
                response.setContentType( "application/pdf" );
                response.setHeader( "Pragma", "public" );
                response.setHeader( "Expires", "0" );
                response.setHeader( "Cache-Control", "must-revalidate,post-check=0,pre-check=0" );

                ba.writeTo( response.getOutputStream(  ) );
                response.getOutputStream(  ).flush(  );
            }
            catch ( IOException e )
            {
                AppLogService.error( e );
            }
        }
    }
}
