/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web.acte;

import fr.paris.lutece.portal.service.admin.AccessDeniedException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


public interface IActeJspBean
{
    String RIGHT_ODS_ACTE_GESTION_AVAL = "ODS_SUIVI_ACTES_GESTION_AVAL";
    String JSP_URL_GESTION_ACTE = "jsp/admin/plugins/ods/acte/ActeGestionAval.jsp";

    /**
    * Methode qui fait appelle a la methode init() de PluginAdminPageJspBean
    * et initialise les variables se sessions si elles n'ont pas tes auparavant initialises
    * @param request request
    * @param strRight droits
    * @throws AccessDeniedException AccessDeniedException
    */
    void init( HttpServletRequest request, HttpServletResponse response, String strRight )
        throws AccessDeniedException;

    /**
    * Methode qui fait appelle a la methode init() de PluginAdminPageJspBean
    * et initialise les variables se sessions si elles n'ont pas tes auparavant initialises
    * @param request request
    * @param strRight droits
    * @throws AccessDeniedException AccessDeniedException
    */
    void initWithoutArrondissement( HttpServletRequest request, HttpServletResponse response, String strRight )
        throws AccessDeniedException;

    /**
     * Methode qui fait appelle a la methode init() de PluginAdminPageJspBean
     * et qui initialise la session
     * @param request la requete
     * @param strRight les droits
     * @throws AccessDeniedException AccessDeniedException
     */
    void initSession( HttpServletRequest request, HttpServletResponse response, String strRight )
        throws AccessDeniedException;

    /**
     * Initialisation du PDD
     * @param request la requte HTTP
     */
    void initPDD( HttpServletRequest request );

    String getRight(  );

    /**
     * retourne l'interface de gestion des actes
     * @param request la requete HTTP
     * @return  String url de l'interface de gestion des actes
     */
    String getActeList( HttpServletRequest request );

    /**
     * Retourne l'ecran de prparation d'acte
     *
     * @param request la requete HTTP
     * @return String template
     */
    String getPreparationActe( HttpServletRequest request );

    /**
     * Retourne l'ecran de visualisation de l'acte suivant le type d'entit
     *
     * @param request la requete HTTP
     * @return String template
     */
    String getModificationActe( HttpServletRequest request );

    /**
     * Retourne l'ecran de visualisation d'acte pour un pdd
     *
     * @param request la requete HTTP
     * @return String template
     */
    String getVisualisationActe( HttpServletRequest request );

    /**
     * Retourne l'ecran de confirmation de finalisation
     * @param request la requete HTTP
     * @return String template
     */
    String getConfirmFinaliser( HttpServletRequest request );

    /**
     * Retourne l'ecran de confirmation de rejet
     *
     * @param request la requete HTTP
     * @return String template
     */
    String getConfirmRejeter( HttpServletRequest request );

    /**
     * Retourne l'ecran de confirmation de reception de l'acte
     *
     * @param request la requete HTTP
     * @return String template
     */
    String getReceptionActe( HttpServletRequest request );

    /**
     * Retourne l'interface de confirmation de la suppression de l'acte
     *
     * @param request
     *            la requte Http
     * @return l'interface de confirmation de la suppression de l'acte
     */
    String getSuppressionActe( HttpServletRequest request );

    /**
     * Retourne l'ecran d'annulation de l'acte
     *
     * @param request la requete HTTP
     * @return String template
     */
    String getAnnulationActe( HttpServletRequest request );

    /**
     * Cr la modification de l'acte
     *
     * @param request la requte HTTP
     * @return String l'URL de retour
     */
    String doPreparationActe( HttpServletRequest request );

    /**
     * Modifie l'acte
     *
     * @param request la requete HTTP
     * @param nTypeRetour l'id du type de retour
     * @return String l'URL de destination
     */
    String doModificationActe( HttpServletRequest request, int nTypeRetour );

    /**
     * Permet de finaliser un acte
     * @param request la requte HTTP
     * @return strUrl url de retour
     */
    String doFinaliserActe( HttpServletRequest request );

    /**
     * Transmet l'acte
     * @param request la requete
     * @return String la page de retour
     */
    String doTransmissionActe( HttpServletRequest request );

    /**
     * Permet de transmettre manuellement un acte
     * @param request la requte HTTP
     * @return strUrl url de retour
     */
    String doTransmissionManuelleActe( HttpServletRequest request );

    /**
     * Permet de receptionner un acte (si transmis manuellement)
     * @param request la requte HTTP
     * @return strUrl url de retour
     */
    String doReceptionActe( HttpServletRequest request );

    /**
     * Permet de retourner  la liste des actes
     * @param request la requte HTTP
     * @return strUrl url de retour
     */
    String doVisualiserActe( HttpServletRequest request );

    /**
     * Permet de retourner  la liste de suivi des actes
     * @param request la requte HTTP
     * @return strUrl url de retour
     */
    String doVisualiserActeForSuiviActe( HttpServletRequest request );

    /**
     * Permet de rejeter un acte
     * @param request la requte HTTP
     * @return strUrl url de retour
     */
    String doRejeterActe( HttpServletRequest request );

    /**
     * Permet d'annuler un acte
     * @param request la requte HTTP
     * @return strUrl url de retour
     */
    String doAnnulerActe( HttpServletRequest request );

    /**
     * Permet de cloner un acte
     * @param request la requte HTTP
     * @return strUrl url de retour
     */
    String doClonerActe( HttpServletRequest request );

    /**
     * Effectue la suppression d'acte  partir des informations
     * contenues dans la requte
     *
     * @param request
     *            la requte Http
     * @return l'url de gestion des actes
     */
    String doSuppressionActe( HttpServletRequest request );

    /**
     * Telecharge un fichier. Renvoie un flux de bytes du fichier dans l'objet
     * HttpServletResponse response
     *
     * @param request
     *            la requete HTTP
     * @param response
     *            la reponse HTTP
     */
    void doDownloadFichier( HttpServletRequest request, HttpServletResponse response );

    /**
     * Retourne l'ecran de modififcation de l'acte lorsque l'on est sur l'cran de suivi des actes
     *
     * @param request la requete HTTP
     * @return String template
     */
    String getModificationActePDD( HttpServletRequest request );

    /**
     * Retourne l'ecran de confirmation de l'erreur de transmission si la personne est autorisee
     * sinon retourne un msg d'erreur
     * @param request la requete HTTP
     * @return String template
     */
    String getErreurTransmissionActe( HttpServletRequest request );

    /**
     * Permet de passer l'acte a l'tat "Erreur de transmission"
     *
     * @param request
     *            la requte HTTP
     * @return strUrl url de retour
     */
    String doErreurTransmissionActe( HttpServletRequest request );
}
