/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web.arrete;

import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.service.arrete.IArreteService;
import fr.paris.lutece.plugins.ods.service.role.ArreteGestionAvalResourceIdService;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.business.rbac.RBAC;
import fr.paris.lutece.portal.service.admin.AccessDeniedException;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.rbac.RBACService;
import fr.paris.lutece.portal.service.template.AppTemplateService;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.web.admin.AdminPageJspBean;
import fr.paris.lutece.util.html.HtmlTemplate;

import org.springframework.beans.factory.annotation.Autowired;

import java.io.ByteArrayOutputStream;
import java.io.IOException;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


/**
 *
 * AbstractArreteJspBean
 *
 */
public abstract class AbstractArreteJspBean<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>>
    extends AdminPageJspBean implements IArreteJspBean
{
    private static final long serialVersionUID = 9175155991345993466L;
    private static final String TEMPLATE_HISTORIQUE = "admin/plugins/ods/historique/historique.html";
    private static final String PROPERTY_PAGE_CREATION_ARRETE = "ods.arretes.creation.pageTitle";
    private static final String PROPERTY_PAGE_MODIFICATION_ARRETE = "ods.arretes.modification.pageTitle";
    private static final String PROPERTY_PAGE_HISTORIQUE = "ods.historique.pageTitle";
    @Autowired
    private IArreteService<GSeance, GFichier> _arreteService;

    /**
     * Retourne l'adresse du template de liste des arrts.
     * @return le chemin du template de liste
     */
    protected abstract String getTemplateListeArretes(  );

    /**
     * Retourne l'adresse du template de cration d'un arrt
     * @return le chemin du template de cration
     */
    protected abstract String getTemplateCreationArrete(  );

    /**
     * Retourne l'adresse du template de modification d'un arrt
     * @return le chemin du template de cration
     */
    protected abstract String getTemplateModificationArrete(  );

    /**
     * Retourne l'adresse template de l'historique
     * @return le chemin du template de l'historique
     */
    protected String getTemplateHistorique(  )
    {
        return TEMPLATE_HISTORIQUE;
    }

    /**
         * Reinitialise tous les attributs de session
         * @param request requete HTTP
         * @param response response http
         * @param strRight droits
         * @throws AccessDeniedException   AccessDeniedException
         */
    public void reinitSession( HttpServletRequest request, HttpServletResponse response, String strRight )
        throws AccessDeniedException
    {
        super.init( request, strRight );

        _arreteService.reinitSession( request );
    }

    /**
     * Initialise le Bean
     * @param request HttpServletRequest
     * @param response HttpServletResponse
     * @param strRight le droit
     * @throws AccessDeniedException si l'utilisateur n'est pas autoris  accder  cette ressource.
     */
    public void init( HttpServletRequest request, HttpServletResponse response, String strRight )
        throws AccessDeniedException
    {
        super.init( request, strRight );
    }

    /**
         * Retourne l'interface de gestion des arretes.
         * @param request la requte http
         * @return la page affichant la liste des arretes
         */
    public String getArreteList( HttpServletRequest request )
    {
        setPageTitleProperty( OdsConstants.CONSTANTE_CHAINE_VIDE );

        HashMap<String, Object> model = _arreteService.getArreteList( request );

        /*
         * Pour la gestion des profils
         */
        ajouterPermissionsDansHashmap( model );

        HtmlTemplate template = AppTemplateService.getTemplate( getTemplateListeArretes(  ), getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
         * Publication d'un arrete
         * Si "OdsParameters.ARRETER_IS_PUBLIER" gale  TRUE, alors la mthode <b>publie</b> l'arrete et incrmente la version<BR>
         *                 Dans ce cas, un historique est cre
         * Sinon la mthode change le statut enLigne d'un arrete a FALSE     *
         * @param request la requte http
         * @return l'url
         */
    public String doPublicationArrete( HttpServletRequest request )
    {
        return _arreteService.doPublicationArrete( request );
    }

    /**
         * retourne le formulaire de cration d'un arrete
         * @param request la requte http
         * @return la page de creation d'un arrete
         */
    public String getCreationArrete( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_PAGE_CREATION_ARRETE );

        HashMap<String, Object> model = _arreteService.getCreationArrete( request );
        HtmlTemplate template = AppTemplateService.getTemplate( getTemplateCreationArrete(  ), getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
         * retourne le formulaire de modification d'un arrete
         * @param request la requte http
         * @return la page de modification d'un arrete
         */
    public String getModificationArrete( HttpServletRequest request )
    {
        HtmlTemplate template;

        /*
         * GESTION DES PROFILS
         * -------------------
         * Si l'utilisateur n'est pas autoris a effectuer cette fonction il est renvoy
         * vers une page lui indiquant qu'il n'a pas les droits ncessaires.
         */
        if ( !RBACService.isAuthorized( ArreteGestionAvalResourceIdService.ARRETE_GESTION_AVAL,
                    RBAC.WILDCARD_RESOURCES_ID, ArreteGestionAvalResourceIdService.PERMISSION_MISE_A_JOUR, getUser(  ) ) )
        {
            template = AppTemplateService.getTemplate( OdsProperties.TEMPLATE_PROFIL_ERROR, getLocale(  ) );
        }
        else
        {
            HashMap<String, Object> model = _arreteService.getModificationArrete( request );

            if ( model == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            setPageTitleProperty( PROPERTY_PAGE_MODIFICATION_ARRETE );
            template = AppTemplateService.getTemplate( getTemplateModificationArrete(  ), getLocale(  ), model );
        }

        return getAdminPage( template.getHtml(  ) );
    }

    /**
         * retourne l'ecran de visualisation de l'historique des versions d'un arrete
         * @param request la requte http
         * @return la page de l'historique de l'arrete
         */
    public String getHistoriqueArrete( HttpServletRequest request )
    {
        HashMap<String, Object> model = _arreteService.getHistoriqueArrete( request );

        if ( model == null )
        {
            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                AdminMessage.TYPE_ERROR );
        }

        setPageTitleProperty( PROPERTY_PAGE_HISTORIQUE );

        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_HISTORIQUE, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
         * retourne une demande de confirmation pour la suppression de l'arrete choisi
         * @param request la requte http
         * @return la page de confirmation de suppression d'un arrete
         */
    public String getSuppressionArrete( HttpServletRequest request )
    {
        return _arreteService.getSuppressionArrete( request );
    }

    /**
         * supprime l'arrete choisi
         * @param request la requte http
         * @return l'url de suppression de l'arrete
         */
    public String doSuppressionArrete( HttpServletRequest request )
    {
        String strUrl = OdsConstants.CONSTANTE_CHAINE_VIDE;

        /*
         * GESTION DES PROFILS
         * -------------------
         * Si l'utilisateur n'est pas autoris a effectuer cette fonction il est renvoy
         * vers une page lui indiquant qu'il n'a pas les droits ncessaires.
         */
        if ( !RBACService.isAuthorized( ArreteGestionAvalResourceIdService.ARRETE_GESTION_AVAL,
                    RBAC.WILDCARD_RESOURCES_ID, ArreteGestionAvalResourceIdService.PERMISSION_SUPPRESSION, getUser(  ) ) )
        {
            strUrl = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }
        else
        {
            strUrl = _arreteService.doSuppressionArrete( request );
        }

        return strUrl;
    }

    /**
         * Modifie l'arrete
         * Si l'arrete est en ligne, alors la version est incrmente de 1 et ajout d'un historique
         * si tous les champs obligatoires du formulaire de modification d'un arrete ont t renseigns,<BR>
         * la mthode retourne l'url de gestion des arretes,<BR>
         * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
         * @param request la requte http
         * @return l'url de modification d'un arrete
         */
    public String doModificationArrete( HttpServletRequest request )
    {
        String strUrl = OdsConstants.CONSTANTE_CHAINE_VIDE;

        /*
         * GESTION DES PROFILS
         * -------------------
         * Si l'utilisateur n'est pas autoris a effectuer cette fonction il est renvoy
         * vers une page lui indiquant qu'il n'a pas les droits ncessaires.
         */
        if ( !RBACService.isAuthorized( ArreteGestionAvalResourceIdService.ARRETE_GESTION_AVAL,
                    RBAC.WILDCARD_RESOURCES_ID, ArreteGestionAvalResourceIdService.PERMISSION_MISE_A_JOUR, getUser(  ) ) )
        {
            return AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                AdminMessage.TYPE_ERROR );
        }
        else
        {
            strUrl = _arreteService.doModificationArrete( request );
        }

        return strUrl;
    }

    /**
         * Creation d'un arrete
         * si tous les champs obligatoires du formulaire de cration d'un arrete ont t renseign,<BR>
         * la mthode retourne l'url de gestion des arretes,<BR>
         * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
         * @param request la requte http
         * @return l'url de creation d'un arrete
         */
    public String doCreationArrete( HttpServletRequest request )
    {
        String strUrl = OdsConstants.CONSTANTE_CHAINE_VIDE;

        /*
         * GESTION DES PROFILS
         * -------------------
         * Si l'utilisateur n'est pas autoris a effectuer cette fonction,
         * il est renvoy vers une page lui indiquant qu'il n'a pas les droits ncessaires.
         */
        if ( !RBACService.isAuthorized( ArreteGestionAvalResourceIdService.ARRETE_GESTION_AVAL,
                    RBAC.WILDCARD_RESOURCES_ID, ArreteGestionAvalResourceIdService.PERMISSION_CREATION, getUser(  ) ) )
        {
            return AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                AdminMessage.TYPE_ERROR );
        }
        else
        {
            strUrl = _arreteService.doCreationArrete( request );
        }

        return strUrl;
    }

    /**
         * Tlcharge un fichier. Renvoie un flux de bytes du fichier dans l'objet
         * HttpServletResponse response
         *
         * @param request
         *            la requte HTTP
         * @param response
         *            la reponse HTTP
         */
    public void doDownloadFichier( HttpServletRequest request, HttpServletResponse response )
    {
        GFichier fichierTelecharger = _arreteService.doDownloadFichier( request );

        if ( fichierTelecharger != null )
        {
            try
            {
                String strFileName = fichierTelecharger.getNom(  ) + "." + fichierTelecharger.getExtension(  );

                ByteArrayOutputStream ba = new ByteArrayOutputStream(  );
                ba.write( fichierTelecharger.getFichier(  ).getDonnees(  ) );
                response.setContentLength( ba.size(  ) );
                response.setHeader( "Content-Disposition", "attachment ;filename=\"" + strFileName + "\"" );
                response.setContentType( "application/pdf" );
                response.setHeader( "Pragma", "public" );
                response.setHeader( "Expires", "0" );
                response.setHeader( "Cache-Control", "must-revalidate,post-check=0,pre-check=0" );

                ba.writeTo( response.getOutputStream(  ) );
                response.getOutputStream(  ).flush(  );
            }
            catch ( IOException e )
            {
                AppLogService.error( e );
            }
        }
    }

    /**
     * Permet de stocker toutes les permissions afin de grer les profils au niveau des templates
     * @param model le hashmap contenant les parametres qui vont tre envoys au template
     */
    private void ajouterPermissionsDansHashmap( Map<String, Object> model )
    {
        /*
         * Permission de dpublier
         */
        boolean bPermissionDepublication = true;

        if ( !RBACService.isAuthorized( ArreteGestionAvalResourceIdService.ARRETE_GESTION_AVAL,
                    RBAC.WILDCARD_RESOURCES_ID, ArreteGestionAvalResourceIdService.PERMISSION_DEPUBLICATION, getUser(  ) ) )
        {
            bPermissionDepublication = false;
        }

        model.put( OdsMarks.MARK_PERMISSION_DEPUBLICATION, bPermissionDepublication );

        /*
         * Permission de modifier
         */
        boolean bPermissionModification = true;

        if ( !RBACService.isAuthorized( ArreteGestionAvalResourceIdService.ARRETE_GESTION_AVAL,
                    RBAC.WILDCARD_RESOURCES_ID, ArreteGestionAvalResourceIdService.PERMISSION_MISE_A_JOUR, getUser(  ) ) )
        {
            bPermissionModification = false;
        }

        model.put( OdsMarks.MARK_PERMISSION_MODIFICATION, bPermissionModification );

        /*
         * Permission de supprimer
         */
        boolean bPermissionSuppression = true;

        if ( !RBACService.isAuthorized( ArreteGestionAvalResourceIdService.ARRETE_GESTION_AVAL,
                    RBAC.WILDCARD_RESOURCES_ID, ArreteGestionAvalResourceIdService.PERMISSION_SUPPRESSION, getUser(  ) ) )
        {
            bPermissionSuppression = false;
        }

        model.put( OdsMarks.MARK_PERMISSION_SUPPRESSION, bPermissionSuppression );
    }
}
