/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web.categoriedeliberation;

import fr.paris.lutece.plugins.ods.service.categoriedeliberation.ICategorieDeliberationService;
import fr.paris.lutece.plugins.ods.service.role.AdminSaufOdjResourceIdService;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.business.rbac.RBAC;
import fr.paris.lutece.portal.service.admin.AccessDeniedException;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.rbac.RBACService;
import fr.paris.lutece.portal.service.template.AppTemplateService;
import fr.paris.lutece.portal.web.admin.PluginAdminPageJspBean;
import fr.paris.lutece.util.html.HtmlTemplate;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


public abstract class AbstractCategorieDeliberationJspBean extends PluginAdminPageJspBean
    implements ICategorieDeliberationJspBean
{
    private static final long serialVersionUID = -549569235992892279L;
    public static final String RIGHT_ODS_CATEGORIE_DELIBERATION = "ODS_CATEGORIES";
    private static final String PROPERTY_PAGE_CREATION_CATEGORIE = "ods.categoriedeliberation.creation.pageTitle";
    private static final String PROPERTY_PAGE_MODIFICATION_CATEGORIE = "ods.categoriedeliberation.modification.pageTitle";
    @Autowired
    private ICategorieDeliberationService _categorieDeliberationService;

    protected abstract String getListTemplate(  );

    protected abstract String getCreationTemplate(  );

    protected abstract String getModificationTemplate(  );

    public void init( HttpServletRequest request, HttpServletResponse response )
        throws AccessDeniedException
    {
        super.init( request, getRight(  ) );
    }

    protected String getRight(  )
    {
        return RIGHT_ODS_CATEGORIE_DELIBERATION;
    }

    public void initCategorieDeliberationBean(  )
    {
        //Aucune action
    }

    /**
     * Retourne l'interface de gestion des catgories de dlibration
     * @param request le requte HTTP
     * @return l'interface de gestion des catgories de dlibration
     */
    public String getCategorieList( HttpServletRequest request )
    {
        setPageTitleProperty( OdsConstants.CONSTANTE_CHAINE_VIDE );

        HashMap<String, Object> model = _categorieDeliberationService.getCategorieList( request );

        ajouterPermissionsDansHashmap( model );

        HtmlTemplate template = AppTemplateService.getTemplate( getListTemplate(  ), getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Retourne l'interface de cration d'une catgorie de dlibration.
     * @param request le requte HTTP
     * @return l'interface de cration d'une catgorie de dlibration
     */
    public String getCreationCategorie( HttpServletRequest request )
    {
        HtmlTemplate template;

        if ( RBACService.isAuthorized( AdminSaufOdjResourceIdService.ADMINISTRATION_SAUF_ODJ,
                    RBAC.WILDCARD_RESOURCES_ID, AdminSaufOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            setPageTitleProperty( PROPERTY_PAGE_CREATION_CATEGORIE );

            HashMap<String, Object> model = _categorieDeliberationService.getCreationCategorie( request );

            template = AppTemplateService.getTemplate( getCreationTemplate(  ), getLocale(  ), model );
        }
        else
        {
            template = AppTemplateService.getTemplate( OdsProperties.TEMPLATE_PROFIL_ERROR, getLocale(  ) );
        }

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Cre une catgorie de dlibration.
     * Si tous les champs obligatoires sont renseigns,
     * et si la valeur du code n'est pas dj utilise par une autre catgorie,
     * la mthode retourne l'url de gestion des catgories de dlibration.
     * Sinon la mthode retourne l'url affichant les messages d'erreur
     * @param request la requte HTTP
     * @return url de la page de gestion si tous les champs sont renseigns et le code valide,<br>
     *  sinon un message d'erreur
     */
    public String doCreationCategorie( HttpServletRequest request )
    {
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( RBACService.isAuthorized( AdminSaufOdjResourceIdService.ADMINISTRATION_SAUF_ODJ,
                    RBAC.WILDCARD_RESOURCES_ID, AdminSaufOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            strReturn = _categorieDeliberationService.doCreationCategorie( request );

            if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                strReturn = getHomeUrl( request );
            }
        }
        else
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }

        return strReturn;
    }

    /**
     * Retourne l'interface de modification d'une catgorie de dlibration.
     * @param request le requte HTTP
     * @return l'interface de modification d'une catgorie de dlibration
     */
    public String getModificationCategorie( HttpServletRequest request )
    {
        HtmlTemplate template;

        if ( RBACService.isAuthorized( AdminSaufOdjResourceIdService.ADMINISTRATION_SAUF_ODJ,
                    RBAC.WILDCARD_RESOURCES_ID, AdminSaufOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            setPageTitleProperty( PROPERTY_PAGE_MODIFICATION_CATEGORIE );

            HashMap<String, Object> model = _categorieDeliberationService.getModificationCategorie( request );
            template = AppTemplateService.getTemplate( getModificationTemplate(  ), getLocale(  ), model );
        }
        else
        {
            template = AppTemplateService.getTemplate( OdsProperties.TEMPLATE_PROFIL_ERROR, getLocale(  ) );
        }

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Modifie une catgorie de dlibration.
     * Si tous les champs obligatoires sont renseigns,
     * et si la valeur du code n'est pas dj utilise par une autre catgorie,
     * la mthode retourne l'url de gestion des catgories de dlibration.
     * Sinon la mthode retourne l'url affichant les messages d'erreur
     * @param request la requte HTTP
     * @return une url
     */
    public String doModificationCategorie( HttpServletRequest request )
    {
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( RBACService.isAuthorized( AdminSaufOdjResourceIdService.ADMINISTRATION_SAUF_ODJ,
                    RBAC.WILDCARD_RESOURCES_ID, AdminSaufOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            strReturn = _categorieDeliberationService.doModificationCategorie( request );

            if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                strReturn = getHomeUrl( request );
            }
        }
        else
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }

        return strReturn;
    }

    /**
     * Retourne l'interface de suppression d'une catgorie de dlibration.
     * @param request le requte HTTP
     * @return l'interface de cration d'une catgorie de dlibration
     */
    public String getSuppressionCategorie( HttpServletRequest request )
    {
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( RBACService.isAuthorized( AdminSaufOdjResourceIdService.ADMINISTRATION_SAUF_ODJ,
                    RBAC.WILDCARD_RESOURCES_ID, AdminSaufOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            strReturn = _categorieDeliberationService.getSuppressionCategorie( request );

            if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                strReturn = getHomeUrl( request );
            }
        }
        else
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }

        return strReturn;
    }

    /**
     * Supprime une catgorie de dlibration.
     * @param request la requte HTTP
     * @return url de gestion des catgories de dlibration
     */
    public String doSuppressionCategorie( HttpServletRequest request )
    {
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( RBACService.isAuthorized( AdminSaufOdjResourceIdService.ADMINISTRATION_SAUF_ODJ,
                    RBAC.WILDCARD_RESOURCES_ID, AdminSaufOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            strReturn = _categorieDeliberationService.doSuppressionCategorie( request );

            if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                strReturn = getHomeUrl( request );
            }
        }
        else
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }

        return strReturn;
    }

    /**
     * Permet de stocker toutes les permissions afin de grer les profils au niveau des templates
     * @param model le hashmap contenant les parametres qui vont tre envoys au template
     */
    public void ajouterPermissionsDansHashmap( Map<String, Object> model )
    {
        /*
         * Permission de grer
         */
        boolean bPermissionGestion = true;

        if ( !RBACService.isAuthorized( AdminSaufOdjResourceIdService.ADMINISTRATION_SAUF_ODJ,
                    RBAC.WILDCARD_RESOURCES_ID, AdminSaufOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            bPermissionGestion = false;
        }

        model.put( OdsMarks.MARK_PERMISSION_GESTION, bPermissionGestion );
    }
}
