/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web.certificataffichage;

import fr.paris.lutece.plugins.ods.service.certificataffichage.ICertificatAffichageService;
import fr.paris.lutece.plugins.ods.service.manager.DocumentManager;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.template.AppTemplateService;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.web.admin.PluginAdminPageJspBean;
import fr.paris.lutece.util.html.HtmlTemplate;

import org.springframework.beans.factory.annotation.Autowired;

import java.io.IOException;
import java.io.OutputStream;

import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


/**
 * Cette classe permet de grer l'interface de gestion des certificat d'affichage.
 */
public class CertificatAffichageJspBean extends PluginAdminPageJspBean
{
    private static final long serialVersionUID = -6322208496788424116L;
    public static final String RIGHT_ODS_CERTIFICAT_AFFICHAGE = "ODS_CERTIFICAT_AFFICHAGE";
    public static final String JSP_URL_CERTIFICAT = "jsp/admin/plugins/ods/certificatAffichage/CertificatAffichage.jsp";
    private static final String MESSAGE_PROBLEME_CONVERSION_JCOPIST = "ods.odj.message.problemJCopist";
    private static final String TEMPLATE_CERTIFICAT_AFFICHAGE = "admin/plugins/ods/certificatAffichage/certificat.html";
    private static final String PROPERTY_MESSAGE_NO_MODELE = "ods.certificatAffichage.error.noModele";
    private static final String PROPERTY_MESSAGE_NO_ENTETE_PIED = "ods.certificatAffichage.error.noEntetePied";
    @Autowired
    private ICertificatAffichageService _certificatAffichageService;

    /**
     * Retourne l'interface de gestion des certificats d'affichage
     * @param request le requte HTTP
     * @return l'interface de gestion des certificats d'affichage
     */
    public String getCertificatList( HttpServletRequest request )
    {
        setPageTitleProperty( OdsConstants.CONSTANTE_CHAINE_VIDE );

        HashMap<String, Object> model = _certificatAffichageService.getCertificatList( request );

        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_CERTIFICAT_AFFICHAGE, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     *
     * @param request la requete HTTP
     * @return l'URL de visualisation du certificat
     */
    public String getVisualisationCertificat( HttpServletRequest request )
    {
        return _certificatAffichageService.getVisualisationCertificat( request );
    }

    /**
     * Retourne l'ordre du jour sous format PDF ou Word
     * @param request la requete Http
     * @param response la reponse
     * @throws IOException IOException
     */
    public void doVisualisationCertificat( HttpServletRequest request, HttpServletResponse response )
        throws IOException
    {
        if ( !DocumentManager.checkJCopist(  ) )
        {
            try
            {
                response.sendRedirect( AdminMessageService.getMessageUrl( request, MESSAGE_PROBLEME_CONVERSION_JCOPIST,
                        AdminMessage.TYPE_STOP ) );
            }
            catch ( IOException ioe )
            {
                AppLogService.error( I18nService.getLocalizedString( MESSAGE_PROBLEME_CONVERSION_JCOPIST, getLocale(  ) ) );

                return;
            }
        }

        String strXML = _certificatAffichageService.doVisualisationCertificat( request );

        if ( ( strXML != null ) &&
                ( strXML.equals( PROPERTY_MESSAGE_NO_MODELE ) || strXML.equals( PROPERTY_MESSAGE_NO_ENTETE_PIED ) ) )
        {
            response.sendRedirect( AdminMessageService.getMessageUrl( request, strXML, AdminMessage.TYPE_ERROR ) );
        }

        try
        {
            byte[] xml = ( OdsConstants.CONSTANTE_ENTETE_XML + strXML ).getBytes( OdsConstants.UTF8 );
            byte[] bConvertedDoc = DocumentManager.convert( DocumentManager.PDF_CONVERSION, "certificat", xml );

            if ( ( bConvertedDoc != null ) && ( bConvertedDoc.length > 0 ) )
            {
                OutputStream os = response.getOutputStream(  );
                String strFileName = "certificat.pdf";
                response.setHeader( "Content-Disposition", "attachment ;filename=\"" + strFileName + "\"" );
                response.setContentType( "application/pdf" );
                response.setHeader( "Pragma", "public" );
                response.setHeader( "Expires", "0" );
                response.setHeader( "Cache-Control", "must-revalidate,post-check=0,pre-check=0" );
                os.write( bConvertedDoc );
                os.close(  );
            }
        }
        catch ( Exception e )
        {
            AppLogService.error( e );
        }
    }
}
