/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web.deliberationdesignation;

import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.service.deliberationdesignation.IDeliberationDesignationService;
import fr.paris.lutece.plugins.ods.service.role.DeliberationDesignationGestionAvalResourceIdService;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.business.rbac.RBAC;
import fr.paris.lutece.portal.service.admin.AccessDeniedException;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.rbac.RBACService;
import fr.paris.lutece.portal.service.template.AppTemplateService;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.service.util.AppPathService;
import fr.paris.lutece.portal.web.admin.PluginAdminPageJspBean;
import fr.paris.lutece.util.html.HtmlTemplate;

import org.springframework.beans.factory.annotation.Autowired;

import java.io.ByteArrayOutputStream;
import java.io.IOException;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


public abstract class AbstractDeliberationDesignationJspBean<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>>
    extends PluginAdminPageJspBean implements IDeliberationDesignationJspBean
{
    private static final long serialVersionUID = 8295423897057499317L;
    protected static final String JSP_URL_LISTE_DELIBERATIONS = "jsp/admin/plugins/ods/deliberationsdesignation/DeliberationsDesignationGestionAvalNoReinit.jsp";
    protected static final String TEMPLATE_HISTORIQUE = "admin/plugins/ods/historique/historique.html";
    protected static final String TEMPLATE_CONFIRMATION_FINALISATION = "admin/plugins/ods/deliberationsdesignation/confirmation_finaliser_acte.html";
    protected static final String PROPERTY_PAGE_CREATION_DELIBERATIONS_DESIGNATION = "ods.deliberationsdesignation.creation.pageTitle";
    protected static final String PROPERTY_PAGE_MODIFICATION_DELIBERATIONS_DESIGNATION = "ods.deliberationsdesignation.modification.pageTitle";
    protected static final String PROPERTY_PAGE_HISTORIQUE = "ods.historique.pageTitle";
    @Autowired
    protected IDeliberationDesignationService<GSeance, GFichier> _deliberationDesignationService;

    protected abstract String getTemplateListeDeliberationsDesignation(  );

    protected abstract String getTemplateCreationDeliberationDesignation(  );

    protected abstract String getTemplateModificationDeliberationDesignation(  );

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.deliberationdesignation.IDeliberationDesignationJspBean#reinitSession(javax.servlet.http.HttpServletRequest, java.lang.String)
         */
    public void init( HttpServletRequest request, HttpServletResponse response, String strRight )
        throws AccessDeniedException
    {
        super.init( request, strRight );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.deliberationdesignation.IDeliberationDesignationJspBean#reinitSession(javax.servlet.http.HttpServletRequest, java.lang.String)
         */
    public void init( HttpServletRequest request, String strRight )
        throws AccessDeniedException
    {
        super.init( request, strRight );
    }

    public void comeFromActe(  )
    {
        _deliberationDesignationService.comeFromActe(  );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.deliberationdesignation.IDeliberationDesignationJspBean#reinitSession(javax.servlet.http.HttpServletRequest, java.lang.String)
         */
    public void reinitSession( HttpServletRequest request, HttpServletResponse response, String strRight )
        throws AccessDeniedException
    {
        super.init( request, strRight );
        _deliberationDesignationService.reinitSession( request );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.deliberationdesignation.IDeliberationDesignationJspBean#initUrlRetour(java.lang.String, javax.servlet.http.HttpServletRequest)
         */
    public void initUrlRetour( String strUrl, HttpServletRequest request )
    {
        _deliberationDesignationService.initUrlRetour( strUrl, request );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.deliberationdesignation.IDeliberationDesignationJspBean#getDeliberationDesignationList(javax.servlet.http.HttpServletRequest)
         */
    public String getDeliberationDesignationList( HttpServletRequest request )
    {
        setPageTitleProperty( OdsConstants.CONSTANTE_CHAINE_VIDE );

        HashMap<String, Object> model = _deliberationDesignationService.getDeliberationDesignationList( request,
                getHomeUrl( request ) );

        /*
         * Pour la gestion des profils
         */
        ajouterPermissionsDansHashmap( model );

        HtmlTemplate template = AppTemplateService.getTemplate( getTemplateListeDeliberationsDesignation(  ),
                getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.deliberationdesignation.IDeliberationDesignationJspBean#doPublicationDeliberationDesignation(javax.servlet.http.HttpServletRequest)
         */
    public String doPublicationDeliberationDesignation( HttpServletRequest request )
    {
        String strReturn = _deliberationDesignationService.doPublicationDeliberationDesignation( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.deliberationdesignation.IDeliberationDesignationJspBean#getConfirmFinaliser(javax.servlet.http.HttpServletRequest)
         */
    public String getConfirmFinaliser( HttpServletRequest request )
    {
        HashMap<String, Object> model = _deliberationDesignationService.getConfirmFinaliser( request );

        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_CONFIRMATION_FINALISATION, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.deliberationdesignation.IDeliberationDesignationJspBean#getCreationDeliberationDesignation(javax.servlet.http.HttpServletRequest)
         */
    public String getCreationDeliberationDesignation( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_PAGE_CREATION_DELIBERATIONS_DESIGNATION );

        HashMap<String, Object> model = _deliberationDesignationService.getCreationDeliberationDesignation( request );

        HtmlTemplate template = AppTemplateService.getTemplate( getTemplateCreationDeliberationDesignation(  ),
                getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.deliberationdesignation.IDeliberationDesignationJspBean#getModificationDeliberationDesignation(javax.servlet.http.HttpServletRequest)
         */
    public String getModificationDeliberationDesignation( HttpServletRequest request )
    {
        HtmlTemplate template;

        /*
         * GESTION DES PROFILS
         * -------------------
         * Si l'utilisateur n'est pas autoris a effectuer cette fonction il est renvoy
         * vers une page lui indiquant qu'il n'a pas les droits ncessaires.
         */
        if ( _deliberationDesignationService.isPublie( request ) &&
                !RBACService.isAuthorized( 
                    DeliberationDesignationGestionAvalResourceIdService.DELIBERATION_GESTION_AVAL,
                    RBAC.WILDCARD_RESOURCES_ID,
                    DeliberationDesignationGestionAvalResourceIdService.PERMISSION_MISE_A_JOUR, getUser(  ) ) )
        {
            template = AppTemplateService.getTemplate( OdsProperties.TEMPLATE_PROFIL_ERROR, getLocale(  ) );
        }
        else
        {
            setPageTitleProperty( PROPERTY_PAGE_MODIFICATION_DELIBERATIONS_DESIGNATION );

            HashMap<String, Object> model = _deliberationDesignationService.getModificationDeliberationDesignation( request );

            if ( model == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            template = AppTemplateService.getTemplate( getTemplateModificationDeliberationDesignation(  ),
                    getLocale(  ), model );
        }

        return getAdminPage( template.getHtml(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.deliberationdesignation.IDeliberationDesignationJspBean#getHistoriqueDeliberationDesignation(javax.servlet.http.HttpServletRequest)
         */
    public String getHistoriqueDeliberationDesignation( HttpServletRequest request )
    {
        HashMap<String, Object> model = _deliberationDesignationService.getHistoriqueDeliberationDesignation( request );

        if ( model == null )
        {
            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                AdminMessage.TYPE_ERROR );
        }

        setPageTitleProperty( PROPERTY_PAGE_HISTORIQUE );

        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_HISTORIQUE, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.deliberationdesignation.IDeliberationDesignationJspBean#getSuppressionDeliberationDesignation(javax.servlet.http.HttpServletRequest)
         */
    public String getSuppressionDeliberationDesignation( HttpServletRequest request )
    {
        String strReturn = _deliberationDesignationService.getSuppressionDeliberationDesignation( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.deliberationdesignation.IDeliberationDesignationJspBean#doSuppressionDeliberationDesignation(javax.servlet.http.HttpServletRequest)
         */
    public String doSuppressionDeliberationDesignation( HttpServletRequest request )
    {
        /*
        * GESTION DES PROFILS
        * -------------------
        * Si l'utilisateur n'est pas autoris a effectuer cette fonction il est renvoy
        * vers une page lui indiquant qu'il n'a pas les droits ncessaires.
        */
        if ( _deliberationDesignationService.isPublie( request ) &&
                !RBACService.isAuthorized( 
                    DeliberationDesignationGestionAvalResourceIdService.DELIBERATION_GESTION_AVAL,
                    RBAC.WILDCARD_RESOURCES_ID,
                    DeliberationDesignationGestionAvalResourceIdService.PERMISSION_SUPPRESSION, getUser(  ) ) )
        {
            return AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                AdminMessage.TYPE_ERROR );
        }

        String strReturn = _deliberationDesignationService.doSuppressionDeliberationDesignation( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.deliberationdesignation.IDeliberationDesignationJspBean#doModificationDeliberationDesignation(javax.servlet.http.HttpServletRequest)
         */
    public String doModificationDeliberationDesignation( HttpServletRequest request )
    {
        /*
         * GESTION DES PROFILS
         * -------------------
         * Si l'utilisateur n'est pas autoris a effectuer cette fonction il est renvoy
         * vers une page lui indiquant qu'il n'a pas les droits ncessaires.
         */
        if ( _deliberationDesignationService.isPublie( request ) &&
                !RBACService.isAuthorized( 
                    DeliberationDesignationGestionAvalResourceIdService.DELIBERATION_GESTION_AVAL,
                    RBAC.WILDCARD_RESOURCES_ID,
                    DeliberationDesignationGestionAvalResourceIdService.PERMISSION_MISE_A_JOUR, getUser(  ) ) )
        {
            return AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                AdminMessage.TYPE_ERROR );
        }

        String strReturn = _deliberationDesignationService.doModificationDeliberationDesignation( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.deliberationdesignation.IDeliberationDesignationJspBean#doCreationDeliberationDesignation(javax.servlet.http.HttpServletRequest)
         */
    public String doCreationDeliberationDesignation( HttpServletRequest request )
    {
        /*
         * GESTION DES PROFILS
         * -------------------
         * Si l'utilisateur n'est pas autoris a effectuer cette fonction,
         * il est renvoy vers une page lui indiquant qu'il n'a pas les droits ncessaires.
         */
        if ( !RBACService.isAuthorized( DeliberationDesignationGestionAvalResourceIdService.DELIBERATION_GESTION_AVAL,
                    RBAC.WILDCARD_RESOURCES_ID,
                    DeliberationDesignationGestionAvalResourceIdService.PERMISSION_CREATION, getUser(  ) ) )
        {
            return AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                AdminMessage.TYPE_ERROR );
        }

        String strReturn = _deliberationDesignationService.doCreationDeliberationDesignation( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.deliberationdesignation.IDeliberationDesignationJspBean#doDownloadFichier(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
         */
    public void doDownloadFichier( HttpServletRequest request, HttpServletResponse response )
    {
        GFichier fichierTelecharger = _deliberationDesignationService.doDownloadFichier( request );

        if ( fichierTelecharger != null )
        {
            try
            {
                String strFileName = fichierTelecharger.getNom(  ) + "." + fichierTelecharger.getExtension(  );

                ByteArrayOutputStream ba = new ByteArrayOutputStream(  );
                ba.write( fichierTelecharger.getFichier(  ).getDonnees(  ) );
                response.setContentLength( ba.size(  ) );
                response.setHeader( "Content-Disposition", "attachment ;filename=\"" + strFileName + "\"" );
                response.setContentType( "application/pdf" );
                response.setHeader( "Pragma", "public" );
                response.setHeader( "Expires", "0" );
                response.setHeader( "Cache-Control", "must-revalidate,post-check=0,pre-check=0" );

                ba.writeTo( response.getOutputStream(  ) );
                response.getOutputStream(  ).flush(  );
            }
            catch ( IOException e )
            {
                AppLogService.error( e );
            }
        }
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.deliberationdesignation.IDeliberationDesignationJspBean#doFinaliserActe(javax.servlet.http.HttpServletRequest)
         */
    public String doFinaliserActe( HttpServletRequest request )
    {
        String strReturn = _deliberationDesignationService.doFinaliserActe( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /**
     * Permet de stocker toutes les permissions afin de grer les profils au niveau des templates
     * @param model le hashmap contenant les parametres qui vont tre envoys au template
     */
    protected void ajouterPermissionsDansHashmap( Map<String, Object> model )
    {
        /*
         * Permission de dpublier
         */
        boolean bPermissionDepublication = true;

        if ( !RBACService.isAuthorized( DeliberationDesignationGestionAvalResourceIdService.DELIBERATION_GESTION_AVAL,
                    RBAC.WILDCARD_RESOURCES_ID,
                    DeliberationDesignationGestionAvalResourceIdService.PERMISSION_DEPUBLICATION, getUser(  ) ) )
        {
            bPermissionDepublication = false;
        }

        model.put( OdsMarks.MARK_PERMISSION_DEPUBLICATION, bPermissionDepublication );

        /*
         * Permission de modifier
         */
        boolean bPermissionModification = true;

        if ( !RBACService.isAuthorized( DeliberationDesignationGestionAvalResourceIdService.DELIBERATION_GESTION_AVAL,
                    RBAC.WILDCARD_RESOURCES_ID,
                    DeliberationDesignationGestionAvalResourceIdService.PERMISSION_MISE_A_JOUR, getUser(  ) ) )
        {
            bPermissionModification = false;
        }

        model.put( OdsMarks.MARK_PERMISSION_MODIFICATION, bPermissionModification );

        /*
         * Permission de supprimer
         */
        boolean bPermissionSuppression = true;

        if ( !RBACService.isAuthorized( DeliberationDesignationGestionAvalResourceIdService.DELIBERATION_GESTION_AVAL,
                    RBAC.WILDCARD_RESOURCES_ID,
                    DeliberationDesignationGestionAvalResourceIdService.PERMISSION_SUPPRESSION, getUser(  ) ) )
        {
            bPermissionSuppression = false;
        }

        model.put( OdsMarks.MARK_PERMISSION_SUPPRESSION, bPermissionSuppression );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.deliberationdesignation.IDeliberationDesignationJspBean#getHomeUrl(javax.servlet.http.HttpServletRequest)
         */
    public String getHomeUrl( HttpServletRequest request )
    {
        String strJspReturn = AppPathService.getBaseUrl( request );
        strJspReturn += JSP_URL_LISTE_DELIBERATIONS;

        return strJspReturn;
    }
}
