/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web.deliberationdesignation;

import fr.paris.lutece.portal.service.admin.AccessDeniedException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


public interface IDeliberationDesignationJspBean
{
    String RIGHT_ODS_DELIBERATIONS_DESIGNATION_GESTION_AVAL = "ODS_DELIBERATIONS_DESIGNATION_GESTION_AVAL";

    /**
     * Reinitialise tous les attributs de session
     * @param request requete HTTP
     * @param strRight droits
     * @throws AccessDeniedException   AccessDeniedException
     */
    void reinitSession( HttpServletRequest request, HttpServletResponse response, String strRight )
        throws AccessDeniedException;

    void init( HttpServletRequest request, HttpServletResponse response, String strRight )
        throws AccessDeniedException;

    void init( HttpServletRequest request, String strRight )
        throws AccessDeniedException;

    void comeFromActe(  );

    /**
     * Permet d'initialiser l'url de retour
     * @param strUrl url de retour
     * @param request la requte http
     */
    void initUrlRetour( String strUrl, HttpServletRequest request );

    /**
     * Retourne l'interface de gestion des deliberations de designation.
     * @param request la requte http
     * @return la page affichant la liste des dlibrations de dsignation
     */
    String getDeliberationDesignationList( HttpServletRequest request );

    /**
     * Publication d'une dlibration de dsignation
     * Si "OdsParameters.DELIBERATION_IS_PUBLIER" gale  TRUE, alors la mthode <b>publie</b> la dlibration et incrmente la version<BR>
     *                 Dans ce cas, un historique est cre
     * Sinon la mthode change le statut enLigne d'une dlibration a FALSE     *
     * @param request la requte http
     * @return l'url
     */
    String doPublicationDeliberationDesignation( HttpServletRequest request );

    /**
     * retourne le formulaire de confirmation d'envoi manuel de l'acte
     * @param request la requte http
     * @return la page de confirmation
     */
    String getConfirmFinaliser( HttpServletRequest request );

    /**
     * retourne le formulaire de cration d'une dlibration de dsignation
     * @param request la requte http
     * @return la page de creation d'une dlibration de dsignation
     */
    String getCreationDeliberationDesignation( HttpServletRequest request );

    /**
     * retourne le formulaire de modification d'une dlibration de dsignation
     * @param request la requte http
     * @return la page de modification d'une dlibration de dsignation
     */
    String getModificationDeliberationDesignation( HttpServletRequest request );

    /**
     * retourne l'ecran de visualisation de l'historique des versions d'une dlibration de dsignation
     * @param request la requte http
     * @return la page de l'historique de la dlibration
     */
    String getHistoriqueDeliberationDesignation( HttpServletRequest request );

    /**
     * retourne une demande de confirmation pour la suppression de l dlibration choisie
     * @param request la requte http
     * @return la page de confirmation de suppression d'une dlibration
     */
    String getSuppressionDeliberationDesignation( HttpServletRequest request );

    /**
     * supprime la dlibration de dsignation choisie
     * @param request la requte http
     * @return l'url de suppression de la dlibration de dsignation
     */
    String doSuppressionDeliberationDesignation( HttpServletRequest request );

    /**
     * Modifie la dlibration de dsignation
     * Si la dlibration est en ligne, alors la version est incrmente de 1 et ajout d'un historique
     * si tous les champs obligatoires du formulaire de modification d'une dlibration ont t renseigns,<BR>
     * la mthode retourne l'url de gestion des dlibrations,<BR>
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request la requte http
     * @return l'url de modification d'une dlibration
     */
    String doModificationDeliberationDesignation( HttpServletRequest request );

    /**
     * Creation d'une dlibration de dsignation
     * si tous les champs obligatoires du formulaire de cration d'une dlibration ont t renseign,<BR>
     * la mthode retourne l'url de gestion des dlibrations,<BR>
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request la requte http
     * @return l'url de creation d'une dlibration de dsignation
     */
    String doCreationDeliberationDesignation( HttpServletRequest request );

    /**
     * Tlcharge un fichier. Renvoie un flux de bytes du fichier dans l'objet
     * HttpServletResponse response
     *
     * @param request
     *            la requte HTTP
     * @param response
     *            la reponse HTTP
     */
    void doDownloadFichier( HttpServletRequest request, HttpServletResponse response );

    /**
     * Permet de confirmer la finalisation d'un acte
     *
     * @param request
     *            la requete HTTP
     * @return strUrl url de retour
     */
    String doFinaliserActe( HttpServletRequest request );

    /**
     * retourne l'URL de la page d'accueil des arretes
     * @param request la requete
     * @return l'URL de la page d'accueil des arretes
     */
    String getHomeUrl( HttpServletRequest request );
}
