/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web.elu;

import fr.paris.lutece.portal.service.admin.AccessDeniedException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


public interface IEluJspBean
{
    String RIGHT_ODS_ELU = "ODS_ELUS";

    /**
     * Retourne le formulaire de cration d'un lu
     * @param request la requete HTTP
     * @return formulaire de cration d'un lu
     */
    String getCreationElu( HttpServletRequest request );

    /**
     * Initialise la page
     * @param request la requete HTTP
     * @param response la repose
     * @param strRight le droit
     * @throws AccessDeniedException droits insuffisants
     */
    void init( HttpServletRequest request, HttpServletResponse response, String strRight )
        throws AccessDeniedException;

    /**
     * cre lu.
     * si tous les champs obligatoires du formulaire de cration d'un lu ont t renseign,
     * la mthode retourne l'url de gestion des lus,
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request request
     * @return url url
     */
    String doCreationElu( HttpServletRequest request );

    /**
     * Retourne le formulaire de modification d'un lu  pr remplis avec  les critres  de l'lu choisi
     * @param request la requete HTTP
     * @return formulaire de modification d'un lu
     */
    String getModificationElu( HttpServletRequest request );

    /**
     * Modifie un lu.
     * si tous les champs obligatoires du formulaire de modification d'un lu ont t renseign,
     * la mthode retourne l'url de gestion des lus,
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request request
     * @return url url
     */
    String doModificationElu( HttpServletRequest request );

    /**
     * Retourne une demande pour la suppression de l'lu choisi
     * @param request la requete HTTP
     * @return interface de confirmation de suppression
     */
    String getSuppressionElu( HttpServletRequest request );

    /**
     * supprime l'lu choisi
     * @param request la requete HTTP
     * @return url de gestion des groupes poliques
     */
    String doSuppressionElu( HttpServletRequest request );

    /**
     * Retourne l'interface de gestion des lus
     * @param request la requete HTTP
     * @return   interface de gestion des lus
     */
    String getEluList( HttpServletRequest request );

    void init( HttpServletRequest request, String strRight )
        throws AccessDeniedException;
}
