/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web.expression;

import fr.paris.lutece.plugins.ods.service.expression.IExpressionUsuelleService;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.admin.AccessDeniedException;
import fr.paris.lutece.portal.service.template.AppTemplateService;
import fr.paris.lutece.portal.web.admin.PluginAdminPageJspBean;
import fr.paris.lutece.util.html.HtmlTemplate;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


public abstract class AbstractExpressionUsuelleJspBean extends PluginAdminPageJspBean
    implements IExpressionUsuelleJspBean
{
    private static final long serialVersionUID = 6921212836759455701L;
    private static final String TEMPLATE_LISTE_EXPRESSIONS = "admin/plugins/ods/expression/liste_expression_usuelle.html";
    private static final String TEMPLATE_SELECTION_EXPRESSIONS = "admin/plugins/ods/expression/selection_expression_usuelle.html";
    private static final String TEMPLATE_CREATION_EXPRESSION = "admin/plugins/ods/expression/creation_expression_usuelle.html";
    private static final String TEMPLATE_MODIFICATION_EXPRESSION = "admin/plugins/ods/expression/modification_expression_usuelle.html";
    private static final String PROPERTY_PAGE_CREATION_EXPRESSION = "ods.expressionusuelle.creation.pageTitle";
    private static final String PROPERTY_PAGE_MODIFICATION_EXPRESSION = "ods.expressionusuelle.modification.pageTitle";
    private static final String RIGHT_ODS_EXPRESSION = "ODS_EXPRESSIONS";
    @Autowired
    private IExpressionUsuelleService _expressionUsuelleService;

    /**
    * Init
    * @param request
    * @throws AccessDeniedException
    */
    public void init( HttpServletRequest request, HttpServletResponse response )
        throws AccessDeniedException
    {
        super.init( request, getRight(  ) );
    }

    protected String getRight(  )
    {
        return RIGHT_ODS_EXPRESSION;
    }

    /**
     * Cre une expression usuelle.
     * Si le champ obligatoire est renseign,
     * la mthode retourne l'url de gestion des expressions usuelles.
     * Sinon la mthode retourne l'url affichant les messages d'erreur
     * @param request la requte HTTP
     * @return une url
     */
    public String doCreationExpression( HttpServletRequest request )
    {
        String strReturn = _expressionUsuelleService.doCreationExpression( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /**
    * Modifie une expression usuelle.
    * Si le champ obligatoire est renseign,
    * la mthode retourne l'url de gestion des expressions usuelles.
    * Sinon la mthode retourne l'url affichant les messages d'erreur
    * @param request la requte HTTP
    * @return une url
    */
    public String doModificationExpression( HttpServletRequest request )
    {
        String strReturn = _expressionUsuelleService.doModificationExpression( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /**
    * Supprime une expression usuelle.
    * @param request la requte HTTP
    * @return url de gestion des expressions usuelles
    */
    public String doSuppressionExpression( HttpServletRequest request )
    {
        String strReturn = _expressionUsuelleService.doSuppressionExpression( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /**
    * Retourne l'interface de cration d'une expression usuelle.
    * @param request le requte HTTP
    * @return l'interface de cration d'une expression usuelle
    */
    public String getCreationExpression( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_PAGE_CREATION_EXPRESSION );

        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_CREATION_EXPRESSION, getLocale(  ) );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
    * Retourne l'interface de gestion des expressions usuelles.
    * @param request le requte HTTP
    * @return l'interface de gestion des expressions usuelles
    */
    public String getExpressionList( HttpServletRequest request )
    {
        setPageTitleProperty( OdsConstants.CONSTANTE_CHAINE_VIDE );

        HashMap<String, Object> model = _expressionUsuelleService.getExpressionList( request, getHomeUrl( request ) );

        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_LISTE_EXPRESSIONS, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
    * Retourne l'interface de modification d'une expression usuelle.
    * @param request le requte HTTP
    * @return l'interface de modification d'une expression usuelle
    */
    public String getModificationExpression( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_PAGE_MODIFICATION_EXPRESSION );

        HashMap<String, Object> model = _expressionUsuelleService.getModificationExpression( request );

        if ( model == null )
        {
            return getExpressionList( request );
        }

        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_MODIFICATION_EXPRESSION, getLocale(  ), model );

        return ( getAdminPage( template.getHtml(  ) ) );
    }

    /**
    * Retourne l'interface de selection des expressions usuelles.
    * @param request le requte HTTP
    * @return l'interface de selection des expressions usuelles.
    */
    public String getSelectionExpression( HttpServletRequest request )
    {
        setPageTitleProperty( OdsConstants.CONSTANTE_CHAINE_VIDE );

        HashMap<String, Object> model = _expressionUsuelleService.getSelectionExpression( request );

        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_SELECTION_EXPRESSIONS, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
    * Retourne l'interface de suppression d'une expression usuelle.
    * @param request le requte HTTP
    * @return l'interface de cration d'une expression usuelle
    */
    public String getSuppressionExpression( HttpServletRequest request )
    {
        String strReturn = _expressionUsuelleService.getSuppressionExpression( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    public void initExpressionUsuelleBean(  )
    {
        // TODO Auto-generated method stub
    }
}
