/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web.fichier;

import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.service.fichier.IFichierService;
import fr.paris.lutece.plugins.ods.service.role.FichiersResourceIdService;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.business.rbac.RBAC;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.rbac.RBACService;
import fr.paris.lutece.portal.service.template.AppTemplateService;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.web.admin.PluginAdminPageJspBean;
import fr.paris.lutece.util.html.HtmlTemplate;

import org.springframework.beans.factory.annotation.Autowired;

import java.io.IOException;
import java.io.OutputStream;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


public abstract class AbstractFichierJspBean<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>>
    extends PluginAdminPageJspBean implements IFichierJspBean
{
    private static final long serialVersionUID = -4506650320636279314L;
    private static final String TEMPLATE_HISTORIQUE = "admin/plugins/ods/historique/historique.html";
    protected static final String PROPERTY_PAGE_CREATION_FICHIER = "ods.fichier.creation.pageTitle";
    private static final String PROPERTY_PAGE_MODIFICATION_FICHIER = "ods.fichier.modification.pageTitle";
    private static final String PROPERTY_PAGE_HISTORIQUE = "ods.historique.pageTitle";
    @Autowired
    protected IFichierService<GSeance, GFichier> _fichierService;

    protected abstract String getTemplateListeFichiers(  );

    protected abstract String getTemplateCreationFichier(  );

    protected abstract String getTemplateCreationPieceAnnexe(  );

    protected abstract String getTemplateModificationFichier(  );

    protected abstract String getTemplateModificationPieceAnnexe(  );

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.fichier.IFichierJspBean#getFichiersList(javax.servlet.http.HttpServletRequest, boolean)
         */
    public String getFichiersList( HttpServletRequest request, boolean bIsGestionAval )
    {
        setPageTitleProperty( OdsConstants.CONSTANTE_CHAINE_VIDE );

        HashMap<String, Object> model = _fichierService.getFichiersList( request, bIsGestionAval, getHomeUrl( request ) );

        /*
         * Pour la gestion des profils
         */
        ajouterPermissionsDansHashmap( model );

        HtmlTemplate template = AppTemplateService.getTemplate( getTemplateListeFichiers(  ), getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.fichier.IFichierJspBean#doPublicationFichier(javax.servlet.http.HttpServletRequest, boolean)
         */
    public String doPublicationFichier( HttpServletRequest request, boolean bNoReinit )
    {
        /*
         * GESTION DES PROFILS
         * -------------------
         * Si l'utilisateur n'est pas autoris a effectuer cette fonction il est renvoy
         * vers une page lui indiquant qu'il n'a pas les droits ncessaires.
         */
        if ( !_fichierService.isPublier( request ) &&
                !RBACService.isAuthorized( FichiersResourceIdService.FICHIERS, RBAC.WILDCARD_RESOURCES_ID,
                    FichiersResourceIdService.PERMISSION_DEPUBLICATION, getUser(  ) ) )
        {
            return AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                AdminMessage.TYPE_ERROR );
        }

        return _fichierService.doPublicationFichier( request, bNoReinit, getHomeUrl( request ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.fichier.IFichierJspBean#doDownloadFichier(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
         */
    public void doDownloadFichier( HttpServletRequest request, HttpServletResponse response )
    {
        GFichier fichierTelecharger = _fichierService.doDownloadFichier( request );

        if ( fichierTelecharger == null )
        {
            try
            {
                response.sendRedirect( AdminMessageService.getMessageUrl( request,
                        OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND, AdminMessage.TYPE_ERROR ) );
            }
            catch ( IOException ioe )
            {
                AppLogService.error( ioe );

                return;
            }
        }
        else
        {
            try
            {
                String strFileName = fichierTelecharger.getNom(  ) + "." + fichierTelecharger.getExtension(  );

                response.setHeader( "Content-Disposition", "attachment ;filename=\"" + strFileName + "\"" );
                response.setContentType( "application/pdf" );
                response.setHeader( "Pragma", "public" );
                response.setHeader( "Expires", "0" );
                response.setHeader( "Cache-Control", "must-revalidate,post-check=0,pre-check=0" );
                response.addHeader( "Accept-Ranges", "bytes" );
                response.setContentLength( (int) fichierTelecharger.getFichier(  ).getDonnees(  ).length );

                OutputStream os = response.getOutputStream(  );
                os.write( fichierTelecharger.getFichier(  ).getDonnees(  ) );
                os.close(  );
            }
            catch ( IOException e )
            {
                AppLogService.error( e );
            }
        }
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.fichier.IFichierJspBean#getCreationFichier(javax.servlet.http.HttpServletRequest, boolean)
         */
    public String getCreationFichier( HttpServletRequest request, boolean bIsGestionAval )
    {
        setPageTitleProperty( PROPERTY_PAGE_CREATION_FICHIER );

        HashMap<String, Object> model = _fichierService.getCreationFichier( request, bIsGestionAval );

        HtmlTemplate template = AppTemplateService.getTemplate( getTemplateCreationFichier(  ), getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.fichier.IFichierJspBean#getCreationFichierForPDD(javax.servlet.http.HttpServletRequest, int, java.lang.String)
         */
    public String getCreationFichierForPDD( HttpServletRequest request, int nIdTypeDocument, String strUrlRetour )
    {
        setPageTitleProperty( PROPERTY_PAGE_CREATION_FICHIER );

        HashMap<String, Object> model = _fichierService.getCreationFichierForPDD( request, nIdTypeDocument, strUrlRetour );

        HtmlTemplate template = AppTemplateService.getTemplate( getTemplateCreationPieceAnnexe(  ), getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.fichier.IFichierJspBean#getModificationFichierForPDD(javax.servlet.http.HttpServletRequest, int, java.lang.String)
         */
    public String getModificationFichierForPDD( HttpServletRequest request, int nIdTypeDocument, String strUrlRetour )
    {
        setPageTitleProperty( PROPERTY_PAGE_CREATION_FICHIER );

        HashMap<String, Object> model = _fichierService.getModificationFichierForPDD( request, nIdTypeDocument,
                strUrlRetour );

        HtmlTemplate template = AppTemplateService.getTemplate( getTemplateModificationPieceAnnexe(  ), getLocale(  ),
                model );

        return getAdminPage( template.getHtml(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.fichier.IFichierJspBean#getModificationFichier(javax.servlet.http.HttpServletRequest, boolean)
         */
    public String getModificationFichier( HttpServletRequest request, boolean bIsGestionAval )
    {
        HtmlTemplate template;

        /*
         * GESTION DES PROFILS
         * -------------------
         * Si l'utilisateur n'est pas autoris a effectuer cette fonction il est renvoy
         * vers une page lui indiquant qu'il n'a pas les droits ncessaires.
         */
        int nIdFichier = OdsUtils.getIntegerParameter( OdsParameters.ID_FICHIER, request );

        if ( _fichierService.isPublier( nIdFichier ) &&
                !RBACService.isAuthorized( FichiersResourceIdService.FICHIERS, RBAC.WILDCARD_RESOURCES_ID,
                    FichiersResourceIdService.PERMISSION_MISE_A_JOUR, getUser(  ) ) )
        {
            template = AppTemplateService.getTemplate( OdsProperties.TEMPLATE_PROFIL_ERROR, getLocale(  ) );
        }
        else
        {
            setPageTitleProperty( PROPERTY_PAGE_MODIFICATION_FICHIER );

            HashMap<String, Object> model = _fichierService.getModificationFichier( request, bIsGestionAval );
            template = AppTemplateService.getTemplate( getTemplateModificationFichier(  ), getLocale(  ), model );
        }

        return getAdminPage( template.getHtml(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.fichier.IFichierJspBean#getHistoriqueFichier(javax.servlet.http.HttpServletRequest)
         */
    public String getHistoriqueFichier( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_PAGE_HISTORIQUE );

        HashMap<String, Object> model = _fichierService.getHistoriqueFichier( request );

        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_HISTORIQUE, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.fichier.IFichierJspBean#getSuppressionFichier(javax.servlet.http.HttpServletRequest, boolean)
         */
    public String getSuppressionFichier( HttpServletRequest request, boolean bIsGestionAval )
    {
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        /*
        * GESTION DES PROFILS
        * -------------------
        * Si l'utilisateur n'est pas autoris a effectuer cette fonction il est renvoy
        * vers une page lui indiquant qu'il n'a pas les droits ncessaires.
        */
        int nIdFichier = OdsUtils.getIntegerParameter( OdsParameters.ID_FICHIER, request );

        if ( _fichierService.isPublier( nIdFichier ) &&
                !RBACService.isAuthorized( FichiersResourceIdService.FICHIERS, RBAC.WILDCARD_RESOURCES_ID,
                    FichiersResourceIdService.PERMISSION_SUPPRESSION, getUser(  ) ) )
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }
        else
        {
            strReturn = _fichierService.getSuppressionFichier( request, bIsGestionAval );

            if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                strReturn = getHomeUrl( request );
            }
        }

        return strReturn;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.fichier.IFichierJspBean#doSuppressionFichier(javax.servlet.http.HttpServletRequest, boolean)
         */
    public String doSuppressionFichier( HttpServletRequest request, boolean bNoReinit )
    {
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        /*
         * GESTION DES PROFILS
         * -------------------
         * Si l'utilisateur n'est pas autoris a effectuer cette fonction il est renvoy
         * vers une page lui indiquant qu'il n'a pas les droits ncessaires.
         */
        int nIdFichier = OdsUtils.getIntegerParameter( OdsParameters.ID_FICHIER, request );

        if ( _fichierService.isPublier( nIdFichier ) &&
                !RBACService.isAuthorized( FichiersResourceIdService.FICHIERS, RBAC.WILDCARD_RESOURCES_ID,
                    FichiersResourceIdService.PERMISSION_SUPPRESSION, getUser(  ) ) )
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }
        else
        {
            strReturn = _fichierService.doSuppressionFichier( request, bNoReinit, getHomeUrl( request ) );
        }

        return strReturn;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.fichier.IFichierJspBean#doModificationFichier(javax.servlet.http.HttpServletRequest, boolean)
         */
    public String doModificationFichier( HttpServletRequest request, boolean bNoReinit )
    {
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        /*
         * GESTION DES PROFILS
         * -------------------
         * Si l'utilisateur n'est pas autoris a effectuer cette fonction il est renvoy
         * vers une page lui indiquant qu'il n'a pas les droits ncessaires.
         */
        int nIdFichier = OdsUtils.getIntegerParameter( OdsParameters.ID_FICHIER, request );

        if ( _fichierService.isPublier( nIdFichier ) &&
                !RBACService.isAuthorized( FichiersResourceIdService.FICHIERS, RBAC.WILDCARD_RESOURCES_ID,
                    FichiersResourceIdService.PERMISSION_MISE_A_JOUR, getUser(  ) ) )
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }
        else
        {
            strReturn = _fichierService.doModificationFichier( request, bNoReinit, getHomeUrl( request ) );
        }

        return strReturn;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.fichier.IFichierJspBean#doCreationFichier(javax.servlet.http.HttpServletRequest, boolean)
         */
    public String doCreationFichier( HttpServletRequest request, boolean bNoReinit )
    {
        String strReturn = _fichierService.doCreationFichier( request, bNoReinit, getHomeUrl( request ) );

        return strReturn;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.fichier.IFichierJspBean#setProchaineSeance()
         */
    public void setProchaineSeance(  )
    {
        _fichierService.setProchaineSeance(  );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.fichier.IFichierJspBean#setAllSeances(javax.servlet.http.HttpServletRequest)
         */
    public void setAllSeances( HttpServletRequest request )
    {
        _fichierService.setAllSeances( request );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.fichier.IFichierJspBean#setAllTypeDocuments(javax.servlet.http.HttpServletRequest, boolean)
         */
    public void setAllTypeDocuments( HttpServletRequest request, boolean bIsGestionAval )
    {
        _fichierService.setAllTypeDocuments( request, bIsGestionAval );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.fichier.IFichierJspBean#setAllPublication(javax.servlet.http.HttpServletRequest)
         */
    public void setAllPublication( HttpServletRequest request )
    {
        _fichierService.setAllPublication( request );
    }

    /**
     * Permet de stocker toutes les permissions afin de grer les profils au niveau des templates
     * @param model le hashmap contenant les parametres qui vont tre envoys au template
     */
    protected void ajouterPermissionsDansHashmap( Map<String, Object> model )
    {
        /*
         * Permission de dpublier
         */
        boolean bPermissionDepublication = true;

        if ( !RBACService.isAuthorized( FichiersResourceIdService.FICHIERS, RBAC.WILDCARD_RESOURCES_ID,
                    FichiersResourceIdService.PERMISSION_DEPUBLICATION, getUser(  ) ) )
        {
            bPermissionDepublication = false;
        }

        model.put( OdsMarks.MARK_PERMISSION_DEPUBLICATION, bPermissionDepublication );

        /*
         * Permission de modifier
         */
        boolean bPermissionModification = true;

        if ( !RBACService.isAuthorized( FichiersResourceIdService.FICHIERS, RBAC.WILDCARD_RESOURCES_ID,
                    FichiersResourceIdService.PERMISSION_MISE_A_JOUR, getUser(  ) ) )
        {
            bPermissionModification = false;
        }

        model.put( OdsMarks.MARK_PERMISSION_MODIFICATION, bPermissionModification );

        /*
         * Permission de supprimer
         */
        boolean bPermissionSuppression = true;

        if ( !RBACService.isAuthorized( FichiersResourceIdService.FICHIERS, RBAC.WILDCARD_RESOURCES_ID,
                    FichiersResourceIdService.PERMISSION_SUPPRESSION, getUser(  ) ) )
        {
            bPermissionSuppression = false;
        }

        model.put( OdsMarks.MARK_PERMISSION_SUPPRESSION, bPermissionSuppression );
    }
}
