/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web.fichier;

import fr.paris.lutece.portal.service.admin.AccessDeniedException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


public interface IFichierJspBean
{
    String RIGHT_ODS_FICHIERS = "ODS_FICHIERS";
    String RIGHT_ODS_FICHIERS_GESTION_AVAL = "ODS_FICHIERS_GESTION_AVAL";

    /**
     * Initialisation et vrification des droits
     * @param request HttpServletRequest
     * @param response HttpServletResponse
     * @throws AccessDeniedException si l'utilisateur n'a pas les droits suffisants
     */
    void init( HttpServletRequest request, HttpServletResponse response, String strRight )
        throws AccessDeniedException;

    /**
     * Retourne l'interface de gestion des fichiers.
     * @param request la requte http
     * @param bIsGestionAval si on est dans la gestion aval ou pas
     * @return la page affichant la liste des fichiers
     */
    String getFichiersList( HttpServletRequest request, boolean bIsGestionAval );

    /**
     * Publication d'un fichier
     * Si "OdsParameters.FICHIER_IS_PUBLIER" gale  TRUE, alors la mthode <b>publie</b> le fichier et incrmente la version<BR>
     *                 Dans ce cas, un historique est cre
     * Sinon la mthode change le statut enLigne d'un fichier a FALSE
     * @param request la requte http
     * @param bNoReinit FALSE si on reinitialise, TRUE sinon
     * @return l'url
     */
    String doPublicationFichier( HttpServletRequest request, boolean bNoReinit );

    /**
     * Tlecharge un fichier.
     * Renvoie un flux de bytes du fichier dans l'objet HttpServletResponse response
     * @param request la requte http
     * @param response la rponse
     */
    void doDownloadFichier( HttpServletRequest request, HttpServletResponse response );

    /**
     * retourne le formulaire de cration d'un fichier
     * @param request la requte http
     * @param bIsGestionAval TRUE si on est dans la gestion aval, FALSE sinon
     * @return la page de creation de fichier
     */
    String getCreationFichier( HttpServletRequest request, boolean bIsGestionAval );

    /**
     * retourne le formulaire de cration d'une piece annexe pour un pdd
     * @param request la requte http
     * @return la page de cration de fichier pour un PDD
     */
    String getCreationFichierForPDD( HttpServletRequest request, int nIdTypeDocument, String strUrlRetour );

    /**
     * retourne le formulaire de modification d'une piece annexe pour un pdd
     * @param request la requte http
     * @return la page de modification d'un fichier pour un PDD
     */
    String getModificationFichierForPDD( HttpServletRequest request, int nIdTypeDocument, String strUrlRetour );

    /**
     * retourne le formulaire de modification d'un fichier
     * @param request la requte http
     * @param bIsGestionAval TRUE si on est dans la gestion aval, FALSE sinon
     * @return la page de modification de fichier
     */
    String getModificationFichier( HttpServletRequest request, boolean bIsGestionAval );

    /**
     * retourne l'ecran de visualisation de l'historique des versions d'un fichier
     * @param request la requte http
     * @return la page de l'historique d'un fichier
     */
    String getHistoriqueFichier( HttpServletRequest request );

    /**
     * retourne une demande de confirmation pour la suppression du fichier choisie
     * @param request la requte http
     * @param bIsGestionAval TRUE si on est dans la gestion aval, FALSE sinon
     * @return la page de confirmation de suppression d'un fichier
     */
    String getSuppressionFichier( HttpServletRequest request, boolean bIsGestionAval );

    /**
     * supprime le fichier choisi
     * @param request la requte http
     * @param bNoReinit FALSE si on reinitialise, TRUE sinon
     * @return l'url de suppression de fichier
     */
    String doSuppressionFichier( HttpServletRequest request, boolean bNoReinit );

    /**
     * Modifie le fichier et le fichier physique
     * Si le fichie est en ligne, alors la version est incrment de 1 et ajout d'un historique
     * si tous les champs obligatoires du formulaire de modification d'un fichier ont t renseign,<BR>
     * la mthode retourne l'url de gestion des fichiers,<BR>
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request la requte http
     * @param bNoReinit FALSE si on reinitialise, TRUE sinon
     * @return l'url de modification d'un fichier
     */
    String doModificationFichier( HttpServletRequest request, boolean bNoReinit );

    /**
     * Creation d'un fichier
     * si tous les champs obligatoires du formulaire de cration d'un fichier ont t renseign,<BR>
     * la mthode retourne l'url de gestion des fichiers,<BR>
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request la requte http
     * @param bNoReinit FALSE si on reinitialise, TRUE sinon
     * @return l'url de creation d'un fichier
     */
    String doCreationFichier( HttpServletRequest request, boolean bNoReinit );

    /**
     * Permet d'initialiser la prochaine seance
     */
    void setProchaineSeance(  );

    /**
     * Permet d'initialiser la dernire seance
     * @param request la requete Http
     */
    void setAllSeances( HttpServletRequest request );

    /**
     * Permet d'initialiser la liste des diffrents types de documents
     * @param request la requte http
     * @param bIsGestionAval TRUE si on est dans la gestion aval, FALSE sinon
     */
    void setAllTypeDocuments( HttpServletRequest request, boolean bIsGestionAval );

    /**
     * Permet d'initialiser la liste des choix de publications
     * Les choix possibles: OUI, NON ou Pas d'importance
     * @param request la requte http
     */
    void setAllPublication( HttpServletRequest request );
}
