/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web.groupepolitique;

import fr.paris.lutece.plugins.ods.service.groupepolitique.IGroupePolitiqueService;
import fr.paris.lutece.plugins.ods.service.role.AdminSaufOdjResourceIdService;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.business.rbac.RBAC;
import fr.paris.lutece.portal.service.admin.AccessDeniedException;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.rbac.RBACService;
import fr.paris.lutece.portal.service.template.AppTemplateService;
import fr.paris.lutece.portal.web.admin.PluginAdminPageJspBean;
import fr.paris.lutece.util.html.HtmlTemplate;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


/**
 *
 * gestion des groupes politiques
 *
 */
public abstract class AbstractGroupePolitiqueJspBean extends PluginAdminPageJspBean implements IGroupePolitiqueJspBean
{
    private static final long serialVersionUID = 6312193547755167116L;
    public static final String RIGHT_ODS_GROUPE_POLITIQUE = "ODS_GROUPES_POLITIQUES";
    private static final String TEMPLATE_LISTE_GROUPES = "admin/plugins/ods/groupepolitique/liste_groupes.html";
    private static final String TEMPLATE_CREATION_GROUPE = "admin/plugins/ods/groupepolitique/creation_groupe.html";
    private static final String TEMPLATE_MODIFICATION_GROUPE = "admin/plugins/ods/groupepolitique/modification_groupe.html";
    private static final String PROPERTY_PAGE_MODIFICATION_GROUPE = "ods.modificationgroupe.label.title";
    private static final String PROPERTY_PAGE_CREATION_GROUPE = "ods.creationgroupe.label.title";
    @Autowired
    private IGroupePolitiqueService _groupePolitiqueService;

    /**
     * Init
     * @param request
     * @throws AccessDeniedException
     */
    public void init( HttpServletRequest request, HttpServletResponse response )
        throws AccessDeniedException
    {
        super.init( request, getRight(  ) );
    }

    protected String getRight(  )
    {
        return RIGHT_ODS_GROUPE_POLITIQUE;
    }

    /**
     * retourne l'interface de gestion des groupes politiques.
     * @param request la requete HTTP
     * @return   interface de gestion des groupes politiques
     */
    public String getGroupeList( HttpServletRequest request )
    {
        setPageTitleProperty( OdsConstants.CONSTANTE_CHAINE_VIDE );

        HashMap<String, Object> model = _groupePolitiqueService.getGroupeList( request );

        ajouterPermissionsDansHashmap( model );

        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_LISTE_GROUPES, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * retourne le formulaire de cration d'un groupe politique
     * @param request la requete HTTP
     * @return formulaire de cration d'un groupe politique
     */
    public String getCreationGroupe( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_PAGE_CREATION_GROUPE );

        HtmlTemplate template;

        if ( RBACService.isAuthorized( AdminSaufOdjResourceIdService.ADMINISTRATION_SAUF_ODJ,
                    RBAC.WILDCARD_RESOURCES_ID, AdminSaufOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            template = AppTemplateService.getTemplate( TEMPLATE_CREATION_GROUPE, getLocale(  ) );
        }
        else
        {
            template = AppTemplateService.getTemplate( OdsProperties.TEMPLATE_PROFIL_ERROR, getLocale(  ) );
        }

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * cre un groupe politique.
     * si tous les champs obligatoires du formulaire de cration d'un groupe politique ont t renseign,
     * la mthode retourne l'url de gestion des groupes poliques,
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request request
     * @return url url de retour
     */
    public String doCreationGroupe( HttpServletRequest request )
    {
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( RBACService.isAuthorized( AdminSaufOdjResourceIdService.ADMINISTRATION_SAUF_ODJ,
                    RBAC.WILDCARD_RESOURCES_ID, AdminSaufOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            strReturn = _groupePolitiqueService.doCreationGroupe( request );

            if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                strReturn = getHomeUrl( request );
            }
        }
        else
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }

        return strReturn;
    }

    /**
     * retourne le formulaire de modification d'un groupe politique  pr remplis avec  les critres  du groupe politique choisi
     * @param request la requete HTTP
     * @return formulaire de modification d'un groupe politique
     */
    public String getModificationGroupe( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_PAGE_MODIFICATION_GROUPE );

        HtmlTemplate template;

        if ( RBACService.isAuthorized( AdminSaufOdjResourceIdService.ADMINISTRATION_SAUF_ODJ,
                    RBAC.WILDCARD_RESOURCES_ID, AdminSaufOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            HashMap<String, Object> model = _groupePolitiqueService.getModificationGroupe( request );

            if ( model == null )
            {
                return getGroupeList( request );
            }

            template = AppTemplateService.getTemplate( TEMPLATE_MODIFICATION_GROUPE, getLocale(  ), model );
        }
        else
        {
            template = AppTemplateService.getTemplate( OdsProperties.TEMPLATE_PROFIL_ERROR, getLocale(  ) );
        }

        return ( getAdminPage( template.getHtml(  ) ) );
    }

    /**
     * Modifie un groupe politique.
     * si tous les champs obligatoires du formulaire de modification d'un groupe politique ont t renseign,
     * la mthode retourne l'url de gestion des groupes poliques,
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request request
     * @return url url de retour
     */
    public String doModificationGroupe( HttpServletRequest request )
    {
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( RBACService.isAuthorized( AdminSaufOdjResourceIdService.ADMINISTRATION_SAUF_ODJ,
                    RBAC.WILDCARD_RESOURCES_ID, AdminSaufOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            strReturn = _groupePolitiqueService.doModificationGroupe( request );

            if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                strReturn = getHomeUrl( request );
            }
        }
        else
        {
            return AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                AdminMessage.TYPE_ERROR );
        }

        return strReturn;
    }

    /**
     * retourne une demande de confirmation pour la suppression du  groupe politique  choisi
     * @param request la requete HTTP
     * @return interface de confirmation de suppression
     */
    public String getSuppressionGroupe( HttpServletRequest request )
    {
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( RBACService.isAuthorized( AdminSaufOdjResourceIdService.ADMINISTRATION_SAUF_ODJ,
                    RBAC.WILDCARD_RESOURCES_ID, AdminSaufOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            strReturn = _groupePolitiqueService.getSuppressionGroupe( request );

            if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                strReturn = getHomeUrl( request );
            }
        }
        else
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }

        return strReturn;
    }

    /**
     * supprime le groupe politique choisi
     * @param request la requete HTTP
     * @return url de gestion des groupes poliques
     */
    public String doSuppressionGroupe( HttpServletRequest request )
    {
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( RBACService.isAuthorized( AdminSaufOdjResourceIdService.ADMINISTRATION_SAUF_ODJ,
                    RBAC.WILDCARD_RESOURCES_ID, AdminSaufOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            strReturn = _groupePolitiqueService.doSuppressionGroupe( request );

            if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                strReturn = getHomeUrl( request );
            }
        }
        else
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }

        return strReturn;
    }

    /**
     * Permet de stocker toutes les permissions afin de grer les profils au niveau des templates
     * @param model le hashmap contenant les parametres qui vont tre envoys au template
     */
    public void ajouterPermissionsDansHashmap( Map<String, Object> model )
    {
        /*
         * Permission de grer
         */
        boolean bPermissionGestion = true;

        if ( !RBACService.isAuthorized( AdminSaufOdjResourceIdService.ADMINISTRATION_SAUF_ODJ,
                    RBAC.WILDCARD_RESOURCES_ID, AdminSaufOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            bPermissionGestion = false;
        }

        model.put( OdsMarks.MARK_PERMISSION_GESTION, bPermissionGestion );
    }

    public void initGroupePolitiqueBean(  )
    {
        // rien
    }
}
