/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web.horodatage;

import fr.paris.lutece.plugins.ods.service.horodatage.IHorodatageService;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.template.AppTemplateService;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.web.admin.PluginAdminPageJspBean;
import fr.paris.lutece.util.html.HtmlTemplate;

import org.springframework.beans.factory.annotation.Autowired;

import java.io.IOException;
import java.io.OutputStream;

import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


/**
 *
 * HorodatageJspBean
 *
 */
public class HorodatageJspBean extends PluginAdminPageJspBean
{
    private static final long serialVersionUID = -7637544051529008374L;
    public static final String RIGHT_ODS_HORODATAGE = "ODS_HORODATAGE";
    private static final String TEMPLATE_LISTE_HORODATAGE = "admin/plugins/ods/horodatage/horodatage.html";
    private static final String MESSAGE_ERROR_DOWNLOAD = "ods.horodatage.message.errorDownload";
    @Autowired
    private IHorodatageService _horodatageService;

    /**
     * Retourne l'interface de gestion de l'horodatage
     *
     * @param request le requte HTTP
     * @return l'interface de gestion des directions
     */
    public String getHorodatage( HttpServletRequest request )
    {
        setPageTitleProperty( OdsConstants.CONSTANTE_CHAINE_VIDE );

        HashMap<String, Object> model = _horodatageService.getHorodatage( request );

        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_LISTE_HORODATAGE, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Permet de renvoyer le flux de donnes du fichier a telecharger
     *
     * @param request HttpServletRequest
     * @param response HttpServletResponse
     * @return Le flux de donnes du fichier a telecharger
     */
    public String doDownloadCSV( HttpServletRequest request, HttpServletResponse response )
    {
        String strReturn = _horodatageService.doDownloadCSV( request );

        if ( ( strReturn != null ) && strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            try
            {
                download( response );
                strReturn = getHorodatage( request );
            }
            catch ( IOException e )
            {
                AppLogService.error( e );

                strReturn = AdminMessageService.getMessageUrl( request, MESSAGE_ERROR_DOWNLOAD, AdminMessage.TYPE_STOP );
            }
        }

        return strReturn;
    }

    /**
     * Permet de crer le flux de donnes du fichier a tlcharger
     *
     * @param response HttpServletResponse
     * @param typeHorodatage Type PUBLICATION ou NOTIFICATION
     * @param dateStart Filtre sur la date de dbut des traces
     * @param dateEnd Filtre sur la date de fin des traces
     * @throws IOException IOException
     */
    private void download( HttpServletResponse response )
        throws IOException
    {
        byte[] fichier = _horodatageService.getFichier(  );

        if ( fichier != null )
        {
            String strFileName = "export.csv";

            response.setHeader( "Content-Disposition", "attachment ;filename=\"" + strFileName + "\"" );
            response.setContentType( "application/text" );
            response.setHeader( "Pragma", "public" );
            response.setHeader( "Expires", "0" );
            response.setHeader( "Cache-Control", "must-revalidate,post-check=0,pre-check=0" );
            response.setContentLength( (int) fichier.length );

            OutputStream os = response.getOutputStream(  );
            os.write( fichier );
            os.close(  );
        }
    }
}
