/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web.indexer;

import fr.paris.lutece.plugins.ods.service.indexer.IIndexerService;
import fr.paris.lutece.portal.service.template.AppTemplateService;
import fr.paris.lutece.portal.web.admin.PluginAdminPageJspBean;
import fr.paris.lutece.util.html.HtmlTemplate;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;


/**
 * Gestion des indexations compltes
 */
public class IndexerJspBean extends PluginAdminPageJspBean
{
    private static final long serialVersionUID = -5644987481361434597L;
    public static final String RIGHT_ODS_INDEXER = "ODS_INDEXER";
    private static final String TEMPLATE_LISTE_INDEXER = "admin/plugins/ods/indexer/liste_indexer.html";
    private static final String PROPERTY_PAGE_TITLE = "ods.listeindexer.page.title";
    @Autowired
    private IIndexerService _indexerService;

    /**
     * Retourne la page de gestion des indexers
     * @param request la requte Http
     * @return la liste des indexers
     */
    public String getListeIndexer( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_PAGE_TITLE );

        HashMap<String, Object> model = _indexerService.getListeIndexer( request );

        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_LISTE_INDEXER, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Retourne la page de confirmation de demande d'indexation
     * @param request la requte Http
     * @return un message de confirmation
     */
    public String getDemandeIndexation( HttpServletRequest request )
    {
        return _indexerService.getDemandeIndexation( request );
    }

    /**
     * Demande ou annule la demande d'indexation complte
     * @param request la erqute Http
     * @return la page de gestion des inedexers
     */
    public String doDemandeIndexation( HttpServletRequest request )
    {
        _indexerService.doDemandeIndexation( request );

        return getHomeUrl( request );
    }
}
