/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web.listemembrepresent;

import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.service.listemembrepresent.IListeMembrePresentService;
import fr.paris.lutece.plugins.ods.service.role.DeliberationDesignationGestionAvalResourceIdService;
import fr.paris.lutece.plugins.ods.service.role.ListeMembrePresentGestionAvalResourceIdService;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.business.rbac.RBAC;
import fr.paris.lutece.portal.service.admin.AccessDeniedException;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.rbac.RBACService;
import fr.paris.lutece.portal.service.template.AppTemplateService;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.service.util.AppPathService;
import fr.paris.lutece.portal.web.admin.PluginAdminPageJspBean;
import fr.paris.lutece.util.html.HtmlTemplate;

import org.springframework.beans.factory.annotation.Autowired;

import java.io.ByteArrayOutputStream;
import java.io.IOException;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


public abstract class AbstractListeMembrePresentJspBean<GSeance extends ISeance, GFichier extends IFichier<GSeance, GFichier>>
    extends PluginAdminPageJspBean implements IListeMembrePresentJspBean
{
    private static final long serialVersionUID = -5152857573861357101L;
    private static final String TEMPLATE_HISTORIQUE = "admin/plugins/ods/historique/historique.html";
    private static final String TEMPLATE_CONFIRMATION_FINALISATION = "admin/plugins/ods/listemembrepresent/confirmation_finaliser_acte.html";
    private static final String PROPERTY_PAGE_CREATION_LISTE_MEMBRE_PRESENT = "ods.listemembrepresent.creation.pageTitle";
    private static final String PROPERTY_PAGE_MODIFICATION_LISTE_MEMBRE_PRESENT = "ods.listemembrepresent.modification.pageTitle";
    private static final String PROPERTY_PAGE_HISTORIQUE = "ods.historique.pageTitle";
    @Autowired
    protected IListeMembrePresentService<GSeance, GFichier> _listeMembrePresentService;

    protected abstract String getTemplateListMembrePresent(  );

    protected abstract String getTemplateCreationListeMembrePresent(  );

    protected abstract String getTemplateModificationListeMembrePresent(  );

    public void init( HttpServletRequest request, HttpServletResponse response, String strRight )
        throws AccessDeniedException
    {
        super.init( request, strRight );
    }

    public void init( HttpServletRequest request, String strRight )
        throws AccessDeniedException
    {
        super.init( request, strRight );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.listemembrepresent.IListeMembrePresentJspBean#reinitSession(javax.servlet.http.HttpServletRequest, java.lang.String)
         */
    public void reinitSession( HttpServletRequest request, HttpServletResponse response, String strRight )
        throws AccessDeniedException
    {
        super.init( request, strRight );

        _listeMembrePresentService.reinitSession( request );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.listemembrepresent.IListeMembrePresentJspBean#initUrlRetour(java.lang.String, javax.servlet.http.HttpServletRequest)
         */
    public void initUrlRetour( String strUrl, HttpServletRequest request )
    {
        _listeMembrePresentService.initUrlRetour( strUrl, request );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.listemembrepresent.IListeMembrePresentJspBean#getAdminListeMembrePresent(javax.servlet.http.HttpServletRequest)
         */
    public String getAdminListeMembrePresent( HttpServletRequest request )
    {
        setPageTitleProperty( OdsConstants.CONSTANTE_CHAINE_VIDE );

        HashMap<String, Object> model = _listeMembrePresentService.getAdminListeMembrePresent( request );

        ajouterPermissionsDansHashmap( model );

        HtmlTemplate template = AppTemplateService.getTemplate( getTemplateListMembrePresent(  ), getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.listemembrepresent.IListeMembrePresentJspBean#doChangementEtatListeMembrePresent(javax.servlet.http.HttpServletRequest)
         */
    public String doChangementEtatListeMembrePresent( HttpServletRequest request )
    {
        String strReturn = _listeMembrePresentService.doChangementEtatListeMembrePresent( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.listemembrepresent.IListeMembrePresentJspBean#getCreationListeMembrePresent(javax.servlet.http.HttpServletRequest)
         */
    public String getCreationListeMembrePresent( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_PAGE_CREATION_LISTE_MEMBRE_PRESENT );

        HashMap<String, Object> model = _listeMembrePresentService.getCreationListeMembrePresent( request );

        HtmlTemplate template = AppTemplateService.getTemplate( getTemplateCreationListeMembrePresent(  ),
                getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.listemembrepresent.IListeMembrePresentJspBean#doFinaliserActe(javax.servlet.http.HttpServletRequest)
         */
    public String doFinaliserActe( HttpServletRequest request )
    {
        String strReturn = _listeMembrePresentService.doFinaliserActe( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.listemembrepresent.IListeMembrePresentJspBean#getConfirmFinaliser(javax.servlet.http.HttpServletRequest)
         */
    public String getConfirmFinaliser( HttpServletRequest request )
    {
        HashMap<String, Object> model = _listeMembrePresentService.getConfirmFinaliser( request );

        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_CONFIRMATION_FINALISATION, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.listemembrepresent.IListeMembrePresentJspBean#getModificationListeMembrePresent(javax.servlet.http.HttpServletRequest)
         */
    public String getModificationListeMembrePresent( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_PAGE_MODIFICATION_LISTE_MEMBRE_PRESENT );

        HtmlTemplate template;

        /*
         * GESTION DES PROFILS
         * -------------------
         * Si l'utilisateur n'est pas autoris a effectuer cette fonction il est renvoy
         * vers une page lui indiquant qu'il n'a pas les droits ncessaires.
         */
        int nIdListeMembre = OdsUtils.getIntegerParameter( OdsParameters.ID_LISTE_MEMBRE_PRESENT, request );

        if ( _listeMembrePresentService.isPublier( nIdListeMembre ) &&
                !RBACService.isAuthorized( 
                    ListeMembrePresentGestionAvalResourceIdService.TYPE_LISTE_MEMBRE_PRESENT_GESTION_AVAL,
                    RBAC.WILDCARD_RESOURCES_ID, ListeMembrePresentGestionAvalResourceIdService.PERMISSION_MISE_A_JOUR,
                    getUser(  ) ) )
        {
            template = AppTemplateService.getTemplate( OdsProperties.TEMPLATE_PROFIL_ERROR, getLocale(  ) );

            return getAdminPage( template.getHtml(  ) );
        }

        HashMap<String, Object> model = _listeMembrePresentService.getModificationListeMembrePresent( request );
        template = AppTemplateService.getTemplate( getTemplateModificationListeMembrePresent(  ), getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.listemembrepresent.IListeMembrePresentJspBean#getHistoriqueListeMembrePresent(javax.servlet.http.HttpServletRequest)
         */
    public String getHistoriqueListeMembrePresent( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_PAGE_HISTORIQUE );

        HashMap<String, Object> model = _listeMembrePresentService.getHistoriqueListeMembrePresent( request );

        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_HISTORIQUE, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.listemembrepresent.IListeMembrePresentJspBean#getSuppressionListeMembrePresent(javax.servlet.http.HttpServletRequest)
         */
    public String getSuppressionListeMembrePresent( HttpServletRequest request )
    {
        return _listeMembrePresentService.getSuppressionListeMembrePresent( request );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.listemembrepresent.IListeMembrePresentJspBean#doSuppressionListeMembrePresent(javax.servlet.http.HttpServletRequest)
         */
    public String doSuppressionListeMembrePresent( HttpServletRequest request )
    {
        int nIdListeMembre = OdsUtils.getIntegerParameter( OdsParameters.ID_LISTE_MEMBRE_PRESENT, request );

        if ( nIdListeMembre == -1 )
        {
            return getHomeUrl( request );
        }

        /*
         * GESTION DES PROFILS
         * -------------------
         * Si l'utilisateur n'est pas autoris a effectuer cette fonction il est renvoy
         * vers une page lui indiquant qu'il n'a pas les droits ncessaires.
         */
        if ( _listeMembrePresentService.isPublier( nIdListeMembre ) &&
                !RBACService.isAuthorized( 
                    ListeMembrePresentGestionAvalResourceIdService.TYPE_LISTE_MEMBRE_PRESENT_GESTION_AVAL,
                    RBAC.WILDCARD_RESOURCES_ID, ListeMembrePresentGestionAvalResourceIdService.PERMISSION_SUPPRESSION,
                    getUser(  ) ) )
        {
            return AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                AdminMessage.TYPE_ERROR );
        }

        String strReturn = _listeMembrePresentService.doSuppressionListeMembrePresent( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.listemembrepresent.IListeMembrePresentJspBean#doModificationListeMembrePresent(javax.servlet.http.HttpServletRequest)
         */
    public String doModificationListeMembrePresent( HttpServletRequest request )
    {
        /*
         * GESTION DES PROFILS
         * -------------------
         * Si l'utilisateur n'est pas autorise a effectuer cette fonction,
         * il est renvoye vers une page lui indiquant qu'il n'a pas les droits ncessaires.
         */
        if ( !RBACService.isAuthorized( 
                    ListeMembrePresentGestionAvalResourceIdService.TYPE_LISTE_MEMBRE_PRESENT_GESTION_AVAL,
                    RBAC.WILDCARD_RESOURCES_ID, ListeMembrePresentGestionAvalResourceIdService.PERMISSION_MISE_A_JOUR,
                    getUser(  ) ) )
        {
            return AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                AdminMessage.TYPE_ERROR );
        }

        String strReturn = _listeMembrePresentService.doModificationListeMembrePresent( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.listemembrepresent.IListeMembrePresentJspBean#doCreationListeMembrePresent(javax.servlet.http.HttpServletRequest)
         */
    public String doCreationListeMembrePresent( HttpServletRequest request )
    {
        /*
         * GESTION DES PROFILS
         * -------------------
         * Si l'utilisateur n'est pas autoris a effectuer cette fonction,
         * il est renvoy vers une page lui indiquant qu'il n'a pas les droits ncessaires.
         */
        if ( !RBACService.isAuthorized( 
                    ListeMembrePresentGestionAvalResourceIdService.TYPE_LISTE_MEMBRE_PRESENT_GESTION_AVAL,
                    RBAC.WILDCARD_RESOURCES_ID, ListeMembrePresentGestionAvalResourceIdService.PERMISSION_CREATION,
                    getUser(  ) ) )
        {
            return AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                AdminMessage.TYPE_ERROR );
        }

        String strReturn = _listeMembrePresentService.doCreationListeMembrePresent( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.listemembrepresent.IListeMembrePresentJspBean#doDownloadFichier(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
         */
    public void doDownloadFichier( HttpServletRequest request, HttpServletResponse response )
    {
        GFichier fichier = _listeMembrePresentService.getFichier( request );

        if ( fichier != null )
        {
            try
            {
                String strFileName = fichier.getNom(  ) + "." + fichier.getExtension(  );

                ByteArrayOutputStream ba = new ByteArrayOutputStream(  );
                ba.write( fichier.getFichier(  ).getDonnees(  ) );
                response.setContentLength( ba.size(  ) );
                response.setHeader( "Content-Disposition", "attachment ;filename=\"" + strFileName + "\"" );
                response.setContentType( "application/pdf" );
                response.setHeader( "Pragma", "public" );
                response.setHeader( "Expires", "0" );
                response.setHeader( "Cache-Control", "must-revalidate,post-check=0,pre-check=0" );

                ba.writeTo( response.getOutputStream(  ) );
                response.getOutputStream(  ).flush(  );
            }
            catch ( IOException e )
            {
                AppLogService.error( e );
            }
        }
    }

    /**
     * Permet de stocker toutes les permissions afin de grer les profils au niveau des templates
     * @param model le hashmap contenant les parametres qui vont tre envoys au template
     */
    protected void ajouterPermissionsDansHashmap( Map<String, Object> model )
    {
        //Permission de dpublier
        boolean bPermissionDepublication = true;

        if ( !RBACService.isAuthorized( DeliberationDesignationGestionAvalResourceIdService.DELIBERATION_GESTION_AVAL,
                    RBAC.WILDCARD_RESOURCES_ID,
                    DeliberationDesignationGestionAvalResourceIdService.PERMISSION_DEPUBLICATION, getUser(  ) ) )
        {
            bPermissionDepublication = false;
        }

        model.put( OdsMarks.MARK_PERMISSION_DEPUBLICATION, bPermissionDepublication );

        // Permission de modifier
        boolean bPermissionModification = true;

        if ( !RBACService.isAuthorized( DeliberationDesignationGestionAvalResourceIdService.DELIBERATION_GESTION_AVAL,
                    RBAC.WILDCARD_RESOURCES_ID,
                    DeliberationDesignationGestionAvalResourceIdService.PERMISSION_MISE_A_JOUR, getUser(  ) ) )
        {
            bPermissionModification = false;
        }

        model.put( OdsMarks.MARK_PERMISSION_MODIFICATION, bPermissionModification );

        // Permission de supprimer
        boolean bPermissionSuppression = true;

        if ( !RBACService.isAuthorized( DeliberationDesignationGestionAvalResourceIdService.DELIBERATION_GESTION_AVAL,
                    RBAC.WILDCARD_RESOURCES_ID,
                    DeliberationDesignationGestionAvalResourceIdService.PERMISSION_SUPPRESSION, getUser(  ) ) )
        {
            bPermissionSuppression = false;
        }

        model.put( OdsMarks.MARK_PERMISSION_SUPPRESSION, bPermissionSuppression );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.listemembrepresent.IListeMembrePresentJspBean#getHomeUrl(javax.servlet.http.HttpServletRequest)
         */
    public String getHomeUrl( HttpServletRequest request )
    {
        String strJspReturn = AppPathService.getBaseUrl( request );
        strJspReturn += JSP_URL_LISTE_MEMBRE_PRESENT;

        return strJspReturn;
    }
}
