/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web.listemembrepresent;

import fr.paris.lutece.portal.service.admin.AccessDeniedException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


public interface IListeMembrePresentJspBean
{
    String RIGHT_ODS_LISTE_MEMBRE_PRESENT_GESTION_AVAL = "ODS_LISTES_MEMBRES_PRESENTS_GESTION_AVAL";
    String JSP_URL_LISTE_MEMBRE_PRESENT = "jsp/admin/plugins/ods/listemembrepresent/AdminListeMembrePresentNoReinit.jsp";

    /**
     * Reinitialise tous les attributs de session
     * @param request requete HTTP
     * @param strRight droits
     * @throws AccessDeniedException   AccessDeniedException
     */
    void reinitSession( HttpServletRequest request, HttpServletResponse response, String strRight )
        throws AccessDeniedException;

    void init( HttpServletRequest request, HttpServletResponse response, String strRight )
        throws AccessDeniedException;

    void init( HttpServletRequest request, String strRight )
        throws AccessDeniedException;

    /**
     * Permet d'initialiser l'url de retour
     * @param strUrl url de retour
     * @param request la requte http
     */
    void initUrlRetour( String strUrl, HttpServletRequest request );

    /**
     * Retourne l'interface de gestion de la liste des membres prsents.
     * @param request la requte http
     * @return la page affichant la liste des membres prsents
     */
    String getAdminListeMembrePresent( HttpServletRequest request );

    /**
     * Publication ou dpublication  d'une liste de membre prsent
     *
     * @param request la requte http
     * @return l'url
     */
    String doChangementEtatListeMembrePresent( HttpServletRequest request );

    /**
     * retourne le formulaire de cration d'une liste de membre prsent
     * @param request la requte http
     * @return la page de cration d'une liste de membre prsent
     */
    String getCreationListeMembrePresent( HttpServletRequest request );

    /**
     * Permet de confirmer la finalisation d'un acte
     *
     * @param request
     *            la requete HTTP
     * @return strUrl url de retour
     */
    String doFinaliserActe( HttpServletRequest request );

    /**
     * retourne le formulaire de confirmation d'envoi manuel de l'acte
     * @param request la requte http
     * @return la page de confirmation
     */
    String getConfirmFinaliser( HttpServletRequest request );

    /**
     * retourne le formulaire de modificationd'une liste de membre prsent
     * @param request la requte http
     * @param
     * @return la page de modification d'une liste de membre prsent
     */
    String getModificationListeMembrePresent( HttpServletRequest request );

    /**
     * retourne l'ecran de visualisation de l'historique des versions d'une liste de membre prsent
     * @param request la requte http
     * @return la page de l'historique des membres prsent
     */
    String getHistoriqueListeMembrePresent( HttpServletRequest request );

    /**
     * retourne une demande de confirmation pour la suppression de la liste des membres prsents
     * @param request la requte http
     * @return la page de confirmation de suppression de la liste des membres prsents
     */
    String getSuppressionListeMembrePresent( HttpServletRequest request );

    /**
     * supprime la liste de membre prsent choisie
     * @param request la requte http
     * @return l'url de suppression de la dlibration de dsignation
     */
    String doSuppressionListeMembrePresent( HttpServletRequest request );

    /**
     * Modifie la dlibration de dsignation
     * Si la dlibration est en ligne, alors la version est incrmente de 1 et ajout d'un historique
     * si tous les champs obligatoires du formulaire de modification d'une dlibration ont t renseigns,<BR>
     * la mthode retourne l'url de gestion des dlibrations,<BR>
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request la requte http
     * @return l'url de modification d'une dlibration
     */
    String doModificationListeMembrePresent( HttpServletRequest request );

    /**
     * Creation d'une liste de membre prsent
     * si tous les champs obligatoires du formulaire de cration d'une liste de membre prsent ont t renseign,<BR>
     * la mthode retourne l'url de gestion des dlibrations,<BR>
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request la requte http
     * @return l'url de creation d'une dlibration de dsignation
     */
    String doCreationListeMembrePresent( HttpServletRequest request );

    /**
     * Tlcharge un fichier. Renvoie un flux de bytes du fichier dans l'objet
     * HttpServletResponse response
     *
     * @param request
     *            la requte HTTP
     * @param response
     *            la reponse HTTP
     */
    void doDownloadFichier( HttpServletRequest request, HttpServletResponse response );

    /**
     * Retourne l'URL de la page d'accueil des arretes
     * @param request la requete
     * @return l'URL de la page d'accueil des arretes
     */
    String getHomeUrl( HttpServletRequest request );
}
