/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web.modeleordredujour;

import fr.paris.lutece.plugins.ods.dto.fichier.FichierPhysique;
import fr.paris.lutece.plugins.ods.service.modeleordredujour.IModeleOrdreDuJourService;
import fr.paris.lutece.plugins.ods.service.role.ModelesOdjResourceIdService;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.business.rbac.RBAC;
import fr.paris.lutece.portal.service.admin.AccessDeniedException;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.rbac.RBACService;
import fr.paris.lutece.portal.service.template.AppTemplateService;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.web.admin.PluginAdminPageJspBean;
import fr.paris.lutece.util.html.HtmlTemplate;

import org.springframework.beans.factory.annotation.Autowired;

import java.io.IOException;
import java.io.OutputStream;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


/**
 * ModeleOrdreDuJourJspBean
 */
public abstract class AbstractModeleOrdreDuJourJspBean extends PluginAdminPageJspBean
    implements IModeleOrdreDuJourJspBean
{
    private static final long serialVersionUID = 1549747135565041217L;
    private static final String TEMPLATE_LISTE_MODELESODJ = "admin/plugins/ods/modeleordredujour/liste_modeles.html";
    private static final String TEMPLATE_CREATION_MODELEODJ = "admin/plugins/ods/modeleordredujour/creation_modele.html";
    private static final String TEMPLATE_MODIFICATION_MODELEODJ = "admin/plugins/ods/modeleordredujour/modification_modele.html";
    @Autowired
    private IModeleOrdreDuJourService _modeleOrdreDuJourService;

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.modeleordredujour.IModeleOrdreDuJourJspBean#init(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse, java.lang.String)
         */
    public void init( HttpServletRequest request, HttpServletResponse response, String strRight )
        throws AccessDeniedException
    {
        super.init( request, strRight );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.modeleordredujour.IModeleOrdreDuJourJspBean#getModelesOdjList(javax.servlet.http.HttpServletRequest)
         */
    public String getModelesOdjList( HttpServletRequest request )
    {
        setPageTitleProperty( OdsConstants.CONSTANTE_CHAINE_VIDE );

        HashMap<String, Object> model = _modeleOrdreDuJourService.getModelesOdjList( request );

        ajouterPermissionsDansHashmap( model );

        HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_LISTE_MODELESODJ, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.modeleordredujour.IModeleOrdreDuJourJspBean#getCreationModeleOdj(javax.servlet.http.HttpServletRequest)
         */
    public String getCreationModeleOdj( HttpServletRequest request )
    {
        setPageTitleProperty( OdsConstants.CONSTANTE_CHAINE_VIDE );

        HtmlTemplate template;

        if ( RBACService.isAuthorized( ModelesOdjResourceIdService.MODELES_ODJ, RBAC.WILDCARD_RESOURCES_ID,
                    ModelesOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            HashMap<String, Object> model = _modeleOrdreDuJourService.getCreationModeleOdj( request );
            template = AppTemplateService.getTemplate( TEMPLATE_CREATION_MODELEODJ, getLocale(  ), model );
        }
        else
        {
            template = AppTemplateService.getTemplate( OdsProperties.TEMPLATE_PROFIL_ERROR, getLocale(  ) );
        }

        return getAdminPage( template.getHtml(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.modeleordredujour.IModeleOrdreDuJourJspBean#getModificationModeleOdj(javax.servlet.http.HttpServletRequest)
         */
    public String getModificationModeleOdj( HttpServletRequest request )
    {
        setPageTitleProperty( OdsConstants.CONSTANTE_CHAINE_VIDE );

        HtmlTemplate template;

        if ( RBACService.isAuthorized( ModelesOdjResourceIdService.MODELES_ODJ, RBAC.WILDCARD_RESOURCES_ID,
                    ModelesOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            HashMap<String, Object> model = _modeleOrdreDuJourService.getModificationModeleOdj( request );
            template = AppTemplateService.getTemplate( TEMPLATE_MODIFICATION_MODELEODJ, getLocale(  ), model );
        }
        else
        {
            template = AppTemplateService.getTemplate( OdsProperties.TEMPLATE_PROFIL_ERROR, getLocale(  ) );
        }

        return getAdminPage( template.getHtml(  ) );
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.modeleordredujour.IModeleOrdreDuJourJspBean#doCreationModeleOdj(javax.servlet.http.HttpServletRequest)
         */
    public String doCreationModeleOdj( HttpServletRequest request )
    {
        String strReturn;

        if ( RBACService.isAuthorized( ModelesOdjResourceIdService.MODELES_ODJ, RBAC.WILDCARD_RESOURCES_ID,
                    ModelesOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            strReturn = _modeleOrdreDuJourService.doCreationModeleOdj( request );

            if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                strReturn = getHomeUrl( request );
            }
        }
        else
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }

        return strReturn;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.modeleordredujour.IModeleOrdreDuJourJspBean#doModificationModeleOdj(javax.servlet.http.HttpServletRequest)
         */
    public String doModificationModeleOdj( HttpServletRequest request )
    {
        String strReturn;

        if ( RBACService.isAuthorized( ModelesOdjResourceIdService.MODELES_ODJ, RBAC.WILDCARD_RESOURCES_ID,
                    ModelesOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            strReturn = _modeleOrdreDuJourService.doModificationModeleOdj( request );

            if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                strReturn = getHomeUrl( request );
            }
        }
        else
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }

        return strReturn;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.modeleordredujour.IModeleOrdreDuJourJspBean#getSuppressionModele(javax.servlet.http.HttpServletRequest)
         */
    public String getSuppressionModele( HttpServletRequest request )
    {
        String strReturn;

        if ( RBACService.isAuthorized( ModelesOdjResourceIdService.MODELES_ODJ, RBAC.WILDCARD_RESOURCES_ID,
                    ModelesOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            strReturn = _modeleOrdreDuJourService.getSuppressionModele( request );

            if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                strReturn = getHomeUrl( request );
            }
        }
        else
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }

        return strReturn;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.modeleordredujour.IModeleOrdreDuJourJspBean#doSuppressionModele(javax.servlet.http.HttpServletRequest)
         */
    public String doSuppressionModele( HttpServletRequest request )
    {
        String strReturn;

        if ( RBACService.isAuthorized( ModelesOdjResourceIdService.MODELES_ODJ, RBAC.WILDCARD_RESOURCES_ID,
                    ModelesOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            strReturn = _modeleOrdreDuJourService.doSuppressionModele( request );

            if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                strReturn = getHomeUrl( request );
            }
        }
        else
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }

        return strReturn;
    }

    /* (non-Javadoc)
         * @see fr.paris.lutece.plugins.ods.web.modeleordredujour.IModeleOrdreDuJourJspBean#doDownloadFichier(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
         */
    public void doDownloadFichier( HttpServletRequest request, HttpServletResponse response )
    {
        String strType = request.getParameter( OdsParameters.TYPE );
        FichierPhysique fichierPhysique = _modeleOrdreDuJourService.getFichier( request );

        try
        {
            String strFileName = ( strType != null ) ? strType
                                                     : ( OdsConstants.CONSTANTE_CHAINE_VIDE +
                OdsConstants.CONSTANTE_POINT + OdsConstants.CONSTANTE_XML );

            response.setHeader( "Content-Disposition", "attachment ;filename=\"" + strFileName + "\"" );
            response.setContentType( "application/x-msdownload" );
            response.setHeader( "Pragma", "public" );
            response.setHeader( "Expires", "0" );
            response.setHeader( "Cache-Control", "must-revalidate,post-check=0,pre-check=0" );

            OutputStream os = response.getOutputStream(  );

            if ( fichierPhysique != null )
            {
                os.write( fichierPhysique.getDonnees(  ) );
            }

            os.close(  );
        }
        catch ( IOException e )
        {
            AppLogService.error( e );
        }
    }

    /**
     * Permet de stocker toutes les permissions afin de grer les profils au niveau des templates
     * @param model le hashmap contenant les parametres qui vont tre envoys au template
     */
    private void ajouterPermissionsDansHashmap( Map<String, Object> model )
    {
        // Permission de grer
        boolean bPermissionGestion = true;

        if ( !RBACService.isAuthorized( ModelesOdjResourceIdService.MODELES_ODJ, RBAC.WILDCARD_RESOURCES_ID,
                    ModelesOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            bPermissionGestion = false;
        }

        model.put( OdsMarks.MARK_PERMISSION_GESTION, bPermissionGestion );
    }

    protected String getRight(  )
    {
        return RIGHT_ODS_MODELES_ODJ;
    }
}
