/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web.ordredujour;

import au.com.bytecode.opencsv.CSVWriter;

import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.service.manager.DocumentManager;
import fr.paris.lutece.plugins.ods.service.ordredujour.IOrdreDuJourService;
import fr.paris.lutece.plugins.ods.service.role.ConstitutionOdjResourceIdService;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.business.rbac.RBAC;
import fr.paris.lutece.portal.service.admin.AccessDeniedException;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.rbac.RBACService;
import fr.paris.lutece.portal.service.template.AppTemplateService;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.web.admin.PluginAdminPageJspBean;
import fr.paris.lutece.util.html.HtmlTemplate;

import java.io.IOException;
import java.io.OutputStream;
import java.io.StringWriter;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


/**
 * AbstractOrdreDuJourJspBean
 */
public abstract class AbstractOrdreDuJourJspBean<GOrdreDuJour extends IOrdreDuJour<GEntreeOrdreDuJour, GSeance, GFichier, GElu, GOrdreDuJour>, GEntreeOrdreDuJour extends IEntreeOrdreDuJour<GElu, GSeance, GFichier, GEntreeOrdreDuJour, GOrdreDuJour>, GSeance extends ISeance, GElu extends IElu, GFichier extends IFichier<GSeance, GFichier>, GOrdreDuJourFilter extends IOrdreDuJourFilter>
    extends PluginAdminPageJspBean implements IOrdreDuJourJspBean
{
    private static final long serialVersionUID = 7557832931683041188L;
    private static final String RIGHT_ODS_ODJ = "ODS_ODJ";
    private static final String RIGHT_ODS_ODJ_GESTION_AVAL = "ODS_ODJ_GESTION_AVAL";
    protected static final String TEMPLATE_CONFIRMATION_SUPPRESSION_ORDRE_DU_JOUR = "admin/plugins/ods/ordredujour/confirmation_suppression_ordre_du_jour.html";
    protected static final String TEMPLATE_MODIFICATION_ENTETE_PIED_DE_PAGE = "admin/plugins/ods/ordredujour/modification_entete_pied_de_page.html";
    private static final String MESSAGE_PROBLEME_DE_CREATION_ODJ_PDF = "ods.odj.message.problemeCreationOdjPdf";
    private static final String MESSAGE_PROBLEME_CONVERSION_JCOPIST = "ods.odj.message.problemJCopist";

    protected abstract String getListTemplate(  );

    protected abstract String getListTemplateAval(  );

    protected abstract String getCreationEntreeTemplate(  );

    protected abstract String getModificationEntreeTemplate(  );

    protected abstract String getSelectionEntreeTemplate(  );

    protected abstract String getModificationOrdreDuJourSpec( HttpServletRequest request );

    protected abstract IOrdreDuJourService<GOrdreDuJour, GEntreeOrdreDuJour, GOrdreDuJourFilter, GSeance, GElu, GFichier> getOrdreDuJourService(  );

    public void initSession( HttpServletRequest request, HttpServletResponse response, boolean bIsGestionAval )
        throws AccessDeniedException
    {
        init( request, response, bIsGestionAval );
        getOrdreDuJourService(  ).initSession(  );
    }

    /**
     * Init
     * @param request
     * @throws AccessDeniedException
     */
    public void init( HttpServletRequest request, HttpServletResponse response, boolean bIsGestionAval )
        throws AccessDeniedException
    {
        String strRight = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( bIsGestionAval )
        {
            strRight = getRightAval(  );
        }
        else
        {
            strRight = getRight(  );
        }

        super.init( request, strRight );
    }

    protected String getRight(  )
    {
        return RIGHT_ODS_ODJ;
    }

    protected String getRightAval(  )
    {
        return RIGHT_ODS_ODJ_GESTION_AVAL;
    }

    /**
     * Rcupre la page de gestion de l'ordre du jour
     *
     * @param request la requte Http
     * @return la page de gestion de l'ordre du jour
     */
    public String getOrdreDuJourList( HttpServletRequest request )
    {
        HashMap<String, Object> model = getOrdreDuJourService(  ).getOrdreDuJourList( request );

        ajouterPermissionsDansHashmap( model );

        HtmlTemplate template = AppTemplateService.getTemplate( getListTemplate(  ), getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Rcupre la page de gestion de l'ordre du jour en gestion aval
     *
     * @param request la requte Http
     * @return la page de gestion de l'ordre du jour en gestion aval
     */
    public String getOrdreDuJourGestionAvalList( HttpServletRequest request )
    {
        HashMap<String, Object> model = getOrdreDuJourService(  ).getOrdreDuJourGestionAvalList( request );

        ajouterPermissionsDansHashmap( model );

        HtmlTemplate template = AppTemplateService.getTemplate( getListTemplateAval(  ), getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Effectue la cration d'un ordre du jour  partir des informations
     * contenues dans la requte
     *
     * @param request
     *            la requte Http
     * @return l'url de gestion des ordres du jour si la requte est valide
     */
    public String doCreationOrdreDuJour( HttpServletRequest request )
    {
        String strReturn = getOrdreDuJourService(  ).doCreationOrdreDuJour( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /**
     * Retourne l'interface de modification d'un ordre du jour.
     *
     * @param request la requte Http
     * @return l'interface de modification d'un ordre du jour
     */
    public String getModificationOrdreDuJour( HttpServletRequest request )
    {
        HashMap<String, Object> model = getOrdreDuJourService(  ).getModificationOrdreDuJour( request );

        if ( model == null )
        {
            return getOrdreDuJourList( request );
        }

        String strTemplate = getModificationOrdreDuJourSpec( request );
        HtmlTemplate template = AppTemplateService.getTemplate( strTemplate, getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Retourne l'interface de modification d'entete ou de pied de page de l'odj
     *
     * @param request la requte Http
     * @return l'interface de modification d'entete ou de pied de page de l'odj
     */
    public String getModificationEntetePiedPage( HttpServletRequest request )
    {
        HashMap<String, Object> model = getOrdreDuJourService(  ).getModificationEntetePiedPage( request );

        if ( model == null )
        {
            return getModificationOrdreDuJour( request );
        }
        else
        {
            HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_MODIFICATION_ENTETE_PIED_DE_PAGE,
                    getLocale(  ), model );

            return getAdminPage( template.getHtml(  ) );
        }
    }

    /**
     * Effectue la modification de l'entte ou du pied de page d'un ODJ
     *
     * @param request
     *            la requte Http
     * @return l'url de modification de l'ordre du jour
     */
    public String doModificationEntetePiedPage( HttpServletRequest request )
    {
        return getOrdreDuJourService(  ).doModificationEntetePiedPage( request );
    }

    /**
     * Retourne l'interface de confirmation de suppression d'une entre d'une entte
     *                                 ou d'un pied de page
     *
     * @param request
     *                    la requte Http
     * @return 'interface de confirmation de suppression d'une entre d'une entte
     *                                 ou d'un pied de page
     */
    public String getSuppressionEntreeEntetePiedPage( HttpServletRequest request )
    {
        String strReturn = getOrdreDuJourService(  ).getSuppressionEntreeEntetePiedPage( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /**
     * Effectue la suppression d'une entre d'entte ou pied de page d'un ordre du jour  partir des
     * informations contenues dans la requte
     *
     * @param request
     *            la requte Http
     * @return l'url de modification d'entte ou pied de page de l' ordre du jour
     */
    public String doSuppressionEntreeEntetePiedPage( HttpServletRequest request )
    {
        String strReturn = getOrdreDuJourService(  ).doSuppressionEntreeEntetePiedPage( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /**
     * Retourne l'ordre du jour sous format PDF ou Word
     * @param request la requete Http
     * @param response la reponse
     * @return l'ordre du jour sous format PDF ou Word
     */
    public String doVisualisationODJ( HttpServletRequest request, HttpServletResponse response )
    {
        if ( !DocumentManager.checkJCopist(  ) )
        {
            return AdminMessageService.getMessageUrl( request, MESSAGE_PROBLEME_CONVERSION_JCOPIST,
                AdminMessage.TYPE_STOP );
        }

        GOrdreDuJour ordreDuJour = getOrdreDuJourService(  ).doVisualisationODJ( request );

        if ( ordreDuJour == null )
        {
            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                AdminMessage.TYPE_ERROR );
        }

        try
        {
            String strTypeConversion = request.getParameter( OdsParameters.FORMAT );
            String strXml = ordreDuJour.getXml(  );

            byte[] xml = ( OdsConstants.CONSTANTE_ENTETE_XML + strXml ).getBytes( OdsConstants.UTF8 );
            byte[] bConvertedDoc = null;

            if ( strTypeConversion.trim(  ).equals( DocumentManager.PDF_CONVERSION ) )
            {
                bConvertedDoc = DocumentManager.convert( DocumentManager.PDF_CONVERSION,
                        OdsConstants.CONSTANTE_TEMPLATE_XML_ODJ, xml );
            }
            else if ( strTypeConversion.trim(  ).equals( DocumentManager.PDF_WORD ) )
            {
                bConvertedDoc = DocumentManager.convert( DocumentManager.PDF_WORD,
                        OdsConstants.CONSTANTE_TEMPLATE_XML_ODJ, xml );
            }

            if ( ( bConvertedDoc != null ) && ( bConvertedDoc.length > 0 ) )
            {
                OutputStream os = response.getOutputStream(  );

                if ( strTypeConversion.trim(  ).equals( DocumentManager.PDF_CONVERSION ) )
                {
                    String strFileName = ordreDuJour.getIntitule(  ) + ".pdf";
                    response.setHeader( "Content-Disposition", "attachment ;filename=\"" + strFileName + "\"" );
                    response.setContentType( "application/pdf" );
                    response.setHeader( "Pragma", "public" );
                    response.setHeader( "Expires", "0" );
                    response.setHeader( "Cache-Control", "must-revalidate,post-check=0,pre-check=0" );
                }
                else if ( strTypeConversion.trim(  ).equals( DocumentManager.PDF_WORD ) )
                {
                    String strFileName = ordreDuJour.getIntitule(  ) + ".doc";
                    response.setHeader( "Content-Disposition", "attachment ;filename=\"" + strFileName + "\"" );
                    response.setContentType( "application/msword" );
                    response.setHeader( "Pragma", "public" );
                    response.setHeader( "Expires", "0" );
                    response.setHeader( "Cache-Control", "must-revalidate,post-check=0,pre-check=0" );
                }

                os.write( bConvertedDoc );
                os.flush(  );
                os.close(  );
            }
            else
            {
                return AdminMessageService.getMessageUrl( request, MESSAGE_PROBLEME_CONVERSION_JCOPIST,
                    AdminMessage.TYPE_STOP );
            }
        }
        catch ( Exception e )
        {
            AppLogService.error( e.getMessage(  ) );

            return AdminMessageService.getMessageUrl( request, MESSAGE_PROBLEME_CONVERSION_JCOPIST,
                AdminMessage.TYPE_STOP );
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * genre les odj de la prochaine sance au format word et les enregistrer sur le File System
     * @param request la requte HTTP
     * @return 0 si OK, 10 sinon
     * @throws IOException IOException
     */
    public String doGenereODJ( HttpServletRequest request )
        throws IOException
    {
        return getOrdreDuJourService(  ).doGenereODJ( request );
    }

    /**
     * Retourne l'ordre du jour sous format csv
     * @param request la requete Http
     * @param response la reponse
     */
    public void doVisualisationODJcsv( HttpServletRequest request, HttpServletResponse response )
    {
        List<String[]> listToCSVWriter = getOrdreDuJourService(  ).doVisualisationODJcsv( request );

        if ( listToCSVWriter != null )
        {
            String strCsvSeparator = OdsConstants.CONSTANTE_POINT_VIRGULE;
            StringWriter strWriter = new StringWriter(  );

            CSVWriter csvWriter = new CSVWriter( strWriter, strCsvSeparator.toCharArray(  )[0] );
            csvWriter.writeAll( listToCSVWriter );

            byte[] byteFileOutPut = strWriter.toString(  ).getBytes(  );

            try
            {
                response.setHeader( "Content-Disposition", "attachment; filename=\"odj.csv \";" );
                response.setHeader( "Pragma", "public" );
                response.setHeader( "Expires", "0" );
                response.setHeader( "Cache-Control", "must-revalidate,post-check=0,pre-check=0" );
                response.setContentType( "enctype=multipart/form-data" );

                OutputStream os = response.getOutputStream(  );
                os.write( byteFileOutPut );
                os.close(  );

                csvWriter.close(  );
                strWriter.close(  );
            }
            catch ( IOException e )
            {
                AppLogService.error( e );
            }
        }
    }

    /**
      * Effectue la modification d'un ordre du jour  partir des informations
      * contenues dans la requte
      *
      * @param request
      *            la requte Http
      * @return l'url de gestion des ordres du jour
      */
    public String doModificationOrdreDuJour( HttpServletRequest request )
    {
        return getOrdreDuJourService(  ).doModificationOrdreDuJour( request );
    }

    /**
     * Retourne l'interface de confirmation de la suppression de l'ordre du jour
     *
     * @param request
     *            la requte Http
     * @return l'interface de confirmation de la suppression de l'ordre du jour
     */
    public String getSuppressionOrdreDuJour( HttpServletRequest request )
    {
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        /*
         * GESTION DES PROFILS
         * -------------------
         * Si l'utilisateur n'est pas autoris a effectuer cette fonction,
         *  il est renvoy vers une page lui indiquant qu'il n'a pas les droits ncessaires.
         */
        if ( !RBACService.isAuthorized( ConstitutionOdjResourceIdService.CONSTITUTION_ODJ, RBAC.WILDCARD_RESOURCES_ID,
                    ConstitutionOdjResourceIdService.PERMISSION_SUPPRESSION, getUser(  ) ) )
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }
        else
        {
            HashMap<String, String> model = getOrdreDuJourService(  ).getSuppressionOrdreDuJour( request );

            if ( model == null )
            {
                strReturn = getHomeUrl( request );
            }
            else
            {
                HtmlTemplate template = AppTemplateService.getTemplate( TEMPLATE_CONFIRMATION_SUPPRESSION_ORDRE_DU_JOUR,
                        getLocale(  ), model );
                strReturn = template.getHtml(  );
            }
        }

        return strReturn;
    }

    /**
     * Effectue la suppression d'un ordre du jour  partir des informations
     * contenues dans la requte
     *
     * @param request
     *            la requte Http
     * @return l'url de gestion des ordres du jour
     */
    public String doSuppressionOrdreDuJour( HttpServletRequest request )
    {
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        /*
         * GESTION DES PROFILS
         * -------------------
         * Si l'utilisateur n'est pas autoris a effectuer cette fonction,
         *  il est renvoy vers une page lui indiquant qu'il n'a pas les droits ncessaires.
         */
        if ( !RBACService.isAuthorized( ConstitutionOdjResourceIdService.CONSTITUTION_ODJ, RBAC.WILDCARD_RESOURCES_ID,
                    ConstitutionOdjResourceIdService.PERMISSION_SUPPRESSION, getUser(  ) ) )
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }
        else
        {
            strReturn = getOrdreDuJourService(  ).doSuppressionOrdreDuJour( request );
        }

        return strReturn;
    }

    /**
     * Retourne l'interface de confirmation de publication si au moins un pdd
     * rattach  l'ordre du jour n'est pas publi sinon redirection vers le jsp
     * de publication
     *
     * @param request
     *            la requte Http
     * @return l'interface de confirmation de publication si au moins un pdd
     *         rattach  l'ordre du jour n'est pas publi sinon redirection
     *         vers le jsp de publication
     */
    public String getPublicationOrdreDuJour( HttpServletRequest request )
    {
        String strReturn = getOrdreDuJourService(  ).getPublicationOrdreDuJour( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /**
     * Retourne l'interface de confirmation de depublication
     *
     * @param request
     *            la requte Http
     * @return l'interface de confirmation de publication si au moins un pdd
     *         rattach  l'ordre du jour n'est pas publi sinon redirection
     *         vers le jsp de publication
     */
    public String getDepublicationOrdreDuJour( HttpServletRequest request )
    {
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        /*
         * GESTION DES PROFILS
         * -------------------
         * Si l'utilisateur n'est pas autoris a effectuer cette fonction,
         *  il est renvoy vers une page lui indiquant qu'il n'a pas les droits ncessaires.
         */
        if ( !RBACService.isAuthorized( ConstitutionOdjResourceIdService.CONSTITUTION_ODJ, RBAC.WILDCARD_RESOURCES_ID,
                    ConstitutionOdjResourceIdService.PERMISSION_DEPUBLICATION, getUser(  ) ) )
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }
        else
        {
            strReturn = getOrdreDuJourService(  ).getDepublicationOrdreDuJour( request );

            if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                strReturn = getHomeUrl( request );
            }
        }

        return strReturn;
    }

    /**
     * Effectue la publication d'un ordre du jour  partir de l'id de l'odj
     * contenu dans la requette
     *
     * @param request
     *            la requte Http
     * @return l'url de gestion des ordres du jour si la requte est valide, un
     *         message d'erreur sinon
     */
    public String doPublicationOrdreDuJour( HttpServletRequest request )
    {
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        /*
         * GESTION DES PROFILS
         * -------------------
         * Si l'utilisateur n'est pas autoris a effectuer cette fonction,
         *  il est renvoy vers une page lui indiquant qu'il n'a pas les droits ncessaires.
         */
        if ( !RBACService.isAuthorized( ConstitutionOdjResourceIdService.CONSTITUTION_ODJ, RBAC.WILDCARD_RESOURCES_ID,
                    ConstitutionOdjResourceIdService.PERMISSION_PUBLICATION, getUser(  ) ) )
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }
        else if ( !DocumentManager.checkJCopist(  ) )
        {
            strReturn = AdminMessageService.getMessageUrl( request, MESSAGE_PROBLEME_DE_CREATION_ODJ_PDF,
                    AdminMessage.TYPE_STOP );
        }
        else
        {
            strReturn = getOrdreDuJourService(  ).doPublicationOrdreDuJour( request );
        }

        return strReturn;
    }

    /**
     * Effectue la dpublication d'un ordre du jour  partir de l'id de l'odj
     * contenu dans la requette
     *
     * @param request
     *            la requte Http
     * @return l'url de gestion des ordres du jour si la requte est valide, un
     *         message d'erreur sinon
     */
    public String doDepublicationOrdreDuJour( HttpServletRequest request )
    {
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        /*
         * GESTION DES PROFILS
         * -------------------
         * Si l'utilisateur n'est pas autoris a effectuer cette fonction,
         *  il est renvoy vers une page lui indiquant qu'il n'a pas les droits ncessaires.
         */
        if ( !RBACService.isAuthorized( ConstitutionOdjResourceIdService.CONSTITUTION_ODJ, RBAC.WILDCARD_RESOURCES_ID,
                    ConstitutionOdjResourceIdService.PERMISSION_DEPUBLICATION, getUser(  ) ) )
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }
        else
        {
            strReturn = getOrdreDuJourService(  ).doDepublicationOrdreDuJour( request );
        }

        return strReturn;
    }

    /**
     * Retourne l'interface de cration d'une entre de l'ordre du jour
     *
     * @param request
     *            la requte Http
     * @return l'interface de cration d'une entre de l'ordre du jour
     */
    public String getCreationEntreeOrdreDuJour( HttpServletRequest request )
    {
        HashMap<String, Object> model = getOrdreDuJourService(  ).getCreationEntreeOrdreDuJour( request );

        if ( model == null )
        {
            return getOrdreDuJourList( request );
        }
        else
        {
            setPageTitleProperty( getOrdreDuJourService(  ).getPageTitleCreationEntreeODJ(  ) );

            HtmlTemplate template = AppTemplateService.getTemplate( getCreationEntreeTemplate(  ),
                    request.getLocale(  ), model );

            return getAdminPage( template.getHtml(  ) );
        }
    }

    /**
     * Effectue l'ensemble des traitements necessaire lors de la cration d'une
     * entre de l'ordre du jour  partir des informations contenues dans la
     * requte creation d'une nouvelle entre ,ajout d'un rapporteur ,d'une
     * commission  l'entre en cours de creation.....
     *
     * @param request
     *            la requte Http
     * @return en fonction des diffrents traitements retour vers l'url de
     *         modification d'un ordre du jour ,de creation de l'entre en cours
     *         de creation...
     */
    public String doCreationEntreeOrdreDuJour( HttpServletRequest request )
    {
        return getOrdreDuJourService(  )
                   .doCreationEntreeOrdreDuJour( IOrdreDuJourService.JSP_CREATION_ENTREE_ORDRE_DU_JOUR, request );
    }

    /**
     * Retourne l'interface de modification de l'entre d'ordre du jour
     * selectionne
     *
     * @param request la requte Http
     * @return l'interface de modification de l'entre d'ordre du jour
     *         selectionne
     */
    public String getModificationEntreeOrdreDuJour( HttpServletRequest request )
    {
        HashMap<String, Object> model = getOrdreDuJourService(  ).getModificationEntreeOrdreDuJour( request );

        setPageTitleProperty( getOrdreDuJourService(  ).getPageTitleModificationEntreeODJ(  ) );

        HtmlTemplate template = AppTemplateService.getTemplate( getModificationEntreeTemplate(  ), getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Effectue l'ensemble des traitements necessaire lors d'une modification
     * d'une entre d'ordre du jour  partir des informations contenues dans la
     * requte modification d'une entre ,ajout d'un rapporteur ,d'une
     * commission  l'entre en cours de modification.....
     *
     * @param request la requte Http
     * @return en fonction des diffrents traitements retour vers l'url de
     *         modification d'un ordre du jour ,de creation de l'entre en cours
     *         de creation...
     */
    public String doModificationEntreeOrdreDuJour( HttpServletRequest request )
    {
        return getOrdreDuJourService(  )
                   .doModificationEntreeOrdreDuJour( IOrdreDuJourService.JSP_MODIFICATION_ENTREE_ORDRE_DU_JOUR, request );
    }

    /**
     * Retourne l'interface de confirmation de suppression d'une entre d'ordre
     * du jour
     *
     * @param request
     *            la requte Http
     * @return l'interface de confirmation de suppression d'une entre d'ordre
     *         du jour
     */
    public String getSuppressionEntreeOrdreDuJour( HttpServletRequest request )
    {
        String strReturn = getOrdreDuJourService(  ).getSuppressionEntreeOrdreDuJour( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /**
     * Effectue la suppression d'une entre ordre du jour  partir des
     * informations contenues dans la requte
     *
     * @param request
     *            la requte Http
     * @return l'url de modification de l' ordre du jour en cours de
     *         modification
     */
    public String doSuppressionEntreeOrdreDuJour( HttpServletRequest request )
    {
        String strReturn = getOrdreDuJourService(  ).doSuppressionEntreeOrdreDuJour( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /**
     * Effectue la suppression de le liste des entes d'ordre du jour
     * selectionnes par l'utisateur
     *
     * @param request
     *            la requte Http
     * @return l'url de modification de l' ordre du jour en cours de
     *         modification
     */
    public String doSuppressionListeEntree( HttpServletRequest request )
    {
        String strReturn = getOrdreDuJourService(  ).doSuppressionListeEntree( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /**
     * insere le PDD selectionn dans l'entre d'ordre du jour en session
     *
     * @param request la requte Http
     * @return retourne sur l'entre d'ordre du jour en cours de traitement
     */
    public String doSelectionPDD( HttpServletRequest request )
    {
        String strReturn = getOrdreDuJourService(  ).doSelectionPDD( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /**
     * methode qui cre pour chaque Pdds selectionns une entre d'ordre du jour
     *
     * @param request
     *            la requte Http
     * @return retour sur l'ordre du jour en cours de modification
     */
    public String doSelectionPDDs( HttpServletRequest request )
    {
        String strReturn = getOrdreDuJourService(  ).doSelectionPDDs( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /**
     * insere une expression usuelle dans l'entre d'ordre du jour en session
     *
     * @param request
     *            la requte Http
     * @return l'url de traitement de l'entre d'ordre du jour
     */
    public String doSelectionExpression( HttpServletRequest request )
    {
        String strReturn = getOrdreDuJourService(  ).doSelectionExpression( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /**
     * Insre un voeu dans l'entre de l'ordre du jour
     *
     * @param request la requte Http
     * @return l'url de traitement de l'entre d'ordre du jour
     */
    public String doSelectionVoeuAmendement( HttpServletRequest request )
    {
        return getOrdreDuJourService(  ).doSelectionVoeuAmendement( request );
    }

    /**
     * methode qui cre pour chaque voeux selectionns une entre d'ordre du
     * jour
     *
     * @param request
     *            la requte Http
     * @return retour sur l'ordre du jour en cours de modification
     */
    public String doSelectionVoeuxAmendements( HttpServletRequest request )
    {
        String strReturn = getOrdreDuJourService(  ).doSelectionVoeuxAmendements( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /**
     * methode qui gere l'ensemble des traitements effectus sur l'ordre du jour en paramtre de la request
     *
     * @param request request
     * @return l'url de modification de l'ordre du jour en cours de traitement
     */
    public String doTraitementOdj( HttpServletRequest request )
    {
        return getOrdreDuJourService(  ).doTraitementOdj( request, getHomeUrl( request ) );
    }

    /**
     * methode qui gere l'ensemble des traitements effectus sur la liste des
     * entres d'ordre du jour en cours de traitement
     *
     * @param request
     *            request
     * @return l'url de modification de l'ordre du jour en cours de traitement
     */
    public String doTraitementEntreeOdj( HttpServletRequest request )
    {
        String strReturn = getOrdreDuJourService(  ).doTraitementEntreeOdj( request, getHomeUrl( request ) );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /**
     * Retourne l'interface de selection d'une entre d'ordre du jour.
     *
     * @param request la requte Http
     * @return l'interface de selection d'une entre d'ordre du jour.
     */
    public String getSelectionEntreeOrdreDuJour( HttpServletRequest request )
    {
        HashMap<String, Object> model = getOrdreDuJourService(  ).getSelectionEntreeOrdreDuJour( request );

        if ( model == null )
        {
            return getOrdreDuJourList( request );
        }
        else
        {
            HtmlTemplate template = AppTemplateService.getTemplate( getSelectionEntreeTemplate(  ), getLocale(  ), model );

            return getAdminPage( template.getHtml(  ) );
        }
    }

    /**
     * Numerote les voeux et amendements de l'ordre du jour definitif
     *
     * @param request
     *            request
     * @return l'url de modification de l'ordre du jour en cours de traitement
     */
    public String doNumeroteVa( HttpServletRequest request )
    {
        String strReturn = getOrdreDuJourService(  ).doNumeroteVa( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /**
     * Deplace la liste des entres selectionnes  la position choisie par
     * l'utilisateur
     *
     * @param request
     *            request
     * @return l'url de modification de l'ordre du jour en cours de traitement
     */
    public String doDeplaceListeEntree( HttpServletRequest request )
    {
        String strReturn = getOrdreDuJourService(  ).doDeplaceListeEntree( request );

        if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strReturn = getHomeUrl( request );
        }

        return strReturn;
    }

    /**
     * Permet de stocker toutes les permissions afin de grer les profils au niveau des templates
     * @param model le hashmap contenant les parametres qui vont tre envoys au template
     */
    protected void ajouterPermissionsDansHashmap( Map<String, Object> model )
    {
        /*
         * Permission de publier
         */
        boolean bPermissionPublication = true;

        if ( !RBACService.isAuthorized( ConstitutionOdjResourceIdService.CONSTITUTION_ODJ, RBAC.WILDCARD_RESOURCES_ID,
                    ConstitutionOdjResourceIdService.PERMISSION_PUBLICATION, getUser(  ) ) )
        {
            bPermissionPublication = false;
        }

        model.put( OdsMarks.MARK_PERMISSION_PUBLICATION, bPermissionPublication );

        /*
         * Permission de depublier
         */
        boolean bPermissionDePublication = true;

        if ( !RBACService.isAuthorized( ConstitutionOdjResourceIdService.CONSTITUTION_ODJ, RBAC.WILDCARD_RESOURCES_ID,
                    ConstitutionOdjResourceIdService.PERMISSION_DEPUBLICATION, getUser(  ) ) )
        {
            bPermissionDePublication = false;
        }

        model.put( OdsMarks.MARK_PERMISSION_DEPUBLICATION, bPermissionDePublication );

        /*
         * Permission de supprimer
         */
        boolean bPermissionSuppression = true;

        if ( !RBACService.isAuthorized( ConstitutionOdjResourceIdService.CONSTITUTION_ODJ, RBAC.WILDCARD_RESOURCES_ID,
                    ConstitutionOdjResourceIdService.PERMISSION_SUPPRESSION, getUser(  ) ) )
        {
            bPermissionSuppression = false;
        }

        model.put( OdsMarks.MARK_PERMISSION_SUPPRESSION, bPermissionSuppression );
    }
}
