/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web.seance;

import fr.paris.lutece.plugins.ods.service.role.AdminSaufOdjResourceIdService;
import fr.paris.lutece.plugins.ods.service.seance.ISeanceService;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.business.rbac.RBAC;
import fr.paris.lutece.portal.service.admin.AccessDeniedException;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.rbac.RBACService;
import fr.paris.lutece.portal.service.template.AppTemplateService;
import fr.paris.lutece.portal.web.admin.PluginAdminPageJspBean;
import fr.paris.lutece.util.html.HtmlTemplate;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


/**
 * Manage la fonctionnalit de la Sance
 */
public abstract class AbstractSeanceJspBean extends PluginAdminPageJspBean implements ISeanceJspBean
{
    protected static final String PROPERTY_PAGE_CREATION_SEANCE = "ods.seance.creation.pageTitle";
    protected static final String PROPERTY_PAGE_MODIFICATION_SEANCE = "ods.seance.modification.pageTitle";
    protected static final String PROPERTY_PAGE_LISTE_SEANCE = "ods.listeseances.label.titleListe";
    private static final long serialVersionUID = 779366419921338366L;
    private static final String RIGHT_ODS_SEANCE = "ODS_SEANCES";
    @Autowired
    private ISeanceService _seanceService;

    /**
     *
     * @return la template de liste
     */
    protected abstract String getListTemplate(  );

    /**
     *
     * @return la template de modification
     */
    protected abstract String getModificationTemplate(  );

    /**
     *
     * @return la template de creation
     */
    protected abstract String getCreationTemplate(  );

    /**
     * Init
     * @param request la requete HTTP
     * @param response la reponse HTTP
     * @throws AccessDeniedException AccessDeniedException
     */
    public void init( HttpServletRequest request, HttpServletResponse response )
        throws AccessDeniedException
    {
        super.init( request, getRight(  ) );
    }

    /**
     *
     * @return le right
     */
    protected String getRight(  )
    {
        return RIGHT_ODS_SEANCE;
    }

    /**
     * initalise la sance
     */
    public void initSeanceBean(  )
    {
        _seanceService.doResetSeance(  );
    }

    /**
     * retourne le formulaire de cration d'une sance
     * @param request la requete HTTP
     * @return Template avec le formulaire de cration d'un sance
     */
    public String getCreateSeance( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_PAGE_CREATION_SEANCE );

        HtmlTemplate template;

        if ( RBACService.isAuthorized( AdminSaufOdjResourceIdService.ADMINISTRATION_SAUF_ODJ,
                    RBAC.WILDCARD_RESOURCES_ID, AdminSaufOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            HashMap<String, Object> model = _seanceService.getCreateSeance( request );

            template = AppTemplateService.getTemplate( getCreationTemplate(  ), getLocale(  ), model );
        }
        else
        {
            template = AppTemplateService.getTemplate( OdsProperties.TEMPLATE_PROFIL_ERROR, getLocale(  ) );
        }

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * cre une sance
     * si tous les champs obligatoires du formulaire de cration d'une sance ont t renseign,<BR>
     * la mthode retourne l'url de gestion des sances,<BR>
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request La requete HTTP
     * @return String template liste des seances
     */
    public String doCreateAndRegisterSeance( HttpServletRequest request )
    {
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( RBACService.isAuthorized( AdminSaufOdjResourceIdService.ADMINISTRATION_SAUF_ODJ,
                    RBAC.WILDCARD_RESOURCES_ID, AdminSaufOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            strReturn = _seanceService.doCreateAndRegisterSeance( request );

            if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                strReturn = getHomeUrl( request );
            }
        }
        else
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }

        return strReturn;
    }

    /**
     * rinitialise l'objet seance stock en session
     * @param request La requete HTTP
     * @return String template liste des seances
     */
    public String doResetSeance( HttpServletRequest request )
    {
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( RBACService.isAuthorized( AdminSaufOdjResourceIdService.ADMINISTRATION_SAUF_ODJ,
                    RBAC.WILDCARD_RESOURCES_ID, AdminSaufOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            _seanceService.doResetSeance(  );
            strReturn = getHomeUrl( request );
        }
        else
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }

        return strReturn;
    }

    /**
     * Permet de calculer les dates automatiquement en fonction de la date de dbut et de fin de sance
     * Ne crer pas la sance.
     * @param request La requete HTTP
     * @return String template de creation de seance
     */
    public String doCreateSeance( HttpServletRequest request )
    {
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( RBACService.isAuthorized( AdminSaufOdjResourceIdService.ADMINISTRATION_SAUF_ODJ,
                    RBAC.WILDCARD_RESOURCES_ID, AdminSaufOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            strReturn = _seanceService.doCreateSeance( request );
        }
        else
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }

        return strReturn;
    }

    /**
     * Modifie une sance
     * si tous les champs obligatoires du formulaire de modification d'une sance ont t renseign,<BR>
     * la mthode retourne l'url de gestion des sances,<BR>
     * sinon l'url retourne est celle de l'interface spcifiant les messages d'erreurs.
     * @param request La requete HTTP
     * @return String template liste des seances
     */
    public String doUpdateAndRegisterSeance( HttpServletRequest request )
    {
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( RBACService.isAuthorized( AdminSaufOdjResourceIdService.ADMINISTRATION_SAUF_ODJ,
                    RBAC.WILDCARD_RESOURCES_ID, AdminSaufOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            strReturn = _seanceService.doUpdateAndRegisterSeance( request );

            if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                strReturn = getHomeUrl( request );
            }
        }
        else
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }

        return strReturn;
    }

    /**
     * retourne le formulaire de modification d'une sance pr remplis avec  les critres de la sance choisi
     * @param request la requete HTTP
     * @return String template de modification d'une seance
     */
    public String getModificationSeance( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_PAGE_MODIFICATION_SEANCE );

        HtmlTemplate template;

        if ( RBACService.isAuthorized( AdminSaufOdjResourceIdService.ADMINISTRATION_SAUF_ODJ,
                    RBAC.WILDCARD_RESOURCES_ID, AdminSaufOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            HashMap<String, Object> model = _seanceService.getModificationSeance( request );

            if ( model == null )
            {
                return getSeanceList( request );
            }
            else
            {
                template = AppTemplateService.getTemplate( getModificationTemplate(  ), getLocale(  ), model );
            }
        }
        else
        {
            template = AppTemplateService.getTemplate( OdsProperties.TEMPLATE_PROFIL_ERROR, getLocale(  ) );
        }

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Permet de calculer les dates automatiquement en fonction de la date de dbut et de fin de sance
     * Ne modifie pas la sance.
     *
     * @param request la requete HTTP
     * @return String template de liste des seances
     */
    public String doModificationSeance( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_PAGE_MODIFICATION_SEANCE );

        HtmlTemplate template;

        if ( RBACService.isAuthorized( AdminSaufOdjResourceIdService.ADMINISTRATION_SAUF_ODJ,
                    RBAC.WILDCARD_RESOURCES_ID, AdminSaufOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            HashMap<String, Object> model = _seanceService.doModificationSeance( request );
            template = AppTemplateService.getTemplate( getModificationTemplate(  ), getLocale(  ), model );
        }
        else
        {
            template = AppTemplateService.getTemplate( OdsProperties.TEMPLATE_PROFIL_ERROR, getLocale(  ) );
        }

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * retourne une demande de confirmation pour la suppression de la sance choisie
     * @param request la requete HTTP
     * @return String Template de confirmation de suppression des seances
     */
    public String getSupressionSeance( HttpServletRequest request )
    {
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( RBACService.isAuthorized( AdminSaufOdjResourceIdService.ADMINISTRATION_SAUF_ODJ,
                    RBAC.WILDCARD_RESOURCES_ID, AdminSaufOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            strReturn = _seanceService.getSupressionSeance( request );

            if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                strReturn = getHomeUrl( request );
            }
        }
        else
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }

        return strReturn;
    }

    /**
     * Supprime la sance choisi
     * @param request la requete HTTP
     * @return String template de liste des seances
     */
    public String doSupressionSeance( HttpServletRequest request )
    {
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( RBACService.isAuthorized( AdminSaufOdjResourceIdService.ADMINISTRATION_SAUF_ODJ,
                    RBAC.WILDCARD_RESOURCES_ID, AdminSaufOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            strReturn = _seanceService.doSupressionSeance( request );

            if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                strReturn = getHomeUrl( request );
            }
        }
        else
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }

        return strReturn;
    }

    /**
     * retourne l'interface de gestion des sances<BR>
     * Seuls les sances dont la date de dbut est comprise entre les deux dates du filtre seront affichs
     * @param request request la requete HTTP
     * @return String template des listes des seances
     */
    public String applySeanceFilter( HttpServletRequest request )
    {
        setPageTitleProperty( OdsConstants.CONSTANTE_CHAINE_VIDE );

        HashMap<String, Object> model = _seanceService.applySeanceFilter( request );

        ajouterPermissionsDansHashmap( model );

        HtmlTemplate template = AppTemplateService.getTemplate( getListTemplate(  ), getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Retourne l'interface de gestion des sances<BR>
     * Seuls les sances dont la date de debut est suprieure ou gale  la date du jour seront affichs
     *
     * @param request la requete HTTP
     * @return String template des listes des seances
     */
    public String getSeanceList( HttpServletRequest request )
    {
        setPageTitleProperty( PROPERTY_PAGE_LISTE_SEANCE );

        HashMap<String, Object> model = _seanceService.getSeanceList( request );

        ajouterPermissionsDansHashmap( model );

        HtmlTemplate template = AppTemplateService.getTemplate( getListTemplate(  ), getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Permet de stocker toutes les permissions afin de grer les profils au niveau des templates
     * @param model le hashmap contenant les parametres qui vont tre envoys au template
     */
    private void ajouterPermissionsDansHashmap( Map<String, Object> model )
    {
        // Permission de grer
        boolean bPermissionGestion = true;

        if ( !RBACService.isAuthorized( AdminSaufOdjResourceIdService.ADMINISTRATION_SAUF_ODJ,
                    RBAC.WILDCARD_RESOURCES_ID, AdminSaufOdjResourceIdService.PERMISSION_GESTION, getUser(  ) ) )
        {
            bPermissionGestion = false;
        }

        model.put( OdsMarks.MARK_PERMISSION_GESTION, bPermissionGestion );
    }
}
