/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web.voeuamendement;

import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.service.role.VoeuxAmendementsResourceIdService;
import fr.paris.lutece.plugins.ods.service.voeuamendement.IVoeuAmendementService;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.business.rbac.RBAC;
import fr.paris.lutece.portal.service.admin.AccessDeniedException;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.rbac.RBACService;
import fr.paris.lutece.portal.service.template.AppTemplateService;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.web.admin.PluginAdminPageJspBean;
import fr.paris.lutece.util.html.HtmlTemplate;

import org.springframework.beans.factory.annotation.Autowired;

import java.io.IOException;
import java.io.OutputStream;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


/**
 *
 * VoeuAmendementJspBean Classe qui gre les fonctionnalits dadministrations
 * des vux rattachs, des vux non rattachs et des amendements
 *
 * @param <GVoeuAmendement> IVoeuAmendement
 * @param <GVoeuAmendementService> IVoeuAmendementService
 * @param <GEntreeOrdreDuJour> IEntreeOrdreDuJour
 * @param <GOrdreDuJour> IOrdreDuJour
 * @param <GSeance> ISeance
 * @param <GElu> IElu
 * @param <GFichier> IFichier
 */
public abstract class AbstractVoeuAmendementJspBean<GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>, GVoeuAmendementService extends IVoeuAmendementService<GVoeuAmendement, GOrdreDuJour, GEntreeOrdreDuJour, GSeance, GElu, GFichier, GPdd>, GEntreeOrdreDuJour extends IEntreeOrdreDuJour<GElu, GSeance, GFichier, GEntreeOrdreDuJour, GOrdreDuJour>, GOrdreDuJour extends IOrdreDuJour<GEntreeOrdreDuJour, GSeance, GFichier, GElu, GOrdreDuJour>, GSeance extends ISeance, GElu extends IElu, GFichier extends IFichier<GSeance, GFichier>, GPdd extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>>
    extends PluginAdminPageJspBean implements IVoeuAmendementJspBean
{
    public static final String CONSTANTE_GENERE_CSV = "genere_csv";
    public static final String CONSTANTE_GENERE_PDF = "genere_pdf";
    protected static final String PROPERTY_PAGE_LISTE_AMENDEMENT = "ods.listeamendements.label.title";
    protected static final String PROPERTY_PAGE_LISTE_AMENDEMENT_AVAL = "ods.listeamendements.label.titleAval";
    protected static final String PROPERTY_PAGE_TITLE_CREATION_AMENDEMENT = "ods.creationamendement.page.title";
    protected static final String PROPERTY_PAGE_TITLE_MODIFICATION_AMENDEMENT = "ods.modificationamendement.page.title";
    protected static final String PROPERTY_PAGE_LISTE_VOEU = "ods.listevoeux.label.title";
    protected static final String PROPERTY_PAGE_LISTE_VOEU_AVAL = "ods.listevoeux.label.titleAval";
    protected static final String PROPERTY_PAGE_LISTE_VOEU_RATTACHE = "ods.listevoeuxrattaches.label.title";
    protected static final String PROPERTY_PAGE_LISTE_VOEU_RATTACHE_AVAL = "ods.listevoeuxrattaches.label.titleAval";
    protected static final String PROPERTY_PAGE_TITLE_CREATION_VOEU = "ods.creationvoeu.page.title";
    protected static final String PROPERTY_PAGE_TITLE_MODIFICATION_VOEU = "ods.modificationvoeu.page.title";
    private static final long serialVersionUID = -7235528701737262398L;
    private static final String TEMPLATE_HISTORIQUE = "admin/plugins/ods/historique/historique.html";
    private static final String TEMPLATE_CONFIRMATION_FINALISATION = "admin/plugins/ods/voeuamendement/confirmation_finaliser_acte.html";
    @Autowired
    protected GVoeuAmendementService _voeuAmendementService;

    protected abstract String getListeAmendementTemplate(  );

    protected abstract String getCreationAmendementTemplate(  );

    protected abstract String getModificationAmendementTemplate(  );

    protected abstract String getListeVoeuTemplate(  );

    protected abstract String getCreationVoeuTemplate(  );

    protected abstract String getModificationVoeuTemplate(  );

    /**
     * Retourne le chemin du template pour la modification du statut d'un voeu
     * @return le chemin du template
     */
    protected abstract String getModificationStatutVoeuTemplate(  );

    /**
     * Retourne le chemin du template pour la modification du statut d'un amendement
     * @return le chemin du template
     */
    protected abstract String getModificationStatutAmendementTemplate(  );

    protected abstract String getModificationStatutVaTemplate(  );

    protected abstract String getLiasseTemplate(  );

    protected abstract String getListeAmendementSelectionTemplate(  );

    protected abstract String getListeVoeuRattacheSelectionTemplate(  );

    protected abstract String getListeVoeuNonRattacheSelectionTemplate(  );

    protected abstract void getVoeuAmendementListSpec(  );

    public String getHistoriqueTemplate(  )
    {
        return TEMPLATE_HISTORIQUE;
    }

    public String getConfirmationFinalisationTemplate(  )
    {
        return TEMPLATE_CONFIRMATION_FINALISATION;
    }

    /**
     * Permet d'initialiser l'url de retour
     * @param strUrl url de retour
     * @param request la requte http
     */
    public void initUrlRetour( String strUrl, HttpServletRequest request )
    {
        _voeuAmendementService.initUrlRetour( strUrl, request );
    }

    /**
     * Rinitialise tous les attributs de session
     * @param request Requte HTTP
     * @param response Rponse HTTP
     * @param strRight Droits
     * @param bIsGestionAval TRUE si gestion aval, FALSE sinon
     * @throws AccessDeniedException AccessDeniedException
     */
    public void reinitSession( HttpServletRequest request, HttpServletResponse response, String strRight,
        boolean bIsGestionAval ) throws AccessDeniedException
    {
        super.init( request, strRight );
        _voeuAmendementService.reinitSession( request, bIsGestionAval );
    }

    /**
     * Appel  la mthode init() de PluginAdminPageJspBean (initialise les
     * variables de session si elles n'ont pas t auparavant initialises)
     * @param request Requte HTTP
     * @param response Rponse HTTP
     * @param strRight Droits
     * @param bIsGestionAval TRUE si gestion aval, FALSE sinon
     * @throws AccessDeniedException AccessDeniedException
     */
    public void init( HttpServletRequest request, HttpServletResponse response, String strRight, boolean bIsGestionAval )
        throws AccessDeniedException
    {
        super.init( request, strRight );
        _voeuAmendementService.init( request, bIsGestionAval );
    }

    /**
     * Appel  la mthode init() de PluginAdminPageJspBean (initialise les
     * variables de session si elles n'ont pas t auparavant initialises)
     * @param request Requte HTTP
     * @param strRight Droits
     * @param bIsGestionAval TRUE si gestion aval, FALSE sinon
     * @throws AccessDeniedException AccessDeniedException
     */
    public void init( HttpServletRequest request, String strRight, boolean bIsGestionAval )
        throws AccessDeniedException
    {
        super.init( request, strRight );
        _voeuAmendementService.init( request, bIsGestionAval );
    }

    /**
     *
     */
    public void comeFromActe(  )
    {
        _voeuAmendementService.comeFromActe(  );
    }

    /**
     * Retourne l'cran de cration d'un amendement, d'un voeu rattach
     * ou d'un voeu non rattach en fonction du type de cration demand
     * @param request Requte HTTP
     * @param strType Type de VA (amendement, voeu rattach, voeu non rattach)
     * @param doReinitVA Doit-on rinitialiser le VA en session
     * @return template Template HTML
     */
    public String getCreationVoeuAmendement( HttpServletRequest request, String strType, boolean doReinitVA )
    {
        HtmlTemplate template;
        HashMap<String, Object> model = _voeuAmendementService.getCreationVoeuAmendement( request, strType, doReinitVA );

        // Test sur le type de cration demand (amendement, voeu rattach, voeu non ratach)
        if ( strType.equals( OdsConstants.CONSTANTE_TYPE_A ) )
        {
            setPageTitleProperty( PROPERTY_PAGE_TITLE_CREATION_AMENDEMENT );
            template = AppTemplateService.getTemplate( getCreationAmendementTemplate(  ), getLocale(  ), model );
        }
        else
        {
            setPageTitleProperty( PROPERTY_PAGE_TITLE_CREATION_VOEU );
            template = AppTemplateService.getTemplate( getCreationVoeuTemplate(  ), getLocale(  ), model );
        }

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Methode qui effectue la creation d'un amendement, d'un voeu rattach ou d'un voeu non rattach
     * en fonction du type de cration demand.
     * Redirection vers l'interface d'administration(amendement,voeu rattach ou voeu non rattach)
     * @param request request
     * @param strType type de cration demand
     * @return url l'url
     */
    public String doCreationVoeuAmendement( HttpServletRequest request, String strType )
    {
        return _voeuAmendementService.doCreationVoeuAmendement( request, strType, getHomeUrl( request ) );
    }

    /**
     * Retourne l'cran de modification d'un amendement, d'un voeu rattach ou d'un
     * voeu non rattach en fonction du type de VA slectionn dans la liste des VA
     * @param request Requte HTTP
     * @return template Template HTML
     */
    public String getModificationVoeuAmendement( HttpServletRequest request )
    {
        HashMap<String, Object> model = _voeuAmendementService.getModificationVoeuAmendement( request,
                getHomeUrl( request ) );

        if ( model != null )
        {
            // Permission de modifier
            boolean bPermissionModification = true;

            if ( !RBACService.isAuthorized( VoeuxAmendementsResourceIdService.VOEUX_AMENDEMENTS,
                        RBAC.WILDCARD_RESOURCES_ID, VoeuxAmendementsResourceIdService.PERMISSION_MISE_A_JOUR,
                        getUser(  ) ) )
            {
                bPermissionModification = false;
            }

            model.put( OdsMarks.MARK_PERMISSION_MODIFICATION, bPermissionModification );

            HtmlTemplate template;

            // Test sur le type de cration demand (amendement, voeu rattach, voeu non ratach)
            String strTypeVA = _voeuAmendementService.getTypeVa(  );

            if ( strTypeVA.equals( OdsConstants.CONSTANTE_TYPE_A ) ||
                    strTypeVA.equals( OdsConstants.CONSTANTE_TYPE_LR ) )
            {
                setPageTitleProperty( PROPERTY_PAGE_TITLE_MODIFICATION_AMENDEMENT );
                template = AppTemplateService.getTemplate( getModificationAmendementTemplate(  ), getLocale(  ), model );
            }
            else
            {
                setPageTitleProperty( PROPERTY_PAGE_TITLE_MODIFICATION_VOEU );
                template = AppTemplateService.getTemplate( getModificationVoeuTemplate(  ), getLocale(  ), model );
            }

            return getAdminPage( template.getHtml(  ) );
        }
        else
        {
            return getVoeuAmendementList( request, OdsConstants.CONSTANTE_TYPE_A );
        }
    }

    /**
     * Methode qui effectue la modification d'un amendement, d'un voeu rattach
     * en fonction du type du VoeuAmendement prsent en session.
     * Redirection vers l'interface d'administration(amendement,voeu non rattach,voeu rattach)
     * @param request request
     * @return template
     */
    public String doModificationVoeuAmendement( HttpServletRequest request )
    {
        return _voeuAmendementService.doModificationVoeuAmendement( request, getHomeUrl( request ), getUser(  ) );
    }

    /**
     * retourne une demande pour la suppression du VA choisi
     * @param request la requete HTTP
     * @return interface de confirmation de suppression
     */
    public String getSuppressionVoeuAmendement( HttpServletRequest request )
    {
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        /*
         * GESTION DES PROFILS
         * -------------------
         * Si l'utilisateur n'est pas autoris a effectuer cette fonction il est renvoy
         * vers une page lui indiquant qu'il n'a pas les droits ncessaires.
         */
        int nIdVoeuAmendement = OdsUtils.getIntegerParameter( OdsParameters.ID_VOEUAMENDEMENT, request );

        if ( _voeuAmendementService.isPublier( nIdVoeuAmendement ) &&
                !RBACService.isAuthorized( VoeuxAmendementsResourceIdService.VOEUX_AMENDEMENTS,
                    RBAC.WILDCARD_RESOURCES_ID, VoeuxAmendementsResourceIdService.PERMISSION_SUPPRESSION, getUser(  ) ) )
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }
        else
        {
            strReturn = _voeuAmendementService.getSuppressionVoeuAmendement( request );

            if ( strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                strReturn = getVoeuAmendementList( request, OdsConstants.CONSTANTE_CHAINE_VIDE );
            }
        }

        return strReturn;
    }

    /**
     * retourne un avertissement pour signaler que le VA n'est associ  aucun PDD
     * @param request la requete HTTP
     * @return interface de confirmation de suppression
     */
    public String getAvertissementEnregistrementSansPDD( HttpServletRequest request )
    {
        return _voeuAmendementService.getAvertissementEnregistrementSansPDD( request, getHomeUrl( request ) );
    }

    /**
     * supprime le voeuAmendement choisi
     * @param request la requete HTTP
     * @return url de gestion des voeux amendements
     */
    public String doSuppressionVoeuAmendement( HttpServletRequest request )
    {
        return _voeuAmendementService.doSuppressionVoeuAmendement( request, getHomeUrl( request ) );
    }

    /**
     * Retourne l'interface de gestion des amendements, des voeux rattachs
     * ou des voeux non rattachs en fonction du type demand
     * @param request Requte HTTP
     * @param strType Type de VA (amendement, voeu rattach, voeu non rattach)
     * @return Interface de gestion des VA
     */
    public String getVoeuAmendementList( HttpServletRequest request, String strType )
    {
        HashMap<String, Object> model = _voeuAmendementService.getVoeuAmendementList( request, strType,
                getHomeUrl( request ) );

        // Pour la gestion des profils
        ajouterPermissionsDansHashmap( model );

        HtmlTemplate template;

        if ( strType.equals( OdsConstants.CONSTANTE_TYPE_A ) || strType.equals( OdsConstants.CONSTANTE_TYPE_LR ) )
        {
            if ( !isGestionAval(  ) )
            {
                setPageTitleProperty( PROPERTY_PAGE_LISTE_AMENDEMENT );
            }
            else
            {
                setPageTitleProperty( PROPERTY_PAGE_LISTE_AMENDEMENT_AVAL );
            }

            template = AppTemplateService.getTemplate( getListeAmendementTemplate(  ), getLocale(  ), model );
        }
        else
        {
            if ( !isGestionAval(  ) )
            {
                setPageTitleProperty( PROPERTY_PAGE_LISTE_VOEU );
            }
            else
            {
                setPageTitleProperty( PROPERTY_PAGE_LISTE_VOEU_AVAL );
            }

            template = AppTemplateService.getTemplate( getListeVoeuTemplate(  ), getLocale(  ), model );
        }

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * retourne l'interface de gestion des amendements, des voeux  rattachs ou des voeux non rattachs en fonction du type demand
     * @param request la requete HTTP
     * @param strType (amendement,voeu rattache,voeu non rattache)
     * @param strUrlJSP l'url du JSP
     * @param strUrlRetour l'url de retour
     * @return   interface de gestion des lus
     */
    public String getVoeuAmendementList( HttpServletRequest request, String strType, String strUrlJSP,
        String strUrlRetour )
    {
        HashMap<String, Object> model = _voeuAmendementService.getVoeuAmendementList( request, strType, strUrlJSP,
                strUrlRetour );

        HtmlTemplate template;

        if ( strType.equals( OdsConstants.CONSTANTE_TYPE_VR ) )
        {
            if ( !isGestionAval(  ) )
            {
                setPageTitleProperty( PROPERTY_PAGE_LISTE_VOEU_RATTACHE );
            }
            else
            {
                setPageTitleProperty( PROPERTY_PAGE_LISTE_VOEU_RATTACHE_AVAL );
            }

            template = AppTemplateService.getTemplate( getListeVoeuRattacheSelectionTemplate(  ), getLocale(  ), model );
        }
        else if ( strType.equals( OdsConstants.CONSTANTE_TYPE_VNR ) )
        {
            getVoeuAmendementListSpec(  );
            template = AppTemplateService.getTemplate( getListeVoeuNonRattacheSelectionTemplate(  ), getLocale(  ),
                    model );
        }
        else
        {
            if ( !isGestionAval(  ) )
            {
                setPageTitleProperty( PROPERTY_PAGE_LISTE_AMENDEMENT );
            }
            else
            {
                setPageTitleProperty( PROPERTY_PAGE_LISTE_AMENDEMENT_AVAL );
            }

            template = AppTemplateService.getTemplate( getListeAmendementSelectionTemplate(  ), getLocale(  ), model );
        }

        return getAdminPage( template.getHtml(  ) );
    }

    /**
      * methode qui ajoute un dpositaire dans la liste des dpositaires du VA .
      * Retour sur l'interface de cration d'un amendement,voeu non rattach ou
      * voeu rattach en fonction du type du VA
      * @param request la requete
      * @return url l'interface de cration d'un va
      */
    public String doAddDepositaireCreation( HttpServletRequest request )
    {
        return _voeuAmendementService.doAddDepositaireCreation( request, getHomeUrl( request ) );
    }

    /**
     * methode qui supprime  un dpositaire  en dans la liste des dpositaires du VA .
     * Retour sur l'interface de cration d'un amendement,voeu non rattach ou
     * voeu rattach en fonction du type du VA
     * @param request la requete
     * @return url de l'interface de cration d'un VA
     */
    public String doRemoveDepositaireCreation( HttpServletRequest request )
    {
        return _voeuAmendementService.doRemoveDepositaireCreation( request, getHomeUrl( request ) );
    }

    /**
     * methode qui ajoute un dpositaire dans la liste des dpositaires du VA .
     * Retour sur l'interface de modification d'un amendement,voeu non rattach ou
     * voeu rattach en fonction du type du VA
     * @param request la requete
     * @return url de l'inteface de modification
     */
    public String doAddDepositaireModification( HttpServletRequest request )
    {
        return _voeuAmendementService.doAddDepositaireModification( request, getHomeUrl( request ) );
    }

    /**
     * methode qui supprime  un dpositaire en dans la liste des dpositaires du VA .
     * Retour sur l'interface de modification d'un amendement,voeu non rattach ou
     * voeu rattach en fonction du type du VA
     * @param request la requete
     * @return l'url de modification d'un va
     */
    public String doRemoveDepositaireModification( HttpServletRequest request )
    {
        return _voeuAmendementService.doRemoveDepositaireModification( request, getHomeUrl( request ) );
    }

    /**
     * methode qui supprime  un Pdd dans la liste des pdds du VA .
     * Retour sur l'interface de modification d'un amendement,ou
     * voeu rattach en fonction du type du VA
     * @param request la requete
     * @return l'url de modificaion d'un VA
     */
    public String doRemovePdd( HttpServletRequest request )
    {
        return _voeuAmendementService.doRemovePdd( request, getHomeUrl( request ) );
    }

    /**
     * methode qui ajoutes  les  Pdds selectionns par l'utilisateur dans la liste des pdds du VA .
     * Retour sur l'interface de modification d'un amendement,ou
     * voeu rattach en fonction du type du VA
     * @param request la requete
     * @return l'url de modification d'un VA
     */
    public String doAddPdds( HttpServletRequest request )
    {
        return _voeuAmendementService.doAddPdds( request, getHomeUrl( request ) );
    }

    /**
     * rinitialise le voeuAmendement en session
     * @param request la requete
     * @return l'url en fonction di type du VA
     */
    public String doResetVoeuAmendement( HttpServletRequest request )
    {
        return _voeuAmendementService.doResetVoeuAmendement( request, getHomeUrl( request ) );
    }

    /**
     * mthode qui gre la publication du voeu amendement
     * @param request request
     * @param strType le type de VA
     * @return interface d'administration
     */
    public String doPublicationVoeuAmendement( HttpServletRequest request, String strType )
    {
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        /*
         * GESTION DES PROFILS
         * -------------------
         * Si l'utilisateur n'est pas autoris a effectuer cette fonction il est renvoy
         * vers une page lui indiquant qu'il n'a pas les droits ncessaires.
         */
        int nIdVA = OdsUtils.getIntegerParameter( OdsParameters.ID_VOEUAMENDEMENT, request );

        if ( _voeuAmendementService.isPublier( nIdVA ) &&
                !RBACService.isAuthorized( VoeuxAmendementsResourceIdService.VOEUX_AMENDEMENTS,
                    RBAC.WILDCARD_RESOURCES_ID, VoeuxAmendementsResourceIdService.PERMISSION_DEPUBLICATION, getUser(  ) ) )
        {
            strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                    AdminMessage.TYPE_ERROR );
        }
        else
        {
            strReturn = _voeuAmendementService.doPublicationVoeuAmendement( request, strType, getHomeUrl( request ) );
        }

        return strReturn;
    }

    /**
     * retourne l'ecran de visualisation du voeuAmendement choisi
     * @param request la requete
     * @return l'ecran de visualisation du voeuAmendement choisi
     */
    public String getHistoriqueVoeuAmendement( HttpServletRequest request )
    {
        HashMap<String, Object> model = _voeuAmendementService.getHistoriqueVoeuAmendement( request );
        HtmlTemplate template = AppTemplateService.getTemplate( getHistoriqueTemplate(  ), getLocale(  ), model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Tlecharge un fichier.
     * Renvoie un flux de bytes du fichier dans l'objet HttpServletResponse response
     *
     * @param request la requete
     * @param response la rponse
     */
    public void doDownloadFichier( HttpServletRequest request, HttpServletResponse response )
    {
        GFichier fichierTelecharger = _voeuAmendementService.doDownloadFichier( request );

        if ( fichierTelecharger != null )
        {
            try
            {
                String strFileName = fichierTelecharger.getNom(  ) + OdsConstants.CONSTANTE_POINT +
                    fichierTelecharger.getExtension(  );

                response.setHeader( "Content-Disposition", "attachment ;filename=\"" + strFileName + "\"" );
                response.setContentType( "application/pdf" );
                response.setHeader( "Pragma", "public" );
                response.setHeader( "Expires", "0" );
                response.setHeader( "Cache-Control", "must-revalidate,post-check=0,pre-check=0" );
                response.setContentLength( (int) fichierTelecharger.getFichier(  ).getDonnees(  ).length );

                OutputStream os = response.getOutputStream(  );
                os.write( fichierTelecharger.getFichier(  ).getDonnees(  ) );
                os.close(  );
            }
            catch ( IOException e )
            {
                AppLogService.error( e );
            }
        }
    }

    /**
     * Modifie le statut selectionn pour lui attribu le statut demand
     * @param request la requte
     * @return l'url de retour
     */
    public String doAssignStatut( HttpServletRequest request )
    {
        return _voeuAmendementService.doAssignStatut( request );
    }

    /**
     * Supprime le parent du voeu/amendement.
     * @param request la requete
     * @param strAction marqueur de l'action  effecuer (ajout ou suppression)
     * @param strReturn marqueur sur la page de retour (cration ou modification)
     * @return la page de cration ou de modification
     */
    public String doAddRemoveParent( HttpServletRequest request, String strAction, String strReturn )
    {
        return _voeuAmendementService.doAddRemoveParent( request, strAction, strReturn, getHomeUrl( request ) );
    }

    /**
     * Retourne TRUE si gestion aval, FALSE sinon
     * @return TRUE si gestion aval, FALSE sinon
     */
    public boolean isGestionAval(  )
    {
        return _voeuAmendementService.isGestionAval(  );
    }

    /**
     * @return _b boolean
     */
    public boolean isUrLRetour(  )
    {
        return _voeuAmendementService.isUrLRetour(  );
    }

    /**
     * affecte la variable bUrlRetour retour
     * @param bUrlRetour =true retour a l'interface de gestion de jsp _strUrlRetour
     */
    public void setBurlRetour( boolean bUrlRetour )
    {
        _voeuAmendementService.setBurlRetour( bUrlRetour );
    }

    /**
     * affecte  l'url de retour
     * @param strUrl url de retour
     */
    public void setUrlRetour( String strUrl )
    {
        _voeuAmendementService.setUrlRetour( strUrl );
    }

    /**
     * retourne l'url de retour
     * @return l'url de retour
     */
    public String getUrlRetour(  )
    {
        return _voeuAmendementService.getUrlRetour(  );
    }

    /**
     * Permet de confirmer la finalisation d'un acte
     *
     * @param request
     *            la requete HTTP
     * @return strUrl url de retour
     */
    public String doFinaliserActe( HttpServletRequest request )
    {
        return _voeuAmendementService.doFinaliserActe( request, getHomeUrl( request ) );
    }

    /**
     * retourne le formulaire de confirmation d'envoi manuel de l'acte
     * @param request la requte http
     * @return la page de confirmation
     */
    public String getConfirmFinaliser( HttpServletRequest request )
    {
        HashMap<String, Object> model = _voeuAmendementService.getConfirmFinaliser( request );
        HtmlTemplate template = AppTemplateService.getTemplate( getConfirmationFinalisationTemplate(  ), getLocale(  ),
                model );

        return getAdminPage( template.getHtml(  ) );
    }

    /**
     * Permet de stocker toutes les permissions afin de grer les profils au niveau des templates
     * @param model le hashmap contenant les parametres qui vont tre envoys au template
     */
    protected void ajouterPermissionsDansHashmap( Map<String, Object> model )
    {
        /*
         * Permission de dpublier
         */
        boolean bPermissionDepublication = true;

        if ( !RBACService.isAuthorized( VoeuxAmendementsResourceIdService.VOEUX_AMENDEMENTS,
                    RBAC.WILDCARD_RESOURCES_ID, VoeuxAmendementsResourceIdService.PERMISSION_DEPUBLICATION, getUser(  ) ) )
        {
            bPermissionDepublication = false;
        }

        model.put( OdsMarks.MARK_PERMISSION_DEPUBLICATION, bPermissionDepublication );

        /*
         * Permission de modifier
         */
        boolean bPermissionModification = true;

        if ( !RBACService.isAuthorized( VoeuxAmendementsResourceIdService.VOEUX_AMENDEMENTS,
                    RBAC.WILDCARD_RESOURCES_ID, VoeuxAmendementsResourceIdService.PERMISSION_MISE_A_JOUR, getUser(  ) ) )
        {
            bPermissionModification = false;
        }

        model.put( OdsMarks.MARK_PERMISSION_MODIFICATION, bPermissionModification );

        /*
         * Permission de supprimer
         */
        boolean bPermissionSuppression = true;

        if ( !RBACService.isAuthorized( VoeuxAmendementsResourceIdService.VOEUX_AMENDEMENTS,
                    RBAC.WILDCARD_RESOURCES_ID, VoeuxAmendementsResourceIdService.PERMISSION_SUPPRESSION, getUser(  ) ) )
        {
            bPermissionSuppression = false;
        }

        model.put( OdsMarks.MARK_PERMISSION_SUPPRESSION, bPermissionSuppression );
    }
}
