/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web.xpage.ordredujour;

import fr.paris.lutece.plugins.ods.service.ordredujour.IOrdreDuJourService;
import fr.paris.lutece.plugins.ods.service.utilisateur.IUtilisateurService;
import fr.paris.lutece.plugins.ods.service.xpage.ordredujour.IOrdreDuJourAppService;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.web.xpages.XPage;
import fr.paris.lutece.portal.web.xpages.XPageApplication;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;


/**
 * Interface d'affichage de l'ordre du jour
 */
public class OrdreDuJourApp implements XPageApplication
{
    @Autowired
    private IOrdreDuJourAppService _ordreDuJourAppService;
    @Autowired
    private IUtilisateurService _utilisateurService;
    @Autowired
    private IOrdreDuJourService<?, ?, ?, ?, ?, ?> _ordreDuJourService;

    /**
    * @param request la requte HTTP
    * @param nMode le mode
    * @param plugin le plugin
    * @return XPage template
    */
    public XPage getPage( HttpServletRequest request, int nMode, Plugin plugin )
    {
        HashMap<String, Object> model = _ordreDuJourAppService.getPage( request, nMode, plugin );

        _utilisateurService.gestionUtilisateur( model, request, plugin );

        int nIdOdj = -1;

        if ( request.getParameter( OdsParameters.ID_ODJ ) != null )
        {
            try
            {
                nIdOdj = Integer.parseInt( request.getParameter( OdsParameters.ID_ODJ ) );
            }
            catch ( NumberFormatException ne )
            {
                AppLogService.error( ne );
            }
        }

        String strContent = _ordreDuJourService.getFrontHtml( nIdOdj, plugin, request.getLocale(  ), model );
        String strPageTitle = I18nService.getLocalizedString( _ordreDuJourAppService.getPropertyPageTitle(  ),
                request.getLocale(  ) );
        String strPagePathLabel = I18nService.getLocalizedString( _ordreDuJourAppService.getPropertyPathLabel(  ),
                request.getLocale(  ) );

        XPage page = new XPage(  );
        page.setTitle( strPageTitle );
        page.setPathLabel( strPagePathLabel );
        page.setContent( strContent );

        return page;
    }
}
