/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web.xpage.panier;

import fr.paris.lutece.plugins.ods.service.xpage.panier.IPanierConfirmationAppService;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.template.AppTemplateService;
import fr.paris.lutece.portal.web.xpages.XPage;
import fr.paris.lutece.portal.web.xpages.XPageApplication;
import fr.paris.lutece.util.html.HtmlTemplate;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;


/**
 * PanierConfirmationApp: a pour responsabilit d'afficher la popup de tlchargement du panier
 */
public class PanierConfirmationApp implements XPageApplication
{
    private static final String CONSTANTE_DATEAJOUT_NULL = "null";
    @Autowired
    private IPanierConfirmationAppService _panierConfirmationAppService;

    /**
     * renvoie la page.
     * @param request le requte http
     * @param nMode le mode
     * @param plugin le Plugin actif
     * @return la page
     */
    public XPage getPage( HttpServletRequest request, int nMode, Plugin plugin )
    {
        HashMap<String, Object> model = _panierConfirmationAppService.getPage( request, nMode, plugin );

        // Rcupre la date de la demande de tlchargment du panier
        String dateAjout = request.getParameter( OdsMarks.MARK_DATE_AJOUT );
        HtmlTemplate template;

        // Si la date d'ajout est correcte alors on vrifie si l'archive est prsente dans la base de donnes
        if ( ( dateAjout != null ) && !CONSTANTE_DATEAJOUT_NULL.equals( dateAjout ) )
        {
            template = AppTemplateService.getTemplate( _panierConfirmationAppService.getTemplate(  ),
                    request.getLocale(  ), model );
        }
        else
        {
            template = AppTemplateService.getTemplate( _panierConfirmationAppService.getTemplateKO(  ),
                    request.getLocale(  ), model );
        }

        String strPageTitle = I18nService.getLocalizedString( _panierConfirmationAppService.getPropertyPageTitle(  ),
                request.getLocale(  ) );
        String strPagePathLabel = I18nService.getLocalizedString( _panierConfirmationAppService.getPropertyPathLabel(  ),
                request.getLocale(  ) );

        XPage page = new XPage(  );
        page.setTitle( strPageTitle );
        page.setPathLabel( strPagePathLabel );
        page.setContent( template.getHtml(  ) );

        return page;
    }
}
