/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web.xpage.pdd;

import fr.paris.lutece.plugins.ods.business.pdd.IPDDHome;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.service.xpage.pdd.IPddAppService;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.template.AppTemplateService;
import fr.paris.lutece.portal.web.xpages.XPage;
import fr.paris.lutece.portal.web.xpages.XPageApplication;
import fr.paris.lutece.util.html.HtmlTemplate;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;


/**
 * PddApp: a pour responsabilit d'afficher la fiche d'un PDD
 */
public class PddApp implements XPageApplication
{
    @Autowired
    private IPddAppService _pddAppService;
    @Autowired
    private IPDDHome<?, ?, ?, ?, ?, ?, ?, ?> _projetDeliberationHome;

    /**
     * @param request la requte HTTP
     * @param nMode le mode
     * @param plugin le plugin
     * @return XPage template
     */
    public XPage getPage( HttpServletRequest request, int nMode, Plugin plugin )
    {
        HashMap<String, Object> model = _pddAppService.getPage( request, nMode, plugin );
        HtmlTemplate template = AppTemplateService.getTemplate( _pddAppService.getTemplate(  ), request.getLocale(  ),
                model );

        String strPageTitle = getPageTitle( request, plugin );
        String strPagePathLabel = I18nService.getLocalizedString( _pddAppService.getPropertyPathLabel(  ),
                request.getLocale(  ) );

        XPage page = new XPage(  );
        page.setTitle( strPageTitle );
        page.setPathLabel( strPagePathLabel );
        page.setContent( template.getHtml(  ) );

        return page;
    }

    /**
     * Returns the page title
     * @param request The HTTP request
     * @param plugin the plugin
     * @return the page title
     */
    public String getPageTitle( HttpServletRequest request, Plugin plugin )
    {
        String strPageTitle = OdsConstants.CONSTANTE_CHAINE_VIDE;

        int nIdPDD = OdsUtils.getIntegerParameter( OdsParameters.ID_PDD, request );

        if ( nIdPDD > -1 )
        {
            IPDD<?, ?, ?, ?, ?> pdd = _projetDeliberationHome.findByPrimaryKey( nIdPDD, plugin );

            if ( ( pdd != null ) && pdd.isEnLigne(  ) )
            {
                if ( pdd.getTypePdd(  ).equals( IPDD.CONSTANTE_TYPE_PROJET ) )
                {
                    strPageTitle = I18nService.getLocalizedString( _pddAppService.getPropertyPageTitlePj(  ),
                            request.getLocale(  ) );
                }
                else
                {
                    strPageTitle = I18nService.getLocalizedString( _pddAppService.getPropertyPageTitlePp(  ),
                            request.getLocale(  ) );
                }
            }
        }

        return strPageTitle;
    }
}
