/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web.xpage.search;

import fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.service.xpage.search.ICriteresDeRechercheAppService;
import fr.paris.lutece.plugins.ods.utils.commons.DateUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.security.SecurityService;
import fr.paris.lutece.portal.service.template.AppTemplateService;
import fr.paris.lutece.portal.web.xpages.XPage;
import fr.paris.lutece.portal.web.xpages.XPageApplication;
import fr.paris.lutece.util.html.HtmlTemplate;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;


/**
 *
 * AbstractCriteresDeRechercheApp
 *
 * @param <GRequeteUtilisateur> IRequeteUtilisateur
 */
public abstract class AbstractCriteresDeRechercheApp<GSeance extends ISeance, GSeanceFilter extends ISeanceFilter, GRequeteUtilisateur extends IRequeteUtilisateur<GSeance, GSeanceFilter>>
    implements XPageApplication
{
    private static final String PARAM_SAVE_CRITERE = "save";
    private static final String PARAM_NOM_REQUETE = "nom_requete";
    private static final String PARAM_VALUE_NEXT = "next";
    private static final String PARAM_TYPE_REQUETE = "type_requete";
    private static final String PARAM_DATE_1 = "date_1";
    private static final String PARAM_DATE_2 = "date_2";
    private static final String PARAM_CHAMP_RECHERCHE = "champ_recherche";
    private static final String PARAM_TYPES_DOCUMENTS = "selected_types_document";
    private static final String PARAM_FORMATIONS_CONSEIL = "selected_formations_conseil";
    private static final String PARAM_RAPPORTEURS = "selected_rapporteurs";
    private static final String PARAM_ARRONDISSEMENTS = "selected_arrondissements";
    private static final String PARAM_DIRECTIONS = "selected_directions";
    private static final String PARAM_CATEGORIES_DELIBERATION = "selected_categories_deliberation";
    private static final String PARAM_GROUPES_DEPOSITAIRES = "selected_groupes_depositaires";
    private static final String PARAM_ELUS_DEPOSITAIRES = "selected_elus_depositaires";
    private static final String PARAM_DEPOSE_EXECUTIF_SELECTED = "depose_executif_selected";
    private static final String PARAM_SELECTED_STATUTS_VOEUX_AMENDEMENTS = "selected_statuts_voeux_amendements";
    private static final String PARAM_SELECTED_STATUTS_PROPOSITIONS = "selected_statuts_propositions";
    private static final String PARAM_SELECTED_STATUTS_PROJETS = "selected_statuts_projets";
    private static final String PROPERTY_PAGE_ERREUR_TITRE = "ods.front.error.page.title";
    private static final String PROPERTY_PAGE_ERREUR_REQUETE = "ods.front.error.page.title.requete";
    private static final String ERROR_NO_REQUETE_NAME = "ods.front.mescriteres.error.noname";
    private static final String ERROR_NO_ACCESS = "ods.front.mescriteres.error.noaccess";
    private static final String CONSTANTE_DEUX_POINTS = ":";
    @Autowired
    private ICriteresDeRechercheAppService<GSeance, GSeanceFilter, GRequeteUtilisateur> _criteresDeRechercheAppService;

    /**
     *
     * @return une instance de GRequeteUtilisateur
     */
    protected abstract GRequeteUtilisateur newRequeteUtilisateurInstance(  );

    /**
     *
     * @param request la requete HTTP
     * @param requeteUtilisateur la requete utilisateur
     */
    protected abstract void prepareRequeteUtilisateurSpec( HttpServletRequest request,
        GRequeteUtilisateur requeteUtilisateur );

    /**
     *
     * @return le template de requete non valide
     */
    protected abstract String getTemplateRequeteNonValide(  );

    /**
     * renvoie la page.
     * @param request le requte http
     * @param nMode le mode
     * @param plugin le Plugin actif
     * @return la page
     */
    public XPage getPage( HttpServletRequest request, int nMode, Plugin plugin )
    {
        // Ce paramtre vient de la page de rsultat d'une recherche. Si cette page
        // est appele avec une requte d'enregistrement de celle-ci, on enregistre
        // la requte de recherche si le nom affect  celui-ci n'est pas null ou vide.
        if ( request.getParameter( PARAM_SAVE_CRITERE ) != null )
        {
            GRequeteUtilisateur requeteUtilisateur = prepareRequeteUtilisateur( request );

            if ( requeteUtilisateur != null )
            {
                if ( ( request.getParameter( PARAM_NOM_REQUETE ) == null ) ||
                        request.getParameter( PARAM_NOM_REQUETE ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                {
                    return getErrorPage( request, requeteUtilisateur, ERROR_NO_REQUETE_NAME, plugin );
                }

                _criteresDeRechercheAppService.saveCriteria( requeteUtilisateur, plugin );
            }
        }

        HashMap<String, Object> model = _criteresDeRechercheAppService.getPage( request, nMode, plugin );

        if ( model == null )
        {
            return getErrorPage( request, null, ERROR_NO_ACCESS, plugin );
        }

        HtmlTemplate template = AppTemplateService.getTemplate( _criteresDeRechercheAppService.getTemplate(  ),
                request.getLocale(  ), model );

        String strPageTitle = I18nService.getLocalizedString( _criteresDeRechercheAppService.getPropertyPageTitle(  ),
                request.getLocale(  ) );
        String strPagePathLabel = I18nService.getLocalizedString( _criteresDeRechercheAppService.getPropertyPathLabel(  ),
                request.getLocale(  ) );

        XPage page = new XPage(  );
        page.setTitle( strPageTitle );
        page.setPathLabel( strPagePathLabel );
        page.setContent( template.getHtml(  ) );

        return page;
    }

    /**
     * Retourne une page d'erreur avec le message adapt
     * @param request la requete Http
     * @param requeteUtilisateur la requete utilisateur
     * @param strErrorMessage le message  afficher
     * @param plugin le plugin ODS
     * @return une page d'erreur
     */
    private XPage getErrorPage( HttpServletRequest request, GRequeteUtilisateur requeteUtilisateur,
        String strErrorMessage, Plugin plugin )
    {
        HashMap<String, Object> model = _criteresDeRechercheAppService.getErrorPage( request, requeteUtilisateur,
                strErrorMessage, plugin );

        HtmlTemplate template = AppTemplateService.getTemplate( getTemplateRequeteNonValide(  ), request.getLocale(  ),
                model );

        String strPageTitle = I18nService.getLocalizedString( PROPERTY_PAGE_ERREUR_TITRE, request.getLocale(  ) ) +
            OdsConstants.CONSTANTE_ESPACE + CONSTANTE_DEUX_POINTS + OdsConstants.CONSTANTE_ESPACE +
            I18nService.getLocalizedString( PROPERTY_PAGE_ERREUR_REQUETE, request.getLocale(  ) );
        String strPagePathLabel = I18nService.getLocalizedString( _criteresDeRechercheAppService.getPropertyPathLabel(  ),
                request.getLocale(  ) );

        XPage page = new XPage(  );
        page.setTitle( strPageTitle );
        page.setPathLabel( strPagePathLabel );
        page.setContent( template.getHtml(  ) );

        return page;
    }

    /**
     * Cre la requete utilisateur  partir de la requete Http
     * @param request la requete Http
     * @return la requete utilisateur
     */
    private GRequeteUtilisateur prepareRequeteUtilisateur( HttpServletRequest request )
    {
        if ( SecurityService.getInstance(  ).getRegisteredUser( request ) != null )
        {
            GRequeteUtilisateur requeteUtilisateur = newRequeteUtilisateurInstance(  );
            String strUserName = SecurityService.getInstance(  ).getRegisteredUser( request ).getName(  );
            String strNomRequete = request.getParameter( PARAM_NOM_REQUETE );
            requeteUtilisateur.setUserName( strUserName );
            requeteUtilisateur.setNomRequete( strNomRequete );
            requeteUtilisateur.setRechercheArchive( !request.getParameter( PARAM_SAVE_CRITERE ).equals( PARAM_VALUE_NEXT ) );

            requeteUtilisateur.setTypeRequete( request.getParameter( PARAM_TYPE_REQUETE ) );
            requeteUtilisateur.setChampRecherche( request.getParameter( PARAM_CHAMP_RECHERCHE ) );
            requeteUtilisateur.setListeTypesDocument( request.getParameter( PARAM_TYPES_DOCUMENTS ) );
            requeteUtilisateur.setListeFormationsConseil( request.getParameter( PARAM_FORMATIONS_CONSEIL ) );

            String strDate1 = request.getParameter( PARAM_DATE_1 );
            String strDate2 = request.getParameter( PARAM_DATE_2 );

            if ( strDate1 != null )
            {
                requeteUtilisateur.setPremiereDate( DateUtils.getDate( strDate1, true ) );
            }

            if ( strDate2 != null )
            {
                requeteUtilisateur.setDeuxiemeDate( DateUtils.getDate( strDate2, true ) );
            }

            requeteUtilisateur.setListeRapporteurs( request.getParameter( PARAM_RAPPORTEURS ) );
            requeteUtilisateur.setListeArrondissements( request.getParameter( PARAM_ARRONDISSEMENTS ) );
            requeteUtilisateur.setListeDirections( request.getParameter( PARAM_DIRECTIONS ) );
            requeteUtilisateur.setListeCategoriesDeliberation( request.getParameter( PARAM_CATEGORIES_DELIBERATION ) );
            requeteUtilisateur.setListeGroupesDepositaires( request.getParameter( PARAM_GROUPES_DEPOSITAIRES ) );
            requeteUtilisateur.setListeElusDepositaires( request.getParameter( PARAM_ELUS_DEPOSITAIRES ) );
            requeteUtilisateur.setNotifie( false );

            String strDeposeExecutif = request.getParameter( PARAM_DEPOSE_EXECUTIF_SELECTED );

            if ( ( strDeposeExecutif != null ) && strDeposeExecutif.trim(  ).equals( OdsConstants.INTEGER_TRUE ) )
            {
                requeteUtilisateur.setDeposeExecutif( true );
            }

            requeteUtilisateur.setListeStatutsProjets( request.getParameter( PARAM_SELECTED_STATUTS_PROJETS ) );
            requeteUtilisateur.setListeStatutsPropositions( request.getParameter( PARAM_SELECTED_STATUTS_PROPOSITIONS ) );
            requeteUtilisateur.setListeStatutsVoeuxAmendements( request.getParameter( 
                    PARAM_SELECTED_STATUTS_VOEUX_AMENDEMENTS ) );
            prepareRequeteUtilisateurSpec( request, requeteUtilisateur );

            return requeteUtilisateur;
        }

        return null;
    }
}
