/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web.xpage.search;

import fr.paris.lutece.plugins.ods.dto.categoriedeliberation.CategorieDeliberation;
import fr.paris.lutece.plugins.ods.dto.direction.Direction;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.groupepolitique.IGroupePolitique;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheAppService;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.util.AppPathService;
import fr.paris.lutece.portal.service.util.AppPropertiesService;
import fr.paris.lutece.portal.web.xpages.XPage;

import org.springframework.beans.factory.annotation.Autowired;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.StringTokenizer;

import javax.servlet.http.HttpServletRequest;


/**
 * AbstractRechercheApp
 */
public abstract class AbstractRechercheApp<GSeance extends ISeance, GSeanceFilter extends ISeanceFilter, GRequeteUtilisateur extends IRequeteUtilisateur<GSeance, GSeanceFilter>, GElu extends IElu, GFichier extends IFichier<GSeance, GFichier>, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>>
    implements IRechercheApp
{
    protected static final String MARK_LISTE_SELECTED_TYPES_DOCUMENT = "liste_selected_types_document";
    protected static final String MARK_VALUE_URL_FORMULAIRE = "/jsp/site/Portal.jsp?page=ods_recherche";
    protected static final String MARK_URL_FORMULAIRE = "url_formulaire";
    protected static final String MARK_URL_PDD = "url_pdd";
    protected static final String MARK_VALUE_URL_PDD = "jsp/site/Portal.jsp?page=ods_pdd";
    protected static final String MARK_RAPPORTEURS = "rapporteurs";
    protected static final String MARK_ARRONDISSEMENTS = "arrondissements";
    protected static final String MARK_DIRECTIONS = "directions";
    protected static final String MARK_CATEGORIES_DELIBERATION = "categories_deliberation";
    protected static final String MARK_GROUPES = "groupes";
    protected static final String MARK_ELUS = "elus";
    protected static final String MARK_CHAMP_RECHERCHE = "champ_recherche";
    protected static final String PARAM_PROJET = "projet";
    protected static final String PARAM_PROPOSITION = "proposition";
    protected static final String PARAM_AMENDEMENT = "amendement";
    protected static final String PARAM_VOEU = "voeu";
    protected static final String PARAM_ARRONDISSEMENT1 = "arrondissement1";
    protected static final String PARAM_ARRONDISSEMENT2 = "arrondissement2";
    protected static final String PARAM_REQUETE = "id_requete";
    protected static final String PARAM_SELECTED_TYPES_DOC = "selected_types_document";
    protected static final String PARAM_SELECTED_RAPPORTEURS = "selected_rapporteurs";
    protected static final String PARAM_SELECTED_ARRONDISSEMENTS = "selected_arrondissements";
    protected static final String PARAM_SELECTED_DIRECTIONS = "selected_directions";
    protected static final String PARAM_SELECTED_CATEGORIES_DELIBERATION = "selected_categories_deliberation";
    protected static final String PARAM_SELECTED_GROUPES = "selected_groupes_depositaires";
    protected static final String PARAM_DEPOSE_EXECUTIF_SELECTED = "depose_executif_selected";
    protected static final String PARAM_SELECTED_ELUS = "selected_elus_depositaires";
    protected static final String PARAM_CHAMP_RECHERCHE = "champ_recherche";
    protected static final String MARK_LISTE_RAPPORTEURS = "liste_elus_rapporteurs";
    protected static final String MARK_LISTE_DIRECTIONS = "liste_directions";
    protected static final String MARK_LISTE_CATEGORIES = "liste_categories";
    protected static final String MARK_LISTE_GROUPES = "liste_groupes";
    protected static final String MARK_LISTE_ELUS = "liste_elus";
    protected static final String MARK_LISTE_SELECTED_ARRONDISSEMENTS = "liste_selected_arrondissements";
    protected static final String MARK_LISTE_SELECTED_RAPPORTEURS = "liste_selected_elus_rapporteurs";
    protected static final String MARK_LISTE_SELECTED_DIRECTIONS = "liste_selected_directions";
    protected static final String MARK_LISTE_SELECTED_CATEGORIES = "liste_selected_categories";
    protected static final String MARK_LISTE_SELECTED_GROUPES = "liste_selected_groupes";
    protected static final String MARK_DEPOSE_EXECUTIF_SELECTED = "depose_executif_selected";
    protected static final String MARK_LISTE_SELECTED_ELUS = "liste_selected_elus";
    protected static final String MARK_LISTE_PROJETS = "liste_projets";
    protected static final String MARK_LISTE_PROPOSITIONS = "liste_propositions";
    protected static final String MARK_LISTE_AMENDEMENTS = "liste_amendements";
    protected static final String MARK_LISTE_VOEUX = "liste_voeux";
    protected static final String MARK_NOMBRE_FICHIERS = "nombre_fichiers";
    protected static final String MARK_NOMBRE_OBJETS = "nombre_objets";
    protected static final String MARK_REQUETE_UTILISATEUR = "user_search";
    protected static final String MARK_LISTE_PDD_RATTACHE_ODJ = "liste_pdds_rattache_odj";
    protected static final String MARK_TROP_DE_RESULTATS = "trop_de_resultats";
    protected static final String MARK_MESSAGE = "message";
    protected static final String MARK_ERROR_MESSAGE = "error_message";
    protected static final String ERREUR_REQUETE_SIMPLE_NON_VALIDE = "ods.front.rechercheresultats.message.nonValidSimpleRequest";
    protected static final String ERREUR_RECHERCHE_MOT_CLE_NON_VALIDE = "ods.front.rechercheresultats.message.nonValidChampMotCle";
    protected static final String ERREUR_REQUETE_UTILISATEUR = "ods.front.mescriteres.error.noaccess";
    protected static final String ERREUR_TROP_DE_RESULTATS = "ods.front.rechercheresultats.message.tooManyResults";
    protected static final String TYPE_PROJET = "projet";
    protected static final String TYPE_PROPOSITION = "proposition";
    protected static final String TYPE_AMENDEMENT = "amendement";
    protected static final String TYPE_VOEU = "voeu";
    protected static final int DEFAUT_MAX_NOMBRE_FICHIERS = 50;
    protected static final String PROPERTY_MAX_NOMBRE_FICHIERS = "ods.search.maxNombreFichiers";
    protected static final int MAX_NOMBRE_FICHIERS = AppPropertiesService.getPropertyInt( PROPERTY_MAX_NOMBRE_FICHIERS,
            DEFAUT_MAX_NOMBRE_FICHIERS );
    protected static final String MARK_NOMBRE_OBJETS_DETAILLES = "nombre_objets_detailles";
    protected static final String PARAM_SEARCH = "search_input";
    protected static final String TYPE_REQUETE_SIMPLE = "simple";
    private static final String PARAM_RESULT = "result";
    private static final String PARAM_CATEGORIES1 = "categorie1";
    private static final String PARAM_CATEGORIES2 = "categorie2";
    private static final String PARAM_GROUPES1 = "groupe1";
    private static final String PARAM_GROUPES2 = "groupe2";
    private static final String PARAM_ELUS = "elu";
    private static final String PARAM_DIRECTION = "direction";
    private static final String PARAM_RAPPORTEURS = "rapporteur";
    private static final String PARAM_SIMPLE_SEARCH = "search";
    private static final String PARAM_REFERENCE = "code_pdd";
    private static final String ERREUR_PAS_DE_PROCHAINE_SEANCE = "ods.front.rechercheresultats.message.aucuneProchaineSeance";
    @Autowired
    private IRechercheAppService<GSeance, GSeanceFilter, GRequeteUtilisateur, GElu, GFichier, GVoeuAmendement, GPDD> _rechercheAppService;

    /**
     *
     * @param request la requete HTTP
     * @param requeteUtilisateur la requete utilisateur
     */
    protected abstract void prepareRequeteUtilisateurSimpleSpec( HttpServletRequest request,
        GRequeteUtilisateur requeteUtilisateur );

    /**
     * Renvoie la page
     * @param request la requete HTTP
     * @param nMode le mode d'affichage
     * @param plugin le plugin
     * @return XPage la page
     */
    public XPage getPage( HttpServletRequest request, int nMode, Plugin plugin )
    {
        HashMap<String, Object> model = new HashMap<String, Object>(  );
        GSeance prochaineSeance = _rechercheAppService.getProchaineSeance( plugin );

        // On met  jour la liste des utilisateurs et gre la session de l'utilisateur
        _rechercheAppService.gestionUtilisateur( model, request, plugin );

        model.put( OdsMarks.MARK_URL_WEBAPP, AppPathService.getBaseUrl( request ) );

        //On rcupre la sance en cours si elle existe
        ISeance seanceEnCours = _rechercheAppService.getSeanceEnCours( plugin );

        if ( seanceEnCours != null )
        {
            model.put( OdsMarks.MARK_ID_SEANCE_EN_COURS, seanceEnCours.getIdSeance(  ) );
        }

        // Si il n'y a pas de prochaine sance, on affiche une page d'erreur
        if ( prochaineSeance == null )
        {
            return getErrorPage( model, ERREUR_PAS_DE_PROCHAINE_SEANCE, request.getLocale(  ) );
        }

        String strResult = request.getParameter( PARAM_RESULT );
        String strIdRequete = request.getParameter( PARAM_REQUETE );

        if ( ( ( strResult == null ) || strResult.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) ) &&
                ( strIdRequete == null ) )
        {
            return getPageFormulaire( model, request, plugin );
        }

        return getPageResultat( model, request, prochaineSeance, plugin );
    }

    /**
     * Retourne la page du formulaire de recherche multi-criteres
     * @param request la requete Http
     * @param model  le modle pour le template
     * @param plugin le plugin ods
     * @return la page du formulaire de recherche
     */
    public abstract XPage getPageFormulaire( Map<String, Object> model, HttpServletRequest request, Plugin plugin );

    /**
     * Retourne le rsultat d'une recherche dans les documents et objets d'ODS
     * @param model  le modle pour le template
     * @param request la requte Http
     * @param prochaineSeance la prochaine sance
     * @param plugin le plugin
     * @return la page de rsultat d'une recherche
     */
    public abstract XPage getPageResultat( Map<String, Object> model, HttpServletRequest request,
        GSeance prochaineSeance, Plugin plugin );

    /**
     * Retourne une page d'erreur avec le message adapt
     * @param strErrorMessage le message  afficher
     * @param locale la locale du navigateur
     * @param model  le modle pour le template
     * @return une page d'erreur
     */
    public abstract XPage getErrorPage( Map<String, Object> model, String strErrorMessage, Locale locale );

    /**
     * Indique si une requte d'une recherche par rfrence de code de PDD est valide,
     * c'est  dire que le champ de recherche est rempli
     * @param request la requte
     * @return <b>true</b> si la requte est valide, <b>false</b> sinon
     */
    protected boolean isReferenceValideForm( HttpServletRequest request )
    {
        return ( ( request.getParameter( PARAM_REFERENCE ) != null ) &&
        !request.getParameter( PARAM_REFERENCE ).trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) );
    }

    /**
     * Indique si une requte d'une recherche "simple" par mot-cl est valide, c'est  dire que
     * le champ de recherche est rempli
     * @param request la requte
     * @return <b>true</b> si la requte est valide, <b>false</b> sinon
     */
    protected boolean isSimpleValideForm( HttpServletRequest request )
    {
        return ( ( request.getParameter( PARAM_SIMPLE_SEARCH ) != null ) &&
        !request.getParameter( PARAM_SIMPLE_SEARCH ).trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) );
    }

    /**
     * Prpare l'objet RequeteUtilisateur avec les paramtres pour une recherche par mot-cl.
     * Si l'utilisateur dcide de l'enregistrer l'objet sera ainsi dj cr ( transmettre en attribut de la requte)
     * @param request la requte
     * @param requeteUtilisateur la requte de l'utilisateur
     */
    protected void prepareRequeteUtilisateurReference( HttpServletRequest request,
        GRequeteUtilisateur requeteUtilisateur )
    {
        requeteUtilisateur.setRechercheArchive( false );
        requeteUtilisateur.setTypeRequete( OdsParameters.REFERENCE );
        requeteUtilisateur.setChampRecherche( request.getParameter( PARAM_REFERENCE ).trim(  ) );
        requeteUtilisateur.setNotifie( false );
    }

    /**
     * Prpare l'objet RequeteUtilisateur avec les paramtres pour une recherche par rfrence de PDD.
     * Si l'utilisateur dcide de l'enregistrer l'objet sera ainsi dj cr ( transmettre en attribut de la requte)
     * @param request la requte
     * @param requeteUtilisateur la requte de l'utilisateur
     */
    protected void prepareRequeteUtilisateurSimple( HttpServletRequest request, GRequeteUtilisateur requeteUtilisateur )
    {
        requeteUtilisateur.setRechercheArchive( false );
        requeteUtilisateur.setTypeRequete( TYPE_REQUETE_SIMPLE );
        requeteUtilisateur.setChampRecherche( request.getParameter( PARAM_SIMPLE_SEARCH ).trim(  ) );
        requeteUtilisateur.setNotifie( false );
        prepareRequeteUtilisateurSimpleSpec( request, requeteUtilisateur );
    }

    /**
     * Vrifie que le champ de recherche par mot-cl ne contient pas de caractre spcial qui altrerait le
     * rsultat de la recherche dans l'index Lucene
     * @param request la requte
     * @return <b>true</b> si le champ de recherche est correct, <b>false</b> sinon
     */
    protected boolean isChampMotCleValide( HttpServletRequest request )
    {
        String strRequete = request.getParameter( PARAM_SEARCH );

        if ( ( strRequete != null ) && !strRequete.trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strRequete = strRequete.trim(  );

            if ( ( strRequete.indexOf( " -" ) > -1 ) || strRequete.startsWith( "-" ) )
            {
                return false;
            }

            String strDelimiteurs = " &~|!*?+(){}[]^:\"";
            String[] tabMotsExclus = 
                {
                    "&", "|", "~", "!", "*", "?", "+", "(", ")", "{", "}", "[", "]", "^", ":", "\"", "\\", "AND", "and",
                    "OR", "or",
                };
            List<String> listMotsExclus = new ArrayList<String>(  );

            for ( String strMotExclu : tabMotsExclus )
            {
                listMotsExclus.add( strMotExclu );
            }

            StringTokenizer stringTokenizer = new StringTokenizer( strRequete, strDelimiteurs, true );

            while ( stringTokenizer.hasMoreTokens(  ) )
            {
                if ( listMotsExclus.contains( stringTokenizer.nextToken(  ) ) )
                {
                    return false;
                }
            }
        }

        return true;
    }

    /**
     * Retourne la liste des catgories de dlibration slectionnes  partir de la requte Http
     * @param request la requte
     * @param plugin le plugin
     * @return la liste de dirctions slectionnes
     */
    protected List<CategorieDeliberation> getListeCategoriesDeliberationSelectionnees( HttpServletRequest request,
        Plugin plugin )
    {
        List<CategorieDeliberation> listeCategoriesDeliberation = new ArrayList<CategorieDeliberation>(  );

        String[] tabCategoriesDeliberation1 = request.getParameterValues( PARAM_CATEGORIES1 );
        String[] tabCategoriesDeliberation2 = request.getParameterValues( PARAM_CATEGORIES2 );

        String[] tabCategoriesDeliberation = null;

        if ( tabCategoriesDeliberation1 != null )
        {
            tabCategoriesDeliberation = tabCategoriesDeliberation1;
        }

        if ( ( tabCategoriesDeliberation2 != null ) && ( tabCategoriesDeliberation2.length > 0 ) )
        {
            tabCategoriesDeliberation = tabCategoriesDeliberation2;
        }

        if ( tabCategoriesDeliberation != null )
        {
            for ( String strCategoriesDeliberation : tabCategoriesDeliberation )
            {
                listeCategoriesDeliberation.add( _rechercheAppService.findCategorieDeliberationByPrimaryKey( 
                        Integer.parseInt( strCategoriesDeliberation ), plugin ) );
            }
        }

        return listeCategoriesDeliberation;
    }

    /**
     * Retourne la liste des gropes politiques dpositaires slectionns  partir de la requte Http
     * @param request la requte Http
     * @param plugin le plugin
     * @return la liste des groupes dpositaires slectionns
     */
    protected List<IGroupePolitique> getListeGroupesDepositairesSelectionnes( HttpServletRequest request, Plugin plugin )
    {
        List<IGroupePolitique> listeGroupesDepositaires = new ArrayList<IGroupePolitique>(  );

        String[] tabGroupesDepositaires1 = request.getParameterValues( PARAM_GROUPES1 );
        String[] tabGroupesDepositaires2 = request.getParameterValues( PARAM_GROUPES2 );

        String[] tabGroupesDepositaires = null;

        if ( tabGroupesDepositaires2 != null )
        {
            tabGroupesDepositaires = tabGroupesDepositaires2;
        }

        if ( ( tabGroupesDepositaires1 != null ) && ( tabGroupesDepositaires1.length > 0 ) )
        {
            tabGroupesDepositaires = tabGroupesDepositaires1;
        }

        if ( tabGroupesDepositaires != null )
        {
            for ( String strGroupesDepositaires : tabGroupesDepositaires )
            {
                if ( !strGroupesDepositaires.equals( OdsConstants.CONSTANTE_ENTITEE_DEPOSITAIRE_DEPOSE_PAR_EXECUTIF ) )
                {
                    listeGroupesDepositaires.add( _rechercheAppService.findGroupePolitiqueByPrimaryKey( 
                            Integer.parseInt( strGroupesDepositaires ), plugin ) );
                }
            }
        }

        return listeGroupesDepositaires;
    }

    /**
     * Retourne la liste des lus dpositaires slectionns  partir de la requte
     * @param request la requte
     * @param plugin le plugin
     * @return la liste de lus dpositaires slectionns
     */
    protected List<IElu> getListeElusDepositairesSelectionnes( HttpServletRequest request, Plugin plugin )
    {
        List<IElu> listeElusDepositaires = new ArrayList<IElu>(  );
        String[] tabElusDepositaires = request.getParameterValues( PARAM_ELUS );

        if ( tabElusDepositaires != null )
        {
            for ( String strEluDepositaire : tabElusDepositaires )
            {
                listeElusDepositaires.add( _rechercheAppService.findEluByPrimaryKey( Integer.parseInt( 
                            strEluDepositaire ), plugin ) );
            }
        }

        return listeElusDepositaires;
    }

    /**
     * Retourne true si l'item dpos par l'executif a t slectionn dans les entits dpositaires
     * @param request la requte Http
      *@return true  si l'item dpos par l'executif a t slectionn dans les entits dpositaires ,false sinon
     */
    protected boolean isDeposeParExecutifSelectionne( HttpServletRequest request )
    {
        String[] tabGroupesDepositaires1 = request.getParameterValues( PARAM_GROUPES1 );

        if ( tabGroupesDepositaires1 != null )
        {
            for ( String strGroupesDepositaires : tabGroupesDepositaires1 )
            {
                if ( strGroupesDepositaires.equals( OdsConstants.CONSTANTE_ENTITEE_DEPOSITAIRE_DEPOSE_PAR_EXECUTIF ) )
                {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Retourne la liste des directions slectionnes  partir de la requte Http
     * @param request la requte
     * @param plugin le plugin
     * @return la liste de dirctions slectionnes
     */
    protected List<Direction> getListeDirectionsSelectionnees( HttpServletRequest request, Plugin plugin )
    {
        List<Direction> listeDirections = new ArrayList<Direction>(  );
        String[] tabDirections = request.getParameterValues( PARAM_DIRECTION );

        if ( tabDirections != null )
        {
            for ( String strDirection : tabDirections )
            {
                listeDirections.add( _rechercheAppService.findDirectionByPrimaryKey( Integer.parseInt( strDirection ),
                        plugin ) );
            }
        }

        return listeDirections;
    }

    /**
     * Retourne la lsite des lus slectionns  partir de la requte
     * @param request la requte
     * @param plugin le plugin
     * @return la liste de lus slectionns
     */
    protected List<IElu> getListeRapporteursSelectionnes( HttpServletRequest request, Plugin plugin )
    {
        List<IElu> listeRapporteurs = new ArrayList<IElu>(  );
        String[] tabRapporteurs = request.getParameterValues( PARAM_RAPPORTEURS );

        if ( tabRapporteurs != null )
        {
            for ( String strRapporteur : tabRapporteurs )
            {
                listeRapporteurs.add( _rechercheAppService.findEluByPrimaryKey( Integer.parseInt( strRapporteur ),
                        plugin ) );
            }
        }

        return listeRapporteurs;
    }
}
