/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.web.xpage.search;

import fr.paris.lutece.plugins.ods.dto.categoriedeliberation.CategorieDeliberation;
import fr.paris.lutece.plugins.ods.dto.direction.Direction;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.groupepolitique.IGroupePolitique;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur;
import fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.statut.Statut;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.service.xpage.search.IRechercheArchivesAppService;
import fr.paris.lutece.plugins.ods.utils.commons.DateConvertException;
import fr.paris.lutece.plugins.ods.utils.commons.DateUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.template.AppTemplateService;
import fr.paris.lutece.portal.service.util.AppPathService;
import fr.paris.lutece.portal.service.util.AppPropertiesService;
import fr.paris.lutece.portal.web.xpages.XPage;
import fr.paris.lutece.portal.web.xpages.XPageApplication;
import fr.paris.lutece.util.html.HtmlTemplate;

import org.springframework.beans.factory.annotation.Autowired;

import java.sql.Timestamp;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.StringTokenizer;

import javax.servlet.http.HttpServletRequest;


/**
 *
 * AbstractRechercheArchivesApp
 *
 * @param <GSeance> ISeance
 * @param <GSeanceFilter> ISeanceFilter
 * @param <GRequeteUtilisateur> IRequeteUtilisateur
 * @param <GResultatRequete> IResultatRequete
 */
public abstract class AbstractRechercheArchivesApp<GSeance extends ISeance, GSeanceFilter extends ISeanceFilter, GRequeteUtilisateur extends IRequeteUtilisateur<GSeance, GSeanceFilter>, GResultatRequete extends IResultatRequete<GSeance, GSeanceFilter, GRequeteUtilisateur, GElu, GFichier, GVoeuAmendement, GPDD>, GFichier extends IFichier<GSeance, GFichier>, GElu extends IElu, GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>, GPDD extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPDD>>
    implements XPageApplication, IRechercheArchivesApp
{
    protected static final String MARK_LISTE_SELECTED_RAPPORTEURS = "liste_selected_elus_rapporteurs";
    protected static final String MARK_LISTE_SELECTED_DIRECTIONS = "liste_selected_directions";
    protected static final String MARK_LISTE_SELECTED_CATEGORIES = "liste_selected_categories";
    protected static final String MARK_LISTE_SELECTED_GROUPES = "liste_selected_groupes";
    protected static final String MARK_LISTE_SELECTED_ELUS = "liste_selected_elus";
    protected static final String MARK_LISTE_SELECTED_ARRONDISSEMENTS = "liste_selected_arrondissements";
    protected static final String MARK_VALUE_URL_FORMULAIRE = "jsp/site/Portal.jsp?page=ods_recherche_archives";
    protected static final String MARK_URL_FORMULAIRE = "url_formulaire";
    protected static final String MARK_URL_PDD = "url_pdd";
    protected static final String MARK_VALUE_URL_PDD = "jsp/site/Portal.jsp?page=ods_pdd";
    protected static final String MARK_URL_OLD_SEANCES = "url_seance";
    protected static final String MARK_VALUE_URL_OLD_SEANCE = "jsp/site/Portal.jsp?page=ods_seance_archives&id_seance=";
    protected static final String MARK_FORMATION_CONSEIL = "formation_conseil";
    protected static final String MARK_RAPPORTEURS = "rapporteurs";
    protected static final String MARK_ARRONDISSEMENTS = "arrondissements";
    protected static final String MARK_DIRECTIONS = "directions";
    protected static final String MARK_CATEGORIES_DELIBERATION = "categories_deliberation";
    protected static final String MARK_GROUPES = "groupes";
    protected static final String MARK_ELUS = "elus";
    protected static final String MARK_DATE_1 = "date_1";
    protected static final String MARK_DATE_2 = "date_2";
    protected static final String MARK_CHAMP_RECHERCHE = "champ_recherche";
    protected static final String PARAM_PROJET = "projet";
    protected static final String PARAM_DESIGNATION = "designation";
    protected static final String PARAM_PROPOSITION = "proposition";
    protected static final String PARAM_AMENDEMENT = "amendement";
    protected static final String PARAM_VOEU = "voeu";
    protected static final String PARAM_SEANCE = "seance";
    protected static final String PARAM_AUTRES = "autres";
    protected static final String PARAM_CONSEIL = "conseil";
    protected static final String PARAM_REQUETE = "id_requete";
    protected static final String PARAM_SELECTED_TYPES_DOC = "selected_types_document";
    protected static final String PARAM_SELECTED_FORMATIONS_CONSEIL = "selected_formations_conseil";
    protected static final String PARAM_SELECTED_RAPPORTEURS = "selected_rapporteurs";
    protected static final String PARAM_SELECTED_ARRONDISSEMENTS = "selected_arrondissements";
    protected static final String PARAM_SELECTED_DIRECTIONS = "selected_directions";
    protected static final String PARAM_SELECTED_CATEGORIES_DELIBERATION = "selected_categories_deliberation";
    protected static final String PARAM_SELECTED_GROUPES = "selected_groupes_depositaires";
    protected static final String PARAM_SELECTED_ELUS = "selected_elus_depositaires";
    protected static final String PARAM_DEPOSE_EXECUTIF_SELECTED = "depose_executif_selected";
    protected static final String PARAM_SELECTED_STATUTS_VOEUX_AMENDEMENTS = "selected_statuts_voeux_amendements";
    protected static final String PARAM_SELECTED_STATUTS_PROPOSITIONS = "selected_statuts_propositions";
    protected static final String PARAM_SELECTED_STATUTS_PROJETS = "selected_statuts_projets";
    protected static final String PARAM_DATE_1 = "date_1";
    protected static final String PARAM_DATE_2 = "date_2";
    protected static final String PARAM_CHAMP_RECHERCHE = "champ_recherche";
    protected static final String MARK_LISTE_RAPPORTEURS = "liste_elus_rapporteurs";
    protected static final String MARK_LISTE_DIRECTIONS = "liste_directions";
    protected static final String MARK_LISTE_CATEGORIES = "liste_categories";
    protected static final String MARK_LISTE_GROUPES = "liste_groupes";
    protected static final String MARK_LISTE_ELUS = "liste_elus";
    protected static final String MARK_LISTE_CONSEILS = "liste_conseils";
    protected static final String MARK_DEPOSE_EXECUTIF_SELECTED = "depose_executif_selected";
    protected static final String MARK_SELECTED_STATUTS_VOEUX_AMENDEMENTS = "selected_statuts_voeux_amendements";
    protected static final String MARK_SELECTED_STATUTS_PROPOSITIONS = "selected_statuts_propositions";
    protected static final String MARK_SELECTED_STATUTS_PROJETS = "selected_statuts_projets";
    protected static final String MARK_LISTE_STATUTS_PDDS = "liste_statuts_pdds";
    protected static final String MARK_LISTE_STATUTS_VOEUX_AMENDEMENTS = "liste_statuts_voeux_amendements";
    protected static final String MARK_LISTE_PROJETS = "liste_projets";
    protected static final String MARK_LISTE_PROPOSITIONS = "liste_propositions";
    protected static final String MARK_LISTE_AMENDEMENTS = "liste_amendements";
    protected static final String MARK_LISTE_DESIGNATIONS = "liste_designations";
    protected static final String MARK_LISTE_VOEUX = "liste_voeux";
    protected static final String MARK_LISTE_SEANCES = "liste_seances";
    protected static final String MARK_LISTE_AUTRES = "liste_autres";
    protected static final String MARK_NOMBRE_FICHIERS = "nombre_fichiers";
    protected static final String MARK_NOMBRE_OBJETS_DETAILLES = "nombre_objets_detailles";
    protected static final String MARK_REQUETE_UTILISATEUR = "user_search";
    protected static final String ERREUR_REQUETE_REFERENCE_NON_VALIDE = "ods.front.rechercheresultats.message.nonValidSimpleRequest";
    protected static final String ERREUR_RECHERCHE_MOT_CLE_NON_VALIDE = "ods.front.rechercheresultats.message.nonValidChampMotCle";
    protected static final String ERREUR_REQUETE_UTILISATEUR = "ods.front.mescriteres.error.noaccess";
    protected static final String TYPE_PROJET = "projet";
    protected static final String TYPE_PROPOSITION = "proposition";
    protected static final String TYPE_DESIGNATION = "designation";
    protected static final String TYPE_AMENDEMENT = "amendement";
    protected static final String TYPE_VOEU = "voeu";
    protected static final String TYPE_SEANCE = "seance";
    protected static final String TYPE_AUTRES = "autres";
    protected static final String PARAM_ARRONDISSEMENT1 = "arrondissement1";
    protected static final String PARAM_ARRONDISSEMENT2 = "arrondissement2";
    protected static final String MARK_LISTE_SELECTED_TYPES_DOCUMENT = "liste_selected_types_document";
    protected static final String PARAM_REFERENCE = "code_pdd";
    protected static final String PARAM_SEARCH = "search_input";
    protected static final String ERREUR_TROP_DE_RESULTATS = "ods.front.rechercheresultats.message.tooManyResults";
    private static final String MARK_TROP_DE_RESULTATS = "trop_de_resultats";
    private static final String MARK_IS_ARCHIVE = "isArchive";
    private static final String MARK_MESSAGE = "message";
    private static final String MARK_ERROR_MESSAGE = "error_message";
    private static final String PARAM_PAGE = "page";
    private static final String PARAM_CATEGORIES1 = "categorie1";
    private static final String PARAM_CATEGORIES2 = "categorie2";
    private static final String PARAM_DIRECTION = "direction";
    private static final String PARAM_GROUPES1 = "groupe1";
    private static final String PARAM_GROUPES2 = "groupe2";
    private static final String PARAM_RAPPORTEURS = "rapporteur";
    private static final String PARAM_STATUTS_PROJETS = "statuts_projets";
    private static final String PARAM_STATUTS_PROPOSITIONS = "statuts_propositions";
    private static final String PARAM_STATUTS_VOEUX_AMENDEDEMENTS = "statuts_voeux_amendements";
    private static final String PARAM_ELUS = "elu";
    private static final String PARAM_J_DATE1 = "j_date1";
    private static final String PARAM_M_DATE1 = "m_date1";
    private static final String PARAM_A_DATE1 = "a_date1";
    private static final String PARAM_J_DATE2 = "j_date2";
    private static final String PARAM_M_DATE2 = "m_date2";
    private static final String PARAM_A_DATE2 = "a_date2";
    private static final String PARAM_RESULT = "result";
    private static final String ERREUR_PAS_DE_SEANCE_ARCHIVEE = "ods.front.recherchearchivesresultats.message.aucuneSeanceArchivee";
    private static final String ERREUR_DATES_1_AFTER_2 = "ods.front.recherchearchivesresultats.message.date1AfterDate2";
    private static final String ERREUR_DATES_2_AFTER_CURRENT = "ods.front.recherchearchivesresultats.message.date2AfterCurrent";
    private static final String ERREUR_DATES_1_AFTER_CURRENT = "ods.front.recherchearchivesresultats.message.date1AfterCurrent";
    private static final String ERREUR_DATES_NO_FIRST = "ods.front.recherchearchivesresultats.message.noFirstDate";
    private static final String ERREUR_DATES_FORMAT_1 = "ods.front.recherchearchivesresultats.message.formatDate1";
    private static final String ERREUR_DATES_FORMAT_2 = "ods.front.recherchearchivesresultats.message.formatDate2";
    private static final int DEFAUT_MAX_NOMBRE_FICHIERS = 50;
    private static final String PROPERTY_MAX_NOMBRE_FICHIERS = "ods.search.maxNombreFichiers";
    private static final int MAX_NOMBRE_FICHIERS = AppPropertiesService.getPropertyInt( PROPERTY_MAX_NOMBRE_FICHIERS,
            DEFAUT_MAX_NOMBRE_FICHIERS );
    private static final String ODS_RECHERCHE_ARCHIVES = "ods_recherche_archives";
    @Autowired
    private IRechercheArchivesAppService<GSeance, GSeanceFilter, GRequeteUtilisateur, GResultatRequete, GFichier, GElu, GVoeuAmendement, GPDD> _rechercheArchivesAppService;

    /**
     * Renvoie la page
     * @param request la requete HTTP
     * @param nMode le mode d'affichage
     * @param plugin le plugin
     * @return XPage la page
     */
    public XPage getPage( HttpServletRequest request, int nMode, Plugin plugin )
    {
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        _rechercheArchivesAppService.gestionUtilisateur( model, request, plugin );

        model.put( OdsMarks.MARK_URL_WEBAPP, AppPathService.getBaseUrl( request ) );

        // Si il n'y a pas de sance archive, on affiche une page d'erreur
        GSeance derniereSeance = _rechercheArchivesAppService.getDerniereSeance( plugin );
        GSeance prochaineSeance = _rechercheArchivesAppService.getProchaineSeance( plugin );

        //On rcupre la sance en cours si elle existe
        GSeance seanceEnCours = _rechercheArchivesAppService.getSeanceEnCours( plugin );

        if ( seanceEnCours != null )
        {
            model.put( OdsMarks.MARK_ID_SEANCE_EN_COURS, seanceEnCours.getIdSeance(  ) );
        }

        if ( derniereSeance == null )
        {
            return getErrorPage( request, model, ERREUR_PAS_DE_SEANCE_ARCHIVEE, request.getLocale(  ) );
        }

        String strResult = request.getParameter( PARAM_RESULT );
        String strIdRequete = request.getParameter( PARAM_REQUETE );

        if ( ( ( strResult == null ) || strResult.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) ) &&
                ( strIdRequete == null ) )
        {
            return getPageFormulaire( model, request, plugin );
        }

        return getPageResultat( model, request, prochaineSeance, plugin );
    }

    /**
     * Retourne le formulaire pour la recherche dans les archives
     * @param request la requte HTTP
     * @param model le modle pour le template
     * @param plugin le plugin
     * @return la page de formulaire de recherche dans les archives
     */
    public abstract XPage getPageFormulaire( Map<String, Object> model, HttpServletRequest request, Plugin plugin );

    /**
     * Retourne la page de rsultat de la recherche dans les archives
     * @param model le modle pour le template
     * @param request la requte HTTP
     * @param prochaineSeance la prochaine sance
     * @param plugin le plugin
     * @return XPage template
     */
    public abstract XPage getPageResultat( Map<String, Object> model, HttpServletRequest request,
        GSeance prochaineSeance, Plugin plugin );

    /**
     * Retourne une page d'erreur avec le message adapt
     * @param request la requete HTTP
     * @param model le modle pour le template
     * @param strErrorMessage le message  afficher
     * @param locale la locale du navigateur
     * @return une page d'erreur
     */
    public XPage getErrorPage( HttpServletRequest request, Map<String, Object> model, String strErrorMessage,
        Locale locale )
    {
        XPage page = new XPage(  );

        if ( strErrorMessage.equals( ERREUR_TROP_DE_RESULTATS ) )
        {
            String[] listArguments = { OdsConstants.CONSTANTE_CHAINE_VIDE + MAX_NOMBRE_FICHIERS };
            String strErrorMsg = I18nService.getLocalizedString( ERREUR_TROP_DE_RESULTATS, listArguments, locale );
            model.put( MARK_ERROR_MESSAGE, strErrorMsg );
            model.put( MARK_TROP_DE_RESULTATS, true );
        }
        else
        {
            model.put( MARK_MESSAGE, strErrorMessage );
        }

        String strPage = request.getParameter( PARAM_PAGE );

        if ( ( strPage != null ) && strPage.equals( ODS_RECHERCHE_ARCHIVES ) )
        {
            model.put( MARK_IS_ARCHIVE, true );
        }

        HtmlTemplate template = AppTemplateService.getTemplate( _rechercheArchivesAppService.getTemplateRechercheNonValide(  ),
                locale, model );

        String strPageTitle = I18nService.getLocalizedString( _rechercheArchivesAppService.getPropertyPageErreurTitle(  ),
                locale );
        String strPagePathLabel = I18nService.getLocalizedString( _rechercheArchivesAppService.getPropertyPathLabel(  ),
                locale );
        page.setTitle( strPageTitle );
        page.setContent( template.getHtml(  ) );
        page.setPathLabel( strPagePathLabel );

        return page;
    }

    /**
     * Prpare l'objet RequeteUtilisateur avec les paramtres pour une recherche par mot-cl.
     * Si l'utilisateur dcide de l'enregistrer l'objet sera ainsi dj cr ( transmettre en attribut de la requte)
     * @param request la requte
     * @param requeteUtilisateur la requete de l'utilisateur
     */
    protected void prepareRequeteUtilisateurReference( GRequeteUtilisateur requeteUtilisateur,
        HttpServletRequest request )
    {
        requeteUtilisateur.setRechercheArchive( true );
        requeteUtilisateur.setTypeRequete( OdsParameters.REFERENCE );
        requeteUtilisateur.setChampRecherche( request.getParameter( PARAM_REFERENCE ).trim(  ) );
        requeteUtilisateur.setNotifie( false );
    }

    /**
     * Indique si une requte d'une recherche par rfrence de code de PDD est valide,
     * c'est  dire que le champ de recherche est rempli (et que le format/template est bon?)
     * @param request la requte
     * @return <b>true</b> si la requte est valide, <b>false</b> sinon
     */
    protected boolean isReferenceValideForm( HttpServletRequest request )
    {
        return ( ( request.getParameter( PARAM_REFERENCE ) != null ) &&
        !request.getParameter( PARAM_REFERENCE ).trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) );
    }

    /**
     * Vrifie que le champ de recherche par mot-cl ne contient pas de caractre spcial qui altrerait le
     * rsultat de la recherche dans l'index Lucene
     * @param request la requte
     * @return <b>true</b> si le champ de recherche est correct, <b>false</b> sinon
     */
    protected boolean isChampMotCleValide( HttpServletRequest request )
    {
        String strRequete = request.getParameter( PARAM_SEARCH );

        if ( ( strRequete != null ) && !strRequete.trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            strRequete = strRequete.trim(  );

            if ( ( strRequete.indexOf( " -" ) > -1 ) || strRequete.startsWith( "-" ) )
            {
                return false;
            }

            String strDelimiteurs = " &~|!*?+(){}[]^:\"";
            String[] tabMotsExclus = 
                {
                    "&", "|", "~", "!", "*", "?", "+", "(", ")", "{", "}", "[", "]", "^", ":", "\"", "\\", "AND", "and",
                    "OR", "or",
                };
            List<String> listMotsExclus = new ArrayList<String>(  );

            for ( String strMotExclu : tabMotsExclus )
            {
                listMotsExclus.add( strMotExclu );
            }

            StringTokenizer stringTokenizer = new StringTokenizer( strRequete, strDelimiteurs, true );

            while ( stringTokenizer.hasMoreTokens(  ) )
            {
                if ( listMotsExclus.contains( stringTokenizer.nextToken(  ) ) )
                {
                    return false;
                }
            }
        }

        return true;
    }

    /**
     * Retourne la liste des catgories de dlibration slectionnes  partir de la requte Http
     * @param request la requte
     * @param plugin le plugin
     * @return la liste de dirctions slectionnes
     */
    protected List<CategorieDeliberation> getListeCategoriesDeliberationSelectionnees( HttpServletRequest request,
        Plugin plugin )
    {
        List<CategorieDeliberation> listeCategoriesDeliberation = new ArrayList<CategorieDeliberation>(  );

        String[] tabCategoriesDeliberation1 = request.getParameterValues( PARAM_CATEGORIES1 );
        String[] tabCategoriesDeliberation2 = request.getParameterValues( PARAM_CATEGORIES2 );

        String[] tabCategoriesDeliberation = null;

        if ( tabCategoriesDeliberation1 != null )
        {
            tabCategoriesDeliberation = tabCategoriesDeliberation1;
        }

        if ( ( tabCategoriesDeliberation2 != null ) && ( tabCategoriesDeliberation2.length > 0 ) )
        {
            tabCategoriesDeliberation = tabCategoriesDeliberation2;
        }

        if ( tabCategoriesDeliberation != null )
        {
            for ( String strCategoriesDeliberation : tabCategoriesDeliberation )
            {
                listeCategoriesDeliberation.add( _rechercheArchivesAppService.findCategorieDeliberationByPrimaryKey( 
                        Integer.parseInt( strCategoriesDeliberation ), plugin ) );
            }
        }

        return listeCategoriesDeliberation;
    }

    /**
     * Retourne la liste des directions slectionnes  partir de la requte Http
     * @param request la requte
     * @param plugin le plugin
     * @return la liste de dirctions slectionnes
     */
    protected List<Direction> getListeDirectionsSelectionnees( HttpServletRequest request, Plugin plugin )
    {
        List<Direction> listeDirections = new ArrayList<Direction>(  );
        String[] tabDirections = request.getParameterValues( PARAM_DIRECTION );

        if ( tabDirections != null )
        {
            for ( String strDirection : tabDirections )
            {
                listeDirections.add( _rechercheArchivesAppService.findDirectionByPrimaryKey( Integer.parseInt( 
                            strDirection ), plugin ) );
            }
        }

        return listeDirections;
    }

    /**
     * Retourne la liste des gropes politiques dpositaires slectionns  partir de la requte Http
     * @param request la requte Http
     * @param plugin le plugin
     * @return la liste des groupes dpositaires slectionns
     */
    protected List<IGroupePolitique> getListeGroupesDepositairesSelectionnes( HttpServletRequest request, Plugin plugin )
    {
        List<IGroupePolitique> listeGroupesDepositaires = new ArrayList<IGroupePolitique>(  );

        String[] tabGroupesDepositaires1 = request.getParameterValues( PARAM_GROUPES1 );
        String[] tabGroupesDepositaires2 = request.getParameterValues( PARAM_GROUPES2 );

        String[] tabGroupesDepositaires = null;

        if ( tabGroupesDepositaires2 != null )
        {
            tabGroupesDepositaires = tabGroupesDepositaires2;
        }

        if ( ( tabGroupesDepositaires1 != null ) && ( tabGroupesDepositaires1.length > 0 ) )
        {
            tabGroupesDepositaires = tabGroupesDepositaires1;
        }

        if ( tabGroupesDepositaires != null )
        {
            for ( String strGroupesDepositaires : tabGroupesDepositaires )
            {
                if ( !strGroupesDepositaires.equals( OdsConstants.CONSTANTE_ENTITEE_DEPOSITAIRE_DEPOSE_PAR_EXECUTIF ) )
                {
                    listeGroupesDepositaires.add( _rechercheArchivesAppService.findGroupePolitiqueByPrimaryKey( 
                            Integer.parseInt( strGroupesDepositaires ), plugin ) );
                }
            }
        }

        return listeGroupesDepositaires;
    }

    /**
     * Retourne la lsite des lus slectionns  partir de la requte
     * @param request la requte
     * @param plugin le plugin
     * @return la liste de lus slectionns
     */
    protected List<IElu> getListeRapporteursSelectionnes( HttpServletRequest request, Plugin plugin )
    {
        List<IElu> listeRapporteurs = new ArrayList<IElu>(  );
        String[] tabRapporteurs = request.getParameterValues( PARAM_RAPPORTEURS );

        if ( tabRapporteurs != null )
        {
            for ( String strRapporteur : tabRapporteurs )
            {
                listeRapporteurs.add( _rechercheArchivesAppService.findEluByPrimaryKey( Integer.parseInt( strRapporteur ),
                        plugin ) );
            }
        }

        return listeRapporteurs;
    }

    /**
     * Retourne true si l'item dpos par l'executif a t slectionn dans les entits dpositaires
     * @param request la requte Http
      *@return true  si l'item dpos par l'executif a t slectionn dans les entits dpositaires ,false sinon
     */
    protected boolean isDeposeParExecutifSelectionne( HttpServletRequest request )
    {
        String[] tabGroupesDepositaires1 = request.getParameterValues( PARAM_GROUPES1 );

        if ( tabGroupesDepositaires1 != null )
        {
            for ( String strGroupesDepositaires : tabGroupesDepositaires1 )
            {
                if ( strGroupesDepositaires.equals( OdsConstants.CONSTANTE_ENTITEE_DEPOSITAIRE_DEPOSE_PAR_EXECUTIF ) )
                {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Retourne la liste des lus dpositaires slectionns  partir de la requte
     * @param request la requte
     * @param plugin le plugin
     * @return la liste de lus dpositaires slectionns
     */
    protected List<IElu> getListeElusDepositairesSelectionnes( HttpServletRequest request, Plugin plugin )
    {
        List<IElu> listeElusDepositaires = new ArrayList<IElu>(  );
        String[] tabElusDepositaires = request.getParameterValues( PARAM_ELUS );

        if ( tabElusDepositaires != null )
        {
            for ( String strEluDepositaire : tabElusDepositaires )
            {
                listeElusDepositaires.add( _rechercheArchivesAppService.findEluByPrimaryKey( Integer.parseInt( 
                            strEluDepositaire ), plugin ) );
            }
        }

        return listeElusDepositaires;
    }

    /**
     * Retourne la liste des arrondissements slectionns  partir de la requte
     * @param request la requte
     * @return la liste des arrondissements slectionns
     */
    protected int[] getListeArrondissementSelectionnes( HttpServletRequest request )
    {
        String[] tabArrondissement1 = request.getParameterValues( PARAM_ARRONDISSEMENT1 );
        String[] tabArrondissement2 = request.getParameterValues( PARAM_ARRONDISSEMENT2 );

        String[] tabArrondissement = null;

        if ( tabArrondissement1 != null )
        {
            tabArrondissement = tabArrondissement1;
        }

        if ( ( tabArrondissement2 != null ) && ( tabArrondissement2.length > 0 ) )
        {
            tabArrondissement = tabArrondissement2;
        }

        int[] listeArrondissement = new int[0];

        if ( tabArrondissement != null )
        {
            listeArrondissement = new int[tabArrondissement.length];

            int nIndex = 0;

            for ( String strArrondissement : tabArrondissement )
            {
                listeArrondissement[nIndex++] = Integer.parseInt( strArrondissement );
            }
        }

        return listeArrondissement;
    }

    /**
     * Retourne la liste des statuts slectionns  pour les projets  partir de la requte
     * @param request la requte
     * @param plugin le plugin
     * @return la liste des statuts slectionns pour les projets
     */
    protected List<Statut> getListeStatutsProjetsSelectionnes( HttpServletRequest request, Plugin plugin )
    {
        List<Statut> listeStatutsProjets = new ArrayList<Statut>(  );
        String[] tabStatusProjets = request.getParameterValues( PARAM_STATUTS_PROJETS );

        if ( tabStatusProjets != null )
        {
            for ( String strStatut : tabStatusProjets )
            {
                listeStatutsProjets.add( _rechercheArchivesAppService.findStatutByPrimaryKey( Integer.parseInt( 
                            strStatut ), plugin ) );
            }
        }

        return listeStatutsProjets;
    }

    /**
     * Retourne la liste des statuts slectionns pour les propositions  partir de la requte
     * @param request la requte
     * @param plugin le plugin
     * @return la liste des statuts dslectionns pour les propositions
     */
    protected List<Statut> getListeStatutsPropositionsSelectionnes( HttpServletRequest request, Plugin plugin )
    {
        List<Statut> listeStatutsPropositions = new ArrayList<Statut>(  );
        String[] tabStatusPropositions = request.getParameterValues( PARAM_STATUTS_PROPOSITIONS );

        if ( tabStatusPropositions != null )
        {
            for ( String strStatut : tabStatusPropositions )
            {
                listeStatutsPropositions.add( _rechercheArchivesAppService.findStatutByPrimaryKey( Integer.parseInt( 
                            strStatut ), plugin ) );
            }
        }

        return listeStatutsPropositions;
    }

    /**
     * Retourne la liste des statuts slectionns pour les voeux et amendements  partir de la requte
     * @param request la requte
     * @param plugin le plugin
     * @return la liste des statuts slectionns pour les voeux et amendements  partir de la requte
     */
    protected List<Statut> getListeStatutsVoeuxAmendementsSelectionnes( HttpServletRequest request, Plugin plugin )
    {
        List<Statut> listeStatutsVoeuxAmendements = new ArrayList<Statut>(  );
        String[] tabStatusPropositions = request.getParameterValues( PARAM_STATUTS_VOEUX_AMENDEDEMENTS );

        if ( tabStatusPropositions != null )
        {
            for ( String strStatut : tabStatusPropositions )
            {
                listeStatutsVoeuxAmendements.add( _rechercheArchivesAppService.findStatutByPrimaryKey( Integer.parseInt( 
                            strStatut ), plugin ) );
            }
        }

        return listeStatutsVoeuxAmendements;
    }

    /**
     * Teste les champs dates rentrs par l'utilisateur et renseigne les champs de la requte
     * @param request la requte Http
     * @param requeteUtilisateur la requete de l'utilisateur
     * @throws DateConvertException si l'utilisateur fait une opration non permise:<br>
     *                 si la date de fin est saisie alors la date de dbut doit tre saisie<br>
     *                 les dates doivent tre au format jj/mm/aaaa (en numrique)
     */
    protected void prepareDates( GRequeteUtilisateur requeteUtilisateur, HttpServletRequest request )
        throws DateConvertException
    {
        String strDateComplete1;
        String strDateComplete2;
        Timestamp tsDate1 = null;
        Timestamp tsDate2 = null;

        String strDateJ1 = request.getParameter( PARAM_J_DATE1 );
        String strDateM1 = request.getParameter( PARAM_M_DATE1 );
        String strDateA1 = request.getParameter( PARAM_A_DATE1 );
        String strDateJ2 = request.getParameter( PARAM_J_DATE2 );
        String strDateM2 = request.getParameter( PARAM_M_DATE2 );
        String strDateA2 = request.getParameter( PARAM_A_DATE2 );

        if ( ( ( strDateJ2 != null ) && ( strDateJ2 != OdsConstants.CONSTANTE_CHAINE_VIDE ) ) ||
                ( ( strDateM2 != null ) && ( strDateM2 != OdsConstants.CONSTANTE_CHAINE_VIDE ) ) ||
                ( ( strDateA2 != null ) && ( strDateA2 != OdsConstants.CONSTANTE_CHAINE_VIDE ) ) )
        {
            if ( ( ( strDateJ1 == null ) || ( strDateJ1 == OdsConstants.CONSTANTE_CHAINE_VIDE ) ) &&
                    ( ( strDateM1 == null ) || ( strDateM1 == OdsConstants.CONSTANTE_CHAINE_VIDE ) ) &&
                    ( ( strDateA1 == null ) || ( strDateA1 == OdsConstants.CONSTANTE_CHAINE_VIDE ) ) )
            {
                throw new DateConvertException( ERREUR_DATES_NO_FIRST );
            }

            strDateComplete1 = strDateJ1 + "/" + strDateM1 + "/" + strDateA1;
            tsDate1 = DateUtils.getDate( strDateComplete1, true );

            if ( tsDate1 == null )
            {
                throw new DateConvertException( ERREUR_DATES_FORMAT_1 );
            }

            strDateComplete2 = strDateJ2 + "/" + strDateM2 + "/" + strDateA2;
            tsDate2 = DateUtils.getDate( strDateComplete2, true );

            if ( tsDate2 == null )
            {
                requeteUtilisateur.setPremiereDate( tsDate1 );
                throw new DateConvertException( ERREUR_DATES_FORMAT_2 );
            }

            if ( !tsDate2.before( DateUtils.getCurrentDate(  ) ) )
            {
                requeteUtilisateur.setPremiereDate( tsDate1 );
                requeteUtilisateur.setDeuxiemeDate( tsDate2 );
                throw new DateConvertException( ERREUR_DATES_2_AFTER_CURRENT );
            }

            if ( tsDate2.before( tsDate1 ) )
            {
                requeteUtilisateur.setPremiereDate( tsDate1 );
                requeteUtilisateur.setDeuxiemeDate( tsDate2 );
                throw new DateConvertException( ERREUR_DATES_1_AFTER_2 );
            }
        }
        else
        {
            if ( ( ( strDateJ1 != null ) && ( strDateJ1 != OdsConstants.CONSTANTE_CHAINE_VIDE ) ) ||
                    ( ( strDateM1 != null ) && ( strDateM1 != OdsConstants.CONSTANTE_CHAINE_VIDE ) ) ||
                    ( ( strDateA1 != null ) && ( strDateA1 != OdsConstants.CONSTANTE_CHAINE_VIDE ) ) )
            {
                strDateComplete1 = strDateJ1 + "/" + strDateM1 + "/" + strDateA1;
                tsDate1 = DateUtils.getDate( strDateComplete1, true );

                if ( tsDate1 == null )
                {
                    throw new DateConvertException( ERREUR_DATES_FORMAT_1 );
                }

                if ( !tsDate1.before( DateUtils.getCurrentDate(  ) ) )
                {
                    requeteUtilisateur.setPremiereDate( tsDate1 );
                    throw new DateConvertException( ERREUR_DATES_1_AFTER_CURRENT );
                }
            }
        }

        requeteUtilisateur.setPremiereDate( tsDate1 );
        requeteUtilisateur.setDeuxiemeDate( tsDate2 );
    }
}
