/*
 * Copyright (c) 2002-2010, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.ods.service.voeuamendement;

import fr.paris.lutece.plugins.ods.business.acte.ActeHome;
import fr.paris.lutece.plugins.ods.business.acte.NatureActeHome;
import fr.paris.lutece.plugins.ods.business.acte.StatutActeHome;
import fr.paris.lutece.plugins.ods.business.categoriedeliberation.ICategorieDeliberationHome;
import fr.paris.lutece.plugins.ods.business.elu.IEluHome;
import fr.paris.lutece.plugins.ods.business.fichier.FichierPhysiqueHome;
import fr.paris.lutece.plugins.ods.business.fichier.IFichierHome;
import fr.paris.lutece.plugins.ods.business.groupepolitique.GroupePolitiqueHome;
import fr.paris.lutece.plugins.ods.business.historique.HistoriqueHome;
import fr.paris.lutece.plugins.ods.business.ordredujour.IEntreeOrdreDuJourHome;
import fr.paris.lutece.plugins.ods.business.ordredujour.IOrdreDuJourHome;
import fr.paris.lutece.plugins.ods.business.panier.PanierHome;
import fr.paris.lutece.plugins.ods.business.pdd.IPDDHome;
import fr.paris.lutece.plugins.ods.business.seance.ISeanceHome;
import fr.paris.lutece.plugins.ods.business.statut.StatutHome;
import fr.paris.lutece.plugins.ods.business.voeuamendement.IVoeuAmendementHome;
import fr.paris.lutece.plugins.ods.dto.IBusinessItem;
import fr.paris.lutece.plugins.ods.dto.IDepositaire;
import fr.paris.lutece.plugins.ods.dto.acte.Acte;
import fr.paris.lutece.plugins.ods.dto.acte.IActeFilter;
import fr.paris.lutece.plugins.ods.dto.acte.NatureActe;
import fr.paris.lutece.plugins.ods.dto.acte.NatureActeEnum;
import fr.paris.lutece.plugins.ods.dto.acte.StatutActe;
import fr.paris.lutece.plugins.ods.dto.acte.StatutActeEnum;
import fr.paris.lutece.plugins.ods.dto.categoriedeliberation.CategorieDeliberation;
import fr.paris.lutece.plugins.ods.dto.certificataffichage.IEntiteCertificatAffichage;
import fr.paris.lutece.plugins.ods.dto.elu.IElu;
import fr.paris.lutece.plugins.ods.dto.fichier.FichierPhysique;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichier;
import fr.paris.lutece.plugins.ods.dto.fichier.IFichierFilter;
import fr.paris.lutece.plugins.ods.dto.fichier.ITypeDocument;
import fr.paris.lutece.plugins.ods.dto.groupepolitique.GroupePolitique;
import fr.paris.lutece.plugins.ods.dto.groupepolitique.IGroupePolitique;
import fr.paris.lutece.plugins.ods.dto.historique.Historique;
import fr.paris.lutece.plugins.ods.dto.horodatage.HorodatageActionEnum;
import fr.paris.lutece.plugins.ods.dto.horodatage.IEntiteHorodatage;
import fr.paris.lutece.plugins.ods.dto.ordredujour.EntreeOrdreDuJourFilter;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IEntreeOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJour;
import fr.paris.lutece.plugins.ods.dto.ordredujour.IOrdreDuJourFilter;
import fr.paris.lutece.plugins.ods.dto.pdd.IArrondissement;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDD;
import fr.paris.lutece.plugins.ods.dto.pdd.IPDDFilter;
import fr.paris.lutece.plugins.ods.dto.pdd.IPddFormBean;
import fr.paris.lutece.plugins.ods.dto.requete.IRequeteUtilisateur;
import fr.paris.lutece.plugins.ods.dto.requete.IResultatRequete;
import fr.paris.lutece.plugins.ods.dto.seance.ISeance;
import fr.paris.lutece.plugins.ods.dto.seance.ISeanceFilter;
import fr.paris.lutece.plugins.ods.dto.statut.IStatutFilter;
import fr.paris.lutece.plugins.ods.dto.statut.Statut;
import fr.paris.lutece.plugins.ods.dto.statut.StatutEnum;
import fr.paris.lutece.plugins.ods.dto.typeentite.ITypeEntite;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendement;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.IVoeuAmendementFilter;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.VoeuAmendementCertificatComparator;
import fr.paris.lutece.plugins.ods.dto.voeuamendement.VoeuAmendementComparator;
import fr.paris.lutece.plugins.ods.service.acte.IActeService;
import fr.paris.lutece.plugins.ods.service.categoriedeliberation.ICategorieDeliberationService;
import fr.paris.lutece.plugins.ods.service.direction.IDirectionService;
import fr.paris.lutece.plugins.ods.service.groupepolitique.IGroupePolitiqueService;
import fr.paris.lutece.plugins.ods.service.horodatage.IHorodatageService;
import fr.paris.lutece.plugins.ods.service.ordredujour.IOrdreDuJourService;
import fr.paris.lutece.plugins.ods.service.pdd.IProjetDeDeliberationService;
import fr.paris.lutece.plugins.ods.service.role.ActesResourceIdService;
import fr.paris.lutece.plugins.ods.service.role.VoeuxAmendementsResourceIdService;
import fr.paris.lutece.plugins.ods.service.search.ISearchService;
import fr.paris.lutece.plugins.ods.utils.business.OdsUtils;
import fr.paris.lutece.plugins.ods.utils.commons.DateUtils;
import fr.paris.lutece.plugins.ods.utils.commons.ShaUtils;
import fr.paris.lutece.plugins.ods.utils.commons.StringUtils;
import fr.paris.lutece.plugins.ods.utils.constants.OdsConstants;
import fr.paris.lutece.plugins.ods.utils.constants.OdsMarks;
import fr.paris.lutece.plugins.ods.utils.constants.OdsParameters;
import fr.paris.lutece.plugins.ods.utils.constants.OdsProperties;
import fr.paris.lutece.portal.business.rbac.RBAC;
import fr.paris.lutece.portal.business.user.AdminUser;
import fr.paris.lutece.portal.service.i18n.I18nService;
import fr.paris.lutece.portal.service.message.AdminMessage;
import fr.paris.lutece.portal.service.message.AdminMessageService;
import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.rbac.RBACService;
import fr.paris.lutece.portal.service.util.AppException;
import fr.paris.lutece.portal.service.util.AppLogService;
import fr.paris.lutece.portal.service.util.AppPathService;
import fr.paris.lutece.portal.service.util.AppPropertiesService;
import fr.paris.lutece.portal.web.constants.Messages;
import fr.paris.lutece.portal.web.upload.MultipartHttpServletRequest;
import fr.paris.lutece.util.ReferenceList;
import fr.paris.lutece.util.html.Paginator;
import fr.paris.lutece.util.url.UrlItem;

import org.apache.commons.fileupload.FileItem;

import org.springframework.beans.factory.annotation.Autowired;

import java.sql.SQLException;
import java.sql.Timestamp;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;

import javax.servlet.http.HttpServletRequest;


/**
 * AbstractVoeuAmendementService
 */
public abstract class AbstractVoeuAmendementService<GVoeuAmendement extends IVoeuAmendement<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>, GVoeuAmendementFilter extends IVoeuAmendementFilter, GVoeuAmendementHome extends IVoeuAmendementHome<GVoeuAmendement, GVoeuAmendementFilter, GSeance, GElu, GFichier, GPdd>, GEntreeOrdreDuJour extends IEntreeOrdreDuJour<GElu, GSeance, GFichier, GEntreeOrdreDuJour, GOrdreDuJour>, GEntreeOrdreDuJourHome extends IEntreeOrdreDuJourHome<GOrdreDuJour, GEntreeOrdreDuJour, GElu, GSeance, GFichier>, GOrdreDuJour extends IOrdreDuJour<GEntreeOrdreDuJour, GSeance, GFichier, GElu, GOrdreDuJour>, GOrdreDuJourFilter extends IOrdreDuJourFilter, GOrdreDuJourHome extends IOrdreDuJourHome<GOrdreDuJour, GEntreeOrdreDuJour, GOrdreDuJourFilter, GSeance, GElu, GFichier>, GOrdreDuJourService extends IOrdreDuJourService<GOrdreDuJour, GEntreeOrdreDuJour, GOrdreDuJourFilter, GSeance, GElu, GFichier>, GSeance extends ISeance, GElu extends IElu, GFichier extends IFichier<GSeance, GFichier>, GSeanceFilter extends ISeanceFilter, GActeFilter extends IActeFilter<GFichier, GSeance>, GPdd extends IPDD<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>, GArrondissement extends IArrondissement, GPDDFormBean extends IPddFormBean<GSeance, GFichier, GElu, GVoeuAmendement, GPdd, GArrondissement>, GPDDFilter extends IPDDFilter, GRequeteUtilisateur extends IRequeteUtilisateur<GSeance, GSeanceFilter>, GResultatRequete extends IResultatRequete<GSeance, GSeanceFilter, GRequeteUtilisateur, GElu, GFichier, GVoeuAmendement, GPdd>>
    implements IVoeuAmendementService<GVoeuAmendement, GOrdreDuJour, GEntreeOrdreDuJour, GSeance, GElu, GFichier, GPdd>
{
    protected static final String CONSTANTE_TYPE_STATUT_VA = "STATUTVA";
    protected static final String CONSTANTE_ACTION_SAVE = "save";
    private static final String CONSTANTE_ACTION_ADD_PARENT = "add_parent";
    private static final String CONSTANTE_ACTION_STATUT_VA = "statutva";
    private static final String CONSTANTE_ACTION_REMOVE_PARENT = "remove_parent";
    private static final String CONSTANTE_RETOUR_CREATION = "creation";
    private static final String CONSTANTE_RETOUR_MODIFICATION = "modification";
    private static final String CONSTANTE_URL_JSP_APPELANT_AMENDEMENT_PARENT_MODIFICATION = "jsp/admin/plugins/ods/voeuamendement/AmendementsForParentModification.jsp";
    private static final String CONSTANTE_URL_JSP_APPELANT_AMENDEMENT_PARENT_CREATION = "jsp/admin/plugins/ods/voeuamendement/AmendementsForParentCreation.jsp";
    private static final String CONSTANTE_URL_JSP_APPELANT_VOEU_RATTACHE_PARENT_MODIFICATION = "jsp/admin/plugins/ods/voeuamendement/VoeuxRattachesForParentModification.jsp";
    private static final String CONSTANTE_URL_JSP_APPELANT_VOEU_RATTACHE_PARENT_CREATION = "jsp/admin/plugins/ods/voeuamendement/VoeuxRattachesForParentCreation.jsp";
    private static final String CONSTANTE_URL_JSP_APPELANT_VOEU_NON_RATTACHE_PARENT_MODIFICATION = "jsp/admin/plugins/ods/voeuamendement/VoeuxNonRattachesForParentModification.jsp";
    private static final String CONSTANTE_URL_JSP_APPELANT_VOEU_NON_RATTACHE_PARENT_CREATION = "jsp/admin/plugins/ods/voeuamendement/VoeuxNonRattachesForParentCreation.jsp";
    private static final String CONSTANTE_CANCEL = "cancel";
    private static final String CONSTANTE_CREATION_FICHIER_INTITULE = "Texte Dpos";
    private static final String CONSTANTE_FIN_INTITULE_FICHIER_DELIBERATION = "dlibration";
    private static final String CONSTANTE_PAGE_FIRST = "1";
    private static final String CONSTANTE_AJOUTER_VOEU = "ajouter_voeu";
    private static final int CONSTANTE_CODE_CATEGORIE_VOEUX = 60;
    private static final String CONSTANTE_ORDREDUJOUR = "ordredujour";
    private static final String CONSTANTE_URL_JSP_APPELANT_PROJET = "jsp/admin/plugins/ods/projetdeliberation/ProjetDeliberationForVa.jsp";
    private static final String CONSTANTE_URL_JSP_APPELANT_PROJET_AVAL = "jsp/admin/plugins/ods/projetdeliberation/ProjetDeliberationForVaGestionAval.jsp";
    private static final String CONSTANTE_URL_JSP_APPELANT_PROPOSITION = "jsp/admin/plugins/ods/projetdeliberation/PropositionDeliberationForVa.jsp";
    private static final String CONSTANTE_URL_JSP_APPELANT_PROPOSITION_AVAL = "jsp/admin/plugins/ods/projetdeliberation/PropositionDeliberationForVaGestionAval.jsp";
    protected static final String JSP_URL_GET_STATUT_VA = "jsp/admin/plugins/ods/modules/cp/voeuamendement/StatutVAGestionAvalNoReinit.jsp";
    private static final String JSP_URL_GET_MODIFICATION_VOEU_FOR_ACTE = "jsp/admin/plugins/ods/voeuamendement/ModificationVoeuForActe.jsp";
    private static final String JSP_URL_GET_MODIFICATION_VOEU = "jsp/admin/plugins/ods/voeuamendement/ModificationVoeu.jsp";
    private static final String JSP_URL_GET_VISUALISATION_VOEU_FOR_ACTE = "jsp/admin/plugins/ods/voeuamendement/VisualisationVoeuForActe.jsp";
    private static final String JSP_URL_DO_SUPPRESSIONAMENDEMENT = "jsp/admin/plugins/ods/voeuamendement/DoSuppressionAmendement.jsp";
    private static final String JSP_URL_DO_SUPPRESSIONVOEU = "jsp/admin/plugins/ods/voeuamendement/DoSuppressionVoeu.jsp";
    private static final String JSP_URL_GET_MODIFICATION_AMENDEMENT = "jsp/admin/plugins/ods/voeuamendement/ModificationAmendement.jsp";
    private static final String JSP_URL_GET_CREATION_AMENDEMENT = "jsp/admin/plugins/ods/voeuamendement/CreationAmendementNoReinit.jsp";
    private static final String JSP_URL_GET_AMENDEMENTS_LIST = "jsp/admin/plugins/ods/voeuamendement/AmendementsNoReinit.jsp";
    private static final String JSP_URL_GET_AMENDEMENTS_LIST_GESTION_AVAL = "jsp/admin/plugins/ods/voeuamendement/AmendementsGestionAvalNoReinit.jsp";
    private static final String JSP_URL_GET_CREATION_VOEU = "jsp/admin/plugins/ods/voeuamendement/CreationVoeuNoReinit.jsp";
    private static final String JSP_URL_GET_VOEUX_LIST_GESTION_AVAL = "jsp/admin/plugins/ods/voeuamendement/VoeuxGestionAvalNoReinit.jsp";
    private static final String JSP_URL_GET_VOEUX_LIST = "jsp/admin/plugins/ods/voeuamendement/VoeuxNoReinit.jsp";
    private static final String JSP_URL_GET_MODIFICATION_STATUT_VA = "jsp/admin/plugins/ods/modules/cp/voeuamendement/ModificationStatutVA.jsp";
    private static final String JSP_URL_GET_AVERTISSEMENT_ENREGISTREMENT_VOEU_RATTACHE = "jsp/admin/plugins/ods/voeuamendement/AvertissementEnregistrementVoeuRattache.jsp";
    private static final String JSP_URL_GET_AVERTISSEMENT_ENREGISTREMENT_AMENDEMENT = "jsp/admin/plugins/ods/voeuamendement/AvertissementEnregistrementAmendement.jsp";
    private static final String JSP_URL_FINALISATION_ACTE = "jsp/admin/plugins/ods/voeuamendement/FinalisationActe.jsp";
    private static final String JSP_RETOUR_PDDS_IN_VOEURATTACHE = "jsp/admin/plugins/ods/voeuamendement/AddPddsInVoeuRattache.jsp";
    private static final String JSP_RETOUR_PDDS_IN_AMENDEMENT = "jsp/admin/plugins/ods/voeuamendement/AddPddsInAmendement.jsp";
    protected static final String MESSAGE_SESSION_EXPIREE = "ods.voeuamendement.message.sessionExpiree";
    private static final String MESSAGE_CONFIRMDELETEVOEU = "ods.message.confirmdeletevoeu";
    private static final String MESSAGE_CONFIRMDELETEVOEUINSCRITODJ = "ods.message.confirmdeletevoeuinscritodj";
    private static final String MESSAGE_CANTDELETEVOEUINSCRITODJPUBLIE = "ods.message.cantdeletevoeuinscritodj";
    private static final String MESSAGE_ERROR_DELETE_VOEU = "ods.message.errorDeleteVoeu";
    private static final String MESSAGE_CONFIRM_DELETE_VOEU_WITH_ACTES_ANNULES = "ods.message.confirmDeleteVoeuWithActes";
    private static final String MESSAGE_PAS_DE_PROJET_CHOISIE = "ods.voeuamendement.message.pasdeprojetchoisie";
    private static final String MESSAGE_PAS_DE_PROPOSITION_CHOISIE = "ods.voeuamendement.message.pasdepropositionchoisie";
    private static final String MESSAGE_LABEL_FICHIER_NO_PDF = "ods.voeuamendement.message.fichier.nopdf";
    private static final String MESSAGE_CONFIRMDELETEAMENDEMENT = "ods.message.confirmdeleteamendement";
    private static final String MESSAGE_AVERTISSEMENT_ENREGISTREMENT = "ods.voeuamendement.message.avertissementEnregistrement";
    private static final String MESSAGE_AVERTISSEMENT_PUBLICATION = "ods.voeuamendement.message.avertissementPublication";
    private static final String MESSAGE_REFERENCE_TOO_LONG = "ods.voeuamendement.message.refTooLong";
    private static final String MESSAGE_NO_PARENT_SELECTED = "ods.voeuamendement.message.noparentselected";
    private static final String MESSAGE_CANNOT_DELETE_VA = "ods.voeuamendement.message.cannotdelete";
    private static final String MESSAGE_PAS_DE_VOEU_CHOISI = "ods.entreeodj.message.pasdevoeuchoisi";
    private static final String MESSAGE_EXISTE_ELEMENT_PAS_PUBLIE = "ods.odj.message.elementPasPublie";
    private static final String PROPERTY_TITLE_CREATION_ENTREE_ODJ = "ods.odj.creation.label.modificationEntreeVNR";
    private static final String PROPERTY_TITLE_MODIFICATION_ENTREE_ODJ = "ods.odj.creation.label.modificationEntreeVNR";
    protected static final String MARK_LISTE_SEANCES = "liste_seances";
    protected static final String MARK_ID_SEANCE_SELECTED = "id_seance_selected";
    protected static final String MARK_LISTE_STATUTS = "liste_statuts";
    protected static final String MARK_ID_STATUT_SELECTED = "id_statut_selected";
    protected static final String MARK_DEPOSITAIRE_TYPE_ELU = "depositaire_type_elu";
    protected static final String MARK_DEPOSITAIRE_TYPE_GROUPE = "depositaire_type_groupe";
    protected static final String MARK_LISTE_VOEUAMENDEMENT = "liste_voeuamendement";
    private static final String MARK_CREATION_FROM_ENTREE_ODJ = "creation_from_entreeODJ";
    private static final String MARK_LISTE_GROUPE = "liste_groupe";
    private static final String MARK_LISTE_PUBLIE = "liste_publie";
    private static final String MARK_LISTE_ELU = "liste_elu";
    private static final String MARK_LISTE_ELU_A_AFFICHER = "liste_elu_a_afficher";
    private static final String MARK_LISTE_DIRECTION = "liste_direction";
    private static final String MARK_LISTE_HISTORIQUES = "liste_historiques";
    private static final String MARK_LISTE_VOEU = "liste_voeux";
    private static final String MARK_LISTE_CATEGORIE_DELIBERATION = "liste_categorie_deliberation";
    private static final String MARK_LISTE_NATURE_DOCUMENT = "liste_nature_document";
    private static final String MARK_ID_VOEU_SELECTED = "id_voeu_selected";
    private static final String MARK_ID_PUBLICATION_SELECTED = "id_publication_selected";
    private static final String MARK_ID_ACTE = "id_acte";
    private static final String MARK_URL_JSP_APPELANT = "url_jsp_appelant";
    private static final String MARK_URL_RETOUR = "url_retour";
    private static final String MARK_STATUT_VA = "statut_va";
    private static final String MARK_ENTREE = "entree";
    private static final String MARK_STATUT_ACTE_SELECTED = "statut_acte_selected";
    private static final String MARK_SAVE = "save";
    private static final String MARK_ACTE = "acte";
    private static final String MARK_FORMATION_CONSEIL = "formation_conseil";
    private static final String PROPERTY_FICHIERS_PER_PAGE = "ods.itemPerPage.label";
    private static final String PROPERTY_LABEL_OBJET = "ods.voeuamendement.label.objet";
    private static final String PROPERTY_LABEL_FICHIER = "ods.voeuamendement.label.fichier";
    private static final String PROPERTY_LABEL_VOEUX_RATTACHES = "ods.voeuamendement.label.voeuxrattaches";
    private static final String PROPERTY_LABEL_VOEUX_NON_RATTACHES = "ods.voeuamendement.label.voeuxnonrattaches";
    private static final String PROPERTY_LABEL_DATE_VOTE = "ods.modificationvoeuamendement.label.datevote";
    private static final String PROPERTY_LIBELLE_CREATION_ENTREE_ODJ = "ods.odj.modificationOdj.select.value.voeu";

    /* -------------------- */
    /* Variables de session */
    /* -------------------- */
    protected GVoeuAmendement _voeuAmendementBean;
    protected GSeance _seance;
    protected boolean _bIsGestionAval;

    // Permet de savoir si le retour doit se faire sur l'interface de gestion des VAs
    protected boolean _bUrlRetour;

    // Stocke l'url de retour si le retour ne ce fait pas sur l'interface de gestion des VAs 
    protected String _strUrlRetour;

    // Gre s'il y a eu des changements dans un VA avant enregistrement
    protected boolean _bHasChanged;

    // Gre la page du Paginator
    protected String _strCurrentPageIndex;

    // Gre le nombre d'items par page
    protected int _nItemsPerPage;
    protected int _nDefaultItemsPerPage = -1;

    /* ---------------------------- */
    /* Filtres de recherche communs */
    /* ---------------------------- */

    // Id de la sance slectionne
    protected int _nIdSeanceSelected = -1;

    // Id du type de voeu slectionn (rattach ou non)
    protected int _nIdVoeuSelected = -1;

    // Id de la publication slectionne
    protected int _nIdPublicationSelected = -1;

    // Gre le statut de l'acte slectionn
    protected int _nStatutActeSelected = -1;

    // Id du statut slectionn
    protected int _nIdStatutSelected = -1;

    // Id de la date de vote slectionne
    protected int _nIdDatesVoteSelected = -1;
    protected boolean _bComeFromActe;

    /* -------------------- */
    /* Instanciation Spring */
    /* -------------------- */
    @Autowired
    protected GVoeuAmendementHome _voeuAmendementHome;
    @Autowired
    protected GOrdreDuJourService _ordreDuJourService;
    @Autowired
    protected GOrdreDuJourHome _ordreDuJourHome;
    @Autowired
    protected IEluHome<GElu> _eluHome;
    @Autowired
    protected StatutHome _statutHome;
    @Autowired
    protected IFichierHome<GFichier, IFichierFilter, ITypeDocument, GSeance> _fichierHome;
    @Autowired
    private GEntreeOrdreDuJourHome _entreeOrdreDuJourHome;
    @Autowired
    private ActeHome<GSeance, GFichier, GActeFilter> _acteHome;
    @Autowired
    private NatureActeHome _natureActeHome;
    @Autowired
    private StatutActeHome _statutActeHome;
    @Autowired
    private ICategorieDeliberationHome _categorieDeliberationHome;
    @Autowired
    private FichierPhysiqueHome _fichierPhysiqueHome;
    @Autowired
    private GroupePolitiqueHome _groupePolitiqueHome;
    @Autowired
    private IGroupePolitiqueService _groupePolitiqueService;
    @Autowired
    private IHorodatageService _horodatageService;
    @Autowired
    private ISearchService<GSeance, GSeanceFilter, GRequeteUtilisateur, GResultatRequete, GElu, GFichier, GVoeuAmendement, GPdd> _searchService;
    @Autowired
    private IActeService<GSeance, GFichier, GActeFilter> _acteService;
    @Autowired
    private ICategorieDeliberationService _categorieDeliberationService;
    @Autowired
    private IDirectionService _directionService;
    @Autowired
    private HistoriqueHome _historiqueHome;
    @Autowired
    private PanierHome _panierHome;
    @Autowired
    private IPDDHome<GPdd, IPDDFilter, GVoeuAmendementFilter, GSeance, GElu, GVoeuAmendement, GFichier, GArrondissement> _projetDeliberationHome;
    @Autowired
    private IProjetDeDeliberationService<GOrdreDuJour, GEntreeOrdreDuJour, GVoeuAmendement, GSeance, GElu, GPdd, GFichier, GPDDFormBean, GPDDFilter, GArrondissement> _projetDeliberationService;

    /* ------------------------------------------ */
    /* Mthodes abstraites (services spcifiques) */
    /* ------------------------------------------ */
    protected abstract GVoeuAmendement newVoeuAmendement(  );

    protected abstract GVoeuAmendementFilter newVoeuAmendementFilter(  );

    protected abstract GOrdreDuJour newOrdreDuJour(  );

    protected abstract IStatutFilter newStatutFilter(  );

    protected abstract GFichier newFichierInstance(  );

    protected abstract int getTypeODJEnumDefinitif(  );

    protected abstract void reinitSessionSpec(  );

    protected abstract GVoeuAmendementFilter getVoeuAmendementListFilterSpec( HttpServletRequest request );

    protected abstract HashMap<String, Object> getVoeuAmendementListSpec( HttpServletRequest request );

    protected abstract HashMap<String, Object> getCreationVoeuAmendementSpec( HttpServletRequest request );

    protected abstract boolean isFieldRequiredSpec( HttpServletRequest request, List<String> champsNonSaisie );

    protected abstract boolean isValidFormSpec( HttpServletRequest request );

    protected abstract boolean setVoeuAmendementBeanSpec( HttpServletRequest request );

    protected abstract HashMap<String, Object> getModificationVoeuAmendementSpec( HttpServletRequest request );

    protected abstract void getModificationVoeuAmendementSpecEnd( HashMap<String, Object> model,
        HttpServletRequest request );

    protected abstract GVoeuAmendementFilter getVoeuAmendementSelectionListFilterSpec( HttpServletRequest request,
        String strUrlJSP );

    protected abstract HashMap<String, Object> getVoeuAmendementSelectionListSpec( HttpServletRequest request,
        List<GVoeuAmendement> voeuAmendements, List<GVoeuAmendement> vaToDelete, String strUrlJSP );

    protected abstract void initRefListDatesVote( int nIdSeance, ReferenceList refList, List<Timestamp> listDatesVote,
        HttpServletRequest request ) throws ParseException;

    protected abstract boolean getQuestionsDiverses( HttpServletRequest request, List<String> listErrorMessage );

    protected abstract boolean isCancel( HttpServletRequest request );

    protected abstract boolean isIdPDDs( HttpServletRequest request );

    protected abstract String[] getIdPDDs( HttpServletRequest request );

    // Spcifique CP
    protected abstract String isAlreadyExistInFasciculeSpec( HttpServletRequest request );

    protected abstract void ajouteVAaReleveDesTravauxSpec(  );

    protected abstract String checkModificationSpec( HttpServletRequest request, FileItem fileDelib );

    protected abstract void updateReleveSpec(  );

    protected abstract void removeReleveSpec( GVoeuAmendement voeuAmendement );

    protected abstract void setSeanceSpec(  );

    protected abstract void setSeanceSpec( GVoeuAmendement voeuAmendement );

    protected abstract void setFasciculeSpec( boolean bSetSeance );

    protected abstract void setFasciculeSpec( GVoeuAmendement voeuAmendement );

    protected abstract void doAddRemoveParentSpec( GVoeuAmendement parent );

    protected abstract void initSeanceSpec(  );

    protected abstract void getVoeuAmendementListSpec( HttpServletRequest request,
        GVoeuAmendementFilter voeuAmendementFilter );

    protected abstract ISeanceHome<GSeance, GSeanceFilter, GFichier, GElu, GVoeuAmendement, GPdd> getSeanceHome(  );

    protected abstract Plugin getPlugin(  );

    protected abstract String getParameterPluginName(  );

    protected abstract String getIntituleFichierDeliberationSpec( HttpServletRequest request );

    /**
     * Retourne l'URL de retour pour un voeu rattach
     * @return l'URL de retour pour un voeu rattach
     */
    public String getURLRetourVoeuRattache(  )
    {
        return JSP_RETOUR_PDDS_IN_VOEURATTACHE;
    }

    /**
     * Retourne l'URL de retour pour un amendement
     * @return l'URL de retour pour un amendement
     */
    public String getURLRetourAmendement(  )
    {
        return JSP_RETOUR_PDDS_IN_AMENDEMENT;
    }

    /**
    * Appel  la mthode init() de PluginAdminPageJspBean (initialise les
    * variables de session si elles n'ont pas t auparavant initialises)
    * @param request Requte HTTP
    * @param bIsGestionAval TRUE si gestion aval, FALSE sinon
    */
    public void init( HttpServletRequest request, boolean bIsGestionAval )
    {
        _bIsGestionAval = bIsGestionAval;
        _bComeFromActe = false;
        initSeance( request, false );

        if ( _nDefaultItemsPerPage == -1 )
        {
            _nDefaultItemsPerPage = AppPropertiesService.getPropertyInt( PROPERTY_FICHIERS_PER_PAGE, 10 );
        }
    }

    /**
     * Rinitialise tous les attributs de session
     * @param request Requte HTTP
     * @param bIsGestionAval TRUE si gestion aval, FALSE sinon
     */
    public void reinitSession( HttpServletRequest request, boolean bIsGestionAval )
    {
        _voeuAmendementBean = null;
        _seance = null;
        _bIsGestionAval = bIsGestionAval;
        _bHasChanged = false;
        _bUrlRetour = false;
        _strUrlRetour = null;
        _nIdPublicationSelected = -1;
        _nIdSeanceSelected = -1;
        _nIdStatutSelected = -1;
        _nStatutActeSelected = -1;
        _nIdVoeuSelected = -1;
        _bComeFromActe = false;

        if ( _nDefaultItemsPerPage == -1 )
        {
            _nDefaultItemsPerPage = AppPropertiesService.getPropertyInt( PROPERTY_FICHIERS_PER_PAGE, 10 );
        }

        initSeance( request, true );

        if ( _seance != null )
        {
            _nIdSeanceSelected = _seance.getIdSeance(  );
        }

        // Rinitialisation des attributs de sessions spcifiques
        reinitSessionSpec(  );
    }

    public void comeFromActe(  )
    {
        _bComeFromActe = true;
    }

    /**
     * Permet d'initialiser l'url de retour
     * @param strUrl url de retour
     * @param request la requte http
     */
    public void initUrlRetour( String strUrl, HttpServletRequest request )
    {
        _strUrlRetour = strUrl;
        _bUrlRetour = true;

        _strUrlRetour += ( OdsConstants.CONSTANTE_POINT_INTERROGATION + getParameterPluginName(  ) );

        if ( request.getParameter( OdsParameters.ID_ODJ ) != null )
        {
            _strUrlRetour += ( OdsConstants.CONSTANTE_ET + OdsParameters.ID_ODJ + OdsConstants.CONSTANTE_EGAL +
            request.getParameter( OdsParameters.ID_ODJ ) + OdsConstants.CONSTANTE_ET + "creation=creation" );
        }
        else if ( request.getParameter( OdsParameters.IS_CERTIFICAT ) != null )
        {
            _strUrlRetour += ( OdsConstants.CONSTANTE_ET + OdsParameters.ID_VOEUAMENDEMENT +
            OdsConstants.CONSTANTE_EGAL + request.getParameter( OdsParameters.ID_VOEUAMENDEMENT ) );
        }
    }

    /**
     * @return _bIsGestionAval boolean
     */
    public boolean isGestionAval(  )
    {
        return _bIsGestionAval;
    }

    /**
     * Rinitialise l'index de la page courante
     */
    protected void resetPageIndex(  )
    {
        _strCurrentPageIndex = CONSTANTE_PAGE_FIRST;
    }

    /**
     * Retourne l'interface de gestion des amendements, des voeux rattachs
     * ou des voeux non rattachs en fonction du type demand
     * @param request Requte HTTP
     * @param strType (amendement, voeu rattach, voeu non rattach)
     * @return Interface de gestion des VA
     */
    public HashMap<String, Object> getVoeuAmendementList( HttpServletRequest request, String strType,
        String strReturnDefault )
    {
        Plugin plugin = getPlugin(  );

        // Rinitialisation des variables de session
        _bHasChanged = false;
        _bUrlRetour = false;
        _strUrlRetour = null;
        _strCurrentPageIndex = Paginator.getPageIndex( request, Paginator.PARAMETER_PAGE_INDEX, _strCurrentPageIndex );
        _nItemsPerPage = Paginator.getItemsPerPage( request, Paginator.PARAMETER_ITEMS_PER_PAGE, _nItemsPerPage,
                _nDefaultItemsPerPage );

        // Suppression de l'acte en session
        if ( _voeuAmendementBean != null )
        {
            _voeuAmendementBean.setActeCourant( null );
        }

        // Rinitialisation de l'index de la page courante
        if ( ( _seance != null ) && ( _nIdSeanceSelected != _seance.getIdSeance(  ) ) )
        {
            resetPageIndex(  );
        }

        // Initialisation du filtre
        GVoeuAmendementFilter voeuAmendementFilter = getVoeuAmendementListFilterSpec( request );

        if ( ( strType != null ) &&
                ( strType.equals( OdsConstants.CONSTANTE_TYPE_A ) || strType.equals( OdsConstants.CONSTANTE_TYPE_LR ) ) )
        {
            voeuAmendementFilter.setType( strType );
        }

        // Id de la sance slectionne
        if ( _seance == null )
        {
            _nIdSeanceSelected = getIdSeance( request, voeuAmendementFilter );
        }
        else
        {
            _nIdSeanceSelected = _seance.getIdSeance(  );
        }

        // Initialisation du modle
        HashMap<String, Object> model = getVoeuAmendementListSpec( request );

        // Cration de la ReferenceList de publication
        ReferenceList refListPublie = new ReferenceList(  );
        initRefListPublie( refListPublie, request );
        _nIdPublicationSelected = getIdPublie( request, voeuAmendementFilter );
        model.put( MARK_LISTE_PUBLIE, refListPublie );
        model.put( MARK_ID_PUBLICATION_SELECTED, _nIdPublicationSelected );

        // Cration de la ReferenceList des voeux rattachs ou non
        if ( ( strType != null ) && strType.equals( OdsConstants.CONSTANTE_TYPE_ALLV ) )
        {
            ReferenceList refListVoeu = new ReferenceList(  );
            initRefListVoeu( refListVoeu, request );
            _nIdVoeuSelected = getIdVoeu( request, voeuAmendementFilter );
            model.put( MARK_LISTE_VOEU, refListVoeu );
            model.put( MARK_ID_VOEU_SELECTED, _nIdVoeuSelected );
        }

        // Prochaine sance
        if ( !isGestionAval(  ) )
        {
            if ( _seance != null )
            {
                voeuAmendementFilter.setIdSeance( _seance.getIdSeance(  ) );
            }

            model.put( OdsMarks.MARK_GESTION_AVAL, false );
            model.put( OdsMarks.MARK_SEANCE, _seance );
        }

        // Gestion aval
        else
        {
            IStatutFilter statutFilter = newStatutFilter(  );

            // Voeu
            if ( strType.equals( OdsConstants.CONSTANTE_TYPE_VR ) || strType.equals( OdsConstants.CONSTANTE_TYPE_VNR ) ||
                    strType.equals( OdsConstants.CONSTANTE_TYPE_ALLV ) )
            {
                statutFilter.setPourVoeu( true );
            }

            // Amendement
            if ( strType.equals( OdsConstants.CONSTANTE_TYPE_A ) || strType.equals( OdsConstants.CONSTANTE_TYPE_LR ) )
            {
                statutFilter.setPourAmendement( true );
            }

            ReferenceList refListStatuts = new ReferenceList(  );
            initRefListStatut( statutFilter, refListStatuts );

            // Statut de l'acte
            if ( request.getParameter( OdsParameters.STATUT_ACTE ) != null )
            {
                _nStatutActeSelected = OdsUtils.getIntegerParameter( OdsParameters.STATUT_ACTE, request );
            }

            voeuAmendementFilter.setStatutActe( _nStatutActeSelected );

            _nIdStatutSelected = getIdStatut( request, voeuAmendementFilter );
            voeuAmendementFilter.setIdSeance( _nIdSeanceSelected );

            GSeance seance = getSeanceHome(  ).findByPrimaryKey( _nIdSeanceSelected, plugin );

            if ( seance != null )
            {
                voeuAmendementFilter.setDateSeance( seance.getDateSeance(  ) );
            }

            model.put( OdsMarks.MARK_GESTION_AVAL, true );
            model.put( MARK_LISTE_SEANCES, getSeanceHome(  ).findOldSeance( plugin ) );
            model.put( MARK_LISTE_STATUTS, refListStatuts );
            model.put( MARK_ID_SEANCE_SELECTED, _nIdSeanceSelected );
            model.put( MARK_ID_STATUT_SELECTED, _nIdStatutSelected );
            model.put( MARK_STATUT_ACTE_SELECTED, _nStatutActeSelected );

            // Ajout des statuts Adopt et Amend pour la cration des actes
            model.put( OdsMarks.MARK_STATUT_AMENDE, StatutEnum.AMENDE.getId(  ) );
            model.put( OdsMarks.MARK_STATUT_ADOPTE, StatutEnum.ADOPTE.getId(  ) );

            // Rcupration de l'identifiant de la sance  partir de laquelle les crations d'acte sont autorises
            int nIdFirstSeanceForActe = AppPropertiesService.getPropertyInt( OdsProperties.PROPERTY_CREATION_ID_SEANCE,
                    1 );
            model.put( OdsMarks.MARK_ACTE_CREATION_ID_SEANCE, nIdFirstSeanceForActe );
        }

        // Rcupration de la liste des amendements
        List<GVoeuAmendement> listVoeuxAmendements = _voeuAmendementHome.findVoeuAmendementListByFilter( voeuAmendementFilter,
                plugin );
        Collections.sort( listVoeuxAmendements,
            new VoeuAmendementComparator<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>(  ) );

        String strHomeUrl = getHomeUrl( request, strType, strReturnDefault );
        Paginator paginator = new Paginator( listVoeuxAmendements, _nItemsPerPage, strHomeUrl,
                Paginator.PARAMETER_PAGE_INDEX, _strCurrentPageIndex );

        model.put( MARK_LISTE_VOEUAMENDEMENT, paginator.getPageItems(  ) );
        model.put( OdsMarks.MARK_NOMBRE_RESULTATS, listVoeuxAmendements.size(  ) );
        model.put( OdsMarks.MARK_PAGINATOR, paginator );
        model.put( OdsMarks.MARK_NB_ITEMS_PER_PAGE, OdsConstants.CONSTANTE_CHAINE_VIDE + _nItemsPerPage );

        return model;
    }

    /**
     * Retourne la map contenant les lments utiles au template HTML de la creation
     * @param request la requete
     * @param strType le type de VA
     * @param doReinitVA doit-on reinitialiser le VA en sessio
     * @return model la map contenant les lments utiles au template HTML de la creation
     */
    public HashMap<String, Object> getCreationVoeuAmendement( HttpServletRequest request, String strType,
        boolean bDoReinitVA )
    {
        Plugin plugin = getPlugin(  );

        // Initialisation du VA en session
        if ( bDoReinitVA )
        {
            _voeuAmendementBean = newVoeuAmendement(  );
            _voeuAmendementBean.setType( strType );
            // Par dfaut : deposeExecutif = true
            _voeuAmendementBean.setDeposeExecutif( false );
        }

        // Cration de la liste des lus
        List<GElu> listElu;

        if ( !isGestionAval(  ) )
        {
            listElu = _eluHome.findElusActifs( plugin );
        }
        else
        {
            listElu = _eluHome.findEluList( plugin );
        }

        // Initialisation de la ReferenceList des lus et cration du
        // tableau Javascript associant un groupe  une liste d'lus
        ReferenceList refListElu = new ReferenceList(  );
        List<GElu> listEluAAfficher = new ArrayList<GElu>(  );
        initListElu( refListElu, listElu, listEluAAfficher, request.getLocale(  ) );

        // Dans le cas de la gestion aval, et si la sance est nulle,
        // on met en session la dernire sance par dfaut
        _nIdSeanceSelected = 1;

        // Cration de la ReferenceList des groupes politiques
        ReferenceList refListGroupe = _groupePolitiqueService.initRefListGroupeDepositaire( isGestionAval(  ) );

        // Initialisation du modle
        HashMap<String, Object> model = getCreationVoeuAmendementSpec( request );

        // Prochaine sance
        if ( !isGestionAval(  ) )
        {
            model.put( OdsMarks.MARK_GESTION_AVAL, false );
        }

        // Gestion aval
        else
        {
            IStatutFilter statutfilter = newStatutFilter(  );

            // Si le VA en session a t rinitialis, on affiche un message d'erreur
            if ( _voeuAmendementBean == null )
            {
                throw new AppException( I18nService.getLocalizedString( MESSAGE_SESSION_EXPIREE, request.getLocale(  ) ) );
            }

            // Voeu
            if ( _voeuAmendementBean.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_VR ) ||
                    _voeuAmendementBean.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_VNR ) )
            {
                statutfilter.setPourVoeu( true );

                if ( _voeuAmendementBean.getDeliberation(  ) != null )
                {
                    GFichier deliberation = _fichierHome.findByPrimaryKey( _voeuAmendementBean.getDeliberation(  )
                                                                                              .getId(  ), plugin );
                    _voeuAmendementBean.setDeliberation( deliberation );
                }
            }

            // Amendement
            if ( _voeuAmendementBean.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_A ) ||
                    _voeuAmendementBean.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_LR ) )
            {
                statutfilter.setPourAmendement( true );
            }

            ReferenceList refListStatuts = new ReferenceList(  );
            initRefListStatut( statutfilter, refListStatuts );

            // Elments ncessaires pour la gestion aval
            model.put( OdsMarks.MARK_GESTION_AVAL, true );
            model.put( MARK_LISTE_SEANCES, getSeanceHome(  ).findOldSeance( plugin ) );
            model.put( MARK_ID_SEANCE_SELECTED, _nIdSeanceSelected );
            model.put( MARK_LISTE_STATUTS, refListStatuts );
        }

        model.put( MARK_LISTE_ELU_A_AFFICHER, listEluAAfficher );
        model.put( MARK_DEPOSITAIRE_TYPE_ELU, IDepositaire.DEPOSITAIRE_TYPE_ELU );
        model.put( MARK_DEPOSITAIRE_TYPE_GROUPE, IDepositaire.DEPOSITAIRE_TYPE_GROUPE );
        model.put( OdsMarks.MARK_VOEUAMENDEMENT, _voeuAmendementBean );
        model.put( OdsMarks.MARK_SEANCE, _seance );
        model.put( MARK_LISTE_GROUPE, refListGroupe );
        model.put( MARK_LISTE_ELU, refListElu );
        model.put( OdsMarks.MARK_URL_WEBAPP, AppPathService.getBaseUrl( request ) );

        // Cas o l'on souhaite crer un VA (en tant qu'entre de l'ODJ) depuis un ODJ
        if ( ( _strUrlRetour != null ) && _strUrlRetour.contains( CONSTANTE_ORDREDUJOUR ) )
        {
            model.put( MARK_CREATION_FROM_ENTREE_ODJ, OdsConstants.INTEGER_TRUE );
        }
        else
        {
            model.put( MARK_CREATION_FROM_ENTREE_ODJ, OdsConstants.INTEGER_FALSE );
        }

        return model;
    }

    /**
     * Mthode qui effectue la cration d'un amendement, d'un voeu rattach ou d'un voeu
     * non rattach, en fonction du type de cration demand. Redirection vers l'interface
     * d'administration (amendement, voeu rattach ou voeu non rattach)
     * @param request Requte HTTP
     * @param strType Type de cration demand
     * @param strReturnDefault URL de retour par dfaut
     * @return url URL de retour
     */
    public String doCreationVoeuAmendement( HttpServletRequest request, String strType, String strReturnDefault )
    {
        Plugin plugin = getPlugin(  );

        // Cas o l'utilisateur a cliqu sur le bouton "Annuler"
        if ( ( request.getParameter( OdsParameters.A_FAIRE ) != null ) &&
                request.getParameter( OdsParameters.A_FAIRE ).trim(  ).equals( OdsParameters.CONSTANTE_ACTION_CANCEL ) )
        {
            String strReturn = null;

            // Si le VA en session a t rinitialis, on affiche un message d'erreur
            if ( _voeuAmendementBean == null )
            {
                UrlItem url = new UrlItem( getHomeUrl( request, strType, strReturnDefault ) );

                return AdminMessageService.getMessageUrl( request, MESSAGE_SESSION_EXPIREE, url.getUrl(  ),
                    AdminMessage.TYPE_ERROR );
            }

            if ( _bUrlRetour )
            {
                strReturn = AppPathService.getBaseUrl( request ) + _strUrlRetour + OdsConstants.CONSTANTE_ET +
                    OdsParameters.ID_VOEUAMENDEMENT + OdsConstants.CONSTANTE_EGAL + _voeuAmendementBean.getId(  );

                if ( _ordreDuJourService.getUrlSelectionVoeuRetour(  ).equals( _strUrlRetour ) )
                {
                    strReturn += ( OdsConstants.CONSTANTE_ET + OdsParameters.ID_VOEUAMENDEMENT +
                    OdsConstants.CONSTANTE_EGAL + _voeuAmendementBean.getId(  ) );
                }

                strReturn = AppPathService.getBaseUrl( request ) + _strUrlRetour + OdsConstants.CONSTANTE_ET +
                    OdsParameters.A_FAIRE + OdsConstants.CONSTANTE_EGAL + CONSTANTE_CANCEL;

                _bUrlRetour = false;
                _strUrlRetour = null;

                return strReturn;
            }
            else
            {
                return getHomeUrl( request, _voeuAmendementBean.getType(  ), strReturnDefault );
            }
        }

        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        FileItem fileItem = multipartRequest.getFile( OdsParameters.FICHIER_PHYSIQUE );

        List<String> champsNonSaisie = new ArrayList<String>(  );
        List<String> fileProblems = new ArrayList<String>(  );

        // Test des paramtres de l'interface de saisie
        if ( isFieldRequired( request, champsNonSaisie ) )
        {
            if ( ( request.getParameter( OdsParameters.REFERENCE ) != null ) &&
                    ( request.getParameter( OdsParameters.REFERENCE ).trim(  ).length(  ) > 10 ) )
            {
                return AdminMessageService.getMessageUrl( request, MESSAGE_REFERENCE_TOO_LONG, AdminMessage.TYPE_STOP );
            }

            // Si le VA en session a t rinitialis, on affiche un message d'erreur
            if ( _voeuAmendementBean == null )
            {
                UrlItem url = new UrlItem( getHomeUrl( request, strType, strReturnDefault ) );

                return AdminMessageService.getMessageUrl( request, MESSAGE_SESSION_EXPIREE, url.getUrl(  ),
                    AdminMessage.TYPE_ERROR );
            }

            List<String> listErrorMessage = new ArrayList<String>(  );

            if ( setVoeuAmendementBean( request, listErrorMessage ) )
            {
                if ( isValidFile( fileItem, fileProblems ) )
                {
                    // Pour la gestion en aval, on doit aussi traiter le fichier de dlibration
                    FileItem fileDelib = null;

                    if ( isGestionAval(  ) && !_voeuAmendementBean.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_A ) )
                    {
                        fileDelib = multipartRequest.getFile( OdsParameters.FICHIER_DELIBERATION );

                        if ( !isValidFile( fileDelib, fileProblems ) )
                        {
                            String strFileProblem = ( fileProblems.get( 0 ) != null ) ? fileProblems.get( 0 ) : null;

                            if ( ( strFileProblem != null ) && strFileProblem.equals( MESSAGE_LABEL_FICHIER_NO_PDF ) )
                            {
                                return AdminMessageService.getMessageUrl( request, MESSAGE_LABEL_FICHIER_NO_PDF,
                                    AdminMessage.TYPE_STOP );
                            }
                        }
                    }

                    // Les lus peuvent tre saisis uniquement si le voeu ou amendement nest
                    // pas dpos par lexcutif (on vide la liste des lus rattachs au VA)
                    if ( _voeuAmendementBean.getDeposeExecutif(  ) &&
                            ( _voeuAmendementBean.getDepositaires(  ) != null ) )
                    {
                        _voeuAmendementBean.getDepositaires(  ).clear(  );
                    }

                    // Appel au service spcifique
                    String strReturn = isAlreadyExistInFasciculeSpec( request );

                    if ( ( strReturn != null ) && !strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                    {
                        return strReturn;
                    }

                    // Cration du fichier et stockage en session du fichier cr
                    _voeuAmendementBean.setFichier( doCreationFichier( fileItem, false, request ) );

                    // Pour la gestion en aval, on traite aussi le fichier de dlibration
                    if ( isGestionAval(  ) && !_voeuAmendementBean.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_A ) &&
                            ( fileDelib != null ) && ( fileDelib.getName(  ) != null ) &&
                            !fileDelib.getName(  ).trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                    {
                        GFichier fichierDelib = doCreationFichier( fileDelib, true, request );
                        _voeuAmendementBean.setDeliberation( fichierDelib );
                    }

                    int nIdVA = -1;

                    try
                    {
                        if ( !_voeuAmendementBean.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_VR ) )
                        {
                            _voeuAmendementBean.setPdds( null );
                        }

                        nIdVA = _voeuAmendementHome.create( _voeuAmendementBean, plugin );

                        GSeance nextSeance = getSeanceHome(  ).getProchaineSeance( plugin );
                        boolean bArchive = true;

                        if ( ( nextSeance != null ) && ( _voeuAmendementBean != null ) &&
                                ( _voeuAmendementBean.getSeance(  ) != null ) )
                        {
                            GSeance seanceByVA = _voeuAmendementBean.getSeance(  );

                            if ( seanceByVA.equals( nextSeance ) )
                            {
                                bArchive = false;
                            }
                        }

                        _searchService.addObjectToIndex( _voeuAmendementBean, bArchive, plugin );
                    }
                    catch ( AppException ae )
                    {
                        if ( ae.getInitialException(  ) instanceof SQLException )
                        {
                            return AdminMessageService.getMessageUrl( request,
                                OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND, AdminMessage.TYPE_ERROR );
                        }
                    }

                    _voeuAmendementBean.setId( nIdVA );

                    // Appel au service spcifique
                    ajouteVAaReleveDesTravauxSpec(  );

                    boolean bIsPDDListEmpty = ( ( _voeuAmendementBean.getPdds(  ) == null ) ||
                        _voeuAmendementBean.getPdds(  ).isEmpty(  ) );

                    if ( request.getParameter( OdsParameters.A_FAIRE ).equals( OdsParameters.CONSTANTE_ACTION_APPLY ) )
                    {
                        strReturn = getModificationUrl( request, _voeuAmendementBean.getType(  ), nIdVA,
                                strReturnDefault );

                        if ( isGestionAval(  ) )
                        {
                            strReturn += ( OdsConstants.CONSTANTE_ET + OdsParameters.ID_SEANCE +
                            OdsConstants.CONSTANTE_EGAL + _seance.getIdSeance(  ) );
                        }

                        return strReturn;
                    }
                    else
                    {
                        if ( bIsPDDListEmpty &&
                                _voeuAmendementBean.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_VR ) )
                        {
                            return AppPathService.getBaseUrl( request ) +
                            JSP_URL_GET_AVERTISSEMENT_ENREGISTREMENT_VOEU_RATTACHE;
                        }

                        if ( bIsPDDListEmpty &&
                                ( _voeuAmendementBean.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_LR ) ||
                                _voeuAmendementBean.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_A ) ) )
                        {
                            return AppPathService.getBaseUrl( request ) +
                            JSP_URL_GET_AVERTISSEMENT_ENREGISTREMENT_AMENDEMENT;
                        }

                        strReturn = null;

                        if ( _bUrlRetour )
                        {
                            strReturn = ( AppPathService.getBaseUrl( request ) + _strUrlRetour +
                                OdsConstants.CONSTANTE_ET + OdsParameters.ID_VOEUAMENDEMENT +
                                OdsConstants.CONSTANTE_EGAL + _voeuAmendementBean.getId(  ) );

                            if ( _ordreDuJourService.getUrlSelectionVoeuRetour(  ).equals( _strUrlRetour ) )
                            {
                                strReturn += ( OdsConstants.CONSTANTE_ET + OdsParameters.ID_VOEUAMENDEMENT +
                                OdsConstants.CONSTANTE_EGAL + _voeuAmendementBean.getId(  ) );
                            }

                            _bUrlRetour = false;
                            _strUrlRetour = null;

                            return strReturn;
                        }
                        else
                        {
                            return getHomeUrl( request, _voeuAmendementBean.getType(  ), strReturnDefault );
                        }
                    }
                }
                else
                {
                    String message = OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED;
                    String strFileProblem = ( fileProblems.get( 0 ) != null ) ? fileProblems.get( 0 ) : null;
                    Object[] messagesArgs = { I18nService.getLocalizedString( strFileProblem, request.getLocale(  ) ) };

                    if ( ( strFileProblem != null ) && strFileProblem.equals( MESSAGE_LABEL_FICHIER_NO_PDF ) )
                    {
                        message = MESSAGE_LABEL_FICHIER_NO_PDF;
                    }

                    return AdminMessageService.getMessageUrl( request, message, messagesArgs, AdminMessage.TYPE_STOP );
                }
            }
            else if ( ( listErrorMessage != null ) && !listErrorMessage.isEmpty(  ) )
            {
                return listErrorMessage.get( 0 );
            }
            else
            {
                return AdminMessageService.getMessageUrl( request, Messages.MANDATORY_FIELDS, AdminMessage.TYPE_STOP );
            }
        }
        else
        {
            String strChamp = ( champsNonSaisie.get( 0 ) != null ) ? champsNonSaisie.get( 0 ) : null;
            Object[] messagesArgs = { I18nService.getLocalizedString( strChamp, request.getLocale(  ) ) };

            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED,
                messagesArgs, AdminMessage.TYPE_STOP );
        }
    }

    /**
     * Teste si les champs obligatoires ont bien t saisis
     * @param request Requte HTTP
     * @param champsNonSaisie Liste des champs non saisis
     * @return true si les champs obligatoires ont bien t saisis
     */
    private boolean isFieldRequired( HttpServletRequest request, List<String> champsNonSaisie )
    {
        boolean bIsFieldRequired = true;

        if ( ( request.getParameter( OdsParameters.OBJET ) == null ) ||
                request.getParameter( OdsParameters.OBJET ).trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            champsNonSaisie.add( PROPERTY_LABEL_OBJET );
            bIsFieldRequired = false;
        }

        // Appel au service spcifique
        else if ( !isFieldRequiredSpec( request, champsNonSaisie ) )
        {
            bIsFieldRequired = false;
        }

        return bIsFieldRequired;
    }

    /**
     * Mthode qui recupre les paramtres fournis par l'utilisateur en vue de les insrer dans le voeuamendementBean stock en session
     * @param request Requte HTTP
     * @param listErrorMessage Liste d'erreurs renvoye
     * @return true si aucune erreur ne s'est produite
     */
    private boolean setVoeuAmendementBean( HttpServletRequest request, List<String> listErrorMessage )
    {
        Plugin plugin = getPlugin(  );

        if ( isValidForm( request ) )
        {
            // Si le VA en session a t rinitialis, on renvoie FALSE
            if ( _voeuAmendementBean == null )
            {
                return false;
            }

            // Rcupration du texte du fichier pour la recherche
            String strFichierTexte = request.getParameter( OdsParameters.FICHIER_TXT_RECHERCHE );

            if ( strFichierTexte != null )
            {
                strFichierTexte = strFichierTexte.trim(  );
            }

            // Rcupration du fichier en vue de l'insrer dans le VA en session
            if ( ( _voeuAmendementBean.getFichier(  ) == null ) ||
                    ( ( _voeuAmendementBean.getFichier(  ).getTexte(  ) == null ) && ( strFichierTexte != null ) ) ||
                    ( ( _voeuAmendementBean.getFichier(  ).getTexte(  ) != null ) && ( strFichierTexte == null ) ) ||
                    ( ( _voeuAmendementBean.getFichier(  ).getTexte(  ) != null ) && ( strFichierTexte != null ) &&
                    !_voeuAmendementBean.getFichier(  ).getTexte(  ).equals( strFichierTexte ) ) )
            {
                _bHasChanged = true;
            }

            if ( _voeuAmendementBean.getFichier(  ) == null )
            {
                _voeuAmendementBean.setFichier( newFichierInstance(  ) );
            }

            _voeuAmendementBean.getFichier(  ).setTexte( strFichierTexte );

            // Rcupration de la proprit "dpos par l'excutif"
            int nDeposeExecutif = -1;

            try
            {
                if ( request.getParameter( OdsParameters.DEPOSE_EXECUTIF ) != null )
                {
                    nDeposeExecutif = Integer.parseInt( request.getParameter( OdsParameters.DEPOSE_EXECUTIF ) );
                }
            }
            catch ( NumberFormatException nfe )
            {
                AppLogService.error( nfe );

                return false;
            }

            if ( ( _voeuAmendementBean.getDeposeExecutif(  ) && ( nDeposeExecutif != 1 ) ) ||
                    ( !_voeuAmendementBean.getDeposeExecutif(  ) && ( nDeposeExecutif == 1 ) ) )
            {
                _bHasChanged = true;
            }

            if ( nDeposeExecutif == 1 )
            {
                _voeuAmendementBean.setDeposeExecutif( true );
            }
            else
            {
                _voeuAmendementBean.setDeposeExecutif( false );
            }

            // Rcupration de la proprit "voeu rattach" (si nous sommes dans le cas d'un voeu et non d'un amendement)
            if ( !_voeuAmendementBean.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_A ) &&
                    !_voeuAmendementBean.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_LR ) )
            {
                int nVoeuRattache = -1;

                try
                {
                    if ( request.getParameter( OdsParameters.VOEU_RATTACHE ) != null )
                    {
                        nVoeuRattache = Integer.parseInt( request.getParameter( OdsParameters.VOEU_RATTACHE ) );
                    }
                }
                catch ( NumberFormatException nfe )
                {
                    AppLogService.error( nfe );

                    return false;
                }

                if ( nVoeuRattache == 1 )
                {
                    _voeuAmendementBean.setType( OdsConstants.CONSTANTE_TYPE_VR );
                }
                else
                {
                    _voeuAmendementBean.setType( OdsConstants.CONSTANTE_TYPE_VNR );
                }
            }

            // Rcupration de la rfrence du VA
            String strReference = request.getParameter( OdsParameters.REFERENCE ).trim(  );

            if ( strReference.length(  ) > 10 )
            {
                return false;
            }
            else if ( ( ( _voeuAmendementBean.getReference(  ) == null ) &&
                    !strReference.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) ) ||
                    ( ( _voeuAmendementBean.getReference(  ) != null ) &&
                    !_voeuAmendementBean.getReference(  ).equals( strReference ) ) )
            {
                _bHasChanged = true;

                if ( strReference.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                {
                    _voeuAmendementBean.setReference( null );
                    _voeuAmendementBean.setReferenceFront( null );
                }
                else
                {
                    _voeuAmendementBean.setReference( strReference );
                    _voeuAmendementBean.setReferenceFront( strReference );
                }
            }

            // Rcupration de l'objet
            String strObjet = request.getParameter( OdsParameters.OBJET );

            if ( ( _voeuAmendementBean.getObjet(  ) == null ) || !_voeuAmendementBean.getObjet(  ).equals( strObjet ) )
            {
                _voeuAmendementBean.setObjet( strObjet );
                _bHasChanged = true;
            }

            // Appel au service spcifique
            if ( !setVoeuAmendementBeanSpec( request ) )
            {
                return false;
            }

            // Gestion aval
            if ( isGestionAval(  ) )
            {
                // Rcupration de la sance
                if ( _seance == null )
                {
                    return false;
                }
                else
                {
                    _voeuAmendementBean.setSeance( _seance );
                }

                // Rcupration du statut
                int nStatut;

                if ( ( request.getParameter( OdsParameters.ID_STATUT ) != null ) &&
                        !request.getParameter( OdsParameters.ID_STATUT ).trim(  )
                                    .equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                {
                    try
                    {
                        nStatut = Integer.parseInt( request.getParameter( OdsParameters.ID_STATUT ) );

                        Statut statut = _statutHome.findByPrimaryKey( nStatut, plugin );
                        _voeuAmendementBean.setStatut( statut );
                    }
                    catch ( NumberFormatException ne )
                    {
                        AppLogService.error( ne );

                        return false;
                    }
                }

                // Rcupration de la date de vote
                if ( ( request.getParameter( OdsParameters.DATE_VOTE ) != null ) &&
                        !request.getParameter( OdsParameters.DATE_VOTE ).trim(  )
                                    .equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                {
                    String strDateVote = request.getParameter( OdsParameters.DATE_VOTE ).trim(  );
                    Timestamp timeStampDateVote = DateUtils.getDate( strDateVote, true );

                    if ( timeStampDateVote == null )
                    {
                        Object[] messagesArgs = 
                            {
                                I18nService.getLocalizedString( PROPERTY_LABEL_DATE_VOTE, request.getLocale(  ) ),
                            };

                        listErrorMessage.add( AdminMessageService.getMessageUrl( request,
                                OdsProperties.PROPERTY_MESSAGE_DATE_FORMAT_ERROR, messagesArgs, AdminMessage.TYPE_ERROR ) );

                        return false;
                    }
                    else
                    {
                        _voeuAmendementBean.setDateVote( timeStampDateVote );
                    }
                }
                else if ( ( request.getParameter( OdsParameters.DATE_VOTE ) != null ) &&
                        request.getParameter( OdsParameters.DATE_VOTE ).trim(  )
                                   .equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                {
                    _voeuAmendementBean.setDateVote( null );
                }

                // Appel au service spcifique
                if ( !getQuestionsDiverses( request, listErrorMessage ) )
                {
                    return false;
                }

                // Rcupration du numro de voeu adopt
                if ( ( request.getParameter( OdsParameters.NUMERO_VOEU_ADOPTE ) != null ) &&
                        !request.getParameter( OdsParameters.NUMERO_VOEU_ADOPTE ).trim(  )
                                    .equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                {
                    String strNumeroVoeu = request.getParameter( OdsParameters.NUMERO_VOEU_ADOPTE );
                    _voeuAmendementBean.setNumeroDeliberation( strNumeroVoeu );
                }
                else if ( ( request.getParameter( OdsParameters.NUMERO_VOEU_ADOPTE ) != null ) &&
                        request.getParameter( OdsParameters.NUMERO_VOEU_ADOPTE ).trim(  )
                                   .equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                {
                    _voeuAmendementBean.setNumeroDeliberation( null );
                }
            }

            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * Mthode qui teste si dans le formulaire de saisie ou de modification d'un VA,
     * l'ensemble des critres communs aux deux formulaires sont prsents dans la requte
     * @param request Requte HTTP
     * @return true si le formulaire est valide
     */
    private boolean isValidForm( HttpServletRequest request )
    {
        if ( ( request.getParameter( OdsParameters.OBJET ) == null ) ||
                ( request.getParameter( OdsParameters.REFERENCE ) == null ) ||
                ( request.getParameter( OdsParameters.FICHIER_TXT_RECHERCHE ) == null ) )
        {
            return false;
        }

        // Appel au service spcifique
        if ( !isValidFormSpec( request ) )
        {
            return false;
        }

        return true;
    }

    /**
     * Teste si le fichier fourni est valide
     * @param fileItem Fichier  tester
     * @param fileProblems Liste des problmes lis au fichier
     * @return true si le fichier est valide
     */
    private boolean isValidFile( FileItem fileItem, List<String> fileProblems )
    {
        boolean nIsFileGood = true;

        if ( ( fileItem != null ) && ( fileItem.getName(  ) != null ) )
        {
            if ( fileItem.getName(  ).trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                fileProblems.add( PROPERTY_LABEL_FICHIER );
                nIsFileGood = false;
            }
            else
            {
                String extensionFile = fileItem.getName(  ).substring( fileItem.getName(  ).length(  ) - 3 )
                                               .toUpperCase(  );

                if ( !OdsConstants.CONSTANTE_PDF.equals( extensionFile ) )
                {
                    fileProblems.add( MESSAGE_LABEL_FICHIER_NO_PDF );
                    nIsFileGood = false;
                }
            }
        }
        else
        {
            nIsFileGood = false;
            fileProblems.add( PROPERTY_LABEL_FICHIER );
        }

        return nIsFileGood;
    }

    /**
     * Mthode qui cre le  fichier rcupr de la requte
     * @param fileItem Fichier associ au voeu (rcupr de la requte)
     * @param bEstDeliberation true si il est de dlibration, false sinon
     * @param strNumeroDeliberation le numro de dlibration
     * @return Fichier cr
     */
    protected GFichier doCreationFichier( FileItem fileItem, boolean bEstDeliberation, HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        GFichier newFichier = newFichierInstance(  );
        String strTexteRecherche = null;

        if ( bEstDeliberation )
        {
            _voeuAmendementBean.setDeliberation( newFichier );
        }
        else
        {
            strTexteRecherche = _voeuAmendementBean.getFichier(  ).getTexte(  );
            _voeuAmendementBean.setFichier( newFichier );
        }

        if ( strTexteRecherche != null )
        {
            strTexteRecherche = strTexteRecherche.trim(  );
        }

        newFichier.setTexte( strTexteRecherche );

        // Cration de l'objet reprsentant le fichier physique
        FichierPhysique newFichierPhysique = new FichierPhysique(  );
        newFichierPhysique.setDonnees( fileItem.get(  ) );

        int nIdFichierPhysique = _fichierPhysiqueHome.create( newFichierPhysique, plugin );
        newFichierPhysique.setIdFichier( nIdFichierPhysique );

        // Cration de l'objet reprsentant le fichier
        newFichier.setFichier( newFichierPhysique );
        newFichier.setNom( StringUtils.getNameCourt( fileItem ) );
        newFichier.setExtension( fileItem.getName(  ).substring( fileItem.getName(  ).length(  ) - 3 ).toUpperCase(  ) );
        newFichier.setTaille( (int) fileItem.getSize(  ) );
        newFichier.setSeance( _seance );

        if ( bEstDeliberation )
        {
            newFichier.setTypdeDocument( _fichierHome.findTypeDocumentsById( ITypeDocument.DELIBERATION, plugin ) );

            Calendar calSeance = new GregorianCalendar(  );
            calSeance.setTimeInMillis( _seance.getDateSeance(  ).getTime(  ) );

            newFichier.setIntitule( OdsConstants.CONSTANTE_CHAINE_VIDE + calSeance.get( Calendar.YEAR ) +
                OdsConstants.CONSTANTE_ESPACE + OdsConstants.CONSTANTE_TYPE_VR +
                getIntituleFichierDeliberationSpec( request ) + OdsConstants.CONSTANTE_ESPACE +
                CONSTANTE_FIN_INTITULE_FICHIER_DELIBERATION );

            if ( _voeuAmendementBean.getEnLigne(  ) )
            {
                newFichier.setEnLigne( true );
                newFichier.setVersion( 1 );

                Timestamp currentTime = new Timestamp( System.currentTimeMillis(  ) );
                newFichier.setDatePublication( currentTime );
            }
            else
            {
                newFichier.setEnLigne( false );
            }
        }
        else
        {
            if ( _voeuAmendementBean.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_A ) ||
                    _voeuAmendementBean.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_LR ) )
            {
                newFichier.setTypdeDocument( _fichierHome.findTypeDocumentsById( ITypeDocument.AMENDEMENT, plugin ) );
            }
            else
            {
                newFichier.setTypdeDocument( _fichierHome.findTypeDocumentsById( ITypeDocument.VOEU, plugin ) );
            }

            newFichier.setIntitule( CONSTANTE_CREATION_FICHIER_INTITULE );
            newFichier.setEnLigne( false );
        }

        newFichier.setId( _fichierHome.create( newFichier, plugin ) );

        if ( newFichier.getSeance(  ) != null )
        {
            ISeance seanceByFichier = newFichier.getSeance(  );
            GSeance nextSeance = getSeanceHome(  ).getProchaineSeance( plugin );

            if ( ( nextSeance != null ) && !seanceByFichier.equals( nextSeance ) &&
                    ( ( newFichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.DOCUMENT_LIBRE ) ||
                    ( newFichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.COMPTE_RENDU ) ||
                    ( newFichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.LISTE_DE_DESIGNATION ) ) )
            {
                _searchService.addObjectToIndex( newFichier, true, plugin );
            }
        }

        return newFichier;
    }

    /**
     * retourne l'URL de la page de modification d'un VA
     * @param request la requete
     * @param strTypeVa le type de VA
     * @param nIdVA l'id du Va
     * @param strReturnDefault l'URL de retour par dfaut
     * @return l'URL de la page de modification d'un VA
     */
    protected String getModificationUrl( HttpServletRequest request, String strTypeVa, int nIdVA,
        String strReturnDefault )
    {
        if ( strTypeVa != null )
        {
            return getModificationUrl( request, strTypeVa, strReturnDefault ) +
            OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_VOEUAMENDEMENT + OdsConstants.CONSTANTE_EGAL +
            nIdVA;
        }

        return strReturnDefault;
    }

    /**
     * retourne l'URL de la page de modification d'un VA
     * @param request la requete
     * @param typeVa le type de VA
     * @param strReturnDefault l'URL de retour par dfaut
     * @return l'URL de la page de modification d'un VA
     */
    protected String getModificationUrl( HttpServletRequest request, String typeVa, String strReturnDefault )
    {
        String strJspReturn = AppPathService.getBaseUrl( request );

        if ( typeVa != null )
        {
            if ( request.getParameter( MARK_ID_ACTE ) != null )
            {
                strJspReturn += JSP_URL_GET_VISUALISATION_VOEU_FOR_ACTE;
            }
            else if ( typeVa.equals( OdsConstants.CONSTANTE_TYPE_A ) ||
                    typeVa.equals( OdsConstants.CONSTANTE_TYPE_LR ) )
            {
                strJspReturn += JSP_URL_GET_MODIFICATION_AMENDEMENT;
            }
            else if ( typeVa.equals( OdsConstants.CONSTANTE_TYPE_VR ) ||
                    typeVa.equals( OdsConstants.CONSTANTE_TYPE_VNR ) ||
                    typeVa.equals( OdsConstants.CONSTANTE_TYPE_ALLV ) )
            {
                strJspReturn += JSP_URL_GET_MODIFICATION_VOEU;
            }
            else if ( typeVa.equals( CONSTANTE_TYPE_STATUT_VA ) )
            {
                strJspReturn += JSP_URL_GET_MODIFICATION_STATUT_VA;
            }

            return strJspReturn;
        }

        return strReturnDefault;
    }

    /**
     * retourne l'URL du home
     * @param request la requete
     * @param typeVa le type de VA
     * @param strReturnDefault l'URL de retour par dfaut
     * @return l'url du Home
     */
    protected String getHomeUrl( HttpServletRequest request, String typeVa, String strReturnDefault )
    {
        String strJspReturn = AppPathService.getBaseUrl( request );

        if ( typeVa != null )
        {
            if ( isGestionAval(  ) )
            {
                if ( typeVa.equals( OdsConstants.CONSTANTE_TYPE_A ) || typeVa.equals( OdsConstants.CONSTANTE_TYPE_LR ) )
                {
                    strJspReturn += JSP_URL_GET_AMENDEMENTS_LIST_GESTION_AVAL;
                }
                else if ( typeVa.equals( OdsConstants.CONSTANTE_TYPE_VR ) ||
                        typeVa.equals( OdsConstants.CONSTANTE_TYPE_VNR ) ||
                        typeVa.equals( OdsConstants.CONSTANTE_TYPE_ALLV ) )
                {
                    strJspReturn += JSP_URL_GET_VOEUX_LIST_GESTION_AVAL;
                }
                else if ( typeVa.equals( CONSTANTE_TYPE_STATUT_VA ) )
                {
                    strJspReturn += JSP_URL_GET_STATUT_VA;
                }
                else
                {
                    strJspReturn = strReturnDefault;
                }
            }
            else
            {
                if ( typeVa.equals( OdsConstants.CONSTANTE_TYPE_A ) || typeVa.equals( OdsConstants.CONSTANTE_TYPE_LR ) )
                {
                    strJspReturn += JSP_URL_GET_AMENDEMENTS_LIST;
                }
                else if ( typeVa.equals( OdsConstants.CONSTANTE_TYPE_VR ) ||
                        typeVa.equals( OdsConstants.CONSTANTE_TYPE_VNR ) ||
                        typeVa.equals( OdsConstants.CONSTANTE_TYPE_ALLV ) )
                {
                    strJspReturn += JSP_URL_GET_VOEUX_LIST;
                }
                else
                {
                    strJspReturn = strReturnDefault;
                }
            }
        }
        else
        {
            strJspReturn = strReturnDefault;
        }

        return strJspReturn;
    }

    /**
     * Retourne true si le VA est publi, false sinon
     * @param nIdVA l'id du VA
     * @return true si le VA est publi, false sinon
     */
    public boolean isPublier( int nIdVA )
    {
        Plugin plugin = getPlugin(  );

        GVoeuAmendement va = _voeuAmendementHome.findByPrimaryKey( nIdVA, plugin );

        return va.getEnLigne(  );
    }

    /**
     * Retourne le type du va en session
     * @return le type du va en session
     */
    public String getTypeVa(  )
    {
        return _voeuAmendementBean.getType(  );
    }

    /**
     * Retourne la map contenant les lments utiles au template HTML de modification
     * @param request request
     * @param strReturnDefault l'URL de retour par dfaut
     * @return model la map contenant les lments utiles au template HTML de modification
     */
    public HashMap<String, Object> getModificationVoeuAmendement( HttpServletRequest request, String strReturnDefault )
    {
        Plugin plugin = getPlugin(  );

        int nIdVoeuAmendement = OdsUtils.getIntegerParameter( OdsParameters.ID_VOEUAMENDEMENT, request );

        if ( ( nIdVoeuAmendement == -1 ) && ( _voeuAmendementBean == null ) )
        {
            return getVoeuAmendementList( request, OdsConstants.VA_TYPE_V, strReturnDefault );
        }

        if ( nIdVoeuAmendement > -1 )
        {
            if ( _voeuAmendementBean != null )
            {
                Acte<GSeance, GFichier> acteCourant = _voeuAmendementBean.getActeCourant(  );
                _voeuAmendementBean = _voeuAmendementHome.findByPrimaryKey( nIdVoeuAmendement, plugin );

                if ( _voeuAmendementBean.getActeCourant(  ) == null )
                {
                    _voeuAmendementBean.setActeCourant( acteCourant );
                }
            }
            else
            {
                _voeuAmendementBean = _voeuAmendementHome.findByPrimaryKey( nIdVoeuAmendement, plugin );
            }
        }

        OdsUtils.testNullObject( _voeuAmendementBean, request );

        if ( _voeuAmendementBean != null )
        {
            // Initialisation du modle
            HashMap<String, Object> model = getModificationVoeuAmendementSpec( request );

            if ( ( request.getParameter( OdsParameters.A_FAIRE ) != null ) &&
                    request.getParameter( OdsParameters.A_FAIRE ).equals( CONSTANTE_ACTION_STATUT_VA ) )
            {
                IStatutFilter statutFilter = newStatutFilter(  );

                if ( _voeuAmendementBean.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_VR ) ||
                        _voeuAmendementBean.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_VNR ) )
                {
                    statutFilter.setPourVoeu( true );

                    if ( _voeuAmendementBean.getDeliberation(  ) != null )
                    {
                        GFichier deliberation = _fichierHome.findByPrimaryKey( _voeuAmendementBean.getDeliberation(  )
                                                                                                  .getId(  ), plugin );
                        _voeuAmendementBean.setDeliberation( deliberation );
                    }
                }

                if ( _voeuAmendementBean.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_A ) ||
                        _voeuAmendementBean.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_LR ) )
                {
                    statutFilter.setPourAmendement( true );
                }

                ReferenceList refListStatuts = new ReferenceList(  );
                initRefListStatut( statutFilter, refListStatuts );

                model.put( MARK_LISTE_STATUTS, refListStatuts );
                model.put( OdsMarks.MARK_GESTION_AVAL, false );
                model.put( MARK_STATUT_VA, true );
            }
            else if ( isGestionAval(  ) )
            {
                List<GSeance> listSeance = getSeanceHome(  ).findOldSeance( plugin );
                IStatutFilter statutFilter = newStatutFilter(  );

                if ( _voeuAmendementBean.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_VR ) ||
                        _voeuAmendementBean.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_VNR ) )
                {
                    statutFilter.setPourVoeu( true );

                    if ( _voeuAmendementBean.getDeliberation(  ) != null )
                    {
                        GFichier deliberation = _fichierHome.findByPrimaryKey( _voeuAmendementBean.getDeliberation(  )
                                                                                                  .getId(  ), plugin );
                        _voeuAmendementBean.setDeliberation( deliberation );
                    }
                }

                if ( _voeuAmendementBean.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_A ) ||
                        _voeuAmendementBean.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_LR ) )
                {
                    statutFilter.setPourAmendement( true );
                }

                ReferenceList refListStatuts = new ReferenceList(  );
                initRefListStatut( statutFilter, refListStatuts );

                if ( request.getParameter( OdsParameters.PREPARER_ACTE ) != null )
                {
                    Acte<GSeance, GFichier> acteCourant = new Acte<GSeance, GFichier>(  );
                    acteCourant.setReference( _voeuAmendementBean.getReferenceComplete(  ) );

                    NatureActe natureActe = new NatureActe(  );
                    natureActe.setIdNatureDocument( NatureActeEnum.DELIBERATION_VOEU.getId(  ) );
                    acteCourant.setNatureDocument( natureActe );

                    StatutActe statut = _statutActeHome.findByPrimaryKey( StatutActeEnum.A_PREPARER.getId(  ), plugin );
                    acteCourant.setStatutActe( statut );

                    CategorieDeliberation categorie = _categorieDeliberationHome.findCategorieByCode( CONSTANTE_CODE_CATEGORIE_VOEUX,
                            plugin );
                    acteCourant.setCategorie( categorie );

                    acteCourant.setSeance( _seance );
                    acteCourant.setFormationConseil( _voeuAmendementBean.getFormationConseil(  ) );
                    acteCourant.setIdEntite( nIdVoeuAmendement );
                    acteCourant.setIdTypeEntite( ITypeEntite.VOEUX );

                    GFichier deliberation = _voeuAmendementBean.getDeliberation(  );

                    if ( deliberation != null )
                    {
                        acteCourant.setDeliberationFinale( deliberation );
                        acteCourant.setPoids( deliberation.getTaille(  ) );

                        int nMaxPoids = AppPropertiesService.getPropertyInt( OdsProperties.PROPERTY_POIDS_MAX_ACTE,
                                OdsConstants.DEFAULT_POIDS_MAX_ACTE );

                        if ( acteCourant.getPoids(  ) > nMaxPoids )
                        {
                            acteCourant.setIdTransmission( 0 );
                        }
                        else
                        {
                            acteCourant.setIdTransmission( 1 );
                        }
                    }
                    else
                    {
                        acteCourant.setIdTransmission( 1 );
                    }

                    _voeuAmendementBean.setActeCourant( acteCourant );
                }

                // Elments ncessaires pour la gestion aval
                model.put( OdsMarks.MARK_GESTION_AVAL, true );
                model.put( MARK_STATUT_VA, false );
                model.put( MARK_LISTE_SEANCES, listSeance );
                model.put( MARK_ID_SEANCE_SELECTED, _nIdSeanceSelected );
                model.put( MARK_LISTE_STATUTS, refListStatuts );
                model.put( MARK_LISTE_CATEGORIE_DELIBERATION,
                    _categorieDeliberationService.initRefListCategorieDeliberation( plugin, false ) );

                // Ajout des statuts Adopt et Amend pour la cration des actes
                model.put( OdsMarks.MARK_STATUT_AMENDE, StatutEnum.AMENDE.getId(  ) );
                model.put( OdsMarks.MARK_STATUT_ADOPTE, StatutEnum.ADOPTE.getId(  ) );

                // Rcupration de l'identifiant de la sance  partir de laquelle les crations d'acte sont autorises
                int nIdFirstSeanceForActe = AppPropertiesService.getPropertyInt( OdsProperties.PROPERTY_CREATION_ID_SEANCE,
                        1 );
                model.put( OdsMarks.MARK_ACTE_CREATION_ID_SEANCE, nIdFirstSeanceForActe );

                // Rcupration des natures de document       
                model.put( MARK_LISTE_NATURE_DOCUMENT, _acteService.getRefListNatureActe(  ) );
                model.put( MARK_LISTE_DIRECTION, _directionService.initRefListDirection( true, false ) );

                // Si l'on a cliqu sur 'visualiser l'acte'  partir de la liste de suivi des actes
                int nIdActe = OdsUtils.getIntegerParameter( MARK_ID_ACTE, request );

                if ( nIdActe != -1 )
                {
                    model.put( MARK_ID_ACTE, nIdActe );

                    Acte<GSeance, GFichier> acte = _acteHome.findByPrimaryKey( nIdActe, plugin );
                    model.put( MARK_ACTE, acte );
                }
            }
            else
            {
                model.put( OdsMarks.MARK_GESTION_AVAL, false );
                model.put( MARK_STATUT_VA, false );
            }

            // Rcupration du fichier li au VA
            GFichier fichier = _fichierHome.findByPrimaryKey( _voeuAmendementBean.getFichier(  ).getId(  ), plugin );
            _voeuAmendementBean.setFichier( fichier );

            // Cration de la liste des lus
            List<GElu> listElu;

            if ( !isGestionAval(  ) )
            {
                listElu = _eluHome.findElusActifs( plugin );
            }
            else
            {
                listElu = _eluHome.findEluList( plugin );
            }

            // Initialisation de la ReferenceList des lus et cration du
            // tableau Javascript associant un groupe  une liste d'lus
            ReferenceList refListElu = new ReferenceList(  );
            List<GElu> listEluAAfficher = new ArrayList<GElu>(  );
            initListElu( refListElu, listElu, listEluAAfficher, request.getLocale(  ) );

            // Cration de la ReferenceList des groupes politiques
            ReferenceList refListGroupe = _groupePolitiqueService.initRefListGroupeDepositaire( isGestionAval(  ) );

            model.put( MARK_DEPOSITAIRE_TYPE_ELU, IDepositaire.DEPOSITAIRE_TYPE_ELU );
            model.put( MARK_DEPOSITAIRE_TYPE_GROUPE, IDepositaire.DEPOSITAIRE_TYPE_GROUPE );
            model.put( MARK_LISTE_ELU_A_AFFICHER, listEluAAfficher );
            model.put( OdsMarks.MARK_VOEUAMENDEMENT, _voeuAmendementBean );
            model.put( OdsMarks.MARK_SEANCE, _seance );
            model.put( MARK_LISTE_ELU, refListElu );
            model.put( MARK_LISTE_GROUPE, refListGroupe );
            model.put( OdsMarks.MARK_URL_WEBAPP, AppPathService.getBaseUrl( request ) );

            // Cas o l'on souhaite crer un VA (en tant qu'entre de l'ODJ) depuis un ODJ
            if ( ( _strUrlRetour != null ) && _strUrlRetour.contains( CONSTANTE_ORDREDUJOUR ) )
            {
                model.put( MARK_CREATION_FROM_ENTREE_ODJ, OdsConstants.INTEGER_TRUE );
            }
            else
            {
                model.put( MARK_CREATION_FROM_ENTREE_ODJ, OdsConstants.INTEGER_FALSE );
            }

            getModificationVoeuAmendementSpecEnd( model, request );

            return model;
        }
        else
        {
            return null;
        }
    }

    /**
     * Methode qui effectue la modification d'un amendement, d'un voeu rattach
     * en fonction du type du VoeuAmendement prsent en session.
     * Redirection vers l'interface d'administration(amendement,voeu non rattach,voeu rattach)
     * @param request request
     * @param strReturnDefault l'URL de retour par dfaut
     * @param user l'utilisateur courant
     * @return template
     */
    public String doModificationVoeuAmendement( HttpServletRequest request, String strReturnDefault, AdminUser user )
    {
        Plugin plugin = getPlugin(  );

        // Cas o l'utilisateur a cliqu sur le bouton "Annuler"
        if ( request.getParameter( OdsParameters.A_FAIRE ).equals( OdsParameters.CONSTANTE_ACTION_CANCEL ) )
        {
            if ( _bUrlRetour )
            {
                String strReturn = AppPathService.getBaseUrl( request ) + _strUrlRetour;

                _bUrlRetour = false;
                _strUrlRetour = null;

                return strReturn;
            }
            else
            {
                // Si le VA en session a t rinitialis, on affiche un message d'erreur
                if ( _voeuAmendementBean == null )
                {
                    UrlItem url = new UrlItem( strReturnDefault );

                    return AdminMessageService.getMessageUrl( request, MESSAGE_SESSION_EXPIREE, url.getUrl(  ),
                        AdminMessage.TYPE_ERROR );
                }

                return getHomeUrl( request, _voeuAmendementBean.getType(  ), strReturnDefault );
            }
        }

        // Si le VA en session a t rinitialis, on affiche un message d'erreur
        if ( _voeuAmendementBean == null )
        {
            UrlItem url = new UrlItem( strReturnDefault );

            return AdminMessageService.getMessageUrl( request, MESSAGE_SESSION_EXPIREE, url.getUrl(  ),
                AdminMessage.TYPE_ERROR );
        }

        // Cas o le VA a t supprim
        GVoeuAmendement va = _voeuAmendementHome.findByPrimaryKey( _voeuAmendementBean.getId(  ), plugin );

        if ( va == null )
        {
            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                AdminMessage.TYPE_ERROR );
        }

        // Gestion des profils
        // Si l'utilisateur n'est pas autoris  effectuer cette action, il est renvoy
        // vers une page lui indiquant qu'il n'a pas les droits ncessaires
        if ( !isGestionAval(  ) && _voeuAmendementBean.getEnLigne(  ) &&
                !RBACService.isAuthorized( VoeuxAmendementsResourceIdService.VOEUX_AMENDEMENTS,
                    RBAC.WILDCARD_RESOURCES_ID, VoeuxAmendementsResourceIdService.PERMISSION_MISE_A_JOUR, user ) )
        {
            return AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                AdminMessage.TYPE_ERROR );
        }

        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        List<String> champsNonSaisie = new ArrayList<String>(  );

        if ( ( request.getParameter( OdsParameters.REFERENCE ) != null ) &&
                ( request.getParameter( OdsParameters.REFERENCE ).trim(  ).length(  ) > 10 ) )
        {
            return AdminMessageService.getMessageUrl( request, MESSAGE_REFERENCE_TOO_LONG, AdminMessage.TYPE_STOP );
        }

        String strMessage;
        List<String> listErrorMessage = new ArrayList<String>(  );

        // Dans le cas o le VA est en ligne et que l'utilisateur est un charg de mise en ligne, il ne peut
        // modifier que les attributs propres au statut du VA (seules ces modifications sont prises en compte)
        if ( isGestionAval(  ) && _voeuAmendementBean.getEnLigne(  ) &&
                !RBACService.isAuthorized( VoeuxAmendementsResourceIdService.VOEUX_AMENDEMENTS,
                    RBAC.WILDCARD_RESOURCES_ID, VoeuxAmendementsResourceIdService.PERMISSION_MISE_A_JOUR, user ) )
        {
            // Rcupration du statut
            int nStatut = OdsUtils.getIntegerParameter( OdsParameters.ID_STATUT, request );

            if ( nStatut != -1 )
            {
                Statut statut = _statutHome.findByPrimaryKey( nStatut, plugin );
                _voeuAmendementBean.setStatut( statut );
            }

            // Rcupration de la date de vote
            if ( ( request.getParameter( OdsParameters.DATE_VOTE ) != null ) &&
                    !request.getParameter( OdsParameters.DATE_VOTE ).trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                String strDateVote = request.getParameter( OdsParameters.DATE_VOTE ).trim(  );
                Timestamp timeStampDateVote = DateUtils.getDate( strDateVote, true );

                if ( timeStampDateVote == null )
                {
                    Object[] messagesArgs = 
                        {
                            I18nService.getLocalizedString( PROPERTY_LABEL_DATE_VOTE, request.getLocale(  ) ),
                        };

                    return AdminMessageService.getMessageUrl( request,
                        OdsProperties.PROPERTY_MESSAGE_DATE_FORMAT_ERROR, messagesArgs, AdminMessage.TYPE_ERROR );
                }
                else
                {
                    _voeuAmendementBean.setDateVote( timeStampDateVote );
                }
            }
            else if ( ( request.getParameter( OdsParameters.DATE_VOTE ) != null ) &&
                    request.getParameter( OdsParameters.DATE_VOTE ).trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                _voeuAmendementBean.setDateVote( null );
            }

            // Appel au service spcifique
            if ( !getQuestionsDiverses( request, listErrorMessage ) )
            {
                return listErrorMessage.get( 0 );
            }

            // Rcupration du numro du voeu adopt
            if ( ( request.getParameter( OdsParameters.NUMERO_VOEU_ADOPTE ) != null ) &&
                    !request.getParameter( OdsParameters.NUMERO_VOEU_ADOPTE ).trim(  )
                                .equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                String strNumeroVoeu = request.getParameter( OdsParameters.NUMERO_VOEU_ADOPTE );
                _voeuAmendementBean.setNumeroDeliberation( strNumeroVoeu );
            }
            else if ( ( request.getParameter( OdsParameters.NUMERO_VOEU_ADOPTE ) != null ) &&
                    request.getParameter( OdsParameters.NUMERO_VOEU_ADOPTE ).trim(  )
                               .equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                _voeuAmendementBean.setNumeroDeliberation( null );
            }

            // Pour la gestion en aval, on doit traiter le fichier de dlibration
            FileItem fileDelib = null;

            if ( !_voeuAmendementBean.getType(  ).trim(  ).equals( OdsConstants.CONSTANTE_TYPE_A ) &&
                    ( multipartRequest.getFile( OdsParameters.FICHIER_DELIBERATION ) != null ) &&
                    ( multipartRequest.getFile( OdsParameters.FICHIER_DELIBERATION ).getName(  ) != null ) &&
                    !multipartRequest.getFile( OdsParameters.FICHIER_DELIBERATION ).getName(  )
                                         .equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                fileDelib = multipartRequest.getFile( OdsParameters.FICHIER_DELIBERATION );

                String strReturn = checkModification( request, fileDelib );

                if ( !strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
                {
                    return strReturn;
                }

                Timestamp dateForVersion = new Timestamp( System.currentTimeMillis(  ) );

                if ( _voeuAmendementBean.getDeliberation(  ) != null )
                {
                    doModificationFichier( fileDelib, dateForVersion, true, request );
                }
                else
                {
                    _voeuAmendementBean.setDeliberation( newFichierInstance(  ) );
                    _voeuAmendementBean.setDeliberation( doCreationFichier( fileDelib, true, request ) );
                }
            }

            boolean bArchive = true;
            GSeance nextSeance = getSeanceHome(  ).getProchaineSeance( plugin );

            // Appel au service spcifique
            setSeanceSpec(  );

            if ( ( nextSeance != null ) && ( _voeuAmendementBean != null ) &&
                    ( _voeuAmendementBean.getSeance(  ) != null ) )
            {
                GSeance seanceByVA = _voeuAmendementBean.getSeance(  );

                if ( seanceByVA.equals( nextSeance ) )
                {
                    bArchive = false;
                }
            }

            _searchService.updateObjectInIndex( _voeuAmendementBean, bArchive, plugin );
            _voeuAmendementHome.update( _voeuAmendementBean, plugin );

            if ( _voeuAmendementBean.getEnLigne(  ) )
            {
                // Appel au service spcifique
                setFasciculeSpec( false );

                _horodatageService.tracePublication( _voeuAmendementBean, HorodatageActionEnum.MODIFICATION );
            }

            String strType = _voeuAmendementBean.getType(  );
            int nIdVA = _voeuAmendementBean.getId(  );
            boolean bIsPDDListEmpty = ( ( _voeuAmendementBean.getPdds(  ) == null ) ||
                _voeuAmendementBean.getPdds(  ).isEmpty(  ) );

            // Enregistrement du nouveau VA en base de donnes
            if ( request.getParameter( OdsParameters.A_FAIRE ).trim(  ).equals( OdsParameters.CONSTANTE_ACTION_APPLY ) )
            {
                // Initialisation de l'id du VA en session
                _voeuAmendementBean.setId( nIdVA );

                strMessage = getModificationUrl( request, strType, nIdVA, strReturnDefault );
                strMessage += ( OdsConstants.CONSTANTE_ET + OdsParameters.ID_SEANCE + OdsConstants.CONSTANTE_EGAL +
                _seance.getIdSeance(  ) );

                if ( request.getParameter( MARK_ID_ACTE ) != null )
                {
                    int nIdActe = OdsUtils.getIntegerParameter( MARK_ID_ACTE, request );
                    strMessage += ( OdsConstants.CONSTANTE_ET + OdsParameters.ID_ACTE + OdsConstants.CONSTANTE_EGAL +
                    nIdActe );
                }
            }

            // Si "_nliasse" est gal  true, on retourne sur la liasse des VAs, sinon getHomeUrl
            else if ( request.getParameter( OdsParameters.A_FAIRE ).trim(  ).equals( CONSTANTE_ACTION_SAVE ) &&
                    _bUrlRetour )
            {
                // On vide l'objet VA en session
                _voeuAmendementBean = newVoeuAmendement(  );

                if ( bIsPDDListEmpty )
                {
                    if ( strType.equals( OdsConstants.CONSTANTE_TYPE_VR ) )
                    {
                        return AppPathService.getBaseUrl( request ) +
                        JSP_URL_GET_AVERTISSEMENT_ENREGISTREMENT_VOEU_RATTACHE;
                    }

                    if ( strType.equals( OdsConstants.CONSTANTE_TYPE_LR ) ||
                            strType.equals( OdsConstants.CONSTANTE_TYPE_A ) )
                    {
                        return AppPathService.getBaseUrl( request ) +
                        JSP_URL_GET_AVERTISSEMENT_ENREGISTREMENT_AMENDEMENT;
                    }
                }

                strMessage = AppPathService.getBaseUrl( request ) + _strUrlRetour;

                _bUrlRetour = false;
                _strUrlRetour = null;
            }
            else
            {
                if ( bIsPDDListEmpty )
                {
                    if ( strType.equals( OdsConstants.CONSTANTE_TYPE_VR ) )
                    {
                        return AppPathService.getBaseUrl( request ) +
                        JSP_URL_GET_AVERTISSEMENT_ENREGISTREMENT_VOEU_RATTACHE;
                    }

                    if ( strType.equals( OdsConstants.CONSTANTE_TYPE_LR ) ||
                            strType.equals( OdsConstants.CONSTANTE_TYPE_A ) )
                    {
                        return AppPathService.getBaseUrl( request ) +
                        JSP_URL_GET_AVERTISSEMENT_ENREGISTREMENT_AMENDEMENT;
                    }
                }

                strMessage = getHomeUrl( request, strType, strReturnDefault );
            }

            return strMessage;
        }

        // Dans le cas o l'utilisateur peut modifier tous les attributs d'un VA,
        // alors on effectue la procdure de mise  jour suivante
        else if ( ( request.getParameter( OdsParameters.ID_VOEUAMENDEMENT ) != null ) &&
                isFieldRequired( request, champsNonSaisie ) && setVoeuAmendementBean( request, listErrorMessage ) )
        {
            // Test pour savoir si le fichier a t modifi et si c'est le cas, si le fichier est bien au format PDF
            FileItem fileItem = multipartRequest.getFile( OdsParameters.FICHIER_PHYSIQUE );

            if ( ( fileItem != null ) && ( fileItem.getName(  ) != null ) &&
                    !fileItem.getName(  ).trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                _bHasChanged = true;
            }

            // Pour la gestion en aval, on doit aussi traiter le fichier de dlibration
            FileItem fileDelib = multipartRequest.getFile( OdsParameters.FICHIER_DELIBERATION );

            String strReturn = checkModification( request, fileDelib );

            if ( !strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                return strReturn;
            }

            int nIdVa = OdsUtils.getIntegerParameter( OdsParameters.ID_VOEUAMENDEMENT, request );
            _voeuAmendementBean.setId( nIdVa );

            // Les lus peuvent tre saisis uniquement si le VA nest pas dpos
            // par lexcutif (on vide la liste des lus ratachs au VA)
            if ( _voeuAmendementBean.getDeposeExecutif(  ) && ( _voeuAmendementBean.getDepositaires(  ) != null ) )
            {
                _voeuAmendementBean.getDepositaires(  ).clear(  );
            }

            Timestamp dateForVersion = new Timestamp( System.currentTimeMillis(  ) );

            // Gestion de l'historique du VA			    		
            if ( _voeuAmendementBean.getEnLigne(  ) && _bHasChanged )
            {
                _voeuAmendementBean.setVersion( _voeuAmendementBean.getVersion(  ) + 1 );
                _voeuAmendementBean.setEnLigne( true );
                _voeuAmendementBean.setDatePublication( dateForVersion );

                // Ajout d'un historique
                Historique historique = new Historique(  );
                historique.setVersion( _voeuAmendementBean.getVersion(  ) );
                historique.setDatePublication( dateForVersion );
                historique.setIdEntite( _voeuAmendementBean.getId(  ) );
                historique.setIdTypeEntite( ITypeEntite.VA );
                _historiqueHome.create( historique, plugin );
            }

            _bHasChanged = false;

            // Modification du fichier
            doModificationFichier( fileItem, dateForVersion, false, request );

            // Pour la gestion en aval, on traite aussi le fichier de dlibration
            if ( isGestionAval(  ) &&
                    !_voeuAmendementBean.getType(  ).trim(  ).equals( OdsConstants.CONSTANTE_TYPE_A ) &&
                    ( fileDelib != null ) && ( fileDelib.getName(  ) != null ) &&
                    !fileDelib.getName(  ).trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                if ( _voeuAmendementBean.getDeliberation(  ) != null )
                {
                    doModificationFichier( fileDelib, dateForVersion, true, request );
                }
                else
                {
                    _voeuAmendementBean.setDeliberation( newFichierInstance(  ) );
                    _voeuAmendementBean.setDeliberation( doCreationFichier( fileDelib, true, request ) );
                }
            }

            if ( !_voeuAmendementBean.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_A ) &&
                    !_voeuAmendementBean.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_LR ) )
            {
                // Gestion du type de voeu : rattach ou non rattach
                String strVoeuRattache = request.getParameter( OdsParameters.VOEU_RATTACHE );

                // Voeu rattach
                if ( strVoeuRattache != null )
                {
                    _voeuAmendementBean.setType( OdsConstants.CONSTANTE_TYPE_VR );
                }

                // Voeu non rattach
                else
                {
                    _voeuAmendementBean.setType( OdsConstants.CONSTANTE_TYPE_VNR );
                }
            }

            // Appel au service spcifique
            updateReleveSpec(  );

            try
            {
                boolean bArchive = true;
                GSeance nextSeance = getSeanceHome(  ).getProchaineSeance( plugin );

                // Appel au service spcifique
                setSeanceSpec(  );

                if ( ( nextSeance != null ) && ( _voeuAmendementBean != null ) &&
                        ( _voeuAmendementBean.getSeance(  ) != null ) )
                {
                    GSeance seanceByVA = _voeuAmendementBean.getSeance(  );

                    if ( seanceByVA.equals( nextSeance ) )
                    {
                        bArchive = false;
                    }
                }

                _searchService.updateObjectInIndex( _voeuAmendementBean, bArchive, plugin );
                _voeuAmendementHome.update( _voeuAmendementBean, plugin );
            }
            catch ( AppException ae )
            {
                if ( ae.getInitialException(  ) instanceof SQLException )
                {
                    return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                        AdminMessage.TYPE_ERROR );
                }
            }

            if ( _voeuAmendementBean.getEnLigne(  ) )
            {
                // Appel au service spcifique
                setFasciculeSpec( false );

                _horodatageService.tracePublication( _voeuAmendementBean, HorodatageActionEnum.MODIFICATION );
            }

            // Mise  jour de l'acte
            Acte<GSeance, GFichier> acteCourant = _voeuAmendementBean.getActeCourant(  );

            int nMaxPoids = AppPropertiesService.getPropertyInt( OdsProperties.PROPERTY_POIDS_MAX_ACTE,
                    OdsConstants.DEFAULT_POIDS_MAX_ACTE );

            if ( request.getParameter( OdsParameters.PREPARER_ACTE ) != null )
            {
                acteCourant = new Acte<GSeance, GFichier>(  );
                acteCourant.setStatutActe( StatutActeEnum.A_PREPARER.getStatutOnlyWidthId(  ) );
            }

            if ( ( request.getParameter( MARK_ID_ACTE ) == null ) && isGestionAval(  ) && ( acteCourant != null ) &&
                    ( request.getParameter( OdsParameters.PREPARER_ACTE_BUTTON ) == null ) &&
                    ( _voeuAmendementBean.getDateVote(  ) != null ) &&
                    ( ( acteCourant.getStatutActe(  ).getIdStatut(  ) == StatutActeEnum.A_PREPARER.getId(  ) ) ||
                    ( acteCourant.getStatutActe(  ).getIdStatut(  ) == StatutActeEnum.EN_PREPARATION.getId(  ) ) ) )
            {
                // Ajout de la catgorie de dlibration
                CategorieDeliberation categorie = _categorieDeliberationHome.findByPrimaryKey( Integer.parseInt( 
                            request.getParameter( OdsParameters.ID_CATEGORIE ) ), plugin );
                acteCourant.setCategorie( categorie );
                acteCourant.setSeance( _seance );
                acteCourant.setReference( _voeuAmendementBean.getReferenceCompleteCertificatAffichage(  ) );

                int natureActeID = Integer.parseInt( request.getParameter( OdsParameters.NATURE_DOCUMENT ) );
                acteCourant.setNatureDocument( _natureActeHome.findByPrimaryKey( natureActeID, plugin ) );

                GFichier deliberation = _voeuAmendementBean.getDeliberation(  );

                if ( deliberation != null )
                {
                    acteCourant.setDeliberationFinale( deliberation );
                    acteCourant.setPoids( deliberation.getTaille(  ) );

                    int nIdModeTransmission = Integer.parseInt( request.getParameter( OdsParameters.MODE_TRANSMISSION ) );

                    if ( ( nIdModeTransmission == 1 ) && ( acteCourant.getPoids(  ) > nMaxPoids ) )
                    {
                        // Mode de transmission automatique, mais le poids est suprieur
                        return AdminMessageService.getMessageUrl( request,
                            OdsProperties.PROPERTY_MESSAGE_MODE_TRANSMISSION_ERROR, AdminMessage.TYPE_STOP );
                    }

                    acteCourant.setIdTransmission( nIdModeTransmission );
                }

                acteCourant.setIdEntite( _voeuAmendementBean.getId(  ) );
                acteCourant.setIdTypeEntite( ITypeEntite.VOEUX );

                StatutActe statut = _statutActeHome.findByPrimaryKey( StatutActeEnum.EN_PREPARATION.getStatutOnlyWidthId(  )
                                                                                                   .getIdStatut(  ),
                        plugin );
                acteCourant.setStatutActe( statut );
                acteCourant.setSeance( _seance );
                acteCourant.setFormationConseil( _voeuAmendementBean.getFormationConseil(  ) );
                acteCourant.setEnLigne( false );
                _voeuAmendementBean.setActeCourant( acteCourant );

                if ( request.getParameter( OdsParameters.PREPARER_ACTE ) != null )
                {
                    _acteHome.create( acteCourant, plugin );
                }
                else
                {
                    _acteHome.update( acteCourant, plugin );
                }
            }

            String strType = _voeuAmendementBean.getType(  );
            int nIdVA = _voeuAmendementBean.getId(  );

            boolean bIsPDDListEmpty = ( ( _voeuAmendementBean.getPdds(  ) == null ) ||
                _voeuAmendementBean.getPdds(  ).isEmpty(  ) );

            // Enregistrement du nouveau VA en base de donnes
            if ( request.getParameter( OdsParameters.A_FAIRE ).trim(  ).equals( OdsParameters.CONSTANTE_ACTION_APPLY ) )
            {
                // On initialise l'id du VA en session
                _voeuAmendementBean.setId( nIdVA );
                strMessage = getModificationUrl( request, strType, nIdVA, strReturnDefault );

                // Gestion aval
                if ( isGestionAval(  ) )
                {
                    strMessage += ( OdsConstants.CONSTANTE_ET + OdsParameters.ID_SEANCE + OdsConstants.CONSTANTE_EGAL +
                    _seance.getIdSeance(  ) );
                }

                if ( request.getParameter( MARK_ID_ACTE ) != null )
                {
                    int nIdActe = OdsUtils.getIntegerParameter( MARK_ID_ACTE, request );
                    strMessage += ( OdsConstants.CONSTANTE_ET + OdsParameters.ID_ACTE + OdsConstants.CONSTANTE_EGAL +
                    nIdActe );
                }
            }

            else if ( request.getParameter( OdsParameters.A_FAIRE ).trim(  ).equals( CONSTANTE_ACTION_SAVE ) &&
                    _bUrlRetour && ( request.getParameter( MARK_SAVE ) != null ) )
            {
                // On vide l'objet VA en session
                _voeuAmendementBean = newVoeuAmendement(  );

                if ( bIsPDDListEmpty )
                {
                    if ( strType.equals( OdsConstants.CONSTANTE_TYPE_VR ) )
                    {
                        return AppPathService.getBaseUrl( request ) +
                        JSP_URL_GET_AVERTISSEMENT_ENREGISTREMENT_VOEU_RATTACHE;
                    }

                    if ( strType.equals( OdsConstants.CONSTANTE_TYPE_LR ) ||
                            strType.equals( OdsConstants.CONSTANTE_TYPE_A ) )
                    {
                        return AppPathService.getBaseUrl( request ) +
                        JSP_URL_GET_AVERTISSEMENT_ENREGISTREMENT_AMENDEMENT;
                    }
                }

                strMessage = AppPathService.getBaseUrl( request ) + _strUrlRetour;

                _bUrlRetour = false;
                _strUrlRetour = null;
            }

            // Si le bouton "preparer acte" est cliqu, on recharge la page avec les champs de prparation de l'acte
            else if ( request.getParameter( OdsParameters.PREPARER_ACTE_BUTTON ) != null )
            {
                if ( RBACService.isAuthorized( ActesResourceIdService.ACTES, RBAC.WILDCARD_RESOURCES_ID,
                            ActesResourceIdService.PERMISSION_CREATION, user ) )
                {
                    strMessage = getModificationUrl( request, strType, nIdVA, strReturnDefault );
                    strMessage += "&preparer_acte=preparer_acte";
                }
                else
                {
                    strMessage = AdminMessageService.getMessageUrl( request, OdsProperties.MESSAGE_PROFIL_NO_RIGHT,
                            AdminMessage.TYPE_ERROR );
                }
            }

            // Si le bouton "finaliser l'acte" est cliqu
            else if ( request.getParameter( OdsParameters.FINALISER_ACTE ) != null )
            {
                if ( ( acteCourant.getIdTransmission(  ) == 1 ) &&
                        ( _voeuAmendementBean.getDeliberation(  ).getTaille(  ) > nMaxPoids ) )
                {
                    strMessage = AppPathService.getBaseUrl( request ) + JSP_URL_FINALISATION_ACTE +
                        OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_VOEUAMENDEMENT +
                        OdsConstants.CONSTANTE_EGAL + _voeuAmendementBean.getId(  ) + OdsConstants.CONSTANTE_ET +
                        getParameterPluginName(  );

                    return strMessage;
                }
                else
                {
                    if ( _bUrlRetour )
                    {
                        strMessage = AppPathService.getBaseUrl( request ) + _strUrlRetour;

                        _bUrlRetour = false;
                        _strUrlRetour = null;
                    }
                    else
                    {
                        strMessage = getHomeUrl( request, strType, strReturnDefault );
                    }

                    acteCourant = _voeuAmendementBean.getActeCourant(  );

                    StatutActe statut = _statutActeHome.findByPrimaryKey( StatutActeEnum.A_TRANSMETTRE.getStatutOnlyWidthId(  )
                                                                                                      .getIdStatut(  ),
                            plugin );
                    acteCourant.setStatutActe( statut );
                    _acteHome.update( acteCourant, plugin );
                }
            }
            else
            {
                if ( bIsPDDListEmpty )
                {
                    if ( strType.equals( OdsConstants.CONSTANTE_TYPE_VR ) )
                    {
                        return AppPathService.getBaseUrl( request ) +
                        JSP_URL_GET_AVERTISSEMENT_ENREGISTREMENT_VOEU_RATTACHE;
                    }

                    if ( strType.equals( OdsConstants.CONSTANTE_TYPE_LR ) ||
                            strType.equals( OdsConstants.CONSTANTE_TYPE_A ) )
                    {
                        return AppPathService.getBaseUrl( request ) +
                        JSP_URL_GET_AVERTISSEMENT_ENREGISTREMENT_AMENDEMENT;
                    }
                }

                strMessage = getHomeUrl( request, strType, strReturnDefault );
            }

            return strMessage;
        }

        // Sinon certains champs obligatoires n'ont pas t convenablement remplis
        // On indique donc  l'utilisateur quel champ doit tre saisi  nouveau
        else if ( ( listErrorMessage != null ) && !listErrorMessage.isEmpty(  ) )
        {
            return listErrorMessage.get( 0 );
        }
        else
        {
            String strChamp = ( champsNonSaisie.get( 0 ) != null ) ? champsNonSaisie.get( 0 )
                                                                   : OdsConstants.CONSTANTE_CHAINE_VIDE;
            Object[] messagesArgs = { I18nService.getLocalizedString( strChamp, request.getLocale(  ) ) };

            return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_FIELDREQUIRED,
                messagesArgs, AdminMessage.TYPE_STOP );
        }
    }

    /**
     * Modifie le fichier et le fichier physique li au voeuAmendement
     * @param fileItem le fichier  modifier
     * @param dateForVersion la date pour la nouvelle version
     * @param bEstDeliberation true si il est de dlibration, false sinon
     */
    protected void doModificationFichier( FileItem fileItem, Timestamp dateForVersion, boolean bEstDeliberation,
        HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        if ( _voeuAmendementBean.getFichier(  ).getId(  ) != -1 )
        {
            GFichier fichier;

            if ( bEstDeliberation )
            {
                fichier = _fichierHome.findByPrimaryKey( _voeuAmendementBean.getDeliberation(  ).getId(  ), plugin );

                Calendar calSeance = new GregorianCalendar(  );
                calSeance.setTimeInMillis( _seance.getDateSeance(  ).getTime(  ) );
                fichier.setIntitule( OdsConstants.CONSTANTE_CHAINE_VIDE + calSeance.get( Calendar.YEAR ) +
                    OdsConstants.CONSTANTE_ESPACE + OdsConstants.CONSTANTE_TYPE_VR +
                    getIntituleFichierDeliberationSpec( request ) + OdsConstants.CONSTANTE_ESPACE +
                    CONSTANTE_FIN_INTITULE_FICHIER_DELIBERATION );
            }
            else
            {
                fichier = _fichierHome.findByPrimaryKey( _voeuAmendementBean.getFichier(  ).getId(  ), plugin );
                fichier.setTexte( _voeuAmendementBean.getFichier(  ).getTexte(  ) );
            }

            FichierPhysique fichierPhysique = _fichierPhysiqueHome.findByPrimaryKey( fichier.getFichier(  )
                                                                                            .getIdFichier(  ), plugin );

            // Rcupration des modifications
            if ( ( fileItem != null ) && ( fileItem.getName(  ) != null ) &&
                    !OdsConstants.CONSTANTE_CHAINE_VIDE.equals( fileItem.getName(  ).trim(  ) ) )
            {
                // Modification de l'objet reprsentant le fichier physique
                fichierPhysique.setDonnees( fileItem.get(  ) );
                _fichierPhysiqueHome.update( fichierPhysique, plugin );

                fichier.setFichier( fichierPhysique );
                fichier.setNom( StringUtils.getNameCourt( fileItem ) );
                fichier.setExtension( fileItem.getName(  ).substring( fileItem.getName(  ).length(  ) - 3 ).toUpperCase(  ) );
                fichier.setTaille( (int) fileItem.getSize(  ) );

                // Modification de l'objet reprsentant le Fichier
                if ( !fichier.getEnLigne(  ) )
                {
                    // On incrmente le numro de version
                    fichier.setDatePublication( dateForVersion );

                    if ( !bEstDeliberation )
                    {
                        fichier.setVersion( fichier.getVersion(  ) + 1 );

                        // Ajout de l'historique
                        Historique historique = new Historique(  );
                        historique.setVersion( fichier.getVersion(  ) );
                        historique.setDatePublication( dateForVersion );
                        historique.setIdEntite( fichier.getId(  ) );
                        historique.setIdTypeEntite( ITypeEntite.FICHIERS );
                        _historiqueHome.create( historique, plugin );
                    }
                }
            }

            if ( fichier.getSeance(  ) != null )
            {
                ISeance seanceByFichier = fichier.getSeance(  );
                GSeance nextSeance = getSeanceHome(  ).getProchaineSeance( plugin );

                if ( ( nextSeance != null ) && !seanceByFichier.equals( nextSeance ) &&
                        ( ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.DOCUMENT_LIBRE ) ||
                        ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.COMPTE_RENDU ) ||
                        ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.LISTE_DE_DESIGNATION ) ) )
                {
                    _searchService.updateObjectInIndex( fichier, true, plugin );
                }
            }

            _fichierHome.update( fichier, plugin );
        }
    }

    /**
     *
     */
    protected String checkModification( HttpServletRequest request, FileItem fileDelib )
    {
        if ( isGestionAval(  ) && !_voeuAmendementBean.getType(  ).trim(  ).equals( OdsConstants.CONSTANTE_TYPE_A ) )
        {
            List<String> fileProblems = new ArrayList<String>(  );

            if ( !isValidFile( fileDelib, fileProblems ) )
            {
                String strFileProblem = ( fileProblems.get( 0 ) != null ) ? fileProblems.get( 0 ) : null;

                if ( ( strFileProblem != null ) && strFileProblem.trim(  ).equals( MESSAGE_LABEL_FICHIER_NO_PDF ) )
                {
                    return AdminMessageService.getMessageUrl( request, MESSAGE_LABEL_FICHIER_NO_PDF,
                        AdminMessage.TYPE_STOP );
                }
            }
        }

        // Appel au service spcifique
        String strReturn = checkModificationSpec( request, fileDelib );

        if ( ( strReturn != null ) && !strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            return strReturn;
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     *
     */
    @SuppressWarnings( "unchecked" )
    public ISeance getSeance( IBusinessItem entite )
    {
        Plugin plugin = getPlugin(  );

        GVoeuAmendement voeuAmendement = (GVoeuAmendement) entite;

        // Appel au service spcifique
        setFasciculeSpec( voeuAmendement );

        GSeance seance;

        if ( voeuAmendement.getSeance(  ) != null )
        {
            seance = voeuAmendement.getSeance(  );
        }
        else
        {
            seance = getSeanceHome(  ).getProchaineSeance( plugin );
        }

        return seance;
    }

    /**
     * Retourne la liste des signatures numriques SHA224 des fichiers (les
     * signatures sont spares par le caractre '-')
     *
     * @param voeuAmendement
     *            VoeuAmendement
     * @return La liste des signatures
     */
    @SuppressWarnings( "unchecked" )
    public String getSignature( IEntiteHorodatage entite )
    {
        GVoeuAmendement voeuAmendement = (GVoeuAmendement) entite;

        String signature = null;

        IFichier fichier = getFichier( voeuAmendement );

        if ( fichier != null )
        {
            if ( fichier.getFichier(  ) != null )
            {
                byte[] donnees = fichier.getFichier(  ).getDonnees(  );

                signature = ShaUtils.toSha224String( donnees );

                if ( signature == null )
                {
                    signature = "error_" + fichier.getNom(  );

                    AppLogService.error( "Erreur lors de la recuperation de la signature SHA 224 du fichier : " +
                        fichier.getNom(  ) );
                }
            }
        }

        return signature;
    }

    /**
    * retourne une demande pour la suppression du VA choisi
    * @param request la requete HTTP
    * @return interface de confirmation de suppression
    */
    public String getSuppressionVoeuAmendement( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        String strMessage = OdsConstants.CONSTANTE_CHAINE_VIDE;
        UrlItem url;

        int nIdVoeuAmendement = OdsUtils.getIntegerParameter( OdsParameters.ID_VOEUAMENDEMENT, request );

        if ( nIdVoeuAmendement != -1 )
        {
            GVoeuAmendement voeuAmendement = _voeuAmendementHome.findByPrimaryKey( nIdVoeuAmendement, plugin );

            if ( voeuAmendement == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            if ( voeuAmendement.getActeCourant(  ) != null )
            {
                return AdminMessageService.getMessageUrl( request, MESSAGE_ERROR_DELETE_VOEU, AdminMessage.TYPE_STOP );
            }
            else if ( ( voeuAmendement.getActes(  ) != null ) && !voeuAmendement.getActes(  ).isEmpty(  ) )
            {
                url = new UrlItem( JSP_URL_DO_SUPPRESSIONVOEU );
                url.addParameter( OdsParameters.ID_VOEUAMENDEMENT,
                    OdsConstants.CONSTANTE_CHAINE_VIDE + nIdVoeuAmendement );

                return AdminMessageService.getMessageUrl( request, MESSAGE_CONFIRM_DELETE_VOEU_WITH_ACTES_ANNULES,
                    url.getUrl(  ), AdminMessage.TYPE_CONFIRMATION );
            }

            if ( voeuAmendement.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_A ) ||
                    voeuAmendement.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_LR ) )
            {
                strMessage = MESSAGE_CONFIRMDELETEAMENDEMENT;
                url = new UrlItem( JSP_URL_DO_SUPPRESSIONAMENDEMENT );
            }
            else if ( voeuAmendement.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_VR ) )
            {
                strMessage = MESSAGE_CONFIRMDELETEVOEU;
                url = new UrlItem( JSP_URL_DO_SUPPRESSIONVOEU );
            }
            else
            {
                GSeance prochaineSeance = getSeanceHome(  ).getProchaineSeance( plugin );
                boolean bInscritOdjProchaineSeance = false;

                EntreeOrdreDuJourFilter filter = new EntreeOrdreDuJourFilter(  );
                filter.setIdTypeEntite( ITypeEntite.VOEUX );
                filter.setIdEntite( nIdVoeuAmendement );

                List<GEntreeOrdreDuJour> listEntreeOdj = _entreeOrdreDuJourHome.findEntreesListByFilter( filter, plugin );

                for ( GEntreeOrdreDuJour entree : listEntreeOdj )
                {
                    entree.setOrdreDuJour( _ordreDuJourHome.findByPrimaryKey( 
                            entree.getOrdreDuJour(  ).getIdOrdreDuJour(  ), plugin ) );

                    // On vrifie que le voeu non rattach n'est pas inscrit dans un ordre du jour publi
                    if ( entree.getOrdreDuJour(  ).getPublie(  ) )
                    {
                        return AdminMessageService.getMessageUrl( request, MESSAGE_CANTDELETEVOEUINSCRITODJPUBLIE,
                            AdminMessage.TYPE_ERROR );
                    }

                    // On vrifie que le voeu non rattach n'est pas inscrit dans un ordre du jour de la prochaine sance
                    else if ( entree.getOrdreDuJour(  ).getSeance(  ).getIdSeance(  ) == prochaineSeance.getIdSeance(  ) )
                    {
                        bInscritOdjProchaineSeance = true;
                    }
                }

                url = new UrlItem( JSP_URL_DO_SUPPRESSIONVOEU );

                if ( bInscritOdjProchaineSeance )
                {
                    strMessage = MESSAGE_CONFIRMDELETEVOEUINSCRITODJ;
                }
                else
                {
                    strMessage = MESSAGE_CONFIRMDELETEVOEU;
                }
            }

            url.addParameter( OdsParameters.ID_VOEUAMENDEMENT, OdsConstants.CONSTANTE_CHAINE_VIDE + nIdVoeuAmendement );

            return AdminMessageService.getMessageUrl( request, strMessage, url.getUrl(  ),
                AdminMessage.TYPE_CONFIRMATION );
        }
        else
        {
            return OdsConstants.CONSTANTE_CHAINE_VIDE;
        }
    }

    /**
     * supprime le voeuAmendement choisi
     * @param request la requete HTTP
     * @param strReturnDefault l'URL de retour par dfaut
     * @return url de gestion des voeux amendements
     */
    public String doSuppressionVoeuAmendement( HttpServletRequest request, String strReturnDefault )
    {
        Plugin plugin = getPlugin(  );

        if ( request.getParameter( OdsParameters.ID_VOEUAMENDEMENT ) != null )
        {
            int nIdVA = OdsUtils.getIntegerParameter( OdsParameters.ID_VOEUAMENDEMENT, request );
            GVoeuAmendement voeuAmendement = _voeuAmendementHome.findByPrimaryKey( nIdVA, plugin );

            if ( voeuAmendement == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            if ( voeuAmendement.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_VNR ) )
            {
                // On vrifie que le voeu non rattach n'est pas inscrit dans un ordre du jour publi
                EntreeOrdreDuJourFilter filter = new EntreeOrdreDuJourFilter(  );
                filter.setIdTypeEntite( ITypeEntite.VOEUX );
                filter.setIdEntite( voeuAmendement.getId(  ) );

                List<GEntreeOrdreDuJour> listEntreeOdj = _entreeOrdreDuJourHome.findEntreesListByFilter( filter, plugin );

                for ( GEntreeOrdreDuJour entree : listEntreeOdj )
                {
                    entree.setOrdreDuJour( _ordreDuJourHome.findByPrimaryKey( 
                            entree.getOrdreDuJour(  ).getIdOrdreDuJour(  ), plugin ) );

                    // On vrifie que le voeu non rattach n'est pas inscrit dans un ordre du jour publi
                    if ( entree.getOrdreDuJour(  ).getPublie(  ) )
                    {
                        return AdminMessageService.getMessageUrl( request, MESSAGE_CANTDELETEVOEUINSCRITODJPUBLIE,
                            AdminMessage.TYPE_ERROR );
                    }
                }

                // On supprime l'ensemble des entres de l'ordre du jour associ au VA
                for ( GEntreeOrdreDuJour entree : listEntreeOdj )
                {
                    _entreeOrdreDuJourHome.remove( entree, plugin );
                }
            }

            try
            {
                voeuAmendement.setFichier( _fichierHome.findByPrimaryKey( voeuAmendement.getFichier(  ).getId(  ),
                        plugin ) );

                FichierPhysique fichierPhysique = _fichierPhysiqueHome.findByPrimaryKey( voeuAmendement.getFichier(  )
                                                                                                       .getFichier(  )
                                                                                                       .getIdFichier(  ),
                        plugin );

                if ( isGestionAval(  ) && ( voeuAmendement.getDeliberation(  ) != null ) )
                {
                    voeuAmendement.setDeliberation( _fichierHome.findByPrimaryKey( 
                            voeuAmendement.getDeliberation(  ).getId(  ), plugin ) );
                }

                // Appel au service spcifique
                removeReleveSpec( voeuAmendement );

                // Suppression de l'historique du VA 
                _historiqueHome.remove( voeuAmendement, plugin );

                boolean bArchive = false;
                GSeance nextSeance = getSeanceHome(  ).getProchaineSeance( plugin );

                // Appel au service spcifique
                setSeanceSpec( voeuAmendement );

                if ( ( nextSeance != null ) && ( voeuAmendement != null ) && ( voeuAmendement.getSeance(  ) != null ) )
                {
                    GSeance seanceByVA = voeuAmendement.getSeance(  );

                    if ( seanceByVA.equals( nextSeance ) )
                    {
                        bArchive = false;
                    }
                }

                _searchService.deleteObjectFromIndex( voeuAmendement, bArchive, plugin );

                // Suppression du VA en base de donnes
                _voeuAmendementHome.remove( voeuAmendement, plugin );

                // Suppression du panier
                _panierHome.removeDocument( voeuAmendement.getFichier(  ).getId(  ), plugin );

                // Suppression de l'historique du fichier associ au VA 
                _historiqueHome.remove( voeuAmendement.getFichier(  ), plugin );

                // Suppression du fichier
                GFichier fichier = voeuAmendement.getFichier(  );

                if ( ( fichier != null ) && ( fichier.getSeance(  ) != null ) )
                {
                    ISeance seanceByFichier = fichier.getSeance(  );
                    nextSeance = getSeanceHome(  ).getProchaineSeance( plugin );

                    if ( ( nextSeance != null ) && !seanceByFichier.equals( nextSeance ) &&
                            ( ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.DOCUMENT_LIBRE ) ||
                            ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.COMPTE_RENDU ) ||
                            ( fichier.getTypdeDocument(  ).getId(  ) == ITypeDocument.LISTE_DE_DESIGNATION ) ) )
                    {
                        _searchService.deleteObjectFromIndex( fichier, true, plugin );
                    }
                }

                _fichierHome.remove( fichier, plugin );
                _fichierPhysiqueHome.remove( fichierPhysique, plugin );

                // Suppression des actes rattachs qui sont annuls et de leur dlibration
                if ( isGestionAval(  ) && ( voeuAmendement.getActes(  ) != null ) &&
                        !voeuAmendement.getActes(  ).isEmpty(  ) )
                {
                    for ( Acte<GSeance, GFichier> acte : voeuAmendement.getActes(  ) )
                    {
                        acte = _acteHome.findByPrimaryKey( acte.getId(  ), plugin );

                        if ( acte.getDeliberationFinale(  ) != null )
                        {
                            fichier = _fichierHome.findByPrimaryKey( acte.getDeliberationFinale(  ).getId(  ), plugin );
                            _acteHome.remove( acte, plugin );
                            _historiqueHome.remove( fichier, plugin );
                            _panierHome.removeDocument( fichier.getId(  ), plugin );
                            _fichierHome.remove( fichier, plugin );
                            _fichierPhysiqueHome.remove( fichier.getFichier(  ), plugin );
                        }
                        else
                        {
                            _acteHome.remove( acte, plugin );
                        }
                    }
                }

                // On vide l'objet VA en session
                String strType = voeuAmendement.getType(  );
                _voeuAmendementBean = newVoeuAmendement(  );

                return getHomeUrl( request, strType, strReturnDefault );
            }
            catch ( AppException ae )
            {
                AppLogService.error( ae );

                if ( ae.getInitialException(  ) instanceof SQLException )
                {
                    return AdminMessageService.getMessageUrl( request, MESSAGE_CANNOT_DELETE_VA, AdminMessage.TYPE_STOP );
                }
            }
        }

        return getHomeUrl( request, _voeuAmendementBean.getType(  ), strReturnDefault );
    }

    /**
     * Retourne la map contenant les lments utiles au template HTML de la liste
     * @param request Requte HTTP
     * @param strType Amendement, voeu rattach, voeu non rattach
     * @param strUrlJSP URL de la JSP
     * @param strUrlRetour URL de retour
     * @return model Map contenant les lments utiles au template HTML de la liste
     */
    public HashMap<String, Object> getVoeuAmendementList( HttpServletRequest request, String strType, String strUrlJSP,
        String strUrlRetour )
    {
        Plugin plugin = getPlugin(  );

        // Rinitialisation des variables de session
        _bHasChanged = false;
        _strCurrentPageIndex = Paginator.getPageIndex( request, Paginator.PARAMETER_PAGE_INDEX, _strCurrentPageIndex );
        _nItemsPerPage = Paginator.getItemsPerPage( request, Paginator.PARAMETER_ITEMS_PER_PAGE, _nItemsPerPage,
                _nDefaultItemsPerPage );

        // Initialisation des variables locales
        int nIdOdj = OdsUtils.getIntegerParameter( OdsParameters.ID_ODJ, request );
        String strUrlAppelant = strUrlJSP;
        GOrdreDuJour ordreDuJour = null;
        GSeance seanceLocal = null;

        // Rcupration de la sance
        if ( ( request.getParameter( OdsParameters.ID_SEANCE ) != null ) &&
                !request.getParameter( OdsParameters.ID_SEANCE ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            int nIdSeanceLocal = OdsUtils.getIntegerParameter( OdsParameters.ID_SEANCE, request );
            seanceLocal = getSeanceHome(  ).findByPrimaryKey( nIdSeanceLocal, plugin );
        }

        if ( ( seanceLocal == null ) && isGestionAval(  ) )
        {
            // Si le VA en session a t rinitialis, on affiche un message d'erreur
            if ( _voeuAmendementBean == null )
            {
                throw new AppException( I18nService.getLocalizedString( MESSAGE_SESSION_EXPIREE, request.getLocale(  ) ) );
            }

            if ( _voeuAmendementBean.getSeance(  ) == null )
            {
                // Appel au service spcifique
                setFasciculeSpec( true );

                seanceLocal = _voeuAmendementBean.getSeance(  );
            }
            else
            {
                seanceLocal = _seance;
            }
        }
        else if ( ( seanceLocal == null ) && !isGestionAval(  ) )
        {
            seanceLocal = getSeanceHome(  ).getProchaineSeance( plugin );
        }

        // Initialisation du filtre
        GVoeuAmendementFilter voeuAmendementFilter = getVoeuAmendementSelectionListFilterSpec( request, strUrlJSP );
        voeuAmendementFilter.setType( strType );

        if ( strUrlJSP.equals( _ordreDuJourService.getUrlSelectionVoeu(  ) ) )
        {
            // Si la liste des VAs est appele par le module d'administration de l'ordre du jour, les VAs
            // doivent tre de la mme formation de conseil et de la mme sance que l'ordre du jour 
            if ( nIdOdj != -1 )
            {
                ordreDuJour = _ordreDuJourHome.findByPrimaryKey( nIdOdj, plugin );
                OdsUtils.testNullObject( ordreDuJour, request );

                // Utilis par le paginator
                strUrlAppelant += ( OdsConstants.CONSTANTE_POINT_INTERROGATION + getParameterPluginName(  ) +
                OdsConstants.CONSTANTE_ET + OdsParameters.ID_ODJ + OdsConstants.CONSTANTE_EGAL + nIdOdj );

                if ( ordreDuJour.getFormationConseil(  ) != null )
                {
                    voeuAmendementFilter.setIdFormationConseil( ordreDuJour.getFormationConseil(  )
                                                                           .getIdFormationConseil(  ) );
                }

                if ( ordreDuJour.getSeance(  ) != null )
                {
                    voeuAmendementFilter.setIdSeance( ordreDuJour.getSeance(  ).getIdSeance(  ) );

                    seanceLocal = getSeanceHome(  ).findByPrimaryKey( ordreDuJour.getSeance(  ).getIdSeance(  ), plugin );
                }

                getVoeuAmendementListSpec( request, voeuAmendementFilter );
            }
        }
        else if ( estPourSelectionParents( strUrlJSP ) )
        {
            voeuAmendementFilter.setType( voeuAmendementFilter.getAllString(  ) );
            voeuAmendementFilter.setTypeOrdreDuJour( getTypeODJEnumDefinitif(  ) );
            _nIdPublicationSelected = getIdPublie( request, voeuAmendementFilter );
            voeuAmendementFilter.setIdSeance( seanceLocal.getIdSeance(  ) );
        }
        else
        {
            _nIdPublicationSelected = getIdPublie( request, voeuAmendementFilter );
            voeuAmendementFilter.setIdSeance( seanceLocal.getIdSeance(  ) );
        }

        // Liste des VAs
        List<GVoeuAmendement> voeuAmendements = _voeuAmendementHome.findVoeuAmendementListByFilter( voeuAmendementFilter,
                plugin );

        OdsUtils.testNullObject( voeuAmendements, request );

        // On instancie un tableau qui va contenir tous les lments qui vont tre supprims
        List<GVoeuAmendement> vaToDelete = new ArrayList<GVoeuAmendement>(  );

        // Initialisation du modle
        HashMap<String, Object> model = getVoeuAmendementSelectionListSpec( request, voeuAmendements, vaToDelete,
                strUrlJSP );

        // Traitement pour les parents lors d'une modification d'un VA
        if ( estPourSelectionParents( strUrlJSP ) )
        {
            // Si le VA en session a t rinitialis, on affiche un message d'erreur
            if ( _voeuAmendementBean == null )
            {
                throw new AppException( I18nService.getLocalizedString( MESSAGE_SESSION_EXPIREE, request.getLocale(  ) ) );
            }

            // Liste des VAs avec descendants
            List<GVoeuAmendement> voeuAmendementsWithDescendant = new ArrayList<GVoeuAmendement>(  );
            getVoeuxAmendementsDescendant( voeuAmendementsWithDescendant, voeuAmendements, plugin );

            for ( GVoeuAmendement va : voeuAmendementsWithDescendant )
            {
                // Si la formation du conseil rfrence est diffrente => A SUPPRIMER
                // Si la sance est diffrente => A SUPPRIMER
                // Si le VA est le voeuamendement courant => A SUPPRIMER
                if ( ( va.getId(  ) == _voeuAmendementBean.getId(  ) ) ||
                        ( ( _voeuAmendementBean.getFormationConseil(  ) != null ) &&
                        ( va.getFormationConseil(  ).getIdFormationConseil(  ) != _voeuAmendementBean.getFormationConseil(  )
                                                                                                         .getIdFormationConseil(  ) ) ) ||
                        ( ( va.getSeance(  ) != null ) && ( _voeuAmendementBean.getSeance(  ) != null ) &&
                        ( va.getSeance(  ).getIdSeance(  ) != _voeuAmendementBean.getSeance(  ).getIdSeance(  ) ) ) )
                {
                    vaToDelete.add( va );
                }
                else
                {
                    // On vrifie que les VAs proposs ne sont pas dans la liste des VAs descendants du VA courant
                    List<GVoeuAmendement> voeuAmendementsWithDescendantsSaufCourant = new ArrayList<GVoeuAmendement>(  );
                    getVoeuxAmendementsDescendant( _voeuAmendementBean.getId(  ), plugin,
                        voeuAmendementsWithDescendantsSaufCourant );

                    for ( GVoeuAmendement voeuAmendementDescendant : voeuAmendementsWithDescendantsSaufCourant )
                    {
                        if ( va.getId(  ) == voeuAmendementDescendant.getId(  ) )
                        {
                            vaToDelete.add( va );

                            break;
                        }
                    }
                }
            }

            if ( _voeuAmendementBean.getFormationConseil(  ) != null )
            {
                model.put( MARK_FORMATION_CONSEIL, _voeuAmendementBean.getFormationConseil(  ) );
            }

            voeuAmendements = voeuAmendementsWithDescendant;
        }

        OdsUtils.testNullObject( voeuAmendements, request );

        // Traitement pour les PDDs
        int pddPrimaryKey = OdsUtils.getIntegerParameter( OdsParameters.ID_PDD, request );

        if ( pddPrimaryKey != -1 )
        {
            model.put( OdsParameters.ID_PDD, OdsConstants.CONSTANTE_CHAINE_VIDE + pddPrimaryKey );

            GPdd pdd = _projetDeliberationHome.findByPrimaryKey( pddPrimaryKey, plugin );
            OdsUtils.testNullObject( pdd, request );

            model.put( MARK_FORMATION_CONSEIL, pdd.getFormationConseil(  ) );

            for ( GVoeuAmendement va : voeuAmendements )
            {
                // Les VAs disponibles sont ceux de la mme formation de conseil que le projet 
                if ( ( va.getFormationConseil(  ) == null ) ||
                        ( va.getFormationConseil(  ).getIdFormationConseil(  ) != pdd.getFormationConseil(  )
                                                                                         .getIdFormationConseil(  ) ) )
                {
                    vaToDelete.add( va );
                }

                // Et qui ne sont pas dj rattachs  ce projet
                else if ( pdd.containsVoeuxAmandements( va ) )
                {
                    vaToDelete.add( va );
                }
            }
        }

        // Traitement pour l'ordre du jour
        if ( strUrlJSP.equals( _ordreDuJourService.getUrlSelectionVoeu(  ) ) )
        {
            List<GEntreeOrdreDuJour> listEntree = _entreeOrdreDuJourHome.findEntreesListByIdOrdreDuJour( nIdOdj, plugin );

            if ( listEntree != null )
            {
                for ( GVoeuAmendement va : voeuAmendements )
                {
                    for ( GEntreeOrdreDuJour entreeOrdreDuJour : listEntree )
                    {
                        if ( ( entreeOrdreDuJour.getIdTypeEntite(  ) == ITypeEntite.VOEUX ) &&
                                ( entreeOrdreDuJour.getIdEntite(  ) == va.getId(  ) ) )
                        {
                            vaToDelete.add( va );

                            break;
                        }
                    }
                }
            }
        }

        // Suppression des VAs qui ne doivent pas tre affichs
        voeuAmendements.removeAll( vaToDelete );
        Collections.sort( voeuAmendements,
            new VoeuAmendementComparator<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>(  ) );

        ReferenceList refListPublie = new ReferenceList(  );
        initRefListPublie( refListPublie, request );

        Paginator paginator = new Paginator( voeuAmendements, _nItemsPerPage, strUrlAppelant,
                Paginator.PARAMETER_PAGE_INDEX, _strCurrentPageIndex );

        if ( ordreDuJour != null )
        {
            model.put( GOrdreDuJour.MARK_ODJ, ordreDuJour );
        }

        model.put( OdsMarks.MARK_GESTION_AVAL, isGestionAval(  ) );
        model.put( MARK_LISTE_PUBLIE, refListPublie );
        model.put( MARK_ID_PUBLICATION_SELECTED, _nIdPublicationSelected );
        model.put( MARK_LISTE_VOEUAMENDEMENT, paginator.getPageItems(  ) );
        model.put( OdsMarks.MARK_NOMBRE_RESULTATS, voeuAmendements.size(  ) );
        model.put( OdsMarks.MARK_SEANCE, seanceLocal );
        model.put( OdsMarks.MARK_PAGINATOR, paginator );
        model.put( OdsMarks.MARK_NB_ITEMS_PER_PAGE, OdsConstants.CONSTANTE_CHAINE_VIDE + _nItemsPerPage );
        model.put( MARK_URL_JSP_APPELANT, strUrlJSP );
        model.put( MARK_URL_RETOUR, strUrlRetour );

        return model;
    }

    /**
     * permet de savoir si l'url donn en paramtre correspond  l'url pour la gestion des parents d'un VA
     * @param strUrl l'url  tester
     * @return VRAI si il s'agit de l'url de gestion des parents, FAUX sinon
     */
    protected boolean estPourSelectionParents( String strUrl )
    {
        Boolean bRetour = false;

        if ( strUrl.equals( CONSTANTE_URL_JSP_APPELANT_AMENDEMENT_PARENT_CREATION ) ||
                strUrl.equals( CONSTANTE_URL_JSP_APPELANT_AMENDEMENT_PARENT_MODIFICATION ) ||
                strUrl.equals( CONSTANTE_URL_JSP_APPELANT_VOEU_RATTACHE_PARENT_CREATION ) ||
                strUrl.equals( CONSTANTE_URL_JSP_APPELANT_VOEU_RATTACHE_PARENT_MODIFICATION ) ||
                strUrl.equals( CONSTANTE_URL_JSP_APPELANT_VOEU_NON_RATTACHE_PARENT_CREATION ) ||
                strUrl.equals( CONSTANTE_URL_JSP_APPELANT_VOEU_NON_RATTACHE_PARENT_MODIFICATION ) )
        {
            bRetour = true;
        }

        return bRetour;
    }

    /**
     *
     * retourne une liste de voeux amendements contenant la liste des va passes en paramtre plus pour chaque va de la liste les  descendants du va
     * @param listVoeuAmendementsWithDescendant la liste des VA avec descendants
     * @param listVoeuAmendements la liste des VA
     * @param plugin le plugin
     */
    protected void getVoeuxAmendementsDescendant( List<GVoeuAmendement> listVoeuAmendementsWithDescendant,
        List<GVoeuAmendement> listVoeuAmendements, Plugin plugin )
    {
        for ( GVoeuAmendement va : listVoeuAmendements )
        {
            if ( !listVoeuAmendementsWithDescendant.contains( va ) )
            {
                listVoeuAmendementsWithDescendant.add( va );
            }

            getVoeuxAmendementsDescendant( va.getId(  ), plugin, listVoeuAmendementsWithDescendant );
        }
    }

    /**
     *
     * retourne la liste des voeux amendements descendant du voeu amendement d'id nIdVoeuAmendement
     * @param nIdVoeuAmendement l'id du voeu amendement
     * @param plugin yhe plugin
     * @param listVaDescendant la liste des VA descendants
     */
    public void getVoeuxAmendementsDescendant( int nIdVoeuAmendement, Plugin plugin,
        List<GVoeuAmendement> listVaDescendant )
    {
        List<GVoeuAmendement> listEnfant = _voeuAmendementHome.findVoeuxAmendementsEnfant( nIdVoeuAmendement, plugin );

        if ( !listEnfant.isEmpty(  ) )
        {
            for ( GVoeuAmendement vaEnfant : listEnfant )
            {
                if ( !listVaDescendant.contains( vaEnfant ) )
                {
                    listVaDescendant.add( vaEnfant );
                }
            }

            for ( GVoeuAmendement va : listEnfant )
            {
                getVoeuxAmendementsDescendant( va.getId(  ), plugin, listVaDescendant );
            }
        }
    }

    /**
     * retourne un avertissement pour signaler que le VA n'est associ  aucun PDD
     * @param request la requete HTTP
     * @param strReturnDefault l'URL de retour par dfaut
     * @return interface de confirmation de suppression
     */
    public String getAvertissementEnregistrementSansPDD( HttpServletRequest request, String strReturnDefault )
    {
        UrlItem url;

        // Si le VA en session a t rinitialis, on affiche un message d'erreur
        if ( _voeuAmendementBean == null )
        {
            url = new UrlItem( strReturnDefault );

            return AdminMessageService.getMessageUrl( request, MESSAGE_SESSION_EXPIREE, url.getUrl(  ),
                AdminMessage.TYPE_ERROR );
        }

        if ( _bUrlRetour )
        {
            String strReturn = null;
            strReturn = AppPathService.getBaseUrl( request ) + _strUrlRetour + OdsConstants.CONSTANTE_ET +
                OdsParameters.ID_VOEUAMENDEMENT + OdsConstants.CONSTANTE_EGAL + _voeuAmendementBean.getId(  );

            if ( _ordreDuJourService.getUrlSelectionVoeuRetour(  ).equals( _strUrlRetour ) )
            {
                strReturn += ( OdsConstants.CONSTANTE_ET + OdsParameters.ID_VOEUAMENDEMENT +
                OdsConstants.CONSTANTE_EGAL + _voeuAmendementBean.getId(  ) );
            }

            url = new UrlItem( strReturn );

            _bUrlRetour = false;
            _strUrlRetour = null;
        }
        else
        {
            url = new UrlItem( getHomeUrl( request, _voeuAmendementBean.getType(  ), strReturnDefault ) );
        }

        return AdminMessageService.getMessageUrl( request, MESSAGE_AVERTISSEMENT_ENREGISTREMENT, url.getUrl(  ),
            AdminMessage.TYPE_WARNING );
    }

    /**
     * Retourne la map contenant les lments utiles au template HTML de la liste pour l'ODJ
     * @param request la requete HTTP
     * @return model la map contenant les lments utiles au template HTML de la liste pour l'ODJ
     */
    public HashMap<String, Object> getVoeuListForOdj( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        GOrdreDuJour ordreDuJour = newOrdreDuJour(  );
        GSeance seanceLocal = null;

        // Initialisation du filtre 
        GVoeuAmendementFilter filter = newVoeuAmendementFilter(  );
        filter.setType( OdsConstants.CONSTANTE_TYPE_VNR );

        // Les VAs doivent tre de la mme formation de conseil et de la mme sance que l'ordre du jour 
        int nIdEntree = OdsUtils.getIntegerParameter( OdsParameters.ID_ENTREE, request );

        if ( nIdEntree != -1 )
        {
            GEntreeOrdreDuJour entreeOrdreDuJour = _entreeOrdreDuJourHome.findByPrimaryKey( nIdEntree, plugin );

            ordreDuJour = _ordreDuJourHome.findByPrimaryKey( entreeOrdreDuJour.getOrdreDuJour(  ).getIdOrdreDuJour(  ),
                    plugin );
            entreeOrdreDuJour.setOrdreDuJour( ordreDuJour );
            model.put( MARK_ENTREE, entreeOrdreDuJour );

            if ( ordreDuJour.getFormationConseil(  ) != null )
            {
                filter.setIdFormationConseil( ordreDuJour.getFormationConseil(  ).getIdFormationConseil(  ) );
            }

            if ( ordreDuJour.getSeance(  ) != null )
            {
                filter.setIdSeance( ordreDuJour.getSeance(  ).getIdSeance(  ) );
                seanceLocal = getSeanceHome(  ).findByPrimaryKey( ordreDuJour.getSeance(  ).getIdSeance(  ), plugin );
            }
        }

        // On rcupre la liste des VAs 
        List<GVoeuAmendement> voeuAmendements = _voeuAmendementHome.findVoeuAmendementListByFilter( filter, plugin );

        // On instancie un tableau qui va contenir tous les lments qui vont tre supprims
        List<GVoeuAmendement> vaToDelete = new ArrayList<GVoeuAmendement>(  );

        // Traitement pour l'ordre du jour
        List<GEntreeOrdreDuJour> listEntree = _entreeOrdreDuJourHome.findEntreesListByIdOrdreDuJour( ordreDuJour.getIdOrdreDuJour(  ),
                plugin );

        if ( listEntree != null )
        {
            for ( GVoeuAmendement va : voeuAmendements )
            {
                for ( GEntreeOrdreDuJour entreeOrdreDuJour : listEntree )
                {
                    if ( ( entreeOrdreDuJour.getIdTypeEntite(  ) == ITypeEntite.VOEUX ) &&
                            ( entreeOrdreDuJour.getIdEntite(  ) == va.getId(  ) ) )
                    {
                        vaToDelete.add( va );

                        break;
                    }
                }
            }
        }

        // Suppression des VAs qui ne doivent pas tre affichs
        voeuAmendements.removeAll( vaToDelete );
        Collections.sort( voeuAmendements,
            new VoeuAmendementComparator<GVoeuAmendement, GSeance, GElu, GFichier, GPdd>(  ) );

        model.put( MARK_LISTE_VOEUAMENDEMENT, voeuAmendements );
        model.put( OdsMarks.MARK_NOMBRE_RESULTATS, voeuAmendements.size(  ) );
        model.put( OdsMarks.MARK_SEANCE, seanceLocal );

        return model;
    }

    /**
     * Retourne la map contenant les lments utiles au template HTML de confirmation de finalisation
     * @param request la requte http
     * @return model la map contenant les lments utiles au template HTML de confirmation de finalisation
     */
    public HashMap<String, Object> getConfirmFinaliser( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        int nIdVoeu = OdsUtils.getIntegerParameter( OdsParameters.ID_VOEUAMENDEMENT, request );
        GVoeuAmendement voeu = _voeuAmendementHome.findByPrimaryKey( nIdVoeu, plugin );
        OdsUtils.testNullObject( nIdVoeu, request );

        model.put( OdsMarks.MARK_VOEUAMENDEMENT, voeu );

        return model;
    }

    /**
     * Retourne la map contenant les lments utiles au template HTML de l'historique
     * @param request la requete
     * @return model la map contenant les lments utiles au template HTML de l'historique
     */
    public HashMap<String, Object> getHistoriqueVoeuAmendement( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );
        HashMap<String, Object> model = new HashMap<String, Object>(  );

        int nIdVoeuAmendement = OdsUtils.getIntegerParameter( OdsParameters.ID_VOEUAMENDEMENT, request );
        GVoeuAmendement voeuAmendement = _voeuAmendementHome.findByPrimaryKey( nIdVoeuAmendement, plugin );
        OdsUtils.testNullObject( voeuAmendement, request );

        List<Historique> historiques = _historiqueHome.getHistoriqueList( voeuAmendement, plugin );
        model.put( MARK_LISTE_HISTORIQUES, historiques );

        return model;
    }

    /**
     * Permet de savoir si il s'agit d'une demande de slection de PDD pour Voeu/Amendement
     * @param strUrlJspAppelant l'url de la JSP source.
     * @return TRUE, si c'est pour le traitement des Voeux/amendement, FALSE sinon
     */
    public boolean estPourSelectionVoeuAmendement( String strUrlJspAppelant )
    {
        boolean bRetour = false;

        if ( strUrlJspAppelant.equals( CONSTANTE_URL_JSP_APPELANT_PROJET ) ||
                strUrlJspAppelant.equals( CONSTANTE_URL_JSP_APPELANT_PROJET_AVAL ) ||
                strUrlJspAppelant.equals( CONSTANTE_URL_JSP_APPELANT_PROPOSITION ) ||
                strUrlJspAppelant.equals( CONSTANTE_URL_JSP_APPELANT_PROPOSITION_AVAL ) )
        {
            bRetour = true;
        }

        return bRetour;
    }

    /**
     * Rinitialise le voeuAmendement en session
     * @param request la requete
     * @param strReturnDefault l'URL de retour par dfaut
     * @return l'url en fonction di type du VA
     */
    public String doResetVoeuAmendement( HttpServletRequest request, String strReturnDefault )
    {
        _voeuAmendementBean = newVoeuAmendement(  );

        return getHomeUrl( request, _voeuAmendementBean.getType(  ), strReturnDefault );
    }

    /**
     * Mthode qui ajoute un dpositaire dans la liste des dpositaires du VA .
     * Retour sur l'interface de cration d'un amendement,voeu non rattach ou
     * voeu rattach en fonction du type du VA
     * @param request la requete
     * @param strReturnDefault l'URL de retour par dfaut
     * @return url l'interface de cration d'un va
     */
    public String doAddDepositaireCreation( HttpServletRequest request, String strReturnDefault )
    {
        // Mthode commune
        String strReturn = doAddDepositaire( request, strReturnDefault );

        if ( ( strReturn != null ) && !strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            return strReturn;
        }

        return getCreationUrl( request, _voeuAmendementBean.getType(  ), strReturnDefault );
    }

    /**
     * Mthode qui ajoute un dpositaire dans la liste des dpositaires du VA .
     * Retour sur l'interface de modification d'un amendement,voeu non rattach ou
     * voeu rattach en fonction du type du VA
     * @param request la requete
     * @param strReturnDefault l'URL de retour par dfaut
     * @return url de l'inteface de modification
     */
    public String doAddDepositaireModification( HttpServletRequest request, String strReturnDefault )
    {
        // Mthode commune
        String strReturn = doAddDepositaire( request, strReturnDefault );

        if ( ( strReturn != null ) && !strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            return strReturn;
        }

        strReturn = getModificationUrl( request, _voeuAmendementBean.getType(  ), strReturnDefault );
        strReturn += ( OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_SEANCE +
        OdsConstants.CONSTANTE_EGAL + _seance.getIdSeance(  ) );

        if ( request.getParameter( MARK_ID_ACTE ) != null )
        {
            int nIdActe = OdsUtils.getIntegerParameter( MARK_ID_ACTE, request );
            strReturn += ( OdsConstants.CONSTANTE_ET + OdsParameters.ID_ACTE + OdsConstants.CONSTANTE_EGAL + nIdActe );
        }

        return strReturn;
    }

    /**
     *
     */
    private String doAddDepositaire( HttpServletRequest request, String strReturnDefault )
    {
        Plugin plugin = getPlugin(  );

        int nIdElu = OdsUtils.getIntegerParameter( OdsParameters.ID_ELU, request );
        int nIdGroupe = OdsUtils.getIntegerParameter( OdsParameters.ID_GROUPE, request );

        // Si le VA en session a t rinitialis, on affiche un message d'erreur
        if ( _voeuAmendementBean == null )
        {
            UrlItem url = new UrlItem( strReturnDefault );

            return AdminMessageService.getMessageUrl( request, MESSAGE_SESSION_EXPIREE, url.getUrl(  ),
                AdminMessage.TYPE_ERROR );
        }

        List<String> listErrorMessage = new ArrayList<String>(  );

        // On sauvegarde en session les valeurs saisies par l'utilisateur
        setVoeuAmendementBean( request, listErrorMessage );

        // Si le VA stock en session a une liste d'lus nulle, on l'initialise 
        if ( _voeuAmendementBean.getDepositaires(  ) == null )
        {
            _voeuAmendementBean.setDepositaires( new ArrayList<IDepositaire>(  ) );
        }

        if ( ( request.getParameter( OdsParameters.ID_ELU ) != null ) &&
                ( request.getParameter( OdsParameters.ID_GROUPE ) != null ) )
        {
            if ( ( nIdElu != -1 ) && !isInTheDepositaireList( nIdElu, IDepositaire.DEPOSITAIRE_TYPE_ELU ) )
            {
                GElu elu = _eluHome.findByPrimaryKey( nIdElu, plugin );

                if ( elu != null )
                {
                    _voeuAmendementBean.getDepositaires(  ).add( elu );
                    _bHasChanged = true;
                }
            }
            else if ( ( nIdElu == -1 ) && ( nIdGroupe != -1 ) &&
                    !isInTheDepositaireList( nIdGroupe, IDepositaire.DEPOSITAIRE_TYPE_GROUPE ) )
            {
                IGroupePolitique groupe = _groupePolitiqueHome.findByPrimaryKey( nIdGroupe, plugin );
                _voeuAmendementBean.getDepositaires(  ).add( groupe );
                _bHasChanged = true;
            }
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * methode qui supprime  un dpositaire  en dans la liste des dpositaires du VA .
     * Retour sur l'interface de cration d'un amendement,voeu non rattach ou
     * voeu rattach en fonction du type du VA
     * @param request la requete
     * @param strReturnDefault l'URL de retour par dfaut
     * @return url de l'interface de cration d'un VA
     */
    public String doRemoveDepositaireCreation( HttpServletRequest request, String strReturnDefault )
    {
        // Mthode commune
        String strReturn = doRemoveDeposition( request, strReturnDefault );

        if ( ( strReturn != null ) && !strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            return strReturn;
        }

        return getCreationUrl( request, _voeuAmendementBean.getType(  ), strReturnDefault );
    }

    /**
     * methode qui supprime  un dpositaire en dans la liste des dpositaires du VA .
     * Retour sur l'interface de modification d'un amendement,voeu non rattach ou
     * voeu rattach en fonction du type du VA
     * @param request la requete
     * @param strReturnDefault l'URL de retour par dfaut
     * @return l'url de modification d'un va
     */
    public String doRemoveDepositaireModification( HttpServletRequest request, String strReturnDefault )
    {
        // Mthode commune
        String strReturn = doRemoveDeposition( request, strReturnDefault );

        if ( ( strReturn != null ) && !strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            return strReturn;
        }

        return getModificationUrl( request, _voeuAmendementBean.getType(  ), strReturnDefault );
    }

    /**
     *
     * @param request
     * @param strReturnDefault
     * @return
     */
    private String doRemoveDeposition( HttpServletRequest request, String strReturnDefault )
    {
        // Si le VA en session a t rinitialis, on affiche un message d'erreur
        if ( _voeuAmendementBean == null )
        {
            UrlItem url = new UrlItem( strReturnDefault );

            return AdminMessageService.getMessageUrl( request, MESSAGE_SESSION_EXPIREE, url.getUrl(  ),
                AdminMessage.TYPE_ERROR );
        }

        List<String> listErrorMessage = new ArrayList<String>(  );

        // On sauvegarde en session les valeurs saisies par l'utilisateur
        setVoeuAmendementBean( request, listErrorMessage );

        // Si le VA stock en session a une liste d'lus nulle, on l'initialise 
        if ( _voeuAmendementBean.getDepositaires(  ) == null )
        {
            _voeuAmendementBean.setDepositaires( new ArrayList<IDepositaire>(  ) );
        }

        int nIdElu = OdsUtils.getIntegerParameter( OdsParameters.ID_ELU_TO_REMOVE, request );
        int nIdGroupe = OdsUtils.getIntegerParameter( OdsParameters.ID_GROUPE_TO_REMOVE, request );

        if ( nIdElu != -1 )
        {
            removeDepositaireInDepositaireList( nIdElu, IDepositaire.DEPOSITAIRE_TYPE_ELU );
            _bHasChanged = true;
        }
        else if ( nIdGroupe != -1 )
        {
            removeDepositaireInDepositaireList( nIdGroupe, IDepositaire.DEPOSITAIRE_TYPE_GROUPE );
            _bHasChanged = true;
        }

        return OdsConstants.CONSTANTE_CHAINE_VIDE;
    }

    /**
     * Supprime le dpositaire d'id nkey de la liste de la liste des dpositaires du VA
     * @param nKey id du dpositaire  supprimer
     * @param nType le type de dpositaire  supprimer
     */
    @SuppressWarnings( "unchecked" )
    private void removeDepositaireInDepositaireList( int nKey, int nType )
    {
        IDepositaire depositaireToRemove = null;

        // Si le VA en session a t rinitialis, on ne fait rien
        if ( _voeuAmendementBean == null )
        {
            return;
        }

        List<IDepositaire> depositaires = _voeuAmendementBean.getDepositaires(  );

        for ( IDepositaire depositaire : depositaires )
        {
            if ( depositaire.getType(  ) == nType )
            {
                if ( ( nType == IDepositaire.DEPOSITAIRE_TYPE_ELU ) && ( ( (GElu) depositaire ).getIdElu(  ) == nKey ) )
                {
                    depositaireToRemove = depositaire;

                    break;
                }
                else if ( ( nType == IDepositaire.DEPOSITAIRE_TYPE_GROUPE ) &&
                        ( ( (GroupePolitique) depositaire ).getIdGroupe(  ) == nKey ) )
                {
                    depositaireToRemove = depositaire;

                    break;
                }
            }
        }

        if ( depositaireToRemove != null )
        {
            depositaires.remove( depositaireToRemove );
        }
    }

    /**
     * Modifie le statut slectionn pour lui attribuer le statut demand
     * @param request la requte
     * @return l'url de retour
     */
    public String doAssignStatut( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        if ( ( request.getParameter( OdsParameters.ID_VOEUAMENDEMENT ) != null ) &&
                !request.getParameter( OdsParameters.ID_VOEUAMENDEMENT ).trim(  )
                            .equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            int nIdVA = OdsUtils.getIntegerParameter( OdsParameters.ID_VOEUAMENDEMENT, request );
            int nIdStatut = OdsUtils.getIntegerParameter( OdsParameters.ID_STATUT, request );

            if ( nIdVA != -1 )
            {
                GVoeuAmendement va = _voeuAmendementHome.findByPrimaryKey( nIdVA, plugin );

                if ( va == null )
                {
                    return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                        AdminMessage.TYPE_ERROR );
                }

                if ( nIdStatut != -1 )
                {
                    Statut st = _statutHome.findByPrimaryKey( nIdStatut, plugin );
                    va.setStatut( st );
                }
                else
                {
                    va.setStatut( null );
                }

                _voeuAmendementHome.update( va, plugin );

                return AppPathService.getBaseUrl( request ) + JSP_URL_GET_STATUT_VA +
                OdsConstants.CONSTANTE_POINT_INTERROGATION + OdsParameters.ID_SEANCE + +_nIdSeanceSelected;
            }
        }

        return AdminMessageService.getMessageUrl( request, Messages.MANDATORY_FIELDS, AdminMessage.TYPE_STOP );
    }

    /**
     * Supprime le parent du voeu/amendement.
     * @param request la requete
     * @param strAction marqueur de l'action  effecuer (ajout ou suppression)
     * @param strReturn marqueur sur la page de retour (cration ou modification)
     * @param strReturnDefault l'URL de retour par dfaut
     * @return la page de cration ou de modification
     */
    public String doAddRemoveParent( HttpServletRequest request, String strAction, String strReturn,
        String strReturnDefault )
    {
        Plugin plugin = getPlugin(  );

        if ( ( ( request.getParameter( OdsParameters.A_FAIRE ) == null ) ||
                !request.getParameter( OdsParameters.A_FAIRE ).trim(  ).equals( OdsParameters.CONSTANTE_ACTION_CANCEL ) ) &&
                strAction.equals( CONSTANTE_ACTION_ADD_PARENT ) )
        {
            String[] strListIdParent = request.getParameterValues( OdsParameters.ID_PARENT );
            int nIdParent = -1;
            List<GPdd> listPddsParents = new ArrayList<GPdd>(  );

            if ( ( strListIdParent == null ) || ( strListIdParent.length == 0 ) )
            {
                return AdminMessageService.getMessageUrl( request, MESSAGE_NO_PARENT_SELECTED, AdminMessage.TYPE_STOP );
            }

            // Si le VA en session a t rinitialis, on affiche un message d'erreur
            if ( _voeuAmendementBean == null )
            {
                UrlItem url = new UrlItem( strReturnDefault );

                return AdminMessageService.getMessageUrl( request, MESSAGE_SESSION_EXPIREE, url.getUrl(  ),
                    AdminMessage.TYPE_ERROR );
            }

            int ncpt = 1;
            _voeuAmendementBean.setParents( new ArrayList<GVoeuAmendement>(  ) );

            for ( String strIdParent : strListIdParent )
            {
                try
                {
                    nIdParent = Integer.parseInt( strIdParent );
                }
                catch ( NumberFormatException ne )
                {
                    AppLogService.error( ne );
                }

                if ( ( nIdParent != -1 ) && ( nIdParent != _voeuAmendementBean.getId(  ) ) )
                {
                    GVoeuAmendement parent = _voeuAmendementHome.findByPrimaryKey( nIdParent, plugin );

                    if ( parent == null )
                    {
                        return AdminMessageService.getMessageUrl( request,
                            OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND, AdminMessage.TYPE_ERROR );
                    }

                    _voeuAmendementBean.getParents(  ).add( parent );

                    if ( ncpt == 1 )
                    {
                        // Initialisation des informations du premier voeu ou amendement slectionn
                        _voeuAmendementBean.setFormationConseil( parent.getFormationConseil(  ) );
                        _voeuAmendementBean.setDeposeExecutif( parent.getDeposeExecutif(  ) );

                        // Appel au service spcifique
                        doAddRemoveParentSpec( parent );

                        if ( ( parent.getReference(  ) != null ) && parent.getReference(  ).endsWith( "bis" ) )
                        {
                            _voeuAmendementBean.setReference( parent.getReference(  )
                                                                    .substring( 0,
                                    parent.getReference(  ).length(  ) - 3 ) + "ter" );
                            _voeuAmendementBean.setReferenceFront( parent.getReference(  )
                                                                         .substring( 0,
                                    parent.getReference(  ).length(  ) - 3 ) + "ter" );
                        }

                        else if ( parent.getReference(  ) != null )
                        {
                            _voeuAmendementBean.setReference( parent.getReference(  ) + "bis" );
                            _voeuAmendementBean.setReferenceFront( parent.getReference(  ) + "bis" );
                        }

                        ncpt++;
                    }

                    // On rcupre l'ensemble des PDDs des parents
                    for ( GPdd pdd : parent.getPdds(  ) )
                    {
                        if ( !isInThePddList( pdd.getId(  ), listPddsParents ) )
                        {
                            listPddsParents.add( pdd );
                        }
                    }
                }

                _voeuAmendementBean.setPdds( listPddsParents );
                _bHasChanged = true;
            }
        }
        else if ( strAction.equals( CONSTANTE_ACTION_REMOVE_PARENT ) )
        {
            _voeuAmendementBean.setParents( null );
        }

        if ( strReturn.equals( CONSTANTE_RETOUR_CREATION ) )
        {
            return getCreationUrl( request, _voeuAmendementBean.getType(  ), strReturnDefault );
        }

        if ( strReturn.equals( CONSTANTE_RETOUR_MODIFICATION ) )
        {
            return getModificationUrl( request, _voeuAmendementBean.getType(  ), strReturnDefault );
        }

        return getHomeUrl( request, _voeuAmendementBean.getType(  ), strReturnDefault );
    }

    /**
     * methode qui test si le pdd d'id nkey se trouve dans la liste des
     * des pdds
     *
     * @param listPdd la liste des pdds
     * @param nKey id du pdd
     * @return true si le pdd se trouve dans la liste false sinon
     */
    private boolean isInThePddList( int nKey, List<GPdd> listPdd )
    {
        for ( GPdd pdd : listPdd )
        {
            if ( pdd.getId(  ) == nKey )
            {
                return true;
            }
        }

        return false;
    }

    /**
     * Tlecharge un fichier.
     * Renvoie un flux de bytes du fichier dans l'objet HttpServletResponse response
     *
     * @param request la requete
     * @return fichierTelecharger le fichier  tlcharger
     */
    public GFichier doDownloadFichier( HttpServletRequest request )
    {
        Plugin plugin = getPlugin(  );

        GFichier fichierTelecharger = null;

        int nIdFichier = OdsUtils.getIntegerParameter( OdsParameters.ID_FICHIER, request );

        if ( nIdFichier != -1 )
        {
            fichierTelecharger = _fichierHome.findByPrimaryKey( nIdFichier, plugin );

            FichierPhysique fichierPhysique = _fichierPhysiqueHome.findByPrimaryKey( fichierTelecharger.getFichier(  )
                                                                                                       .getIdFichier(  ),
                    plugin );
            fichierTelecharger.setFichier( fichierPhysique );
        }

        return fichierTelecharger;
    }

    /**
     * Permet d'initialiser la liste des choix de publications
     * Les choix possibles: OUI,NON
     * @param refList la liste pour le menu droulant
     */
    protected void initRefListPublie( ReferenceList refList, HttpServletRequest request )
    {
        refList.addItem( -1, OdsConstants.CONSTANTE_CHAINE_VIDE );
        refList.addItem( 1, I18nService.getLocalizedString( OdsConstants.CONSTANTE_OUI, request.getLocale(  ) ) );
        refList.addItem( 0, I18nService.getLocalizedString( OdsConstants.CONSTANTE_NON, request.getLocale(  ) ) );
    }

    /**
     * methode qui initialise la liste des lus  afficher dans les interfaces de modification et crations
     * des Voeux et Amendements.
     * @param refListElu referenceList  initialiser
     * @param listElu liste des lus prsent en base
     * @param listEluAAfficher liste des lus  afficher
     * @param locale la locale
     */
    private void initListElu( ReferenceList refListElu, List<GElu> listElu, List<GElu> listEluAAfficher, Locale locale )
    {
        refListElu.addItem( -1, OdsConstants.CONSTANTE_CHAINE_VIDE );

        // Si le VA en session a t rinitialis, on ne fait rien
        if ( _voeuAmendementBean == null )
        {
            return;
        }

        for ( GElu elu : listElu )
        {
            // Si l'lu se trouve dj dans la liste des lus du VA stock en session, on ne l'affiche pas dans la combo lu		    
            if ( ( _voeuAmendementBean.getDepositaires(  ) != null ) &&
                    ( _voeuAmendementBean.getDepositaires(  ).size(  ) != 0 ) &&
                    isInTheDepositaireList( elu.getIdElu(  ), IDepositaire.DEPOSITAIRE_TYPE_ELU ) )
            {
                AppLogService.debug( _voeuAmendementBean );
            }
            else
            {
                listEluAAfficher.add( elu );

                // Construction pour chaque item de la liste lu du libell  afficher
                StringBuffer strLibelleElu = new StringBuffer(  );

                strLibelleElu.append( elu.getCivilite(  ) );
                strLibelleElu.append( OdsConstants.CONSTANTE_ESPACE );
                strLibelleElu.append( elu.getPrenomElu(  ) );
                strLibelleElu.append( OdsConstants.CONSTANTE_ESPACE );
                strLibelleElu.append( elu.getNomElu(  ) );
                strLibelleElu.append( OdsConstants.CONSTANTE_ESPACE );
                strLibelleElu.append( OdsConstants.CONSTANTE_OUVERTURE_ACCOLADE );
                strLibelleElu.append( elu.getGroupe(  ).getNomGroupe(  ) );
                strLibelleElu.append( OdsConstants.CONSTANTE_FERMETURE_ACCOLADE );

                refListElu.addItem( elu.getIdElu(  ), strLibelleElu.toString(  ) );
            }
        }
    }

    /**
      * Permet d'initialiser la liste des diffrents statuts
      * @param filter le filtre de statut
      * @param refList la liste pour le menu droulant
      */
    protected void initRefListStatut( IStatutFilter filter, ReferenceList refList )
    {
        Plugin plugin = getPlugin(  );

        List<Statut> listStatuts;

        if ( filter != null )
        {
            listStatuts = _statutHome.findStatutWithFilterList( filter, plugin );
        }
        else
        {
            listStatuts = _statutHome.findStatutList( plugin );
        }

        // Ajout dans la referenceList de l'ensemble des items reprsentant les statuts
        refList.addItem( -1, OdsConstants.CONSTANTE_CHAINE_VIDE );

        if ( ( listStatuts != null ) && !listStatuts.isEmpty(  ) )
        {
            for ( Statut statut : listStatuts )
            {
                refList.addItem( statut.getIdStatut(  ), statut.getLibelle(  ) );
            }
        }
    }

    /**
     * Permet d'initialiser la liste des choix de type de voeux (rattachs ou non)
     * @param refList la liste pour le menu droulant
     */
    private void initRefListVoeu( ReferenceList refList, HttpServletRequest request )
    {
        // Ajout dans la referenceList de l'item chane vide
        refList.addItem( -1, OdsConstants.CONSTANTE_CHAINE_VIDE );

        refList.addItem( 0, I18nService.getLocalizedString( PROPERTY_LABEL_VOEUX_NON_RATTACHES, request.getLocale(  ) ) );
        refList.addItem( 1, I18nService.getLocalizedString( PROPERTY_LABEL_VOEUX_RATTACHES, request.getLocale(  ) ) );
    }

    /**
     * Initialise la variable de session de seance
     * @param request la requte HTTP
     * @param bReinit true s'il faut rinitiliser la sance, false sinon
     */
    private void initSeance( HttpServletRequest request, boolean bReinit )
    {
        Plugin plugin = getPlugin(  );

        // Cas o l'on donne en paramtre l'id de la sance sur laquelle s'initialiser.
        if ( ( request.getParameter( OdsParameters.ID_SEANCE ) != null ) &&
                !request.getParameter( OdsParameters.ID_SEANCE ).trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            int nIdSeance = OdsUtils.getIntegerParameter( OdsParameters.ID_SEANCE, request );

            _seance = getSeanceHome(  ).findByPrimaryKey( nIdSeance, plugin );
        }

        // Cas o l'utilisateur a slectionn une sance dans un menu droulant
        else if ( ( _nIdSeanceSelected != -1 ) && ( _seance == null ) )
        {
            _seance = getSeanceHome(  ).findByPrimaryKey( _nIdSeanceSelected, plugin );
        }

        // Cas o la sance est nulle et que l'on est en gestion aval
        // La sance est donc la dernire sance archive
        else if ( isGestionAval(  ) &&
                ( bReinit || ( _seance == null ) ||
                ( ( getSeanceHome(  ).getProchaineSeance( plugin ) != null ) &&
                ( _seance.getIdSeance(  ) == getSeanceHome(  ).getProchaineSeance( plugin ).getIdSeance(  ) ) ) ) )
        {
            _seance = getSeanceHome(  ).getDerniereSeance( plugin );
        }

        // Cas o l'on n'est pas en gestion aval
        // La sance est donc la prochaine sance
        else if ( !isGestionAval(  ) && ( getSeanceHome(  ).getProchaineSeance( plugin ) != null ) )
        {
            _seance = getSeanceHome(  ).getProchaineSeance( plugin );
        }

        else
        {
            // Appel au service spcifique
            initSeanceSpec(  );
        }
    }

    /**
     * Retourne l'id du type de publication slectionn et initialise le filtre si une selection a t faite
     * @param request request
     * @param filter filtre initialis si une selection a t faite sur le type de publication
     * @return id du type de publication
     */
    protected int getIdPublie( HttpServletRequest request, GVoeuAmendementFilter filter )
    {
        int nCodePublieFilter = filter.getAllInt(  );

        if ( request.getParameter( OdsParameters.ID_PUBLICATION ) != null )
        {
            try
            {
                nCodePublieFilter = Integer.parseInt( ( request.getParameter( OdsParameters.ID_PUBLICATION ) ) );
            }
            catch ( NumberFormatException ne )
            {
                AppLogService.error( ne );
                nCodePublieFilter = _nIdPublicationSelected;
            }

            // Si la slection diffre de la slection stocke en session,
            // on rinitialise l'index de la page courante
            if ( nCodePublieFilter != _nIdPublicationSelected )
            {
                resetPageIndex(  );
            }
        }
        else
        {
            nCodePublieFilter = _nIdPublicationSelected;
        }

        filter.setIdPublie( nCodePublieFilter );

        return nCodePublieFilter;
    }

    /**
     * Retourne l'id du type de voeu  slectionn et initialise le filtre si une selection a t faite
     * @param request request
     * @param filter filtre initialis si une selection a t faite sur le type de commission
     * @return id de la commission
     */
    private int getIdVoeu( HttpServletRequest request, GVoeuAmendementFilter filter )
    {
        int nCodeVoeuFilter = filter.getAllInt(  );

        if ( request.getParameter( OdsParameters.ID_VOEUAMENDEMENT ) != null )
        {
            try
            {
                nCodeVoeuFilter = Integer.parseInt( ( request.getParameter( OdsParameters.ID_VOEUAMENDEMENT ) ) );
            }
            catch ( NumberFormatException ne )
            {
                AppLogService.error( ne );
                nCodeVoeuFilter = _nIdVoeuSelected;
            }

            // Si la slection diffre de la slection stocke en session,
            // On rinitialise l'index de la page courante
            if ( nCodeVoeuFilter != _nIdVoeuSelected )
            {
                resetPageIndex(  );
            }
        }
        else
        {
            nCodeVoeuFilter = _nIdVoeuSelected;
        }

        switch ( nCodeVoeuFilter )
        {
            case -1:
                filter.setType( OdsConstants.CONSTANTE_TYPE_ALLV );

                break;

            case 0:
                filter.setType( OdsConstants.CONSTANTE_TYPE_VNR );

                break;

            case 1:
                filter.setType( OdsConstants.CONSTANTE_TYPE_VR );

                break;

            default:
                break;
        }

        return nCodeVoeuFilter;
    }

    /**
     * retourne l'id de la sance slectionn et initialise le filtre si une selection a t faite
     * @param request request
     * @param filter filtre initialis si une selection a t faite sur le type de publication
     * @return id de la sance
     */
    protected int getIdSeance( HttpServletRequest request, GVoeuAmendementFilter filter )
    {
        if ( ( request.getParameter( OdsParameters.ID_SEANCE ) != null ) &&
                !request.getParameter( OdsParameters.ID_SEANCE ).trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            int nIdSeance = OdsUtils.getIntegerParameter( OdsParameters.ID_SEANCE, request );

            if ( nIdSeance > 0 )
            {
                filter.setIdSeance( nIdSeance );

                return nIdSeance;
            }
        }

        return -1;
    }

    /**
     * retourne l'id du statut slectionn et initialise le filtre si une selection a t faite
     * @param request request
     * @param filter filtre initialis si une selection a t faite sur le statut
     * @return id du statut
     */
    private int getIdStatut( HttpServletRequest request, GVoeuAmendementFilter filter )
    {
        int nCodeStatutFilter = filter.getAllInt(  );

        if ( request.getParameter( OdsParameters.ID_STATUT ) != null )
        {
            try
            {
                nCodeStatutFilter = Integer.parseInt( ( request.getParameter( OdsParameters.ID_STATUT ).trim(  ) ) );
            }
            catch ( NumberFormatException ne )
            {
                AppLogService.error( ne );
                nCodeStatutFilter = _nIdStatutSelected;
            }

            // Si la slection diffre de la slection stocke en session,
            // On rinitialise l'index de la page courante
            if ( nCodeStatutFilter != _nIdStatutSelected )
            {
                resetPageIndex(  );
            }
        }
        else
        {
            nCodeStatutFilter = _nIdStatutSelected;
        }

        filter.setIdStatut( nCodeStatutFilter );

        return nCodeStatutFilter;
    }

    /**
     * methode qui test si l'elu est deja dans la liste des personnes ayant dpos l'amendement trait
     * @param nKey id de l'lu
     * @param nTypeDepositaire le type du dpositaire (lu ou groupe)
     * @return true si le dpositaire est dj prsent dans la liste des dpositaires, false sinon
     */
    @SuppressWarnings( "unchecked" )
    private boolean isInTheDepositaireList( int nKey, int nTypeDepositaire )
    {
        // Si le VA en session a t rinitialis, on renvoi FALSE
        if ( _voeuAmendementBean == null )
        {
            return false;
        }

        for ( IDepositaire depositaire : _voeuAmendementBean.getDepositaires(  ) )
        {
            if ( depositaire.getType(  ) == nTypeDepositaire )
            {
                if ( ( nTypeDepositaire == IDepositaire.DEPOSITAIRE_TYPE_ELU ) &&
                        ( ( (GElu) depositaire ).getIdElu(  ) == nKey ) )
                {
                    return true;
                }
                else if ( ( nTypeDepositaire == IDepositaire.DEPOSITAIRE_TYPE_GROUPE ) &&
                        ( ( (GroupePolitique) depositaire ).getIdGroupe(  ) == nKey ) )
                {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * methode qui test si le pdd  est deja dans la liste des pdds du voeuAmendement
     * @param nKey id du pdd
     * @return true si le pdd  est deja dans la liste des pdds du voeuAmendement, false sinon
     */
    private boolean isInThePddList( int nKey )
    {
        // Si le VA en session a t rinitialis, on renvoi FALSE
        if ( _voeuAmendementBean == null )
        {
            return false;
        }

        for ( GPdd pdd : _voeuAmendementBean.getPdds(  ) )
        {
            if ( pdd.getId(  ) == nKey )
            {
                return true;
            }
        }

        return false;
    }

    /**
     * supprime le pdd d'id nkey de la liste des pdds du VA
     * @param nKey id du pdd  supprimer
     */
    private void removePddInPddList( int nKey )
    {
        // Si le VA en session a t rinitialis, on ne fait rien
        if ( _voeuAmendementBean == null )
        {
            return;
        }

        GPdd pddToRemove = null;
        List<GPdd> pdds = _voeuAmendementBean.getPdds(  );

        for ( GPdd pdd : pdds )
        {
            if ( pdd.getId(  ) == nKey )
            {
                pddToRemove = pdd;
            }
        }

        if ( pddToRemove != null )
        {
            pdds.remove( pddToRemove );
        }
    }

    /**
     * retourne l'URL de la page de cration d'un VA
     * @param request la requete
     * @param typeVa le type de VA
     * @param strReturnDefault l'URL de retour par dfaut
     * @return l'URL de la page de modification d'un VA
     */
    private String getCreationUrl( HttpServletRequest request, String typeVa, String strReturnDefault )
    {
        String strJspReturn = AppPathService.getBaseUrl( request );

        if ( typeVa != null )
        {
            if ( typeVa.equals( OdsConstants.CONSTANTE_TYPE_A ) || typeVa.equals( OdsConstants.CONSTANTE_TYPE_LR ) )
            {
                strJspReturn += JSP_URL_GET_CREATION_AMENDEMENT;
            }
            else if ( typeVa.equals( OdsConstants.CONSTANTE_TYPE_VR ) ||
                    typeVa.equals( OdsConstants.CONSTANTE_TYPE_VNR ) ||
                    typeVa.equals( OdsConstants.CONSTANTE_TYPE_ALLV ) )
            {
                strJspReturn += JSP_URL_GET_CREATION_VOEU;
            }

            return strJspReturn;
        }

        return strReturnDefault;
    }

    /**
     * Permet de confirmer la finalisation d'un acte
     *
     * @param request la requete HTTP
     * @param strReturnDefault l'URL de retour par dfaut
     * @return strUrl url de retour
     */
    public String doFinaliserActe( HttpServletRequest request, String strReturnDefault )
    {
        Plugin plugin = getPlugin(  );

        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        int nIdVoeu = OdsUtils.getIntegerParameter( OdsParameters.ID_VOEUAMENDEMENT, request );
        GVoeuAmendement voeu = _voeuAmendementHome.findByPrimaryKey( nIdVoeu, plugin );
        OdsUtils.testNullObject( nIdVoeu, request );

        Acte<GSeance, GFichier> acteCourant = voeu.getActeCourant(  );
        String strType = voeu.getType(  );

        // Si on a cliqu sur le bouton "Annuler"
        if ( request.getParameter( OdsParameters.CONSTANTE_ACTION_CANCEL ) != null )
        {
            strReturn = getModificationUrl( request, strType, nIdVoeu, strReturnDefault );
        }
        else
        {
            String strActe = request.getParameter( OdsParameters.CHECKBOX_DELETE );

            if ( strActe != null )
            {
                StatutActe statut = _statutActeHome.findByPrimaryKey( StatutActeEnum.A_TRANSMETTRE.getStatutOnlyWidthId(  )
                                                                                                  .getIdStatut(  ),
                        plugin );
                acteCourant.setStatutActe( statut );
                acteCourant.setIdTransmission( 0 );
                _acteHome.update( acteCourant, plugin );

                if ( _bUrlRetour )
                {
                    strReturn = AppPathService.getBaseUrl( request ) + _strUrlRetour;
                    _bUrlRetour = false;
                    _strUrlRetour = null;
                }
                else
                {
                    strReturn = getHomeUrl( request, strType, strReturnDefault );
                }
            }
            else
            {
                strReturn = AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_CHECKBOX,
                        AdminMessage.TYPE_STOP );
            }
        }

        return strReturn;
    }

    /**
     * @return _b boolean
     */
    public boolean isUrLRetour(  )
    {
        return _bUrlRetour;
    }

    /**
     * affecte la variable bUrlRetour retour
     * @param bUrlRetour =true retour a l'interface de gestion de jsp _strUrlRetour
     */
    public void setBurlRetour( boolean bUrlRetour )
    {
        _bUrlRetour = bUrlRetour;
    }

    /**
     * affecte  l'url de retour
     * @param strUrl url de retour
     */
    public void setUrlRetour( String strUrl )
    {
        _strUrlRetour = strUrl;
    }

    /**
     * retourne l'url de retour
     * @return l'url de retour
     */
    public String getUrlRetour(  )
    {
        return _strUrlRetour;
    }

    /**
    * Retourne le fichier appartenant au voeuAmendement
    *
    * @param voeuAmendement
    *            VoeuAmendement
    * @return fichier appartenant au voeuAmendement
    */
    private GFichier getFichier( GVoeuAmendement voeuAmendement )
    {
        Plugin plugin = getPlugin(  );

        GFichier fichier = _fichierHome.findByPrimaryKey( voeuAmendement.getFichier(  ).getId(  ), plugin );
        fichier.setFichier( _fichierPhysiqueHome.findByPrimaryKey( fichier.getFichier(  ).getIdFichier(  ), plugin ) );

        return fichier;
    }

    /**
     *
     */
    public IEntiteCertificatAffichage getEntite( Acte<GSeance, GFichier> acte )
    {
        Plugin plugin = getPlugin(  );

        GVoeuAmendement va = _voeuAmendementHome.findByPrimaryKey( acte.getIdEntite(  ), plugin );

        if ( ( va.getStatut(  ) != null ) &&
                ( ( va.getStatut(  ).getIdStatut(  ) == StatutEnum.ADOPTE.getId(  ) ) ||
                ( va.getStatut(  ).getIdStatut(  ) == StatutEnum.AMENDE.getId(  ) ) ) )
        {
            return va;
        }
        else
        {
            return null;
        }
    }

    /**
     *
     */
    @SuppressWarnings( "unchecked" )
    public Comparator<IBusinessItem> getComparatorClass(  )
    {
        return (Comparator) new VoeuAmendementCertificatComparator(  );
    }

    /**
     *
     */
    @SuppressWarnings( "unchecked" )
    public int getIndexerAction( IBusinessItem item )
    {
        int nActionIndexer;

        GVoeuAmendement va = (GVoeuAmendement) item;

        if ( va.getType(  ).equals( OdsConstants.VA_TYPE_A ) || va.getType(  ).equals( OdsConstants.VA_TYPE_LR ) )
        {
            nActionIndexer = ITypeEntite.AMENDEMENTS;
        }
        else
        {
            nActionIndexer = ITypeEntite.VOEUX;
        }

        return nActionIndexer;
    }

    /**
     *
     */
    public IBusinessItem createItemToDeleteInIndexer( int nIdItem )
    {
        Plugin plugin = getPlugin(  );

        return _voeuAmendementHome.findByPrimaryKey( nIdItem, plugin );
    }

    /**
     *
     */
    public boolean bIsNoEntitySelected( GEntreeOrdreDuJour entree )
    {
        return ( entree.getEntite(  ) == null ) ? true : false;
    }

    public String getErrorNoEntitySelected(  )
    {
        return MESSAGE_PAS_DE_VOEU_CHOISI;
    }

    /**
     *
     */
    @SuppressWarnings( "unchecked" )
    public void validateNumerotation( GEntreeOrdreDuJour entree )
    {
        Plugin plugin = getPlugin(  );

        if ( entree.getEntite(  ) != null )
        {
            GVoeuAmendement voeuAmendement = (GVoeuAmendement) entree.getEntite(  );
            voeuAmendement.setReferenceFront( voeuAmendement.getReference(  ) );

            // Appel au service spcifique
            setSeanceSpec( voeuAmendement );

            _searchService.updateObjectInIndex( voeuAmendement, false, plugin );
            _voeuAmendementHome.updateReferenceFront( voeuAmendement, plugin );
        }
    }

    /**
     *
     */
    @SuppressWarnings( "unchecked" )
    public String getPublicationOrdreDuJour( GEntreeOrdreDuJour entree, GOrdreDuJour ordreDuJour )
    {
        String strReturn = OdsConstants.CONSTANTE_CHAINE_VIDE;

        if ( ( ordreDuJour.getTypeOrdreDuJour(  ).getIdTypeOrdreDuJour(  ) == getTypeODJEnumDefinitif(  ) ) &&
                ( entree.getEntite(  ) != null ) && !( (GVoeuAmendement) entree.getEntite(  ) ).getEnLigne(  ) )
        {
            // Si le VA non rattach n'est pas publi
            return MESSAGE_EXISTE_ELEMENT_PAS_PUBLIE;
        }

        return strReturn;
    }

    /**
    * Mthode qui gre la publication du voeu amendement
    * @param request request
    * @param strType le type de VA
    * @param strReturnDefault l'URL de retour par dfaut
    * @return interface d'administration
    */
    public String doPublicationVoeuAmendement( HttpServletRequest request, String strType, String strReturnDefault )
    {
        Plugin plugin = getPlugin(  );

        Timestamp dateForVersion = new Timestamp( System.currentTimeMillis(  ) );
        int nIdVoeuAmendement = OdsUtils.getIntegerParameter( OdsParameters.ID_VOEUAMENDEMENT, request );
        int nIsPublier = OdsUtils.getIntegerParameter( OdsParameters.IS_PUBLIER_OR_DEPUBLIER, request );

        if ( ( nIdVoeuAmendement != -1 ) && ( nIsPublier != -1 ) )
        {
            boolean bEstPublication = ( nIsPublier == 1 );
            GVoeuAmendement voeuAmendement = _voeuAmendementHome.findByPrimaryKey( nIdVoeuAmendement, plugin );
            OdsUtils.testNullObject( voeuAmendement, request );

            if ( ( bEstPublication && voeuAmendement.getEnLigne(  ) ) ||
                    ( !bEstPublication && !voeuAmendement.getEnLigne(  ) ) )
            {
                return strReturnDefault;
            }

            if ( !voeuAmendement.getEnLigne(  ) )
            {
                if ( !isGestionAval(  ) && !voeuAmendement.getType(  ).equals( OdsConstants.CONSTANTE_TYPE_VNR ) &&
                        ( ( voeuAmendement.getPdds(  ) == null ) || voeuAmendement.getPdds(  ).isEmpty(  ) ) )
                {
                    return AdminMessageService.getMessageUrl( request, MESSAGE_AVERTISSEMENT_PUBLICATION,
                        AdminMessage.TYPE_CONFIRMATION );
                }
                else
                {
                    voeuAmendement.setVersion( voeuAmendement.getVersion(  ) + 1 );
                    voeuAmendement.setEnLigne( true );
                    voeuAmendement.setDatePublication( dateForVersion );

                    // Gestion de l'historique du VA
                    Historique historique = new Historique(  );
                    historique.setVersion( voeuAmendement.getVersion(  ) );
                    historique.setDatePublication( dateForVersion );
                    historique.setIdEntite( voeuAmendement.getId(  ) );
                    historique.setIdTypeEntite( ITypeEntite.VA );
                    _historiqueHome.create( historique, plugin );
                }
            }
            else
            {
                voeuAmendement.setEnLigne( false );
            }

            // Gestion du fichier associ
            GFichier fichier = _fichierHome.findByPrimaryKey( voeuAmendement.getFichier(  ).getId(  ), plugin );
            _fichierHome.publication( fichier.getId(  ), dateForVersion, fichier.getVersion(  ),
                voeuAmendement.getEnLigne(  ), plugin );

            // Si c'est une publication, on modifie l'historique du fichier
            if ( voeuAmendement.getEnLigne(  ) )
            {
                // Ajout de l'historique
                Historique historique = new Historique(  );
                historique.setVersion( fichier.getVersion(  ) );
                historique.setDatePublication( dateForVersion );
                historique.setIdEntite( fichier.getId(  ) );
                historique.setIdTypeEntite( ITypeEntite.FICHIERS );
                _historiqueHome.create( historique, plugin );
            }

            // Suppression du panier
            if ( !bEstPublication )
            {
                _panierHome.removeDocument( fichier.getId(  ), plugin );
            }

            // Dpublication de l'acte rattach s'il existe
            if ( !bEstPublication && isGestionAval(  ) )
            {
                if ( voeuAmendement.getActeCourant(  ) != null )
                {
                    _acteHome.publication( voeuAmendement.getActeCourant(  ).getId(  ), bEstPublication, plugin );
                    _fichierHome.publication( voeuAmendement.getActeCourant(  ).getDeliberationFinale(  ).getId(  ),
                        dateForVersion, voeuAmendement.getVersion(  ), bEstPublication, plugin );
                }
            }

            // Gestion du fichier de dlibration associ
            if ( isGestionAval(  ) && ( voeuAmendement.getDeliberation(  ) != null ) )
            {
                GFichier deliberation = _fichierHome.findByPrimaryKey( voeuAmendement.getDeliberation(  ).getId(  ),
                        plugin );
                _fichierHome.publication( deliberation.getId(  ), dateForVersion, deliberation.getVersion(  ),
                    voeuAmendement.getEnLigne(  ), plugin );
            }

            // Appel au service spcifique
            setFasciculeSpec( voeuAmendement );

            // Mise jour du VA en base
            _voeuAmendementHome.update( voeuAmendement, plugin );

            // On horodate la (d)publication
            HorodatageActionEnum action = ( bEstPublication ) ? HorodatageActionEnum.PUBLICATION
                                                              : HorodatageActionEnum.DE_PUBLICATION;
            _horodatageService.tracePublication( voeuAmendement, action );
        }

        return getHomeUrl( request, strType, strReturnDefault );
    }

    /**
     * Mthode qui ajoutes  les  Pdds selectionns par l'utilisateur dans la liste des pdds du VA .
     * Retour sur l'interface de modification d'un amendement,ou
     * voeu rattach en fonction du type du VA
     * @param request la requete
     * @param strReturnDefault l'URL de retour par dfaut
     * @return l'url de modification d'un VA
     */
    public String doAddPdds( HttpServletRequest request, String strReturnDefault )
    {
        Plugin plugin = getPlugin(  );

        if ( isCancel( request ) )
        {
            // Si le VA en session a t rinitialis, on affiche un message d'erreur
            if ( _voeuAmendementBean == null )
            {
                UrlItem url = new UrlItem( strReturnDefault );

                return AdminMessageService.getMessageUrl( request, MESSAGE_SESSION_EXPIREE, url.getUrl(  ),
                    AdminMessage.TYPE_ERROR );
            }

            String strReturn = getModificationUrl( request, _voeuAmendementBean.getType(  ),
                    _voeuAmendementBean.getId(  ), strReturnDefault );

            if ( isGestionAval(  ) )
            {
                strReturn += ( OdsConstants.CONSTANTE_ET + OdsParameters.ID_SEANCE + OdsConstants.CONSTANTE_EGAL +
                _seance.getIdSeance(  ) );
            }

            return strReturn;
        }
        else if ( isIdPDDs( request ) )
        {
            // On rcupre le VA de la base
            _voeuAmendementBean = _voeuAmendementHome.findByPrimaryKey( _voeuAmendementBean.getId(  ), plugin );

            if ( _voeuAmendementBean == null )
            {
                return AdminMessageService.getMessageUrl( request, OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND,
                    AdminMessage.TYPE_ERROR );
            }

            // Rcupration de la liste des Id pdds  ajouter au VA
            String[] strListePdd = getIdPDDs( request );

            if ( strListePdd.length != 0 )
            {
                for ( String strIdPdd : strListePdd )
                {
                    int nIdPdd;

                    try
                    {
                        nIdPdd = Integer.parseInt( strIdPdd );
                    }
                    catch ( NumberFormatException ne )
                    {
                        return getHomeUrl( request, _voeuAmendementBean.getType(  ), strReturnDefault );
                    }

                    if ( !isInThePddList( nIdPdd ) )
                    {
                        GPdd pdd = _projetDeliberationHome.findByPrimaryKey( nIdPdd, plugin );

                        if ( pdd == null )
                        {
                            return AdminMessageService.getMessageUrl( request,
                                OdsProperties.PROPERTY_MESSAGE_OBJECT_NOT_FOUND, AdminMessage.TYPE_ERROR );
                        }

                        if ( pdd.getModeClassement(  ).equals( OdsConstants.CONSTANTE_TYPE_AUTOMATIQUE ) )
                        {
                            // Mode de classement automatique => on doit classer les VA
                            _projetDeliberationService.classeVA( _voeuAmendementBean, pdd );
                        }
                        else if ( pdd.getModeClassement(  ).equals( OdsConstants.CONSTANTE_TYPE_MANUEL ) )
                        {
                            // Mode de classement manuel => les VA sont insrs en fin de liste
                            _voeuAmendementBean.getPdds(  ).add( pdd );
                            _voeuAmendementHome.update( _voeuAmendementBean, plugin );
                        }
                    }
                }

                String strReturn = getModificationUrl( request, _voeuAmendementBean.getType(  ),
                        _voeuAmendementBean.getId(  ), strReturnDefault );

                if ( isGestionAval(  ) )
                {
                    strReturn += ( OdsConstants.CONSTANTE_ET + OdsParameters.ID_SEANCE + +_seance.getIdSeance(  ) );
                }

                return strReturn;
            }
            else
            {
                String strMessage = null;

                if ( ( request.getParameter( OdsParameters.TYPE ) != null ) &&
                        request.getParameter( OdsParameters.TYPE ).equals( IPDD.CONSTANTE_TYPE_PROJET ) )
                {
                    strMessage = MESSAGE_PAS_DE_PROJET_CHOISIE;
                }
                else
                {
                    strMessage = MESSAGE_PAS_DE_PROPOSITION_CHOISIE;
                }

                return AdminMessageService.getMessageUrl( request, strMessage, AdminMessage.TYPE_STOP );
            }
        }
        else
        {
            String strMessage = null;

            if ( ( request.getParameter( OdsParameters.TYPE ) != null ) &&
                    request.getParameter( OdsParameters.TYPE ).equals( IPDD.CONSTANTE_TYPE_PROJET ) )
            {
                strMessage = MESSAGE_PAS_DE_PROJET_CHOISIE;
            }
            else
            {
                strMessage = MESSAGE_PAS_DE_PROPOSITION_CHOISIE;
            }

            return AdminMessageService.getMessageUrl( request, strMessage, AdminMessage.TYPE_STOP );
        }
    }

    /**
     * Mthode qui supprime  un Pdd dans la liste des pdds du VA .
     * Retour sur l'interface de modification d'un amendement,ou
     * voeu rattach en fonction du type du VA
     * @param request la requete
     * @param strReturnDefault l'URL de retour par dfaut
     * @return l'url de modificaion d'un VA
     */
    public String doRemovePdd( HttpServletRequest request, String strReturnDefault )
    {
        Plugin plugin = getPlugin(  );

        // Si le VA en session a t rinitialis, on affiche un message d'erreur
        if ( _voeuAmendementBean == null )
        {
            UrlItem url = new UrlItem( strReturnDefault );

            return AdminMessageService.getMessageUrl( request, MESSAGE_SESSION_EXPIREE, url.getUrl(  ),
                AdminMessage.TYPE_ERROR );
        }

        if ( ( _voeuAmendementBean != null ) && ( _voeuAmendementBean.getPdds(  ) == null ) )
        {
            _voeuAmendementBean.setPdds( new ArrayList<GPdd>(  ) );
        }

        int nIdPdd = OdsUtils.getIntegerParameter( OdsParameters.ID_PDD, request );

        if ( nIdPdd != -1 )
        {
            removePddInPddList( nIdPdd );
            _voeuAmendementHome.update( _voeuAmendementBean, plugin );
        }

        return getModificationUrl( request, _voeuAmendementBean.getType(  ), strReturnDefault );
    }

    /**
     *
     */
    public void doPublicationActe( Acte<GSeance, GFichier> acte, boolean bIsPublie, Timestamp dateForVersion )
    {
        // Aucune action
    }

    /**
     *
     */
    public String getAddParameter(  )
    {
        return CONSTANTE_AJOUTER_VOEU;
    }

    /**
     *
     */
    public String getTypeEntree(  )
    {
        return OdsConstants.CONSTANTE_ENTREE_TYPE_VOEU;
    }

    /**
     *
     */
    public String getIdParameter(  )
    {
        return OdsParameters.ID_VOEUAMENDEMENT;
    }

    /**
     *
     */
    public String getMarkForCertificat( int nIdTypeEntite )
    {
        return OdsMarks.MARK_LISTE_VAS;
    }

    public String getJspUrlGetModificationForActe( HttpServletRequest request, int nIdEntite, int nIdActe )
    {
        return AppPathService.getBaseUrl( request ) + JSP_URL_GET_MODIFICATION_VOEU_FOR_ACTE +
        OdsConstants.CONSTANTE_POINT_INTERROGATION + getParameterPluginName(  ) + OdsConstants.CONSTANTE_ET +
        getIdParameter(  ) + OdsConstants.CONSTANTE_EGAL + nIdEntite;
    }

    /**
     *
     */
    public boolean isForPositionNewEntree( GEntreeOrdreDuJour entree )
    {
        return false;
    }

    /**
     *
     */
    public boolean isOtherDataEnabled(  )
    {
        return true;
    }

    /**
     *
     */
    public boolean isRapporteurEnabled(  )
    {
        return true;
    }

    /**
     *
     */
    public boolean isForCsv(  )
    {
        return true;
    }

    /**
     *
     */
    public boolean isForModeClassement(  )
    {
        return true;
    }

    /**
     *
     */
    public boolean isFieldRequired( GEntreeOrdreDuJour entree, List<String> listErreurs )
    {
        return true;
    }

    /**
     *
     */
    public int createEntiteForODJ( GEntreeOrdreDuJour entree )
    {
        return -1;
    }

    /**
     *
     */
    public void deleteEntiteForODJ( int nIdEntite )
    {
        // Aucune action		
    }

    /**
     *
     */
    public void updateEntiteForODJ( GEntreeOrdreDuJour entree )
    {
        // Aucune action		
    }

    /**
     *
     */
    public String getLibelleCreationEntree( Locale locale )
    {
        return I18nService.getLocalizedString( PROPERTY_LIBELLE_CREATION_ENTREE_ODJ, locale );
    }

    /**
     *
     */
    public String getPageTitleCreationEntreeODJ(  )
    {
        return PROPERTY_TITLE_CREATION_ENTREE_ODJ;
    }

    /**
     *
     */
    public String getPageTitleModificationEntreeODJ(  )
    {
        return PROPERTY_TITLE_MODIFICATION_ENTREE_ODJ;
    }

    /**
     *
     */
    public void initEntreeOrdreDuJourSpec( HashMap<String, Object> model, Locale locale )
    {
        // Aucune action		
    }

    /**
    * Methode qui effectue la modification d'un amendement, d'un voeu rattach
    * en fonction du type du VoeuAmendement prsent en session.
    * Redirection vers l'interface d'administration des statuts VA
    * @param request request
    * @param strReturnDefault l'URL de retour par dfaut
    * @return template
    */
    public String doModificationStatutVA( HttpServletRequest request, String strReturnDefault )
    {
        Plugin plugin = getPlugin(  );

        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;

        if ( ( null != request.getParameter( OdsParameters.ID_VOEUAMENDEMENT ) ) &&
                !request.getParameter( OdsParameters.ID_VOEUAMENDEMENT ).trim(  )
                            .equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
        {
            // Pour la gestion en aval, on doit aussi traiter le fichier de dlibration
            FileItem fileDelib = multipartRequest.getFile( OdsParameters.FICHIER_DELIBERATION );

            // Si le VA en session a t rinitialis, on affiche un message d'erreur
            if ( _voeuAmendementBean == null )
            {
                UrlItem url = new UrlItem( strReturnDefault );

                return AdminMessageService.getMessageUrl( request, MESSAGE_SESSION_EXPIREE, url.getUrl(  ),
                    AdminMessage.TYPE_ERROR );
            }

            String strReturn = checkModification( request, fileDelib );

            if ( !strReturn.equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                return strReturn;
            }

            int nIdVa = OdsUtils.getIntegerParameter( OdsParameters.ID_VOEUAMENDEMENT, request );
            _voeuAmendementBean.setId( nIdVa );

            Timestamp dateForVersion = new Timestamp( System.currentTimeMillis(  ) );

            // Pour la gestion en aval on traite aussi le fichier de dlibration
            if ( !_voeuAmendementBean.getType(  ).trim(  ).equals( OdsConstants.CONSTANTE_TYPE_A ) &&
                    ( null != fileDelib ) && ( null != fileDelib.getName(  ) ) &&
                    !fileDelib.getName(  ).trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                if ( _voeuAmendementBean.getDeliberation(  ) != null )
                {
                    doModificationFichier( fileDelib, dateForVersion, true, request );
                }
                else
                {
                    _voeuAmendementBean.setDeliberation( newFichierInstance(  ) );
                    _voeuAmendementBean.setDeliberation( doCreationFichier( fileDelib, true, request ) );
                }
            }

            // Rcupration du statut
            int nStatut = OdsUtils.getIntegerParameter( OdsParameters.ID_STATUT, request );

            if ( nStatut != -1 )
            {
                Statut statut = _statutHome.findByPrimaryKey( nStatut, plugin );
                _voeuAmendementBean.setStatut( statut );
            }

            DateFormat dateFormat = null;
            dateFormat = new SimpleDateFormat( OdsConstants.CONSTANTE_PATTERN_DATE );
            dateFormat.setLenient( false );

            // Rcupration de la date du vote
            if ( ( request.getParameter( OdsParameters.DATE_VOTE ) != null ) &&
                    !request.getParameter( OdsParameters.DATE_VOTE ).trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                String strDateVote = request.getParameter( OdsParameters.DATE_VOTE );
                Timestamp timeStampDateVote;

                try
                {
                    timeStampDateVote = new Timestamp( dateFormat.parse( strDateVote ).getTime(  ) );
                    _voeuAmendementBean.setDateVote( timeStampDateVote );
                }
                catch ( ParseException pe )
                {
                    AppLogService.error( pe );
                }
            }
            else if ( ( request.getParameter( OdsParameters.DATE_VOTE ) != null ) &&
                    request.getParameter( OdsParameters.DATE_VOTE ).trim(  ).equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                _voeuAmendementBean.setDateVote( null );
            }

            // Rcupration du numro du voeu adopt
            if ( ( request.getParameter( OdsParameters.NUMERO_VOEU_ADOPTE ) != null ) &&
                    !request.getParameter( OdsParameters.NUMERO_VOEU_ADOPTE ).trim(  )
                                .equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                String strNumeroVoeu = request.getParameter( OdsParameters.NUMERO_VOEU_ADOPTE );
                _voeuAmendementBean.setNumeroDeliberation( strNumeroVoeu );
            }
            else if ( ( request.getParameter( OdsParameters.NUMERO_VOEU_ADOPTE ) != null ) &&
                    request.getParameter( OdsParameters.NUMERO_VOEU_ADOPTE ).trim(  )
                               .equals( OdsConstants.CONSTANTE_CHAINE_VIDE ) )
            {
                _voeuAmendementBean.setNumeroDeliberation( null );
            }

            _voeuAmendementHome.update( _voeuAmendementBean, plugin );

            int nIdVA = _voeuAmendementBean.getId(  );
            String strMessage;

            if ( request.getParameter( OdsParameters.A_FAIRE ).trim(  ).equals( CONSTANTE_ACTION_SAVE ) )
            {
                // On vide l'objet VA en session
                _voeuAmendementBean = newVoeuAmendement(  );
            }

            if ( request.getParameter( OdsParameters.A_FAIRE ).trim(  ).equals( OdsParameters.CONSTANTE_ACTION_APPLY ) )
            {
                // Enregistrement du nouveau VA, en base
                // On initialise l'id du VA en session  
                _voeuAmendementBean.setId( nIdVA );
                strMessage = getModificationUrl( request, CONSTANTE_TYPE_STATUT_VA, nIdVA, strReturnDefault );
            }
            else
            {
                strMessage = getHomeUrl( request, CONSTANTE_TYPE_STATUT_VA, strReturnDefault );
            }

            return strMessage;
        }

        return AdminMessageService.getMessageUrl( request, Messages.MANDATORY_FIELDS, AdminMessage.TYPE_STOP );
    }

    /**
     * Renvoie l'objet de l'acte utilis lors de l'envoi vers la plate-forme
     *                         V NumroAdopte (Fascicule pour ODS CP) Objet
     * @param acte l'acte que l'on va transmettre
     * @return strObjet l'objet de l'acte
     */
    public String getObjetActe( Acte<GSeance, GFichier> acte )
    {
        Plugin plugin = getPlugin(  );
        String strObjet = OdsConstants.CONSTANTE_CHAINE_VIDE;

        GVoeuAmendement va = _voeuAmendementHome.findByPrimaryKey( acte.getIdEntite(  ), plugin );

        if ( va != null )
        {
            strObjet = va.getReferenceCompleteCertificatAffichage(  ) + OdsConstants.CONSTANTE_ESPACE +
                va.getObjet(  );
        }

        return strObjet;
    }
}
